/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "IBQuickBuildFeedbackController.h"

#import "ICQuickBuildWindow.h"

@implementation IBQuickBuildFeedbackController

+ (IBQuickBuildFeedbackController *) sharedController
{
	static IBQuickBuildFeedbackController * sQuickBuildFeedbackController=nil;
	
	if (sQuickBuildFeedbackController==nil)
	{
		sQuickBuildFeedbackController=[IBQuickBuildFeedbackController new];
	}
	
	return sQuickBuildFeedbackController;
}

- (void) dealloc
{
	[viewsArray_ release];
	
	[viewsDictionary_ release];

	[IBwindow_ release];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	[super dealloc];
}

- (id) init
{
	self=[super init];
	
	if (self!=nil)
	{
		NSRect tBounds;
		
		IBwindow_=[[ICQuickBuildWindow alloc] init];
		
		IBmainView_=[IBwindow_ contentView];
		
		tBounds=[IBmainView_ bounds];
		
		IBbuildingLabelView_=[[ICQuickBuildTextField alloc] initWithFrame:NSMakeRect(10.0,NSMaxY(tBounds)-44.0f,IC_QUICKBUILD_WINDOW_WIDTH-20.0f,31.0f)];
		
		if (IBbuildingLabelView_!=nil)
		{
			[IBbuildingLabelView_ setDrawsBackground:NO];
		
			[IBbuildingLabelView_ setBezeled:NO];
			
			[IBbuildingLabelView_ setBordered:NO];
			
			[IBbuildingLabelView_ setEditable:NO];
			
			[IBbuildingLabelView_ setSelectable:NO];
			
			[IBbuildingLabelView_ setAlignment:NSCenterTextAlignment];
			
			[IBbuildingLabelView_ setFont:[NSFont systemFontOfSize:20.0f]];
			
			[IBbuildingLabelView_ setStringValue:NSLocalizedString(@"Building",@"")];
			
			[IBmainView_ addSubview:IBbuildingLabelView_];
			
			// Register for notifications
			
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(viewFrameDidChange:) name:NSViewFrameDidChangeNotification object:IBmainView_];
			
			[IBbuildingLabelView_ release];
		}
		
		viewsArray_=[NSMutableArray new];
		
		viewsDictionary_=[NSMutableDictionary new];
	}
	
	return self;
}

#pragma mark -

- (void) addViewForUUID:(NSString *) inUUID fileName:(NSString *) inFileName
{
	NSView * tBuildView;
	NSRect tBounds;
	NSUInteger i,tCount;
	NSRect tBuildViewFrame;
	
	tCount=[viewsArray_ count];
	
	tBuildViewFrame=NSMakeRect(tCount*IC_QUICKBUILD_WINDOW_WIDTH,0.0f,IC_QUICKBUILD_WINDOW_WIDTH,IC_QUICKBUILD_WINDOW_WIDTH);
	
	tBuildView=[[NSView alloc] initWithFrame:tBuildViewFrame];
	
	if (tBuildView!=nil) 
	{
		ICQuickBuildTextField * tTextField;
		ICQuickBuildStatusView * tStatusView=nil;
		NSImageView * tImageView;
		NSDictionary * tViewDictionary;
		
		tBounds=[tBuildView bounds];
		
		// Background Image
		
		tImageView=[[NSImageView alloc] initWithFrame:NSMakeRect(NSMidX(tBounds)-64.0f,NSMidY(tBounds)-64.0f,128.0f,128.0f)];
		
		if (tImageView!=nil)
		{
			
			
			[tImageView setImage:[NSImage imageNamed:@"BuildPackageGray128"]];
		
			[tBuildView addSubview:tImageView];
			
			
			tStatusView=[[ICQuickBuildStatusView alloc] initWithFrame:NSMakeRect(35.5f,35.5f,57.0f,57.0f)];
			
			if (tStatusView!=nil)
			{
				[tImageView addSubview:tStatusView];
				
				[tStatusView autorelease];
			}
			
			[tImageView release];
		}
		
		// Text labels
		
		tTextField=[[ICQuickBuildTextField alloc] initWithFrame:NSMakeRect(10.0f,NSMinY(tBounds)+10.0f,IC_QUICKBUILD_WINDOW_WIDTH-20.0f,25.0f)];
		
		if (tTextField!=nil)
		{
			[[tTextField cell] setLineBreakMode:NSLineBreakByTruncatingMiddle];
			
			[tTextField setDrawsBackground:NO];
		
			[tTextField setBezeled:NO];
			
			[tTextField setBordered:NO];
			
			[tTextField setEditable:NO];
			
			[tTextField setSelectable:NO];
			
			[tTextField setAlignment:NSCenterTextAlignment];
			
			[tTextField setFont:[NSFont systemFontOfSize:14.0f]];
			
			[tTextField setStringValue:inFileName];
		
			[tBuildView addSubview:tTextField];
		
			[tTextField release];
		}
		
		
		tViewDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tBuildView,@"BUILD_VIEW",
																   tStatusView,@"STATUS_VIEW",
																   nil];
		
		[viewsArray_ addObject:tViewDictionary];
		
		[viewsDictionary_ setObject:tViewDictionary forKey:inUUID];
		
		[IBmainView_ addSubview:tBuildView];
		
		[tBuildView release];
		
		
		tCount=[viewsArray_ count];
		
		for(i=0;i<tCount;i++)
		{
			NSDictionary * tDictionary;
			NSView * tView;
			
			
			tDictionary=[viewsArray_ objectAtIndex:i];
			
			tView=[tDictionary objectForKey:@"BUILD_VIEW"];
			
			if (tView!=nil)
			{
				[tView setAutoresizingMask:NSViewMaxXMargin];
			}
		}
		
		if (tCount==1)
		{
			[IBwindow_ makeKeyAndOrderFront:self];
		}
		else
		{
			NSRect tContentViewFrame;
			NSRect tNewWindowBounds;
			NSRect tOldWindowFrame;
			NSRect tNewWindowFrame;
			/*NSDictionary * tAnimationDictionary;
			NSViewAnimation * tAnimation;*/
			
			tContentViewFrame=[[IBwindow_ contentView] frame];
			
			tContentViewFrame.size.width=tCount*IC_QUICKBUILD_WINDOW_WIDTH;
		
			tNewWindowBounds=[IBwindow_ frameRectForContentRect:tContentViewFrame];
			
			tOldWindowFrame=[IBwindow_ frame];
			
			tNewWindowFrame=NSMakeRect(_CGFloatFloor(NSMidX(tOldWindowFrame)-NSWidth(tNewWindowBounds)*0.5),tOldWindowFrame.origin.y,NSWidth(tNewWindowBounds),NSHeight(tOldWindowFrame));
			
			// Animation
			
			[IBwindow_ setFrame:tNewWindowFrame display:YES animate:YES];
			
			/*tAnimationDictionary=[NSDictionary dictionaryWithObjectsAndKeys:IBwindow_,NSViewAnimationTargetKey,
																			[NSValue valueWithRect: tNewWindowFrame], NSViewAnimationEndFrameKey,
																			nil];
			
			tAnimation=[[NSViewAnimation alloc] initWithViewAnimations: [NSArray arrayWithObject:tAnimationDictionary]];
			
			if (tAnimation!=nil)
			{
				[tAnimation setAnimationBlockingMode:NSAnimationBlocking];
				
				[tAnimation setDuration: 0.15f]; // or however long you want it for

				[tAnimation startAnimation];
				
				[tAnimation autorelease];
			}*/
		}
	}
}

- (void) removeViewForUUID:(NSString *) inUUID
{
	NSUInteger i,tCount;
	NSDictionary * tDictionary;
	
	tCount=[viewsArray_ count];
	
	tDictionary=[viewsDictionary_ objectForKey:inUUID];
	
	if (tDictionary!=nil)
	{
		NSView * tView;
		NSUInteger tIndex;
		
		tView=[tDictionary objectForKey:@"BUILD_VIEW"];
		
		if (tView!=nil)
		{
			
		}
		
		tIndex=[viewsArray_ indexOfObject:tDictionary];
		
		if (tIndex!=NSNotFound)
		{
			NSView * tOtherView;
			NSDictionary * tOtherDictionary;
			
			for(i=0;i<tIndex;i++)
			{
				tOtherDictionary=[viewsArray_ objectAtIndex:i];
			
				tOtherView=[tOtherDictionary objectForKey:@"BUILD_VIEW"];
				
				if (tOtherView!=nil)
				{
					[tOtherView setAutoresizingMask:NSViewMaxXMargin];
				}
			}
			
			for(i=tIndex;i<tCount;i++)
			{
				tOtherDictionary=[viewsArray_ objectAtIndex:i];
			
				tOtherView=[tOtherDictionary objectForKey:@"BUILD_VIEW"];
				
				if (tOtherView!=nil)
				{
					[tOtherView setAutoresizingMask:NSViewMinXMargin];
				}
			}
			
			[viewsArray_ removeObjectAtIndex:tIndex];
		}
		
		[viewsDictionary_ removeObjectForKey:inUUID];
		
		[tView removeFromSuperview];
		
		if (tCount==1)
		{
			[IBwindow_ orderOut:self];
		}
		else
		{
			NSRect tContentViewFrame;
			NSRect tNewWindowBounds;
			NSRect tOldWindowFrame;
			NSRect tNewWindowFrame;
			/*NSDictionary * tAnimationDictionary;
			NSViewAnimation * tAnimation;*/
			
			tContentViewFrame=[[IBwindow_ contentView] frame];
			
			tCount=[viewsArray_ count];
			
			tContentViewFrame.size.width=tCount*IC_QUICKBUILD_WINDOW_WIDTH;
			
			tNewWindowBounds=[IBwindow_ frameRectForContentRect:tContentViewFrame];
			
			tOldWindowFrame=[IBwindow_ frame];
			
			tNewWindowFrame=NSMakeRect(_CGFloatFloor(NSMidX(tOldWindowFrame)-NSWidth(tNewWindowBounds)*0.5),tOldWindowFrame.origin.y,NSWidth(tNewWindowBounds),NSHeight(tOldWindowFrame));
			
			// Animation
			
			[IBwindow_ setFrame:tNewWindowFrame display:YES animate:YES];
			
			/*tAnimationDictionary=[NSDictionary dictionaryWithObjectsAndKeys:IBwindow_,NSViewAnimationTargetKey,
																			[NSValue valueWithRect: tNewWindowFrame], NSViewAnimationEndFrameKey,
																			nil];
			
			tAnimation=[[NSViewAnimation alloc] initWithViewAnimations: [NSArray arrayWithObject:tAnimationDictionary]];
			
			if (tAnimation!=nil)
			{
				[tAnimation setAnimationBlockingMode:NSAnimationBlocking];
				
				[tAnimation setDuration: 0.15f]; // or however long you want it for

				[tAnimation startAnimation];
				
				[tAnimation autorelease];
			}*/
		}
	}
	
	if ([viewsArray_ count]==0)
	{
		[IBwindow_ orderOut:self];
	}
}

- (void) setStatus:(int) inStatus forUUID:(NSString *) inUUID
{
	NSDictionary * tDictionary;
	
	tDictionary=[viewsDictionary_ objectForKey:inUUID];
	
	if (tDictionary!=nil)
	{
		ICQuickBuildStatusView * tStatusView;
		
		tStatusView=[tDictionary objectForKey:@"STATUS_VIEW"];
		
		if (tStatusView!=nil)
		{
			[tStatusView setStatus:inStatus];
		}
	}
}

#pragma mark -

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	// We want to make sure the no display text label is centered
	
	NSRect tViewFrame;
	NSRect tLabelFrame;
	
	tViewFrame=[IBmainView_ frame];
	
	tLabelFrame=[IBbuildingLabelView_ frame];
	
	tLabelFrame.origin.x=_CGFloatRound(NSMidX(tViewFrame)-NSWidth(tLabelFrame)*0.5f);
	
	[IBbuildingLabelView_ setFrame:tLabelFrame];
}

@end
