/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICBackgroundUtilities.h"

#import "ICDocumentFormat+Constants.h"

#import "ICPresentation+Notifications.h"

NSString * const ICBACKGROUNDUTILITIES_INSTALLER_PATH=@"/System/Library/CoreServices/Installer.app";

NSString * const ICBACKGROUNDUTILITIES_DEFAULT_BACKGROUND_NAME=@"defaultBackground";

@implementation ICBackgroundUtilities

+ (NSImage *) defaultBackgroundPicture
{
	static NSImage * sDefaultBackgroundPicture=nil;
	
	if (sDefaultBackgroundPicture==nil)
	{
		NSBundle * tBundle;
		
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5

        tBundle=[NSBundle bundleWithPath:ICBACKGROUNDUTILITIES_INSTALLER_PATH];
#else
		tBundle=[NSBundle mainBundle];
#endif
		
		if (tBundle!=nil)
		{
			NSString * tPath;
			
			tPath=[tBundle pathForResource:ICBACKGROUNDUTILITIES_DEFAULT_BACKGROUND_NAME ofType:@"tiff"];
			
			if (tPath!=nil)
			{
				sDefaultBackgroundPicture=[[NSImage alloc] initWithContentsOfFile:tPath];
		
				if (sDefaultBackgroundPicture!=nil)
				{
					[sDefaultBackgroundPicture setScalesWhenResized:YES];
				}
			}
		}
	}
	
	return sDefaultBackgroundPicture;
}

+ (NSMutableDictionary *) defaultBackgroundDictionary
{
	return [NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:NO],ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_CUSTOM,
															 [NSMutableDictionary dictionary],ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH,
															 [NSNumber numberWithInt:ICALIGNMENT_LEFT],ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_ALIGNMENT,
															 [NSNumber numberWithInt:ICSCALING_PROPORTIONALLY],ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_SCALING,
															 nil];
}

+ (void) notifyBackgroundDidChangeWithSender:(ICDocumentSecondaryController *) sender
{
	[[NSNotificationCenter defaultCenter] postNotificationName:ICPRESENTATION_NOTIFICATION_BACKGROUND_SETTINGS_DID_CHANGE
														object:[sender document]
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:sender,ICPRESENTATION_NOTIFICATION_SENDER,nil]];
}

+ (NSArray *) backgroundImageTypes
{
	return [NSArray arrayWithObjects:@"'TIFF'",
									 @"TIF",
									 @"tif",
									 @"TIFF",
									 @"tiff",
									 @"'JPEG'",
									 @"JPG",
									 @"jpg",
									 @"JPEG",
									 @"jpeg",
									 @"'GIFf'",
									 @"GIF",
									 @"gif",
									 @"'PDF '",
									 @"PDF",
									 @"pdf",
									 @"'PICT'",
									 @"PCT",
									 @"pct",
									 @"PICT",
									 @"pict",
									 @"'EPSF'",
									 @"EPSI",
									 @"epsi",
									 @"EPSF",
									 @"epsf",
									 @"EPI",
									 @"epi",
									 @"EPS",
									 @"eps",
									 @"'PNGf'",
									 @"png",
									 nil];
}

+ (NSArray *) backgroundImageUTIs
{
	return [NSArray arrayWithObjects:(NSString *) kUTTypeGIF,
									 kUTTypeJPEG,
									 kUTTypePDF,
									 kUTTypePICT,
									 kUTTypePNG,
									 kUTTypeTIFF,
									 nil];
}

+ (NSSize) sizeOfImageAtPath:(NSString *) inPath
{
	NSSize tSize=NSZeroSize;
	
	if (inPath!=nil)
	{
		NSURL * tURL;
	
		tURL=[NSURL fileURLWithPath:inPath];
		
		if (tURL!=nil)
		{
			CGImageSourceRef tImageSource;
			
			tImageSource = CGImageSourceCreateWithURL((CFURLRef) tURL, NULL);
			
			if (tImageSource!=NULL)
			{
				CFDictionaryRef tDictionaryRef;
	
				tDictionaryRef = CGImageSourceCopyPropertiesAtIndex(tImageSource,0, NULL);
				
				if (tDictionaryRef!=NULL)
				{
					tSize.width=[[((NSDictionary *) tDictionaryRef) objectForKey:(NSString *) kCGImagePropertyPixelWidth] floatValue];
					
					tSize.height=[[((NSDictionary *) tDictionaryRef) objectForKey:(NSString *) kCGImagePropertyPixelHeight] floatValue];
					
					CFRelease(tDictionaryRef);
				}
				
				// Release Memory
				
				CFRelease(tImageSource);
			}
		}
	}
	
	return tSize;
}

@end
