/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICBuildConclusionCell.h"

#define ICON_SIZE	30.0f

@implementation ICBuildConclusionCell

- (void) dealloc
{
	[icon_ release];
	
    [title_ release];
	
	[subTitle_ release];
	
	[super dealloc];
}

- (id) copyWithZone:(NSZone *)zone
{
    ICBuildConclusionCell *cell = (ICBuildConclusionCell *) [super copyWithZone:zone];
    
	cell->icon_ = [icon_ retain];
	
	cell->title_ = [title_ retain];
	
	cell->subTitle_ = [subTitle_ retain];
    
	return cell;
}

#pragma mark -

+ (NSDictionary *) whiteTitleAttributes
{
	static NSDictionary * sWhiteTitleAttributes=nil;
	
	if (sWhiteTitleAttributes==nil)
	{
		sWhiteTitleAttributes=[[NSDictionary alloc] initWithObjectsAndKeys:[NSColor colorWithCalibratedWhite:1.0f alpha:1.0f],NSForegroundColorAttributeName,
																	  [NSFont boldSystemFontOfSize:11.0f],NSFontAttributeName,
																	  nil]; 
	}
	
	return sWhiteTitleAttributes;
}

+ (NSDictionary *) whiteSubTitleAttributes
{
	static NSDictionary * sWhiteSubTitleAttributes=nil;
	
	if (sWhiteSubTitleAttributes==nil)
	{
		sWhiteSubTitleAttributes=[[NSDictionary alloc] initWithObjectsAndKeys:[NSColor colorWithCalibratedWhite:1.0f alpha:1.0f],NSForegroundColorAttributeName,
																		 [NSFont systemFontOfSize:11.0f],NSFontAttributeName,
																		 nil]; 
	}
	
	return sWhiteSubTitleAttributes;
}

+ (NSDictionary *) titleAttributes
{
	static NSDictionary * sTitleAttributes=nil;
	
	if (sTitleAttributes==nil)
	{
		sTitleAttributes=[[NSDictionary alloc] initWithObjectsAndKeys:[NSColor colorWithCalibratedWhite:0.15f alpha:1.0f],NSForegroundColorAttributeName,
																	  [NSFont boldSystemFontOfSize:11.0f],NSFontAttributeName,
																	  nil]; 
	}
	
	return sTitleAttributes;
}

+ (NSDictionary *) subTitleAttributes
{
	static NSDictionary * sSubTitleAttributes=nil;
	
	if (sSubTitleAttributes==nil)
	{
		sSubTitleAttributes=[[NSDictionary alloc] initWithObjectsAndKeys:[NSColor colorWithCalibratedWhite:0.15f alpha:1.0f],NSForegroundColorAttributeName,
																		 [NSFont systemFontOfSize:11.0f],NSFontAttributeName,
																		 nil]; 
	}
	
	return sSubTitleAttributes;
}

#pragma mark -

- (void) setStatus:(int) inStatus
{
	status_=inStatus;
}

- (int) status
{
	return status_;
}

- (void) setIcon:(NSImage *) inIcon
{
    if (inIcon != icon_)
	{
        [icon_ release];
		
        icon_ = [inIcon retain];
    }
}

- (NSImage *) icon
{
    return icon_;
}

- (void) setTitle:(NSString *) inTitle
{
	if (inTitle != title_)
	{
        [title_ release];
		
        title_ = [inTitle retain];
    }
}

- (NSString *) title
{
	return title_;
}

- (void) setSubTitle:(NSString *) inSubTitle
{
	if (inSubTitle != subTitle_)
	{
        [subTitle_ release];
		
        subTitle_ = [inSubTitle retain];
    }
}

- (NSString *) subTitle
{
	return subTitle_;
}

#pragma mark -

- (void) drawWithFrame:(NSRect) inFrame inView:(NSView *) controlView
{
    NSRect tIconFrame;
	
	if ([self isHighlighted]==NO)
	{
		NSRect tRowFrame;
		
		tRowFrame=NSMakeRect(0.0f,NSMinY(inFrame),NSWidth([controlView bounds]),NSHeight(inFrame));
		
		if (status_==1)
		{
			[[NSColor colorWithCalibratedRed:0.8471f green:0.9647f blue:0.8510f alpha:1.0f] set];
		}
		else
		{
			[[NSColor colorWithCalibratedRed:0.9647f green:0.8471f blue:0.8510f alpha:1.0f] set];
		}
		
		NSRectFill(tRowFrame);
	
		if (status_==1)
		{
			[[NSColor colorWithCalibratedRed:0.1647f green:0.7882f blue:0.0902f alpha:1.0f] set];
		}
		else
		{
			[[NSColor colorWithCalibratedRed:0.7882f green:0.1647f blue:0.0902f alpha:1.0f] set];
		}
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(tRowFrame),NSMinY(tRowFrame)-0.5f) toPoint:NSMakePoint(NSMaxX(tRowFrame),NSMinY(tRowFrame)-0.5f)];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(tRowFrame),NSMaxY(tRowFrame)+0.5f) toPoint:NSMakePoint(NSMaxX(tRowFrame),NSMaxY(tRowFrame)+0.5f)];
	}
	
	tIconFrame=NSMakeRect(NSMinX(inFrame)+4.0f,NSMinY(inFrame)+5.0f,ICON_SIZE,ICON_SIZE);
	
	// Draw the icon
	
	if (icon_!=nil)
	{
		NSRect tSourceFrame;
		
		tSourceFrame.origin=NSZeroPoint;
		
		tSourceFrame.size=[icon_ size];
		
		[icon_ setFlipped:[controlView isFlipped]];
		
		[icon_ drawInRect:tIconFrame fromRect:tSourceFrame operation:NSCompositeSourceOver fraction:1.0f];
	}
	else
	{
		[[NSColor grayColor] set];
		
		NSFrameRect(tIconFrame);
	}
	
	// Draw the title
	
	if (title_!=nil)
	{
		NSRect tTitleFrame;
		NSSize tSize;
		
		tSize=inFrame.size;
		
		tTitleFrame=[title_ boundingRectWithSize:tSize options:0 attributes:[ICBuildConclusionCell titleAttributes]];
		
		tTitleFrame.origin=NSMakePoint(NSMaxX(tIconFrame)+5.0f,NSMinY(inFrame)+5.0f);
		
		if ([self isHighlighted]==NO)
		{
			[title_ drawInRect:tTitleFrame withAttributes:[ICBuildConclusionCell titleAttributes]];
		}
		else
		{
			[title_ drawInRect:tTitleFrame withAttributes:[ICBuildConclusionCell whiteTitleAttributes]];
		}
	}
	
	// Draw the subtitle
	
	if (subTitle_!=nil)
	{
		NSRect tSubTitleFrame;
		NSSize tSize;
		
		tSize=inFrame.size;
		
		tSubTitleFrame=[subTitle_ boundingRectWithSize:tSize options:0 attributes:[ICBuildConclusionCell subTitleAttributes]];
		
		tSubTitleFrame.origin=NSMakePoint(NSMaxX(tIconFrame)+5.0f,NSMinY(inFrame)+20.0f);
		
		if ([self isHighlighted]==NO)
		{
			[subTitle_ drawInRect:tSubTitleFrame withAttributes:[ICBuildConclusionCell subTitleAttributes]];
		}
		else
		{
			[subTitle_ drawInRect:tSubTitleFrame withAttributes:[ICBuildConclusionCell whiteSubTitleAttributes]];
		}
	}
}

@end
