/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICBuildTree.h"

@implementation ICBuildNodeData

+ (id) nodeOfType:(int) inType withTitle:(NSString *) inTitle subTitle:(NSString *) inSubTitle status:(int) inStatus
{
	return [[[ICBuildNodeData alloc] initWithType:inType title:inTitle subTitle:inSubTitle status:inStatus] autorelease];
}

- (id) initWithType:(int) inType title:(NSString *) inTitle subTitle:(NSString *) inSubTitle status:(int) inStatus
{
	 self=[super init];
    
    if (self!=nil)
    {
        type_=inType;
		
		status_=inStatus;
		
		title_=[inTitle retain];
        
		subTitle_=[inSubTitle retain];
    }
    
    return self;
}

+ (id) rootNode
{
    return [ICBuildNodeData  nodeOfType:ICBUILDTREE_TYPE_ROOT withTitle:nil subTitle:nil status:0];
}

- (void) dealloc
{
	[title_ release];
	
	[subTitle_ release];
	
	[super dealloc];
}

#pragma mark -

- (BOOL) isLeaf
{
    if ((type_==ICBUILDTREE_TYPE_STEP && status_!=ICBUILDTREE_STATUS_FAILURE) ||
		type_==ICBUILDTREE_TYPE_ERROR_DESCRIPTION ||
		type_==ICBUILDTREE_TYPE_CONCLUSION)
	{
		return YES;
	}
    
    return NO;
}

- (int) type
{
    return type_;
}

- (void) setType:(int) inType
{
    type_=inType;
}

- (int) status
{
    return status_;
}

- (void) setStatus:(int) inStatus
{
    status_=inStatus;
}

- (NSString *) title
{
    return [[title_ retain] autorelease]; 
}

- (void) setTitle:(NSString *) inTitle
{
    if (title_!=inTitle)
    {
        [title_ release];
        
        title_=[inTitle copy];
    }
}

- (NSString *) subTitle
{
    return [[subTitle_ retain] autorelease]; 
}

- (void) setSubTitle:(NSString *) inSubTitle
{
    if (subTitle_!=inSubTitle)
    {
        [subTitle_ release];
        
        subTitle_=[inSubTitle copy];
    }
}

@end

@implementation ICBuildTreeNode

+ (id) buildTree
{
    ICBuildTreeNode * rootNode=nil;
    
    rootNode=[[ICBuildTreeNode alloc] initWithData:[ICBuildNodeData rootNode]
                                            parent:nil
                                          children:[NSArray array]];
    
    return [rootNode autorelease];
}

@end