/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICBuildWindowController.h"

#import "ICDocumentFormat+MinimumConstants.h"

#import "ICProjectBuilder+Notifications.h"

#import "ICBuildImageAndTextCell.h"

#import "ICBuildStepCell.h"

#import "ICBuildConclusionCell.h"

#import "ICDocument.h"


#define SAFE_BUILDNODE(n) 	((ICBuildTreeNode*)((n!=nil)?(n):(tree_)))


@implementation ICBuildWindowController

- (void) dealloc
{
	[dateFormatter_ release];
	
	dateFormatter_=nil;
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [[NSDistributedNotificationCenter defaultCenter] removeObserver:self];
        
    if (tree_!=nil)
    {
        [tree_ release];
    }
    
    [super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{
	//NSTableColumn *tableColumn = nil;
	
	dateFormatter_=[NSDateFormatter new];
	
	if (dateFormatter_!=nil)
	{
		[dateFormatter_ setFormatterBehavior:NSDateFormatterBehavior10_4];
	
		[dateFormatter_ setDateStyle:NSDateFormatterShortStyle];
		
		[dateFormatter_ setTimeStyle:NSDateFormatterShortStyle];
	}
	
	// A COMPLETER
	
	
	defaults_=[NSUserDefaults standardUserDefaults];
    
    
	
	[IBoutlineView_ setAutoresizesOutlineColumn:NO];
	
	
	// Window Title
    
    [IBwindow_ setTitle:[NSString stringWithFormat:NSLocalizedStringFromTable(@"%@ - Build Log",@"Build",@"No commets"),[document_ displayName]]];
	
	[IBwindow_ center];
	
	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
                                            selector:@selector(buildDidStart:)
                                                name:PACKAGES_BUILDER_NOTIFICATION_BUILD_DID_START
                                              object:document_];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
                                            selector:@selector(cleanDidStart:)
                                                name:PACKAGES_BUILDER_NOTIFICATION_CLEAN_DID_START
                                              object:document_];
	
	[[NSDistributedNotificationCenter defaultCenter] addObserver:self
                                                        selector:@selector(builderNotification:)
                                                            name:PACKAGES_BUILDER_NOTIFICATION
                                                          object:nil
                                                          suspensionBehavior:NSNotificationSuspensionBehaviorDeliverImmediately];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(builderNotification:)
                                                 name:PACKAGES_BUILDER_NOTIFICATION
                                               object:nil];
}

#pragma mark -

+ (NSImage *) iconForBuildStep:(int) inBuildStep
{
	NSString * tPath=nil;

	if (inBuildStep==ICDOCUMENT_TYPE_DISTRIBUTION)
	{
		static NSImage * sDistributionProjectIcon=nil;
		
		if (sDistributionProjectIcon==nil)
		{
			sDistributionProjectIcon=[[[NSWorkspace sharedWorkspace] iconForFileType:@".mpkg"] copy];
			
			[sDistributionProjectIcon setScalesWhenResized:YES];
		}
		
		return sDistributionProjectIcon;
	}
	else if (inBuildStep==ICDOCUMENT_TYPE_PACKAGE)
	{
		static NSImage * sPackageProjectIcon=nil;
		
		if (sPackageProjectIcon==nil)
		{
			sPackageProjectIcon=[[[NSWorkspace sharedWorkspace] iconForFileType:@".pkg"] copy];
			
			[sPackageProjectIcon setScalesWhenResized:YES];
		}
		
		return sPackageProjectIcon;
	}
	else if (inBuildStep==ICBUILDTREE_TYPE_DISTRIBUTION_SCRIPT)
	{
		static NSImage * sDistributionScriptProjectIcon=nil;
		
		if (sDistributionScriptProjectIcon==nil)
		{
			tPath=[[NSBundle mainBundle] pathForResource:@"XML_File_32" ofType:@"tif"];
			
			if (tPath!=nil)
			{
				sDistributionScriptProjectIcon=[[NSImage alloc] initWithContentsOfFile:tPath];
			}
			
			[sDistributionScriptProjectIcon setScalesWhenResized:YES];
		}
		
		return sDistributionScriptProjectIcon;
	}
	
	return nil;
}

+ (NSImage *) iconForStepStatus:(int) inStatus
{
	NSString * tPath=nil;

	if (inStatus==ICBUILDTREE_STATUS_RUNNING)
	{
		static NSImage * sStepRunningIcon=nil;
		
		if (sStepRunningIcon==nil)
		{
			tPath=[[NSBundle mainBundle] pathForResource:@"Running_12" ofType:@"tif"];
			
			if (tPath!=nil)
			{
				sStepRunningIcon=[[NSImage alloc] initWithContentsOfFile:tPath];
			}
		}
		
		return sStepRunningIcon;
	}
	else if (inStatus==ICBUILDTREE_STATUS_SUCCESS)
	{
		static NSImage * sStepSuccessIcon=nil;
		
		if (sStepSuccessIcon==nil)
		{
			tPath=[[NSBundle mainBundle] pathForResource:@"Success_12" ofType:@"tif"];
			
			if (tPath!=nil)
			{
				sStepSuccessIcon=[[NSImage alloc] initWithContentsOfFile:tPath];
			}
		}
		
		return sStepSuccessIcon;
	}
	else if (inStatus==ICBUILDTREE_STATUS_FAILURE)
	{
		static NSImage * sStepFailureIcon=nil;
		
		if (sStepFailureIcon==nil)
		{
			tPath=[[NSBundle mainBundle] pathForResource:@"Failure_12" ofType:@"tif"];
			
			if (tPath!=nil)
			{
				sStepFailureIcon=[[NSImage alloc] initWithContentsOfFile:tPath];
			}
		}
		
		return sStepFailureIcon;
	}
	
	return nil;
}

#pragma mark -

- (NSWindow *) window
{
    return IBwindow_;
}

- (BOOL) validateMenuItem:(NSMenuItem *)aMenuItem
{    
    SEL tAction=[aMenuItem action];
    
    if (tAction==@selector(showHideBuildWindow:))
    {
        [aMenuItem setTitle:NSLocalizedStringFromTable(@"Hide Build Log Window",@"Build",@"No comment")];
		
		return YES;
    }
    
    return [document_ validateMenuItem:aMenuItem];
}

#pragma mark -

- (IBAction) showHideBuildWindow:(id) sender
{
    [IBwindow_ orderOut:nil];
}

- (IBAction) build:(id) sender
{
    if (document_!=nil)
    {
        [document_ build:sender];
    }
}

- (IBAction) buildAndRun:(id) sender
{
    if (document_!=nil)
    {
        [document_ buildAndRun:sender];
    }
}

- (IBAction) buildAndDebug:(id) sender
{
    if (document_!=nil)
    {
        [document_ buildAndDebug:sender];
    }
}

- (IBAction) clean:(id) sender
{
    if (document_!=nil)
    {
        [document_ clean:sender];
    }
}

- (IBAction) hideWindow:(id)sender
{
    [IBwindow_ orderOut:sender];
}

- (IBAction) showWindow:(id)sender cleanWindow:(BOOL) inCleanWindow
{
	if (inCleanWindow==YES)
	{
		[IBstatusLabel_ setStringValue:@""];
	}

    [IBwindow_ makeKeyAndOrderFront:sender];
}

#pragma mark -

- (id) outlineView:(NSOutlineView *) inOutlineView child:(NSInteger) inIndex ofItem:(id) inItem
{
    if (inOutlineView==IBoutlineView_)
	{
		if (tree_!=nil)
		{
			return [SAFE_BUILDNODE(inItem) childAtIndex:inIndex];
		}
    }
	
    return nil;
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView isItemExpandable:(id) inItem
{
    if (inOutlineView==IBoutlineView_)
	{
		ICBuildNodeData * tNodeData;
		
		tNodeData=ICBUILDNODE_DATA(inItem);
		
		if ([tNodeData type]==ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE)
		{
			return ([SAFE_BUILDNODE(inItem) numberOfChildren]>0);
		}
		
		return ![tNodeData isLeaf];
	}
	
	return NO;

}

- (NSInteger) outlineView:(NSOutlineView *) inOutlineView numberOfChildrenOfItem:(id) inItem
{
    if (inOutlineView==IBoutlineView_)
	{
		if (tree_!=nil)
		{
			return [SAFE_BUILDNODE(inItem) numberOfChildren];
		}
    }
	
    return 0;
}

- (id) outlineView:(NSOutlineView *) inOutlineView objectValueForTableColumn:(NSTableColumn *) inColumn byItem:(id) inItem
{
	return nil;
}

- (id) outlineView:(NSOutlineView *) inOutlineView column:(NSTableColumn *) inColumn dataCellForItem:(id) inItem
{
	if (inOutlineView==IBoutlineView_)
	{
		if (inItem!=nil)
		{
			int tType;
			ICBuildNodeData * tNodeData;
			
			tNodeData=ICBUILDNODE_DATA(inItem);
			
			tType=[tNodeData type];
			
			switch(tType)
			{
				case ICBUILDTREE_TYPE_PROJECT:
				case ICBUILDTREE_TYPE_DISTRIBUTION_SCRIPT:
				case ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE:
				case ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE_PROJECT:
				case ICBUILDTREE_TYPE_PACKAGE:
					
					return [[ICBuildStepCell new] autorelease];
					
				case ICBUILDTREE_TYPE_STEP:
				case ICBUILDTREE_TYPE_STEP_PARENT:
				case ICBUILDTREE_TYPE_ERROR_DESCRIPTION:
				{
					ICBuildImageAndTextCell * tImageAndTextCell;
					
					tImageAndTextCell=[[ICBuildImageAndTextCell alloc] initTextCell:@""];
					
					if (tImageAndTextCell!=nil)
					{
						[tImageAndTextCell setFont:[NSFont systemFontOfSize:11.0f]];
					}
					
					return [tImageAndTextCell autorelease];
				
				}
				case ICBUILDTREE_TYPE_CONCLUSION:
					
					return [[ICBuildConclusionCell new] autorelease];
					
					break;
			}
		}
	}
	
	

    return nil;
}

- (CGFloat) outlineView:(NSOutlineView *) inOutlineView heightOfRowByItem:(id) inItem
{
	if (inOutlineView==IBoutlineView_)
	{
		int tType;
        ICBuildNodeData * tNodeData;
		
		tNodeData=ICBUILDNODE_DATA(inItem);
		
        tType=[tNodeData type];
		
		switch(tType)
		{
			case ICBUILDTREE_TYPE_PROJECT:
			case ICBUILDTREE_TYPE_DISTRIBUTION_SCRIPT:
			case ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE:
			case ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE_PROJECT:
			case ICBUILDTREE_TYPE_PACKAGE:
				
				return 34.0f;
				
				break;
				
			case ICBUILDTREE_TYPE_STEP:
			case ICBUILDTREE_TYPE_ERROR_DESCRIPTION:
			case ICBUILDTREE_TYPE_STEP_PARENT:
			
				return 15.0f;
				
				break;
			
			case ICBUILDTREE_TYPE_CONCLUSION:
				
				return 40.0f;
				
				break;
		}
	}
	
	return 17.0f;
}

- (void) outlineView:(NSOutlineView *) inOutlineView willDisplayCell:(id) inCell forTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{
	if (inOutlineView==IBoutlineView_)
	{
		if ([[inTableColumn identifier] isEqualToString: @"Event"])
		{
			int tType;
			ICBuildNodeData * tNodeData;
			ICBuildStepCell * tBuildStepCell;
			ICBuildImageAndTextCell * tImageAndTextCell;
			ICBuildConclusionCell * tBuildConclusionCell;
			
			tNodeData=ICBUILDNODE_DATA(inItem);
			
			tType=[tNodeData type];
			
			switch(tType)
			{
				case ICBUILDTREE_TYPE_PROJECT:
				case ICBUILDTREE_TYPE_DISTRIBUTION_SCRIPT:
				case ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE:
				case ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE_PROJECT:
				case ICBUILDTREE_TYPE_PACKAGE:
				
					tBuildStepCell=(ICBuildStepCell *) inCell;
					
					if (tBuildStepCell!=nil)
					{
						NSImage * tStatusIcon=nil;
						NSImage * tBadgeIcon=nil;
						int tStatus;
						
						// Icon
						
						switch(tType)
						{
							case ICBUILDTREE_TYPE_PROJECT:
							
								tStatusIcon=[ICBuildWindowController iconForBuildStep:[document_ documentType]];
								
								break;
								
							case ICBUILDTREE_TYPE_DISTRIBUTION_SCRIPT:
								
								tStatusIcon=[ICBuildWindowController iconForBuildStep:ICBUILDTREE_TYPE_DISTRIBUTION_SCRIPT];
								
								break;
								
							case ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE:
							case ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE_PROJECT:
							
								tStatusIcon=[ICBuildWindowController iconForBuildStep:ICDOCUMENT_TYPE_PACKAGE];
								
								break;
								
							case ICBUILDTREE_TYPE_PACKAGE:
						
								break;
						}
						
						[tBuildStepCell setIcon:tStatusIcon];
						
						// Badge
						
						tStatus=[tNodeData status];
						
						switch(tStatus)
						{
							case ICBUILDTREE_STATUS_SUCCESS:
							
								tBadgeIcon=[ICBuildWindowController iconForStepStatus:tStatus];
								
								break;
								
							case ICBUILDTREE_STATUS_FAILURE:
								
								tBadgeIcon=[ICBuildWindowController iconForStepStatus:tStatus];
								
								break;
						}
						
						[tBuildStepCell setBadge:tBadgeIcon];
						
						// Title
						
						[tBuildStepCell setTitle:[tNodeData title]];
						
						// SubTitle
						
						[tBuildStepCell setSubTitle:[tNodeData subTitle]];
					}
				
					break;
					
				case ICBUILDTREE_TYPE_STEP:
				case ICBUILDTREE_TYPE_STEP_PARENT:
				
					tImageAndTextCell=(ICBuildImageAndTextCell *) inCell;
					
					if (tImageAndTextCell!=nil)
					{
						NSImage * tStatusIcon=nil;
						
						// Icon
						
						tStatusIcon=[ICBuildWindowController iconForStepStatus:[tNodeData status]];
						
						[tImageAndTextCell setImage:tStatusIcon];
						
						// Title
						
						[tImageAndTextCell setTitle:[tNodeData title]];
						
						[tImageAndTextCell setTextColor:[NSColor blackColor]];
					}
				
					break;
				
				case ICBUILDTREE_TYPE_ERROR_DESCRIPTION:
				
					tImageAndTextCell=(ICBuildImageAndTextCell *) inCell;
					
					if (tImageAndTextCell!=nil)
					{
						NSString * tTitle;
						
						// Title
						
						tTitle=[tNodeData title];
						
						if (tTitle==nil)
						{
							tTitle=@"";
						}
						
						[tImageAndTextCell setTitle:tTitle];
						
						[tImageAndTextCell setTextColor:[NSColor redColor]];
					}
				
					break;
				
				case ICBUILDTREE_TYPE_CONCLUSION:
					
					tBuildConclusionCell=(ICBuildConclusionCell *) inCell;
					
					if (tBuildConclusionCell!=nil)
					{
						int tStatus;
						
						tStatus=[tNodeData status];
						
						// Icon
						
						switch(tStatus)
						{
							case ICBUILDTREE_STATUS_SUCCESS:
							
								[tBuildConclusionCell setIcon:[NSImage imageNamed:@"Conclusion_Success_32"]];
								
								break;
								
							case ICBUILDTREE_STATUS_FAILURE:
								
								[tBuildConclusionCell setIcon:[NSImage imageNamed:@"Conclusion_Failure_32"]];
								
								break;
						}
						
						// Status
						
						[tBuildConclusionCell setStatus:tStatus];
						
						// Icon
						
						// Title
						
						[tBuildConclusionCell setTitle:[tNodeData title]];
						
						// SubTitle
						
						[tBuildConclusionCell setSubTitle:[tNodeData subTitle]];
					}
					
					break;
				
			}
		}
        /*NSImage * tImage=nil;
        int tType;
        
        tType=[ICBUILDNODE_DATA(inItem) type];
        
        switch([ICBUILDNODE_DATA(inItem) status])
        {
            case ICBUILDTREE_STATUS_RUNNING:
                tImage=sBuildInProgressIcon;
        	break;
            caseICBUILDTREE_STATUS_SUCCESS:
                tImage=sBuildSuccessIcon;
                break;
            case ICBUILDTREE_STATUS_FAILURE:
                tImage=sBuildFailureIcon;
                break;
        }
        
        [(ImageAndTextCell*)cell setImage: tImage];
        
        if (tType==ICBUILDTREE_TYPE_STEP_FAILED ||
            tType==ICBUILDTREE_TYPE_STEP)
        {
            [(ImageAndTextCell*)cell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
        }
        else
        {
            [(ImageAndTextCell*)cell setFont:[NSFont boldSystemFontOfSize:[NSFont smallSystemFontSize]]];
        }*/
    }
}

#pragma mark -

- (void) buildDidStart:(NSNotification *) inNotification
{
    if (tree_!=nil)
    {
        [tree_ release];
		
        tree_=nil;
    }
    
	currentBuildNode_=nil;
	
	numberOfPackagesToBuild_=0;
	
	building_=YES;
	
	buildFailed_=NO;
	
    tree_=[[ICBuildTreeNode buildTree] retain];
    
    if (tree_!=nil)
    {
        ICBuildNodeData * tProjectNodeData;
    
        tProjectNodeData=[ICBuildNodeData nodeOfType:ICBUILDTREE_TYPE_PROJECT
										   withTitle:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Build %@",@"Build",@"No comment"),[document_ displayName]]
										    subTitle:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Project %@",@"Build",@"No comment"),[document_ displayName]]	// A COMPLETER (Debug)
											  status:ICBUILDTREE_STATUS_RUNNING];
		
		if (tProjectNodeData!=nil)
        {
			currentBuildNode_=[[ICBuildTreeNode alloc] initWithData:tProjectNodeData
															 parent:nil
														   children:[NSArray array]];
        
			if (currentBuildNode_!=nil)
			{
				[tree_ insertChild: currentBuildNode_
						   atIndex: 0];
					   
				[currentBuildNode_ release];
			}
		}
	}
	
	[IBoutlineView_ reloadData];
	
	// Disclose the content of the project tree
	
	if (currentBuildNode_!=nil)
	{
		[IBoutlineView_ expandItem:currentBuildNode_];
	}
}

- (void) cleanDidStart:(NSNotification *) inNotification
{
	if (tree_!=nil)
    {
        [tree_ release];
		
        tree_=nil;
    }
    
    tree_=[[ICBuildTreeNode buildTree] retain];
	
	if (tree_!=nil)
    {
		ICBuildNodeData * tProjectNodeData;
    
        tProjectNodeData=[ICBuildNodeData nodeOfType:ICBUILDTREE_TYPE_PROJECT
										   withTitle:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Clean %@",@"Build",@"No comment"),[document_ displayName]]
										    subTitle:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Project %@",@"Build",@"No comment"),[document_ displayName]]	// A COMPLETER (Debug)
											  status:ICBUILDTREE_STATUS_RUNNING];
		
		if (tProjectNodeData!=nil)
        {
			currentBuildNode_=[[ICBuildTreeNode alloc] initWithData:tProjectNodeData
															 parent:nil
														   children:[NSArray array]];
        
			if (currentBuildNode_!=nil)
			{
				[tree_ insertChild: currentBuildNode_
						   atIndex: 0];
					   
				[currentBuildNode_ release];
			}
		}
	}
	
	[IBoutlineView_ reloadData];
	
	// Disclose the content of the project tree
	
	if (currentBuildNode_!=nil)
	{
		[IBoutlineView_ expandItem:currentBuildNode_];
	}
}

- (void) builderNotification:(NSNotification *)notification
{
    NSDictionary * tUserInfo;
    static int sProcessID=-1;
    
    if (sProcessID==-1)
    {
        sProcessID=[[NSProcessInfo processInfo] processIdentifier];
    }
    
    tUserInfo=[notification userInfo];
    
    if (tUserInfo!=nil)
    {
        NSNumber * tNumber;
        
        tNumber=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_PID];
        
        if (tNumber!=nil)
        {
            if ([tNumber intValue]==sProcessID)
            {
                NSString * tPath;
                
                tPath=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH];
                
                if (tPath!=nil)
                {
                    if ([tPath isEqualToString:[document_ fileName]]==YES)
                    {
						int tEventInfoID;
						id tExtraData;
						//NSDictionary * tStackMetadata;
						int tStep;
						int tState;
							
						tEventInfoID=[[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_ID] intValue];
						
						tExtraData=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA];
						
						//tStackMetadata=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_INFORMATION_STACK_METADATA];
						
						// Build label status
						
						if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_BUILD_PROGRESSION)
						{
							tNumber=[tExtraData objectForKey:IC_BUILDER_INFORMATION_BUILD_PROGRESSION_PACKAGES_NUMBER];
							
							if (tNumber!=nil)
							{
								numberOfPackagesToBuild_=[tNumber unsignedLongValue];
							}
							else
							{
								NSLog(@"[ICBuildWindowController builderNotification:] Missing IC_BUILDER_INFORMATION_BUILD_PROGRESSION_PACKAGES_NUMBER data");
							}
						}
						else if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_STANDARD)
						{
							int tStep;
							int tState;
							
							tStep=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP] intValue];

							tState=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STATE] intValue];
							
							if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
							{
								switch( tStep)
								{
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION:
									
										indexOfPackagesBeingBuilt_=0;
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_PACKAGE:
								
										[IBstatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Building package...",@"BuildNotification",@"")];
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_CREATE:
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_REFERENCE:
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_IMPORT:
									
										if (numberOfPackagesToBuild_>0)
										{
											indexOfPackagesBeingBuilt_++;
										
											[IBstatusLabel_ setStringValue:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Building %ld of %ld packages...",@"BuildNotification",@""),indexOfPackagesBeingBuilt_,numberOfPackagesToBuild_]];
										}
										
										break;
								}
							}
							else if (tState==IC_BUILDER_BUILD_STATE_END_FAILURE)
							{
								if (building_==YES)
								{
									building_=NO;
								
									[IBstatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Build failed",@"BuildNotification",@"")];
								}
							}
							else
							{
								if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
								{
									if (tStep==IC_BUILDER_INFORMATION_STEP_PROJECT)
									{
										building_=NO;
										
										[IBstatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Build succeeded",@"BuildNotification",@"")];
									}
								}
							}
						}
						else if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN)
						{
							int tStep;
							int tState;
							
							tStep=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP] intValue];

							tState=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STATE] intValue];
							
							if (tStep==IC_BUILDER_INFORMATION_STEP_CLEAN)
							{
								if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
								{
									[IBstatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Clean succeeded",@"BuildNotification",@"")];
								}
								else if (tState==IC_BUILDER_BUILD_STATE_END_FAILURE)
								{
									[IBstatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Clean failed",@"BuildNotification",@"")];
								}
							}
							else if (tStep==IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT)
							{
							}
							else
							{
								NSLog(@"[ICBuildWindowController builderNotification:] Unknown Step");
							}
						}
						else if (tEventInfoID==IC_BUILDER_NOTIFICATION_REQUEST)
						{
							// Just Skip
							
							return;
						}
						else
						{
							NSLog(@"[ICBuildWindowController builderNotification:] Unknown Event Info ID");
						}
					
						// Build Log
						
						if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_STANDARD)
						{
							//NSString * tStepString=@"";
							ICBuildNodeData * tBuildNodeData=nil;
							ICBuildTreeNode * tBuildTreeNode;
							NSString * tString;
							
							tNumber=[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP];
							
							tStep=[tNumber intValue];
							
							//tStepString=[tNumber stringValue];

							tState=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STATE] intValue];
							
							if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
							{
								NSString * tTitle=@"";
								NSString * tSubTitle=@"";
								int tType=ICBUILDTREE_TYPE_STEP;
								
								switch(tStep)
								{
									case IC_BUILDER_INFORMATION_STEP_PROJECT:
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION:
									case IC_BUILDER_INFORMATION_STEP_PACKAGE:
									
										return;
									
									case IC_BUILDER_INFORMATION_STEP_PROJECT_BUILD_FOLDER:
									
										tTitle=NSLocalizedStringFromTable(@"Create build folder",@"Build",@"");
									
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_WELCOME_MESSAGE:
									
										tTitle=NSLocalizedStringFromTable(@"Copy introduction documents",@"Build",@"");
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_README_MESSAGE:
									
										tTitle=NSLocalizedStringFromTable(@"Copy readme documents",@"Build",@"");
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_LICENSE_DOCUMENT:
									
										tTitle=NSLocalizedStringFromTable(@"Copy license documents",@"Build",@"");
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_CONCLUSION_MESSAGE:
									
										tTitle=NSLocalizedStringFromTable(@"Copy conclusion documents",@"Build",@"");
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_BACKGROUND_IMAGE:
									
										tTitle=NSLocalizedStringFromTable(@"Copy background image",@"Build",@"");
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_SCRIPT:
									
										tType=ICBUILDTREE_TYPE_DISTRIBUTION_SCRIPT;
										
										tTitle=NSLocalizedStringFromTable(@"Create distribution script",@"Build",@"");
										
										tSubTitle=NSLocalizedStringFromTable(@"Distribution script and other resources",@"Build",@"");
										
										break;
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_CHOICES_HIERARCHIES:
										
										tTitle=NSLocalizedStringFromTable(@"Create hierarchy of choices",@"Build",@"");
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_INSTALLATION_VOLUME_REQUIREMENTS:
									
										tTitle=NSLocalizedStringFromTable(@"Create installation and volume requirements",@"Build",@"");
									
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_JAVASCRIPT:
									
										tTitle=NSLocalizedStringFromTable(@"Create JavaScript code",@"Build",@"");
									
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_INSTALLER_PLUGINS:
									
										tTitle=NSLocalizedStringFromTable(@"Copy Installer plugins",@"Build",@"");
									
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_RESOURCES:
									
										tTitle=NSLocalizedStringFromTable(@"Copy extra resources",@"Build",@"");
									
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_SCRIPTS:
									
										tType=ICBUILDTREE_TYPE_STEP_PARENT;
										
										tTitle=NSLocalizedStringFromTable(@"Scripts payload",@"Build",@"");
									
										break;
									
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_CREATE:
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_REFERENCE:
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_IMPORT:
									{
										NSString * tPackageName;
										
										NSMutableDictionary * tMutableDictionary;
										
										tMutableDictionary=[document_ dictionaryOfPackageWithUUID:[tExtraData objectForKey:IC_BUILDER_INFORMATION_PACKAGE_UUID]];
										
										tType=ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE;
										
										tPackageName=[tExtraData objectForKey:IC_BUILDER_INFORMATION_PACKAGE_NAME];
										
										if (tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE_IMPORT)
										{
											tSubTitle=[[tMutableDictionary objectForKey:ICDOCUMENT_PACKAGE_REFERENCE_PATH] objectForKey:ICDOCUMENT_PATH];
											
											tString=@"Import package";
										}
										else if (tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE_REFERENCE)
										{
											NSDictionary * tSettingsDictionary;
											
											tString=@"Create reference to package";
											
											tSettingsDictionary=[tMutableDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS];
											
											if (tSettingsDictionary!=nil)
											{
												tNumber=[tSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION];
					
												if (tNumber!=nil)
												{
													int tLocationType;
													NSString * tLocationPath;
													
													tLocationType=[tNumber intValue];
													
													tLocationPath=[tSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
													
													if (tLocationPath!=nil && [tLocationPath length]>0)
													{
														NSString * tURLPrefix=nil;
														
														if (tLocationType==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_HTTP_URL)
														{
															tURLPrefix=@"http://";
														}
														else if (tLocationType==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_REMOVABLE_MEDIA)
														{
															tURLPrefix=@"x-disc://";
														}
														
														if (tURLPrefix!=nil)
														{
															if ([tLocationPath hasPrefix:tURLPrefix]==YES)
															{
																tLocationPath=[tLocationPath substringFromIndex:[tURLPrefix length]];
															}
															
															if ([tLocationPath hasPrefix:@"/"]==YES)
															{
																tLocationPath=[tLocationPath substringFromIndex:1];
															}
																
															tLocationPath=[tLocationPath stringByAppendingPathComponent:tPackageName];
																
															if (tLocationPath!=nil)
															{
																tSubTitle=[NSString stringWithFormat:@"%@%@.pkg",tURLPrefix,tLocationPath];
															}
														}
													}
													else
													{
														tSubTitle=[NSString stringWithFormat:@"-://%@.pkg",tPackageName];
													}
												}
											}
										}
										else
										{
											tType=ICBUILDTREE_TYPE_DISTRIBUTION_PACKAGE_PROJECT;
											
											if ([document_ documentType]==ICDOCUMENT_TYPE_PACKAGE)
											{
												return;
											}
											
											tSubTitle=tPackageName;
											
											tString=@"Create package";
										}
																				
										tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(tString,@"Build",@""),tPackageName];
										
										break;
									}
									
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_PAYLOAD:
									
										tType=ICBUILDTREE_TYPE_STEP_PARENT;
										
										tTitle=NSLocalizedStringFromTable(@"Create payload",@"Build",@"");
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_SCRIPTS_PAYLOAD:
									
										tType=ICBUILDTREE_TYPE_STEP_PARENT;
										
										tTitle=NSLocalizedStringFromTable(@"Scripts payload",@"Build",@"");
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_INFO:
									
										tTitle=NSLocalizedStringFromTable(@"Create PackageInfo document",@"Build",@"");
									
										break;
										
										
									case IC_BUILDER_INFORMATION_STEP_PAYLOAD_ASSEMBLE:
									
										tTitle=NSLocalizedStringFromTable(@"Assemble payload files",@"Build",@"");
									
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PAYLOAD_SPLIT:
									
										tTitle=NSLocalizedStringFromTable(@"Split forks",@"Build",@"");
									
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PAYLOAD_BOM:
									
										tTitle=NSLocalizedStringFromTable(@"Create bill of materials",@"Build",@"");
									
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PAYLOAD_PAX:
									
										tTitle=NSLocalizedStringFromTable(@"Create pax archive",@"Build",@"");
									
										break;
								}
								
								tBuildNodeData=[ICBuildNodeData nodeOfType:tType
																 withTitle:tTitle
																  subTitle:tSubTitle
																	status:ICBUILDTREE_STATUS_RUNNING];
								
								if (tBuildNodeData!=nil)
								{
									tBuildTreeNode=[[ICBuildTreeNode alloc] initWithData:tBuildNodeData
																				  parent:nil
																				children:[NSArray array]];
									
									if (tBuildTreeNode!=nil)
									{
										[currentBuildNode_ insertChild: tBuildTreeNode
															   atIndex: [currentBuildNode_ numberOfChildren]];
															   
										currentBuildNode_=tBuildTreeNode;
										
										[IBoutlineView_ reloadData];
										
										// Disclose the content of the project tree
	
										if ([ICBUILDNODE_DATA(currentBuildNode_) isLeaf]==NO)
										{
											[IBoutlineView_ expandItem:currentBuildNode_];
										}
									}
								}
								
								
							}
							else if (tState==IC_BUILDER_BUILD_STATE_END_FAILURE)
							{
								ICBuildNodeData * tBuildNodeData=nil;
								
								if (buildFailed_==NO)
								{
									NSString * tTitle=@"";
									
									buildFailed_=YES;
									
									if (tExtraData!=nil)
									{
										tNumber=[tExtraData objectForKey:IC_BUILDER_FAILURE_REASON];
									
										if (tNumber!=nil)
										{
											NSUInteger tFailureReason;
											
											tFailureReason=[tNumber unsignedLongValue];
										
											if (tFailureReason==IC_BUILDER_FAILURE_REASON_UNKNOWN)
											{
											}
											else if (tFailureReason==IC_BUILDER_FAILURE_REASON_OUT_OF_MEMORY)
											{
												tTitle=NSLocalizedStringFromTable(@"Not enough memory to perform operation",@"Build",@"No comment");
											}
											else
											{
												NSString * tTag;
												NSString * tFilePath;
												BOOL isFolder;
												BOOL isTool;
												
												tTag=[tExtraData objectForKey:IC_BUILDER_FAILURE_TAG_NAME];
												
												tFilePath=[tExtraData objectForKey:IC_BUILDER_INFORMATION_FILE_PATH];
												
												isFolder=[[tExtraData objectForKey:IC_BUILDER_FAILURE_FILE_IS_FOLDER] boolValue];
												
												isTool=[[tExtraData objectForKey:IC_BUILDER_FAILURE_FILE_IS_TOOL] boolValue];
												
												switch(tFailureReason)
												{
													case IC_BUILDER_FAILURE_REASON_MISSING_INFORMATION:
														
														tTitle=nil;
														
														if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH]==YES)
														{
															tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"The location of the referenced package has not been fully defined.",@"Build",@""),tTag];
														}
														
														if (tTitle==nil)
														{
															tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Missing information for tag '%@'",@"Build",@""),tTag];
														}
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_MISSING_BUILD_DATA:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Missing build data for tag '%@'",@"Build",@""),tTag];
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_INCORRECT_VALUE:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Incorrect value for object with tag '%@'",@"Build",@""),tTag];
														
														break;
													
													case IC_BUILDER_FAILURE_REASON_FILE_INCORRECT_TYPE:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Incorrect type for file at path \"%@\"",@"Build",@""),tFilePath];
														
														break;
													
													case IC_BUILDER_FAILURE_REASON_FILE_INSUFFICIENT_PERMISSIONS_WRITE:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Insufficient privileges to write at path \"%@\"",@"Build",@""),tFilePath];
													
														break;
													
													case IC_BUILDER_FAILURE_REASON_FILE_INSUFFICIENT_PERMISSIONS_SET:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Insufficient privileges to set permissions for path \"%@\"",@"Build",@""),tFilePath];
													
														break;
													
													case IC_BUILDER_FAILURE_REASON_OPERATION_FAILED:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"%@ returned error code '%d'",@"Build",@""),[tFilePath lastPathComponent],[[tExtraData objectForKey:IC_BUILDER_FAILURE_TOOL_RETURN_CODE] intValue]];
														
														break;
													
													case IC_BUILDER_FAILURE_REASON_SPLIT_NON_HFS_VOLUME:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"The Temporary Build Path is on a non HFS volume",@"Build",@""),tTag];
													
														break;
													
													case IC_BUILDER_FAILURE_REASON_LICENSE_MISSING_TEMPLATE:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"License template for \"%@\" can not be found",@"Build",@""),tTag];
														
														break;
													
													case IC_BUILDER_FAILURE_REASON_PACKAGE_SAME_NAME:
														
														// A COMPLETER
														
														break;
													
													case IC_BUILDER_FAILURE_REASON_STRING_EMPTY:
													
														tTitle=nil;
														
														if (tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE_INFO)
														{
															if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER]==YES)
															{
																tTitle=NSLocalizedStringFromTable(@"The identifier of the package can not be empty.",@"Build",@"");
															}
															else if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_VERSION]==YES)
															{
																tTitle=NSLocalizedStringFromTable(@"The version of the package can not be empty.",@"Build",@"");
															}
														}
														else if (tStep==IC_BUILDER_INFORMATION_STEP_DISTRIBUTION)
														{
															if ([tTag isEqualToString:ICDOCUMENT_PROJECT_SETTINGS_NAME]==YES)
															{
																tTitle=NSLocalizedStringFromTable(@"The name of the project can not be empty.",@"Build",@"");
															}
															
														}
														else if (tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE_CREATE)
														{
															if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_NAME]==YES)
															{
																tTitle=NSLocalizedStringFromTable(@"The name of the package can not be empty.",@"Build",@"");
															}
														}
														else
														{
															if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH]==YES)
															{
																tTitle=NSLocalizedStringFromTable(@"The location of the referenced package has not been fully defined.",@"Build",@"");
															}
														}
														
														if (tTitle==nil)
														{
															tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"String can not be empty for tag '%@'",@"Build",@""),tTag];
														}
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_REQUIREMENT_MISSING_CONVERTER:
														
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Converter not found for requirement of type '%@'",@"Build",@""),tTag];
														
														break;
													
													case IC_BUILDER_FAILURE_REASON_REQUIREMENT_MISSING_CODE:
														
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"No code generated for requirement of type '%@'",@"Build",@""),tTag];
														
														break;
													
													case IC_BUILDER_FAILURE_REASON_LOCATOR_MISSING_CONVERTER:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Converter not found for locator of type '%@'",@"Build",@""),tTag];
														
														break;
															
													case IC_BUILDER_FAILURE_REASON_LOCATOR_CONVERSION_ERROR:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"No code generated for locator of type '%@'",@"Build",@""),tTag];
														
														//IC_BUILDER_FAILURE_SECONDARY_REASON
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_FOUND:
													
														tTitle=nil;
														
														if (tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE_IMPORT)
														{
															if (tTag!=nil)
															{
																if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_REFERENCE_PATH]==YES)
																{
																	tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Unable to find package at path '%@'",@"Build",@""),tFilePath];
																}
															}
														}
														
														if (tTitle==nil)
														{
															if (isTool==YES)
															{
																tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Unable to find tool '%@'",@"Build",@""),[tFilePath lastPathComponent]];
															}
															else if (isFolder==YES)
															{
																tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Unable to find folder at path '%@'",@"Build",@""),tFilePath];
															}
															else
															{
																tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Unable to find file at path '%@'",@"Build",@""),tFilePath];
															}
														}
														
														break;
													
													case IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_CREATED:
													
														if (isFolder==YES)
														{
															tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Unable to create folder at path '%@'",@"Build",@""),tFilePath];
														}
														else
														{
															tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Unable to create file at path '%@'",@"Build",@""),tFilePath];
														}
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_COPIED:
													
														tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Unable to copy item at path '%@'",@"Build",@""),tFilePath];
														
														tNumber=[tExtraData objectForKey:IC_BUILDER_FAILURE_SECONDARY_REASON];
														
														if (tNumber!=nil)
														{
															if ([tNumber unsignedLongValue]==IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_FOUND)
															{
																tTitle=[tTitle stringByAppendingString:NSLocalizedStringFromTable(@" because the item could not be found",@"Build",@"")];
															}
														}
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_DELETED:
													
														if (isFolder==YES)
														{
															tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Unable to delete folder at path '%@'",@"Build",@""),tFilePath];
														}
														else
														{
															tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Unable to delete file at path '%@'",@"Build",@""),tFilePath];
														}
														
														break;
													
													case IC_BUILDER_FAILURE_REASON_SIGNATURE_GENERIC_ERROR:
														
														tTitle=NSLocalizedStringFromTable(@"Unable to sign the data",@"Build",@"");
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_SIGNATURE_TIME_OUT:
														
														tTitle=NSLocalizedStringFromTable(@"Signing operation timed out",@"Build",@"");
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_SIGNATURE_AUTH_DENIED:
													
														tTitle=NSLocalizedStringFromTable(@"Signing operation denied",@"Build",@"");
													
														break;
														
													case IC_BUILDER_FAILURE_REASON_SIGNATURE_CERTIFICATE_NOT_FOUND:
													
														tTitle=NSLocalizedStringFromTable(@"Unable to find signing certificate",@"Build",@"");
														
														break;
												}
											}
										}
									}
								
									tBuildNodeData=[ICBuildNodeData nodeOfType:ICBUILDTREE_TYPE_ERROR_DESCRIPTION
																		withTitle:tTitle
																		 subTitle:nil
																		   status:ICBUILDTREE_STATUS_FAILURE];
									
									if (tBuildNodeData!=nil)
									{
										ICBuildTreeNode * tBuildTreeNode;
										
										tBuildTreeNode=[[ICBuildTreeNode alloc] initWithData:tBuildNodeData
																					  parent:nil
																				    children:[NSArray array]];
									
										if (tBuildTreeNode!=nil)
										{
											NSString * tConclusionTitle=nil;
											NSString * tConclusionSubTitle=nil;
										
											[currentBuildNode_ insertChild: tBuildTreeNode
																   atIndex: [currentBuildNode_ numberOfChildren]];
												   
											[tBuildTreeNode release];
											
											// Set the status to failure for the parents hierarchy
											
											tBuildTreeNode=currentBuildNode_;
											
											tBuildNodeData=ICBUILDNODE_DATA(tBuildTreeNode);
											
											while ([tBuildNodeData type]!=ICBUILDTREE_TYPE_ROOT)
											{
												[tBuildNodeData setStatus:ICBUILDTREE_STATUS_FAILURE];
												
												tBuildTreeNode=(ICBuildTreeNode *) [tBuildTreeNode nodeParent];
												
												tBuildNodeData=ICBUILDNODE_DATA(tBuildTreeNode);
											}
											
											tConclusionTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Build Failed \t%@",@"Build",@"No comment"),[dateFormatter_ stringForObjectValue:[NSDate date]]];
										
											tConclusionSubTitle=NSLocalizedStringFromTable(@"1 error",@"Build",@"No comment");
											
											tBuildNodeData=[ICBuildNodeData nodeOfType:ICBUILDTREE_TYPE_CONCLUSION
																			   withTitle:tConclusionTitle
																				subTitle:tConclusionSubTitle
																				  status:ICBUILDTREE_STATUS_FAILURE];
																				  
											if (tBuildNodeData!=nil)
											{
												ICBuildTreeNode * tBuildTreeNode;
												
												tBuildTreeNode=[[ICBuildTreeNode alloc] initWithData:tBuildNodeData
																							  parent:nil
																							children:[NSArray array]];
											
												if (tBuildTreeNode!=nil)
												{
													[tree_ insertChild: tBuildTreeNode
															   atIndex: [tree_ numberOfChildren]];
														   
													[tBuildTreeNode release];
												}
											}
									
											[IBoutlineView_ reloadData];
											
											[IBoutlineView_ expandItem:currentBuildNode_];
										}
									}
									
									buildFailed_=YES;
								}
							}
							else if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
							{
								ICBuildNodeData * tBuildNodeData=nil;
								
								if (tStep==IC_BUILDER_INFORMATION_STEP_PROJECT)
								{
									NSString * tTitle;
									
									tBuildNodeData=ICBUILDNODE_DATA(currentBuildNode_);
									
									[tBuildNodeData setStatus:ICBUILDTREE_STATUS_SUCCESS];
									
									tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Build Succeeded \t%@",@"Build",@"No comment"),[dateFormatter_ stringForObjectValue:[NSDate date]]];
									
									tBuildNodeData=[ICBuildNodeData nodeOfType:ICBUILDTREE_TYPE_CONCLUSION
																    withTitle:tTitle
																	 subTitle:NSLocalizedStringFromTable(@"No issues",@"Build",@"No comment")
																	   status:ICBUILDTREE_STATUS_SUCCESS];
								
									if (tBuildNodeData!=nil)
									{
										ICBuildTreeNode * tBuildTreeNode;
										
										tBuildTreeNode=[[ICBuildTreeNode alloc] initWithData:tBuildNodeData
																 parent:nil
															   children:[NSArray array]];
									
										if (tBuildTreeNode!=nil)
										{
											[tree_ insertChild: tBuildTreeNode
													   atIndex: [tree_ numberOfChildren]];
												   
											[tBuildTreeNode release];
											
											[IBoutlineView_ reloadData];
											
											[IBoutlineView_ collapseItem:[tree_ childAtIndex:0]];
										}
									}
								}
								else
								{
									if (tStep==IC_BUILDER_INFORMATION_STEP_DISTRIBUTION ||
										tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE ||
										(tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE_CREATE && 
											[document_ documentType]==ICDOCUMENT_TYPE_PACKAGE))
									{
										return;
									}
									
									tBuildNodeData=ICBUILDNODE_DATA(currentBuildNode_);
									
									[tBuildNodeData setStatus:ICBUILDTREE_STATUS_SUCCESS];
									
									[IBoutlineView_ reloadData];
									
									if ([ICBUILDNODE_DATA(currentBuildNode_) isLeaf]==NO)
									{
										[IBoutlineView_ collapseItem:currentBuildNode_];
									}
									
									currentBuildNode_=(ICBuildTreeNode *) [currentBuildNode_ nodeParent];
								}
							}
						}
						else if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN)
						{
							tStep=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP] intValue];

							tState=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STATE] intValue];
							
							if (tStep==IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT)
							{
								ICBuildNodeData * tProjectNodeData=nil;
								ICBuildNodeData * tBuildNodeData=nil;
								NSString * tTitle;
								ICBuildTreeNode * tBuildTreeNode;
								
								if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
								{
									tTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Clean %@",@"Build",@""),[tExtraData objectForKey:IC_BUILDER_INFORMATION_FILE_PATH]];
									
									tProjectNodeData=[ICBuildNodeData nodeOfType:ICBUILDTREE_TYPE_STEP
																	   withTitle:tTitle
																		subTitle:nil
																		  status:ICBUILDTREE_STATUS_RUNNING];
								}
								else if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
								{
									tBuildNodeData=ICBUILDNODE_DATA(currentBuildNode_);
									
									[tBuildNodeData setStatus:ICBUILDTREE_STATUS_SUCCESS];
									
									currentBuildNode_=(ICBuildTreeNode *) [currentBuildNode_ nodeParent];
								}
								else if (tState==IC_BUILDER_BUILD_STATE_END_FAILURE)
								{
									tBuildNodeData=ICBUILDNODE_DATA(currentBuildNode_);
									
									[tBuildNodeData setStatus:ICBUILDTREE_STATUS_FAILURE];
									
									tProjectNodeData=[ICBuildNodeData nodeOfType:ICBUILDTREE_TYPE_ERROR_DESCRIPTION
																	   withTitle:NSLocalizedStringFromTable([tExtraData objectForKey:IC_BUILDER_FAILURE_REASON],@"Build",@"")
																		subTitle:nil
																		  status:ICBUILDTREE_STATUS_FAILURE];
								}
								
								if (tProjectNodeData!=nil)
								{
									
									
									tBuildTreeNode=[[ICBuildTreeNode alloc] initWithData:tProjectNodeData
															 parent:nil
														   children:[NSArray array]];
								
									if (tBuildTreeNode!=nil)
									{
										[currentBuildNode_ insertChild: tBuildTreeNode
															   atIndex: [currentBuildNode_ numberOfChildren]];
											   
										[IBoutlineView_ reloadData];
										
										// Disclose the content of the project tree
	
										if ([ICBUILDNODE_DATA(currentBuildNode_) isLeaf]==NO)
										{
											[IBoutlineView_ expandItem:currentBuildNode_];
										}
										
										if (tState!=IC_BUILDER_BUILD_STATE_END_FAILURE)
										{
											currentBuildNode_=tBuildTreeNode;
										}
										else
										{
											currentBuildNode_=(ICBuildTreeNode *) [currentBuildNode_ nodeParent];
										}
										
										[tBuildTreeNode release];
									}
								}
								else
								{
									[IBoutlineView_ reloadData];
								}
							}
							else if (tStep==IC_BUILDER_INFORMATION_STEP_CLEAN)
							{
								ICBuildNodeData * tProjectNodeData=nil;
								NSString * tConclusionTitle;
								
								if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
								{
									tConclusionTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Clean Succeeded \t%@",@"Build",@"No comment"),[dateFormatter_ stringForObjectValue:[NSDate date]]];
									
									tProjectNodeData=[ICBuildNodeData nodeOfType:ICBUILDTREE_TYPE_CONCLUSION
																	   withTitle:tConclusionTitle
																		subTitle:NSLocalizedStringFromTable(@"No issues",@"Build",@"No comment")
																		  status:ICBUILDTREE_STATUS_SUCCESS];
								}
								else if (tState==IC_BUILDER_BUILD_STATE_END_FAILURE)
								{
									NSInteger tErrorCount;
									NSString * tSubTitle=nil;
									
									tConclusionTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Clean Failed \t%@",@"Build",@"No comment"),[dateFormatter_ stringForObjectValue:[NSDate date]]];
									
									tErrorCount=[tExtraData unsignedLongValue];
									
									if (tErrorCount==1)
									{
										tSubTitle=NSLocalizedStringFromTable(@"1 error",@"Build",@"No comment");
									}
									else
									{
										tSubTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"%ld errors",@"Build",@"No comment"),tErrorCount];
									}
									
									tProjectNodeData=[ICBuildNodeData nodeOfType:ICBUILDTREE_TYPE_CONCLUSION
																	   withTitle:tConclusionTitle
																		subTitle:tSubTitle
																		  status:ICBUILDTREE_STATUS_FAILURE];
								}
								
								if (tProjectNodeData!=nil)
								{
									ICBuildTreeNode * tBuildTreeNode;
									
									tBuildTreeNode=[[ICBuildTreeNode alloc] initWithData:tProjectNodeData
															 parent:nil
														   children:[NSArray array]];
								
									if (tBuildTreeNode!=nil)
									{
										[tree_ insertChild: tBuildTreeNode
												   atIndex: [tree_ numberOfChildren]];
											   
										[tBuildTreeNode release];
										
										[IBoutlineView_ reloadData];
										
										currentBuildNode_=tBuildTreeNode;
									
										if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
										{
											[IBoutlineView_ collapseItem:[tree_ childAtIndex:0]];
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
}

@end
