/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICCertificatePanel.h"

#import <SecurityInterface/SFCertificateView.h>

@implementation ICCertificatePanel

+ (ICCertificatePanel *) certificatePanel
{
	ICCertificatePanel * tCertificatePanel;
	
	tCertificatePanel=[ICCertificatePanel alloc];
	
	if (tCertificatePanel!=nil)
	{
		if ([NSBundle loadNibNamed:@"CertificatePanel" owner:tCertificatePanel]==NO)
		{
			NSBeep();
			
			NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"CertificatePanel"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
		}
	}
	else
	{
		// A COMPLETER
	}
	
	return [tCertificatePanel autorelease];
}

- (void) dealloc
{
	// A COMPLETER

	[super dealloc];
}

- (void) awakeFromNib
{
	[IBcertificateView_ setDisplayTrust:NO];
	
	[IBcertificateView_ setEditableTrust:NO];
	
	[IBcertificateView_ setDisplayDetails:YES]; 
}

#pragma mark -

- (void) beginSheetForCertificate:(SecCertificateRef) inCertificate modalForWindow:(NSWindow *) inWindow modalDelegate:(id) inDelegate didEndSelector:(SEL) inSelector contextInfo:(void *) inContextInfo
{
	[self retain];
	
	[IBcertificateView_ setCertificate:inCertificate];
	
	delegate=inDelegate;
	
	endSelector_=inSelector;
	
	[NSApp beginSheet:IBwindow_ 
	   modalForWindow:inWindow 
		modalDelegate:self 
	   didEndSelector:@selector(sheetPanelDidEnd:returnCode:contextInfo:)
	      contextInfo:inContextInfo];
}

- (void) sheetPanelDidEnd:(NSWindow *) inSheet returnCode:(NSInteger) inReturnCode contextInfo:(void *) inContextInfo
{
	if (delegate!=nil)
	{
		NSMethodSignature * tSignature;
	
		tSignature=[[delegate class] instanceMethodSignatureForSelector:endSelector_];
		
		if (tSignature!=nil)
		{
			NSInvocation * tInvocation;
		
			tInvocation=[NSInvocation invocationWithMethodSignature:tSignature];
			
			if (tInvocation!=nil)
			{
				[tInvocation setTarget:delegate];

				[tInvocation setSelector:endSelector_];
				
				[tInvocation setArgument:&IBwindow_ atIndex:2];
				
				[tInvocation setArgument:&inReturnCode atIndex:3];
				
				[tInvocation setArgument:&inContextInfo atIndex:4];
				
				[tInvocation retainArguments];
				
				[tInvocation invoke];
			}
		}
	}
	
	[IBwindow_ orderOut:self];
	
	[self autorelease];
}

- (IBAction) ok:(id) sender
{
	[NSApp endSheet:IBwindow_ returnCode:NSOKButton];
}

@end
