/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* Contains code from Apple Sample Code */

#import "ICCertificateStampView.h"

#define OUTER_RADIUS	36.0f

#define TOOTH_HEIGHT	3.5f

#define TEETH_COUNT		40

#define DEG_TO_RAD		((2*3.141592654)/360.0f)

#define ALPHA	0.29f

@implementation ICCertificateStampView

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
	
    if (self)
	{
        outerRadius_=OUTER_RADIUS;
		
		toothHeight_=TOOTH_HEIGHT;
		
		teethCount_=TEETH_COUNT;
		
		// Initialization code here.
		
		
		textStorage_ = [[NSTextStorage alloc] initWithString:@""];
		
		layoutManager_ = [[NSLayoutManager alloc] init];
		
		textContainer_ = [[NSTextContainer alloc] init];
		
		[layoutManager_ addTextContainer:textContainer_];
		
		[textContainer_ release];
		
		[textStorage_ addLayoutManager:layoutManager_];
		
		[layoutManager_ release];


		[layoutManager_ setUsesScreenFonts:NO];
		
		[self setStringValue:@"CERTIFICATE OF AUTHENTICITY"];
    }
	
    return self;
}

#pragma mark -

- (BOOL) isFlipped
{
	return NO;
}

- (BOOL) isOpaque
{
	return NO;
}

- (IBAction) takeOuterRadiusFrom:(id) sender
{
	outerRadius_=[sender floatValue];
	
	[self setNeedsDisplay:YES];
}

- (IBAction) takeToothHeightFrom:(id) sender
{
	toothHeight_=[sender floatValue];
	
	[self setNeedsDisplay:YES];
}

- (IBAction) takeTeethCountFrom:(id) sender
{
	teethCount_=[sender intValue];
	
	[self setNeedsDisplay:YES];
}

#pragma mark -

- (void) setIsMissing:(BOOL) isMissing
{
	if (isMissing!=isMissing_)
	{
		isMissing_=isMissing;
		
		[self setNeedsDisplay:YES];
	}
}

- (void) setStringValue:(NSString *) inString
{
	[textStorage_ replaceCharactersInRange:NSMakeRange(0, [textStorage_ length]) withString:inString];
	
	[textStorage_ addAttribute:NSFontAttributeName value:[NSFont boldSystemFontOfSize:7.0f] range:NSMakeRange(0, [textStorage_ length])];
	
    [self setNeedsDisplay:YES];
}

- (void) drawRect:(NSRect) inRect
{
    NSBezierPath * tBezierPath;
	NSRect tBounds;
	CGFloat tMidX,tMidY;
	CGFloat tCircleRadius;
	CGFloat tInnerRadius;
	CGFloat tFactor;
	
	tBounds=[self bounds];
	
	tMidX=NSMidX(tBounds);
		
	tMidY=NSMidY(tBounds);
	
	tInnerRadius=outerRadius_-toothHeight_;
	
	tBezierPath=[NSBezierPath bezierPath];
	
	if (pushed_==NO)
	{
		tFactor=1.0f;//ALPHA;
	}
	else
	{
		tFactor=0.5f;
	}
	
	[textStorage_ addAttribute:NSForegroundColorAttributeName value:[NSColor colorWithDeviceWhite:1.0f alpha:1.0f]  range:NSMakeRange(0, [textStorage_ length])];
	
	if (tBezierPath!=nil)
	{
		NSUInteger i;
		CGFloat tAngle=0.0f;
		CGFloat tDeltaAngle;
		CGFloat tSemiDeltaAngle;
		
		// Draw the edge
		
		tDeltaAngle=360.0f/teethCount_;
		
		tSemiDeltaAngle=tDeltaAngle*0.5f;
		
		
		
		[tBezierPath moveToPoint:NSMakePoint(tMidX+tInnerRadius*cosf(tAngle*DEG_TO_RAD),tMidY+tInnerRadius*sinf(tAngle*DEG_TO_RAD))];
		
		for(i=0;i<teethCount_;i++)
		{
			[tBezierPath lineToPoint:NSMakePoint(tMidX+outerRadius_*cosf((tAngle+tSemiDeltaAngle)*DEG_TO_RAD),tMidY+outerRadius_*sinf((tAngle+tSemiDeltaAngle)*DEG_TO_RAD))];
			
			tAngle+=tDeltaAngle;
			
			[tBezierPath lineToPoint:NSMakePoint(tMidX+tInnerRadius*cosf(tAngle*DEG_TO_RAD),tMidY+tInnerRadius*sinf(tAngle*DEG_TO_RAD))];
		}
		
		[tBezierPath closePath];
		
		if (isMissing_==NO)
		{
			[[NSColor colorWithDeviceWhite:0.485f*tFactor alpha:1.0f] set];
		}
		else
		{
			[[NSColor colorWithDeviceRed:202.0/255.0 green:46.0f/255.0f blue:50.0f/255.0f alpha:1.0f] set];
		}
		
		[tBezierPath fill];
		
		if (isMissing_==NO)
		{
			[[NSColor colorWithDeviceWhite:0.325f*tFactor alpha:1.0f] set];
		}
		else
		{
			[[NSColor colorWithDeviceRed:137.0/255.0 green:46.0f/255.0f blue:50.0f/255.0f alpha:1.0f] set];
		}
		
		[tBezierPath stroke];
		
		[tBezierPath removeAllPoints];
	}
	
	tCircleRadius=tInnerRadius*0.95f;
	
	tBezierPath=[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(tMidX-tCircleRadius,tMidY-tCircleRadius,tCircleRadius*2.0f,tCircleRadius*2.0f)];
	
	if (tBezierPath!=nil)
	{
#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_5
		NSGradient * tGradient=nil;
		
		if (isMissing_==NO)
		{
			tGradient=[[NSGradient alloc] initWithStartingColor:[NSColor colorWithDeviceWhite:0.7f*tFactor alpha:1.0f] endingColor:[NSColor colorWithDeviceWhite:0.1f*tFactor alpha:1.0f]];
		}
		else
		{
			// A COMPLETER
		}
		
		[tGradient drawInBezierPath:tBezierPath angle:-90.0f];
		
		[tGradient release];
		
#else
		[tBezierPath fill];
#endif

		[tBezierPath removeAllPoints];
	}
	
	if (isMissing_==NO)
	{
		[[NSColor colorWithDeviceWhite:0.425f*tFactor alpha:1.0f] set];
	}
	else
	{
		[[NSColor colorWithDeviceRed:110.0/255.0 green:46.0f/255.0f blue:50.0f/255.0f alpha:1.0f] set];
	}
	
	tCircleRadius=tInnerRadius*0.89f;
	
	tBezierPath=[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(tMidX-tCircleRadius,tMidY-tCircleRadius,tCircleRadius*2.0f,tCircleRadius*2.0f)];
	
	if (tBezierPath!=nil)
	{
		[tBezierPath setLineWidth:2.0f];
		
		[tBezierPath stroke];
	}
	
	// Draw the text (code from some Apple Sample code)
	
	NSUInteger tGlyphIndex;
    NSRange tGlyphRange;
    NSRect usedRect;
	CGFloat startingAngle;
	
	startingAngle=-M_PI_2;

	tCircleRadius=tInnerRadius*0.58;

    // Note that usedRectForTextContainer: does not force layout, so it must 
    // be called after glyphRangeForTextContainer:, which does force layout.
	
    tGlyphRange = [layoutManager_ glyphRangeForTextContainer:textContainer_];
	
    usedRect = [layoutManager_ usedRectForTextContainer:textContainer_];

    for (tGlyphIndex = tGlyphRange.location; tGlyphIndex < NSMaxRange(tGlyphRange); tGlyphIndex++)
	{
        NSGraphicsContext *context = [NSGraphicsContext currentContext];
		NSRect lineFragmentRect = [layoutManager_ lineFragmentRectForGlyphAtIndex:tGlyphIndex effectiveRange:NULL];
		NSPoint viewLocation, layoutLocation = [layoutManager_ locationForGlyphAtIndex:tGlyphIndex];
        CGFloat angle, distance;
        NSAffineTransform * transform = [NSAffineTransform transform];
    
        // Here layoutLocation is the location (in container coordinates) where the glyph was laid out. 
        layoutLocation.x += lineFragmentRect.origin.x;
        layoutLocation.y += lineFragmentRect.origin.y;

        // We then use the layoutLocation to calculate an appropriate position for the glyph 
        // around the circle (by angle and distance, or viewLocation in rectangular coordinates).
        distance = tCircleRadius + usedRect.size.height - layoutLocation.y;
        angle = startingAngle + layoutLocation.x / distance;

        viewLocation.x = tMidX + distance * sin(angle);
        viewLocation.y = tMidY + distance * cos(angle);
        
        // We use a different affine transform for each glyph, to position and rotate it
        // based on its calculated position around the circle.  
        [transform translateXBy:viewLocation.x yBy:viewLocation.y];
        [transform rotateByRadians:-angle];

        // We save and restore the graphics state so that the transform applies only to this glyph.
        [context saveGraphicsState];
        [transform concat];
        // drawGlyphsForGlyphRange: draws the glyph at its laid-out location in container coordinates.
        // Since we are using the transform to place the glyph, we subtract the laid-out location here.
        [layoutManager_ drawGlyphsForGlyphRange:NSMakeRange(tGlyphIndex, 1) atPoint:NSMakePoint(-layoutLocation.x, -layoutLocation.y)];
        [context restoreGraphicsState];
    }
	
	tCircleRadius=tInnerRadius*0.54f;
	
	tBezierPath=[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(tMidX-tCircleRadius,tMidY-tCircleRadius,tCircleRadius*2.0f,tCircleRadius*2.0f)];
	
	if (tBezierPath!=nil)
	{
#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_5
		NSGradient * tGradient=nil;
		
		if (isMissing_==NO)
		{
			tGradient=[[NSGradient alloc] initWithStartingColor:[NSColor colorWithDeviceWhite:0.7f*tFactor alpha:1.0f] endingColor:[NSColor colorWithDeviceWhite:0.1f*tFactor alpha:1.0f]];
		}
		else
		{
			// A COMPLETER
		}
		
		[tGradient drawInBezierPath:tBezierPath angle:-90.0f];
		
		[tGradient release];
#else

		[tBezierPath fill];

#endif		
		[tBezierPath removeAllPoints];
	}
}

- (void) mouseDown:(NSEvent *) inEvent
{
	if ([self isEnabled]==YES)
	{
		NSRect tBounds;
		NSBezierPath * tBezierPath;
		NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
		
		tBounds=[self bounds];
		
		tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tBounds];
		
		if ([tBezierPath containsPoint:tMouseLoc]==YES)
		{
			pushed_=YES;
				
			[self setNeedsDisplay:YES];
		}
	}
}

- (void) mouseDragged:(NSEvent *) inEvent
{
    if ([self isEnabled]==YES)
	{
		NSRect tBounds;
		NSBezierPath * tBezierPath;
		NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
		
		tBounds=[self bounds];
		
		tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tBounds];
		
		if ([tBezierPath containsPoint:tMouseLoc]==YES)
		{
			if (pushed_==NO)
			{
				pushed_=YES;
				
				[self setNeedsDisplay:YES];
			}
		}
		else
		{
			if (pushed_==YES)
			{
				pushed_=NO;
				
				[self setNeedsDisplay:YES];
			}
		}
	}
}

- (void) mouseUp:(NSEvent *) inEvent
{
    if ([self isEnabled]==YES)
	{
		NSRect tBounds;
		NSBezierPath * tBezierPath;
		NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
		
		pushed_=NO;
		
		tBounds=[self bounds];
		
		tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tBounds];
		
		if ([tBezierPath containsPoint:tMouseLoc]==YES)
		{
			[self sendAction:[self action] to:[self target]];
			
			[self setNeedsDisplay:YES];
		}
	}
}

@end
