/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICChoicePopUpButton.h"

#import "ICInstallationType+Constants.h"

#import "NSBezierPath+RoundedRect.h"

@implementation NSObject (ICChoicePopUpButton)

- (BOOL) popUpButton:(id) inPopUpButton canSelectChoice:(NSString *) inUUID
{
	return NO;
}

- (void) selectItemOfPopUpButton:(id) inPopUpButton forChoice:(NSString *) inUUID
{
}

@end

@implementation ICChoicePopUpButton

- (void) dealloc
{
	[delegate release];

	[super dealloc];
}

#pragma mark -

- (void) setDelegate:(id) inDelegate
{
	if (delegate!=inDelegate)
	{
		[delegate release];
		
		delegate=[inDelegate retain];
		
		if (delegate!=nil)
		{
			// Register for Drop
		
			[self unregisterDraggedTypes];
		
			[self registerForDraggedTypes:[NSArray arrayWithObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_UUID_PASTEBOARDTYPE]];
		}
	}
}

- (void)drawRect:(NSRect) inFrame
{
	[super drawRect:inFrame];
	
	if (highlighted_==YES)
	{
		NSBezierPath * tPath;
		
		tPath=[NSBezierPath bezierPathWithRoundedRect:NSInsetRect([self bounds],2.0f,2.0f) radius:8.0f];
		
		if (tPath!=nil)
		{
			[tPath setLineWidth:3.0f];
			
			[[NSColor alternateSelectedControlColor] set];
			
			[tPath stroke];
		}
	}
}

- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>)sender
{
    if (delegate!=nil)
	{
		NSPasteboard * tPasteBoard;
		
		tPasteBoard = [sender draggingPasteboard];

		if ( [[tPasteBoard types] containsObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_UUID_PASTEBOARDTYPE] )
		{
			NSArray * tItems = [tPasteBoard propertyListForType:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_UUID_PASTEBOARDTYPE];
			
			if (tItems!=nil && [tItems count]==1)
			{
				if ([delegate popUpButton:self canSelectChoice:[tItems objectAtIndex:0]])
				{
					highlighted_=YES;
			
					[self setNeedsDisplay:YES];
					
					return NSDragOperationLink;
				}
			}
        }
    }
    
    return NSDragOperationNone;
}

- (BOOL) wantsPeriodicDraggingUpdates
{
	return NO;
}

- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
{
	return [self draggingEntered:sender];
}

- (BOOL) performDragOperation:(id <NSDraggingInfo>)sender
{
    if (delegate!=nil)
	{
		NSPasteboard * tPasteBoard;
		
		tPasteBoard = [sender draggingPasteboard];

		if ( [[tPasteBoard types] containsObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_UUID_PASTEBOARDTYPE] )
		{
			NSArray * tItems = [tPasteBoard propertyListForType:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_UUID_PASTEBOARDTYPE];
			
			if (tItems!=nil && [tItems count]==1)
			{
				
				[delegate selectItemOfPopUpButton:self forChoice:[tItems objectAtIndex:0]];
				
				return YES;
			}
		}
	}
	
    return NO;
}

- (BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	highlighted_=NO;
	
    [self setNeedsDisplay:YES];
    
    return YES;
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
	highlighted_=NO;
	
    [self setNeedsDisplay:YES];
}

@end
