/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICChoiceUtilities.h"

#import "TreeNode+Indentation.h"

#import "ICDocumentFormat+Constants.h"

NSString * const ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY=@"ENABLED_DEPENDENCY_AVAILABILITY";

NSString * const ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY=@"SELECTED_DEPENDENCY_AVAILABILITY";

NSString * const ICCHOICE_ENABLED_DEPENDENCIES=@"ENABLED_DEPENDENCIES";

NSString * const ICCHOICE_SELECTED_DEPENDENCIES=@"SELECTED_DEPENDENCIES";

NSString * const ICCHOICE_IS_GROUP=@"IS_GROUP";

NSString * const ICCHOICE_NODE=@"NODE";

NSString * const ICCHOICE_UUID=@"UUID";

NSString * const ICCHOICE_STATE_OBJECT=@"STATE";

@implementation NSString (NumericComparison)

- (NSComparisonResult) compareNumeric:(NSString *) inString
{
	return [self compare:inString options:NSNumericSearch];
}

@end

@implementation ICChoiceUtilities

+ (BOOL) isEnabledStateConstantForNode:(ICInstallationTypeTreeNode *) inTreeNode
{
	BOOL tIsEnabledStateConstant=YES;
	
	if (inTreeNode!=nil)
	{
		ICInstallationTypeNodeData * tNodeData;
		
		tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(inTreeNode);
	
		if (tNodeData!=nil)
		{
			if ([tNodeData type]!=kInstallationTypeRootNode)
			{
				NSDictionary * tChoiceDictionary;
				
				tChoiceDictionary=[tNodeData dictionary];
				
				if (tChoiceDictionary!=nil)
				{
					// Check the Type of Choice
					
					NSNumber * tNumber;
					
					tNumber=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
					
					if (tNumber!=nil)
					{
						NSDictionary * tOptionsDictionary;
						BOOL isGroup=NO;
					
						tOptionsDictionary=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
						
						if (tOptionsDictionary!=nil)
						{
							if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
							{
								tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
									
								isGroup=([tNumber boolValue]==NO);
							}
						}
						
						if (tOptionsDictionary!=nil)
						{
							int tOptionsState;
							NSDictionary * tDependencyDictionary;
							
							tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
							
							tOptionsState=[tNumber intValue];
							
							if (isGroup==NO)
							{
								if (tOptionsState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENT)
								{
									tDependencyDictionary=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
									
									if (tDependencyDictionary!=nil)
									{
										NSNumber * tEnabledStateModeNumber;
										
										tEnabledStateModeNumber=[tDependencyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE];
										
										if (tEnabledStateModeNumber!=nil && [tEnabledStateModeNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_DEPENDENT)
										{
											tIsEnabledStateConstant=NO;
										}
									}
								}
							}
							else
							{
								if (tOptionsState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_DEPENDENT)
								{
									tIsEnabledStateConstant=NO;
								}
							}
						}
					}
				}
			}
		}
	}

	return tIsEnabledStateConstant;
}

+ (void) addChoiceDependencyForBranch:(NSDictionary *) inBranch toArray:(NSMutableArray *) inMutableArray
{
	if (inMutableArray!=nil && inBranch!=nil)
	{
		NSDictionary * tDictionary;
	
		// Is it a branch or leaf?
		
		tDictionary=[inBranch objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP];
		
		if (tDictionary==nil)
		{
			// Leaf
			
			NSString * tChoiceUUID;
			NSNumber * tStateObject;
			
			tChoiceUUID=[inBranch objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID];
			
			tStateObject=[inBranch objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT];
			
			if (tChoiceUUID!=nil && tStateObject!=nil)
			{
				NSDictionary * tBranchDependency;
				
				tBranchDependency=[NSDictionary dictionaryWithObjectsAndKeys:tChoiceUUID,ICCHOICE_UUID,
																			 tStateObject,ICCHOICE_STATE_OBJECT,
																			 nil];
				
				if (tBranchDependency!=nil && ([inMutableArray containsObject:tBranchDependency]==NO))
				{
					[inMutableArray addObject:tBranchDependency];
				}
			}
		}
		else
		{
			// Branch
			
			[ICChoiceUtilities addChoiceDependencyForBranch:tDictionary toArray:inMutableArray];
			
			tDictionary=[inBranch objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM];
			
			[ICChoiceUtilities addChoiceDependencyForBranch:tDictionary toArray:inMutableArray];
		}
	}
}

+ (NSMutableArray *) enabledDependenciesFromDependencyDictionary:(NSDictionary *) inDependencyDictionary
{
	NSMutableArray * tMutableArray=nil;
	
	if (inDependencyDictionary!=nil)
	{
		NSNumber * tNumber;
		
		tNumber=[inDependencyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE];
		
		if (tNumber!=nil && [tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_DEPENDENT)
		{
			NSDictionary * tEnabledDependencies;
			
			tEnabledDependencies=[inDependencyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
			
			if (tEnabledDependencies!=nil)
			{
				tMutableArray=[NSMutableArray array];
				
				[ICChoiceUtilities addChoiceDependencyForBranch:tEnabledDependencies toArray:tMutableArray];
			}
		}
	}
	
	return tMutableArray;
}

+ (NSMutableArray *) selectedDependenciesFromDependencyDictionary:(NSDictionary *) inDependencyDictionary
{
	NSMutableArray * tMutableArray=nil;
	
	if (inDependencyDictionary!=nil)
	{
		NSDictionary * tSelectedDependencies;
			
		tSelectedDependencies=[inDependencyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_SELECTED_DICTIONARY];
		
		if (tSelectedDependencies!=nil)
		{
			tMutableArray=[NSMutableArray array];
			
			[ICChoiceUtilities addChoiceDependencyForBranch:tSelectedDependencies toArray:tMutableArray];
		}
	}
	
	return tMutableArray;
}

#pragma mark -

+ (void) addDependenciesOfNode:(ICInstallationTypeTreeNode *) inNode toDictionary:(NSMutableDictionary *) inDictionary
{
	if (inDictionary!=nil && inNode!=nil)
	{
		ICInstallationTypeNodeData * tNodeData;
		NSArray * tChildren;
		
		tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(inNode);
	
		if (tNodeData!=nil)
		{
			if ([tNodeData type]!=kInstallationTypeRootNode)
			{
				NSDictionary * tChoiceDictionary;
				
				tChoiceDictionary=[tNodeData dictionary];
				
				if (tChoiceDictionary!=nil)
				{
					// Check the Type of Choice
					
					NSNumber * tNumber;
					
						
					tNumber=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
					
					if (tNumber!=nil)
					{
						NSDictionary * tOptionsDictionary;
						BOOL tIsHiddenGroup=NO;
						BOOL isGroup=NO;
					
						tOptionsDictionary=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
						
						if (tOptionsDictionary!=nil)
						{
							if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
							{
								tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
									
								if (tNumber!=nil && [tNumber boolValue]==YES)
								{
									tIsHiddenGroup=YES;
								}
								else
								{
									isGroup=YES;
								}
							}
						}
						
						if (tOptionsDictionary!=nil)
						{
							int tOptionsState;
							NSDictionary * tDependencyDictionary;
							
							
							
							tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
							
							tOptionsState=[tNumber intValue];
							
							if (isGroup==NO)
							{
								NSMutableArray * tSelectedDependencies=nil;
								NSMutableArray * tEnabledDependencies=nil;
								
								if (tOptionsState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENT)
								{
									tDependencyDictionary=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
									
									if (tDependencyDictionary!=nil)
									{
										NSNumber * tEnabledStateModeNumber;
										
										tEnabledStateModeNumber=[tDependencyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE];
										
										if (tEnabledStateModeNumber!=nil && [tEnabledStateModeNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_DEPENDENT)
										{
											tEnabledDependencies=[ICChoiceUtilities enabledDependenciesFromDependencyDictionary:tDependencyDictionary];
										}
									
										tSelectedDependencies=[ICChoiceUtilities selectedDependenciesFromDependencyDictionary:tDependencyDictionary];
									}
								}

								[inDictionary setObject:[NSMutableDictionary dictionaryWithObjectsAndKeys:inNode,ICCHOICE_NODE,
																								   [NSNumber numberWithBool:NO],ICCHOICE_IS_GROUP,
																								   [NSNumber numberWithBool:YES],ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY,
																								   [NSNumber numberWithBool:YES],ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY,
																								   tSelectedDependencies,ICCHOICE_SELECTED_DEPENDENCIES,
																								   tEnabledDependencies,ICCHOICE_ENABLED_DEPENDENCIES,
																								   nil]
												 forKey:[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID]];
							}
							else
							{
								NSMutableArray * tEnabledDependencies=nil;
								
								if (tOptionsState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_DEPENDENT)
								{
									tDependencyDictionary=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
								
									if (tDependencyDictionary!=nil)
									{
										tEnabledDependencies=[ICChoiceUtilities enabledDependenciesFromDependencyDictionary:tDependencyDictionary];
									}
								}
								
								[inDictionary setObject:[NSMutableDictionary dictionaryWithObjectsAndKeys:inNode,ICCHOICE_NODE,
																								   [NSNumber numberWithBool:YES],ICCHOICE_IS_GROUP,
																								   [NSNumber numberWithBool:YES],ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY,
																								   [NSNumber numberWithBool:NO],ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY,
																								   tEnabledDependencies,ICCHOICE_ENABLED_DEPENDENCIES,		// Can be nil
																								   nil]
												 forKey:[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID]];
							}
						}
						
						if (tIsHiddenGroup==YES)
						{
							return;
						}
					}
				}
			}
		}
		
		tChildren=[inNode children];
		
		if (tChildren!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[tChildren objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				ICInstallationTypeTreeNode * tChildNode;
				
				while (tChildNode=[tEnumerator nextObject])
				{
					[self addDependenciesOfNode:tChildNode toDictionary:inDictionary];
				}
			}
		}
	}
}

+ (NSMutableDictionary *) allDependenciesForTree:(ICInstallationTypeTreeNode *) inTreeNode
{
	NSMutableDictionary * tMutableDictionary=nil;
	
	if (inTreeNode!=nil)
	{
		tMutableDictionary=[NSMutableDictionary dictionary];
		
		if (tMutableDictionary!=nil)
		{
			[ICChoiceUtilities addDependenciesOfNode:inTreeNode toDictionary:tMutableDictionary];
		}
	}
	
	return tMutableDictionary;
}

+ (NSMutableDictionary *) availableDependenciesDictionaryForEnabledStateOfGroupNode:(ICInstallationTypeTreeNode *) inTreeNode
{
	ICInstallationTypeTreeNode * tRootNode;
	NSMutableDictionary * tAvailableDependenciesDictionary=nil;
	
	tRootNode=[inTreeNode rootNode];
	
	if (tRootNode!=nil)
	{
		NSMutableDictionary * tCompleteDependenciesDictionary;
		
		tCompleteDependenciesDictionary=[ICChoiceUtilities allDependenciesForTree:tRootNode];
		
		if (tCompleteDependenciesDictionary!=nil)
		{
			NSString * tChoiceUUID;
			
			tChoiceUUID=[inTreeNode choiceUUID];
			
			if (tChoiceUUID!=nil)
			{
				NSArray * tAllKeys;
				NSMutableArray * tDependentChoicesArray;
				
				// Remove the group itself
				
				[tCompleteDependenciesDictionary removeObjectForKey:tChoiceUUID];
				
				// Remove the items which are children of this group
				
				tAllKeys=[tCompleteDependenciesDictionary allKeys];
				
				if (tAllKeys!=nil)
				{
					NSUInteger tCount,i;
					
					tCount=[tAllKeys count];
					
					for(i=tCount;i>0;i--)
					{
						NSString * tUUID;
						NSDictionary * tDictionary;
						
						tUUID=[tAllKeys objectAtIndex:i-1];
						
						tDictionary=[tCompleteDependenciesDictionary objectForKey:tUUID];
						
						if (tDictionary!=nil)
						{
							ICInstallationTypeTreeNode * tNode;
						
							tNode=[tDictionary objectForKey:ICCHOICE_NODE];
							
							if (tNode!=nil)
							{
								if ([tNode isDescendantOfNode:inTreeNode]==YES)
								{
									[tCompleteDependenciesDictionary removeObjectForKey:tUUID];
								}
							}
						}
					}
				}
				
				// Remove the items creating a loop
				
				tDependentChoicesArray=[NSMutableArray arrayWithObject:[NSDictionary dictionaryWithObjectsAndKeys:tChoiceUUID,ICCHOICE_UUID,
																												  [NSNumber numberWithInt:ICSTATE_ENABLED],ICCHOICE_STATE_OBJECT,
																												  nil]];
			
				if (tDependentChoicesArray!=nil)
				{
					while ([tDependentChoicesArray count]>0)
					{
						NSDictionary * tStateDependencyDictionary;
						
						tStateDependencyDictionary=[tDependentChoicesArray objectAtIndex:0];
						
						tAllKeys=[tCompleteDependenciesDictionary allKeys];
						
						if (tAllKeys!=nil)
						{
							NSEnumerator * tKeyEnumerator;
							
							tKeyEnumerator=[tAllKeys objectEnumerator];
						
							if (tKeyEnumerator!=nil)
							{
								NSString * tKey;
							
								while (tKey=[tKeyEnumerator nextObject])
								{
									NSMutableDictionary * tMutableDictionary;
									
									tMutableDictionary=[tCompleteDependenciesDictionary objectForKey:tKey];
									
									if (tMutableDictionary!=nil)
									{
										// Is it a Group or Not?
										
										NSNumber * tNumber;
										NSArray * tEnabledDependencies;
										
										tNumber=[tMutableDictionary objectForKey:ICCHOICE_IS_GROUP];
										
										if (tNumber!=nil && [tNumber boolValue]==NO)
										{
											// It's not a group, we need to inspect the selected dependencies too
											
											NSArray * tSelectedDependencies;
											
											tSelectedDependencies=[tMutableDictionary objectForKey:ICCHOICE_SELECTED_DEPENDENCIES];
											
											if (tSelectedDependencies!=nil)
											{
												if ([tSelectedDependencies containsObject:tStateDependencyDictionary]==YES)
												{
													[tDependentChoicesArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:tKey,ICCHOICE_UUID,
																																 [NSNumber numberWithInt:ICSTATE_SELECTED],ICCHOICE_STATE_OBJECT,
																																 nil]];
																																 
													// The Selected state of this item can't be used for dependency
													
													[tMutableDictionary setObject:[NSNumber numberWithBool:NO] forKey:ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY];
												}
											}
										}
										
										tEnabledDependencies=[tMutableDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCIES];
										
										if (tEnabledDependencies!=nil)
										{
											if ([tEnabledDependencies containsObject:tStateDependencyDictionary]==YES)
											{
												[tDependentChoicesArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:tKey,ICCHOICE_UUID,
																															 [NSNumber numberWithInt:ICSTATE_ENABLED],ICCHOICE_STATE_OBJECT,
																															 nil]];
																															 
												// The Enabled state of this item can't be used for dependency
												
												[tMutableDictionary setObject:[NSNumber numberWithBool:NO] forKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
											}
										}
										
										tNumber=[tMutableDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
										
										if (tNumber!=nil && [tNumber boolValue]==NO)
										{
											tNumber=[tMutableDictionary objectForKey:ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY];
										
											if (tNumber!=nil && [tNumber boolValue]==NO)
											{
												// The item selected and enabled state can't be used for dependencies
												
												[tCompleteDependenciesDictionary removeObjectForKey:tKey];
											}
										}
									}
								}
							}
							
							[tDependentChoicesArray removeObjectAtIndex:0];
						}
					}
				}
				
				tAvailableDependenciesDictionary=tCompleteDependenciesDictionary;
			}
		}
	}
	
	return tAvailableDependenciesDictionary;
}

+ (NSMutableDictionary *) availableDependenciesDictionaryForEnabledStateOfLeafNode:(ICInstallationTypeTreeNode *) inTreeNode skipEnabledIfConstant:(BOOL) inSkipEnabled
{
	ICInstallationTypeTreeNode * tRootNode;
	NSMutableDictionary * tAvailableDependenciesDictionary=nil;
	
	tRootNode=[inTreeNode rootNode];
	
	if (tRootNode!=nil)
	{
		NSMutableDictionary * tCompleteDependenciesDictionary;
		
		tCompleteDependenciesDictionary=[ICChoiceUtilities allDependenciesForTree:tRootNode];
		
		if (tCompleteDependenciesDictionary!=nil)
		{
			NSString * tChoiceUUID;
			
			tChoiceUUID=[inTreeNode choiceUUID];
			
			if (tChoiceUUID!=nil)
			{
				NSArray * tAllKeys;
				NSMutableArray * tDependentChoicesArray;
				ICInstallationTypeTreeNode * tNode;
				
				// Remove the leaf itself
				
				[tCompleteDependenciesDictionary removeObjectForKey:tChoiceUUID];
				
				// Remove the parents of this leaf
				
				tAllKeys=[tCompleteDependenciesDictionary allKeys];
				
				if (tAllKeys!=nil)
				{
					NSUInteger tCount,i;
					
					tCount=[tAllKeys count];
					
					for(i=tCount;i>0;i--)
					{
						NSString * tUUID;
						NSDictionary * tDictionary;
						
						tUUID=[tAllKeys objectAtIndex:i-1];
						
						tDictionary=[tCompleteDependenciesDictionary objectForKey:tUUID];
						
						if (tDictionary!=nil)
						{
							tNode=[tDictionary objectForKey:ICCHOICE_NODE];
							
							if (tNode!=nil)
							{
								if ([inTreeNode isDescendantOfNode:tNode]==YES)
								{
									[tCompleteDependenciesDictionary removeObjectForKey:tUUID];
								}
							}
						}
					}
				}
				
				// Remove the items creating a loop
				
				tDependentChoicesArray=[NSMutableArray arrayWithObject:[NSDictionary dictionaryWithObjectsAndKeys:tChoiceUUID,ICCHOICE_UUID,
																												  [NSNumber numberWithInt:ICSTATE_SELECTED],ICCHOICE_STATE_OBJECT,
																												  nil]];
				
				if (inSkipEnabled==NO || [ICChoiceUtilities isEnabledStateConstantForNode:inTreeNode]==NO)
				{
					[tDependentChoicesArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:tChoiceUUID,ICCHOICE_UUID,
																								 [NSNumber numberWithInt:ICSTATE_ENABLED],ICCHOICE_STATE_OBJECT,
																								 nil]];
																		
				}
				
				// We need to take into account the parent choices of the item
			
				tNode=(ICInstallationTypeTreeNode *) [inTreeNode nodeParent];
				
				while (tNode!=nil)
				{
					ICInstallationTypeNodeData * tNodeData;
			
					tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tNode);
					
					if (tNodeData!=nil)
					{
						if ([tNodeData type]!=kInstallationTypeRootNode)
						{
							[tDependentChoicesArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:[tNode choiceUUID],ICCHOICE_UUID,
																										 [NSNumber numberWithInt:ICSTATE_ENABLED],ICCHOICE_STATE_OBJECT,
																										 nil]];
						}
						else
						{
							break;
						}
					}
					
					tNode=(ICInstallationTypeTreeNode *) [tNode nodeParent];
				}
				
				if (tDependentChoicesArray!=nil)
				{
					while ([tDependentChoicesArray count]>0)
					{
						NSDictionary * tStateDependencyDictionary;
						
						tStateDependencyDictionary=[tDependentChoicesArray objectAtIndex:0];
						
						tAllKeys=[tCompleteDependenciesDictionary allKeys];
						
						if (tAllKeys!=nil)
						{
							NSEnumerator * tKeyEnumerator;
							
							tKeyEnumerator=[tAllKeys objectEnumerator];
						
							if (tKeyEnumerator!=nil)
							{
								NSString * tKey;
							
								while (tKey=[tKeyEnumerator nextObject])
								{
									NSMutableDictionary * tMutableDictionary;
									
									tMutableDictionary=[tCompleteDependenciesDictionary objectForKey:tKey];
									
									if (tMutableDictionary!=nil)
									{
										// Is it a Group or Not?
										
										NSNumber * tNumber;
										NSArray * tEnabledDependencies;
										
										tNumber=[tMutableDictionary objectForKey:ICCHOICE_IS_GROUP];
										
										if (tNumber!=nil && [tNumber boolValue]==NO)
										{
											// It's not a group, we need to inspect the selected dependencies too
											
											NSArray * tSelectedDependencies;
											
											tSelectedDependencies=[tMutableDictionary objectForKey:ICCHOICE_SELECTED_DEPENDENCIES];
											
											if (tSelectedDependencies!=nil)
											{
												if ([tSelectedDependencies containsObject:tStateDependencyDictionary]==YES)
												{
													[tDependentChoicesArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:tKey,ICCHOICE_UUID,
																																 [NSNumber numberWithInt:ICSTATE_SELECTED],ICCHOICE_STATE_OBJECT,
																																 nil]];
													// The Selected state of this item can't be used for dependency
													
													[tMutableDictionary setObject:[NSNumber numberWithBool:NO] forKey:ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY];
												}
											}
										}
										
										tEnabledDependencies=[tMutableDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCIES];
										
										if (tEnabledDependencies!=nil)
										{
											if ([tEnabledDependencies containsObject:tStateDependencyDictionary]==YES)
											{
												[tDependentChoicesArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:tKey,ICCHOICE_UUID,
																															 [NSNumber numberWithInt:ICSTATE_ENABLED],ICCHOICE_STATE_OBJECT,
																															 nil]];
																															 
												// The Enabled state of this item can't be used for dependency
												
												[tMutableDictionary setObject:[NSNumber numberWithBool:NO] forKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
											}
										}
										
										tNumber=[tMutableDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
										
										if (tNumber!=nil && [tNumber boolValue]==NO)
										{
											tNumber=[tMutableDictionary objectForKey:ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY];
										
											if (tNumber!=nil && [tNumber boolValue]==NO)
											{
												// The item selected and enabled state can't be used for dependencies
												
												[tCompleteDependenciesDictionary removeObjectForKey:tKey];
											}
										}
									}
								}
							}
							
							[tDependentChoicesArray removeObjectAtIndex:0];
						}
					}
				}
				
				tAvailableDependenciesDictionary=tCompleteDependenciesDictionary;
			}
		}
	}
	
	return tAvailableDependenciesDictionary;
}

+ (NSMutableDictionary *) availableDependenciesDictionaryForEnabledStateOfLeafNode:(ICInstallationTypeTreeNode *) inTreeNode
{
	return [ICChoiceUtilities availableDependenciesDictionaryForEnabledStateOfLeafNode:inTreeNode skipEnabledIfConstant:YES];
}

+ (NSMutableDictionary *) availableDependenciesDictionaryForSelectedStateOfLeafNode:(ICInstallationTypeTreeNode *) inTreeNode
{
	return [ICChoiceUtilities availableDependenciesDictionaryForEnabledStateOfLeafNode:inTreeNode];
}

#pragma mark -

+ (NSMutableArray *) availableListOfChoicesFromAvailableDependenciesDictionary:(NSDictionary *) inDictionary
{
	NSMutableArray * tMutableArray=nil;
	
	if (inDictionary!=nil)
	{
		NSEnumerator * tUUIDEnumerator;
	
		tUUIDEnumerator=[inDictionary keyEnumerator];
		
		if (tUUIDEnumerator!=nil)
		{
			NSString * tUUID;
			
			tMutableArray=[NSMutableArray array];
			
			while (tUUID=[tUUIDEnumerator nextObject])
			{
				NSDictionary * tDictionary;
				
				tDictionary=[inDictionary objectForKey:tUUID];
				
				if (tDictionary!=nil)
				{
					ICInstallationTypeTreeNode * tChoiceNode;
					
					tChoiceNode=[tDictionary objectForKey:ICCHOICE_NODE];
					
					if (tChoiceNode!=nil)
					{
						NSString * tChoiceIndex;
									
						tChoiceIndex=[NSString stringWithFormat:NSLocalizedString(@"Choice %@",@""),[tChoiceNode indentationString]];
						
						if (tChoiceIndex!=nil)
						{
							[tMutableArray addObject:tChoiceIndex];
						}
					}
				}
			}
			
			[tMutableArray sortUsingSelector:@selector(compareNumeric:)];
		}
	}
	
	return tMutableArray;
}

+ (NSMutableDictionary *) availableChoicesDictionaryFromAvailableDependenciesDictionary:(NSDictionary *) inDictionary
{
	NSMutableDictionary * tMutableDictionary=nil;
	
	if (inDictionary!=nil)
	{
		NSEnumerator * tUUIDEnumerator;
	
		tUUIDEnumerator=[inDictionary keyEnumerator];
		
		if (tUUIDEnumerator!=nil)
		{
			NSString * tUUID;
			
			tMutableDictionary=[NSMutableDictionary dictionary];
			
			while (tUUID=[tUUIDEnumerator nextObject])
			{
				NSDictionary * tDictionary;
				
				tDictionary=[inDictionary objectForKey:tUUID];
				
				if (tDictionary!=nil)
				{
					ICInstallationTypeTreeNode * tChoiceNode;
					
					tChoiceNode=[tDictionary objectForKey:ICCHOICE_NODE];
					
					if (tChoiceNode!=nil)
					{
						NSString * tChoiceIndex;
									
						tChoiceIndex=[NSString stringWithFormat:NSLocalizedString(@"Choice %@",@""),[tChoiceNode indentationString]];
						
						if (tChoiceIndex!=nil)
						{
							[tMutableDictionary setObject:[NSDictionary dictionaryWithObjectsAndKeys:tUUID,ICCHOICE_UUID,
																									 [tDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY],ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY,
																									 [tDictionary objectForKey:ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY],ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY,
																									 nil]
												   forKey:tChoiceIndex];
						}
					}
				}
			}
		}
	}
	
	return tMutableDictionary;
}

+ (NSMutableDictionary *) UUIDDictionaryFromAvailableDependenciesDictionary:(NSDictionary *) inDictionary
{
	NSMutableDictionary * tMutableDictionary=nil;
	
	if (inDictionary!=nil)
	{
		NSEnumerator * tUUIDEnumerator;
	
		tUUIDEnumerator=[inDictionary keyEnumerator];
		
		if (tUUIDEnumerator!=nil)
		{
			NSString * tUUID;
			
			tMutableDictionary=[NSMutableDictionary dictionary];
			
			while (tUUID=[tUUIDEnumerator nextObject])
			{
				NSDictionary * tDictionary;
				
				tDictionary=[inDictionary objectForKey:tUUID];
				
				if (tDictionary!=nil)
				{
					ICInstallationTypeTreeNode * tChoiceNode;
					
					tChoiceNode=[tDictionary objectForKey:ICCHOICE_NODE];
					
					if (tChoiceNode!=nil)
					{
						NSString * tChoiceIndex;
									
						tChoiceIndex=[NSString stringWithFormat:NSLocalizedString(@"Choice %@",@""),[tChoiceNode indentationString]];
						
						if (tChoiceIndex!=nil)
						{
							[tMutableDictionary setObject:tChoiceIndex
												   forKey:tUUID];
						}
					}
				}
			}
		}
	}
	
	return tMutableDictionary;
}

@end
