/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICComponentController.h"

@implementation ICComponentController

- (void) awakeFromNib
{
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(viewDidResize:) name:NSViewFrameDidChangeNotification object:IBview_];
}

- (void) dealloc
{
	// A COMPLETER

	[currentSegmentedView_ removeFromSuperview];
	
	[currentSegmentedController_ release];

	[super dealloc];
}

#pragma mark -

- (ICSegmentedController *) currentSegmentedController
{
	return [[currentSegmentedController_ retain] autorelease];
}

- (ICSegmentedController *) segmentedControllerWithTag:(NSInteger) inTag
{
	NSLog(@"Override this method");
	
	return nil;
}

- (IBAction) showView:(id) sender
{
	if (currentSegmentedController_!=nil)
	{
		if ([currentSegmentedController_ viewCanBeRemoved:self]==NO)
		{
			return;
		}
		
		[currentSegmentedController_ viewWillBeRemoved:self];
		
		if (currentSegmentedView_!=nil)
		{
			[currentSegmentedView_ removeFromSuperview];
			
			currentSegmentedView_=nil;
		}
		
		[currentSegmentedController_ release];
	}
	
	//NSLog(@"%@",sender);
	
	currentSegmentedController_=[self segmentedControllerWithTag:[sender selectedSegment]];
	
	if (currentSegmentedController_!=nil)
	{
		[currentSegmentedController_ retain];
		
		currentSegmentedView_=[currentSegmentedController_ view];
		
		// Resize view
		
		[currentSegmentedView_ setFrame:[IBcontentView_ bounds]];
		
		// Initialize View widgets
		
		[currentSegmentedController_ initializeWithDictionary:dictionary_ forDocument:document_];
		
		[IBcontentView_ addSubview:currentSegmentedView_];
	}
	else
	{
		NSLog(@"Missing controller for tag: %d",[sender selectedSegment]);
	}
}

#pragma mark -

- (void) viewWillBeRemoved:(id) sender
{
	if (currentSegmentedController_!=nil)
	{
		[currentSegmentedController_ viewWillBeRemoved:self];
	}
}

#pragma mark -

- (void) viewDidResize:(NSNotification *) inNotification
{
	NSRect tFrame;
	NSInteger i,tSegmentCount;
	CGFloat tSegmentWidth;

	tSegmentCount=[IBsegmentedView_ segmentCount];
	
	tFrame=[IBsegmentedView_ frame];
	
	tFrame.origin.x=-7.0f;
	
	tFrame.size.width=NSWidth([IBview_ frame])+7.0f;
	
	tSegmentWidth=tFrame.size.width/tSegmentCount;
	
	for(i=0;i<(tSegmentCount-1);i++)
	{
		[IBsegmentedView_ setWidth:tSegmentWidth forSegment:i];
	}
	
	[IBsegmentedView_ setWidth:tFrame.size.width-(tSegmentCount-1)*tSegmentWidth forSegment:(tSegmentCount-1)];
	
	[IBsegmentedView_ setFrame:tFrame];
}

@end
