/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICDependencyContainerView.h"
#import "ICPresentationInspectorInstallationTypeChoiceDependencyController.h"

@implementation ICDependencyContainerView

- (void) dealloc
{
	[dictionary_ release];

	[parentDictionary_ release];
	
	[controller_ release];

	[super dealloc];
}

#pragma mark -

- (NSMutableDictionary *) dictionary
{
	return [[dictionary_ retain] autorelease];
}

- (void) setDictionary:(NSDictionary *) inDictionary
{
	if (dictionary_!=inDictionary)
	{
		[dictionary_ release];
	
		dictionary_=[inDictionary retain];
	}
}

- (NSMutableDictionary *) parentDictionary
{
	return [[parentDictionary_ retain] autorelease];
}

- (void) setParentDictionary:(NSDictionary *) inParentDictionary
{
	if (parentDictionary_!=inParentDictionary)
	{
		[parentDictionary_ release];
	
		parentDictionary_=[inParentDictionary retain];
	}
}

- (id) controller
{
	return [[controller_ retain] autorelease];
}

- (void) setController:(id) inController
{
	if (controller_!=inController)
	{
		[controller_ release];
	
		controller_=[inController retain];
	}
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{
	SEL tAction;
	
	tAction=[inMenuItem action];
	
	if (tAction==@selector(switchEnabledStateObject:) ||
		tAction==@selector(switchSelectedStateObject:))
	{
		if (controller_!=nil)
		{
			return [controller_ validateMenuItem:inMenuItem forDictionary:dictionary_];
		}
	}
	
	return YES;
}

- (void) switchEnabledStateObject:(id) sender
{
	if (controller_!=nil)
	{
		[controller_ switchEnabledStateObject:sender];
	}
}

- (void) switchSelectedStateObject:(id) sender
{
	if (controller_!=nil)
	{
		[controller_ switchSelectedStateObject:sender];
	}
}

/*- (void) drawRect:(NSRect) inFrame
{
	[[NSColor redColor] set];
	
	NSFrameRect([self bounds]);
}*/

@end
