/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICDispatchBuildUtilities.h"

#import "ICPreferences+Constants.h"

@implementation ICDispatchBuildUtilities

+ (int) buildProjectAtPath:(NSString *) inProjectPath withNotificationPath:(NSString *) inNotificationPath locationEnabled:(BOOL) isLocationEnabled
{
	CFMessagePortRef tRemote;
	NSString * tScratchPath;
    NSString * tSplitForksToolName;
	NSUserDefaults * tUserDefaults;
	
	tUserDefaults=[NSUserDefaults standardUserDefaults];
	
	tScratchPath=[tUserDefaults stringForKey:ICPREFERENCES_TEMPORARY_BUILD_LOCATION];
        
    if (tScratchPath==nil)
    {
        tScratchPath=ICPREFERENCES_DEFAULT_TEMPORARY_BUILD_LOCATION;
    }
    
	tSplitForksToolName=[tUserDefaults stringForKey:ICPREFERENCES_FILES_SPLITFORKS_TOOL];
	
	if (tSplitForksToolName==nil)
	{
		tSplitForksToolName=SPLITFORKSTOOL_NAME_GOLDIN;
	}

	
	tRemote=CFMessagePortCreateRemote(kCFAllocatorDefault,(CFStringRef) ICDISPATCH_MESSAGEPORT_NAME);

#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_5

    if (tRemote==NULL)
	{
		int tCount1=0;
	
		do
		{
			if ([ICDispatchBuildUtilities _wakeupDispatcher]==YES)
			{
				int tCount2=0;
				struct timespec tSleepDuration={0,100000000};
				
				do
				{
					nanosleep(&tSleepDuration,NULL);
					
					tRemote=CFMessagePortCreateRemote(kCFAllocatorDefault,(CFStringRef) ICDISPATCH_MESSAGEPORT_NAME);
					
					tCount2++;
				}
				while (tCount2<20 && tRemote==NULL);
			}
			
			tCount1++;
		}
		while (tCount1<2 && tRemote==NULL);
	}

#endif

	if (tRemote!=NULL)
    {
        CFDataRef tDataRef;
        NSDictionary * tDictionary;
        
        tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:inProjectPath,ICDISPATCH_PROJECT_PATH,
                                                               inNotificationPath,ICDISPATCH_NOTIFICATION_PATH,
                                                               [NSNumber numberWithInt:getuid()],ICDISPATCH_USER_ID,
                                                               [NSNumber numberWithInt:getgid()],ICDISPATCH_GROUP_ID,
                                                               [NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],ICDISPATCH_PROCESS_ID,
															   [NSNumber numberWithBool:!isLocationEnabled],ICDISPATCH_DEBUG_MODE,
                                                               tSplitForksToolName,ICDISPATCH_SPLITFORKS_TOOL,
															   tScratchPath,ICDISPATCH_SCRATCH_PATH,
                                                               nil];
        
        
        if (tDictionary!=nil)
        {
            tDataRef=CFPropertyListCreateXMLData(kCFAllocatorDefault,(CFPropertyListRef) tDictionary);
        
            if (tDataRef!=NULL)
            {
                if (CFMessagePortSendRequest(tRemote,0,tDataRef, 20, 10, NULL, NULL)!=kCFMessagePortSuccess)
                {
                    CFRelease(tDataRef);
					
					CFRelease(tRemote);
					
					NSBeep();
                    
                    return NO;
                }
				
                // Release Memory
                
                CFRelease(tDataRef);
            }
        }
        
        // Release memory
        
        CFRelease(tRemote);
		
		return YES;
    }
	else
	{
		return -1;
	}
	
	return NO;
}

+ (BOOL) _wakeupDispatcher
{
	// Send a Mach Port message to start the dipatcher
		
	NSMachBootstrapServer * tBootstrapServer;
	
	tBootstrapServer = [NSMachBootstrapServer sharedInstance];
	
	if (tBootstrapServer!=nil)
	{
		NSMachPort * tMachPort;
		
		tMachPort=(NSMachPort *) [tBootstrapServer portForName:@ICDISPATCH_MACHPORT_NAME];
		
		if (tMachPort!=nil)
		{
			NSPortMessage * tPortMessage;
			
			tPortMessage= [[NSPortMessage alloc] initWithSendPort:tMachPort
													  receivePort:nil
													   components:nil];
			
			if (tPortMessage)
			{
				[tPortMessage setMsgid:0];
				
				[tPortMessage sendBeforeDate:[NSDate distantFuture]];
				
				[tPortMessage release];
				
				return YES;
			}
		}
	}
	
	return NO;
}

@end
