/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICDocument+BuildingNotification.h"

#import "ICProjectBuilder+Notifications.h"

#import "ICProjectBuilder+SignatureRequest.h"

#import "ICPreferences+Constants.h"

#import "ICSignatureFactory.h"

@implementation ICDocument (BuildingNotification)

- (void) builderNotification:(NSNotification *)notification
{
    NSDictionary * tUserInfo;
    static int sProcessID=-1;
    
    if (sProcessID==-1)
    {
        sProcessID=[[NSProcessInfo processInfo] processIdentifier];
    }
    
    tUserInfo=[notification userInfo];
    
    if (tUserInfo!=nil)
    {
        NSNumber * tNumber;
        
        tNumber=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_PID];
        
        if (tNumber!=nil)
        {
            if ([tNumber intValue]==sProcessID)
            {
                NSString * tPath;
                
                tPath=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH];
                
                if (tPath!=nil)
                {
                    if ([tPath isEqualToString:[self fileName]]==YES)
                    {
                        int tEventInfoID;
						NSDictionary * tExtraData;
						NSDictionary * tStackMetadata;
						
						if (building_==YES)
						{
							if ([IBbuildStatusLabel_ isHidden]==YES)
							{
								[IBbuildStatusLabel_ setHidden:NO];
							}
							
							if ([IBbuildStatusProgressIndicator_ isHidden]==YES)
							{
								[IBbuildStatusProgressIndicator_ setHidden:NO];
							}
						}
						
						tEventInfoID=[[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_ID] intValue];
						
						tExtraData=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA];
						
						tStackMetadata=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_INFORMATION_STACK_METADATA];
						
						if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_BUILD_PROGRESSION)
						{
							tNumber=[tExtraData objectForKey:IC_BUILDER_INFORMATION_BUILD_PROGRESSION_PACKAGES_NUMBER];
							
							if (tNumber!=nil)
							{
								unsigned long tMaxValue;
								
								numberOfPackagesToBuild_=[tNumber unsignedLongValue];
								
								tMaxValue=numberOfPackagesToBuild_*3+3;
								
								[IBbuildStatusProgressIndicator_ setMaxValue:tMaxValue];
							}
							else
							{
								NSLog(@"[ICDocument builderNotification:] Missing IC_BUILDER_INFORMATION_BUILD_PROGRESSION_PACKAGES_NUMBER data");
							}
						}
						else if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_DEBUG)
						{
							NSLog(@"-------------- Debug -----------------");
							
							NSLog(@"%@",tExtraData);
							
							NSLog(@"--------------------------------------");
						}
						else if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_STANDARD)
						{
							int tStep;
							int tState;
							
							tStep=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP] intValue];

							tState=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STATE] intValue];
							
							if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
							{
								switch( tStep)
								{
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION:
									
										indexOfPackagesBeingBuilt_=0;
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_PACKAGE:
								
										[IBbuildStatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Building package...",@"BuildNotification",@"")];
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_CREATE:
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_REFERENCE:
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_IMPORT:
									
										if (numberOfPackagesToBuild_>0)
										{
											indexOfPackagesBeingBuilt_++;
										
											[IBbuildStatusLabel_ setStringValue:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Building %ld of %ld packages...",@"BuildNotification",@""),indexOfPackagesBeingBuilt_,numberOfPackagesToBuild_]];
										}
										
										break;
								}
							}
							else if (tState==IC_BUILDER_BUILD_STATE_END_FAILURE)
							{
								if (building_==YES)
								{
									NSString * tSoundName;
		
									tSoundName=[defaults_ stringForKey:ICPREFERENCES_BUILD_PLAY_ERRORS];
										
									building_=NO;
								
									[IBbuildStatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Build failed",@"BuildNotification",@"")];
								
									[IBbuildStatusProgressIndicator_ setHidden:YES];
									
									if ([tSoundName length]>0)
									{
										[[NSSound soundNamed:tSoundName] play];
									}
								}
							}
							else
							{
								if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
								{
									// Progress Indicator
									
									switch(tStep)
									{
										case IC_BUILDER_INFORMATION_STEP_PROJECT:
										case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION:
										case IC_BUILDER_INFORMATION_STEP_PACKAGE:
										
										case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_SCRIPT:
										
										case IC_BUILDER_INFORMATION_STEP_PACKAGE_CREATE:
										case IC_BUILDER_INFORMATION_STEP_PACKAGE_INFO:
										case IC_BUILDER_INFORMATION_STEP_PACKAGE_PAYLOAD:
										
											[IBbuildStatusProgressIndicator_ setDoubleValue:[IBbuildStatusProgressIndicator_ doubleValue]+1];
											
											break;
											
										case IC_BUILDER_INFORMATION_STEP_PACKAGE_REFERENCE:
										case IC_BUILDER_INFORMATION_STEP_PACKAGE_IMPORT:
										
											[IBbuildStatusProgressIndicator_ setDoubleValue:[IBbuildStatusProgressIndicator_ doubleValue]+3];
											
											break;
										
									}
									
									if (tStep==IC_BUILDER_INFORMATION_STEP_PROJECT)
									{
										NSString * tSoundName;
		
										tSoundName=[defaults_ stringForKey:ICPREFERENCES_BUILD_PLAY_SUCCESS];
	
										building_=NO;
										
										[IBbuildStatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Build succeeded",@"BuildNotification",@"")];
										
										[IBbuildStatusProgressIndicator_ setHidden:YES];
										
										if ([tSoundName length]>0)
										{
											[[NSSound soundNamed:tSoundName] play];
										}
										
										if (launchAfterBuild_==YES)
										{
											NSDictionary * tProjectMetadataDictionary;
											
											tProjectMetadataDictionary=[tStackMetadata objectForKey:[[NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_PROJECT] stringValue]];
											
											if (tProjectMetadataDictionary!=nil)
											{
												NSString * tLaunchPath;
												
												tLaunchPath=[tProjectMetadataDictionary objectForKey:IC_BUILDER_INFORMATION_FILE_PATH];
												
												if (tLaunchPath!=nil)
												{
													if ([[NSWorkspace sharedWorkspace] openFile:tLaunchPath]==NO)
													{
														NSLog(@"[ICDocument builderNotification:] Error when opening file \'%@\'",tLaunchPath);
													}
												}
												else
												{
													NSLog(@"[ICDocument builderNotification:] Missing metadata IC_BUILDER_INFORMATION_FILE_PATH");
												}
											}
											else
											{
												NSLog(@"[ICDocument builderNotification:] Missing stack metadata for IC_BUILDER_INFORMATION_STEP_PROJECT");
											}
										}

									}
								}
							}
						}
						else if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN)
						{
							int tStep;
							int tState;
							
							tStep=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP] intValue];

							tState=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STATE] intValue];
							
							if (tStep==IC_BUILDER_INFORMATION_STEP_CLEAN)
							{
								if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
								{
									if ([IBbuildStatusLabel_ isHidden]==YES)
									{
										[IBbuildStatusLabel_ setHidden:NO];
									}
									
									if ([IBbuildStatusProgressIndicator_ isHidden]==YES)
									{
										[IBbuildStatusProgressIndicator_ setHidden:NO];
										
										[IBbuildStatusProgressIndicator_ startAnimation:self];
									}
								}
								else if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
								{
									[IBbuildStatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Clean succeeded",@"BuildNotification",@"")];
								
									[IBbuildStatusProgressIndicator_ stopAnimation:self];
								
									[IBbuildStatusProgressIndicator_ setHidden:YES];
								}
								else if (tState==IC_BUILDER_BUILD_STATE_END_FAILURE)
								{
									[IBbuildStatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Clean failed",@"BuildNotification",@"")];
									
									[IBbuildStatusProgressIndicator_ stopAnimation:self];
									
									[IBbuildStatusProgressIndicator_ setHidden:YES];
								}
								else
								{
									NSLog(@"[ICDocument builderNotification:] Unknown State");
								}
							}
							else if (tStep==IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT)
							{
								// Skip
							}
							else
							{
								NSLog(@"[ICDocument builderNotification:] Unknown Step");
							}
						}
						else if (tEventInfoID==IC_BUILDER_NOTIFICATION_REQUEST)
						{
							int tRequestID;
							
							tRequestID=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP] intValue];
							
							if (tRequestID==IC_BUILDER_NOTIFICATION_REQUEST_SIGN_DATA)
							{
								if ([ICSignatureFactory performSignatureRequestWithDictionary:tExtraData]==NO)
								{
									// A COMPLETER
								}
							}
						}
						else
						{
							NSLog(@"[ICDocument builderNotification:] Unknown Event Info ID");
						}
						
					}
				}
			}
		}
	}
}


@end
