/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import <Cocoa/Cocoa.h>

#import "ICDocument+Constants.h"

#import "ICDocumentTree.h"

#import "ICDocumentWindow.h"

#import "ICComponentController.h"

#import "ICFileBouncer.h"

@interface ICDocument : NSDocument
{

	IBOutlet id buildWindowController_;
	
	IBOutlet id IBsplitView_;
	
	IBOutlet id IBsourceListView_;
	
	IBOutlet id IBaddPackageButton_;
	
	IBOutlet id IBremovePackageButton_;
	
	IBOutlet id IBrightView_;
	
	IBOutlet id IBbuildStatusLabel_;
	
	IBOutlet id IBbuildStatusProgressIndicator_;
	
	// Permissions & Reference Dialog
	
	IBOutlet id IBreferenceStyle_;
	
	/// Data
	
	int documentType_;
	
	BOOL nibDidLoad_;
	
	NSUserDefaults * defaults_;
	
	NSFileManager * fileManager_;
	
	ICTreeNode * tree_;
	
	NSMutableDictionary * dictionary_;
	
	ICDocumentSecondaryController * currentComponentController_;
	
	id currentComponentView_;
	
	NSMutableDictionary * documentDefaults_;
	
	ICFileBouncer * fileBouncer_;
	
	CGFloat savedSourceListWidth_;
	
	BOOL editingInstallationTypeChoice_;
	
	NSArray * _cachedRemovedPackagesList;
	
		// Build
	
	NSString * cleanErrorString_;
	
	BOOL launchAfterBuild_;
	
	BOOL building_;
	
	NSUInteger numberOfPackagesToBuild_;
	NSUInteger indexOfPackagesBeingBuilt_;
}

- (NSString *) pathFromPathDictionary:(NSDictionary *) inPathDictionary;

- (NSMutableDictionary *) dictionary;

- (BOOL) isFlatDistribution;

- (int) documentType;

- (NSString *) absoluteBuildPath;

- (NSString *) referenceFolderPath;

- (id) rootObjectForKey:(NSString *) inKey;

- (void) setRootObject:(id) inObject forKey:(NSString *) inKey;

- (ICFileBouncer *) fileBouncer;

- (void) updateFileBouncer;

- (NSArray *) packagesArray;

- (NSDictionary *) projectSettingsDictionary;

- (NSDictionary *) projectPresentationDictionary;

- (NSMutableDictionary *) dictionaryOfPackageWithUUID:(NSString *) inUUID;

- (id) componentControllerForTreeNodeType:(int) inType;

- (IBAction) revealInFinder:(id) sender;


- (id) documentDefaultForKey:(NSString *) inKey;

- (void) setDocumentDefault:(id) inObject forKey:(NSString *) inKey;

- (void) removeDocumentDefaultForKey:(NSString *) inKey;

// File Menu

- (BOOL) canPrint;

- (NSView *) printableView;

- (void) printOperationDidRun:(NSPrintOperation *) inPrintOperation success:(BOOL) inSuccess contextInfo:(void *) info;

// Edit Menu

- (IBAction) duplicate:(id) sender;

// Build Menu

- (IBAction) showHideBuildWindow:(id) sender;

- (IBAction) build:(id) sender;

- (IBAction) buildAndRun:(id) sender;

- (IBAction) buildAndDebug:(id) sender;

- (IBAction) clean:(id) sender;


- (BOOL) _buildWithSearchLocationEnabled:(BOOL) inLocationEnabled;

- (BOOL) buildAsynchronousWithSearchLocationEnabled:(BOOL) inLocationEnabled;

- (NSString *) temporaryProjectPath;

- (void) cleanSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;

- (void) cleanBuildFolder;


- (IBAction) showProject:(id) sender;

- (IBAction) addPackage:(id) sender;

- (IBAction) addPackageReference:(id) sender;

- (IBAction) importPackage:(id) sender;

- (void) importPackageDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo;

- (void) beginImportPackagesReferenceSheet:(NSArray *) inFiles;

- (IBAction) endimportPackageReferenceSheet:(id) sender;

- (void) importPackageReferenceSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;

- (void) _importPackages:(NSArray *) inArray referenceStyle:(NSInteger) inReferenceStyle;

- (IBAction) rename:(id) sender;

- (IBAction) removePackage:(id) sender;

- (void) removePackageDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo;

- (IBAction) upgradeToDistribution:(id) sender;

// Proxy selectors

- (void) performIfPossibleSelector:(SEL) inSelector withObject:(id) inObject;

	// Project > Settings
	
- (IBAction) changeCertificate:(id) sender;

- (IBAction) removeCertificate:(id) sender;

	// Project > Presentation

- (IBAction) group:(id) sender;

- (IBAction) ungroup:(id) sender;

- (IBAction) merge:(id) sender;

- (IBAction) separate:(id) sender;

	// Packages > Files
	
- (IBAction) addFiles:(id) sender;

- (IBAction) newFolder:(id) sender;

- (IBAction) expandAll:(id) sender;

- (IBAction) expand:(id) sender;

- (IBAction) expandOneLevel:(id) sender;

- (IBAction) setAsDefaultLocation:(id)sender;

// Notifications

- (void) optionKeyDidChange:(NSNotification *) inNotification;

- (void) documentDidChange:(NSNotification *) inNotification;

- (void) packagesDidAdd:(NSNotification *) inNotification;

- (void) packagesDidRemove:(NSNotification *) inNotification;

- (void) documentShouldHideSourceList:(NSNotification *) inNotification;

- (void) documentShouldShowSourceList:(NSNotification *) inNotification;

- (void) documentShouldReloadSourceList:(NSNotification *) inNotification;

- (void) documentStatusDidChange:(NSNotification *) inNotification;

@end
