/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICDocument.h"

#import "ICApplicationController.h"

#import "ICBuildWindowController.h"

#import "ICProjectController.h"

#import "ICPackageController.h"

#import "ICProjectController_Package.h"

#import "ICMultiplePackagesSelectionController.h"

#import "ICUUIDFactory.h"

#import "NSDictionary+MutableDeepCopy.h"

#import "ICPackageUtilities.h"
#import "ICPackageUtilities+Default.h"

#import "ICTitleUtilities.h"

#import "ICImageAndTextCell.h"

#import "ICInstallationTypeUtilities+Extras.h"

#import "ICPreferences+Constants.h"

#import "ICInstallationType+Constants.h"

#import "ICPackageFilesController+Constants.h"

#import "ICDispatchBuildUtilities.h"

#import "NSImage+Tint.h"
#import "NSImage+Composition.h"

#import "ICProjectBuilder+Notifications.h"

#import "ICProjectAssistantController.h"



#define ICDOCUMENT_RIGHTVIEW_MIN_WIDTH		1026

#define SAFENODE(n) 	((ICTreeNode*)((n!=nil)?(n):(tree_)))

@implementation ICDocument

- (id) init
{
    self = [super init];
	
    if (self)
	{
		documentDefaults_=[[NSMutableDictionary alloc] initWithCapacity:10];
    
		[[NSApp delegate] setLaunchedNormally:YES];
	}
	
    return self;
}

- (void) dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	[[NSDistributedNotificationCenter defaultCenter] removeObserver:self];
	
	[currentComponentController_ release];
	
	[documentDefaults_ release];
	
	[dictionary_ release];
	
	[fileBouncer_ release];

	[super dealloc];
}

#pragma mark -

- (BOOL) isFlatDistribution
{
	if (documentType_==ICDOCUMENT_TYPE_DISTRIBUTION)
	{
		NSDictionary * tProjectDictionary;
	
		tProjectDictionary=[self projectSettingsDictionary];
	
		if (tProjectDictionary!=nil)
		{
			NSNumber * tNumber;
			
			tNumber=[tProjectDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT];
			
			if (tNumber==nil || [tNumber intValue]==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_FLAT)
			{
				return YES;
			}
		}
	}

	return NO;
}

- (int) documentType
{
	return documentType_;
}

- (NSString *) referenceFolderPath
{
	NSDictionary * tProjectDictionary;
	NSString * tReferenceFolderPath=nil;
	
	tProjectDictionary=[self projectSettingsDictionary];
	
	if (tProjectDictionary!=nil)
	{
		tReferenceFolderPath=[tProjectDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH];
	}
	
	if (tReferenceFolderPath==nil)
	{
		tReferenceFolderPath=[self folder];
	}
	
	return tReferenceFolderPath;
}

- (NSString *) absoluteBuildPath
{
	NSDictionary * tProjectDictionary;
	NSString * tBuildPath=nil;
	
	tProjectDictionary=[self projectSettingsDictionary];
	
	if (tProjectDictionary!=nil)
	{
		NSDictionary * tBuildPathDictionary;
		
		tBuildPathDictionary=[tProjectDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_PATH];
		
		if (tBuildPathDictionary!=nil)
		{
			tBuildPath=[self pathFromPathDictionary:tBuildPathDictionary];
		}
	}
	
	return tBuildPath;
}

- (NSString *) pathFromPathDictionary:(NSDictionary *) inPathDictionary
{
	if (inPathDictionary!=nil)
	{
		NSString * tPath;
		
		tPath=[inPathDictionary objectForKey:ICDOCUMENT_PATH];
		
		if (tPath!=nil)
		{
			NSNumber * tNumber;
			
			tNumber=[inPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
			
			if (tNumber!=nil)
			{
				int tPathType;
				
				tPathType=[tNumber intValue];
				
				if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
                {
                    tPath=[tPath stringByAbsolutingWithPath:[self folder]];
                }
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
                {
                    tPath=[tPath stringByAbsolutingWithPath:[self referenceFolderPath]];
                }
				
				return tPath;
			}
		}
	}
	
	return nil;
}

- (NSMutableDictionary *) dictionary
{
	return [[dictionary_ retain] autorelease];
}

- (id) rootObjectForKey:(NSString *) inKey
{
	NSMutableDictionary * tMutableDictionary;
	
	tMutableDictionary=[dictionary_ objectForKey:ICDOCUMENT_SHARED_GLOBAL_DATA];
	
	if (tMutableDictionary!=nil)
	{
		return [tMutableDictionary objectForKey:inKey];
	}
	
	return nil;
}

- (void) setRootObject:(id) inObject forKey:(NSString *) inKey
{
	if (inKey!=nil && inObject!=nil)
	{
		NSMutableDictionary * tMutableDictionary;
		
		tMutableDictionary=[dictionary_ objectForKey:ICDOCUMENT_SHARED_GLOBAL_DATA];
		
		if (tMutableDictionary==nil)
		{
			tMutableDictionary=[NSMutableDictionary dictionary];
			
			if (tMutableDictionary!=nil)
			{
				[dictionary_ setObject:tMutableDictionary forKey:ICDOCUMENT_SHARED_GLOBAL_DATA];
			}
		}
		
		if (tMutableDictionary!=nil)
		{
			[tMutableDictionary setObject:inObject forKey:inKey];
			
			[self documentDidChange:nil];
		}
	}
}

- (NSMutableDictionary *) sharedGlobalData
{
	NSMutableDictionary * tMutableDictionary;
	
	tMutableDictionary=[dictionary_ objectForKey:ICDOCUMENT_SHARED_GLOBAL_DATA];
	
	if (tMutableDictionary==nil)
	{
		tMutableDictionary=[NSMutableDictionary dictionary];
		
		[dictionary_ setObject:tMutableDictionary forKey:ICDOCUMENT_SHARED_GLOBAL_DATA];
	}
	
	return tMutableDictionary;
}

- (ICFileBouncer *) fileBouncer
{
	return [[fileBouncer_ retain] autorelease];
}

- (void) updateFileBouncer
{
	if (dictionary_!=nil)
	{
		NSDictionary * tProjectSettingsDictionary;
		
		tProjectSettingsDictionary=[self projectSettingsDictionary];
		
		if (tProjectSettingsDictionary!=nil)
		{
			NSArray * tExcludedFilesArray;
			
			tExcludedFilesArray=[tProjectSettingsDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES];
			
			if (tExcludedFilesArray!=nil)
			{
				if (fileBouncer_==nil)
				{
					fileBouncer_=[[ICFileBouncer alloc] initWithBlackList:tExcludedFilesArray];
				}
				else
				{
					[fileBouncer_ setBlackList:tExcludedFilesArray];
				}
			}
		}
	}
}

- (NSArray *) packagesArray
{
	return [dictionary_ objectForKey:ICDOCUMENT_PACKAGES];
}

- (NSDictionary *) projectSettingsDictionary
{
	NSDictionary * tProjectDictionary=nil;
	
	tProjectDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT];
		
	if (tProjectDictionary!=nil)
	{
		return [tProjectDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS];
	}
	
	return nil;
}

- (NSDictionary *) projectPresentationDictionary
{
	NSDictionary * tProjectDictionary;
			
	tProjectDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT];
			
	if (tProjectDictionary!=nil)
	{
		return [tProjectDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION];
	}
	
	return nil;
}

- (NSMutableDictionary *) dictionaryOfPackageWithUUID:(NSString *) inUUID
{
	if (inUUID!=nil)
	{
		NSArray * tPackagesArray;
		
		tPackagesArray=[self packagesArray];
			
		if (tPackagesArray!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[tPackagesArray objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSMutableDictionary * tPackageDictionary;
				
				while (tPackageDictionary=[tEnumerator nextObject])
				{
					NSString * tPackageUUID;
					
					tPackageUUID=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID];
					
					if (tPackageUUID!=nil && [inUUID isEqualToString:tPackageUUID]==YES)
					{
						return tPackageDictionary;
					}
				}
			}
		}
	}
	
	return nil;
}

- (id) documentDefaultForKey:(NSString *) inKey
{
	id tObject=nil;
	
	if (inKey!=nil)
	{
		tObject=[documentDefaults_ objectForKey:inKey];
	}
	
	return tObject;
}

- (void) setDocumentDefault:(id) inObject forKey:(NSString *) inKey
{
	if (inKey!=nil && inObject!=nil)
	{
		[documentDefaults_ setObject:inObject forKey:inKey];
	}
}

- (void) removeDocumentDefaultForKey:(NSString *) inKey
{
	if (inKey!=nil)
	{
		[documentDefaults_ removeObjectForKey:inKey];
	}
}

#pragma mark -

- (NSString *)windowNibName
{
    return @"ICDocument";
}

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
    //NSLog(@"windowControllerDidLoadNib");
	
	[super windowControllerDidLoadNib:aController];
    
	
	
	nibDidLoad_=YES;
	
	defaults_=[NSUserDefaults standardUserDefaults];
	
	fileManager_=[NSFileManager defaultManager];
	
	// Set the bottom bar
	
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5

	[[IBsourceListView_ window] setContentBorderThickness:33.0f forEdge:NSMinYEdge];

#endif
	
	[IBbuildStatusLabel_ setHidden:YES];
	
	[IBbuildStatusProgressIndicator_ setHidden:YES];
	
	[IBbuildStatusProgressIndicator_ setUsesThreadedAnimation:YES];
	
	[IBbuildStatusProgressIndicator_ setIndeterminate:NO];
	
	[IBbuildStatusProgressIndicator_ setMinValue:0];
	
	if (documentType_==ICDOCUMENT_TYPE_PACKAGE)
	{
		id tRightView;
		NSRect tFrame;
		NSView * tSuperview;
		NSRect tWindowFrame;
		
		[IBaddPackageButton_ setHidden:YES];
		
		tFrame=[IBbuildStatusLabel_ frame];
		
		tFrame.origin.x=20.0f;
		
		[IBbuildStatusLabel_ setFrame:tFrame];
		
		
		tRightView=[IBrightView_ retain];
		
		[tRightView removeFromSuperview];
		
		tFrame=[IBsplitView_ frame];
		
		tSuperview=[IBsplitView_ superview];
		
		[IBsplitView_ removeFromSuperview];
		
		IBsourceListView_=nil;
		
		[tRightView setFrame:tFrame];
		
		[tSuperview addSubview:tRightView];
		
		
		currentComponentController_=[[ICProjectController_Package projectController] retain];
		
		currentComponentView_=[currentComponentController_ view];
				
		// Resize Window
		
		[[self windowForSheet] setMinSize:NSMakeSize(ICDOCUMENT_RIGHTVIEW_MIN_WIDTH,613)];
		
		tFrame=[[self windowForSheet] frameRectForContentRect:NSMakeRect(0,0,ICDOCUMENT_RIGHTVIEW_MIN_WIDTH,613)];
		
		tWindowFrame=[[self windowForSheet] frame];
		
		tFrame.origin.y=NSMinY(tWindowFrame)+NSHeight(tWindowFrame)-NSHeight(tFrame);
		
		[[self windowForSheet] setMinSize:NSMakeSize(ICDOCUMENT_RIGHTVIEW_MIN_WIDTH,613)];
		
		[[self windowForSheet] setFrame:tFrame display:NO];
		
		// Resize view
				
		[currentComponentView_ setFrame:[IBrightView_ bounds]];
				
		// Initialize View widgets
				
		[currentComponentController_ initializeWithDictionary:dictionary_ forDocument:self];
				
		[IBrightView_ addSubview:currentComponentView_];
	}
	else if (documentType_==ICDOCUMENT_TYPE_DISTRIBUTION)
	{
		if (tree_==nil)
		{
			// A COMPLETER
		}
		else
		{
			NSTableColumn * tTableColumn = nil;
			NSUInteger i,tCount;
			
			tTableColumn = [IBsourceListView_ tableColumnWithIdentifier: @"Name"];
			
			if (tTableColumn!=nil)
			{
				ICImageAndTextCell * tPrototypeTextCell=nil;
			
				tPrototypeTextCell=[[ICImageAndTextCell alloc] initTextCell:@""];
		
				if (tPrototypeTextCell!=nil)
				{
					[tPrototypeTextCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
					
					[tPrototypeTextCell setEditable:YES];
					
					[tPrototypeTextCell setLineBreakMode:NSLineBreakByTruncatingTail];
					
					[tTableColumn setDataCell:tPrototypeTextCell];
					
					[tPrototypeTextCell release];
				}
			
				[IBsourceListView_ setOutlineTableColumn:tTableColumn]; 
			}
			
			[IBsourceListView_ setAutoresizesOutlineColumn:NO];
			
			[IBsourceListView_ reloadData];
			
			tCount=[tree_ numberOfChildren];
			
			for(i=tCount-1;i>0;i--)
			{
				id tItem;
			
				tItem=[tree_ childAtIndex:i];
					
				if (tItem!=nil && [IBsourceListView_ isItemExpanded:tItem]==NO)
				{
					[IBsourceListView_ expandItem:tItem];
				}
			}
			
			// Define which types of pasteboard are accepted
			
			[IBsourceListView_ registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType,
																				 ICDOCUMENT_PROJECT_PACKAGES_TRANSFER_PASTEBOARDTYPE,
																				 nil]];
			
			// Select the Project Row
			
			[IBsourceListView_ selectRowIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
			
			// Register for Notifications
			
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(packagesDidAdd:) 
														 name:ICDOCUMENT_NOTIFICATION_PACKAGESDIDADD 
													   object:self];
														
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(packagesDidRemove:) 
														 name:ICDOCUMENT_NOTIFICATION_PACKAGESDIDREMOVE 
													   object:self];
		
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(documentShouldHideSourceList:) 
														 name:ICDOCUMENT_NOTIFICATION_DOCUMENT_SOURCELIST_SHOULD_HIDE
													   object:self];
												   
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(documentShouldShowSourceList:) 
														 name:ICDOCUMENT_NOTIFICATION_DOCUMENT_SOURCELIST_SHOULD_SHOW
													   object:self];
			
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(documentShouldReloadSourceList:) 
														 name:ICDOCUMENT_NOTIFICATION_DOCUMENT_SOURCELIST_SHOULD_RELOAD
													   object:self];
			
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(optionKeyDidChange:)
														 name:ICWINDOW_NOTIFICATION_OPTIONKEY_DID_CHANGE
													   object:[IBsourceListView_ window]];
		}
	}
	
	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(documentDidChange:)
												 name:ICDOCUMENT_NOTIFICATION_DOCUMENT_DID_CHANGE
											   object:self];

	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(documentStatusDidChange:)
												 name:ICDOCUMENT_NOTIFICATION_STATUS_DID_CHANGE
											   object:self];

	// Register for Builder Notifications
	
	[[NSDistributedNotificationCenter defaultCenter] addObserver:self
														selector:@selector(builderNotification:)
															name:PACKAGES_BUILDER_NOTIFICATION
														  object:nil
														  suspensionBehavior:NSNotificationSuspensionBehaviorDeliverImmediately];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(builderNotification:)
												 name:PACKAGES_BUILDER_NOTIFICATION
											   object:nil];
}

- (NSData *) dataOfType:(NSString *)typeName error:(NSError **)outError
{
    NSData * tData;
	NSString * tErrorString;
	
	tData=[NSPropertyListSerialization dataFromPropertyList:dictionary_
													 format:NSPropertyListXMLFormat_v1_0
										   errorDescription:&tErrorString];
	
	if (tData==nil)
	{
		NSDictionary * tMetadataDictionary=nil;
		
		tMetadataDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[self fileURL],ICDOCUMENT_ERROR_METADATA,nil];
	
		if (tErrorString!=nil)
		{
			NSLog(@"%@",tErrorString);
			
			[tErrorString release];
		}
		
		*outError=[[NSError alloc] initWithDomain:ICDOCUMENT_ERROR_DOMAIN
											 code:ICDOCUMENT_ERROR_UNKNOWN_ERROR
										 userInfo:tMetadataDictionary];
		
		return nil;
	}
	
	return tData;
}

- (BOOL) readFromData:(NSData *) inData ofType:(NSString *) typeName error:(NSError **) outError
{
    NSString * tErrorString;
	NSPropertyListFormat tFormat;
	NSDictionary * tMetadataDictionary=nil;
		
	tMetadataDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[self fileURL],ICDOCUMENT_ERROR_METADATA,nil];
		
	dictionary_=[NSPropertyListSerialization propertyListFromData:inData
												 mutabilityOption:NSPropertyListMutableContainersAndLeaves
														   format:&tFormat
												 errorDescription:&tErrorString];
        
	if (dictionary_==nil)
	{
		if (tErrorString!=nil)
		{
			NSLog(@"%@",tErrorString);
			
			[tErrorString release];
		}
		
		*outError=[[NSError alloc] initWithDomain:ICDOCUMENT_ERROR_DOMAIN
											 code:ICDOCUMENT_ERROR_LOAD_INVALID_DOCUMENT
										 userInfo:tMetadataDictionary];
		
		return NO;
	}
	else
	{
		NSNumber * tNumber;
		
		// Check the version
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_VERSION];
		
		if (tNumber==nil)
		{
			*outError=[[NSError alloc] initWithDomain:ICDOCUMENT_ERROR_DOMAIN
											 code:ICDOCUMENT_ERROR_LOAD_INVALID_DOCUMENT
										 userInfo:tMetadataDictionary];
			
			dictionary_=nil;
			
			return NO;
		}
		else
		{
			int tVersionValue;
			
			tVersionValue=[tNumber intValue];
			
			if (tVersionValue>=ICDOCUMENT_VERSION_NEXT_INCOMPATIBLE_VERSION)
			{
				*outError=[[NSError alloc] initWithDomain:ICDOCUMENT_ERROR_DOMAIN
													 code:ICDOCUMENT_ERROR_LOAD_VERSION_TOO_NEW
												 userInfo:tMetadataDictionary];
			
				dictionary_=nil;
				
				return NO;
			}
		}
		
		// Check the document type
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_TYPE];
		
		if (tNumber==nil)
		{
			*outError=[[NSError alloc] initWithDomain:ICDOCUMENT_ERROR_DOMAIN
											 code:ICDOCUMENT_ERROR_LOAD_INVALID_DOCUMENT
										 userInfo:tMetadataDictionary];
			
			dictionary_=nil;
			
			return NO;
		}
		else
		{
			int tDocumentType;
			
			tDocumentType=[tNumber intValue];
			
			if (tDocumentType==ICDOCUMENT_TYPE_DISTRIBUTION)
			{
				tree_=[ICTreeNode documentTreeNodeWithDictionary:dictionary_];
		
				if (tree_!=nil)
				{
					[dictionary_ retain];
				
					[tree_ retain];
					
					[self updateFileBouncer];
				}
				else
				{
					*outError=[[NSError alloc] initWithDomain:ICDOCUMENT_ERROR_DOMAIN
														 code:ICDOCUMENT_ERROR_UNKNOWN_ERROR
													 userInfo:tMetadataDictionary];
					
					dictionary_=nil;
					
					return NO;
				}
			}
			else if (tDocumentType==ICDOCUMENT_TYPE_PACKAGE)
			{
				[dictionary_ retain];
			}
			else 
			{
				*outError=[[NSError alloc] initWithDomain:ICDOCUMENT_ERROR_DOMAIN
											 code:ICDOCUMENT_ERROR_LOAD_INVALID_DOCUMENT
										 userInfo:tMetadataDictionary];
			
				dictionary_=nil;
				
				return NO;
			}
			
			documentType_=tDocumentType;
		}
	}
	
	return YES;
}

#pragma mark -

- (void)splitView:(NSSplitView *) inSplitView resizeSubviewsWithOldSize:(NSSize)oldSize
{
    NSArray * tSubviews;
    NSView * tViewLeft,* tViewRight;
    NSRect tLeftFrame,tRightFrame;
    NSRect tSplitViewFrame=[inSplitView frame];
    
    tSubviews=[inSplitView subviews];
    
    tViewLeft=[tSubviews objectAtIndex:0];
    
    tLeftFrame=[tViewLeft frame];
    
    tViewRight=[tSubviews objectAtIndex:1];
    
    tRightFrame=[tViewRight frame];
        
	tRightFrame.size.width=NSWidth(tSplitViewFrame)-[inSplitView dividerThickness]-NSWidth(tLeftFrame);
	
	if (NSWidth(tRightFrame)<ICDOCUMENT_RIGHTVIEW_MIN_WIDTH)
	{
		tRightFrame.size.width=ICDOCUMENT_RIGHTVIEW_MIN_WIDTH;
		
		tLeftFrame.size.width=NSWidth(tSplitViewFrame)-[inSplitView dividerThickness]-NSWidth(tRightFrame);
		
		if (NSWidth(tLeftFrame)<0)
		{
			tLeftFrame.size.width=0;
		}
	}
	
	tRightFrame.size.height=NSHeight(tSplitViewFrame);
	
	tRightFrame.origin.y=0;
	
	[tViewRight setFrame:tRightFrame];
    
    tLeftFrame.size.height=NSHeight(tSplitViewFrame);
        
    tLeftFrame.origin.y=0;
        
    [tViewLeft setFrame:tLeftFrame];
	
    [inSplitView adjustSubviews];
}

- (BOOL) splitView:(NSSplitView *)sender canCollapseSubview:(NSView *)subview
{
    return NO;
}

- (CGFloat) splitView:(NSSplitView *) inSplitView constrainMaxCoordinate:(CGFloat)proposedMin ofSubviewAt:(NSInteger)offset
{
    return (NSWidth([inSplitView frame])-(ICDOCUMENT_RIGHTVIEW_MIN_WIDTH+[inSplitView dividerThickness]));
}

#pragma mark -

#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5

- (BOOL) outlineView:(NSOutlineView *) inOutlineView isGroupItem:(id)item
{
	if (inOutlineView==IBsourceListView_)
	{
		int tType;
		
		tType=[NODE_DATA(item) type];
		
		if (tType==kProjectPackagesGroupNode ||
			tType==kImportedPackagesGroupNode ||
			tType==kReferencedPackagesGroupNode)
		{
			return YES;
		}
	}
	
	return NO;
}

#endif

- (id)outlineView:(NSOutlineView *) inOutlineView child:(NSInteger) inIndex ofItem:(id)item
{
    if (inOutlineView==IBsourceListView_ && nibDidLoad_==YES)
	{
		if (tree_!=nil)
		{
			return [SAFENODE(item) childAtIndex:inIndex];
		}
    }
	
    return nil;
}

- (BOOL)outlineView:(NSOutlineView *) inOutlineView isItemExpandable:(id)item
{
    if (inOutlineView==IBsourceListView_)
	{
		return ![NODE_DATA(item) isLeaf];
	}
	
	return NO;
}

- (NSInteger) outlineView:(NSOutlineView *) inOutlineView numberOfChildrenOfItem:(id)item
{
    if (inOutlineView==IBsourceListView_ && nibDidLoad_==YES)
	{
		if (tree_!=nil)
		{
			return [SAFENODE(item) numberOfChildren];
		}
    }
	
    return 0;
}

- (id) outlineView:(NSOutlineView *) inOutlineView objectValueForTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
    if (inOutlineView==IBsourceListView_)
	{
		if (tree_!=nil && inItem!=nil)
		{
			NSString * tColumnIdentifier;
			ICDocumentNodeData * tNodeData;
			int tType;
			
			tColumnIdentifier=[inTableColumn identifier];
			
			tNodeData=(ICDocumentNodeData *) NODE_DATA(inItem);
			
			tType=[tNodeData type];
			
			if ([tColumnIdentifier isEqualToString: @"State"])
			{
				if (tType==kPackageNode)
				{
					// A COMPLETER
				}
			}
			else
			if ([tColumnIdentifier isEqualToString: @"Name"])
			{
				NSDictionary * tPackageDictionary;
				
				switch(tType)
				{
					case kProjectNode:
					
						return NSLocalizedString(@"Project",@"");
					
					case kProjectPackagesGroupNode:
					
						return NSLocalizedString(@"PACKAGES",@"");
					
					case kImportedPackagesGroupNode:
					
						return NSLocalizedString(@"IMPORTED",@"");
					
					case kReferencedPackagesGroupNode:
					
						return NSLocalizedString(@"REFERENCED",@"");
					
					case kPackageNode:
					
						tPackageDictionary=[tNodeData dictionary];
						
						if (tPackageDictionary!=nil)
						{
							NSString * tPackageName;
							
							
							tPackageName=[[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
							
							return tPackageName;
						}
						
						break;
						
				}
			}
		}
	}
	
	return nil;
}

- (void)outlineView:(NSOutlineView *) inOutlineView setObjectValue:(id)object forTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
	if (inOutlineView==IBsourceListView_)
	{
		if (tree_!=nil && inItem!=nil)
		{
			NSString * tColumnIdentifier;
			ICDocumentNodeData * tNodeData;
			int tType;
			
			tColumnIdentifier=[inTableColumn identifier];
			
			tNodeData=(ICDocumentNodeData *) NODE_DATA(inItem);
			
			tType=[tNodeData type];
			
			if ([tColumnIdentifier isEqualToString:@"Name"])
			{
				if (tType==kPackageNode)
				{
					NSMutableDictionary * tPackageDictionary;
					
					tPackageDictionary=[tNodeData dictionary];
						
					if (tPackageDictionary!=nil)
					{
						NSMutableDictionary * tPackageSettingsDictionary;
						
						tPackageSettingsDictionary=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS];
						
						if (tPackageSettingsDictionary!=nil)
						{
							NSString * tPackageName;
							
							tPackageName=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
							
							if (tPackageName==nil || [tPackageName isEqualToString:object]==NO)
							{
								if ([ICPackageUtilities isUniquePackageName:object forPackage:tPackageDictionary withExistingPackages:[self packagesArray]]==YES)
								{
									NSInteger tRowIndex;
									
									[tPackageSettingsDictionary setObject:object forKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
									
									// Sort Packages by name
									
									[tree_ sortPackages];
									
									[IBsourceListView_ reloadData];
									
									// Restore Selection
									
									tRowIndex=[IBsourceListView_ rowForItem:inItem];
				
									[IBsourceListView_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
									
									[self documentDidChange:nil];
									
									return;
								}
								else
								{
									NSBeep();
				
									[IBsourceListView_ editColumn:[IBsourceListView_ columnWithIdentifier:@"Name"] row:[IBsourceListView_ rowForItem:inItem] withEvent:nil select:YES];
								}
							}
						}
					}
				}
			}
		}
	}
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView shouldCollapseItem:(id) inItem
{
	return NO;
}

- (void) outlineView:(NSOutlineView *) inOutlineView willDisplayCell:(NSCell *)cell forTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{
	if (inOutlineView==IBsourceListView_)
	{
		if (tree_!=nil && inItem!=nil)
		{
			NSString * tColumnIdentifier;
			ICDocumentNodeData * tNodeData;
			int tType;
			
			tColumnIdentifier=[inTableColumn identifier];
			
			tNodeData=(ICDocumentNodeData *) NODE_DATA(inItem);
			
			tType=[tNodeData type];
			
			if ([tColumnIdentifier isEqualToString: @"State"])
			{
				if (tType==kPackageNode)
				{
					// A COMPLETER
				}
			}
			else
			if ([tColumnIdentifier isEqualToString: @"Name"])
			{
				NSImage * tImage=nil;
				
				if (tType==kProjectNode)
				{
					static NSImage * sProjectIcon=nil;
							
					if (sProjectIcon==nil)
					{
						sProjectIcon=[[NSImage imageNamed:@"Project16"] retain];
					}
					
					tImage=sProjectIcon;
					
					[(ICImageAndTextCell *)cell setTextColor:[NSColor blackColor]];
				}
				else if (tType==kPackageNode)
				{
					NSMutableDictionary * tPackageDictionary;
					
					tPackageDictionary=[tNodeData dictionary];
						
					[(ICImageAndTextCell *)cell setTextColor:[NSColor blackColor]];
					
					if (tPackageDictionary!=nil)
					{
						NSNumber * tNumber;
						int tPackageType;
						BOOL tRemoved=NO;
						
						if (_cachedRemovedPackagesList!=nil)
						{
							if ([_cachedRemovedPackagesList containsObject:[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID]]==YES)
							{
								tRemoved=YES;
								
								[(ICImageAndTextCell *)cell setTextColor:[NSColor grayColor]];
							}
						}
						else
						{
							[(ICImageAndTextCell *)cell setTextColor:[NSColor blackColor]];
						}
						
						tPackageType=ICDOCUMENT_PACKAGE_TYPE_PROJECT;
						
						tNumber=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
						
						if (tNumber!=nil)
						{
							tPackageType=[tNumber intValue];
						}
						
						if (tPackageType==ICDOCUMENT_PACKAGE_TYPE_PROJECT)
						{
							static NSImage * sPackageProjectIcon=nil;
							
							if (sPackageProjectIcon==nil)
							{
								sPackageProjectIcon=[[NSImage imageNamed:@"package16"] retain];
							}
							
							tImage=sPackageProjectIcon;
						}
						else if (tPackageType==ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
						{
							static NSImage * sPackageReferenceIcon=nil;
							
							if (sPackageReferenceIcon==nil)
							{
								sPackageReferenceIcon=[[NSImage imageNamed:@"packageReference16"] retain];
							}
							
							tImage=sPackageReferenceIcon;
						}
						else if (tPackageType==ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
						{
							static NSImage * sPackageImportedIcon=nil;
							
							if (sPackageImportedIcon==nil)
							{
								sPackageImportedIcon=[[NSImage imageNamed:@"packageImported16"] retain];
							}
							
							tImage=sPackageImportedIcon;
							
							tNumber=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
							
							if (tNumber!=nil)
							{
								if ([tNumber intValue]==ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
								{
									// We need to check the file still exists
									
									NSDictionary * tPathDictionary;
									
									tPathDictionary=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_REFERENCE_PATH];
				
									if (tPathDictionary!=nil)
									{
										NSInteger tPathType=ICDOCUMENT_PATH_TYPE_GLOBAL;
										NSString * tPath;
										BOOL isDirectory;
										
										tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
							
										tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
										
										if (tNumber!=nil)
										{
											tPathType=[tNumber integerValue];
										}
										
										if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
										{
											tPath=[tPath stringByAbsolutingWithPath:[self folder]];
										}
										else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
										{
											tPath=[tPath stringByAbsolutingWithPath:[self referenceFolderPath]];
										}
										
										if ([fileManager_ fileExistsAtPath:tPath isDirectory:&isDirectory]==NO || isDirectory==YES)
										{
											[(ICImageAndTextCell *)cell setTextColor:[NSColor redColor]];
											
											tImage=[tImage tintWithColor:[NSColor redColor]];
											
											tRemoved=NO;
										}
									}
								}
							}
							
							
						}
						
						if (tRemoved==YES)
						{
							tImage=[tImage imageWithAlpha:0.5f];
						}
					}
				}
				else
				{
					[(ICImageAndTextCell *)cell setTextColor:[NSColor blackColor]];
				}
				
				[(ICImageAndTextCell*)cell setImage: tImage];
			}
		}
	}
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView shouldEditTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{
	if (inOutlineView==IBsourceListView_)
	{
		if (tree_!=nil && inItem!=nil)
		{
			NSString * tColumnIdentifier;
			ICDocumentNodeData * tNodeData;
			int tType;
			
			tColumnIdentifier=[inTableColumn identifier];
			
			tNodeData=(ICDocumentNodeData *) NODE_DATA(inItem);
			
			tType=[tNodeData type];
			
			if ([tColumnIdentifier isEqualToString: @"Name"])
			{
				if (tType==kPackageNode)
				{
					NSDictionary * tPackageDictionary;
					
					tPackageDictionary=[tNodeData dictionary];
						
					if (tPackageDictionary!=nil)
					{
						NSNumber * tNumber;
						int tPackageType;
						
						tPackageType=ICDOCUMENT_PACKAGE_TYPE_PROJECT;
						
						tNumber=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
						
						if (tNumber!=nil)
						{
							tPackageType=[tNumber intValue];
						}
						
						if (tPackageType!=ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
						{
							return YES;
						}
					}
				}
			}
		}
	}
    
    return NO;
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView shouldSelectItem:(id)item
{
	if (inOutlineView==IBsourceListView_)
	{
		int tType;
		
		tType=[NODE_DATA(item) type];
		
		if (tType==kProjectPackagesGroupNode ||
			tType==kImportedPackagesGroupNode ||
			tType==kReferencedPackagesGroupNode)
		{
			return NO;
		}
	}
	
	return YES;
}

- (CGFloat) outlineView:(NSOutlineView *) inOutlineView heightOfRowByItem:(id)item
{
	if (inOutlineView==IBsourceListView_)
	{
		int tType;
		
		tType=[NODE_DATA(item) type];
		
		if (tType==kProjectPackagesGroupNode ||
			tType==kImportedPackagesGroupNode ||
			tType==kReferencedPackagesGroupNode)
		{
			return 20.0f;
		}
	}
	
	return 20.0f;
}
#pragma mark -

- (BOOL) outlineView:(NSOutlineView *) inOutlineView validateAction:(SEL) inSelector
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[inOutlineView selectedRowIndexes];
	
	if (tIndexSet!=nil)
	{
		NSUInteger tSelectedRow;
		
		switch ([tIndexSet count])
		{
			case 0:		// Empty Selection
			
				return NO;
				
			default:
			
				tSelectedRow=[tIndexSet firstIndex];
				
				if (tSelectedRow<2)
				{
					return NO;
				}
				
				return YES;
		}
	}
	
	return NO;
}

- (void) outlineView:(NSOutlineView *) inOutlineView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inConfirmationRequired==YES)
	{
		[self removePackage:nil];
	}
	else
	{
		[self removePackageDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
	}
}

#pragma mark -

- (BOOL) outlineView:(NSOutlineView *) inOutlineView writeItems:(NSArray *) inItems toPasteboard:(NSPasteboard*) inPasteboard
{
	if (inOutlineView==IBsourceListView_)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inItems objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSMutableArray * tMutableArray;
			NSMutableArray * tMutableTransferArray;
			
			tMutableArray=[NSMutableArray array];
			
			tMutableTransferArray=[NSMutableArray array];
			
			if (tMutableArray!=nil && tMutableTransferArray!=nil)
			{
				ICTreeNode * tTreeNode;
			
				while (tTreeNode=[tEnumerator nextObject])
				{
					ICDocumentNodeData * tNodeData;
				
					tNodeData=NODE_DATA(tTreeNode);
				
					if (tNodeData!=nil)
					{
						int tType;
						NSDictionary * tItemDictionary;
						
						tType=[tNodeData type];
						
						switch(tType)
						{
							case kProjectNode:
							case kProjectPackagesGroupNode:
							case kImportedPackagesGroupNode:
							case kReferencedPackagesGroupNode:
							
								return NO;
							
							case kPackageNode:
						
								tItemDictionary=[tNodeData dictionary];
								
								if (tItemDictionary!=nil)
								{
									NSString * tPackageUUID;
									
									tPackageUUID=[tItemDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID];
									
									if (tPackageUUID!=nil)
									{
										[tMutableArray addObject:tPackageUUID];
									}
									else
									{
										return NO;
									}
									
									[tMutableTransferArray addObject:tItemDictionary];
								}
								
								break;
						}
					}
				}
				
				[inPasteboard declareTypes:[NSArray arrayWithObjects:ICDOCUMENT_PROJECT_PACKAGES_PASTEBOARDTYPE,ICDOCUMENT_PROJECT_PACKAGES_TRANSFER_PASTEBOARDTYPE,nil] owner:self];
				
				if ([tMutableArray count]>0)
				{
					[inPasteboard setPropertyList:tMutableArray forType:ICDOCUMENT_PROJECT_PACKAGES_PASTEBOARDTYPE];
				}
				
				if ([tMutableTransferArray count]>0)
				{
					[inPasteboard setPropertyList:tMutableTransferArray forType:ICDOCUMENT_PROJECT_PACKAGES_TRANSFER_PASTEBOARDTYPE];
				}
				
				if ([tMutableArray count]>0 || [tMutableTransferArray count]>0)
				{
					return YES;
				}
			}
		}
	}
	
	return NO;
}

- (NSDragOperation) outlineView:(NSOutlineView *) inOutlineView validateDrop:(id <NSDraggingInfo>) inDraggingInfo proposedItem:(id) inItem proposedChildIndex:(NSInteger) inChildIndex
{
	if (inChildIndex!=NSOutlineViewDropOnItemIndex)
	{
		if (inOutlineView==IBsourceListView_)
		{
			NSPasteboard * tPasteBoard;
				
			tPasteBoard=[inDraggingInfo draggingPasteboard];
			
			if (tPasteBoard!=nil)
			{
				if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
				{
					// We need to check that the plugins are not already in the list
					
					NSArray * tArray;
					
					tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
								
					if (tArray!=nil)
					{
						NSEnumerator * tEnumerator;
			
						tEnumerator=[tArray objectEnumerator];
						
						if (tEnumerator!=nil)
						{
							NSString * tFilePath;
							
							while (tFilePath=[tEnumerator nextObject])
							{
								BOOL isDirectory;
						
								if ([fileManager_ fileExistsAtPath:tFilePath isDirectory:&isDirectory]==NO || isDirectory==YES)
								{
									return NSDragOperationNone;
								}
								else
								{
									// Check it's a xar FLAT package file
									
									if ([ICPackageUtilities isFlatPackageAtPath:tFilePath]==NO)
									{
										return NSDragOperationNone;
									}
								}
							}
							
							[IBsourceListView_ setDropItem:nil dropChildIndex:-1];
							
							return NSDragOperationCopy;
						}
					}
				}
				else if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:ICDOCUMENT_PROJECT_PACKAGES_TRANSFER_PASTEBOARDTYPE]]!=nil)
				{
					if ([[inDraggingInfo draggingSource] window]!=[IBsourceListView_ window])
					{
						// We can't accept drop from ourselves
						
						[IBsourceListView_ setDropItem:nil dropChildIndex:-1];
						
						return NSDragOperationCopy;
					}
				}
			}
		}
	}
	
	return NSDragOperationNone;
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView acceptDrop:(id <NSDraggingInfo>) inDraggingInfo item:(id) inItem childIndex:(NSInteger) inIndex
{
	if (inOutlineView==IBsourceListView_)
	{
		NSPasteboard * tPasteBoard;
				
		tPasteBoard=[inDraggingInfo draggingPasteboard];
		
		if (tPasteBoard!=nil)
		{
			if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
			{
				NSArray * tArray;
					
				tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
							
				if (tArray!=nil)
				{
					BOOL tShowCustomizationDialog=YES;
					id tObject;
					
					tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
					
					tObject=[defaults_ objectForKey: ICPREFERENCES_PACKAGES_SHOW_CUSTOMIZATION_DIALOG];
				
					if (tObject!=nil)
					{
						tShowCustomizationDialog=[defaults_ boolForKey:ICPREFERENCES_PACKAGES_SHOW_CUSTOMIZATION_DIALOG];
					}
		
					if (tShowCustomizationDialog==YES)
					{
						[self beginImportPackagesReferenceSheet:tArray];
					}
					else
					{
						NSInteger tDefaultReferenceStyle;
						
						tDefaultReferenceStyle=[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
						
						[self _importPackages:tArray referenceStyle:tDefaultReferenceStyle];
					}
					
					return YES;
				}
			}
			else if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:ICDOCUMENT_PROJECT_PACKAGES_TRANSFER_PASTEBOARDTYPE]]!=nil)
			{
				NSArray * tArray;
						
				tArray=(NSArray *) [tPasteBoard propertyListForType:ICDOCUMENT_PROJECT_PACKAGES_TRANSFER_PASTEBOARDTYPE];
				
				if (tArray!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[tArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSMutableDictionary * tMutablePackageDictionary;
						NSMutableArray * tItemArray;
						NSMutableIndexSet * tMutableIndexSet;
						
						[IBsourceListView_ deselectAll:self];
						
						tItemArray=[NSMutableArray array];
						
						while (tMutablePackageDictionary=[tEnumerator nextObject])
						{
							NSMutableArray * tPackagesArray;
							NSString * tPackageName;
							ICTreeNode * tPackageNode;
							
							// Set a Unique UUID
							
							[tMutablePackageDictionary setObject:[ICUUIDFactory string] forKey:ICDOCUMENT_PACKAGE_UUID];
							
							// Change Package Name if needed
							
							tPackageName=[[tMutablePackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
							
							tPackagesArray=[dictionary_ objectForKey:ICDOCUMENT_PACKAGES];
							
							if ([ICPackageUtilities isUniquePackageName:tPackageName forPackage:nil withExistingPackages:tPackagesArray]==NO)
							{
								[[tMutablePackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] setObject:[ICPackageUtilities uniquePackageNameWithExistingPackages:tPackagesArray baseName:tPackageName] forKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
							}
							
							tPackageNode=[tree_ addPackageTreeNodeWithDictionary:tMutablePackageDictionary];
					
							if (tPackageNode!=nil)
							{
								[tItemArray addObject:tPackageNode];
							
								[tPackagesArray addObject:tMutablePackageDictionary];
							}
						}
						
						[tree_ sortPackages];
						
						[IBsourceListView_ reloadData];
						
						tMutableIndexSet=[NSMutableIndexSet indexSet];
						
						if (tMutableIndexSet!=nil)
						{
							tEnumerator=[tItemArray objectEnumerator];
							
							if (tEnumerator!=nil)
							{
								ICTreeNode * tPackageNode;
								
								while (tPackageNode=[tEnumerator nextObject])
								{
									NSInteger tRowIndex;
									
									tRowIndex=[IBsourceListView_ rowForItem:tPackageNode];
									
									[tMutableIndexSet addIndex:tRowIndex];
								}
							}
							
							[IBsourceListView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:NO];
						}
						
						// Post Notification
						
						[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_PACKAGESDIDADD
																object:self
															  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:tArray,ICDOCUMENT_NOTIFICATION_PACKAGES_LIST,nil]];
					
						[IBsourceListView_ reloadData];
						
						[self documentDidChange:nil];
						
						return YES;
					}
				}
			}
		}
	}

	return NO;
}

#pragma mark -

- (id) componentControllerForTreeNodeType:(int) inType
{
	switch(inType)
	{
		case kProjectNode:
		
			return [ICProjectController projectController];
	
		case kPackageNode:
		
			return [ICPackageController packageController];
			
			break;
	}
	
	return nil;
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{
    if (editingInstallationTypeChoice_==NO)
	{
		SEL tAction=[inMenuItem action];
		
		if (tAction==@selector(printDocument:))
		{
			return [self canPrint];
		}
		
		if (tAction==@selector(addFiles:) ||
			tAction==@selector(newFolder:) ||
			tAction==@selector(expandAll:) ||
			tAction==@selector(expand:) ||
			tAction==@selector(expandOneLevel:) ||
			tAction==@selector(setAsDefaultLocation:) ||
			
			tAction==@selector(changeCertificate:) ||
			tAction==@selector(removeCertificate:) ||
			
			tAction==@selector(group:) ||
			tAction==@selector(ungroup:) ||
			tAction==@selector(merge:) ||
			tAction==@selector(separate:))
		{
			if ([currentComponentController_ isKindOfClass:[ICComponentController class]]==YES)
			{
				ICSegmentedController * tCurrentSegmentedController;
			
				tCurrentSegmentedController=[((ICComponentController *) currentComponentController_) currentSegmentedController];
			
				if ([tCurrentSegmentedController respondsToSelector:tAction]==YES)
				{
					return [tCurrentSegmentedController validateMenuItem:inMenuItem];
				}
			}
			
			return NO;
		}
		else
		{
			if (tAction==@selector(build:) ||
				tAction==@selector(buildAndRun:) ||
				tAction==@selector(buildAndDebug:))
			{
				return !building_;
			}
			else if (tAction==@selector(upgradeToDistribution:))
			{
				return (documentType_==ICDOCUMENT_TYPE_PACKAGE);
			}
			else if (tAction==@selector(revealInFinder:))
			{
				NSIndexSet * tIndexSet;
				
				tIndexSet=[IBsourceListView_ selectedRowIndexes];
				
				if (tIndexSet!=nil && ([tIndexSet count]>0))
				{
					NSUInteger tIndex;
					
					tIndex=[tIndexSet firstIndex];
			
					while (tIndex!=NSNotFound)
					{
						id tItem;
						ICDocumentNodeData * tNodeData;
				
						tItem=[IBsourceListView_ itemAtRow:tIndex];
			
						tNodeData=(ICDocumentNodeData *) NODE_DATA(tItem);
				
						if ([tNodeData type]!=kPackageNode)
						{
							return NO;
						}
						else
						{
							NSDictionary * tDictionary;
							
							tDictionary=[tNodeData dictionary];
					
							if (tDictionary!=nil)
							{
								NSNumber * tNumber;
								
								tNumber=[tDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
								
								if (tNumber!=nil)
								{
									if ([tNumber intValue]!=ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
									{
										return NO;
									}
								}
								else
								{
									return NO;
								}
							}
						}
						
						tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
					}
					
					return YES;
				}
				
				return NO;
			}
			else if (tAction==@selector(rename:))
			{
				NSIndexSet * tIndexSet;
				
				tIndexSet=[IBsourceListView_ selectedRowIndexes];
				
				if (tIndexSet!=nil && ([tIndexSet count]==1))
				{
					NSUInteger tIndex;
					id tItem;
					ICDocumentNodeData * tNodeData;
					
					tIndex=[tIndexSet firstIndex];
			
					tItem=[IBsourceListView_ itemAtRow:tIndex];
		
					tNodeData=(ICDocumentNodeData *) NODE_DATA(tItem);
			
					if ([tNodeData type]!=kPackageNode)
					{
						return NO;
					}
					else
					{
						NSDictionary * tDictionary;
						
						tDictionary=[tNodeData dictionary];
				
						if (tDictionary!=nil)
						{
							NSNumber * tNumber;
							
							tNumber=[tDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
							
							if (tNumber!=nil && [tNumber intValue]==ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
							{
								return NO;
							}
						}
					}
					
					return YES;
				}
				
				return NO;
			}
			else if (tAction==@selector(duplicate:))
			{
				NSIndexSet * tIndexSet;
				
				tIndexSet=[IBsourceListView_ selectedRowIndexes];
				
				if (tIndexSet!=nil && ([tIndexSet count]>0))
				{
					NSUInteger tIndex;
					
					tIndex=[tIndexSet firstIndex];
			
					while (tIndex!=NSNotFound)
					{
						id tItem;
						ICDocumentNodeData * tNodeData;
				
						tItem=[IBsourceListView_ itemAtRow:tIndex];
			
						tNodeData=(ICDocumentNodeData *) NODE_DATA(tItem);
				
						if ([tNodeData type]!=kPackageNode)
						{
							return NO;
						}
						else
						{
							NSDictionary * tDictionary;
							
							tDictionary=[tNodeData dictionary];
					
							if (tDictionary!=nil)
							{
								NSNumber * tNumber;
								
								tNumber=[tDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
								
								if (tNumber!=nil && [tNumber intValue]!=ICDOCUMENT_PACKAGE_TYPE_PROJECT)
								{
									return NO;
								}
							}
						}
						
						tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
					}
					
					return YES;
				}
				
				return NO;
			}
			else if (tAction==@selector(removePackage:))
			{
				NSIndexSet * tIndexSet;
				
				tIndexSet=[IBsourceListView_ selectedRowIndexes];
				
				if (tIndexSet!=nil && ([tIndexSet count]>0))
				{
					NSUInteger tIndex;
					
					tIndex=[tIndexSet firstIndex];
					
					if (tIndex<2)
					{
						return NO;
					}
					
					return YES;
				}
				
				return NO;
			}
		}
		
		return YES;
	}
	
	return NO;
}

- (void) performIfPossibleSelector:(SEL) inSelector withObject:(id) inObject
{
	ICSegmentedController * tCurrentSegmentedController;
	
	tCurrentSegmentedController=[((ICComponentController *) currentComponentController_) currentSegmentedController];
	
	if ([tCurrentSegmentedController respondsToSelector:inSelector]==YES)
	{
		[tCurrentSegmentedController performSelector:inSelector withObject:inObject];
	}
}

#pragma mark -

- (BOOL) canPrint
{
	return NO;
}

- (NSView *) printableView
{
	return nil;
}

- (void) printOperationDidRun:(NSPrintOperation *) inPrintOperation success:(BOOL) inSuccess contextInfo:(void *) info
{
}

- (IBAction) printDocument:(id) sender
{
	NSPrintOperation * tPrintOperation;
	
	tPrintOperation=[NSPrintOperation printOperationWithView:[self printableView]
												   printInfo:[self printInfo]];
    
	[tPrintOperation runOperationModalForWindow:[self windowForSheet]
									   delegate:self
								 didRunSelector:@selector(printOperationDidRun:success:contextInfo:)
									contextInfo:NULL];
}

#pragma mark -

- (IBAction) revealInFinder:(id) sender
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBsourceListView_ selectedRowIndexes];
	
	if (tIndexSet!=nil && ([tIndexSet count]>0))
	{
		NSMutableArray * tPackages;
		NSUInteger tIndex;
		NSWorkspace * tWorkSpace;
		
		tWorkSpace=[NSWorkspace sharedWorkspace];
		
		tPackages=[dictionary_ objectForKey:ICDOCUMENT_PACKAGES];
		
		tIndex=[tIndexSet firstIndex];
		
		while (tIndex!=NSNotFound)
		{
			id tItem;
			ICDocumentNodeData * tNodeData;
	
			tItem=[IBsourceListView_ itemAtRow:tIndex];

			tNodeData=(ICDocumentNodeData *) NODE_DATA(tItem);
	
			if ([tNodeData type]!=kPackageNode)
			{
				break;
			}
			else
			{
				NSDictionary * tPackageDictionary;
				
				tPackageDictionary=[tNodeData dictionary];
		
				if (tPackageDictionary!=nil)
				{
					NSNumber * tNumber;
					
					tNumber=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
					
					if (tNumber!=nil)
					{
						if ([tNumber intValue]==ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
						{
							NSDictionary * tPathDictionary;
									
							tPathDictionary=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_REFERENCE_PATH];
		
							if (tPathDictionary!=nil)
							{
								NSInteger tPathType=ICDOCUMENT_PATH_TYPE_GLOBAL;
								NSString * tPath;
								BOOL isDirectory;
								
								tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
					
								tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
								
								if (tNumber!=nil)
								{
									tPathType=[tNumber integerValue];
								}
								
								if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
								{
									tPath=[tPath stringByAbsolutingWithPath:[self folder]];
								}
								else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
								{
									tPath=[tPath stringByAbsolutingWithPath:[self referenceFolderPath]];
								}
								
								if ([fileManager_ fileExistsAtPath:tPath isDirectory:&isDirectory]==YES && isDirectory==NO)
								{
									[tWorkSpace selectFile:tPath inFileViewerRootedAtPath:@""];
								}
								else
								{
									NSBeep();
								
									break;
								}
							}
						}
					}
				}
			}
			
			tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
		}
	}
}

- (IBAction) duplicate:(id) sender
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBsourceListView_ selectedRowIndexes];
	
	if (tIndexSet!=nil && ([tIndexSet count]>0))
	{
		NSMutableArray * tPackages;
		NSMutableArray * tNewPackageNodesArray;
		NSMutableArray * tNewPackageDictionaryArray;
		
		tPackages=[dictionary_ objectForKey:ICDOCUMENT_PACKAGES];
		
		tNewPackageNodesArray=[NSMutableArray array];
		
		tNewPackageDictionaryArray=[NSMutableArray array];
		
		if (tNewPackageNodesArray!=nil && tPackages!=nil && tNewPackageDictionaryArray!=nil)
		{
			NSUInteger tIndex;
			
			tIndex=[tIndexSet firstIndex];
		
			while (tIndex!=NSNotFound)
			{
				id tItem;
				ICDocumentNodeData * tNodeData;
		
				tItem=[IBsourceListView_ itemAtRow:tIndex];

				tNodeData=(ICDocumentNodeData *) NODE_DATA(tItem);
		
				if ([tNodeData type]!=kPackageNode)
				{
					break;
				}
				else
				{
					NSDictionary * tDictionary;
					
					tDictionary=[tNodeData dictionary];
			
					if (tDictionary!=nil)
					{
						NSMutableDictionary * tNewPackageDictionary;
						
						tNewPackageDictionary=[[tDictionary mutableDeepCopy] autorelease];
						
						if (tNewPackageDictionary!=nil)
						{
							NSMutableDictionary * tPackageSettings;
							ICTreeNode * tPackageNode;
							
							// Set a new Package UUID
							
							[tNewPackageDictionary setObject:[ICUUIDFactory string] forKey:ICDOCUMENT_PACKAGE_UUID];
							
							tPackageSettings=[tNewPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS];
							
							if (tPackageSettings!=nil)
							{
								NSString * tName;
								NSString * tBaseName=nil;
								NSString * tBaseBundleIdentifier;
								NSString * tBundleIdentifier=nil;
								
								// Set a unique name for the package
								
								tName=[tPackageSettings objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
								
								if (tName!=nil && [tName length]>0)
								{
									NSRange tRange;
									NSUInteger tLength;
									
									tBaseName=[tName stringByAppendingString:NSLocalizedString(@" copy",@"")];
									
									tRange=[tName rangeOfString:NSLocalizedString(@" copy",@"") options:NSCaseInsensitiveSearch+NSBackwardsSearch];
									
									if (tRange.location!=NSNotFound)
									{
										tLength=[tName length];
										
										tBaseName=[[tName copy] autorelease];
										
										if ((tRange.location+tRange.length)<tLength)
										{
											tBaseName=[tName stringByAppendingString:NSLocalizedString(@" copy",@"")];
											
											if ([tName characterAtIndex:tRange.location+tRange.length]==' ')
											{
												NSString * tEndingString;
												
												tRange.length+=1;
												
												tEndingString=[tName substringFromIndex:tRange.location+tRange.length];
												
												tLength=[tEndingString length];
												
												if (tLength>0)
												{
													NSUInteger tIndex;
												
													tIndex=0;
													
													while (tIndex<tLength)
													{
														unichar tCharacter;
														
														tCharacter=[tEndingString characterAtIndex:tIndex];
														
														if (tCharacter<'0' || tCharacter>'9')
														{
															break;
														}
														
														tIndex++;
													}
													
													if (tIndex==tLength)
													{
														tBaseName=[tName substringToIndex:tRange.location+tRange.length-1];
													}
												}
											}
										}
									}
								}
								else
								{
									tBaseName=NSLocalizedString(@"untitled package",@"No comment");
								}
								
								tName=[ICPackageUtilities uniquePackageNameWithExistingPackages:tPackages baseName:tBaseName];
								
								if (tName!=nil)
								{
									[tPackageSettings setObject:tName forKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
								}
								
								// Change the bundle identifier
							
								tBaseBundleIdentifier=[ICPackageUtilities packageIdentifierFromName:tBaseName];
								
								if (tBaseBundleIdentifier!=nil && [tBaseBundleIdentifier length]>0)
								{
									tBundleIdentifier=[ICPackageUtilities uniquePackageIdentifierWithExistingPackages:tPackages originalIdentifier:tBaseBundleIdentifier];
								}
								
								if (tBundleIdentifier!=nil)
								{
									[tPackageSettings setObject:tBundleIdentifier forKey:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER];
								}
								
							}
							else
							{
								break;
							}
							
							// Insert duplicate package
							
							tPackageNode=[tree_ addPackageTreeNodeWithDictionary:tNewPackageDictionary];
							
							if (tPackageNode!=nil)
							{
								[tNewPackageNodesArray addObject:tPackageNode];
								
								[tPackages addObject:tNewPackageDictionary];
							
								[tNewPackageDictionaryArray addObject:tNewPackageDictionary];
							}
							else
							{
								break;
							}
						}
						else
						{
							break;
						}
					}
				}
				
				tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
			}
			
			if ([tNewPackageDictionaryArray count]>0)
			{
				NSMutableIndexSet * tMutableIndexSet;
				
				//[IBsourceListView_ deselectAll:self];
							
				[IBsourceListView_ reloadData];
				
				tMutableIndexSet=[NSMutableIndexSet indexSet];
				
				if (tMutableIndexSet!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[tNewPackageNodesArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						ICTreeNode * tPackageNode;
						
						while (tPackageNode=[tEnumerator nextObject])
						{
							NSInteger tRowIndex;
				
							tRowIndex=[IBsourceListView_ rowForItem:tPackageNode];
							
							[tMutableIndexSet addIndex:tRowIndex];
						}
					}
				
					[IBsourceListView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:NO];
				}
				
				// Post Notification
				
				[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_PACKAGESDIDADD
														object:self
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:tNewPackageDictionaryArray,ICDOCUMENT_NOTIFICATION_PACKAGES_LIST,nil]];
			
				[IBsourceListView_ reloadData];
				
				[self documentDidChange:nil];
			}
		}
	}
}

- (IBAction) rename:(id) sender
{
	[IBsourceListView_ editColumn:[IBsourceListView_ columnWithIdentifier:@"Name"] row:[IBsourceListView_ selectedRow] withEvent:nil select:YES];
}

#pragma mark -

- (IBAction) changeCertificate:(id) sender
{
	[self performIfPossibleSelector:@selector(changeCertificate:) withObject:sender];
}

- (IBAction) removeCertificate:(id) sender
{
	[self performIfPossibleSelector:@selector(removeCertificate:) withObject:sender];
}

- (IBAction) group:(id) sender
{
	[self performIfPossibleSelector:@selector(group:) withObject:sender];
}


- (IBAction) ungroup:(id) sender
{
	[self performIfPossibleSelector:@selector(ungroup:) withObject:sender];
}

- (IBAction) merge:(id) sender
{
	[self performIfPossibleSelector:@selector(merge:) withObject:sender];
}

- (IBAction) separate:(id) sender
{
	[self performIfPossibleSelector:@selector(separate:) withObject:sender];
}


- (IBAction) addFiles:(id) sender
{
	[self performIfPossibleSelector:@selector(addFiles:) withObject:sender];
}

- (IBAction) newFolder:(id) sender
{
	[self performIfPossibleSelector:@selector(newFolder:) withObject:sender];
}

- (IBAction) expandAll:(id) sender
{
	[self performIfPossibleSelector:@selector(expandAll:) withObject:sender];
}

- (IBAction) expand:(id) sender
{
	[self performIfPossibleSelector:@selector(expand:) withObject:sender];
}

- (IBAction) expandOneLevel:(id) sender
{
	[self performIfPossibleSelector:@selector(expandOneLevel:) withObject:sender];
}

- (IBAction) setAsDefaultLocation:(id) sender
{
	[self performIfPossibleSelector:@selector(setAsDefaultLocation:) withObject:sender];
}

#pragma mark -

- (IBAction) showHideBuildWindow:(id) sender
{
    if ([[buildWindowController_ window] isVisible]==YES)
    {
        [[buildWindowController_ window] orderOut:self];
    }
    else
    {
        [buildWindowController_ showWindow:self cleanWindow:NO];
    }
}

- (IBAction) build:(id) sender
{
	[self _buildWithSearchLocationEnabled:YES];
}

- (IBAction) buildAndRun:(id) sender
{
	if ([self _buildWithSearchLocationEnabled:YES]==YES)
    {
		launchAfterBuild_=YES;
	}
}

- (IBAction) buildAndDebug:(id) sender
{
    if ([self _buildWithSearchLocationEnabled:NO]==YES)
    {
		launchAfterBuild_=YES;
	}
}

- (BOOL) _buildWithSearchLocationEnabled:(BOOL) inLocationEnabled
{
    BOOL tResult;
	
	[IBbuildStatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Building...",@"Build",@"")];
	
	[IBbuildStatusProgressIndicator_ setIndeterminate:NO];
	
	[IBbuildStatusProgressIndicator_ setDoubleValue:0];
	
	[IBbuildStatusProgressIndicator_ setMaxValue:4];
	
	
	
	tResult=[self buildAsynchronousWithSearchLocationEnabled:inLocationEnabled];
	
	if (tResult==YES)
	{
		NSInteger tShowBuildWindowBehavior;
    
		// Notify the Build Window
    
		[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION_BUILD_DID_START object:self];
		
		tShowBuildWindowBehavior=[defaults_ integerForKey:ICPREFERENCES_BUILD_SHOW_WINDOW];
    
		if (tShowBuildWindowBehavior==ICPREFERENCES_BUILD_SHOWWINDOW_ALWAYS)
		{
			[buildWindowController_ showWindow:self cleanWindow:YES];
		}
	}
	
	return tResult;
}

- (BOOL) buildAsynchronousWithSearchLocationEnabled:(BOOL) inLocationEnabled
{
    NSString * tProjectPath;
	int tResult;
	
    launchAfterBuild_=NO;
    
    tProjectPath=[self fileName];
    
    if ([self isDocumentEdited]==YES)
    {
        NSInteger tBehavior;
        
        tBehavior=[defaults_ integerForKey:ICPREFERENCES_BUILD_UNSAVED_PROJECT];
        
        switch(tBehavior)
        {
            case ICPREFERENCES_BUILD_UNSAVEDPROJECT_ASKBEFOREBUILD:
                {
					NSInteger tReturnCode;
					NSString * tAlertTitle;
					
					tAlertTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Do you want to save the changes you made in the project \"%@\" before building it?",@"Build",@"No comment"),[[[self fileName] lastPathComponent] stringByDeletingPathExtension]];
					
					tReturnCode=NSRunAlertPanel(tAlertTitle,@"",NSLocalizedString(@"Save",@"No comment"),NSLocalizedString(@"Don't Save",@"No comment"),NSLocalizedString(@"Cancel",@"No comment"));
					
					switch(tReturnCode)
					{
						case NSAlertDefaultReturn:		// Save
						
							[self saveDocument:nil];
							
							break;
							
						case NSAlertAlternateReturn:	// Don't Save
						
							tProjectPath=[self temporaryProjectPath];
							
							break;
							
						case NSAlertOtherReturn:		// Cancel
							
							return NO;
					}
				}
                
                break;
				
            case ICPREFERENCES_BUILD_UNSAVEDPROJECT_ALWAYSSAVE:
                
                [self saveDocument:nil];
				
                break;
        
            case ICPREFERENCES_BUILD_UNSAVEDPROJECT_NEVERSAVE:
                
                tProjectPath=[self temporaryProjectPath];
                
                break;
        }
    }
    
    if (tProjectPath==nil)
	{
		return NO;
	}
	
	tResult=[ICDispatchBuildUtilities  buildProjectAtPath:tProjectPath withNotificationPath:[self fileName] locationEnabled:inLocationEnabled];
	
	switch(tResult)
	{
		case -1:
	
			NSBeep();
        
			NSBeginAlertSheet(NSLocalizedStringFromTable(@"No signal from the packages_dispatcher process",@"Build",@"No comment"),
							  nil,
							  nil,
							  nil,
							  [self windowForSheet],
							  nil,
							  nil,
							  nil,
							  NULL,
							  NSLocalizedStringFromTable(@"The packages_dispatcher process is not responding. Packages can't build any project when this process is not running.",@"Build",@"No comment"));
        
			return NO;
	
		case NO:
		
			NSBeep();
                    
			return NO;
		
		case YES:
		
			building_=YES;
			
			break;
	}
    
    return YES;
}

- (NSString *) temporaryProjectPath
{
    NSString * tTemporaryProjectPath=nil;
    NSString * tTempPath;
    BOOL isDirectory;
    NSFileManager * tFileManager;
    NSMutableDictionary * tNotificationInfo;
    NSString * tProjectPath;
    NSString * tExplanationString=nil;
	
    tProjectPath=[self fileName];
    
    tTempPath=[[NSString stringWithString:ICPREFERENCES_DEFAULT_TEMPORARY_BUILD_LOCATION] stringByAppendingPathComponent:[NSString stringWithFormat:@"%d",getuid()]];
    
    tFileManager=[NSFileManager defaultManager];
    
    tNotificationInfo=[NSMutableDictionary dictionary];
    
    [tNotificationInfo setObject:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]] forKey:@"Process ID"];
    
    [tNotificationInfo setObject:tProjectPath forKey:@"Project Path"];
    
    if ([tFileManager fileExistsAtPath:tTempPath isDirectory:&isDirectory]==NO)
    {
        if ([tFileManager createDirectoryAtPath:tTempPath attributes:nil]==NO)
        {
            tExplanationString=NSLocalizedStringFromTable(@"Creation of temporary folder failed.",@"Build",@"");
			
			goto bail;
        }
    }
    else
    {
        if (isDirectory==NO)
        {
            if ([tFileManager removeFileAtPath:tTempPath handler:nil]==NO)
            {
                 tExplanationString=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Removing file at path '%@' failed.",@"Build",@""),tTempPath];
					
				goto bail;
            }
            else
            {
                if ([tFileManager createDirectoryAtPath:tTempPath attributes:nil]==NO)
                {
					tExplanationString=NSLocalizedStringFromTable(@"Creation of temporary folder failed.",@"Build",@"");
					
					goto bail;
                }
            }
        }
    }

    tTemporaryProjectPath=[NSString stringWithFormat:@"%@/%d/%@",ICPREFERENCES_DEFAULT_TEMPORARY_BUILD_LOCATION,getuid(),[[self fileName] lastPathComponent]];
    
    if (tTemporaryProjectPath!=nil)
	{
		NSData * tData;
		NSString * tErrorString;
		
		tData=[NSPropertyListSerialization dataFromPropertyList:dictionary_
														 format:NSPropertyListXMLFormat_v1_0
											   errorDescription:&tErrorString];
		
		if (tData!=nil)
		{
			NSError * tError;
			
			if ([tData writeToFile:tTemporaryProjectPath options:NSAtomicWrite error:&tError]==NO)
			{
				tExplanationString=[tError localizedDescription];
			
				goto bail;
			}
		}
		else
		{
			tExplanationString=[tErrorString autorelease];
		
			goto bail;
		}
	}
    
    return tTemporaryProjectPath;
	
bail:
	
	NSBeep();
	  
	NSBeginAlertSheet(NSLocalizedStringFromTable(@"Packages is unable to build the project because a temporary project file can not be created.",@"Build",@""),
					  nil,
					  nil,
					  nil,
					  [self windowForSheet],
					  nil,
					  nil,
					  nil,
					  NULL,
					  tExplanationString);
	
	return nil;
}

- (IBAction) clean:(id) sender
{
    // Display the Warning sheet
    
    NSBeginAlertSheet(NSLocalizedStringFromTable(@"clean title",@"Build",@"No comment"),
                      NSLocalizedStringFromTable(@"Clean",@"Build",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [self windowForSheet],
                      self,
                      @selector(cleanSheetDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedStringFromTable(@"clean message",@"Build",@"No comment"));
}

- (void) cleanSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		// Notify the Build Window
    
		[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION_CLEAN_DID_START object:self];
	
        [self cleanBuildFolder];
    }
}

- (void) cleanBuildFolder
{
	NSString * tBuildPath;
	NSDictionary * tDictionary;
	BOOL tCleanSucceeded=YES;
	id tMetadata=nil;
	NSInteger tErrorCounts=0;
	
	[IBbuildStatusLabel_ setStringValue:NSLocalizedStringFromTable(@"Cleaning...",@"Build",@"")];
	
	[IBbuildStatusProgressIndicator_ setIndeterminate:YES];
	
	tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
														   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
														   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
														   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN],IC_BUILDER_INFORMATION_STEP,
														   [NSNumber numberWithUnsignedLong:IC_BUILDER_BUILD_STATE_BEGIN],IC_BUILDER_INFORMATION_STATE,
														   nil];
	
	[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
														object:nil 
													  userInfo:tDictionary];
	
	tBuildPath=[self absoluteBuildPath];
	
	if (tBuildPath!=nil)
	{
		BOOL isDirectory;
		
		if ([fileManager_ fileExistsAtPath:tBuildPath isDirectory:&isDirectory]==YES && isDirectory==YES)
		{
			NSDictionary * tProjectDictionary;
			NSString * tProjectName;
			BOOL tSuccess;
			
			tProjectDictionary=[self projectSettingsDictionary];
			
			tProjectName=[tProjectDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_NAME];
			
			if (tProjectName!=nil)
			{
				NSString * tExtension=nil;
				
				if (documentType_==ICDOCUMENT_TYPE_DISTRIBUTION)
				{
					tExtension=@"mpkg";
				}
				else if (documentType_==ICDOCUMENT_TYPE_PACKAGE)
				{
					tExtension=@"pkg";
				}
				
				if (tExtension!=nil)
				{
					NSString * tPath;
					
					tPath=[[tBuildPath stringByAppendingPathComponent:tProjectName] stringByAppendingPathExtension:tExtension];
					
					if (tPath!=nil)
					{
						if ([fileManager_ fileExistsAtPath:tPath]==YES)
						{
							tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tPath,IC_BUILDER_INFORMATION_FILE_PATH,nil];
							
							tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																				   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																				   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																				   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																				   [NSNumber numberWithUnsignedLong:IC_BUILDER_BUILD_STATE_BEGIN],IC_BUILDER_INFORMATION_STATE,
																				   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																				   nil];
	
							[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
													object:nil 
												  userInfo:tDictionary];

							tSuccess=[fileManager_ removeFileAtPath:tPath handler:self];
							
							if (tSuccess==NO)
							{
								NSBeep();

								tCleanSucceeded=NO;
								
								tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tPath,IC_BUILDER_INFORMATION_FILE_PATH,
																					 cleanErrorString_,IC_BUILDER_FAILURE_REASON,
																					 nil];
							
								[cleanErrorString_ release];
								
								cleanErrorString_=nil;
							
								tErrorCounts++;
							}
							else
							{
								tMetadata=nil;
							}
							
								
							tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																				   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																				   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																				   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																				   [NSNumber numberWithUnsignedLong:(tSuccess==YES) ? IC_BUILDER_BUILD_STATE_END_SUCCESS : IC_BUILDER_BUILD_STATE_END_FAILURE],IC_BUILDER_INFORMATION_STATE,
																				   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																				   nil];
	
							[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
																				object:nil 
																			  userInfo:tDictionary];
						}
					}
				}
			}
			
			if (documentType_==ICDOCUMENT_TYPE_DISTRIBUTION)
			{
				NSArray * tArray;
				
				tArray=[dictionary_ objectForKey:ICDOCUMENT_PACKAGES];
				
				if (tArray!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[tArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSDictionary * tPackageDictionary;
						BOOL shouldCleanHTTPFolder=NO;
						BOOL shouldCleanXDISCFolder=NO;
						BOOL shouldCleanPackagesFolder=NO;
						NSString * tFolderPath;
						
						while (tPackageDictionary=[tEnumerator nextObject])
						{
							NSDictionary * tSettingsDictionary;
							
							tSettingsDictionary=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS];
	
							if (tSettingsDictionary!=nil)
							{
								NSNumber * tNumber;
								int tPackageType=ICDOCUMENT_PACKAGE_TYPE_PROJECT;
								
								tNumber=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
		
								if (tNumber!=nil)
								{
									tPackageType=[tNumber intValue];
								}
								
								if (tPackageType!=ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
								{
									int tLocationType=ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED;
									
									tNumber=[tSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION];
				
									if (tNumber!=nil)
									{
										tLocationType=[tNumber intValue];
									}
									
									if (tLocationType!=ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED)
									{
										if (tLocationType==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_HTTP_URL)
										{
											shouldCleanHTTPFolder=YES;
										}
										else if (tLocationType==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_REMOVABLE_MEDIA)
										{
											shouldCleanXDISCFolder=YES;
										}
										else if (tLocationType==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_CUSTOM_PATH)
										{
											NSString * tLocationPath;
												
											tLocationPath=[tSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
									
											if (tLocationPath!=nil)
											{
												
												
												if ([tLocationPath hasPrefix:@"file:"]==YES)
												{
													tLocationPath=[tLocationPath substringFromIndex:5];
												}
												
												if ([tLocationPath length]>0)
												{
													if ([tLocationPath isEqualToString:@"./"]==YES ||
														[tLocationPath isEqualToString:@"../"]==YES)
													{
														NSString * tPackageName=nil;
														
														if (tPackageType==ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
														{
															NSDictionary * tPathDictionary;
					
															tPathDictionary=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_REFERENCE_PATH];
					
															if (tPathDictionary!=nil)
															{
																NSString * tImportedPackagePath;
																
																tImportedPackagePath=[self pathFromPathDictionary:tPathDictionary]; 
																
																if (tImportedPackagePath!=nil)
																{
																	// Get the Package Name
																	
																	tPackageName=[tImportedPackagePath lastPathComponent];
																}
															}
														}
														else if (tPackageType==ICDOCUMENT_PACKAGE_TYPE_PROJECT)
														{
															tPackageName=[[[tSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME] stringByDeletingPathExtension] stringByAppendingPathExtension:@"pkg"];
														}
														
														if (tPackageName!=nil)
														{
															NSString * tRelativePath;
															
															tRelativePath=[tLocationPath stringByAppendingPathComponent:tPackageName];
															
															if (tRelativePath!=nil)
															{
																NSString * tPackagePath;
																
																tPackagePath=[[tBuildPath stringByAppendingPathComponent:tRelativePath] stringByStandardizingPath];
																
																if (tPackagePath!=nil && [fileManager_ fileExistsAtPath:tPackagePath]==YES)
																{
																	tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tPackagePath,IC_BUILDER_INFORMATION_FILE_PATH,nil];
																	
																	tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																														   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																														   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																														   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																														   [NSNumber numberWithUnsignedLong:IC_BUILDER_BUILD_STATE_BEGIN],IC_BUILDER_INFORMATION_STATE,
																														   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																														   nil];
											
																	[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
																							object:nil 
																						  userInfo:tDictionary];

																	tSuccess=[fileManager_ removeFileAtPath:tPackagePath handler:self];
																	
																	if (tSuccess==NO)
																	{
																		if (tCleanSucceeded==YES)
																		{
																			NSBeep();

																			tCleanSucceeded=NO;
																		}
																		
																		tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tPackagePath,IC_BUILDER_INFORMATION_FILE_PATH,
																															 cleanErrorString_,IC_BUILDER_FAILURE_REASON,
																															 nil];
																	
																		[cleanErrorString_ release];
																		
																		cleanErrorString_=nil;
																	
																		tErrorCounts++;
																	}
																	else
																	{
																		tMetadata=nil;
																	}
																	
																		
																	tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																														   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																														   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																														   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																														   [NSNumber numberWithUnsignedLong:(tSuccess==YES) ? IC_BUILDER_BUILD_STATE_END_SUCCESS : IC_BUILDER_BUILD_STATE_END_FAILURE],IC_BUILDER_INFORMATION_STATE,
																														   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																														   nil];
											
																	[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
																														object:nil 
																													  userInfo:tDictionary];
																}
															}
														}
													}
													else	
													{
														shouldCleanPackagesFolder=YES;
													}
												}
											}
										}
									}
								}
							}
						}
						
						if (shouldCleanPackagesFolder==YES)
						{
							tFolderPath=[tBuildPath stringByAppendingPathComponent:@"packages"];
							
							if (tFolderPath!=nil && [fileManager_ fileExistsAtPath:tFolderPath isDirectory:&isDirectory]==YES && isDirectory==YES)
							{
								tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tFolderPath,IC_BUILDER_INFORMATION_FILE_PATH,nil];
								
								tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																					   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																					   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																					   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																					   [NSNumber numberWithUnsignedLong:IC_BUILDER_BUILD_STATE_BEGIN],IC_BUILDER_INFORMATION_STATE,
																					   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																					   nil];
		
								[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
														object:nil 
													  userInfo:tDictionary];

								tSuccess=[fileManager_ removeFileAtPath:tFolderPath handler:self];
								
								if (tSuccess==NO)
								{
									if (tCleanSucceeded==YES)
									{
										NSBeep();

										tCleanSucceeded=NO;
									}
									
									tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tFolderPath,IC_BUILDER_INFORMATION_FILE_PATH,
																						 cleanErrorString_,IC_BUILDER_FAILURE_REASON,
																						 nil];
								
									[cleanErrorString_ release];
									
									cleanErrorString_=nil;
								
									tErrorCounts++;
								}
								else
								{
									tMetadata=nil;
								}
								
									
								tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																					   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																					   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																					   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																					   [NSNumber numberWithUnsignedLong:(tSuccess==YES) ? IC_BUILDER_BUILD_STATE_END_SUCCESS : IC_BUILDER_BUILD_STATE_END_FAILURE],IC_BUILDER_INFORMATION_STATE,
																					   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																					   nil];
		
								[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
																					object:nil 
																				  userInfo:tDictionary];
							}
						}
						
						if (shouldCleanHTTPFolder==YES)
						{
							tFolderPath=[tBuildPath stringByAppendingPathComponent:@"http"];
							
							if (tFolderPath!=nil && [fileManager_ fileExistsAtPath:tFolderPath isDirectory:&isDirectory]==YES && isDirectory==YES)
							{
								tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tFolderPath,IC_BUILDER_INFORMATION_FILE_PATH,nil];
								
								tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																					   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																					   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																					   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																					   [NSNumber numberWithUnsignedLong:IC_BUILDER_BUILD_STATE_BEGIN],IC_BUILDER_INFORMATION_STATE,
																					   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																					   nil];
		
								[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
														object:nil 
													  userInfo:tDictionary];

								tSuccess=[fileManager_ removeFileAtPath:tFolderPath handler:self];
								
								if (tSuccess==NO)
								{
									if (tCleanSucceeded==YES)
									{
										NSBeep();

										tCleanSucceeded=NO;
									}
									
									tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tFolderPath,IC_BUILDER_INFORMATION_FILE_PATH,
																						 cleanErrorString_,IC_BUILDER_FAILURE_REASON,
																						 nil];
								
									[cleanErrorString_ release];
									
									cleanErrorString_=nil;
								
									tErrorCounts++;
								}
								else
								{
									tMetadata=nil;
								}
								
									
								tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																					   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																					   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																					   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																					   [NSNumber numberWithUnsignedLong:(tSuccess==YES) ? IC_BUILDER_BUILD_STATE_END_SUCCESS : IC_BUILDER_BUILD_STATE_END_FAILURE],IC_BUILDER_INFORMATION_STATE,
																					   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																					   nil];
		
								[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
																					object:nil 
																				  userInfo:tDictionary];
							}
						}
						
						if (shouldCleanXDISCFolder==YES)
						{
							tFolderPath=[tBuildPath stringByAppendingPathComponent:@"x-disc"];
							
							if (tFolderPath!=nil && [fileManager_ fileExistsAtPath:tFolderPath isDirectory:&isDirectory]==YES && isDirectory==YES)
							{
								tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tFolderPath,IC_BUILDER_INFORMATION_FILE_PATH,nil];
								
								tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																					   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																					   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																					   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																					   [NSNumber numberWithUnsignedLong:IC_BUILDER_BUILD_STATE_BEGIN],IC_BUILDER_INFORMATION_STATE,
																					   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																					   nil];
		
								[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
														object:nil 
													  userInfo:tDictionary];

								tSuccess=[fileManager_ removeFileAtPath:tFolderPath handler:self];
								
								if (tSuccess==NO)
								{
									if (tCleanSucceeded==YES)
									{
										NSBeep();

										tCleanSucceeded=NO;
									}
									
									tMetadata=[NSDictionary dictionaryWithObjectsAndKeys:tFolderPath,IC_BUILDER_INFORMATION_FILE_PATH,
																						 cleanErrorString_,IC_BUILDER_FAILURE_REASON,
																						 nil];
								
									[cleanErrorString_ release];
									
									cleanErrorString_=nil;
								
									tErrorCounts++;
								}
								else
								{
									tMetadata=nil;
								}
								
									
								tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
																					   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
																					   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
																					   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN_OBJECT],IC_BUILDER_INFORMATION_STEP,
																					   [NSNumber numberWithUnsignedLong:(tSuccess==YES) ? IC_BUILDER_BUILD_STATE_END_SUCCESS : IC_BUILDER_BUILD_STATE_END_FAILURE],IC_BUILDER_INFORMATION_STATE,
																					   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
																					   nil];
		
								[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
																					object:nil 
																				  userInfo:tDictionary];
							}
						}
					}
				}
			}
		}
	}
	else
	{
		tCleanSucceeded=NO;
	}
	
	if (tCleanSucceeded==NO)
	{
		tMetadata=[NSNumber numberWithUnsignedLong:tErrorCounts];
	}
	else
	{
		tMetadata==nil;
	}
	
	tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:[[NSProcessInfo processInfo] processIdentifier]],IC_BUILDER_NOTIFICATION_INFO_PID,
														   [self fileName],IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
														   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_CLEAN],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
														   [NSNumber numberWithUnsignedLong:IC_BUILDER_INFORMATION_STEP_CLEAN],IC_BUILDER_INFORMATION_STEP,
														   [NSNumber numberWithUnsignedLong:(tCleanSucceeded==YES) ? IC_BUILDER_BUILD_STATE_END_SUCCESS : IC_BUILDER_BUILD_STATE_END_FAILURE],IC_BUILDER_INFORMATION_STATE,
														   tMetadata,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
														   nil];
	
	[[NSNotificationCenter defaultCenter] postNotificationName:PACKAGES_BUILDER_NOTIFICATION
														object:nil 
													  userInfo:tDictionary];
}

- (BOOL) fileManager:(NSFileManager *) inManager shouldProceedAfterError:(NSDictionary *) inErrorDictionary
{
	cleanErrorString_=[[inErrorDictionary objectForKey:@"Error"] retain];
	
	return NO;
}

#pragma mark -

- (IBAction) showProject:(id) sender
{
	[IBsourceListView_ selectRow:0 byExtendingSelection:NO];
}

- (IBAction) addPackage:(id) sender
{
	if (dictionary_!=nil)
	{
		NSMutableArray * tPackagesArray;
		NSMutableDictionary * tNewPackageDictionary;
		
		tPackagesArray=[dictionary_ objectForKey:ICDOCUMENT_PACKAGES];
		
		if (tPackagesArray==nil)
		{
			tPackagesArray=[NSMutableArray array];
			
			if (tPackagesArray==nil)
			{
				NSBeep();
			
				NSLog(@"[ICDocument addPackage:] Not enough memory");
			
				[NSApp terminate:self];
			}
			
			[dictionary_ setObject:tPackagesArray forKey:ICDOCUMENT_PACKAGES];
		}
		
		// Create a default Package dictionary
		
		tNewPackageDictionary=[ICPackageUtilities defaultPackageDictionaryWithExistingPackages:tPackagesArray];
		
		if (tNewPackageDictionary!=nil)
		{
			ICTreeNode * tPackageNode;
			
			// Update the Tree
			
			tPackageNode=[tree_ addPackageTreeNodeWithDictionary:tNewPackageDictionary];
			
			if (tPackageNode!=nil)
			{
				NSInteger tRowIndex;
				id tItem;
				
				[IBsourceListView_ deselectAll:self];
				
				[IBsourceListView_ reloadData];
				
				tItem=[IBsourceListView_ itemAtRow:1];
				
				if (tItem!=nil && [IBsourceListView_ isItemExpanded:tItem]==NO)
				{
					[IBsourceListView_ expandItem:tItem];
				}
				
				tRowIndex=[IBsourceListView_ rowForItem:tPackageNode];
				
				[IBsourceListView_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
			
				[IBsourceListView_ editColumn:[IBsourceListView_ columnWithIdentifier:@"Name"] row:tRowIndex withEvent:nil select:YES];
				
				[tPackagesArray addObject:tNewPackageDictionary];
				
				// Post Notification
				
				[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_PACKAGESDIDADD
														object:self
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:[NSArray arrayWithObject:tNewPackageDictionary],ICDOCUMENT_NOTIFICATION_PACKAGES_LIST,nil]];
			
				[IBsourceListView_ reloadData];
				
				[self documentDidChange:nil];
			}
			else
			{
				// A COMPLETER
			}
		}
	}
}

- (IBAction) addPackageReference:(id) sender
{
	if (dictionary_!=nil)
	{
		NSMutableArray * tPackagesArray;
		NSMutableDictionary * tNewPackageDictionary;
		
		tPackagesArray=[dictionary_ objectForKey:ICDOCUMENT_PACKAGES];
		
		if (tPackagesArray==nil)
		{
			tPackagesArray=[NSMutableArray array];
			
			if (tPackagesArray==nil)
			{
				NSBeep();
			
				NSLog(@"[ICDocument addReferencedPackage:] Not enough memory");
			
				[NSApp terminate:self];
			}
			
			[dictionary_ setObject:tPackagesArray forKey:ICDOCUMENT_PACKAGES];
		}
		
		// Create a default Package dictionary
		
		tNewPackageDictionary=[ICPackageUtilities defaultReferencedPackageDictionaryWithExistingPackages:tPackagesArray];
		
		if (tNewPackageDictionary!=nil)
		{
			ICTreeNode * tPackageNode;
			
			// Update the Tree
			
			tPackageNode=[tree_ addPackageTreeNodeWithDictionary:tNewPackageDictionary];
			
			if (tPackageNode!=nil)
			{
				NSInteger tRowIndex;
				id tItem;
				
				[IBsourceListView_ deselectAll:self];
				
				[IBsourceListView_ reloadData];
				
				if ([tree_ numberOfChildren]==3)
				{
					tItem=[tree_ childAtIndex:2];
				}
				else
				{
					tItem=[tree_ childAtIndex:3];
				}
				
				if (tItem!=nil && [IBsourceListView_ isItemExpanded:tItem]==NO)
				{
					[IBsourceListView_ expandItem:tItem];
				}
				
				tRowIndex=[IBsourceListView_ rowForItem:tPackageNode];
				
				[IBsourceListView_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
			
				[IBsourceListView_ editColumn:[IBsourceListView_ columnWithIdentifier:@"Name"] row:tRowIndex withEvent:nil select:YES];
				
				[tPackagesArray addObject:tNewPackageDictionary];
				
				// Post Notification
				
				[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_PACKAGESDIDADD
														object:self
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:[NSArray arrayWithObject:tNewPackageDictionary],ICDOCUMENT_NOTIFICATION_PACKAGES_LIST,nil]];
			
				[IBsourceListView_ reloadData];
				
				[self documentDidChange:nil];
			}
			else
			{
				// A COMPLETER
			}
		}
	}
}

- (IBAction) importPackage:(id) sender
{
	NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setResolvesAliases:YES];
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:NO];
    [tOpenPanel setAllowsMultipleSelection:YES];
    [tOpenPanel setTreatsFilePackagesAsDirectories:YES];
    [tOpenPanel setDelegate:self];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Import",@"No comment")];
    
    [tOpenPanel beginSheetForDirectory:nil
                                  file:nil
                                 types:nil
                        modalForWindow:[self windowForSheet]
                         modalDelegate:self
                        didEndSelector:@selector(importPackageDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (BOOL) panel:(id)sender shouldShowFilename:(NSString *) inFileName
{
    BOOL isDirectory;
	
	if ([[NSFileManager defaultManager] fileExistsAtPath:inFileName isDirectory:&isDirectory]==YES)
	{
		if (isDirectory==NO)
		{
			return [ICPackageUtilities isFlatPackageAtPath:inFileName];
		}
	}
	
	return YES;
}

- (void) importPackageDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        NSArray * tFileNames;
		
		tFileNames=[sheet filenames];
		
		if (tFileNames!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[tFileNames objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				BOOL tShowCustomizationDialog=YES;
				id tObject;
				
				tObject=[defaults_ objectForKey:ICPREFERENCES_PACKAGES_SHOW_CUSTOMIZATION_DIALOG];
	
				if (tObject!=nil)
				{
					tShowCustomizationDialog=[defaults_ boolForKey:ICPREFERENCES_PACKAGES_SHOW_CUSTOMIZATION_DIALOG];
				}
				
				if (tShowCustomizationDialog==YES)
				{
					[self performSelector:@selector(beginImportPackagesReferenceSheet:) withObject:tFileNames afterDelay:0.f];
				}
				else
				{
					NSInteger tDefaultReferenceStyle;
					
					tDefaultReferenceStyle=[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
					
					[self _importPackages:tFileNames referenceStyle:tDefaultReferenceStyle];
				}
			}
		}
    }
}

- (void) beginImportPackagesReferenceSheet:(NSArray *) inFiles
{
	NSInteger tDefaultReferenceStyle;
	
	tDefaultReferenceStyle=[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
    
    inFiles=[inFiles copy];
    
    // Prepare the sheet
    
    [IBreferenceStyle_ selectItemAtIndex:[IBreferenceStyle_ indexOfItemWithTag:tDefaultReferenceStyle]];
    
    [NSApp beginSheet:[IBreferenceStyle_ window]
       modalForWindow:[self windowForSheet]
        modalDelegate:self
       didEndSelector:@selector(importPackageReferenceSheetDidEnd:returnCode:contextInfo:)
          contextInfo:inFiles];
}

- (IBAction) endimportPackageReferenceSheet:(id) sender
{
    [NSApp endSheet:[IBreferenceStyle_ window] returnCode:[sender tag]];
}

- (void) importPackageReferenceSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
    NSArray * tArray;
    
    tArray=(NSArray *) contextInfo;
    
    if (returnCode==NSOKButton)
    {
		[self _importPackages:tArray referenceStyle:[[IBreferenceStyle_ selectedItem] tag]];
	}
    
    [tArray release];

    [sheet orderOut:self];
}

- (void) _importPackages:(NSArray *) inArray referenceStyle:(NSInteger) inReferenceStyle
{
	NSEnumerator * tEnumerator;
		
	tEnumerator=[inArray objectEnumerator];
	
	if (tEnumerator!=nil)
	{
		NSString * tFilePath;
		NSMutableArray * tItemArray;
		NSMutableIndexSet * tMutableIndexSet;
		NSMutableArray * tImportedPackageDictionaryArray;
		NSUInteger i,tCount;
		
		tImportedPackageDictionaryArray=[NSMutableArray array];
		
		tItemArray=[NSMutableArray array];
		
		while (tFilePath=[tEnumerator nextObject])
		{
			NSMutableDictionary * tMutablePackageDictionary;
			
			tMutablePackageDictionary=[ICPackageUtilities importedPackageDictionaryWithContentsOfFile:tFilePath];
			
			if (tMutablePackageDictionary!=nil)
			{
				NSMutableDictionary * tPathDictionary;
				
				tPathDictionary=[NSMutableDictionary dictionary];
				
				if (tPathDictionary!=nil)
				{
					NSMutableArray * tPackagesArray;
					NSString * tPackageName;
					ICTreeNode * tPackageNode;
							
					[tPathDictionary setObject:[NSNumber numberWithInteger:inReferenceStyle] forKey:ICDOCUMENT_PATH_TYPE];
					
					if (inReferenceStyle==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						tFilePath=[tFilePath stringByRelativizingToPath:[self folder]];
					}
					else if (inReferenceStyle==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						tFilePath=[tFilePath stringByRelativizingToPath:[self referenceFolderPath]];
					}
					
					[tPathDictionary setObject:tFilePath forKey:ICDOCUMENT_PATH];
					
					[tMutablePackageDictionary setObject:tPathDictionary forKey:ICDOCUMENT_PACKAGE_REFERENCE_PATH];
					
					// Change Package Name if needed
					
					tPackageName=[[tMutablePackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
					
					tPackagesArray=[dictionary_ objectForKey:ICDOCUMENT_PACKAGES];
					
					if ([ICPackageUtilities isUniquePackageName:tPackageName forPackage:nil withExistingPackages:tPackagesArray]==NO)
					{
						[[tMutablePackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] setObject:[ICPackageUtilities uniquePackageNameWithExistingPackages:tPackagesArray baseName:tPackageName] forKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
					}
					
					tPackageNode=[tree_ addPackageTreeNodeWithDictionary:tMutablePackageDictionary];
			
					if (tPackageNode!=nil)
					{
						[tItemArray addObject:tPackageNode];
					
						[tPackagesArray addObject:tMutablePackageDictionary];
						
						[tImportedPackageDictionaryArray addObject:tMutablePackageDictionary];
					}
				}
			}
		}
		
		[tree_ sortPackages];
		
		[IBsourceListView_ reloadData];
		
		// Expand the groups if needed
		
		tCount=[tree_ numberOfChildren];
			
		for(i=tCount-1;i>0;i--)
		{
			id tItem;
			
			tItem=[tree_ childAtIndex:i];
				
			if (tItem!=nil && [IBsourceListView_ isItemExpanded:tItem]==NO)
			{
				[IBsourceListView_ expandItem:tItem];
			}
		}
		
		tMutableIndexSet=[NSMutableIndexSet indexSet];
		
		if (tMutableIndexSet!=nil)
		{
			tEnumerator=[tItemArray objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				ICTreeNode * tPackageNode;
				
				while (tPackageNode=[tEnumerator nextObject])
				{
					NSInteger tRowIndex;
					
					tRowIndex=[IBsourceListView_ rowForItem:tPackageNode];
					
					[tMutableIndexSet addIndex:tRowIndex];
				}
			}
			
			[IBsourceListView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:NO];
		}
		
		// Post Notification
		
		[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_PACKAGESDIDADD
												object:self
											  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:tImportedPackageDictionaryArray,ICDOCUMENT_NOTIFICATION_PACKAGES_LIST,nil]];
	
		[IBsourceListView_ reloadData];
		
		[self documentDidChange:nil];
	}
}

- (IBAction) removePackage:(id) sender
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBsourceListView_ selectedRowIndexes];
		
	if (tIndexSet!=nil)
	{
		NSString * tAlertTitle=@"";
		
		if ([tIndexSet count]==1)
		{
			tAlertTitle=NSLocalizedString(@"Do you really want to remove this package?",@"No comment");
		}
		else
		{
			tAlertTitle=NSLocalizedString(@"Do you really want to remove these packages?",@"No comment");
		}
	
		NSBeginAlertSheet(tAlertTitle,
						  NSLocalizedString(@"Remove",@"No comment"),
						  NSLocalizedString(@"Cancel",@"No comment"),
						  nil,
						  [self windowForSheet],
						  self,
						  @selector(removePackageDidEnd:returnCode:contextInfo:),
						  nil,
						  NULL,
						  NSLocalizedString(@"This cannot be undone.",@"No comment"));

	}
}

- (void) removePackageDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
		NSMutableArray * tPackagesArray;
		
		tPackagesArray=[dictionary_ objectForKey:ICDOCUMENT_PACKAGES];
		
		tIndexSet=[IBsourceListView_ selectedRowIndexes];
	
		if (tIndexSet!=nil && tPackagesArray!=nil)
		{
			NSUInteger tIndex;
			NSUInteger tFirstIndex;
			NSMutableArray * tRemovedPackagesList;
			
			tFirstIndex=[tIndexSet firstIndex];
			
			tIndex=[tIndexSet lastIndex];
			
			tRemovedPackagesList=[NSMutableArray array];
			
			while (tIndex!=NSNotFound)
			{
				id tItem;
				id tParent;
				
				NSDictionary * tDictionary;
				ICDocumentNodeData * tNodeData;
				
				tItem=[IBsourceListView_ itemAtRow:tIndex];
			
				tNodeData=(ICDocumentNodeData *) NODE_DATA(tItem);
				
				// Update the array
				
				tDictionary=[tNodeData dictionary];
				
				if (tDictionary!=nil)
				{
					NSString * tPackageUUID;
					
					tPackageUUID=[tDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID];
					
					if (tPackageUUID!=nil)
					{
						// Remove archived selection
						
						[self removeDocumentDefaultForKey:[NSString stringWithFormat:ICFILE_HIERARCHY_SELECTION,tPackageUUID]];
						
						// Remove archived disclosed state
						
						[self removeDocumentDefaultForKey:[NSString stringWithFormat:ICFILE_HIERARCHY_DISCLOSED_STATE,tPackageUUID]];
					}
					
					[tRemovedPackagesList addObject:tDictionary];
					
					[tPackagesArray removeObject:tDictionary];
				}
				
				// Update the Tree
				
				tParent=[tItem nodeParent];
				
				[tItem removeFromParent];
				
				if ([tParent numberOfChildren]==0)
				{
					tNodeData=(ICDocumentNodeData *) NODE_DATA(tParent);
					
					if ([tNodeData type]!=kProjectPackagesGroupNode)
					{
						[tParent removeFromParent];
					}
				}
				
				tIndex=[tIndexSet indexLessThanIndex:tIndex];
			}
			
			// Post Notification
					
			[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_PACKAGESDIDREMOVE
														object:self
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:tRemovedPackagesList,ICDOCUMENT_NOTIFICATION_PACKAGES_LIST,nil]];
			
			[IBsourceListView_ deselectAll:self];
			
			tIndex=tFirstIndex;
			
			while (tIndex!=0)
			{
				id tItem;
				ICDocumentNodeData * tNodeData;
				
				tIndex=tIndex-1;
				
				tItem=[IBsourceListView_ itemAtRow:tIndex];
			
				tNodeData=(ICDocumentNodeData *) NODE_DATA(tItem);
				
				if ([tNodeData type]==kPackageNode)
				{
					[IBsourceListView_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tIndex] byExtendingSelection:NO];
					
					break;
				}
			}
			
			if (tIndex==0)
			{
				[IBsourceListView_ selectRowIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
			}
				
			[IBsourceListView_ reloadData];
			
			[self documentDidChange:nil];
			
			[self outlineViewSelectionDidChange:nil];
		}
	}
}

- (IBAction) upgradeToDistribution:(id) sender
{
	NSBeginAlertSheet(NSLocalizedString(@"Do you really want to upgrade this raw package project to a distribution project?",@"No comment"),
					  NSLocalizedString(@"Upgrade",@"No comment"),
					  NSLocalizedString(@"Cancel",@"No comment"),
					  nil,
					  [self windowForSheet],
					  self,
					  @selector(upgradeToDistributionDidEnd:returnCode:contextInfo:),
					  nil,
					  NULL,
					  NSLocalizedString(@"This will save all the changes you made and can not be undone.",@"No comment"));
}

- (void) upgradeToDistributionDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
	if (returnCode==NSAlertDefaultReturn)
    {
		NSMutableArray * tMutableArray;
		NSMutableDictionary * tProjectDictionary;
		
		// Change project type
		
		[dictionary_ setObject:[NSNumber numberWithInt:ICDOCUMENT_TYPE_DISTRIBUTION] forKey:ICDOCUMENT_TYPE];
		
		// Create the Presentation Dictionary and update the Settings Dictionary
		
		tProjectDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT];
		
		if (tProjectDictionary!=nil)
		{
			NSMutableDictionary * tPresentationDictionary;
			NSMutableDictionary * tProjectSettingsDictionary;
			
			// Set the Build Format
			
			tProjectSettingsDictionary=[tProjectDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS];
			
			if (tProjectSettingsDictionary!=nil)
			{
				[tProjectSettingsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_BUNDLE] forKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT];
			}
			
			tPresentationDictionary=[NSMutableDictionary dictionary];
			
			if (tPresentationDictionary!=nil)
			{
				NSMutableDictionary * tTitleDictionary;
				
				tTitleDictionary=[ICTitleUtilities defaultTitleDictionaryForDocument:self];
				
				if (tTitleDictionary!=nil)
				{
					[tPresentationDictionary setObject:tTitleDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_TITLE];
				}
				
				[tProjectDictionary setObject:tPresentationDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION];
			}
		}
		
		// Create the List of packages
		
		tMutableArray=[NSMutableArray array];
		
		if (tMutableArray!=nil)
		{
			NSMutableDictionary * tPackageDictionary;
			
			tPackageDictionary=[NSMutableDictionary dictionary];
			
			if (tPackageDictionary!=nil)
			{
				NSMutableDictionary * tMutableDictionary;
				
				// Package Settings
				
				tMutableDictionary=[tProjectDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS];
				
				if (tMutableDictionary!=nil)
				{
					// Set the location type
					
					[tMutableDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED] forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION];
				
					// Set the name
					
					[tMutableDictionary setObject:[[[self fileName] lastPathComponent] stringByDeletingPathExtension] forKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
					
					
					[tPackageDictionary setObject:tMutableDictionary forKey:ICDOCUMENT_PACKAGE_SETTINGS];
					
					[tProjectDictionary removeObjectForKey:ICDOCUMENT_PACKAGE_SETTINGS];
				}
				
				// Package payload
				
				tMutableDictionary=[tProjectDictionary objectForKey:ICDOCUMENT_PACKAGE_FILES];
				
				if (tMutableDictionary!=nil)
				{
					[tPackageDictionary setObject:tMutableDictionary forKey:ICDOCUMENT_PACKAGE_FILES];
					
					[tProjectDictionary removeObjectForKey:ICDOCUMENT_PACKAGE_FILES];
				}
				
				// Package scripts
				
				tMutableDictionary=[tProjectDictionary objectForKey:ICDOCUMENT_PACKAGE_SCRIPTS];
				
				if (tMutableDictionary!=nil)
				{
					[tPackageDictionary setObject:tMutableDictionary forKey:ICDOCUMENT_PACKAGE_SCRIPTS];
					
					[tProjectDictionary removeObjectForKey:ICDOCUMENT_PACKAGE_SCRIPTS];
				}
				
				// UUID
				
				[tPackageDictionary setObject:[ICUUIDFactory string] forKey:ICDOCUMENT_PACKAGE_UUID];
				
				// Add the package
				
				[tMutableArray addObject:tPackageDictionary];
			}
			
		
			[dictionary_ setObject:tMutableArray forKey:ICDOCUMENT_PACKAGES];
		}
		
		// Save the file on disk
		
		[self saveDocumentWithDelegate:[ICProjectAssistantController sharedProjectAssistantController] didSaveSelector:@selector(upgradedDocument:didSave:contextInfo:) contextInfo:NULL];
    }
}

#pragma mark -

- (void) outlineViewSelectionDidChange:(NSNotification *) notification
{
	if (tree_!=nil && nibDidLoad_==YES)
	{
		NSIndexSet * tIndexSet;
		
		if (currentComponentController_!=nil)
		{
			if ([currentComponentController_ viewCanBeRemoved:self]==NO)
			{
				return;
			}
			
			[currentComponentController_ viewWillBeRemoved:self];
			
			if (currentComponentView_!=nil)
			{
				[currentComponentView_ removeFromSuperview];
			}
				
			[currentComponentController_ release];
		}
		
		currentComponentController_=nil;
		
		tIndexSet=[IBsourceListView_ selectedRowIndexes];
		
		if (tIndexSet!=nil)
		{
			NSUInteger tSelectedRow;
			NSMutableDictionary * tNodeDictionary=nil;
			
			switch ([tIndexSet count])
			{
				case 0:		// Empty Selection
				
					// A COMPLETER
				
					break;
					
				case 1:
				
					tSelectedRow=[tIndexSet firstIndex];
					
					if (tSelectedRow==0)
					{
						currentComponentController_=[self componentControllerForTreeNodeType:kProjectNode];
						
						tNodeDictionary=dictionary_;
					}
					else
					{
						id tItem;
						
						tItem=[IBsourceListView_ itemAtRow:tSelectedRow];
						
						if (tItem!=nil)
						{
							ICDocumentNodeData * tNodeData;
							
							currentComponentController_=[self componentControllerForTreeNodeType:kPackageNode];
							
							tNodeData=(ICDocumentNodeData *) NODE_DATA(tItem);
							
							if (tNodeData!=nil)
							{
								tNodeDictionary=[tNodeData dictionary];
							}
							else
							{
								currentComponentController_=nil;
							}
						}
					}
				
					break;
					
				default:	// Multiple Selection
					
					currentComponentController_=[ICMultiplePackagesSelectionController multiplePackagesSelectionController];
					
					tNodeDictionary=[NSDictionary dictionary];
					
					break;
			}
		
			if (currentComponentController_!=nil && tNodeDictionary!=nil)
			{
				[currentComponentController_ retain];
				
				currentComponentView_=[currentComponentController_ view];
				
				// Resize view
				
				[currentComponentView_ setFrame:[IBrightView_ bounds]];
				
				// Initialize View widgets
				
				[currentComponentController_ initializeWithDictionary:tNodeDictionary forDocument:self];
				
				[IBrightView_ addSubview:currentComponentView_];
			}
		}
	}
}

- (void) documentDidChange:(NSNotification *) inNotification
{
	 [self updateChangeCount:NSChangeDone];

	// A COMPLETER
}

- (void) packagesDidAdd:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			NSArray * tNewPackages;
			
			tNewPackages=[tUserInfo objectForKey:ICDOCUMENT_NOTIFICATION_PACKAGES_LIST];
			
			if (tNewPackages!=nil && [tNewPackages count]>0)
			{
				if (dictionary_!=nil)
				{
					NSMutableDictionary * tProjectDictionary;
					
					tProjectDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT];
					
					if (tProjectDictionary!=nil)
					{
						NSMutableDictionary * tPresentationDictionary;
						
						tPresentationDictionary=[tProjectDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION];
						
						if (tPresentationDictionary!=nil)
						{
							NSMutableDictionary * tInstallationTypeDictionary;
							
							tInstallationTypeDictionary=[tPresentationDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE];
							
							if (tInstallationTypeDictionary!=nil)
							{
								[ICInstallationTypeUtilities updateInstallationTypeDictionary:tInstallationTypeDictionary withNewPackages:tNewPackages];
							}
						}
					}
				}
			}
		}
	}
}

- (void) packagesDidRemove:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			NSArray * tRemovedPackages;
			
			tRemovedPackages=[tUserInfo objectForKey:ICDOCUMENT_NOTIFICATION_PACKAGES_LIST];
			
			if (tRemovedPackages!=nil && [tRemovedPackages count]>0)
			{
				if (dictionary_!=nil)
				{
					NSMutableDictionary * tProjectDictionary;
					
					tProjectDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT];
					
					if (tProjectDictionary!=nil)
					{
						NSMutableDictionary * tPresentationDictionary;
						
						tPresentationDictionary=[tProjectDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION];
						
						if (tPresentationDictionary!=nil)
						{
							NSMutableDictionary * tInstallationTypeDictionary;
							
							tInstallationTypeDictionary=[tPresentationDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE];
							
							if (tInstallationTypeDictionary!=nil)
							{
								[ICInstallationTypeUtilities updateInstallationTypeDictionary:tInstallationTypeDictionary withRemovedPackages:tRemovedPackages];
							}
						}
					}
				}
			}
		}
	}
}

- (void) optionKeyDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tDictionary;
		
		tDictionary=[inNotification userInfo];
		
		if (tDictionary!=nil)
		{
			NSNumber * tNumber;
			
			tNumber=[tDictionary objectForKey:ICWINDOW_OPTIONKEY_STATE];
			
			if (tNumber!=nil)
			{
				if ([tNumber boolValue]==YES)
				{
					[IBaddPackageButton_ setAction:@selector(addPackageReference:)];
					
					[IBaddPackageButton_ setImage:[NSImage imageNamed:@"Reference"]];
				}
				else
				{
					[IBaddPackageButton_ setAction:@selector(addPackage:)];
					
					[IBaddPackageButton_ setImage:[NSImage imageNamed:@"Plus"]];
				}
			}
		}
	}
}

- (void) documentShouldHideSourceList:(NSNotification *) inNotification
{
	NSArray * tSubViews;
	
	tSubViews=[IBsplitView_ subviews];
	
	editingInstallationTypeChoice_=YES;
	
	if (tSubViews!=nil && [tSubViews count]>0)
	{
		NSView * tSourceView;
		NSRect tFrame;
		NSRect tSplitViewBounds;
		
		tSourceView=[tSubViews objectAtIndex:0];
		
		tFrame=[tSourceView frame];
		
		savedSourceListWidth_=NSWidth(tFrame);
		
		tFrame.size.width=0.0f;
		
		[tSourceView setFrame:tFrame];
		
		tSplitViewBounds=[IBsplitView_ bounds];
		
		[IBrightView_ setFrame:tSplitViewBounds];
		
		tFrame=[IBsplitView_ frame];
		
		tFrame.size.width+=[IBsplitView_ dividerThickness];
		
		tFrame.origin.x-=[IBsplitView_ dividerThickness];
		
		[IBsplitView_ setFrame:tFrame];
		
		[IBsplitView_ display];
		
		[IBaddPackageButton_ setHidden:YES];
	}
}

- (void) documentShouldShowSourceList:(NSNotification *) inNotification
{
	NSArray * tSubViews;
	
	tSubViews=[IBsplitView_ subviews];
	
	editingInstallationTypeChoice_=NO;
	
	if (tSubViews!=nil && [tSubViews count]>0)
	{
		NSView * tSourceView;
		NSRect tFrame;
		NSRect tSplitViewBounds;
		
		tSourceView=[tSubViews objectAtIndex:0];

		tFrame=[tSourceView frame];
		
		tFrame.size.width=savedSourceListWidth_;
		
		[tSourceView setFrame:tFrame];
		
		tFrame=[IBsplitView_ frame];
		
		tFrame.size.width-=[IBsplitView_ dividerThickness];
		
		tFrame.origin.x+=[IBsplitView_ dividerThickness];
		
		[IBsplitView_ setFrame:tFrame];
		
		tSplitViewBounds=[IBsplitView_ bounds];
		
		tFrame=[IBrightView_ frame];
		
		tFrame.origin.x=savedSourceListWidth_+[IBsplitView_ dividerThickness];
		
		tFrame.size.width=NSWidth(tSplitViewBounds)-tFrame.origin.x;
		
		[IBrightView_ setFrame:tSplitViewBounds];
		
		[IBsplitView_ display];
		
		[IBaddPackageButton_ setHidden:NO];
	}
}

- (void) documentShouldReloadSourceList:(NSNotification *) inNotification
{
	NSDictionary * tUserInfo;
	
	[_cachedRemovedPackagesList release];
	
	_cachedRemovedPackagesList=nil;
	
	tUserInfo=[inNotification userInfo];
	
	if (tUserInfo!=nil)
	{
		_cachedRemovedPackagesList=[tUserInfo objectForKey:ICDOCUMENT_NOTIFICATION_HIERARCHY_PACKAGES_REMOVED_LIST];
		
		if (_cachedRemovedPackagesList!=nil)
		{
			[_cachedRemovedPackagesList retain];
		}
	}
	
	[IBsourceListView_ reloadData];
}

- (void) documentStatusDidChange:(NSNotification *) inNotification
{
	NSDictionary * tDictionary;
	
	tDictionary=[inNotification userInfo];
	
	if (tDictionary!=nil)
	{
		if (building_==NO)
		{
			NSString * tString;
			NSNumber * tNumber;
			
			tString=[tDictionary objectForKey:ICDOCUMENT_STATUS_STRING];
			
			if (tString!=nil && [tString length]>0)
			{
				[IBbuildStatusLabel_ setStringValue:tString];
				
				[IBbuildStatusLabel_ setHidden:NO];
			}
			else
			{
				[IBbuildStatusLabel_ setStringValue:@""];
				
				[IBbuildStatusLabel_ setHidden:YES];
			}
			
			tNumber=[tDictionary objectForKey:ICDOCUMENT_STATUS_ID];
			
			if (tNumber!=nil)
			{
				int tValue;
				
				tValue=[tNumber intValue];
				
				if (tValue==-1)
				{
					[IBbuildStatusProgressIndicator_ setIndeterminate:YES];
					
					[IBbuildStatusProgressIndicator_ setHidden:NO];
					
					[IBbuildStatusProgressIndicator_ startAnimation:nil];
				}
				else
				{
					// To be continued
				}
			}
			else
			{
				[IBbuildStatusProgressIndicator_ stopAnimation:nil];
				
				[IBbuildStatusProgressIndicator_ setHidden:YES];
			}
			
		}
	}
}

@end
