/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICDocumentSecondaryController.h"

@implementation ICDocumentSecondaryController

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[dictionary_ release];
	
	dictionary_=nil;
	
	document_=inDocument;
	
	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(documentWillBeSaved:) 
												 name:ICDOCUMENT_NOTIFICATION_DOCUMENT_WILL_BE_SAVED 
											   object:inDocument];
}

- (void) dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[dictionary_ release];

	[super dealloc];
}

#pragma mark -

- (NSMutableDictionary *) dictionary
{
	return [[dictionary_ retain] autorelease];
}

- (id) document
{
	return document_;
}

#pragma mark -

- (void) noteDocumentChange
{
	[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_DOCUMENT_DID_CHANGE object:document_];
}

#pragma mark -

- (void) documentWillBeSaved:(NSNotification *) inNotification
{
}

@end
