/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICDocumentSourceListView.h"

#import "ICHighlightWindow.h"

@implementation ICDocumentSourceListView

- (NSMenu*) menuForEvent: (NSEvent*) inEvent
{
    NSPoint where;
    NSInteger tRow;

    where = [self convertPoint: [inEvent locationInWindow] fromView: [[self window] contentView]];
    
	tRow = [self rowAtPoint: where];
    
    if (tRow < 0)
    {
        return nil;
    }
    
    if ([self isRowSelected:tRow]==NO)
    {
        [self selectRowIndexes:[NSIndexSet indexSetWithIndex:tRow] byExtendingSelection: NO];
    }
    
    return [self menu];
}

#pragma mark -

- (void) _drawDropHighlight
{
}

- (NSRect) frameOfOutlineCellAtRow:(NSInteger) inRow
{
	return NSZeroRect;
}

- (void) showHideHighlight:(BOOL) inShow
{
	if (inShow==YES && highlightWindow_==nil)
	{
		NSRect tFrame;
		NSRect tPackagesLabelFrame;
		
		tPackagesLabelFrame=[self rectOfRow:1];
		
		tFrame=[[self superview] bounds];
		
		if (NSMaxY(tPackagesLabelFrame)>NSMinY(tFrame))
		{
			tFrame.size.height-=(NSMaxY(tPackagesLabelFrame)-NSMinY(tFrame));
			
			tFrame.origin.y=NSMaxY(tPackagesLabelFrame);
		}
		
		highlightWindow_=[[ICHighlightWindow alloc] initForView:self withFrame:tFrame];
		
		if (highlightWindow_!=nil)
		{
			[[self window] addChildWindow:highlightWindow_ ordered:NSWindowAbove];
			
			[highlightWindow_ orderFront:self];
		}
	}
	else if (inShow==NO && highlightWindow_!=nil)
	{
		[[self window] removeChildWindow:highlightWindow_];
		
		[highlightWindow_ orderOut:self];
		
		[highlightWindow_ release];
		
		highlightWindow_=nil;
	}
}

- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>) sender
{
	NSDragOperation tDragOperation;
	
	tDragOperation=[super draggingEntered:sender];
	
	if (tDragOperation!=NSDragOperationNone)
	{
		[self showHideHighlight:YES];
	}
	
	return tDragOperation;
}

- (BOOL) performDragOperation: (id<NSDraggingInfo>) sender
{
	[self showHideHighlight:NO];
	
	return [super performDragOperation:sender];
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
	[self showHideHighlight:NO];
	
	[super draggingExited:sender];
}

- (void) draggingEnded:(id <NSDraggingInfo>)sender
{
	[self showHideHighlight:NO];
}

@end
