/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICDocumentTree.h"
#import "ICDocumentFormat+Constants.h"

@implementation ICDocumentNodeData

+ (id) documentNodeDataOfType:(int) inType withDictionary:(NSMutableDictionary *) inDictionary
{
	return [[[ICDocumentNodeData alloc] initWithType:inType andDictionary:inDictionary] autorelease];
}

+ (id) rootNodeData
{
	return [ICDocumentNodeData documentNodeDataOfType:kRootNode withDictionary:nil];
}

- (id) initWithType:(int) inType andDictionary:(NSMutableDictionary *) inDictionary
{
	self=[super init];
	
	if (self!=nil)
	{
		type_=inType;
		
		dictionary_=[inDictionary retain];
	}
	
	return self;
}

- (void) dealloc
{
	[dictionary_ release];

	[super dealloc];
}

#pragma mark -

- (BOOL) isLeaf
{
	if (type_==kProjectNode ||
		type_==kPackageNode)
	{
		return YES;
	}
	
	return NO;
}

- (int) type
{
	return type_;
}

- (NSMutableDictionary *) dictionary
{
	return [[dictionary_ retain] autorelease];
}

@end

@implementation ICTreeNode

+ (id) documentTreeNodeWithDictionary:(NSMutableDictionary *) inDictionary
{
	ICTreeNode * tRootNode;
	
	tRootNode=[[ICTreeNode alloc] initWithData:[ICDocumentNodeData rootNodeData]
										parent:nil
									  children:[NSArray array]];
	
	if (tRootNode!=nil)
	{
		NSMutableDictionary * tDictionary;
		
		tDictionary=[inDictionary objectForKey:ICDOCUMENT_PROJECT];
		
		if (tDictionary!=nil)
		{
			ICTreeNode * tProjectNode;
			
			tProjectNode=[[ICTreeNode alloc] initWithData:[ICDocumentNodeData documentNodeDataOfType:kProjectNode withDictionary:tDictionary]
												   parent:tRootNode
												 children:[NSArray array]];
		
			if (tProjectNode!=nil)
			{
				ICTreeNode * tProjectPackagesGroupNode;
				ICTreeNode * tImportedPackagesGroupNode=nil;
				ICTreeNode * tReferencedPackagesGroupNode=nil;
				
				[tRootNode insertChild:tProjectNode
							   atIndex: 0];
			
				[tProjectNode release];
						
				tProjectPackagesGroupNode=[[ICTreeNode alloc] initWithData:[ICDocumentNodeData documentNodeDataOfType:kProjectPackagesGroupNode withDictionary:nil]
															 parent:tRootNode
														   children:[NSArray array]];
				
				
				if (tProjectPackagesGroupNode!=nil)
				{
					NSArray * tArray;
					int tGroupIndex=1;
					
					[tRootNode insertChild:tProjectPackagesGroupNode
								   atIndex:tGroupIndex];
					
					tGroupIndex++;
					
					[tProjectPackagesGroupNode release];
					
					tArray=[inDictionary objectForKey:ICDOCUMENT_PACKAGES];
					
					if (tArray!=nil)
					{
						NSEnumerator * tEnumerator;
						
						tEnumerator=[tArray objectEnumerator];
						
						if (tEnumerator!=nil)
						{
							int tProjectPackagesIndex=0;
							int tImportedPackagesIndex=0;
							int tReferencedPackagesIndex=0;
							
							while (tDictionary=[tEnumerator nextObject])
							{
								ICTreeNode * tPackageNode;
								
								tPackageNode=[ICTreeNode packageTreeNodeWithDictionary:tDictionary];
							
								if (tPackageNode!=nil)
								{
									NSNumber * tTypeNumber;
									NSUInteger tType=ICDOCUMENT_PACKAGE_TYPE_PROJECT;
									
									tTypeNumber=[tDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
									
									if (tTypeNumber!=nil)
									{
										tType=[tTypeNumber unsignedLongValue];
									}
									
									if (tType==ICDOCUMENT_PACKAGE_TYPE_PROJECT)
									{
										[tProjectPackagesGroupNode insertChild:tPackageNode
																	  atIndex:tProjectPackagesIndex];
																	  
										tProjectPackagesIndex++;
									}
									else
									{
										if (tType==ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
										{
											if (tImportedPackagesGroupNode==nil)
											{
												tImportedPackagesGroupNode=[[ICTreeNode alloc] initWithData:[ICDocumentNodeData documentNodeDataOfType:kImportedPackagesGroupNode withDictionary:nil]
															 parent:tRootNode
														   children:[NSArray array]];
				
				
												if (tImportedPackagesGroupNode!=nil)
												{
													[tRootNode insertChild:tImportedPackagesGroupNode
																   atIndex:tGroupIndex];
																   
													tGroupIndex++;
													
													[tImportedPackagesGroupNode release];
												}
												else
												{
													// A COMPLETER
												}
											}
											
											[tImportedPackagesGroupNode insertChild:tPackageNode
																			atIndex:tImportedPackagesIndex];
																		  
											tImportedPackagesIndex++;
										}
										else
										{
											if (tReferencedPackagesGroupNode==nil)
											{
												tReferencedPackagesGroupNode=[[ICTreeNode alloc] initWithData:[ICDocumentNodeData documentNodeDataOfType:kReferencedPackagesGroupNode withDictionary:nil]
															 parent:tRootNode
														   children:[NSArray array]];
				
				
												if (tReferencedPackagesGroupNode!=nil)
												{
													[tRootNode insertChild:tReferencedPackagesGroupNode
																   atIndex:tGroupIndex];
																   
													tGroupIndex++;
													
													[tReferencedPackagesGroupNode release];
												}
												else
												{
													// A COMPLETER
												}
											}
											
											[tReferencedPackagesGroupNode insertChild:tPackageNode
																			atIndex:tReferencedPackagesIndex];
																		  
											tReferencedPackagesIndex++;
										}
									}
												   
									
								}
								else
								{
									// A COMPLETER
								}
							}
						}
						else
						{
							// A COMPLETER
						}
					}
					else
					{
					}
					
					// Sort Packages by Name
					
					[tProjectPackagesGroupNode recursiveSortChildren];
				}
				else
				{
					// A COMPLETER
				}
			}
			else
			{
				// A COMPLETER
			}
		}
		else
		{
			// A COMPLETER
		}
	}
	else
	{
		// A COMPLETER
	}
	
	return [tRootNode autorelease];
}

+ (id) packageTreeNodeWithDictionary:(NSMutableDictionary *) inDictionary
{
	ICTreeNode * tPackageNode=nil;
	
	if (inDictionary!=nil)
	{
		tPackageNode=[[ICTreeNode alloc] initWithData:[ICDocumentNodeData documentNodeDataOfType:kPackageNode withDictionary:inDictionary]
																						 parent:nil
																					   children:[NSArray array]];
	}
	
	return [tPackageNode autorelease];
}

- (id) addPackageTreeNodeWithDictionary:(NSMutableDictionary *) inDictionary
{
	ICTreeNode * tPackageNode=nil;
	
	if (inDictionary!=nil)
	{
		// Check that we are a root node
		
		ICDocumentNodeData * tNodeData;
		
		tNodeData=(ICDocumentNodeData *) [self nodeData];
		
		if (tNodeData!=nil)
		{
			if ([tNodeData type]==kRootNode)
			{
				tPackageNode=[ICTreeNode packageTreeNodeWithDictionary:inDictionary];
				
				if (tPackageNode!=nil)
				{
					NSNumber * tTypeNumber;
					NSUInteger tType=ICDOCUMENT_PACKAGE_TYPE_PROJECT;
					ICTreeNode * tProjectPackagesGroupNode=nil;
					ICTreeNode * tImportedPackagesGroupNode=nil;
					ICTreeNode * tReferencedPackagesGroupNode=nil;
					NSArray * tChildrenArray;
					
					tTypeNumber=[inDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
					
					if (tTypeNumber!=nil)
					{
						tType=[tTypeNumber unsignedLongValue];
					}
					
					tChildrenArray=[self children];
					
					if (tChildrenArray!=nil)
					{
						NSEnumerator * tEnumerator;
						
						tEnumerator=[tChildrenArray objectEnumerator];
						
						if (tEnumerator!=nil)
						{
							ICTreeNode * tNode;
							
							while (tNode=(ICTreeNode *) [tEnumerator nextObject])
							{
								switch([NODE_DATA(tNode) type])
								{
									case kProjectPackagesGroupNode:
										
										tProjectPackagesGroupNode=tNode;
									
										break;
									case kImportedPackagesGroupNode:
									
										tImportedPackagesGroupNode=tNode;
										
										break;
									case kReferencedPackagesGroupNode:
										
										tReferencedPackagesGroupNode=tNode;
										
										break;
								}
							}
						}
					}
					
					if (tType==ICDOCUMENT_PACKAGE_TYPE_PROJECT)
					{
						[tProjectPackagesGroupNode insertChild:tPackageNode
												atIndex:[tProjectPackagesGroupNode numberOfChildren]];
					}
					else
					{
						if (tType==ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
						{
							if (tImportedPackagesGroupNode==nil)
							{
								tImportedPackagesGroupNode=[[ICTreeNode alloc] initWithData:[ICDocumentNodeData documentNodeDataOfType:kImportedPackagesGroupNode withDictionary:nil]
											 parent:self
										   children:[NSArray array]];


								if (tImportedPackagesGroupNode!=nil)
								{
									[self insertChild:tImportedPackagesGroupNode
												   atIndex:2];
												   
									[tImportedPackagesGroupNode release];
								}
								else
								{
									// A COMPLETER
								}
							}
							
							[tImportedPackagesGroupNode insertChild:tPackageNode
												atIndex:[tImportedPackagesGroupNode numberOfChildren]];
						
							
						}
						else
						{
							if (tReferencedPackagesGroupNode==nil)
							{
								tReferencedPackagesGroupNode=[[ICTreeNode alloc] initWithData:[ICDocumentNodeData documentNodeDataOfType:kReferencedPackagesGroupNode withDictionary:nil]
											 parent:self
										   children:[NSArray array]];


								if (tReferencedPackagesGroupNode!=nil)
								{
									[self insertChild:tReferencedPackagesGroupNode
												   atIndex:(tImportedPackagesGroupNode==nil) ? 2 : 3];
												   
									[tReferencedPackagesGroupNode release];
								}
								else
								{
									// A COMPLETER
								}
							}
							
							[tReferencedPackagesGroupNode insertChild:tPackageNode
												atIndex:[tReferencedPackagesGroupNode numberOfChildren]];
						}
						
						[self sortPackages];
					}
				}
			}
		}
	}
	
	return tPackageNode;
}

- (NSComparisonResult) compare:(ICTreeNode *) other
{
	ICDocumentNodeData * tNodeData;
		
	tNodeData=(ICDocumentNodeData *) [self nodeData];
	
	if (tNodeData!=nil)
	{
		ICDocumentNodeData * tOtherNodeData;
		NSDictionary * tPackageDictionary;
		
		tPackageDictionary=[tNodeData dictionary];
		
		tOtherNodeData=(ICDocumentNodeData *) [other nodeData];
		
		if (tOtherNodeData!=nil && tPackageDictionary!=nil)
		{
			NSDictionary * tOtherPackageDictionary;
			NSString * tPackageName;
			
			tOtherPackageDictionary=[tOtherNodeData dictionary];
			
			tPackageName=[[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
			
			if (tOtherPackageDictionary!=nil && tPackageName!=nil)
			{
				NSString * tOtherPackageName;
			
				tOtherPackageName=[[tOtherPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
				
				if (tOtherPackageName!=nil)
				{
					return [tPackageName compare:tOtherPackageName options:NSNumericSearch+NSCaseInsensitiveSearch];
				}
			}
		}
	}
	
	return NSOrderedSame;
}

- (void) sortPackages
{
	// Check that we are a root node
		
	ICDocumentNodeData * tNodeData;
	
	tNodeData=(ICDocumentNodeData *) [self nodeData];
	
	if (tNodeData!=nil)
	{
		if ([tNodeData type]==kRootNode)
		{
			NSArray * tChildrenArray;
			
			tChildrenArray=[self children];
					
			if (tChildrenArray!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tChildrenArray objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					ICTreeNode * tNode;
					
					while (tNode=(ICTreeNode *) [tEnumerator nextObject])
					{
						switch([NODE_DATA(tNode) type])
						{
							case kProjectPackagesGroupNode:
							case kImportedPackagesGroupNode:
							case kReferencedPackagesGroupNode:
								
								[tNode recursiveSortChildren];
								
								break;
						}
					}
				}
			}
		}
	}
}

@end
