/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICFileBouncer.h"

#import "ICDocumentFormat+Constants.h"

@implementation ICFileBouncer

- (id) initWithBlackList:(NSArray *) inList
{
	if (inList!=nil)
	{
		self=[super init];
	
		if (self!=nil)
		{
			blackList_=[inList retain];
		}
	
		return self;
	}
	
	[self dealloc];
	
	NSLog(@"[ICFileBouncer initWithBlackList:] Missing blackList");
	
	return nil;
}

- (void) dealloc
{
	[blackList_ release];

	[super dealloc];
}

#pragma mark -

- (void) setBlackList:(NSArray *) inList
{
	if (inList==nil)
	{
		NSLog(@"[ICFileBouncer setBlackList:] blackList can't be nil");
		
		return;
	}
	
	if (blackList_!=inList)
	{
		[blackList_ release];
		
		blackList_=[inList retain];
	}
}

#pragma mark -

- (BOOL) shouldBounceItemNamed:(NSString *) inName ofType:(unsigned int) inType
{
	if (inName!=nil && inType<=ICFILE_TYPE_FOLDER)
	{
		NSEnumerator * tRuleEnumerator;
		
		tRuleEnumerator=[blackList_ objectEnumerator];
		
		if (tRuleEnumerator!=nil)
		{
			NSDictionary * tDictionary;
			
			while (tDictionary=[tRuleEnumerator nextObject])
			{
				NSNumber * tNumber;
				
				tNumber=[tDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_STATE];
				
				if (tNumber==nil || [tNumber boolValue]==YES)
				{
					NSArray * tPatternsArray;
					
					tPatternsArray=[tDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY];
					
					if (tPatternsArray!=nil)
					{
						NSEnumerator * tPatternEnumerator;
						
						tPatternEnumerator=[tPatternsArray objectEnumerator];
						
						if (tPatternEnumerator!=nil)
						{
							NSDictionary * tPatternDictionary;
							
							while (tPatternDictionary=[tPatternEnumerator nextObject])
							{
								unsigned int tType;
							
								tNumber=[tPatternDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE];
								
								tType=[tNumber unsignedIntValue];
								
								if (tType==inType || tType==ICFILE_TYPE_FILE_OR_FOLDER)
								{
									NSString * tPattern;
									
									tPattern=[tPatternDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_STRING];
									
									if (tPattern!=nil && [tPattern length]>1)
									{
										tNumber=[tPatternDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_REGULAR_EXPRESSION];
									
										if ([tNumber boolValue]==NO)
										{	
											if ([inName caseInsensitiveCompare:tPattern]==NSOrderedSame)
											{
												return YES;
											}
										}
										else
										{
											// Regular Expression
											
											if ([inName isMatchedByRegex:tPattern options:RKLNoOptions inRange:((NSRange){0, NSUIntegerMax}) error:NULL]==YES)
											{
												return YES;
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	else
	{
		if (inName==nil)
		{
			NSLog(@"[ICFileBouncer shouldBounceItemNamed:ofType] Name is nil");
		}
		
		if (inType>ICFILE_TYPE_FOLDER)
		{
			NSLog(@"[ICFileBouncer shouldBounceItemNamed:ofType] Type is unknown (%u)",inType);
		}
	}
	
	return NO;
}

@end
