/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICFileDropNotifierView.h"

@implementation NSObject (ICFileDropNotifierViewDelegate)

- (BOOL) validateDropFiles:(NSArray *) inFilenames
{
	return NO;
}

- (BOOL) acceptDropFiles:(NSArray *) inFilenames
{
	return NO;
}

@end

@implementation ICFileDropNotifierView

- (id) initWithFrame:(NSRect) inFrame
{
	self=[super initWithFrame:inFrame];
	
	if (self!=nil)
	{
		// A COMPLETER
		
		// Register for Drop
		
		[self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	}
	
	return self;
}

#pragma mark -

- (id) delegate
{
	return delegate_;
}

- (void) setDelegate:(id) inDelegate
{
	delegate_=inDelegate;
}

#pragma mark -

- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>)sender
{
    NSPasteboard * tPasteBoard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
	
    tPasteBoard = [sender draggingPasteboard];

    if (delegate_!=nil && [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
    {
        if (sourceDragMask & NSDragOperationCopy)
        {
            NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
			if (tFiles!=nil)
			{
				if ([delegate_ validateDropFiles:tFiles]==YES)
				{
					highlighted_=YES;
					
					[self setNeedsDisplay:YES];
				
					return NSDragOperationCopy;
				}
			}
        }
    }
    
    return NSDragOperationNone;
}

- (BOOL) performDragOperation:(id <NSDraggingInfo>)sender
{
    NSPasteboard * tPasteBoard;
	
    tPasteBoard = [sender draggingPasteboard];

    if (delegate_!=nil && [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
    {
        NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
		if (tFiles!=nil)
		{
			return [delegate_ acceptDropFiles:tFiles];
		}
    }
	
    return NO;
}

- (BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    highlighted_=NO;
	
    [self setNeedsDisplay:YES];
	
    return YES;
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
	highlighted_=NO;
	
    [self setNeedsDisplay:YES];
}

@end
