/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICFileDropView.h"
#import "NSBezierPath+RoundedRect.h"
#import "ICIconsUtilities.h"

NSString * const ICDropViewFilePathDidChangeNotification=@"ICDropViewFilePathDidChangeNotification";

@implementation NSObject (ICFileDropViewNotification)

- (void) dropViewFilePathDidChange:(NSNotification *) inNotification
{
}

@end

@implementation ICFileDropView

- (id) initWithFrame:(NSRect) inFrame
{
	self=[super initWithFrame:inFrame];
	
	if (self!=nil)
	{
		// A COMPLETER
		
		// Register for Drop
		
		[self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	}
	
	return self;
}

- (void) dealloc
{
	[filePath_ release];
		
	[cachedIcon_ release];
		
	[super dealloc];
}

#pragma mark -

- (void) viewWillMoveToWindow:(NSWindow *) inWindow
{
	[super viewWillMoveToWindow:inWindow];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	if (inWindow!=nil)
	{
		// Register for Notification
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowDidBecomeMain:)
													 name:NSWindowDidBecomeMainNotification
												   object:inWindow];
												   
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowDidResignMain:)
													 name:NSWindowDidResignMainNotification
												   object:inWindow];
	}
}

- (NSString *) filePath
{
	return [[filePath_ retain] autorelease];
}

- (void) setFilePath:(NSString *) inPath
{
	if (inPath!=filePath_)
	{
		[filePath_ release];
		
		[cachedIcon_ release];
		
		if (inPath!=nil && [[NSFileManager defaultManager] fileExistsAtPath:inPath]==NO)
		{
			inPath=nil;
		}
		
		filePath_=[inPath copy];
		
		if (filePath_!=nil)
		{
			cachedIcon_=[[[NSWorkspace sharedWorkspace] iconForFile:filePath_] retain];
			
			[cachedIcon_ setScalesWhenResized:YES];
			
			[cachedIcon_ setSize:NSMakeSize(128.0f,128.0f)];
		}
		else
		{
			cachedIcon_=nil;
		}
		
		[self setNeedsDisplay:YES];
		
		
	}
}

- (void) _drawBackground
{
	NSBezierPath * tBezierPath;
	NSRect tBounds;
	
	tBounds=[self bounds];	
	
	tBounds=NSInsetRect(tBounds,2,2);
	
	tBezierPath=[NSBezierPath bezierPathWithRoundedRect:tBounds radius:5];
	
	if (tBezierPath!=nil)
	{
		CGFloat tArray[2]={5.0f,2.0f};
		
		[[NSColor colorWithDeviceWhite:0.85f alpha:0.5f] set];
		
		[tBezierPath fill];
		
		[tBezierPath setLineDash:tArray count:2 phase:0.5f];
		
		[[NSColor colorWithDeviceWhite:0.7f alpha:0.5f] set];
		
		[tBezierPath setLineWidth:2.0f];
		
		[tBezierPath stroke];
	}
}

- (void) drawRect:(NSRect) inRect
{
    NSRect tBounds;
	
	tBounds=[self bounds];
		
	if (filePath_==nil)
	{
		[self _drawBackground];
	}
	else
	{
		if (filePath_!=nil && [[NSFileManager defaultManager] fileExistsAtPath:filePath_]==NO)
		{
			NSImage * tImage;
			NSRect tQuestionMarkRect;
			NSRect tSourceRect;
			
			[self _drawBackground];
			
			// Draw Question Mark
			
			tImage=[[NSWorkspace sharedWorkspace] iconForFileType:NSFileTypeForHFSTypeCode(kQuestionMarkIcon)];
			
			[tImage setScalesWhenResized:YES];
			
			[tImage setSize:NSMakeSize(_CGFloatRound(NSWidth(tBounds)*0.6f),_CGFloatRound(NSHeight(tBounds)*0.6f))];
			
			tSourceRect.origin=NSZeroPoint;
			tSourceRect.size=[tImage size];
			
			tQuestionMarkRect.size=tSourceRect.size;
			tQuestionMarkRect.origin=NSMakePoint(_CGFloatRound(NSWidth(tBounds)*0.2f),_CGFloatRound(NSHeight(tBounds)*0.2f));
			
			[tImage drawInRect:tQuestionMarkRect fromRect:tSourceRect operation:NSCompositeSourceOver fraction:1.0f];
		}
		else
		{
			NSRect tSourceRect;
			NSGraphicsContext * tContext;
			
			tSourceRect.origin=NSZeroPoint;
			tSourceRect.size=[cachedIcon_ size];
			
			tContext=[NSGraphicsContext currentContext];
			
			[tContext saveGraphicsState];
			
			[tContext setImageInterpolation:NSImageInterpolationHigh];
			
			[cachedIcon_ drawInRect:tBounds fromRect:tSourceRect operation:NSCompositeSourceOver fraction:1.0f];
			
			[tContext restoreGraphicsState];
		}
	}
	
	if (highlighted_==YES)
	{
		NSBezierPath * tPath;
		
		tPath=[NSBezierPath bezierPathWithRoundedRect:NSInsetRect(tBounds,2.0f,2.0f) radius:8.0f];
		
		if (tPath!=nil)
		{
			[tPath setLineWidth:2.0f];
			
			[[NSColor alternateSelectedControlColor] set];
			
			[tPath stroke];
		}
	}
}

#pragma mark -

- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>)sender
{
    NSPasteboard * tPasteBoard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
	
    tPasteBoard = [sender draggingPasteboard];

    if ( [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
    {
        if (sourceDragMask & NSDragOperationCopy)
        {
            NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
			if (tFiles!=nil && [tFiles count]==1)
			{
				NSString * tFilePath;
				BOOL isDirectory=NO;
				
				tFilePath=[tFiles objectAtIndex:0];
				
				if ([[NSFileManager defaultManager] fileExistsAtPath:tFilePath isDirectory:&isDirectory]==YES && isDirectory==NO)
				{
					highlighted_=YES;
			
					[self setNeedsDisplay:YES];
            
					return NSDragOperationCopy;
				}
			}
        }
    }
    
    return NSDragOperationNone;
}

- (BOOL) performDragOperation:(id <NSDraggingInfo>)sender
{
    NSPasteboard * tPasteBoard;
	
    tPasteBoard = [sender draggingPasteboard];

    if ( [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
    {
        NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
		if (tFiles!=nil && [tFiles count]==1)
		{
			NSString * tFilePath;
			
			tFilePath=[tFiles objectAtIndex:0];
			
			[self setFilePath:tFilePath];
			
			[[NSNotificationCenter defaultCenter] postNotificationName:ICDropViewFilePathDidChangeNotification
																object:self
															  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:tFilePath,@"Path",nil]];
			
			return YES;
		}
    }
	
    return NO;
}

- (BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	highlighted_=NO;
	
    [self setNeedsDisplay:YES];
    
    return YES;
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
    highlighted_=NO;
	
    [self setNeedsDisplay:YES];
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
}

#pragma mark -

- (void) windowDidBecomeMain:(NSNotification *) inNotification
{
	[self setNeedsDisplay:YES];
}

- (void) windowDidResignMain:(NSNotification *) inNotification
{
	[self setNeedsDisplay:YES];
}

@end
