/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICSegmentedController.h"

#include <sys/stat.h>

#import "ICFileTree.h"

#import "ImageAndTextCell.h"

#import "ICDocument.h"

#import "ICFileNameFormatter.h"

#import "ICPreferences+Constants.h"

// Inspectors

#import "ICFileHierarchyInspectorEmptySelectionController.h"

#import "ICFileHierarchyInspectorSelectionController.h"

#define SAFEFILENODE(n) 	((ICFileTree*)((n!=nil)?(n):(fileTree_)))

extern NSString * const ICFileHierarchyPboardType;

extern NSString * const ICFileHierarchyExternalPboardType;

extern NSString * const IC_ADD_FILES;

extern NSString * const IC_ADD_PARENTS;

@interface ICFileHierarchyController : ICSegmentedController
{
	IBOutlet id IBoutlineView_;
	
	IBOutlet id IBaddButton_;
	
	IBOutlet id IBremoveButton_;
	
	IBOutlet id IBinspectorView_;
	
	IBOutlet id IBreferenceWindow_;
	
	IBOutlet id IBreferenceStyle_;
	
	// Data
	
	NSString * cachedPackageUUID_;
	
	NSUserDefaults * defaults_;
	
	ICFileTree * fileTree_;
	
	ICFileTree * parentFileTree_;
	
	NSImage * genericFolderIcon_;
	
	NSImage * genericDisabledFolderIcon_;
	
	BOOL hierarchyChanged_;
	
	ICFileNameFormatter * fileNameFormatter_;
	
	id emptySelectionInspectorController_;
	
	id selectionInspectorController_;
	
	id currentInspectorController_;
	
	BOOL cachedHighlightExcludedFiles_;
	
	ICFileBouncer * cachedFileBouncer_;
	
	NSMutableArray * internalDragArray_;
}

- (void) delayedHierarchyLoad:(id) sender;

- (NSString *) selectionKey;

- (NSString *) disclosedStateKey;

- (NSString *) disclosedStateFormatKey;

- (NSString *) selectionFormatKey;

- (IBAction) addFiles:(id)sender;


- (void) addFilesPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo;

- (void) beginFileSheet:(NSDictionary *) inDictionary;

- (void) beginFileSheetForFiles:(NSArray *) inFiles parents:(id) inParents;

- (IBAction) endFileSheet:(id) sender;

- (void) fileSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;


- (void) expandItem:(ICFileTree *) inFileTree ifKeyOfDictionary:(NSMutableDictionary *) inDictionary;

- (BOOL) defaultRestoreForItem:(id) inItem;

- (void) restoreDisclosedStates;


- (ICFileTree *) fileTreeWithSelectionPath:(NSString *) inPath;

- (NSString *) selectionPathWithFileTree:(ICFileTree *) inFileTree;

- (void) archiveSelection;

- (void) restoreSelection;


- (void) _addFiles:(NSArray *) inFilesArray  toParents:(NSArray *) inParents keepOwnerAndGroup:(BOOL) inKeepOwnerAndGroup referenceStyle:(NSInteger) inReferenceStyle;

- (IBAction) removeFile:(id)sender;

- (void) removeFileSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;

- (IBAction) revealInFinder:(id) sender;

- (IBAction) newFolder:(id) sender;

- (IBAction) expandAll:(id) sender;

- (void) delayedExpandAll:(id) inObject;

- (IBAction) expand:(id) sender;

- (void) delayedExpand:(id) inObject;

- (IBAction) expandOneLevel:(id) sender;

- (IBAction) contract:(id) sender;



- (IBAction) updateFiles:(id) sender;

// Notifications

- (void) folderNameShouldChange:(NSNotification *) inNotification;

- (void) fileAttributesDidChange:(NSNotification *) inNotification;

- (void) highlightExcludedFilesDidChange:(NSNotification *) inNotification;

@end
