/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICFileHierarchyController.h"

#import "ICIconsUtilities.h"

#import "NSImage+Composition.h"

#import "ICDocument+Constants.h"

NSString * const ICFileHierarchyPboardType=@"ICFileHierarchyPboardType";

NSString * const ICFileHierarchyExternalPboardType=@"ICFileHierarchyExternalPboardType";


NSString * const IC_ADD_FILES=@"IC_ADD_FILES";

NSString * const IC_ADD_PARENTS=@"IC_ADD_PARENTS";


@implementation ICFileHierarchyController

- (void) dealloc
{
	[internalDragArray_ release];
	
	[genericFolderIcon_ release];
	
	[genericDisabledFolderIcon_ release];
	
	[fileTree_ release];
	
	[emptySelectionInspectorController_ release];
	
	[selectionInspectorController_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{
	ImageAndTextCell * tImageTextCell;
    NSTableColumn * tTableColumn;
	NSSize tSize;
    id tPrototypeCell;
	NSFont * tSmallLabelFont;
	
	defaults_=[NSUserDefaults standardUserDefaults];
	
	cachedHighlightExcludedFiles_=[defaults_ boolForKey:ICPREFERENCES_FILES_HIGHLIGHT_EXCLUDED_FILES];
	
	tSmallLabelFont=[NSFont labelFontOfSize:[NSFont smallSystemFontSize]];
	
	// Cached Icons
	
	//	Generic Folder Icon
		
	genericFolderIcon_=[[ICIconsUtilities smallIconForType:NSFileTypeForHFSTypeCode(kGenericFolderIcon)] retain];
	
	genericDisabledFolderIcon_=[[genericFolderIcon_ imageWithAlpha:0.5f] retain];
	
	// Outline View
	
	fileNameFormatter_=[ICFileNameFormatter new];
	
	tTableColumn = [IBoutlineView_ tableColumnWithIdentifier: @"File"];
	
    tImageTextCell = [[ImageAndTextCell alloc] init];
	
	if (tImageTextCell!=nil)
	{
		[tImageTextCell setEditable:YES];
		
		[tImageTextCell setFormatter:fileNameFormatter_];
		
		[tImageTextCell setFont:tSmallLabelFont];
		
		[tTableColumn setDataCell:tImageTextCell];
		
		[tImageTextCell release];
    }
	
	tTableColumn = [IBoutlineView_ tableColumnWithIdentifier: @"Permissions"];
    
	tPrototypeCell=[tTableColumn dataCell];
	
	if (tPrototypeCell!=nil)
	{
		[tPrototypeCell setFont:tSmallLabelFont];
    }
    
    [IBoutlineView_ setAutoresizesOutlineColumn:NO];
    
    tSize=[IBoutlineView_ intercellSpacing];
    
    tSize.height=2.0f;
    
    [IBoutlineView_ setIntercellSpacing:tSize];
	
	// Register for notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(highlightExcludedFilesDidChange:)
                                                 name:ICPREFERENCES_NOTIFICATION_FILES_HIGHLIGHT_EXCLUDED_FILES_DID_CHANGE
                                               object:nil];
}

#pragma mark -

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{	
	[fileTree_ release];
	
	fileTree_=nil;
	
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
}

- (void) delayedHierarchyLoad:(id) sender
{
}

- (void) viewWillBeRemoved:(id) sender
{
	[NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayedHierarchyLoad:) object:nil];
	
	// A COMPLETER
	
	if (fileTree_!=nil)
	{
		// Archive Selection
	
		[self archiveSelection];
	}
}

- (IBAction) updateFiles:(id) sender
{
}

#pragma mark -

- (NSString *) selectionKey
{
	return [NSString stringWithFormat:[self selectionFormatKey],cachedPackageUUID_];
}

- (NSString *) disclosedStateKey
{
	return [NSString stringWithFormat:[self disclosedStateFormatKey],cachedPackageUUID_];
}

- (NSString *) disclosedStateFormatKey
{
	return nil;
}

- (NSString *) selectionFormatKey
{
	return nil;
}

#pragma mark -

- (id) outlineView:(NSOutlineView *) inOutlineView child:(NSInteger) inIndex ofItem:(id)inItem
{
    return [SAFEFILENODE(inItem) childAtIndex:inIndex];
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView isItemExpandable:(id)inItem
{
    return ![FILENODE_DATA(inItem) isLeaf];
}

- (NSInteger) outlineView:(NSOutlineView *) inOutlineView numberOfChildrenOfItem:(id)inItem
{
    return [SAFEFILENODE(inItem) numberOfChildren];
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView shouldEditTableColumn:(NSTableColumn *) inTableColumn item:(id)inItem
{
    BOOL tShouldEdit=NO;
	NSTableColumn * tTableColumn;
	
	tTableColumn = [inOutlineView tableColumnWithIdentifier: @"File"];
	
	if (inTableColumn==tTableColumn)
	{
		tShouldEdit=([FILENODE_DATA(inItem) type] == ICDOCUMENT_FILE_NEWFOLDER_NODE);
	}
	
	return tShouldEdit;
}

- (void) outlineView:(NSOutlineView *) inOutlineView setObjectValue:(id) inObject forTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
    if ([[inTableColumn identifier] isEqualToString: @"File"])
    {	
		NSString * tNewName;
		NSUInteger tLength;
		
		tNewName=(NSString *) inObject;
		
		tLength=[tNewName length];
		
		if (tLength==0 ||
			tLength>256 ||
			[tNewName isEqualToString:@".."]==YES ||
			[tNewName isEqualToString:@"."]==YES ||
			[tNewName rangeOfString:@"/"].location!=NSNotFound)
		{
			NSBeep();
		}
		else
		{
			ICFileTree * tTree;
			NSDictionary * tDictionary;
			
			if ([[FILENODE_DATA(inItem) path] compare:tNewName options:NSCaseInsensitiveSearch]!=NSOrderedSame)
			{
				if ([((ICFileTree *)[inItem nodeParent]) containsTreeWithName:tNewName]==YES)
				{
					NSBeep();
					
					[inOutlineView editColumn:[inOutlineView columnWithIdentifier:@"File"] row:[inOutlineView rowForItem:inItem] withEvent:nil select:YES];
					
					return;
				}
			}
			else
			{
				if ([[FILENODE_DATA(inItem) path] compare:tNewName]==NSOrderedSame)
				{
					return;
				}
			}
			
			tTree=(ICFileTree *) inItem;
			
			// Post Notification
			
			tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tTree,ICFILEHIERARCHYINSPECTOR_FILETREE,
																   tNewName,ICFILEHIERARCHYINSPECTOR_NEWNAME,
																   nil];
			
			if (tDictionary!=nil)
			{
				[[NSNotificationCenter defaultCenter] postNotificationName:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_FOLDERNAME_SHOULD_CHANGE
																	object:[self document]
																  userInfo:tDictionary];
			}
		}
	}
}

#pragma mark -

- (void) expandItem:(ICFileTree *) inFileTree ifKeyOfDictionary:(NSMutableDictionary *) inDictionary
{
	if (inFileTree!=nil && [inDictionary count]>0)
	{
		if ([FILENODE_DATA(inFileTree) isLeaf]==NO)
		{
			NSString * tFilePath;
			NSArray * tChildren;
			NSEnumerator * tEnumerator;
			NSNumber * tNumber;
			
			
			tFilePath=[inFileTree filePath];
			
			[IBoutlineView_ expandItem:inFileTree];
			
			// Check children
			
			tChildren=[inFileTree children];
				
			tEnumerator=[tChildren objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				ICFileTree * tFileTree;
				
				while (tFileTree=[tEnumerator nextObject])
				{
					[self expandItem:tFileTree ifKeyOfDictionary:inDictionary];
				}
			}
			
			tNumber=[inDictionary objectForKey:tFilePath];
			
			if (tNumber==nil || [tNumber boolValue]==NO)
			{
				[IBoutlineView_ collapseItem:inFileTree];
			}
		}
	}
}

- (BOOL) defaultRestoreForItem:(id) inItem
{
	if (inItem!=nil)
	{
		ICFileNode * tFileNode;
		int tFileType;
		
		tFileNode=FILENODE_DATA(inItem);
        
		tFileType=[tFileNode type];
		
		if (tFileType==ICDOCUMENT_FILE_BASE_NODE_INVISIBLE ||
			tFileType==ICDOCUMENT_FILE_BASE_NODE ||
			tFileType==ICDOCUMENT_FILE_ROOT_NODE)
		{
			NSArray * tChildren;
			BOOL tRealItemFound=NO;
			
			if (tFileType==ICDOCUMENT_FILE_BASE_NODE_INVISIBLE ||
				tFileType==ICDOCUMENT_FILE_BASE_NODE)
			{
				if ([inItem hasRealChildren]==NO)
				{
					return NO;
				}
				
				tRealItemFound=YES;
				
				// Expand the item
					
				[IBoutlineView_ expandItem:inItem];
			}	
					
			// Inspect the children
					
			tChildren=[inItem children];
					
			if (tChildren!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tChildren objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					id tChild;
					
					while (tChild=[tEnumerator nextObject])
					{
						if ([self defaultRestoreForItem:tChild]==YES)
						{
							tRealItemFound=YES;
						}
					}
				}
			}
			
			return tRealItemFound;
		}
	}

	return NO;
}

- (void) restoreDisclosedStates
{
	NSDictionary * tDictionary;
	
	tDictionary=[document_ documentDefaultForKey:[self disclosedStateKey]];
			
	if (tDictionary!=nil)
	{
		NSMutableDictionary * tDictionaryCopy;
		
		tDictionaryCopy=[tDictionary mutableCopy];
		
		if (tDictionaryCopy!=nil)
		{
			NSArray * tChildren;
			
			tChildren=[fileTree_ children];
			
			if (tChildren!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tChildren objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					ICFileTree * tFileTree;
					
					while (tFileTree=[tEnumerator nextObject])
					{
						[self expandItem:tFileTree ifKeyOfDictionary:tDictionaryCopy];
					}
				}
			}
		
			[tDictionaryCopy release];
		}
	}
}

#pragma mark -

- (ICFileTree *) fileTreeWithSelectionPath:(NSString *) inPath
{
	return [((ICFileTree *) [fileTree_ firstChild]) fileTreeAtPath:inPath];
}

- (NSString *) selectionPathWithFileTree:(ICFileTree *) inFileTree
{
	return [inFileTree filePath];
}

- (void) archiveSelection
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBoutlineView_ selectedRowIndexes];
	
	if (tIndexSet!=nil)
	{
		NSMutableArray * tMutableArray;
		
		tMutableArray=[NSMutableArray array];
		
		if (tMutableArray!=nil)
		{
			NSUInteger tIndex;
			
			tIndex=[tIndexSet firstIndex];
			
			while (tIndex!=NSNotFound)
			{
				ICFileTree * tFileTree;
					
				tFileTree=[IBoutlineView_ itemAtRow:tIndex];
				
				if (tFileTree!=nil)
				{
					NSString * tPath;
					
					tPath=[self selectionPathWithFileTree:tFileTree];
					
					if (tPath!=nil)
					{
						[tMutableArray addObject:tPath];
					}
				}
				
			
				tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
			}
			
			[document_ setDocumentDefault:tMutableArray forKey:[self selectionKey]];
		}
		else
		{
			NSLog(@"[ICFileHierarchyController archiveSelection]: Low Memory");
			
			[document_ removeDocumentDefaultForKey:[self selectionKey]];
		}
	}
}

- (void) restoreSelection
{
	NSArray * tArray;
	
	tArray=[document_ documentDefaultForKey:[self selectionKey]];
			
	if (tArray!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[tArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSMutableIndexSet * tMutableIndexSet;
			
			tMutableIndexSet=[NSMutableIndexSet indexSet];
			
			if (tMutableIndexSet!=nil)
			{
				NSString * tPath;
			
				while (tPath=[tEnumerator nextObject])
				{
					ICFileTree * tFileTree;
					
					tFileTree=[self fileTreeWithSelectionPath:tPath];
				
					if (tFileTree!=nil)
					{
						NSInteger tRow;
						
						tRow=[IBoutlineView_ rowForItem:tFileTree];
						
						if (tRow!=-1)
						{
							[tMutableIndexSet addIndex:tRow];
						}
					}
				}
				
				[IBoutlineView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:NO];
			}
		}
	}
}

#pragma mark -

- (void) outlineView:(NSOutlineView *) inOutlineView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inConfirmationRequired==YES)
	{
		[self removeFile:nil];
	}
	else
	{
		[self removeFileSheetDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
	}
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView validateAction:(SEL) inSelector
{
	if (inSelector==@selector(delete:))
	{
		NSIndexSet * tIndexSet;
	
		tIndexSet=[inOutlineView selectedRowIndexes];
		
		if (tIndexSet!=nil && [tIndexSet count]>0)
		{
			NSUInteger tIndex;
				
			tIndex=[tIndexSet firstIndex];
			
			while (tIndex!=NSNotFound)
			{
				ICFileTree * tFileTree;
				int tType;
				
				tFileTree=(ICFileTree *) [IBoutlineView_ itemAtRow:tIndex];
			
				tType=[FILENODE_DATA(tFileTree) type];
			
				if (tType==ICDOCUMENT_FILE_BASE_NODE_INVISIBLE ||
					tType==ICDOCUMENT_FILE_BASE_NODE)
				{
					return NO;
				}
				
				tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
			}
			
			return YES;
		}
	}
	
	return NO;
}

#pragma mark -

- (BOOL)validateMenuItem:(NSMenuItem *) inMenuItem
{    
    ICFileTree * tFileTree;
    NSInteger tSelectedCount;
    SEL tAction;
	
	tAction=[inMenuItem action];
	
	if (tAction==@selector(expandAll:))
    {
        // A COMPLETER
		
		return YES;
    }
    
    tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
    
	if (tAction==@selector(addFiles:))
	{
		return (tSelectedCount<=1);
	}
	
    if (tSelectedCount>0)
	{
		NSIndexSet * tIndexSet;
				
		tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
		if (tIndexSet!=nil)
		{
			NSUInteger tIndex;
			
			if (tAction==@selector(removeFile:))
			{
				tIndex=[tIndexSet firstIndex];
				
				while (tIndex!=NSNotFound)
				{
					int tType;
					
					tFileTree=(ICFileTree *) [IBoutlineView_ itemAtRow:tIndex];
				
					tType=[FILENODE_DATA(tFileTree) type];
					
					if (tType==ICDOCUMENT_FILE_BASE_NODE_INVISIBLE ||
						tType==ICDOCUMENT_FILE_BASE_NODE)
					{
						return NO;
					}
					
					tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
				}
				
				return YES;
			}
			else if (tAction==@selector(revealInFinder:))
			{
				tIndex=[tIndexSet firstIndex];
				
				while (tIndex!=NSNotFound)
				{
					tFileTree=(ICFileTree *) [IBoutlineView_ itemAtRow:tIndex];
				
					if ([FILENODE_DATA(tFileTree) type]!=ICDOCUMENT_FILE_REALFILE_NODE)
					{
						return NO;
					}
					
					tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
				}
				
				return YES;
			}
		}
	}
	
	if (tSelectedCount==1)
    {
        NSInteger tSelectedRow;
        
        tSelectedRow=[IBoutlineView_ selectedRow];
    
        if (tSelectedRow>=0)
        {
            int tType;
			
			tFileTree=[IBoutlineView_ itemAtRow:tSelectedRow];
			
			tType=[FILENODE_DATA(tFileTree) type];
			
			if (tAction==@selector(newFolder:))
			{
				return YES;
			}
			else
			{
				if (tType==ICDOCUMENT_FILE_REALFILE_NODE)
				{
					
					if ((tAction==@selector(expand:)			||
						tAction==@selector(expandOneLevel:))	&& 
						[tFileTree numberOfChildren]==0)
					{
						NSString * tPath;
						NSFileManager * tFileManager;
						BOOL isDirectory;
						
						tPath=[FILENODE_DATA(tFileTree) path];
						
						tFileManager=[NSFileManager defaultManager];
						
						if ([tFileManager fileExistsAtPath:tPath isDirectory:&isDirectory]==YES && isDirectory==YES)
						{
							NSDictionary * tFileAttributes;
							NSString * tString;
							
							tFileAttributes=[tFileManager fileAttributesAtPath:tPath traverseLink:NO];
							
							tString=[tFileAttributes objectForKey:NSFileType];
							
							if (tString!=nil)
							{
								return ([tString isEqualToString:NSFileTypeDirectory]);
							}
						}
					}
					else if (tAction==@selector(contract:))
					{
						if ([tFileTree numberOfChildren]!=0)
						{
							return YES;
						}
					}
				}
            }
			
            return NO;
        }
    }
    else
    {
        return NO;
    }
    
    return YES;
}

- (IBAction) addFiles:(id)sender
{
}

- (BOOL) panel:(id)sender shouldShowFilename:(NSString *) inFileName
{
    // A AMELIORER (We need to be able to add items from these places)
	
	BOOL isDirectory;
	
	if ([[NSFileManager defaultManager] fileExistsAtPath:inFileName isDirectory:&isDirectory]==YES)
	{
		if (isDirectory==NO)
		{
			return ![parentFileTree_ containsTreeWithName:[inFileName lastPathComponent]];
		}
	}
	
	return YES;
}

- (void) addFilesPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
}

- (void) beginFileSheet:(NSDictionary *) inDictionary
{
	[self beginFileSheetForFiles:[inDictionary objectForKey:IC_ADD_FILES] parents:[inDictionary objectForKey:IC_ADD_PARENTS]];

	[inDictionary autorelease];
}

- (void) beginFileSheetForFiles:(NSArray *) inFiles parents:(id) inParents
{
}

- (void) fileSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
}

- (IBAction) endFileSheet:(id) sender
{
    [NSApp endSheet:IBreferenceWindow_ returnCode:[sender tag]];
}

- (IBAction) removeFile:(id)sender
{
    NSString * tAlertTitle;
    
    if ([IBoutlineView_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this item?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these items?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeFileSheetDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeFileSheetDidEnd:(NSWindow *) sheet returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
        NSIndexSet * tIndexSet;
        NSMutableDictionary * tDisclosedDictionary;
					
		tDisclosedDictionary=[document_ documentDefaultForKey:[self disclosedStateKey]];

		tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
		if (tIndexSet!=nil)
		{
			NSMutableArray * tSelectedNodesMutableArray;
			NSArray * tCoverageNodesArray;
			
			tSelectedNodesMutableArray=[NSMutableArray array];
			
			if (tSelectedNodesMutableArray!=nil)
			{
				NSUInteger tIndex;
			
				tIndex=[tIndexSet lastIndex];
			
				while (tIndex!=NSNotFound)
				{
					ICFileTree * tNode;
					
					tNode=[IBoutlineView_ itemAtRow:tIndex];
					
					if (tNode!=nil)
					{
						[tSelectedNodesMutableArray addObject:tNode];
					}
					
					tIndex=[tIndexSet indexLessThanIndex:tIndex];
				}
				
				// Create the coverage array of nodes
				
				tCoverageNodesArray=[TreeNode minimumNodeCoverFromNodesInArray:tSelectedNodesMutableArray];
				
				if (tCoverageNodesArray!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[tCoverageNodesArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						ICFileTree * tNode;
						
						while (tNode=[tEnumerator nextObject])
						{
							NSMutableArray * tChildFoldersArray;
							
							// Removed item and descendants from disclosed state dictionary (because the key will change)
				
							tChildFoldersArray=[tNode descendantFoldersIncludingSelf:YES];
							
							if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
							{
								NSUInteger i,tCount;
								
								tCount=[tChildFoldersArray count];
								
								for(i=tCount;i>0;i--)
								{
									ICFileTree * tGroupFileTree;
									NSString * tPath;
									
									tGroupFileTree=[tChildFoldersArray objectAtIndex:(i-1)];
									
									tPath=[tGroupFileTree filePath];
									
									if (tPath!=nil)
									{
										[tDisclosedDictionary removeObjectForKey:tPath];
									}
								}
							}
							
							// Remove the node
							
							[tNode removeFromParent];
						}
					}
				}
			}
		}
        
        [IBoutlineView_ deselectAll:nil];
        
        hierarchyChanged_=YES;
        
        [IBoutlineView_ reloadData];
        
        [self updateFiles:IBoutlineView_];
    }
}

- (IBAction) revealInFinder:(id) sender
{
    NSWorkspace * tWorkSpace;
    NSIndexSet * tIndexSet;
     
    tWorkSpace=[NSWorkspace sharedWorkspace];
    
	tIndexSet=[IBoutlineView_ selectedRowIndexes];
    
	if (tIndexSet!=nil)
	{
		NSUInteger tIndex;
		
		tIndex=[tIndexSet firstIndex];
		
		while (tIndex!=NSNotFound)
		{
			ICFileTree * tNode;
			
			tNode=[IBoutlineView_ itemAtRow:tIndex];
			
			[tWorkSpace selectFile:[FILENODE_DATA(tNode) path] inFileViewerRootedAtPath:@""];
			
			tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
		}
	}
}

- (IBAction) newFolder:(id) sender
{
    NSInteger tRow;
    id tItem;
    ICFileTree * tParentTree=nil;
    ICFileNode * tParentNode;
    ICFileTree * tNewFolderTree=nil;
    
    tRow=[IBoutlineView_ selectedRow];

    tItem=[IBoutlineView_ itemAtRow:tRow];
    
    switch([FILENODE_DATA(tItem) type])
    {
        case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
		case ICDOCUMENT_FILE_BASE_NODE:
        case ICDOCUMENT_FILE_NEWFOLDER_NODE:
            tParentTree=(ICFileTree *) tItem;
            break;
        case ICDOCUMENT_FILE_REALFILE_NODE:
            tParentTree=(ICFileTree *) [tItem nodeParent];
            break;
    }
    
    if (tParentTree!=nil)
    {
        if ([IBoutlineView_ isItemExpanded:tParentTree]==NO)
        {
            [IBoutlineView_ expandItem:tParentTree];
        }
    
        tParentNode=FILENODE_DATA(tParentTree);
            
        
        
        tNewFolderTree=[[ICFileTree alloc] initWithData:[ICFileNode folderFileNodeWithName:[ICFileTree uniqueNameWithParentFileTree:tParentTree]
                                                                                    icon:nil
                                                                                    user:[tParentNode uid]
                                                                                    group:[tParentNode gid]
                                                                                privileges:([tParentNode privileges] & ACCESSPERMS)]
                                                parent:nil
                                                children:[NSArray array]];
        
        if (tNewFolderTree!=nil)
        {
            [tParentTree insertSortedChild:tNewFolderTree];
            
            [tNewFolderTree release];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadData];
            
            tRow=[IBoutlineView_ rowForItem:tNewFolderTree];
            
            [IBoutlineView_ scrollRowToVisible:tRow];
            
            [IBoutlineView_ selectRow:tRow byExtendingSelection:NO];
            
            [IBoutlineView_ editColumn:[IBoutlineView_ columnWithIdentifier:@"File"] row:tRow withEvent:nil select:YES];
            
            [self updateFiles:IBoutlineView_];
        }
    }
}

- (IBAction) expandAll:(id) sender
{
    NSDictionary * tDictionary;
            
    tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Expanding...",@"No comment"),ICDOCUMENT_STATUS_STRING,
                                                           [NSNumber numberWithInt:-1],ICDOCUMENT_STATUS_ID,
														   nil];
            
    [[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_STATUS_DID_CHANGE
                                                        object:document_
                                                        userInfo:tDictionary];
                                                            
    [self performSelector:@selector(delayedExpandAll:)
               withObject:[NSNumber numberWithBool:(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)!=0)]
               afterDelay:0.1f];
}

- (void) delayedExpandAll:(id) inObject
{
    BOOL tExpandedOne=NO;
			
	tExpandedOne=[fileTree_ expandAll:[inObject boolValue] 
					  withProjectPath:[document_ folder] 
				  referenceFolderPath:[document_ referenceFolderPath]];
	
	[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_STATUS_DID_CHANGE
														object:document_
													  userInfo:[NSDictionary dictionary]];
	
	if (tExpandedOne==YES)
	{
		hierarchyChanged_=YES;
	
		[IBoutlineView_ reloadData];

		[self updateFiles:IBoutlineView_];
	}
}

- (IBAction) expand:(id) sender
{
    NSDictionary * tDictionary;
            
    tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Expanding...",@"No comment"),ICDOCUMENT_STATUS_STRING,
														   [NSNumber numberWithInt:-1],ICDOCUMENT_STATUS_ID,
														    nil];
            
    [[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_STATUS_DID_CHANGE
                                                        object:document_
                                                        userInfo:tDictionary];
                                                            
    [self performSelector:@selector(delayedExpand:)
               withObject:[NSNumber numberWithBool:(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)!=0)]
               afterDelay:0.1f];
}

- (void) delayedExpand:(id) inObject
{
    ICFileTree * tFileTree;
    NSInteger tSelectedCount;
        
    tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
        
    if (tSelectedCount==1)
    {
        NSInteger tSelectedRow;
        
        tSelectedRow=[IBoutlineView_ selectedRow];
    
        if (tSelectedRow>=0)
        {
            tFileTree=[IBoutlineView_ itemAtRow:tSelectedRow];
            
            [tFileTree expand:[inObject boolValue]];
            
            [[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_STATUS_DID_CHANGE
                                                                object:document_
                                                              userInfo:[NSDictionary dictionary]];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadItem:tFileTree];
        
            [self updateFiles:IBoutlineView_];
        }
    }
}

- (IBAction) contract:(id) sender
{
    ICFileTree * tFileTree;
    NSInteger tSelectedCount;
        
    tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
        
    if (tSelectedCount==1)
    {
        NSInteger tSelectedRow;
        
        tSelectedRow=[IBoutlineView_ selectedRow];
    
        if (tSelectedRow>=0)
        {
            tFileTree=[IBoutlineView_ itemAtRow:tSelectedRow];
            
            [IBoutlineView_ collapseItem:tFileTree];
            
            [tFileTree contract];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadItem:tFileTree];
            
            [self updateFiles:IBoutlineView_];
        }
    }
}

- (IBAction) expandOneLevel:(id) sender
{
    ICFileTree * tFileTree;
    NSInteger tSelectedCount;
        
    tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
        
    if (tSelectedCount==1)
    {
        NSInteger tSelectedRow;
        
        tSelectedRow=[IBoutlineView_ selectedRow];
    
        if (tSelectedRow>=0)
        {
            tFileTree=[IBoutlineView_ itemAtRow:tSelectedRow];
            
            [tFileTree expandOneLevel:(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)!=0)];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadItem:tFileTree];
            
            [self updateFiles:IBoutlineView_];
        }
    }
}

#pragma mark -

- (void) _addFiles:(NSArray *) inFilesArray toParents:(NSArray *) inParents keepOwnerAndGroup:(BOOL) inKeepOwnerAndGroup referenceStyle:(NSInteger) inReferenceStyle
{
	NSMutableArray * tNewSelectionArray;
	NSUInteger i,tCount;
	NSMutableIndexSet * tMutableIndexSet;
	BOOL hasCommonParent=NO;
	ICFileTree * tParentFileTree=nil;
	
	if ([inParents count]==1)
	{
		hasCommonParent=YES;
		
		tParentFileTree=[inParents objectAtIndex:0];
	}
	
	
	tNewSelectionArray=[NSMutableArray array];
	
	tCount=[inFilesArray count];
	
	for(i=0;i<tCount;i++)
	{
		ICFileTree * tFileTree;
		id tFileNode;
		NSString * tPath;
		NSString * tLastPathComponent;
		
		tPath=[inFilesArray objectAtIndex:i];
		
		tLastPathComponent=[tPath lastPathComponent];
		
				
		// 01/03/07 : Removed fileExistsAtPath:isDirectory: call
		
		tFileNode=[ICFileNode fileNodeWithType:ICDOCUMENT_FILE_REALFILE_NODE
										  path:tPath
									  pathType:inReferenceStyle];
										
		if (tFileNode!=nil)
		{
			struct stat tStat;

			if (hasCommonParent==NO)
			{
				tParentFileTree=[inParents objectAtIndex:i];
			}
			
			if ([tParentFileTree containsTreeWithName:tLastPathComponent]==NO)
			{
				if (lstat([tPath fileSystemRepresentation], &tStat)==0)
				{
					if (inKeepOwnerAndGroup==NO)
					{
						ICFileNode * tTargetNode=nil;
						
						tTargetNode=FILENODE_DATA(tParentFileTree);
						
						[tFileNode setUid:[tTargetNode uid]];
						[tFileNode setGid:[tTargetNode gid]];
					}
					else
					{
						[tFileNode setUid:tStat.st_uid];
						[tFileNode setGid:tStat.st_gid];
					}
					
					[tFileNode setPrivileges: (tStat.st_mode & ALLPERMS)];
					
					tFileTree=[[ICFileTree alloc] initWithData:tFileNode
													parent:tParentFileTree
												  children:nil];

					if (tFileTree!=nil)
					{
						[tParentFileTree insertSortedChild:tFileTree];
						
						[tNewSelectionArray addObject:tFileTree];
						
						[tFileTree release];
					}
				}
				else
				{
					NSLog(@"");
				}
			}
			else
			{
				NSLog(@"");
			}
		}
	}
	
	[IBoutlineView_ deselectAll:nil];

	hierarchyChanged_=YES;
	
	[IBoutlineView_ reloadData];
	
	if (hasCommonParent==YES)
	{
		tCount=1;
		
	}
	
	for(i=0;i<tCount;i++)
	{
		BOOL couldNeedToBeExpanded=NO;
		
		tParentFileTree=[inParents objectAtIndex:i];
		
		switch([FILENODE_DATA(tParentFileTree) type])
		{
			case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
			case ICDOCUMENT_FILE_BASE_NODE:
			case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				couldNeedToBeExpanded=YES;
				break;
			case ICDOCUMENT_FILE_REALFILE_NODE:
				break;
		}
		
		if (couldNeedToBeExpanded==YES)
		{
			if ([IBoutlineView_ isItemExpanded:tParentFileTree]==NO)
			{
				[IBoutlineView_ expandItem:tParentFileTree];
			}
		}
	}
	
	tCount=[tNewSelectionArray count];
	
	tMutableIndexSet=[NSMutableIndexSet indexSet];
	
	for(i=0;i<tCount;i++)
	{
		[tMutableIndexSet addIndex:[IBoutlineView_ rowForItem:[tNewSelectionArray objectAtIndex:i]]];
	}
	
	[IBoutlineView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:NO];
	
	[self updateFiles:IBoutlineView_];
}

#pragma mark -

- (void) outlineViewItemDidExpand:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			ICFileTree * tTreeNode;
			
			tTreeNode=(ICFileTree *) [tUserInfo objectForKey:@"NSObject"];
			
			if (tTreeNode!=nil)
			{
				NSString * tFilePath;
				
				tFilePath=[tTreeNode filePath];
				
				if (tFilePath!=nil)
				{
					NSMutableDictionary * tDisclosedDictionary;
					NSString * tKey;
					
					tKey=[self disclosedStateKey];
					
					tDisclosedDictionary=[document_ documentDefaultForKey:tKey];
					
					if (tDisclosedDictionary==nil)
					{
						tDisclosedDictionary=[NSMutableDictionary dictionary];
						
						if (tDisclosedDictionary!=nil)
						{
							[document_ setDocumentDefault:tDisclosedDictionary forKey:tKey];
						}
						else
						{
							NSLog(@"[ICFileHierarchyController outlineViewItemDidExpand:] Low memory");
						}
					}
					
					if (tDisclosedDictionary!=nil)
					{
						[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tFilePath];
					}
				}
			}
		}
	}
}

- (void) outlineViewItemWillCollapse:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			ICFileTree * tTreeNode;
			
			tTreeNode=(ICFileTree *) [tUserInfo objectForKey:@"NSObject"];
			
			if (tTreeNode!=nil)
			{
				NSString * tFilePath;
				
				tFilePath=[tTreeNode filePath];
				
				if (tFilePath!=nil)
				{
					NSMutableDictionary * tDisclosedDictionary;
					
					tDisclosedDictionary=[document_ documentDefaultForKey:[self disclosedStateKey]];
					
					if (tDisclosedDictionary!=nil)
					{
						// Check if the option key is down or not
						
						NSEvent * tCurrentEvent;
						
						tCurrentEvent=[NSApp currentEvent];
						
						if (tCurrentEvent==nil || (([tCurrentEvent modifierFlags] & NSAlternateKeyMask)==0))
						{
							if ([tFilePath isEqualToString:@"/"]==NO)
							{
								// Check the parents state
							
								NSString * tParentPath;
								NSNumber * tNumber;
								
								tParentPath=[NSString stringWithString:tFilePath];
							
								do
								{
									tParentPath=[tParentPath stringByDeletingLastPathComponent];
									
									tNumber=[tDisclosedDictionary objectForKey:tParentPath];
									
									if (tNumber==nil || [tNumber boolValue]==NO)
									{
										// Parent is hidden
									
										return;
									}
									
									if ([tParentPath isEqualToString:@"/"]==YES)
									{
										break;
									}
								}
								while (1);
							}
						}
						
						[tDisclosedDictionary removeObjectForKey:tFilePath];
					}
				}
			}
		}
	}
}

- (void) folderNameShouldChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
    {
    	NSDictionary * tUserInfo;
        
        tUserInfo=[inNotification userInfo];
        
        if (tUserInfo!=nil)
        {
			ICFileTree * tFileTree;
			NSString * tNewName;
			
			tFileTree=[tUserInfo objectForKey:ICFILEHIERARCHYINSPECTOR_FILETREE];
			
			tNewName=[tUserInfo objectForKey:ICFILEHIERARCHYINSPECTOR_NEWNAME];
			
			if (tFileTree!=nil && tNewName!=nil)
			{
				ICFileTree * tParentTree;
				NSMutableArray * tChildFoldersArray;
				NSMutableDictionary * tDisclosedDictionary;
				ICFileNode * tFileNode;
				NSNumber * tNumber;
				
				tNumber=[tUserInfo objectForKey:ICFILEHIERARCHYINSPECTOR_INSPECTOR];
				
				if (tNumber!=nil && [tNumber boolValue]==YES)
				{
					if ([IBoutlineView_ editedRow]!=-1)
					{
						NSText * tFieldEditor;
						
						// Get the field editor
						
						tFieldEditor=[[IBoutlineView_ window] fieldEditor:NO forObject:IBoutlineView_];
						
						if (tFieldEditor!=nil)
						{
							[tFieldEditor setString:[FILENODE_DATA(tFileTree) fileName]];
						}
					}
				}
				
				tDisclosedDictionary=[document_ documentDefaultForKey:[self disclosedStateKey]];
		
				// Removed item and descendants from disclosed state dictionary (because the key will change)
				
				tChildFoldersArray=[tFileTree descendantFoldersIncludingSelf:YES];
				
				if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
				{
					NSUInteger i,tCount;
					
					tCount=[tChildFoldersArray count];
					
					for(i=tCount;i>0;i--)
					{
						ICFileTree * tGroupFileTree;
						NSString * tPath;
						
						tGroupFileTree=[tChildFoldersArray objectAtIndex:(i-1)];
						
						tPath=[tGroupFileTree filePath];
						
						if (tPath!=nil)
						{
							BOOL tRemoved=NO;
							
							tNumber=[tDisclosedDictionary objectForKey:tPath];
							
							if (tNumber==nil)
							{
								tRemoved=YES;
							}
							else
							{
								if ([tNumber boolValue]==NO)
								{
									tRemoved=YES;
								}
								
								[tDisclosedDictionary removeObjectForKey:tPath];
							}
							
							if (tRemoved==YES)
							{
								[tChildFoldersArray removeObject:tGroupFileTree];
							}
						}
					}
				}
				
				tFileNode=FILENODE_DATA(tFileTree);
				
				if (tFileNode!=nil)
				{
					[tFileNode setPath:tNewName];
				}
				
				
				tParentTree=(ICFileTree *) [tFileTree nodeParent];
		
				[tFileTree retain];
				
				[tFileTree removeFromParent];
				
				[tParentTree insertSortedChild:tFileTree];
				
				[tFileTree release];
				
				// Put back the item and descendant folders disclosed state
				
				if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[tChildFoldersArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						ICFileTree * tGroupFileTree;
						
						while (tGroupFileTree=[tEnumerator nextObject])
						{
							NSString * tPath;
							
							tPath=[tGroupFileTree filePath];
						
							if (tPath!=nil)
							{
								[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tPath];
							}
						}
					}
				}
				
				// Refresh list
				
				[IBoutlineView_ deselectAll:nil];
				
				if ([FILENODE_DATA(tParentTree) type]==ICDOCUMENT_FILE_ROOT_NODE)
				{
					[IBoutlineView_ reloadData];
				}
				else
				{
					[IBoutlineView_ reloadItem:tParentTree reloadChildren:YES];
				}
				
				[IBoutlineView_ selectRow:[IBoutlineView_ rowForItem:tFileTree] byExtendingSelection:NO];
				
				// Update hierarchy dictionary
				
				hierarchyChanged_=YES;
				
				[self updateFiles:IBoutlineView_];
			}
		}
	}
}

- (void) fileAttributesDidChange:(NSNotification *) inNotification
{
    if (inNotification!=nil)
    {
    	NSDictionary * tUserInfo;
        
        tUserInfo=[inNotification userInfo];
        
        if (tUserInfo!=nil)
        {
            NSArray * tSelectedFiles;
			
			tSelectedFiles=[tUserInfo objectForKey:ICFILEHIERARCHYINSPECTOR_SELECTED_FILES];
			
			if (tSelectedFiles!=nil)
			{
				NSEnumerator * tEnumerator;
				
				hierarchyChanged_=YES;
				
				tEnumerator=[tSelectedFiles objectEnumerator];
                
				if (tEnumerator!=nil)
				{
					ICFileTree * tFileTree;
					
					// Refresh the appropriate rows
				
					while (tFileTree=[tEnumerator nextObject])
					{
						[IBoutlineView_ reloadItem:tFileTree reloadChildren:[IBoutlineView_ isItemExpanded:tFileTree]];
					}
				}
					
				[self updateFiles:IBoutlineView_];
            }
        }
    }
}

- (void) highlightExcludedFilesDidChange:(NSNotification *) inNotification
{
	cachedHighlightExcludedFiles_=[defaults_ boolForKey:ICPREFERENCES_FILES_HIGHLIGHT_EXCLUDED_FILES];
	
	[IBoutlineView_ reloadData];
}

@end
