/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICFileHierarchyInspectorSelectionController.h"

#import "ICIconsUtilities.h"

#import "ICDocument.h"

#define ICFILEHIERARCHYINSPECTOR_MAX_ICON_CACHE_SIZE		100

static NSMutableDictionary * sRealFileIconsCache=nil;
static NSMutableArray * sRealFileLastAccessedIcons=nil;

@implementation ICFileHierarchyInspectorSelectionController

- (void) dealloc
{
	[dateFormatter_ release];
	
	[array_ release];
	
	[super dealloc];
}

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn;
	id tPrototypeCell;
	
	sharedWorkspace_=[NSWorkspace sharedWorkspace];
	
	dateFormatter_=[NSDateFormatter new];
	
	if (dateFormatter_!=nil)
	{
		[dateFormatter_ setFormatterBehavior:NSDateFormatterBehavior10_4];
	
		[dateFormatter_ setDateStyle:NSDateFormatterMediumStyle];
		
		[dateFormatter_ setTimeStyle:NSDateFormatterShortStyle];
	}
	
	// Permission Array
    
    [IBfilePermissionsArray_ setIntercellSpacing:NSMakeSize(3.0f,1.0f)];
    
    tPrototypeCell = [[NSButtonCell alloc] initTextCell: @""];
    
	if (tPrototypeCell!=nil)
	{
		[tPrototypeCell setControlSize:NSMiniControlSize];
		[tPrototypeCell setEditable:YES];
		[tPrototypeCell setButtonType: NSSwitchButton];
		[tPrototypeCell setAllowsMixedState:YES];
		[tPrototypeCell setImagePosition: NSImageOnly];
		
		
		tTableColumn = [IBfilePermissionsArray_ tableColumnWithIdentifier: @"Read"];
		[tTableColumn setDataCell:tPrototypeCell];
		
		tTableColumn = [IBfilePermissionsArray_ tableColumnWithIdentifier: @"Write"];
		[tTableColumn setDataCell:tPrototypeCell];
		
		tTableColumn = [IBfilePermissionsArray_ tableColumnWithIdentifier: @"Search"];
		[tTableColumn setDataCell:tPrototypeCell];
		
		[tPrototypeCell release];
    }
	
    tTableColumn = [IBfilePermissionsArray_ tableColumnWithIdentifier: @"Owner"];
	
    tPrototypeCell = [tTableColumn dataCell];
	
    [tPrototypeCell setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
    
    
    
    // Special Bits Array
    
    [IBfileSpecialBitsArray_ setIntercellSpacing:NSMakeSize(3.0f,1.0f)];
    
    tPrototypeCell = [[NSButtonCell alloc] initTextCell: @""];
    
	if (tPrototypeCell!=nil)
	{
		[tPrototypeCell setControlSize:NSMiniControlSize];
		[tPrototypeCell setEditable:YES];
		[tPrototypeCell setButtonType: NSSwitchButton];
		[tPrototypeCell setAllowsMixedState:YES];
		[tPrototypeCell setImagePosition: NSImageOnly];
    
    
		tTableColumn = [IBfileSpecialBitsArray_ tableColumnWithIdentifier: @"Bit"];
		[tTableColumn setDataCell:tPrototypeCell];
		
		[tPrototypeCell release];
	}
		
	tTableColumn = [IBfileSpecialBitsArray_ tableColumnWithIdentifier: @"Name"];
		
	tPrototypeCell = [tTableColumn dataCell];
		
	[tPrototypeCell setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
}

#pragma mark -

- (void) initializeWithArray:(NSMutableArray *) inArray forDocument:(id) inDocument
{
	[array_ release];
	
	array_=nil;
	
	[super initializeWithDictionary:nil forDocument:inDocument];
}

- (void) setFileNameEditable:(BOOL) inEditable
{
	if (inEditable==NO)
	{
		[IBfileEditableName_ setHidden:YES];
		
		[IBfileSelectableName_ setHidden:NO];
	}
	else
	{
		[IBfileEditableName_ setHidden:NO];
		
		[IBfileSelectableName_ setHidden:YES];
	}
}

+ (void) runRealFileIconsCacheMaintenance
{
	if ([sRealFileLastAccessedIcons count]>ICFILEHIERARCHYINSPECTOR_MAX_ICON_CACHE_SIZE)
	{
		NSInteger i,tCount;
		
		tCount=[sRealFileLastAccessedIcons count]-ICFILEHIERARCHYINSPECTOR_MAX_ICON_CACHE_SIZE;
		
		for(i=tCount;i>0;i--)
		{
			NSString * tPath;
			
			tPath=[sRealFileLastAccessedIcons objectAtIndex:i-1];
			
			[sRealFileIconsCache removeObjectForKey:tPath];
		}
		
		[sRealFileLastAccessedIcons removeObjectsInRange:NSMakeRange(0,tCount)];
	}
}

- (NSImage *) iconForTreeNode:(ICFileTree *) inFileTree
{
	NSImage * tImage=nil;
	
	if (inFileTree!=nil)
	{
		int tType;
		ICFileNode * tFileNode;
		
		tFileNode=FILENODE_DATA(inFileTree);
		
		tType=[tFileNode type];
		
		if (tType==ICDOCUMENT_FILE_NEWFOLDER_NODE || tType==ICDOCUMENT_FILE_BASE_NODE_INVISIBLE)
		{
			static NSImage * sIconFolder32=nil;
			
			if (sIconFolder32==nil)
			{
				sIconFolder32=[[ICIconsUtilities classicIconForType:NSFileTypeForHFSTypeCode(kGenericFolderIcon)] copy];
			}
				
			tImage=sIconFolder32;
		}
		else
		{
			NSString * tFilePath;
			
			if (tType==ICDOCUMENT_FILE_BASE_NODE)
			{
				tFilePath=[inFileTree filePath];
			
				if (tFilePath!=nil)
				{
					static NSMutableDictionary * sBaseNodeIconsCache=nil;
				
					if (sBaseNodeIconsCache==nil)
					{
						sBaseNodeIconsCache=[[NSMutableDictionary alloc] initWithCapacity:50];
					}
					else
					{
						tImage=[sBaseNodeIconsCache objectForKey:tFilePath];
					}
					
					if (tImage==nil)
					{
						tImage=[sharedWorkspace_ iconForFile:tFilePath];
					
						if (tImage!=nil)
						{
							[sBaseNodeIconsCache setObject:tImage forKey:tFilePath];
						}
					}
					
				}
			}
			else if (tType==ICDOCUMENT_FILE_REALFILE_NODE)
			{
				tFilePath=[tFileNode path];
				
				if (tFilePath!=nil)
				{
					if (sRealFileIconsCache==nil)
					{
						sRealFileIconsCache=[[NSMutableDictionary alloc] initWithCapacity:ICFILEHIERARCHYINSPECTOR_MAX_ICON_CACHE_SIZE];
		
						sRealFileLastAccessedIcons=[[NSMutableArray alloc] initWithCapacity:ICFILEHIERARCHYINSPECTOR_MAX_ICON_CACHE_SIZE];
					}
					else
					{
						tImage=[sRealFileIconsCache objectForKey:tFilePath];
						
						if (tImage!=nil)
						{
							NSUInteger tIndex;
							
							tIndex=[sRealFileLastAccessedIcons indexOfObject:tFilePath];
							
							if (tIndex!=NSNotFound)
							{
								NSUInteger tCount;
								
								tCount=[sRealFileLastAccessedIcons count];
								
								if (tCount>ICFILEHIERARCHYINSPECTOR_MAX_ICON_CACHE_SIZE)
								{
									if (tIndex<(tCount-ICFILEHIERARCHYINSPECTOR_MAX_ICON_CACHE_SIZE))
									{
										[sRealFileLastAccessedIcons removeObjectAtIndex:tIndex];
										
										[sRealFileLastAccessedIcons addObject:tFilePath];
									}
								}
							}
						}
					}
					
					if (tImage==nil)
					{
						tImage=[sharedWorkspace_ iconForFile:tFilePath];
					
						if (tImage!=nil)
						{
							[sRealFileIconsCache setObject:tImage forKey:tFilePath];
							
							[sRealFileLastAccessedIcons addObject:tFilePath];
							
							if ([sRealFileLastAccessedIcons count]>ICFILEHIERARCHYINSPECTOR_MAX_ICON_CACHE_SIZE)
							{
								[NSObject cancelPreviousPerformRequestsWithTarget:[self class] selector:@selector(runRealFileIconsCacheMaintenance) object:nil];
							
								[[self class] performSelector:@selector(runRealFileIconsCacheMaintenance) withObject:nil afterDelay:1.0f];
							}
						}
					}
				}
			}
		}
	}

	return tImage;
}

- (void) updateSingleSelection
{
	ICFileTree * tFileTree;
	ICFileNode * tFileNode;
	ICFileNode * tPermissionNode;
	NSString * tString=nil;
	NSTextField * tTextField=nil;
	BOOL isSymbolicLink=NO;
	int tType;
	NSString * tFilePath;
	NSImage * tImage=nil;
	BOOL tTypeAlreadySet=NO;
	
	tFileTree=[array_ objectAtIndex:0];
	
	tFileNode=FILENODE_DATA(tFileTree);
	
	// *** General Section

	tType=[tFileNode type];
	
	tFilePath=[tFileNode path];
	
	// File Icon
	
	tImage=[self iconForTreeNode:tFileTree];
	
	[IBfileIcon_ setImage:tImage];
	
	// File Name
	
	switch(tType)
	{
		case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
		case ICDOCUMENT_FILE_BASE_NODE:
			
			[self setFileNameEditable:NO];
			
			tTextField=IBfileSelectableName_;
			
			break;
			
		case ICDOCUMENT_FILE_NEWFOLDER_NODE:
			
			[self setFileNameEditable:YES];
			
			tTextField=IBfileEditableName_;
			
			break;
			
		case ICDOCUMENT_FILE_REALFILE_NODE:
			
			[self setFileNameEditable:NO];
			
			tTextField=IBfileSelectableName_;
			
			break;
	}
	
	[tTextField setTextColor:[NSColor blackColor]];
	
	[tTextField setStringValue:[tFileNode fileName]];
	
	[IBfileBigName_ setStringValue:[tFileNode fileName]];
	
	// Last Modification Date
	
	[IBfileLastModifiedDate_ setStringValue:@"-"];
	
	if (tType==ICDOCUMENT_FILE_REALFILE_NODE)
	{
		NSDictionary * tAttributesDictionary;
				
		tAttributesDictionary=[[NSFileManager defaultManager] fileAttributesAtPath:tFilePath traverseLink:NO];
				
		if (tAttributesDictionary!=nil)
		{
			NSDate * tModificationDate;
			
			tModificationDate=[tAttributesDictionary objectForKey:NSFileModificationDate];
			
			if (tModificationDate!=nil)
			{
				if (dateFormatter_!=nil)
				{
					[IBfileLastModifiedDate_ setStringValue:[dateFormatter_ stringForObjectValue:tModificationDate]];
				}
				else
				{
					[IBfileLastModifiedDate_ setObjectValue:tModificationDate];
				}
			}
		}
	}
	
	// Architectures
	
	[IBfileArchitecturesLabel_ setStringValue:NSLocalizedString(@"Architectures:",@"")];
	
	[IBfileArchitectures_ setStringValue:@"-"];
	
	if (tType==ICDOCUMENT_FILE_REALFILE_NODE)
	{
		struct stat tStat;
		
		if (lstat([tFilePath fileSystemRepresentation], &tStat)==0)
		{
			mode_t tMode=(tStat.st_mode & S_IFMT);
			
			if (tMode!=S_IFLNK && tMode!=S_IFSOCK)
			{
				NSBundle * tBundle;
				NSArray * tArchitecturesArray;
				NSString * tExecutablePath;
				
				tBundle=[NSBundle bundleWithPath:tFilePath];
			
				if (tBundle!=nil)
				{
					tExecutablePath=[tBundle executablePath];
				}
				else
				{
					tExecutablePath=tFilePath;
				}
		
				tArchitecturesArray=[ICArchitectureUtilities architecturesOfFile:tExecutablePath];
				
				if (tArchitecturesArray!=nil && [tArchitecturesArray count]>0)
				{
					NSString * tExtension;
					
					// Label
						
					if ([tArchitecturesArray count]==1)
					{
						[IBfileArchitecturesLabel_ setStringValue:NSLocalizedString(@"Architecture:",@"")];
					}
					
					[IBfileArchitectures_ setStringValue:[tArchitecturesArray componentsJoinedByString:@" | "]];
					
					// File Type
			
					tString=nil;
				
					if (tMode==S_IFDIR)
					{
						tExtension=[tFilePath pathExtension];
							
						if ([tExtension caseInsensitiveCompare:@"app"]==NSOrderedSame)
						{
							tString=NSLocalizedString(@"Application",@"No comment");
						}
						else if ([tExtension caseInsensitiveCompare:@"colorPicker"]==NSOrderedSame)
						{
							tString=NSLocalizedString(@"Color Picker Plug-in",@"No comment");
						}
						else if ([tExtension caseInsensitiveCompare:@"component"]==NSOrderedSame)
						{
							tString=NSLocalizedString(@"Component",@"No comment");
						}
						else if ([tExtension caseInsensitiveCompare:@"kext"]==NSOrderedSame)
						{
							tString=NSLocalizedString(@"Kernel Extension",@"No comment");
						}
						else if ([tExtension caseInsensitiveCompare:@"prefPane"]==NSOrderedSame)
						{
							tString=NSLocalizedString(@"Preference Pane",@"No comment");
						}
						else if ([tExtension caseInsensitiveCompare:@"wdgt"]==NSOrderedSame)
						{
							tString=NSLocalizedString(@"Widget",@"No comment");
						}
						else if ([tExtension caseInsensitiveCompare:@"framework"]==NSOrderedSame)
						{
							tString=NSLocalizedString(@"Framework",@"No comment");
						}
						else if ([tExtension caseInsensitiveCompare:@"saver"]==NSOrderedSame)
						{
							tString=NSLocalizedString(@"Screen Saver",@"No comment");
						}
						else if ([tExtension caseInsensitiveCompare:@"plugin"]==NSOrderedSame)
						{
							tString=NSLocalizedString(@"Plug-in",@"No comment");
						}
						else
						{
							tString=NSLocalizedString(@"Bundle",@"No comment");
						}
					}
					else
					{
						tString=NSLocalizedString(@"Executable",@"No comment");
					}
				
					if (tString!=nil)
					{
						[IBfileType_ setStringValue:tString];
					
						tTypeAlreadySet=YES;
					}
				}
			}
		}
	}
	
	// File Type
	
	[IBfileType_ setTextColor:[NSColor blackColor]];
	
	if (tTypeAlreadySet==NO)
	{
		switch(tType)
		{
			case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
			case ICDOCUMENT_FILE_BASE_NODE:
			
				tString=NSLocalizedString(@"Standard folder",@"No comment");
				
				break;
				
			case ICDOCUMENT_FILE_NEWFOLDER_NODE:
			
				tString=NSLocalizedString(@"Custom folder",@"No comment");
				
				break;
				
			case ICDOCUMENT_FILE_REALFILE_NODE:
				{
					struct stat tStat;
					
					if (lstat([[tFileNode path] fileSystemRepresentation], &tStat)==0)
					{
						switch((tStat.st_mode & S_IFMT))
						{
							case S_IFDIR:
								tString=NSLocalizedString(@"Folder",@"No comment");
								break;
							case S_IFLNK:
								tString=NSLocalizedString(@"Symbolic link",@"No comment");
								break;
							default:
								tString=NSLocalizedString(@"File",@"No comment");
								break;
						}
					}
					else
					{
						tString=NSLocalizedString(@"N/A",@"No comment");
					}
				}
				break;
		}
		
		[IBfileType_ setStringValue:tString];
	}
	
	// File Reference
	
	cachedPathTypeIndex_=0;
	
	switch(tType)
    {
        case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
		case ICDOCUMENT_FILE_BASE_NODE:
		case ICDOCUMENT_FILE_NEWFOLDER_NODE:
		
			[IBnonEditablePathType_ setHidden:NO];
		
			[IBpathType_ setHidden:YES];
			
            break;
			
        case ICDOCUMENT_FILE_REALFILE_NODE:
            
			cachedPathTypeIndex_=[IBpathType_ indexOfItemWithTag:[tFileNode pathType]];
			
			[IBnonEditablePathType_ setHidden:YES];
		
			[IBpathType_ setHidden:NO];
			
			[IBpathType_ selectItemAtIndex:cachedPathTypeIndex_];
			
            break;
    }

	
	// File Source
	
	[IBfileSource_ setTextColor:[NSColor blackColor]];
	
	switch(tType)
    {
        case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
		case ICDOCUMENT_FILE_BASE_NODE:
        case ICDOCUMENT_FILE_NEWFOLDER_NODE:
		
            tString=@"-";
			
            break;
        case ICDOCUMENT_FILE_REALFILE_NODE:
            
			tString=[tFileNode path];
                            
			if ([tFileNode pathType]==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tString=[tString stringByRelativizingToPath:[document_ folder]];
			}
			else if ([tFileNode pathType]==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tString=[tString stringByRelativizingToPath:[document_ referenceFolderPath]];
			}
							
            break;
    }
	
	[IBfileSource_ setStringValue:tString];
	
	// File Destination
	
	[IBfileDestination_ setTextColor:[NSColor blackColor]];
	
	if (IBfileDestination_!=nil)
	{
		[IBfileDestination_ setStringValue:[tFileTree filePath]];
	}
	
	// OwnerShip and Permissions
	
	tPermissionNode=tFileNode;
	
	switch(tType)
    {
        case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
		case ICDOCUMENT_FILE_BASE_NODE:
		
			canEditFilePermissions_=NO;
			
			break;
		
		case ICDOCUMENT_FILE_NEWFOLDER_NODE:
		
			canEditFilePermissions_=YES;
			
			break;
		
		case ICDOCUMENT_FILE_REALFILE_NODE:
            
			isSymbolicLink=[tFileNode link];
			
			if (isSymbolicLink==YES)
			{
				ICFileTree * tParentTree;
			
				tParentTree=(ICFileTree *) [tFileTree nodeParent];
				
				if (tParentTree!=nil)
				{
					tPermissionNode=FILENODE_DATA(tParentTree);
				}
				
				canEditFilePermissions_=NO;
			}
			else
			{
				canEditFilePermissions_=YES;
			}
			
            break;
    }
                    
	if (isSymbolicLink==NO)
	{
		[IBfilePermissions_ setStringValue:[tFileNode privilegesStringRepresentation]];
	}
	else
	{
		[IBfilePermissions_ setStringValue:[tPermissionNode privilegesStringRepresentationForLink]];
	}
}

- (void) updateMultipleSelection
{
	NSEnumerator * tEnumerator;
	BOOL tCanEditProperty;
	NSInteger tCommonPathType=-1;
	BOOL tFirstSeen;
	static NSImage * sIconMultipleSelection=nil;
	
	// *** General Section

	// File Icon
	
	if (sIconMultipleSelection==nil)
	{
		sIconMultipleSelection=[[ICIconsUtilities classicIconForType:NSFileTypeForHFSTypeCode(kUnknownFSObjectIcon)] copy];
	}
	
	[IBfileIcon_ setImage:sIconMultipleSelection];
	
	// Big Name
	
	[IBfileBigName_ setStringValue:NSLocalizedString(@"Multiple Selection",@"No comment")];
	
	// Last Modification Date
	
	[IBfileLastModifiedDate_ setStringValue:@"-"];
	
	// File Name
	
	[self setFileNameEditable:NO];
	
	[IBfileSelectableName_ setStringValue:NSLocalizedString(@"Multiple Selection",@"No comment")];
	
	[IBfileSelectableName_ setTextColor:[NSColor grayColor]];
	
	// Architectures
	
	[IBfileArchitecturesLabel_ setStringValue:NSLocalizedString(@"Architectures:",@"No comment")];
	
	[IBfileArchitectures_ setStringValue:@"-"];
	
	// File Type
	
	[IBfileType_ setStringValue:NSLocalizedString(@"Multiple Selection",@"No comment")];
	
	[IBfileType_ setTextColor:[NSColor grayColor]];
	
	// File Reference
	
	tCommonPathType=-1;
	
	tCanEditProperty=YES;
	
	tEnumerator=[array_ objectEnumerator];
	
	if (tEnumerator!=nil)
	{
		ICFileTree * tFileTree;
		
		while ((tFileTree=[tEnumerator nextObject]) && tCanEditProperty==YES && tCommonPathType!=-2)
		{
			ICFileNode * tFileNode;
			
			tFileNode=FILENODE_DATA(tFileTree);
			
			switch([tFileNode type])
			{
				case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
				case ICDOCUMENT_FILE_BASE_NODE:
				case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				
					tCanEditProperty=NO;
					
					break;
				
				case ICDOCUMENT_FILE_REALFILE_NODE:
					
					if (tCommonPathType==-1)
					{
						tCommonPathType=[tFileNode pathType];
					}
					else if ([tFileNode pathType]!=tCommonPathType)
					{
						tCommonPathType=-2;
					}
					
					break;
			}

		}
	}
	
	if (tCanEditProperty==NO)
	{
		[IBnonEditablePathType_ setHidden:NO];
		
		[IBpathType_ setHidden:YES];
	}
	else
	{
		[IBnonEditablePathType_ setHidden:YES];
		
		[IBpathType_ setHidden:NO];
		
		if (tCommonPathType==-2)
		{
			id tMenuItem;
			NSImage * tImage;
			
			cachedPathTypeIndex_=-1;
			
			[IBpathType_ setEnabled:YES];
			
			[IBpathType_ insertItemWithTitle:NSLocalizedString(@"Mixed",@"No comment")
									 atIndex:0];
                                    
                                    
			tMenuItem=[IBpathType_ itemAtIndex:0];
			
			tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Mixed" ofType:@"tif"]];
        
			if (tImage!=nil)
			{
				[tMenuItem setImage:tImage];
			
				[tImage release];
			}

			
			[tMenuItem setTarget:nil];
			
			[tMenuItem setEnabled:NO];
			
			[tMenuItem setTag:ICDOCUMENT_PATH_TYPE_MIXED];
			
			[IBpathType_ selectItemAtIndex:0];
			
			[IBpathType_ setNeedsDisplay:YES];
		}
		else
		{
			cachedPathTypeIndex_=[IBpathType_ indexOfItemWithTag:tCommonPathType];
			
			[IBpathType_ selectItemAtIndex:cachedPathTypeIndex_];
		}
	}
	
	// File Source
	
	[IBfileSource_ setStringValue:NSLocalizedString(@"Multiple Selection",@"No comment")];
	
	[IBfileSource_ setTextColor:[NSColor grayColor]];
	
	// File Destination
	
	[IBfileDestination_ setStringValue:NSLocalizedString(@"Multiple Selection",@"No comment")];
	
	[IBfileDestination_ setTextColor:[NSColor grayColor]];
	
	// OwnerShip and Permissions
	
	// File Permissions
	
	cachedStatType_=0;
	
	cachedFilePermissions_=0;
	
	mixedFilePermission_=0;
	
	canEditFilePermissions_=YES;
	
	tFirstSeen=YES;
	
	tEnumerator=[array_ objectEnumerator];
	
	if (tEnumerator!=nil)
	{
		ICFileTree * tFileTree;
		
		while ((tFileTree=[tEnumerator nextObject]))
		{
			ICFileNode * tFileNode;
			
			tFileNode=FILENODE_DATA(tFileTree);
			
			switch([tFileNode type])
			{
				case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
				case ICDOCUMENT_FILE_BASE_NODE:
				
					canEditFilePermissions_=NO;
				
				case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				case ICDOCUMENT_FILE_REALFILE_NODE:
					
					if ([tFileNode link]==YES)
					{
						ICFileTree * tParentTree;
						
						canEditFilePermissions_=NO;
						
						
						tParentTree=(ICFileTree *) [tFileTree nodeParent];
						
						if (tParentTree!=nil)
						{
							tFileNode=FILENODE_DATA(tParentTree);
						}
						
						cachedStatType_='l';
					}
					
					if (tFirstSeen==YES)
					{
						cachedFilePermissions_=[tFileNode privileges];
					}
					else
					{
						int tXoredPermissions;
										
						tXoredPermissions=cachedFilePermissions_^[tFileNode privileges];
						
						mixedFilePermission_=mixedFilePermission_|tXoredPermissions;
					}
					
					break;
			}
			
			if (tFirstSeen==YES)
			{
				if (cachedStatType_!='l')
				{
					cachedStatType_=[tFileNode statType];
				}
				
				tFirstSeen=NO;
			}
			else
			{
				if (cachedStatType_!=0)
				{
					if (cachedStatType_!=[tFileNode statType])
					{
						cachedStatType_=0;
					}
				}
			}
		}
		
		[IBfilePermissions_ setStringValue:[ICFileNode privilegesStringRepresentationWithPermission:cachedFilePermissions_ mixedPermission:mixedFilePermission_ statType:cachedStatType_]];
	}
}

#pragma mark -

- (NSInteger)numberOfRowsInTableView:(NSTableView *) inTableView
{
    return 3;
}

- (id)tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{    
    if (IBfileSpecialBitsArray_==inTableView)
    {
        if ([[inTableColumn identifier] isEqualToString: @"Name"])
        {
            switch(inRowIndex)
            {
                case 0:
                    return @"SetUID";
                case 1:
                    return @"SetGID";
                case 2:
                    return @"Sticky";
            }
        }
        else
        {
            if ([[inTableColumn identifier] isEqualToString: @"Bit"])
            {
                int tValue=NSOffState;
            
                if ([array_ count]==1)
				{
                    int tPermissions;
                
                    tPermissions=[FILENODE_DATA([array_ objectAtIndex:0]) privileges];
                
                    switch(inRowIndex)
                    {
                        case 0:
                            tValue=((tPermissions & S_ISUID)==S_ISUID);
                            break;
                        case 1:
                            tValue=((tPermissions & S_ISGID)==S_ISGID);
                            break;
                        case 2:
                            tValue=((tPermissions & S_ISTXT)==S_ISTXT);
                            break;
                    }
                }
                else
				{
					int tFlag=-1;
				
					switch(inRowIndex)
					{
						case 0:
							tFlag=S_ISUID;
							break;
						case 1:
							tFlag=S_ISGID;
							break;
						case 2:
							tFlag=S_ISTXT;
							break;
					}
					
					if ((mixedFilePermission_ & tFlag)==tFlag)
					{
						tValue=NSMixedState;
					}
					else
					{
						tValue=((cachedFilePermissions_ & tFlag)==tFlag);
					}
				}
            
                return [NSNumber numberWithInt:tValue];
            }
        }
    }
    else if (IBfilePermissionsArray_==inTableView)
    {
        if ([[inTableColumn identifier] isEqualToString: @"Owner"])
        {
            switch(inRowIndex)
            {
                case 0:
                    return @"Owner";
                case 1:
                    return @"Group";
                case 2:
                    return @"Others";
            }
        }
        else
        {
			int tValue=NSOffState;
            
            if ([array_ count]==1)
            {
                int tPermissions;
                
                tPermissions=[FILENODE_DATA([array_ objectAtIndex:0]) privileges];
                
                if ([[inTableColumn identifier] isEqualToString: @"Read"])
                {
                    switch(inRowIndex)
                    {
                        case 0:
                            tValue=((tPermissions & S_IRUSR)==S_IRUSR);
                            break;
                        case 1:
                            tValue=((tPermissions & S_IRGRP)==S_IRGRP);
                            break;
                        case 2:
                            tValue=((tPermissions & S_IROTH)==S_IROTH);
                            break;
                    }
                }
                else
                if ([[inTableColumn identifier] isEqualToString: @"Write"])
                {
                    switch(inRowIndex)
                    {
                        case 0:
                            tValue=((tPermissions & S_IWUSR)==S_IWUSR);
                            break;
                        case 1:
                            tValue=((tPermissions & S_IWGRP)==S_IWGRP);
                            break;
                        case 2:
                            tValue=((tPermissions & S_IWOTH)==S_IWOTH);
                            break;
                    }
                }
                else
                if ([[inTableColumn identifier] isEqualToString: @"Search"])
                {
                    switch(inRowIndex)
                    {
                        case 0:
                            tValue=((tPermissions & S_IXUSR)==S_IXUSR);
                            break;
                        case 1:
                            tValue=((tPermissions & S_IXGRP)==S_IXGRP);
                            break;
                        case 2:
                            tValue=((tPermissions & S_IXOTH)==S_IXOTH);
                            break;
                    }
                }
                
                return [NSNumber numberWithBool:tValue];
            }
            else
			{
				int tFlag=-1;
					
				if ([[inTableColumn identifier] isEqualToString: @"Read"])
				{
					switch(inRowIndex)
					{
						case 0:
							tFlag=S_IRUSR;
							break;
						case 1:
							tFlag=S_IRGRP;
							break;
						case 2:
							tFlag=S_IROTH;
							break;
					}
				}
				else
				if ([[inTableColumn identifier] isEqualToString: @"Write"])
				{
					switch(inRowIndex)
					{
						case 0:
							tFlag=S_IWUSR;
							break;
						case 1:
							tFlag=S_IWGRP;
							break;
						case 2:
							tFlag=S_IWOTH;
							break;
					}
				}
				else
				if ([[inTableColumn identifier] isEqualToString: @"Search"])
				{
					switch(inRowIndex)
					{
						case 0:
							tFlag=S_IXUSR;
							break;
						case 1:
							tFlag=S_IXGRP;
							break;
						case 2:
							tFlag=S_IXOTH;
							break;
					}
				}
				
				if ((mixedFilePermission_ & tFlag)==tFlag)
				{
					tValue=NSMixedState;
				}
				else
				{
					tValue=((cachedFilePermissions_ & tFlag)==tFlag);
				}
            }
			
            return [NSNumber numberWithInt:tValue];
        }
    }
    
    return nil;
}

- (void)tableView:(NSTableView *) inTableView setObjectValue:(id)object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
    int tPermissions;
	ICFileNode * tFileNode=nil;
	
	if ([array_ count]==1)
	{
		tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
		
		tPermissions=[tFileNode privileges];
	}
	else
	{
		tPermissions=cachedFilePermissions_;
	}
		
	if (IBfilePermissionsArray_==inTableView)
	{
		int tMask=-1;
		
		if ([[inTableColumn identifier] isEqualToString: @"Read"])
		{
			switch(inRowIndex)
			{
				case 0:
					tMask=S_IRUSR;
					break;
				case 1:
					tMask=S_IRGRP;
					break;
				case 2:
					tMask=S_IROTH;
					break;
			}
		}
		else
		if ([[inTableColumn identifier] isEqualToString: @"Write"])
		{
			switch(inRowIndex)
			{
				case 0:
					tMask=S_IWUSR;
					break;
				case 1:
					tMask=S_IWGRP;
					break;
				case 2:
					tMask=S_IWOTH;
					break;
			}
		}
		else
		if ([[inTableColumn identifier] isEqualToString: @"Search"])
		{
			switch(inRowIndex)
			{
				case 0:
					tMask=S_IXUSR;
					break;
				case 1:
					tMask=S_IXGRP;
					break;
				case 2:
					tMask=S_IXOTH;
					break;
			}
		}
		
		if ([object intValue]!=NSOffState)
		{
			tPermissions|=tMask;
		}
		else
		{
			tPermissions&=~tMask;
		}
		
		mixedFilePermission_&=~tMask;
	}
	else if (IBfileSpecialBitsArray_==inTableView)
	{
		if ([[inTableColumn identifier] isEqualToString: @"Bit"])
		{
			int tMask=-1;
			
			switch(inRowIndex)
			{
				case 0:
					tMask=S_ISUID;
					break;
				case 1:
					tMask=S_ISGID;
					break;
				case 2:
					tMask=S_ISTXT;
					
					break;
			}
			
			if ([object intValue]!=NSOffState)
			{
				tPermissions|=tMask;
			}
			else
			{
				tPermissions&=~tMask;
			}
			
			mixedFilePermission_&=~tMask;
		}
	}
	
	if ([array_ count]==1)
	{
		[tFileNode setPrivileges:tPermissions];
		
		[IBfilePermissions_ setStringValue:[tFileNode privilegesStringRepresentation]];
	}
	else
	{
		NSEnumerator * tEnumerator;
		
		cachedFilePermissions_=tPermissions;
		
		tEnumerator=[array_ objectEnumerator];
		
		while (tFileNode=FILENODE_DATA([tEnumerator nextObject]))
		{
			[tFileNode setPrivileges:(cachedFilePermissions_ & ~mixedFilePermission_)|([tFileNode privileges]&mixedFilePermission_)];
		}
		
		[IBfilePermissions_ setStringValue:[ICFileNode privilegesStringRepresentationWithPermission:cachedFilePermissions_ mixedPermission:mixedFilePermission_ statType:cachedStatType_]];
	}
	
	// Post Notification
	
	[self postAttributesDidChangeNotification];
}

- (void)tableView:(NSTableView *) inTableView willDisplayCell:(id)aCell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
    if (IBfilePermissionsArray_==inTableView)
    {
        if ([[inTableColumn identifier] isEqualToString: @"Owner"])
		{
			[aCell setEnabled:YES];
        }
		else
		{
			[aCell setEnabled:canEditFilePermissions_];
		}
    }
    else if (IBfileSpecialBitsArray_==inTableView)
    {
        if ([[inTableColumn identifier] isEqualToString: @"Name"])
		{
			[aCell setEnabled:YES];
		}
		else
		{
			BOOL tEnabledCell=canEditFilePermissions_;
        
			if ([array_ count]!=0)
			{
				ICFileNode * tFileNode;
				
				tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
				
				if (([tFileNode type]==ICDOCUMENT_FILE_REALFILE_NODE && [tFileNode link]==NO))
				{
					struct stat tStat;
					
					if (lstat([[tFileNode path] fileSystemRepresentation], &tStat)==0)
					{
						if (inRowIndex==2 && (tStat.st_mode & S_IFMT)!=S_IFDIR)
						{
							tEnabledCell=NO;
						}
					}
				}
			}
			else
			{
				if (inRowIndex==2 && cachedStatType_!='d')
				{
					tEnabledCell=NO;
				}
			}
			
			[aCell setEnabled:tEnabledCell];
		}
	}
}

#pragma mark -

- (IBAction) setFolderName:(id) sender
{
    if ([array_ count]==1)
    {
        ICFileNode * tFileNode;
        ICFileTree * tFileTree;
		
		tFileTree=[array_ objectAtIndex:0];
		
        tFileNode=FILENODE_DATA(tFileTree);
        
        if ([tFileNode type]==ICDOCUMENT_FILE_NEWFOLDER_NODE)
        {
            NSString * tName;
            
            tName=[IBfileEditableName_ stringValue];
        
            if ([tName isEqualToString:[tFileNode path]]==NO)
            {
                NSDictionary * tDictionary;
				
				// Post Notification
				
				tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tFileTree,ICFILEHIERARCHYINSPECTOR_FILETREE,
																	   tName,ICFILEHIERARCHYINSPECTOR_NEWNAME,
																	   [NSNumber numberWithBool:YES],ICFILEHIERARCHYINSPECTOR_INSPECTOR,
																	   nil];
		
				if (tDictionary!=nil)
				{
					[[NSNotificationCenter defaultCenter] postNotificationName:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_FOLDERNAME_SHOULD_CHANGE
																		object:[self document]
																	  userInfo:tDictionary];
				}
				
				
				 // Update Destination
				
				[IBfileDestination_ setStringValue:[[array_ objectAtIndex:0] filePath]];
            }
        }
    }
}

- (IBAction) switchPathType:(id) sender
{
	if (cachedPathTypeIndex_!=[IBpathType_ indexOfSelectedItem])
    {
		NSString * tPath=nil;
        ICFileNode * tFileNode;
        NSString * tProjectReferencePath;
		NSString * tFolderReferencePath;
		NSEnumerator * tEnumerator;
		NSInteger tPathType;
		
		tPathType=[[IBpathType_ selectedItem] tag];
		
        cachedPathTypeIndex_=[IBpathType_ indexOfSelectedItem];
    
        tProjectReferencePath=[document_ folder];
		
		tFolderReferencePath=[document_ referenceFolderPath];
        
		tEnumerator=[array_ objectEnumerator];
        
		if (tEnumerator!=nil)
		{
			while (tFileNode=FILENODE_DATA([tEnumerator nextObject]))
			{
				[tFileNode setPathType:tPathType];
			
				// Update File Path drawing
				
				tPath=[tFileNode path];
										
				if ([tFileNode pathType]==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
				{
					tPath=[tPath stringByRelativizingToPath:tProjectReferencePath];
				}
				else if ([tFileNode pathType]==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
				{
					tPath=[tPath stringByRelativizingToPath:tFolderReferencePath];
				}
			}
		}
		
		if ([array_ count]==1)
		{
			[IBfileSource_ setStringValue:tPath];
		}
		
		// Post Notification
		
		[self postAttributesDidChangeNotification];
	}
}

- (IBAction) revealSourceInFinder:(id) sender
{
    NSEnumerator * tEnumerator;
	
	tEnumerator=[array_ objectEnumerator];
	
	if (tEnumerator!=nil)
	{
		ICFileNode * tFileNode;
		
		while (tFileNode=FILENODE_DATA([tEnumerator nextObject]))
		{
			[sharedWorkspace_ selectFile:[tFileNode path] inFileViewerRootedAtPath:@""];
		}
	}
}

- (IBAction) switchPermissions:(id) sender
{
    int tPermissions;
    ICFileNode * tFileNode;
	
    tPermissions=(int) [[sender selectedItem] tag];
    
    if ([array_ count]==1)
    {
		tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
		
		if (tPermissions!=[tFileNode privileges])
        {
            [tFileNode setPrivileges:tPermissions];
            
            [IBfilePermissions_ setStringValue:[tFileNode privilegesStringRepresentation]];
            
            [IBfilePermissionsArray_ reloadData];
            
            [IBfileSpecialBitsArray_ reloadData];
            
            // Post Notification
			
			[self postAttributesDidChangeNotification];
        }
    }
    else
    {
        if (tPermissions!=(cachedFilePermissions_ & ~mixedFilePermission_))
        {
            NSEnumerator * tEnumerator;
            
			cachedFilePermissions_=tPermissions;
            
            mixedFilePermission_=0;
            
            tEnumerator=[array_ objectEnumerator];
            
            while (tFileNode=FILENODE_DATA([tEnumerator nextObject]))
            {
                [tFileNode setPrivileges:cachedFilePermissions_];
            }
            
            [IBfilePermissions_ setStringValue:[ICFileNode privilegesStringRepresentationWithPermission:cachedFilePermissions_ mixedPermission:mixedFilePermission_ statType:cachedStatType_]];
            
            [IBfilePermissionsArray_ reloadData];
            
            [IBfileSpecialBitsArray_ reloadData];
            
            // Post Notification
			
			[self postAttributesDidChangeNotification];
        }
    }
}

#pragma mark -

- (void) postAttributesDidChangeNotification
{
    NSDictionary * tDictionary;
    
	tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:array_,ICFILEHIERARCHYINSPECTOR_SELECTED_FILES,
														  nil]; 
	
    // Post notification
    
    [[NSNotificationCenter defaultCenter] postNotificationName:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_ATTRIBUTES_DID_CHANGE
                                                        object:document_
                                                      userInfo:tDictionary];
}

@end
