/*
Copyright (c) 2008, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import <Cocoa/Cocoa.h>

#import "TreeNode.h"

#import "ICFileBouncer.h"

#define FILETREE(n)			((ICFileTree*)n)
#define FILENODE_DATA(n) 	((ICFileNode *)[FILETREE((n)) nodeData])

typedef struct __FTFlags
{
#ifdef __BIG_ENDIAN__
    unsigned int		link:1;
    unsigned int		folder:1;
    unsigned int		existsOnDisk:1;
#else
    unsigned int 		existsOnDisk:1;
    unsigned int		folder:1;
    unsigned int		link:1;
#endif
} _FTFlags; 

@interface ICFileNode: NSObject
{
    int type_;
    NSString * path_;
    NSInteger pathType_;
    NSImage * icon_;
    int uid_,gid_;
    int privileges_;
    
    _FTFlags flags_;
    
    NSMutableDictionary * extras_;
}

+ (id) rootFileNode;



+ (id) invisibleBaseFileNodeWithName:(NSString *) inName icon:(NSImage *) inIcon user:(int) inUID group:(int) inGID privileges:(int) inPrivileges;


+ (id) baseFileNodeWithName:(NSString *) inName icon:(NSImage *) inIcon user:(int) inUID group:(int) inGID privileges:(int) inPrivileges;

+ (id) folderFileNodeWithName:(NSString *) inName icon:(NSImage *) inIcon user:(int) inUID group:(int) inGID privileges:(int) inPrivileges;

+ (id) fileNodeWithType:(int) inType path:(NSString *) inPath;
+ (id) fileNodeWithType:(int) inType path:(NSString *) inPath pathType:(NSInteger) inPathType;

+ (NSImage *) setIconAsDefaultLocation:(NSImage *) inIcon;

+ (NSImage *) cachedIconForBaseNodeAtPath:(NSString *) tPath;

+ (NSImage *) cachedIconForPath:(NSString *) tPath;

+ (NSImage *) newSmallIconFromImage:(NSImage *) inImage;
+ (NSImage *) newIconFromPath:(NSString *) inPath;

- (id) initWithType:(int) inType path:(NSString *) inPath pathType:(NSInteger) inPathType;

- (BOOL) link;

- (char) statType;

- (int) type;

- (NSString *) path;

- (void) setPath:(NSString *) inPath;

- (NSInteger) pathType;

- (void) setPathType:(NSInteger) inPathType;

- (NSString *) fileName;

- (NSImage *) icon;

- (int) uid;

- (void) setUid:(int) inUID;

- (int) gid;

- (void) setGid:(int) inGID;

- (int) privileges;

- (void) setPrivileges:(int) inPrivileges;

- (NSNumber *) allowDowngradeNumber;

- (BOOL) allowDowngrade;

- (void) setAllowDowngrade:(BOOL) inAllowDowngrade;

- (NSMutableArray *) locators;

- (void) setLocators:(NSArray *) inLocators;

- (NSMutableDictionary *) preInstallPath;

- (void) setPreInstallPath:(NSDictionary *) inPostInstallPath;

- (NSMutableDictionary *) postInstallPath;

- (void) setPostInstallPath:(NSDictionary *) inPostInstallPath;

+ (NSString *) privilegesStringRepresentationWithPermission:(int) inPermission mixedPermission:(int) inMixedPermission statType:(char) inType;

- (NSString *) privilegesStringRepresentationForLink;

- (NSString *) privilegesStringRepresentation;

- (void) setFolder:(BOOL) inFolder;

- (BOOL) isLeaf;

- (BOOL) isLink;

- (BOOL) fileExistenceOnDiskChanged;

- (BOOL) existsOnDisk;

@end


@interface ICFileTree : TreeNode
{
}

+ (ICFileTree *) fileTreeWithDictionary:(NSDictionary *) inDictionary projectPath:(NSString *) inProjectPath referenceFolderPath:(NSString *) inReferenceFolderPath;

+ (ICFileTree *) fileTreeWithArray:(NSArray *) inArray projectPath:(NSString *) inProjectPath referenceFolderPath:(NSString *) inReferenceFolderPath;

+ (ICFileTree *) fileNodeWithDictionary:(NSDictionary *) inDictionary projectPath:(NSString *) inProjectPath referenceFolderPath:(NSString *) inReferenceFolderPath;

- (ICFileTree *) createMissingBranchesForPath:(NSString *) inPath projectPath:(NSString *) inProjectPath referenceFolderPath:(NSString *) inReferenceFolderPath;

/*+ (ICFileTree *) defaultFileTree;*/

- (void) recursiveAddBranchChildrenToArray:(NSMutableArray *) inMutableArray;

- (NSMutableArray *) descendantFoldersIncludingSelf:(BOOL) inIncludeSelf;

- (ICFileTree *) fileTreeAtPath:(NSString *) inPath;

- (NSString *) filePath;

- (NSArray *) arrayWithProjectAtPath:(NSString *) inProjectPath referenceFolderPath:(NSString *) inReferenceFolderPath;

- (NSDictionary *) dictionaryWithProjectAtPath:(NSString *) inProjectPath referenceFolderPath:(NSString *) inReferenceFolderPath;

- (BOOL) containsTreeWithName:(NSString *) inName;

- (void) insertSortedChildReverse:(ICFileTree *) inChild;

- (void) insertSortedChild:(ICFileTree *) inChild;

- (void) insertSortedChildren:(NSArray *) inChildren;

- (BOOL) expandAll:(BOOL) inKeepOwnerAndGroup withProjectPath:(NSString *) inProjectPath referenceFolderPath:(NSString *) inReferenceFolderPath;

- (void) expand:(BOOL) inKeepOwnerAndGroup;

- (void) expandOneLevel:(BOOL) inKeepOwnerAndGroup;

- (void) contract;

- (BOOL) hasRealChildren;

- (BOOL) isItemExcludedByBouncer:(ICFileBouncer *) inBouncer;

+ (NSString *) uniqueNameWithParentFileTree:(ICFileTree *) inParentTree;

@end
