/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICImageAndTextCell.h"

@implementation ICImageAndTextCell

- copyWithZone:(NSZone *)zone
{
    ICImageAndTextCell * tCell;
	
	tCell = (ICImageAndTextCell *) [super copyWithZone:zone];
	
	if (tCell!=nil)
	{
		tCell->image_ = [image_ retain];
	}
	
    return tCell;
}

- (void) dealloc
{
    [image_ release];
	
    [super dealloc];
}

#pragma mark -

- (void) setImage:(NSImage *) inImage
{
    if (image_ != inImage)
	{
        [image_ release];
		
        image_ = [inImage copy];
    }
}

- (NSImage *) image
{
    return [[image_ retain] autorelease];
}

#pragma mark -

- (NSRect) imageFrameForCellFrame:(NSRect)cellFrame
{
    if (image_ != nil)
	{
        NSRect imageFrame;
		
        imageFrame.size = [image_ size];
        imageFrame.origin = cellFrame.origin;
        imageFrame.origin.x += 3;
        imageFrame.origin.y += _CGFloatCeil((cellFrame.size.height - imageFrame.size.height) / 2);
		
        return imageFrame;
    }
    
	return NSZeroRect;
}

- (void)editWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject event:(NSEvent *)theEvent
{
    NSRect textFrame, imageFrame;
	
    NSDivideRect (aRect, &imageFrame, &textFrame, 5 + [image_ size].width, NSMinXEdge);
	
	textFrame=NSInsetRect(textFrame,0,2);
	
	textFrame.size.width-=4.0f;
	
    [super editWithFrame: textFrame inView: controlView editor:textObj delegate:anObject event: theEvent];
}

- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject start:(NSInteger)selStart length:(NSInteger)selLength
{
    NSRect textFrame, imageFrame;
	
    NSDivideRect (aRect, &imageFrame, &textFrame, 5 + [image_ size].width, NSMinXEdge);
	
	textFrame=NSInsetRect(textFrame,0,2);
	
	textFrame.size.width-=4.0f;
	
    [super selectWithFrame: textFrame inView: controlView editor:textObj delegate:anObject start:selStart length:selLength];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    if (image_ != nil)
	{
        NSSize	imageSize;
        NSRect	imageFrame;

        imageSize = [image_ size];
        
		NSDivideRect(cellFrame, &imageFrame, &cellFrame, 5 + imageSize.width, NSMinXEdge);
		
		cellFrame=NSInsetRect(cellFrame,0,2);
		
        if ([self drawsBackground])
		{
            [[self backgroundColor] set];
            NSRectFill(imageFrame);
        }
		
        imageFrame.origin.x += 3;
        imageFrame.size = imageSize;

        if ([controlView isFlipped])
		{
            imageFrame.origin.y += _CGFloatCeil((cellFrame.size.height + imageFrame.size.height) / 2) + 2.0f;
        }
		else
        {
		    imageFrame.origin.y += _CGFloatCeil((cellFrame.size.height - imageFrame.size.height) / 2);
		}
		
        [image_ compositeToPoint:imageFrame.origin operation:NSCompositeSourceOver];
    }
	
    [super drawWithFrame:cellFrame inView:controlView];
}

- (NSSize)cellSize
{
    NSSize cellSize = [super cellSize];
	
    cellSize.width += (image_ ? [image_ size].width : 0) + 3;
	
    return cellSize;
}

@end

