/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICInstallationStepsUtilities.h"

#import "ICPresentation+Constants.h"

#import "ICDocumentFormat+Constants.h"

#import "NSDocument+Iceberg.h"

#import "ICDocument.h"

#import "NSString+Iceberg.h"

@implementation ICInstallationStepsUtilities

+ (NSMutableArray *) defaultInstallationSteps
{
	NSMutableArray * tInstallationSteps=nil;
	
	tInstallationSteps=[NSMutableArray array];
	
	if (tInstallationSteps!=nil)
	{
		NSDictionary * tDictionary;
		
		// Introduction
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:@"ICPresentationViewIntroductionController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
															   @"Introduction",ICPRESENTATIONVIEWCONTROLLER_PLUGIN_NAME,
															   @"InstallerSectionTitle",ICPRESENTATIONVIEWCONTROLLER_LIST_TITLE_KEY,
															   nil];
		
		if (tDictionary!=nil)
		{
			[tInstallationSteps addObject:tDictionary];
		}
		else
		{
			// A COMPLETER
		}
		
		// Read Me
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:@"ICPresentationViewReadMeController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
															   @"ReadMe",ICPRESENTATIONVIEWCONTROLLER_PLUGIN_NAME,
															   @"InstallerSectionTitle",ICPRESENTATIONVIEWCONTROLLER_LIST_TITLE_KEY,
															   nil];
		
		if (tDictionary!=nil)
		{
			[tInstallationSteps addObject:tDictionary];
		}
		else
		{
			// A COMPLETER
		}
		
		// License
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:@"ICPresentationViewLicenseController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
															   @"License",ICPRESENTATIONVIEWCONTROLLER_PLUGIN_NAME,
															   @"InstallerSectionTitle",ICPRESENTATIONVIEWCONTROLLER_LIST_TITLE_KEY,
															   nil];
		
		if (tDictionary!=nil)
		{
			[tInstallationSteps addObject:tDictionary];
		}
		else
		{
			// A COMPLETER
		}
		
		// Destination Select
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:@"ICPresentationViewDestinationSelectController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
															   @"TargetSelect",ICPRESENTATIONVIEWCONTROLLER_PLUGIN_NAME,
															   @"InstallerSectionTitle",ICPRESENTATIONVIEWCONTROLLER_LIST_TITLE_KEY,
															   nil];
		
		if (tDictionary!=nil)
		{
			[tInstallationSteps addObject:tDictionary];
		}
		else
		{
			// A COMPLETER
		}
	
		// Installation Type
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:@"ICPresentationViewInstallationTypeController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
															   @"PackageSelection",ICPRESENTATIONVIEWCONTROLLER_PLUGIN_NAME,
															   @"InstallerSectionTitle",ICPRESENTATIONVIEWCONTROLLER_LIST_TITLE_KEY,
															   nil];
		
		if (tDictionary!=nil)
		{
			[tInstallationSteps addObject:tDictionary];
		}
		else
		{
			// A COMPLETER
		}
		
		// Installation
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:@"ICPresentationViewInstallationController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
															   @"Install",ICPRESENTATIONVIEWCONTROLLER_PLUGIN_NAME,
															   @"InstallerSectionTitle",ICPRESENTATIONVIEWCONTROLLER_LIST_TITLE_KEY,
															   nil];
		
		if (tDictionary!=nil)
		{
			[tInstallationSteps addObject:tDictionary];
		}
		else
		{
			// A COMPLETER
		}
		
		// Summary
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:@"ICPresentationViewSummaryController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
															   @"Summary",ICPRESENTATIONVIEWCONTROLLER_PLUGIN_NAME,
															   @"InstallerSectionTitle",ICPRESENTATIONVIEWCONTROLLER_LIST_TITLE_KEY,
															   nil];
		
		if (tDictionary!=nil)
		{
			[tInstallationSteps addObject:tDictionary];
		}
		else
		{
			// A COMPLETER
		}
	}
	
	return tInstallationSteps;
}

+ (BOOL) installationSteps:(NSArray *) inInstallationSteps ofDocument:(id) inDocument containsInstallerPluginAtPath:(NSString *) inPath
{
	if (inInstallationSteps!=nil && inDocument!=nil && inPath!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inInstallationSteps objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tDictionary;
			
			while (tDictionary=[tEnumerator nextObject])
			{
				NSString * tClassName;
				
				tClassName=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
				
				if (tClassName!=nil)
				{
					if ([tClassName isEqualToString:@"ICPresentationViewInstallerPluginController"]==YES)
					{
						NSDictionary * tPathDictionary;
						
						tPathDictionary=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_INSTALLERPLUGIN_PATH];
					
						if (tPathDictionary!=nil)
						{
							NSString * tPath;
							NSInteger tPathType;
							
							tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
								
							tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
							
							if (tPath!=nil && [tPath length]>0)
							{
								if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
								{
									tPath=[tPath stringByAbsolutingWithPath:[inDocument folder]];
								}
								else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
								{
									tPath=[tPath stringByAbsolutingWithPath:[inDocument referenceFolderPath]];
								}
							
								if (tPath!=nil && [tPath isEqualToString:inPath])
								{
									return YES;
								}
							}
						}
					}
				}
			}
		}
	}

	return NO;
}

+ (BOOL) projectPresentationHasInstallerPlugins:(NSDictionary *) inProjectPresentationDictionary
{
	if (inProjectPresentationDictionary!=nil)
	{
		NSArray * tInstallationSteps;
		
		tInstallationSteps=[inProjectPresentationDictionary objectForKey:ICDOCUMENT_PROJECT_INSTALLATION_STEPS];
		
		if (tInstallationSteps!=nil)
		{
			NSEnumerator * tEnumerator;
		
			tEnumerator=[tInstallationSteps objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSDictionary * tDictionary;
				
				while (tDictionary=[tEnumerator nextObject])
				{
					NSString * tClassName;
					
					tClassName=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
					
					if (tClassName!=nil)
					{
						if ([tClassName isEqualToString:@"ICPresentationViewInstallerPluginController"]==YES)
						{
							return YES;
						}
					}
				}
			}
		}
	}
	
	return NO;
}

@end
