/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICInstallationTypeCheckBoxCell.h"

#import "ICInstallationTypeTree.h"

@implementation ICInstallationTypeCheckBoxCell

- (void) setInvisible:(BOOL) inInvisible
{
	invisible_=inInvisible;
}

- (void) setDependent:(BOOL) inDependent
{
	dependent_=inDependent;
}

#pragma mark -

- (BOOL) trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)flag
{
	return YES;
}

#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5

- (NSUInteger) hitTestForEvent:(NSEvent *) inEvent inRect:(NSRect) inFrame ofView:(NSView *) inView
{
	return NSCellHitContentArea;
}

#endif

#pragma mark -

- (void) drawImage:(NSImage*) inImage withFrame:(NSRect) inFrame inView:(NSView*) inView
{
	NSImage * tImage;
	NSSize tSize;
	
	if (invisible_==YES && inImage!=nil)
	{
		NSImage * tStrippedImage;
		
		tImage=[NSImage imageNamed:@"Strip32Composite"];
		
		tSize=[inImage size];
		
		tStrippedImage=[[NSImage alloc] initWithSize:tSize];
		
		if (tStrippedImage!=nil)
		{
			[tStrippedImage lockFocus];
			
#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5
			if (NSAppKitVersionNumber>949)
#endif
			{
				[tImage compositeToPoint:NSMakePoint(3.0f,0.0f) fromRect:NSMakeRect(0.0f,0.0f,13.0f,17.0f) operation:NSCompositeSourceOver];
			}
#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5
			else
			{
				[tImage compositeToPoint:NSZeroPoint operation:NSCompositeSourceOver];
			}
#endif
			
			[inImage compositeToPoint:NSZeroPoint operation:NSCompositeSourceIn];
			
			if (dependent_==YES)
			{
				// Draw the small gear
				
				tImage=[NSImage imageNamed:@"GearCheckBox"];
				
#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5
				[tImage compositeToPoint:NSMakePoint(2.0f,2.0f) operation:NSCompositeSourceIn];
#else
				[tImage compositeToPoint:NSMakePoint(4.0f,3.0f) operation:NSCompositeSourceIn];
#endif
			}
			
			[tStrippedImage unlockFocus];
			
			[super drawImage:tStrippedImage withFrame:inFrame inView:inView];
			
			[tStrippedImage release];
			
			return;
		}
	}
	
	[super drawImage:inImage withFrame:inFrame inView:inView];
	
	if (dependent_==YES)
	{
		// Draw the small gear
		
		tImage=[NSImage imageNamed:@"GearCheckBox"];
				
		tSize=[tImage size];
				
		[tImage compositeToPoint:NSMakePoint(_CGFloatRound(NSMidX(inFrame)-tSize.width*0.5f),_CGFloatRound(NSMidY(inFrame)+tSize.height*0.5f)) operation:NSCompositeSourceOver];
	}
}

- (NSRect) drawTitle:(NSAttributedString *) inAttributedString withFrame:(NSRect) inFrame inView:(NSView*) inView
{
	inFrame.origin.y-=1.0f;
	
	if (invisible_==YES)
	{
		NSMutableAttributedString * tMutableAttributedString;
		
		tMutableAttributedString=[[NSMutableAttributedString alloc] initWithAttributedString:inAttributedString];
		
		if (tMutableAttributedString!=nil)
		{
			NSImage * tImage;
			
			if ([self isEnabled]==YES)
			{
				tImage=[NSImage imageNamed:@"Strip32"];
			}
			else
			{
				tImage=[NSImage imageNamed:@"Strip32Disabled"];
			}
		
			if (tImage!=nil)
			{
				NSRect tRect;
	
				NSGraphicsContext * tGraphicContext;
				
				tGraphicContext = [NSGraphicsContext currentContext];
				
				[NSGraphicsContext saveGraphicsState];
				
				[tMutableAttributedString addAttribute:NSForegroundColorAttributeName 
												 value:[NSColor colorWithPatternImage:tImage] 
												 range:NSMakeRange(0,[tMutableAttributedString length])];
				
				[tGraphicContext setPatternPhase:NSMakePoint(0.0f,[inView convertPoint:NSZeroPoint toView:nil].y)];
				
				tRect=[super drawTitle:tMutableAttributedString withFrame:inFrame inView:inView];
			
				[NSGraphicsContext restoreGraphicsState];
				
				[tMutableAttributedString release];
			
				return tRect;
			}
			else
			{
				[tMutableAttributedString release];
			}

		}
	}
	
	return [super drawTitle:inAttributedString withFrame:inFrame inView:inView];
}

@end
