/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICInstallationTypeOutlineView.h"

@implementation ICInstallationTypeCornerView

- (id) initWithFrame:(NSRect) aFrame
{
	self=[super initWithFrame:aFrame];
	
	if (self!=nil)
	{
		_drawCell=[[NSTableHeaderCell alloc] initImageCell:nil];
	}
	
	return self;
}

#pragma mark -

- (BOOL) isOpaque
{
	return NO;
}

- (void) setDashedBorder:(BOOL) inDashedBorder
{
	dashedBorder_=inDashedBorder;
}

- (BOOL) dashedBorder
{
	return dashedBorder_;
}

- (void) showPackagesName:(BOOL) inBool
{
	if (showPackagesName_!=inBool)
	{
		showPackagesName_=inBool;
		
		[self setNeedsDisplay:YES];
	}
}

- (void) drawRect:(NSRect) aFrame
{
	if (dashedBorder_==YES)
	{
		NSRect tBounds;
		NSBezierPath * tBezierPath;
		
		tBounds=[self bounds];
	
		tBezierPath=[NSBezierPath bezierPath];
		
		if (tBezierPath!=nil)
		{
			const CGFloat tDash[2]={2.0f,2.0f};
			
			[[NSColor whiteColor] set];
			
			[tBezierPath moveToPoint:NSMakePoint(0.0f,0.0f)];
			
			[tBezierPath lineToPoint:NSMakePoint(NSMaxX(tBounds),0.0f)];
			
			[tBezierPath stroke];
		
			[tBezierPath setLineDash:tDash count:2 phase:0.0f];
		
			[[NSColor colorWithDeviceWhite:0.209f alpha:1.0f] set];
			
			[tBezierPath stroke];
		}
		
		tBounds.size.height-=1;
	
		tBounds.origin.y+=1.0f;
		
		[NSBezierPath clipRect:tBounds];
	}

	[super drawRect:aFrame];
	
	if (showPackagesName_==YES)
	{
		NSBezierPath * tBezierPath;
		
		
		tBezierPath=[NSBezierPath bezierPath];
		
		if (tBezierPath!=nil)
		{
			NSRect tRect;
			
			tRect=[self bounds];
			
			[tBezierPath moveToPoint:NSMakePoint(NSMinX(tRect),NSMinY(tRect))];
			
			[tBezierPath lineToPoint:NSMakePoint(NSMaxX(tRect),NSMinY(tRect))];
			
			[tBezierPath lineToPoint:NSMakePoint(NSMaxX(tRect),NSMaxY(tRect))];
		
			[tBezierPath closePath];
			
			[[NSColor colorWithDeviceWhite:0.0f alpha:0.5f] set];
			
			[tBezierPath fill];
		}
	}
}

@end

@implementation ICInstallationTypeOutlineHeaderView

- (void) setDashedBorder:(BOOL) inDashedBorder
{
	dashedBorder_=inDashedBorder;
}

- (BOOL) dashedBorder
{
	return dashedBorder_;
}

- (void) drawRect:(NSRect) aFrame
{
	if (dashedBorder_==YES)
	{
		NSRect tBounds;
		NSBezierPath * tBezierPath;
		
		tBounds=[self bounds];
	
		tBezierPath=[NSBezierPath bezierPath];
		
		if (tBezierPath!=nil)
		{
			const CGFloat tDash[2]={2.0f,2.0f};
			
			[[NSColor whiteColor] set];
			
			[tBezierPath moveToPoint:NSMakePoint(0.0f,0.0f)];
			
			[tBezierPath lineToPoint:NSMakePoint(NSMaxX(tBounds),0.0f)];
			
			[tBezierPath stroke];
		
			[tBezierPath setLineDash:tDash count:2 phase:0.0f];
		
			[[NSColor colorWithDeviceWhite:0.209f alpha:1.0f] set];
			
			[tBezierPath stroke];
		}
		
		tBounds.size.height-=1;
	
		tBounds.origin.y+=1.0f;
		
		[NSBezierPath clipRect:tBounds];
	}

	[super drawRect:aFrame];
}

@end


@implementation ICInstallationTypeScrollView

- (BOOL) isOpaque
{
	return YES;
}

- (void) setDashedBorder:(BOOL) inDashedBorder
{
	if (dashedBorder_!=inDashedBorder)
	{
		dashedBorder_=inDashedBorder;
		
		[self setNeedsDisplay:YES];
	}
}

- (BOOL) dashedBorder
{
	return dashedBorder_;
}

- (void) drawRect:(NSRect) inRect
{
	if (dashedBorder_==YES)
	{
		NSRect tBounds=[self bounds];
		NSBezierPath * tBezierPath;
		
		[[NSColor colorWithDeviceWhite:0.530f alpha:1.0f] set];
		
		tBezierPath=[NSBezierPath bezierPath];
		
		if (tBezierPath!=nil)
		{
			const CGFloat tDash[2]={2.0f,2.0f};
			
			[tBezierPath moveToPoint:NSMakePoint(0.0f,0.0f)];
			
			[tBezierPath lineToPoint:NSMakePoint(0.0f,NSMaxY(tBounds))];
			
			[tBezierPath lineToPoint:NSMakePoint(NSMaxX(tBounds),NSMaxY(tBounds))];
			
			[tBezierPath lineToPoint:NSMakePoint(NSMaxX(tBounds),0.0f)];
			
			[tBezierPath setLineDash:tDash count:2 phase:0.0f];
			
			[tBezierPath stroke];
			
			[tBezierPath removeAllPoints];
			
			if ([[self documentView] isKindOfClass:[NSTextView class]]==YES)
			{
				[[NSColor colorWithDeviceWhite:0.209f alpha:1.0f] set];
			
				[tBezierPath moveToPoint:NSMakePoint(0.0f,0.0f)];
			
				[tBezierPath lineToPoint:NSMakePoint(NSMaxX(tBounds),0.0f)];
			
				[tBezierPath stroke];
			}
		}
	}
	else
	{
		[super drawRect:inRect];
	}
}

@end

@implementation NSObject (ICInstallationTypeOutlineView)

- (void) switchShowPackagesName
{
}

@end

@implementation ICInstallationTypeOutlineView

- (BOOL) performKeyEquivalent:(NSEvent *) inEvent
{
	unichar tChar;
		
	tChar=[[inEvent charactersIgnoringModifiers] characterAtIndex:0];
		
	if (tChar=='.')
	{
		if ((NSCommandKeyMask+NSShiftKeyMask)& [inEvent modifierFlags])
		{
			[[self delegate] switchShowPackagesName];
			
			return YES;
		}
	}
	
	return [super performKeyEquivalent:inEvent];
}

- (BOOL) _canDragRowForClickOnCell:(id) inCell column:(int) inColumn row:(int) inRowIndex atPoint:(NSPoint) inPoint
{
	return YES;
}

@end
