/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICInstallationTypeTree.h"
#import "ICDocumentFormat+Constants.h"


@implementation ICInstallationTypeNodeData

+ (id) installationTypeNodeDataOfType:(int) inType withDictionary:(NSMutableDictionary *) inDictionary
{
	return [[[ICInstallationTypeNodeData alloc] initWithType:inType andDictionary:inDictionary] autorelease];
}

+ (id) rootNodeData
{
	return [ICInstallationTypeNodeData installationTypeNodeDataOfType:kInstallationTypeRootNode withDictionary:nil];
}

- (id) initWithType:(int) inType andDictionary:(NSMutableDictionary *) inDictionary
{
	self=[super init];
	
	if (self!=nil)
	{
		type_=inType;
		
		dictionary_=[inDictionary retain];
	}
	
	return self;
}

- (void) dealloc
{
	[dictionary_ release];

	[super dealloc];
}

#pragma mark -

- (BOOL) isLeaf
{
	if (type_==kInstallationTypeChoiceNode)
	{
		NSNumber * tType;
		
		tType=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
		
		if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
		{
			return YES;
		}
	}
	
	return NO;
}

- (int) type
{
	return type_;
}

- (NSMutableDictionary *) dictionary
{
	return [[dictionary_ retain] autorelease];
}

@end


@implementation ICInstallationTypeTreeNode

+ (id) installationTypeTreeNodeWithArray:(NSMutableArray *) inArray
{
	ICInstallationTypeTreeNode * tRootNode;
	
	tRootNode=[[ICInstallationTypeTreeNode alloc] initWithData:[ICInstallationTypeNodeData rootNodeData]
														parent:nil
													  children:[NSArray array]];
	
	if (tRootNode!=nil)
	{
		if (inArray!=nil)
		{
			NSEnumerator * tEnumerator;
		
			tEnumerator=[inArray objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSMutableDictionary * tItemDictionary;
				
				while (tItemDictionary=[tEnumerator nextObject])
				{
					ICInstallationTypeTreeNode * tChildChoiceNode=nil;
					
					tChildChoiceNode=[ICInstallationTypeTreeNode choiceTreeNodeWithDictionary:tItemDictionary];
					
					if (tChildChoiceNode!=nil)
					{
						[tRootNode insertChild:tChildChoiceNode
									   atIndex:[tRootNode numberOfChildren]];
					}
				}
			}
		}
	}
	
	return tRootNode;
}

+ (id) choiceTreeNodeWithDictionary:(NSMutableDictionary *) inDictionary
{
	ICInstallationTypeTreeNode * tChoiceNode=nil;
	
	if (inDictionary!=nil)
	{
		NSNumber * tNumber;
		BOOL isGroup=NO;
		
		tNumber=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
		
		if (tNumber!=nil && [tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
		{
			isGroup=YES;
		}
		
		tChoiceNode=[[ICInstallationTypeTreeNode alloc] initWithData:[ICInstallationTypeNodeData installationTypeNodeDataOfType:kInstallationTypeChoiceNode withDictionary:inDictionary]
															  parent:nil
															children:[NSArray array]];
		
		if (tChoiceNode!=nil)
		{
			if (isGroup==YES)
			{
				NSArray * tChildrenArray;
				
				tChildrenArray=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
				
				if (tChildrenArray!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[tChildrenArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSMutableDictionary * tItemDictionary;
				
						while (tItemDictionary=[tEnumerator nextObject])
						{
							ICInstallationTypeTreeNode * tChildChoiceNode=nil;
							
							tChildChoiceNode=[ICInstallationTypeTreeNode choiceTreeNodeWithDictionary:tItemDictionary];
							
							if (tChildChoiceNode!=nil)
							{
								[tChoiceNode insertChild:tChildChoiceNode
												 atIndex:[tChoiceNode numberOfChildren]];
							}
						}
					}
				}
			}
		}
	}
	
	return [tChoiceNode autorelease];
}

- (id) addChoiceTreeNodeWithDictionary:(NSMutableDictionary *) inDictionary
{
	ICInstallationTypeTreeNode * tChoiceNode=nil;
	
	if (inDictionary!=nil)
	{
		tChoiceNode=[ICInstallationTypeTreeNode choiceTreeNodeWithDictionary:inDictionary];
				
		if (tChoiceNode!=nil)
		{
			[self insertChild:tChoiceNode
					  atIndex:[self numberOfChildren]];
		}
	}
	
	return tChoiceNode;
}

- (BOOL) isEqualToOrChildOfChoicesInArray:(NSArray *) inChoiceTreeArray
{
	if (inChoiceTreeArray!=nil)
	{
		if ([inChoiceTreeArray containsObject:self]==YES)
		{
			return YES;
		}
		else
		{
			if ([self isDescendantOfNodeInArray:inChoiceTreeArray]==YES)
			{
				return YES;
			}
		}
	}
	
	return NO;
}

- (int) selectedState
{
	ICInstallationTypeNodeData * tNodeData;
			
	tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(self);
	
	if (tNodeData!=nil)
	{
		if ([tNodeData type]!=kInstallationTypeRootNode)
		{
			NSDictionary * tDictionary;
			
			tDictionary=[tNodeData dictionary];
		
			if (tDictionary!=nil)
			{
				NSNumber * tType;
				NSDictionary * tOptions;
			
				tOptions=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
					
				tType=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
		
				if (tOptions!=nil)
				{
					NSNumber * tStateNumber;
					int tState;
					
					tStateNumber=[tOptions objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
					
					tState=[tStateNumber intValue];
					
					if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
					{
						if (tState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENT)
						{
							return kInstallationTypeDependentState;
						}
						else
						{
							return (tState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_REQUIRED || tState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED);
						}
					}
					else
					{
						if (tState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_DEPENDENT)
						{
							return kInstallationTypeDependentState;
						}
						else
						{
							NSNumber * tNumber;
									
							tNumber=[tOptions objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
							
							if (tNumber!=nil && [tNumber boolValue]==YES)
							{
								return (tState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_REQUIRED || tState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED);
							}
							else
							{
								NSArray * tChildrenArray;
								
								tChildrenArray=[self children];
								
								if (tChildrenArray!=nil)
								{
									NSEnumerator * tEnumerator;
									int tFinalState=-5;
									
									tEnumerator=[tChildrenArray objectEnumerator];
									
									if (tEnumerator!=nil)
									{
										ICInstallationTypeTreeNode * tChild;
										
										while (tChild=[tEnumerator nextObject])
										{
											int tSelectedState;
											
											tSelectedState=[tChild selectedState];
											
											if (tFinalState==-5)
											{
												tFinalState=tSelectedState;
											}
											else
											{
												if (tFinalState!=tSelectedState)
												{
													return kInstallationTypeMixedState;
												}
											}
										}
									}
									
									if (tFinalState==kInstallationTypeDependentState)
									{
										tFinalState=kInstallationTypeMixedState;
									}
									
									if (tFinalState!=-5)
									{
										return tFinalState;
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	return kInstallationTypeOffState;
}

- (BOOL) isEnabled
{
	ICInstallationTypeNodeData * tNodeData;
	
			
	tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(self);
	
	if (tNodeData!=nil)
	{
		if ([tNodeData type]!=kInstallationTypeRootNode)
		{
			id tParentNode;
			NSDictionary * tDictionary;
				
			tDictionary=[tNodeData dictionary];
			
			tParentNode=[self nodeParent];
			
			if (tDictionary!=nil)
			{
				NSNumber * tType;
				NSDictionary * tOptions;
				
				tType=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
		
				if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
				{
					if ([self isInHiddenGroup]==YES)
					{
						return [tParentNode isEnabled];
					}
				}
				
				tOptions=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
				
				if (tOptions!=nil)
				{
					NSNumber * tNumber;
									
					tNumber=[tOptions objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
					
					if (tNumber!=nil &&
						([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_REQUIRED ||
						 [tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_NEVER))
					{
						return NO;
					}
				} 
			}
			
			tParentNode=[self nodeParent];
			
			if (tParentNode!=nil)
			{
				return [tParentNode isEnabled];
			}
		}
	}
	
	return YES;
}

- (void) setExpanded:(BOOL) inExpanded
{
	isExpanded_=inExpanded;
}

- (BOOL) isExpanded
{
	return isExpanded_;
}

- (BOOL) isInvisible
{
	ICInstallationTypeNodeData * tNodeData;
	
	tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(self);
	
	if (tNodeData!=nil)
	{
		if ([tNodeData type]!=kInstallationTypeRootNode)
		{
			id tParentNode;
			NSDictionary * tDictionary;
				
			tDictionary=[tNodeData dictionary];
			
			tParentNode=[self nodeParent];
			
			if (tDictionary!=nil)
			{
				NSNumber * tType;
				NSDictionary * tOptions;
				
				tType=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
		
				if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
				{
					if ([self isInHiddenGroup]==YES)
					{
						return [tParentNode isInvisible];
					}
				}
				
				tOptions=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
				
				if (tOptions!=nil)
				{
					NSNumber * tNumber;
									
					tNumber=[tOptions objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN];
					
					if (tNumber!=nil && [tNumber boolValue]==YES)
					{
						return YES;
					}
				} 
			}
			
			if (tParentNode!=nil)
			{
				return [tParentNode isInvisible];
			}
		}
	}
	
	return NO;
}

- (BOOL) isInHiddenGroup
{
	ICInstallationTypeTreeNode * tParentNode;
	
	tParentNode=(ICInstallationTypeTreeNode *) [self nodeParent];
	
	if (tParentNode!=nil)
	{
		ICInstallationTypeNodeData * tParentNodeData;

		tParentNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tParentNode);
		
		if (tParentNodeData!=nil)
		{
			if ([tParentNodeData type]!=kInstallationTypeRootNode)
			{
				NSDictionary * tParentDictionary;
				
				tParentDictionary=[tParentNodeData dictionary];
				
				if (tParentDictionary!=nil)
				{
					NSDictionary * tParentOptionsDictionary;
					
					tParentOptionsDictionary=[tParentDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
					
					if (tParentOptionsDictionary!=nil)
					{
						NSNumber * tNumber;
							
						tNumber=[tParentOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
						
						if (tNumber!=nil && [tNumber boolValue]==YES)
						{
							return YES;
						}
					}
					else
					{
						NSLog(@"[ICInstallationTypeTreeNode isInHiddenGroup] Missing ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS dictionary");
					}
				}
				else
				{
					NSLog(@"[ICInstallationTypeTreeNode isInHiddenGroup] Missing dictionary");
				}
			}
		}
	}
	
	return NO;
}

- (BOOL) isGroup
{
	ICInstallationTypeNodeData * tNodeData;

	tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(self);
	
	if (tNodeData!=nil)
	{
		if ([tNodeData type]!=kInstallationTypeRootNode)
		{
			NSDictionary * tDictionary;
			
			tDictionary=[tNodeData dictionary];
			
			if (tDictionary!=nil)
			{
				NSNumber * tNumber;
				
				tNumber=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
				
				if (tNumber!=nil)
				{
					if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
					{
						NSDictionary * tOptionsDictionary;
					
						tOptionsDictionary=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
					
						if (tOptionsDictionary!=nil)
						{
							tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
						
							if (tNumber!=nil && [tNumber boolValue]==YES)
							{
								return NO;
							}
						}
						
						return YES;
					}
				}
				else
				{
					NSLog(@"[ICInstallationTypeTreeNode isGroup] Missing ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE dictionary");
				}
			}
			else
			{
				NSLog(@"[ICInstallationTypeTreeNode isGroup] Missing dictionary");
			}
		}
	}
	
	return NO;
}

- (ICInstallationTypeTreeNode *) rootNode
{
	ICInstallationTypeTreeNode * tParentNode;
	ICInstallationTypeTreeNode * tNode;
	
	tNode=self;
	
	while (tParentNode=((ICInstallationTypeTreeNode *) [tNode nodeParent]))
	{
		tNode=tParentNode;
	}
	
	return tNode;
}

- (ICInstallationTypeTreeNode *) nodeWithChoiceUUID:(NSString *) inChoiceUUID
{
	if (inChoiceUUID!=nil)
	{
		NSArray * tChildren;
		NSEnumerator * tEnumerator;
		
		if ([[self choiceUUID] isEqualToString:inChoiceUUID]==YES)
		{
			return self;
		}
		
		tChildren=[self children];
		
		tEnumerator=[tChildren objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			ICInstallationTypeTreeNode * tChoiceNode;
			
			
			while (tChoiceNode=((ICInstallationTypeTreeNode *) [tEnumerator nextObject]))
			{
				ICInstallationTypeTreeNode * tResult;
				
				tResult=[tChoiceNode nodeWithChoiceUUID:inChoiceUUID];
				
				if (tResult!=nil)
				{
					return tResult;
				}
			}
		}
	}
	
	return nil;
}

+ (NSMutableArray *) choiceUUIDOfReverseHierarchyFromNode:(ICInstallationTypeTreeNode *) inNode
{
	NSMutableArray * tMutableArray=nil;
	
	if (inNode!=nil)
	{
		tMutableArray=[NSMutableArray array];
		
		if (tMutableArray!=nil)
		{
			ICInstallationTypeTreeNode * tParentNode;
			ICInstallationTypeTreeNode * tNode;
	
			tNode=inNode;
	
			while (tParentNode=((ICInstallationTypeTreeNode *) [tNode nodeParent]))
			{
				NSString * tChoiceUUID;
				
				tChoiceUUID=[tNode choiceUUID];
				
				if (tChoiceUUID!=nil)
				{
					[tMutableArray addObject:tChoiceUUID];
				}
				
				tNode=tParentNode;
			}
		}
	}
	
	return tMutableArray;
}

- (NSString *) choiceUUID
{
	ICInstallationTypeNodeData * tNodeData;
	NSString * tUUID=nil;
	
	tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(self);

	if (tNodeData!=nil)
	{
		if ([tNodeData type]!=kInstallationTypeRootNode)
		{
			NSDictionary * tChoiceDictionary;
			
			tChoiceDictionary=[tNodeData dictionary];
			
			if (tChoiceDictionary!=nil)
			{
				// Check the Type of Choice
				
				tUUID=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
			}
		}
	}
	
	return tUUID;
}

@end