/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICInstallationTypeUtilities+Extras.h"

#import "ICUUIDFactory.h"

#import "ICDocumentFormat+Constants.h"

#import "ICLocalizationUtilities.h"

@implementation ICInstallationTypeUtilities (Extras)

+ (void) removeLocalizationsInItem:(NSMutableDictionary *) inItemDictionary
{
	if (inItemDictionary!=nil)
	{
		NSNumber * tNumber;
		
		// Remove Title Localizations
		
		[inItemDictionary setObject:[NSMutableArray array] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TITLE];
		
		// Remove Description Localizations
		
		[inItemDictionary setObject:[NSMutableArray array] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_DESCRIPTION];
		
		// Remove Tooltip Localizations
		
		[inItemDictionary setObject:[NSMutableArray array] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TOOLTIP];
		
		// Check the Item Type
		
		tNumber=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
		
		if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
		{
			// It's a group
			
			NSMutableArray * tChildrenArray;
			
			tChildrenArray=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
			
			if (tChildrenArray!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tChildrenArray objectEnumerator];
			
				if (tEnumerator!=nil)
				{
					NSMutableDictionary * tChildDictionary;
					
					while (tChildDictionary=[tEnumerator nextObject])
					{
						[ICInstallationTypeUtilities removeLocalizationsInItem:tChildDictionary];
					}
				}
			}
		}
	}
}

+ (void) removeLocalizationsInHierarchy:(NSMutableDictionary *) inHierarchyDictionary
{
	if (inHierarchyDictionary!=nil)
	{
		NSArray * tArray;
		NSDictionary * tDictionary;
		NSEnumerator * tEnumerator;
		
		tArray=[inHierarchyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST];
		
		if (tArray!=nil)
		{
			tEnumerator=[tArray objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSMutableDictionary * tItemDictionary;
				
				while (tItemDictionary=[tEnumerator nextObject])
				{
					[ICInstallationTypeUtilities removeLocalizationsInItem:tItemDictionary];
				}
			}
		}
		
		tDictionary=[inHierarchyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED];
		
		if (tDictionary!=nil)
		{
			tEnumerator=[tDictionary objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSMutableDictionary * tItemDictionary;
				
				while (tItemDictionary=[tEnumerator nextObject])
				{
					[ICInstallationTypeUtilities removeLocalizationsInItem:tItemDictionary];
				}
			}
		}
	}
}

+ (BOOL) isPackageWithUUID:(NSString *) inPackagesUUID inListItems:(NSArray *) inListItems
{
	if (inPackagesUUID!=nil && inListItems!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inListItems objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tItemDictionary;
			
			while (tItemDictionary=[tEnumerator nextObject])
			{
				NSNumber * tNumber;
		
				// Check the Item Type
				
				tNumber=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
				
				if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
				{
					// It's a package
				
					NSString * tPackageUUID;
					
					tPackageUUID=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_PACKAGE_UUID];
					
					if ([inPackagesUUID isEqualToString:tPackageUUID]==YES)
					{
						return YES;
					}
				}
				else
				{
					// It's a group
					
					NSMutableArray * tChildrenArray;
					
					tChildrenArray=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
					
					if (tChildrenArray!=nil)
					{
						if ([ICInstallationTypeUtilities isPackageWithUUID:inPackagesUUID inListItems:tChildrenArray]==YES)
						{
							return YES;
						}
					}
				}
			}
		}
	}
	
	return NO;
}

+ (BOOL) isPackageWithUUID:(NSString *) inPackagesUUID partOfHierarchy:(NSDictionary *) inHierarchyDictionary part:(NSString **) outPart
{
	*outPart=nil;
	
	
	if (inPackagesUUID!=nil && inHierarchyDictionary!=nil)
	{
		NSArray * tArray;
		NSDictionary * tDictionary;
		
		// Check the hierarchy then
		
		tDictionary=[inHierarchyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED];
		
		if (tDictionary!=nil)
		{
			if ([tDictionary objectForKey:inPackagesUUID]!=nil)
			{
				*outPart=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED;
				
				return YES;
			}
		}
		
		// Check the removed first
	
		tArray=[inHierarchyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST];
		
		if (tArray!=nil)
		{
			if ([ICInstallationTypeUtilities isPackageWithUUID:inPackagesUUID inListItems:tArray]==YES)
			{
				*outPart=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST;
				
				return YES;
			}
		}
		
		
	}
	
	return NO;
}

+ (void) flattenItem:(NSDictionary *) inItemDictionary intoList:(NSMutableArray *) inList
{
	if (inItemDictionary!=nil && inList!=nil)
	{
		if ([inList containsObject:inItemDictionary]==NO)
		{
			NSNumber * tNumber;
			
			[inList addObject:inItemDictionary];
			
			// Check the Item Type
				
			tNumber=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
				
			if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
			{
				// It's a group
				
				NSMutableArray * tChildrenArray;
				
				tChildrenArray=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
				
				if (tChildrenArray!=nil)
				{
					NSEnumerator * tEnumerator;
			
					tEnumerator=[tChildrenArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSDictionary * tItemDictionary;
						
						while (tItemDictionary=[tEnumerator nextObject])
						{
							[ICInstallationTypeUtilities flattenItem:tItemDictionary intoList:inList];
						}
					}
				}
			}
		}
	}
}

+ (NSMutableArray *) flattenItemsFromItemsList:(NSArray *) inItemsList
{
	NSMutableArray * tMutableArray=nil;
	
	if (inItemsList!=nil)
	{
		tMutableArray=[NSMutableArray array];
		
		if (tMutableArray!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[inItemsList objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSDictionary * tItemDictionary;
				
				while (tItemDictionary=[tEnumerator nextObject])
				{
					[ICInstallationTypeUtilities flattenItem:tItemDictionary intoList:tMutableArray];
				}
			}
		}
		else
		{
			NSLog(@"[ICInstallationTypeUtilities flattenItemsFromItemsList:] Not enough memory");
		}
	}

	return tMutableArray;
}

+ (BOOL) removePackagesWithUUID:(NSArray *) inPackagesUUID fromHierarchyNode:(NSMutableDictionary *) inItemDictionary andUpdateRemovedChoices:(NSMutableArray *) outChoicesUUID
{
	if (inItemDictionary!=nil && [inPackagesUUID count]>0)
	{
		NSNumber * tNumber;
		NSString * tChoiceUUID;
		
		// Check the Item Type
		
		tNumber=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
		
		if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
		{
			// It's a package
		
			NSString * tPackageUUID;
			NSInteger tIndex;
			
			tPackageUUID=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_PACKAGE_UUID];
			
			tIndex=[inPackagesUUID indexOfObject:tPackageUUID];
			
			if (tIndex!=NSNotFound)
			{
				tChoiceUUID=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
				
				if (tChoiceUUID!=nil)
				{
					[outChoicesUUID addObject:tChoiceUUID];
				}
				
				return YES;
			}
			
			return NO;
		}
		else
		{
			// It's a group
			
			NSMutableArray * tChildrenArray;
			
			tChildrenArray=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
			
			if (tChildrenArray!=nil)
			{
				NSUInteger tCount,i;
		
				tCount=[tChildrenArray count];
		
				for(i=tCount;i>0;i--)
				{
					NSMutableDictionary * tItemDictionary;
					
					tItemDictionary=[tChildrenArray objectAtIndex:i-1];
					
					if ([ICInstallationTypeUtilities removePackagesWithUUID:inPackagesUUID fromHierarchyNode:tItemDictionary andUpdateRemovedChoices:outChoicesUUID]==YES)
					{
						[tChildrenArray removeObjectAtIndex:i-1];
					}
				}
				
				if ([tChildrenArray count]==0)
				{
					NSDictionary * tOptionsDictionary;
					
					tOptionsDictionary=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
					
					if (tOptionsDictionary!=nil)
					{
						tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN];
						
						if (tNumber!=nil && [tNumber boolValue]==YES)
						{
							tChoiceUUID=[inItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
				
							if (tChoiceUUID!=nil)
							{
								[outChoicesUUID addObject:tChoiceUUID];
							}
							
							return YES;
						}
					}
				}
			}
		}
	}
	
	return NO;
}

+ (void) removeDependenciesToChoices:(NSMutableArray *) inChoicesUUID fromDependenciesDictionary:(NSMutableDictionary *) inDependenciesDictionary
{
	if (inChoicesUUID!=nil && inDependenciesDictionary!=nil)
	{
		// Is it a Leaf or a branch
	
		NSMutableDictionary * tBranch;
		
		tBranch=[inDependenciesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP];
		
		if (tBranch==nil)
		{
			// It's a leaf, honey!
			
			NSString * tUUID;
			
			tUUID=[inDependenciesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID];
			
			if (tUUID!=nil && [inChoicesUUID containsObject:tUUID]==YES)
			{
				[inDependenciesDictionary removeAllObjects];
			}
		}
		else
		{
			BOOL tRemovedTopBranch=NO;
			BOOL tRemovedBottomBranch=NO;
			
			[ICInstallationTypeUtilities removeDependenciesToChoices:inChoicesUUID fromDependenciesDictionary:tBranch];
			
			if ([tBranch count]==0)
			{
				tRemovedTopBranch=YES;
			}
			
			tBranch=[inDependenciesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM];
			
			[ICInstallationTypeUtilities removeDependenciesToChoices:inChoicesUUID fromDependenciesDictionary:tBranch];
			
			if ([tBranch count]==0)
			{
				tRemovedBottomBranch=YES;
			}
			
			if (tRemovedTopBranch==YES || tRemovedBottomBranch==YES)
			{
				if (tRemovedTopBranch==YES && tRemovedBottomBranch==YES)
				{
					[inDependenciesDictionary removeAllObjects];
				}
				else if (tRemovedTopBranch==YES)
				{
					tBranch=[inDependenciesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM];
					
					if (tBranch!=nil)
					{
						[tBranch retain];
					
						[inDependenciesDictionary setDictionary:tBranch];
					
						[tBranch release];
					}
					else
					{
						[inDependenciesDictionary removeAllObjects];
					}
				}
				else
				{
					tBranch=[inDependenciesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP];
					
					if (tBranch!=nil)
					{
						[tBranch retain];
					
						[inDependenciesDictionary setDictionary:tBranch];
					
						[tBranch release];
					}
					else
					{
						[inDependenciesDictionary removeAllObjects];
					}
				}
			}
		}
	}
}

+ (void) removeDependenciesToChoices:(NSMutableArray *) inChoicesUUID fromChoiceDictionary:(NSMutableDictionary *) inChoiceDictionary
{
	if (inChoicesUUID!=nil && inChoiceDictionary!=nil)
	{
		NSNumber * tNumber;
		NSMutableDictionary * tOptionsDictionary;
		
		// Check the Item Type
		
		tNumber=[inChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
		
		if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
		{
			// It's a group
			
			NSMutableArray * tChildrenArray;
			
			tChildrenArray=[inChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
			
			if (tChildrenArray!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tChildrenArray objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSMutableDictionary * tChoiceDictionary;
					
					while (tChoiceDictionary=[tEnumerator nextObject])
					{
						[ICInstallationTypeUtilities removeDependenciesToChoices:inChoicesUUID fromChoiceDictionary:tChoiceDictionary];
					}
				}
			}
		}
		
		tOptionsDictionary=[inChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
		
		if (tOptionsDictionary!=nil)
		{
			tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
			
			if (tNumber!=nil)
			{
				int tOptionsState;
				NSMutableDictionary * tOptionsStateDependenciesDictionary;
				NSMutableDictionary * tDependenciesDictionary;
				
				tOptionsStateDependenciesDictionary=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
				
				tOptionsState=[tNumber intValue];
				
				if (tOptionsState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENT)
				{
					tNumber=[tOptionsStateDependenciesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE];
					
					if (tNumber!=nil)
					{
						if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_DEPENDENT)
						{
							tDependenciesDictionary=[tOptionsStateDependenciesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
							
							[ICInstallationTypeUtilities removeDependenciesToChoices:inChoicesUUID fromDependenciesDictionary:tDependenciesDictionary];
							
							if ([tDependenciesDictionary count]==0)
							{
								[tOptionsStateDependenciesDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_ALWAYS] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE];
							
								[tOptionsStateDependenciesDictionary removeObjectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
							}
						}
					}
					
					tDependenciesDictionary=[tOptionsStateDependenciesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_SELECTED_DICTIONARY];
					
					[ICInstallationTypeUtilities removeDependenciesToChoices:inChoicesUUID fromDependenciesDictionary:tDependenciesDictionary];
					
					if ([tDependenciesDictionary count]==0)
					{
						[tOptionsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
							
						[tOptionsDictionary removeObjectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
					}
				}
				else if (tOptionsState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_DEPENDENT)
				{
					tDependenciesDictionary=[tOptionsStateDependenciesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
					
					[ICInstallationTypeUtilities removeDependenciesToChoices:inChoicesUUID fromDependenciesDictionary:tDependenciesDictionary];
					
					if ([tDependenciesDictionary count]==0)
					{
						[tOptionsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_ALWAYS] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
							
						[tOptionsDictionary removeObjectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
					}
				}
			}
		}
	}
}

+ (void) removeDependenciesToChoices:(NSMutableArray *) inChoicesUUID fromHierarchy:(NSMutableArray *) inHierarchy
{
	if (inChoicesUUID!=nil && inHierarchy!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inHierarchy objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSMutableDictionary * tChoiceDictionary;
			
			while (tChoiceDictionary=[tEnumerator nextObject])
			{
				[ICInstallationTypeUtilities removeDependenciesToChoices:inChoicesUUID fromChoiceDictionary:tChoiceDictionary];
			}
		}
	}
}

+ (void) removePackagesWithUUID:(NSMutableArray *) inPackagesUUID fromHierarchy:(NSMutableArray *) inHierarchy
{
	if (inPackagesUUID!=nil && inHierarchy!=nil)
	{
		NSUInteger tCount,i;
		NSMutableArray * tRemovedChoicesUUID;
		
		tRemovedChoicesUUID=[NSMutableArray arrayWithCapacity:[inPackagesUUID count]];
		
		tCount=[inHierarchy count];
		
		for(i=tCount;i>0;i--)
		{
			NSMutableDictionary * tItemDictionary;
			BOOL tNeedsToBeRemoved;
			
			tItemDictionary=[inHierarchy objectAtIndex:i-1];
			
			tNeedsToBeRemoved=[ICInstallationTypeUtilities removePackagesWithUUID:inPackagesUUID fromHierarchyNode:tItemDictionary andUpdateRemovedChoices:tRemovedChoicesUUID];
			
			if (tNeedsToBeRemoved==YES)
			{
				[inHierarchy removeObjectAtIndex:i-1];
			}
		}
		
		// Update the dependencies
		
		if ([tRemovedChoicesUUID count]>0)
		{
			[ICInstallationTypeUtilities removeDependenciesToChoices:tRemovedChoicesUUID fromHierarchy:inHierarchy];
		}
	}
}

+ (void) updateInstallationTypeDictionary:(NSMutableDictionary *) inDictionary withNewPackages:(NSArray *) inPackages
{
	if (inDictionary!=nil)
	{
		NSDictionary * tHierarchiesDictionary;
		BOOL tNeedsToCreate=YES;
		
		tHierarchiesDictionary=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
		
		if (tHierarchiesDictionary!=nil)
		{
			NSEnumerator * tHierarchyEnumerator;
			
			tHierarchyEnumerator=[tHierarchiesDictionary objectEnumerator];
			
			if (tHierarchyEnumerator!=nil)
			{
				NSMutableDictionary * tHierarchyDictionary;
				
				while (tHierarchyDictionary=[tHierarchyEnumerator nextObject])
				{
					NSMutableArray * tListItemsArray;
					
					// Visible Hierarchy
					
					tListItemsArray=[tHierarchyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST];
					
					if (tListItemsArray!=nil)
					{
						NSEnumerator * tEnumerator;
					
						tNeedsToCreate=NO;
						
						// We need to add the remaining packages to the end
							
						tEnumerator=[inPackages objectEnumerator];
					
						if (tEnumerator!=nil)
						{
							NSDictionary * tPackageDictionary;
									
							while (tPackageDictionary=[tEnumerator nextObject])
							{
								NSMutableDictionary * tItemDictionary;
								
								tItemDictionary=[ICInstallationTypeUtilities defaultItemDictionaryWithPackageDictionary:tPackageDictionary];
							
								if (tItemDictionary!=nil)
								{
									[tListItemsArray addObject:tItemDictionary];
								}
							}
						}
					}
				}
			}
		}
		
		if (tNeedsToCreate==YES)
		{
			NSMutableDictionary * tNewDictionary;
					
			tNewDictionary=[ICInstallationTypeUtilities defaultInstallationTypeDictionaryWithPackages:inPackages];
			
			if (tNewDictionary!=nil)
			{
				[inDictionary addEntriesFromDictionary:tNewDictionary];
			}
		}
	}
}

+ (void) updateInstallationTypeDictionary:(NSMutableDictionary *) inDictionary withRemovedPackages:(NSArray *) inPackages
{
	if (inDictionary!=nil)
	{
		NSDictionary * tHierarchiesDictionary;
		
		tHierarchiesDictionary=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
		
		if (tHierarchiesDictionary!=nil)
		{
			NSEnumerator * tHierarchyEnumerator;
			
			tHierarchyEnumerator=[tHierarchiesDictionary objectEnumerator];
			
			if (tHierarchyEnumerator!=nil)
			{
				NSMutableDictionary * tHierarchyDictionary;
				
				while (tHierarchyDictionary=[tHierarchyEnumerator nextObject])
				{
					NSMutableArray * tRemovedMutablePackagesList;
					
					tRemovedMutablePackagesList=[inPackages mutableCopy];
					
					if (tRemovedMutablePackagesList!=nil)
					{
						NSMutableDictionary * tRemovedGroup;
						
						// REMOVED
						
						tRemovedGroup=[tHierarchyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED];
						
						if (tRemovedGroup!=nil)
						{
							NSUInteger i,tCount;
							
							tCount=[tRemovedMutablePackagesList count];
							
							for(i=tCount;i>0;i--)
							{
								NSDictionary * tPackageDictionary;
								NSString * tUUID;
								
								tPackageDictionary=[tRemovedMutablePackagesList objectAtIndex:i-1];
								
								tUUID=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID];
								
								if (tUUID!=nil)
								{
									if ([tRemovedGroup objectForKey:tUUID]!=nil)
									{
										[tRemovedGroup removeObjectForKey:tUUID];
										
										[tRemovedMutablePackagesList removeObjectAtIndex:i-1];
									}
								}
								else
								{
									// A COMPLETER (corruption)
								}
							}
						}
						
						// LIST
						
						if ([tRemovedMutablePackagesList count]>0)
						{
							NSMutableArray * tListArray;
							
							tListArray=[tHierarchyDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST];
						
							if (tListArray!=nil)
							{
								NSMutableArray * tPackagesUUID;
							
								tPackagesUUID=[ICInstallationTypeUtilities allUUIDSForPackages:inPackages];
							
								[ICInstallationTypeUtilities removePackagesWithUUID:tPackagesUUID fromHierarchy:tListArray];
							}
						}
						
						[tRemovedMutablePackagesList release];
					}
				}
			}
		}
	}
}

#pragma mark -

+ (void) notifyInstallationTypeDidChangeWithSender:(ICDocumentSecondaryController *) sender
{
	[[NSNotificationCenter defaultCenter] postNotificationName:ICPRESENTATION_NOTIFICATION_INSTALLATIONTYPE_SETTINGS_DID_CHANGE
														object:[sender document]
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:sender,ICPRESENTATION_NOTIFICATION_SENDER,nil]];
}

@end
