#import "ICInstallerPluginLocalizer.h"

#import "ICLanguageConverter.h"

//#import "PBInstallerLocator.h"

@implementation ICInstallerPluginLocalizer

+ (ICInstallerPluginLocalizer *) defaultLocalizer
{
    static ICInstallerPluginLocalizer * sInstallerPluginLocalizer=nil;
    
    if (sInstallerPluginLocalizer==nil)
    {
        sInstallerPluginLocalizer=[[ICInstallerPluginLocalizer alloc] init];
    }
    
    return sInstallerPluginLocalizer;
}

+ (NSString *) localizedString:(NSString *) inString forLanguage:(NSString *) inLanguage inExternalBundle:(NSBundle *) inBundle
{
    NSString * tString;
	
	tString=inString;
	
	if (inBundle!=nil)
	{
		NSString * tISOLanguage;
		NSString * tPath;
		NSDictionary * tResourceDictionary;
		NSMutableDictionary * tLocalizedDictionary;
			
		tISOLanguage=[[ICLanguageConverter defaultConverter] ISOFromEnglish:inLanguage];
		
		tLocalizedDictionary=[NSMutableDictionary dictionary];
			
		tPath=[inBundle pathForResource:@"Localizable"
								ofType:@"strings"
						inDirectory:nil
					forLocalization:inLanguage];
		
		if (tPath==nil)
		{
			tPath=[inBundle pathForResource:@"Localizable"
									ofType:@"strings"
							   inDirectory:nil
						   forLocalization:tISOLanguage];
		
			if (tPath==nil)
			{
				tPath=[inBundle pathForResource:@"Localizable"
										ofType:@"strings"];
			}
		}
		
		if (tPath!=nil)
		{
			tResourceDictionary=[NSDictionary dictionaryWithContentsOfFile:tPath];
		
			if (tResourceDictionary!=nil)
			{
				[tLocalizedDictionary addEntriesFromDictionary:tResourceDictionary];
			}
		}
		
		tPath=[inBundle pathForResource:@"InfoPlist"
								ofType:@"strings"
						inDirectory:nil
					forLocalization:inLanguage];
		
		if (tPath==nil)
		{
			tPath=[inBundle pathForResource:@"InfoPlist"
									ofType:@"strings"
							   inDirectory:nil
						   forLocalization:tISOLanguage];
						   
			if (tPath==nil)
			{
				tPath=[inBundle pathForResource:@"InfoPlist"
										ofType:@"strings"];
			}
		}
			
		if (tPath!=nil)
		{
			tResourceDictionary=[NSDictionary dictionaryWithContentsOfFile:tPath];
		
			if (tResourceDictionary!=nil)
			{
				[tLocalizedDictionary addEntriesFromDictionary:tResourceDictionary];
			}
		}
			
		
        if (tLocalizedDictionary!=nil)
        {
            tString=[tLocalizedDictionary objectForKey:inString];
        }
    }
    
    return tString;
}

#pragma mark -

- (id) init
{
    self=[super init];
    
    if (self!=nil)
    {
        NSString * tPath;
        
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5

        tPath=@"/System/Library/CoreServices/Installer.app";

#else

		tPath=@"/Applications/Utilities/Installer.app";
		 
#endif
        
        if (tPath!=nil)
        {
            mainBundle_=[[NSBundle bundleWithPath:tPath] retain];
            
            tPath=nil;
            
            if (mainBundle_!=nil)
            {
                NSString * tPluginPath;
                
                tPluginPath=[mainBundle_ builtInPlugInsPath];
                
                if (tPluginPath!=nil)
                {
                    NSFileManager * tFileManager;
                    NSArray * tArray;
                    NSEnumerator * tEnumerator;
                    NSString * tBundleName;
                    NSMutableDictionary * tMutableDictionary;
                    
                    tFileManager=[NSFileManager defaultManager];
                    
                    tArray=[tFileManager directoryContentsAtPath:tPluginPath];
                    
                    bundleDictionary_=[[NSMutableDictionary dictionary] retain];
                    
                    tMutableDictionary=[[NSMutableDictionary alloc] initWithObjectsAndKeys:mainBundle_,@"Bundle",
                                                                                           nil];
                                
                    [bundleDictionary_ setObject:tMutableDictionary forKey:@"Main"];
                    
                    [tMutableDictionary release];
                    
                    tMutableDictionary=[[NSMutableDictionary alloc] initWithObjectsAndKeys:[NSBundle mainBundle],@"Bundle",
                                                                                           nil];
                                
                    [bundleDictionary_ setObject:tMutableDictionary forKey:@"self"];
                    
                    [tMutableDictionary release];
                    
                    tEnumerator=[tArray objectEnumerator];
                    
                    while (tBundleName=[tEnumerator nextObject])
                    {
                        if ([[tBundleName pathExtension] isEqualToString:@"bundle"]==YES)
                        {
                            NSBundle * tBundle;
                            
                            tBundle=[NSBundle bundleWithPath:[tPluginPath stringByAppendingPathComponent:tBundleName]];
                            
                            if (tBundle!=nil)
                            {
                                tMutableDictionary=[[NSMutableDictionary alloc] initWithObjectsAndKeys:tBundle,@"Bundle",
                                                                                                       nil];
                                
                                [bundleDictionary_ setObject:tMutableDictionary forKey:[tBundleName stringByDeletingPathExtension]];
                                
                                [tMutableDictionary release];
                            }
                        }
                    }
                }
            }
        }
    }
    
    return self;
}

- (NSArray *) localizations
{
    if (mainBundle_!=nil)
    {
        return [mainBundle_ localizations];
    }
    
    return nil;
}



- (NSString *) localizedString:(NSString *) inString forLanguage:(NSString *) inLanguage inBundle:(NSString *) inBundleName
{
    NSString * tString=nil;
    
    NSMutableDictionary * tBundleDictionary;
    
    tBundleDictionary=[bundleDictionary_ objectForKey:inBundleName];
    
    if (tBundleDictionary!=nil)
    {
        NSMutableDictionary * tLocalizedDictionary;
        
        tLocalizedDictionary=[tBundleDictionary objectForKey:inLanguage];
		
		if (tLocalizedDictionary==nil)
        {
            // We need to build it
            
            NSBundle * tBundle;
            
            tBundle=[tBundleDictionary objectForKey:@"Bundle"];
            
            if (tBundle!=nil)
            {
                NSString * tISOLanguage;
				
				tISOLanguage=[[ICLanguageConverter defaultConverter] ISOFromEnglish:inLanguage];
				
				if ([inBundleName isEqualToString:@"self"]==NO)
                {
                    NSString * tPath;
                    NSDictionary * tResourceDictionary;
                    
                    tLocalizedDictionary=[NSMutableDictionary dictionary];
                    
                    tPath=[tBundle pathForResource:@"Localizable"
                                            ofType:@"strings"
                                    inDirectory:nil
                                forLocalization:inLanguage];
                    
					if (tPath==nil)
					{
						tPath=[tBundle pathForResource:@"Localizable"
												ofType:@"strings"
										   inDirectory:nil
									   forLocalization:tISOLanguage];
					}
					
                    if (tPath!=nil)
					{
						tResourceDictionary=[[NSDictionary alloc] initWithContentsOfFile:tPath];
                    
						if (tResourceDictionary!=nil)
						{
							[tLocalizedDictionary addEntriesFromDictionary:tResourceDictionary];
						
							[tResourceDictionary release];
						}
                    }
					
                    tPath=[tBundle pathForResource:@"InfoPlist"
                                            ofType:@"strings"
                                    inDirectory:nil
                                forLocalization:inLanguage];
                    
                    if (tPath==nil)
					{
						tPath=[tBundle pathForResource:@"InfoPlist"
												ofType:@"strings"
										   inDirectory:nil
									   forLocalization:tISOLanguage];
					}
					
					if (tPath!=nil)
					{
						tResourceDictionary=[[NSDictionary alloc] initWithContentsOfFile:tPath];
                    
						if (tResourceDictionary!=nil)
						{
							[tLocalizedDictionary addEntriesFromDictionary:tResourceDictionary];
						
							[tResourceDictionary release];
						}
                    }
					
                    [tBundleDictionary setObject:tLocalizedDictionary forKey:inLanguage];
                    
                    [bundleDictionary_ setObject:tBundleDictionary forKey:inBundleName];
                }
                else
                {
                    NSString * tPath;
                    NSDictionary * tResourceDictionary;
                    
                    tLocalizedDictionary=[NSMutableDictionary dictionary];
                    
                    tPath=[tBundle pathForResource:@"PresentationSimulator"
                                            ofType:@"strings"
                                    inDirectory:nil
                                forLocalization:inLanguage];
                    
                    if (tPath==nil)
					{
						tPath=[tBundle pathForResource:@"PresentationSimulator"
												ofType:@"strings"
										   inDirectory:nil
									   forLocalization:tISOLanguage];
					}
					
					if (tPath!=nil)
					{
						tResourceDictionary=[[NSDictionary alloc] initWithContentsOfFile:tPath];
                    
						if (tResourceDictionary!=nil)
						{
							[tLocalizedDictionary addEntriesFromDictionary:tResourceDictionary];
						
							[tResourceDictionary release];
						}
                    }
					
                    [tBundleDictionary setObject:tLocalizedDictionary forKey:inLanguage];
                    
                    [bundleDictionary_ setObject:tBundleDictionary forKey:inBundleName];
                }
            }
        }
        
        if (tLocalizedDictionary!=nil)
        {
            tString=[tLocalizedDictionary objectForKey:inString];
        }
    }
    
    return tString;
}

@end
