/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICIntroductionUtilities.h"

#import "ICPresentation+Notifications.h"

#import "ICLanguageConverter.h"

@implementation ICIntroductionUtilities

+ (NSMutableDictionary *) defaultIntroductionDictionary
{
	NSMutableDictionary * tDefaultDictionary=nil;
	
	tDefaultDictionary=[NSMutableDictionary dictionary];
		
	if (tDefaultDictionary!=nil)
	{
		[tDefaultDictionary setObject:[NSMutableArray array] forKey:ICDOCUMENT_LOCALIZATIONS];
	}
	
	return tDefaultDictionary;
}

+ (void) notifyIntroductionDidChangeWithSender:(ICDocumentSecondaryController *) sender
{
	[[NSNotificationCenter defaultCenter] postNotificationName:ICPRESENTATION_NOTIFICATION_INTRODUCTION_SETTINGS_DID_CHANGE
														object:[sender document]
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:sender,ICPRESENTATION_NOTIFICATION_SENDER,nil]];
}

+ (NSString *) defaultIntroductionDocumentPathForLanguage:(NSString *) inLanguage
{
	static NSBundle * sIntroductionBundle=nil;
	NSString * tDocumentPath=nil;
	
	if (sIntroductionBundle==nil)
	{
		NSString * tPath;
        
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5

        tPath=@"/System/Library/CoreServices/Installer.app";

#else

		tPath=@"/Applications/Utilities/Installer.app";
		 
#endif
        
        if (tPath!=nil)
        {
            NSBundle * tBundle;
			
			// Check that this is the one we're looking for
            
            tBundle=[NSBundle bundleWithPath:tPath];
            
            tPath=nil;
            
            if (tBundle!=nil)
            {
                tPath=[[tBundle builtInPlugInsPath] stringByAppendingPathComponent:@"Introduction.bundle"];
                
				if (tPath!=nil)
                {
                    sIntroductionBundle=[[NSBundle alloc] initWithPath:tPath];
                }
            }
        }

	}
	
	if (sIntroductionBundle!=nil)
	{
		tDocumentPath=[sIntroductionBundle pathForResource:@"Default"
													ofType:@"rtf"
											   inDirectory:nil
										   forLocalization:inLanguage];
										   
		if (tDocumentPath==nil)
		{
			inLanguage=[[ICLanguageConverter defaultConverter] ISOFromEnglish:inLanguage];
			
			tDocumentPath=[sIntroductionBundle pathForResource:@"Default"
													ofType:@"rtf"
											   inDirectory:nil
										   forLocalization:inLanguage];
										   
			if (tDocumentPath==nil)
			{
				tDocumentPath=[sIntroductionBundle pathForResource:@"Default"
																ofType:@"rtf"
														   inDirectory:nil
													   forLocalization:@"English"];
			}
		}
	}
	
	return tDocumentPath;
}

@end
