/*
Copyright (c) 2004-2007, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICLanguageConverter.h"

@implementation ICLanguageConverter

+ (ICLanguageConverter *) defaultConverter
{
    static ICLanguageConverter * sLanguageConverter=nil;
    
    if (sLanguageConverter==nil)
    {
        sLanguageConverter=[[ICLanguageConverter alloc] init];
    }
    
    return sLanguageConverter;
}

- (id) init
{
    self=[super init];
    
    if (self!=nil)
    {
        allEnglishNames_=[[NSMutableArray alloc] initWithObjects:@"Danish",
																 @"Dutch",
																 @"English",
																 @"Finnish",
																 @"French",
																 @"German",
																 @"Italian",
																 @"Japanese",
																 @"Korean",
																 @"Norwegian",
																 @"Polish",
																 @"Brazilian Portuguese",
																 @"Portuguese",
																 @"Russian",
																 @"Spanish",
																 @"Swedish",
																 @"Simplified Chinese",
																 @"Traditional Chinese",
																 nil];
		
		
		if (allEnglishNames_!=nil)
		{
			englishToISODictionary_=[[NSMutableDictionary alloc] initWithObjectsAndKeys:@"da",@"Danish",
																					    @"nl",@"Dutch",
																					    @"en",@"English",
																					    @"fi",@"Finnish",
																					    @"fr",@"French",
																					    @"de",@"German",
																					    @"it",@"Italian",
																					    @"ja",@"Japanese",
																					    @"ko",@"Korean",
																					    @"no",@"Norwegian",
																					    @"pl",@"Polish",
																					    @"pt",@"Brazilian Portuguese",
																					    @"pt_PT",@"Portuguese",
																					    @"ru",@"Russian",
																					    @"es",@"Spanish",
																						@"sv",@"Swedish",
																					    @"zh_CN",@"Simplified Chinese",
																					    @"zh_TW",@"Traditional Chinese",
																					    nil];
			
			ISOToEnglishDictionary_=[[NSMutableDictionary alloc] initWithObjectsAndKeys:@"Danish",@"da",
																						@"Dutch",@"nl",
																						@"English",@"en",
																						@"Finnish",@"fi",
																						@"French",@"fr",
																						@"German",@"de",
																						@"Italian",@"it",
																						@"Japanese",@"ja",
																						@"Korean",@"ko",
																						@"Norwegian",@"no",
																						@"Polish",@"pl",
																						@"Brazilian Portuguese",@"pt",
																						@"Portuguese",@"pt_PT",
																						@"Russian",@"ru",
																						@"Spanish",@"es",
																						@"Swedish",@"sv",
																						@"Simplified Chinese",@"zh_CN",
																						@"Traditional Chinese",@"zh_TW",
																						nil];
		}
		else
		{
			NSLog(@"[ICLanguageConverter init] Memory is low");
		}
    }
    
    return self;
}

- (void) dealloc
{
	[allEnglishNames_ release];

	[super dealloc];
}

#pragma mark -

- (NSArray *) allEnglishNames
{
	return [[allEnglishNames_ retain] autorelease];
}

- (NSString *) englishFromISO:(NSString *) inISOName
{
    NSString * tEnglishName;
	
	if (ISOToEnglishDictionary_==nil)
    {
        return inISOName;
    }
    
	tEnglishName=[ISOToEnglishDictionary_ objectForKey:inISOName];
	
	if (tEnglishName==nil)
	{
		tEnglishName=inISOName;
	}
	
    return tEnglishName;
}

- (NSString *) ISOFromEnglish:(NSString *) inEnglishName
{
    NSString * tISOName;
	
	if (englishToISODictionary_==nil)
    {
        return inEnglishName;
    }
    
    tISOName=[englishToISODictionary_ objectForKey:inEnglishName];
	
	if (tISOName==nil)
	{
		tISOName=inEnglishName;
	}
	
    return tISOName;
}

@end
