/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICLicenseUtilities.h"

#import "ICPresentation+Notifications.h"

#import "ICLanguageConverter.h"

#import "ICLocalizationUtilities.h"

#import "ICLicenseProvider.h"

#import "ICLicenseProvider+LocalizationUtilities.h"

#import "ICDocument.h"

@implementation ICLicenseUtilities

+ (NSMutableDictionary *) defaultLicenseDictionary
{
	NSMutableDictionary * tDefaultDictionary=nil;
	
	tDefaultDictionary=[NSMutableDictionary dictionary];
		
	if (tDefaultDictionary!=nil)
	{
		[tDefaultDictionary setObject:[NSNumber numberWithInt:ICLICENSE_MODE_CUSTOM] forKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_MODE];
		
		[tDefaultDictionary setObject:[NSMutableArray array] forKey:ICDOCUMENT_LOCALIZATIONS];
		
		[tDefaultDictionary setObject:[NSMutableDictionary dictionary] forKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_KEYWORDS];
	}
	
	return tDefaultDictionary;
}

+ (NSString *) licenseDocumentPathFromDictionary:(NSDictionary *) inDictionary forLanguage:(NSString *) inLanguage inDocument:(id) inDocument
{
	NSNumber * tNumber;
	NSString * tDocumentPath=nil;
	
	tNumber=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_MODE];
	
	if (tNumber!=nil)
	{
		NSArray * tLocalizations;
		
		switch([tNumber intValue])
		{
			case ICLICENSE_MODE_CUSTOM:
				
				tLocalizations=[inDictionary objectForKey:ICDOCUMENT_LOCALIZATIONS];
				
				if (tLocalizations!=nil)
				{
					NSMutableDictionary * tPathDictionary=nil;
					
					tPathDictionary=[ICLocalizationUtilities localizedValueForLanguage:inLanguage inLocalizations:tLocalizations];
					
					if (tPathDictionary!=nil)
					{
						NSString * tPath;
						NSInteger tPathType;
							
						tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
							
						tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
						
						if (tPath!=nil && [tPath length]>0)
						{
							if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
							{
								tDocumentPath=[tPath stringByAbsolutingWithPath:[inDocument folder]];
							}
							else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
							{
								tDocumentPath=[tPath stringByAbsolutingWithPath:[inDocument referenceFolderPath]];
							}
							else
							{
								tDocumentPath=tPath;
							}
						}
					}
				}
				
				break;
				
			case ICLICENSE_MODE_TEMPLATE:
				
				{
					NSString * tTemplateName;
					
					tTemplateName=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_NAME];
					
					if (tTemplateName!=nil)
					{
						return [[ICLicenseProvider defaultProvider] pathForLicenseWithName:tTemplateName language:inLanguage];
					}
				}
				
				break;
		}
	}
	
	return tDocumentPath;
}

+ (void) notifyLicenseDidChangeWithSender:(ICDocumentSecondaryController *) sender
{
	[[NSNotificationCenter defaultCenter] postNotificationName:ICPRESENTATION_NOTIFICATION_LICENSE_SETTINGS_DID_CHANGE
														object:[sender document]
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:sender,ICPRESENTATION_NOTIFICATION_SENDER,nil]];
}

@end
