/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICLocatorEditorController.h"

#import "NSDictionary+MutableDeepCopy.h"

#import "ICLocatorPluginsManager.h"

#import "ICPluginsManager+AppKit.h"

#import "ICDocumentWindow.h"

#import "ICLocatorPlugin+Constants.h"

@implementation ICLocatorEditorController

+ (ICLocatorEditorController *) newController
{
	ICLocatorEditorController * nController=nil;
    
    nController=[ICLocatorEditorController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICBundleLocatorEditor" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICBundleLocatorEditor"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return nController;
}

- (void) dealloc
{
	[dictionary_ release];
	
	[cachedValues_ release];
	
	[super dealloc];
}



- (void) awakeFromNib
{
	NSArray * tTitles;
	
	defaultContentWidth_=NSWidth([[IBwindow_ contentView] frame]);
	
	// Locator Popup
	
	[IBlocatorTypePopupButton_ removeAllItems];
		
	tTitles=[[ICLocatorPluginsManager defaultManager] allPluginsNameSorted];
	
	if (tTitles!=nil)
	{
		[IBlocatorTypePopupButton_ addItemsWithTitles:tTitles];
	}
}

#pragma mark -

- (id) delegate
{
	return delegate;
}

- (NSMutableDictionary *) dictionary
{
	return [[dictionary_ retain] autorelease];
}

#pragma mark -

- (void) updateMinMaxWindowSize
{
	NSSize tSize;
		
	tSize=[[IBwindow_ contentView] frame].size;
		
	if ([currentUIController_ windowCanBeResized]==YES)
	{
		NSRect tContentFrame;
		NSRect tLocatortFrame;
		
		tContentFrame=[[IBwindow_ contentView] frame];
		
		tLocatortFrame=[IBlocatorView_ frame];
		
		tContentFrame.size.height=NSHeight(tContentFrame)-NSHeight(tLocatortFrame)+[currentUIController_ minHeight];
		
		[IBwindow_ setContentMinSize:NSMakeSize(defaultContentWidth_, NSHeight(tContentFrame))];
		
		
		
		[IBwindow_ setContentMaxSize:NSMakeSize(2000.0f,2000.0f)];
	
		//NSLog(@"Min: %@ Max: %@",NSStringFromSize(NSMakeSize(defaultContentWidth_, NSHeight(tContentFrame))),NSStringFromSize(NSMakeSize(2000.0f,2000.0f)) );
	}
	else
	{
		tSize.width=defaultContentWidth_;
		
		[IBwindow_ setContentMinSize:tSize];
		
		[IBwindow_ setContentMaxSize:tSize];
		
		//NSLog(@"Min: %@ Max: %@",NSStringFromSize(tSize),NSStringFromSize(tSize) );
	}
}


- (void) switchIdentifierView
{
	NSString * tLocatorIdentifier;
	NSView * tPreviousKeyView;
	
	if (currentUIController_!=nil)
	{
		[currentUIController_ viewWillBeRemoved:self];
	
		[currentUIController_ release];
		
		currentUIController_=nil;
	}
	
	if (currentView_!=nil)
	{
		[currentView_ removeFromSuperview];
	
		currentView_=nil;
	}

	tLocatorIdentifier=[dictionary_ objectForKey:ICDOCUMENT_LOCATOR_IDENTIFIER];
	
	if (tLocatorIdentifier!=nil)
	{
		[IBlocatorTypeIcon_ setImage:[[ICLocatorPluginsManager defaultManager] iconForIdentifier:tLocatorIdentifier]];
		
		currentUIController_=[[ICLocatorPluginsManager defaultManager] createPluginUIControllerForIdentifier:tLocatorIdentifier];
		
		if (currentUIController_!=nil)
		{
			currentView_=[currentUIController_ view];
			
			if (currentView_!=nil)
			{
				NSRect tBounds;
				NSRect tCurrentViewBounds;
				NSRect tOldWindowFrame;
				NSRect tNewWindowFrame;
				NSRect tComputeRect;
				
				NSMutableDictionary * tValuesDictionary;
				
				tBounds=[IBlocatorView_ bounds];
				
				tCurrentViewBounds=[currentView_ bounds];
				
				if ([currentUIController_ windowCanBeResized]==YES)
				{
					NSString * tSizeString;
					NSString * tKey;
					
					tKey=[NSString stringWithFormat:@"%@.size",[dictionary_ objectForKey:ICDOCUMENT_LOCATOR_IDENTIFIER]];
					
					tSizeString=[[NSUserDefaults standardUserDefaults] objectForKey:tKey];
					
					if (tSizeString!=nil)
					{
						tCurrentViewBounds=NSRectFromString(tSizeString);
					}
					
					[IBwindow_ setShowsResizeIndicator:YES];
				}
				else
				{
					[IBwindow_ setShowsResizeIndicator:NO];
				}
				
				[IBwindow_ setMinSize:NSMakeSize(50.0f,50.0f)];
				
				
				tOldWindowFrame=[IBwindow_ frame];
				
				tComputeRect=NSMakeRect(0,0,NSWidth(tCurrentViewBounds)-NSWidth(tBounds),NSHeight(tCurrentViewBounds)-NSHeight(tBounds));
				
				tComputeRect=[NSWindow frameRectForContentRect:tComputeRect styleMask:NSBorderlessWindowMask];
				
				tNewWindowFrame.size=NSMakeSize(NSWidth(tOldWindowFrame)+NSWidth(tComputeRect),NSHeight(tOldWindowFrame)+NSHeight(tComputeRect));
				
				tNewWindowFrame.origin.x=_CGFloatFloor(NSMidX(tOldWindowFrame)-NSWidth(tNewWindowFrame)*0.5f);
				tNewWindowFrame.origin.y=NSMaxY(tOldWindowFrame)-NSHeight(tNewWindowFrame);
				
				
				
				
				tValuesDictionary=[dictionary_ objectForKey:ICDOCUMENT_LOCATOR_DICTIONARY];
				
				if (tValuesDictionary==nil)
				{
					tValuesDictionary=[currentUIController_ defaultDictionaryWithCommonValues:cachedCommonValues_];
					
					if (tValuesDictionary!=nil)
					{
						[dictionary_ setObject:tValuesDictionary forKey:ICDOCUMENT_LOCATOR_DICTIONARY];
					}
					else
					{
						// A COMPLETER
					}
				}
				
				[currentUIController_ initializeWithDictionary:tValuesDictionary forDocument:[delegate document]];
				
				[IBwindow_ setFrame:tNewWindowFrame display:YES animate:NO];
				
				tBounds=[IBlocatorView_ bounds];
				
				if ([currentUIController_ windowCanBeResized]==YES)
				{
					[currentView_ setAutoresizingMask:NSViewWidthSizable+NSViewHeightSizable];
				}
				
				[currentView_ setFrame:tBounds];
				
				[IBlocatorView_ addSubview:currentView_];
			}
		}
	}
	
	[self updateMinMaxWindowSize];
	
	tPreviousKeyView=[currentUIController_ previousKeyView];
	
	if (tPreviousKeyView!=nil)
	{
		[currentUIController_ setNextKeyView:tPreviousKeyView];
		
		[IBwindow_ makeFirstResponder:tPreviousKeyView];
	}
	else
	{
		[IBwindow_ makeFirstResponder:nil];
	}
}

- (void) prepareUI
{
	NSString * tLocatorIdentifier;
	
	// Set the Locator
	
	tLocatorIdentifier=[dictionary_ objectForKey:ICDOCUMENT_LOCATOR_IDENTIFIER];
	
	if (tLocatorIdentifier!=nil)
	{
		NSString * tLocalizedName;
	
		tLocalizedName=[[ICLocatorPluginsManager defaultManager] localizedPluginNameForIdentifier:tLocatorIdentifier];
		
		[IBlocatorTypePopupButton_ selectItemWithTitle:tLocalizedName];
		
		[self switchIdentifierView];
	}
	else
	{
		NSLog(@"[ICLocatorEditorController prepareUI]: Missing ICDOCUMENT_LOCATOR_IDENTIFIER value");
	}
	
	// Convert to Script button
	
	/*if ([tLocatorIdentifier isEqualToString:@""]==YES)	// A COMPLETER
	{
		[IBlocatorConvertToScriptButton_ setEnabled:NO];
	}
	else
	{
		[IBlocatorConvertToScriptButton_ setEnabled:YES];
	}*/
}

- (void) showEditorForWindow:(NSWindow *) inWindow withDictionary:(NSMutableDictionary *) inDictionary file:(ICFileTree *) inFileTree delegate:(id) inDelegate
{
	[cachedCommonValues_ release];
	
	cachedCommonValues_=nil;
	
	[dictionary_ release];
	
	dictionary_=nil;
	
	
	if (inWindow!=nil && inDictionary!=nil && inDelegate!=nil && inFileTree!=nil)
	{
		if (cachedValues_==nil)
		{
			cachedValues_=[[NSMutableDictionary alloc] initWithCapacity:[[[ICLocatorPluginsManager defaultManager] allPluginsIdentifier] count]];
		}
		
		dictionary_=[inDictionary mutableDeepCopy];
		
		delegate=inDelegate;
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(optionKeyDidChange:)
													 name:ICWINDOW_NOTIFICATION_OPTIONKEY_DID_CHANGE
												   object:IBwindow_];
		
		// Prepare the common values
		
		cachedCommonValues_=[[NSMutableDictionary alloc] initWithCapacity:2];
		
		if (cachedCommonValues_!=nil)
		{
			NSString * tPath;
			ICFileNode * tFileNode;
			NSBundle * tBundle;
			
			// Path
			
			tPath=[inFileTree filePath];
			
			if (tPath!=nil)
			{
				[cachedCommonValues_ setObject:tPath forKey:IC_LOCATOR_PATH];
			}
			else
			{
				// A COMPLETER
			}
			
			// Bundle Identifier
			
			tFileNode=FILENODE_DATA(inFileTree);
	
			tPath=[tFileNode path];
	
			tBundle=[NSBundle bundleWithPath:tPath];
		
			if (tBundle!=nil)
			{
				NSString * tBundleIdentifier;
				
				tBundleIdentifier=[tBundle bundleIdentifier];
			 
				if (tBundleIdentifier!=nil)
				{
					[cachedCommonValues_ setObject:tBundleIdentifier forKey:IC_LOCATOR_BUNDLE_IDENTIFIER];
				}
				else
				{
					// A COMPLETER
				}
			}
			else
			{
				// A COMPLETER
			}
		}
		else
		{
			// A COMPLETER
		}
		
		[self prepareUI];
		
		
		
		[NSApp beginSheet:IBwindow_
		   modalForWindow:inWindow
		    modalDelegate:self
		   didEndSelector:nil
		      contextInfo:NULL];
	}
}

- (IBAction) endDialog:(id) sender
{
	NSMutableDictionary * tLocatorDictionary;
	
	[IBwindow_ makeFirstResponder:nil];
	
	if ([currentUIController_ windowCanBeResized]==YES)
	{
		if (currentView_!=nil)
		{
			NSRect tBounds;
			NSString * tKey;
			
			tBounds=[currentView_ bounds];
			
			tKey=[NSString stringWithFormat:@"%@.size",[dictionary_ objectForKey:ICDOCUMENT_LOCATOR_IDENTIFIER]];
			
			[[NSUserDefaults standardUserDefaults] setObject:NSStringFromRect(tBounds) forKey:tKey];
			
			[currentView_ setAutoresizingMask:0];
		}
	}
	
	tLocatorDictionary=[currentUIController_ dictionary];

	if (tLocatorDictionary!=nil)
	{
		[dictionary_ setObject:tLocatorDictionary forKey:ICDOCUMENT_LOCATOR_DICTIONARY];
	
		if (delegate!=nil)
		{
			NSString * tLocatorName;
			
			tLocatorName=[dictionary_ objectForKey:ICDOCUMENT_LOCATOR_NAME];
			
			if (tLocatorName==nil)
			{
				[dictionary_ setObject:[IBlocatorTypePopupButton_ titleOfSelectedItem] forKey:@"SuggestedName"];
			}
			
			[delegate locatorEditionDidEnd:self returnCode:[sender tag]];
		}
	}
	else
	{
		NSLog(@"[ICLocatorEditorController endDialog:] A COMPLETER");
	}
	
	[cachedValues_ release];
	
	cachedValues_=nil;
	
	[cachedCommonValues_ release];
	
	cachedCommonValues_=nil;
	
	[NSApp endSheet:IBwindow_];
	
	[IBwindow_ orderOut:self];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self name:ICWINDOW_NOTIFICATION_OPTIONKEY_DID_CHANGE object:IBwindow_];
}

#pragma mark - 

- (IBAction) switchLocatorType:(id) sender
{
	NSString * tIdentifier;
	
	tIdentifier=[[ICLocatorPluginsManager defaultManager] identifierForLocalizedPluginName:[sender titleOfSelectedItem]];
	
	if ([tIdentifier isEqualToString:[dictionary_ objectForKey:ICDOCUMENT_LOCATOR_IDENTIFIER]]==NO)
	{
		NSMutableDictionary * tMutableDictionary;
		
		[IBwindow_ makeFirstResponder:nil];
		
		if (currentUIController_!=nil)
		{
			tMutableDictionary=[currentUIController_ dictionary];
		
			if (tMutableDictionary!=nil)
			{
				[cachedValues_ setObject:tMutableDictionary forKey:[dictionary_ objectForKey:ICDOCUMENT_LOCATOR_IDENTIFIER]];
			}
			
			if ([currentUIController_ windowCanBeResized]==YES)
			{
				if (currentView_!=nil)
				{
					NSRect tBounds;
					NSString * tKey;
					
					tBounds=[currentView_ bounds];
					
					tKey=[NSString stringWithFormat:@"%@.size",[dictionary_ objectForKey:ICDOCUMENT_LOCATOR_IDENTIFIER]];
					
					[[NSUserDefaults standardUserDefaults] setObject:NSStringFromRect(tBounds) forKey:tKey];
					
					[currentView_ setAutoresizingMask:0];
				}
			}
		}
		
		[dictionary_ setObject:tIdentifier forKey:ICDOCUMENT_LOCATOR_IDENTIFIER];
		
		tMutableDictionary=[cachedValues_ objectForKey:tIdentifier];
		
		if (tMutableDictionary!=nil)
		{
			[dictionary_ setObject:tMutableDictionary forKey:ICDOCUMENT_LOCATOR_DICTIONARY];
		}
		else
		{
			[dictionary_ removeObjectForKey:ICDOCUMENT_LOCATOR_DICTIONARY];
		}
		
		[self switchIdentifierView];
	}
}

- (IBAction) convertToScript:(id) sender
{
}

#pragma mark -

- (void) optionKeyDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tDictionary;
		
		tDictionary=[inNotification userInfo];
		
		if (tDictionary!=nil)
		{
			NSNumber * tNumber;
			
			tNumber=[tDictionary objectForKey:ICWINDOW_OPTIONKEY_STATE];
			
			if (tNumber!=nil)
			{
				[currentUIController_ optionKeyStateDidChange:[tNumber boolValue]];
			}
		}
	}
}

@end

@implementation NSObject (LocatorEdition)

- (void) locatorEditionDidEnd:(ICLocatorEditorController *) inEditor returnCode:(NSInteger) inReturnCode
{
}

@end
