/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICPackageController.h"

#import "ICDocument.h"

#import "ICPackageSettingsController.h"
#import "ICPackageFilesController.h"
#import "ICPackageScriptsController.h"

#import "ICPreferences+Constants.h"

@implementation ICPackageController

+ (ICPackageController *) packageController
{
	ICPackageController * nController=nil;
    
    nController=[ICPackageController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Package_Pane" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Package_Pane"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) awakeFromNib
{
	[super awakeFromNib];
}

#pragma mark -

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=inDictionary;
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
		NSInteger tSelectedSegment=0;
		int tPackageType=ICDOCUMENT_PACKAGE_TYPE_PROJECT;
		
		[dictionary_ retain];
		
		// Check the Package Type
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_TYPE];
		
		if (tNumber!=nil)
		{
			tPackageType=[tNumber intValue];
		}
		
		[IBsegmentedView_ setEnabled:(tPackageType==ICDOCUMENT_PACKAGE_TYPE_PROJECT) forSegment:1];
		
		[IBsegmentedView_ setEnabled:(tPackageType==ICDOCUMENT_PACKAGE_TYPE_PROJECT) forSegment:2];
		
		// Select the appropriate Segment
		
		tNumber=[document_ documentDefaultForKey:[NSString stringWithFormat:@"ui.package[%@].selected.segment",[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_UUID]]];
		
		if (tNumber!=nil)
		{
			tSelectedSegment=[tNumber intValue];
		}
		else
		{
			tSelectedSegment=[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_PACKAGE_PANE];
		
			if (tPackageType!=ICDOCUMENT_PACKAGE_TYPE_PROJECT &&
				(tSelectedSegment==1 || tSelectedSegment==2))
			{
				tSelectedSegment=0;
			}
		}
		
		[IBsegmentedView_ setSelectedSegment:tSelectedSegment];
		
		[self showView:IBsegmentedView_];
	}
}
	

- (ICSegmentedController *) segmentedControllerWithTag:(NSInteger) inTag
{
	[document_ setDocumentDefault:[NSNumber numberWithInt:inTag] forKey:[NSString stringWithFormat:@"ui.package[%@].selected.segment",[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_UUID]]];
	
	switch(inTag)
	{
		case ICPACKAGE_PANE_SETTINGS:
		
			return [ICPackageSettingsController packageSettingsController];
			
		case ICPACKAGE_PANE_FILES:
		
			return [ICPackageFilesController packageFilesController];
			
		case ICPACKAGE_PANE_SCRIPTS:
		
			return [ICPackageScriptsController packageScriptsController];
	
			break;
	}
	
	return nil;
}

@end
