#import "ICPackageFilesController.h"

#include <sys/types.h>
#include <sys/stat.h>

#import "ImageAndTextCell.h"

#import "ICPackageFileUtility.h"

#import "ICIconsUtilities.h"

#import "NSImage+Composition.h"

#import "ICPreferences+Constants.h"

#import "ICPackageFilesController+Constants.h"

// Inspectors

#import "ICPackageFilesInspectorEmptySelectionController.h"

#import "ICPackageFilesInspectorSelectionController.h"

#define SAFEFILENODE(n) 	((ICFileTree*)((n!=nil)?(n):(fileTree_)))

NSString * const ICFilePboardType=@"ICFilePboardType";

NSString * const ICFileExternalPboardType=@"ICFileExternalPboardType";

@implementation ICPackageFilesController

+ (ICPackageFilesController *) packageFilesController
{
	ICPackageFilesController * nController=nil;
    
    nController=[ICPackageFilesController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Package_Files" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Iceberg can't find the \"%@\".nib resource file.",@"No comment"),@"Package_Files"],NSLocalizedString(@"Iceberg will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) dealloc
{
	[internalDragArray_ release];
	
	[fileTree_ release];
	
	[emptySelectionInspectorController_ release];
	
	[selectionInspectorController_ release];
	
	[genericFolderIcon_ release];
	
	[genericDisabledFolderIcon_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{
	ImageAndTextCell * tImageTextCell;
    NSTableColumn * tTableColumn;
	NSSize tSize;
    id tPrototypeCell;
	NSFont * tSmallLabelFont;
	
	defaults_=[NSUserDefaults standardUserDefaults];
	
	cachedHighlightExcludedFiles_=[defaults_ boolForKey:ICPREFERENCES_FILES_HIGHLIGHT_EXCLUDED_FILES];
	
	tSmallLabelFont=[NSFont labelFontOfSize:[NSFont smallSystemFontSize]];
	
	// Cached Icons
	
	//	Generic Folder Icon
		
	genericFolderIcon_=[[ICIconsUtilities smallIconForType:NSFileTypeForHFSTypeCode(kGenericFolderIcon)] retain];
	
	genericDisabledFolderIcon_=[[genericFolderIcon_ imageWithAlpha:0.5f] retain];
	
	// Outline View
	
	fileNameFormatter_=[ICFileNameFormatter new];
	
	tTableColumn = [IBoutlineView_ tableColumnWithIdentifier: @"File"];
	
    tImageTextCell = [[ImageAndTextCell alloc] init];
	
	if (tImageTextCell!=nil)
	{
		[tImageTextCell setEditable:YES];
		
		[tImageTextCell setFormatter:fileNameFormatter_];
		
		[tImageTextCell setFont:tSmallLabelFont];
		
		[tTableColumn setDataCell:tImageTextCell];
		
		[tImageTextCell release];
    }
	
    tTableColumn = [IBoutlineView_ tableColumnWithIdentifier: @"Owner"];
	
    tPrototypeCell=[tTableColumn dataCell];
	
	if (tPrototypeCell!=nil)
	{
		[tPrototypeCell setFont:tSmallLabelFont];
    }
	
    tTableColumn = [IBoutlineView_ tableColumnWithIdentifier: @"Group"];
    
	tPrototypeCell=[tTableColumn dataCell];
	
	if (tPrototypeCell!=nil)
	{
		[tPrototypeCell setFont:tSmallLabelFont];
    }
    
    tTableColumn = [IBoutlineView_ tableColumnWithIdentifier: @"Permissions"];
    
	tPrototypeCell=[tTableColumn dataCell];
	
	if (tPrototypeCell!=nil)
	{
		[tPrototypeCell setFont:tSmallLabelFont];
    }
    
    [IBoutlineView_ setAutoresizesOutlineColumn:NO];
    
    tSize=[IBoutlineView_ intercellSpacing];
    
    tSize.height=2.0f;
    
    [IBoutlineView_ setIntercellSpacing:tSize];
	
	[IBoutlineView_ registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType,
                                                                      ICFilePboardType,
                                                                      ICFileExternalPboardType,
                                                                      nil]];
	
	directoryServicesManager_=[ICDirectoryServicesManager defaultManager];
	
	// Register for notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(highlightExcludedFilesDidChange:)
                                                 name:ICPREFERENCES_NOTIFICATION_FILES_HIGHLIGHT_EXCLUDED_FILES_DID_CHANGE
                                               object:nil];
}

#pragma mark -

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{	
	[fileTree_ release];
	
	fileTree_=nil;

	[super initializeWithDictionary:inDictionary forDocument:inDocument];

	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PACKAGE_FILES];
	
	if (dictionary_==nil)
	{
		// Build the default dictionary
		
		dictionary_=[ICPackageFileUtility defaultPackageFileDictionary];
		
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PACKAGE_FILES];
		}
	}
	
	if (dictionary_!=nil)
	{
		[cachedPackageUUID_ release];
		
		cachedPackageUUID_=[[inDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID] retain];
		
		[document_ updateFileBouncer];
		
		cachedFileBouncer_=[[document_ fileBouncer] retain];
		
		[dictionary_ retain];
		
		[self performSelector:@selector(delayedHierarchyLoad:) withObject:nil afterDelay:0.1f];
		
		// Register for Notifications
		
		[[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(folderNameShouldChange:)
                                                 name:ICPACKAGEFILESINSPECTOR_NOTIFICATION_FOLDERNAME_SHOULD_CHANGE
                                               object:document_];
	
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(fileAttributesDidChange:)
													 name:ICPACKAGEFILESINSPECTOR_NOTIFICATION_ATTRIBUTES_DID_CHANGE
												   object:document_];
	}
}

- (void) delayedHierarchyLoad:(id) sender
{
	NSString * tPath;
	
	// Default Install Location
		
	tPath=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_FILES_DEFAULT_INSTALL_LOCATION];
	
	if (tPath==nil)
	{
		tPath=@"/";
		
		[dictionary_ setObject:tPath forKey:ICDOCUMENT_PACKAGE_FILES_DEFAULT_INSTALL_LOCATION];
	}
	
	[IBdefaultLocationPath_ setStringValue:tPath];
	
	//[IBsetButton_ setEnabled
	
	// File Hierarchy
	
	fileTree_=[ICFileTree fileTreeWithDictionary:[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_FILES_FILE_HIERARCHY] projectPath:[document_ folder]];
	
	if (fileTree_!=nil)
	{
		[fileTree_ retain];
		
		[IBoutlineView_ reloadData];
		
		[self restoreDisclosedStates];
	}
	else
	{
	
		// A COMPLETER
	}
	
	defaultLocation_=[fileTree_ fileTreeAtPath:tPath];
	
	// A COMPLETER
	
	// Restore Selection
	
	[self restoreSelection];
	
	[self outlineViewSelectionDidChange:nil];
}

- (void) viewWillBeRemoved:(id) sender
{
	[NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayedHierarchyLoad:) object:nil];
	
	// A COMPLETER
	
	if (fileTree_!=nil)
	{
		// Archive Selection
	
		[self archiveSelection];
	}
}

- (IBAction) updateFiles:(id) sender
{
	// Save Options
	
	//	Default Install Location
	
	//[dictionary_ setObject:[IBdefaultLocationPath_ stringValue] forKey:ICDOCUMENT_PACKAGE_FILES_DEFAULT_INSTALL_LOCATION];
	
	// Save File Hierarchy if needed
	
	if (hierarchyChanged_==YES)
	{
		ICFileTree * tHierarchyTree_;
		NSDictionary * tDictionary;
		
		tHierarchyTree_=(ICFileTree *) [fileTree_ childAtIndex:0];
		
		if (tHierarchyTree_!=nil)
		{
			tDictionary=[tHierarchyTree_ dictionaryWithProjectAtPath:[document_ folder]];
			
			if (tDictionary!=nil)
			{
				[dictionary_ setObject:tDictionary forKey:ICDOCUMENT_PACKAGE_FILES_FILE_HIERARCHY];
			}
		}
		
		hierarchyChanged_=NO;
	}
	
	// Notify Document Change
			
	[self noteDocumentChange];
}

#pragma mark -

- (id) outlineView:(NSOutlineView *) inOutlineView child:(int)index ofItem:(id)inItem
{
    return [SAFEFILENODE(inItem) childAtIndex:index];
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView isItemExpandable:(id)inItem
{
    return ![FILENODE_DATA(inItem) isLeaf];
}

- (int) outlineView:(NSOutlineView *) inOutlineView numberOfChildrenOfItem:(id)inItem
{
    return [SAFEFILENODE(inItem) numberOfChildren];
}

- (id) outlineView:(NSOutlineView *) inOutlineView objectValueForTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
    id objectValue = nil;
    ICFileNode * tFileNode;
    
    tFileNode=FILENODE_DATA(inItem);
    
    if([[inTableColumn identifier] isEqualToString: @"File"])
    {
        objectValue = [tFileNode fileName];
	}
    else
    if([[inTableColumn identifier] isEqualToString: @"Owner"])
    {
        if ([tFileNode link]==YES)
        {
            tFileNode=FILENODE_DATA([inItem nodeParent]);
        }
        
       	objectValue=[directoryServicesManager_ userAccountForUID:[tFileNode uid]];
    }
    else
    if([[inTableColumn identifier] isEqualToString: @"Group"])
    {
        if ([tFileNode link]==YES)
        {
            tFileNode=FILENODE_DATA([inItem nodeParent]);
        }
        
        objectValue=[directoryServicesManager_ groupForGID:[tFileNode gid]];
    }
    else
    if([[inTableColumn identifier] isEqualToString: @"Permissions"])
    {
        if ([tFileNode link]==YES)
        {
            objectValue= [FILENODE_DATA([inItem nodeParent]) privilegesStringRepresentationForLink];
        }
        else
        {
            objectValue= [tFileNode privilegesStringRepresentation];
        }
    }
    
    return objectValue;
}

- (void) outlineView:(NSOutlineView *) inOutlineView setObjectValue:(id)object forTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
    if ([[inTableColumn identifier] isEqualToString: @"File"])
    {	
		ICFileTree * tTree;
		NSDictionary * tDictionary;
		
		if ([[FILENODE_DATA(inItem) path] compare:object options:NSCaseInsensitiveSearch]!=NSOrderedSame)
		{
			if ([((ICFileTree *)[inItem nodeParent]) containsTreeWithName:object]==YES)
            {
                NSBeep();
                
                //[inOutlineView selectRow:[inOutlineView rowForItem:item] byExtendingSelection:NO];
                
                [inOutlineView editColumn:[inOutlineView columnWithIdentifier:@"File"] row:[inOutlineView rowForItem:inItem] withEvent:nil select:YES];
				
				return;
            }
		}
		else
		{
			if ([[FILENODE_DATA(inItem) path] compare:object]==NSOrderedSame)
			{
				return;
			}
		}
		
		tTree=(ICFileTree *) inItem;
		
		// Post Notification
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tTree,ICPACKAGEFILESINSPECTOR_FILETREE,
															   object,ICPACKAGEFILESINSPECTOR_NEWNAME,
															   nil];
		
		if (tDictionary!=nil)
		{
			[[NSNotificationCenter defaultCenter] postNotificationName:ICPACKAGEFILESINSPECTOR_NOTIFICATION_FOLDERNAME_SHOULD_CHANGE
																object:[self document]
															  userInfo:tDictionary];
		}
    }
}


- (BOOL) outlineView:(NSOutlineView *) inOutlineView shouldEditTableColumn:(NSTableColumn *) inTableColumn item:(id)inItem
{
    BOOL tShouldEdit=NO;
	NSTableColumn * tTableColumn;
	
	tTableColumn = [inOutlineView tableColumnWithIdentifier: @"File"];
	
	if (inTableColumn==tTableColumn)
	{
		tShouldEdit=([FILENODE_DATA(inItem) type] == ICDOCUMENT_FILE_NEWFOLDER_NODE);
	}
	
	return tShouldEdit;
}

- (void) outlineView:(NSOutlineView *) inOutlineView willDisplayCell:(NSCell *)cell forTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{    
    if ([[inTableColumn identifier] isEqualToString: @"File"])
    {
		ICFileNode * tFileNode;
        BOOL tFather=YES;
        NSImage * tIcon=nil;
		NSString * tFileName;
		NSString * tFilePath;
		int tFileType;
		NSColor * tColor;
		NSMutableDictionary * tMutableAttributesDictionary;
		
        tFileNode=FILENODE_DATA(inItem);
        
		tFileType=[tFileNode type];
		
		tFileName=[tFileNode fileName];
		
		tColor=[NSColor blackColor];
		
        switch(tFileType)
        {
            case ICDOCUMENT_FILE_BASE_NODE:
			
                tFather=[inItem hasRealChildren];
				
				tFilePath=[tFileNode path];
				
				if (tFather==YES)
				{
					tIcon=[ICFileNode cachedIconForBaseNodeAtPath:tFilePath];
				}
				else
				{
					tIcon=[ICFileNode cachedIconForBaseNodeAtPath:[tFilePath stringByAppendingString:@"+Disabled"]];
				}
				
            case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				
				if (tIcon==nil)
				{
					if (tFather==YES)
					{
						tIcon=genericFolderIcon_;
					}
					else
					{
						tIcon=genericDisabledFolderIcon_;
					}
                }
				
				if (tIcon!=nil)
				{
					if (defaultLocation_==inItem)
					{
						tIcon=[ICFileNode setIconAsDefaultLocation:tIcon];
					}
					
					[(ImageAndTextCell*)cell setImage:tIcon];
				}
				
				break;
            case ICDOCUMENT_FILE_REALFILE_NODE:
                
                [tFileNode fileExistenceOnDiskChanged];
                
                if ([tFileNode existsOnDisk]==NO)
                {
                    tColor=[NSColor redColor];
                }
                
                [(ImageAndTextCell*)cell setImage: [FILENODE_DATA(inItem) icon]];
				
                break;
        }
		
		tMutableAttributesDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tColor,NSForegroundColorAttributeName,
																					   [cell font],NSFontAttributeName,
																					   nil];
		
		//if (tFileType==ICDOCUMENT_FILE_NEWFOLDER_NODE || tFileType==ICDOCUMENT_FILE_REALFILE_NODE)
		{
			if (cachedHighlightExcludedFiles_==YES && cachedFileBouncer_!=nil)
			{
				if ([inItem isItemExcludedByBouncer:cachedFileBouncer_]==YES)
				{
					[tMutableAttributesDictionary setObject:[NSNumber numberWithInt:NSUnderlineStyleSingle] forKey:NSStrikethroughStyleAttributeName];
				}
			}
		}
		
		[fileNameFormatter_ setCachedAttributes:tMutableAttributesDictionary];
			
		[(ImageAndTextCell*)cell setStringValue:tFileName];
	}
}

#pragma mark -

- (BOOL) outlineView:(NSOutlineView *) inOutlineView writeItems:(NSArray*) inItems toPasteboard:(NSPasteboard*)pboard
{
    int i,tCount;
    ICFileTree * tTree;
    NSMutableArray * tExternalDragArray;
	
    // We need to simplify the hierarchy and check no items is a base node
    
    tCount=[inItems count];
    
	[internalDragArray_ release];
	
    internalDragArray_=[[NSMutableArray alloc] initWithCapacity:tCount];
    
    tExternalDragArray=[NSMutableArray arrayWithCapacity:tCount];
    
    for(i=tCount-1;i>=0;i--)
    {
        int j;
		
		tTree=[inItems objectAtIndex:i];
        
        if ([FILENODE_DATA(tTree) type]==ICDOCUMENT_FILE_BASE_NODE)
        {
            return NO;
        }
        
        for(j=i-1;j>=0;j--)
        {
            if ([tTree isDescendantOfNode:[inItems objectAtIndex:j]]==YES)
            {
                break;
            }
        }
        
        if (j<0)
        {
            [internalDragArray_ insertObject:tTree atIndex:0];
            
            [tExternalDragArray insertObject:[tTree dictionaryWithProjectAtPath:nil] atIndex:0];
        }
    }
    
    [pboard declareTypes:[NSArray arrayWithObjects: ICFilePboardType,ICFileExternalPboardType,nil] owner:self];
    
    [pboard setPropertyList:tExternalDragArray forType:ICFileExternalPboardType];
	
    [pboard setPropertyList:[NSData data] forType:ICFilePboardType]; 
    
    return YES;
}

- (unsigned int) outlineView:(NSOutlineView*) inOutlineView validateDrop:(id <NSDraggingInfo>)info proposedItem:(id) inItem proposedChildIndex:(int)childIndex
{
    NSPasteboard * tPasteBoard;
    NSArray * tArray;
    int i,tCount;
    ICFileNode * tFileNode;
    
    tFileNode=FILENODE_DATA(inItem);
    
    tPasteBoard=[info draggingPasteboard];
    
    switch(childIndex)
    {
        case NSOutlineViewDropOnItemIndex:
            break;
        default:
            if ([tFileNode type]==ICDOCUMENT_FILE_ROOT_NODE)
            {
                return NSDragOperationNone;
            }
            
            if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject: NSFilenamesPboardType]]!=nil)
            {
                NSString * tPath;
                ICFileTree * tChildTree;
                NSArray * tChildren;
                NSString * tName;
                int k,tChildrenCount;
                BOOL find=NO;
                
                tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
                        
                tCount=[tArray count];
                        
                for(i=0;i<tCount;i++)
                {
                    tPath=[tArray objectAtIndex:i];
                            
                    if ([inItem containsTreeWithName:[tPath lastPathComponent]]==YES)
                    {
                        return NSDragOperationNone;
                    }
                }
                
                tPath=[tArray objectAtIndex:0];
                
                tName=[tPath lastPathComponent];
                
                tChildren=[inItem children];
                
                tChildrenCount=[tChildren count];
                
                for(k=0;k<tChildrenCount;k++)
                {
                    tChildTree=[tChildren objectAtIndex:k];
                    
                    if ([tName compare:[FILENODE_DATA(tChildTree) fileName] options:NSCaseInsensitiveSearch]!=NSOrderedDescending)
                    {
                        [inOutlineView setDropItem:inItem dropChildIndex:k];
                        find=YES;
                        
                        break;
                    }
                }
                
                if (find==NO)
                {
                    [inOutlineView setDropItem:inItem dropChildIndex:k];
                }
                
                return NSDragOperationCopy;//[info draggingSourceOperationMask];
            }
            else
            {
                ICFileTree * tFileTree;
                NSString * tName=nil;
                
                if ([info draggingSource]==IBoutlineView_)
                {
					if ([inItem isDescendantOfNodeInArray:internalDragArray_]==YES)
                    {
                        return NSDragOperationNone;
                    }
                    
                    // We need to check the name and eventually switch the drop location
                    
                    tCount=[internalDragArray_ count];
                    
                    for(i=0;i<tCount;i++)
                    {
                        tFileTree=[internalDragArray_ objectAtIndex:i];
                        
                        tName=[FILENODE_DATA(tFileTree) fileName];
                        
                        if ([inItem containsTreeWithName:tName]==YES)
                        {
                            return NSDragOperationNone;
                        }
                    }
                    
                    tFileTree=[internalDragArray_ objectAtIndex:0];
                    
                    tName=[FILENODE_DATA(tFileTree) fileName];
                }
                else
                {
                    // External Drag
                    
                    NSArray * tExternalArray;
                    NSDictionary * tDictionary;
                    
                    tExternalArray=[[info draggingPasteboard] propertyListForType:ICFileExternalPboardType];
                    
                    // We need to check the name and eventually switch the drop location
                    
                    tCount=[tExternalArray count];
                    
                    for(i=0;i<tCount;i++)
                    {
                        tDictionary=[tExternalArray objectAtIndex:i];
                        
                        if (tDictionary!=nil)
                        {
                            tName=[tDictionary objectForKey:@"Path"];
                        
                            if (tName==nil)
                            {
                                return NSDragOperationNone;
                            }
                            else
                            {
                                tName=[tName lastPathComponent];
                                
                                if ([inItem containsTreeWithName:tName]==YES)
                                {
                                    return NSDragOperationNone;
                                }
                            }
                        }
                    }
                    
                    tDictionary=[tExternalArray objectAtIndex:0];
                    
                    tName=[[tDictionary objectForKey:@"Path"] lastPathComponent];
                }
                    
                // Find the appropriate drop location based on the first name of the dropped objects
                
                {
                    ICFileTree * tChildTree;
                    NSArray * tChildren;
                    int k,tChildrenCount;
                    BOOL find=NO;
                
                    tChildren=[inItem children];
                
                    tChildrenCount=[tChildren count];
                    
                    for(k=0;k<tChildrenCount;k++)
                    {
                        tChildTree=[tChildren objectAtIndex:k];
                        
                        if ([tName compare:[FILENODE_DATA(tChildTree) fileName] options:NSCaseInsensitiveSearch]!=NSOrderedDescending)
                        {
                            [inOutlineView setDropItem:inItem dropChildIndex:k];
							
                            find=NO;
                            
                            break;
                        }
                    }
                    
                    if (find==NO)
                    {
                        [inOutlineView setDropItem:inItem dropChildIndex:k];
                    }
                }
                
                return NSDragOperationGeneric;
            }
            break;
    }
    
    return NSDragOperationNone;
}

- (BOOL)outlineView:(NSOutlineView*) inOutlineView acceptDrop:(id <NSDraggingInfo>)info item:(id)targetItem childIndex:(int)childIndex
{
    ICFileTree * tFileTree;
    NSPasteboard * tPasteBoard;
    NSMutableArray * tNewSelectionArray;
    int k,tNewCount;
    
    tPasteBoard=[info draggingPasteboard];
    
    tNewSelectionArray=[NSMutableArray array];
    
    if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject: NSFilenamesPboardType]]!=nil)
    {
        // Drag & Drop of a real item
        
        NSArray * tArray;
        BOOL tShowCustomizationDialog=YES;
		id tObject;
		
        parentFileTree_=targetItem;
        
        tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
		tObject=[defaults_ objectForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
	
		if (tObject!=nil)
		{
			tShowCustomizationDialog=[defaults_ boolForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
		}
		
		if (tShowCustomizationDialog==YES)
		{
			[self beginFileSheet:tArray];
		}
		else
		{
			BOOL tDefaultKeepPermissionMode;
			int tDefaultReferenceStyle;
			
			tDefaultKeepPermissionMode=[defaults_ boolForKey:ICPREFERENCES_FILES_KEEP_PERMISSION];
			
			tDefaultReferenceStyle=[defaults_ integerForKey:ICPREFERENCES_FILES_DEFAULT_REFERENCE_STYLE];
			
			[self _addFiles:tArray keepOwnerAndGroup:tDefaultKeepPermissionMode referenceStyle:tDefaultReferenceStyle];
		}
        
        return YES;
    }
    else
    {
        int i,tCount;
		
		if ([info draggingSource]==IBoutlineView_)
        {
            NSMutableDictionary * tDisclosedDictionary;
			NSArray * tCoverageNodesArray;
			
			// Internal Drag, we need to move
            
			tDisclosedDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICFILE_HIERARCHY_DISCLOSED_STATE,cachedPackageUUID_]];
			
            // Find the coverage tree
			
			tCoverageNodesArray=[TreeNode minimumNodeCoverFromNodesInArray:internalDragArray_];
			
			if (tCoverageNodesArray!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tCoverageNodesArray objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSNumber * tNumber;
					
					while (tFileTree=[tEnumerator nextObject])
					{
						NSMutableArray * tChildFoldersArray;
		
						// Removed item and descendants from disclosed state dictionary (because the key will change)
						
						tChildFoldersArray=[tFileTree descendantFoldersIncludingSelf:YES];
						
						if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
						{
							unsigned int i,tCount;
							
							tCount=[tChildFoldersArray count];
							
							for(i=tCount;i>0;i--)
							{
								ICFileTree * tGroupFileTree;
								NSString * tPath;
								
								tGroupFileTree=[tChildFoldersArray objectAtIndex:(i-1)];
								
								tPath=[tGroupFileTree filePath];
								
								if (tPath!=nil)
								{
									BOOL tRemoved=NO;
									
									tNumber=[tDisclosedDictionary objectForKey:tPath];
									
									if (tNumber==nil)
									{
										tRemoved=YES;
									}
									else
									{
										if ([tNumber boolValue]==NO)
										{
											tRemoved=YES;
										}
										
										[tDisclosedDictionary removeObjectForKey:tPath];
									}
									
									if (tRemoved==YES)
									{
										[tChildFoldersArray removeObject:tGroupFileTree];
									}
								}
							}
						}
						
						[tFileTree retain];
						
						[tFileTree removeFromParent];
						
						[targetItem insertSortedChild:tFileTree];
						
						[tFileTree release];
						
						[tNewSelectionArray addObject:tFileTree];
						
						// Put back the item and descendant folders disclosed state
			
						if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
						{
							NSEnumerator * tEnumerator;
							
							tEnumerator=[tChildFoldersArray objectEnumerator];
							
							if (tEnumerator!=nil)
							{
								ICFileTree * tGroupFileTree;
								
								while (tGroupFileTree=[tEnumerator nextObject])
								{
									NSString * tPath;
									
									tPath=[tGroupFileTree filePath];
								
									if (tPath!=nil)
									{
										[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tPath];
									}
								}
							}
						}

					}
				}
			}
        }
        else
        {
            // External Drag, we copy
            
            NSArray * tExternalArray;
		
            tExternalArray=[tPasteBoard propertyListForType:ICFileExternalPboardType];
            
            tCount=[tExternalArray count];
            
            for(i=0;i<tCount;i++)
            {
                NSDictionary * tDictionary;
                
                tDictionary=[tExternalArray objectAtIndex:i];
                
                tFileTree=[ICFileTree fileNodeWithDictionary:tDictionary projectPath:nil];
                
                [targetItem insertSortedChild:tFileTree];
                
                [tNewSelectionArray addObject:tFileTree];
            }
        }
    }
    
    [IBoutlineView_ deselectAll:nil];
    
    [IBoutlineView_ reloadData];
    
    tNewCount=[tNewSelectionArray count];
    
    for(k=0;k<tNewCount;k++)
    {
        [IBoutlineView_ selectRow:[IBoutlineView_ rowForItem:[tNewSelectionArray objectAtIndex:k]]
             byExtendingSelection:YES];
    }
    
    hierarchyChanged_=YES;
	
	[self updateFiles:IBoutlineView_];
    
    return YES;
}

#pragma mark -

- (void) expandItem:(ICFileTree *) inFileTree ifKeyOfDictionary:(NSMutableDictionary *) inDictionary
{
	if (inFileTree!=nil && [inDictionary count]>0)
	{
		if ([FILENODE_DATA(inFileTree) isLeaf]==NO)
		{
			NSString * tFilePath;
			NSArray * tChildren;
			NSEnumerator * tEnumerator;
			NSNumber * tNumber;
			
			
			tFilePath=[inFileTree filePath];
			
			[IBoutlineView_ expandItem:inFileTree];
			
			// Check children
			
			tChildren=[inFileTree children];
				
			tEnumerator=[tChildren objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				ICFileTree * tFileTree;
				
				while (tFileTree=[tEnumerator nextObject])
				{
					[self expandItem:tFileTree ifKeyOfDictionary:inDictionary];
				}
			}
			
			tNumber=[inDictionary objectForKey:tFilePath];
			
			if (tNumber==nil || [tNumber boolValue]==NO)
			{
				[IBoutlineView_ collapseItem:inFileTree];
			}
		}
	}
}

- (BOOL) defaultRestoreForItem:(id) inItem
{
	if (inItem!=nil)
	{
		ICFileNode * tFileNode;
		int tFileType;
		
		tFileNode=FILENODE_DATA(inItem);
        
		tFileType=[tFileNode type];
		
		if (tFileType==ICDOCUMENT_FILE_BASE_NODE || tFileType==ICDOCUMENT_FILE_ROOT_NODE)
		{
			NSArray * tChildren;
			BOOL tRealItemFound=NO;
			
			if (tFileType==ICDOCUMENT_FILE_BASE_NODE)
			{
				if ([inItem hasRealChildren]==NO)
				{
					return NO;
				}
				
				tRealItemFound=YES;
				
				// Expand the item
					
				[IBoutlineView_ expandItem:inItem];
			}	
					
			// Inspect the children
					
			tChildren=[inItem children];
					
			if (tChildren!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tChildren objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					id tChild;
					
					while (tChild=[tEnumerator nextObject])
					{
						if ([self defaultRestoreForItem:tChild]==YES)
						{
							tRealItemFound=YES;
						}
					}
				}
			}
			
			return tRealItemFound;
		}
	}

	return NO;
}

- (void) restoreDisclosedStates
{
	NSDictionary * tDictionary;
	
	tDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICFILE_HIERARCHY_DISCLOSED_STATE,cachedPackageUUID_]];
			
	if (tDictionary!=nil)
	{
		NSMutableDictionary * tDictionaryCopy;
		
		tDictionaryCopy=[tDictionary mutableCopy];
		
		if (tDictionaryCopy!=nil)
		{
			[self expandItem:((ICFileTree *) [fileTree_ firstChild]) ifKeyOfDictionary:tDictionaryCopy];
		
			[tDictionaryCopy release];
		}
	}
	else
	{
		if ([self defaultRestoreForItem:fileTree_]==NO)
		{
			ICFileTree * tFileTree;
					
			// Expand '/'
			
			[IBoutlineView_ expandItem:[IBoutlineView_ itemAtRow:0]];
		
			// Expand the /Applications folder
			
			tFileTree=[fileTree_ fileTreeAtPath:@"/Applications"];
			
			if (tFileTree!=nil)
			{
				[IBoutlineView_ expandItem:tFileTree];
			}
			
			// Expand the /Library folder
			
			tFileTree=[fileTree_ fileTreeAtPath:@"/Library"];
			
			if (tFileTree!=nil)
			{
				[IBoutlineView_ expandItem:tFileTree];
			}
		}
	}
}

#pragma mark -

- (void) archiveSelection
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBoutlineView_ selectedRowIndexes];
	
	if (tIndexSet!=nil)
	{
		NSMutableArray * tMutableArray;
		
		tMutableArray=[NSMutableArray array];
		
		if (tMutableArray!=nil)
		{
			unsigned int tIndex;
			
			tIndex=[tIndexSet firstIndex];
			
			while (tIndex!=NSNotFound)
			{
				ICFileTree * tFileTree;
					
				tFileTree=[IBoutlineView_ itemAtRow:tIndex];
				
				if (tFileTree!=nil)
				{
					NSString * tPath;
					
					tPath=[tFileTree filePath];
					
					if (tPath!=nil)
					{
						[tMutableArray addObject:tPath];
					}
				}
				
			
				tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
			}
			
			[document_ setDocumentDefault:tMutableArray forKey:[NSString stringWithFormat:ICFILE_HIERARCHY_SELECTION,cachedPackageUUID_]];
		}
		else
		{
			NSLog(@"[ICPackageFilesController archiveSelection]: Low Memory");
			
			[document_ removeDocumentDefaultForKey:[NSString stringWithFormat:ICFILE_HIERARCHY_SELECTION,cachedPackageUUID_]];
		}
	}
}

- (void) restoreSelection
{
	NSArray * tArray;
	
	tArray=[document_ documentDefaultForKey:[NSString stringWithFormat:ICFILE_HIERARCHY_SELECTION,cachedPackageUUID_]];
			
	if (tArray!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[tArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSMutableIndexSet * tMutableIndexSet;
			
			tMutableIndexSet=[NSMutableIndexSet indexSet];
			
			if (tMutableIndexSet!=nil)
			{
				NSString * tPath;
			
				while (tPath=[tEnumerator nextObject])
				{
					ICFileTree * tFileTree;
					
					tFileTree=[fileTree_ fileTreeAtPath:tPath];
				
					if (tFileTree!=nil)
					{
						int tRow;
						
						tRow=[IBoutlineView_ rowForItem:tFileTree];
						
						if (tRow!=-1)
						{
							[tMutableIndexSet addIndex:tRow];
						}
					}
				}
				
				[IBoutlineView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:NO];
			}
		}
	}
}

#pragma mark -

- (BOOL)validateMenuItem:(NSMenuItem *) inMenuItem
{    
    ICFileTree * tFileTree;
    int tSelectedCount;
    SEL tAction;
	
	tAction=[inMenuItem action];
	
	if (tAction==@selector(expandAll:))
    {
        // A COMPLETER
		
		return YES;
    }
    
    tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
    
	if (tAction==@selector(addFiles:))
	{
		return (tSelectedCount<=1);
	}
	
    if (tSelectedCount>0)
	{
		NSIndexSet * tIndexSet;
				
		tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
		if (tIndexSet!=nil)
		{
			unsigned int tIndex;
			
			if (tAction==@selector(removeFile:))
			{
				tIndex=[tIndexSet firstIndex];
				
				while (tIndex!=NSNotFound)
				{
					ICFileTree * tFileTree;
					
					tFileTree=(ICFileTree *) [IBoutlineView_ itemAtRow:tIndex];
				
					if ([FILENODE_DATA(tFileTree) type]==ICDOCUMENT_FILE_BASE_NODE)
					{
						return NO;
					}
					
					tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
				}
				
				return YES;
			}
			else if (tAction==@selector(revealInFinder:))
			{
				tIndex=[tIndexSet firstIndex];
				
				while (tIndex!=NSNotFound)
				{
					ICFileTree * tFileTree;
					
					tFileTree=(ICFileTree *) [IBoutlineView_ itemAtRow:tIndex];
				
					if ([FILENODE_DATA(tFileTree) type]!=ICDOCUMENT_FILE_REALFILE_NODE)
					{
						return NO;
					}
					
					tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
				}
				
				return YES;
			}
		}
	}
	
	if (tSelectedCount==1)
    {
        int tSelectedRow;
        
        tSelectedRow=[IBoutlineView_ selectedRow];
    
        if (tSelectedRow>=0)
        {
            int tType;
			
			tFileTree=[IBoutlineView_ itemAtRow:tSelectedRow];
			
			tType=[FILENODE_DATA(tFileTree) type];
			
			if (tAction==@selector(newFolder:))
			{
				return YES;
			}
			else
			{
				if (tAction==@selector(setAsDefaultLocation:) &&
					(tType==ICDOCUMENT_FILE_BASE_NODE || tType==ICDOCUMENT_FILE_NEWFOLDER_NODE))
				{
					if (tFileTree!=defaultLocation_)
					{
						return YES;
					}
				}
				else if (tType==ICDOCUMENT_FILE_REALFILE_NODE)
				{
					
					if ((tAction==@selector(expand:)			||
						tAction==@selector(expandOneLevel:))	&& 
						[tFileTree numberOfChildren]==0)
					{
						NSString * tPath;
						NSFileManager * tFileManager;
						BOOL isDirectory;
						
						tPath=[FILENODE_DATA(tFileTree) path];
						
						tFileManager=[NSFileManager defaultManager];
						
						if ([tFileManager fileExistsAtPath:tPath isDirectory:&isDirectory]==YES && isDirectory==YES)
						{
							NSDictionary * tFileAttributes;
							NSString * tString;
							
							tFileAttributes=[tFileManager fileAttributesAtPath:tPath traverseLink:NO];
							
							tString=[tFileAttributes objectForKey:NSFileType];
							
							if (tString!=nil)
							{
								return ([tString isEqualToString:NSFileTypeDirectory]);
							}
						}
					}
					else if (tAction==@selector(contract:))
					{
						if ([tFileTree numberOfChildren]!=0)
						{
							return YES;
						}
					}
				}
            }
			
            return NO;
        }
    }
    else
    {
        return NO;
    }
    
    return YES;
}

- (IBAction) newFolder:(id) sender
{
    int tRow;
    id tItem;
    ICFileTree * tParentTree=nil;
    ICFileNode * tParentNode;
    ICFileTree * tNewFolderTree=nil;
    
    tRow=[IBoutlineView_ selectedRow];

    tItem=[IBoutlineView_ itemAtRow:tRow];
    
    switch([FILENODE_DATA(tItem) type])
    {
        case ICDOCUMENT_FILE_BASE_NODE:
        case ICDOCUMENT_FILE_NEWFOLDER_NODE:
            tParentTree=(ICFileTree *) tItem;
            break;
        case ICDOCUMENT_FILE_REALFILE_NODE:
            tParentTree=(ICFileTree *) [tItem nodeParent];
            break;
    }
    
    if (tParentTree!=nil)
    {
        if ([IBoutlineView_ isItemExpanded:tParentTree]==NO)
        {
            [IBoutlineView_ expandItem:tParentTree];
        }
    
        tParentNode=FILENODE_DATA(tParentTree);
            
        
        
        tNewFolderTree=[[ICFileTree alloc] initWithData:[ICFileNode newFolderFileNodeWithName:[ICFileTree uniqueNameWithParentFileTree:tParentTree]
                                                                                    icon:nil
                                                                                    user:[tParentNode uid]
                                                                                    group:[tParentNode gid]
                                                                                privileges:([tParentNode privileges] & ACCESSPERMS)]
                                                parent:nil
                                                children:[NSArray array]];
        
        if (tNewFolderTree!=nil)
        {
            [tParentTree insertSortedChild:tNewFolderTree];
            
            [tNewFolderTree release];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadData];
            
            tRow=[IBoutlineView_ rowForItem:tNewFolderTree];
            
            [IBoutlineView_ scrollRowToVisible:tRow];
            
            [IBoutlineView_ selectRow:tRow byExtendingSelection:NO];
            
            [IBoutlineView_ editColumn:[IBoutlineView_ columnWithIdentifier:@"File"] row:tRow withEvent:nil select:YES];
            
            [self updateFiles:IBoutlineView_];
        }
    }
}

- (IBAction) addFiles:(id)sender
{
    NSOpenPanel * tOpenPanel;
    int tSelectedRow;
    
    tSelectedRow=[IBoutlineView_ selectedRow];
    
    if (tSelectedRow==-1)
	{
		parentFileTree_=[fileTree_ fileTreeAtPath:@"/"];
	}
	else
	{
		parentFileTree_=[IBoutlineView_ itemAtRow:tSelectedRow];
	
		if ([FILENODE_DATA(parentFileTree_) type]==ICDOCUMENT_FILE_REALFILE_NODE)
		{
			parentFileTree_=(ICFileTree *) [parentFileTree_ nodeParent];
		}
    }
	
    tOpenPanel=[NSOpenPanel openPanel];
    
	[[tOpenPanel _navView] setShowsHiddenFiles:YES];
	
    [tOpenPanel setResolvesAliases:NO];
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:YES];
    [tOpenPanel setAllowsMultipleSelection:YES];
    [tOpenPanel setTreatsFilePackagesAsDirectories:YES];
    [tOpenPanel setDelegate:self];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Add...",@"No comment")];
    
    [tOpenPanel beginSheetForDirectory:nil
                                  file:nil
                                 types:nil
                        modalForWindow:[document_ windowForSheet]
                         modalDelegate:self
                        didEndSelector:@selector(addFilesPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (BOOL) panel:(id)sender shouldShowFilename:(NSString *) inFileName
{
    // A AMELIORER (We need to be able to add items from these places)
	
	BOOL isDirectory;
	
	if ([[NSFileManager defaultManager] fileExistsAtPath:inFileName isDirectory:&isDirectory]==YES)
	{
		if (isDirectory==NO)
		{
			return ![parentFileTree_ containsTreeWithName:[inFileName lastPathComponent]];
		}
	}
	
	return YES;
}

- (void) addFilesPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        NSArray * tFileNames;
		
		tFileNames=[sheet filenames];
		
		if (tFileNames!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[tFileNames objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSString * tPath;
				BOOL tShowCustomizationDialog=YES;
				id tObject;
		
				// A AMELIORER
				
				while (tPath=[tEnumerator nextObject])
				{
					if ([parentFileTree_ containsTreeWithName:[tPath lastPathComponent]]==YES)
					{
						NSBeep();
						
						return;
					}
				}
				
				tObject=[defaults_ objectForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
	
				if (tObject!=nil)
				{
					tShowCustomizationDialog=[defaults_ boolForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
				}
				
				if (tShowCustomizationDialog==YES)
				{
					[self performSelector:@selector(beginFileSheet:) withObject:tFileNames afterDelay:0.f];
				}
				else
				{
					BOOL tDefaultKeepPermissionMode;
					int tDefaultReferenceStyle;
					
					tDefaultKeepPermissionMode=[defaults_ boolForKey:ICPREFERENCES_FILES_KEEP_PERMISSION];
					
					tDefaultReferenceStyle=[defaults_ integerForKey:ICPREFERENCES_FILES_DEFAULT_REFERENCE_STYLE];
					
					[self _addFiles:tFileNames keepOwnerAndGroup:tDefaultKeepPermissionMode referenceStyle:tDefaultReferenceStyle];
				}
			}
		}
    }
}

- (void) beginFileSheet:(NSArray *) inFiles
{
    int tDefaultReferenceStyle;
    BOOL tDefaultKeepPermissionMode;
	
	tDefaultKeepPermissionMode=[defaults_ boolForKey:ICPREFERENCES_FILES_KEEP_PERMISSION];
	
    tDefaultReferenceStyle=[defaults_ integerForKey:ICPREFERENCES_FILES_DEFAULT_REFERENCE_STYLE];
    
    inFiles=[inFiles copy];
    
    // Prepare the sheet
    
    [IBownerAndGroup_ setState:(tDefaultKeepPermissionMode==YES) ? NSOnState : NSOffState];
    
    [IBreferenceStyle_ selectItemAtIndex:[IBreferenceStyle_ indexOfItemWithTag:tDefaultReferenceStyle]];
    
    [NSApp beginSheet:[IBownerAndGroup_ window]
       modalForWindow:[document_ windowForSheet]
        modalDelegate:self
       didEndSelector:@selector(fileSheetDidEnd:returnCode:contextInfo:)
          contextInfo:inFiles];
}

- (IBAction) endFileSheet:(id) sender
{
    [NSApp endSheet:[IBownerAndGroup_ window] returnCode:[sender tag]];
}

- (void) fileSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    NSArray * tArray;
    
    tArray=(NSArray *) contextInfo;
    
    if (returnCode==NSOKButton)
    {
		[self _addFiles:tArray keepOwnerAndGroup:([IBownerAndGroup_ state]==NSOnState) referenceStyle:[[IBreferenceStyle_ selectedItem] tag]];
	}
    
    [tArray release];

    [sheet orderOut:self];
}

- (IBAction) expandAll:(id) sender
{
    NSDictionary * tDictionary;
            
    tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Expanding...",@"No comment"),@"Status String",
                                                                   [NSNumber numberWithInt:1],@"Status ID",
                                                                   nil];
            
    [[NSNotificationCenter defaultCenter] postNotificationName:@"PBDocumentStatusChanged"
                                                        object:document_
                                                        userInfo:tDictionary];
                                                            
    [self performSelector:@selector(delayedExpandAll:)
               withObject:[NSNumber numberWithBool:(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)!=0)]
               afterDelay:0.1f];
}

- (void) delayedExpandAll:(id) inObject
{
    BOOL tExpandedOne=NO;
			
	tExpandedOne=[fileTree_ expandAll:[inObject boolValue] withProjectPath:[document_ folder]];
	
	[[NSNotificationCenter defaultCenter] postNotificationName:@"PBDocumentStatusChanged"
														object:document_
													  userInfo:[NSDictionary dictionary]];
	
	if (tExpandedOne==YES)
	{
		hierarchyChanged_=YES;
	
		[IBoutlineView_ reloadData];

		[self updateFiles:IBoutlineView_];
	}
}

- (IBAction) expand:(id) sender
{
    NSDictionary * tDictionary;
            
    tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Expanding...",@"No comment"),@"Status String",
                                                                   [NSNumber numberWithInt:1],@"Status ID",
                                                                   nil];
            
    [[NSNotificationCenter defaultCenter] postNotificationName:@"PBDocumentStatusChanged"
                                                        object:document_
                                                        userInfo:tDictionary];
                                                            
    [self performSelector:@selector(delayedExpand:)
               withObject:[NSNumber numberWithBool:(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)!=0)]
               afterDelay:0.1f];
}

- (void) delayedExpand:(id) inObject
{
    ICFileTree * tFileTree;
    int tSelectedCount;
        
    tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
        
    if (tSelectedCount==1)
    {
        int tSelectedRow;
        
        tSelectedRow=[IBoutlineView_ selectedRow];
    
        if (tSelectedRow>=0)
        {
            tFileTree=[IBoutlineView_ itemAtRow:tSelectedRow];
            
            [tFileTree expand:[inObject boolValue]];
            
            [[NSNotificationCenter defaultCenter] postNotificationName:@"PBDocumentStatusChanged"
                                                                object:document_
                                                              userInfo:[NSDictionary dictionary]];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadItem:tFileTree];
        
            [self updateFiles:IBoutlineView_];
        }
    }
}

- (IBAction) contract:(id) sender
{
    ICFileTree * tFileTree;
    int tSelectedCount;
        
    tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
        
    if (tSelectedCount==1)
    {
        int tSelectedRow;
        
        tSelectedRow=[IBoutlineView_ selectedRow];
    
        if (tSelectedRow>=0)
        {
            tFileTree=[IBoutlineView_ itemAtRow:tSelectedRow];
            
            [IBoutlineView_ collapseItem:tFileTree];
            
            [tFileTree contract];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadItem:tFileTree];
            
            [self updateFiles:IBoutlineView_];
        }
    }
}

- (IBAction) expandOneLevel:(id) sender
{
    ICFileTree * tFileTree;
    int tSelectedCount;
        
    tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
        
    if (tSelectedCount==1)
    {
        int tSelectedRow;
        
        tSelectedRow=[IBoutlineView_ selectedRow];
    
        if (tSelectedRow>=0)
        {
            tFileTree=[IBoutlineView_ itemAtRow:tSelectedRow];
            
            [tFileTree expandOneLevel:(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)!=0)];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadItem:tFileTree];
            
            [self updateFiles:IBoutlineView_];
        }
    }
}

- (void) outlineView:(NSOutlineView *) inOutlineView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inConfirmationRequired==YES)
	{
		[self removeFile:nil];
	}
	else
	{
		[self removeFileSheetDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
	}
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView validateAction:(SEL) inSelector
{
	if (inSelector==@selector(delete:))
	{
		NSIndexSet * tIndexSet;
	
		tIndexSet=[inOutlineView selectedRowIndexes];
		
		if (tIndexSet!=nil && [tIndexSet count]>0)
		{
			unsigned int tIndex;
				
			tIndex=[tIndexSet firstIndex];
			
			while (tIndex!=NSNotFound)
			{
				ICFileTree * tFileTree;
				
				tFileTree=(ICFileTree *) [IBoutlineView_ itemAtRow:tIndex];
			
				if ([FILENODE_DATA(tFileTree) type]==ICDOCUMENT_FILE_BASE_NODE)
				{
					return NO;
				}
				
				tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
			}
			
			return YES;
		}
	}
	
	return NO;
}

- (IBAction) removeFile:(id)sender
{
    NSString * tAlertTitle;
    
    if ([IBoutlineView_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this item?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these items?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeFileSheetDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeFileSheetDidEnd:(NSWindow *) sheet returnCode:(int) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
        NSIndexSet * tIndexSet;
        NSMutableDictionary * tDisclosedDictionary;
					
		tDisclosedDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICFILE_HIERARCHY_DISCLOSED_STATE,cachedPackageUUID_]];

		tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
		if (tIndexSet!=nil)
		{
			NSMutableArray * tSelectedNodesMutableArray;
			NSArray * tCoverageNodesArray;
			
			tSelectedNodesMutableArray=[NSMutableArray array];
			
			if (tSelectedNodesMutableArray!=nil)
			{
				unsigned int tIndex;
			
				tIndex=[tIndexSet lastIndex];
			
				while (tIndex!=NSNotFound)
				{
					ICFileTree * tNode;
					
					tNode=[IBoutlineView_ itemAtRow:tIndex];
					
					if (tNode!=nil)
					{
						[tSelectedNodesMutableArray addObject:tNode];
					}
					
					tIndex=[tIndexSet indexLessThanIndex:tIndex];
				}
				
				// Create the coverage array of nodes
				
				tCoverageNodesArray=[TreeNode minimumNodeCoverFromNodesInArray:tSelectedNodesMutableArray];
				
				if (tCoverageNodesArray!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[tCoverageNodesArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						ICFileTree * tNode;
						
						while (tNode=[tEnumerator nextObject])
						{
							NSMutableArray * tChildFoldersArray;
							
							// Removed item and descendants from disclosed state dictionary (because the key will change)
				
							tChildFoldersArray=[tNode descendantFoldersIncludingSelf:YES];
							
							if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
							{
								unsigned int i,tCount;
								
								tCount=[tChildFoldersArray count];
								
								for(i=tCount;i>0;i--)
								{
									ICFileTree * tGroupFileTree;
									NSString * tPath;
									
									tGroupFileTree=[tChildFoldersArray objectAtIndex:(i-1)];
									
									tPath=[tGroupFileTree filePath];
									
									if (tPath!=nil)
									{
										[tDisclosedDictionary removeObjectForKey:tPath];
									}
								}
							}
							
							// Remove the node
							
							[tNode removeFromParent];
						}
					}
				}
			}
		}
        
        [IBoutlineView_ deselectAll:nil];
        
        hierarchyChanged_=YES;
        
        [IBoutlineView_ reloadData];
        
        [self updateFiles:IBoutlineView_];
    }
}

- (IBAction) revealInFinder:(id) sender
{
    NSWorkspace * tWorkSpace;
    NSIndexSet * tIndexSet;
     
    tWorkSpace=[NSWorkspace sharedWorkspace];
    
	tIndexSet=[IBoutlineView_ selectedRowIndexes];
    
	if (tIndexSet!=nil)
	{
		unsigned int tIndex;
		
		tIndex=[tIndexSet firstIndex];
		
		while (tIndex!=NSNotFound)
		{
			ICFileTree * tNode;
			
			tNode=[IBoutlineView_ itemAtRow:tIndex];
			
			[tWorkSpace selectFile:[FILENODE_DATA(tNode) path] inFileViewerRootedAtPath:@""];
			
			tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
		}
	}
}

- (IBAction) setAsDefaultLocation:(id)sender
{
    int tRow;
    
    tRow=[IBoutlineView_ selectedRow];

    if (defaultLocation_!=[IBoutlineView_ itemAtRow:tRow])
    {
        defaultLocation_=[IBoutlineView_ itemAtRow:tRow];
    
        [IBoutlineView_ reloadData];
    
    	[IBdefaultLocationPath_ setStringValue:[defaultLocation_ filePath]];
    
        [self updateFiles:IBsetButton_];
		
		[IBsetButton_ setEnabled:NO];
    }
}

#pragma mark -

- (void) _addFiles:(NSArray *) inFilesArray keepOwnerAndGroup:(BOOL) inKeepOwnerAndGroup referenceStyle:(int) inReferenceStyle
{
	NSFileManager * tFileManager;
	NSString * tPath;
	NSEnumerator * tEnumerator;
	NSMutableArray * tNewSelectionArray;
	BOOL couldNeedToBeExpanded=NO;
	ICFileNode * tTargetNode=nil;
	int i,tCount;
	NSMutableIndexSet * tMutableIndexSet;
	
	switch([FILENODE_DATA(parentFileTree_) type])
	{
		case ICDOCUMENT_FILE_BASE_NODE:
		case ICDOCUMENT_FILE_NEWFOLDER_NODE:
			couldNeedToBeExpanded=YES;
			break;
		case ICDOCUMENT_FILE_REALFILE_NODE:
			break;
	}
	
	if (inKeepOwnerAndGroup==NO)
	{
		tTargetNode=FILENODE_DATA(parentFileTree_);
	}
	
	tNewSelectionArray=[NSMutableArray array];
	
	tFileManager=[NSFileManager defaultManager];
	
	tEnumerator=[inFilesArray objectEnumerator];
	
	while (tPath=[tEnumerator nextObject])
	{
		ICFileTree * tFileTree;
		id tFileNode;
		
		// 01/03/07 : Removed fileExistsAtPath:isDirectory: call
		
		tFileNode=[ICFileNode fileNodeWithType:ICDOCUMENT_FILE_REALFILE_NODE
										  path:tPath
									  pathType:inReferenceStyle];
										
		if (tFileNode!=nil)
		{
			struct stat tStat;

			if (lstat([tPath fileSystemRepresentation], &tStat)==0)
			{
				if (inKeepOwnerAndGroup==NO)
				{
					[tFileNode setUid:[tTargetNode uid]];
					[tFileNode setGid:[tTargetNode gid]];
				}
				else
				{
					[tFileNode setUid:tStat.st_uid];
					[tFileNode setGid:tStat.st_gid];
				}
				
				[tFileNode setPrivileges: (tStat.st_mode & ALLPERMS)];
			}
			else
			{
				// A COMPLETER
			}
			
			tFileTree=[[ICFileTree alloc] initWithData:tFileNode
												parent:parentFileTree_
											  children:nil];

			[parentFileTree_ insertSortedChild:tFileTree];
								
			[tFileTree release];
			
			[tNewSelectionArray addObject:tFileTree];
		}
	}
	
	[IBoutlineView_ deselectAll:nil];

	hierarchyChanged_=YES;
	
	[IBoutlineView_ reloadData];
	
	if (couldNeedToBeExpanded==YES)
	{
		if ([IBoutlineView_ isItemExpanded:parentFileTree_]==NO)
		{
			[IBoutlineView_ expandItem:parentFileTree_];
		}
	}
	
	tCount=[tNewSelectionArray count];
	
	tMutableIndexSet=[NSMutableIndexSet indexSet];
	
	for(i=0;i<tCount;i++)
	{
		[tMutableIndexSet addIndex:[IBoutlineView_ rowForItem:[tNewSelectionArray objectAtIndex:i]]];
	}
	
	[IBoutlineView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:YES];
	
	[self updateFiles:IBoutlineView_];
}

#pragma mark -

- (void) outlineViewItemDidExpand:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			ICFileTree * tTreeNode;
			
			tTreeNode=(ICFileTree *) [tUserInfo objectForKey:@"NSObject"];
			
			if (tTreeNode!=nil)
			{
				NSString * tFilePath;
				
				tFilePath=[tTreeNode filePath];
				
				if (tFilePath!=nil)
				{
					NSMutableDictionary * tDisclosedDictionary;
					NSString * tKey;
					
					tKey=[NSString stringWithFormat:ICFILE_HIERARCHY_DISCLOSED_STATE,cachedPackageUUID_];
					
					tDisclosedDictionary=[document_ documentDefaultForKey:tKey];
					
					if (tDisclosedDictionary==nil)
					{
						tDisclosedDictionary=[NSMutableDictionary dictionary];
						
						if (tDisclosedDictionary!=nil)
						{
							[document_ setDocumentDefault:tDisclosedDictionary forKey:tKey];
						}
						else
						{
							NSLog(@"[ICPackageFilesController outlineViewItemDidExpand:] Low memory");
						}
					}
					
					if (tDisclosedDictionary!=nil)
					{
						[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tFilePath];
					}
				}
			}
		}
	}
}

- (void) outlineViewItemWillCollapse:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			ICFileTree * tTreeNode;
			
			tTreeNode=(ICFileTree *) [tUserInfo objectForKey:@"NSObject"];
			
			if (tTreeNode!=nil)
			{
				NSString * tFilePath;
				
				tFilePath=[tTreeNode filePath];
				
				if (tFilePath!=nil)
				{
					NSMutableDictionary * tDisclosedDictionary;
					
					tDisclosedDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICFILE_HIERARCHY_DISCLOSED_STATE,cachedPackageUUID_]];
					
					if (tDisclosedDictionary!=nil)
					{
						// Check if the option key is down or not
						
						NSEvent * tCurrentEvent;
						
						tCurrentEvent=[NSApp currentEvent];
						
						if (tCurrentEvent==nil || (([tCurrentEvent modifierFlags] & NSAlternateKeyMask)==0))
						{
							if ([tFilePath isEqualToString:@"/"]==NO)
							{
								// Check the parents state
							
								NSString * tParentPath;
								NSNumber * tNumber;
								
								tParentPath=[NSString stringWithString:tFilePath];
							
								do
								{
									tParentPath=[tParentPath stringByDeletingLastPathComponent];
									
									tNumber=[tDisclosedDictionary objectForKey:tParentPath];
									
									if (tNumber==nil || [tNumber boolValue]==NO)
									{
										// Parent is hidden
									
										return;
									}
									
									if ([tParentPath isEqualToString:@"/"]==YES)
									{
										break;
									}
								}
								while (1);
							}
						}
						
						[tDisclosedDictionary removeObjectForKey:tFilePath];
					}
				}
			}
		}
	}
}

- (void) folderNameShouldChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
    {
    	NSDictionary * tUserInfo;
        
        tUserInfo=[inNotification userInfo];
        
        if (tUserInfo!=nil)
        {
			ICFileTree * tFileTree;
			NSString * tNewName;
			
			tFileTree=[tUserInfo objectForKey:ICPACKAGEFILESINSPECTOR_FILETREE];
			
			tNewName=[tUserInfo objectForKey:ICPACKAGEFILESINSPECTOR_NEWNAME];
			
			if (tFileTree!=nil && tNewName!=nil)
			{
				ICFileTree * tParentTree;
				NSMutableArray * tChildFoldersArray;
				NSMutableDictionary * tDisclosedDictionary;
				ICFileNode * tFileNode;
				NSNumber * tNumber;
				
				tNumber=[tUserInfo objectForKey:ICPACKAGEFILESINSPECTOR_INSPECTOR];
				
				if (tNumber!=nil && [tNumber boolValue]==YES)
				{
					if ([IBoutlineView_ editedRow]!=-1)
					{
						NSText * tFieldEditor;
						
						// Get the field editor
						
						tFieldEditor=[[IBoutlineView_ window] fieldEditor:NO forObject:IBoutlineView_];
						
						if (tFieldEditor!=nil)
						{
							[tFieldEditor setString:[FILENODE_DATA(tFileTree) fileName]];
						}
					}
				}
				
				tDisclosedDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICFILE_HIERARCHY_DISCLOSED_STATE,cachedPackageUUID_]];
		
				// Removed item and descendants from disclosed state dictionary (because the key will change)
				
				tChildFoldersArray=[tFileTree descendantFoldersIncludingSelf:YES];
				
				if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
				{
					unsigned int i,tCount;
					
					tCount=[tChildFoldersArray count];
					
					for(i=tCount;i>0;i--)
					{
						ICFileTree * tGroupFileTree;
						NSString * tPath;
						
						tGroupFileTree=[tChildFoldersArray objectAtIndex:(i-1)];
						
						tPath=[tGroupFileTree filePath];
						
						if (tPath!=nil)
						{
							BOOL tRemoved=NO;
							
							tNumber=[tDisclosedDictionary objectForKey:tPath];
							
							if (tNumber==nil)
							{
								tRemoved=YES;
							}
							else
							{
								if ([tNumber boolValue]==NO)
								{
									tRemoved=YES;
								}
								
								[tDisclosedDictionary removeObjectForKey:tPath];
							}
							
							if (tRemoved==YES)
							{
								[tChildFoldersArray removeObject:tGroupFileTree];
							}
						}
					}
				}
				
				tFileNode=FILENODE_DATA(tFileTree);
				
				if (tFileNode!=nil)
				{
					[tFileNode setPath:tNewName];
				}
				
				
				tParentTree=(ICFileTree *) [tFileTree nodeParent];
		
				[tFileTree retain];
				
				[tFileTree removeFromParent];
				
				[tParentTree insertSortedChild:tFileTree];
				
				[tFileTree release];
				
				// Put back the item and descendant folders disclosed state
				
				if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[tChildFoldersArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						ICFileTree * tGroupFileTree;
						
						while (tGroupFileTree=[tEnumerator nextObject])
						{
							NSString * tPath;
							
							tPath=[tGroupFileTree filePath];
						
							if (tPath!=nil)
							{
								[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tPath];
							}
						}
					}
				}
				
				// Refresh list
				
				[IBoutlineView_ deselectAll:nil];
				
				[IBoutlineView_ reloadItem:tParentTree reloadChildren:YES];
				
				[IBoutlineView_ selectRow:[IBoutlineView_ rowForItem:tFileTree] byExtendingSelection:NO];
				
				// Update hierarchy dictionary
				
				hierarchyChanged_=YES;
				
				[self updateFiles:IBoutlineView_];
			}
		}
	}
}

- (void) fileAttributesDidChange:(NSNotification *) inNotification
{
    if (inNotification!=nil)
    {
    	NSDictionary * tUserInfo;
        
        tUserInfo=[inNotification userInfo];
        
        if (tUserInfo!=nil)
        {
            NSArray * tSelectedFiles;
			
			tSelectedFiles=[tUserInfo objectForKey:ICPACKAGEFILESINSPECTOR_SELECTED_FILES];
			
			if (tSelectedFiles!=nil)
			{
				NSEnumerator * tEnumerator;
				
				hierarchyChanged_=YES;
				
				tEnumerator=[tSelectedFiles objectEnumerator];
                
				if (tEnumerator!=nil)
				{
					ICFileTree * tFileTree;
					
					// Refresh the appropriate rows
				
					while (tFileTree=[tEnumerator nextObject])
					{
						[IBoutlineView_ reloadItem:tFileTree reloadChildren:[IBoutlineView_ isItemExpanded:tFileTree]];
					}
				}
					
				[self updateFiles:IBoutlineView_];
            }
        }
    }
}

- (void) outlineViewSelectionDidChange:(NSNotification *) inNotification
{
	int tSelectedCount;
	NSRect tBounds;
	NSView * tView;
	
	tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
	
	if (tSelectedCount==0)
	{
		if (emptySelectionInspectorController_==nil)
		{
			emptySelectionInspectorController_=[ICPackageFilesInspectorEmptySelectionController newController];
		}
		
		if (currentInspectorController_!=emptySelectionInspectorController_ || currentInspectorController_==nil)
		{
			tView=[currentInspectorController_ view];
		
			[tView removeFromSuperview];
		
			currentInspectorController_=emptySelectionInspectorController_;
			
			tView=[currentInspectorController_ view];
			
			tBounds=[IBinspectorView_ bounds];
			
			[tView setFrame:tBounds];
			
			[IBinspectorView_ addSubview:tView];
		}
		
		[IBsetButton_ setEnabled:NO];
		
		[IBremoveButton_ setEnabled:NO];
	}
	else
	{
		NSIndexSet * tIndexSet;
		
		[IBremoveButton_ setEnabled:YES];
		
		if (selectionInspectorController_==nil)
		{
			selectionInspectorController_=[ICPackageFilesInspectorSelectionController newController];
		}
		
		if (currentInspectorController_!=selectionInspectorController_ || currentInspectorController_==nil)
		{
			tView=[currentInspectorController_ view];
		
			[tView removeFromSuperview];
		
			currentInspectorController_=selectionInspectorController_;
			
			tView=[currentInspectorController_ view];
			
			tBounds=[IBinspectorView_ bounds];
			
			[tView setFrame:tBounds];
			
			[IBinspectorView_ addSubview:tView];
		}
		
		tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
		if (tIndexSet!=nil)
		{
			NSMutableArray * tSelectedItems;
			
			tSelectedItems=[NSMutableArray array];
			
			if (tSelectedItems!=nil)
			{
				unsigned int tIndex;
				BOOL tDisableRemoveButton=NO;
				
				tIndex=[tIndexSet firstIndex];
				
				while (tIndex!=NSNotFound)
				{
					id tItem;
					
					tItem=[IBoutlineView_ itemAtRow:tIndex];
					
					if (tItem!=nil)
					{
						if (tDisableRemoveButton==NO)
						{
							if ([FILENODE_DATA(tItem) type]==ICDOCUMENT_FILE_BASE_NODE)
							{
								tDisableRemoveButton=YES;
							}
						}
						
						[tSelectedItems addObject:tItem];
						
						tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
					}
				}
				
				if (tDisableRemoveButton==YES)
				{
					[IBremoveButton_ setEnabled:NO];
				}
				
				[IBsetButton_ setEnabled:NO];
				
				if (tSelectedCount==1)
				{
					int tType;
					ICFileTree * tFileTree;
					
					tFileTree=[tSelectedItems lastObject];
					
					tType=[FILENODE_DATA(tFileTree) type];
		
					if (tType==ICDOCUMENT_FILE_BASE_NODE || tType==ICDOCUMENT_FILE_NEWFOLDER_NODE)
					{
						if (tFileTree!=defaultLocation_)
						{
							[IBsetButton_ setEnabled:YES];
						}
					}
					
					[IBaddButton_ setEnabled:YES];
				}
				else
				{
					[IBaddButton_ setEnabled:NO];
				}
				
				[selectionInspectorController_ initializeWithArray:tSelectedItems forDocument:document_];
			}
		}
	}
}

- (void) highlightExcludedFilesDidChange:(NSNotification *) inNotification
{
	cachedHighlightExcludedFiles_=[defaults_ boolForKey:ICPREFERENCES_FILES_HIGHLIGHT_EXCLUDED_FILES];
	
	[IBoutlineView_ reloadData];
}

@end
