/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICPackageFilesInspectorSelectionController.h"

#import "ICDirectoryServicesManager.h"

#import "ICLocatorEditorController.h"

#import "ICFileDropView.h"

#import "ICDocument.h"

#import "ICPreferences+Constants.h"

@implementation ICPackageFilesInspectorSelectionController

+ (ICPackageFilesInspectorSelectionController *) newController
{
	ICPackageFilesInspectorSelectionController * nController;
    
    nController=[ICPackageFilesInspectorSelectionController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICPayloadSelectionInspectorView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICPayloadSelectionInspectorView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return nController;
}

- (void) awakeFromNib
{
	ICDirectoryServicesManager * tDirectoryServicesManager;
	NSMutableArray * tPasswordArray;
    NSMutableArray * tGroupArray;
	NSUInteger i,tCount;
	NSTableColumn * tTableColumn = nil;
	
	[super awakeFromNib];
	
	// Ownership and Permission
	
	tDirectoryServicesManager=[ICDirectoryServicesManager defaultManager];
	
	[IBfileOwner_ removeAllItems];
    
    tPasswordArray=[tDirectoryServicesManager usersArray];
    
    [tPasswordArray sortUsingSelector:@selector(compareUserGroupName:)];
    
    tCount=[tPasswordArray count];
    
    for(i=0;i<tCount;i++)
    {
        NSDictionary * tDictionary;
        NSString * tKey;
        
        tDictionary=[tPasswordArray objectAtIndex:i];
        
        tKey=[tDictionary objectForKey:@"Name"];
        
        [IBfileOwner_ addItemWithTitle:tKey];
        
        [[IBfileOwner_ itemWithTitle:tKey] setTag:[[tDictionary objectForKey:@"ID"] intValue]];
    }
	
	[IBfileGroup_ removeAllItems];
    
    tGroupArray=[tDirectoryServicesManager groupsArray];
    
    [tGroupArray sortUsingSelector:@selector(compareUserGroupName:)];
    
    tCount=[tGroupArray count];
    
    for(i=0;i<tCount;i++)
    {
        NSDictionary * tDictionary;
        NSString * tKey;
        
        tDictionary=[tGroupArray objectAtIndex:i];
        
        tKey=[tDictionary objectForKey:@"Name"];
        
        [IBfileGroup_ addItemWithTitle:tKey];
        
        [[IBfileGroup_ itemWithTitle:tKey] setTag:[[tDictionary objectForKey:@"ID"] intValue]];
    }
    
    // Locators
	
	[IBlocatorsArray_ setDoubleAction:@selector(editLocator:)];
	
	[IBlocatorsArray_ setTarget:self];
	
	tTableColumn = [IBlocatorsArray_ tableColumnWithIdentifier:@"State"];
    
	if (tTableColumn!=nil)
	{
		NSButtonCell * tButttonCell;
		
		tButttonCell = [[NSButtonCell alloc] initTextCell: @""];
    
		if (tButttonCell!=nil)
		{
			[tButttonCell setControlSize:NSMiniControlSize];
			[tButttonCell setEditable: YES];
			[tButttonCell setButtonType: NSSwitchButton];
			[tButttonCell setImagePosition: NSImageOnly];
		
			[tTableColumn setDataCell:tButttonCell];
		
			[tButttonCell release];
		}
	}
	
	// Name
	
	tTableColumn = [IBlocatorsArray_ tableColumnWithIdentifier:@"Name"];
    
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
	
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
	}
	
    // Scripts
	
	[[NSNotificationCenter defaultCenter]  addObserver:self
											 selector:@selector(dropViewFilePathDidChange:) 
												 name:ICDropViewFilePathDidChangeNotification 
											   object:IBpreInstallIcon_];
											   
	[[NSNotificationCenter defaultCenter]  addObserver:self
											 selector:@selector(dropViewFilePathDidChange:) 
												 name:ICDropViewFilePathDidChangeNotification 
											   object:IBpostInstallIcon_];
	
	/*[IBfileSpecialBitsArray_ setStripesColor:[NSColor colorWithDeviceRed:1.0f
                                                               green:213.0/255 blue:202.0/255.0 alpha:1.0f]];*/
}

- (void) initializeWithArray:(NSMutableArray *) inArray forDocument:(id) inDocument
{
	[super initializeWithDictionary:nil forDocument:inDocument];
	
	array_=inArray;
	
	if (array_!=nil)
	{
		[array_ retain];
		
		if ([[IBfileOwner_ itemTitleAtIndex:0] isEqualToString:NSLocalizedString(@"Mixed",@"No comment")]==YES)
		{
			[IBfileOwner_ removeItemAtIndex:0];
		}
		
		if ([[IBfileGroup_ itemTitleAtIndex:0] isEqualToString:NSLocalizedString(@"Mixed",@"No comment")]==YES)
		{
			[IBfileGroup_ removeItemAtIndex:0];
		}
		
		if ([[IBpathType_ itemTitleAtIndex:0] isEqualToString:NSLocalizedString(@"Mixed",@"No comment")]==YES)
		{
			[IBpathType_ removeItemAtIndex:0];
		}
		
		if ([array_ count]==1)
		{
			[self updateSingleSelection];
		}
		else
		{
			[self updateMultipleSelection];
		}
		
		[IBfilePermissionsArray_ deselectAll:self];
		[IBfilePermissionsArray_ reloadData];
		
		[IBfileSpecialBitsArray_ deselectAll:self];
		[IBfileSpecialBitsArray_ reloadData];
	}
}

- (void) dealloc
{
	[super dealloc];
}

#pragma mark -

- (void) updateSingleSelection
{
	ICFileTree * tFileTree;
	ICFileNode * tFileNode;
	ICFileNode * tPermissionNode;
	BOOL isSymbolicLink=NO;
	int tType;
	NSString * tFilePath;
	BOOL tHideTab=YES;
	NSBundle * tBundle=nil;
	
	tFileTree=[array_ objectAtIndex:0];
	
	tFileNode=FILENODE_DATA(tFileTree);
	
	[super updateSingleSelection];
	
	// *** General Section

	tType=[tFileNode type];
	
	tFilePath=[tFileNode path];
	
	// Bundle ?
	
	if (tType==ICDOCUMENT_FILE_REALFILE_NODE)
	{
		if ([tFileNode link]==NO)
		{
			// Is it a Bundle?
		
			tBundle=[NSBundle bundleWithPath:tFilePath];
		
			if (tBundle!=nil && [tBundle bundleIdentifier]!=nil)
			{
				tHideTab=NO;
			}
		}
	}
	
	// Tabs
	
	if (tHideTab==YES)
	{
		if (tabViewBundleItem_==nil)
		{
			tabViewBundleItem_=[IBtabView_ tabViewItemAtIndex:[IBtabView_ indexOfTabViewItemWithIdentifier:@"Bundle"]];
		
			[tabViewBundleItem_ retain];
		
			[IBtabView_ removeTabViewItem:tabViewBundleItem_];
		}
		
		if (tabViewScriptsItem_==nil)
		{
			tabViewScriptsItem_=[IBtabView_ tabViewItemAtIndex:[IBtabView_ indexOfTabViewItemWithIdentifier:@"Scripts"]];
		
			[tabViewScriptsItem_ retain];
		
			[IBtabView_ removeTabViewItem:tabViewScriptsItem_];
		}
		
		// Unregister for notifications
			
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidBecomeMainNotification object:[[self document] windowForSheet]];
			
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidResignMainNotification object:[[self document] windowForSheet]];
	}
	else
	{
		if (tabViewBundleItem_!=nil)
		{
			[IBtabView_ addTabViewItem:tabViewBundleItem_];
		
			[tabViewBundleItem_ release];
		
			tabViewBundleItem_=nil;
		}
		
		if (tabViewScriptsItem_!=nil)
		{
			[IBtabView_ addTabViewItem:tabViewScriptsItem_];
		
			[tabViewScriptsItem_ release];
		
			tabViewScriptsItem_=nil;
		}
		
		// Unregister for notifications
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidBecomeMainNotification object:[[self document] windowForSheet]];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidResignMainNotification object:[[self document] windowForSheet]];
		
		// Register for notifications
		
		[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(windowStateDidChange:) 
												 name:NSWindowDidBecomeMainNotification 
											   object:[[self document] windowForSheet]];
	
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidResignMainNotification 
												   object:[[self document] windowForSheet]];
	}
	
	// OwnerShip and Permissions
	
	tPermissionNode=tFileNode;
	
	switch(tType)
    {
        case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
		case ICDOCUMENT_FILE_BASE_NODE:
		
			canEditFilePermissions_=NO;
			
			break;
		
		case ICDOCUMENT_FILE_NEWFOLDER_NODE:
		
			canEditFilePermissions_=YES;
			
			break;
		
		case ICDOCUMENT_FILE_REALFILE_NODE:
            
			isSymbolicLink=[tFileNode link];
			
			if (isSymbolicLink==YES)
			{
				ICFileTree * tParentTree;
			
				tParentTree=(ICFileTree *) [tFileTree nodeParent];
				
				if (tParentTree!=nil)
				{
					tPermissionNode=FILENODE_DATA(tParentTree);
				}
				
				canEditFilePermissions_=NO;
			}
			else
			{
				canEditFilePermissions_=YES;
			}
			
            break;
    }
	
	[IBfileOwner_ setEnabled:canEditFilePermissions_];
			
	[IBfileGroup_ setEnabled:canEditFilePermissions_];
	
	
	cachedOwnerIndex_=[IBfileOwner_ indexOfItemWithTag:[tPermissionNode uid]];
                    
	if (cachedOwnerIndex_==-1)
	{
		[IBfileOwnerTextField_ setHidden:NO];
		
		[IBfileOwner_ setHidden:YES];
		
		// A COMPLETER
	}
	else
	{
		[IBfileOwnerTextField_ setHidden:YES];
	
		[IBfileOwner_ setHidden:NO];
		
		[IBfileOwner_ selectItemAtIndex:cachedOwnerIndex_];
	}
	
	cachedGroupIndex_=[IBfileGroup_ indexOfItemWithTag:[tPermissionNode gid]];
                    
	if (cachedGroupIndex_==-1)
	{
		[IBfileGroupTextField_ setHidden:NO];
		
		[IBfileGroup_ setHidden:YES];
		
		// A COMPLETER
	}
	else
	{
		[IBfileGroupTextField_ setHidden:YES];
		
		[IBfileGroup_ setHidden:NO];
		
		[IBfileGroup_ selectItemAtIndex:cachedGroupIndex_];
	}
	
	// Bundle
	
	[cachedLocators_ release];
	
	cachedLocators_=nil;
	
	if (tHideTab==NO)
	{
		BOOL tAllowDowngrade;
		
		tAllowDowngrade=[tFileNode allowDowngrade];
		
		[IBallowDowngradeCheckbox_ setState:(tAllowDowngrade==YES) ? NSOnState : NSOffState];
	
	
		cachedLocators_=[tFileNode locators];
		
		if (cachedLocators_==nil)
		{
			cachedLocators_=[NSMutableArray array];
			
			[tFileNode setLocators:cachedLocators_];
		}
		
		if (cachedLocators_!=nil)
		{
			[cachedLocators_ retain];
		}
		
		[IBlocatorsArray_ reloadData];
	}
	
	// Scripts
	
	if (tHideTab==NO)
	{
		NSDictionary * tPathDictionary;
		NSString * tPath;
		NSString * tAbsolutePath;
		NSNumber * tNumber;
		NSInteger tPathType;
		
		// Pre-install
		
		cachedPreInstallPathTypeIndex_=0;
		
		tPathDictionary=[tFileNode preInstallPath];
		
		if (tPathDictionary!=nil)
		{
			tPathType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			
			tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
		
			tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
		
			if (tNumber!=nil)
			{
				tPathType=[tNumber integerValue];
			}
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
			}
			else
			{
				tAbsolutePath=tPath;
			}
			
			[IBpreInstallName_ setToolTip:tPath];
			
			[self updatePreInstallScriptWithPath:tAbsolutePath];
		}
		else
		{
			tPathType=[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
		
			[self updatePreInstallScriptWithPath:nil];
		}
		
		// Type
			
		// File Reference
		
		cachedPreInstallPathTypeIndex_=[IBpreInstallPathType_ indexOfItemWithTag:tPathType];
				
		[IBpreInstallPathType_ selectItemAtIndex:cachedPreInstallPathTypeIndex_];
		
		// Post-install
		
		cachedPostInstallPathTypeIndex_=0;
		
		tPathDictionary=[tFileNode postInstallPath];
		
		if (tPathDictionary!=nil)
		{
			tPathType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			
			tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
		
			tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
		
			if (tNumber!=nil)
			{
				tPathType=[tNumber integerValue];
			}
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
			}
			else
			{
				tAbsolutePath=tPath;
			}
			
			[IBpostInstallName_ setToolTip:tPath];

			
			[self updatePostInstallScriptWithPath:tAbsolutePath];
		}
		else
		{
			tPathType=[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
			
			[self updatePostInstallScriptWithPath:nil];
		}
		
		// Type
			
		// File Reference
			
		cachedPostInstallPathTypeIndex_=[IBpostInstallPathType_ indexOfItemWithTag:tPathType];
					
		[IBpostInstallPathType_ selectItemAtIndex:cachedPostInstallPathTypeIndex_];
	}
}

- (void) updateMultipleSelection
{
	NSEnumerator * tEnumerator;
	BOOL tCanEditProperty;
	BOOL tFirstSeen;
	NSInteger tCommonID=NSNotFound;
	BOOL tMixedProperty;
	
	[super updateMultipleSelection];
				
	// No Scripts or Locators for multiple selection
	
	if (tabViewScriptsItem_==nil)
	{
		tabViewScriptsItem_=[IBtabView_ tabViewItemAtIndex:[IBtabView_ indexOfTabViewItemWithIdentifier:@"Scripts"]];
	
		[tabViewScriptsItem_ retain];
	
		[IBtabView_ removeTabViewItem:tabViewScriptsItem_];
	}
	
	if (tabViewBundleItem_==nil)
	{
		tabViewBundleItem_=[IBtabView_ tabViewItemAtIndex:[IBtabView_ indexOfTabViewItemWithIdentifier:@"Bundle"]];
	
		[tabViewBundleItem_ retain];
	
		[IBtabView_ removeTabViewItem:tabViewBundleItem_];
	}
	
	// OwnerShip and Permissions
	
	// Owner
	
	tCanEditProperty=YES;
	
	tFirstSeen=YES;
	
	tMixedProperty=NO;
	
	tEnumerator=[array_ objectEnumerator];
	
	if (tEnumerator!=nil)
	{
		ICFileTree * tFileTree;
		
		while ((tFileTree=[tEnumerator nextObject]) && tCanEditProperty==YES && tMixedProperty==NO)
		{
			ICFileNode * tFileNode;
			
			tFileNode=FILENODE_DATA(tFileTree);
			
			switch([tFileNode type])
			{
				case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
				case ICDOCUMENT_FILE_BASE_NODE:
				
					tCanEditProperty=NO;
				
				case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				case ICDOCUMENT_FILE_REALFILE_NODE:
					
					if ([tFileNode link]==YES)
					{
						ICFileTree * tParentTree;
						
						tCanEditProperty=NO;
						
						tParentTree=(ICFileTree *) [tFileTree nodeParent];
						
						if (tParentTree!=nil)
						{
							tFileNode=FILENODE_DATA(tParentTree);
						}
					}
					
					if (tFirstSeen==YES)
					{
						tCommonID=[tFileNode uid];
						
						tFirstSeen=NO;
					}
					else
					{
						if (tCommonID!=[tFileNode uid])
						{
							tMixedProperty=YES;
						}
					}
					
					break;
			}

		}
	}
	
	[IBfileOwner_ setEnabled:tCanEditProperty];
	
	if (tMixedProperty==NO)
	{
		cachedOwnerIndex_=[IBfileOwner_ indexOfItemWithTag:tCommonID];
				
		if (cachedOwnerIndex_==-1)
		{
			[IBfileOwnerTextField_ setHidden:NO];
			
			[IBfileOwner_ setHidden:YES];
		}
		else
		{
			[IBfileOwnerTextField_ setHidden:YES];
			
			[IBfileOwner_ setHidden:NO];
			
			[IBfileOwner_ selectItemAtIndex:cachedOwnerIndex_];
		}
	}
	else
	{
		cachedOwnerIndex_=0;
								
		[IBfileOwner_ insertItemWithTitle:NSLocalizedString(@"Mixed",@"No comment")
							  atIndex:0];
		
		[[IBfileOwner_ itemAtIndex:0] setTarget:nil];
		
		[[IBfileOwner_ itemAtIndex:0] setEnabled:NO];
		
		[IBfileOwnerTextField_ setHidden:YES];
		
		[IBfileOwner_ setHidden:NO];
		
		[IBfileOwner_ selectItemAtIndex:0];
		
		[IBfileOwner_ setNeedsDisplay:YES];
	}
	
	// Group
	
	tCanEditProperty=YES;
	
	tFirstSeen=YES;
	
	tMixedProperty=NO;
	
	tEnumerator=[array_ objectEnumerator];
	
	if (tEnumerator!=nil)
	{
		ICFileTree * tFileTree;
		
		while ((tFileTree=[tEnumerator nextObject]) && tCanEditProperty==YES && tMixedProperty==NO)
		{
			ICFileNode * tFileNode;
			
			tFileNode=FILENODE_DATA(tFileTree);
			
			switch([tFileNode type])
			{
				case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
				case ICDOCUMENT_FILE_BASE_NODE:
				
					tCanEditProperty=NO;
				
				case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				case ICDOCUMENT_FILE_REALFILE_NODE:
					
					if ([tFileNode link]==YES)
					{
						tCanEditProperty=NO;
						
						ICFileTree * tParentTree;
						
						tCanEditProperty=NO;
						
						tParentTree=(ICFileTree *) [tFileTree nodeParent];
						
						if (tParentTree!=nil)
						{
							tFileNode=FILENODE_DATA(tParentTree);
						}
					}
					
					if (tFirstSeen==YES)
					{
						tCommonID=[tFileNode gid];
						
						tFirstSeen=NO;
					}
					else
					{
						if (tCommonID!=[tFileNode gid])
						{
							tMixedProperty=YES;
						}
					}
					
					break;
			}

		}
	}
	
	[IBfileGroup_ setEnabled:tCanEditProperty];
	
	if (tMixedProperty==NO)
	{
		cachedGroupIndex_=[IBfileGroup_ indexOfItemWithTag:tCommonID];
				
		if (cachedGroupIndex_==-1)
		{
			[IBfileGroupTextField_ setHidden:NO];
			
			[IBfileGroup_ setHidden:YES];
		}
		else
		{
			[IBfileGroupTextField_ setHidden:YES];
			
			[IBfileGroup_ setHidden:NO];
			
			[IBfileGroup_ selectItemAtIndex:cachedGroupIndex_];
		}
	}
	else
	{
		cachedGroupIndex_=0;
								
		[IBfileGroup_ insertItemWithTitle:NSLocalizedString(@"Mixed",@"No comment")
							  atIndex:0];
		
		[[IBfileGroup_ itemAtIndex:0] setTarget:nil];
		
		[[IBfileGroup_ itemAtIndex:0] setEnabled:NO];
		
		[IBfileGroupTextField_ setHidden:YES];
		
		[IBfileGroup_ setHidden:NO];
		
		[IBfileGroup_ selectItemAtIndex:0];
		
		[IBfileGroup_ setNeedsDisplay:YES];
	}
}

#pragma mark -

- (BOOL)validateMenuItem:(NSMenuItem *) inMenuItem
{
    SEL tAction=[inMenuItem action];
        
    if (tAction==@selector(switchOwner:) ||
        tAction==@selector(switchGroup:))
    {
        if ([inMenuItem tag]<0 || [inMenuItem tag]>UINT16_MAX)
        {
            return NO;
        }
    }
    else
    if (tAction==@selector(revealSourceInFinder:))
    {
        if ([array_ count]==1)
		{
			ICFileNode * tFileNode;
        
			tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
		
			return ([[IBfileSource_ stringValue] isEqualTo:@"-"]==NO && [tFileNode type]==ICDOCUMENT_FILE_REALFILE_NODE);
		}
    }
    else if (tAction==@selector(switchPermissions:))
    {
        /*if (selectedFile_!=nil)
        {
            return (([tFileNode type]==kRealItemNode && [tFileNode link]==NO) || [tFileNode type]==kNewFolderNode);
        }
        else*/
        {
            return canEditFilePermissions_;
        }
    }
    
    return YES;
}



- (IBAction) switchOwner:(id) sender
{
    if (cachedOwnerIndex_!=[IBfileOwner_ indexOfSelectedItem])
    {
        ICFileNode * tFileNode;
        NSInteger tTag;
        NSEnumerator * tEnumerator;
		
        cachedOwnerIndex_=[IBfileOwner_ indexOfSelectedItem];
        
        tTag=[[IBfileOwner_ selectedItem] tag];
        
		tEnumerator=[array_ objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			while (tFileNode=FILENODE_DATA([tEnumerator nextObject]))
			{
				[tFileNode setUid:tTag];
			}
        }
		
        // Notify Document Change
		
		[self postAttributesDidChangeNotification];
    }
}

- (IBAction) switchGroup:(id) sender
{
    if (cachedGroupIndex_!=[IBfileGroup_ indexOfSelectedItem])
    {
        ICFileNode * tFileNode;
        NSInteger tTag;
		NSEnumerator * tEnumerator;
		
        cachedGroupIndex_=[IBfileGroup_ indexOfSelectedItem];
    
        tTag=[[IBfileGroup_ selectedItem] tag];
        
        tEnumerator=[array_ objectEnumerator];
            
		if (tEnumerator!=nil)
		{
			while (tFileNode=FILENODE_DATA([tEnumerator nextObject]))
			{
				[tFileNode setGid:tTag];
			}
		}
		
        // Notify Document Change
		
		[self postAttributesDidChangeNotification];
    }
}

#pragma mark -

- (IBAction) switchAllowDowngrade:(id) sender
{
	ICFileNode * tFileNode;
	
	tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
		
	if (tFileNode!=nil)
	{
		BOOL tAllowDowngrade;
		
		tAllowDowngrade=([sender state]==NSOnState);
		
		[tFileNode setAllowDowngrade:tAllowDowngrade];
		
		// Post Notification
	
		[self postAttributesDidChangeNotification];
	}
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (inTableView==IBlocatorsArray_)
	{
		if (cachedLocators_!=nil)
		{
			return [cachedLocators_ count];
		}
	}
	else
	{
		return [super numberOfRowsInTableView:inTableView];
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (inTableView==IBlocatorsArray_)
	{
		if (cachedLocators_!=nil)
		{
			NSDictionary * tDictionary;
			NSString * tColumnIdentifier;
			
			tDictionary=[cachedLocators_ objectAtIndex:inRowIndex];
			
			tColumnIdentifier=[inTableColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:@"State"])
			{
				return [tDictionary objectForKey:ICDOCUMENT_LOCATOR_STATE];
			}
			else if ([tColumnIdentifier isEqualToString:@"Name"])
			{
				return [tDictionary objectForKey:ICDOCUMENT_LOCATOR_NAME];
			}
		}
	}
	else
	{
		return [super tableView:inTableView objectValueForTableColumn:inTableColumn row:inRowIndex];
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	
	if (inTableView==IBlocatorsArray_)
	{
		if (cachedLocators_!=nil)
		{
			NSMutableDictionary * tDictionary;
			
			tColumnIdentifier=[inTableColumn identifier];
		
			tDictionary=[cachedLocators_ objectAtIndex:inRowIndex];
				
			if ([tColumnIdentifier isEqualToString:@"State"])
			{
				NSNumber * tNumber;
				
				tNumber=[tDictionary objectForKey:ICDOCUMENT_LOCATOR_STATE];
				
				if ([tNumber boolValue]!=[object boolValue])
				{
					[tDictionary setObject:object forKey:ICDOCUMENT_LOCATOR_STATE];
					
					// Notify Document Change
					
					[self postAttributesDidChangeNotification];
				}
			}
			else if ([tColumnIdentifier isEqualToString:@"Name"])
			{
				NSString * tString;
							
				tString=[tDictionary objectForKey:ICDOCUMENT_LOCATOR_NAME];
							
				if ([tString isEqualToString:object]==NO)
				{
					NSEnumerator * tEnumerator;
			
					tEnumerator=[cachedLocators_ objectEnumerator];
			
					if (tEnumerator!=nil)
					{
						NSDictionary * tOtherDictionary;
						
						while (tOtherDictionary=[tEnumerator nextObject])
						{
							if (tOtherDictionary!=tDictionary)
							{
								NSString * tOtherName;
							
								tOtherName=[tOtherDictionary objectForKey:ICDOCUMENT_LOCATOR_NAME];
							
								if (tOtherName!=nil)
								{
									if ([tOtherName isEqualToString:object]==YES)
									{
										NSBeep();
										
										[IBlocatorsArray_ editColumn:[IBlocatorsArray_ columnWithIdentifier:@"Name"] row:inRowIndex withEvent:nil select:YES];
										
										return;
									}
								}
							}
						}
						
						[tDictionary setObject:object forKey:ICDOCUMENT_LOCATOR_NAME];
						
						// Notify Document Change
						
						[self postAttributesDidChangeNotification];
					}
				}
			}
		}
	}
	else
	{
		[super tableView:inTableView setObjectValue:object forTableColumn:inTableColumn row:inRowIndex];
	}
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBlocatorsArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeLocator:IBremoveLocatorButton_];
		}
		else
		{
			[self removeLocatorsDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBlocatorsArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBremoveLocatorButton_ isEnabled];
		}
	}
	
	return NO;
}

#pragma mark -

- (IBAction) addLocator:(id) sender
{
	NSMutableDictionary * tMutableDictionary;
	
	editLocatorMode_=NO;
	
	tMutableDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],ICDOCUMENT_LOCATOR_STATE,
																		 @"fr.whitebox.Packages.locator.standard",ICDOCUMENT_LOCATOR_IDENTIFIER,
																		 nil];

	if (tMutableDictionary!=nil)
	{
		if (locatorController_==nil)
		{
			locatorController_=[ICLocatorEditorController newController];
		}
	
		if (locatorController_!=nil)
		{
			[locatorController_ showEditorForWindow:[document_ windowForSheet] withDictionary:tMutableDictionary file:[array_ objectAtIndex:0] delegate:self];
		}
	}
}

- (IBAction) editLocator:(id) sender
{
	NSInteger tSelectedRow;
	
	tSelectedRow=[IBlocatorsArray_ selectedRow];
	
	if (tSelectedRow!=NSNotFound)
	{
		if (locatorController_==nil)
		{
			locatorController_=[ICLocatorEditorController newController];
		}
	
		if (locatorController_!=nil)
		{
			NSMutableDictionary * tLocatorDictionary;
	
			tLocatorDictionary=[cachedLocators_ objectAtIndex:tSelectedRow];
			
			editLocatorMode_=YES;
						
			[locatorController_ showEditorForWindow:[document_ windowForSheet] withDictionary:tLocatorDictionary file:[array_ objectAtIndex:0] delegate:self];
		}
	}
}

- (IBAction) removeLocator:(id) sender
{
	NSString * tAlertTitle;
    
    if ([IBlocatorsArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this locator?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these locators?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeLocatorsDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeLocatorsDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBlocatorsArray_ selectedRowIndexes];
		
		[IBlocatorsArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedLocators_ removeObjectsAtIndexes:tIndexSet];
			
			[IBlocatorsArray_ reloadData];
			
			[IBremoveLocatorButton_ setEnabled:NO];
			
			// Notify Document Change
			
			[self postAttributesDidChangeNotification];
		}
	}
}

- (NSString *) uniqueLocatorNameWithBaseName:(NSString *) inBaseName
{
	NSString * tString;
    
	tString=nil;
	
	if (inBaseName!=nil)
	{
		NSUInteger _sIndex=0;
		NSUInteger i,tCount;
    
		tCount=[cachedLocators_ count];
    
		do
		{
			NSDictionary * tRequirementDictionary;
			
			if (_sIndex>0)
			{
				tString=[[NSString alloc] initWithFormat:@"%@ %lu",inBaseName,_sIndex];
			}
			else
			{
				tString=[[NSString alloc] initWithString:inBaseName];
			}
			
			_sIndex++;
			
			for(i=0;i<tCount;i++)
			{
				tRequirementDictionary=[cachedLocators_ objectAtIndex:i];
				
				if ([[tRequirementDictionary objectForKey:ICDOCUMENT_LOCATOR_NAME] caseInsensitiveCompare:tString]==NSOrderedSame)
				{
					break;
				}
			}
			
			if (i==tCount)
			{
				break;
			}
			
			[tString release];
		}
		while (_sIndex<65535);
    }
	
    return [tString autorelease];
}

#pragma mark -

- (void) updatePreInstallScriptWithPath:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSFileManager * tFileManager;
		
		tFileManager=[NSFileManager defaultManager];
		
		[IBpreInstallRemoveButton_ setHidden:NO];
		
		
		// Icon
			
		[IBpreInstallIcon_ setFilePath:inPath];
		
		// Name
		
		[IBpreInstallName_ setStringValue:[inPath lastPathComponent]];
		
		
		if( [tFileManager fileExistsAtPath:inPath]==YES)
		{
			NSDictionary * tAttributesDictionary;
			
			// Name
			
			[IBpreInstallName_ setTextColor:[NSColor blackColor]];
			
			// Last Modified
			
			tAttributesDictionary=[tFileManager fileAttributesAtPath:inPath traverseLink:NO];
					
			if (tAttributesDictionary!=nil)
			{
				NSDate * tModificationDate;
				
				tModificationDate=[tAttributesDictionary objectForKey:NSFileModificationDate];
				
				if (tModificationDate!=nil)
				{
					if (dateFormatter_!=nil)
					{
						[IBpreInstallLastModifiedDate_ setStringValue:[dateFormatter_ stringForObjectValue:tModificationDate]];
					}
					else
					{
						[IBpreInstallLastModifiedDate_ setObjectValue:tModificationDate];
					}
				}
			}
		}
		else
		{
			// Name
			
			[IBpreInstallName_ setTextColor:[NSColor redColor]];
			
			// Last Modified
			
			[IBpreInstallLastModifiedDate_ setStringValue:@"-"];
		}
	}
	else
	{
		[IBpreInstallIcon_ setFilePath:nil];
		
		[IBpreInstallName_ setToolTip:@""];
		
		[IBpreInstallName_ setTextColor:[NSColor colorWithDeviceWhite:0.0f alpha:0.15f]];
		
		[IBpreInstallName_ setStringValue:NSLocalizedString(@"No Script Set",@"")];
		
		[IBpreInstallLastModifiedDate_ setStringValue:@"-"];
	
		[IBpreInstallRemoveButton_ setHidden:YES];
	}
}

- (IBAction) switchPreInstallPathType:(id) sender
{
	if (cachedPreInstallPathTypeIndex_!=[IBpreInstallPathType_ indexOfSelectedItem])
    {
		ICFileNode * tFileNode;
		NSMutableDictionary * tPathDictionary;
	   
		cachedPreInstallPathTypeIndex_=[IBpreInstallPathType_ indexOfSelectedItem];
	   
        tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
		
		tPathDictionary=[tFileNode preInstallPath];
		
		if (tPathDictionary!=nil)
		{
			NSString * tPath;
			NSString * tAbsolutePath=nil;
			NSInteger tPathType;
			
			// Get the absolute path
			
			tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
			
			tPathType=[[IBpreInstallPathType_ itemAtIndex:cachedPreInstallPathTypeIndex_] tag];
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
			{
				tAbsolutePath=tPath;
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
			}
			
			// Convert to the new path type
			
			cachedPreInstallPathTypeIndex_=[IBpreInstallPathType_ indexOfSelectedItem];
			
			tPathType=[[IBpreInstallPathType_ itemAtIndex:cachedPreInstallPathTypeIndex_] tag];
			
			[tPathDictionary setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
			{
				tPath=tAbsolutePath;
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tPath=[tAbsolutePath stringByRelativizingToPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tPath=[tAbsolutePath stringByRelativizingToPath:[document_ referenceFolderPath]];
			}
			
			[IBpreInstallName_ setToolTip:tPath];
			
			[tPathDictionary setObject:tPath forKey:ICDOCUMENT_PATH];
			
			// Notify Document Change
			
			[self postAttributesDidChangeNotification];
		}
	}
}

- (IBAction) setPreInstallScript:(id) sender
{
	ICFileNode * tFileNode;
	NSString * tAbsolutePath=nil;
	NSOpenPanel * tOpenPanel;
	NSDictionary * tPathDictionary;
	
	tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
	
	tPathDictionary=[tFileNode preInstallPath];
		
	if (tPathDictionary!=nil)
	{
		NSString * tPath;
		NSNumber * tNumber;
		int tType;
		
		tType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			
		tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
	
		tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
	
		if (tNumber!=nil)
		{
			tType=[tNumber intValue];
		}
		
		if (tType==ICDOCUMENT_PATH_TYPE_GLOBAL)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
		}
		else
		{
			tAbsolutePath=tPath;
		}
	}
	
	tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:NO];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Set",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:NO];
    
   
					
	[tOpenPanel beginSheetForDirectory:tAbsolutePath
						  file:nil
						 types:nil
				modalForWindow:[document_ windowForSheet]
				 modalDelegate:self
				didEndSelector:@selector(setPreInstallScriptDidEnd:returnCode:contextInfo:)
				   contextInfo:NULL];
}

- (void) _setPreInstallScriptPath:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSInteger tPathType;
		NSDictionary * tPathDictionary;
		
		
			
		tPathType=[[IBpreInstallPathType_ itemAtIndex:cachedPreInstallPathTypeIndex_] tag];
		
		if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
		{
			inPath=[inPath stringByRelativizingToPath:[document_ folder]];
		}
		else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
		{
			inPath=[inPath stringByRelativizingToPath:[document_ referenceFolderPath]];
		}
			
		tPathDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																   inPath,ICDOCUMENT_PATH,
																   nil];
		
		if (tPathDictionary!=nil)
		{
			ICFileNode * tFileNode;
			
			tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
			
			[tFileNode setPreInstallPath:tPathDictionary];
			
			// Post Notification
		
			[self postAttributesDidChangeNotification];
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (void) setPreInstallScriptDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        NSString * tPath;
		
		tPath=[sheet filename];
		
		[self updatePreInstallScriptWithPath:tPath];
		
		[self _setPreInstallScriptPath:tPath];
	}
}

- (IBAction) removePreInstallScript:(id) sender
{
	NSString * tAlertTitle;
    
    tAlertTitle=NSLocalizedString(@"Do you really want to remove the preinstall script for this bundle?",@"No comment");
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removePreInstallScriptDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removePreInstallScriptDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		ICFileNode * tFileNode;
			
		tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
		
		[tFileNode setPreInstallPath:nil];
		
		[self updatePreInstallScriptWithPath:nil];
		
		// Post Notification
	
		[self postAttributesDidChangeNotification];
	}
}

- (void) updatePostInstallScriptWithPath:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSFileManager * tFileManager;
		
		tFileManager=[NSFileManager defaultManager];
		
		[IBpostInstallRemoveButton_ setHidden:NO];
		
		// Icon
			
		[IBpostInstallIcon_ setFilePath:inPath];
		
		// Name
		
		[IBpostInstallName_ setToolTip:inPath];
		
		[IBpostInstallName_ setStringValue:[inPath lastPathComponent]];
		
		if( [tFileManager fileExistsAtPath:inPath]==YES)
		{
			NSDictionary * tAttributesDictionary;
			
			// Name
			
			[IBpostInstallName_ setTextColor:[NSColor blackColor]];
			
			// Last Modified
			
			tAttributesDictionary=[tFileManager fileAttributesAtPath:inPath traverseLink:NO];
					
			if (tAttributesDictionary!=nil)
			{
				NSDate * tModificationDate;
				
				tModificationDate=[tAttributesDictionary objectForKey:NSFileModificationDate];
				
				if (tModificationDate!=nil)
				{
					if (dateFormatter_!=nil)
					{
						[IBpostInstallLastModifiedDate_ setStringValue:[dateFormatter_ stringForObjectValue:tModificationDate]];
					}
					else
					{
						[IBpostInstallLastModifiedDate_ setObjectValue:tModificationDate];
					}
				}
			}
		}
		else
		{
			// Name
			
			[IBpostInstallName_ setTextColor:[NSColor redColor]];
			
			[IBpostInstallLastModifiedDate_ setStringValue:@"-"];
		}
	}
	else
	{
		[IBpostInstallIcon_ setFilePath:nil];
		
		[IBpostInstallName_ setTextColor:[NSColor colorWithDeviceWhite:0.0f alpha:0.15f]];
		
		[IBpostInstallName_ setToolTip:@""];
		
		[IBpostInstallName_ setStringValue:NSLocalizedString(@"No Script Set",@"")];
		
		[IBpostInstallLastModifiedDate_ setStringValue:@"-"];
	
		[IBpostInstallRemoveButton_ setHidden:YES];
	}
}

- (IBAction) switchPostInstallPathType:(id) sender
{
	if (cachedPostInstallPathTypeIndex_!=[IBpostInstallPathType_ indexOfSelectedItem])
    {
		ICFileNode * tFileNode;
		NSMutableDictionary * tPathDictionary;
	   
	   cachedPostInstallPathTypeIndex_=[IBpostInstallPathType_ indexOfSelectedItem];
	   
        tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
		
		tPathDictionary=[tFileNode postInstallPath];
		
		if (tPathDictionary!=nil)
		{
			NSString * tPath;
			NSString * tAbsolutePath=nil;
			NSInteger tPathType;
			
			// Get the absolute path
			
			tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
			
			tPathType=[[IBpostInstallPathType_ itemAtIndex:cachedPostInstallPathTypeIndex_] tag];
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
			{
				tAbsolutePath=tPath;
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
			}
			
			// Convert to the new path type
			
			cachedPostInstallPathTypeIndex_=[IBpostInstallPathType_ indexOfSelectedItem];
			
			tPathType=[[IBpostInstallPathType_ itemAtIndex:cachedPostInstallPathTypeIndex_] tag];
			
			[tPathDictionary setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
			{
				tPath=tAbsolutePath;
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tPath=[tAbsolutePath stringByRelativizingToPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tPath=[tAbsolutePath stringByRelativizingToPath:[document_ referenceFolderPath]];
			}
			
			[IBpostInstallName_ setToolTip:tPath];
			
			[tPathDictionary setObject:tPath forKey:ICDOCUMENT_PATH];
			
			// Notify Document Change
			
			[self postAttributesDidChangeNotification];
		}
	}
}

- (IBAction) setPostInstallScript:(id) sender
{
	ICFileNode * tFileNode;
	NSString * tAbsolutePath=nil;
	NSOpenPanel * tOpenPanel;
	NSDictionary * tPathDictionary;
	
	tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
	
	tPathDictionary=[tFileNode postInstallPath];
		
	if (tPathDictionary!=nil)
	{
		NSString * tPath;
		NSNumber * tNumber;
		int tType;
		
		tType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			
		tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
	
		tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
	
		if (tNumber!=nil)
		{
			tType=[tNumber intValue];
		}
		
		if (tType==ICDOCUMENT_PATH_TYPE_GLOBAL)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
		}
		else
		{
			tAbsolutePath=tPath;
		}
	}
	
	tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:NO];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Set",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:NO];
    
   
					
	[tOpenPanel beginSheetForDirectory:tAbsolutePath
						  file:nil
						 types:nil
				modalForWindow:[document_ windowForSheet]
				 modalDelegate:self
				didEndSelector:@selector(setPostInstallScriptDidEnd:returnCode:contextInfo:)
				   contextInfo:NULL];
}

- (void) _setPostInstallScriptPath:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSInteger tPathType;
		NSDictionary * tPathDictionary;
		
		tPathType=[[IBpostInstallPathType_ itemAtIndex:cachedPostInstallPathTypeIndex_] tag];
		
		if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
		{
			inPath=[inPath stringByRelativizingToPath:[document_ folder]];
		}
		else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
		{
			inPath=[inPath stringByRelativizingToPath:[document_ referenceFolderPath]];
		}
			
		tPathDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																   inPath,ICDOCUMENT_PATH,
																   nil];
		
		if (tPathDictionary!=nil)
		{
			ICFileNode * tFileNode;
			
			tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
			
			[tFileNode setPostInstallPath:tPathDictionary];
			
			// Post Notification
		
			[self postAttributesDidChangeNotification];
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (void) setPostInstallScriptDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        NSString * tPath;
		
		tPath=[sheet filename];
		
		[self updatePostInstallScriptWithPath:tPath];
		
		[self _setPostInstallScriptPath:tPath];
	}
}

- (IBAction) removePostInstallScript:(id) sender
{
	NSString * tAlertTitle;
    
    tAlertTitle=NSLocalizedString(@"Do you really want to remove the postinstall script for this bundle?",@"No comment");
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removePostInstallScriptDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removePostInstallScriptDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		ICFileNode * tFileNode;
			
		tFileNode=FILENODE_DATA([array_ objectAtIndex:0]);
		
		[tFileNode setPostInstallPath:nil];
		
		[self updatePostInstallScriptWithPath:nil];
		
		// Post Notification
	
		[self postAttributesDidChangeNotification];
	}
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBlocatorsArray_)
    {
		NSInteger tNumberOfSelectedRows;
		
		tNumberOfSelectedRows=[IBlocatorsArray_ numberOfSelectedRows];
		
		[IBeditLocatorButton_ setEnabled:(tNumberOfSelectedRows==1)];
		
		[IBremoveLocatorButton_ setEnabled:(tNumberOfSelectedRows!=0)];
	}
	
}

- (void) locatorEditionDidEnd:(ICLocatorEditorController *) inEditor returnCode:(NSInteger) inReturnCode
{
	if (inReturnCode==NSOKButton)
	{
		NSMutableDictionary * tLocatorDictionary;
		
		tLocatorDictionary=[inEditor dictionary];
		
		if (tLocatorDictionary!=nil)
		{
			NSInteger tRowIndex;
			NSString * tSuggestedName;
			
			tSuggestedName=[tLocatorDictionary objectForKey:@"SuggestedName"];
			
			if (tSuggestedName!=nil)
			{
				NSString * tUniqueName;
				
				tUniqueName=[self uniqueLocatorNameWithBaseName:tSuggestedName];
				
				[tLocatorDictionary removeObjectForKey:@"SuggestedName"];
				
				[tLocatorDictionary setObject:tUniqueName forKey:ICDOCUMENT_LOCATOR_NAME];
			}
			
			if (editLocatorMode_==YES)
			{
				editLocatorMode_=NO;
				
				tRowIndex=[IBlocatorsArray_ selectedRow];
				
				if (tRowIndex!=NSNotFound)
				{
					[cachedLocators_ replaceObjectAtIndex:tRowIndex withObject:tLocatorDictionary];
					
					[IBlocatorsArray_ reloadData];
					
					[IBlocatorsArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				}
			}
			else
			{
				[cachedLocators_ addObject:tLocatorDictionary];
				
				[IBlocatorsArray_ reloadData];
				
				tRowIndex=[cachedLocators_ indexOfObject:tLocatorDictionary];
				
				[IBlocatorsArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
				[IBlocatorsArray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
			}
			
			// Notify Document Change
			
			[self postAttributesDidChangeNotification];
		}
	}
}

- (void) windowStateDidChange:(NSNotification *) inNotification
{
	[self updateSingleSelection];
}

- (void) dropViewFilePathDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		id tObject;
		NSString * tPath;
		
		tObject=[inNotification object];
		
		tPath=[[inNotification userInfo] objectForKey:@"Path"];
		
		if (tObject==IBpreInstallIcon_)
		{
			[self _setPreInstallScriptPath:tPath];
			
			[self updatePreInstallScriptWithPath:tPath];
		}
		else if (tObject==IBpostInstallIcon_)
		{
			[self _setPostInstallScriptPath:tPath];
			
			[self updatePostInstallScriptWithPath:tPath];
		}
	}
}

@end
