/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICPackageScriptsController.h"

#import "ICPackageScriptsUtility.h"

#import "ICPackageScriptsInspectorSelectionController.h"

#import "ICFileDropView.h"

@implementation ICPackageScriptsController

+ (ICPackageScriptsController *) packageScriptsController
{
	ICPackageScriptsController * nController=nil;
    
    nController=[ICPackageScriptsController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Package_Scripts" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Package_Scripts"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) dealloc
{
	
	[super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{
	[super awakeFromNib];
	
	dateFormatter_=[NSDateFormatter new];
	
	if (dateFormatter_!=nil)
	{
		[dateFormatter_ setFormatterBehavior:NSDateFormatterBehavior10_4];
	
		[dateFormatter_ setDateStyle:NSDateFormatterMediumStyle];
		
		[dateFormatter_ setTimeStyle:NSDateFormatterShortStyle];
	}
	
	// Outline View
	
	[IBoutlineView_ registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType,
                                                                      ICFileHierarchyPboardType,
																	  ICFileHierarchyExternalPboardType,
                                                                      nil]];

	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(scriptsViewDidResize:)
											     name:NSViewFrameDidChangeNotification
											   object:IBscriptsView_];

	[[NSNotificationCenter defaultCenter]  addObserver:self
											 selector:@selector(dropViewFilePathDidChange:) 
												 name:ICDropViewFilePathDidChangeNotification 
											   object:IBpreInstallIcon_];
											   
	[[NSNotificationCenter defaultCenter]  addObserver:self
											 selector:@selector(dropViewFilePathDidChange:) 
												 name:ICDropViewFilePathDidChangeNotification 
											   object:IBpostInstallIcon_];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{	
	[super initializeWithDictionary:inDictionary forDocument:inDocument];

	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PACKAGE_SCRIPTS];
	
	if (dictionary_==nil)
	{
		// Build the default dictionary
		
		dictionary_=[ICPackageScriptsUtility defaultPackageScriptsDictionary];
		
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PACKAGE_SCRIPTS];
		}
	}
	
	if (dictionary_!=nil)
	{
		[cachedPackageUUID_ release];
		
		cachedPackageUUID_=[[inDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID] retain];
		
		[document_ updateFileBouncer];
		
		cachedFileBouncer_=[[document_ fileBouncer] retain];
		
		[dictionary_ retain];
		
		[self performSelector:@selector(delayedHierarchyLoad:) withObject:nil afterDelay:0.1f];
		
		// Register for Notifications
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_FOLDERNAME_SHOULD_CHANGE object:document_];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(folderNameShouldChange:)
                                                 name:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_FOLDERNAME_SHOULD_CHANGE
                                               object:document_];
	
		[[NSNotificationCenter defaultCenter] removeObserver:self name:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_ATTRIBUTES_DID_CHANGE object:document_];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(fileAttributesDidChange:)
													 name:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_ATTRIBUTES_DID_CHANGE
												   object:document_];
	}
}

- (void) delayedHierarchyLoad:(id) sender
{
	NSDictionary * tPathDictionary;
	NSString * tPath;
	NSString * tAbsolutePath;
	NSNumber * tNumber;
	int tType;
		
	// Pre-install Path
	
	cachedPreInstallPathTypeIndex_=0;
	
	tPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SCRIPTS_PREINSTALL_PATH];
	
	if (tPathDictionary!=nil)
	{
		tType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			
		tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
	
		tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
	
		if (tNumber!=nil)
		{
			tType=[tNumber intValue];
		}
		
		if (tType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
		}
		else if (tType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
		}
		else
		{
			tAbsolutePath=tPath;
		}
		
		[IBpreInstallName_ setToolTip:tPath];
		
		[self updatePreInstallScriptWithPath:tAbsolutePath];
		
		// Type
		
		// File Reference
		
		cachedPreInstallPathTypeIndex_=[IBpreInstallPathType_ indexOfItemWithTag:tType];
				
		[IBpreInstallPathType_ selectItemAtIndex:cachedPreInstallPathTypeIndex_];

	}
	else
	{
		[self updatePreInstallScriptWithPath:nil];
	}
	
	// Post-install Path
	
	tPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SCRIPTS_POSTINSTALL_PATH];
	
	cachedPostInstallPathTypeIndex_=0;
	
	if (tPathDictionary!=nil)
	{
		tType=ICDOCUMENT_PATH_TYPE_GLOBAL;
		
		tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
	
		tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
	
		if (tNumber!=nil)
		{
			tType=[tNumber intValue];
		}
		
		if (tType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
		}
		else if (tType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
		}
		else
		{
			tAbsolutePath=tPath;
		}
		
		[IBpostInstallName_ setToolTip:tPath];

		
		[self updatePostInstallScriptWithPath:tAbsolutePath];
		
		// Type
		
		// File Reference
		
		cachedPostInstallPathTypeIndex_=[IBpostInstallPathType_ indexOfItemWithTag:tType];
				
		[IBpostInstallPathType_ selectItemAtIndex:cachedPostInstallPathTypeIndex_];

	}
	else
	{
		[IBpostInstallName_ setToolTip:@""];
		
		[self updatePostInstallScriptWithPath:nil];
	}
	
	// File Hierarchy
	
	fileTree_=[ICFileTree fileTreeWithArray:[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SCRIPTS_FILE_HIERARCHY] projectPath:[document_ folder] referenceFolderPath:[document_ referenceFolderPath]];
	
	if (fileTree_!=nil)
	{
		[fileTree_ retain];
		
		[IBoutlineView_ reloadData];
		
		[self restoreDisclosedStates];
	}
	else
	{
	
		// A COMPLETER
	}
	
	
	
	// Restore Selection
	
	[self restoreSelection];
	
	[self outlineViewSelectionDidChange:nil];
}

- (IBAction) updateFiles:(id) sender
{
	// Save File Hierarchy if needed
	
	if (hierarchyChanged_==YES)
	{
		NSArray * tArray;
		
		tArray=[fileTree_ arrayWithProjectAtPath:[document_ folder] referenceFolderPath:[document_ referenceFolderPath]];
			
		if (tArray!=nil)
		{
			[dictionary_ setObject:tArray forKey:ICDOCUMENT_PACKAGE_SCRIPTS_FILE_HIERARCHY];
		}
		
		hierarchyChanged_=NO;
	}
	
	// Notify Document Change
			
	[self noteDocumentChange];
}

#pragma mark -

- (NSString *) disclosedStateFormatKey
{
	return ICSCRIPTS_HIERARCHY_DISCLOSED_STATE;
}

- (NSString *) selectionFormatKey
{
	return ICSCRIPTS_HIERARCHY_SELECTION;
}

#pragma mark -

- (id) outlineView:(NSOutlineView *) inOutlineView objectValueForTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
    id objectValue = nil;
    ICFileNode * tFileNode;
    
    tFileNode=FILENODE_DATA(inItem);
    
    if([[inTableColumn identifier] isEqualToString: @"File"])
    {
        objectValue = [tFileNode fileName];
	}
    else
    if([[inTableColumn identifier] isEqualToString: @"Permissions"])
    {
        if ([tFileNode link]==YES)
        {
            objectValue= [FILENODE_DATA([inItem nodeParent]) privilegesStringRepresentationForLink];
        }
        else
        {
            objectValue= [tFileNode privilegesStringRepresentation];
        }
    }
    
    return objectValue;
}

- (void) outlineView:(NSOutlineView *) inOutlineView willDisplayCell:(NSCell *)cell forTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{    
    if ([[inTableColumn identifier] isEqualToString: @"File"])
    {
		ICFileNode * tFileNode;
        BOOL tFather=YES;
        NSImage * tIcon=nil;
		NSString * tFileName;
		int tFileType;
		NSColor * tColor;
		NSMutableDictionary * tMutableAttributesDictionary;
		
        tFileNode=FILENODE_DATA(inItem);
        
		tFileType=[tFileNode type];
		
		tFileName=[tFileNode fileName];
		
		tColor=[NSColor blackColor];
		
        switch(tFileType)
        {
           case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				
				if (tIcon==nil)
				{
					if (tFather==YES)
					{
						tIcon=genericFolderIcon_;
					}
					else
					{
						tIcon=genericDisabledFolderIcon_;
					}
                }
				
				if (tIcon!=nil)
				{
					[(ImageAndTextCell*)cell setImage:tIcon];
				}
				
				break;
            case ICDOCUMENT_FILE_REALFILE_NODE:
                
                [tFileNode fileExistenceOnDiskChanged];
                
                if ([tFileNode existsOnDisk]==NO)
                {
                    tColor=[NSColor redColor];
                }
                
                [(ImageAndTextCell*)cell setImage: [FILENODE_DATA(inItem) icon]];
				
                break;
        }
		
		tMutableAttributesDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tColor,NSForegroundColorAttributeName,
																					   [cell font],NSFontAttributeName,
																					   nil];
		
		if (cachedHighlightExcludedFiles_==YES && cachedFileBouncer_!=nil)
		{
			if ([inItem isItemExcludedByBouncer:cachedFileBouncer_]==YES)
			{
				[tMutableAttributesDictionary setObject:[NSNumber numberWithInt:NSUnderlineStyleSingle] forKey:NSStrikethroughStyleAttributeName];
			}
		}
		
		[fileNameFormatter_ setCachedAttributes:tMutableAttributesDictionary];
			
		[(ImageAndTextCell*)cell setStringValue:tFileName];
	}
}

#pragma mark -

- (BOOL) outlineView:(NSOutlineView *) inOutlineView writeItems:(NSArray*) inItems toPasteboard:(NSPasteboard*)pboard
{
    NSUInteger i,tCount;
    ICFileTree * tTree;
    NSMutableArray * tExternalDragArray;
	
    // We need to simplify the hierarchy and check no items is a base node
    
    tCount=[inItems count];
    
	[internalDragArray_ release];
	
    internalDragArray_=[[NSMutableArray alloc] initWithCapacity:tCount];
    
    tExternalDragArray=[NSMutableArray arrayWithCapacity:tCount];
    
    for(i=tCount;i>0;i--)
    {
        NSUInteger j;
		int tType;
		
		tTree=[inItems objectAtIndex:i-1];
        
        tType=[FILENODE_DATA(tTree) type];
		
		if (tType==ICDOCUMENT_FILE_BASE_NODE_INVISIBLE ||
			tType==ICDOCUMENT_FILE_BASE_NODE)
        {
            return NO;
        }
        
        for(j=i-1;j>0;j--)
        {
            if ([tTree isDescendantOfNode:[inItems objectAtIndex:j-1]]==YES)
            {
                break;
            }
        }
        
        if (j==0)
        {
            [internalDragArray_ insertObject:tTree atIndex:0];
            
            [tExternalDragArray insertObject:[tTree dictionaryWithProjectAtPath:nil referenceFolderPath:nil] atIndex:0];
        }
    }
    
    [pboard declareTypes:[NSArray arrayWithObjects: ICFileHierarchyPboardType,ICFileHierarchyExternalPboardType,nil] owner:self];
    
    [pboard setPropertyList:tExternalDragArray forType:ICFileHierarchyExternalPboardType];
	
    [pboard setPropertyList:[NSData data] forType:ICFileHierarchyPboardType]; 
    
    return YES;
}

- (NSDragOperation) outlineView:(NSOutlineView*) inOutlineView validateDrop:(id <NSDraggingInfo>)info proposedItem:(id) inItem proposedChildIndex:(NSInteger)childIndex
{
    NSPasteboard * tPasteBoard;
    NSArray * tArray;
    NSUInteger i,tCount;
    
    tPasteBoard=[info draggingPasteboard];
    
    switch(childIndex)
    {
        case NSOutlineViewDropOnItemIndex:
            break;
        default:
            
            if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject: NSFilenamesPboardType]]!=nil)
            {
                if ([fileTree_ numberOfChildren]==0)
				{
					[inOutlineView setDropItem:inItem dropChildIndex:-1];
				}
				else
				{
					NSString * tPath;
					ICFileTree * tChildTree;
					NSArray * tChildren;
					NSString * tName;
					NSUInteger k,tChildrenCount;
					BOOL find=NO;
					
					tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
							
					tCount=[tArray count];
							
					for(i=0;i<tCount;i++)
					{
						tPath=[tArray objectAtIndex:i];
								
						if ([inItem containsTreeWithName:[tPath lastPathComponent]]==YES)
						{
							return NSDragOperationNone;
						}
					}
					
					tPath=[tArray objectAtIndex:0];
					
					tName=[tPath lastPathComponent];
					
					tChildren=[inItem children];
					
					tChildrenCount=[tChildren count];
					
					for(k=0;k<tChildrenCount;k++)
					{
						tChildTree=[tChildren objectAtIndex:k];
						
						if ([tName compare:[FILENODE_DATA(tChildTree) fileName] options:NSCaseInsensitiveSearch]!=NSOrderedDescending)
						{
							[inOutlineView setDropItem:inItem dropChildIndex:k];
							find=YES;
							
							break;
						}
					}
					
					if (find==NO)
					{
						[inOutlineView setDropItem:inItem dropChildIndex:k];
					}
                }
				
                return NSDragOperationCopy;//[info draggingSourceOperationMask];
            }
            else
            {
                ICFileTree * tFileTree;
                NSString * tName=nil;
                
                if ([info draggingSource]==IBoutlineView_)
                {
					if ([inItem isDescendantOfNodeInArray:internalDragArray_]==YES)
                    {
                        return NSDragOperationNone;
                    }
                    
                    // We need to check the name and eventually switch the drop location
                    
                    tCount=[internalDragArray_ count];
                    
                    for(i=0;i<tCount;i++)
                    {
                        tFileTree=[internalDragArray_ objectAtIndex:i];
                        
                        tName=[FILENODE_DATA(tFileTree) fileName];
                        
                        if ([inItem containsTreeWithName:tName]==YES)
                        {
                            return NSDragOperationNone;
                        }
                    }
                    
                    tFileTree=[internalDragArray_ objectAtIndex:0];
                    
                    tName=[FILENODE_DATA(tFileTree) fileName];
                }
                else
                {
                    // External Drag
                    
                    NSArray * tExternalArray;
                    NSDictionary * tDictionary;
                    
                    tExternalArray=[[info draggingPasteboard] propertyListForType:ICFileHierarchyExternalPboardType];
                    
                    // We need to check the name and eventually switch the drop location
                    
                    tCount=[tExternalArray count];
                    
                    for(i=0;i<tCount;i++)
                    {
                        tDictionary=[tExternalArray objectAtIndex:i];
                        
                        if (tDictionary!=nil)
                        {
                            tName=[tDictionary objectForKey:ICDOCUMENT_PATH];
                        
                            if (tName==nil)
                            {
                                return NSDragOperationNone;
                            }
                            else
                            {
                                tName=[tName lastPathComponent];
                                
                                if ([inItem containsTreeWithName:tName]==YES)
                                {
                                    return NSDragOperationNone;
                                }
                            }
                        }
                    }
                    
                    tDictionary=[tExternalArray objectAtIndex:0];
                    
                    tName=[[tDictionary objectForKey:ICDOCUMENT_PATH] lastPathComponent];
                }
                    
                // Find the appropriate drop location based on the first name of the dropped objects
                
                {
                    ICFileTree * tChildTree;
                    NSArray * tChildren;
                    NSUInteger k,tChildrenCount;
                    BOOL find=NO;
                
                    tChildren=[inItem children];
                
                    tChildrenCount=[tChildren count];
                    
                    for(k=0;k<tChildrenCount;k++)
                    {
                        tChildTree=[tChildren objectAtIndex:k];
                        
                        if ([tName compare:[FILENODE_DATA(tChildTree) fileName] options:NSCaseInsensitiveSearch]!=NSOrderedDescending)
                        {
                            [inOutlineView setDropItem:inItem dropChildIndex:k];
							
                            find=NO;
                            
                            break;
                        }
                    }
                    
                    if (find==NO)
                    {
                        [inOutlineView setDropItem:inItem dropChildIndex:k];
                    }
                }
                
                return NSDragOperationGeneric;
            }
            break;
    }
    
    return NSDragOperationNone;
}

- (BOOL) outlineView:(NSOutlineView*) inOutlineView acceptDrop:(id <NSDraggingInfo>)info item:(id)targetItem childIndex:(NSInteger)childIndex
{
    ICFileTree * tFileTree;
    NSPasteboard * tPasteBoard;
    NSMutableArray * tNewSelectionArray;
    NSUInteger k,tNewCount;
    NSMutableIndexSet * tMutableIndexSet;
    tPasteBoard=[info draggingPasteboard];
    
	if (targetItem==nil)
	{
		targetItem=fileTree_;
	}
	
    tNewSelectionArray=[NSMutableArray array];
    
    if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject: NSFilenamesPboardType]]!=nil)
    {
        // Drag & Drop of a real item
        
        NSArray * tArray;
        BOOL tShowCustomizationDialog=YES;
		id tObject;
		
        tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
		tObject=[defaults_ objectForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
	
		if (tObject!=nil)
		{
			tShowCustomizationDialog=[defaults_ boolForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
		}
		
		if (tShowCustomizationDialog==YES)
		{
			[self beginFileSheetForFiles:tArray parents:[NSArray arrayWithObject:targetItem]];
		}
		else
		{
			NSInteger tDefaultReferenceStyle;
			
			tDefaultReferenceStyle=[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
			
			[self _addFiles:tArray toParents:[NSArray arrayWithObject:targetItem] keepOwnerAndGroup:NO referenceStyle:tDefaultReferenceStyle];
		}
        
        return YES;
    }
    else
    {
        if ([info draggingSource]==IBoutlineView_)
        {
            NSMutableDictionary * tDisclosedDictionary;
			NSArray * tCoverageNodesArray;
			
			// Internal Drag, we need to move
            
			tDisclosedDictionary=[document_ documentDefaultForKey:[self disclosedStateKey]];
			
            // Find the coverage tree
			
			tCoverageNodesArray=[TreeNode minimumNodeCoverFromNodesInArray:internalDragArray_];
			
			if (tCoverageNodesArray!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tCoverageNodesArray objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSNumber * tNumber;
					
					while (tFileTree=[tEnumerator nextObject])
					{
						NSMutableArray * tChildFoldersArray;
		
						// Removed item and descendants from disclosed state dictionary (because the key will change)
						
						tChildFoldersArray=[tFileTree descendantFoldersIncludingSelf:YES];
						
						if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
						{
							NSUInteger i,tCount;
							
							tCount=[tChildFoldersArray count];
							
							for(i=tCount;i>0;i--)
							{
								ICFileTree * tGroupFileTree;
								NSString * tPath;
								
								tGroupFileTree=[tChildFoldersArray objectAtIndex:(i-1)];
								
								tPath=[tGroupFileTree filePath];
								
								if (tPath!=nil)
								{
									BOOL tRemoved=NO;
									
									tNumber=[tDisclosedDictionary objectForKey:tPath];
									
									if (tNumber==nil)
									{
										tRemoved=YES;
									}
									else
									{
										if ([tNumber boolValue]==NO)
										{
											tRemoved=YES;
										}
										
										[tDisclosedDictionary removeObjectForKey:tPath];
									}
									
									if (tRemoved==YES)
									{
										[tChildFoldersArray removeObject:tGroupFileTree];
									}
								}
							}
						}
						
						[tFileTree retain];
						
						[tFileTree removeFromParent];
						
						[targetItem insertSortedChild:tFileTree];
						
						[tFileTree release];
						
						[tNewSelectionArray addObject:tFileTree];
						
						// Put back the item and descendant folders disclosed state
			
						if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
						{
							NSEnumerator * tChildEnumerator;
							
							tChildEnumerator=[tChildFoldersArray objectEnumerator];
							
							if (tChildEnumerator!=nil)
							{
								ICFileTree * tGroupFileTree;
								
								while (tGroupFileTree=[tChildEnumerator nextObject])
								{
									NSString * tPath;
									
									tPath=[tGroupFileTree filePath];
								
									if (tPath!=nil)
									{
										[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tPath];
									}
								}
							}
						}

					}
				}
			}
        }
        else
        {
            // External Drag, we copy
            
            NSArray * tExternalArray;
			NSUInteger i,tCount;
			NSNumber * tPathStyleNumber;
			
			tPathStyleNumber=[NSNumber numberWithLong:[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE]];
			
            tExternalArray=[tPasteBoard propertyListForType:ICFileHierarchyExternalPboardType];
            
            tCount=[tExternalArray count];
            
            for(i=0;i<tCount;i++)
            {
                NSMutableDictionary * tDictionary;
                
                tDictionary=[tExternalArray objectAtIndex:i];
                
				[tDictionary setObject:tPathStyleNumber forKey:ICDOCUMENT_PATH_TYPE];
				
                tFileTree=[ICFileTree fileNodeWithDictionary:tDictionary projectPath:[document_ folder] referenceFolderPath:[document_ referenceFolderPath]];
                
                [targetItem insertSortedChild:tFileTree];
                
                [tNewSelectionArray addObject:tFileTree];
            }
        }
    }
    
    [IBoutlineView_ deselectAll:nil];
    
    [IBoutlineView_ reloadData];
    
    tMutableIndexSet=[NSMutableIndexSet indexSet];
	
	if (tMutableIndexSet!=nil)
	{
		tNewCount=[tNewSelectionArray count];
		
		for(k=0;k<tNewCount;k++)
		{
			[tMutableIndexSet addIndex:[IBoutlineView_ rowForItem:[tNewSelectionArray objectAtIndex:k]]];
		}
		
		[IBoutlineView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:NO];
	}
    
    hierarchyChanged_=YES;
	
	[self updateFiles:IBoutlineView_];
    
    return YES;
}

#pragma mark -

- (ICFileTree *) fileTreeWithSelectionPath:(NSString *) inPath
{
	return [fileTree_ fileTreeAtPath:inPath];
}

- (NSString *) selectionPathWithFileTree:(ICFileTree *) inFileTree
{
	return [inFileTree filePath];
}

#pragma mark -

- (void) updatePreInstallScriptWithPath:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSFileManager * tFileManager;
		
		tFileManager=[NSFileManager defaultManager];
		
		[IBpreInstallRemoveButton_ setHidden:NO];
		
		
		// Icon
			
		[IBpreInstallIcon_ setFilePath:inPath];
		
		// Name
		
		[IBpreInstallName_ setStringValue:[inPath lastPathComponent]];
		
		
		if( [tFileManager fileExistsAtPath:inPath]==YES)
		{
			NSDictionary * tAttributesDictionary;
			
			// Name
			
			[IBpreInstallName_ setTextColor:[NSColor blackColor]];
			
			// Last Modified
			
			tAttributesDictionary=[tFileManager fileAttributesAtPath:inPath traverseLink:NO];
					
			if (tAttributesDictionary!=nil)
			{
				NSDate * tModificationDate;
				
				tModificationDate=[tAttributesDictionary objectForKey:NSFileModificationDate];
				
				if (tModificationDate!=nil)
				{
					if (dateFormatter_!=nil)
					{
						[IBpreInstallLastModifiedDate_ setStringValue:[dateFormatter_ stringForObjectValue:tModificationDate]];
					}
					else
					{
						[IBpreInstallLastModifiedDate_ setObjectValue:tModificationDate];
					}
				}
			}
		}
		else
		{
			// Name
			
			[IBpreInstallName_ setTextColor:[NSColor redColor]];
			
			// Last Modified
			
			[IBpreInstallLastModifiedDate_ setStringValue:@"-"];
		}
	}
	else
	{
		[IBpreInstallIcon_ setFilePath:nil];
		
		[IBpreInstallName_ setToolTip:@""];
		
		[IBpreInstallName_ setTextColor:[NSColor colorWithDeviceWhite:0.0f alpha:0.15f]];
		
		[IBpreInstallName_ setStringValue:NSLocalizedString(@"No Script Set",@"")];
		
		[IBpreInstallLastModifiedDate_ setStringValue:@"-"];
	
		[IBpreInstallRemoveButton_ setHidden:YES];
	}
}

- (IBAction) switchPreInstallPathType:(id) sender
{
	if (cachedPreInstallPathTypeIndex_!=[IBpreInstallPathType_ indexOfSelectedItem])
    {
		NSMutableDictionary * tPathDictionary;
		
		tPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SCRIPTS_PREINSTALL_PATH];
		
		if (tPathDictionary!=nil)
		{
			NSString * tPath;
			NSString * tAbsolutePath=nil;
			NSInteger tPathType;
				
			// Get the absolute path
			
			tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
			
			if (tPath!=nil)
			{
				tPathType=[[IBpreInstallPathType_ itemAtIndex:cachedPreInstallPathTypeIndex_] tag];
				
				if (tPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
				{
					tAbsolutePath=tPath;
				}
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
				{
					tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
				}
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
				{
					tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
				}
			}
				
			// Convert to the new path type
				
			cachedPreInstallPathTypeIndex_=[IBpreInstallPathType_ indexOfSelectedItem];
				
			tPathType=[[IBpreInstallPathType_ itemAtIndex:cachedPreInstallPathTypeIndex_] tag];
				
			if (tPath!=nil)
			{	
				if (tPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
				{
					tPath=tAbsolutePath;
				}
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
				{
					tPath=[tAbsolutePath stringByRelativizingToPath:[document_ folder]];
				}
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
				{
					tPath=[tAbsolutePath stringByRelativizingToPath:[document_ referenceFolderPath]];
				}
				
				[IBpreInstallName_ setToolTip:tPath];
				
				[tPathDictionary setObject:tPath forKey:ICDOCUMENT_PATH];
			}
			
			[tPathDictionary setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
	}
}

- (IBAction) setPreInstallScript:(id) sender
{
	NSString * tAbsolutePath=nil;
	NSOpenPanel * tOpenPanel;
	NSDictionary * tPathDictionary;
	
	tPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SCRIPTS_PREINSTALL_PATH];
		
	if (tPathDictionary!=nil)
	{
		NSString * tPath;
		NSNumber * tNumber;
		int tType;
		
		tType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			
		tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
	
		tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
	
		if (tNumber!=nil)
		{
			tType=[tNumber intValue];
		}
		
		if (tType==ICDOCUMENT_PATH_TYPE_GLOBAL)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
		}
		else
		{
			tAbsolutePath=tPath;
		}
	}
	
	tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:NO];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Set",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:NO];
    
   
					
	[tOpenPanel beginSheetForDirectory:tAbsolutePath
						  file:nil
						 types:nil
				modalForWindow:[document_ windowForSheet]
				 modalDelegate:self
				didEndSelector:@selector(setPreInstallScriptDidEnd:returnCode:contextInfo:)
				   contextInfo:NULL];
}

- (void) _setPreInstallScriptPath:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSInteger tPathType;
		NSDictionary * tPathDictionary;
		
		tPathType=[[IBpreInstallPathType_ itemAtIndex:cachedPreInstallPathTypeIndex_] tag];
		
		if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
		{
			inPath=[inPath stringByRelativizingToPath:[document_ folder]];
		}
		else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
		{
			inPath=[inPath stringByRelativizingToPath:[document_ referenceFolderPath]];
		}
		
		[IBpreInstallName_ setToolTip:inPath];
		
		tPathDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																   inPath,ICDOCUMENT_PATH,
																   nil];
		
		if (tPathDictionary!=nil)
		{
			[dictionary_ setObject:tPathDictionary forKey:ICDOCUMENT_PACKAGE_SCRIPTS_PREINSTALL_PATH];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (void) setPreInstallScriptDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        NSString * tPath;
		
		tPath=[sheet filename];
		
		[self updatePreInstallScriptWithPath:tPath];
		
		[self _setPreInstallScriptPath:tPath];
	}
}

- (IBAction) removePreInstallScript:(id) sender
{
	NSString * tAlertTitle;
    
    tAlertTitle=NSLocalizedString(@"Do you really want to remove the preinstall script for this package?",@"No comment");
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removePreInstallScriptDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removePreInstallScriptDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		[dictionary_ removeObjectForKey:ICDOCUMENT_PACKAGE_SCRIPTS_PREINSTALL_PATH];
		
		[self updatePreInstallScriptWithPath:nil];
		
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (void) updatePostInstallScriptWithPath:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSFileManager * tFileManager;
		
		tFileManager=[NSFileManager defaultManager];
		
		[IBpostInstallRemoveButton_ setHidden:NO];
		
		// Icon
			
		[IBpostInstallIcon_ setFilePath:inPath];
		
		// Name
		
		[IBpostInstallName_ setStringValue:[inPath lastPathComponent]];
		
		if( [tFileManager fileExistsAtPath:inPath]==YES)
		{
			NSDictionary * tAttributesDictionary;
			
			// Name
			
			[IBpostInstallName_ setTextColor:[NSColor blackColor]];
			
			// Last Modified
			
			tAttributesDictionary=[tFileManager fileAttributesAtPath:inPath traverseLink:NO];
					
			if (tAttributesDictionary!=nil)
			{
				NSDate * tModificationDate;
				
				tModificationDate=[tAttributesDictionary objectForKey:NSFileModificationDate];
				
				if (tModificationDate!=nil)
				{
					if (dateFormatter_!=nil)
					{
						[IBpostInstallLastModifiedDate_ setStringValue:[dateFormatter_ stringForObjectValue:tModificationDate]];
					}
					else
					{
						[IBpostInstallLastModifiedDate_ setObjectValue:tModificationDate];
					}
				}
			}
		}
		else
		{
			// Name
			
			[IBpostInstallName_ setTextColor:[NSColor redColor]];
			
			[IBpostInstallLastModifiedDate_ setStringValue:@"-"];
		}
	}
	else
	{
		[IBpostInstallIcon_ setFilePath:nil];
		
		[IBpostInstallName_ setTextColor:[NSColor colorWithDeviceWhite:0.0f alpha:0.15f]];
		
		[IBpostInstallName_ setToolTip:@""];
		
		[IBpostInstallName_ setStringValue:NSLocalizedString(@"No Script Set",@"")];
		
		[IBpostInstallLastModifiedDate_ setStringValue:@"-"];
	
		[IBpostInstallRemoveButton_ setHidden:YES];
	}
}

- (IBAction) switchPostInstallPathType:(id) sender
{
	if (cachedPostInstallPathTypeIndex_!=[IBpostInstallPathType_ indexOfSelectedItem])
    {
		NSMutableDictionary * tPathDictionary;
	   
	   cachedPostInstallPathTypeIndex_=[IBpostInstallPathType_ indexOfSelectedItem];
	   
	   tPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SCRIPTS_POSTINSTALL_PATH];
		
		if (tPathDictionary!=nil)
		{
			NSString * tPath;
			NSString * tAbsolutePath=nil;
			NSInteger tPathType;
				
			// Get the absolute path
			
			tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
			
			if (tPath!=nil)
			{
				tPathType=[[IBpostInstallPathType_ itemAtIndex:cachedPostInstallPathTypeIndex_] tag];
				
				if (tPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
				{
					tAbsolutePath=tPath;
				}
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
				{
					tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
				}
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
				{
					tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
				}
			}
			
			// Convert to the new path type
				
			cachedPostInstallPathTypeIndex_=[IBpostInstallPathType_ indexOfSelectedItem];
				
			tPathType=[[IBpostInstallPathType_ itemAtIndex:cachedPostInstallPathTypeIndex_] tag];
			
			if (tPath!=nil)
			{
				if (tPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
				{
					tPath=tAbsolutePath;
				}
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
				{
					tPath=[tAbsolutePath stringByRelativizingToPath:[document_ folder]];
				}
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
				{
					tPath=[tAbsolutePath stringByRelativizingToPath:[document_ referenceFolderPath]];
				}
				
				[IBpostInstallName_ setToolTip:tPath];
				
				[tPathDictionary setObject:tPath forKey:ICDOCUMENT_PATH];
			}
			
			[tPathDictionary setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
	}
}

- (IBAction) setPostInstallScript:(id) sender
{
	NSString * tAbsolutePath=nil;
	NSOpenPanel * tOpenPanel;
	NSDictionary * tPathDictionary;
	
	tPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SCRIPTS_POSTINSTALL_PATH];
		
	if (tPathDictionary!=nil)
	{
		NSString * tPath;
		NSNumber * tNumber;
		int tType;
		
		tType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			
		tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
	
		tNumber=[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
	
		if (tNumber!=nil)
		{
			tType=[tNumber intValue];
		}
		
		if (tType==ICDOCUMENT_PATH_TYPE_GLOBAL)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
		}
		else
		{
			tAbsolutePath=tPath;
		}
	}
	
	tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:NO];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Set",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:NO];
    
   
					
	[tOpenPanel beginSheetForDirectory:tAbsolutePath
						  file:nil
						 types:nil
				modalForWindow:[document_ windowForSheet]
				 modalDelegate:self
				didEndSelector:@selector(setPostInstallScriptDidEnd:returnCode:contextInfo:)
				   contextInfo:NULL];
}

- (void) _setPostInstallScriptPath:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSInteger tPathType;
		NSDictionary * tPathDictionary;
		
		tPathType=[[IBpostInstallPathType_ itemAtIndex:cachedPostInstallPathTypeIndex_] tag];
		
		if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
		{
			inPath=[inPath stringByRelativizingToPath:[document_ folder]];
		}
		else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
		{
			inPath=[inPath stringByRelativizingToPath:[document_ referenceFolderPath]];
		}
			
		[IBpostInstallName_ setToolTip:inPath];
		
		tPathDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																		  inPath,ICDOCUMENT_PATH,
																		  nil];
		
		if (tPathDictionary!=nil)
		{
			[dictionary_ setObject:tPathDictionary forKey:ICDOCUMENT_PACKAGE_SCRIPTS_POSTINSTALL_PATH];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (void) setPostInstallScriptDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        NSString * tPath;
		
		tPath=[sheet filename];
		
		[self updatePostInstallScriptWithPath:tPath];
		
		[self _setPostInstallScriptPath:tPath];
	}
}

- (IBAction) removePostInstallScript:(id) sender
{
	NSString * tAlertTitle;
    
    tAlertTitle=NSLocalizedString(@"Do you really want to remove the postinstall script for this package?",@"No comment");
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removePostInstallScriptDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removePostInstallScriptDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		[dictionary_ removeObjectForKey:ICDOCUMENT_PACKAGE_SCRIPTS_POSTINSTALL_PATH];
		
		[self updatePostInstallScriptWithPath:nil];
		
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
	SEL tAction;
	
	tAction=[inMenuItem action];
	
	if (tAction==@selector(newFolder:))
	{
		NSInteger tSelectedCount;
		
		tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
		 
		if (tSelectedCount<=1)
		{
			return YES;
		}
		
		return NO;
	}
	else
	{
		if (tAction==@selector(switchPreInstallPathType:) ||
			tAction==@selector(switchPostInstallPathType:))
		{
			return YES;
		}
	}

	return [super validateMenuItem:inMenuItem];
}

- (IBAction) newFolder:(id) sender
{
    NSInteger tRow;
    
    ICFileTree * tParentTree=nil;
    ICFileNode * tParentNode;
    ICFileTree * tNewFolderTree=nil;
    
    tRow=[IBoutlineView_ selectedRow];

    if (tRow==-1)
	{
		tParentTree=fileTree_;
	}
	else
	{
		id tItem;
		
		tItem=[IBoutlineView_ itemAtRow:tRow];
		
		switch([FILENODE_DATA(tItem) type])
		{
			case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				tParentTree=(ICFileTree *) tItem;
				break;
			case ICDOCUMENT_FILE_REALFILE_NODE:
				tParentTree=(ICFileTree *) [tItem nodeParent];
				break;
		}
    }
	
    if (tParentTree!=nil)
    {
        if ([IBoutlineView_ isItemExpanded:tParentTree]==NO)
        {
            [IBoutlineView_ expandItem:tParentTree];
        }
    
        tParentNode=FILENODE_DATA(tParentTree);
            
        
        
        tNewFolderTree=[[ICFileTree alloc] initWithData:[ICFileNode folderFileNodeWithName:[ICFileTree uniqueNameWithParentFileTree:tParentTree]
                                                                                    icon:nil
                                                                                    user:[tParentNode uid]
                                                                                    group:[tParentNode gid]
                                                                                privileges:(0775 & ACCESSPERMS)]
                                                parent:nil
                                                children:[NSArray array]];
        
        if (tNewFolderTree!=nil)
        {
            [tParentTree insertSortedChild:tNewFolderTree];
            
            [tNewFolderTree release];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadData];
            
            tRow=[IBoutlineView_ rowForItem:tNewFolderTree];
            
            [IBoutlineView_ scrollRowToVisible:tRow];
            
            [IBoutlineView_ selectRow:tRow byExtendingSelection:NO];
            
            [IBoutlineView_ editColumn:[IBoutlineView_ columnWithIdentifier:@"File"] row:tRow withEvent:nil select:YES];
            
            [self updateFiles:IBoutlineView_];
        }
    }
}

- (IBAction) addFiles:(id)sender
{
    NSOpenPanel * tOpenPanel;
    NSInteger tSelectedRow;
	
    tSelectedRow=[IBoutlineView_ selectedRow];
    
    if (tSelectedRow==-1)
	{
		parentFileTree_=fileTree_;
	}
	else
	{
		parentFileTree_=[IBoutlineView_ itemAtRow:tSelectedRow];
	
		if ([FILENODE_DATA(parentFileTree_) type]==ICDOCUMENT_FILE_REALFILE_NODE)
		{
			parentFileTree_=(ICFileTree *) [parentFileTree_ nodeParent];
		}
    }
	
	[parentFileTree_ retain];
	
    tOpenPanel=[NSOpenPanel openPanel];
    
	if ([[NSUserDefaults standardUserDefaults] boolForKey:ICPREFERENCES_FILES_SHOW_ALL_FILES]==YES)
	{
		[[tOpenPanel _navView] setShowsHiddenFiles:YES];	// @"AppleShowAllFiles" = YES
	}
	
    [tOpenPanel setResolvesAliases:YES];
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:YES];
    [tOpenPanel setAllowsMultipleSelection:YES];
    [tOpenPanel setTreatsFilePackagesAsDirectories:YES];
    [tOpenPanel setDelegate:self];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Add...",@"No comment")];
    
    [tOpenPanel beginSheetForDirectory:nil
                                  file:nil
                                 types:nil
                        modalForWindow:[document_ windowForSheet]
                         modalDelegate:self
                        didEndSelector:@selector(addFilesPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) addFilesPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
   if (returnCode==NSOKButton)
    {
        NSArray * tFileNames;
		
		tFileNames=[sheet filenames];
		
		if (tFileNames!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[tFileNames objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSString * tPath;
				BOOL tShowCustomizationDialog=YES;
				id tObject;
				
				// A AMELIORER
				
				while (tPath=[tEnumerator nextObject])
				{
					if ([parentFileTree_ containsTreeWithName:[tPath lastPathComponent]]==YES)
					{
						NSBeep();
						
						return;
					}
				}
				
				tObject=[defaults_ objectForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
	
				if (tObject!=nil)
				{
					tShowCustomizationDialog=[defaults_ boolForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
				}
				
				if (tShowCustomizationDialog==YES)
				{
					NSDictionary * nContextualInformation;
					 
					 nContextualInformation=[[NSDictionary alloc] initWithObjectsAndKeys:tFileNames,IC_ADD_FILES,
																						 [NSArray arrayWithObject:parentFileTree_],IC_ADD_PARENTS,
																						 nil];
					 
					 
					
					[self performSelector:@selector(beginFileSheet:) withObject:nContextualInformation afterDelay:0.f];
				}
				else
				{
					[self _addFiles:tFileNames
						  toParents:[NSArray arrayWithObject:parentFileTree_] 
				  keepOwnerAndGroup:NO 
				     referenceStyle:[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE]];
				}
			}
		}
    }
	
	[parentFileTree_ release];
	
	parentFileTree_=nil;
}

- (void)  beginFileSheetForFiles:(NSArray *) inFiles parents:(id) inParents
{
    NSInteger tDefaultReferenceStyle;
	NSDictionary * nContextualInformation;
	
    tDefaultReferenceStyle=[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
    
    nContextualInformation=[[NSDictionary alloc] initWithObjectsAndKeys:inFiles,IC_ADD_FILES,
																		inParents,IC_ADD_PARENTS,
																		nil];
    
    // Prepare the sheet
    
    [IBreferenceStyle_ selectItemAtIndex:[IBreferenceStyle_ indexOfItemWithTag:tDefaultReferenceStyle]];
    
    [NSApp beginSheet:IBreferenceWindow_
       modalForWindow:[document_ windowForSheet]
        modalDelegate:self
       didEndSelector:@selector(fileSheetDidEnd:returnCode:contextInfo:)
          contextInfo:nContextualInformation];
}

- (void) fileSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
    NSDictionary * nContextualInformation;
    
    nContextualInformation=(NSDictionary *) contextInfo;
    
    if (returnCode==NSOKButton)
    {
		[self _addFiles:[nContextualInformation objectForKey:IC_ADD_FILES] 
			  toParents:[nContextualInformation objectForKey:IC_ADD_PARENTS] 
	  keepOwnerAndGroup:NO
	     referenceStyle:[[IBreferenceStyle_ selectedItem] tag]];
	}
    
    [nContextualInformation release];

    [sheet orderOut:self];
}

#pragma mark -

- (void) scriptsViewDidResize:(NSNotification *) inNotification
{
	NSRect tRect;
	CGFloat tWidth;
	
	tRect=[IBscriptsView_ bounds];
	
	tWidth=_CGFloatRound(NSWidth(tRect)*0.5f);
	
	tRect=[IBpreInstallView_ frame];
	
	tRect.size.width=tWidth;
	
	[IBpreInstallView_ setFrame:tRect];
	
	tRect=[IBpostInstallView_ frame];
	
	tRect.origin.x=tWidth;
	
	tRect.size.width=tWidth;
	
	[IBpostInstallView_ setFrame:tRect];
}

- (void) dropViewFilePathDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		id tObject;
		NSString * tPath;
		
		tObject=[inNotification object];
		
		tPath=[[inNotification userInfo] objectForKey:@"Path"];
		
		if (tObject==IBpreInstallIcon_)
		{
			[self _setPreInstallScriptPath:tPath];
			
			[self updatePreInstallScriptWithPath:tPath];
		}
		else if (tObject==IBpostInstallIcon_)
		{
			[self _setPostInstallScriptPath:tPath];
			
			[self updatePostInstallScriptWithPath:tPath];
		}
	}
}

- (void) outlineViewSelectionDidChange:(NSNotification *) inNotification
{
	NSInteger tSelectedCount;
	NSRect tBounds;
	NSView * tView;
	
	tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
	
	if (tSelectedCount==0)
	{
		if (emptySelectionInspectorController_==nil)
		{
			emptySelectionInspectorController_=[ICFileHierarchyInspectorEmptySelectionController newController];
		}
		
		if (currentInspectorController_!=emptySelectionInspectorController_ || currentInspectorController_==nil)
		{
			tView=[currentInspectorController_ view];
		
			[tView removeFromSuperview];
		
			currentInspectorController_=emptySelectionInspectorController_;
			
			tView=[currentInspectorController_ view];
			
			tBounds=[IBinspectorView_ bounds];
			
			[tView setFrame:tBounds];
			
			[IBinspectorView_ addSubview:tView];
		}
		
		[IBremoveButton_ setEnabled:NO];
	}
	else
	{
		NSIndexSet * tIndexSet;
		
		[IBremoveButton_ setEnabled:YES];
		
		if (selectionInspectorController_==nil)
		{
			selectionInspectorController_=[ICPackageScriptsInspectorSelectionController newController];
		}
		
		if (currentInspectorController_!=selectionInspectorController_ || currentInspectorController_==nil)
		{
			tView=[currentInspectorController_ view];
		
			[tView removeFromSuperview];
		
			currentInspectorController_=selectionInspectorController_;
			
			tView=[currentInspectorController_ view];
			
			tBounds=[IBinspectorView_ bounds];
			
			[tView setFrame:tBounds];
			
			[IBinspectorView_ addSubview:tView];
		}
		
		tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
		if (tIndexSet!=nil)
		{
			NSMutableArray * tSelectedItems;
			
			tSelectedItems=[NSMutableArray array];
			
			if (tSelectedItems!=nil)
			{
				NSUInteger tIndex;
				
				tIndex=[tIndexSet firstIndex];
				
				while (tIndex!=NSNotFound)
				{
					id tItem;
					
					tItem=[IBoutlineView_ itemAtRow:tIndex];
					
					if (tItem!=nil)
					{
						[tSelectedItems addObject:tItem];
						
						tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
					}
				}
				
				[selectionInspectorController_ initializeWithArray:tSelectedItems forDocument:document_];
			}
		}
	}
}

@end
