/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICPackageSettingsController.h"

#import "ICPackageUtilities.h"

#import "ICReferencedUtilities.h"

#import "ICPackageLocationTextField.h"

#import "ICBundleIdentifierFormatter.h"

#import "ICDocument.h"

@implementation ICPackageSettingsController

+ (ICPackageSettingsController *) packageSettingsController
{
	ICPackageSettingsController * nController=nil;
    
    nController=[ICPackageSettingsController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Package_Settings" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Package_Settings"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

#pragma mark -

- (void) awakeFromNib
{
	NSMenu * tMenu;
	ICBundleIdentifierFormatter * tFormatter;
	
	tMenu=[ICReferencedUtilities referenceStyleMenuForTarget:self andAction:@selector(switchReferencePathType:)];
	
	if (tMenu!=nil)
	{
		[IBsourcePopupButton_ setMenu:tMenu];
	}
	
	[IBreferencedPackageSection_ setColor:[NSColor colorWithDeviceWhite:0.8392f alpha:1.0f]];
	
	[IBsourceSection_ setColor:[NSColor colorWithDeviceRed:0.7529f green:0.7843f blue:0.8392f alpha:1.0f]];
	
	
	// Bundle Identifier Formatter
	
	tFormatter=[ICBundleIdentifierFormatter new];
	
	if (tFormatter!=nil)
	{
		[IBidentifierTextField_ setFormatter:tFormatter];
		
		[tFormatter release];
	}
}


- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{	
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS];
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;

		[dictionary_ retain];
		
		// Check the Package Type
		
		cachedPackageType_=ICDOCUMENT_PACKAGE_TYPE_PROJECT;
		
		tNumber=[inDictionary objectForKey:ICDOCUMENT_PACKAGE_TYPE];
		
		if (tNumber!=nil)
		{
			cachedPackageType_=[tNumber intValue];
		}
		
		switch (cachedPackageType_)
		{
			case ICDOCUMENT_PACKAGE_TYPE_PROJECT:
		
				// We can edit the settings and select any type of package location
			
				break;
			
			case ICDOCUMENT_PACKAGE_TYPE_IMPORTED:
			
				// We can not edit but select any type of package location
				
				// We read from the file directly for most settings
				
				
				
				cachedReferencePath_=[inDictionary objectForKey:ICDOCUMENT_PACKAGE_REFERENCE_PATH];
				
				if (cachedReferencePath_!=nil)
				{
					NSInteger tPathType=ICDOCUMENT_PATH_TYPE_GLOBAL;
					
					cachedAbsolutePath_=[cachedReferencePath_ objectForKey:ICDOCUMENT_PATH];
		
					tNumber=[cachedReferencePath_ objectForKey:ICDOCUMENT_PATH_TYPE];
					
					if (tNumber!=nil)
					{
						tPathType=[tNumber integerValue];
					}
					
					if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						cachedAbsolutePath_=[cachedAbsolutePath_ stringByAbsolutingWithPath:[document_ folder]];
					}
					else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						cachedAbsolutePath_=[cachedAbsolutePath_ stringByAbsolutingWithPath:[document_ referenceFolderPath]];
					}
				}
				
				break;
				
			case ICDOCUMENT_PACKAGE_TYPE_REFERENCED:
			
				// We can edit but only select some types of package location (Internal forbidden)
		
				// A COMPLETER
				
				break;
		}
		
		[self updateSettings];
	}
}

#pragma mark -

- (void) showHideSourceSection:(BOOL) inHidden
{
	id tView;
	NSRect tViewBounds;
	NSRect tSectionFrame;
	CGFloat tCumulatedHeight=0;
	
	tView=[self view];
	
	tViewBounds=[tView bounds];
	
	if (inHidden==YES && [IBsourceSection_ isHidden]==NO)
	{
		// Hide Source Section
		
		// Source Section
		
		[IBsourceSection_ setHidden:YES];
		
		// Referenced Section
		
		[IBreferencedPackageSection_ setHidden:YES];
		
		// Tag Section
		
		tSectionFrame=[IBtagSection_ frame];
		
		tCumulatedHeight+=NSHeight(tSectionFrame);
		
		tSectionFrame.origin.y=NSMaxY(tViewBounds)-tCumulatedHeight;
		
		[IBtagSection_ setFrame:tSectionFrame];
		
		tCumulatedHeight=NSHeight(tSectionFrame);
		
		// Options Section
		
		tSectionFrame=[IBpostInstallationSection_ frame];
		
		tCumulatedHeight+=NSHeight(tSectionFrame);
		
		tSectionFrame.origin.y=NSMaxY(tViewBounds)-tCumulatedHeight;
		
		[IBpostInstallationSection_ setFrame:tSectionFrame];
		
		// Post-Installation Section
		
		tSectionFrame=[IBlocationSection_ frame];
		
		tCumulatedHeight+=NSHeight(tSectionFrame);
		
		tSectionFrame.origin.y=NSMaxY(tViewBounds)-tCumulatedHeight;
		
		[IBlocationSection_ setFrame:tSectionFrame];
		
		// Location Section
		
		tSectionFrame=[IBoptionsSection_ frame];
		
		tCumulatedHeight+=NSHeight(tSectionFrame);
		
		tSectionFrame.origin.y=NSMaxY(tViewBounds)-tCumulatedHeight;
		
		[IBoptionsSection_ setFrame:tSectionFrame];
		
		[tView setNeedsDisplay:YES];
	}
	else if (inHidden==NO)
	{
		BOOL tMoveOtherSections=NO;
		
		if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
		{
			[IBreferencedPackageSection_ setHidden:YES];
			
			tSectionFrame=[IBsourceSection_ frame];
				
			tCumulatedHeight+=NSHeight(tSectionFrame);
			
			if ([IBsourceSection_ isHidden]==YES)
			{
				// Show Source Section
				
				// Source Section
				
				tSectionFrame.origin.y=NSMaxY(tViewBounds)-tCumulatedHeight;
				
				[IBsourceSection_ setFrame:tSectionFrame];
				
				[IBsourceSection_ setHidden:NO];
			}
			
			tMoveOtherSections=YES;
		}
		
		if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
		{
			[IBsourceSection_ setHidden:YES];
			
			tSectionFrame=[IBreferencedPackageSection_ frame];
				
			tCumulatedHeight+=NSHeight(tSectionFrame);
			
			if ([IBreferencedPackageSection_ isHidden]==YES)
			{
				// Show Source Section
				
				// Source Section
				
				tSectionFrame.origin.y=NSMaxY(tViewBounds)-tCumulatedHeight;
				
				[IBreferencedPackageSection_ setFrame:tSectionFrame];
				
				[IBreferencedPackageSection_ setHidden:NO];
			}
			
			tMoveOtherSections=YES;
		}
		
		if (tMoveOtherSections==YES)
		{
			CGFloat tMaxY;
			CGFloat tHeight;
			
			// Tag Section
			
			tSectionFrame=[IBtagSection_ frame];
			
			tCumulatedHeight+=NSHeight(tSectionFrame);
			
			tSectionFrame.origin.y=NSMaxY(tViewBounds)-tCumulatedHeight;
			
			[IBtagSection_ setFrame:tSectionFrame];
			
			// Post-Installation Section
			
			tSectionFrame=[IBpostInstallationSection_ frame];
			
			tCumulatedHeight+=NSHeight(tSectionFrame);
			
			tSectionFrame.origin.y=NSMaxY(tViewBounds)-tCumulatedHeight;
			
			[IBpostInstallationSection_ setFrame:tSectionFrame];
			
			// Location Section
			
			tSectionFrame=[IBlocationSection_ frame];
			
			tCumulatedHeight+=NSHeight(tSectionFrame);
			
			tSectionFrame.origin.y=NSMaxY(tViewBounds)-tCumulatedHeight;
			
			[IBlocationSection_ setFrame:tSectionFrame];
			
			// Options Section
			
			tSectionFrame=[IBoptionsSection_ frame];
			
			//tCumulatedHeight+=NSHeight(tSectionFrame);
			
			tMaxY=NSMaxY(tSectionFrame);
			
			if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
			{
				[IBrelocatableCheckBox_ setHidden:YES];
			
				[IBoverwritePermissionsCheckBox_ setHidden:YES];
				
				[IBfollowSymbolicLinksCheckBox_ setHidden:YES];
				
				[IBuseHFSPlusPermissionCheckBox_ setHidden:YES];
				
				[IBuseHFSPlusPermissionExplanation_ setHidden:YES];
				
				tHeight=tMaxY-(NSMinY(tSectionFrame)+NSMinY([IBrootAuthenticationCheckBox_ frame])-20.0f);
				
				tSectionFrame.size.height=tHeight;
				
				tSectionFrame.origin.y=NSMinY([IBlocationSection_ frame])-tHeight;
			}
			else
			{
				[IBrelocatableCheckBox_ setHidden:NO];
				
				[IBoverwritePermissionsCheckBox_ setHidden:NO];
				
				[IBfollowSymbolicLinksCheckBox_ setHidden:NO];
				
				[IBuseHFSPlusPermissionCheckBox_ setHidden:NO];
				
				[IBuseHFSPlusPermissionExplanation_ setHidden:NO];
				
				tHeight=tMaxY-(NSMinY(tSectionFrame)+NSMinY([IBfollowSymbolicLinksCheckBox_ frame])-20.0f);
				
				tSectionFrame.size.height=tHeight;
				
				tSectionFrame.origin.y=NSMinY([IBlocationSection_ frame])-tHeight;
			}
			
			[IBoptionsSection_ setFrame:tSectionFrame];
			
			[tView setNeedsDisplay:YES];
		}
	}
}

- (void) updateSettings
{
	NSNumber * tNumber;
	NSString * tString;
	int tLocationType;
	
	if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_IMPORTED)
	{
		NSMutableDictionary * tPackageSettingsDictionary;
		
		tPackageSettingsDictionary=[ICPackageUtilities packageSettingsDictionaryWithContentsOfFile:cachedAbsolutePath_];
		
		// Source
		
		[self showHideSourceSection:NO];
		
		if (cachedReferencePath_!=nil)
		{
			NSInteger tPathType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			NSString * tPath;
			
			// Popup Button
			
			tNumber=[cachedReferencePath_ objectForKey:ICDOCUMENT_PATH_TYPE];
			
			if (tNumber!=nil)
			{
				tPathType=[tNumber integerValue];
			}
			
			[IBsourcePopupButton_ selectItemWithTag:tPathType];
			
			// Text Field
			
			tPath=[cachedReferencePath_ objectForKey:ICDOCUMENT_PATH];
			
			[IBsourceTextField_ setStringValue:tPath];
		}
		
		// Identifier
		
		tString=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER];
		
		if (tString==nil)
		{
			tString=@"";
		}
		
		[IBidentifierTextField_ setEnabled:NO];
		
		[IBidentifierTextField_ setStringValue:tString];
		
		// Version
		
		tString=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_VERSION];
		
		if (tString==nil)
		{
			tString=@"";
		}
		
		[IBversionTextField_ setEnabled:NO];
		
		[IBversionTextField_ setStringValue:tString];
		
		// Root Authentication
		
		tNumber=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION];
		
		[IBrootAuthenticationCheckBox_ setEnabled:NO];
		
		if (tNumber==nil)
		{
			[IBrootAuthenticationCheckBox_ setState:NSOffState];
		}
		else
		{
			[IBrootAuthenticationCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
		}
	
		// Relocatable
		
		tNumber=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_RELOCATABLE];
		
		[IBrelocatableCheckBox_ setEnabled:NO];
		
		if (tNumber==nil)
		{
			[IBrelocatableCheckBox_ setState:NSOffState];
		}
		else
		{
			[IBrelocatableCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
		}
		
		// Overwrite Permissions
		
		tNumber=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS];
		
		[IBoverwritePermissionsCheckBox_ setEnabled:NO];
		
		if (tNumber==nil)
		{
			[IBoverwritePermissionsCheckBox_ setState:NSOffState];
		}
		else
		{
			[IBoverwritePermissionsCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
		}
		
		// Follow Symbolic Links
		
		tNumber=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_FOLLOW_SYMBOLIC_LINKS];
		
		[IBfollowSymbolicLinksCheckBox_ setEnabled:NO];
		
		if (tNumber==nil)
		{
			[IBfollowSymbolicLinksCheckBox_ setState:NSOffState];
		}
		else
		{
			[IBfollowSymbolicLinksCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
		}
		
		// Use HFS+ Compression
		
		tNumber=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_USE_HFSPLUS_COMPRESSION];
		
		[IBuseHFSPlusPermissionCheckBox_ setEnabled:NO];
		
		if (tNumber==nil)
		{
			[IBuseHFSPlusPermissionCheckBox_ setState:NSOffState];
		}
		else
		{
			[IBuseHFSPlusPermissionCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
		}
		
		// On Conclusion
		
		tNumber=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
		
		[IBonConclusionPopupButton_ setEnabled:NO];
		
		if (tNumber==nil)
		{
			[IBonConclusionPopupButton_ selectItemWithTag:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_NONE];
		}
		else
		{
			[IBonConclusionPopupButton_ selectItemWithTag:[tNumber intValue]];
		}
	}
	else
	{
		// Source or Referenced
		
		[self showHideSourceSection:(cachedPackageType_!=ICDOCUMENT_PACKAGE_TYPE_REFERENCED)];
		
		// Identifier
		
		tString=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER];
		
		if (tString==nil)
		{
			tString=@"";
		}
		
		[IBidentifierTextField_ setEnabled:YES];
		
		[IBidentifierTextField_ setStringValue:tString];
		
		// Version
		
		tString=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_VERSION];
		
		if (tString==nil)
		{
			tString=@"";
		}
		
		[IBversionTextField_ setEnabled:YES];
		
		[IBversionTextField_ setStringValue:tString];
		
		// On Conclusion
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
		
		[IBonConclusionPopupButton_ setEnabled:YES];
		
		if (tNumber==nil)
		{
			[IBonConclusionPopupButton_ selectItemWithTag:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_NONE];
		}
		else
		{
			[IBonConclusionPopupButton_ selectItemWithTag:[tNumber intValue]];
		}
		
		// Root Authentication
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION];
		
		[IBrootAuthenticationCheckBox_ setEnabled:YES];
		
		if (tNumber==nil)
		{
			[IBrootAuthenticationCheckBox_ setState:NSOffState];
		}
		else
		{
			[IBrootAuthenticationCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
		}
		
		if (cachedPackageType_!=ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
		{
			
			// Relocatable
		
			tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_RELOCATABLE];
			
			[IBrelocatableCheckBox_ setEnabled:YES];
			
			if (tNumber==nil)
			{
				[IBrelocatableCheckBox_ setState:NSOffState];
			}
			else
			{
				[IBrelocatableCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
			}
			
			// Overwrite Permissions
			
			tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS];
			
			[IBoverwritePermissionsCheckBox_ setEnabled:YES];
			
			if (tNumber==nil)
			{
				[IBoverwritePermissionsCheckBox_ setState:NSOffState];
			}
			else
			{
				[IBoverwritePermissionsCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
			}
			
			// Follow Symbolic Links
			
			tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_FOLLOW_SYMBOLIC_LINKS];
			
			[IBfollowSymbolicLinksCheckBox_ setEnabled:YES];
			
			if (tNumber==nil)
			{
				[IBfollowSymbolicLinksCheckBox_ setState:NSOffState];
			}
			else
			{
				[IBfollowSymbolicLinksCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
			}
			
			// Use HFS+ Compression
			
			tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_USE_HFSPLUS_COMPRESSION];
			
			[IBoverwritePermissionsCheckBox_ setEnabled:YES];
			
			if (tNumber==nil)
			{
				[IBuseHFSPlusPermissionCheckBox_ setState:NSOffState];
			}
			else
			{
				[IBuseHFSPlusPermissionCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
			}
		}
		
	}
	
	// Location
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION];
	
	if (tNumber==nil)
	{
		if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
		{
			tLocationType=ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_HTTP_URL;
		}
		else
		{
			tLocationType=ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED;
		}
	}
	else
	{
		tLocationType=[tNumber intValue];
		
		if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
		{
			if (tLocationType==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED ||
				tLocationType==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_CUSTOM_PATH)
			{
				tLocationType=ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_HTTP_URL;
			}
		}
	}
	
	[IBlocationPopupButton_ selectItemWithTag:tLocationType];
	
	if (tLocationType==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED)
	{
		[IBlocationTip_ setStringValue:@""];
		
		[IBlocationLabel_ setHidden:YES];
			
		[IBlocationTextField_ setHidden:YES];
		
		[IBversionTextField_ setNextKeyView:IBidentifierTextField_];
		
		[IBlocationTextField_ setStringValue:@""];
	}
	else
	{
		NSString * tPath;
		
		tPath=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
	
		[IBlocationLabel_ setHidden:NO];
			
		[IBlocationTextField_ setHidden:NO];
		
		[IBversionTextField_ setNextKeyView:IBlocationTextField_];
			
		[IBlocationTextField_ setNextKeyView:IBidentifierTextField_];
		
		[IBlocationLabel_ setStringValue:NSLocalizedString(@"Path:","")];
		
		switch(tLocationType)
		{
			case ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_CUSTOM_PATH:
			
				[IBlocationTip_ setStringValue:NSLocalizedString(@"( './' represents the parent folder of the metapackage )",@"")];
				
				if (tPath==nil)
				{
					tPath=@"";
				}
			
				break;
			
			case ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_HTTP_URL:
			
				[IBlocationTip_ setStringValue:NSLocalizedString(@"( URL of the folder containing the package on the HTTP server )",@"")];
				
				[IBlocationLabel_ setStringValue:NSLocalizedString(@"URL:","")];
				
				if (tPath==nil)
				{
					tPath=@"";
				}
				
				break;
				
			case ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_REMOVABLE_MEDIA:
			
				[IBlocationTip_ setStringValue:@""];
				
				if (tPath==nil)
				{
					tPath=@"x-disc://";
				}
				
				break;
		}
		
		[IBlocationTextField_ setStringValue:tPath];
	}
}

#pragma mark -

- (IBAction) switchReferencePathType:(id) sender
{
	NSInteger tTag;
	
	tTag=[sender tag];
	
	if (cachedReferencePath_!=nil)
	{
		NSInteger tOldPathType=ICDOCUMENT_PATH_TYPE_GLOBAL;
		NSNumber * tNumber;
		
		tNumber=[cachedReferencePath_ objectForKey:ICDOCUMENT_PATH_TYPE];
		
		if (tNumber!=nil)
		{
			tOldPathType=[tNumber integerValue];
		}
		
		if (tOldPathType!=tTag)
		{
			NSString * tPath;
			NSString * tAbsolutePath=nil;

			
			// Get the absolute path
			
			tPath=[cachedReferencePath_ objectForKey:ICDOCUMENT_PATH];
			
			if (tOldPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
			{
				tAbsolutePath=tPath;
			}
			else if (tOldPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
			}
			else if (tOldPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
			}
			
			// Convert to the new path type
			
			if (tTag==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tPath=[tAbsolutePath stringByRelativizingToPath:[document_ folder]];
			}
			else if (tTag==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tPath=[tAbsolutePath stringByRelativizingToPath:[document_ referenceFolderPath]];
			}
			else
			{
				tPath=tAbsolutePath;
			}
			
			[cachedReferencePath_ setObject:tPath forKey:ICDOCUMENT_PATH];
			
			[cachedReferencePath_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PATH_TYPE];
			
			
			[IBsourcePopupButton_ selectItemWithTag:tTag];
			
			[IBsourceTextField_ setStringValue:tPath];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
	}
}

- (BOOL)validateMenuItem:(NSMenuItem *) inMenuItem
{
    SEL tAction=[inMenuItem action];
        
    if (tAction==@selector(switchLocation:))
    {
		if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
		{
			if ([inMenuItem tag]==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED ||
				[inMenuItem tag]==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_CUSTOM_PATH)
			{
				return NO;
			}
		}
	}
	
	return YES;
}

- (IBAction) switchRootAuthentication:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchRelocatable:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_RELOCATABLE];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchOverwritePermissions:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchFollowSymbolicLinks:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_FOLLOW_SYMBOLIC_LINKS];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchUseHFSPlusCompression:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_USE_HFSPLUS_COMPRESSION];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchOnConclusion:(id) sender
{
	NSInteger tTag;
	NSInteger tOldTag=ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_NONE;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
		
	if (tNumber!=nil)
	{
		tOldTag=[tNumber intValue];
	}
	
	if (tOldTag!=tTag)
	{
		[dictionary_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
		
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

#pragma mark -

- (IBAction) switchLocation:(id) sender
{
	NSInteger tTag;
	NSInteger tOldTag=ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION];
		
	if (tNumber!=nil)
	{
		tOldTag=[tNumber intValue];
	}
	
	if (tOldTag!=tTag)
	{
		[dictionary_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION];
		
		if (tTag==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED)
		{
			[IBlocationLabel_ setHidden:YES];
				
			[IBlocationTextField_ setHidden:YES];
			
			[IBlocationTextField_ setStringValue:@""];
			
			[IBlocationTip_ setStringValue:@""];
			
			[IBversionTextField_ setNextKeyView:IBidentifierTextField_];
		}
		else
		{
			NSString * tPath;
			
			tPath=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
		
			[IBlocationLabel_ setHidden:NO];
				
			[IBlocationTextField_ setHidden:NO];
			
			[IBversionTextField_ setNextKeyView:IBlocationTextField_];
			
			[IBlocationTextField_ setNextKeyView:IBidentifierTextField_];
			
			[IBlocationLabel_ setStringValue:NSLocalizedString(@"Path:","")];
			
			switch(tTag)
			{
				case ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_CUSTOM_PATH:
				
					[IBlocationTip_ setStringValue:NSLocalizedString(@"( './' represents the parent folder of the metapackage )",@"")];
					
					if (tPath==nil)
					{
						tPath=@"";
					}
					else
					{
						if ([tPath hasPrefix:@"http://"]==YES)
						{
							tPath=[tPath substringFromIndex:7];
							
							[dictionary_ setObject:tPath forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
						}
						else if ([tPath hasPrefix:@"x-disc://"]==YES)
						{
							tPath=[tPath substringFromIndex:9];
							
							[dictionary_ setObject:tPath forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
						}
					}
				
					break;
				
				case ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_HTTP_URL:
				
					[IBlocationTip_ setStringValue:NSLocalizedString(@"( URL of the folder containing the package on the HTTP server )",@"")];
					
					[IBlocationLabel_ setStringValue:NSLocalizedString(@"URL:","")];
					
					if (tPath==nil)
					{
						tPath=@"";
					}
					else
					{
						if ([tPath hasPrefix:@"file:"]==YES)
						{
							tPath=[tPath substringFromIndex:5];
							
							[dictionary_ setObject:tPath forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
						}
						else if ([tPath hasPrefix:@"x-disc://"]==YES)
						{
							tPath=[tPath substringFromIndex:9];
							
							[dictionary_ setObject:tPath forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
						}
					}
					
					break;
					
				case ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_REMOVABLE_MEDIA:
				
					[IBlocationTip_ setStringValue:@""];
					
					if (tPath==nil)
					{
						tPath=@"x-disc://";
					}
					else
					{
						if ([tPath hasPrefix:@"file:"]==YES)
						{
							tPath=[tPath substringFromIndex:5];
							
							[dictionary_ setObject:tPath forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
						}
						else if ([tPath hasPrefix:@"http://"]==YES)
						{
							tPath=[tPath substringFromIndex:7];
							
							[dictionary_ setObject:tPath forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
						}
						
						if ([tPath hasPrefix:@"x-disc://"]==NO)
						{
							tPath=[[NSString stringWithString:@"x-disc://"] stringByAppendingString:tPath];
							
							[dictionary_ setObject:tPath forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
						}
					}

					break;
			}
			
			[IBlocationTextField_ setStringValue:tPath];
		}
		
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

#pragma mark -

- (NSDragOperation) validatePackageDrop:(id <NSDraggingInfo>)info
{
	if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
	{
		NSPasteboard * tPasteBoard;
			
		tPasteBoard=[info draggingPasteboard];
	
		if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
		{
			// We need to check that the plugins are not already in the list
			
			NSArray * tArray;
			
			tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
						
			if (tArray!=nil)
			{
				if ([tArray count]==1)
				{
					NSString * tFilePath;
					
					tFilePath=[tArray objectAtIndex:0];
					
					if ([ICPackageUtilities isFlatPackageAtPath:tFilePath]==YES)
					{
						return NSDragOperationLink;
					}
				}
			}
		}
	}
	
	return NSDragOperationNone;
}

- (BOOL) acceptPackageDrop:(id <NSDraggingInfo>)info
{
	if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
	{
		NSPasteboard * tPasteBoard;
			
		tPasteBoard=[info draggingPasteboard];
	
		if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
		{
			// We need to check that the plugins are not already in the list
			
			NSArray * tArray;
			
			tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
						
			if (tArray!=nil)
			{
				if ([tArray count]==1)
				{
					NSString * tFilePath;
					NSMutableDictionary * tMutableDictionary;
					
					tFilePath=[tArray objectAtIndex:0];
					
					tMutableDictionary=[ICPackageUtilities packageSettingsDictionaryWithContentsOfFile:tFilePath];
					
					if (tMutableDictionary!=nil)
					{
						[tMutableDictionary removeObjectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
						
						[tMutableDictionary removeObjectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION];
						
						[dictionary_ addEntriesFromDictionary:tMutableDictionary];
						
						[self updateSettings];
						
						// Notify Document Change
			
						[self noteDocumentChange];
						
						return YES;
					}
				}
			}
		}
	}
	
	return NO;
}

- (NSDragOperation) textField:(ICFilePathTextField *) inTextField validateDrop:(id <NSDraggingInfo>)info;
{
	NSPasteboard * tPasteBoard;
	int tPackageLocation;
			
	tPasteBoard=[info draggingPasteboard];
	
	tPackageLocation=[[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION] intValue];
	
	/*if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSURLPboardType]]!=nil)
	{
		if (tPackageLocation==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_HTTP_URL)
		{
			return NSDragOperationCopy;
		}
	}
	else*/ if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSStringPboardType]]!=nil)
	{
		NSString * tString;
		
		tString=[tPasteBoard stringForType:NSStringPboardType];
		
		if (tString!=nil)
		{
			switch(tPackageLocation)
			{
				case ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_CUSTOM_PATH:
				
					if ([tString hasPrefix:@"http://"]==NO &&
						[tString hasPrefix:@"x-disc://"]==NO)
					{
						return NSDragOperationCopy;
					}
					
					break;
				
				case ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_HTTP_URL:
				
					if ([tString hasPrefix:@"http://"]==YES)
					{
						return NSDragOperationCopy;
					}
					
					break;
					
				case ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_REMOVABLE_MEDIA:
			
					if ([tString hasPrefix:@"x-disc://"]==YES)
					{
						return NSDragOperationCopy;
					}
					
					break;
			}
		}
	}
	else if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
	{
		if (tPackageLocation==ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_CUSTOM_PATH)
		{
			NSArray * tArray;
			
			tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
						
			if (tArray!=nil)
			{
				if ([tArray count]==1)
				{
					NSString * tFilePath;
					BOOL isDirectory;
					
					tFilePath=[tArray objectAtIndex:0];
					
					if ([[NSFileManager defaultManager] fileExistsAtPath:tFilePath isDirectory:&isDirectory]==YES && isDirectory==YES)
					{
						return NSDragOperationCopy;
					}
				}
			}
		}
	}
	
	
	
	return NSDragOperationNone;
}

- (BOOL) textField:(ICFilePathTextField *) inTextField acceptDrop:(id <NSDraggingInfo>)info;
{
    NSPasteboard * tPasteBoard;
			
	tPasteBoard=[info draggingPasteboard];
	
	/*if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSURLPboardType]]!=nil)
	{
		{
			NSString * tString;
			
			tString=[tPasteBoard stringForType:NSURLPboardType];
			
			NSLog(@"%@",tString);
		}
	}
	else*/ if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSStringPboardType]]!=nil)
	{
		NSString * tString;
		
		tString=[tPasteBoard stringForType:NSStringPboardType];
		
		if (tString!=nil)
		{
			if ([tString hasPrefix:@"http://"]==YES)
			{
				NSString * tLastComponent;
				
				tString=[tString stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
				
				tString=[tString substringFromIndex:7];
				
				tLastComponent=[tString lastPathComponent];
				
				if (tLastComponent!=nil)
				{
					if ([[tLastComponent pathExtension] caseInsensitiveCompare:@"pkg"]==NSOrderedSame)
					{
						tString=[tString stringByDeletingLastPathComponent];
						
						if (cachedPackageType_==ICDOCUMENT_PACKAGE_TYPE_REFERENCED)
						{
							tLastComponent=[tLastComponent stringByDeletingPathExtension];
							
							[dictionary_ setObject:tLastComponent forKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
						}
					}
					
					tString=[[NSString stringWithString:@"http://"] stringByAppendingString:tString];
					
					
					[IBlocationTextField_ setStringValue:tString];
					
					[dictionary_ setObject:tString forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
				}
				else
				{
					return NO;
				}
			}
			else
			{
				[IBlocationTextField_ setStringValue:tString];
			
				[dictionary_ setObject:tString forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
			}
			
			// Notify Document Change
			
			[self noteDocumentChange];

			return YES;
		}
	}	
	else if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
	{
		NSArray * tArray;
			
		tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
					
		if (tArray!=nil)
		{
			if ([tArray count]==1)
			{
				NSString * tFilePath;
				BOOL isDirectory;
				
				tFilePath=[tArray objectAtIndex:0];
				
				if ([[NSFileManager defaultManager] fileExistsAtPath:tFilePath isDirectory:&isDirectory]==YES && isDirectory==YES)
				{
					[IBlocationTextField_ setStringValue:tFilePath];
			
					[dictionary_ setObject:tFilePath forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
					
					// Notify Document Change
					
					[self noteDocumentChange];

					return YES;
				}
			}
		}
	}
		
	return NO;
}

#pragma mark -

- (void) control:(NSControl *) inControl didFailToValidatePartialString:(NSString *)string errorDescription:(NSString *) inErrorDescription
{
	if ([inErrorDescription isEqualToString:@"Error"]==YES)
	{
		NSBeep();
	}
}

- (void) controlTextDidChange:(NSNotification *) inNotification
{
	NSString * tValue;
        
	tValue=[[[[inNotification userInfo] objectForKey:@"NSFieldEditor"] string] copy];
	
	if (tValue!=nil)
	{
		if ([inNotification object]==IBidentifierTextField_)
		{
			[dictionary_ setObject:tValue forKey:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER];
		}
		else if ([inNotification object]==IBversionTextField_)
		{
			[dictionary_ setObject:tValue forKey:ICDOCUMENT_PACKAGE_SETTINGS_VERSION];
		}
		else if ([inNotification object]==IBlocationTextField_)
		{
			[dictionary_ setObject:tValue forKey:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH];
		}
		
		[tValue release];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

@end
