/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPackageSettingsController_Package.h"

#import "ICPackageUtilities.h"

#import "ICReferencedUtilities.h"

#import "ICBundleIdentifierFormatter.h"

@implementation ICPackageSettingsController_Package

+ (ICPackageSettingsController_Package *) packageSettingsController
{
	ICPackageSettingsController_Package * nController=nil;
    
    nController=[ICPackageSettingsController_Package alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Package_Settings_Package" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Package_Settings_Package"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

#pragma mark -

- (void) dealloc
{
	[super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{	
	ICBundleIdentifierFormatter * tFormatter;
	
	tFormatter=[ICBundleIdentifierFormatter new];
	
	if (tFormatter!=nil)
	{
		[IBidentifierTextField_ setFormatter:tFormatter];
		
		[tFormatter release];
	}
}


- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{	
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS];
	
	if (dictionary_!=nil)
	{
		[dictionary_ retain];
		
		// Check the Package Type
		
		cachedPackageType_=ICDOCUMENT_PACKAGE_TYPE_PROJECT;
		
		[self updateSettings];
	}
}

#pragma mark -

- (void) updateSettings
{
	NSNumber * tNumber;
	NSString * tString;
	
	// Identifier
	
	tString=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER];
	
	if (tString==nil)
	{
		tString=@"";
	}
	
	[IBidentifierTextField_ setStringValue:tString];
	
	// Version
	
	tString=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_VERSION];
	
	if (tString==nil)
	{
		tString=@"";
	}
	
	[IBversionTextField_ setStringValue:tString];
	
	// Root Authentication
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION];
	
	if (tNumber==nil)
	{
		[IBrootAuthenticationCheckBox_ setState:NSOffState];
	}
	else
	{
		[IBrootAuthenticationCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
	}
	
	// Relocatable
		
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_RELOCATABLE];
	
	if (tNumber==nil)
	{
		[IBrelocatableCheckBox_ setState:NSOffState];
	}
	else
	{
		[IBrelocatableCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
	}

	// Overwrite Permissions
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS];
	
	if (tNumber==nil)
	{
		[IBoverwritePermissionsCheckBox_ setState:NSOffState];
	}
	else
	{
		[IBoverwritePermissionsCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
	}
	
	// Follow Symbolic Links
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_FOLLOW_SYMBOLIC_LINKS];
	
	if (tNumber==nil)
	{
		[IBfollowSymbolicLinksCheckBox_ setState:NSOffState];
	}
	else
	{
		[IBfollowSymbolicLinksCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
	}
	
	// Use HFS+ Compression
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_USE_HFSPLUS_COMPRESSION];
	
	if (tNumber==nil)
	{
		[IBuseHFSPlusPermissionCheckBox_ setState:NSOffState];
	}
	else
	{
		[IBuseHFSPlusPermissionCheckBox_ setState:([tNumber boolValue]==YES) ? NSOnState : NSOffState];
	}
	
	// On Conclusion
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
	
	if (tNumber==nil)
	{
		[IBonConclusionPopupButton_ selectItemWithTag:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_NONE];
	}
	else
	{
		[IBonConclusionPopupButton_ selectItemWithTag:[tNumber intValue]];
	}
}

#pragma mark -

- (IBAction) switchRootAuthentication:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchRelocatable:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_RELOCATABLE];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchOverwritePermissions:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchFollowSymbolicLinks:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_FOLLOW_SYMBOLIC_LINKS];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchUseHFSPlusCompression:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PACKAGE_SETTINGS_USE_HFSPLUS_COMPRESSION];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

- (IBAction) switchOnConclusion:(id) sender
{
	NSInteger tTag;
	NSInteger tOldTag=ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_NONE;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
		
	if (tNumber!=nil)
	{
		tOldTag=[tNumber intValue];
	}
	
	if (tOldTag!=tTag)
	{
		[dictionary_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
		
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

#pragma mark -

- (void) control:(NSControl *) inControl didFailToValidatePartialString:(NSString *)string errorDescription:(NSString *) inErrorDescription
{
	if ([inErrorDescription isEqualToString:@"Error"]==YES)
	{
		NSBeep();
	}
}

- (void) controlTextDidChange:(NSNotification *) inNotification
{
	NSString * tValue;
        
	tValue=[[[[inNotification userInfo] objectForKey:@"NSFieldEditor"] string] copy];
	
	if (tValue!=nil)
	{
		if ([inNotification object]==IBidentifierTextField_)
		{
			[dictionary_ setObject:tValue forKey:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER];
		}
		else if ([inNotification object]==IBversionTextField_)
		{
			[dictionary_ setObject:tValue forKey:ICDOCUMENT_PACKAGE_SETTINGS_VERSION];
		}
		
		[tValue release];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

@end
