/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICPackageSettingsView.h"

#import "ICHighlightWindow.h"

@implementation ICPackageSettingsView

- (id) initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
	
    if (self)
	{
        // Initialization code here.
		
		// Register for Drop
		
		[self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
    }
	
    return self;
}

- (void) showHideHighlight:(BOOL) inShow
{
	if (inShow==YES && highlightWindow_==nil)
	{
		highlightWindow_=[[ICHighlightWindow alloc] initForView:self];
		
		if (highlightWindow_!=nil)
		{
			[[self window] addChildWindow:highlightWindow_ ordered:NSWindowAbove];
			
			[highlightWindow_ orderFront:self];
		}
	}
	else if (inShow==NO && highlightWindow_!=nil)
	{
		[[self window] removeChildWindow:highlightWindow_];
		
		[highlightWindow_ orderOut:self];
		
		[highlightWindow_ release];
		
		highlightWindow_=nil;
	}
}

#pragma mark -

- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>) sender
{
	NSDragOperation tDragOperation=NSDragOperationNone;
	
	if (delegate!=nil && [delegate respondsToSelector: @selector(validatePackageDrop:)]==YES)
	{
		tDragOperation=[delegate validatePackageDrop:sender];
	}
	
	if (tDragOperation!=NSDragOperationNone)
	{
		[self showHideHighlight:YES];
	}
	
	return tDragOperation;
}

- (BOOL) performDragOperation: (id<NSDraggingInfo>)sender
{
	[self showHideHighlight:NO];
	
	if (delegate!=nil && [delegate respondsToSelector: @selector(acceptPackageDrop:)]==YES)
    {
		return [delegate acceptPackageDrop:sender];
	}
	
    return NO;
}

- (BOOL) prepareForDragOperation: (id<NSDraggingInfo>)sender
{
	return YES;
}

- (void) concludeDragOperation:(id <NSDraggingInfo>)sender
{
	//[self setNeedsDisplay:YES];
}

- (void) draggingEnded:(id <NSDraggingInfo>)sender
{
	[self showHideHighlight:NO];
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
	[self showHideHighlight:NO];
}

@end

@implementation NSObject (ICPackageSettings)

- (NSDragOperation) validatePackageDrop:(id <NSDraggingInfo>)info
{
	return NSDragOperationNone;
}

- (BOOL) acceptPackageDrop:(id <NSDraggingInfo>)info
{
	return NO;
}

@end