/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPackageUtilities+Default.h"

#import "ICDocumentFormat+Constants.h"

#import "ICUUIDFactory.h"

#import "ICPreferences+Constants.h"

#import "ICPreferencesUtilities.h"

@implementation ICPackageUtilities (Default)

+ (NSString *) packageIdentifierFromName:(NSString *) inPackageName
{
	NSString * tPackageIdentifier=nil;
	
	if (inPackageName!=nil)
	{
		NSUserDefaults * tDefaults;
		NSString * tDefaultIdentifierPrefix;
		NSArray * tNameComponents;
		
		tDefaults=[NSUserDefaults standardUserDefaults];
		
		// Package Identifier
			
		tNameComponents=[inPackageName componentsSeparatedByString:@" "];
		
		tPackageIdentifier=[tNameComponents componentsJoinedByString:@"_"];
		
		if (tPackageIdentifier==nil)
		{
			tPackageIdentifier=@"";
		}
		
		tDefaultIdentifierPrefix=[[tDefaults objectForKey:ICPREFERENCES_TEMPLATES_KEYWORDS] objectForKey:ICPREFERENCES_TEMPLATES_COMPANY_IDENTIFIER_PREFIX];
		
		if (tDefaultIdentifierPrefix==nil)
		{
			tDefaultIdentifierPrefix=[ICPreferencesUtilities defaultPackageIdentifierPrefix];
		}
		
		if (tDefaultIdentifierPrefix!=nil)
		{
			NSString * tFormat=@"%@%@";
			
			if ([tDefaultIdentifierPrefix hasSuffix:@"."]==NO)
			{
				tFormat=@"%@.%@";
			}
			
			tPackageIdentifier=[NSString stringWithFormat:tFormat,tDefaultIdentifierPrefix,tPackageIdentifier];
		}
	}
	
	return tPackageIdentifier;
}

+ (NSMutableDictionary *) defaultPackageDictionaryWithExistingPackages:(NSArray *) inPackages
{
	NSMutableDictionary * tMutablePackageDictionary;
	
	tMutablePackageDictionary=[NSMutableDictionary dictionary];
	
	if (tMutablePackageDictionary!=nil)
	{
		NSMutableDictionary * tMutableSettingsDictionary;
		NSString * tPackageName;
		NSString * tPackageIdentifier;
		
		// PACKAGE TYPE
		
		[tMutablePackageDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGE_TYPE_PROJECT] forKey:ICDOCUMENT_PACKAGE_TYPE];
		
		// UUID
		
		[tMutablePackageDictionary setObject:[ICUUIDFactory string] forKey:ICDOCUMENT_PACKAGE_UUID];
		
		// SETTINGS
		
		// Package Name
		
		tPackageName=[ICPackageUtilities uniquePackageNameWithExistingPackages:inPackages];
		
		if (tPackageName==nil)
		{
			tPackageName=@"";
		}
		
		// Package Identifier
		
		tPackageIdentifier=[ICPackageUtilities packageIdentifierFromName:tPackageName];
		
		if (tPackageIdentifier!=nil)
		{
			tPackageIdentifier=[ICPackageUtilities uniquePackageIdentifierWithExistingPackages:inPackages originalIdentifier:tPackageIdentifier];
		}
		else
		{
			NSLog(@"[ICPackageUtilities defaultPackageDictionaryWithExistingPackages:] Unable to create package identifier");
			
			return nil;
		}
		
		// Default Settings
		
		tMutableSettingsDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tPackageName,ICDOCUMENT_PACKAGE_SETTINGS_NAME,
																					 tPackageIdentifier,ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER,
																					 @"1.0",ICDOCUMENT_PACKAGE_SETTINGS_VERSION,
																					 [NSNumber numberWithBool:NO],ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS,
																					 [NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_AUTHENTICATION_ROOT],ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION,
																					 [NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_NONE],ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION,
																					 [NSNumber numberWithInt:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED],ICDOCUMENT_PACKAGE_SETTINGS_LOCATION,
																					 nil];
																					 
																					 
		if (tMutableSettingsDictionary!=nil)
		{
			[tMutablePackageDictionary setObject:tMutableSettingsDictionary forKey:ICDOCUMENT_PACKAGE_SETTINGS];
		}
		else
		{
			NSLog(@"[ICPackageUtilities defaultPackageDictionaryWithExistingPackages:] Unable to create new package Settings dictionary");
			
			tMutablePackageDictionary=nil;
		}
	}
	else
	{
		NSLog(@"[ICPackageUtilities defaultPackageDictionaryWithExistingPackages:] Unable to create new package dictionary");
	}
	
	return tMutablePackageDictionary;
}

+ (NSMutableDictionary *) defaultReferencedPackageDictionaryWithExistingPackages:(NSArray *) inPackages
{
	NSMutableDictionary * tMutablePackageDictionary;
	
	tMutablePackageDictionary=[NSMutableDictionary dictionary];
	
	if (tMutablePackageDictionary!=nil)
	{
		NSMutableDictionary * tMutableSettingsDictionary;
		NSString * tPackageName;
		NSString * tPackageIdentifier;
		
		// PACKAGE TYPE
		
		[tMutablePackageDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGE_TYPE_REFERENCED] forKey:ICDOCUMENT_PACKAGE_TYPE];
		
		// UUID
		
		[tMutablePackageDictionary setObject:[ICUUIDFactory string] forKey:ICDOCUMENT_PACKAGE_UUID];
		
		// SETTINGS
		
		// Package Name
		
		tPackageName=[ICPackageUtilities uniquePackageNameWithExistingPackages:inPackages];
		
		if (tPackageName==nil)
		{
			tPackageName=@"";
		}
		
		// Package Identifier
		
		tPackageIdentifier=[ICPackageUtilities packageIdentifierFromName:tPackageName];
		
		// Default Settings
		
		tMutableSettingsDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tPackageName,ICDOCUMENT_PACKAGE_SETTINGS_NAME,
																					 tPackageIdentifier,ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER,
																					 @"1.0",ICDOCUMENT_PACKAGE_SETTINGS_VERSION,
																					 [NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_AUTHENTICATION_NONE],ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION,
																					 [NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_NONE],ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION,
																					 [NSNumber numberWithInt:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_HTTP_URL],ICDOCUMENT_PACKAGE_SETTINGS_LOCATION,
																					 nil];
																					 
																					 
		if (tMutableSettingsDictionary!=nil)
		{
			[tMutablePackageDictionary setObject:tMutableSettingsDictionary forKey:ICDOCUMENT_PACKAGE_SETTINGS];
		}
		else
		{
			NSLog(@"[ICPackageUtilities defaultReferencedPackageDictionaryWithExistingPackages:] Unable to create new package Settings dictionary");
			
			tMutablePackageDictionary=nil;
		}
	}
	else
	{
		NSLog(@"[ICPackageUtilities defaultReferencedPackageDictionaryWithExistingPackages:] Unable to create new package dictionary");
	}
	
	return tMutablePackageDictionary;
}

@end
