/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPaneTitleView.h"

@implementation ICPaneTitleView

- (void) viewWillMoveToWindow:(NSWindow *) inWindow
{
	if (inWindow==nil)
	{
		[NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayedScroll:) object:nil];
	}
}

- (void) setStringValue:(NSString *) inStringValue
{
	NSRect tBounds;
	NSAttributedString * tAttributedString;
	
	[NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayedScroll:) object:nil];
	
	[super setStringValue:inStringValue];
	
	tBounds=[self bounds];
	
	tAttributedString=[self attributedStringValue];
	
	boundingRect_=[tAttributedString boundingRectWithSize:tBounds.size options:NSStringDrawingUsesLineFragmentOrigin];
	
	boundingRect_.size.width+=5.0f;
	
	offset_=0.0f;
	
	if (NSWidth(boundingRect_)>NSWidth(tBounds))
	{
		
		[self performSelector:@selector(delayedScroll:) withObject:nil afterDelay:2.5f];
	}
}

- (void) setAttributedStringValue:(NSAttributedString *) inAttributedStringValue
{
	NSRect tBounds;
	
	[NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayedScroll:) object:nil];
	
	[super setAttributedStringValue:inAttributedStringValue];
	
	tBounds=[self bounds];
	
	boundingRect_=[inAttributedStringValue boundingRectWithSize:tBounds.size options:NSStringDrawingUsesLineFragmentOrigin];
	
	boundingRect_.size.width+=5.0f;
	
	offset_=0.0f;
	
	if (NSWidth(boundingRect_)>NSWidth(tBounds))
	{
		
		
		[self performSelector:@selector(delayedScroll:) withObject:nil afterDelay:2.5f];
	}
}

- (void) setFrame:(NSRect) inFrame
{
	NSRect tBounds;
	NSAttributedString * tAttributedString;
	
	[NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayedScroll:) object:nil];
	
	[super setFrame:inFrame];
	
	tBounds=[self bounds];
	
	tAttributedString=[self attributedStringValue];
	
	boundingRect_=[tAttributedString boundingRectWithSize:tBounds.size options:NSStringDrawingUsesLineFragmentOrigin];
	
	boundingRect_.size.width+=5.0f;
	
	offset_=0.0f;
	
	if (NSWidth(boundingRect_)>NSWidth(tBounds))
	{
		[self performSelector:@selector(delayedScroll:) withObject:nil afterDelay:2.5f];
	}
}

#pragma mark -

- (void) delayedScroll:(id) sender
{
	if ((NSMaxX(boundingRect_)-offset_)<=NSWidth([self bounds]))
	{
		offset_=0.0f;
		
		[self performSelector:@selector(delayedScroll:) withObject:nil afterDelay:3.0f];
	}
	else
	{
		offset_+=1.0f;
		
		[self setNeedsDisplay:YES];
		
		[self performSelector:@selector(delayedScroll:) withObject:nil afterDelay:0.05f];
	}
}

- (void) drawRect:(NSRect) inRect
{
	NSRect tRect;
	
	tRect=NSOffsetRect(boundingRect_,-offset_,0.0f);

	[[self cell] drawWithFrame:tRect inView:self];
}

@end
