/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPluginsManager.h"

#import "ICPlugin+Constants.h"

NSString * const ICPLUGINSMANAGER_UI_BUNDLE=@"UI_BUNDLE";

NSString * const ICPLUGINSMANAGER_CONVERTER_BUNDLE=@"CONVERTER_BUNDLE";

@implementation ICPluginsManager

- (id) init
{
	self=[super init];
	
	if (self!=nil)
	{
		NSFileManager * tFileManager;
        NSArray * tRequirementsList;
        NSString * tFolderName;
		
		tFileManager=[NSFileManager defaultManager];
		
        // Find the Requirements plugin folder
        
        tFolderName=[self folderName];
        
		if (tFolderName!=nil)
		{
			NSString * tFolderPath;
			NSUInteger i,tPluginsCount;
			
			tFolderPath=[[NSString stringWithString:@"/Library/PrivilegedHelperTools/fr.whitebox.packages"] stringByAppendingPathComponent:tFolderName];
        
			reverseDictionary_=[[NSMutableDictionary alloc] initWithCapacity:20];
		
			dictionary_=[[NSMutableDictionary alloc] initWithCapacity:20];
		
			tRequirementsList=[tFileManager directoryContentsAtPath:tFolderPath];
			
			tPluginsCount=[tRequirementsList count];
			
			for(i=0;i<tPluginsCount;i++)
			{
				 NSString * tSubPath=nil;
				 
				 tSubPath=[tRequirementsList objectAtIndex:i];
				
				if ([[tSubPath pathExtension] isEqualToString:@"plugin"]==YES)
				{
					NSString * tPluginPath;
					NSBundle * tBundle;
					
					tPluginPath=[tFolderPath stringByAppendingPathComponent:tSubPath];
					
					tBundle=[NSBundle bundleWithPath:tPluginPath];
					
					if (tBundle!=nil)
					{
						NSDictionary * tInfoDictionary;
						
						tInfoDictionary=[tBundle infoDictionary];
						
						if (tInfoDictionary!=nil)
						{
							NSString * tBundleIdentifier;
							
							tBundleIdentifier=[tInfoDictionary objectForKey:@"CFBundleIdentifier"];
							
							if (tBundleIdentifier!=nil)
							{
								NSString * tUIPluginPath;
								NSString * tConverterPluginPath;
								
								tUIPluginPath=[tBundle pathForResource:@"ui" ofType:@"bundle"];
								
								tConverterPluginPath=[tBundle pathForResource:@"converter" ofType:@"bundle"];
								
								if (tUIPluginPath!=nil /*&& tConverterPluginPath!=nil*/)
								{
									NSMutableDictionary * tPluginDictionary;
									
									tPluginDictionary=[NSMutableDictionary dictionary];
									
									if (tPluginDictionary!=nil)
									{
										NSBundle * tUIBundle;
										NSBundle * tConverterBundle;
										NSString * tLocalizedName;

										tUIBundle=[NSBundle bundleWithPath:tUIPluginPath];
										
										if (tUIBundle!=nil)
										{
											[tPluginDictionary setObject:tUIBundle forKey:ICPLUGINSMANAGER_UI_BUNDLE];
											
											tLocalizedName=[tUIBundle objectForInfoDictionaryKey:ICPLUGIN_NAME];
										
											if (tLocalizedName!=nil)
											{
												[tPluginDictionary setObject:tLocalizedName forKey:ICPLUGIN_NAME];
												
												[reverseDictionary_ setObject:tBundleIdentifier forKey:tLocalizedName];
											}
										}
										else
										{
											NSLog(@"[ICPluginsManager init] Error when creating bundle at \'%@\'",tUIPluginPath);
										}
										
										tConverterBundle=[NSBundle bundleWithPath:tConverterPluginPath];
										
										if (tConverterBundle!=nil)
										{
											[tPluginDictionary setObject:tConverterBundle forKey:ICPLUGINSMANAGER_CONVERTER_BUNDLE];
										}
										else
										{
											NSLog(@"[ICPluginsManager init] Error when creating bundle at \'%@\'",tConverterPluginPath);
										}
										
										if (tUIBundle!=nil /*&& tConverterBundle!=nil*/)
										{
											[dictionary_ setObject:tPluginDictionary forKey:tBundleIdentifier];
										}
									}
									else
									{
										NSLog(@"[ICPluginsManager init] Low Memory");
									}
								}
								else
								{
									if (tUIPluginPath==nil)
									{
										NSLog(@"[ICPluginsManager init] Missing UI plugin (%@)",tPluginPath);
									}
									
									if (tConverterPluginPath==nil)
									{
										NSLog(@"[ICPluginsManager init] Missing Converter plugin (%@)",tPluginPath);
									}
								}
							}
							else
							{
								NSLog(@"[ICPluginsManager init] Missing CFBundleIdentifier in \'%@\'",tPluginPath);
							}
						}
					}
				}
			}
		}
	}
	
	return self;
}

- (void) dealloc
{
	// A COMPLETER
	
	[dictionary_ release];
	
	[reverseDictionary_ release];
	
	[super dealloc];
}

#pragma mark -

- (NSString *) folderName
{
	return nil;
}

- (NSArray *) allPluginsIdentifier
{
	return [dictionary_ allKeys];
}

- (NSArray *) allPluginsNameSorted
{
	NSMutableArray * tAllKeys;
	
	tAllKeys=[[reverseDictionary_ allKeys] mutableCopy];
	
	if (tAllKeys!=nil)
	{
		[tAllKeys sortUsingSelector:@selector(caseInsensitiveCompare:)];
	}
	
	return [tAllKeys autorelease];
}

- (NSString *) localizedPluginNameForIdentifier:(NSString *) inIdentifier
{
	if (inIdentifier!=nil)
	{
		return [[dictionary_ objectForKey:inIdentifier] objectForKey:ICPLUGIN_NAME];
	}
	
	return nil;
}

- (NSString *) identifierForLocalizedPluginName:(NSString *) inLocalizedName
{
	if (inLocalizedName!=nil)
	{
		return [reverseDictionary_ objectForKey:inLocalizedName];
	}

	return nil;
}

#pragma mark -

- (id) createPluginConverterControllerForIdentifier:(NSString *) inIdentifier
{
	return [self createPluginConverterControllerForIdentifier:inIdentifier documentDictionary:nil];
}

- (id) createPluginConverterControllerForIdentifier:(NSString *) inIdentifier documentDictionary:(NSDictionary *) inDictionary
{
	if (dictionary_!=nil && inIdentifier!=nil)
	{
		NSMutableDictionary * tPluginDictionary;
		
		tPluginDictionary=[dictionary_ objectForKey:inIdentifier];
		
		if (tPluginDictionary!=nil)
		{
			NSBundle * tBundle;
			
			tBundle=[tPluginDictionary objectForKey:ICPLUGINSMANAGER_CONVERTER_BUNDLE];
			
			if (tBundle!=nil)
			{
				Class tPrincipalClass;
				
				tPrincipalClass=[tBundle principalClass];
				
				if (tPrincipalClass!=nil)
				{
					id tInstance;
					
					tInstance=[[tPrincipalClass alloc] init];
					
					if (tInstance!=nil && inDictionary!=nil)
					{
						if ([tInstance respondsToSelector:@selector(setDocumentDictionary:)]==YES)
						{
							[tInstance performSelector:@selector(setDocumentDictionary:) withObject:inDictionary];
						}
					}
					
					return tInstance;
				}
				else
				{
					NSLog(@"[ICPluginsManager createPluginConverterControllerForIdentifier:] Principal Class not found");
				}
			}
		}
		else
		{
			NSLog(@"[ICPluginsManager createPluginConverterControllerForIdentifier:] No plugin found for identifier %@",inIdentifier);
		}
	}
	
	return nil;
}

@end
