/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICPreferencePaneBuildController.h"
#import "ICFileTextField.h"

@implementation ICPreferencePaneBuildController

- (void) awakeFromNib
{
	[super awakeFromNib];
	
	[IBscratchFrame_ setDelegate:self];
	
	[IBplaySoundOnSuccessPopUpButton_ setMenu:[ICPreferencePaneBuildController soundMenu]];
	
	[IBplaySoundOnErrorsPopUpButton_ setMenu:[ICPreferencePaneBuildController soundMenu]];

	
	//[IBscratchLocation_
}

- (void) updateWithDefaults
{
    NSString * tPath;
    NSInteger tTag;
	BOOL isDirectory;
	NSImage * tImage;
	NSString * tSoundName;
	
	
    // Temporary Build Location
    
    tPath=[defaults_ objectForKey:ICPREFERENCES_TEMPORARY_BUILD_LOCATION];
    
    if (tPath==nil || [tPath length]==0)
    {
        tPath=ICPREFERENCES_DEFAULT_TEMPORARY_BUILD_LOCATION;
    }
    
    [IBscratchLocation_ setStringValue:tPath];
    
	if ([[NSFileManager defaultManager] fileExistsAtPath:tPath isDirectory:&isDirectory]==YES)
	{
		tImage=[[NSWorkspace sharedWorkspace] iconForFile:tPath];
		
		if (isDirectory==NO)
		{
			[IBscratchLocation_ setTextColor:[NSColor redColor]];
		}
	}
	else
	{
		tImage=[[NSWorkspace sharedWorkspace] iconForFileType:NSFileTypeForHFSTypeCode(kQuestionMarkIcon)];
			
		[IBscratchLocation_ setTextColor:[NSColor redColor]];
	}
	
	[tImage setScalesWhenResized:YES];
			
	[tImage setSize:NSMakeSize(32.0f,32.0f)];
	
	[IBscratchIcon_ setImage:tImage];
	
	// Build Window
    
    tTag=[defaults_ integerForKey:ICPREFERENCES_BUILD_SHOW_WINDOW];
    
    [IBshowBuildWindowPopUpButton_ selectItemWithTag:tTag];
    
    tTag=[defaults_ integerForKey:ICPREFERENCES_BUILD_HIDE_WINDOW];
    
    [IBhideBuildWindowPopUpButton_ selectItemWithTag:tTag];
    
    tTag=[defaults_ integerForKey:ICPREFERENCES_BUILD_UNSAVED_PROJECT];
    
	[IBunsavedProjectPopUpButton_ selectItemWithTag:tTag];
	
	// Build Result
	
	tSoundName=[defaults_ stringForKey:ICPREFERENCES_BUILD_PLAY_SUCCESS];
	
	if (tSoundName==nil || [tSoundName length]==0)
	{
		[IBplaySoundOnSuccessPopUpButton_ selectItemWithTag:-1];
	}
	else
	{
		[IBplaySoundOnSuccessPopUpButton_ selectItemAtIndex:[IBplaySoundOnSuccessPopUpButton_ indexOfItemWithTitle:tSoundName]];
	}
	
	tSoundName=[defaults_ stringForKey:ICPREFERENCES_BUILD_PLAY_ERRORS];
	
	if (tSoundName==nil || [tSoundName length]==0)
	{
		[IBplaySoundOnErrorsPopUpButton_ selectItemWithTag:-1];
	}
	else
	{
		[IBplaySoundOnErrorsPopUpButton_ selectItemAtIndex:[IBplaySoundOnErrorsPopUpButton_ indexOfItemWithTitle:tSoundName]];
	}
}

#pragma mark -

+ (NSMenu *) soundMenu
{
	NSMenu * tMenu=nil;
	NSArray * tLibraryArray;
	NSEnumerator * tEnumerator;
   
	tLibraryArray=NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,NSSystemDomainMask+NSLocalDomainMask,NO);

	tEnumerator=[tLibraryArray objectEnumerator];
	
	if (tEnumerator!=nil)
	{
		NSString * tLibraryPath;
		NSFileManager * tFileManager;
		NSMutableArray * tMutableArray;
		NSUInteger i,tSoundsCount;
		
		tFileManager=[NSFileManager defaultManager];
		
		tMutableArray=[NSMutableArray array];
		
		while (tLibraryPath=[tEnumerator nextObject])
        {
            NSString * tSoundsFolderPath;
			NSArray * tSoundsArray;
			
			
			tSoundsFolderPath=[tLibraryPath stringByAppendingPathComponent:@"Sounds"];
			
			tSoundsArray=[tFileManager directoryContentsAtPath:tSoundsFolderPath];
			
			tSoundsCount=[tSoundsArray count];
			
			for(i=0;i<tSoundsCount;i++)
			{
				NSString * tSoundFile;
				
				tSoundFile=[tSoundsArray objectAtIndex:i];
				
				if ([[tSoundFile pathExtension] caseInsensitiveCompare:@"aiff"]==NSOrderedSame)
				{
					[tMutableArray addObject:[tSoundFile stringByDeletingPathExtension]];
				}
			}
		}
		
		if ([tMutableArray count]>0)
		{
			// Sort by Names
			
			[tMutableArray sortUsingSelector:@selector(caseInsensitiveCompare:)];
			
			tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
    
			if (tMenu!=nil)
			{
				id tMenuItem;
				
				tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Mute",@"No comment")
														 action:nil
												  keyEquivalent:@""];
				
				if (tMenuItem!=nil)
				{
					[tMenuItem setTag:-1];
					
					[tMenu addItem:tMenuItem];
			
					[tMenuItem release];
				}
				
				tMenuItem=[NSMenuItem separatorItem];
				
				if (tMenuItem!=nil)
				{
					[tMenu addItem:tMenuItem];
				}
				
				tSoundsCount=[tMutableArray count];
				
				for(i=0;i<tSoundsCount;i++)
				{
					tMenuItem=[[NSMenuItem alloc] initWithTitle:[tMutableArray objectAtIndex:i]
														 action:nil
												  keyEquivalent:@""];
			
					if (tMenuItem!=nil)
					{
						[tMenu addItem:tMenuItem];
				
						[tMenuItem release];
					}
				}
			}
		}
	}
	
	return [tMenu autorelease];
}

- (IBAction) setScratchLocation:(id) sender
{
    NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:NO];
    [tOpenPanel setCanChooseDirectories:YES];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:YES];
    
	[[tOpenPanel _navView] setShowsHiddenFiles:YES];	// @"AppleShowAllFiles" = YES
	
    [tOpenPanel beginSheetForDirectory:[[IBscratchLocation_ stringValue] stringByExpandingTildeInPath]
                                  file:nil
                                 types:nil
                        modalForWindow:[IBview_ window]
                         modalDelegate:self
                        didEndSelector:@selector(scratchOpenPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) scratchOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        [defaults_ setObject:[sheet filename] forKey:ICPREFERENCES_TEMPORARY_BUILD_LOCATION];
    
		[self updateWithDefaults];
	}
}

#pragma mark -

- (BOOL) validateDropFiles:(NSArray *) inFilenames
{
	if ([inFilenames count]==1)
	{
		BOOL isDirectory;
		
		if ([[NSFileManager defaultManager] fileExistsAtPath:[inFilenames objectAtIndex:0] isDirectory:&isDirectory]==YES && isDirectory==YES)
		{
			return YES;
		}
	}
	
	return NO;
}

- (BOOL) acceptDropFiles:(NSArray *) inFilenames
{
	if ([inFilenames count]==1)
	{
		[defaults_ setObject:[inFilenames objectAtIndex:0] forKey:ICPREFERENCES_TEMPORARY_BUILD_LOCATION];
    
		[self updateWithDefaults];
		
		return YES;
	}
	
	return NO;
}

- (IBAction) switchUnsavedProject:(id) sender
{
	[defaults_ setInteger:[[IBunsavedProjectPopUpButton_ selectedItem] tag] forKey:ICPREFERENCES_BUILD_UNSAVED_PROJECT];
}

- (IBAction) switchShowBuildWindow:(id) sender
{
	[defaults_ setInteger:[[IBshowBuildWindowPopUpButton_ selectedItem] tag] forKey:ICPREFERENCES_BUILD_SHOW_WINDOW];
   
}

- (IBAction) switchHideBuildWindow:(id) sender
{
	[defaults_ setInteger:[[IBhideBuildWindowPopUpButton_ selectedItem] tag] forKey:ICPREFERENCES_BUILD_HIDE_WINDOW];
}

- (IBAction) switchPlaySoundOnSuccess:(id) sender
{
	NSMenuItem * tMenuItem;
	NSInteger tTag;
	NSString * tOldSoundName;
		
	tOldSoundName=[defaults_ stringForKey:ICPREFERENCES_BUILD_PLAY_SUCCESS];
		
	tMenuItem=[sender selectedItem];
	
	tTag=[tMenuItem tag];
	
	if (tTag==-1)
	{
		if ([tOldSoundName length]>0)
		{
			[defaults_ setObject:@"" forKey:ICPREFERENCES_BUILD_PLAY_SUCCESS];
		}
	}
	else
	{
		NSString * tSoundName;
		
		tSoundName=[tMenuItem title];
		
		if ([tOldSoundName length]==0 || [tOldSoundName caseInsensitiveCompare:tSoundName]!=NSOrderedSame)
		{
			[defaults_ setObject:tSoundName forKey:ICPREFERENCES_BUILD_PLAY_SUCCESS];
		}
		
		[[NSSound soundNamed:tSoundName] play];
	}
}

- (IBAction) switchPlaySoundOnErrors:(id) sender
{
	NSMenuItem * tMenuItem;
	NSInteger tTag;
	NSString * tOldSoundName;
		
	tOldSoundName=[defaults_ stringForKey:ICPREFERENCES_BUILD_PLAY_ERRORS];
	
	tMenuItem=[sender selectedItem];
	
	tTag=[tMenuItem tag];
	
	if (tTag==-1)
	{
		if ([tOldSoundName length]>0)
		{
			[defaults_ setObject:@"" forKey:ICPREFERENCES_BUILD_PLAY_ERRORS];
		}
	}
	else
	{
		NSString * tSoundName;
		
		tSoundName=[tMenuItem title];
		
		if ([tOldSoundName length]==0 || [tOldSoundName caseInsensitiveCompare:tSoundName]!=NSOrderedSame)
		{
			[defaults_ setObject:tSoundName forKey:ICPREFERENCES_BUILD_PLAY_ERRORS];
		}
					
		[[NSSound soundNamed:tSoundName] play];
	}
}

@end
