/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICPreferencePaneFilesController.h"

#import "ICDocumentFormat+Constants.h"

@implementation ICPreferencePaneFilesController

- (void) awakeFromNib
{
    id tObject;
    NSAttributedString * tAttributedString;
	
    [super awakeFromNib];
	
	// Default Values
	
	tObject=[defaults_ objectForKey:ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
	
	if (tObject==nil)
	{
		[defaults_ setBool:YES forKey:ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
	}
	
	tObject=[defaults_ objectForKey:ICPREFERENCES_FILES_SPLITFORKS_TOOL];
	
	if (tObject==nil)
	{
		[defaults_ setObject:SPLITFORKSTOOL_NAME_GOLDIN forKey:ICPREFERENCES_FILES_SPLITFORKS_TOOL];
	}
	
	// Set Reference Style Menu
	
	tAttributedString=[[NSAttributedString alloc] initWithString:[IBhighlightExcludedFiles_ title] attributes:[NSDictionary dictionaryWithObjectsAndKeys:[IBhighlightExcludedFiles_ font],NSFontAttributeName,
																																						 [NSNumber numberWithInt:NSUnderlineStyleSingle],NSStrikethroughStyleAttributeName,nil]];

	if (tAttributedString!=nil)
	{
		[IBhighlightExcludedFiles_ setAttributedTitle:tAttributedString];
		
		[tAttributedString release];
	}
}

#pragma mark -

- (void) updateWithDefaults
{
	NSFileManager * tFileManager;
    BOOL tSplitForksInstalled;
	BOOL tState;
	NSString * tSplitForksToolName;
	
	// Show All Files
	
	tState=[defaults_ boolForKey:ICPREFERENCES_FILES_SHOW_ALL_FILES];
	
	[IBshowAllFiles_ setState:(tState==YES) ? NSOnState: NSOffState];
	
	// Default Permission Mode
	
	tState=[defaults_ boolForKey:ICPREFERENCES_FILES_KEEP_PERMISSION];
	
	[IBdefaultKeepPermissionsMode_ setState:(tState==YES) ? NSOnState: NSOffState];
	
	
	
	// Show Customization Dialog
	
	tState=[defaults_ boolForKey:ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
	
	[IBshowCustomizationDialog_ setState:(tState==YES) ? NSOnState: NSOffState];
	
	// SplitForks vs goldin
	
	tFileManager=[NSFileManager defaultManager];
	
	tSplitForksInstalled=[tFileManager fileExistsAtPath:@"/Developer/Tools/SplitForks"];
	
	if (tSplitForksInstalled==NO)
    {
		// SplitForks is MIA
		
		[[IBsplitForksToolName_ cellWithTag:ICPREFERENCES_FILES_SPLITFORKSTOOL_SPLITFORKS_ID] setEnabled:NO];
	}
	
	tSplitForksToolName=[defaults_ objectForKey:ICPREFERENCES_FILES_SPLITFORKS_TOOL];
	
	if ([tSplitForksToolName isEqualToString:SPLITFORKSTOOL_NAME_GOLDIN]==YES || tSplitForksInstalled==NO)
	{
		[IBsplitForksToolName_ selectCellWithTag:ICPREFERENCES_FILES_SPLITFORKSTOOL_GOLDIN_ID];
	}
	else
	{
		[IBsplitForksToolName_ selectCellWithTag:ICPREFERENCES_FILES_SPLITFORKSTOOL_SPLITFORKS_ID];
	}
	
	// Highlight Excluded Files
	
	tState=[defaults_ boolForKey:ICPREFERENCES_FILES_HIGHLIGHT_EXCLUDED_FILES];
	
	[IBhighlightExcludedFiles_ setState:(tState==YES) ? NSOnState: NSOffState];
}

#pragma mark -

- (IBAction) switchShowAllFiles:(id) sender
{
	NSInteger tState;

	tState=[IBshowAllFiles_ state];

	[defaults_ setBool:(tState==NSOnState) forKey:ICPREFERENCES_FILES_SHOW_ALL_FILES];
}

- (IBAction) switchDefaultPermissionsMode:(id) sender
{
	NSInteger tState;
	
	tState=[IBdefaultKeepPermissionsMode_ state];
	
	[defaults_ setBool:(tState==NSOnState) forKey:ICPREFERENCES_FILES_KEEP_PERMISSION];
}



- (IBAction) switchShowCustomizationDialog:(id) sender
{
	NSInteger tState;
	
	tState=[IBshowCustomizationDialog_ state];
	
	[defaults_ setBool:(tState==NSOnState) forKey:ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
}

- (IBAction) switchSplitForksTool:(id) sender
{
	switch([[IBsplitForksToolName_ selectedCell] tag])
	{
		case ICPREFERENCES_FILES_SPLITFORKSTOOL_GOLDIN_ID:
		
			[defaults_ setObject:SPLITFORKSTOOL_NAME_GOLDIN forKey:ICPREFERENCES_FILES_SPLITFORKS_TOOL];
			
			break;
			
		case ICPREFERENCES_FILES_SPLITFORKSTOOL_SPLITFORKS_ID:
		
			[defaults_ setObject:SPLITFORKSTOOL_NAME_SPLITFORKS forKey:ICPREFERENCES_FILES_SPLITFORKS_TOOL];
			
			break;
	}
}

- (IBAction) switchHighlightExcludedFiles:(id) sender
{
	NSInteger tState;
	
	tState=[IBhighlightExcludedFiles_ state];
	
	[defaults_ setBool:(tState==NSOnState) forKey:ICPREFERENCES_FILES_HIGHLIGHT_EXCLUDED_FILES];
	
	// Post Notification
	
	[[NSNotificationCenter defaultCenter] postNotificationName:ICPREFERENCES_NOTIFICATION_FILES_HIGHLIGHT_EXCLUDED_FILES_DID_CHANGE
														object:nil];
}

@end
