/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICPreferencePaneTemplatesController.h"

#import "ICPreferencesUtilities.h"

@interface NSString (CompareTemplateKeyLocalized)

- (NSComparisonResult) compareTemplateKeyLocalized:(NSString *) inString;

@end

@implementation NSString (CompareLocalized)

- (NSComparisonResult) compareTemplateKeyLocalized:(NSString *) inString
{
	return [NSLocalizedStringFromTable(self,@"Preferences",@"") compare:NSLocalizedStringFromTable(inString,@"Preferences",@"")];
}

@end

@implementation ICPreferencePaneTemplatesController

- (void) awakeFromNib
{
	NSTextFieldCell * tPrototypeCell = nil;
    NSTableColumn *tableColumn = nil;
    
    [IBkeywordsArray_ setIntercellSpacing:NSMakeSize(3,0)];
	
    // Key
    
    tableColumn = [IBkeywordsArray_ tableColumnWithIdentifier: @"Key"];
    tPrototypeCell = [tableColumn dataCell];
    [tPrototypeCell setFont:[NSFont boldSystemFontOfSize:[NSFont smallSystemFontSize]]];
    
    // Value
    
    tableColumn = [IBkeywordsArray_ tableColumnWithIdentifier: @"Value"];
    tPrototypeCell = [tableColumn dataCell];
    [tPrototypeCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
    
    [super awakeFromNib];
}

- (void) dealloc
{
	[keysArray_ release];
	
	[keywordsDictionary_ release];

	[super dealloc];
}

#pragma mark -

- (void) updateWithDefaults
{
	NSDictionary * tDictionary;
    
    [keywordsDictionary_ release];
    
    tDictionary=[defaults_ dictionaryForKey:ICPREFERENCES_TEMPLATES_KEYWORDS];
    
    if (tDictionary!=nil)
    {
        keywordsDictionary_=[tDictionary mutableCopy];
    }
    else
    {
        keywordsDictionary_=[[NSMutableDictionary alloc] initWithObjectsAndKeys:[ICPreferencesUtilities defaultCompanyName],ICPREFERENCES_TEMPLATES_KEY_COMPANY_NAME,
																				[ICPreferencesUtilities defaultPackageIdentifierPrefix],ICPREFERENCES_TEMPLATES_COMPANY_IDENTIFIER_PREFIX,
																				nil];
        
        if (keywordsDictionary_!=nil)
        {
			[defaults_ setObject:keywordsDictionary_ forKey:ICPREFERENCES_TEMPLATES_KEYWORDS];
		}
	}
    
    [keysArray_ release];
    
    keysArray_=[[keywordsDictionary_ allKeys] mutableCopy];
	
	[keysArray_ sortUsingSelector:@selector(compareTemplateKeyLocalized:)];
    
    [IBkeywordsArray_ reloadData];
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [keysArray_ count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
{    
    NSString * tKey;
    
    tKey=[keysArray_ objectAtIndex:rowIndex];
    
    if ([[aTableColumn identifier] isEqualToString: @"Key"])
    {
        return NSLocalizedStringFromTable(tKey,@"Preferences",@"");
    }
    else
    if ([[aTableColumn identifier] isEqualToString: @"Value"])
    {
    	return [keywordsDictionary_ objectForKey:tKey];
    }
    
    return nil;
}

- (void)tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
    if ([[tableColumn identifier] isEqualToString: @"Value"])
    {
        NSString * tKey;
        
        tKey=[keysArray_ objectAtIndex:row];
        
		if ([tKey isEqualToString:ICPREFERENCES_TEMPLATES_COMPANY_IDENTIFIER_PREFIX]==YES)
		{
			if ([object hasSuffix:@"."]==YES)
			{
				object=[object substringToIndex:[object length]-1];
			}
		}
		
        [keywordsDictionary_ setObject:object forKey:tKey];
        
        [defaults_ setObject:keywordsDictionary_ forKey:ICPREFERENCES_TEMPLATES_KEYWORDS];
    }
}

@end
