/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPreferencesUtilities.h"

#import <AddressBook/AddressBook.h>

#import "NSArray+Reverse.h"

NSString * const ICPREFERENCES_TEMPLATES_DEFAULT_COMPANY_NAME=@"My Great Company";

NSString * const ICPREFERENCES_TEMPLATES_DEFAULT_COMPANY_IDENTIFIER_PREFIX=@"com.mygreatcompany.pkg";

@implementation ICPreferencesUtilities

+ (NSString *) defaultCompanyName
{
	ABAddressBook * tSharedAddressBook;
	
	tSharedAddressBook=[ABAddressBook sharedAddressBook];
	
	if (tSharedAddressBook!=nil)
	{
		ABPerson * tMe;
		
		tMe=[tSharedAddressBook me];
		
		if (tMe!=nil)
		{
			NSString * tCompanyProperty;
			
			tCompanyProperty=(NSString *) [tMe valueForProperty:kABOrganizationProperty];
			
			if (tCompanyProperty!=nil && [tCompanyProperty length]>0)
			{
				return tCompanyProperty;
			}
		}
	}
	
	return [NSString stringWithString:ICPREFERENCES_TEMPLATES_DEFAULT_COMPANY_NAME];
}

+ (NSArray *) commonISPDomainName
{
	static NSArray * sCommonISPDomainName=nil;
	
	if (sCommonISPDomainName==nil)
	{
		sCommonISPDomainName=[[NSArray alloc] initWithObjects:@"aol.com",
															  @"gmail.com",
															  @"gmx.com",
															  @"hotmail.com",
															  @"live.com",
															  @"mac.com",
															  @"me.com",
															  @"msn.com",
															  @"yahoo.com",
															  @"ymail.com",
															  
															  // Belgium
															  
															  @"mail.be",
															  
															  // Canada
															  
															  @"sympatico.ca",
															  
															  // France
															  
															  @"aliceadsl.fr",
															  @"caramail.com",
															  @"club-internet.fr",
															  @"free.fr",
															  @"freesbee.fr",
															  @"hotmail.fr",
															  @"infonie.fr",
															  @"laposte.net",
															  @"lavache.com",
															  @"libertysurf.fr",
															  @"live.fr",
															  @"orange.fr",
															  @"wanadoo.fr",
															  @"worldonline.fr",
															  @"voila.fr",
															  @"yahoo.fr",
															  
															  // UK
															  
															  @"gmx.co.uk",
															  
															  // US
															  
															  @"gmx.us",
															  
															  // Disposable
															  
															  @"baxomale.ht.cx",
															  @"maboard.com",
															  @"tilien.com",
															  
															  nil];
															 
	}
	
	return sCommonISPDomainName;
}

+ (NSString *) defaultPackageIdentifierPrefix
{
	ABAddressBook * tSharedAddressBook;
	
	tSharedAddressBook=[ABAddressBook sharedAddressBook];
	
	if (tSharedAddressBook!=nil)
	{
		ABPerson * tMe;
		
		tMe=[tSharedAddressBook me];
		
		if (tMe!=nil)
		{
			ABMultiValue * tMailProperty;
			
			tMailProperty=(ABMultiValue *) [tMe valueForProperty:kABEmailProperty];
			
			if (tMailProperty!=nil)
			{
				NSUInteger i,tCount;
				
				tCount=[tMailProperty count];
				
				for(i=0;i<tCount;i++)
				{
					if ([[tMailProperty labelAtIndex:i] isEqualToString:kABEmailWorkLabel]==YES)
					{
						NSString * tMailAddress;
						
						tMailAddress=[[tMailProperty valueAtIndex:i] lowercaseString];
						
						if (tMailAddress!=nil && [tMailAddress length]>4)
						{
							NSArray * tMailAddressComponents;
							
							tMailAddressComponents=[tMailAddress componentsSeparatedByString:@"@"];
							
							if ([tMailAddressComponents count]>1)
							{
								NSString * tDomainName;
							
								tDomainName=[tMailAddressComponents objectAtIndex:1];
							
								if ([tDomainName length]>3)
								{
									// Check it's not a well-known ISP domain name
									
									if ([[ICPreferencesUtilities commonISPDomainName] containsObject:tDomainName]==NO)
									{
										NSArray * tReversedDomainNameComponents;
									
										tReversedDomainNameComponents=[[tDomainName componentsSeparatedByString:@"."] reversedArray];
									
										if (tReversedDomainNameComponents!=nil)
										{
											NSString * tReversedDomainName;
										
											tReversedDomainName=[tReversedDomainNameComponents componentsJoinedByString:@"."];
										
											return [NSString stringWithFormat:@"%@.pkg",tReversedDomainName];
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	return [NSString stringWithString:ICPREFERENCES_TEMPLATES_DEFAULT_COMPANY_IDENTIFIER_PREFIX];
}

@end
