/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPreferencesWindowController.h"
#import "ICPreferencePaneController.h"

NSString * const ICPREFERENCESWINDOW_ORDER=@"Order";
NSString * const ICPREFERENCESWINDOW_DESCRIPTIONS=@"Descriptions";

NSString * const ICPREFERENCESWINDOW_CLASSNAME=@"ClassName";
NSString * const ICPREFERENCESWINDOW_ICON=@"Icon";
NSString * const ICPREFERENCESWINDOW_NAME=@"Name";
NSString * const ICPREFERENCESWINDOW_NIB=@"Nib";

NSString * const ICPREFERENCESWINDOW_SELECTED_IDENTIFIER=@"preferences.ui.selected.identifier";

@implementation ICPreferencesWindowController

+ (ICPreferencesWindowController *) sharedController
{
	static ICPreferencesWindowController * sPreferencesWindowController=nil;
	
	if (sPreferencesWindowController==nil)
	{
		sPreferencesWindowController=[ICPreferencesWindowController alloc];
		
		if (sPreferencesWindowController!=nil)
		{
			if ([NSBundle loadNibNamed:@"ICPreferencesWindow" owner:sPreferencesWindowController]==NO)
			{
				NSBeep();
				
				NSLog(@"[ICPreferencesWindowController sharedController] Packages can't find the \"%@\".nib resource file.",@"ICPreferencesWindow");
			}
		}
		else
		{
			NSLog(@"[ICPreferencesWindowController sharedController] Not enough memory");
		}
	}
	
	return sPreferencesWindowController;
}

- (void) awakeFromNib
{
    NSString * tPath;
    
    tPath=[[NSBundle mainBundle] pathForResource:@"PreferencesWindowDescription" ofType:@"plist"];
    
    if (tPath!=nil)
    {
        panesDictionary_=[[NSDictionary alloc] initWithContentsOfFile:tPath];
        
        if (panesDictionary_!=nil)
        {
            // Create the required controllers
            
            NSDictionary * tDescriptionsDictionary;
            
            tDescriptionsDictionary=[panesDictionary_ objectForKey:ICPREFERENCESWINDOW_DESCRIPTIONS];
            
            if (tDescriptionsDictionary!=nil)
            {
                preferencePaneControllerDictionary_=[[NSMutableDictionary alloc] initWithCapacity:[tDescriptionsDictionary count]];
                
                if (preferencePaneControllerDictionary_!=nil)
                {
                    NSEnumerator * tKeyEnumerator;
                    NSString * tPaneIdentifier;
                    NSToolbar * tToolBar;
                     
                    tKeyEnumerator=[tDescriptionsDictionary keyEnumerator];
                    
                    while (tPaneIdentifier=[tKeyEnumerator nextObject])
                    {
                        NSDictionary * tPaneDescriptionDictionary;
                        
                        tPaneDescriptionDictionary=[tDescriptionsDictionary objectForKey:tPaneIdentifier];
                        
                        if (tPaneDescriptionDictionary!=nil)
                        {
                            NSString * tClassName;
                            NSString * tNibName;
                            
                            tClassName=[tPaneDescriptionDictionary objectForKey:ICPREFERENCESWINDOW_CLASSNAME];
                            
                            tNibName=[tPaneDescriptionDictionary objectForKey:ICPREFERENCESWINDOW_NIB];
                        
                            if (tClassName!=nil && tNibName!=nil)
                            {
                                ICPreferencePaneController * nPaneController;
                                
                                nPaneController=[NSClassFromString(tClassName) alloc];
                                
                                if (nPaneController!=nil)
                                {
                                    if ([NSBundle loadNibNamed:tNibName owner:nPaneController]==YES)
                                    {
                                        [preferencePaneControllerDictionary_ setObject:nPaneController forKey:tPaneIdentifier];
										
										[nPaneController updateWithDefaults];
                                    }
                                    else
                                    {
                                        NSLog(@"[ICPreferencesWindowController awakeFromNib] Unable to load nib named \'%@\'",tNibName);
										
                                    }
									
									[nPaneController release];
                                }
                            }
                        }
                    }
                
                    tToolBar = [[NSToolbar alloc] initWithIdentifier: @"fr.whitebox.packages.preference.toolbar"];
                
                    if (tToolBar!=nil)
                    {
                        NSString * tSelectedIdentifier;
						
						[tToolBar setAllowsUserCustomization: NO];
                    
                        [tToolBar setDisplayMode: NSToolbarDisplayModeIconAndLabel];
                    
                        [tToolBar setDelegate: self];
                    
                        [IBwindow_ setToolbar: tToolBar];
                        
                        
                        
                        // Show the first pane
                        
                        if ([tToolBar respondsToSelector:@selector(setSelectedItemIdentifier:)]==YES)
						{
							[tToolBar setSelectedItemIdentifier:[[self toolbarDefaultItemIdentifiers:tToolBar] objectAtIndex:0]];
						}
						
						tSelectedIdentifier=[[NSUserDefaults standardUserDefaults] objectForKey:ICPREFERENCESWINDOW_SELECTED_IDENTIFIER];
						
						if (tSelectedIdentifier!=nil)
						{
							[tToolBar setSelectedItemIdentifier:tSelectedIdentifier];
							
							[self showPaneWithIdentifier:tSelectedIdentifier];
						}
						else
						{
							[self showPane:[[tToolBar items] objectAtIndex:0]];
						}
						
						[tToolBar release];
					}
                }
            }
        }
        else
        {
            NSLog(@"[ICPreferencesWindowController awakeFromNib] Low memory");
        }
    }
    else
    {
        NSLog(@"[ICPreferencesWindowController awakeFromNib] Missing file: PreferencesWindowDescription.plist");
    }
    
	[IBwindow_ setShowsToolbarButton:NO];
}

#pragma mark -

- (NSToolbarItem *) toolbar:(NSToolbar *) toolbar itemForItemIdentifier:(NSString *) inItemIdentifier willBeInsertedIntoToolbar:(BOOL) willBeInserted
{
    NSToolbarItem * tToolbarItem;
    NSDictionary * tItemDictionary;
    
	tToolbarItem = [[NSToolbarItem alloc] initWithItemIdentifier: inItemIdentifier];
	
    tItemDictionary=[[panesDictionary_ objectForKey:ICPREFERENCESWINDOW_DESCRIPTIONS] objectForKey:inItemIdentifier];
    
    if (tItemDictionary!=nil)
    {
        [tToolbarItem setLabel:[tItemDictionary objectForKey:ICPREFERENCESWINDOW_NAME]];
	
        [tToolbarItem setImage:[NSImage imageNamed:[tItemDictionary objectForKey:ICPREFERENCESWINDOW_ICON]]];
        
        [tToolbarItem setTarget:self];
	
        [tToolbarItem setAction:@selector(showPane:)];
		
		[tToolbarItem autorelease];
    }
    else
    {
        [tToolbarItem release];
		
		tToolbarItem=nil;
    }
    
    return tToolbarItem;
}

- (NSArray *) toolbarDefaultItemIdentifiers: (NSToolbar *) toolbar
{
    return [panesDictionary_ objectForKey:ICPREFERENCESWINDOW_ORDER];
}

- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar
{
    return [panesDictionary_ objectForKey:ICPREFERENCESWINDOW_ORDER];
}

- (NSArray *)toolbarSelectableItemIdentifiers:(NSToolbar *)toolbar
{
	return [panesDictionary_ objectForKey:ICPREFERENCESWINDOW_ORDER];
}

- (BOOL) validateToolbarItem: (NSToolbarItem *) toolbarItem
{
    return YES;
}

#pragma mark -

- (void) showPaneWithIdentifier:(NSString *) inIdentifier
{
	if (inIdentifier!=nil)
	{
		ICPreferencePaneController * tPreferencePaneController;
    
        if (currentView_!=nil)
        {
            [currentView_ removeFromSuperview];
            
            currentView_=nil;
        }
        
        tPreferencePaneController=[preferencePaneControllerDictionary_ objectForKey:inIdentifier];
        
        if (tPreferencePaneController!=nil)
        {
            NSRect tNewContentRect;
			NSRect tOldWindowFrame;
			NSRect tWindowFrame;
			NSDictionary * tItemDictionary;
			
			tOldWindowFrame=[IBwindow_ frame];
			
			currentView_=[tPreferencePaneController view];
			
			tNewContentRect=[[IBwindow_ contentView] bounds];
			
			tNewContentRect.size=[currentView_ frame].size;
			
			tWindowFrame=[IBwindow_ frameRectForContentRect:tNewContentRect];
			
			tWindowFrame.origin.x=NSMinX(tOldWindowFrame);
			
			tWindowFrame.origin.y=NSMaxY(tOldWindowFrame)-NSHeight(tWindowFrame);
			
			tItemDictionary=[[panesDictionary_ objectForKey:ICPREFERENCESWINDOW_DESCRIPTIONS] objectForKey:inIdentifier];
    
			if (tItemDictionary!=nil)
			{
				[IBwindow_ setTitle:[tItemDictionary objectForKey:ICPREFERENCESWINDOW_NAME]];
			}
            
            [IBwindow_ setFrame:tWindowFrame display:YES animate:YES];
            
            [[IBwindow_ contentView] addSubview:currentView_];
			
			[[NSUserDefaults standardUserDefaults] setObject:inIdentifier forKey:ICPREFERENCESWINDOW_SELECTED_IDENTIFIER];
        }
	}
}

- (IBAction) showPane:(id) sender
{
    if (sender!=nil)
    {
        [self showPaneWithIdentifier:[sender itemIdentifier]];
    }
}

- (void) showWindow
{
    if ([IBwindow_ isVisible]==NO)
    {
        [IBwindow_ center];
	}
	
	[IBwindow_ makeKeyAndOrderFront:self];
}

@end
