/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationBox.h"

#define ICPRESENTATIONBOX_CORNER_RADIUS		10.0f

#define ICPRESENTATIONBOX_LINE_WIDTH		2.0f

#define ICPRESENTATIONBOX_TEXT_WHITE_SPACE	6.0f

@implementation ICPresentationBox

- (void) drawRect:(NSRect) inFrame
{
	NSRect tBounds;
	NSRect tTitleRect;
	CGFloat tWidth;
	CGFloat tHeight;
	CGFloat tSideWidth;
	CGFloat tTextHeight;
	NSBezierPath * tBezierPath;
	
	tBounds=[self bounds];
	
	tTitleRect=[self titleRect];
	
	
	
	tTextHeight=16.0f;
	
	tWidth=NSWidth(tBounds);
	tHeight=NSHeight(tBounds);
	
	tTitleRect.origin.x=_CGFloatRound((tWidth-NSWidth(tTitleRect))*0.5f)+2.0f;
	
	tTitleRect.origin.y=_CGFloatRound(tHeight-tTextHeight);
	
	tSideWidth=_CGFloatRound((tWidth-NSWidth(tTitleRect))*0.5f-ICPRESENTATIONBOX_CORNER_RADIUS-ICPRESENTATIONBOX_TEXT_WHITE_SPACE);
	
	// Draw Frame
	
	tBezierPath=[NSBezierPath bezierPath];
	
	if (tBezierPath!=nil)
	{
		[[NSColor colorWithDeviceWhite:0.25f alpha:1.0f] set];
		
		[tBezierPath setLineWidth:ICPRESENTATIONBOX_LINE_WIDTH];
		
		if (tSideWidth>0)
		{
			[tBezierPath moveToPoint:NSMakePoint(ICPRESENTATIONBOX_CORNER_RADIUS+tSideWidth,_CGFloatRound(tHeight-tTextHeight*0.5f-1.0f)-0.5f)];
			
			[tBezierPath lineToPoint:NSMakePoint(_CGFloatRound(ICPRESENTATIONBOX_CORNER_RADIUS+ICPRESENTATIONBOX_LINE_WIDTH*0.5f)+0.5f,_CGFloatRound(tHeight-tTextHeight*0.5f-1.0f))];
		}
		else
		{
			[tBezierPath moveToPoint:NSMakePoint(_CGFloatRound(ICPRESENTATIONBOX_CORNER_RADIUS+ICPRESENTATIONBOX_LINE_WIDTH*0.5f),_CGFloatRound(tHeight-tTextHeight*0.5f-1.0f))];
		}
		
		[tBezierPath appendBezierPathWithArcWithCenter:NSMakePoint(_CGFloatRound(ICPRESENTATIONBOX_CORNER_RADIUS+ICPRESENTATIONBOX_LINE_WIDTH*0.5f)+0.5f,_CGFloatRound(tHeight-tTextHeight*0.5f-1.0f)-ICPRESENTATIONBOX_CORNER_RADIUS)
												radius:ICPRESENTATIONBOX_CORNER_RADIUS
											startAngle:90.0f
											  endAngle:180.0f];
		
		[tBezierPath lineToPoint:NSMakePoint(_CGFloatRound(ICPRESENTATIONBOX_LINE_WIDTH*0.5f)+0.5f,_CGFloatRound(ICPRESENTATIONBOX_LINE_WIDTH*0.5f+ICPRESENTATIONBOX_CORNER_RADIUS)+0.5f)];
		
		[tBezierPath appendBezierPathWithArcWithCenter:NSMakePoint(_CGFloatRound(ICPRESENTATIONBOX_LINE_WIDTH*0.5f+ICPRESENTATIONBOX_CORNER_RADIUS)+0.5f,_CGFloatRound(ICPRESENTATIONBOX_LINE_WIDTH*0.5f+ICPRESENTATIONBOX_CORNER_RADIUS)+0.5f)
												radius:ICPRESENTATIONBOX_CORNER_RADIUS
											startAngle:180.0f
											  endAngle:270.0f];
		
		[tBezierPath lineToPoint:NSMakePoint(tWidth-_CGFloatRound(ICPRESENTATIONBOX_LINE_WIDTH*0.5f+ICPRESENTATIONBOX_CORNER_RADIUS)-0.5f,_CGFloatRound(ICPRESENTATIONBOX_LINE_WIDTH*0.5f)+0.5f)];
		
		[tBezierPath appendBezierPathWithArcWithCenter:NSMakePoint(tWidth-_CGFloatRound(ICPRESENTATIONBOX_LINE_WIDTH*0.5f+ICPRESENTATIONBOX_CORNER_RADIUS)-0.5f,_CGFloatRound(ICPRESENTATIONBOX_LINE_WIDTH*0.5f+ICPRESENTATIONBOX_CORNER_RADIUS)+0.5f)
												radius:ICPRESENTATIONBOX_CORNER_RADIUS
											startAngle:270.0f
											  endAngle:360.0f];
		
		[tBezierPath lineToPoint:NSMakePoint(_CGFloatRound(tWidth-ICPRESENTATIONBOX_LINE_WIDTH*0.5f)-0.5f,_CGFloatRound(tHeight-tTextHeight*0.5f-1.0f)-ICPRESENTATIONBOX_CORNER_RADIUS-0.5f)];
		
		[tBezierPath appendBezierPathWithArcWithCenter:NSMakePoint(_CGFloatRound(tWidth-ICPRESENTATIONBOX_LINE_WIDTH*0.5f)-ICPRESENTATIONBOX_CORNER_RADIUS-0.5f,_CGFloatRound(tHeight-tTextHeight*0.5f-1.0f)-ICPRESENTATIONBOX_CORNER_RADIUS-0.5f)
												radius:ICPRESENTATIONBOX_CORNER_RADIUS
											startAngle:0.0f
											  endAngle:90.0f];
	
		if (tSideWidth>0)
		{
			[tBezierPath lineToPoint:NSMakePoint(tWidth-ICPRESENTATIONBOX_CORNER_RADIUS-tSideWidth,_CGFloatRound(tHeight-tTextHeight*0.5f-1.0f)-0.5f)];
		}
		
		[tBezierPath stroke];
	}
		
	// Draw Text
										
	[[self title] drawInRect:tTitleRect withAttributes:[NSDictionary dictionaryWithObjectsAndKeys:[self titleFont], NSFontAttributeName,
																								  [NSColor colorWithDeviceWhite:0.25f alpha:1.0f],NSForegroundColorAttributeName,
																								  nil]];
}

@end
