/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationImageView.h"

#import "NSFileManager+fileTypes.h"

#import "ICBackgroundUtilities.h"

#import "NSBezierPath+RoundedRect.h"

#import <ApplicationServices/ApplicationServices.h>

#include <Carbon/Carbon.h>

@implementation NSObject (ICPresentationImageView)

- (void) imageDidChange:(NSString *) inPath
{
}

@end

@implementation ICPresentationImageView

- (id) delegate
{
    return delegate;
}

- (void) setDelegate:(id) inDelegate
{
    delegate=inDelegate;
}

- (void) setDrawGrowBox:(BOOL) inDrawGrowBox
{
	if (inDrawGrowBox!=drawGrowBox_)
	{
		drawGrowBox_=inDrawGrowBox;
		
		[self setNeedsDisplay:YES];
	}
}

#pragma mark -

- (void) drawRect:(NSRect) inRect
{
	NSRect tBounds;

    [super drawRect:inRect];
	
	tBounds=[self bounds];
	
	if (drawGrowBox_==YES)	// A COMPLETER
	{
		[NSGraphicsContext saveGraphicsState];
		
		[[NSGraphicsContext currentContext] setShouldAntialias:NO];
		
		[[NSColor colorWithDeviceWhite:0.8345f alpha:0.5451f] set];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+1.0f) toPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+1.0f)];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-6.0f,NSMinY(tBounds)+1.0f) toPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+5.0f)];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-10.0f,NSMinY(tBounds)+1.0f) toPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+9.0f)];
		
		
		[[NSColor colorWithDeviceWhite:0.1304f alpha:0.5412f] set];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-3.0f,NSMinY(tBounds)+1.0f) toPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+2.0f)];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-7.0f,NSMinY(tBounds)+1.0f) toPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+6.0f)];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-11.0f,NSMinY(tBounds)+1.0f) toPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+10.0f)];
		
		
		[[NSColor colorWithDeviceWhite:0.0f alpha:0.1451f] set];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-4.0f,NSMinY(tBounds)+1.0f) toPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+3.0f)];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-8.0f,NSMinY(tBounds)+1.0f) toPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+7.0f)];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-12.0f,NSMinY(tBounds)+1.0f) toPoint:NSMakePoint(NSMaxX(tBounds)-2.0f,NSMinY(tBounds)+11.0f)];
		
		
		[NSGraphicsContext restoreGraphicsState];
		
	}
	
	if (highlighted_==YES)
	{
		NSBezierPath * tPath;
		
		tPath=[NSBezierPath bezierPathWithRoundedRect:NSInsetRect(tBounds,2.0f,2.0f) radius:8.0f];
		
		if (tPath!=nil)
		{
			[tPath setLineWidth:3.0f];
			
			[[NSColor alternateSelectedControlColor] set];
			
			[tPath stroke];
		}
		
	}
}

#pragma mark -

- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>)sender
{
    NSPasteboard * tPasteBoard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
	
    tPasteBoard = [sender draggingPasteboard];

    if ( [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
    {
        if (sourceDragMask & NSDragOperationCopy)
        {
            NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
			if (tFiles!=nil && [tFiles count]==1)
			{
				NSString * tFilePath;
				BOOL tImageFormatSupported=NO;
				
				tFilePath=[tFiles objectAtIndex:0];
				
				tImageFormatSupported=[[NSFileManager defaultManager] IC_fileAtPath:tFilePath matchesTypes:[ICBackgroundUtilities backgroundImageTypes]];
				
				if (tImageFormatSupported==NO)
				{
					NSURL * tURL;
					CGImageSourceRef tSourceRef;
					
					tURL = [NSURL fileURLWithPath:tFilePath];
					
					tSourceRef = CGImageSourceCreateWithURL((CFURLRef) tURL, NULL);
					
					if (tSourceRef!=NULL)
					{
						NSString * tImageUTI;
						
						tImageUTI=(NSString *) CGImageSourceGetType(tSourceRef);
						
						if (tImageUTI!=nil)
						{
							tImageFormatSupported=[[ICBackgroundUtilities backgroundImageUTIs] containsObject:tImageUTI];
						}
						
						// Release Memory
						
						CFRelease(tSourceRef);
					}
				}
				
				if (tImageFormatSupported==YES)
				{
					highlighted_=YES;
			
					[self setNeedsDisplay:YES];
            
					return NSDragOperationCopy;
				}
			}
        }
    }
    
    return NSDragOperationNone;
}

- (BOOL) performDragOperation:(id <NSDraggingInfo>)sender
{
    NSPasteboard * tPasteBoard;
	
    tPasteBoard = [sender draggingPasteboard];

    if ( [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
    {
        NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
		if (tFiles!=nil && [tFiles count]==1)
		{
			NSString * tFilePath;
			NSImage * tImage;
			
			tFilePath=[tFiles objectAtIndex:0];
			
			tImage=[[NSImage alloc] initWithContentsOfFile:tFilePath];
			
			if (tImage!=nil)
			{
				if (delegate!=nil)
				{
					[delegate imageDidChange:tFilePath];
				}
				
				[self setImage:tImage];
				
				[tImage release];
				
				return YES;
			}
			
			/*if ([[NSFileManager defaultManager] IC_fileAtPath:tFilePath matchesTypes:[ICBackgroundUtilities backgroundImageTypes]]==YES)
			{
				if (delegate!=nil)
				{
					[delegate imageDidChange:tFilePath];
				}
				
				return YES;
			}*/
		}
    }
	
    return NO;
}

- (BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	highlighted_=NO;
	
    [self setNeedsDisplay:YES];
    
    return YES;
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
    highlighted_=NO;
	
    [self setNeedsDisplay:YES];
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
}

@end
