/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationInspectorBackgroundController.h"

#import "ICBackgroundUtilities.h"

#import "ICPreferences+Constants.h"

#import "ICReferencedPopupButton.h"

#import "NSString+Iceberg.h"
#import "NSBezierPath+RoundedRect.h"
#import "NSFileManager+fileTypes.h"

#import "ICDocument.h"

#import "NSDocument+Iceberg.h"

@implementation ICPresentationInspectorBackgroundController

+ (ICPresentationInspectorBackgroundController *) controller
{
	ICPresentationInspectorBackgroundController * nController;
    
    nController=[ICPresentationInspectorBackgroundController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICBackgroundInspectorView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICBackgroundInspectorView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND];
	
	if (dictionary_==nil)
	{
		// Create default Background dictionary
		
		dictionary_=[ICBackgroundUtilities defaultBackgroundDictionary];
	
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND];
		}
	}
	
	if (dictionary_!=nil)
	{
		NSWindow * tWindow;
		
		[dictionary_ retain];
		
		[self upgradeBackgroundSettings];
		
		// Register for Notifications
		
		tWindow=[[self document] windowForSheet];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidBecomeMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidBecomeMainNotification 
												   object:tWindow];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidResignMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidResignMainNotification 
												   object:tWindow];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:ICPRESENTATION_NOTIFICATION_BACKGROUND_SETTINGS_DID_CHANGE object:[self document]];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(backgroundSettingsDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_BACKGROUND_SETTINGS_DID_CHANGE 
												   object:[self document]];
	}
}

- (void) dealloc
{	
	[super dealloc];
}

- (void) awakeFromNib
{
	[IBcustomBackgroundPopupButton_ setDelegate:self];
}

#pragma mark -

- (void) upgradeBackgroundSettings
{
	NSNumber * tNumber;
	BOOL tUseCustomBackground=NO;
	NSMutableDictionary * tCustomPath;
		
	// Initialize the controls
		
	[IBsizeLabel_ setStringValue:@""];
	
	// Custom
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_CUSTOM];
	
	if (tNumber==nil || [tNumber boolValue]==NO)
	{
		[IBshowPopupButton_ selectItemWithTag:0];
		
		[IBbottomView_ setHidden:YES];
	}
	else
	{
		[IBshowPopupButton_ selectItemWithTag:1];
		
		[IBbottomView_ setHidden:NO];
		
		tUseCustomBackground=YES;
	}

	// Path
	
	[IBcustomBackgroundPopupButton_ setEnabled:tUseCustomBackground];
	
	if (tUseCustomBackground==YES)
	{
		NSString * tPath=nil;
		NSInteger tPathType;
		BOOL tFileExists=YES;
		
		tPathType=[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
		
		tCustomPath=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
		
		if (tCustomPath!=nil)
		{
			NSString * tAbsolutePath;
			
			tPath=[tCustomPath objectForKey:ICDOCUMENT_PATH];
			
			tPathType=[[tCustomPath objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
			
			tAbsolutePath=tPath;
			
			if (tAbsolutePath!=nil)
			{
				if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
				{
					tAbsolutePath=[tAbsolutePath stringByAbsolutingWithPath:[document_ folder]];
				}
				else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
				{
					tAbsolutePath=[tAbsolutePath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
				}
			
				if (tAbsolutePath!=nil)
				{
					NSFileManager * tFileManager;
					
					tFileManager=[NSFileManager defaultManager];
					
					if (tFileManager!=nil)
					{
						tFileExists=[tFileManager fileExistsAtPath:tAbsolutePath];
						
						[IBcustomBackgroundPopupButton_ setPathDoesNotExist:!tFileExists];
						
						if (tFileExists==YES)
						{
							NSSize tSize;
							
							tSize=[ICBackgroundUtilities sizeOfImageAtPath:tAbsolutePath];
							
							[IBsizeLabel_ setStringValue:[NSString stringWithFormat:@"%ldx%ld",lround(tSize.width),lround(tSize.height)]];
						}
					}
				}
			}
		}
		
		// Path Type
		
		[IBcustomBackgroundPopupButton_ setPathType:tPathType];
		
		// Path
			
		[IBcustomBackgroundPopupButton_ setToolTip:tPath];
		
		[IBcustomBackgroundPopupButton_ setFileNameWithPath:tPath];
	}

	// Alignment
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_ALIGNMENT];
	
	if (tNumber==nil)
	{
		[IBalignmentMatrix_ selectCellWithTag:ICALIGNMENT_LEFT];
	}
	else
	{
		[IBalignmentMatrix_ selectCellWithTag:[tNumber intValue]];
	}

	// Scaling
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_SCALING];

	if (tNumber==nil)
	{
		[IBscalingMatrix_ selectCellWithTag:ICSCALING_PROPORTIONALLY];
	}
	else
	{
		[IBscalingMatrix_ selectCellWithTag:[tNumber intValue]];
	}
	
	// State of controls
	
	[IBcustomBackgroundPopupButton_ setEnabled:tUseCustomBackground];
		
	[IBalignmentMatrix_ setEnabled:tUseCustomBackground];
	
	[IBscalingMatrix_ setEnabled:tUseCustomBackground];
}

- (IBAction) switchBackgroundValue:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_CUSTOM];
	
	if (tNumber==nil || [tNumber integerValue]!=tTag)
	{
		[dictionary_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_CUSTOM];
		
		[self upgradeBackgroundSettings];
		
		[self noteDocumentChange];
		
		// Notify the modification to the simulator
		
		[ICBackgroundUtilities notifyBackgroundDidChangeWithSender:self];
	}
}

- (IBAction) chooseCustomBackground:(id) sender
{
	NSOpenPanel * tOpenPanel;
    NSString * tPath;
	NSInteger tPathType;
	NSString * tAbsolutePath=nil;
    NSMutableDictionary * tCustomPath;
	
	tCustomPath=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
		
	if (tCustomPath!=nil)
	{
		tPath=[tCustomPath objectForKey:ICDOCUMENT_PATH];
		
		if (tPath!=nil && [tPath length]>0)
		{
			tPathType=[[tCustomPath objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
			
			switch(tPathType)
			{
				case ICDOCUMENT_PATH_TYPE_GLOBAL:
				
					tAbsolutePath=tPath;
					
					break;
					
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
				
					tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
					
					break;
					
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
				
					tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
					
					break;
			}
		}
	}
	
	tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    [tOpenPanel setTreatsFilePackagesAsDirectories:YES];
	
	[tOpenPanel setDelegate:self];
	
    [tOpenPanel beginSheetForDirectory:[tAbsolutePath stringByExpandingTildeInPath]
                                  file:nil
                                 types:nil
                        modalForWindow:[IBview_ window]
                         modalDelegate:self
                        didEndSelector:@selector(chooseCustomBackgroundDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (BOOL) acceptsFileAtPath:(NSString *) inPath
{
	static NSFileManager * sFileManager=nil;
	
	if (sFileManager==nil)
	{
		sFileManager=[NSFileManager defaultManager];
	}
	
	if (sFileManager!=nil)
	{
		BOOL isDirectory;
		
		if ([sFileManager fileExistsAtPath:inPath isDirectory:&isDirectory]==YES)
		{
			if (isDirectory==YES)
			{
				return YES;		// A VOIR (Pkoi accepter un directory ?)
			}
			else
			{
				BOOL tImageFormatSupported;
				
				tImageFormatSupported=[sFileManager IC_fileAtPath:inPath matchesTypes:[ICBackgroundUtilities backgroundImageTypes]];
				
				if (tImageFormatSupported==NO)
				{
					NSURL * tURL;
					CGImageSourceRef tSourceRef;
					
					tURL = [NSURL fileURLWithPath:inPath];
					
					tSourceRef = CGImageSourceCreateWithURL((CFURLRef) tURL, NULL);
					
					if (tSourceRef!=NULL)
					{
						NSString * tImageUTI;
						
						tImageUTI=(NSString *) CGImageSourceGetType(tSourceRef);
						
						if (tImageUTI!=nil)
						{
							tImageFormatSupported=[[ICBackgroundUtilities backgroundImageUTIs] containsObject:tImageUTI];
						}
						
						// Release Memory
						
						CFRelease(tSourceRef);
					}
				}
				
				if (tImageFormatSupported==YES)
				{
					return YES;
				}
			}
		}
	}
	
	return NO;
}

- (BOOL)panel:(id) inPanel shouldShowFilename:(NSString *) inFileName
{
	return [self acceptsFileAtPath:inFileName];
}

- (void) chooseCustomBackgroundDidEnd:(NSOpenPanel *) inPanel returnCode:(NSInteger) inReturnCode contextInfo:(void  *) contextInfo
{
    if (inReturnCode==NSOKButton)
    {
        NSMutableDictionary * tCustomPath;
		NSInteger tPathType;
		NSString * tPath;
		NSSize tSize;
		
		tPath=[inPanel filename];
		
		tPathType=[IBcustomBackgroundPopupButton_ pathType];
		
		tCustomPath=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
		
		if (tCustomPath==nil)
		{
			tCustomPath=[NSMutableDictionary dictionaryWithObjectsAndKeys:@"",ICDOCUMENT_PATH,
																		  [NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																		  nil];
		
			[dictionary_ setObject:tCustomPath forKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
		}
		else
		{
			[tCustomPath setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
		}
		
		// Update Menu
		
		[IBcustomBackgroundPopupButton_ setFileNameWithPath:tPath];
		
		[IBcustomBackgroundPopupButton_ selectItemAtIndex:0];
		
		switch(tPathType)
        {
            case ICDOCUMENT_PATH_TYPE_GLOBAL:
			
                break;
				
            case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
			
                tPath=[tPath stringByRelativizingToPath:[document_ folder]];
                
                break;
				
			case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
				
				tPath=[tPath stringByRelativizingToPath:[document_ referenceFolderPath]];
				 
				break;
				
			default:
			
				tPath=nil;
				
				break;
        }
		
		if (tPath!=nil)
		{
			[tCustomPath setObject:tPath forKey:ICDOCUMENT_PATH];
		}
		
		[IBcustomBackgroundPopupButton_ setToolTip:tPath];
		
		[IBcustomBackgroundPopupButton_ setPathDoesNotExist:NO];
		
		// Update size
		
		tSize=[ICBackgroundUtilities sizeOfImageAtPath:tPath];
		
		[IBsizeLabel_ setStringValue:[NSString stringWithFormat:@"%ldx%ld",lround(tSize.width),lround(tSize.height)]];
		
		
		[self noteDocumentChange];
		
		// Notify the modification to the simulator
		
		[ICBackgroundUtilities notifyBackgroundDidChangeWithSender:self];
    }
	else
	{
		[IBcustomBackgroundPopupButton_ selectItemAtIndex:0];
	}
}

- (IBAction) referenceStyleDidChange:(id) sender
{
	NSInteger tPathType;
	NSInteger tOldPathType;
	NSMutableDictionary * tCustomPath;
	
	tPathType=[IBcustomBackgroundPopupButton_ pathType];
	
	tCustomPath=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
		
	if (tCustomPath==nil)
	{
		tCustomPath=[NSMutableDictionary dictionaryWithObjectsAndKeys:@"",ICDOCUMENT_PATH,
																	  [NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																	  nil];
	
		[dictionary_ setObject:tCustomPath forKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
	}
		
	tOldPathType=[[tCustomPath objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
	
	if (tPathType!=tOldPathType)
	{
		NSString * tPath;
		
		tPath=[tCustomPath objectForKey:ICDOCUMENT_PATH];
		
		if (tPath!=nil)
		{
			if (tOldPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
			}
			else if (tOldPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
			}
			
			[tCustomPath setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tPath=[tPath stringByRelativizingToPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tPath=[tPath stringByRelativizingToPath:[document_ referenceFolderPath]];
			}
			
			[tCustomPath setObject:tPath forKey:ICDOCUMENT_PATH];
		}
		
		[self noteDocumentChange];
		
		[IBcustomBackgroundPopupButton_ setToolTip:tPath];
	}
}

- (IBAction) switchAlignment:(id) sender
{
	NSNumber * tNumber;
	NSInteger tTag=[[sender selectedCell] tag];
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_ALIGNMENT];
	
	if (tNumber==nil || [tNumber integerValue]!=tTag)
	{
		[dictionary_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_ALIGNMENT];
		
		[self noteDocumentChange];
		
		// Notify the modification to the simulator
		
		[ICBackgroundUtilities notifyBackgroundDidChangeWithSender:self];
	}
}

- (IBAction) switchScaling:(id) sender
{
	NSNumber * tNumber;
	NSInteger tTag;
	
	tTag=[[sender selectedCell] tag];
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_SCALING];
	
	if (tNumber==nil || [tNumber integerValue]!=tTag)
	{
		[dictionary_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_SCALING];
		
		[self noteDocumentChange];
		
		// Notify the modification to the simulator
		
		[ICBackgroundUtilities notifyBackgroundDidChangeWithSender:self];
	}
}

#pragma mark -

- (BOOL) acceptDrop
{
	return YES;
}

- (BOOL) acceptsDroppedFile:(NSString *) inPath
{
	return [self acceptsFileAtPath:inPath];
}

- (void) referencedFileDidChange:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSMutableDictionary * tCustomPath;
		NSInteger tPathType;
		NSSize tSize;
		
		tPathType=[IBcustomBackgroundPopupButton_ pathType];
		
		tCustomPath=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
		
		if (tCustomPath==nil)
		{
			tCustomPath=[NSMutableDictionary dictionaryWithObjectsAndKeys:@"",ICDOCUMENT_PATH,
																		  [NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																		  nil];
		
			[dictionary_ setObject:tCustomPath forKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
		}
		
		// Update Menu
		
		[IBcustomBackgroundPopupButton_ setFileNameWithPath:inPath];
		
		[IBcustomBackgroundPopupButton_ selectItemAtIndex:0];
		
		switch(tPathType)
        {
            case ICDOCUMENT_PATH_TYPE_GLOBAL:
			
                break;
				
            case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
			
                inPath=[inPath stringByRelativizingToPath:[document_ folder]];
                
                break;
				
			case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
				
				inPath=[inPath stringByRelativizingToPath:[document_ referenceFolderPath]];
                
                break;
        
			default:
				
				inPath=nil;
                
                break;
		}
		
		if (inPath!=nil)
		{
			[tCustomPath setObject:inPath forKey:ICDOCUMENT_PATH];
			
			[tCustomPath setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
		}
		
		[IBcustomBackgroundPopupButton_ setToolTip:inPath];
		
		[IBcustomBackgroundPopupButton_ setPathDoesNotExist:NO];
		
		// Update size
		
		tSize=[ICBackgroundUtilities sizeOfImageAtPath:inPath];
		
		[IBsizeLabel_ setStringValue:[NSString stringWithFormat:@"%ldx%ld",lround(tSize.width),lround(tSize.height)]];
		
		[self noteDocumentChange];
		
		// Notify the modification to the simulator
		
		[ICBackgroundUtilities notifyBackgroundDidChangeWithSender:self];
	}
}

#pragma mark -

- (void) backgroundSettingsDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if ([tUserInfo objectForKey:ICPRESENTATION_NOTIFICATION_SENDER]!=self)
		{
			[self upgradeBackgroundSettings];
		}
	}
}

- (void) windowStateDidChange:(NSNotification *) inNotification
{
	[self upgradeBackgroundSettings];
}

@end
