/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationInspectorInstallationTypeChoiceDependencyController.h"

#import "ICInstallationType+Constants.h"

#import "ICChoiceUtilities.h"

#import "ICDependencyContainerView.h"

#import "ICChoicePopUpButton.h"

#import "NSPopUpButton+OptimizedSize.h"

#define ICDEPENDENCYCONTAINERVIEW_BRANCH_INTERSPACE_VERTICAL	5.0f

//#define ICDEPENDENCYCONTAINERVIEW_LEAF_WIDTH	400

#define ICDEPENDENCYMARGIN_LEFT		13.0f

#define ICDEPENDENCYMARGIN_RIGHT	13.0f

#define ICDEPENDENCYMARGIN_TOP		12.0f

#define ICDEPENDENCYMARGIN_BOTTOM	13.0f

#define BIG_FLOAT	65535.0f

@implementation ICPresentationInspectorInstallationTypeChoiceDependencyController

- (void) awakeFromNib
{
	// A COMPLETER
}

#pragma mark -

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument;
{
	id tObject;
	
	[availableEnabledDependenciesDictionary_ release];
	
	availableEnabledDependenciesDictionary_=nil;
	
	[availableSelectedDependenciesDictionary_ release];
	
	availableSelectedDependenciesDictionary_=nil;
	
	[enabledChoicesDictionary_ release];
	
	enabledChoicesDictionary_=nil;
	
	[selectedChoicesDictionary_ release];
	
	selectedChoicesDictionary_=nil;
	
	[UUIDsDictionary_ release];
	
	UUIDsDictionary_=nil;
	
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	tObject=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTION];
	
	if (tObject!=nil)
	{
		if ([tObject isKindOfClass:[NSDictionary class]]==NO)
		{
			ICInstallationTypeNodeData * tNodeData;
			NSMutableDictionary * tOptionsDictionary;
			
			node_=(ICInstallationTypeTreeNode *) tObject;
			
			tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(node_);
			
			tOptionsDictionary=[[tNodeData dictionary] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
			
			if (tOptionsDictionary!=nil)
			{
				dictionary_=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
				
				if (dictionary_!=nil)
				{
					NSWindow * tWindow;
					
					[dictionary_ retain];
					
					isGroup_=[node_ isGroup];
					
					if (isGroup_==YES)
					{
						availableEnabledDependenciesDictionary_=[ICChoiceUtilities availableDependenciesDictionaryForEnabledStateOfGroupNode:node_];
					}
					else
					{
						availableEnabledDependenciesDictionary_=[ICChoiceUtilities availableDependenciesDictionaryForEnabledStateOfLeafNode:node_ skipEnabledIfConstant:NO];
						
						availableSelectedDependenciesDictionary_=[ICChoiceUtilities availableDependenciesDictionaryForSelectedStateOfLeafNode:node_];
					}
					
					if (availableEnabledDependenciesDictionary_!=nil)
					{
						[availableEnabledDependenciesDictionary_ retain];
						
						enabledChoicesDictionary_=[ICChoiceUtilities availableChoicesDictionaryFromAvailableDependenciesDictionary:availableEnabledDependenciesDictionary_];
						
						if (enabledChoicesDictionary_!=nil)
						{
							[enabledChoicesDictionary_ retain];
						}
						
						UUIDsDictionary_=[ICChoiceUtilities  UUIDDictionaryFromAvailableDependenciesDictionary:availableEnabledDependenciesDictionary_];
					
						if (UUIDsDictionary_!=nil)
						{
							[UUIDsDictionary_ retain];
						}
					}
					
					if (availableSelectedDependenciesDictionary_!=nil)
					{
						[availableSelectedDependenciesDictionary_ retain];
						
						selectedChoicesDictionary_=[ICChoiceUtilities availableChoicesDictionaryFromAvailableDependenciesDictionary:availableSelectedDependenciesDictionary_];
						
						if (selectedChoicesDictionary_!=nil)
						{
							[selectedChoicesDictionary_ retain];
						}
						
						if (UUIDsDictionary_==nil)
						{
							UUIDsDictionary_=[ICChoiceUtilities  UUIDDictionaryFromAvailableDependenciesDictionary:availableSelectedDependenciesDictionary_];
					
							if (UUIDsDictionary_!=nil)
							{
								[UUIDsDictionary_ retain];
							}
						}
						else
						{
							[UUIDsDictionary_ addEntriesFromDictionary:[ICChoiceUtilities  UUIDDictionaryFromAvailableDependenciesDictionary:availableSelectedDependenciesDictionary_]];
						}
					}
					
					if (isGroup_==YES)
					{
						// Set the Text Label
						
						[IBchoiceEnabledDependencyTextLabel_ setStringValue:NSLocalizedStringFromTable(@"Choice is enabled when:",@"",@"")];
						
						[IBchoiceEnabledDependencyTextLabel_ sizeToFit];
					
						[IBchoiceEnabledDependencyPopupButton_ setHidden:YES];
						
						[IBchoiceEnabledDependencyColonLabel_ setHidden:YES];
						
						[IBchoiceEnabledDependencyScrollView_ setHidden:NO];
						
						[IBchoiceSelectedDependencyTextLabel_ setHidden:YES];
						
						[IBchoiceSelectedDependencyScrollView_ setHidden:YES];
					}
					else
					{
						// Set the Text Label
						
						[IBchoiceEnabledDependencyTextLabel_ setStringValue:NSLocalizedStringFromTable(@"Choice is ",@"",@"")];
						
						[IBchoiceEnabledDependencyTextLabel_ sizeToFit];
						
						[IBchoiceEnabledDependencyPopupButton_ setHidden:NO];
						
						[IBchoiceEnabledDependencyColonLabel_ setHidden:NO];
						
						[IBchoiceSelectedDependencyTextLabel_ setHidden:NO];
						
						[IBchoiceSelectedDependencyScrollView_ setHidden:NO];
					}
					
					// Pre-compute popup buttons size
					
					[self computePopupButtonSizes];
					
					// Add the return to button
					
					tWindow=[[self document] windowForSheet];
					
					if (tWindow!=nil)
					{
						NSView * tContentView;
						
						tContentView=[tWindow contentView];
						
						if (tContentView!=nil)
						{
							NSRect tFrame;
							NSRect tContentBounds;
							
							tContentBounds=[tContentView bounds];
							
							tFrame=[IBcloseView_ frame];
							
							
							tFrame.origin.y=3.0f;
							
							tFrame.origin.x=NSWidth(tContentBounds)-NSWidth(tFrame)-20.0f;
							
							[IBcloseView_ setFrame:tFrame];
							
							[tContentView addSubview:IBcloseView_];
						}
					}
					
					[[NSNotificationCenter defaultCenter] removeObserver:self name:NSViewFrameDidChangeNotification object:IBview_];
					
					[[NSNotificationCenter defaultCenter] addObserver:self
															 selector:@selector(viewFrameDidChange:) 
																 name:NSViewFrameDidChangeNotification 
															  object:IBview_];
					
					// A COMPLETER
					
					[self refreshView];
				}
			}
		}
	}
}

#pragma mark -

- (IBAction) switchEnabledStateMode:(id) sender
{
	NSInteger tTag;
	int tEnabledStateMode=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_ALWAYS;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE];
		
	if (tNumber!=nil)
	{
		tEnabledStateMode=[tNumber intValue];
	}
	
	if (tTag!=tEnabledStateMode)
	{
		[dictionary_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE];
		
		[self refreshView];
		
		[self noteDocumentChange];
	}
}

#pragma mark -

- (NSMutableDictionary *) defaultEnabledLeafDictionary
{
	NSMutableDictionary * tMutableDictionary=nil;
	NSMenu * tMenu;
	
	tMenu=[self enabledAvailableChoicesMenu];
	
	if ([tMenu numberOfItems]>0)
	{
		NSString * tDefaultChoiceLabel;
		NSDictionary * tChoiceDictionary;
	
		tDefaultChoiceLabel=[[tMenu itemAtIndex:0] title];
	
		tChoiceDictionary=[enabledChoicesDictionary_ objectForKey:tDefaultChoiceLabel];
		
		if (tChoiceDictionary!=nil)
		{
			NSString * tUUID;
			int tDefaultState=ICSTATE_ENABLED;
			NSNumber * tNumber;
			
			tUUID=[tChoiceDictionary objectForKey:ICCHOICE_UUID];
			
			tNumber=[tChoiceDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
			
			if (tNumber==nil || [tNumber boolValue]==NO)
			{
				tDefaultState=ICSTATE_SELECTED;
			}
			
			tMutableDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tUUID,ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID,
																				 [NSNumber numberWithInt:ICCOMPARATOR_IS],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_COMPARATOR,
																				 [NSNumber numberWithInt:tDefaultState],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT,
																				 nil];
		}
	}
	
	return tMutableDictionary;

}

- (NSMutableDictionary *) defaultSelectedLeafDictionary
{
	NSMutableDictionary * tMutableDictionary=nil;
	NSMenu * tMenu;
	
	tMenu=[self selectedAvailableChoicesMenu];
	
	if ([tMenu numberOfItems]>0)
	{
		NSString * tDefaultChoiceLabel;
		NSDictionary * tChoiceDictionary;
	
		tDefaultChoiceLabel=[[tMenu itemAtIndex:0] title];
	
		tChoiceDictionary=[selectedChoicesDictionary_ objectForKey:tDefaultChoiceLabel];
		
		if (tChoiceDictionary!=nil)
		{
			NSString * tUUID;
			int tDefaultState=ICSTATE_ENABLED;
			NSNumber * tNumber;
			
			tUUID=[tChoiceDictionary objectForKey:ICCHOICE_UUID];
			
			tNumber=[tChoiceDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
			
			if (tNumber==nil || [tNumber boolValue]==NO)
			{
				tDefaultState=ICSTATE_SELECTED;
			}
			
			tMutableDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tUUID,ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID,
																				 [NSNumber numberWithInt:ICCOMPARATOR_IS],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_COMPARATOR,
																				 [NSNumber numberWithInt:tDefaultState],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT,
																				 nil];
		}
	}
	
	return tMutableDictionary;

}

- (void) computePopupButtonSizes
{
	NSPopUpButton * tPopupButton;
	
	tPopupButton=[[NSPopUpButton alloc] initWithFrame:NSMakeRect(0.0f,0.0f,100.0f,17.0f) pullsDown:NO];
				
	if (tPopupButton!=nil)
	{
		NSMenu * tMenu;
		
		[[tPopupButton cell] setControlSize:NSSmallControlSize];
		
		[tPopupButton setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
		
		[tPopupButton setAlignment:NSCenterTextAlignment];
		
		[tPopupButton setBezelStyle:NSRoundRectBezelStyle];
		
		// Available choices
		
		tMenu=[self selectedAvailableChoicesMenu];
		
		if ([tMenu numberOfItems]==0)
		{
			tMenu=[self enabledAvailableChoicesMenu];
		}
		
		[tPopupButton setMenu:tMenu];
		
		availableChoicesPopUpButtonSize_=[tPopupButton optimizedSize];
		
		availableChoicesPopUpButtonSize_.width=100.0f;
		
		// Operators
		
		[tPopupButton setMenu:[self operatorsMenu]];
		
		operatorPopUpButtonSize_=[tPopupButton optimizedSize];
		
		operatorPopUpButtonSize_.width-=25.0f;
		
		// Comparators
		
		[tPopupButton setMenu:[self comparatorsMenu]];
		
		comparatorPopUpButtonSize_=[tPopupButton optimizedSize];
		
		comparatorPopUpButtonSize_.width-=25.0f;
		
		// State Objects
		
		[tPopupButton setMenu:[self stateObjectsMenu]];
		
		stateObjectPopUpButtonSize_=[tPopupButton optimizedSize];
		
		stateObjectPopUpButtonSize_.width-=25.0f;
		
		[tPopupButton release];
	}
	
	leafWidth_=availableChoicesPopUpButtonSize_.width+10.0f+comparatorPopUpButtonSize_.width+10.0f+stateObjectPopUpButtonSize_.width+10.0f+16.0f+5.0f+16.0f;
	
	branchWidth_=operatorPopUpButtonSize_.width+10.0f+16.0f+5.0f+16.0f+5.0f+16.0f;
}

- (NSMenu *) enabledAvailableChoicesMenu
{
	NSMenu * tMenu;
	
	tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] init];
	
	if (tMenu!=nil)
	{
		NSMutableArray * tChoicesList=nil;
		NSEnumerator * tEnumerator;
		
		tChoicesList=[ICChoiceUtilities availableListOfChoicesFromAvailableDependenciesDictionary:availableEnabledDependenciesDictionary_];
		
		tEnumerator=[tChoicesList objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSString * tChoiceName;
			
			while (tChoiceName=[tEnumerator nextObject])
			{
				NSMenuItem * tMenuItem;
		
				tMenuItem=[[NSMenuItem alloc] initWithTitle:tChoiceName action:nil keyEquivalent:@""];
				
				if (tMenuItem!=nil)
				{
					[tMenu addItem:tMenuItem];
					
					[tMenuItem release];
				}
				else
				{
					// A COMPLETER
				}
			}
		}
	}
	
	return [tMenu autorelease];
}

- (NSMenu *) selectedAvailableChoicesMenu
{
	NSMenu * tMenu;
	
	tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] init];
	
	if (tMenu!=nil)
	{
		NSMutableArray * tChoicesList=nil;
		NSEnumerator * tEnumerator;
		
		tChoicesList=[ICChoiceUtilities availableListOfChoicesFromAvailableDependenciesDictionary:availableSelectedDependenciesDictionary_];
		
		tEnumerator=[tChoicesList objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSString * tChoiceName;
			
			while (tChoiceName=[tEnumerator nextObject])
			{
				NSMenuItem * tMenuItem;
		
				tMenuItem=[[NSMenuItem alloc] initWithTitle:tChoiceName action:nil keyEquivalent:@""];
				
				if (tMenuItem!=nil)
				{
					[tMenu addItem:tMenuItem];
					
					[tMenuItem release];
				}
				else
				{
					// A COMPLETER
				}
			}
		}
	}
	
	return [tMenu autorelease];
}

- (NSMenu *) operatorsMenu
{
	NSMenu * tMenu;
	
	tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] init];
	
	if (tMenu!=nil)
	{
		NSMenuItem * tMenuItem;
		
		tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"And",@"",@"") action:nil keyEquivalent:@""];
		
		if (tMenuItem!=nil)
		{
			[tMenuItem setTag:ICOPERATOR_AND];
			
			[tMenu addItem:tMenuItem];
			
			[tMenuItem release];
		}
		else
		{
			// A COMPLETER
		}
		
		tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Or",@"",@"") action:nil keyEquivalent:@""];
		
		if (tMenuItem!=nil)
		{
			[tMenuItem setTag:ICOPERATOR_OR];
			
			[tMenu addItem:tMenuItem];
			
			[tMenuItem release];
		}
		else
		{
			// A COMPLETER
		}
	}
	
	return [tMenu autorelease];
}

- (NSMenu *) comparatorsMenu
{
	NSMenu * tMenu;
	
	tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] init];
	
	if (tMenu!=nil)
	{
		NSMenuItem * tMenuItem;
		
		tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Is",@"",@"") action:nil keyEquivalent:@""];
		
		if (tMenuItem!=nil)
		{
			[tMenuItem setTag:ICCOMPARATOR_IS];
			
			[tMenu addItem:tMenuItem];
			
			[tMenuItem release];
		}
		else
		{
			// A COMPLETER
		}
		
		tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Is Not",@"",@"") action:nil keyEquivalent:@""];
		
		if (tMenuItem!=nil)
		{
			[tMenuItem setTag:ICCOMPARATOR_IS_NOT];
			
			[tMenu addItem:tMenuItem];
			
			[tMenuItem release];
		}
		else
		{
			// A COMPLETER
		}
	}
	
	return [tMenu autorelease];
}

- (NSMenu *) stateObjectsMenu
{
	NSMenu * tMenu;
	
	tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] init];
	
	if (tMenu!=nil)
	{
		NSMenuItem * tMenuItem;
		
		tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Enabled",@"",@"") action:nil keyEquivalent:@""];
		
		if (tMenuItem!=nil)
		{
			[tMenuItem setTag:ICSTATE_ENABLED];
			
			[tMenu addItem:tMenuItem];
			
			[tMenuItem release];
		}
		else
		{
			// A COMPLETER
		}
		
		tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Selected",@"",@"") action:nil keyEquivalent:@""];
		
		if (tMenuItem!=nil)
		{
			[tMenuItem setTag:ICSTATE_SELECTED];
			
			[tMenu addItem:tMenuItem];
			
			[tMenuItem release];
		}
		else
		{
			// A COMPLETER
		}
	}
	
	return [tMenu autorelease];
}

- (void) computeViewSizeWithDictionary:(NSDictionary *) inDictionary maxY:(CGFloat *) inOutMaxY minX:(CGFloat *) outMinX midY:(CGFloat *) outMidY
{
	NSDictionary * tDictionary;
	
	// Is it a branch or leaf?
	
	tDictionary=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP];
	
	if (tDictionary==nil)
	{
		// Leaf
		
		*outMidY=_CGFloatFloor((*inOutMaxY)-comparatorPopUpButtonSize_.height*0.5f);
		
		*outMinX=BIG_FLOAT-leafWidth_-ICDEPENDENCYMARGIN_RIGHT-ICDEPENDENCYMARGIN_LEFT;
		
		*inOutMaxY=(*inOutMaxY)-comparatorPopUpButtonSize_.height;
	}
	else
	{
		// Branch
		
		CGFloat tMinXTop=0;
		CGFloat tMidYTop=0;
		CGFloat tMinXBottom=0;
		CGFloat tMidYBottom=0;
		
		[self computeViewSizeWithDictionary:tDictionary maxY:inOutMaxY minX:&tMinXTop midY:&tMidYTop];
		
		*outMidY=_CGFloatFloor(*inOutMaxY-(ICDEPENDENCYCONTAINERVIEW_BRANCH_INTERSPACE_VERTICAL+(operatorPopUpButtonSize_.height*0.5f)));
		
		*inOutMaxY-=(2*ICDEPENDENCYCONTAINERVIEW_BRANCH_INTERSPACE_VERTICAL+operatorPopUpButtonSize_.height);
		
		tDictionary=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM];
		
		[self computeViewSizeWithDictionary:tDictionary maxY:inOutMaxY minX:&tMinXBottom midY:&tMidYBottom];
		
		if (tMinXTop>tMinXBottom)
		{
			tMinXTop=tMinXBottom;
		}
		
		*outMinX=tMinXTop-(operatorPopUpButtonSize_.width+10.0f);
	}
}

- (NSSize) optimizedSizeForDictionary:(NSDictionary *) inDictionary
{
	NSSize tSize;
	CGFloat tMaxY;
	CGFloat tMidY;
	CGFloat tMinX=0;
	
	tMaxY=BIG_FLOAT;
	
	[self computeViewSizeWithDictionary:inDictionary maxY:&tMaxY minX:&tMinX midY:&tMidY];
	
	tSize.width=BIG_FLOAT-tMinX;
	
	tSize.height=BIG_FLOAT-tMaxY;
	
	return tSize;
}

- (NSButton *) layoutView:(id) inView forDictionary:(NSDictionary *) inDictionary maxY:(CGFloat *) inOutMaxY minX:(CGFloat *) outMinX midY:(CGFloat *) outMidY
{
	NSMutableDictionary * tDictionary;
	ICDependencyContainerView * tContainerView=nil;
	NSButton * tMinusButton=nil;
	NSRect tBounds;
	
	// Is it a branch or leaf?
	
	tDictionary=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP];
	
	if (tDictionary==nil)
	{
		// Leaf
		
		tBounds=[inView bounds];
	
		tContainerView=[[ICDependencyContainerView alloc] initWithFrame:NSMakeRect(NSWidth(tBounds)-leafWidth_-ICDEPENDENCYMARGIN_RIGHT,(*inOutMaxY)-comparatorPopUpButtonSize_.height,leafWidth_,comparatorPopUpButtonSize_.height)];
		
		if (tContainerView!=nil)
		{
			NSPopUpButton * tPopupButton;
			NSRect tContainerBounds;
			NSButton * tButton;
			
			[tContainerView setDictionary:inDictionary];
			
			[tContainerView setController:self];
			
			[inView addSubview:tContainerView];
		
			tContainerBounds=[tContainerView bounds];
		
			// Layout the leaf subviews
			
				// Choices
				
			tPopupButton=[[ICChoicePopUpButton alloc] initWithFrame:NSMakeRect(NSMinX(tContainerBounds),NSMinY(tContainerBounds),availableChoicesPopUpButtonSize_.width,availableChoicesPopUpButtonSize_.height) pullsDown:NO];
				
			if (tPopupButton!=nil)
			{
				NSString * tChoiceLabel;
				
				[tPopupButton setBezelStyle:NSRoundRectBezelStyle];
				
				if (inView==IBchoiceEnabledDependencyView_)
				{
					[tPopupButton setMenu:[self enabledAvailableChoicesMenu]];
				}
				else if (inView==IBchoiceSelectedDependencyView_)
				{
					[tPopupButton setMenu:[self selectedAvailableChoicesMenu]];
				}
				
				[[tPopupButton cell] setControlSize:NSSmallControlSize];
				
				[tPopupButton setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
				
				[tPopupButton setAlignment:NSLeftTextAlignment];
				
				if (inView==IBchoiceEnabledDependencyView_)
				{
					[tPopupButton setAction:@selector(switchEnabledChoice:)];
				}
				else if (inView==IBchoiceSelectedDependencyView_)
				{
					[tPopupButton setAction:@selector(switchSelectedChoice:)];
				}
				
				
				[tPopupButton setTarget:self];
				
				[((ICChoicePopUpButton *) tPopupButton) setDelegate:self];
				
				[tContainerView addSubview:tPopupButton];
				
				tChoiceLabel=[UUIDsDictionary_ objectForKey:[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID]];
				
				if (tChoiceLabel!=nil)
				{
					[tPopupButton selectItemWithTitle:tChoiceLabel];
				}
				else
				{
					// A COMPLETER
				}
				
				[tPopupButton release];
			}
			else
			{
				NSLog(@"[ICPresentationInspectorInstallationTypeChoiceDependencyController layoutView:forDictionary:maxY:minX:midY:] Low memory");
			}
			
			tBounds=[tPopupButton frame];
						
				// Comparator
				
			tPopupButton=[[NSPopUpButton alloc] initWithFrame:NSMakeRect(NSMaxX(tBounds)+10.0f,NSMinY(tBounds),comparatorPopUpButtonSize_.width,comparatorPopUpButtonSize_.height) pullsDown:NO];
				
			if (tPopupButton!=nil)
			{
				[tPopupButton setBezelStyle:NSRoundRectBezelStyle];
				
				[tPopupButton setMenu:[self comparatorsMenu]];
				
				[[tPopupButton cell] setControlSize:NSSmallControlSize];
				
				[tPopupButton setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
				
				[tPopupButton setAlignment:NSLeftTextAlignment];
				
				[tPopupButton setAction:@selector(switchComparator:)];
				
				[tPopupButton setTarget:self];
				
				[tContainerView addSubview:tPopupButton];
				
				[tPopupButton selectItemWithTag:[[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_COMPARATOR] intValue]];
				
				[tPopupButton release];
			}
			else
			{
				NSLog(@"[ICPresentationInspectorInstallationTypeChoiceDependencyController layoutView:forDictionary:maxY:minX:midY:] Low memory");
			}

			tBounds=[tPopupButton frame];
			
				// State 
				
			tPopupButton=[[NSPopUpButton alloc] initWithFrame:NSMakeRect(NSMaxX(tBounds)+10.0f,NSMinY(tBounds),stateObjectPopUpButtonSize_.width,stateObjectPopUpButtonSize_.height) pullsDown:NO];
				
			if (tPopupButton!=nil)
			{
				[[tPopupButton cell] setControlSize:NSSmallControlSize];
				
				[tPopupButton setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
				
				[tPopupButton setAlignment:NSLeftTextAlignment];
				
				[tPopupButton setBezelStyle:NSRoundRectBezelStyle];
				
				[tPopupButton setMenu:[self stateObjectsMenu]];
				
				if (inView==IBchoiceEnabledDependencyView_)
				{
					[tPopupButton setAction:@selector(switchEnabledStateObject:)];
				}
				else if (inView==IBchoiceSelectedDependencyView_)
				{
					[tPopupButton setAction:@selector(switchSelectedStateObject:)];
				}
				
				[tPopupButton setTarget:tContainerView];
				
				[tContainerView addSubview:tPopupButton];
				
				[tPopupButton selectItemWithTag:[[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT] intValue]];
				
				[tPopupButton release];
			}
			else
			{
				NSLog(@"[ICPresentationInspectorInstallationTypeChoiceDependencyController layoutView:forDictionary:maxY:minX:midY:] Low memory");
			}
			
			tBounds=[tPopupButton frame];
				
				// Minus
				
			tMinusButton=tButton=[[NSButton alloc] initWithFrame:NSMakeRect(NSMaxX(tBounds)+10.0f,NSMinY(tBounds)-1,16.0f,19.0f)];
			
			if (tButton!=nil)
			{
				[tButton setImagePosition:NSImageOnly];
				
				[tButton setImage:[NSImage imageNamed:@"Minus-Dependency"]];
				
				[[tButton cell] setControlSize:NSSmallControlSize];
				
				[tButton setBezelStyle:NSRoundRectBezelStyle];
				
				[tButton setEnabled:NO];
				
				// A COMPLETER
				
				[tButton setAction:@selector(removeBranch:)];
				
				[tButton setTarget:self];
				
				[tContainerView addSubview:tButton];
				
				[tButton release];
			}
			else
			{
				NSLog(@"[ICPresentationInspectorInstallationTypeChoiceDependencyController layoutView:forDictionary:maxY:minX:midY:] Low memory");
			}
			
			tBounds=[tButton frame];

				
				// Plus
			
			tButton=[[NSButton alloc] initWithFrame:NSMakeRect(NSMaxX(tBounds)+5.0f,NSMinY(tBounds),16.0f,19.0f)];
			
			if (tButton!=nil)
			{
				[tButton setImagePosition:NSImageOnly];
				
				[tButton setImage:[NSImage imageNamed:@"Plus-Dependency"]];
				
				[[tButton cell] setControlSize:NSSmallControlSize];
				
				[tButton setBezelStyle:NSRoundRectBezelStyle];
				
				[tButton setAction:@selector(addBranch:)];
				
				[tButton setTarget:self];
				
				[tContainerView addSubview:tButton];
				
				[tButton release];
			}
			else
			{
				NSLog(@"[ICPresentationInspectorInstallationTypeChoiceDependencyController layoutView:forDictionary:maxY:minX:midY:] Low memory");
			}
			
			[tContainerView release];
		}
		
		// Compute for the other views;
		
		*outMidY=_CGFloatFloor((*inOutMaxY)-comparatorPopUpButtonSize_.height*0.5f);
		
		*outMinX=NSWidth([inView bounds])-leafWidth_-ICDEPENDENCYMARGIN_RIGHT;
		
		*inOutMaxY=(*inOutMaxY)-comparatorPopUpButtonSize_.height;
	}
	else
	{
		// Branch
		
		CGFloat tMinXTop=0;
		CGFloat tMidYTop=0;
		CGFloat tMinXBottom=0;
		CGFloat tMidYBottom=0;
		CGFloat tMinX=0;
		CGFloat tSavedYMax;
		NSBox * tBox;
		
		tMinusButton=[self layoutView:inView forDictionary:tDictionary maxY:inOutMaxY minX:&tMinXTop midY:&tMidYTop];
		
		if (tMinusButton!=nil)
		{
			[tMinusButton setEnabled:YES];
			
			[((ICDependencyContainerView *) [tMinusButton superview]) setParentDictionary:inDictionary];
		}
		
		tSavedYMax=*inOutMaxY;
		
		*outMidY=_CGFloatFloor(*inOutMaxY-(ICDEPENDENCYCONTAINERVIEW_BRANCH_INTERSPACE_VERTICAL+(operatorPopUpButtonSize_.height*0.5f)));
		
		*inOutMaxY-=(2*ICDEPENDENCYCONTAINERVIEW_BRANCH_INTERSPACE_VERTICAL+operatorPopUpButtonSize_.height);
		
		
		
		tDictionary=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM];
		
		tMinusButton=[self layoutView:inView forDictionary:tDictionary maxY:inOutMaxY minX:&tMinXBottom midY:&tMidYBottom];
		
		if (tMinusButton!=nil)
		{
			[tMinusButton setEnabled:YES];
			
			[((ICDependencyContainerView *) [tMinusButton superview]) setParentDictionary:inDictionary];
		}
		
		tMinX=tMinXTop;
		
		if (tMinX>tMinXBottom)
		{
			tMinX=tMinXBottom;
		}
		
		*outMinX=tMinX-(operatorPopUpButtonSize_.width+10.0f);
		
		// Branches
		
			// Top
			
			tBox=[[NSBox alloc] initWithFrame:NSMakeRect(_CGFloatFloor((*outMinX)+operatorPopUpButtonSize_.width*0.5f),tMidYTop,_CGFloatFloor(tMinXTop-2.0f-((*outMinX)+operatorPopUpButtonSize_.width*0.5f)),1.0f)];
			
			if (tBox!=nil)
			{
				[tBox setBorderType:NSLineBorder];
			
				[tBox setTitlePosition:NSNoTitle];
				
				[tBox setBoxType:NSBoxSeparator];
				
				[inView addSubview:tBox];
				
				[tBox release];
			}
			
			tBox=[[NSBox alloc] initWithFrame:NSMakeRect(_CGFloatFloor((*outMinX)+operatorPopUpButtonSize_.width*0.5f),tSavedYMax-ICDEPENDENCYCONTAINERVIEW_BRANCH_INTERSPACE_VERTICAL+2.0f,1.0f,(tMidYTop)-(tSavedYMax-ICDEPENDENCYCONTAINERVIEW_BRANCH_INTERSPACE_VERTICAL+2.0f))];
			
			if (tBox!=nil)
			{
				[tBox setBorderType:NSLineBorder];
			
				[tBox setTitlePosition:NSNoTitle];
				
				[tBox setBoxType:NSBoxSeparator];
				
				[inView addSubview:tBox];
				
				[tBox release];
			}
			
			// Bottom
			
			// Top
			
			tBox=[[NSBox alloc] initWithFrame:NSMakeRect(_CGFloatFloor((*outMinX)+operatorPopUpButtonSize_.width*0.5f),tMidYBottom,_CGFloatFloor(tMinXBottom-2.0f-((*outMinX)+operatorPopUpButtonSize_.width*0.5f)),1.0f)];
			
			if (tBox!=nil)
			{
				[tBox setBorderType:NSLineBorder];
			
				[tBox setTitlePosition:NSNoTitle];
				
				[tBox setBoxType:NSBoxSeparator];
				
				[inView addSubview:tBox];
				
				[tBox release];
			}
			
			tBox=[[NSBox alloc] initWithFrame:NSMakeRect(_CGFloatFloor((*outMinX)+operatorPopUpButtonSize_.width*0.5f),tMidYBottom+1.0f,1.0f,tSavedYMax-operatorPopUpButtonSize_.height-ICDEPENDENCYCONTAINERVIEW_BRANCH_INTERSPACE_VERTICAL-2.0f-(tMidYBottom+1.0f))];
			
			if (tBox!=nil)
			{
				[tBox setBorderType:NSLineBorder];
			
				[tBox setTitlePosition:NSNoTitle];
				
				[tBox setBoxType:NSBoxSeparator];
				
				[inView addSubview:tBox];
				
				[tBox release];
			}
		
		// Container Operator
		
		tContainerView=[[ICDependencyContainerView alloc] initWithFrame:NSMakeRect(*outMinX,tSavedYMax-operatorPopUpButtonSize_.height-ICDEPENDENCYCONTAINERVIEW_BRANCH_INTERSPACE_VERTICAL,branchWidth_,operatorPopUpButtonSize_.height)];
		
		if (tContainerView!=nil)
		{
			NSPopUpButton * tPopupButton;
			NSRect tContainerBounds;
			NSButton * tButton;
			
			[tContainerView setDictionary:inDictionary];
			
			[inView addSubview:tContainerView];
		
			tContainerBounds=[tContainerView bounds];
		
			// Layout the operator subviews
			
				// Operator
				
			tPopupButton=[[NSPopUpButton alloc] initWithFrame:NSMakeRect(NSMinX(tContainerBounds),NSMinY(tContainerBounds),operatorPopUpButtonSize_.width,operatorPopUpButtonSize_.height) pullsDown:NO];
				
			if (tPopupButton!=nil)
			{
				[tPopupButton setBezelStyle:NSRoundRectBezelStyle];
				
				[tPopupButton setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
				
				[[tPopupButton cell] setControlSize:NSSmallControlSize];
				
				[tPopupButton setAlignment:NSLeftTextAlignment];
				
				[tPopupButton setMenu:[self operatorsMenu]];
				
				[tPopupButton setAction:@selector(switchOperator:)];
				
				[tPopupButton setTarget:self];
				
				[tContainerView addSubview:tPopupButton];
				
				[tPopupButton selectItemWithTag:[[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_OPERATOR] intValue]];
				
				[tPopupButton release];
			}
			else
			{
				NSLog(@"[ICPresentationInspectorInstallationTypeChoiceDependencyController layoutView:forDictionary:maxY:minX:midY:] Low memory");
			}
			
			tBounds=[tPopupButton frame];
				
				// Switch
			
			tButton=[[NSButton alloc] initWithFrame:NSMakeRect(NSMaxX(tBounds)+10.0f,NSMinY(tBounds)-1,16.0f,19.0f)];
			
			if (tButton!=nil)
			{
				[tButton setImagePosition:NSImageOnly];
				
				[tButton setImage:[NSImage imageNamed:@"Switch-Dependency"]];
				
				[[tButton cell] setControlSize:NSSmallControlSize];
				
				[tButton setBezelStyle:NSRoundRectBezelStyle];
				
				[tButton setAction:@selector(switchBranches:)];
				
				[tButton setTarget:self];
				
				[tContainerView addSubview:tButton];
				
				[tButton release];
			}
			else
			{
				NSLog(@"[ICPresentationInspectorInstallationTypeChoiceDependencyController layoutView:forDictionary:maxY:minX:midY:] Low memory");
			}
			
			tBounds=[tButton frame];
				
				// Minus
				
			tMinusButton=tButton=[[NSButton alloc] initWithFrame:NSMakeRect(NSMaxX(tBounds)+5.0f,NSMinY(tBounds),16.0f,19.0f)];
			
			if (tButton!=nil)
			{
				[tButton setImagePosition:NSImageOnly];
				
				[tButton setImage:[NSImage imageNamed:@"Minus-Dependency"]];
				
				[[tButton cell] setControlSize:NSSmallControlSize];
				
				[tButton setBezelStyle:NSRoundRectBezelStyle];
				
				[tButton setEnabled:NO];
				
				// A COMPLETER
				
				[tButton setAction:@selector(removeBranch:)];
				
				[tButton setTarget:self];
				
				[tContainerView addSubview:tButton];
				
				[tButton release];
			}
			else
			{
				NSLog(@"[ICPresentationInspectorInstallationTypeChoiceDependencyController layoutView:forDictionary:maxY:minX:midY:] Low memory");
			}
			
			tBounds=[tButton frame];

				
				// Plus
			
			tButton=[[NSButton alloc] initWithFrame:NSMakeRect(NSMaxX(tBounds)+5.0f,NSMinY(tBounds),16.0f,19.0f)];
			
			if (tButton!=nil)
			{
				[tButton setImagePosition:NSImageOnly];
				
				[tButton setImage:[NSImage imageNamed:@"Plus-Dependency"]];
				
				[[tButton cell] setControlSize:NSSmallControlSize];
				
				[tButton setBezelStyle:NSRoundRectBezelStyle];
				
				// A COMPLETER
				
				[tButton setAction:@selector(addBranch:)];
				
				[tButton setTarget:self];
				
				[tContainerView addSubview:tButton];
				
				[tButton release];
			}
			else
			{
				NSLog(@"[ICPresentationInspectorInstallationTypeChoiceDependencyController layoutView:forDictionary:maxY:minX:midY:] Low memory");
			}
			
			[tContainerView release];
		}
		
		
	}
	
	return tMinusButton;
}

- (void) refreshView
{
	NSArray * tSubviews;
	NSSize tOpimizedEnabledViewSize=NSZeroSize;
	NSSize tOpimizedSelectedViewSize=NSZeroSize;
	NSDictionary * tEnabledDependencyDictionary;
	NSDictionary * tSelectedDependencyDictionary;
	NSRect tEnabledViewFrame;
	NSRect tSelectedViewFrame;
	CGFloat tMaxY;
	CGFloat tMidY;
	CGFloat tMinX;
	
	// Remove all the contents in the Enabled and Selected views
	
	tSubviews=[IBchoiceEnabledDependencyView_ subviews];
	
	if (tSubviews!=nil)
	{
		NSUInteger i,tCount;
		
		tCount=[tSubviews count];
		
		for(i=tCount;i>0;i--)
		{
			[[tSubviews objectAtIndex:i-1] removeFromSuperview];
		}
	}
		
	tSubviews=[IBchoiceSelectedDependencyView_ subviews];
	
	if (tSubviews!=nil)
	{
		NSUInteger i,tCount;
		
		tCount=[tSubviews count];
		
		for(i=tCount;i>0;i--)
		{
			[[tSubviews objectAtIndex:i-1] removeFromSuperview];
		}
	}
	
	// Layout the Inspector
	
	if (isGroup_==YES)
	{
		tEnabledDependencyDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
		
		if (tEnabledDependencyDictionary==nil)
		{
			// Create the default enabled dependency dictionary
			
			tEnabledDependencyDictionary=[self defaultEnabledLeafDictionary];
			
			if (tEnabledDependencyDictionary!=nil)
			{
				[dictionary_ setObject:tEnabledDependencyDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
			}
		}
		
		if (tEnabledDependencyDictionary!=nil)
		{
			tOpimizedEnabledViewSize=[self optimizedSizeForDictionary:tEnabledDependencyDictionary];
	
			tEnabledViewFrame=NSMakeRect(0,0,tOpimizedEnabledViewSize.width,tOpimizedEnabledViewSize.height+ICDEPENDENCYMARGIN_TOP+ICDEPENDENCYMARGIN_BOTTOM);
	
			[IBchoiceEnabledDependencyView_ setFrame:tEnabledViewFrame];
			
			// Relayout
	
			tMaxY=tOpimizedEnabledViewSize.height+ICDEPENDENCYMARGIN_TOP;
			
			tMinX=0;
			
			tMidY=0;
			
			[self layoutView:IBchoiceEnabledDependencyView_ forDictionary:tEnabledDependencyDictionary maxY:&tMaxY minX:&tMinX midY:&tMidY];
		}
		else
		{
			// A COMPLETER
		}
	}
	else
	{
		NSNumber * tNumber;
		int tEnabledStateMode=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_ALWAYS;
		BOOL enabledViewIsHidden=YES;
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE];
		
		if (tNumber!=nil)
		{
			tEnabledStateMode=[tNumber intValue];
		}
		
		[IBchoiceEnabledDependencyPopupButton_ selectItemWithTag:tEnabledStateMode];
		
		if (tEnabledStateMode==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_DEPENDENT)
		{
			enabledViewIsHidden=NO;
			
			if ([IBchoiceEnabledDependencyColonLabel_ isHidden]==YES)
			{
				[IBchoiceEnabledDependencyColonLabel_ setHidden:NO];
			}
			
			if ([IBchoiceEnabledDependencyScrollView_ isHidden]==YES)
			{
				[IBchoiceEnabledDependencyScrollView_ setHidden:NO];
			}
			
			tEnabledDependencyDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
		
			if (tEnabledDependencyDictionary==nil)
			{
				// Create the default enabled dependency dictionary
				
				tEnabledDependencyDictionary=[self defaultEnabledLeafDictionary];
			
				[dictionary_ setObject:tEnabledDependencyDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
			}
			
			if (tEnabledDependencyDictionary!=nil)
			{
				tOpimizedEnabledViewSize=[self optimizedSizeForDictionary:tEnabledDependencyDictionary];
				
				tEnabledViewFrame=NSMakeRect(0,0,tOpimizedEnabledViewSize.width,tOpimizedEnabledViewSize.height+ICDEPENDENCYMARGIN_TOP+ICDEPENDENCYMARGIN_BOTTOM);
	
				[IBchoiceEnabledDependencyView_ setFrame:tEnabledViewFrame];
			}
			else
			{
				// A COMPLETER
			}
		}
		else
		{
			if ([IBchoiceEnabledDependencyColonLabel_ isHidden]==NO)
			{
				[IBchoiceEnabledDependencyColonLabel_ setHidden:YES];
			}
			
			if ([IBchoiceEnabledDependencyScrollView_ isHidden]==NO)
			{
				[IBchoiceEnabledDependencyScrollView_ setHidden:YES];
			}
		}
		
		tSelectedDependencyDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_SELECTED_DICTIONARY];
		
		if (tSelectedDependencyDictionary==nil)
		{
			// Create the default selected dependency dictionary
			
			tSelectedDependencyDictionary=[self defaultSelectedLeafDictionary];
			
			[dictionary_ setObject:tSelectedDependencyDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_SELECTED_DICTIONARY];
		}
		
		//NSLog(@"%@",tSelectedDependencyDictionary);
		
		if (tSelectedDependencyDictionary!=nil)
		{
			tOpimizedSelectedViewSize=[self optimizedSizeForDictionary:tSelectedDependencyDictionary];
			
			tSelectedViewFrame=NSMakeRect(0,0,tOpimizedSelectedViewSize.width,tOpimizedSelectedViewSize.height+ICDEPENDENCYMARGIN_TOP+ICDEPENDENCYMARGIN_BOTTOM);
	
			[IBchoiceSelectedDependencyView_ setFrame:tSelectedViewFrame];
		}
		else
		{
			// A COMPLETER
		}
		
		if (enabledViewIsHidden==NO)
		{
			tMaxY=tOpimizedEnabledViewSize.height+ICDEPENDENCYMARGIN_TOP;
			
			tMinX=0;
		
			tMidY=0;
		
			[self layoutView:IBchoiceEnabledDependencyView_ forDictionary:tEnabledDependencyDictionary maxY:&tMaxY minX:&tMinX midY:&tMidY];
		}
		
		tMaxY=tOpimizedSelectedViewSize.height+ICDEPENDENCYMARGIN_TOP;
			
		tMinX=0;
		
		tMidY=0;
		
		[self layoutView:IBchoiceSelectedDependencyView_ forDictionary:tSelectedDependencyDictionary maxY:&tMaxY minX:&tMinX midY:&tMidY];
	}
	
	[self viewFrameDidChange:nil];
	
	[IBview_ setNeedsDisplay:YES];
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
	SEL tAction=[inMenuItem action];
	
	if (tAction==@selector(switchEnabledStateMode:))
    {
		if (isGroup_==NO)
		{
			if ([inMenuItem tag]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_DEPENDENT)
			{
				NSMutableDictionary * tMutableDictionary;
				
				tMutableDictionary=[ICChoiceUtilities availableDependenciesDictionaryForEnabledStateOfLeafNode:node_ skipEnabledIfConstant:NO];
				
				if (tMutableDictionary==nil || [tMutableDictionary count]==0)
				{
					return NO;
				}
			}
		}
	}
	
    return YES;
}

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem forDictionary:(NSDictionary *) inDictionary
{
	if (inDictionary!=nil)
	{
		SEL tAction;
		
		tAction=[inMenuItem action];
		
		if (tAction==@selector(switchEnabledStateObject:))
		{
			NSString * tUUID;
			
			tUUID=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID];
			
			if (tUUID!=nil)
			{
				NSString * tChoiceLabel;
								
				tChoiceLabel=[UUIDsDictionary_ objectForKey:tUUID];
				
				if (tChoiceLabel!=nil)
				{
					NSDictionary * tChoiceDictionary;

					tChoiceDictionary=[enabledChoicesDictionary_ objectForKey:tChoiceLabel];
			
					if (tChoiceDictionary!=nil)
					{
						NSNumber * tNumber;
						
						if ([inMenuItem tag]==ICSTATE_ENABLED)
						{
							tNumber=[tChoiceDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
					
							if (tNumber!=nil && [tNumber boolValue]==YES)
							{
								return YES;
							}
						}
						else if ([inMenuItem tag]==ICSTATE_SELECTED)
						{
							tNumber=[tChoiceDictionary objectForKey:ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY];
					
							if (tNumber!=nil && [tNumber boolValue]==YES)
							{
								return YES;
							}
						}
					}
				}
			}
		}
		else if (tAction==@selector(switchSelectedStateObject:))
		{
			NSString * tUUID;
			
			tUUID=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID];
			
			if (tUUID!=nil)
			{
				NSString * tChoiceLabel;
								
				tChoiceLabel=[UUIDsDictionary_ objectForKey:tUUID];
				
				if (tChoiceLabel!=nil)
				{
					NSDictionary * tChoiceDictionary;

					tChoiceDictionary=[selectedChoicesDictionary_ objectForKey:tChoiceLabel];
			
					if (tChoiceDictionary!=nil)
					{
						NSNumber * tNumber;
						
						if ([inMenuItem tag]==ICSTATE_ENABLED)
						{
							tNumber=[tChoiceDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
					
							if (tNumber!=nil && [tNumber boolValue]==YES)
							{
								return YES;
							}
						}
						else if ([inMenuItem tag]==ICSTATE_SELECTED)
						{
							tNumber=[tChoiceDictionary objectForKey:ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY];
					
							if (tNumber!=nil && [tNumber boolValue]==YES)
							{
								return YES;
							}
						}
					}
				}
			}
		}
	}
	
	return NO;
}

- (IBAction) switchOperator:(id) sender
{
	ICDependencyContainerView * tContainer;
	
	tContainer=(ICDependencyContainerView *) [sender superview];
	
	if (tContainer!=nil)
	{
		NSMutableDictionary * tDictionary;
		
		tDictionary=[tContainer dictionary];
		
		[tDictionary setObject:[NSNumber numberWithInteger:[[sender selectedItem] tag]] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_OPERATOR];
		
		[self noteDocumentChange];
	}
}

- (IBAction) switchEnabledChoice:(id) sender
{
	ICDependencyContainerView * tContainer;
	
	tContainer=(ICDependencyContainerView *) [sender superview];
	
	if (tContainer!=nil)
	{
		NSDictionary * tChoiceDictionary;
		NSString * tChoiceLabel;
		
		tChoiceLabel=[sender titleOfSelectedItem];
		
		tChoiceDictionary=[enabledChoicesDictionary_ objectForKey:tChoiceLabel];
		
		if (tChoiceDictionary!=nil)
		{
			NSString * tUUID;
			NSMutableDictionary * tDictionary;
			NSNumber * tNumber;
			
			tDictionary=[tContainer dictionary];
			
			tUUID=[tChoiceDictionary objectForKey:ICCHOICE_UUID];
		
			[tDictionary setObject:tUUID forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID];
		
			// Check that we can use the selected state object
		
			tNumber=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT];
			
			if (tNumber!=nil)
			{
				if ([tNumber intValue]==ICSTATE_ENABLED)
				{
					tNumber=[tChoiceDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
					
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						[tDictionary setObject:[NSNumber numberWithInt:ICSTATE_SELECTED] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT];
					}
				}
				else if ([tNumber intValue]==ICSTATE_SELECTED)
				{
					tNumber=[tChoiceDictionary objectForKey:ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY];
					
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						[tDictionary setObject:[NSNumber numberWithInt:ICSTATE_ENABLED] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT];
					}
				}
			}
		}
		
		[self noteDocumentChange];
	}
}

- (IBAction) switchSelectedChoice:(id) sender
{
	ICDependencyContainerView * tContainer;
	
	tContainer=(ICDependencyContainerView *) [sender superview];
	
	if (tContainer!=nil)
	{
		NSDictionary * tChoiceDictionary;
		NSString * tChoiceLabel;
		
		tChoiceLabel=[sender titleOfSelectedItem];
		
		tChoiceDictionary=[selectedChoicesDictionary_ objectForKey:tChoiceLabel];
		
		if (tChoiceDictionary!=nil)
		{
			NSString * tUUID;
			NSMutableDictionary * tDictionary;
			NSNumber * tNumber;
			
			tDictionary=[tContainer dictionary];
			
			tUUID=[tChoiceDictionary objectForKey:ICCHOICE_UUID];
		
			[tDictionary setObject:tUUID forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID];
		
			// Check that we can use the selected state object
		
			tNumber=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT];
			
			if (tNumber!=nil)
			{
				if ([tNumber intValue]==ICSTATE_ENABLED)
				{
					tNumber=[tChoiceDictionary objectForKey:ICCHOICE_ENABLED_DEPENDENCY_AVAILABILITY];
					
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						[tDictionary setObject:[NSNumber numberWithInt:ICSTATE_SELECTED] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT];
					}
				}
				else if ([tNumber intValue]==ICSTATE_SELECTED)
				{
					tNumber=[tChoiceDictionary objectForKey:ICCHOICE_SELECTED_DEPENDENCY_AVAILABILITY];
					
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						[tDictionary setObject:[NSNumber numberWithInt:ICSTATE_ENABLED] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT];
					}
				}
			}
		}
		
		[self noteDocumentChange];
	}
}

- (IBAction) switchComparator:(id) sender
{
	ICDependencyContainerView * tContainer;
	
	tContainer=(ICDependencyContainerView *) [sender superview];
	
	if (tContainer!=nil)
	{
		NSMutableDictionary * tDictionary;
		
		tDictionary=[tContainer dictionary];
		
		[tDictionary setObject:[NSNumber numberWithInteger:[[sender selectedItem] tag]] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_COMPARATOR];
		
		[self noteDocumentChange];
	}
}

- (IBAction) switchEnabledStateObject:(id) sender
{
	ICDependencyContainerView * tContainer;
	
	tContainer=(ICDependencyContainerView *) [sender superview];
	
	if (tContainer!=nil)
	{
		NSMutableDictionary * tDictionary;
		
		tDictionary=[tContainer dictionary];
		
		[tDictionary setObject:[NSNumber numberWithInteger:[[sender selectedItem] tag]] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_STATE_OBJECT];
		
		[self noteDocumentChange];
	}
}

- (IBAction) switchSelectedStateObject:(id) sender
{
	[self switchEnabledStateObject:sender];
}

- (IBAction) addBranch:(id) sender
{
	ICDependencyContainerView * tContainer;
	
	tContainer=(ICDependencyContainerView *) [sender superview];
	
	if (tContainer!=nil)
	{
		NSMutableDictionary * tParentDictionary;
		NSMutableDictionary * tDictionary;
		NSMutableDictionary * tNewDictionary=nil;
		NSMutableDictionary * tNewParentDictionary;
		NSMutableDictionary * tDependencyDictionary;
		
		if ([tContainer superview]==IBchoiceEnabledDependencyView_)
		{
			tNewDictionary=[self defaultEnabledLeafDictionary];
		}
		else if ([tContainer superview]==IBchoiceSelectedDependencyView_)
		{
			tNewDictionary=[self defaultSelectedLeafDictionary];
		}
		
		tParentDictionary=[tContainer parentDictionary];
		
		tDictionary=[tContainer dictionary];
		
		tNewParentDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tDictionary,ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP,
																			  [NSNumber numberWithInt:ICOPERATOR_AND],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_OPERATOR,
																			  tNewDictionary,ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM,
																			  nil];
																				  
		tDependencyDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
		
		if (tDependencyDictionary!=nil && tDictionary==tDependencyDictionary)
		{
			[dictionary_ setObject:tNewParentDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_DICTIONARY];
		}
		else
		{
			tDependencyDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_SELECTED_DICTIONARY];
		
			if (tDependencyDictionary!=nil && tDictionary==tDependencyDictionary)
			{
				[dictionary_ setObject:tNewParentDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_SELECTED_DICTIONARY];
			}
			else
			{
				NSMutableDictionary * tChildDictionary;
				
				tChildDictionary=[tParentDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP];
				
				if (tChildDictionary==tDictionary)
				{
					[tParentDictionary setObject:tNewParentDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP];
				}
				else
				{
					[tParentDictionary setObject:tNewParentDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM];
				}
			}
		}
		
		[self refreshView];
		
		[self noteDocumentChange];
	}
}

- (IBAction) removeBranch:(id) sender
{
	ICDependencyContainerView * tContainer;
	
	tContainer=(ICDependencyContainerView *) [sender superview];
	
	if (tContainer!=nil)
	{
		NSMutableDictionary * tParentDictionary;
		NSMutableDictionary * tDictionary;
		NSMutableDictionary * tChildDictionary, * tOtherChildDictionary;
		
		tParentDictionary=[tContainer parentDictionary];
		
		tDictionary=[tContainer dictionary];
		
		
		tChildDictionary=[tParentDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP];
		
		if (tChildDictionary==tDictionary)
		{
			tOtherChildDictionary=[tParentDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM];
		}
		else
		{
			tOtherChildDictionary=tChildDictionary;
		}
		
		[tOtherChildDictionary retain];
		
		[tParentDictionary removeAllObjects];
			
		[tParentDictionary setDictionary:tOtherChildDictionary];
		
		[tOtherChildDictionary release];
		
		[self refreshView];
		
		[self noteDocumentChange];
	}
}

- (void) switchBranches:(id) sender
{
	ICDependencyContainerView * tContainer;
	
	tContainer=(ICDependencyContainerView *) [sender superview];
	
	if (tContainer!=nil)
	{
		NSMutableDictionary * tDictionary;
		NSMutableDictionary * tTopChildDictionary,* tBottomChildDictionary;
		
		tDictionary=[tContainer dictionary];
		
		tTopChildDictionary=[[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP] retain];
		
		tBottomChildDictionary=[[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM] retain];
		
		[tDictionary setObject:tBottomChildDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_TOP];
		
		[tDictionary setObject:tTopChildDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_BRANCH_BOTTOM];
		
		[tTopChildDictionary release];
		
		[tBottomChildDictionary release];
		
		[self refreshView];
		
		[self noteDocumentChange];
	}
}

#pragma mark -

- (IBAction) closePane:(id) sender
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	// Remove the Return to button
	
	[IBcloseView_ removeFromSuperview];
	
	[availableEnabledDependenciesDictionary_ release];
	
	availableEnabledDependenciesDictionary_=nil;
	
	[availableSelectedDependenciesDictionary_ release];
	
	availableSelectedDependenciesDictionary_=nil;
	
	[enabledChoicesDictionary_ release];
	
	enabledChoicesDictionary_=nil;
	
	[selectedChoicesDictionary_ release];
	
	selectedChoicesDictionary_=nil;
	
	[UUIDsDictionary_ release];
	
	UUIDsDictionary_=nil;
	
	[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_ENDEDIT_DIDREQUEST
														object:document_];
	
	// A COMPLETER
}

#pragma mark -

- (BOOL) popUpButton:(id) inPopUpButton canSelectChoice:(NSString *) inUUID
{
	if (inUUID!=nil)
	{
		if ([UUIDsDictionary_ objectForKey:inUUID]!=nil)
		{
			return YES;
		}
	}
	
	return NO;
}

- (void) selectItemOfPopUpButton:(id) inPopUpButton forChoice:(NSString *) inUUID
{
	if (inUUID!=nil)
	{
		ICDependencyContainerView * tContainer;
	
		tContainer=(ICDependencyContainerView *) [inPopUpButton superview];
	
		if (tContainer!=nil)
		{
			NSMutableDictionary * tDictionary;
		
			tDictionary=[tContainer dictionary];
		
			if ([[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_TREE_LEAF_ITEM_UUID] isEqualToString:inUUID]==NO)
			{
				NSString * tChoiceLabel;
				
				tChoiceLabel=[UUIDsDictionary_ objectForKey:inUUID];
				
				[inPopUpButton selectItemWithTitle:tChoiceLabel];
				
				if ([tContainer superview]==IBchoiceEnabledDependencyView_)
				{
					[self switchEnabledChoice:inPopUpButton];
				}
				else if ([tContainer superview]==IBchoiceSelectedDependencyView_)
				{
					[self switchSelectedChoice:inPopUpButton];
				}
			}
		}
	}
}

#pragma mark -

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	NSRect tScrollViewFrame;
	CGFloat tAvailableVertical;
	NSSize tIdealEnabledScrollSize;
	NSRect tEnabledViewFrame;
	NSRect tSelectedViewFrame;
	NSRect tLabelFrame;
	NSSize tIdealSelectedScrollSize;
	
	if (isGroup_==YES)
	{
		tAvailableVertical=NSMinY([IBchoiceEnabledDependencyTextLabel_ frame])-8.0f -20.0f;
		
		tEnabledViewFrame=[IBchoiceEnabledDependencyView_ frame];
		
		tIdealEnabledScrollSize=[NSScrollView frameSizeForContentSize:tEnabledViewFrame.size hasHorizontalScroller:NO hasVerticalScroller:NO borderType:NSBezelBorder];
		
		if (tIdealEnabledScrollSize.width>NSWidth([IBchoiceEnabledDependencyScrollView_ frame]))
		{
			tIdealEnabledScrollSize.height+=16.0f;
		}
		
		tScrollViewFrame=[IBchoiceEnabledDependencyScrollView_ frame];

		if (tIdealEnabledScrollSize.height>=tAvailableVertical)
		{
			tScrollViewFrame.origin.y=20.0f;
			
			tScrollViewFrame.size.height=tAvailableVertical;
		}
		else
		{
			tScrollViewFrame.size.height=tIdealEnabledScrollSize.height;
			
			tScrollViewFrame.origin.y=NSMinY([IBchoiceEnabledDependencyTextLabel_ frame])-8.0f-tScrollViewFrame.size.height;
		}

		[IBchoiceEnabledDependencyScrollView_ setFrame:tScrollViewFrame];
	}
	else
	{
		NSNumber * tNumber;
		int tEnabledStateMode=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_ALWAYS;
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE];
		
		if (tNumber!=nil)
		{
			tEnabledStateMode=[tNumber intValue];
		}
		
		tSelectedViewFrame=[IBchoiceSelectedDependencyView_ frame];
		
		if (tEnabledStateMode==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_DEPENDENT)
		{
			tEnabledViewFrame=[IBchoiceEnabledDependencyView_ frame];
			
			tAvailableVertical=NSMinY([IBchoiceEnabledDependencyTextLabel_ frame])-20.0f -20.0f -NSHeight([IBchoiceSelectedDependencyTextLabel_ bounds]) -8.0f -20.0f;
			
			tIdealEnabledScrollSize=[NSScrollView frameSizeForContentSize:tEnabledViewFrame.size hasHorizontalScroller:NO hasVerticalScroller:NO borderType:NSBezelBorder];
			
			if (tIdealEnabledScrollSize.width>NSWidth([IBchoiceEnabledDependencyScrollView_ frame]))
			{
				tIdealEnabledScrollSize.height+=16.0f;
			}
			
			tIdealSelectedScrollSize=[NSScrollView frameSizeForContentSize:tSelectedViewFrame.size hasHorizontalScroller:NO hasVerticalScroller:NO borderType:NSBezelBorder];
			
			if (tIdealSelectedScrollSize.width>NSWidth([IBchoiceSelectedDependencyScrollView_ frame]))
			{
				tIdealSelectedScrollSize.height+=16.0f;
			}
			
			if ((tIdealEnabledScrollSize.height+tIdealSelectedScrollSize.height)>=tAvailableVertical)
			{
				if (tIdealEnabledScrollSize.height<(tAvailableVertical*0.5f))
				{
					tScrollViewFrame=[IBchoiceEnabledDependencyScrollView_ frame];
				
					tScrollViewFrame.size.height=tIdealEnabledScrollSize.height;
				
					tScrollViewFrame.origin.y=NSMinY([IBchoiceEnabledDependencyTextLabel_ frame])-20.0f-tScrollViewFrame.size.height;
				
					[IBchoiceEnabledDependencyScrollView_ setFrame:tScrollViewFrame];
					
					
					tLabelFrame=[IBchoiceSelectedDependencyTextLabel_ frame];
			
					tLabelFrame.origin.y=NSMinY([IBchoiceEnabledDependencyScrollView_ frame])-20.0f-tLabelFrame.size.height;
			
					[IBchoiceSelectedDependencyTextLabel_ setFrame:tLabelFrame];
					
					
					tScrollViewFrame=[IBchoiceSelectedDependencyScrollView_ frame];
				
					tScrollViewFrame.size.height=tAvailableVertical-tIdealEnabledScrollSize.height;
				
					tScrollViewFrame.origin.y=NSMinY([IBchoiceSelectedDependencyTextLabel_ frame])-8.0f-tScrollViewFrame.size.height;
				
					[IBchoiceSelectedDependencyScrollView_ setFrame:tScrollViewFrame];
					
					
				}
				else if (tIdealSelectedScrollSize.height<(tAvailableVertical*0.5f))
				{
					tScrollViewFrame=[IBchoiceEnabledDependencyScrollView_ frame];
				
					tScrollViewFrame.size.height=tAvailableVertical-tIdealSelectedScrollSize.height;
				
					tScrollViewFrame.origin.y=NSMinY([IBchoiceEnabledDependencyTextLabel_ frame])-20.0f-tScrollViewFrame.size.height;
				
					[IBchoiceEnabledDependencyScrollView_ setFrame:tScrollViewFrame];
					
					
					tLabelFrame=[IBchoiceSelectedDependencyTextLabel_ frame];
			
					tLabelFrame.origin.y=NSMinY([IBchoiceEnabledDependencyScrollView_ frame])-20.0f-tLabelFrame.size.height;
			
					[IBchoiceSelectedDependencyTextLabel_ setFrame:tLabelFrame];
					
					
					tScrollViewFrame=[IBchoiceSelectedDependencyScrollView_ frame];
				
					tScrollViewFrame.size.height=tIdealSelectedScrollSize.height;
				
					tScrollViewFrame.origin.y=NSMinY([IBchoiceSelectedDependencyTextLabel_ frame])-8.0f-tScrollViewFrame.size.height;
				
					[IBchoiceSelectedDependencyScrollView_ setFrame:tScrollViewFrame];
				}
				else
				{
					// Proprotionnaly scale down
					
					CGFloat tRatio;
					
					tRatio=(tIdealEnabledScrollSize.height+tIdealSelectedScrollSize.height)/tAvailableVertical;
					
					tIdealEnabledScrollSize.height=_CGFloatRound(tIdealEnabledScrollSize.height/tRatio);
					
					tIdealSelectedScrollSize.height=tAvailableVertical-tIdealEnabledScrollSize.height;
					
					
					tScrollViewFrame=[IBchoiceEnabledDependencyScrollView_ frame];
				
					tScrollViewFrame.size.height=tIdealEnabledScrollSize.height;
				
					tScrollViewFrame.origin.y=NSMinY([IBchoiceEnabledDependencyTextLabel_ frame])-20.0f-tScrollViewFrame.size.height;
				
					[IBchoiceEnabledDependencyScrollView_ setFrame:tScrollViewFrame];
					
					
					tLabelFrame=[IBchoiceSelectedDependencyTextLabel_ frame];
			
					tLabelFrame.origin.y=NSMinY([IBchoiceEnabledDependencyScrollView_ frame])-20.0f-tLabelFrame.size.height;
			
					[IBchoiceSelectedDependencyTextLabel_ setFrame:tLabelFrame];
					
					
					tScrollViewFrame=[IBchoiceSelectedDependencyScrollView_ frame];
				
					tScrollViewFrame.size.height=tIdealSelectedScrollSize.height;
				
					tScrollViewFrame.origin.y=NSMinY([IBchoiceSelectedDependencyTextLabel_ frame])-8.0f-tScrollViewFrame.size.height;
				
					[IBchoiceSelectedDependencyScrollView_ setFrame:tScrollViewFrame];
				}
			}
			else
			{
				tScrollViewFrame=[IBchoiceEnabledDependencyScrollView_ frame];
				
				tScrollViewFrame.size.height=tIdealEnabledScrollSize.height;
				
				tScrollViewFrame.origin.y=NSMinY([IBchoiceEnabledDependencyTextLabel_ frame])-20.0f-tScrollViewFrame.size.height;
				
				[IBchoiceEnabledDependencyScrollView_ setFrame:tScrollViewFrame];
				
				
				tLabelFrame=[IBchoiceSelectedDependencyTextLabel_ frame];
			
				tLabelFrame.origin.y=NSMinY([IBchoiceEnabledDependencyScrollView_ frame])-20.0f-tLabelFrame.size.height;
			
				[IBchoiceSelectedDependencyTextLabel_ setFrame:tLabelFrame];
				
				
				tScrollViewFrame=[IBchoiceSelectedDependencyScrollView_ frame];
				
				tScrollViewFrame.size.height=tIdealSelectedScrollSize.height;
				
				tScrollViewFrame.origin.y=NSMinY([IBchoiceSelectedDependencyTextLabel_ frame])-8.0f-tScrollViewFrame.size.height;
				
				[IBchoiceSelectedDependencyScrollView_ setFrame:tScrollViewFrame];
			}
		}
		else
		{
			// Set the position of the label view
			
			tLabelFrame=[IBchoiceSelectedDependencyTextLabel_ frame];
			
			tLabelFrame.origin.y=NSMinY([IBchoiceEnabledDependencyTextLabel_ frame])-20.0f-tLabelFrame.size.height;
			
			[IBchoiceSelectedDependencyTextLabel_ setFrame:tLabelFrame];
			
			tAvailableVertical=NSMinY([IBchoiceEnabledDependencyTextLabel_ frame])-20.0f -NSHeight([IBchoiceSelectedDependencyTextLabel_ bounds])-8.0f       -20.0f;
			
			tIdealSelectedScrollSize=[NSScrollView frameSizeForContentSize:tSelectedViewFrame.size hasHorizontalScroller:NO hasVerticalScroller:NO borderType:NSBezelBorder];
			
			if (tIdealSelectedScrollSize.width>NSWidth([IBchoiceSelectedDependencyScrollView_ frame]))
			{
				tIdealSelectedScrollSize.height+=16.0f;
			}
			
			tScrollViewFrame=[IBchoiceSelectedDependencyScrollView_ frame];
	
			if (tIdealSelectedScrollSize.height>=tAvailableVertical)
			{
				tScrollViewFrame.size.height=tAvailableVertical;
				
				tScrollViewFrame.origin.y=20.0f;
			}
			else
			{
				tScrollViewFrame.size.height=tIdealSelectedScrollSize.height;
				
				tScrollViewFrame.origin.y=NSMinY([IBchoiceSelectedDependencyTextLabel_ frame])-8.0f-tScrollViewFrame.size.height;
			}
	
			[IBchoiceSelectedDependencyScrollView_ setFrame:tScrollViewFrame];
		}
	}
}

@end
