/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationInspectorInstallationTypeChoiceSelectionController.h"

#import "ICLocalizationUtilities.h"

#import "ICInstallationTypeUtilities+Extras.h"

#import "ICChoiceUtilities.h"

#import "ICLanguagePopupButtonCell.h"

#import "ICDocument.h"

#import "ICInstallationType+Constants.h"

#import "NSPopUpButton+OptimizedSize.h"

#import "ICRequirementChoiceEditorController.h"

@implementation ICPresentationInspectorInstallationTypeChoiceSelectionController

- (void) dealloc
{
	//NSLog(@"[ICPresentationInspectorInstallationTypeChoiceSelectionController dealloc]");
	
	[cachedTreeNode_ release];
	
	[cachedTitleLocalizations_ release];
	
	[cachedDescriptionLocalizations_ release];
	
	[cachedRequirements_ release];
	
	[requirementController_ release];
	
	[tabViewItem_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn = nil;
    
    // *** Title
	
	// Reference
    
    tTableColumn = [IBtitleArray_ tableColumnWithIdentifier:ICDOCUMENT_LANGUAGE];
    
	if (tTableColumn!=nil)
	{
		ICLanguagePopupButtonCell * tPopupButtonCell = nil;
    
		tPopupButtonCell=[[ICLanguagePopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
		
		if (tPopupButtonCell!=nil)
		{
			[tPopupButtonCell setControlSize:NSSmallControlSize];
			
			[tPopupButtonCell setMenu:[ICLocalizationUtilities localizationMenu]];
			
			[tPopupButtonCell setBordered:NO];
			
			[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
			
			[tPopupButtonCell setAction:@selector(nothingTitle:)];
			
			[tPopupButtonCell setTarget:self];
			
			[tTableColumn setDataCell:tPopupButtonCell];
			
			[tPopupButtonCell release];
		}
	}
	
	// Name
    
    tTableColumn = [IBtitleArray_ tableColumnWithIdentifier:ICDOCUMENT_VALUE];
	
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
	
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
	}
	
	 // *** Description
	
	// Language
    
    tTableColumn = [IBdescriptionArray_ tableColumnWithIdentifier:ICDOCUMENT_LANGUAGE];
    
	if (tTableColumn!=nil)
	{
		ICLanguagePopupButtonCell * tPopupButtonCell = nil;
    
		tPopupButtonCell=[[ICLanguagePopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
		
		if (tPopupButtonCell!=nil)
		{
			[tPopupButtonCell setControlSize:NSSmallControlSize];
			
			[tPopupButtonCell setMenu:[ICLocalizationUtilities localizationMenu]];
			
			[tPopupButtonCell setBordered:NO];
			
			[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
			
			[tPopupButtonCell setAction:@selector(nothingDescription:)];
			
			[tPopupButtonCell setTarget:self];
			
			[tTableColumn setDataCell:tPopupButtonCell];
			
			[tPopupButtonCell release];
		}
	}
	
	// String
    
    tTableColumn = [IBdescriptionArray_ tableColumnWithIdentifier:ICDOCUMENT_VALUE];
	
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
	
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
		
		[tTextFieldCell setWraps:YES];
	}
	
	// *** Requirements
	
	[IBrequirementsArray_ setDoubleAction:@selector(editRequirement:)];
	
	[IBrequirementsArray_ setTarget:self];
	
	// State
	
	tTableColumn = [IBrequirementsArray_ tableColumnWithIdentifier:@"State"];
    
	if (tTableColumn!=nil)
	{
		NSButtonCell * tButttonCell;
		
		tButttonCell = [[NSButtonCell alloc] initTextCell: @""];
    
		if (tButttonCell!=nil)
		{
			[tButttonCell setControlSize:NSMiniControlSize];
			[tButttonCell setEditable: YES];
			[tButttonCell setButtonType: NSSwitchButton];
			[tButttonCell setImagePosition: NSImageOnly];
		
			[tTableColumn setDataCell:tButttonCell];
		
			[tButttonCell release];
		}
	}
	
	// Name
	
	tTableColumn = [IBrequirementsArray_ tableColumnWithIdentifier:@"Name"];
    
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
	
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
	}
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument;
{
	id tObject;
	
	[cachedTreeNode_ release];
	
	cachedTreeNode_=nil;
	
	[cachedTitleLocalizations_ release];
	
	cachedTitleLocalizations_=nil;
	
	[cachedDescriptionLocalizations_ release];
	
	cachedDescriptionLocalizations_=nil;
	
	[cachedRequirements_ release];
	
	cachedRequirements_=nil;

	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	tObject=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTION];
	
	if (tObject!=nil)
	{
		if ([tObject isKindOfClass:[NSDictionary class]]==NO)
		{
			cachedTreeNode_=(ICInstallationTypeTreeNode *) tObject;
		}
	}
	
	if (cachedTreeNode_!=nil)
	{
		ICInstallationTypeNodeData * tNodeData;
		
		[cachedTreeNode_ retain];
	
		tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(cachedTreeNode_);
					
		dictionary_=[tNodeData dictionary];
	
		if (dictionary_!=nil)
		{
			NSNumber * tNumber;
			int tType=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE;
			NSDictionary * tOptionsDictionary;
			int tState;
			NSRect tFrame;
			
			[dictionary_ retain];
		
			tOptionsDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
			
			// Type
			
			tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
			
			if (tNumber!=nil)
			{
				tType=[tNumber intValue];
			}
			
			if (tType==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
			{
				NSString * tPackageUUID;
				NSDictionary * tDictionary;
				NSString * tPackageName=nil;
				
				[IBchoiceIcon_ setImage:[[NSWorkspace sharedWorkspace] iconForFileType:@".pkg"]];
				
				[IBchoiceType_ setStringValue:NSLocalizedString(@"Package",@"")];
								
				tPackageUUID=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_PACKAGE_UUID];
				
				tDictionary=[[self document] dictionaryOfPackageWithUUID:tPackageUUID];
				
				if (tDictionary!=nil)
				{
					tPackageName=[[tDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
				}
				
				if (tPackageName==nil)
				{
					tPackageName=@"";
				}
				
				[IBchoicePackageName_ setStringValue:tPackageName];
			}
			else
			{
				NSArray * tChildren;
				
				tChildren=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
			
				if (tOptionsDictionary!=nil)
				{
					NSUInteger tCount;
					
					tCount=[tChildren count];
					
					tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
				
					if (tNumber!=nil && [tNumber boolValue]==YES)
					{
						[IBchoiceIcon_ setImage:[NSImage imageNamed:@"PackagesCombination"]];
				
						[IBchoicePackageName_ setStringValue:NSLocalizedString(@"Combination of Packages",@"")];
						
						if (tCount==1)
						{
							[IBchoiceType_ setStringValue:NSLocalizedString(@"1 package",@"")];
						}
						else if (tCount>1)
						{
							[IBchoiceType_ setStringValue:[NSString stringWithFormat:NSLocalizedString(@"%u packages",@""),tCount]];
						}
					}
					else
					{
						[IBchoiceIcon_ setImage:[NSImage imageNamed:@"ChoicesGroup"]];
				
						[IBchoicePackageName_ setStringValue:NSLocalizedString(@"Group of Choices",@"")];
						
						if (tCount==0)
						{
							[IBchoiceType_ setStringValue:NSLocalizedString(@"No sub-choices",@"")];
						}
						else if (tCount==1)
						{
							[IBchoiceType_ setStringValue:NSLocalizedString(@"1 sub-choice",@"")];
						}
						else if (tCount>1)
						{
							[IBchoiceType_ setStringValue:[NSString stringWithFormat:NSLocalizedString(@"%u sub-choices",@""),tCount]];
						}
					}
				}
			}
			
			// Title
			
			cachedTitleLocalizations_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TITLE];
		
			if (cachedTitleLocalizations_==nil)
			{
				
			}
			
			// Descriptions
			
			cachedDescriptionLocalizations_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_DESCRIPTION];
			
			if (cachedDescriptionLocalizations_==nil)
			{
				cachedDescriptionLocalizations_=[NSMutableArray array];
				
				[dictionary_ setObject:cachedDescriptionLocalizations_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_DESCRIPTION];
			}
			
			if (cachedTitleLocalizations_!=nil)
			{
				[cachedTitleLocalizations_ retain];
				
				[IBtitleArray_ deselectAll:self];
				
				[IBtitleArray_ reloadData];
			}
			
			if (cachedDescriptionLocalizations_!=nil)
			{
				[cachedDescriptionLocalizations_ retain];
				
				[IBdescriptionArray_ deselectAll:self];
				
				[IBdescriptionArray_ reloadData];
			}
			
			// Visibility
			
			tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN];
			
			[IBchoicePackageVisibility_ selectItemWithTag:[tNumber boolValue]];
			
			// State
			
			tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
			
			if ([cachedTreeNode_ isGroup]==YES)
			{
				NSMenu * tMenu;
				
				tState=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_ALWAYS;
			
				// Set the Menu Items
				
				[IBchoicePackageState_ removeAllItems];
				
				tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] init];
				
				if (tMenu!=nil)
				{
					NSMenuItem * tMenuItem;
					
					tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Always Enabled",@"",@"") action:nil keyEquivalent:@""];
					
					if (tMenuItem!=nil)
					{
						[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_ALWAYS];
						
						[tMenu addItem:tMenuItem];
						
						[tMenuItem release];
					}
					else
					{
						// A COMPLETER
					}
					
					tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Always Disabled",@"",@"") action:nil keyEquivalent:@""];
					
					if (tMenuItem!=nil)
					{
						[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_NEVER];
						
						[tMenu addItem:tMenuItem];
						
						[tMenuItem release];
					}
					else
					{
						// A COMPLETER
					}
					
					tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Dependent on Other Choices",@"",@"") action:nil keyEquivalent:@""];
					
					if (tMenuItem!=nil)
					{
						[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_DEPENDENT];
						
						[tMenu addItem:tMenuItem];
						
						[tMenuItem release];
					}
					else
					{
						// A COMPLETER
					}
					
					[IBchoicePackageState_ setMenu:tMenu];
					
					[tMenu release];
				}
			}
			else
			{
				NSMenu * tMenu;
				
				tState=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED;
			
				// Set the Menu Items
				
				[IBchoicePackageState_ removeAllItems];
				
				tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] init];
				
				if (tMenu!=nil)
				{
					NSMenuItem * tMenuItem;
					
					tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Required",@"",@"") action:nil keyEquivalent:@""];
					
					if (tMenuItem!=nil)
					{
						[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_REQUIRED];
						
						[tMenu addItem:tMenuItem];
						
						[tMenuItem release];
					}
					else
					{
						// A COMPLETER
					}
					
					tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Selected",@"",@"") action:nil keyEquivalent:@""];
					
					if (tMenuItem!=nil)
					{
						[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED];
						
						[tMenu addItem:tMenuItem];
						
						[tMenuItem release];
					}
					else
					{
						// A COMPLETER
					}
					
					tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Unselected",@"",@"") action:nil keyEquivalent:@""];
					
					if (tMenuItem!=nil)
					{
						[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_UNSELECTED];
						
						[tMenu addItem:tMenuItem];
						
						[tMenuItem release];
					}
					else
					{
						// A COMPLETER
					}
					
					tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Dependent on Other Choices",@"",@"") action:nil keyEquivalent:@""];
					
					if (tMenuItem!=nil)
					{
						[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENT];
						
						[tMenu addItem:tMenuItem];
						
						[tMenuItem release];
					}
					else
					{
						// A COMPLETER
					}
					
					[IBchoicePackageState_ setMenu:tMenu];
					
					[tMenu release];
				}
			}
			
			if (tNumber!=nil)
			{
				tState=[tNumber intValue];
			}
			
			tFrame=[IBchoicePackageState_ frame];
			
			tFrame.size.width=[IBchoicePackageState_ optimizedSize].width;
			
			[IBchoicePackageState_ setFrame:tFrame];
			
			[IBchoicePackageState_ selectItemWithTag:tState];
			
			if ([cachedTreeNode_ isGroup]==YES)
			{
				if (tState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_DEPENDENT)
				{
					[IBchoicePackageStateEditButton_ setHidden:NO];
				}
				else
				{
					[IBchoicePackageStateEditButton_ setHidden:YES];
				}
			}
			else
			{
				if (tState==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENT)
				{
					[IBchoicePackageStateEditButton_ setHidden:NO];
				}
				else
				{
					[IBchoicePackageStateEditButton_ setHidden:YES];
				}
			}
			
			
			
			// Requirements
			
			
			
			if ([cachedTreeNode_ isGroup]==YES)
			{
				if (tabViewItem_==nil)
				{
					tabViewItem_=[IBtabView_ tabViewItemAtIndex:[IBtabView_ indexOfTabViewItemWithIdentifier:@"Requirements"]];
				
					[tabViewItem_ retain];
				
					[IBtabView_ removeTabViewItem:tabViewItem_];
				}
			}
			else
			{
				if (tabViewItem_!=nil)
				{
					[IBtabView_ addTabViewItem:tabViewItem_];
				
					[tabViewItem_ release];
					
					tabViewItem_=nil;
				}
				
				cachedRequirements_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_REQUIREMENTS];
				
				if (cachedRequirements_==nil)
				{
					cachedRequirements_=[NSMutableArray array];
					
					[dictionary_ setObject:cachedRequirements_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_REQUIREMENTS];
				}
				
				if (cachedRequirements_!=nil)
				{
					[cachedRequirements_ retain];
					
					[IBrequirementsEditButton_ setEnabled:NO];
					
					[IBrequirementsRemoveButton_ setEnabled:NO];
					
					[IBrequirementsArray_ deselectAll:self];
					
					[IBrequirementsArray_ reloadData];
					
					[self updateRequirementTabLabel];
				}
			}
			
			[self viewFrameDidChange:nil];
			
			// Register for Notifications
	
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(viewFrameDidChange:) 
														 name:NSViewFrameDidChangeNotification 
													   object:IBview_];
		}
	}
}

#pragma mark -

- (void) viewWillBeRemoved:(id) sender
{
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSViewFrameDidChangeNotification object:IBview_];
}

#pragma mark -

- (IBAction) nothingTitle:(id) sender
{
}

- (IBAction) nothingDescription:(id) sender
{
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (inTableView==IBtitleArray_)
	{
		if (cachedTitleLocalizations_!=nil)
		{
			return [cachedTitleLocalizations_ count];
		}
	}
	else if (inTableView==IBdescriptionArray_)
	{
		if (cachedDescriptionLocalizations_!=nil)
		{
			return [cachedDescriptionLocalizations_ count];
		}
	}
	else if (inTableView==IBrequirementsArray_)
	{
		if (cachedRequirements_!=nil)
		{
			return [cachedRequirements_ count];
		}
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	NSDictionary * tDictionary;
	
	if (inTableView==IBtitleArray_)
	{
		if (cachedTitleLocalizations_!=nil)
		{
			tColumnIdentifier=[inTableColumn identifier];
			
			tDictionary=[cachedTitleLocalizations_ objectAtIndex:inRowIndex];
			
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
			{
				return [tDictionary objectForKey:ICDOCUMENT_VALUE];
			}
		}
	}
	else if (inTableView==IBdescriptionArray_)
	{
		if (cachedTitleLocalizations_!=nil)
		{
			tColumnIdentifier=[inTableColumn identifier];
			
			tDictionary=[cachedDescriptionLocalizations_ objectAtIndex:inRowIndex];
			
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
			{
				return [tDictionary objectForKey:ICDOCUMENT_VALUE];
			}
		}
	}
	else if (inTableView==IBrequirementsArray_)
	{
		if (cachedRequirements_!=nil)
		{
			tColumnIdentifier=[inTableColumn identifier];
			
			tDictionary=[cachedRequirements_ objectAtIndex:inRowIndex];
			
			if ([tColumnIdentifier isEqualToString:@"State"])
			{
				return [tDictionary objectForKey:ICDOCUMENT_REQUIREMENT_STATE];
			}
			else if ([tColumnIdentifier isEqualToString:@"Name"])
			{
				return [tDictionary objectForKey:ICDOCUMENT_REQUIREMENT_NAME];
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	BOOL tDidChange=NO;
	NSString * tColumnIdentifier;
	
	if (inTableView==IBtitleArray_)
	{
		if (cachedTitleLocalizations_!=nil)
		{
			NSMutableDictionary * tDictionary;
			
			tColumnIdentifier=[inTableColumn identifier];
		
			tDictionary=[cachedTitleLocalizations_ objectAtIndex:inRowIndex];
				
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
			{
				NSString * tLanguageName;
				
				// Check that the Language is different
				
				tLanguageName=[[inTableColumn dataCellForRow:inRowIndex] itemTitleAtIndex:[object intValue]];
				
				if ([[tDictionary objectForKey:ICDOCUMENT_LANGUAGE] isEqualToString:tLanguageName]==NO)
				{
					NSEnumerator * tEnumerator;
					NSUInteger tSelectionIndex;
					
					tEnumerator=[cachedTitleLocalizations_ objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSDictionary * tLocalizationDictionary;
						
						while (tLocalizationDictionary=[tEnumerator nextObject])
						{
							NSString * tExistingLanguage;
							
							tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
							
							if ([tExistingLanguage isEqualToString:tLanguageName]==YES)
							{
								NSBeep();
								
								return;
							}
						}
					}
					
					[tDictionary setObject:tLanguageName forKey:ICDOCUMENT_LANGUAGE];
					
					[inTableView deselectAll:self];
					
					[cachedTitleLocalizations_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
					
					[inTableView reloadData];
					
					tSelectionIndex=[cachedTitleLocalizations_ indexOfObject:tDictionary];
					
					if (tSelectionIndex!=NSNotFound)
					{
						[inTableView selectRowIndexes:[NSIndexSet indexSetWithIndex:tSelectionIndex] byExtendingSelection:NO];
					}
					
					tDidChange=YES;
				}
			}
			else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
			{
				NSString * tTitleValue;
				
				tTitleValue=(NSString *) object;
				
				if ([[tDictionary objectForKey:ICDOCUMENT_VALUE] isEqualToString:tTitleValue]==NO)
				{
					[tDictionary setObject:tTitleValue forKey:ICDOCUMENT_VALUE];
					
					tDidChange=YES;
				}
			}
		}
	}
	else if (inTableView==IBdescriptionArray_)
	{
		if (cachedDescriptionLocalizations_!=nil)
		{
			NSMutableDictionary * tDictionary;
			
			tColumnIdentifier=[inTableColumn identifier];
		
			tDictionary=[cachedDescriptionLocalizations_ objectAtIndex:inRowIndex];
				
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
			{
				NSString * tLanguageName;
				
				// Check that the Language is different
				
				tLanguageName=[[inTableColumn dataCellForRow:inRowIndex] itemTitleAtIndex:[object intValue]];
				
				if ([[tDictionary objectForKey:ICDOCUMENT_LANGUAGE] isEqualToString:tLanguageName]==NO)
				{
					NSEnumerator * tEnumerator;
					NSUInteger tSelectionIndex;
					
					tEnumerator=[cachedDescriptionLocalizations_ objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSDictionary * tLocalizationDictionary;
						
						while (tLocalizationDictionary=[tEnumerator nextObject])
						{
							NSString * tExistingLanguage;
							
							tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
							
							if ([tExistingLanguage isEqualToString:tLanguageName]==YES)
							{
								NSBeep();
								
								return;
							}
						}
					}
					
					[tDictionary setObject:tLanguageName forKey:ICDOCUMENT_LANGUAGE];
					
					[inTableView deselectAll:self];
					
					[cachedDescriptionLocalizations_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
					
					[inTableView reloadData];
					
					tSelectionIndex=[cachedDescriptionLocalizations_ indexOfObject:tDictionary];
					
					if (tSelectionIndex!=NSNotFound)
					{
						[inTableView selectRowIndexes:[NSIndexSet indexSetWithIndex:tSelectionIndex] byExtendingSelection:NO];
					}
					
					tDidChange=YES;
				}
			}
			else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
			{
				NSString * tTitleValue;
				
				tTitleValue=(NSString *) object;
				
				if ([[tDictionary objectForKey:ICDOCUMENT_VALUE] isEqualToString:tTitleValue]==NO)
				{
					[tDictionary setObject:tTitleValue forKey:ICDOCUMENT_VALUE];
					
					tDidChange=YES;
				}
			}
		}
	}
	else if (inTableView==IBrequirementsArray_)
	{
		if (cachedRequirements_!=nil)
		{
			NSMutableDictionary * tDictionary;
			
			tColumnIdentifier=[inTableColumn identifier];
		
			tDictionary=[cachedRequirements_ objectAtIndex:inRowIndex];
				
			if ([tColumnIdentifier isEqualToString:@"State"])
			{
				NSNumber * tNumber;
				
				tNumber=[tDictionary objectForKey:ICDOCUMENT_REQUIREMENT_STATE];
				
				if ([tNumber boolValue]!=[object boolValue])
				{
					[tDictionary setObject:object forKey:ICDOCUMENT_REQUIREMENT_STATE];
					
					[self noteDocumentChange];
				}
			}
			else if ([tColumnIdentifier isEqualToString:@"Name"])
			{
				NSString * tString;
							
				tString=[tDictionary objectForKey:ICDOCUMENT_REQUIREMENT_NAME];
							
				if ([tString isEqualToString:object]==NO)
				{
					NSEnumerator * tEnumerator;
			
					tEnumerator=[cachedRequirements_ objectEnumerator];
			
					if (tEnumerator!=nil)
					{
						NSDictionary * tOtherDictionary;
						
						while (tOtherDictionary=[tEnumerator nextObject])
						{
							if (tOtherDictionary!=tDictionary)
							{
								NSString * tOtherName;
							
								tOtherName=[tOtherDictionary objectForKey:ICDOCUMENT_REQUIREMENT_NAME];
							
								if (tOtherName!=nil)
								{
									if ([tOtherName isEqualToString:object]==YES)
									{
										NSBeep();
										
										[IBrequirementsArray_ editColumn:[IBrequirementsArray_ columnWithIdentifier:@"Name"] row:inRowIndex withEvent:nil select:YES];
										
										return;
									}
								}
							}
						}
						
						[tDictionary setObject:object forKey:ICDOCUMENT_REQUIREMENT_NAME];
					
						[self noteDocumentChange];
					}
				}
			}
		}
	}
	
	if (tDidChange==YES)
	{
		[self noteDocumentChange];
		
		// Notify the modification to the simulator
		
		[ICInstallationTypeUtilities notifyInstallationTypeDidChangeWithSender:self];
	}
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{    
    if (inTableView==IBtitleArray_)
	{
		if (cachedTitleLocalizations_!=nil)
		{
			NSString * tColumnIdentifier;
		
			tColumnIdentifier=[inTableColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
			{
				NSDictionary * tDictionary;
			
				tDictionary=[cachedTitleLocalizations_ objectAtIndex:inRowIndex];
			
				[cell selectItemWithTitle:[tDictionary objectForKey:ICDOCUMENT_LANGUAGE]];
			}
		}
	}
	else if (inTableView==IBdescriptionArray_)
	{
		if (cachedDescriptionLocalizations_!=nil)
		{
			NSString * tColumnIdentifier;
		
			tColumnIdentifier=[inTableColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
			{
				NSDictionary * tDictionary;
			
				tDictionary=[cachedDescriptionLocalizations_ objectAtIndex:inRowIndex];
			
				[cell selectItemWithTitle:[tDictionary objectForKey:ICDOCUMENT_LANGUAGE]];
			}
		}
	}
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBtitleArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeTitleLocalization:IBtitleRemoveButton_];
		}
		else
		{
			[self removeTitleLocalizationDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
	else if (inTableView==IBdescriptionArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeDescriptionLocalization:IBdescriptionRemoveButton_];
		}
		else
		{
			[self removeDescriptionLocalizationDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
	else if (inTableView==IBrequirementsArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeRequirements:IBrequirementsRemoveButton_];
		}
		else
		{
			[self removeRequirementsDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBtitleArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBtitleRemoveButton_ isEnabled];
		}
	}
	else if (inTableView==IBdescriptionArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBdescriptionRemoveButton_ isEnabled];
		}
	}
	else if (inTableView==IBrequirementsArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBrequirementsRemoveButton_ isEnabled];
		}
	}
	
	return NO;
}

- (NSString *) tableView:(NSTableView *) inTableView toolTipForCell:(NSCell *)cell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex mouseLocation:(NSPoint)mouseLocation
{
	if (inTableView==IBtitleArray_)
	{
		if (cachedTitleLocalizations_!=nil)
		{
			NSString * tColumnIdentifier;
			NSDictionary * tDictionary;
			
			tDictionary=[cachedTitleLocalizations_ objectAtIndex:inRowIndex];
				
			tColumnIdentifier=[inTableColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
			{
				return [tDictionary objectForKey:ICDOCUMENT_LANGUAGE];
			}
		}
	}
	else if (inTableView==IBdescriptionArray_)
	{
		if (cachedDescriptionLocalizations_!=nil)
		{
			NSString * tColumnIdentifier;
			NSDictionary * tDictionary;
			
			tDictionary=[cachedDescriptionLocalizations_ objectAtIndex:inRowIndex];
				
			tColumnIdentifier=[inTableColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
			{
				return [tDictionary objectForKey:ICDOCUMENT_LANGUAGE];
			}
		}
	}
	
	return nil;
}

#pragma mark -

- (IBAction) addTitleLocalization:(id) sender
{
	if (cachedTitleLocalizations_!=nil)
	{
		NSMutableDictionary * tNewLocalizedTitleDictionary;
		NSString * tLanguageName;
	
		tLanguageName=[[self document] documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
							
		if ([ICLocalizationUtilities localizations:cachedTitleLocalizations_ containsLanguage:tLanguageName]==YES)
		{
			tLanguageName=[ICLocalizationUtilities firstNewLanguageInLocalizations:cachedTitleLocalizations_];
		}
		
		if (tLanguageName!=nil)
		{
			tNewLocalizedTitleDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tLanguageName,ICDOCUMENT_LANGUAGE,
																						   @"",ICDOCUMENT_VALUE,
																						   nil];
			
			if (tNewLocalizedTitleDictionary!=nil)
			{
				NSUInteger tRowIndex;
				
				[cachedTitleLocalizations_ addObject:tNewLocalizedTitleDictionary];
				
				[cachedTitleLocalizations_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[IBtitleArray_ reloadData];
				
				tRowIndex=[cachedTitleLocalizations_ indexOfObject:tNewLocalizedTitleDictionary];
				
				if (tRowIndex!=NSNotFound)
				{
					[IBtitleArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
					[IBtitleArray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
					
					[self viewFrameDidChange:nil];
				}
			}
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (IBAction) removeTitleLocalization:(id) sender
{
	NSString * tAlertTitle;
    
    if ([IBtitleArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this localization?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these localizations?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeTitleLocalizationDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeTitleLocalizationDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBtitleArray_ selectedRowIndexes];
		
		[IBtitleArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedTitleLocalizations_ removeObjectsAtIndexes:tIndexSet];
			
			[IBtitleArray_ reloadData];
			
			[IBtitleRemoveButton_ setEnabled:NO];
			
			// Notify Document Change
			
			[self noteDocumentChange];
			
			[ICInstallationTypeUtilities notifyInstallationTypeDidChangeWithSender:self];
			
			[self viewFrameDidChange:nil];
		}
	}
}

- (IBAction) addDescriptionLocalization:(id) sender
{
	if (cachedDescriptionLocalizations_!=nil)
	{
		NSMutableDictionary * tNewLocalizedDescriptionDictionary;
		NSString * tLanguageName;
	
		tLanguageName=[[self document] documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
							
		if ([ICLocalizationUtilities localizations:cachedDescriptionLocalizations_ containsLanguage:tLanguageName]==YES)
		{
			tLanguageName=[ICLocalizationUtilities firstNewLanguageInLocalizations:cachedDescriptionLocalizations_];
		}
		
		if (tLanguageName!=nil)
		{
			tNewLocalizedDescriptionDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tLanguageName,ICDOCUMENT_LANGUAGE,
																						   @"",ICDOCUMENT_VALUE,
																						   nil];
			
			if (tNewLocalizedDescriptionDictionary!=nil)
			{
				NSUInteger tRowIndex;
				
				[cachedDescriptionLocalizations_ addObject:tNewLocalizedDescriptionDictionary];
				
				[cachedDescriptionLocalizations_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[IBdescriptionArray_ reloadData];
				
				tRowIndex=[cachedDescriptionLocalizations_ indexOfObject:tNewLocalizedDescriptionDictionary];
				
				if (tRowIndex!=NSNotFound)
				{
					[IBdescriptionArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
					[IBdescriptionArray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
					
					[self viewFrameDidChange:nil];
				}
			}
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (IBAction) removeDescriptionLocalization:(id) sender
{
	NSString * tAlertTitle;
    
    if ([IBdescriptionArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this localization?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these localizations?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeDescriptionLocalizationDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeDescriptionLocalizationDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBdescriptionArray_ selectedRowIndexes];
		
		[IBdescriptionArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedDescriptionLocalizations_ removeObjectsAtIndexes:tIndexSet];
			
			[IBdescriptionArray_ reloadData];
			
			[IBdescriptionRemoveButton_ setEnabled:NO];
			
			// Notify Document Change
			
			[self noteDocumentChange];
			
			[ICInstallationTypeUtilities notifyInstallationTypeDidChangeWithSender:self];
			
			[self viewFrameDidChange:nil];
		}
	}
}

- (IBAction) switchPackageVisibility:(id) sender
{
	NSInteger tTag;
	NSMutableDictionary * tOptionsDictionary;
	
	tTag=[[sender selectedItem] tag];
	
	tOptionsDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
	
	if (tTag!=[[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN] boolValue])
	{
		NSNumber * tNumber;
		
		[tOptionsDictionary setObject:[NSNumber numberWithBool:(BOOL) tTag] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN];
		
		// An invisible item can not be unselected as it has no sense at all
		
		tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
		
		if (tNumber!=nil && [tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_UNSELECTED)
		{
			[tOptionsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
		
			[IBchoicePackageState_ selectItemWithTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED];
		}
		
		// Notify Document Change
			
		[self noteDocumentChange];
			
		[ICInstallationTypeUtilities notifyInstallationTypeDidChangeWithSender:self];
	}
}

- (IBAction) switchPackageState:(id) sender
{
	NSInteger tTag;
	NSMutableDictionary * tOptionsDictionary;
	BOOL tPostNotification=NO;
	NSMutableDictionary * tDependencyDictionary=nil;
	
	tTag=[[sender selectedItem] tag];
	
	tOptionsDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
	
	if (tTag!=[[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE] intValue])
	{
		[tOptionsDictionary setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
		
		if (tTag==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENT ||
			tTag==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_DEPENDENT)
		{
			if ([cachedTreeNode_ isGroup]==YES)
			{
				tDependencyDictionary=[NSMutableDictionary dictionary];
			}
			else
			{
				tDependencyDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_ALWAYS],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY_ENABLED_STATE_MODE,
																						nil];
			}
			
			if (tDependencyDictionary!=nil)
			{
				[tOptionsDictionary setObject:tDependencyDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
				
				[IBchoicePackageStateEditButton_ setHidden:NO];
			
				tPostNotification=YES;
			}
		}
		else
		{
			[IBchoicePackageStateEditButton_ setHidden:YES];
			
			// We need to remove the dependency dictionary
			
			[tOptionsDictionary removeObjectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
		}
		
		// Notify Document Change
			
		[self noteDocumentChange];
			
		[ICInstallationTypeUtilities notifyInstallationTypeDidChangeWithSender:self];
		
		if (tPostNotification==YES)
		{
			// Show the dependency pane
			
			[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_EDIT_DIDREQUEST
																object:document_
															  userInfo:nil];
		}
	}
}

- (IBAction) editChoiceState:(id) sender
{
	NSMutableDictionary * tOptionsDictionary;
	
	tOptionsDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
	
	if (tOptionsDictionary!=nil)
	{
		NSMutableDictionary * tDependencyDictionary;
		
		tDependencyDictionary=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
		
		if (tDependencyDictionary!=nil)
		{
			// Show the dependency pane
			
			[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_EDIT_DIDREQUEST
														object:document_
													  userInfo:nil];
		}
	}
}

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{
	SEL tAction;
	
	tAction=[inMenuItem action];
	
	if (tAction==@selector(nothingTitle:))
	{
		NSInteger tSelectedCount;
		
		tSelectedCount=[IBtitleArray_ numberOfSelectedRows];
		
		if (tSelectedCount!=1)
		{
			return NO;
		}
		else
		{
			NSString * tCurrentLanguageName;
			NSString * tTitle;
			
			tCurrentLanguageName=[[cachedTitleLocalizations_ objectAtIndex:[IBtitleArray_ selectedRow]] objectForKey:ICDOCUMENT_LANGUAGE];
			
			tTitle=[inMenuItem title];
			
			if ([tTitle isEqualToString:tCurrentLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[cachedTitleLocalizations_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tTitle]==YES)
						{
							return NO;
						}
					}
				}
			}
		}
	}
	else if (tAction==@selector(nothingDescription:))
	{
		NSInteger tSelectedCount;
		
		tSelectedCount=[IBdescriptionArray_ numberOfSelectedRows];
		
		if (tSelectedCount!=1)
		{
			return NO;
		}
		else
		{
			NSString * tCurrentLanguageName;
			NSString * tTitle;
			
			tCurrentLanguageName=[[cachedDescriptionLocalizations_ objectAtIndex:[IBdescriptionArray_ selectedRow]] objectForKey:ICDOCUMENT_LANGUAGE];
			
			tTitle=[inMenuItem title];
			
			if ([tTitle isEqualToString:tCurrentLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[cachedDescriptionLocalizations_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tTitle]==YES)
						{
							return NO;
						}
					}
				}
			}
		}
	}
	else if (tAction==@selector(switchPackageState:))
	{
		if ([inMenuItem tag]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_UNSELECTED)
		{
			NSMutableDictionary * tOptionsDictionary;
	
			tOptionsDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
		
			// If the choice is invisible, it can't be unselected
			
			if ([[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN] boolValue]==YES)
			{
				if ([inMenuItem tag]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_UNSELECTED /*||
					[inMenuItem tag]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_REQUIRED*/)	// A VOIR : Is it useful?
				{
					return NO;
				}
			}
		}
		else if ([inMenuItem tag]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENT)
		{
			NSMutableDictionary * tMutableDictionary;
			
			//tMutableDictionary=[ICChoiceUtilities availableDependenciesDictionaryForEnabledStateOfGroupNode:cachedTreeNode_];
			
			tMutableDictionary=[ICChoiceUtilities availableDependenciesDictionaryForSelectedStateOfLeafNode:cachedTreeNode_];
			
			if (tMutableDictionary==nil || ([tMutableDictionary count]==0))
			{
				return NO;
			}
		}
		else if ([inMenuItem tag]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_DEPENDENT)
		{
			NSMutableDictionary * tMutableDictionary;
			
			//tMutableDictionary=[ICChoiceUtilities availableDependenciesDictionaryForSelectedStateOfLeafNode:cachedTreeNode_];
			
			tMutableDictionary=[ICChoiceUtilities availableDependenciesDictionaryForEnabledStateOfGroupNode:cachedTreeNode_];
			
			if (tMutableDictionary==nil || ([tMutableDictionary count]==0))
			{
				return NO;
			}
		}
	}
	
	return YES;
}

#pragma mark -

- (void) updateRequirementTabLabel
{
	NSTabViewItem * tTabViewItem;
					
	tTabViewItem=[IBtabView_ tabViewItemAtIndex:[IBtabView_ indexOfTabViewItemWithIdentifier:@"Requirements"]];
	
	if (tTabViewItem!=nil)
	{
		NSUInteger tCount;

		tCount=[cachedRequirements_ count];

		if (tCount>0)
		{
			[tTabViewItem setLabel:[NSString stringWithFormat:NSLocalizedString(@"Requirements (%d)",@""),tCount]];
		}
		else
		{
			[tTabViewItem setLabel:NSLocalizedString(@"Requirements",@"")];
		}
	}
}

- (IBAction) addRequirement:(id) sender
{
	NSMutableDictionary * tMutableDictionary;
	
	// A COMPLETER	(identifier)
	
	tMutableDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],ICDOCUMENT_REQUIREMENT_STATE,
																		 @"fr.whitebox.Packages.requirement.cpu",ICDOCUMENT_REQUIREMENT_IDENTIFIER,
																		 [NSNumber numberWithInt:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR_CHOICE_UNSELECT_AND_DISABLE],ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR,
																		 [NSMutableArray array],ICDOCUMENT_REQUIREMENT_MESSAGE,
																		 nil];

	if (tMutableDictionary!=nil)
	{
		if (requirementController_==nil)
		{
			requirementController_=[ICRequirementChoiceEditorController newController];
		}
	
		if (requirementController_!=nil)
		{
			BOOL tVisible;
			NSDictionary * tOptionsDictionary;
	
			tOptionsDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
	
			tVisible=![[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN] boolValue];
			
			[requirementController_ showEditorForWindow:[document_ windowForSheet] withDictionary:tMutableDictionary visibility:tVisible delegate:self];
		}
	}
}

- (IBAction) editRequirement:(id) sender
{
	NSInteger tSelectedRow;
	
	tSelectedRow=[IBrequirementsArray_ selectedRow];
	
	if (tSelectedRow!=-1)
	{
		if (requirementController_==nil)
		{
			requirementController_=[ICRequirementChoiceEditorController newController];
		}
	
		if (requirementController_!=nil)
		{
			NSMutableDictionary * tRequirementDictionary;
			BOOL tVisible;
			NSDictionary * tOptionsDictionary;
	
			tRequirementDictionary=[cachedRequirements_ objectAtIndex:tSelectedRow];
			
			tOptionsDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
		
			tVisible=![[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN] boolValue];
			
			editRequirementMode_=YES;
						
			[requirementController_ showEditorForWindow:[document_ windowForSheet] withDictionary:tRequirementDictionary visibility:tVisible delegate:self];
		}
	}
}

- (IBAction) removeRequirements:(id) sender
{
	NSString * tAlertTitle;
    
    if ([IBrequirementsArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this requirement?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these requirements?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeRequirementsDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeRequirementsDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBrequirementsArray_ selectedRowIndexes];
		
		[IBrequirementsArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedRequirements_ removeObjectsAtIndexes:tIndexSet];
			
			[IBrequirementsArray_ reloadData];
			
			[IBrequirementsRemoveButton_ setEnabled:NO];
			
			[self updateRequirementTabLabel];
			
			// Notify Document Change
			
			[self noteDocumentChange];
			
			//[ICInstallationTypeUtilities notifyInstallationTypeDidChangeWithSender:self];
		}
	}
}

- (NSString *) uniqueRequirementNameWithBaseName:(NSString *) inBaseName
{
	NSString * tString;
    
	tString=nil;
	
	if (inBaseName!=nil)
	{
		NSUInteger _sIndex=0;
		NSUInteger i,tCount;
    
		tCount=[cachedRequirements_ count];
    
		do
		{
			NSDictionary * tRequirementDictionary;
			
			if (_sIndex>0)
			{
				tString=[[NSString alloc] initWithFormat:@"%@ %lu",inBaseName,_sIndex];
			}
			else
			{
				tString=[[NSString alloc] initWithString:inBaseName];
			}
			
			_sIndex++;
			
			for(i=0;i<tCount;i++)
			{
				tRequirementDictionary=[cachedRequirements_ objectAtIndex:i];
				
				if ([[tRequirementDictionary objectForKey:ICDOCUMENT_REQUIREMENT_NAME] caseInsensitiveCompare:tString]==NSOrderedSame)
				{
					break;
				}
			}
			
			if (i==tCount)
			{
				break;
			}
			
			[tString release];
		}
		while (_sIndex<65535);
    }
	
    return [tString autorelease];
}

- (void) requirementEditionDidEnd:(ICRequirementEditorController *) inEditor returnCode:(NSInteger) inReturnCode
{
	if (inReturnCode==NSOKButton)
	{
		NSMutableDictionary * tRequirementDictionary;
		
		tRequirementDictionary=[inEditor dictionary];
		
		if (tRequirementDictionary!=nil)
		{
			NSInteger tRowIndex;
			NSString * tSuggestedName;
			
			tSuggestedName=[tRequirementDictionary objectForKey:@"SuggestedName"];
			
			if (tSuggestedName!=nil)
			{
				NSString * tUniqueName;
				
				tUniqueName=[self uniqueRequirementNameWithBaseName:tSuggestedName];
				
				[tRequirementDictionary removeObjectForKey:@"SuggestedName"];
				
				[tRequirementDictionary setObject:tUniqueName forKey:ICDOCUMENT_REQUIREMENT_NAME];
			}
			
			if (editRequirementMode_==YES)
			{
				tRowIndex=[IBrequirementsArray_ selectedRow];
				
				if (tRowIndex!=NSNotFound)
				{
					[cachedRequirements_ replaceObjectAtIndex:tRowIndex withObject:tRequirementDictionary];
				}
			}
			else
			{
				[cachedRequirements_ addObject:tRequirementDictionary];
				
				[IBrequirementsArray_ reloadData];
				
				[self updateRequirementTabLabel];
				
				tRowIndex=[cachedRequirements_ count]-1;
				
				[IBrequirementsArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
				[IBrequirementsArray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
			}
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
	}
	
	editRequirementMode_=NO;
}

#pragma mark -

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	NSRect tTitleSectionRect;
	NSRect tDescriptionSectionRect;
	NSRect tExtendedAttributesSectionRect;
	CGFloat tAvailableHeight;
	NSUInteger tLocalizedTitlesCount;
	NSUInteger tLocalizedDescriptionsCount;
	CGFloat tTitleRowHeight;
	CGFloat tDescriptionRowHeight;
	CGFloat tIdealHeightTitleSection;
	CGFloat tIdealHeightDescriptionSection;
	NSSize tTitleSize,tDescriptionSize;
	CGFloat tRatio;
	CGFloat tExtraHeight;
	
	tTitleSectionRect=[IBtitleSection_ frame];
	
	tDescriptionSectionRect=[IBdescriptionSection_ frame];
	
	tExtendedAttributesSectionRect=[IBchoiceExtendedAttributesSection_ frame];
	
	tExtendedAttributesSectionRect.origin.y=-1.0f;
	
	tAvailableHeight=NSHeight([[IBtitleSection_ superview] frame])-NSMaxY(tExtendedAttributesSectionRect)-20.0f;
	
	
	tLocalizedTitlesCount=[cachedTitleLocalizations_ count];
	
	if (tLocalizedTitlesCount<2)
	{
		tLocalizedTitlesCount=2;
	}
	
	tLocalizedDescriptionsCount=[cachedDescriptionLocalizations_ count];
	
	if (tLocalizedDescriptionsCount<2)
	{
		tLocalizedDescriptionsCount=2;
	}
	
	tTitleRowHeight=[IBtitleArray_ rowHeight];
	
	tDescriptionRowHeight=[IBdescriptionArray_ rowHeight];
	
	
	tTitleSize=[IBtitleArray_ intercellSpacing];
	
	tIdealHeightTitleSection=NSHeight(tTitleSectionRect)-NSHeight([IBtitleScrollView_ frame])+tTitleRowHeight*tLocalizedTitlesCount+(tLocalizedTitlesCount-1)*tTitleSize.height+4;
	
	
	tDescriptionSize=[IBdescriptionArray_ intercellSpacing];
	
	tIdealHeightDescriptionSection=NSHeight(tDescriptionSectionRect)-NSHeight([IBdescriptionScrollView_ frame])+tDescriptionRowHeight*tLocalizedDescriptionsCount+(tLocalizedDescriptionsCount-1)*tDescriptionSize.height+4;
	
	
	tRatio=(tDescriptionRowHeight+tDescriptionSize.height)/(tDescriptionRowHeight+tDescriptionSize.height+tTitleRowHeight+tTitleSize.height);
	
	
	if ((tIdealHeightTitleSection+tIdealHeightDescriptionSection)>tAvailableHeight)
	{
		CGFloat tMissingHeight;
		
		tMissingHeight=tIdealHeightTitleSection+tIdealHeightDescriptionSection-tAvailableHeight;
		
		if (tLocalizedTitlesCount>2 && tLocalizedDescriptionsCount>2)
		{
			tIdealHeightDescriptionSection=tIdealHeightDescriptionSection-_CGFloatFloor(tRatio*tMissingHeight);
			
			tIdealHeightTitleSection=tAvailableHeight-tIdealHeightDescriptionSection;
		}
		else
		{
			if (tLocalizedDescriptionsCount>2)
			{
				tIdealHeightDescriptionSection=tIdealHeightDescriptionSection-tMissingHeight;
			}
			else if (tIdealHeightTitleSection>2)
			{
				tIdealHeightTitleSection=tIdealHeightTitleSection-tMissingHeight;
			}
		}
	}
	
	tExtraHeight=tAvailableHeight-(tIdealHeightTitleSection+tIdealHeightDescriptionSection);
	
	if (tExtraHeight>0)
	{
		tDescriptionSectionRect.size.height=tAvailableHeight-tIdealHeightTitleSection;
	}
	else
	{
		tDescriptionSectionRect.size.height=tIdealHeightDescriptionSection;
	}
	
	
	
	[IBchoiceExtendedAttributesSection_ setFrame:tExtendedAttributesSectionRect];
	
	tDescriptionSectionRect.origin.y=NSMaxY(tExtendedAttributesSectionRect);
	
	[IBdescriptionSection_ setFrame:tDescriptionSectionRect];
	
	
	
	tTitleSectionRect.origin.y=NSMaxY(tDescriptionSectionRect);
	tTitleSectionRect.size.height=tAvailableHeight-tDescriptionSectionRect.size.height;
	
	[IBtitleSection_ setFrame:tTitleSectionRect];
	
	[IBview_ setNeedsDisplay:YES];
}

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBtitleArray_)
    {
		[IBtitleAddButton_ setEnabled:![ICLocalizationUtilities allLanguagesUsedInLocalizations:cachedTitleLocalizations_]];
		
		[IBtitleRemoveButton_ setEnabled:([IBtitleArray_ numberOfSelectedRows]!=0)];
	}
	else if ([inNotification object]==IBdescriptionArray_)
    {
		[IBdescriptionAddButton_ setEnabled:![ICLocalizationUtilities allLanguagesUsedInLocalizations:cachedDescriptionLocalizations_]];
		
		[IBdescriptionRemoveButton_ setEnabled:([IBdescriptionArray_ numberOfSelectedRows]!=0)];
	}
	else if ([inNotification object]==IBrequirementsArray_)
    {
		NSInteger tNumberOfSelectedRows;
		
		tNumberOfSelectedRows=[IBrequirementsArray_ numberOfSelectedRows];
		
		[IBrequirementsEditButton_ setEnabled:(tNumberOfSelectedRows==1)];
		
		[IBrequirementsRemoveButton_ setEnabled:(tNumberOfSelectedRows!=0)];
	}
	
}

@end
