/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationInspectorInstallationTypeController.h"
#import "ICInstallationType+Constants.h"

@implementation ICPresentationInspectorInstallationTypeController

+ (ICPresentationInspectorInstallationTypeController *) controller
{
	ICPresentationInspectorInstallationTypeController * nController;
    
    nController=[ICPresentationInspectorInstallationTypeController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICInstallationTypeInspectorView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICInstallationTypeInspectorView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];

	[self installationTypeSelectionDidChange:nil];
	
	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] removeObserver:self name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTIONDIDCHANGE object:[self document]];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(installationTypeSelectionDidChange:) 
												 name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTIONDIDCHANGE
											   object:[self document]];

	[[NSNotificationCenter defaultCenter] removeObserver:self name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_EDIT_DIDREQUEST object:[self document]];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(installationTypeChoiceDependencyEditDidRequest:) 
												 name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_EDIT_DIDREQUEST
											   object:[self document]];
											   
	[[NSNotificationCenter defaultCenter] removeObserver:self name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_ENDEDIT_DIDREQUEST object:[self document]];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(installationTypeChoiceDependencyEndEditDidRequest:) 
												 name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_ENDEDIT_DIDREQUEST
											   object:[self document]];

}

- (void) dealloc
{
	//NSLog(@"[ICPresentationInspectorInstallationTypeController dealloc]");
	
	[emptySelectionController_ release];
	
	[choiceSelectionController_ release];
	
	[choiceDependencyController_ release];
	
	[cachedSelectionDictionary_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) installationTypeSelectionDidChange:(NSNotification *) inNotification
{
	NSDictionary * tUserInfoDictionary;
	
	if (inNotification!=nil)
	{
		tUserInfoDictionary=[inNotification userInfo];
	}
	else
	{
		tUserInfoDictionary=[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedStringFromTable(@"Empty Selection",@"Presentation_InstallationType",@"No comment"),ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTION_LABEL,nil];
	}
	
	if (tUserInfoDictionary!=cachedSelectionDictionary_ || tUserInfoDictionary==nil)
	{
		if (currentController_!=nil)
		{
			if ([currentController_ viewCanBeRemoved:self]==NO)
			{
				return;
			}
			
			[currentController_ viewWillBeRemoved:self];
			
			if (currentView_!=nil)
			{
				[currentView_ removeFromSuperview];
			}
		}
		
		[cachedSelectionDictionary_ release];
		
		cachedSelectionDictionary_=[tUserInfoDictionary retain];
		
		currentController_=nil;
		
		if ([cachedSelectionDictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTION]==nil)
		{
			currentController_=emptySelectionController_;
			
			[emptySelectionController_ setLabel:[cachedSelectionDictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTION_LABEL]];
		}
		else
		{
			currentController_=choiceSelectionController_;
		}
		
		if (currentController_!=nil)
		{
			currentView_=[currentController_ view];
		
			[currentView_ setFrame:[IBview_ bounds]];
			
			// Initialize View widgets
	
			[currentController_ initializeWithDictionary:cachedSelectionDictionary_ forDocument:[self document]];
	
			[IBview_ addSubview:currentView_];
		}
		else
		{
			NSLog(@"[ICPresentationInspectorInstallationTypeController installationTypeSelectionDidChange]: No controller found");
		}
		
	}
}

- (void) appleModeDidChange:(NSNotification *) inNotification
{
	// A COMPLETER
}

- (void) installationTypeChoiceDependencyEditDidRequest:(NSNotification *) inNotification
{
	if (cachedSelectionDictionary_!=nil)
	{
		if (currentController_!=nil)
		{
			if ([currentController_ viewCanBeRemoved:self]==NO)
			{
				return;
			}
			
			[currentController_ viewWillBeRemoved:self];
			
			if (currentView_!=nil)
			{
				[currentView_ removeFromSuperview];
			}
		}
		
		currentController_=choiceDependencyController_;
		
		currentView_=[currentController_ view];
		
		[currentView_ setFrame:[IBview_ bounds]];
		
		// Initialize View widgets

		[currentController_ initializeWithDictionary:cachedSelectionDictionary_ forDocument:[self document]];

		[IBview_ addSubview:currentView_];
		
		// Post Notification to hide the source list
		
		[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_DOCUMENT_SOURCELIST_SHOULD_HIDE
														object:[self document]
													  userInfo:nil];
													  
		// Post Notification to enlarge the Inspector zone
		
		[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_ENLARGE_INSPECTOR
														object:[self document]
													  userInfo:nil];
	}
}

- (void) installationTypeChoiceDependencyEndEditDidRequest:(NSNotification *) inNotification
{
	if (cachedSelectionDictionary_!=nil)
	{
		// Post Notification to reduce the Inspector zone
		
		[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_REDUCE_INSPECTOR
														object:[self document]
													  userInfo:nil];
		
		// Post Notification to show the source list
		
		[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_DOCUMENT_SOURCELIST_SHOULD_SHOW
														object:[self document]
													  userInfo:nil];
		
		if (currentController_!=nil)
		{
			if ([currentController_ viewCanBeRemoved:self]==NO)
			{
				return;
			}
			
			[currentController_ viewWillBeRemoved:self];
			
			if (currentView_!=nil)
			{
				[currentView_ removeFromSuperview];
			}
		}
		
		currentController_=choiceSelectionController_;
		
		currentView_=[currentController_ view];
		
		[currentView_ setFrame:[IBview_ bounds]];
		
		// Initialize View widgets

		[currentController_ initializeWithDictionary:cachedSelectionDictionary_ forDocument:[self document]];

		[IBview_ addSubview:currentView_];
	}
}

@end
