/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationInspectorInstallerPluginController.h"

#import "ICArchitectureUtilities.h"

#import "ICDocument.h"

@implementation ICPresentationInspectorInstallerPluginController

+ (ICPresentationInspectorInstallerPluginController *) controller
{
	ICPresentationInspectorInstallerPluginController * nController;
    
    nController=[ICPresentationInspectorInstallerPluginController alloc];
    
	if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICInstallerPluginInspectorView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICInstallerPluginInspectorView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
	
    return [nController autorelease];
}

#pragma mark -

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];

	dictionary_=inDictionary;
	
	if (dictionary_==nil)
	{
		// A COMPLETER
	}
	else
	{
		NSWindow * tWindow;
		
		[dictionary_ retain];
		
		[self refreshUI];
		
		// Register for Notifications
		
		tWindow=[[self document] windowForSheet];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidBecomeMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(windowStateDidChange:) 
												 name:NSWindowDidBecomeMainNotification 
											   object:tWindow];
	
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidResignMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidResignMainNotification 
												   object:tWindow];
	}
}

- (void) awakeFromNib
{
	dateFormatter_=[NSDateFormatter new];
	
	if (dateFormatter_!=nil)
	{
		[dateFormatter_ setFormatterBehavior:NSDateFormatterBehavior10_4];
	
		[dateFormatter_ setDateStyle:NSDateFormatterMediumStyle];
		
		[dateFormatter_ setTimeStyle:NSDateFormatterShortStyle];
	}
}

- (void) refreshUI
{
	NSString * tPath;
	NSInteger tPathType;
	static NSImage * sPluginIcon=nil;
	
	// Icon
	
	if (sPluginIcon==nil)
	{
		sPluginIcon=[[NSImage alloc] initWithContentsOfFile:@"/System/Library/CoreServices/CoreTypes.bundle/Contents/Resources/KEXT.icns"];
	}
	
	[IBfileIcon_ setImage:sPluginIcon];
	
	cachedPathTypeIndex_=0;
	
	tPathType=[[dictionary_ objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
	
	tPath=[dictionary_ objectForKey:ICDOCUMENT_PATH];
	
	if (tPath==nil)
	{
		// Name
		
		[IBfileBigName_ setStringValue:@"-"];
		
		// Last Modification Date
		
		[IBfileLastModifiedDate_ setStringValue:@"-"];
		
		// Reference
		
		[IBpathType_ selectItemWithTag:tPathType];
		
		[IBpathType_ setEnabled:NO];
		
		// Path
		
		[IBpath_ setStringValue:@""];
		
		// Open Project
		
		[IBopenProjectButton_ setHidden:YES];
	}
	else
	{
		NSFileManager * tFileManager;
		NSString * tAbsolutePath;
		
		// Name
		
		[IBfileBigName_ setStringValue:[[tPath lastPathComponent] stringByDeletingPathExtension]];
		
		// Reference
		
		[IBpathType_ selectItemWithTag:tPathType];
		
		cachedPathTypeIndex_=[IBpathType_ indexOfItemWithTag:tPathType];
		
		// Path
		
		[IBpath_ setStringValue:tPath];
		
		// Check that the plugin exists at that path
		
		if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
		}
		else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
		{
			tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
		}
		else
		{
			tAbsolutePath=tPath;
		}
		
		tFileManager=[NSFileManager defaultManager];
		
		if (tFileManager!=nil)
		{
			BOOL isDirectory;
		
			if ([tFileManager fileExistsAtPath:tAbsolutePath isDirectory:&isDirectory]==NO || isDirectory==NO)
			{
				// Last Modification Date
		
				[IBfileLastModifiedDate_ setStringValue:@"-"];
			
				[IBpathType_ setEnabled:NO];
				
				[IBpath_ setTextColor:[NSColor redColor]];
				
				[IBopenProjectButton_ setHidden:YES];
			}
			else
			{
				NSDictionary * tAttributesDictionary;
				NSBundle * tBundle;

				// Last Modification Date
				
				tAttributesDictionary=[tFileManager fileAttributesAtPath:tAbsolutePath traverseLink:NO];
						
				if (tAttributesDictionary!=nil)
				{
					NSDate * tModificationDate;
					
					tModificationDate=[tAttributesDictionary objectForKey:NSFileModificationDate];
					
					if (tModificationDate!=nil)
					{
						if (dateFormatter_!=nil)
						{
							[IBfileLastModifiedDate_ setStringValue:[dateFormatter_ stringForObjectValue:tModificationDate]];
						}
						else
						{
							[IBfileLastModifiedDate_ setObjectValue:tModificationDate];
						}
					}
				}
				
				// Architectures
				
				[IBfileArchitecturesLabel_ setStringValue:NSLocalizedString(@"Architectures:",@"")];
				
				[IBfileArchitectures_ setStringValue:@"-"];

				tBundle=[NSBundle bundleWithPath:tAbsolutePath];
				
				if (tBundle!=nil)
				{
					NSArray * tArchitecturesArray;
					
					tArchitecturesArray=[ICArchitectureUtilities architecturesOfFile:[tBundle executablePath]];
				
					if (tArchitecturesArray!=nil)
					{
						// Label
						
						if ([tArchitecturesArray count]==1)
						{
							[IBfileArchitecturesLabel_ setStringValue:NSLocalizedString(@"Architecture:",@"")];
						}
						
						[IBfileArchitectures_ setStringValue:[tArchitecturesArray componentsJoinedByString:@" | "]];
					}
				}
				
				[IBpathType_ setEnabled:YES];
				
				[IBpath_ setTextColor:[NSColor blackColor]];
				
				// Check if we can find the Xcode project
				
				// A COMPLETER
			}
		}
	}
}

#pragma mark -

- (BOOL)validateMenuItem:(NSMenuItem *) inMenuItem
{
    SEL tAction=[inMenuItem action];
        
    if (tAction==@selector(revealInFinder:))
    {
        NSString * tPath;
		NSInteger tPathType;
		NSFileManager * tFileManager;
		
		tPathType=[[dictionary_ objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
		
		tPath=[dictionary_ objectForKey:ICDOCUMENT_PATH];
		
		if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
		{
			tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
		}
		else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
		{
			tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
		}	
		
		tFileManager=[NSFileManager defaultManager];
		
		if (tFileManager!=nil && tPath!=nil)
		{
			BOOL isDirectory;
		
			if ([tFileManager fileExistsAtPath:tPath isDirectory:&isDirectory]==YES)
			{
				return YES;
			}
		}
		
		return NO;
    }
    
    return YES;
}


- (IBAction) switchPathType:(id) sender
{
	if (cachedPathTypeIndex_!=[IBpathType_ indexOfSelectedItem])
    {
		NSString * tPath;
		
		tPath=[dictionary_ objectForKey:ICDOCUMENT_PATH];
		
		if (tPath!=nil)
		{
			NSInteger tPathType;
			
			tPathType=[[IBpathType_ itemAtIndex:cachedPathTypeIndex_] tag];
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
			}
			
			tPathType=[[IBpathType_ selectedItem] tag];
			
			cachedPathTypeIndex_=[IBpathType_ indexOfSelectedItem];
			
			[dictionary_ setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tPath=[tPath stringByRelativizingToPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tPath=[tPath stringByRelativizingToPath:[document_ referenceFolderPath]];
			}
			
			[dictionary_ setObject:tPath forKey:ICDOCUMENT_PATH];
			
			[IBpath_ setStringValue:tPath];
		}
		
		// Notify Document Change
		
		[self noteDocumentChange];
	}
}

- (IBAction) revealInFinder:(id) sender
{
	NSString * tPath;
	NSInteger tPathType;
	
	tPathType=[[dictionary_ objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
	
	tPath=[dictionary_ objectForKey:ICDOCUMENT_PATH];
	
	if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
	{
		tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
	}
	else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
	{
		tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
	}
			
	[[NSWorkspace sharedWorkspace] selectFile:tPath inFileViewerRootedAtPath:@""];
}

- (IBAction) openProject:(id) sender
{
	// A COMPLETER
}

#pragma mark -

- (void) windowStateDidChange:(NSNotification *) inNotification
{
	[self refreshUI];
}

@end
