/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationInspectorLicenseController.h"

#import "ICPreferences+Constants.h"

#import "ICLicenseUtilities.h"
#import "ICLocalizationUtilities.h"
#import "NSString+Iceberg.h"
#import "ICDocument.h"
#import "NSFileManager+fileTypes.h"

#import "ICLanguagePopupButtonCell.h"
#import "ICPathReferencePopupButtonCell.h"
#import "ICOffsetedPopupButtonCell.h"

#import "ICLicenseProvider.h"

@implementation ICPresentationInspectorLicenseController

+ (ICPresentationInspectorLicenseController *) controller
{
	ICPresentationInspectorLicenseController * nController;
    
    nController=[ICPresentationInspectorLicenseController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICLicenseInspectorView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICLicenseInspectorView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) dealloc
{
	[IBdocumentsView_ release];
	
	[IBkeywordsView_ release];

	[cachedKeywordsValues_ release];
	
	[cachedLocalizations_ release];

	[super dealloc];
}

#pragma mark -

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	[cachedKeywordsValues_ release];
	
	cachedKeywordsValues_=nil;
	
	[cachedLocalizations_ release];
	
	cachedKeywordsValues_=nil;
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE];
	
	if (dictionary_==nil)
	{
		// Create default License dictionary
		
		dictionary_=[ICLicenseUtilities defaultLicenseDictionary];
	
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE];
		}
	}
	
	if (dictionary_!=nil)
	{
		[dictionary_ retain];
		
		// Cached Localizations
		
		cachedLocalizations_=[dictionary_ objectForKey:ICDOCUMENT_LOCALIZATIONS];
		
		if (cachedLocalizations_==nil)
		{
			cachedLocalizations_=[NSMutableArray array];
			
			if (cachedLocalizations_!=nil)
			{
				[dictionary_ setObject:cachedLocalizations_ forKey:ICDOCUMENT_LOCALIZATIONS];
			}
			else
			{
				// A COMPLETER
			}
		}
		
		if (cachedLocalizations_!=nil)
		{
			[cachedLocalizations_ retain];
		}
		
		cachedKeywordsValues_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_KEYWORDS];
		
		if (cachedKeywordsValues_==nil)
		{
			cachedKeywordsValues_=[NSMutableDictionary dictionary];
			
			if (cachedKeywordsValues_!=nil)
			{
				[dictionary_ setObject:cachedKeywordsValues_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_KEYWORDS];
			}
			else
			{
				// A COMPLETER
			}
		}
		
		if (cachedKeywordsValues_!=nil)
		{
			[cachedKeywordsValues_ retain];
		}
		
		[IBdocumentsArray_ deselectAll:self];
		
		[IBkeywordsArray_ deselectAll:self];
					
		[self upgradeLicenseSettings];
		
		// Register for Notifications
	
		[[NSNotificationCenter defaultCenter] removeObserver:self name:ICPRESENTATION_NOTIFICATION_LICENSE_SETTINGS_DID_CHANGE object:[self document]];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(licenseSettingsDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_LICENSE_SETTINGS_DID_CHANGE 
												   object:[self document]];
	}
}

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn = nil;
    NSMutableArray * tTemplateNames;
	NSTextFieldCell * tPrototypeTextCell=nil;
	
    fileManager_=[NSFileManager defaultManager];
	
	// Populate the Show popup button
	
	tTemplateNames=[[[ICLicenseProvider defaultProvider] allLicenses] mutableCopy];
	
	if (tTemplateNames!=nil)
	{
		NSUInteger i,tCount;
		
		[tTemplateNames sortUsingSelector:@selector(caseInsensitiveCompare:)];
		
		[IBshowPopupButton_ addItemsWithTitles:tTemplateNames];
		
		[tTemplateNames release];
		
		tCount=[IBshowPopupButton_ numberOfItems];
		
		for(i=3;i<tCount;i++)
		{
			NSMenuItem * tMenuItem;
		
			tMenuItem=[IBshowPopupButton_ itemAtIndex:i];
			
			[tMenuItem setTag:ICLICENSE_MODE_TEMPLATE];
		}
	}
	
	// Language
    
    tTableColumn = [IBdocumentsArray_ tableColumnWithIdentifier:ICDOCUMENT_LANGUAGE];
    
	if (tTableColumn!=nil)
	{
		ICLanguagePopupButtonCell * tPopupButtonCell = nil;
    
		tPopupButtonCell=[[ICLanguagePopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
		
		if (tPopupButtonCell!=nil)
		{
			[tPopupButtonCell setControlSize:NSSmallControlSize];
			
			[tPopupButtonCell setMenu:[ICLocalizationUtilities localizationMenu]];
			
			[tPopupButtonCell setBordered:NO];
			
			[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
			
			[tPopupButtonCell setAction:@selector(nothing:)];
			
			[tPopupButtonCell setTarget:self];
			
			[tTableColumn setDataCell:tPopupButtonCell];
			
			[tPopupButtonCell release];
		}
	}
	
	// Path Reference Type
	
	tTableColumn = [IBdocumentsArray_ tableColumnWithIdentifier: ICDOCUMENT_PATH_TYPE];
    
	if (tTableColumn!=nil)
	{
		ICPathReferencePopupButtonCell * tPopupButtonCell = nil;
		
		tPopupButtonCell=[[ICPathReferencePopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
		
		if (tPopupButtonCell!=nil)
		{
			NSMenuItem * tMenuItem;
			
			[tPopupButtonCell setControlSize:NSSmallControlSize];
			
			[tPopupButtonCell addItemWithTitle:NSLocalizedString(@"Relative to Reference Folder",@"No comment")];
			
			tMenuItem=[tPopupButtonCell itemAtIndex:0];
			[tMenuItem setImage:[NSImage imageNamed:@"Reference13.tif"]];
			[tMenuItem setTag:ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER];
			
			
			[tPopupButtonCell addItemWithTitle:NSLocalizedString(@"Relative to Project",@"No comment")];
			
			tMenuItem=[tPopupButtonCell itemAtIndex:1];
			
			[tMenuItem setImage:[NSImage imageNamed:@"Relative13.tif"]];
			[tMenuItem setTag:ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT];
			
			[tPopupButtonCell addItemWithTitle:NSLocalizedString(@"Absolute Path",@"No comment")];
			
			tMenuItem=[tPopupButtonCell itemAtIndex:2];
			
			[tMenuItem setImage:[NSImage imageNamed:@"Absolute13.tif"]];
			[tMenuItem setTag:ICDOCUMENT_PATH_TYPE_GLOBAL];
			
			[tPopupButtonCell setBordered:NO];
			
			[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
			
			[tTableColumn setDataCell:tPopupButtonCell];
			
			[tPopupButtonCell release];
		}
	}
	
	[IBdocumentsArray_ registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	
	// Keywords Array
	
	tTableColumn = [IBkeywordsArray_ tableColumnWithIdentifier: @"Keyword"];
	
	tPrototypeTextCell=[tTableColumn dataCell];
	
	if (tPrototypeTextCell!=nil)
	{
		[tPrototypeTextCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	}
	
	tTableColumn = [IBkeywordsArray_ tableColumnWithIdentifier: @"Value"];
	
	tPrototypeTextCell=[tTableColumn dataCell];
	
	if (tPrototypeTextCell!=nil)
	{
		[tPrototypeTextCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	}
}

#pragma mark -

- (IBAction) switchLicenseValue:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	NSString * tTemplateName;
	BOOL tHasBeenModified=NO;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_MODE];
	
	if (tNumber==nil || [tNumber integerValue]!=tTag)
	{
		[dictionary_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_MODE];
		
		if (tTag==ICLICENSE_MODE_TEMPLATE)
		{
			// Get the name of the template
				
			tTemplateName=[sender titleOfSelectedItem];
				
			[dictionary_ setObject:tTemplateName forKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_NAME];
		}
		
		tHasBeenModified=YES;
	}
	else if (tTag==ICLICENSE_MODE_TEMPLATE)
	{
		tTemplateName=[sender titleOfSelectedItem];
		
		if ([[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_NAME] caseInsensitiveCompare:tTemplateName]!=NSOrderedSame)
		{
			[dictionary_ setObject:tTemplateName forKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_NAME];
			
			tHasBeenModified=YES;
		}
	}
	
	if (tHasBeenModified==YES)
	{
		[self upgradeLicenseSettings];
		
		// Notify Document Change
			
		[self noteDocumentChange];
			
		[ICLicenseUtilities notifyLicenseDidChangeWithSender:self];
		
		[IBwarningLabel_ setHidden:[self sameFileTypeForAllLocalizations]];
	}
}

- (void) upgradeLicenseSettings
{
	NSNumber * tNumber;
	NSString * tTemplateName;
	NSInteger tIndex=4;
	
	[cachedKeywords_ release];
	
	cachedKeywords_=nil;
	
	// Initialize the controls
		
	// License Mode
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_MODE];
	
	cachedLicenseMode_=ICLICENSE_MODE_CUSTOM;
	
	if (tNumber!=nil)
	{
		cachedLicenseMode_=[tNumber intValue];
	}
	
	if (currentView_!=nil)
	{
		[currentView_ removeFromSuperview];
		
		currentView_=nil;
	}
	
	[cachedKeywords_ release];
	
	cachedKeywords_=nil;
		
	switch(cachedLicenseMode_)
	{
		case ICLICENSE_MODE_CUSTOM:
		
			[IBshowPopupButton_ selectItemWithTag:0];
			
			[IBdocumentsArray_ deselectAll:self];
			
			currentView_=IBdocumentsView_;
		
			[IBaddButton_ setEnabled:YES];
			
			[IBremoveButton_ setEnabled:NO];
		
			break;
			
		case ICLICENSE_MODE_TEMPLATE:
		
			currentView_=IBkeywordsView_;
			
			tTemplateName=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_NAME];
				
			if (tTemplateName!=nil)
			{
				tIndex=[IBshowPopupButton_ indexOfItemWithTitle:tTemplateName];
				
				// Get the Keywords for the selected Template
			
				cachedKeywords_=[[[ICLicenseProvider defaultProvider] licenseKeywordsForLicenseWithName:tTemplateName] mutableCopy];
				
				if (cachedKeywords_!=nil)
				{
					[cachedKeywords_ sortUsingSelector:@selector(caseInsensitiveCompare:)];
				}
			}
			
			[IBshowPopupButton_ selectItemAtIndex:tIndex];
		
			[IBkeywordsArray_ deselectAll:self];

			
			break;
	}
	
	if (currentView_!=nil)
	{
		[currentView_ setFrame:[IBbottomView_ bounds]];
		
		[IBbottomView_ addSubview:currentView_];
	}
	
	switch(cachedLicenseMode_)
	{
		case ICLICENSE_MODE_CUSTOM:
		
			[IBdocumentsArray_ reloadData];
			
			[IBwarningLabel_ setHidden:[self sameFileTypeForAllLocalizations]];
		
			break;
			
		case ICLICENSE_MODE_TEMPLATE:
		
			[IBkeywordsArray_ reloadData];
		
			break;
	}
}

- (BOOL) sameFileTypeForAllLocalizations
{
#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5
	
	return YES;
	
#else
	
	if (cachedLicenseMode_==ICLICENSE_MODE_CUSTOM)
	{
		if (cachedLocalizations_!=nil && [cachedLocalizations_ count]>1)
		{
			NSUInteger tIndex,tCount;
			NSDictionary * tLocalizationDictionary;
			NSString * tUTITemplate=nil;
			
			tCount=[cachedLocalizations_ count];
			
			for(tIndex=0;tIndex<tCount;tIndex++)
			{
				NSDictionary * tPathDictionary;
				
				tLocalizationDictionary=[cachedLocalizations_ objectAtIndex:tIndex];
				
				tPathDictionary=[tLocalizationDictionary objectForKey:ICDOCUMENT_VALUE];
				
				if (tPathDictionary!=nil)
				{
					NSString * tAbsolutePath;
					
					tAbsolutePath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
					
					if (tAbsolutePath!=nil)
					{
						NSInteger tPathType;
						
						tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
						
						if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
						{
							tAbsolutePath=[tAbsolutePath stringByAbsolutingWithPath:[document_ folder]];
						}
						else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
						{
							tAbsolutePath=[tAbsolutePath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
						}
					}
					
					if (tAbsolutePath!=nil && [tAbsolutePath length]>0)
					{
						if ([fileManager_ fileExistsAtPath:tAbsolutePath]==YES)
						{
							NSString * tUTIType;
							NSError * tError;
							
							tUTIType=[[NSWorkspace sharedWorkspace] typeOfFile:tAbsolutePath error:&tError];
							
							if (tUTIType!=nil)
							{
								tUTITemplate=tUTIType;
								
								break;
							}
						}
					}
				}
			}
			
			for(;tIndex<tCount;tIndex++)
			{
				NSDictionary * tPathDictionary;
				
				tLocalizationDictionary=[cachedLocalizations_ objectAtIndex:tIndex];
				
				tPathDictionary=[tLocalizationDictionary objectForKey:ICDOCUMENT_VALUE];
				
				if (tPathDictionary!=nil)
				{
					NSString * tAbsolutePath;
					
					tAbsolutePath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
					
					if (tAbsolutePath!=nil)
					{
						NSInteger tPathType;
						
						tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
						
						if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
						{
							tAbsolutePath=[tAbsolutePath stringByAbsolutingWithPath:[document_ folder]];
						}
						else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
						{
							tAbsolutePath=[tAbsolutePath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
						}
					}
					
					if (tAbsolutePath!=nil && [tAbsolutePath length]>0)
					{
						if ([fileManager_ fileExistsAtPath:tAbsolutePath]==YES)
						{
							NSString * tUTIType;
							NSError * tError;
							
							tUTIType=[[NSWorkspace sharedWorkspace] typeOfFile:tAbsolutePath error:&tError];
							
							if (tUTIType!=nil)
							{
								if (UTTypeEqual((CFStringRef) tUTIType,(CFStringRef) tUTITemplate)==FALSE)
								{
									return NO;
								}
							}
						}
					}
				}
			}
		}
	}
	
	return YES;
	
#endif
}

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
    SEL tAction=[inMenuItem action];
		
	if (tAction==@selector(nothing:))
	{
		NSInteger tSelectedCount;
		
		tSelectedCount=[IBdocumentsArray_ numberOfSelectedRows];
		
		if (tSelectedCount!=1)
		{
			return NO;
		}
		else
		{
			NSString * tCurrentLanguageName;
			NSString * tTitle;
			
			tCurrentLanguageName=[[cachedLocalizations_ objectAtIndex:[IBdocumentsArray_ selectedRow]] objectForKey:ICDOCUMENT_LANGUAGE];
			
			tTitle=[inMenuItem title];
			
			if ([tTitle isEqualToString:tCurrentLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[cachedLocalizations_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tTitle]==YES)
						{
							return NO;
						}
					}
				}
			}
		}
	}
	else if (tAction==@selector(switchLicenseValue:))
    {
		if ([inMenuItem tag]==-1)
		{
			return NO;
		}
    }
    
    return YES;
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (inTableView==IBdocumentsArray_)
	{
		if (cachedLocalizations_!=nil)
		{
			return [cachedLocalizations_ count];
		}
	}
	else if (inTableView==IBkeywordsArray_)
	{
		if (cachedKeywords_!=nil)
		{
			return [cachedKeywords_ count];
		}
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	
	if (inTableView==IBkeywordsArray_)
	{
		if (cachedKeywords_!=nil)
		{
			NSString * tKey;
			
			tKey=[cachedKeywords_ objectAtIndex:inRowIndex];
		
			tColumnIdentifier=[inTableColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:@"Keyword"])
			{
				return tKey;
			}
			else if ([tColumnIdentifier isEqualToString:@"Value"])
			{
				return [cachedKeywordsValues_ objectForKey:tKey];
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	
	if (inTableView==IBdocumentsArray_)
	{
		if (cachedLocalizations_!=nil)
		{
			NSMutableDictionary * tDictionary;
			BOOL tDidChange=NO;
			
			tColumnIdentifier=[inTableColumn identifier];
		
			tDictionary=[cachedLocalizations_ objectAtIndex:inRowIndex];
				
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
			{
				NSString * tLanguageName;
				NSUInteger tSelectionIndex;
				
				// Check that the Language is different
				
				tLanguageName=[[inTableColumn dataCellForRow:inRowIndex] itemTitleAtIndex:[object intValue]];
				
				if ([[tDictionary objectForKey:ICDOCUMENT_LANGUAGE] isEqualToString:tLanguageName]==NO)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[cachedLocalizations_ objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSDictionary * tLocalizationDictionary;
						
						while (tLocalizationDictionary=[tEnumerator nextObject])
						{
							NSString * tExistingLanguage;
							
							tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
							
							if ([tExistingLanguage isEqualToString:tLanguageName]==YES)
							{
								NSBeep();
								
								return;
							}
						}
					}
					
					[tDictionary setObject:tLanguageName forKey:ICDOCUMENT_LANGUAGE];
					
					[IBdocumentsArray_ deselectAll:self];
					
					[cachedLocalizations_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
					
					[IBdocumentsArray_ reloadData];
					
					tSelectionIndex=[cachedLocalizations_ indexOfObject:tDictionary];
					
					if (tSelectionIndex!=NSNotFound)
					{
						[IBdocumentsArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tSelectionIndex] byExtendingSelection:NO];
					}
					
					tDidChange=YES;
				}
			}
			else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_PATH_TYPE])
			{
				NSInteger tPathType;
				NSInteger tOldPathType;
				NSMutableDictionary * tPathDictionary;
				
				// Check that the Path Type is different
				
				tPathDictionary=[tDictionary objectForKey:ICDOCUMENT_VALUE];
				
				tPathType=[[[inTableColumn dataCellForRow:inRowIndex] itemAtIndex:[object intValue]] tag];
				
				tOldPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
				
				if (tOldPathType!=tPathType)
				{
					NSString * tPath;
					
					tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
					
					if (tOldPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
					}
					else if (tOldPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
					}
					
					[tPathDictionary setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
					
					if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						tPath=[tPath stringByRelativizingToPath:[document_ folder]];
					}
					else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						tPath=[tPath stringByRelativizingToPath:[document_ referenceFolderPath]];
					}
					
					[tPathDictionary setObject:tPath forKey:ICDOCUMENT_PATH];
					
					tDidChange=YES;
				}
			}
			
			if (tDidChange==YES)
			{
				[self noteDocumentChange];
				
				// Notify the modification to the simulator
				
				[ICLicenseUtilities notifyLicenseDidChangeWithSender:self];
			}
		}
	}
	else if (inTableView==IBkeywordsArray_)
	{
		if (cachedKeywordsValues_!=nil)
		{
			tColumnIdentifier=[inTableColumn identifier];
		
			if ([tColumnIdentifier isEqualToString:@"Value"]==YES)
			{
				NSString * tOldValue;
            
				tOldValue=[cachedKeywordsValues_ objectForKey:[cachedKeywords_ objectAtIndex:inRowIndex]];
            
				if (tOldValue==nil || [tOldValue isEqualToString:object]==NO)
				{
					[cachedKeywordsValues_ setObject:object forKey:[cachedKeywords_ objectAtIndex:inRowIndex]];
					
					[self noteDocumentChange];
				
					// Notify the modification to the simulator
				
					[ICLicenseUtilities notifyLicenseDidChangeWithSender:self];
				}
			}
		}
	}
}

- (id) tableView:(NSTableView *) inTableView column:(NSTableColumn *) inColumn dataCellForRow:(NSInteger) inRowIndex
{
	if (inTableView==IBdocumentsArray_)
	{
		if (inRowIndex>=0 && cachedLocalizations_!=nil)
		{
			ICOffsetedPopupButtonCell * tPopupButtonCell;
				
			tPopupButtonCell=[[ICOffsetedPopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
			
			if (tPopupButtonCell!=nil)
			{
				NSMenu * nPathMenu=nil;
	
				[tPopupButtonCell setBordered:NO];
				
				[tPopupButtonCell setControlSize:NSSmallControlSize];
				
				[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
				
				nPathMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
	
				if (nPathMenu!=nil)
				{
					id tMenuItem;
					NSDictionary * tDictionary;
					NSString * tFileDocumentName=@"-";
					BOOL tFileExists=YES;
					
					tDictionary=[cachedLocalizations_ objectAtIndex:inRowIndex];
				
					if (tDictionary!=nil)
					{
						NSDictionary * tPathDictionary;
						
						tPathDictionary=[tDictionary objectForKey:ICDOCUMENT_VALUE];
						
						if (tPathDictionary!=nil)
						{
							NSString * tFileDocumentPath;
							
							tFileDocumentPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
							
							if (tFileDocumentPath!=nil)
							{
								tFileDocumentName=[[tFileDocumentPath lastPathComponent] stringByDeletingPathExtension];
							}
							
							if (tFileDocumentName==nil || [tFileDocumentName length]==0)
							{
								tFileDocumentName=@"-";
							}
							else
							{
								// Check that the file exists
			
								if (tFileDocumentPath!=nil)
								{
									NSInteger tPathType;
								
									tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
									
									if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
									{
										tFileDocumentPath=[tFileDocumentPath stringByAbsolutingWithPath:[document_ folder]];
									}
									else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
									{
										tFileDocumentPath=[tFileDocumentPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
									}
									
									tFileExists=NO;
									
									if (tFileDocumentPath!=nil)
									{
										tFileExists=[fileManager_ fileExistsAtPath:tFileDocumentPath];
									}
								}
							}
						}
					}
					
					// Path
					
					tMenuItem=[[NSMenuItem alloc] initWithTitle:tFileDocumentName
														 action:nil
												  keyEquivalent:@""];
												  
					if (tMenuItem!=nil)
					{
						if (tFileExists==NO)
						{
							NSAttributedString * tAttributedString;
							
							tAttributedString=[[NSAttributedString alloc] initWithString:tFileDocumentName attributes:[NSDictionary dictionaryWithObjectsAndKeys:[NSColor redColor],NSForegroundColorAttributeName,nil]];
						
							if (tAttributedString!=nil)
							{
								[tMenuItem setAttributedTitle:tAttributedString];
							
								[tAttributedString release];
							}
						}
						else
						{
							if ([tFileDocumentName isEqualToString:@"-"]==NO)
							{
								NSMenu * tSubMenu;
								
								tSubMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
    
								if (tSubMenu!=nil)
								{
									NSMenuItem * tSubMenuItem;
									
									tSubMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Open With Finder",@"No comment")
																		 action:@selector(openWithFinder:)
																  keyEquivalent:@""];
								
								
									if (tSubMenuItem!=nil)
									{
										[tSubMenuItem setTarget:self];
										
										[tSubMenu addItem:tSubMenuItem];
						
										[tSubMenuItem release];
									}
									
									tSubMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Reveal in Finder",@"No comment")
																		 action:@selector(revealInFinder:)
																  keyEquivalent:@""];
								
									if (tSubMenuItem!=nil)
									{
										[tSubMenuItem setTarget:self];
										
										[tSubMenu addItem:tSubMenuItem];
						
										[tSubMenuItem release];
									}
									
									[tMenuItem setSubmenu:tSubMenu];
									
									[tSubMenu release];
								}
							}
						}
						
						[nPathMenu addItem:tMenuItem];
        
						[tMenuItem release];
					}
					
					// Separator
					
					[nPathMenu addItem:[NSMenuItem separatorItem]];
					
					// Choose...
					
					tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Choose...",@"No comment")
														 action:@selector(chooseLocalizedLicensePath:)
												  keyEquivalent:@""];
												  
					if (tMenuItem!=nil)
					{
						[tMenuItem setTarget:self];
						
						[nPathMenu addItem:tMenuItem];
		
						[tMenuItem release];
					}
					
					[tPopupButtonCell setMenu:nPathMenu];
					
					[nPathMenu release];
				}

				return [tPopupButtonCell autorelease];
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{    
    if (inTableView==IBdocumentsArray_)
	{
		if (cachedLocalizations_!=nil)
		{
			NSString * tColumnIdentifier;
			NSDictionary * tDictionary;
			
			tDictionary=[cachedLocalizations_ objectAtIndex:inRowIndex];
				
			tColumnIdentifier=[inTableColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
			{
				[cell selectItemWithTitle:[tDictionary objectForKey:ICDOCUMENT_LANGUAGE]];
			}
			else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_PATH_TYPE])
			{
				NSDictionary * tPathDictionary;
				
				tPathDictionary=[tDictionary objectForKey:ICDOCUMENT_VALUE];
				
				if(tPathDictionary!=nil)
				{
					[cell selectItemWithTag:[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue]];
				}
			}
			else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_PATH])
			{
				NSDictionary * tPathDictionary;
				
				tPathDictionary=[tDictionary objectForKey:ICDOCUMENT_VALUE];
				
				// Check that the file exists
				
				// A COMPLETER
				
				[cell selectItemAtIndex:0];
			}
		}
	}
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBdocumentsArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeLocalization:IBremoveButton_];
		}
		else
		{
			[self removeLocalizationDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBdocumentsArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBremoveButton_ isEnabled];
		}
	}
	
	return NO;
}

- (NSString *) tableView:(NSTableView *) inTableView toolTipForCell:(NSCell *)cell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex mouseLocation:(NSPoint)mouseLocation
{
	if (inTableView==IBdocumentsArray_)
	{
		if (cachedLocalizations_!=nil)
		{
			NSString * tColumnIdentifier;
			NSDictionary * tDictionary;
			
			tDictionary=[cachedLocalizations_ objectAtIndex:inRowIndex];
				
			tColumnIdentifier=[inTableColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
			{
				return [tDictionary objectForKey:ICDOCUMENT_LANGUAGE];
			}
			else
			{
				NSDictionary * tPathDictionary;
				
				tPathDictionary=[tDictionary objectForKey:ICDOCUMENT_VALUE];
				
				if (tPathDictionary!=nil)
				{
					NSString * tPath;
					
					tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
					
					if (tPath!=nil && [tPath length]>0)
					{
						return tPath;
					}
				}
			}
		}
	}
	
	return nil;
}

#pragma mark -

- (NSDragOperation) tableView:(NSTableView*) inTableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(NSInteger) inRow proposedDropOperation:(NSTableViewDropOperation) inOperation
{
    if (inTableView==IBdocumentsArray_)
	{
		NSPasteboard * tPasteBoard;
			
		tPasteBoard=[info draggingPasteboard];
		
		if (cachedLocalizations_!=nil)
		{
			if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
			{
				NSUInteger tCount;
				NSArray * tArray;
					
				tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
							
				tCount=[tArray count];
				
				if (tCount==1)
				{
					BOOL isDirectory;
					NSString * tFilePath;
					
					tFilePath=[tArray objectAtIndex:0];
					
					if ([fileManager_ fileExistsAtPath:tFilePath isDirectory:&isDirectory]==YES)
					{
						if ([fileManager_ IC_fileAtPath:tFilePath matchesTypes:[ICPresentationDocumentUtilities presentationDocumentTypes]]==YES)
						{
							if (NSTableViewDropOn==inOperation)
							{
								return NSDragOperationCopy;
							}
							else if (NSTableViewDropAbove==inOperation)
							{
								NSString * tNewLanguage;
								
								tNewLanguage=[ICLocalizationUtilities possibleLanguageForFileAtPath:tFilePath];
								
								if (tNewLanguage!=nil)
								{
									NSUInteger tIndex;
									
									tIndex=[ICLocalizationUtilities indexOfLanguage:tNewLanguage inLocalizations:cachedLocalizations_];
									
									if (tIndex!=NSNotFound)
									{
										[inTableView setDropRow:tIndex dropOperation:NSTableViewDropOn];
										
										return NSDragOperationCopy;
									}
								}
								
								// Check that some languages are still not used
								
								if ([ICLocalizationUtilities firstNewLanguageInLocalizations:cachedLocalizations_]!=nil)
								{
									return NSDragOperationCopy;
								}
							}
						}
					}
				}
			}
		}
	}
	
	return NSDragOperationNone;
}

- (BOOL) tableView:(NSTableView*) inTableView acceptDrop:(id <NSDraggingInfo>)info row:(NSInteger) inRow dropOperation:(NSTableViewDropOperation) inOperation
{
    if (inTableView==IBdocumentsArray_)
	{
		if (cachedLocalizations_!=nil)
		{
			NSPasteboard * tPasteBoard;
				
			tPasteBoard=[info draggingPasteboard];
			
			if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
			{
				NSUInteger tCount;
				NSArray * tArray;
					
				tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
							
				tCount=[tArray count];
				
				if (tCount==1)
				{
					BOOL isDirectory;
					NSString * tFilePath;

					tFilePath=[tArray objectAtIndex:0];

					if ([fileManager_ fileExistsAtPath:tFilePath isDirectory:&isDirectory]==YES)
					{
						NSMutableDictionary * tLocalizationDictionary;
						
						if (NSTableViewDropOn==inOperation && [cachedLocalizations_ count]>0)
						{
							NSMutableDictionary * tPathDictionary;
							
							tLocalizationDictionary=[cachedLocalizations_ objectAtIndex:inRow];
			
							tPathDictionary=[tLocalizationDictionary objectForKey:ICDOCUMENT_VALUE];
							
							if (tPathDictionary!=nil)
							{
								NSInteger tPathType;
								
								tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
								
								if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
								{
									tFilePath=[tFilePath stringByRelativizingToPath:[document_ folder]];
								}
								else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
								{
									tFilePath=[tFilePath stringByRelativizingToPath:[document_ referenceFolderPath]];
								}
								
								[tPathDictionary setObject:tFilePath forKey:ICDOCUMENT_PATH];
			
								[self noteDocumentChange];
			
								// Notify the modification to the simulator
			
								[ICLicenseUtilities notifyLicenseDidChangeWithSender:self];
								
								[IBwarningLabel_ setHidden:[self sameFileTypeForAllLocalizations]];
							}
						}
						else if (NSTableViewDropAbove==inOperation || [cachedLocalizations_ count]==0)
						{
							// Check that some languages are still not used
							
							NSString * tNewLanguage;
							NSInteger tPathType;
							
							tPathType=[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
							
							tNewLanguage=[ICLocalizationUtilities possibleLanguageForFileAtPath:tFilePath];
						
							if (tNewLanguage==nil)
							{
								tNewLanguage=[[self document] documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
							}
							
							if ([ICLocalizationUtilities localizations:cachedLocalizations_ containsLanguage:tNewLanguage]==YES)
							{
								tNewLanguage=[ICLocalizationUtilities firstNewLanguageInLocalizations:cachedLocalizations_];
							}
							
							if (tNewLanguage!=nil)
							{
								if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
								{
									tFilePath=[tFilePath stringByRelativizingToPath:[document_ folder]];
								}
								else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
								{
									tFilePath=[tFilePath stringByRelativizingToPath:[document_ referenceFolderPath]];
								}
					
								tLocalizationDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tNewLanguage,ICDOCUMENT_LANGUAGE,
																										  [NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																																							tFilePath,ICDOCUMENT_PATH,
																																							nil],ICDOCUMENT_VALUE,
																										  nil];
								if (tLocalizationDictionary!=nil)
								{
									NSUInteger tSelectionIndex;
									
									[cachedLocalizations_ addObject:tLocalizationDictionary];
									
									[IBdocumentsArray_ deselectAll:self];
				
									[cachedLocalizations_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
									
									[IBdocumentsArray_ reloadData];
									
									tSelectionIndex=[cachedLocalizations_ indexOfObject:tLocalizationDictionary];
									
									if (tSelectionIndex!=NSNotFound)
									{
										[IBdocumentsArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tSelectionIndex] byExtendingSelection:NO];
									}
									
									[self noteDocumentChange];
			
									// Notify the modification to the simulator
				
									[ICLicenseUtilities notifyLicenseDidChangeWithSender:self];
									
									[IBwarningLabel_ setHidden:[self sameFileTypeForAllLocalizations]];
								}
							}
						}
					}
				}
			}
		}
	}
	
	return YES;
}

#pragma mark -

- (NSString *) _absolutePathOfSelectedRow
{
	NSInteger tSelectedRow;
	NSString * tAbsolutePath=nil;
	
	tSelectedRow=[IBdocumentsArray_ selectedRow];
	
	if (tSelectedRow!=-1)
	{
		NSDictionary * tDictionary;
				
		tDictionary=[cachedLocalizations_ objectAtIndex:tSelectedRow];
				
		if (tDictionary!=nil)
		{
			NSDictionary * tPathDictionary;
			
			tPathDictionary=[tDictionary objectForKey:ICDOCUMENT_VALUE];
			
			if (tPathDictionary!=nil)
			{
				tAbsolutePath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
				
				if (tAbsolutePath!=nil)
				{
					NSInteger tPathType;
				
					tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
					
					if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						tAbsolutePath=[tAbsolutePath stringByAbsolutingWithPath:[document_ folder]];
					}
					else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						tAbsolutePath=[tAbsolutePath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
					}
				}
			}
		}
	}
	
	return tAbsolutePath;
}

- (IBAction) openWithFinder:(id) sender
{
	NSString * tPath;
	
	tPath=[self _absolutePathOfSelectedRow];
	
	if (tPath!=nil)
	{
		[[NSWorkspace sharedWorkspace] openFile:tPath];
	}
}

- (IBAction) revealInFinder:(id) sender
{
	NSString * tPath;
	
	tPath=[self _absolutePathOfSelectedRow];
	
	if (tPath!=nil)
	{
		[[NSWorkspace sharedWorkspace] selectFile:tPath inFileViewerRootedAtPath:@""];
	}
}

- (IBAction) nothing:(id) sender
{
}

#pragma mark -

- (IBAction) addLocalization:(id) sender
{
	if (cachedLocalizations_!=nil)
	{
		NSMutableDictionary * tNewLocalizedTitleDictionary;
		NSString * tLanguageName;
	
		tLanguageName=[[self document] documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
							
		if ([ICLocalizationUtilities localizations:cachedLocalizations_ containsLanguage:tLanguageName]==YES)
		{
			tLanguageName=[ICLocalizationUtilities firstNewLanguageInLocalizations:cachedLocalizations_];
		}
		
		if (tLanguageName!=nil)
		{
			tNewLocalizedTitleDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tLanguageName,ICDOCUMENT_LANGUAGE,
																						   [NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInteger:[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE]],ICDOCUMENT_PATH_TYPE,
																																			 @"",ICDOCUMENT_PATH,
																																			 nil],ICDOCUMENT_VALUE,
																						   nil];
			
			if (tNewLocalizedTitleDictionary!=nil)
			{
				NSUInteger tRowIndex;
				
				[cachedLocalizations_ addObject:tNewLocalizedTitleDictionary];
				
				[cachedLocalizations_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[IBdocumentsArray_ reloadData];
				
				tRowIndex=[cachedLocalizations_ indexOfObject:tNewLocalizedTitleDictionary];
				
				if (tRowIndex!=NSNotFound)
				{
					[IBdocumentsArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				}
			}
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (IBAction) removeLocalization:(id) sender
{
	NSString * tAlertTitle;
    
    if ([IBdocumentsArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this localization?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these localizations?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeLocalizationDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeLocalizationDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBdocumentsArray_ selectedRowIndexes];
		
		[IBdocumentsArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedLocalizations_ removeObjectsAtIndexes:tIndexSet];
			
			[IBdocumentsArray_ reloadData];
			
			[IBremoveButton_ setEnabled:NO];
			
			// Notify Document Change
			
			[self noteDocumentChange];
			
			[ICLicenseUtilities notifyLicenseDidChangeWithSender:self];
		}
	}
}

- (IBAction) chooseLocalizedLicensePath:(id) sender
{
	if (cachedLocalizations_!=nil)
	{
		NSDictionary * tLocalizationDictionary;
		NSString * tAbsolutePath=nil;
		NSOpenPanel * tOpenPanel;
			
		tLocalizationDictionary=[cachedLocalizations_ objectAtIndex:[IBdocumentsArray_ selectedRow]];
		
		if (tLocalizationDictionary!=nil)
		{
			NSDictionary * tPathDictionary;
			
			
			tPathDictionary=[tLocalizationDictionary objectForKey:ICDOCUMENT_VALUE];
			
			if (tPathDictionary!=nil)
			{
				NSString * tPath;
				NSInteger tPathType;
	
				tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
				
				if (tPath!=nil && [tPath length]>0)
				{
					tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
					
					switch(tPathType)
					{
						case ICDOCUMENT_PATH_TYPE_GLOBAL:
						
							tAbsolutePath=tPath;
							
							break;
							
						case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
						
							tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ folder]];
							
							break;
							
						case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
					
							tAbsolutePath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
						
							break;
					}
				}
			}
		}
	
		tOpenPanel=[NSOpenPanel openPanel];
    
		[tOpenPanel setCanChooseFiles:YES];
		[tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
		
		[tOpenPanel beginSheetForDirectory:[tAbsolutePath stringByExpandingTildeInPath]
									  file:nil
									 types:[ICPresentationDocumentUtilities presentationDocumentTypes]
							modalForWindow:[IBview_ window]
							 modalDelegate:self
							didEndSelector:@selector(chooseLocalizedLicensePathDidEnd:returnCode:contextInfo:)
							   contextInfo:NULL];
	}
}

- (void) chooseLocalizedLicensePathDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        if (cachedLocalizations_!=nil)
		{
			NSMutableDictionary * tLocalizationDictionary;
			
			tLocalizationDictionary=[cachedLocalizations_ objectAtIndex:[IBdocumentsArray_ selectedRow]];
			
			if (tLocalizationDictionary!=nil)
			{
				NSMutableDictionary * tPathDictionary;
				NSInteger tPathType;
				NSString * tPath;
		
				tPathDictionary=[tLocalizationDictionary objectForKey:ICDOCUMENT_VALUE];
			
				tPath=[sheet filename];
		
				tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
		
				switch(tPathType)
				{
					case ICDOCUMENT_PATH_TYPE_GLOBAL:
					
						break;
						
					case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
					
						tPath=[tPath stringByRelativizingToPath:[document_ folder]];
						
						break;
						
					case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
					
						tPath=[tPath stringByRelativizingToPath:[document_ referenceFolderPath]];
						
						break;
						
					default:
					
						tPath=nil;
						
						break;
				}
				
				if (tPath!=nil)
				{
					[tPathDictionary setObject:tPath forKey:ICDOCUMENT_PATH];
				}
		
			}
		
			[self noteDocumentChange];
		
			// Notify the modification to the simulator
		
			[ICLicenseUtilities notifyLicenseDidChangeWithSender:self];
			
			[IBwarningLabel_ setHidden:[self sameFileTypeForAllLocalizations]];
		}
    }
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBdocumentsArray_)
    {
		[IBaddButton_ setEnabled:![ICLocalizationUtilities allLanguagesUsedInLocalizations:cachedLocalizations_]];
		
		[IBremoveButton_ setEnabled:([IBdocumentsArray_ numberOfSelectedRows]!=0)];
	}
}

- (void) licenseSettingsDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if ([tUserInfo objectForKey:ICPRESENTATION_NOTIFICATION_SENDER]!=self)
		{
			[self upgradeLicenseSettings];
		}
	}
}

@end
