/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationInspectorReadMeController.h"

#import "ICReadMeUtilities.h"
#import "ICDocument.h"

@implementation ICPresentationInspectorReadMeController

+ (ICPresentationInspectorReadMeController *) controller
{
	ICPresentationInspectorReadMeController * nController;
    
    nController=[ICPresentationInspectorReadMeController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICReadMeInspectorView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICReadMeInspectorView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	[cachedLocalizations_ release];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_README];
	
	if (dictionary_==nil)
	{
		// Create default ReadMe dictionary
		
		dictionary_=[ICReadMeUtilities defaultReadMeDictionary];
	
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_README];
		}
	}
	
	if (dictionary_!=nil)
	{
		[dictionary_ retain];
		
		// Cached Localizations
		
		cachedLocalizations_=[dictionary_ objectForKey:ICDOCUMENT_LOCALIZATIONS];
		
		if (cachedLocalizations_==nil)
		{
			cachedLocalizations_=[NSMutableArray array];
			
			if (cachedLocalizations_!=nil)
			{
				[dictionary_ setObject:cachedLocalizations_ forKey:ICDOCUMENT_LOCALIZATIONS];
			}
			else
			{
				// A COMPLETER
			}
		}
		
		if (cachedLocalizations_!=nil)
		{
			[cachedLocalizations_ retain];
		
			[IBarray_ deselectAll:self];
			
			[self upgradeSettings];
		}
		
		// Register for Notifications
	
		[[NSNotificationCenter defaultCenter] removeObserver:self name:ICPRESENTATION_NOTIFICATION_README_SETTINGS_DID_CHANGE object:[self document]];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(settingsDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_README_SETTINGS_DID_CHANGE 
												   object:[self document]];
	}
}

#pragma mark -

- (void) notifySettingsDidChange
{
	[ICReadMeUtilities notifyReadMeDidChangeWithSender:self];
}

@end
