/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationInspectorTitleController.h"

#import "ICTitleUtilities.h"
#import "ICLocalizationUtilities.h"

#import "ICLanguagePopupButtonCell.h"

#import "NSString+Iceberg.h"

#import "NSDocument+Iceberg.h"
#import "ICDocument.h"

@implementation ICPresentationInspectorTitleController

+ (ICPresentationInspectorTitleController *) controller
{
	ICPresentationInspectorTitleController * nController;
    
    nController=[ICPresentationInspectorTitleController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICTitleInspectorView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICTitleInspectorView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn = nil;
    
    // Reference
    
    tTableColumn = [IBarray_ tableColumnWithIdentifier:ICDOCUMENT_LANGUAGE];
    
	if (tTableColumn!=nil)
	{
		ICLanguagePopupButtonCell * tPopupButtonCell = nil;
    
		tPopupButtonCell=[[ICLanguagePopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
		
		if (tPopupButtonCell!=nil)
		{
			[tPopupButtonCell setControlSize:NSSmallControlSize];
			
			[tPopupButtonCell setMenu:[ICLocalizationUtilities localizationMenu]];
			
			[tPopupButtonCell setBordered:NO];
			
			[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
			
			[tPopupButtonCell setAction:@selector(nothing:)];
			
			[tPopupButtonCell setTarget:self];
			
			[tTableColumn setDataCell:tPopupButtonCell];
			
			[tPopupButtonCell release];
		}
	}
	
	// Name
    
    tTableColumn = [IBarray_ tableColumnWithIdentifier:ICDOCUMENT_VALUE];
	
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
	
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
	}
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	[cachedLocalizations_ release];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_TITLE];
	
	if (dictionary_==nil)
	{
		dictionary_=[ICTitleUtilities defaultTitleDictionaryForDocument:inDocument];
		
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_TITLE];
		}
	}
	
	if (dictionary_!=nil)
	{
		[dictionary_ retain];
		
		// Cached Localizations
		
		cachedLocalizations_=[dictionary_ objectForKey:ICDOCUMENT_LOCALIZATIONS];
		
		if (cachedLocalizations_==nil)
		{
			cachedLocalizations_=[ICTitleUtilities defaultLocalizationsArrayForDocument:inDocument];
			
			if (cachedLocalizations_!=nil)
			{
				[dictionary_ setObject:cachedLocalizations_ forKey:ICDOCUMENT_LOCALIZATIONS];
			}
			else
			{
				// A COMPLETER
			}
		}
		
		if (cachedLocalizations_!=nil)
		{
			[cachedLocalizations_ retain];
			
			[IBarray_ deselectAll:self];
			
			[IBarray_ reloadData];
		}
	}
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (cachedLocalizations_!=nil)
	{
		return [cachedLocalizations_ count];
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (cachedLocalizations_!=nil)
	{
		NSString * tColumnIdentifier;
		NSDictionary * tDictionary;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		tDictionary=[cachedLocalizations_ objectAtIndex:inRowIndex];
		
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			// A COMPLETER
		}
		else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
		{
			return [tDictionary objectForKey:ICDOCUMENT_VALUE];
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (cachedLocalizations_!=nil)
	{
		NSString * tColumnIdentifier;
		NSMutableDictionary * tDictionary;
		BOOL tDidChange=NO;
		
		tColumnIdentifier=[inTableColumn identifier];
	
		tDictionary=[cachedLocalizations_ objectAtIndex:inRowIndex];
			
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			NSString * tLanguageName;
			
			// Check that the Language is different
			
			tLanguageName=[[inTableColumn dataCellForRow:inRowIndex] itemTitleAtIndex:[object intValue]];
			
			if ([[tDictionary objectForKey:ICDOCUMENT_LANGUAGE] isEqualToString:tLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				NSUInteger tSelectionIndex;
				
				tEnumerator=[cachedLocalizations_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tLanguageName]==YES)
						{
							NSBeep();
							
							return;
						}
					}
				}
				
				[tDictionary setObject:tLanguageName forKey:ICDOCUMENT_LANGUAGE];
				
				[IBarray_ deselectAll:self];
				
				[cachedLocalizations_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[IBarray_ reloadData];
				
				tSelectionIndex=[cachedLocalizations_ indexOfObject:tDictionary];
				
				if (tSelectionIndex!=NSNotFound)
				{
					[IBarray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tSelectionIndex] byExtendingSelection:NO];
				}
				
				tDidChange=YES;
			}
		}
		else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
		{
			NSString * tTitleValue;
			
			tTitleValue=(NSString *) object;
			
			if ([[tDictionary objectForKey:ICDOCUMENT_VALUE] isEqualToString:tTitleValue]==NO)
			{
				[tDictionary setObject:tTitleValue forKey:ICDOCUMENT_VALUE];
				
				tDidChange=YES;
			}
		}
		
		if (tDidChange==YES)
		{
			[self noteDocumentChange];
			
			// Notify the modification to the simulator
			
			[ICTitleUtilities notifyTitleDidChangeWithSender:self];
		}
	}
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{    
    if (cachedLocalizations_!=nil)
	{
        NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			NSDictionary * tDictionary;
		
			tDictionary=[cachedLocalizations_ objectAtIndex:inRowIndex];
		
			[cell selectItemWithTitle:[tDictionary objectForKey:ICDOCUMENT_LANGUAGE]];
        }
    }
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBarray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeLocalization:IBremoveButton_];
		}
		else
		{
			[self removeLocalizationDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBarray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBremoveButton_ isEnabled];
		}
	}
	
	return NO;
}

- (NSString *) tableView:(NSTableView *) inTableView toolTipForCell:(NSCell *)cell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex mouseLocation:(NSPoint)mouseLocation
{
	if (cachedLocalizations_!=nil)
	{
        NSString * tColumnIdentifier;
		NSDictionary * tDictionary;
		
		tDictionary=[cachedLocalizations_ objectAtIndex:inRowIndex];
			
		tColumnIdentifier=[inTableColumn identifier];
		
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			return [tDictionary objectForKey:ICDOCUMENT_LANGUAGE];
        }
    }
	
	return nil;
}

#pragma mark -

- (IBAction) nothing:(id) sender
{
}

#pragma mark -

- (IBAction) addLocalization:(id) sender
{
	if (cachedLocalizations_!=nil)
	{
		NSMutableDictionary * tNewLocalizedTitleDictionary;
		NSString * tLanguageName;
	
		tLanguageName=[[self document] documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
							
		if ([ICLocalizationUtilities localizations:cachedLocalizations_ containsLanguage:tLanguageName]==YES)
		{
			tLanguageName=[ICLocalizationUtilities firstNewLanguageInLocalizations:cachedLocalizations_];
		}
		
		if (tLanguageName!=nil)
		{
			tNewLocalizedTitleDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tLanguageName,ICDOCUMENT_LANGUAGE,
																						   @"",ICDOCUMENT_VALUE,
																						   nil];
			
			if (tNewLocalizedTitleDictionary!=nil)
			{
				NSUInteger tRowIndex;
				
				[cachedLocalizations_ addObject:tNewLocalizedTitleDictionary];
				
				[cachedLocalizations_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[IBarray_ reloadData];
				
				tRowIndex=[cachedLocalizations_ indexOfObject:tNewLocalizedTitleDictionary];
				
				if (tRowIndex!=NSNotFound)
				{
					[IBarray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
					[IBarray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
				}
			}
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (IBAction) removeLocalization:(id) sender
{
	NSString * tAlertTitle;
    
    if ([IBarray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this localization?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these localizations?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeLocalizationDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeLocalizationDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBarray_ selectedRowIndexes];
		
		[IBarray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedLocalizations_ removeObjectsAtIndexes:tIndexSet];
			
			[IBarray_ reloadData];
			
			[IBremoveButton_ setEnabled:NO];
			
			// Notify Document Change
			
			[self noteDocumentChange];
			
			[ICTitleUtilities notifyTitleDidChangeWithSender:self];
		}
	}
}

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
    SEL tAction=[inMenuItem action];
	
    if (tAction==@selector(nothing:))
    {
		NSInteger tSelectedCount;
		
		tSelectedCount=[IBarray_ numberOfSelectedRows];
		
		if (tSelectedCount!=1)
		{
			return NO;
		}
		else
		{
			NSString * tCurrentLanguageName;
			NSString * tTitle;
			
			tCurrentLanguageName=[[cachedLocalizations_ objectAtIndex:[IBarray_ selectedRow]] objectForKey:ICDOCUMENT_LANGUAGE];
			
			tTitle=[inMenuItem title];
			
			if ([tTitle isEqualToString:tCurrentLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[cachedLocalizations_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tTitle]==YES)
						{
							return NO;
						}
					}
				}
			}
		}
	}
	
	return YES;
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBarray_)
    {
		[IBaddButton_ setEnabled:![ICLocalizationUtilities allLanguagesUsedInLocalizations:cachedLocalizations_]];
		
		[IBremoveButton_ setEnabled:([IBarray_ numberOfSelectedRows]!=0)];
	}
}

@end
