/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import <Cocoa/Cocoa.h>

extern NSString * const ICInstallerPlugInPboardType;

#define ICPRESENTATIONVIEW_TOP_BUTTON		-2
#define ICPRESENTATIONVIEW_BOTTOM_BUTTON	-1

enum
{
	ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_NONE,
	ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_TOP,
	ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_BOTTOM,
	ICPRESENTATIONVIEW_MOUSE_MODE_CLICK,
	ICPRESENTATIONVIEW_MOUSE_MODE_DRAG
};

@interface ICPresentationListView : NSView
{
	id delegate;

	id dataSource;

	NSInteger firstVisibleStep_;
	
	NSInteger lastVisibleStep_;
	
	BOOL shouldSeeSelectedStep_;
	
	// Tracking
	
	BOOL mouseTrackInside_;
	
	NSTrackingRectTag trackingTag_;
	NSRect trackingRect_;
	
	NSInteger selectedStep_;
	
	// Mouse Down
	
	BOOL topPushed_;
	
	BOOL bottomPushed_;
	
	BOOL mouseSelectedStepPushed_;
	
	NSInteger mouseSelectedStep_;
	
	NSPoint originalMouseDownPointLocation_;
	
	// Mouse Mode
	
	int mouseMode_;
	
	NSImage * unselectedPaneImage_;
    NSImage * unProcessedPaneImage_;
    NSImage * selectedPaneImage_;
	
	// Drag & Drop
	
	BOOL dragInProgress_;
	
	NSInteger currentDropStep_;
	
	NSInteger oldDropStep_;
	
	NSRect oldDroppingRect_;
	
	NSDragOperation currentDragOperation_;
}

+ (NSImage *) copyOfImageNamed:(NSString *) inName;

+ (NSCursor *) sharedUnselectableCursor;

- (void) selectStepAtIndex:(NSInteger) inIndex;

- (CGFloat) heightOfString:(NSString *) inString forFont:(NSFont *) inFont andMaxWidth:(CGFloat) inMaxWidth;

- (NSInteger) selectedStep;

- (NSRect) frameForStep:(NSInteger) inStep;

- (NSInteger) indexOfStepAtPoint:(NSPoint) inPoint;

- (NSImage *) imageOfStep:(NSInteger) inStep;

@end

@interface NSObject (ListViewDataSource)

- (NSInteger) numberOfStepsInListView:(ICPresentationListView *) inListView;

- (id) listView:(ICPresentationListView *) inListView objectForStep:(NSInteger) inStep;

- (BOOL) listView:(ICPresentationListView *) inListView stepWillBeVisible:(NSInteger) inStep;

- (BOOL) listView:(ICPresentationListView *) inListView shouldSelectStep:(NSInteger) inIdex;

- (void) listViewSelectionDidChange:(NSNotification *)notification;

- (BOOL) listView:(ICPresentationListView *) inListView writeStep:(NSInteger) inStep toPasteboard:(NSPasteboard*) inPasteboard;

- (NSDragOperation) listView:(ICPresentationListView*) inListView validateDrop:(id <NSDraggingInfo>)info proposedStep:(NSInteger) inStep;

- (BOOL) listView:(ICPresentationListView*) inListView acceptDrop:(id <NSDraggingInfo>)info step:(NSInteger) inStep;

@end