/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationListView.h"

#import "NSBezierPath+RoundedRect.h"

#define ICPRESENTATIONLISTVIEW_MORE_HEIGHT	20.0f

#define ICPRESENTATIONLISTVIEW_ROW_HEIGHT	24.0f

#define ICPRESENTATIONLISTVIEW_ROW_X_OFFSET	5.0f

#define ICPRESENTATIONLISTVIEW_ELLIPSIS_DIAMETER	4.0f

#define ICPRESENTATIONLISTVIEW_ELLIPSIS_OFFSET		10.0f

#define ICPRESENTATIONLISTVIEW_FIRST_CLICK_DELAY	0.5f

#define ICPRESENTATIONLISTVIEW_SECOND_CLICK_DELAY	0.25

#define DROP_CIRCLE_RADIUS		3.0f
#define DROP_CIRCLE_X_OFFSET	2.0f

//#define LIST_DEBUG_VIEW 1

NSString * const ICInstallerPlugInPboardType=@"ICInstallerPlugInPboardType";

@implementation ICPresentationListView

+ (NSImage *) copyOfImageNamed:(NSString *) inName
{
	static NSBundle * sInstallerBundle=nil;
	
	if (sInstallerBundle==nil)
	{
		NSString * tPath;
        
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5

        tPath=@"/System/Library/CoreServices/Installer.app";

#else

		tPath=@"/Applications/Utilities/Installer.app";	// A CHANGER
		 
#endif
        
        if (tPath!=nil)
        {
           // Check that this is the one we're looking for
            
            sInstallerBundle=[[NSBundle bundleWithPath:tPath] retain];
		}
	}
	
	if (sInstallerBundle!=nil)
	{
		NSString * tImagePath;
		NSImage * tImage=nil;
		
		tImagePath=[sInstallerBundle pathForResource:inName ofType:@"tif"];
		
		if (tImagePath==nil)
		{
			 tImagePath=[sInstallerBundle pathForResource:inName ofType:@"tiff"];
		}
		
		if (tImagePath!=nil)
		{
			tImage=[[NSImage alloc] initWithContentsOfFile:tImagePath];
		}
		
		return tImage;
	}
	
	return nil;
}

+ (NSCursor *) sharedUnselectableCursor
{
	static NSCursor * sUnselectableCursor=nil;
	
	if (sUnselectableCursor==nil)
	{
		sUnselectableCursor=[[NSCursor alloc] initWithImage:[NSImage imageNamed:@"unselectableCursor"] hotSpot:NSMakePoint(4.0f,4.0f)]; 
	}
	
	return sUnselectableCursor;
}

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
	
    if (self)
	{
		trackingRect_=NSZeroRect;
	
        unselectedPaneImage_=[ICPresentationListView copyOfImageNamed:@"DotGray"];
		
		selectedPaneImage_=[ICPresentationListView copyOfImageNamed:@"DotBlue"];
    
		unProcessedPaneImage_=[ICPresentationListView copyOfImageNamed:@"DotGrayDisabled"];
		
		currentDropStep_=-1;
		
		oldDropStep_ = -1;

		oldDroppingRect_ = NSZeroRect;

		currentDragOperation_ = NSDragOperationEvery;
		
		// Register for Drop
		
		[self registerForDraggedTypes:[NSArray arrayWithObjects:ICInstallerPlugInPboardType,NSFilenamesPboardType,nil]];
    }
	
    return self;
}

- (void) dealloc
{
	[unselectedPaneImage_ release];

	[selectedPaneImage_ release];
	
	[unProcessedPaneImage_ release];

	[super dealloc];
}

#pragma mark -

- (void) viewWillMoveToWindow:(NSWindow *) inWindow
{
	if (inWindow==nil)
	{
		if (trackingTag_!=0)
		{
			[self removeTrackingRect:trackingTag_];
		
			trackingRect_=NSZeroRect;
		}
	}
}


- (NSInteger) selectedStep
{
	return selectedStep_;
}

- (void) selectStepAtIndex:(NSInteger) inIndex
{
	if (selectedStep_!=inIndex)
	{
		selectedStep_=inIndex;
		
		shouldSeeSelectedStep_=YES;
		
		[self setNeedsDisplay:YES];
	}
}

- (NSRect) frameForStep:(NSInteger) inStep
{
	NSRect tStepFrame=NSZeroRect;
	
	if (inStep>=firstVisibleStep_ && inStep<=lastVisibleStep_)
	{
		NSRect tBounds;

		tBounds=[self bounds];
	
		if (dataSource!=nil)
		{
			NSInteger tNumberOfSteps;
			
			tNumberOfSteps=[dataSource numberOfStepsInListView:self];
			
			if (tNumberOfSteps>0)
			{
				// Compute the number of steps that can be displayed
				
				NSInteger tIndex;
				NSFont * tFont;
				CGFloat tTextOriginX;
				CGFloat tTextMaxWidth;
				NSRect tFrame;
				
				NSSize tBulletSize;
				NSImage * tBulletImage;
				
				tBulletImage=selectedPaneImage_;
					
				tBulletSize=[tBulletImage size];
				
				
				tTextOriginX=NSMinX(tBounds)+tBulletSize.width+4.0f+ICPRESENTATIONLISTVIEW_ROW_X_OFFSET;
				
				tTextMaxWidth=NSMaxX(tBounds)-tTextOriginX;
				
				tFrame=tBounds;
				
				tFrame.size.height=ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
				
				tFrame.origin.y=NSMaxY(tBounds)-ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
				
				for(tIndex=firstVisibleStep_;tIndex<=lastVisibleStep_;tIndex++)
				{
					CGFloat tStepHeight;
					NSString * tStepTitle;
					NSRect tClickableFrame;
					
					if (tIndex<=selectedStep_)
					{
						tFont=[NSFont boldSystemFontOfSize:13.0f];
					}
					else
					{
						tFont=[NSFont systemFontOfSize:13.0f];
					}
					
					tStepTitle=[dataSource listView:self objectForStep:tIndex];
				
					tStepHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
					
					if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
					{
						tFrame.size.height=ICPRESENTATIONLISTVIEW_ROW_HEIGHT+1.0f;
					}
					else
					{
						tFrame.size.height=tStepHeight+11.0f;
					}
					
					tFrame.origin.y-=NSHeight(tFrame);
					
					tClickableFrame=tFrame;
					
					tClickableFrame.origin.y+=2.0f;
					
					if (tIndex==inStep)
					{
						tStepFrame=tClickableFrame;
						
						break;
					}
				}
			}
		}
	}
	
	return tStepFrame;
}

- (NSInteger) indexOfStepAtPoint:(NSPoint) inPoint
{
	NSInteger tStepIndex=NSNotFound;
	
	NSRect tBounds;

    tBounds=[self bounds];
	
	if (dataSource!=nil)
	{
		NSInteger tNumberOfSteps;
		
		tNumberOfSteps=[dataSource numberOfStepsInListView:self];
		
		if (tNumberOfSteps>0)
		{
			// Compute the number of steps that can be displayed
			
			NSInteger tIndex;
			NSFont * tFont;
			CGFloat tTextOriginX;
			CGFloat tTextMaxWidth;
			NSRect tFrame;
			
			NSSize tBulletSize;
			NSImage * tBulletImage;
			
			tBulletImage=selectedPaneImage_;
				
			tBulletSize=[tBulletImage size];
			
			
			tTextOriginX=NSMinX(tBounds)+tBulletSize.width+4.0f+ICPRESENTATIONLISTVIEW_ROW_X_OFFSET;
			
			tTextMaxWidth=NSMaxX(tBounds)-tTextOriginX;
			
			tFrame=tBounds;
			
			tFrame.size.height=ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
			tFrame.origin.y=NSMaxY(tBounds)-ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
			if (firstVisibleStep_>0)
			{
				if (NSMouseInRect(inPoint,tFrame,YES)==YES)
				{
					return ICPRESENTATIONVIEW_TOP_BUTTON;
				}
			}
			
			for(tIndex=firstVisibleStep_;tIndex<=lastVisibleStep_;tIndex++)
			{
				CGFloat tStepHeight;
				NSString * tStepTitle;
				NSRect tClickableFrame;
				
				if (tIndex<=selectedStep_)
				{
					tFont=[NSFont boldSystemFontOfSize:13.0f];
				}
				else
				{
					tFont=[NSFont systemFontOfSize:13.0f];
				}
				
				tStepTitle=[dataSource listView:self objectForStep:tIndex];
			
				tStepHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
				
				if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
				{
					tFrame.size.height=ICPRESENTATIONLISTVIEW_ROW_HEIGHT+1.0f;
				}
				else
				{
					tFrame.size.height=tStepHeight+11.0f;
				}
				
				tFrame.origin.y-=NSHeight(tFrame);
				
				tClickableFrame=tFrame;
				
				tClickableFrame.origin.y+=2.0f;
				
				if (NSMouseInRect(inPoint,tClickableFrame,YES)==YES)
				{
					return tIndex;
				}
			}
			
			if (lastVisibleStep_<(tNumberOfSteps-1))
			{
				tFrame.size.height=ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
				tFrame.origin.y=0;
				
				if (NSMouseInRect(inPoint,tFrame,YES)==YES)
				{
					return ICPRESENTATIONVIEW_BOTTOM_BUTTON;
				}
			}
		}
	}
	
	return tStepIndex;
}

#pragma mark -

- (NSImage *) imageOfStep:(NSInteger) inStep
{
	NSImage * tImage=nil;
	NSRect tBounds;

    tBounds=[self bounds];
	
	if (dataSource!=nil)
	{
		CGFloat tHeight;
		NSString * tStepTitle;
		CGFloat tMaxHeight;
		NSFont * tFont;
		CGFloat tTextMaxWidth;
		CGFloat tTextOriginX;
		NSSize tBulletSize;
		NSImage * tBulletImage;
		
		tBulletImage=selectedPaneImage_;
			
		tBulletSize=[tBulletImage size];
		
		
		tTextOriginX=tBulletSize.width+4.0f+ICPRESENTATIONLISTVIEW_ROW_X_OFFSET;
		
		tTextMaxWidth=NSWidth(tBounds)-tTextOriginX;
		
		tMaxHeight=NSHeight(tBounds)-2*ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
		
		if (firstVisibleStep_>selectedStep_)
		{
			tFont=[NSFont boldSystemFontOfSize:13.0f];
		}
		else
		{
			tFont=[NSFont systemFontOfSize:13.0f];
		}
		
		tStepTitle=[dataSource listView:self objectForStep:inStep];
		
		tHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
			
		if (tHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
		{
			tHeight++;
		}
		else
		{
			tHeight+=11.0f;
		}
		
		tImage=[[NSImage alloc] initWithSize:NSMakeSize(NSWidth(tBounds)-ICPRESENTATIONLISTVIEW_ROW_X_OFFSET,tHeight)];
		
		if (tImage!=nil)
		{
			[tImage lockFocus];
			
			tBulletImage=((inStep==selectedStep_) ? selectedPaneImage_ : ((inStep < selectedStep_) ? unselectedPaneImage_ : unProcessedPaneImage_));
				
			[tBulletImage compositeToPoint:NSMakePoint(ICPRESENTATIONLISTVIEW_ROW_X_OFFSET,tHeight-(ICPRESENTATIONLISTVIEW_ROW_HEIGHT+tBulletSize.height)*0.5f) operation:NSCompositeSourceOver fraction:1.0f];
				
			// Draw the text
				
			if (tStepTitle!=nil && [tStepTitle length]>0)
			{
				NSDictionary * tFontAttributes;
				NSColor * tColor;
				NSRect tTextFrame;
				
				if (inStep >selectedStep_)
				{
					tColor=[NSColor colorWithDeviceWhite:0.0f alpha: 0.5f];
				}
				else
				{
					tColor=[NSColor blackColor];
				}
				
				tFontAttributes=[[NSDictionary alloc] initWithObjectsAndKeys:tFont,NSFontAttributeName,
																			 tColor,NSForegroundColorAttributeName,
																			 nil];
				
				tTextFrame=NSMakeRect(tTextOriginX,-1.0f,tTextMaxWidth,tHeight);
			
				[tStepTitle drawInRect:tTextFrame withAttributes:tFontAttributes];
				
				[tFontAttributes release];
			}
			
			[tImage unlockFocus];
		}
	}
	
	return [tImage autorelease];
	
}

- (CGFloat) heightOfString:(NSString *) inString forFont:(NSFont *) inFont andMaxWidth:(CGFloat) inMaxWidth
{
	CGFloat tHeight=0;
	
	if (inString!=nil)
	{
		NSTextStorage * tTextStorage;
			
		tTextStorage=[[NSTextStorage alloc] initWithString:inString];
		
		if (tTextStorage!=nil)
		{
			NSTextContainer * tTextContainer;
			
			tTextContainer = [[NSTextContainer alloc] initWithContainerSize: NSMakeSize(inMaxWidth, FLT_MAX)];
			
			if (tTextContainer!=nil)
			{
				NSLayoutManager * tLayoutManager;
				
				tLayoutManager = [[NSLayoutManager alloc] init];
				
				if (tLayoutManager!=nil)
				{
					[tLayoutManager setTypesetterBehavior:NSTypesetterBehavior_10_2_WithCompatibility];
					
					[tLayoutManager addTextContainer:tTextContainer];
					
					[tTextStorage addLayoutManager:tLayoutManager];

					[tTextStorage addAttribute:NSFontAttributeName value:inFont range:NSMakeRange(0, [tTextStorage length])];
					
					[tTextContainer setLineFragmentPadding:0.0f];
				
					[tLayoutManager glyphRangeForTextContainer:tTextContainer];
					
					tHeight=NSHeight([tLayoutManager usedRectForTextContainer:tTextContainer]);
					
					[tLayoutManager release];
				}
				
				[tTextContainer release];
			}
			
			[tTextStorage release];
		}
	}
	
	if (tHeight<ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
	{
		// Min Value
	
		tHeight=ICPRESENTATIONLISTVIEW_ROW_HEIGHT;
	}
	
	return tHeight;
}

- (void) drawRect:(NSRect) inRect
{
	NSRect tBounds;

    tBounds=[self bounds];
	
	if (dragInProgress_==NO)
	{
		if (NSEqualRects(tBounds,trackingRect_)==NO)
		{
			if (trackingTag_!=0)
			{
				[self removeTrackingRect:trackingTag_];
			}
			
			trackingRect_=tBounds;
			
			trackingTag_=[self addTrackingRect:tBounds owner:self userData:NULL assumeInside:NO];
		}
	}
	
	if (dataSource!=nil)
	{
		NSInteger tNumberOfSteps;
		
		tNumberOfSteps=[dataSource numberOfStepsInListView:self];
		
		if (tNumberOfSteps>0)
		{
			// Compute the number of steps that can be displayed
			
			NSInteger tMinStepIndex;
			NSInteger tMaxStepIndex;
			CGFloat tHeight;
			NSString * tStepTitle;
			NSInteger tIndex;
			CGFloat tMaxHeight;
			NSFont * tFont=nil;
			CGFloat tTextMaxWidth;
			CGFloat tTextOriginX;
			NSRect tFrame;
#ifdef LIST_DEBUG_VIEW
			NSRect tSavedFrame;
#endif
			NSBezierPath * tBezierPath;
				
			
			
			NSSize tBulletSize;
			NSImage * tBulletImage;
			
			NSGraphicsContext * tGraphicContext;
				
			tGraphicContext = [NSGraphicsContext currentContext];
				
			[NSGraphicsContext saveGraphicsState];
			
			tBulletImage=selectedPaneImage_;
				
			tBulletSize=[tBulletImage size];
			
			[tGraphicContext setPatternPhase:NSMakePoint(0.0f,NSMaxY([self frame]))];
			
			tTextOriginX=NSMinX(tBounds)+tBulletSize.width+4.0f+ICPRESENTATIONLISTVIEW_ROW_X_OFFSET;
			
			tTextMaxWidth=NSMaxX(tBounds)-tTextOriginX;
			
			tMaxHeight=NSHeight(tBounds)-2*ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
			if (selectedStep_<0 || selectedStep_>=tNumberOfSteps)
			{
				selectedStep_=0;
			}
			
			
			
			if (shouldSeeSelectedStep_==YES)
			{
				tFont=[NSFont boldSystemFontOfSize:13.0f];
				
				tMinStepIndex=tMaxStepIndex=selectedStep_;
			
				tStepTitle=[dataSource listView:self objectForStep:selectedStep_];
			
				tHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
				
				if (tHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
				{
					tHeight++;
				}
				else
				{
					tHeight+=11.0f;
				}
				
				shouldSeeSelectedStep_=NO;
				
				if (firstVisibleStep_>selectedStep_)
				{
					tFont=[NSFont systemFontOfSize:13.0f];
					
					tIndex=selectedStep_+1;
					
					while (tIndex<tNumberOfSteps && tHeight<tMaxHeight)
					{
						CGFloat tStepHeight;
						
						tStepTitle=[dataSource listView:self objectForStep:tIndex];
				
						tStepHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
						
						if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
						{
							tStepHeight++;
						}
						else
						{
							tStepHeight+=11.0f;
						}
						
						if ((tStepHeight+tHeight)>tMaxHeight)
						{
							break;
						}
						
						tHeight+=tStepHeight;
						
						tMaxStepIndex=tIndex;
						
						tIndex++;
					}
				}
				else
				{
					tIndex=selectedStep_-1;
				
					while (tIndex>=firstVisibleStep_ && tHeight<tMaxHeight)
					{
						CGFloat tStepHeight;
						
						tStepTitle=[dataSource listView:self objectForStep:tIndex];
					
						tStepHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
						
						if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
						{
							tStepHeight++;
						}
						else
						{
							tStepHeight+=11.0f;
						}
						
						if ((tStepHeight+tHeight)>tMaxHeight)
						{
							break;
						}
						
						tHeight+=tStepHeight;
						
						tMinStepIndex=tIndex;
						
						tIndex--;
					}
					
					if (tHeight<tMaxHeight)
					{
						tFont=[NSFont systemFontOfSize:13.0f];
						
						tIndex=selectedStep_+1;
						
						while (tIndex<tNumberOfSteps && tHeight<tMaxHeight)
						{
							CGFloat tStepHeight;
							
							tStepTitle=[dataSource listView:self objectForStep:tIndex];
					
							tStepHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
							
							if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
							{
								tStepHeight++;
							}
							else
							{
								tStepHeight+=11.0f;
							}
							
							if ((tStepHeight+tHeight)>tMaxHeight)
							{
								break;
							}
							
							tHeight+=tStepHeight;
							
							tMaxStepIndex=tIndex;
							
							tIndex++;
						}
					}

				}
				
				firstVisibleStep_=tMinStepIndex;
				
				lastVisibleStep_=tMaxStepIndex;
			}
			else
			{
				tIndex=firstVisibleStep_;
				
				tMinStepIndex=tMaxStepIndex=firstVisibleStep_;
				
				tHeight=0;
				
				while (tIndex<tNumberOfSteps && tHeight<tMaxHeight)
				{
					CGFloat tStepHeight;
					
					tStepTitle=[dataSource listView:self objectForStep:tIndex];
			
					if (tIndex>selectedStep_)
					{
						tFont=[NSFont systemFontOfSize:13.0f];
					}
					else
					{
						tFont=[NSFont boldSystemFontOfSize:13.0f];
					}
					
					tStepHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
					
					if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
					{
						tStepHeight++;
					}
					else
					{
						tStepHeight+=11.0f;
					}
					
					if ((tStepHeight+tHeight)>tMaxHeight)
					{
						break;
					}
					
					tHeight+=tStepHeight;
					
					tMaxStepIndex=tIndex;
					
					tIndex++;
				}
				
				if (tIndex==tNumberOfSteps && tHeight<tMaxHeight)
				{
					tIndex=firstVisibleStep_-1;
					
					while (tIndex>=0 && tHeight<tMaxHeight)
					{
						CGFloat tStepHeight;
						
						tStepTitle=[dataSource listView:self objectForStep:tIndex];
					
						tStepHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
						
						if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
						{
							tStepHeight++;
						}
						else
						{
							tStepHeight+=11.0f;
						}
						
						if ((tStepHeight+tHeight)>tMaxHeight)
						{
							break;
						}
						
						tHeight+=tStepHeight;
						
						tMinStepIndex=tIndex;
						
						tIndex--;
					}
				}
				
				firstVisibleStep_=tMinStepIndex;
				
				lastVisibleStep_=tMaxStepIndex;
			}
			
			// Draw the top more (Debug)
			
			tFrame=tBounds;
			
			tFrame.size.height=ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
			tFrame.origin.y=NSMaxY(tBounds)-ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
			tFrame.origin.x+=ICPRESENTATIONLISTVIEW_ROW_X_OFFSET;
			
			if (firstVisibleStep_>0)
			{
				CGFloat tMiddleX;
				CGFloat tMiddleY;
				
				tMiddleX=_CGFloatRound(NSMidX(tFrame)-ICPRESENTATIONLISTVIEW_ELLIPSIS_DIAMETER*0.5f);
				
				tMiddleY=NSMidY(tFrame);
				
				[[NSColor grayColor] set];
				
				if (mouseTrackInside_==NO && topPushed_==NO)
				{
					NSRect tEllipsisFrame;
				
					tEllipsisFrame=NSMakeRect(tMiddleX,_CGFloatRound(tMiddleY-ICPRESENTATIONLISTVIEW_ELLIPSIS_DIAMETER*0.5f),ICPRESENTATIONLISTVIEW_ELLIPSIS_DIAMETER,ICPRESENTATIONLISTVIEW_ELLIPSIS_DIAMETER);
					
					tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tEllipsisFrame];
					
					[tBezierPath fill];
					
					tEllipsisFrame.origin.x-=ICPRESENTATIONLISTVIEW_ELLIPSIS_OFFSET;
					
					tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tEllipsisFrame];
					
					[tBezierPath fill];
					
					tEllipsisFrame.origin.x+=2*ICPRESENTATIONLISTVIEW_ELLIPSIS_OFFSET;
					
					tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tEllipsisFrame];
					
					[tBezierPath fill];
				}
				else
				{
					if (topPushed_==YES)
					{
						[[NSColor darkGrayColor] set];
					}
					
					tMiddleX+=2.0f;
					
					tBezierPath=[NSBezierPath bezierPath];
					
					if (tBezierPath!=nil)
					{
						[tBezierPath moveToPoint:NSMakePoint(tMiddleX-6.0f,tMiddleY-2.0f)];
						
						[tBezierPath lineToPoint:NSMakePoint(tMiddleX,tMiddleY+3.0f)];
						
						[tBezierPath lineToPoint:NSMakePoint(tMiddleX+6.0f,tMiddleY-2.0f)];
						
						[tBezierPath closePath];
						
						[tBezierPath fill];
					}
				}
			}
			
#ifdef LIST_DEBUG_VIEW
			tSavedFrame=tFrame;
#endif

			for(tIndex=firstVisibleStep_;tIndex<=lastVisibleStep_;tIndex++)
			{
				CGFloat tStepHeight;
				NSRect tTextFrame;
				BOOL tWillBeVisible;
				
				if (currentDropStep_!=-1 && currentDropStep_==tIndex)
				{
					tBezierPath=[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(DROP_CIRCLE_X_OFFSET,NSMinY(tFrame)-1.0f,DROP_CIRCLE_RADIUS*2,DROP_CIRCLE_RADIUS*2)];
					
					if (tBezierPath!=nil)
					{
						[tBezierPath setLineWidth:1.5f];
					
						[[NSColor alternateSelectedControlColor] set];
						
						//[[NSColor colorWithDeviceWhite:0.0f alpha:0.85f] set];
						
						[tBezierPath stroke];
						
						[tBezierPath removeAllPoints];
						
						[tBezierPath setLineWidth:2.0f];
						
						[tBezierPath moveToPoint:NSMakePoint(DROP_CIRCLE_RADIUS*2+DROP_CIRCLE_X_OFFSET,NSMinY(tFrame)+DROP_CIRCLE_RADIUS-1.0f)];
						
						[tBezierPath lineToPoint:NSMakePoint(NSWidth(tBounds),NSMinY(tFrame)+DROP_CIRCLE_RADIUS-1.0f)];
						
						[tBezierPath stroke];
					}
				}
				
				tWillBeVisible=[dataSource listView:self stepWillBeVisible:tIndex];
				
				if (tIndex<=selectedStep_)
				{
					tFont=[NSFont boldSystemFontOfSize:13.0f];
				}
				else
				{
					tFont=[NSFont systemFontOfSize:13.0f];
				}
				
				tStepTitle=[dataSource listView:self objectForStep:tIndex];
			
				tStepHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
					
				if (tIndex==mouseSelectedStep_ && mouseSelectedStepPushed_==YES)
				{
					NSRect tBackgroundFrame;
					CGFloat tBakckgroundHeight;
					
					if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
					{
						tBakckgroundHeight=ICPRESENTATIONLISTVIEW_ROW_HEIGHT+1.0f;
					}
					else
					{
						tBakckgroundHeight=tStepHeight+11.0f;
					}
					
					tBackgroundFrame=NSMakeRect(1,NSMinY(tFrame)-tBakckgroundHeight+1.0f,NSWidth(tBounds)-2.0f,tBakckgroundHeight);
					
					tBezierPath=[NSBezierPath bezierPathWithRoundedRect:tBackgroundFrame radius:5.0f];
	
					if (tBezierPath!=nil)
					{
						/*NSColor * tColor;
						
						tColor=[NSColor alternateSelectedControlColor];
						
						[[tColor colorWithAlphaComponent:0.4f] set];*/
						
						[[NSColor colorWithDeviceWhite:0.0f alpha:0.75f] set];
						
						[tBezierPath fill];
						
						/*[[tColor colorWithAlphaComponent:1.0f] set];*/
						
						[[NSColor colorWithDeviceWhite:0.0f alpha:0.85f] set];
						
						[tBezierPath setLineWidth:2.0f];
						
						[tBezierPath stroke];
					}
				}
				else if (tIndex==selectedStep_ && mouseSelectedStepPushed_==NO)
				{
					NSRect tBackgroundFrame;
					CGFloat tBakckgroundHeight;
					
					if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
					{
						tBakckgroundHeight=ICPRESENTATIONLISTVIEW_ROW_HEIGHT+1.0f;
					}
					else
					{
						tBakckgroundHeight=tStepHeight+11.0f;
					}
					
					tBackgroundFrame=NSMakeRect(1,NSMinY(tFrame)-tBakckgroundHeight+1.0f,NSWidth(tBounds)-2.0f,tBakckgroundHeight);
					
					tBezierPath=[NSBezierPath bezierPathWithRoundedRect:tBackgroundFrame radius:5.0f];
	
					if (tBezierPath!=nil)
					{
						[[NSColor colorWithDeviceWhite:0.0f alpha:0.025f] set];
						
						[tBezierPath fill];
						
						[[NSColor colorWithDeviceWhite:0.0f alpha:0.05f] set];
						
						[tBezierPath setLineWidth:1.0f];
						
						[tBezierPath stroke];
					}
				}
				
				// Draw the bullet
				
				tBulletImage=((tIndex==selectedStep_) ? selectedPaneImage_ : ((tIndex < selectedStep_) ? unselectedPaneImage_ : unProcessedPaneImage_));
				
				if (tIndex!=mouseSelectedStep_ || mouseSelectedStepPushed_==NO)
				{
					NSImage * tBulletProcessedImage=nil;
				
					if (tWillBeVisible==NO)
					{
						NSImage * tImage;
						NSSize tSize;
						
						tImage=[NSImage imageNamed:@"Strip32Composite"];
						
						tSize=[tBulletImage size];
						
						tBulletProcessedImage=[[NSImage alloc] initWithSize:tSize];
						
						if (tBulletProcessedImage!=nil)
						{
							[tBulletProcessedImage lockFocus];
							
							[tImage compositeToPoint:NSZeroPoint operation:NSCompositeSourceOver];
							
							[tBulletImage compositeToPoint:NSZeroPoint operation:NSCompositeSourceIn];
							
							[tBulletProcessedImage unlockFocus];
							
							[tBulletProcessedImage autorelease];
						}
					}
					
					if (tBulletProcessedImage==nil)
					{
						tBulletProcessedImage=tBulletImage;
					}
					
					[tBulletProcessedImage compositeToPoint:NSMakePoint(ICPRESENTATIONLISTVIEW_ROW_X_OFFSET,NSMinY(tFrame)-(ICPRESENTATIONLISTVIEW_ROW_HEIGHT+tBulletSize.height)*0.5f) operation:NSCompositeSourceOver fraction:1.0f];
					
					//[tBulletImage compositeToPoint:NSMakePoint(ICPRESENTATIONLISTVIEW_ROW_X_OFFSET,NSMinY(tFrame)-(ICPRESENTATIONLISTVIEW_ROW_HEIGHT+tBulletSize.height)*0.5f) operation:NSCompositeSourceOver fraction:(tWillBeVisible==YES) ? 1.0f : 0.5f];
				
					/*if (tWillBeVisible==NO)
					{
						tBezierPath=[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(1+ICPRESENTATIONLISTVIEW_ROW_X_OFFSET,NSMinY(tFrame)-(ICPRESENTATIONLISTVIEW_ROW_HEIGHT+tBulletSize.height)*0.5f+2,tBulletSize.width-2,tBulletSize.width-2)];
						
						[[NSColor colorWithDeviceRed:1.0f green:152.0f/255 blue:135.0f/255 alpha:0.75f] set];
						
						[tBezierPath fill];
					}*/
				}
				else
				{
					tBezierPath=[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(1+ICPRESENTATIONLISTVIEW_ROW_X_OFFSET,NSMinY(tFrame)-(ICPRESENTATIONLISTVIEW_ROW_HEIGHT+tBulletSize.height)*0.5f+2,tBulletSize.width-2,tBulletSize.width-2)];
						
					if (tWillBeVisible==NO)
					{
						NSImage * tImage;
						NSColor * tColor=nil;
						
						tImage=[NSImage imageNamed:@"Strip32"];
						
						if (tImage!=nil)
						{
							tColor=[[NSColor colorWithPatternImage:tImage] colorWithAlphaComponent:0.75f];
						}
						
						if (tColor==nil)
						{
							tColor=[NSColor colorWithDeviceWhite:1.0f alpha: 0.75f];
						}
						
						if (tColor!=nil)
						{
							[tColor set];
						}
					}
					else
					{
						[[NSColor whiteColor] set];
					}
					
					[tBezierPath fill];
				}
				
				
				// Draw the text
				
				if (tStepTitle!=nil && [tStepTitle length]>0)
				{
					NSDictionary * tFontAttributes;
					NSColor * tColor;
					
					if (tIndex >selectedStep_)
					{
						if (tWillBeVisible==YES)
						{
							tColor=[NSColor colorWithDeviceWhite:0.0f alpha: 0.5f];
						}
						else
						{
							tColor=[NSColor colorWithPatternImage:[NSImage imageNamed:@"Strip32Disabled"]];
							
							//tColor=[NSColor colorWithDeviceRed:1.0f green:152.0f/255 blue:135.0f/255 alpha:0.75f];
						}
						
						if (tIndex==mouseSelectedStep_ && mouseSelectedStepPushed_==YES)
						{
							tColor=[NSColor colorWithDeviceWhite:1.0f alpha: 0.75f];
						}
					}
					else
					{
						if (tWillBeVisible==YES)
						{
							tColor=[NSColor blackColor];
						}
						else
						{
							tColor=[NSColor colorWithPatternImage:[NSImage imageNamed:@"Strip32"]];
							
							//tColor=[NSColor colorWithDeviceRed:1.0f green:152.0f/255 blue:135.0f/255 alpha:1.0f];
						}
						
						if (tIndex==mouseSelectedStep_ && mouseSelectedStepPushed_==YES)
						{
							tColor=[NSColor whiteColor];
						}
					}
					
					tFontAttributes=[[NSDictionary alloc] initWithObjectsAndKeys:tFont,NSFontAttributeName,
																				 tColor,NSForegroundColorAttributeName,
																				 nil];
					
					tTextFrame=NSMakeRect(tTextOriginX,NSMinY(tFrame)-tStepHeight-1.0f,tTextMaxWidth,tStepHeight);
				
					[tStepTitle drawInRect:tTextFrame withAttributes:tFontAttributes];
					
					[tFontAttributes release];
				}
				
				if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
				{
					tFrame.size.height=ICPRESENTATIONLISTVIEW_ROW_HEIGHT+1;
				}
				else
				{
					tFrame.size.height=tStepHeight+11;
				}
				
				tFrame.origin.y-=NSHeight(tFrame);
				
				/*f (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
				{
					tFrame.origin.y-=ICPRESENTATIONLISTVIEW_ROW_HEIGHT+1;
				}
				else
				{
					tFrame.origin.y-=tStepHeight+11;
				}*/
				
				
#ifdef LIST_DEBUG_VIEW
				tSavedFrame.size.height=ICPRESENTATIONLISTVIEW_ROW_HEIGHT;
			
				tSavedFrame.origin.y-=ICPRESENTATIONLISTVIEW_ROW_HEIGHT;
				
				[[NSColor blueColor] set];
				
				NSFrameRect(tFrame);
				
				[[NSColor redColor] set];
#endif
			}
			
			if (currentDropStep_!=-1 && currentDropStep_==tIndex)
			{
				tBezierPath=[NSBezierPath bezierPathWithOvalInRect:NSMakeRect(DROP_CIRCLE_X_OFFSET,NSMinY(tFrame)-1.0f,DROP_CIRCLE_RADIUS*2,DROP_CIRCLE_RADIUS*2)];
				
				if (tBezierPath!=nil)
				{
					[tBezierPath setLineWidth:1.5f];
				
					[[NSColor alternateSelectedControlColor] set];
					
					//[[NSColor colorWithDeviceWhite:0.0f alpha:0.85f] set];
					
					[tBezierPath stroke];
					
					[tBezierPath removeAllPoints];
					
					[tBezierPath setLineWidth:2.0f];
					
					[tBezierPath moveToPoint:NSMakePoint(DROP_CIRCLE_RADIUS*2+DROP_CIRCLE_X_OFFSET,NSMinY(tFrame)+DROP_CIRCLE_RADIUS-1.0f)];
					
					[tBezierPath lineToPoint:NSMakePoint(NSWidth(tBounds),NSMinY(tFrame)+DROP_CIRCLE_RADIUS-1.0f)];
					
					[tBezierPath stroke];
				}
			}
			
			// Draw the bottom More
			
			tFrame.size.height=ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
			tFrame.origin.y=0;
			
			if (lastVisibleStep_<(tNumberOfSteps-1))
			{
				CGFloat tMiddleX;
				CGFloat tMiddleY;
				
				tMiddleX=_CGFloatRound(NSMidX(tFrame)-ICPRESENTATIONLISTVIEW_ELLIPSIS_DIAMETER*0.5f);
				
				tMiddleY=NSMidY(tFrame);
				
				[[NSColor grayColor] set];
				
				if (mouseTrackInside_==NO && bottomPushed_==NO)
				{
					NSRect tEllipsisFrame;
					
					tEllipsisFrame=NSMakeRect(tMiddleX,_CGFloatRound(tMiddleY-ICPRESENTATIONLISTVIEW_ELLIPSIS_DIAMETER*0.5f),ICPRESENTATIONLISTVIEW_ELLIPSIS_DIAMETER,ICPRESENTATIONLISTVIEW_ELLIPSIS_DIAMETER);
					
					tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tEllipsisFrame];
					
					[tBezierPath fill];
					
					tEllipsisFrame.origin.x-=ICPRESENTATIONLISTVIEW_ELLIPSIS_OFFSET;
					
					tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tEllipsisFrame];
					
					[tBezierPath fill];
					
					tEllipsisFrame.origin.x+=2*ICPRESENTATIONLISTVIEW_ELLIPSIS_OFFSET;
					
					tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tEllipsisFrame];
					
					[tBezierPath fill];
				}
				else
				{
					if (bottomPushed_==YES)
					{
						[[NSColor darkGrayColor] set];
					}
					
					tMiddleX+=2.0f;
					
					tBezierPath=[NSBezierPath bezierPath];
					
					if (tBezierPath!=nil)
					{
						[tBezierPath moveToPoint:NSMakePoint(tMiddleX-6.0f,tMiddleY+2.0f)];
						
						[tBezierPath lineToPoint:NSMakePoint(tMiddleX,tMiddleY-3.0f)];
						
						[tBezierPath lineToPoint:NSMakePoint(tMiddleX+6.0f,tMiddleY+2.0f)];
						
						[tBezierPath closePath];
						
						[tBezierPath fill];
					}
				}
			}
			
			[NSGraphicsContext restoreGraphicsState];
		}
	}
}

- (void) mouseUp:(NSEvent *) inEvent
{
	NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
	
	if (mouseMode_!=ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_NONE)
	{
		NSInteger tStep;
		BOOL tRefreshNeeded=NO;
		
		switch(mouseMode_)
		{
			case ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_TOP:
			
				if (topPushed_==YES)
				{
					topPushed_=NO;
				
					tRefreshNeeded=YES;
				
					[NSObject cancelPreviousPerformRequestsWithTarget:self];
					
					[[self window] performSelector:@selector(invalidateCursorRectsForView:) withObject:self afterDelay:0.1f];
				}
				
				break;
			
			case ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_BOTTOM:
			
				if (bottomPushed_==YES)
				{
					bottomPushed_=NO;
				
					tRefreshNeeded=YES;
				
					[NSObject cancelPreviousPerformRequestsWithTarget:self];
					
					[[self window] performSelector:@selector(invalidateCursorRectsForView:) withObject:self afterDelay:0.1f];
				}
				
				break;
			
			case ICPRESENTATIONVIEW_MOUSE_MODE_CLICK:
				
				mouseSelectedStepPushed_=NO;
				
				tStep=[self indexOfStepAtPoint:tMouseLoc];
				
				if (tStep==mouseSelectedStep_)
				{
					tRefreshNeeded=YES;
					
					if (selectedStep_!=mouseSelectedStep_)
					{
						// Select new step
					
						[self selectStepAtIndex:mouseSelectedStep_];
						
						if (delegate!=nil)
						{
							[delegate listViewSelectionDidChange:nil];
							
							// A COMPLETER
						}
					}
					
					mouseSelectedStep_=NSNotFound;
				}
				
				break;
		}
		
		if (tRefreshNeeded==YES)
		{
			[self setNeedsDisplay:YES];	
		}
	}
}

- (void) mouseDragged:(NSEvent *) inEvent
{
	if (mouseMode_!=ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_NONE)
	{
		NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
		NSInteger tStep;
		BOOL tRefreshNeeded=NO;
	
		tStep=[self indexOfStepAtPoint:tMouseLoc];
	
		switch(mouseMode_)
		{
			case ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_TOP:
			
				if (topPushed_==YES)
				{
					if (tStep!=ICPRESENTATIONVIEW_TOP_BUTTON)
					{
						topPushed_=NO;
						
						[NSObject cancelPreviousPerformRequestsWithTarget:self];
						
						tRefreshNeeded=YES;
					}
				}
				else
				{
					if (tStep==ICPRESENTATIONVIEW_TOP_BUTTON)
					{
						topPushed_=YES;
						
						tRefreshNeeded=YES;
						
						[self performSelector:@selector(delayedScrollTop:) withObject:self afterDelay:ICPRESENTATIONLISTVIEW_FIRST_CLICK_DELAY];
					}
				}
				
				break;
			
			case ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_BOTTOM:
			
				if (bottomPushed_==YES)
				{
					if (tStep!=ICPRESENTATIONVIEW_BOTTOM_BUTTON)
					{
						bottomPushed_=NO;
						
						[NSObject cancelPreviousPerformRequestsWithTarget:self];
						
						tRefreshNeeded=YES;
					}
				}
				else
				{
					if (tStep==ICPRESENTATIONVIEW_BOTTOM_BUTTON)
					{
						bottomPushed_=YES;
						
						tRefreshNeeded=YES;
						
						[self performSelector:@selector(delayedScrollBottom:) withObject:self afterDelay:ICPRESENTATIONLISTVIEW_FIRST_CLICK_DELAY];
					}
				}
				
				break;
			
			case ICPRESENTATIONVIEW_MOUSE_MODE_CLICK:
				
				if (delegate!=nil)
				{
					// We need to check that we are not far enough to start a drag
					
#define ICPRESENTATIONLISTVIEW_DRAGOFFSET	4.0f

					if (fabs(originalMouseDownPointLocation_.x-tMouseLoc.x)>ICPRESENTATIONLISTVIEW_DRAGOFFSET || fabs(originalMouseDownPointLocation_.y-tMouseLoc.y)>ICPRESENTATIONLISTVIEW_DRAGOFFSET)
					{
						NSPasteboard * tPasteboard;
						
						tPasteboard=[NSPasteboard pasteboardWithName:NSDragPboard];
						
						if (tPasteboard!=nil && dataSource!=nil)
						{
							if ([dataSource listView:self writeStep:mouseSelectedStep_ toPasteboard:tPasteboard]==YES)
							{
								NSRect tStepFrame;
								
								mouseMode_=ICPRESENTATIONVIEW_MOUSE_MODE_DRAG;
								
								mouseSelectedStepPushed_=NO;
								
								[self setNeedsDisplay:YES];
								
								tStepFrame=[self frameForStep:mouseSelectedStep_];
								
								[self dragImage:[self imageOfStep:mouseSelectedStep_]
											 at:tStepFrame.origin
										 offset:NSZeroSize
										  event:inEvent
									 pasteboard:tPasteboard
										 source:self
									  slideBack:YES];
									
								return;
							}
						}
					}
					
					// Step is not draggable
						
					// We just need to check whether the click is in or not
					
					if (mouseSelectedStepPushed_==YES)
					{
						if (tStep!=mouseSelectedStep_)
						{
							mouseSelectedStepPushed_=NO;
							
							tRefreshNeeded=YES;
						}
					}
					else
					{
						if (tStep==mouseSelectedStep_)
						{
							mouseSelectedStepPushed_=YES;
							
							tRefreshNeeded=YES;
						}
					}
				}
				
				break;
		}
	
		if (tRefreshNeeded==YES)
		{
			[self setNeedsDisplay:YES];
		}
	}
}

- (void) delayedScrollTop:(id) sender
{
	topPushed_=NO;
	
	if (firstVisibleStep_>0)
	{
		topPushed_=YES;
		
		firstVisibleStep_--;
		
		[self setNeedsDisplay:YES];
		
		[self performSelector:@selector(delayedScrollTop:) withObject:self afterDelay:ICPRESENTATIONLISTVIEW_SECOND_CLICK_DELAY];
	}
}

- (void) delayedScrollBottom:(id) sender
{
	bottomPushed_=NO;
	
	if (dataSource!=nil)
	{
		if (lastVisibleStep_<([dataSource numberOfStepsInListView:self]-1))
		{
			bottomPushed_=YES;
		
			firstVisibleStep_++;
			
			[self setNeedsDisplay:YES];
			
			[self performSelector:@selector(delayedScrollBottom:) withObject:self afterDelay:ICPRESENTATIONLISTVIEW_SECOND_CLICK_DELAY];
		}
	}
}

/*- (void) scrollWheel:(NSEvent *) inEvent
{
	NSLog(@"%f",[inEvent deltaY]);
	
	// A COMPLETER
}*/

- (void) mouseDown:(NSEvent *) inEvent
{
	NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
	NSInteger tStep;
	
	tStep=[self indexOfStepAtPoint:tMouseLoc];
	
	switch(tStep)
	{
		case ICPRESENTATIONVIEW_TOP_BUTTON:
			
			topPushed_=YES;
			
			firstVisibleStep_--;
				
			[self setNeedsDisplay:YES];
			
			mouseMode_=ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_TOP;
			
			// Launch delayed Scroll
			
			[self performSelector:@selector(delayedScrollTop:) withObject:self afterDelay:ICPRESENTATIONLISTVIEW_FIRST_CLICK_DELAY];
			
			break;
		
		case ICPRESENTATIONVIEW_BOTTOM_BUTTON:
		
			bottomPushed_=YES;
			
			firstVisibleStep_++;
				
			[self setNeedsDisplay:YES];
			
			mouseMode_=ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_BOTTOM;
			
			// Launch delayed Scroll
			
			[self performSelector:@selector(delayedScrollBottom:) withObject:self afterDelay:ICPRESENTATIONLISTVIEW_FIRST_CLICK_DELAY];
			
			break;
		
		case NSNotFound:
			
			mouseMode_=ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_NONE;
			
			return;
	
		default:
			
			mouseSelectedStep_=tStep;
			
			mouseMode_=ICPRESENTATIONVIEW_MOUSE_MODE_CLICK_NONE;
			
			if (delegate!=nil)
			{
				if ([delegate listView:self shouldSelectStep:mouseSelectedStep_]==YES)
				{
					originalMouseDownPointLocation_=tMouseLoc;
					
					mouseMode_=ICPRESENTATIONVIEW_MOUSE_MODE_CLICK;
					
					mouseSelectedStepPushed_=YES;
					
					[self setNeedsDisplay:YES];
				}
			}
		
			break;
	}
}

- (void) mouseEntered:(NSEvent *) inEvent
{
    if (dragInProgress_==NO)
	{
		mouseTrackInside_=YES;
    
		[self setNeedsDisplay:YES];
	}
}

- (void) mouseExited:(NSEvent *) inEvent
{
	if (dragInProgress_==NO)
	{
		mouseTrackInside_=NO;

		[self setNeedsDisplay:YES];
	}
}

- (void) resetCursorRects
{
	NSRect tBounds;

    tBounds=[self bounds];
	
	if (dataSource!=nil)
	{
		NSInteger tNumberOfSteps;
		
		tNumberOfSteps=[dataSource numberOfStepsInListView:self];
		
		if (tNumberOfSteps>0)
		{
			// Compute the number of steps that can be displayed
			
			NSInteger tIndex;
			NSFont * tFont;
			CGFloat tTextOriginX;
			CGFloat tTextMaxWidth;
			NSRect tFrame;
			
			NSSize tBulletSize;
			NSImage * tBulletImage;
			
			tBulletImage=selectedPaneImage_;
				
			tBulletSize=[tBulletImage size];
			
			
			tTextOriginX=NSMinX(tBounds)+tBulletSize.width+4.0f+ICPRESENTATIONLISTVIEW_ROW_X_OFFSET;
			
			tTextMaxWidth=NSMaxX(tBounds)-tTextOriginX;
			
			tFrame=tBounds;
			
			tFrame.size.height=ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
			tFrame.origin.y=NSMaxY(tBounds)-ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
			if (firstVisibleStep_>0)
			{
				//[self addCursorRect:tFrame cursor:[NSCursor arrowCursor]];
			}
			
			for(tIndex=firstVisibleStep_;tIndex<=lastVisibleStep_;tIndex++)
			{
				CGFloat tStepHeight;
				NSString * tStepTitle;
				NSRect tClickableFrame;
				
				if (tIndex<=selectedStep_)
				{
					tFont=[NSFont boldSystemFontOfSize:13.0f];
				}
				else
				{
					tFont=[NSFont systemFontOfSize:13.0f];
				}
				
				tStepTitle=[dataSource listView:self objectForStep:tIndex];
			
				tStepHeight=[self heightOfString:tStepTitle forFont:tFont andMaxWidth:tTextMaxWidth];
				
				if (tStepHeight==ICPRESENTATIONLISTVIEW_ROW_HEIGHT)
				{
					tFrame.size.height=ICPRESENTATIONLISTVIEW_ROW_HEIGHT+1.0f;
				}
				else
				{
					tFrame.size.height=tStepHeight+11.0f;
				}
				
				tFrame.origin.y-=NSHeight(tFrame);
				
				tClickableFrame=tFrame;
				
				tClickableFrame.origin.y+=2.0f;
				
				
				if ([delegate listView:self shouldSelectStep:tIndex]==YES)
				{
					//[self addCursorRect:tFrame cursor:[NSCursor arrowCursor]];
				}
				else
				{
					[self addCursorRect:tFrame cursor:[ICPresentationListView sharedUnselectableCursor]];
				}
			}
			
			if (lastVisibleStep_<(tNumberOfSteps-1))
			{
				tFrame.size.height=ICPRESENTATIONLISTVIEW_MORE_HEIGHT;
			
				tFrame.origin.y=0;
				
				//[self addCursorRect:tFrame cursor:[NSCursor arrowCursor]];
			}
		}
	}
}

#pragma mark -

- (NSDragOperation) draggingSourceOperationMaskForLocal: (BOOL) flag
{
  return NSDragOperationEvery;
}

- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>) sender
{
	if (trackingTag_!=0)
	{
		[self removeTrackingRect:trackingTag_];
	}
	
	dragInProgress_=YES;
	
	trackingRect_=NSZeroRect;
	
	currentDropStep_ = -1;

	oldDropStep_ = -1;

	oldDroppingRect_ = NSZeroRect;

	currentDragOperation_ = NSDragOperationEvery;
	
	return currentDragOperation_;
}

- (BOOL) wantsPeriodicDraggingUpdates
{
	// To Scroll when it's still possible
	
	return YES;
}

- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
{
	NSPoint tPoint;
	NSInteger tStep;
	NSRect tFrame;
	BOOL tRefreshNeeded=NO;
	NSInteger tOldCurrentDropStep;
	
	tPoint=[sender draggingLocation];
	
	tPoint=[self convertPoint:tPoint fromView:nil];
	
	tStep=[self indexOfStepAtPoint:tPoint];
	
	tOldCurrentDropStep=currentDropStep_;
	
	switch(tStep)
	{
		case ICPRESENTATIONVIEW_TOP_BUTTON:
			
			if (firstVisibleStep_>0)
			{
				firstVisibleStep_--;
				
				tRefreshNeeded=YES;
			}
			
			tStep=firstVisibleStep_;
			
			currentDropStep_=firstVisibleStep_;
			
			tFrame=[self frameForStep:tStep];
			
			break;
		
		case ICPRESENTATIONVIEW_BOTTOM_BUTTON:
		
			if (lastVisibleStep_<([dataSource numberOfStepsInListView:self]-1))
			{
				firstVisibleStep_++;
				
				tRefreshNeeded=YES;
			}
			
			tStep=lastVisibleStep_;
			
			currentDropStep_=lastVisibleStep_+1;
			
			tFrame=[self frameForStep:tStep];
			
			break;
			
		case NSNotFound:
			
			if (tPoint.y<NSMidY([self bounds]))
			{
				tStep=lastVisibleStep_;
				
				currentDropStep_=lastVisibleStep_+1;
			}
			else
			{
				tStep=firstVisibleStep_;
				
				currentDropStep_=firstVisibleStep_;
			}
			
			tFrame=[self frameForStep:tStep];
			
			break;
			
		default:
			
			tFrame=[self frameForStep:tStep];
			
			if (tPoint.y<NSMidY(tFrame))
			{
				currentDropStep_=tStep+1;
			}
			else
			{
				currentDropStep_=tStep;
			}
			
			break;
	}
	
	if (oldDropStep_!=currentDropStep_)
	{
		oldDropStep_=currentDropStep_;
		
		if ([dataSource respondsToSelector: @selector(listView:validateDrop:proposedStep:)]==YES)
		{	
			currentDragOperation_=[dataSource listView:self
										  validateDrop:sender
										  proposedStep:currentDropStep_];
		
			if (currentDragOperation_==NSDragOperationNone)
			{
				currentDropStep_=-1;
				
			}
			
			if (tRefreshNeeded==NO)
			{
				[self setNeedsDisplayInRect:oldDroppingRect_];
			}
			
			if (currentDropStep_==tStep)
			{
				// Above
				
				oldDroppingRect_=NSMakeRect(0,NSMaxY(tFrame)-5.0f,NSWidth([self bounds]),10.0f);
			}
			else
			{
				// Below
				
				oldDroppingRect_=NSMakeRect(0,NSMinY(tFrame)-5.0f,NSWidth([self bounds]),10.0f);
			}
			
			if (tRefreshNeeded==NO)
			{
				[self displayRect:oldDroppingRect_];;

			}
		}
	}
	else
	{
		currentDropStep_=tOldCurrentDropStep;
	}
	
	if (tRefreshNeeded==YES)
	{
		[self setNeedsDisplay:YES];
	}
	
	return currentDragOperation_;
}

- (BOOL) performDragOperation: (id<NSDraggingInfo>)sender
{
	mouseTrackInside_=YES;

	if (currentDropStep_!=-1 && [dataSource respondsToSelector: @selector(listView:acceptDrop:step:)]==YES)
    {
		BOOL tResult;
		
		tResult=[dataSource listView:self acceptDrop:sender step:currentDropStep_];
    
		currentDropStep_=-1;
		
		return tResult;
	}
	
    return NO;
}

- (BOOL) prepareForDragOperation: (id<NSDraggingInfo>)sender
{
	[self setNeedsDisplayInRect:oldDroppingRect_];

	return YES;
}

- (void) concludeDragOperation:(id <NSDraggingInfo>)sender
{
	dragInProgress_=NO;
	
	currentDropStep_=-1;
	
	[self setNeedsDisplay:YES];
}

- (void) draggingEnded:(id <NSDraggingInfo>)sender
{
	dragInProgress_=NO;
	
	currentDropStep_=-1;
	
	[self setNeedsDisplayInRect:oldDroppingRect_];
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
	dragInProgress_=NO;
	
	currentDropStep_=-1;
	
	[self setNeedsDisplayInRect:oldDroppingRect_];
}

@end

@implementation NSObject (ListViewDataSource)

- (NSInteger) numberOfStepsInListView:(ICPresentationListView *) inListView
{
	return 0;
}

- (id) listView:(ICPresentationListView *) inListView objectForStep:(NSInteger) inStep
{
	return nil;
}

- (BOOL) listView:(ICPresentationListView *) inListView stepWillBeVisible:(NSInteger) inStep
{
	return YES;
}

- (BOOL) listView:(ICPresentationListView *) inListView shouldSelectStep:(NSInteger) inIdex
{
	return YES;
}

- (void) listViewSelectionDidChange:(NSNotification *)notification
{
	
}

- (BOOL) listView:(ICPresentationListView *) inListView writeStep:(NSInteger) inStep toPasteboard:(NSPasteboard*) inPasteboard
{
	return NO;
}

- (NSDragOperation) listView:(ICPresentationListView*) inListView validateDrop:(id <NSDraggingInfo>)info proposedStep:(NSInteger) inStep
{
	return NSDragOperationNone;
}

- (BOOL) listView:(ICPresentationListView*) inListView acceptDrop:(id <NSDraggingInfo>)info step:(NSInteger) inStep
{
	return NO;
}

@end