/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationPluginButton.h"

#define ICPRESENTATION_PLUGIN_BUTTON_LINE_WDITH		2.5f

#define ICPRESENTATION_PLUGIN_BUTTON_ICON_LINE_WIDTH	2.5f

#define ICPRESENTATION_PLUGIN_BUTTON_ICON_WIDTH		5.0f

@implementation ICPresentationPluginButton

- (void) setPlus:(BOOL) aBool
{
	if (plus_!=aBool)
	{
		plus_=aBool;
	
		[self setNeedsDisplay:YES];
	}
}

#pragma mark -

- (void) drawRect:(NSRect) inRect
{
	NSRect tBounds;
	NSBezierPath * tBezierPath;
	
	tBounds=NSInsetRect([self bounds],ICPRESENTATION_PLUGIN_BUTTON_LINE_WDITH,ICPRESENTATION_PLUGIN_BUTTON_LINE_WDITH);
	
	if ([self isEnabled]==YES)
	{
		[[NSColor colorWithDeviceWhite:0.25f alpha:1.0f] set];
	}
	else
	{
		[[NSColor colorWithDeviceWhite:0.65f alpha:1.0f] set];
	}
	
	// Draw the circle
	
	tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tBounds];
	
	[tBezierPath setLineWidth:ICPRESENTATION_PLUGIN_BUTTON_LINE_WDITH];
	
	if (pushed_==YES)
	{
		[tBezierPath fill];
	}
	else
	{
		[tBezierPath stroke];
	}
	
	// Draw the Plus or Minus
	
	[tBezierPath removeAllPoints];
	
	if (pushed_==YES)
	{
		[[NSColor whiteColor] set];
	}
	
	[tBezierPath setLineWidth:ICPRESENTATION_PLUGIN_BUTTON_ICON_LINE_WIDTH];
	
	[tBezierPath moveToPoint:NSMakePoint(NSMidX(tBounds)-ICPRESENTATION_PLUGIN_BUTTON_ICON_WIDTH,NSMidY(tBounds))];
	[tBezierPath lineToPoint:NSMakePoint(NSMidX(tBounds)+ICPRESENTATION_PLUGIN_BUTTON_ICON_WIDTH,NSMidY(tBounds))];
		
	if (plus_==YES)
	{
		[tBezierPath moveToPoint:NSMakePoint(NSMidX(tBounds),NSMidY(tBounds)-ICPRESENTATION_PLUGIN_BUTTON_ICON_WIDTH)];
		[tBezierPath lineToPoint:NSMakePoint(NSMidX(tBounds),NSMidY(tBounds)+ICPRESENTATION_PLUGIN_BUTTON_ICON_WIDTH)];
	}
	
	[tBezierPath stroke];
}

- (void) mouseDown:(NSEvent *) inEvent
{
	if ([self isEnabled]==YES)
	{
		NSRect tBounds;
		NSBezierPath * tBezierPath;
		NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
		
		tBounds=[self bounds];
		
		tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tBounds];
		
		[tBezierPath setLineWidth:ICPRESENTATION_PLUGIN_BUTTON_LINE_WDITH];
		
		if ([tBezierPath containsPoint:tMouseLoc]==YES)
		{
			pushed_=YES;
				
			[self setNeedsDisplay:YES];
		}
	}
}

- (void) mouseDragged:(NSEvent *) inEvent
{
    if ([self isEnabled]==YES)
	{
		NSRect tBounds;
		NSBezierPath * tBezierPath;
		NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
		
		tBounds=[self bounds];
		
		tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tBounds];
		
		[tBezierPath setLineWidth:ICPRESENTATION_PLUGIN_BUTTON_LINE_WDITH];
		
		if ([tBezierPath containsPoint:tMouseLoc]==YES)
		{
			if (pushed_==NO)
			{
				pushed_=YES;
				
				[self setNeedsDisplay:YES];
			}
		}
		else
		{
			if (pushed_==YES)
			{
				pushed_=NO;
				
				[self setNeedsDisplay:YES];
			}
		}
	}
}

- (void) mouseUp:(NSEvent *) inEvent
{
    if ([self isEnabled]==YES)
	{
		NSRect tBounds;
		NSBezierPath * tBezierPath;
		NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
		
		pushed_=NO;
		
		tBounds=[self bounds];
		
		tBezierPath=[NSBezierPath bezierPathWithOvalInRect:tBounds];
		
		[tBezierPath setLineWidth:ICPRESENTATION_PLUGIN_BUTTON_LINE_WDITH];
		
		if ([tBezierPath containsPoint:tMouseLoc]==YES)
		{
			[self sendAction:[self action] to:[self target]];
			
			[self setNeedsDisplay:YES];
		}
	}
}

@end
