/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationTextView.h"

#import "NSFileManager+fileTypes.h"

#import "ICPresentationDocumentUtilities.h"

#import "NSBezierPath+RoundedRect.h"

@implementation NSObject (ICPresentationTextView)

- (void) documentTextDidChange:(NSString *) inPath
{
}

@end

@implementation ICPresentationTextView

- (id) initWithFrame:(NSRect) inFrame
{
	self=[super initWithFrame:inFrame];
	
	if (self!=nil)
	{
		[self unregisterDraggedTypes];
		
		[self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	}

	return self;
}

#pragma mark -

- (id) delegate
{
    return delegate;
}

- (void) setDelegate:(id) inDelegate
{
    delegate=inDelegate;
}

#pragma mark -

- (void) drawRect:(NSRect) inRect
{
    [NSGraphicsContext saveGraphicsState];
	
	[super drawRect:inRect];
	
	[NSGraphicsContext restoreGraphicsState];
	
	if (highlighted_==YES)
	{
		NSBezierPath * tPath;
		
		tPath=[NSBezierPath bezierPathWithRoundedRect:NSInsetRect([self bounds],2.0f,2.0f) radius:8.0f];
		
		if (tPath!=nil)
		{
			[tPath setLineWidth:3.0f];
			
			[[NSColor alternateSelectedControlColor] set];
			
			[tPath stroke];
		}
	}
}

#pragma mark -

- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>)sender
{
    NSPasteboard * tPasteBoard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
	
    tPasteBoard = [sender draggingPasteboard];

    if ( [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
    {
        if (sourceDragMask & NSDragOperationCopy)
        {
            NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
			if (tFiles!=nil && [tFiles count]==1)
			{
				NSString * tFilePath;
				
				tFilePath=[tFiles objectAtIndex:0];
				
				if ([[NSFileManager defaultManager] IC_fileAtPath:tFilePath matchesTypes:[ICPresentationDocumentUtilities presentationDocumentTypes]]==YES)
				{
					highlighted_=YES;
			
					[self setNeedsDisplay:YES];
            
					return NSDragOperationCopy;
				}
			}
        }
    }
    
    return NSDragOperationNone;
}

- (BOOL) wantsPeriodicDraggingUpdates
{
	return NO;
}

- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
{
	return [self draggingEntered:sender];
}

- (BOOL) performDragOperation:(id <NSDraggingInfo>)sender
{
    NSPasteboard * tPasteBoard;
    /*NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];*/
	
    tPasteBoard = [sender draggingPasteboard];

    if ( [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
    {
        NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
		if (tFiles!=nil && [tFiles count]==1)
		{
			NSString * tFilePath;
			
			tFilePath=[tFiles objectAtIndex:0];
			
			if ([[NSFileManager defaultManager] IC_fileAtPath:tFilePath matchesTypes:[ICPresentationDocumentUtilities presentationDocumentTypes]]==YES)
			{
				if (delegate!=nil)
				{
					[delegate documentTextDidChange:tFilePath];
				}
				
				return YES;
			}
		}
    }
	
    return NO;
}

- (BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	highlighted_=NO;
	
    [self setNeedsDisplay:YES];
    
    return YES;
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
    highlighted_=NO;
	
    [self setNeedsDisplay:YES];
}

@end
