/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationViewController.h"
#import "ICPresentation+Constants.h"

#import "ICPresentationDocumentUtilities.h"

#import "ICLanguageConverter.h"

#import "ICInstallerPluginLocalizer.h"

#import "ICLocalizationUtilities.h"

#import "NSFileManager+fileTypes.h"

@implementation ICPresentationViewController

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument andLanguage:(NSString *) inLanguage
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
}

- (void) dealloc
{
	[IBaccessoryView_ release];
	
	[pluginName_ release];
	
	[paneTitleKey_ release];

	[super dealloc];
}

- (void) initLocalizationInformation
{
	static NSDictionary * sInformationDictionary=nil;
		
	if (sInformationDictionary==nil)
	{
		NSString * tPath;
		
		tPath=[[NSBundle mainBundle] pathForResource:@"InstallerViewControllerList" ofType:@"plist"];
		
		if (tPath!=nil)
		{
			sInformationDictionary=[[NSDictionary alloc] initWithContentsOfFile:tPath];
		}
		else
		{
			// A COMPLETER
		}
	}
	
	if (sInformationDictionary!=nil)
	{
		NSDictionary * tViewDictionary;
		
		tViewDictionary=[sInformationDictionary objectForKey:NSStringFromClass([self class])];
		
		if (tViewDictionary!=nil)
		{
			[self setPluginName:[tViewDictionary objectForKey:ICPRESENTATIONVIEWCONTROLLER_PLUGIN_NAME]];
			
			[self setPaneTitleKey:[tViewDictionary objectForKey:ICPRESENTATIONVIEWCONTROLLER_PANE_TITLE_KEY]];
		}
		else
		{
			// A COMPLETER
		}
	}
}

#pragma mark -

- (void) prepareButtons:(NSArray *) inButtonsArray forLanguage:(NSString *) inLanguage
{
	if (inLanguage!=nil)
	{
		ICInstallerPluginLocalizer * tPluginLocalizer;
		NSButton * tButton;
		NSRect tFrame;
		CGFloat tMaxX;
		
		tPluginLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
		
		// Print / Customize
		
		tButton=[inButtonsArray objectAtIndex:0];
		
		[tButton setHidden:YES];
		
		// Save
		
		tButton=[inButtonsArray objectAtIndex:1];
		
		[tButton setHidden:YES];
		
		// Continue
		
		tButton=[inButtonsArray objectAtIndex:3];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"CONTINUE"
			  forLanguage:inLanguage
				 inBundle:@"Main"]];
	
		tFrame=[tButton frame];
		
		tMaxX=NSMaxX(tFrame);
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		tFrame.origin.x=tMaxX-NSWidth(tFrame);
		
		[tButton setFrame:tFrame];
		
		tMaxX=NSMinX(tFrame);
		
		// Go Back
		
		tButton=[inButtonsArray objectAtIndex:2];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"GOBACK"
			  forLanguage:inLanguage
				 inBundle:@"Main"]];
		
		tFrame=[tButton frame];
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		tFrame.origin.x=tMaxX-NSWidth(tFrame);
		
		[tButton setFrame:tFrame];
		
		[[tButton superview] setNeedsDisplay:YES];
	}
}

- (void) refreshViewForLanguage:(NSString *) inLanguage
{
	
}

- (id) accessoryView
{
	return IBaccessoryView_;
}

- (BOOL) isLocalizedValueUsable:(id) inValue
{
	return YES;
}

- (NSArray *) clearedLocalizationsValue:(NSMutableArray *) inArray
{
	NSMutableArray * tClearedArray=nil;
	
	if (inArray!=nil && [inArray count]>0)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tDictionary;
			
			tClearedArray=[NSMutableArray array];
			
			while (tDictionary=[tEnumerator nextObject])
			{
				id tValue;
				
				tValue=[tDictionary objectForKey:ICDOCUMENT_VALUE];
				
				if (tValue!=nil)
				{
					if ([self isLocalizedValueUsable:tValue]==YES)
					{
						[tClearedArray addObject:tDictionary];
					}
				}
			}
		}
	}
	
	return tClearedArray;
}

- (id) localizedValueForLanguage:(NSString *) inLanguage inLocalizations:(NSMutableArray *) inArray
{
	id tValue=nil;
	NSArray * tClearedArray;
	
	tClearedArray=[self clearedLocalizationsValue:inArray];
	
	if (tClearedArray!=nil)
	{
		tValue=[ICLocalizationUtilities localizedValueForLanguage:inLanguage inLocalizations:tClearedArray];
	}
	
	return tValue;
}

- (NSString *) paneTitleForLanguage:(NSString *) inLanguage
{
	NSString * tLocalizedPaneTitle;
	ICInstallerPluginLocalizer * tLocalizer;
	
	tLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
	
	tLocalizedPaneTitle=[tLocalizer localizedString:paneTitleKey_ forLanguage:inLanguage inBundle:pluginName_];
                
	if (tLocalizedPaneTitle==nil)
	{
		inLanguage=[[ICLanguageConverter defaultConverter] ISOFromEnglish:inLanguage];
		
		tLocalizedPaneTitle=[tLocalizer localizedString:paneTitleKey_ forLanguage:inLanguage inBundle:pluginName_];
		
		if (tLocalizedPaneTitle==nil)
		{
			inLanguage=[NSString stringWithString:@"English"];
			
			tLocalizedPaneTitle=[tLocalizer localizedString:paneTitleKey_ forLanguage:inLanguage inBundle:pluginName_];
		}
	}
	
	return tLocalizedPaneTitle;
}

+ (NSDictionary *) defaultStorageOptionsWithPath:(NSString *) inPath
{
    NSDictionary * tOptionsDictionary;
    NSDictionary * tFontDictionary;
    NSString * tBaseURL=nil;
	
    tFontDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSFont fontWithName:@"Helvetica" size:12.0f],NSFontAttributeName,nil];
    
	tBaseURL=[NSURL fileURLWithPath:/*[*/inPath /*stringByDeletingLastPathComponent]*/];
	
	tOptionsDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tFontDictionary,@"DefaultAttributes",
																  tBaseURL,NSBaseURLDocumentOption,
                                                                  nil];
    
    return tOptionsDictionary;
}

#pragma mark -

- (void) setPluginName:(NSString *) inPluginName
{
	if (pluginName_!=inPluginName)
	{
		[pluginName_ release];
		
		pluginName_=[inPluginName copy];
	}
}

- (void) setPaneTitleKey:(NSString *) inPaneTitleKey
{
	if (paneTitleKey_!=inPaneTitleKey)
	{
		[paneTitleKey_ release];
		
		paneTitleKey_=[inPaneTitleKey copy];
	}
}

@end
