/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationViewController.h"

#import "ICInstallationType+Constants.h"

#import "ICInstallationTypeTree.h"

@interface ICPresentationViewInstallationTypeController : ICPresentationViewController
{
	IBOutlet id IBoutlineScrollView_;
	
	IBOutlet id IBoutlineView_;
	
	IBOutlet id IBspaceRequiredLabel_;
	
	IBOutlet id IBremainingLabel_;
	
	IBOutlet id IBdescriptionTextView_;
	
	IBOutlet id IBsplitView_;
	
	IBOutlet id IBhierarchyBox_;
	
	IBOutlet id IBinstallationTypeModePopupButton_;
	
	IBOutlet id IBhierarchyPopupButton_;
	
	IBOutlet id IBheaderViewMenu_;
	
	
	IBOutlet id IBindentationColumn_;
	
	IBOutlet id IBactionColumn_;
	
	IBOutlet id IBsizeColumn_;
	
	// Data
	
	NSArray * cachedButtons_;
	
	NSString * currentHierarchyName_;
	
	ICInstallationTypeTreeNode * tree_;
	
	NSString * cachedLanguage_;
	
	NSArray * internalDragData_;
	
	NSArray * cachedHierarchyNames_;
	
	BOOL cachedShowPackagesName_;
	
	NSUserDefaults * defaults_;
	
	BOOL choiceDependencyEditionOn_;
}

+ (ICPresentationViewInstallationTypeController *) controller;

- (id) outlineScrollView;

- (void) updateBorders;

- (NSMutableArray *) currentHierarchy;

- (void) updateTree;

- (void) removeFromHierarchy:(id) sender;

- (void) removeFromHierarchyDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo;


- (void) embedSelectionInGroup:(BOOL) isGroup named:(NSString *) inName;

- (void) unembed;


- (IBAction) group:(id) sender;

- (IBAction) ungroup:(id) sender;

- (IBAction) merge:(id) sender;

- (IBAction) separate:(id) sender;


- (IBAction) showHideIndentationColumn:(id) sender;


- (void) expandItemIfPossible:(ICInstallationTypeTreeNode *) inItem;

- (void) expandItem:(ICInstallationTypeTreeNode *) inChoiceNode ifKeyOfDictionary:(NSMutableDictionary *) inDictionary;

- (void) restoreDisclosedStates;

- (void) archiveSelection;

- (void) restoreSelection;

- (void) choiceDidExpand:(ICInstallationTypeTreeNode *) inChoiceNode;


- (IBAction) switchInstallationTypeMode:(id) sender;

// Hierarchy PopupButton

- (void) setHiddeHierarchyBox:(BOOL) inHidden;

- (void) refreshHierarchyPopupButton;

- (IBAction) switchHierarchy:(id) sender;

- (IBAction) nothing:(id) sender;

- (IBAction) addHierarchy:(id) sender;

- (IBAction) duplicateHierarchy:(id) sender;

- (IBAction) removeHierarchy:(id) sender;

- (void) removeHierarchyDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo;

// Notifications

- (void) installationTypeSettingsDidChange:(NSNotification *) inNotification;

- (void) currentHierarchyDidChange:(NSNotification *) inNotification;

- (void) appleModeDidChange:(NSNotification *) inNotification;

- (void) installationTypeChoiceDependencyEditDidRequest:(NSNotification *) inNotification;

- (void) installationTypeChoiceDependencyEndEditDidRequest:(NSNotification *) inNotification;

@end
