/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationViewInstallationTypeController.h"

#import "NSDictionary+MutableDeepCopy.h"

#import "TreeNode+Indentation.h"

#import "ICDocument.h"

#import "ICUUIDFactory.h"

#import "ICInstallationTypeOutlineView.h"

#import "ICInstallationType+Constants.h"

#import "ICInstallationTypeUtilities.h"

#import "ICInstallationTypeUtilities+Extras.h"

#import "ICLocalizationUtilities.h"

#import "ICInstallationTypeCheckBoxCell.h"

#import "ICPreferences+Constants.h"

#define SAFENODE(n) 	((ICInstallationTypeTreeNode*)((n!=nil)?(n):(tree_)))

#define ICINSTALLATIONTYPEVIEW_MIN_HEIGHT	121.0f

#define ICDashedBorder		200

#import "ICInstallationTypeOutlineView.h"

NSString * const ICPRESENTATIONVIEWINSTALLATIONTYPE_INDENTATION_COLUMNIDENTIFIER=@"installationType.indentation";

NSString * const ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE=@"ui.project.presentation.installationType.branches.disclosed.%@";

NSString * const ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_SELECTION=@"ui.project.presentation.installationType.branches.selection.%@";

NSString * const ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_SHOWPACKAGES=@"ui.project.presentation.installationType.showPackagesName";

@implementation ICPresentationViewInstallationTypeController

+ (ICPresentationViewInstallationTypeController *) controller
{
	ICPresentationViewInstallationTypeController * nController;
    
    nController=[ICPresentationViewInstallationTypeController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICInstallationTypeView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICInstallationTypeView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
		else
		{
			[nController initLocalizationInformation];
		}
    }
    
    return [nController autorelease];
}

- (void) awakeFromNib
{
	 NSTableColumn * tTableColumn = nil;
	 ICInstallationTypeOutlineHeaderView * tHeaderView;
	 ICInstallationTypeCornerView * tCornerView;
	 
	 defaults_=[NSUserDefaults standardUserDefaults];
	 
	 cachedHierarchyNames_=[[NSArray alloc] initWithObjects:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER,
														   ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE,
														   ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE,
														   nil];
	 
	 tHeaderView=[[ICInstallationTypeOutlineHeaderView alloc] initWithFrame:[[IBoutlineView_ headerView] frame]];
	 
	 if (tHeaderView!=nil)
	 {
		[IBoutlineView_ setHeaderView:tHeaderView];
		
		[tHeaderView setMenu:IBheaderViewMenu_];
		
		[tHeaderView release];
	 }
	 
	 tCornerView=[[ICInstallationTypeCornerView alloc] initWithFrame:[[IBoutlineView_ cornerView] frame]];
	 
	 if (tCornerView!=nil)
	 {
		[IBoutlineView_ setCornerView:tCornerView];
		
		[tCornerView release];
	 }
	 
	 // Indentation
	 
	[IBindentationColumn_ retain];
	 
	[IBoutlineView_ removeTableColumn:IBindentationColumn_];
	
	[IBoutlineView_ sizeToFit];
	 
	 // Name
	 
	 tTableColumn=[IBoutlineView_ tableColumnWithIdentifier: @"Name"];
	 
	 if (tTableColumn!=nil)
	 {
		ICInstallationTypeCheckBoxCell * tPrototypeButtonCell = nil;
		
		tPrototypeButtonCell = [[ICInstallationTypeCheckBoxCell alloc] initTextCell: @""];
		
		if (tPrototypeButtonCell!=nil)
		{
			[tPrototypeButtonCell setControlSize:NSSmallControlSize];
			
			[tPrototypeButtonCell setFont:[NSFont controlContentFontOfSize:[NSFont systemFontSize]]];

			[tPrototypeButtonCell setButtonType: NSSwitchButton];
			
			[tPrototypeButtonCell setEditable:NO];
			
			[tPrototypeButtonCell setCellAttribute:NSPushInCell to:NO];
			
			[tPrototypeButtonCell setAllowsMixedState:YES];
			
			[tTableColumn setDataCell:tPrototypeButtonCell];
			
			[tPrototypeButtonCell release];
		}
	 }
	 
	 [IBoutlineView_ registerForDraggedTypes:[NSArray arrayWithObjects:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_PACKAGES_AND_GROUPS_PASTEBOARDTYPE,
                                                                       ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIDDENCHILDPACKAGES_PASTEBOARDTYPE,
																	   ICDOCUMENT_PROJECT_PACKAGES_PASTEBOARDTYPE,
																	   nil]];

	[IBoutlineView_ setAutoresizesOutlineColumn:NO];
	
	// Register for Notifications

	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(appleModeDidChange:) 
												 name:ICPREFERENCES_NOTIFICATION_ADVANCED_APPLE_MODE_DID_CHANGE
											   object:nil];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument andLanguage:(NSString *) inLanguage
{
	[tree_ release];
	
	tree_=nil;
	
	[cachedLanguage_ release];
	
	cachedLanguage_=nil;
	
	[super initializeWithDictionary:inDictionary forDocument:inDocument andLanguage:inLanguage];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE];
	
	if (dictionary_==nil)
	{
		// Build the dictionary from the list of packages
		
		dictionary_=[ICInstallationTypeUtilities defaultInstallationTypeDictionaryWithPackages:[inDocument packagesArray]];
		
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE];
		}
	}
	/*else
	{
		// We need to check the current list to check none are missing or should be removed
		
		[ICInstallationTypeUtilities updateInstallationTypeDictionary:dictionary_ withPackages:[inDocument packagesArray]];
	}*/
	
	currentHierarchyName_=[[document_ documentDefaultForKey:@"ui.project.presentation.installationType.hierarchy.current"] retain];
	
	cachedShowPackagesName_=[[document_ documentDefaultForKey:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_SHOWPACKAGES] boolValue];
	
	[(ICInstallationTypeCornerView *) [IBoutlineView_ cornerView] showPackagesName:cachedShowPackagesName_];
	
	if (currentHierarchyName_==nil)
	{
		currentHierarchyName_=[[NSString alloc] initWithString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER];
	}
	
	if (dictionary_!=nil)
	{
		NSMutableArray * tCurrentHierarchyArray;
		BOOL tState;
		NSNumber * tNumber;
		int tInstallationTypeMode=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_ALLOW;
		NSDictionary * tHierarchiesDictionary;
		NSArray * tRemovedPackages;
		NSDictionary * tUserInfo=nil;
	
		tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
	
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_MODE];
	
		if (tNumber!=nil)
		{
			tInstallationTypeMode=[tNumber intValue];
		}
		
		[IBinstallationTypeModePopupButton_ selectItemWithTag:tInstallationTypeMode];
		
		
		[dictionary_ retain];
	
		tState=[defaults_ boolForKey:ICPREFERENCES_ADVANCED_APPLE_MODE_STATE];
	
		[self setHiddeHierarchyBox:(tState==NO)];
		//NSLog(@"%@",dictionary_);
		
		[self updateBorders];
		
		// Hierarchy PopupButton
		
		[self refreshHierarchyPopupButton];
		
		// Choices Line
		
		tCurrentHierarchyArray=[[tHierarchiesDictionary objectForKey:currentHierarchyName_] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST];
		
		if (tCurrentHierarchyArray!=nil)
		{
			tree_=[ICInstallationTypeTreeNode installationTypeTreeNodeWithArray:tCurrentHierarchyArray];
		
			cachedLanguage_=[inLanguage retain];
			
			[self refreshViewForLanguage:cachedLanguage_];
			
			// Restore Disclosed State
			
			[self restoreDisclosedStates];
			
			// Restore Selection
			
			[self restoreSelection];
	
			// Refresh Source List
	
			tRemovedPackages=[[[tHierarchiesDictionary objectForKey:currentHierarchyName_] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED] allKeys];
			
			if (tRemovedPackages!=nil)
			{
				tUserInfo=[NSDictionary dictionaryWithObject:tRemovedPackages forKey:ICDOCUMENT_NOTIFICATION_HIERARCHY_PACKAGES_REMOVED_LIST];
			}
			
			[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_DOCUMENT_SOURCELIST_SHOULD_RELOAD
																object:[self document]
															  userInfo:tUserInfo];
			
			
			// Register for Notifications
	
			[[NSNotificationCenter defaultCenter] removeObserver:self name:ICPRESENTATION_NOTIFICATION_INSTALLATIONTYPE_SETTINGS_DID_CHANGE object:[self document]];
			
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(installationTypeSettingsDidChange:) 
														 name:ICPRESENTATION_NOTIFICATION_INSTALLATIONTYPE_SETTINGS_DID_CHANGE 
													   object:[self document]];
			
			[[NSNotificationCenter defaultCenter] removeObserver:self name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_EDIT_DIDREQUEST object:[self document]];
			
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(installationTypeChoiceDependencyEditDidRequest:) 
														 name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_EDIT_DIDREQUEST
													   object:[self document]];
													   
			[[NSNotificationCenter defaultCenter] removeObserver:self name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_ENDEDIT_DIDREQUEST object:[self document]];
			
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(installationTypeChoiceDependencyEndEditDidRequest:) 
														 name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_ENDEDIT_DIDREQUEST
													   object:[self document]];
		}
		else
		{
			NSLog(@"Missing %@ Hierarchy",currentHierarchyName_);
			
			// A COMPLETER
		}
	}
	else
	{
		// A COMPLETER
	}
}

- (void) dealloc
{
	[cachedButtons_ release];
	
	[currentHierarchyName_ release];
	
	[cachedHierarchyNames_ release];
	
	[tree_ release];
	
	[cachedLanguage_ release];
	
	[super dealloc];
}

#pragma mark -

- (id) outlineScrollView
{
	return IBoutlineScrollView_;
}

- (void) updateBorders
{
	BOOL tVisible=YES;
	ICInstallationTypeOutlineHeaderView * tHeaderView;
	ICInstallationTypeCornerView * tCornerView;
	ICInstallationTypeScrollView * tScrollView;
	
	
	if ([currentHierarchyName_ isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER]==YES)
	{
		NSNumber * tNumber;
		int tInstallationTypeMode=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_ALLOW;
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_MODE];
	
		if (tNumber!=nil)
		{
			tInstallationTypeMode=[tNumber intValue];
		}
		
		if (tInstallationTypeMode==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_NEVER)
		{
			tVisible=NO;
		}
	}
	else
	{
		tVisible=NO;
	}
	
	tHeaderView=(ICInstallationTypeOutlineHeaderView *) [IBoutlineView_ headerView];
	
	if (tHeaderView!=nil)
	{
		[tHeaderView setDashedBorder:!tVisible];
	}
	
	tCornerView=(ICInstallationTypeCornerView *) [IBoutlineView_ cornerView];
	
	if (tCornerView!=nil)
	{
		[tCornerView setDashedBorder:!tVisible];
	}
	
	tScrollView=(ICInstallationTypeScrollView *) [IBoutlineView_ enclosingScrollView];
	
	if (tScrollView!=nil)
	{
		[tScrollView setDashedBorder:!tVisible];
	}
	
	tScrollView=(ICInstallationTypeScrollView *) [IBdescriptionTextView_ enclosingScrollView];
	
	if (tScrollView!=nil)
	{
		[tScrollView setDashedBorder:!tVisible];
	}
	
	[IBsplitView_ setNeedsDisplay:YES];
}

- (NSMutableArray *) currentHierarchy
{
	NSDictionary * tHierarchiesDictionary;
	NSMutableArray * tCurrentHierarchyArray=nil;
		
	tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
		
	if (tHierarchiesDictionary!=nil)
	{
		tCurrentHierarchyArray=[[tHierarchiesDictionary objectForKey:currentHierarchyName_] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST];
	}
	
	return tCurrentHierarchyArray;
}

#pragma mark -

- (void) viewWillBeRemoved:(id) sender
{
	// Archive Selection
	
	[self archiveSelection];

	// Refresh Source List
	
	[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_DOCUMENT_SOURCELIST_SHOULD_RELOAD
														object:[self document]];
}

#pragma mark -

- (void) prepareButtons:(NSArray *) inButtonsArray forLanguage:(NSString *) inLanguage
{
	[cachedButtons_ release];
	
	cachedButtons_=[inButtonsArray retain];
	
	if (inLanguage!=nil)
	{
		ICInstallerPluginLocalizer * tPluginLocalizer;
		NSButton * tButton;
		NSRect tFrame;
		CGFloat tMaxX;
		NSNumber * tNumber;
		int tInstallationTypeMode=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_ALLOW;
		
		tPluginLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
		
		// Customize
		
		tButton=[inButtonsArray objectAtIndex:0];
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_MODE];
		
		if (tNumber!=nil)
		{
			tInstallationTypeMode=[tNumber intValue];
		}
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"Standard Install"
				  forLanguage:inLanguage
					 inBundle:@"self"]];
			
		tFrame=[tButton frame];
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		[tButton setFrame:tFrame];
		
		if (tInstallationTypeMode==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_ALLOW)
		{
			[tButton setHidden:NO];
		}
		else
		{
			[tButton setHidden:YES];
		}
		
		// Save
		
		tButton=[inButtonsArray objectAtIndex:1];
		
		[tButton setHidden:YES];
		
		// Continue
		
		tButton=[inButtonsArray objectAtIndex:3];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"CONTINUE"
			  forLanguage:inLanguage
				 inBundle:@"Main"]];
	
		tFrame=[tButton frame];
		
		tMaxX=NSMaxX(tFrame);
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		tFrame.origin.x=tMaxX-NSWidth(tFrame);
		
		[tButton setFrame:tFrame];
		
		tMaxX=NSMinX(tFrame);
		
		// Go Back
		
		tButton=[inButtonsArray objectAtIndex:2];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"GOBACK"
			  forLanguage:inLanguage
				 inBundle:@"Main"]];
		
		tFrame=[tButton frame];
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		tFrame.origin.x=tMaxX-NSWidth(tFrame);
		
		[tButton setFrame:tFrame];
		
		[[tButton superview] setNeedsDisplay:YES];
	}
}

- (NSString *) paneTitleForLanguage:(NSString *) inLanguage
{
	NSString * tLocalizedPaneTitle=nil;
	
	if (inLanguage!=nil)
	{
		NSNumber * tNumber;
		int tInstallationTypeMode=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_ALLOW;
		ICInstallerPluginLocalizer * tPluginLocalizer;
		NSString * tLocalizedStringFormat=nil;
			
		tPluginLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
			
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_MODE];
		
		if (tNumber!=nil)
		{
			tInstallationTypeMode=[tNumber intValue];
		}
		
		// Customize
		
		if (tInstallationTypeMode==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_NEVER)
		{

#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5

			tLocalizedStringFormat=[tPluginLocalizer localizedString:@"StandardTitle"
													  forLanguage:inLanguage
														 inBundle:@"self"];

#else

			tLocalizedStringFormat=[tPluginLocalizer localizedString:@"StandardTitle"
													  forLanguage:inLanguage
														 inBundle:@"PackageSelection"];
#endif		
			
		}
		else
		{
			tLocalizedStringFormat=[tPluginLocalizer localizedString:@"CustomTitle"
													  forLanguage:inLanguage
														 inBundle:@"PackageSelection"];
		}
		
		if (tLocalizedStringFormat!=nil)
		{
			NSString * tMostAppropriateLocalizedTitle=nil;
				
			tMostAppropriateLocalizedTitle=[[NSFileManager defaultManager] displayNameAtPath:@"/"];
				
			if (tMostAppropriateLocalizedTitle!=nil)
			{
				tLocalizedPaneTitle=[NSString stringWithFormat:tLocalizedStringFormat,tMostAppropriateLocalizedTitle];
			}
		}
	}
	
	return tLocalizedPaneTitle;
}
/*
	NSString * tLocalizedPaneTitle=nil;
	NSString * tLocalizedStringFormat;
	
	tLocalizedStringFormat=[super paneTitleForLanguage:inLanguage];
	
	if (tLocalizedStringFormat!=nil)
	{
		NSString * tMostAppropriateLocalizedTitle=nil;
		NSDictionary * tTitleDictionary;
		NSMutableArray * tLocalizationsArray;
		
		tTitleDictionary=[presentationDictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_TITLE];
		
		if (tTitleDictionary!=nil)
		{
			tLocalizationsArray=[tTitleDictionary objectForKey:ICDOCUMENT_LOCALIZATIONS];
			
			if (tLocalizationsArray!=nil)
			{
				tMostAppropriateLocalizedTitle=[self localizedValueForLanguage:inLanguage inLocalizations:tLocalizationsArray];
			}
		}
		
		if (tMostAppropriateLocalizedTitle==nil)
		{
			// Use Project document name
			
			tLocalizationsArray=[ICTitleUtilities defaultLocalizationsArrayForDocument:document_];
			
			if (tLocalizationsArray!=nil)
			{
				tMostAppropriateLocalizedTitle=[self localizedValueForLanguage:inLanguage inLocalizations:tLocalizationsArray];
			}
		}
		
		if (tMostAppropriateLocalizedTitle!=nil)
		{
			tLocalizedPaneTitle=[NSString stringWithFormat:tLocalizedStringFormat,tMostAppropriateLocalizedTitle];
		}
	}
	
	return tLocalizedPaneTitle;
}*/

#pragma mark -

- (void) updateTree
{
	NSDictionary * tHierarchiesDictionary;
	NSArray * tRemovedPackages;
	NSDictionary * tUserInfo=nil;
	
	//NSLog(@"updateTree");
	
	[tree_ release];
	
	tree_=nil;
	
	tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
	
	tree_=[ICInstallationTypeTreeNode installationTypeTreeNodeWithArray:[[tHierarchiesDictionary objectForKey:currentHierarchyName_] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST]];
	
	[self refreshViewForLanguage:cachedLanguage_];
	
	// Refresh Source List
	
	tRemovedPackages=[[[tHierarchiesDictionary objectForKey:currentHierarchyName_] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED] allKeys];
	
	if (tRemovedPackages!=nil)
	{
		tUserInfo=[NSDictionary dictionaryWithObject:tRemovedPackages forKey:ICDOCUMENT_NOTIFICATION_HIERARCHY_PACKAGES_REMOVED_LIST];
	}
	
	[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_NOTIFICATION_DOCUMENT_SOURCELIST_SHOULD_RELOAD
														object:[self document]
													  userInfo:tUserInfo];
}

- (void) refreshViewForLanguage:(NSString *) inLanguage
{
	if (dictionary_!=nil && inLanguage!=nil)
	{
		NSTableColumn * tTableColumn = nil;
		NSCell * tCell;
		ICInstallerPluginLocalizer * tPluginLocalizer;
		NSString * tString;
		
		tPluginLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
		
		if (inLanguage!=cachedLanguage_)
		{
			[cachedLanguage_ release];
	
			cachedLanguage_=[inLanguage retain];
		}
		
		// Refresh the Headers
		
		tTableColumn=[IBoutlineView_ tableColumnWithIdentifier:@"Name"];
	 
		if (tTableColumn!=nil)
		{
			tCell=[tTableColumn headerCell];
			
			tString=[tPluginLocalizer localizedString:@"Package Name" forLanguage:inLanguage inBundle:@"self"];
			
			if (tString==nil)
			{
				tString=@"Package Name";
			}
			
			[tCell setTitle:tString];
		}
		
		tTableColumn=[IBoutlineView_ tableColumnWithIdentifier:@"Action"];
	 
		if (tTableColumn!=nil)
		{
			tCell=[tTableColumn headerCell];
			
			tString=[tPluginLocalizer localizedString:@"Package Action" forLanguage:inLanguage inBundle:@"self"];
			
			if (tString==nil)
			{
				tString=@"Action";
			}
			
			[tCell setTitle:tString];
		}
		
		tTableColumn=[IBoutlineView_ tableColumnWithIdentifier:@"Size"];
	 
		if (tTableColumn!=nil)
		{
			tCell=[tTableColumn headerCell];
			
			tString=[tPluginLocalizer localizedString:@"Package Size" forLanguage:inLanguage inBundle:@"self"];
			
			if (tString==nil)
			{
				tString=@"Size";
			}
			
			[tCell setTitle:tString];
		}
		
		// Refresh the contents of the outline view
		
		[IBoutlineView_ reloadData];
		
		// Refresh the Space Required and Remaining labels
		
		tString=[tPluginLocalizer localizedString:@"Space Required:" forLanguage:inLanguage inBundle:@"self"];
			
		if (tString==nil)
		{
			tString=@"Space Required:";
		}
		
		[IBspaceRequiredLabel_ setStringValue:tString];
	
		
		tString=[tPluginLocalizer localizedString:@"Remaining:" forLanguage:inLanguage inBundle:@"self"];
			
		if (tString==nil)
		{
			tString=@"Remaining:";
		}
		
		[IBremainingLabel_ setStringValue:tString];
		
		// Refresh the description
		
		[self outlineViewSelectionDidChange:nil];
	}
}

#pragma mark -

- (void)splitView:(NSSplitView *) inSplitView resizeSubviewsWithOldSize:(NSSize)oldSize
{
    NSArray * tSubviews;
    NSView * tViewTop,* tViewBottom;
    NSRect tTopFrame,tBottomFrame;
    NSRect tSplitViewFrame=[inSplitView frame]; 
    
    tSubviews=[inSplitView subviews];
    
    tViewTop=[tSubviews objectAtIndex:0];
    
    tTopFrame=[tViewTop frame];
    
    tViewBottom=[tSubviews objectAtIndex:1];
    
    tBottomFrame=[tViewBottom frame];
        
	tTopFrame.size.height=NSHeight(tSplitViewFrame)-[inSplitView dividerThickness]-NSHeight(tBottomFrame);
	
	if (NSHeight(tTopFrame)<ICINSTALLATIONTYPEVIEW_MIN_HEIGHT)
	{
		tTopFrame.size.height=ICINSTALLATIONTYPEVIEW_MIN_HEIGHT;
		
		tBottomFrame.size.height=NSHeight(tSplitViewFrame)-[inSplitView dividerThickness]-NSHeight(tTopFrame);
		
		if (NSHeight(tBottomFrame)<0)
		{
			tBottomFrame.size.height=0;
		}
	}
	
	tBottomFrame.size.width=NSWidth(tSplitViewFrame);
	
	tBottomFrame.origin.x=0;
	
	tBottomFrame.origin.y=NSHeight(tTopFrame)+[inSplitView dividerThickness];
	
	[tViewBottom setFrame:tBottomFrame];
    
    tTopFrame.size.width=NSWidth(tSplitViewFrame);
        
    tTopFrame.origin.x=0;
	
	
        
    [tViewTop setFrame:tTopFrame];
}

- (BOOL) splitView:(NSSplitView *)sender canCollapseSubview:(NSView *)subview
{
    return NO;
}

- (CGFloat) splitView:(NSSplitView *) inSplitView constrainMinCoordinate:(CGFloat)proposedMin ofSubviewAt:(NSInteger) inOffset
{
    if (inOffset==0)
	{
		return ICINSTALLATIONTYPEVIEW_MIN_HEIGHT;
	}
	
	return 0;
}

#pragma mark -

- (BOOL) outlineView:(NSOutlineView *) inOutlineView validateAction:(SEL) inSelector
{
	if (inSelector==@selector(delete:))
	{
		NSIndexSet * tIndexSet;
		
		// We can remove everything
	
		tIndexSet=[inOutlineView selectedRowIndexes];
		
		if (tIndexSet!=nil && [tIndexSet count]>0)
		{
			return YES;
		}
	}
	
	return NO;
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{
	if (choiceDependencyEditionOn_==NO)
	{
		SEL tAction;
		NSIndexSet * tIndexSet;
		
		tAction=[inMenuItem action];
		
		tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
		if (tAction==@selector(switchInstallationTypeMode:))
		{
			return YES;
		}
		else if (tAction==@selector(removeFromHierarchy:))
		{
			// We can remove everything
		
			if (tIndexSet!=nil && [tIndexSet count]>0)
			{
				return YES;
			}
		}
		else if (tAction==@selector(showHideIndentationColumn:))
		{
			// Set the Checkmark or not
			
			NSTableColumn * tColumn;
			
			tColumn=[IBoutlineView_ tableColumnWithIdentifier:ICPRESENTATIONVIEWINSTALLATIONTYPE_INDENTATION_COLUMNIDENTIFIER];
			
			if (tColumn!=nil)
			{
				[inMenuItem setState:NSOnState];
			}
			else
			{
				[inMenuItem setState:NSOffState];
			}
		
			return YES;
		}
		else if (tAction==@selector(switchHierarchy:))
		{
			return YES;
		}
		else if (tAction==@selector(nothing:))
		{
			if ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)
			{
				[inMenuItem setTitle:NSLocalizedStringFromTable(@"Duplicate As Hierarchy",@"Presentation_InstallationType",@"No comment")];
			}
			else
			{
				[inMenuItem setTitle:NSLocalizedStringFromTable(@"Add Hierarchy",@"Presentation_InstallationType",@"No comment")];
			}
			
			return YES;
		}
		else if (tAction==@selector(addHierarchy:) ||
				 tAction==@selector(duplicateHierarchy:))
				
		{
			if ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)
			{
				[inMenuItem setAction:@selector(duplicateHierarchy:)];
			}
			else
			{
				[inMenuItem setAction:@selector(addHierarchy:)];
			}
			
			return YES;
		}
		else if (tAction==@selector(removeHierarchy:))
		{
			NSDictionary * tHierarchiesDictionary;
			
			tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
			
			if (tHierarchiesDictionary!=nil && [tHierarchiesDictionary count]>1)
			{
				return YES;
			}
		}
		else if (tAction==@selector(remove:))
		{
			// We can remove everything
		
			if (tIndexSet!=nil && [tIndexSet count]>0)
			{
				// A COMPLETER
		
				return YES;
			}
		}
		else if (tAction==@selector(ungroup:))
		{
			if (tIndexSet!=nil && [tIndexSet count]==1)
			{
				NSUInteger tRowIndex;
				
				tRowIndex=[tIndexSet firstIndex];
				
				ICInstallationTypeTreeNode * tNode;
					
				tNode=(ICInstallationTypeTreeNode *) [IBoutlineView_ itemAtRow:tRowIndex];
					
				if (tNode!=nil)
				{
					ICInstallationTypeNodeData * tNodeData;
					
					tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tNode);
						
					if ([tNodeData type]!=kInstallationTypeRootNode)
					{
						NSDictionary * tItemDictionary;
					
						tItemDictionary=[tNodeData dictionary];
						
						if (tItemDictionary!=nil)
						{
							NSNumber * tType;
							
							tType=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
							
							if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
							{
								NSDictionary * tOptionsDictionary;
							
								tOptionsDictionary=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
							
								if (tOptionsDictionary!=nil)
								{
									NSNumber * tOptionValue;
								
									tOptionValue=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
								
									if ([tOptionValue boolValue]==NO)
									{
										return YES;
									}
								}
							}
						}
					}
				}
			}
		}
		else if (tAction==@selector(group:))
		{
			// We can only group packages or groups at the same level and packages that are not inside a hidden group
			
			if (tIndexSet!=nil && [tIndexSet count]>=1)
			{
				NSUInteger tRowIndex;
				ICInstallationTypeTreeNode * tSharedParentNode=nil;
				
				tRowIndex=[tIndexSet firstIndex];
				
				while (tRowIndex!=NSNotFound)
				{
					ICInstallationTypeTreeNode * tNode;
					
					tNode=(ICInstallationTypeTreeNode *) [IBoutlineView_ itemAtRow:tRowIndex];
					
					if (tNode!=nil)
					{
						// Check the parent node
						
						ICInstallationTypeTreeNode * tParentNode;
						ICInstallationTypeNodeData * tParentNodeData;
						
						tParentNode=(ICInstallationTypeTreeNode *) [tNode nodeParent];
						
						if (tSharedParentNode==nil)
						{
							tSharedParentNode=tParentNode;
						}
						else
						{
							if (tSharedParentNode!=tParentNode)
							{
								return NO;
							}
						}
						
						tParentNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tParentNode);
						
						if ([tParentNodeData type]!=kInstallationTypeRootNode)
						{
							NSDictionary * tParentItemDictionary;
						
							tParentItemDictionary=[tParentNodeData dictionary];
							
							if (tParentItemDictionary!=nil)
							{
								NSNumber * tType;
								
								tType=[tParentItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
								
								if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
								{
									NSDictionary * tOptionsDictionary;
								
									tOptionsDictionary=[tParentItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
								
									if (tOptionsDictionary!=nil)
									{
										NSNumber * tOptionValue;
									
										tOptionValue=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
									
										if ([tOptionValue boolValue]==YES)
										{
											return NO;
										}
									}
								}
							}
						}
					}
					
					
					
					tRowIndex=[tIndexSet indexGreaterThanIndex:tRowIndex];
				}
				
				return YES;
			}
		}
		else if (tAction==@selector(merge:))
		{
			// We can only merge packages that are at the same level and not inside a hidden group
			
			if (tIndexSet!=nil && [tIndexSet count]>1)
			{
				NSUInteger tRowIndex;
				ICInstallationTypeTreeNode * tSharedParentNode=nil;
				
				tRowIndex=[tIndexSet firstIndex];
				
				while (tRowIndex!=NSNotFound)
				{
					ICInstallationTypeTreeNode * tNode;
					
					tNode=(ICInstallationTypeTreeNode *) [IBoutlineView_ itemAtRow:tRowIndex];
					
					if (tNode!=nil)
					{
						ICInstallationTypeNodeData * tNodeData;
						NSDictionary * tItemDictionary;
						ICInstallationTypeTreeNode * tParentNode;
						ICInstallationTypeNodeData * tParentNodeData;
						
						tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tNode);
						
						tItemDictionary=[tNodeData dictionary];
							
						if (tItemDictionary!=nil)
						{
							NSNumber * tType;
								
							tType=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
								
							if ([tType intValue]!=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
							{
								return NO;
							}
						}
						
						// Check the parent node
						
						tParentNode=(ICInstallationTypeTreeNode *) [tNode nodeParent];
						
						if (tSharedParentNode==nil)
						{
							tSharedParentNode=tParentNode;
						}
						else
						{
							if (tSharedParentNode!=tParentNode)
							{
								return NO;
							}
						}
						
						tParentNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tParentNode);
						
						if ([tParentNodeData type]!=kInstallationTypeRootNode)
						{
							NSDictionary * tParentItemDictionary;
						
							tParentItemDictionary=[tParentNodeData dictionary];
							
							if (tParentItemDictionary!=nil)
							{
								NSNumber * tType;
								
								tType=[tParentItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
								
								if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
								{
									NSDictionary * tOptionsDictionary;
								
									tOptionsDictionary=[tParentItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
								
									if (tOptionsDictionary!=nil)
									{
										NSNumber * tOptionValue;
									
										tOptionValue=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
									
										if ([tOptionValue boolValue]==YES)
										{
											return NO;
										}
									}
								}
							}
						}
					}
					
					tRowIndex=[tIndexSet indexGreaterThanIndex:tRowIndex];
				}
				
				return YES;
			}
		}
		else if (tAction==@selector(separate:))
		{
			if (tIndexSet!=nil && [tIndexSet count]==1)
			{
				NSUInteger tRowIndex;
				
				tRowIndex=[tIndexSet firstIndex];
				
				ICInstallationTypeTreeNode * tNode;
					
				tNode=(ICInstallationTypeTreeNode *) [IBoutlineView_ itemAtRow:tRowIndex];
					
				if (tNode!=nil)
				{
					ICInstallationTypeNodeData * tNodeData;
					
					tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tNode);
						
					if ([tNodeData type]!=kInstallationTypeRootNode)
					{
						NSDictionary * tChoiceDictionary;
					
						tChoiceDictionary=[tNodeData dictionary];
						
						if (tChoiceDictionary!=nil)
						{
							NSNumber * tType;
							
							tType=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
							
							if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
							{
								NSDictionary * tOptionsDictionary;
							
								tOptionsDictionary=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
							
								if (tOptionsDictionary!=nil)
								{
									NSNumber * tOptionValue;
								
									tOptionValue=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
								
									if (tOptionValue!=nil && [tOptionValue boolValue]==YES)
									{
										return YES;
									}
								}
							}
						}
					}
				}
			}
		}
		
	}
	
	return NO;
}

- (void) removeFromHierarchy:(id) sender
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
	if (tIndexSet!=nil)
	{
		NSString * tAlertTitle=@"";
		NSString * tAlertMessage=@"";
		
		if ([tIndexSet count]==1)
		{
			tAlertTitle=NSLocalizedString(@"Do you really want to remove this item?",@"No comment");
			
			tAlertMessage=NSLocalizedStringFromTable(@"This will also remove any dependency on this item for other items. This cannot be undone.",@"Presentation_InstallationType",@"No comment");
		}
		else
		{
			tAlertTitle=NSLocalizedString(@"Do you really want to remove these items?",@"No comment");
			
			tAlertMessage=NSLocalizedStringFromTable(@"This will also remove any dependency on these items for other items. This cannot be undone.",@"Presentation_InstallationType",@"No comment");
		}
	
		NSBeginAlertSheet(tAlertTitle,
						  NSLocalizedString(@"Remove",@"No comment"),
						  NSLocalizedString(@"Cancel",@"No comment"),
						  nil,
						  [document_ windowForSheet],
						  self,
						  @selector(removeFromHierarchyDidEnd:returnCode:contextInfo:),
						  nil,
						  NULL,
						  tAlertMessage);
	}
}

- (void) outlineView:(NSOutlineView *) inOutlineView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inConfirmationRequired==YES)
	{
		[self removeFromHierarchy:nil];
	}
	else
	{
		[self removeFromHierarchyDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
	}
}

- (void) removeFromHierarchyDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
	if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
	
		tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
		if (tIndexSet!=nil)
		{
			NSDictionary * tHierarchiesDictionary;
			
			tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
						
			if (tHierarchiesDictionary!=nil)
			{
				NSMutableDictionary * tCurrentHierarchy;
				
				tCurrentHierarchy=[tHierarchiesDictionary objectForKey:currentHierarchyName_];
				
				if (tCurrentHierarchy!=nil)
				{
					NSMutableArray * tItemsDictionarySelection;
					NSMutableArray * tItemsSelection;
			
					tItemsSelection=[NSMutableArray array];
					
					tItemsDictionarySelection=[NSMutableArray array];
					
					if (tItemsSelection!=nil && tItemsDictionarySelection!=nil)
					{
						NSUInteger tRowIndex;
						NSMutableArray * tFlattenedItemsList;
						NSUInteger i,tCount;
						NSDictionary * tChoiceDictionary;
						
						tRowIndex=[tIndexSet firstIndex];
					
						while (tRowIndex!=NSNotFound)
						{
							ICInstallationTypeTreeNode * tNode;
							
							tNode=(ICInstallationTypeTreeNode *) [IBoutlineView_ itemAtRow:tRowIndex];
							
							if (tNode!=nil)
							{
								ICInstallationTypeNodeData * tNodeData;
								
								[tItemsSelection addObject:tNode];
								
								tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tNode);
								
								tChoiceDictionary=[tNodeData dictionary];
								
								[tItemsDictionarySelection addObject:tChoiceDictionary];
							}
							
							tRowIndex=[tIndexSet indexGreaterThanIndex:tRowIndex];
						}
						
						// Flatten the list (groups included)
						
						tFlattenedItemsList=[ICInstallationTypeUtilities flattenItemsFromItemsList:tItemsDictionarySelection];
						
						// Remove the items from the disclosed state if needed
						
						if (tFlattenedItemsList!=nil)
						{
							NSMutableDictionary * tDisclosedDictionary;
					
							tDisclosedDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,currentHierarchyName_]];
					
							if (tDisclosedDictionary!=nil)
							{
								NSEnumerator * tEnumerator;
							
								tEnumerator=[tFlattenedItemsList objectEnumerator];
								
								if (tEnumerator!=nil)
								{
									while (tChoiceDictionary=[tEnumerator nextObject])
									{
										NSString * tChoiceUUID;
										
										tChoiceUUID=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
										
										if (tChoiceUUID!=nil)
										{
											[tDisclosedDictionary removeObjectForKey:tChoiceUUID];
										}
									}
								}
							}
						}
						
						// We remove the items from the LIST hierarchy
						
						tCount=[tItemsSelection count];
						
						if (tCount>0)
						{
							NSMutableArray * tCurrentListArray=nil;
							
							// Remove the items from LIST
							
							tCurrentListArray=[tCurrentHierarchy objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST];
							
							if (tCurrentListArray!=nil)
							{
								for(i=tCount;i>0;i--)
								{
									ICInstallationTypeTreeNode * tNode;
								
									tNode=(ICInstallationTypeTreeNode *) [tItemsSelection objectAtIndex:i-1];
									
									if (tNode!=nil)
									{
										ICInstallationTypeNodeData * tNodeData;
										NSDictionary * tDictionary;
										
										tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tNode);
									
										tDictionary=[tNodeData dictionary];
										
										if (tDictionary!=nil)
										{
											ICInstallationTypeTreeNode * tParentNode;
											
											tParentNode=(ICInstallationTypeTreeNode *) [tNode nodeParent];
											
											if (tParentNode!=nil)
											{
												ICInstallationTypeNodeData * tParentNodeData;
							
												tParentNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tParentNode);
												
												if (tParentNodeData!=nil)
												{
													if ([tParentNodeData type]!=kInstallationTypeRootNode)
													{
														NSDictionary * tParentDictionary;
														
														tParentDictionary=[tParentNodeData dictionary];
														
														if (tParentDictionary!=nil)
														{
															NSMutableArray * tChildren;
															
															tChildren=[tParentDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
															
															if (tChildren!=nil)
															{
																[tChildren removeObject:tDictionary];
															
																if ([tChildren count]==0)
																{
																	NSDictionary * tParentOptionsDictionary;
													
																	tParentOptionsDictionary=[tParentDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
																	
																	if (tParentOptionsDictionary!=nil)
																	{
																		NSNumber * tNumber;
																			
																		tNumber=[tParentOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
														
																		if (tNumber!=nil && [tNumber boolValue]==YES)
																		{
																			// We need to remove the parent too
																			
																			if ([tFlattenedItemsList containsObject:tParentDictionary]==NO)
																			{
																				[tFlattenedItemsList addObject:tParentDictionary];
																			}
																			
																			[tItemsSelection insertObject:tParentNode atIndex:i-1];
																			
																			i++;
																		}
																	}
																}
															}
														}
													}
													else
													{
														// Just remove from array
														
														[tCurrentListArray removeObject:tDictionary];
													}
												}
											}
										}
									}
								}
							}
							
							if (tFlattenedItemsList!=nil)
							{
								NSMutableDictionary * tCurrentRemovedGroup=nil;
								NSMutableArray * tAllRemovedChoicesUUID;
								
								// We remove all the dependencies to the removed choices
							
								tAllRemovedChoicesUUID=[ICInstallationTypeUtilities allChoicesUUIDForChoices:tFlattenedItemsList];
								
								if (tAllRemovedChoicesUUID!=nil && tCurrentListArray!=nil)
								{
									[ICInstallationTypeUtilities removeDependenciesToChoices:tAllRemovedChoicesUUID fromHierarchy:tCurrentListArray];
								}
								
								// We move the package items to the the REMOVED list
					
								tCurrentRemovedGroup=[tCurrentHierarchy objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED];
						
								if (tCurrentRemovedGroup==nil)
								{
									tCurrentRemovedGroup=[NSMutableDictionary dictionary];
									
									if (tCurrentRemovedGroup!=nil)
									{
										[tCurrentHierarchy setObject:tCurrentRemovedGroup forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED];
									}
									else
									{
										// A COMPLETER (memoire)
									}
								}
						
								if (tCurrentRemovedGroup!=nil)
								{
									NSEnumerator * tItemsDictionaryEnumerator;
									
									tItemsDictionaryEnumerator=[tFlattenedItemsList objectEnumerator];
									
									 if (tItemsDictionaryEnumerator!=nil)
									 {
										while (tChoiceDictionary=[tItemsDictionaryEnumerator nextObject])
										{
											NSNumber * tType;
								
											tType=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
											
											if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
											{
												NSString * tPackageUUID;
												
												tPackageUUID=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_PACKAGE_UUID];
												
												if (tPackageUUID!=nil)
												{
													if ([tCurrentRemovedGroup objectForKey:tPackageUUID]==nil)
													{
														NSMutableDictionary * tOptionsDictionary;
														
														// Set the State to Selected
														
														tOptionsDictionary=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
														
														if (tOptionsDictionary!=nil)
														{
															[tOptionsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
														
															[tOptionsDictionary removeObjectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
														}
														
														[tCurrentRemovedGroup setObject:tChoiceDictionary forKey:tPackageUUID];
													}
												}
												else
												{
													// A COMPLETER (corruption)
												}
											}
										}
									 }
								}
							}
						}
					}
				}
				
				// Refresh the tree
				
				[IBoutlineView_ deselectAll:self];
				
				[self updateTree];
	
				[self restoreDisclosedStates];
				
				// Notify document changed
				
				[self noteDocumentChange];
			}
		}
	}
}

#pragma mark -

- (void) embedSelectionInGroup:(BOOL) isGroup named:(NSString *) inName
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBoutlineView_ selectedRowIndexes];
	
	if (tIndexSet!=nil)
	{
		NSUInteger tRowIndex;
		ICInstallationTypeTreeNode * tNode;
		NSUInteger tInsertionIndex;
		
		tRowIndex=[tIndexSet firstIndex];
			
		// Find the parent node and add a group
		
		tNode=(ICInstallationTypeTreeNode *) [IBoutlineView_ itemAtRow:tRowIndex];
		
		if (tNode!=nil)
		{
			ICInstallationTypeTreeNode * tParentNode;
			ICInstallationTypeNodeData * tParentNodeData;
			NSMutableArray * tChildrenArray=nil;
			
			tParentNode=(ICInstallationTypeTreeNode *) [tNode nodeParent];
			
			tInsertionIndex=[tParentNode indexOfChild:tNode];
			
			tParentNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tParentNode);
					
			if ([tParentNodeData type]==kInstallationTypeRootNode)
			{
				tChildrenArray=[self currentHierarchy];
			}
			else
			{
				NSDictionary * tParentItemDictionary;
				
				tParentItemDictionary=[tParentNodeData dictionary];
						
				if (tParentItemDictionary!=nil)
				{
					tChildrenArray=[tParentItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
				}
			}
			
			if (tChildrenArray!=nil)
			{
				// Add the Group item
				
				NSMutableArray * tGroupChildrenArray;
				
				tGroupChildrenArray=[NSMutableArray array];
				
				if (tGroupChildrenArray!=nil)
				{
					NSMutableDictionary * tGroupDictionary;
					NSMutableDictionary * tDisclosedDictionary;
					NSString * tChoiceUUID;
					
					tDisclosedDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,currentHierarchyName_]];
					
					tChoiceUUID=[ICUUIDFactory string];
					
					[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tChoiceUUID];
					
					tGroupDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE,
																					   tChoiceUUID,ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID,
																					   tGroupChildrenArray,ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN,
																					   nil];
																				   
					if (tGroupDictionary!=nil)
					{
						NSMutableDictionary * tGroupOptionsDictionary;
						NSMutableArray * tLocalizationsArray;
						int tState;
						NSMutableArray * tMergedChoicesUUID;
						
						if (isGroup==YES)
						{
							tState=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_ENABLED_ALWAYS;
						}
						else
						{
							tState=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED;
						}
						
						// OPTIONS
						
						tGroupOptionsDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:tState],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE,
																								  [NSNumber numberWithBool:NO],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN,
																								  [NSNumber numberWithBool:!isGroup],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN,
																								  nil];
																								  
						if (tGroupOptionsDictionary!=nil)
						{
							[tGroupDictionary setObject:tGroupOptionsDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
						}
						else
						{
							NSBeep();
							
							// A COMPLETER
						
							return;
						}
						
						// TITLE
						
						tLocalizationsArray=[ICLocalizationUtilities localizationsArrayWithValue:inName forLanguage:@"English"];

						if (tLocalizationsArray!=nil)
						{
							[tGroupDictionary setObject:tLocalizationsArray forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TITLE];
						}
						else
						{
							// A COMPLETER
						}
						
						// DESCRIPTION
										
						tLocalizationsArray=[NSMutableArray array];//[ICLocalizationUtilities localizationsArrayWithValue:@"" forLanguage:@"English"];

						if (tLocalizationsArray!=nil)
						{
							[tGroupDictionary setObject:tLocalizationsArray forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_DESCRIPTION];
						}
						else
						{
							NSLog(@"[ICPresentationViewInstallationTypeController embedSelectionInGroup:named:] Low Memory");// A COMPLETER
						}
						
						// TOOLTIP
						
						/*tLocalizationsArray=[ICLocalizationUtilities localizationsArrayWithValue:@"" forLanguage:@"English"];

						if (tLocalizationsArray!=nil)
						{
							[tGroupDictionary setObject:tLocalizationsArray forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TOOLTIP];
						}
						else
						{
							// A COMPLETER
						}*/
					
					
						[tChildrenArray insertObject:tGroupDictionary atIndex:tInsertionIndex];
					
						
						tMergedChoicesUUID=[NSMutableArray array];
						
						// We need to remove the children and add them to the new group
						
						while (tRowIndex!=NSNotFound)
						{
							tNode=(ICInstallationTypeTreeNode *) [IBoutlineView_ itemAtRow:tRowIndex];
				
							if (tNode!=nil)
							{
								ICInstallationTypeNodeData * tNodeData;
								NSMutableDictionary * tDictionary;
								
								tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tNode);
								
								tDictionary=[tNodeData dictionary];
								
								if (tDictionary!=nil)
								{
									if (isGroup==NO)
									{
										NSMutableDictionary * tOptionsDictionary;
										
										// We need to remove references of this choice
										
										tOptionsDictionary=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
										
										if (tOptionsDictionary!=nil)
										{
											[tOptionsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE];
											
											[tOptionsDictionary removeObjectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_DEPENDENCY];
											
											[tOptionsDictionary setObject:[NSNumber numberWithBool:NO] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN];
										}
										
										tChoiceUUID=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
										
										if (tChoiceUUID!=nil)
										{
											[tMergedChoicesUUID addObject:tChoiceUUID];
										}
									}
									
									[tGroupChildrenArray addObject:tDictionary];
									
									[tChildrenArray removeObject:tDictionary];
								}
							}
							
							tRowIndex=[tIndexSet indexGreaterThanIndex:tRowIndex];
						}
						
						if (isGroup==NO)
						{
							// We need to remove references to these choices
										
							[ICInstallationTypeUtilities removeDependenciesToChoices:tMergedChoicesUUID fromHierarchy:[self currentHierarchy]];
						}
										
						
						[IBoutlineView_ deselectAll:self];
	
						[self updateTree];
						
						// Find the tree for the new group
						
						//[IBoutlineView_ expandItem:[IBoutlineView_ itemAtRow:tFirstRowIndex]];
						
						[self restoreDisclosedStates];
						
						//[self choiceDidExpand:[IBoutlineView_ itemAtRow:tFirstRowIndex]];
						
						[self noteDocumentChange];
					}
				}
			
			}
		}
	}
}

- (void) unembed
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBoutlineView_ selectedRowIndexes];
	
	if (tIndexSet!=nil && [tIndexSet count]==1)
	{
		NSUInteger tRowIndex;
		ICInstallationTypeTreeNode * tNode;
		
		tRowIndex=[tIndexSet firstIndex];
			
		// Find the node
		
		tNode=(ICInstallationTypeTreeNode *) [IBoutlineView_ itemAtRow:tRowIndex];
		
		if (tNode!=nil)
		{
			ICInstallationTypeTreeNode * tParentNode;
			ICInstallationTypeNodeData * tParentNodeData;
			NSMutableArray * tParentChildrenArray=nil;
			
			tParentNode=(ICInstallationTypeTreeNode *) [tNode nodeParent];
			
			tParentNodeData=INSTALLATION_TYPE_NODE_DATA(tParentNode);
					
			if ([tParentNodeData type]==kInstallationTypeRootNode)
			{
				tParentChildrenArray=[self currentHierarchy];
			}
			else
			{
				NSDictionary * tParentItemDictionary;
				
				tParentItemDictionary=[tParentNodeData dictionary];
						
				if (tParentItemDictionary!=nil)
				{
					tParentChildrenArray=[tParentItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
				}
			}
			
			if (tParentChildrenArray!=nil)
			{
				ICInstallationTypeNodeData * tNodeData;
				NSMutableArray * tChildrenArray;
				NSEnumerator * tEnumerator;
				NSMutableDictionary * tChoiceDictionary;
				NSString * tChoiceUUID;
				
				tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tNode);
				
				tChoiceDictionary=[tNodeData dictionary];
				
				[tChoiceDictionary retain];
				
				tChildrenArray=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
				
				// Remove the group from the hierarchy
				
				tRowIndex=[tParentChildrenArray indexOfObject:tChoiceDictionary];
				
				[tParentChildrenArray removeObjectAtIndex:tRowIndex];
				
				// Shift the items to the left
				
				tEnumerator=[tChildrenArray objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tChildChoiceDictionary;
					
					while (tChildChoiceDictionary=[tEnumerator nextObject])
					{
						[tParentChildrenArray insertObject:tChildChoiceDictionary atIndex:tRowIndex];
						
						tRowIndex++;
					}
					
					[tChildrenArray removeAllObjects];
				}
				
				tChoiceUUID=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
				
				if (tChoiceUUID!=nil)
				{
					NSMutableDictionary * tDisclosedDictionary;
					
					// Remove the dependencies on this group
				
					[ICInstallationTypeUtilities removeDependenciesToChoices:[NSMutableArray arrayWithObject:tChoiceUUID] fromHierarchy:[self currentHierarchy]];
				
					// Remove the item from the disclosed item if needed
				
					tDisclosedDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,currentHierarchyName_]];
					
					if (tDisclosedDictionary!=nil)
					{
						[tDisclosedDictionary removeObjectForKey:tChoiceUUID];
					}
				}
				
				[tChoiceDictionary release];	// A TESTER
				
				// Refresh the list
				
				[IBoutlineView_ deselectAll:self];

				[self updateTree];
				
				// Restore the disclosed state
				
				[self restoreDisclosedStates];
				
				[self noteDocumentChange];
			}
		}
	}
}

- (IBAction) group:(id) sender
{
	[self embedSelectionInGroup:YES named:@"untitled group"];
}

- (IBAction) ungroup:(id) sender
{
	[self unembed];
}

- (IBAction) merge:(id) sender
{
	[self embedSelectionInGroup:NO named:@"untitled item"];
}

- (IBAction) separate:(id) sender
{
	[self unembed];
}

#pragma mark -

- (void) switchShowPackagesName
{
	cachedShowPackagesName_=!cachedShowPackagesName_;
	
	[document_ setDocumentDefault:[NSNumber numberWithBool:cachedShowPackagesName_] forKey:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_SHOWPACKAGES];
	
	[(ICInstallationTypeCornerView *) [IBoutlineView_ cornerView] showPackagesName:cachedShowPackagesName_];
	
	[IBoutlineView_ reloadData];
}

#pragma mark -

- (id)outlineView:(NSOutlineView *) inOutlineView child:(NSInteger)inIndex ofItem:(id) inItem
{
    if (inOutlineView==IBoutlineView_)
	{
		if (tree_!=nil)
		{
			return [SAFENODE(inItem) childAtIndex:inIndex];
		}
    }
	
    return nil;
}

- (BOOL)outlineView:(NSOutlineView *) inOutlineView isItemExpandable:(id) inItem
{
    if (inOutlineView==IBoutlineView_)
	{
		return ![INSTALLATION_TYPE_NODE_DATA(inItem) isLeaf];
	}
	
	return NO;
}

- (NSInteger)outlineView:(NSOutlineView *) inOutlineView numberOfChildrenOfItem:(id) inItem
{
    if (inOutlineView==IBoutlineView_)
	{
		if (tree_!=nil)
		{
			return [SAFENODE(inItem) numberOfChildren];
		}
    }
	
    return 0;
}

- (id) outlineView:(NSOutlineView *) inOutlineView objectValueForTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
    if (inOutlineView==IBoutlineView_)
	{
		if (tree_!=nil && inItem!=nil)
		{
			ICInstallationTypeNodeData * tNodeData;
			
			tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(inItem);
			
			if (tNodeData!=nil)
			{
				NSString * tColumnIdentifier;
				tColumnIdentifier=[inTableColumn identifier];
				
				if (tColumnIdentifier!=nil)
				{
					NSDictionary * tDictionary;
			
					tDictionary=[tNodeData dictionary];
					
					if ([tColumnIdentifier isEqualToString: @"Name"])
					{
						int tState;
						
						tState=[inItem selectedState];
						
						//NSLog(@"Row:%d State:%d",[inOutlineView rowForItem:inItem],[inItem selectedState]);
						
						if (tState==kInstallationTypeDependentState)
						{
							tState=kInstallationTypeOffState;
						}
						
						return [NSNumber numberWithInt:tState];
					}
					else if ([tColumnIdentifier isEqualToString: @"Indentation"])
					{
						if ([((ICInstallationTypeTreeNode *) inItem) isInHiddenGroup]==NO)
						{
							return [((TreeNode *) inItem) indentationString];
						}
					}
					else if ([tColumnIdentifier isEqualToString: @"Action"])
					{
						NSNumber * tNumber;
						int tType;
						
						tNumber=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
						
						tType=[tNumber intValue];
						
						if (tType==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
						{
							// Is it a real group?
							
							NSDictionary * tOptionsDictionary;
					
							tOptionsDictionary=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
							
							if (tOptionsDictionary!=nil)
							{
								NSNumber * tNumber;
									
								tNumber=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
								
								if (tNumber==nil || [tNumber boolValue]==NO)
								{
									return @"";
								}
							}
						}
						else if (tType==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
						{
							// Is it inside a hidden group?
							
							if ([((ICInstallationTypeTreeNode *) inItem) isInHiddenGroup]==YES)
							{
								return @"";
							}
						}
						
						return @"-";
					}
				}
			}
		}
	}
	
	return nil;
}

- (void) outlineView:(NSOutlineView *) inOutlineView willDisplayCell:(NSCell *) inCell forTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{
	if (inOutlineView==IBoutlineView_)
	{
		if (tree_!=nil && inItem!=nil)
		{
			ICInstallationTypeNodeData * tNodeData;
			
			tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(inItem);
			
			if (tNodeData!=nil)
			{
				NSString * tColumnIdentifier;
				
				tColumnIdentifier=[inTableColumn identifier];
				
				if (tColumnIdentifier!=nil)
				{
					NSDictionary * tDictionary;
					
					
					tDictionary=[tNodeData dictionary];
			
					if ([tColumnIdentifier isEqualToString: @"Name"])
					{
						NSArray * tTitleLocalizationsArray;
						ICInstallationTypeCheckBoxCell * tButtonCell;
						NSNumber * tType;
						BOOL isPackageInHiddenGroup=NO;
						BOOL isInvisible=NO;
						NSString * tLocalizedTitle=nil;
						BOOL tUsePackageName=NO;
						int tState;
						
						tButtonCell= (ICInstallationTypeCheckBoxCell *) inCell;
					
						tType=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
						
						if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
						{
							isPackageInHiddenGroup=[inItem isInHiddenGroup];
							
							if (isPackageInHiddenGroup==YES)
							{
								[tButtonCell setImagePosition:NSNoImage];
							}
							else
							{
								[tButtonCell setImagePosition:NSImageLeft];
							}
						}
						else
						{
							[tButtonCell setImagePosition:NSImageLeft];
						}
						
						// Enable or not
						
						[tButtonCell setEnabled:((choiceDependencyEditionOn_==NO) && [inItem isEnabled])];
						
						// Title
						
						// If it's a package inside a hidden group, we display the name of the package
						
						if (isPackageInHiddenGroup==YES || cachedShowPackagesName_==YES)
						{
							tUsePackageName=YES;
						}
						else
						{
							tTitleLocalizationsArray=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TITLE];
							
							if (tTitleLocalizationsArray!=nil)
							{
								tLocalizedTitle=[ICLocalizationUtilities localizedValueForLanguage:cachedLanguage_ inLocalizations:tTitleLocalizationsArray lookForBestMatch:NO];
								
								if (tLocalizedTitle==nil)
								{
									tUsePackageName=YES;
								}
							}
						}
						
						if (tUsePackageName==YES)
						{
							// We need to get the Package ID
							
							NSString * tPackageUUID;
							
							tPackageUUID=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_PACKAGE_UUID];
							
							if (tPackageUUID!=nil)
							{
								NSDictionary * tPackageDictionary;
								
								tPackageDictionary=[document_ dictionaryOfPackageWithUUID:tPackageUUID];
								
								if (tPackageDictionary!=nil)
								{
									NSDictionary * tPackageSettingsDictionary;
							
									tPackageSettingsDictionary=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS];
									
									tLocalizedTitle=[tPackageSettingsDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME];
								}
							}
						}
						
						
						if (tLocalizedTitle==nil)
						{
							tLocalizedTitle=[NSString stringWithFormat:@"Choice %@",[((TreeNode *) inItem) indentationString]];
							
							//tLocalizedTitle=@"";
						}
						
						[tButtonCell setTitle:tLocalizedTitle];
						
						// Check if the item is invisible or has an invisible parent
								
						isInvisible=[inItem isInvisible];
						
						[tButtonCell setInvisible:isInvisible];
						
						// Dependent
						
						tState=[inItem selectedState];
						
						[tButtonCell setDependent:(tState==kInstallationTypeDependentState)];
					}
				}
			}
		}
	}
}

/*- (NSString *) outlineView:(NSOutlineView *) inOutlineView toolTipForCell:(NSCell *) inCell rect:(NSRectPointer) inRect tableColumn:(NSTableColumn *) inTableColumn item:(id) inItem mouseLocation:(NSPoint)mouseLocation
{
	if (inOutlineView==IBoutlineView_)
	{
		if (tree_!=nil && inItem!=nil)
		{
			ICInstallationTypeNodeData * tNodeData;
			
			tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(inItem);
			
			if (tNodeData!=nil)
			{
				NSString * tColumnIdentifier;
				tColumnIdentifier=[inTableColumn identifier];
				
				if (tColumnIdentifier!=nil)
				{
					NSDictionary * tDictionary;
			
					tDictionary=[tNodeData dictionary];
			
					if ([tColumnIdentifier isEqualToString: @"Name"])
					{
						NSArray * tToolTipLocalizationsArray;
						
						tToolTipLocalizationsArray=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TOOLTIP];
						
						if (tToolTipLocalizationsArray!=nil)
						{
							return [ICLocalizationUtilities localizedValueForLanguage:cachedLanguage_ inLocalizations:tToolTipLocalizationsArray];
						}
					}
					else
					{
						return @"-";
					}
				}
			}
		}
	}
	
	return nil;
}*/

#pragma mark -

/* What you can drag:

	(1) A package not part of a group whose packages are hidden
	
	(2) A group
	
	(3) A package within a group whose packages are hidden

	    A combination of 1 and 2
		
	    A combination of packages in (3)
		
	We only allow inside Drag & Drop
*/

- (BOOL) outlineView:(NSOutlineView *) inOutlineView writeItems:(NSArray *) inItems toPasteboard:(NSPasteboard*) inPasteboard
{
	if (inOutlineView==IBoutlineView_)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inItems objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			ICInstallationTypeTreeNode * tTreeNode;
			NSString * tPasteBoardType=nil;
			NSMutableArray * tUUIDArray;
			
			tUUIDArray=[NSMutableArray array];
			
			while (tTreeNode=[tEnumerator nextObject])
			{
				ICInstallationTypeNodeData * tNodeData;
			
				tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tTreeNode);
			
				if (tNodeData!=nil)
				{
					NSDictionary * tItemDictionary;
					
					tItemDictionary=[tNodeData dictionary];
					
					if (tItemDictionary!=nil)
					{
						NSNumber * tType;
						NSString * tUUID;
						
						tUUID=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
						
						tType=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
						
						if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
						{
							// We just need to check that the drag type is compatible
							
							if (tPasteBoardType!=nil)
							{
								if ([tPasteBoardType isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_PACKAGES_AND_GROUPS_PASTEBOARDTYPE]==NO)
								{
									return NO;
								}
							}
							else
							{
								tPasteBoardType=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_PACKAGES_AND_GROUPS_PASTEBOARDTYPE;
								
								[tUUIDArray addObject:tUUID];
							}
						}
						else
						{
							// We need to check if it's a a hidden child package
							
							ICInstallationTypeTreeNode * tParentNode;
							
							// Inspect the parent node
							
							tParentNode=(ICInstallationTypeTreeNode *) [tTreeNode nodeParent];
							
							if (tParentNode!=nil)
							{
								ICInstallationTypeNodeData * tParentNodeData;
			
								tParentNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tParentNode);
			
								if (tParentNodeData!=nil)
								{
									// Is it the root node
									
									if ([tParentNodeData type]==kInstallationTypeRootNode)
									{
										if (tPasteBoardType!=nil)
										{
											if ([tPasteBoardType isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_PACKAGES_AND_GROUPS_PASTEBOARDTYPE]==NO)
											{
												return NO;
											}
										}
										else
										{
											tPasteBoardType=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_PACKAGES_AND_GROUPS_PASTEBOARDTYPE;
											
											[tUUIDArray addObject:tUUID];
										}
									}
									else
									{
										NSDictionary * tParentItemDictionary;
					
										tParentItemDictionary=[tParentNodeData dictionary];
										
										if (tParentItemDictionary!=nil)
										{
											NSDictionary * tOptionsDictionary;
											
											tOptionsDictionary=[tParentItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
											
											if (tOptionsDictionary!=nil)
											{
												NSNumber * tOptionValue;
												
												tOptionValue=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
												
												if ([tOptionValue boolValue]==NO)
												{
													if (tPasteBoardType!=nil)
													{
														if ([tPasteBoardType isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_PACKAGES_AND_GROUPS_PASTEBOARDTYPE]==NO)
														{
															return NO;
														}
													}
													else
													{
														tPasteBoardType=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_PACKAGES_AND_GROUPS_PASTEBOARDTYPE;
														
														[tUUIDArray addObject:tUUID];
													}
												}
												else
												{
													if (tPasteBoardType!=nil)
													{
														if ([tPasteBoardType isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIDDENCHILDPACKAGES_PASTEBOARDTYPE]==NO)
														{
															return NO;
														}
													}
													else
													{
														tPasteBoardType=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIDDENCHILDPACKAGES_PASTEBOARDTYPE;
													}
												}
											}
											else
											{
												NSLog(@"[ICPresentationViewInstallationTypeController outlineView:writeItems:toPasteboard:] Corrupted Tree (4)");
							
												return NO;
											}
										}
										else
										{
											NSLog(@"[ICPresentationViewInstallationTypeController outlineView:writeItems:toPasteboard:] Corrupted Tree (3)");
						
											return NO;
										}
									}
								}
							}
							else
							{
								NSLog(@"[ICPresentationViewInstallationTypeController outlineView:writeItems:toPasteboard:] Corrupted Tree (2)");
					
								return NO;
							}
						}
					}
					else
					{
						NSLog(@"[ICPresentationViewInstallationTypeController outlineView:writeItems:toPasteboard:] Corrupted Tree (1)");
					
						return NO;
					}
				}
			}
			
			[inPasteboard declareTypes:[NSArray arrayWithObjects: tPasteBoardType,ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_UUID_PASTEBOARDTYPE,nil] owner:self];
    
			[inPasteboard setData:[NSData data] forType:tPasteBoardType];
			
			[inPasteboard setPropertyList:tUUIDArray forType:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_UUID_PASTEBOARDTYPE];
			
			internalDragData_=inItems;
			
			return YES;
		}
	}
	
	return NO;
}

- (NSDragOperation) outlineView:(NSOutlineView *) inOutlineView validateDrop:(id <NSDraggingInfo>) inDraggingInfo proposedItem:(id) inItem proposedChildIndex:(NSInteger) inChildIndex
{
	if (choiceDependencyEditionOn_==NO && inChildIndex!=NSOutlineViewDropOnItemIndex)
	{
		if (inOutlineView==IBoutlineView_)
		{
			NSPasteboard * tPasteBoard;
				
			tPasteBoard=[inDraggingInfo draggingPasteboard];
			
			if (tPasteBoard!=nil)
			{
				// Check that it's a local drag
				
				if ([[inDraggingInfo draggingSource] window]==[inOutlineView window])
				{				
					NSArray * tPasteboardTypes;
					
					tPasteboardTypes=[tPasteBoard types];
					
					if ([tPasteboardTypes containsObject:ICDOCUMENT_PROJECT_PACKAGES_PASTEBOARDTYPE]==YES)
					{
						// Drag and Drop from the packages source list
						
						NSArray * tArray;
						
						tArray=(NSArray *) [tPasteBoard propertyListForType:ICDOCUMENT_PROJECT_PACKAGES_PASTEBOARDTYPE];
						
						if (tArray!=nil && [tArray count]>0)
						{
							NSDictionary * tHierarchiesDictionary;
							
							tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
								
							if (tHierarchiesDictionary!=nil)
							{
								NSDictionary * tCurrentHierarchy;
								
								tCurrentHierarchy=[tHierarchiesDictionary objectForKey:currentHierarchyName_];
								
								if (tCurrentHierarchy!=nil)
								{
									// We can drop everywhere
							
									NSUInteger i,tCount;
									
									tCount=[tArray count];
									
									for (i=tCount;i>0;i--)
									{
										NSString * tPackageUUID;
										NSString * tPart;
										
										tPackageUUID=[tArray objectAtIndex:i-1];
										
										if ([ICInstallationTypeUtilities isPackageWithUUID:tPackageUUID partOfHierarchy:tCurrentHierarchy part:&tPart]==YES)
										{
											if ([tPart isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST]==YES)
											{
												return NSDragOperationNone;
											}
										}
									}
									
									return NSDragOperationCopy;
								}
							}
						}
					}
					else
					{
						if ([tPasteboardTypes containsObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_PACKAGES_AND_GROUPS_PASTEBOARDTYPE]==YES)
						{
							// We can drop it everywhere but in a hidden group or inside a real package choice
							
							if (inItem==nil)	// root target
							{
								if ([internalDragData_ count]==1)
								{
									// Check that it's not a drag to the same location
								
									ICInstallationTypeTreeNode * tDraggedTreeNode;
									
									tDraggedTreeNode=[internalDragData_ objectAtIndex:0];
									
									if ([INSTALLATION_TYPE_NODE_DATA([tDraggedTreeNode nodeParent]) type]==kInstallationTypeRootNode)
									{
										NSInteger tOriginalRow;
										
										tOriginalRow=[[tDraggedTreeNode nodeParent] indexOfChild:tDraggedTreeNode];
										
										if (tOriginalRow!=inChildIndex && inChildIndex!=(tOriginalRow+1))
										{
											return NSDragOperationMove;
										}
									}
									else
									{
										return NSDragOperationMove;
									}
								}
								else
								{
									return NSDragOperationMove;
								}
							}
							else
							{
								ICInstallationTypeNodeData * tNodeData;
								NSDictionary * tItemDictionary;
							
								tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(inItem);
								
								tItemDictionary=[tNodeData dictionary];
						
								if (tItemDictionary!=nil)
								{
									NSNumber * tType;
									
									tType=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
								
									if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
									{
										NSDictionary * tOptionsDictionary;
												
										tOptionsDictionary=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
										
										if (tOptionsDictionary!=nil)
										{
											NSNumber * tOptionValue;
											
											tOptionValue=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
											
											if ([tOptionValue boolValue]==NO)
											{
												if ([inItem isEqualToOrChildOfChoicesInArray:internalDragData_]==NO)
												{
													if ([internalDragData_ count]==1)
													{
														// Check that it's not a drag to the same location
													
														ICInstallationTypeTreeNode * tDraggedTreeNode;
														
														tDraggedTreeNode=[internalDragData_ objectAtIndex:0];
														
														if ([tDraggedTreeNode nodeParent]==inItem)
														{
															NSInteger tOriginalRow;
															
															tOriginalRow=[[tDraggedTreeNode nodeParent] indexOfChild:tDraggedTreeNode];
															
															if (tOriginalRow!=inChildIndex && inChildIndex!=(tOriginalRow+1))
															{
																return NSDragOperationMove;
															}
														}
														else
														{
															return NSDragOperationMove;
														}
													}
													else
													{
														return NSDragOperationMove;
													}
												}
											}
										}
									}
								}
							}
						}
						else
						{
							if ([tPasteboardTypes containsObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIDDENCHILDPACKAGES_PASTEBOARDTYPE]==YES)
							{
								ICInstallationTypeTreeNode * tDraggedTreeNode;
								
								// We can only drop them inside the same parent group
								
								tDraggedTreeNode=(ICInstallationTypeTreeNode *) [internalDragData_ objectAtIndex:0];
								
								if ([tDraggedTreeNode nodeParent]==inItem)
								{
									if ([internalDragData_ count]==1)
									{
										// Check that it's not a drag to the same location
													
										NSInteger tOriginalRow;
											
										tOriginalRow=[[tDraggedTreeNode nodeParent] indexOfChild:tDraggedTreeNode];
											
										if (tOriginalRow!=inChildIndex && inChildIndex!=(tOriginalRow+1))
										{
											return NSDragOperationMove;
										}
									}
									else
									{
										return NSDragOperationMove;
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	return NSDragOperationNone;
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView acceptDrop:(id <NSDraggingInfo>) inDraggingInfo item:(id) inItem childIndex:(NSInteger) inIndex
{
	if (inOutlineView==IBoutlineView_)
	{
		if ([[inDraggingInfo draggingSource] window]==[inOutlineView window])
		{
			NSPasteboard * tPasteBoard;
				
			tPasteBoard=[inDraggingInfo draggingPasteboard];
			
			if (tPasteBoard!=nil)
			{
				NSArray * tPasteboardTypes;
				
				tPasteboardTypes=[tPasteBoard types];
				
				if ([tPasteboardTypes containsObject:ICDOCUMENT_PROJECT_PACKAGES_PASTEBOARDTYPE]==YES)
				{
					NSMutableArray * tDestinationChildrenArray;
							
					if ([INSTALLATION_TYPE_NODE_DATA(inItem) type]==kInstallationTypeRootNode)
					{
						tDestinationChildrenArray=[self currentHierarchy];
					}
					else
					{
						tDestinationChildrenArray=[[INSTALLATION_TYPE_NODE_DATA(inItem) dictionary] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
					}
					
					if (tDestinationChildrenArray!=nil)
					{
						// We need to move the package from the REMOVE group to the LIST group
						
						NSDictionary * tHierarchiesDictionary;
							
						tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
							
						if (tHierarchiesDictionary!=nil)
						{
							NSDictionary * tCurrentHierarchy;
							
							tCurrentHierarchy=[tHierarchiesDictionary objectForKey:currentHierarchyName_];
							
							if (tCurrentHierarchy!=nil)
							{
								NSMutableDictionary * tRemovedGroup;
								
								tRemovedGroup=[tCurrentHierarchy objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED];
								
								if (tRemovedGroup!=nil)
								{
									NSArray * tArray;
						
									tArray=(NSArray *) [tPasteBoard propertyListForType:ICDOCUMENT_PROJECT_PACKAGES_PASTEBOARDTYPE];
						
									if (tArray!=nil)
									{
										NSEnumerator * tEnumerator;
										
										tEnumerator=[tArray objectEnumerator];
										
										if (tEnumerator!=nil)
										{
											NSString * tPackageUUID;
											
											while (tPackageUUID=[tEnumerator nextObject])
											{
												NSDictionary * tItemDictionary;
												
												tItemDictionary=[tRemovedGroup objectForKey:tPackageUUID];
												
												if (tItemDictionary!=nil)
												{
													[tDestinationChildrenArray insertObject:tItemDictionary atIndex:inIndex];
									
													inIndex++;
													
													[tRemovedGroup removeObjectForKey:tPackageUUID];
												}
											}
											
											[IBoutlineView_ deselectAll:self];
			
											[self updateTree];
											
											// Restore the other expanded groups
											
											[self restoreDisclosedStates];
											
											[self noteDocumentChange];
											
											return YES;
										}
									}
								}
							}
						}
					}
				}
				else
				{
					if ([tPasteboardTypes containsObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_PACKAGES_AND_GROUPS_PASTEBOARDTYPE]==YES ||
						[tPasteboardTypes containsObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIDDENCHILDPACKAGES_PASTEBOARDTYPE]==YES)
					{
						// We can drop it everywhere but in a hidden group or inside a real package choice
						
						NSEnumerator * tEnumerator;
						
						tEnumerator=[internalDragData_ objectEnumerator];
						
						if (tEnumerator!=nil)
						{
							ICInstallationTypeTreeNode * tDraggedTreeNode;
							NSMutableArray * tDestinationChildrenArray;
							BOOL needsToUpdateDependencies=NO;
							
							if ([INSTALLATION_TYPE_NODE_DATA(inItem) type]==kInstallationTypeRootNode)
							{
								tDestinationChildrenArray=[self currentHierarchy];
							}
							else
							{
								needsToUpdateDependencies=YES;
								
								tDestinationChildrenArray=[[INSTALLATION_TYPE_NODE_DATA(inItem) dictionary] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
							}
							
							if (tDestinationChildrenArray!=nil)
							{
								NSMutableArray * tItemsDictionarySelection;
								
								tItemsDictionarySelection=[NSMutableArray array];
								
								while (tDraggedTreeNode=[tEnumerator nextObject])
								{
									ICInstallationTypeTreeNode * tParentTreeNode;
									NSMutableArray * tChildrenArray;
									NSInteger tOldIndex=-1;
									BOOL tParentRootNode=NO;
									NSDictionary * tItemDictionary;
									
									tParentTreeNode=(ICInstallationTypeTreeNode *) [tDraggedTreeNode nodeParent];
									
									if ([INSTALLATION_TYPE_NODE_DATA([tDraggedTreeNode nodeParent]) type]==kInstallationTypeRootNode)
									{
										tChildrenArray=[self currentHierarchy];
										
										tParentRootNode=YES;
									}
									else
									{
										tChildrenArray=[[INSTALLATION_TYPE_NODE_DATA(tParentTreeNode) dictionary] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
									}
									
									if (tParentTreeNode==inItem || (inItem==nil && tParentRootNode==YES))
									{
										tOldIndex=[tParentTreeNode indexOfChild:tDraggedTreeNode];
										
										if (inIndex>=tOldIndex)
										{
											inIndex--;
										}
									}
									
									tItemDictionary=[INSTALLATION_TYPE_NODE_DATA(tDraggedTreeNode) dictionary];
									
									[tChildrenArray removeObject:tItemDictionary];
									
									[tDestinationChildrenArray insertObject:tItemDictionary atIndex:inIndex];
									
									inIndex++;
									
									[tItemsDictionarySelection addObject:tItemDictionary];
								}
								
								if (needsToUpdateDependencies==YES)
								{
									// Get the Choice UUID of the destination parent hierarchy
								
									NSMutableArray * tTargetHierarchyChoicesUUID;
									
									tTargetHierarchyChoicesUUID=[ICInstallationTypeTreeNode choiceUUIDOfReverseHierarchyFromNode:inItem];
									
									if (tTargetHierarchyChoicesUUID!=nil && [tTargetHierarchyChoicesUUID count]>0)
									{
										[ICInstallationTypeUtilities removeDependenciesToChoices:tTargetHierarchyChoicesUUID fromHierarchy:tItemsDictionarySelection];
									}
								}
								
								[IBoutlineView_ deselectAll:self];
			
								[self updateTree];
								
								// Restore the other expanded groups
											
								[self restoreDisclosedStates];
								
								[self noteDocumentChange];
								
								return YES;
							}
						}
					}
				}
			}
		}
	}
	
	return NO;
}

- (BOOL) selectionShouldChangeInOutlineView:(NSOutlineView *) inOutlineView
{
	return !choiceDependencyEditionOn_;
}

#pragma mark -

- (void) expandItemIfPossible:(ICInstallationTypeTreeNode *) inItem
{
	if ([INSTALLATION_TYPE_NODE_DATA(inItem) isLeaf]==NO)
	{
		NSArray * tChildrenArray;
		
		[IBoutlineView_ expandItem:inItem];
		
		tChildrenArray=[inItem children];
		
		if (tChildrenArray!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[tChildrenArray objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				ICInstallationTypeTreeNode * tChildNode;
				
				while (tChildNode=[tEnumerator nextObject])
				{
					[self expandItemIfPossible:tChildNode];
				}
			}
		}
	}
}

- (void) expandItem:(ICInstallationTypeTreeNode *) inChoiceNode ifKeyOfDictionary:(NSMutableDictionary *) inDictionary
{
	if (inChoiceNode!=nil && [inDictionary count]>0)
	{
		ICInstallationTypeNodeData * tChoiceNodeData;
		NSArray * tChildren;
		NSEnumerator * tEnumerator;
			
		tChoiceNodeData=INSTALLATION_TYPE_NODE_DATA(inChoiceNode);
		
		if ([tChoiceNodeData type]==kInstallationTypeRootNode)
		{
			tChildren=[inChoiceNode children];
				
			tEnumerator=[tChildren objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				ICInstallationTypeTreeNode * tChoiceNode;
				
				while (tChoiceNode=[tEnumerator nextObject])
				{
					[self expandItem:tChoiceNode ifKeyOfDictionary:inDictionary];
				}
			}
		}
		else if ([tChoiceNodeData isLeaf]==NO)
		{
			NSString * tChoiceUUID;
			
			tChoiceUUID=[[tChoiceNodeData dictionary] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
			
			[IBoutlineView_ expandItem:inChoiceNode];
			
			// Check children
			
			tChildren=[inChoiceNode children];
				
			tEnumerator=[tChildren objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				ICInstallationTypeTreeNode * tChoiceNode;
				
				while (tChoiceNode=[tEnumerator nextObject])
				{
					[self expandItem:tChoiceNode ifKeyOfDictionary:inDictionary];
				}
			}
			
			if (tChoiceUUID!=nil)
			{
				NSNumber * tNumber;
				
				tNumber=[inDictionary objectForKey:tChoiceUUID];
			
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					[IBoutlineView_ collapseItem:inChoiceNode];
				}
			}
		}
	}
} 

- (void) restoreDisclosedStates
{
	NSMutableDictionary * tDisclosedDictionary;
	
	tDisclosedDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,currentHierarchyName_]];
			
	if (tDisclosedDictionary!=nil)
	{
		[self expandItem:tree_ ifKeyOfDictionary:tDisclosedDictionary];
	}
	else
	{
		/*
		
		tDisclosedDictionary=[NSMutableDictionary dictionary];
		
		[document_ setDocumentDefault:tDisclosedDictionary forKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,currentHierarchyName_]];*/
		
		// We disclose everything
		
		[self expandItemIfPossible:tree_];
	}
}

- (void) choiceDidExpand:(ICInstallationTypeTreeNode *) inChoiceNode
{
	if (inChoiceNode!=nil)
	{
		NSString * tChoiceUUID;
		
		tChoiceUUID=[[INSTALLATION_TYPE_NODE_DATA(inChoiceNode) dictionary] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
		
		if (tChoiceUUID!=nil)
		{
			NSMutableDictionary * tDisclosedDictionary;
			NSString * tKey;
			
			tKey=[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,currentHierarchyName_];
			
			tDisclosedDictionary=[document_ documentDefaultForKey:tKey];
			
			if (tDisclosedDictionary==nil)
			{
				tDisclosedDictionary=[NSMutableDictionary dictionary];
				
				if (tDisclosedDictionary!=nil)
				{
					[document_ setDocumentDefault:tDisclosedDictionary forKey:tKey];
				}
				else
				{
					NSLog(@"[ICPackageFilesController outlineViewItemDidExpand:] Low memory");
				}
			}
			
			if (tDisclosedDictionary!=nil)
			{
				[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tChoiceUUID];
			}
		}
	}
}

#pragma mark -

- (void) archiveSelection
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBoutlineView_ selectedRowIndexes];
	
	if (tIndexSet!=nil)
	{
		NSMutableArray * tMutableArray;
		
		tMutableArray=[NSMutableArray array];
		
		if (tMutableArray!=nil)
		{
			NSUInteger tIndex;
			
			tIndex=[tIndexSet firstIndex];
			
			while (tIndex!=NSNotFound)
			{
				ICInstallationTypeTreeNode * tChoiceNode;
					
				tChoiceNode=[IBoutlineView_ itemAtRow:tIndex];
				
				if (tChoiceNode!=nil)
				{
					NSString * tChoiceUUID;
			
					tChoiceUUID=[tChoiceNode choiceUUID];
						
					if (tChoiceUUID!=nil)
					{
						[tMutableArray addObject:tChoiceUUID];
					}
				}
			
				tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
			}
			
			[document_ setDocumentDefault:tMutableArray forKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_SELECTION,currentHierarchyName_]];
		}
		else
		{
			NSLog(@"[ICPresentationViewInstallationTypeController archiveSelection]: Low Memory");
			
			[document_ removeDocumentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_SELECTION,currentHierarchyName_]];
		}
	}
}

- (void) restoreSelection
{
	NSArray * tArray;
	
	tArray=[document_ documentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_SELECTION,currentHierarchyName_]];
			
	if (tArray!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[tArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSMutableIndexSet * tMutableIndexSet;
			
			tMutableIndexSet=[NSMutableIndexSet indexSet];
			
			if (tMutableIndexSet!=nil)
			{
				NSString * tChoiceUUID;
			
				while (tChoiceUUID=[tEnumerator nextObject])
				{
					ICInstallationTypeTreeNode * tChoiceNode;
					
					tChoiceNode=[tree_ nodeWithChoiceUUID:tChoiceUUID];
					
					if (tChoiceNode!=nil)
					{
						NSInteger tRow;
						
						tRow=[IBoutlineView_ rowForItem:tChoiceNode];
						
						if (tRow!=-1)
						{
							[tMutableIndexSet addIndex:tRow];
						}
					}
				}
				
				[IBoutlineView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:NO];
			}
		}
	}
}

#pragma mark -

- (IBAction) switchInstallationTypeMode:(id) sender
{
	if ([currentHierarchyName_ isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER]==YES)
	{
		NSInteger tTag;
		NSNumber * tNumber;
		int tInstallationTypeMode=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_ALLOW;
		
		tTag=[[sender selectedItem] tag];
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_MODE];
	
		if (tNumber!=nil)
		{
			tInstallationTypeMode=[tNumber intValue];
		}
		
		if (tInstallationTypeMode!=tTag)
		{
			[dictionary_ setObject:[NSNumber numberWithInteger:tTag] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_MODE];
			
			// Refresh the border
			
			[self updateBorders];
			
			// Update the buttons
			
			if (cachedButtons_!=nil)
			{
				NSButton * tButton;
				
				tButton=(NSButton *) [cachedButtons_ objectAtIndex:0];
				
				if (tTag==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_ALLOW)
				{
					[tButton setHidden:NO];
				}
				else
				{
					[tButton setHidden:YES];
				}
			}
			
			[self noteDocumentChange];
		}
	}
}

#pragma mark -

- (void) setHiddeHierarchyBox:(BOOL) inHidden
{
	if (inHidden==NO)
	{
		[IBhierarchyBox_ setHidden:NO];
	}
	else
	{
		// We need to check that we are not already in a custom hierarchy situation
		
		NSDictionary * tHierarchiesDictionary;
		
		tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
		
		if (tHierarchiesDictionary!=nil)
		{
			if ([tHierarchiesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER]==nil ||
				[tHierarchiesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE]!=nil ||
				[tHierarchiesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE]!=nil)
			{
				return;
			}
		}
		
		[IBhierarchyBox_ setHidden:YES];
	}
}

- (void) refreshHierarchyPopupButton
{
	NSMenu * tHierarchyMenu;
	
	// Remove all items
	
	[IBhierarchyPopupButton_ removeAllItems];
	
	// Set new menu
	
	tHierarchyMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
    
    if (tHierarchyMenu!=nil)
    {
        NSDictionary * tHierarchiesDictionary;
		int tSelectedTag=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER_TAG;
		
		tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
		
		if (tHierarchiesDictionary!=nil)
		{
			id tMenuItem;
			NSImage * tImage;
		
			// Installer?
			
			if ([tHierarchiesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER]!=nil)
			{
				tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER,@"Presentation_InstallationType",@"No comment")
													 action:@selector(switchHierarchy:)
											  keyEquivalent:@""];
											  
				if (tMenuItem!=nil)
				{
					[tMenuItem setTarget:self];
					 
					[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER_TAG];
					 
					tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"InstallerSmall" ofType:@"tiff"]];
        
					if (tImage!=nil)
					{
						[tMenuItem setImage:tImage];

						[tImage release];
					}
					
					[tHierarchyMenu addItem:tMenuItem];
        
					[tMenuItem release];
				}
			}
			
			// Software Update?
			
			if ([tHierarchiesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE]!=nil)
			{
				tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE,@"Presentation_InstallationType",@"No comment")
													 action:@selector(switchHierarchy:)
											  keyEquivalent:@""];
											  
				if (tMenuItem!=nil)
				{
					[tMenuItem setTarget:self];
					 
					[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE_TAG];
					 
					tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"SoftwareUpdateSmall" ofType:@"tiff"]];
        
					if (tImage!=nil)
					{
						[tMenuItem setImage:tImage];

						[tImage release];
					}
					
					[tHierarchyMenu addItem:tMenuItem];
        
					if ([currentHierarchyName_ isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE]==YES)
					{
						tSelectedTag=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE_TAG;
					}
					
					[tMenuItem release];
				}
			}
			
			// Invisible?
			
			if ([tHierarchiesDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE]!=nil)
			{
				tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE,@"Presentation_InstallationType",@"No comment")
													 action:@selector(switchHierarchy:)
											  keyEquivalent:@""];
											  
				if (tMenuItem!=nil)
				{
					[tMenuItem setTarget:self];
					 
					[tMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE_TAG];
					 
					tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"InvisibleSmall" ofType:@"tiff"]];
        
					if (tImage!=nil)
					{
						[tMenuItem setImage:tImage];

						[tImage release];
					}
					
					[tHierarchyMenu addItem:tMenuItem];
					
					if ([currentHierarchyName_ isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE]==YES)
					{
						tSelectedTag=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE_TAG;
					}
					
					[tMenuItem release];
				}
			}
			
			// Separator
			
			tMenuItem=[NSMenuItem separatorItem];
			
			if (tMenuItem!=nil)
			{
				[tHierarchyMenu addItem:tMenuItem];
			}
			
			if ([tHierarchiesDictionary count]<ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_COUNT)
			{
				// Add
				
				tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Add Hierarchy",@"Presentation_InstallationType",@"No comment")
													 action:@selector(nothing:)
											  keyEquivalent:@""];
												  
				if (tMenuItem!=nil)
				{
					NSMenu * tSubMenu;
				
					tSubMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
				
					if (tSubMenu!=nil)
					{
						NSMutableArray * tArray;
						NSArray * tKeys;
						
						tArray=[cachedHierarchyNames_ mutableCopy];
						
						tKeys=[tHierarchiesDictionary allKeys];
						
						[tArray removeObjectsInArray:tKeys];
						
						if ([tArray count]>0)
						{
							NSMenuItem * tSubMenuItem;
							
							// Installer?
							
							if ([tArray containsObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER]==YES)
							{
								tSubMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER,@"Presentation_InstallationType",@"No comment")
																	 action:@selector(addHierarchy:)
															  keyEquivalent:@""];
															  
								if (tSubMenuItem!=nil)
								{
									[tSubMenuItem setTarget:self];
									 
									[tSubMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER_TAG];
									
									tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"InstallerSmall" ofType:@"tiff"]];
        
									if (tImage!=nil)
									{
										[tSubMenuItem setImage:tImage];

										[tImage release];
									}
									
									[tSubMenu addItem:tSubMenuItem];
						
									[tSubMenuItem release];
								}
							}
							
							// Software Update?
							
							if ([tArray containsObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE]==YES)
							{
								tSubMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE,@"Presentation_InstallationType",@"No comment")
																	 action:@selector(addHierarchy:)
															  keyEquivalent:@""];
															  
								if (tSubMenuItem!=nil)
								{
									[tSubMenuItem setTarget:self];
									 
									[tSubMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE_TAG];
									
									tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"SoftwareUpdateSmall" ofType:@"tiff"]];
        
									if (tImage!=nil)
									{
										[tSubMenuItem setImage:tImage];

										[tImage release];
									}
									
									[tSubMenu addItem:tSubMenuItem];
						
									if ([currentHierarchyName_ isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE]==YES)
									{
										tSelectedTag=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE_TAG;
									}
									
									[tSubMenuItem release];
								}
							}
							
							// Invisible?
							
							if ([tArray containsObject:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE]==YES)
							{
								tSubMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE,@"Presentation_InstallationType",@"No comment")
																	 action:@selector(addHierarchy:)
															  keyEquivalent:@""];
															  
								if (tSubMenuItem!=nil)
								{
									[tSubMenuItem setTarget:self];
									 
									[tSubMenuItem setTag:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE_TAG];
									
									tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"InvisibleSmall" ofType:@"tiff"]];
        
									if (tImage!=nil)
									{
										[tSubMenuItem setImage:tImage];

										[tImage release];
									}
									
									[tSubMenu addItem:tSubMenuItem];
									
									if ([currentHierarchyName_ isEqualToString:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE]==YES)
									{
										tSelectedTag=ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE_TAG;
									}
									
									[tSubMenuItem release];
								}
							}
						
							[tMenuItem setSubmenu:tSubMenu];
						}
						
						[tArray release];
						
						[tSubMenu release];
					}
					
					[tMenuItem setTag:-1];
				}
				else
				{
				}
				
				[tMenuItem setTarget:self];
				
				[tHierarchyMenu addItem:tMenuItem];
	
				[tMenuItem release];
			}
			
			// Remove
			
			tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Remove...",@"Presentation_InstallationType",@"No comment")
												 action:@selector(removeHierarchy:)
										  keyEquivalent:@""];
											  
			if (tMenuItem!=nil)
			{
				[tMenuItem setTarget:self];
				 
				[tHierarchyMenu addItem:tMenuItem];
	
				tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"DeleteSmall" ofType:@"tiff"]];
        
				if (tImage!=nil)
				{
					[tMenuItem setImage:tImage];

					[tImage release];
				}
				
				[tMenuItem release];
			}
		}
	
		[IBhierarchyPopupButton_ setMenu:tHierarchyMenu];
		
		[tHierarchyMenu release];
		
		[IBhierarchyPopupButton_ selectItemWithTag:tSelectedTag];
		
		[IBinstallationTypeModePopupButton_ setEnabled:(tSelectedTag==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER_TAG)];
		
	}
}

- (IBAction) switchHierarchy:(id) sender
{
	NSInteger tTag;
	NSString * tNewCurrentHierarchyName;
	
	tTag=[sender tag];
	
NS_DURING

	tNewCurrentHierarchyName=[cachedHierarchyNames_ objectAtIndex:tTag];

NS_HANDLER

	tNewCurrentHierarchyName=nil;

NS_ENDHANDLER
	
	if (tNewCurrentHierarchyName!=nil && [currentHierarchyName_ isEqualToString:tNewCurrentHierarchyName]==NO)
	{
		// Archive Selection
	
		[self archiveSelection];
		
		
		[currentHierarchyName_ release];
		
		currentHierarchyName_=[[NSString alloc] initWithString:tNewCurrentHierarchyName];
		
		if (currentHierarchyName_!=nil)
		{
			[IBinstallationTypeModePopupButton_ setEnabled:(tTag==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER_TAG)];
		
			[self currentHierarchyDidChange:nil];
		}
		else
		{
			// Low Memory
			
			// A COMPLETER
		}
	}
}

- (IBAction) nothing:(id) sender
{
	if (currentHierarchyName_!=nil)
	{
												   
		[IBhierarchyPopupButton_ selectItemWithTag:[cachedHierarchyNames_ indexOfObject:currentHierarchyName_]];
	}
}

- (IBAction) addHierarchy:(id) sender
{
	if (sender!=nil)
	{
		NSInteger tTag;
		
		NSString * tNewCurrentHierarchyName;
		
		tTag=[sender tag];

NS_DURING

		tNewCurrentHierarchyName=[cachedHierarchyNames_ objectAtIndex:tTag];

NS_HANDLER

		tNewCurrentHierarchyName=nil;

NS_ENDHANDLER
		
		if (tNewCurrentHierarchyName!=nil)
		{
			NSMutableDictionary * tHierarchyDictionary;
		
			tHierarchyDictionary=[ICInstallationTypeUtilities defaultHierarchyDictionaryWithPackages:[document_ packagesArray]];
		
			if (tHierarchyDictionary!=nil)
			{
				NSMutableDictionary * tHierarchiesDictionary;
			
				tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
				
				if (tHierarchiesDictionary!=nil)
				{
					// Archive Selection
	
					[self archiveSelection];
					
					
					[tHierarchiesDictionary setObject:tHierarchyDictionary forKey:tNewCurrentHierarchyName];
					
					[currentHierarchyName_ release];
			
					currentHierarchyName_=[[NSString alloc] initWithString:tNewCurrentHierarchyName];
					
					if (currentHierarchyName_!=nil)
					{
						[self currentHierarchyDidChange:nil];
					}
					
					[self refreshHierarchyPopupButton];
					
					[self noteDocumentChange];
				}
			}
		}
	}
}

- (IBAction) duplicateHierarchy:(id) sender
{
	if (sender!=nil)
	{
		NSInteger tTag;
		
		NSString * tNewCurrentHierarchyName;
		
		tTag=[sender tag];
	
NS_DURING

		tNewCurrentHierarchyName=[cachedHierarchyNames_ objectAtIndex:tTag];

NS_HANDLER

		tNewCurrentHierarchyName=nil;

NS_ENDHANDLER
		
		if (tNewCurrentHierarchyName!=nil)
		{
			NSMutableDictionary * tHierarchiesDictionary;
			
			tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
				
			if (tHierarchiesDictionary!=nil)
			{
				NSMutableDictionary * tHierarchyDictionary;
		
				tHierarchyDictionary=[tHierarchiesDictionary objectForKey:currentHierarchyName_];
				
				if (tHierarchyDictionary!=nil)
				{
					NSMutableDictionary * tDuplicateHierarchy=nil;
				
					// Duplicate
					
					tDuplicateHierarchy=[tHierarchyDictionary mutableDeepCopy];
					
					if (tDuplicateHierarchy!=nil)
					{
						//Copy the disclose state too
						
						[document_ setDocumentDefault:[document_ documentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,currentHierarchyName_]]
											   forKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,tNewCurrentHierarchyName]];
						
						if (tTag==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_SOFTWAREUPDATE_TAG ||
							tTag==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INVISIBLE_TAG)
						{
							// Remove the localizations (because we don't care)
							
							[ICInstallationTypeUtilities removeLocalizationsInHierarchy:tDuplicateHierarchy];
						}
					
						// Archive Selection
	
						[self archiveSelection];
						
						
						[tHierarchiesDictionary setObject:tDuplicateHierarchy forKey:tNewCurrentHierarchyName];
						
						[currentHierarchyName_ release];
			
						currentHierarchyName_=[[NSString alloc] initWithString:tNewCurrentHierarchyName];
						
						if (currentHierarchyName_!=nil)
						{
							[self currentHierarchyDidChange:nil];
						}
						
						[self refreshHierarchyPopupButton];
						
						[self noteDocumentChange];
					}
				}
			}
		}
	}
}

- (IBAction) removeHierarchy:(id) sender
{
	if (currentHierarchyName_!=nil)
	{
		NSString * tAlertTitle;
	
		tAlertTitle=[NSString stringWithFormat:NSLocalizedStringFromTable(@"Do you really want to remove the \"%@\" hierachy?",@"Presentation_InstallationType",@"No comment"),NSLocalizedStringFromTable(currentHierarchyName_,@"Presentation_InstallationType",@"No comment")];
	
		NSBeginAlertSheet(tAlertTitle,
						  NSLocalizedString(@"Remove",@"No comment"),
						  NSLocalizedString(@"Cancel",@"No comment"),
						  nil,
						  [document_ windowForSheet],
						  self,
						  @selector(removeHierarchyDidEnd:returnCode:contextInfo:),
						  nil,
						  NULL,
						  NSLocalizedString(@"This cannot be undone.",@"No comment"));
	}
}

- (void) removeHierarchyDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSMutableDictionary * tHierarchiesDictionary;
			
		tHierarchiesDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
				
		if (tHierarchiesDictionary!=nil && currentHierarchyName_!=nil)
		{
			NSEnumerator * tEnumerator;
		
			// Remove the disclosed state for this hiearchy
			
			[document_ removeDocumentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,currentHierarchyName_]];
			
			// Remove the stored selection if needed
			
			[document_ removeDocumentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_SELECTION,currentHierarchyName_]];
			
			// Remove the hierarchy
		
			[tHierarchiesDictionary removeObjectForKey:currentHierarchyName_];
			
			[currentHierarchyName_ release];
			
			currentHierarchyName_=nil;
			
			tEnumerator=[cachedHierarchyNames_ objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSString * tHierarchyName;
				
				while (tHierarchyName=[tEnumerator nextObject])
				{
					if ([tHierarchiesDictionary objectForKey:tHierarchyName]!=nil)
					{
						currentHierarchyName_=[tHierarchyName copy];
						
						[self currentHierarchyDidChange:nil];
						
						[self refreshHierarchyPopupButton];
						
						[self noteDocumentChange];
					}
				}
			}
		}
	}
	else
	{
		if (currentHierarchyName_!=nil)
		{
													   
			[IBhierarchyPopupButton_ selectItemWithTag:[cachedHierarchyNames_ indexOfObject:currentHierarchyName_]];
		}
	}
}

#pragma mark -

- (IBAction) showHideIndentationColumn:(id) sender
{
	NSTableColumn * tColumn;
		
	tColumn=[IBoutlineView_ tableColumnWithIdentifier:ICPRESENTATIONVIEWINSTALLATIONTYPE_INDENTATION_COLUMNIDENTIFIER];
		
	if (tColumn!=nil)
	{
		[IBoutlineView_ removeTableColumn:tColumn];
	}
	else
	{
		tColumn=[[NSTableColumn alloc] initWithIdentifier:ICPRESENTATIONVIEWINSTALLATIONTYPE_INDENTATION_COLUMNIDENTIFIER];
		
		if (tColumn!=nil)
		{
			NSTextFieldCell * tDataCell;
			
			tDataCell=[[NSTextFieldCell alloc] initTextCell:@""];
			
			if (tDataCell!=nil)
			{
				float tCellWidth;
				id tHeaderCell;
				
				tCellWidth=45.0f;
				
				[tDataCell setFont:[NSFont controlContentFontOfSize:[NSFont systemFontSize]]];
				
				/*[tDataCell setBackgroundColor:[NSColor lightGrayColor]];
				
				[tDataCell setDrawsBackground:YES];*/
				
				[tColumn setDataCell:tDataCell];
				
				[tDataCell release];
				
				
				tHeaderCell=[tColumn headerCell];
				
				if (tHeaderCell!=nil)
				{
					[tHeaderCell setStringValue:@""];
				}
				
				[tColumn setWidth:tCellWidth];
				
				[tColumn setMinWidth:tCellWidth];
				
				[tColumn setMaxWidth:tCellWidth];
				
				[tColumn setEditable:NO];
				
				[tColumn setResizingMask:NSTableColumnNoResizing];
				
				// Insert the Column
				
				[IBoutlineView_ addTableColumn:tColumn];
				
				[IBoutlineView_ moveColumn:[IBoutlineView_ numberOfColumns]-1 toColumn:0];
			}
			
			[tColumn release];
		}
		else
		{
			// A COMPLETER (memory)
		}
	}
}

#pragma mark -

- (void) installationTypeSettingsDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if ([tUserInfo objectForKey:ICPRESENTATION_NOTIFICATION_SENDER]!=self)
		{
			[self refreshViewForLanguage:cachedLanguage_];
		}
	}
}

- (void) currentHierarchyDidChange:(NSNotification *) inNotification
{
	[document_ setDocumentDefault:currentHierarchyName_ forKey:@"ui.project.presentation.installationType.hierarchy.current"];
	
	[self updateBorders];
		
	[IBoutlineView_ deselectAll:self];
	
	[self updateTree];
	
	[self restoreDisclosedStates];
	
	// Restore Selection
				
	[self restoreSelection];
}

- (void) outlineViewItemDidExpand:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			ICInstallationTypeTreeNode * tChoiceNode;
			
			tChoiceNode=(ICInstallationTypeTreeNode *) [tUserInfo objectForKey:@"NSObject"];
			
			[self choiceDidExpand:tChoiceNode];
		}
	}
}

- (void) outlineViewItemWillCollapse:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			ICInstallationTypeTreeNode * tChoiceNode;
			
			tChoiceNode=(ICInstallationTypeTreeNode *) [tUserInfo objectForKey:@"NSObject"];
			
			if (tChoiceNode!=nil)
			{
				NSString * tChoiceUUID;
				
				tChoiceUUID=[[INSTALLATION_TYPE_NODE_DATA(tChoiceNode) dictionary] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
				
				if (tChoiceUUID!=nil)
				{
					NSMutableDictionary * tDisclosedDictionary;
					
					tDisclosedDictionary=[document_ documentDefaultForKey:[NSString stringWithFormat:ICPRESENTATIONVIEWINSTALLATIONTYPE_HIERARCHY_DISCLOSED_STATE,currentHierarchyName_]];
					
					if (tDisclosedDictionary!=nil)
					{
						// Check if the option key is down or not
						
						NSEvent * tCurrentEvent;
						
						tCurrentEvent=[NSApp currentEvent];
						
						if (tCurrentEvent==nil || (([tCurrentEvent modifierFlags] & NSAlternateKeyMask)==0))
						{
							ICInstallationTypeTreeNode * tParentNode;
							
							tParentNode=tChoiceNode;
							
							do
							{
								ICInstallationTypeNodeData * tParentNodeData;
								NSNumber * tNumber;
								
								tParentNode=(ICInstallationTypeTreeNode *) [tParentNode nodeParent];
								
								tParentNodeData=INSTALLATION_TYPE_NODE_DATA(tParentNode);
								
								if ([tParentNodeData type]==kInstallationTypeRootNode)
								{
									break;
								}
								
								tChoiceUUID=[[tParentNodeData dictionary] objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
								
								tNumber=[tDisclosedDictionary objectForKey:tChoiceUUID];
								
								if (tNumber==nil || [tNumber boolValue]==NO)
								{
									// Parent is hidden
								
									return;
								}
							}
							while (1);
						}
						
						[tDisclosedDictionary removeObjectForKey:tChoiceUUID];
					}
				}
			}
		}
	}
}


- (void) outlineViewSelectionDidChange:(NSNotification *) inNotification
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBoutlineView_ selectedRowIndexes];
	
	if (tIndexSet!=nil)
	{
		ICInstallationTypeTreeNode * tTreeNode;
		
		switch ([tIndexSet count])
		{
			case 0:
			
				// Display nothing in the description text field
				
				[IBdescriptionTextView_ setString:@""];
				
				// Post Notification (No Inspector displayed)
				
				[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTIONDIDCHANGE 
																		object:document_ 
																	  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedStringFromTable(@"Empty Selection",@"Presentation_InstallationType",@"No comment"),ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTION_LABEL,nil]];
			
				break;
				
			case 1:
			
				// What kind of node is selected
				
				tTreeNode=(ICInstallationTypeTreeNode *) [IBoutlineView_ itemAtRow:[IBoutlineView_ selectedRow]];
				
				if (tTreeNode!=nil)
				{
					//ICInstallationTypeTreeNode * tParentNode;
					//ICInstallationTypeNodeData * tParentNodeData;
					ICInstallationTypeNodeData * tNodeData;
					NSString * tLocalizedDescription=nil;
					NSDictionary * tItemDictionary=nil;
					
					//tParentNode=(ICInstallationTypeTreeNode *) [tTreeNode nodeParent];
					
					//tParentNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tParentNode);
					
					if ([tTreeNode isInHiddenGroup]==YES)
					{
						// It's an invisible group
									
						[IBdescriptionTextView_ setString:@""];
										
						// Post Notification (No Inspector displayed)
										
						[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTIONDIDCHANGE 
																		object:document_ 
																	  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedStringFromTable(@"Not Editable",@"Presentation_InstallationType",@"No comment"),ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTION_LABEL,nil]];
						
						return;
					}
					
					/*if ([tParentNodeData type]!=kInstallationTypeRootNode)
					{
						NSDictionary * tParentItemDictionary;
					
						tParentItemDictionary=[tParentNodeData dictionary];
						
						if (tParentItemDictionary!=nil)
						{
							NSNumber * tType;
							
							tType=[tParentItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
							
							if ([tType intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_GROUP)
							{
								NSDictionary * tOptionsDictionary;
							
								tOptionsDictionary=[tParentItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
							
								if (tOptionsDictionary!=nil)
								{
									NSNumber * tOptionValue;
								
									tOptionValue=[tOptionsDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDE_CHILDREN];
								
									if ([tOptionValue boolValue]==YES)
									{
										// It's an invisible group
									
										[IBdescriptionTextView_ setString:@""];
										
										// Post Notification (No Inspector displayed)
										
										// A COMPLETER
										
										return;
									}
								}
							}
						}
					}*/
					
					tNodeData=(ICInstallationTypeNodeData *) INSTALLATION_TYPE_NODE_DATA(tTreeNode);
					
					if (tNodeData!=nil)
					{
						tItemDictionary=[tNodeData dictionary];
						
						if (tItemDictionary!=nil)
						{
							NSArray * tDescriptionLocalizationsArray;
							
							tDescriptionLocalizationsArray=[tItemDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_DESCRIPTION];
							
							if (tDescriptionLocalizationsArray!=nil)
							{
								tLocalizedDescription=[ICLocalizationUtilities localizedValueForLanguage:cachedLanguage_ inLocalizations:tDescriptionLocalizationsArray lookForBestMatch:NO];
							}
						}
					}
					
					if (tLocalizedDescription==nil)
					{
						tLocalizedDescription=@"";
					}
					
					[IBdescriptionTextView_ setString:tLocalizedDescription];
					
					// Post Notification
					
					[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTIONDIDCHANGE 
																		object:document_ 
																	  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:tTreeNode,ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTION,nil]];
				}
				else
				{
					[IBdescriptionTextView_ setString:@""];
					
					// Post Notification
					
					[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTIONDIDCHANGE 
																		object:document_ 
																	  userInfo:nil];
				}
				
				break;
				
			default:
			
				// Display nothing in the description text field
				
				[IBdescriptionTextView_ setString:@""];
				
				// Post Notification
					
				[[NSNotificationCenter defaultCenter] postNotificationName:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTIONDIDCHANGE 
																		object:document_ 
																	  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedStringFromTable(@"Multiple Selection",@"Presentation_InstallationType",@"No comment"),ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_SELECTION_LABEL,nil]];
				
				break;
		}
	}
}

- (void) appleModeDidChange:(NSNotification *) inNotification
{
	BOOL tState;
	
	tState=[defaults_ boolForKey:ICPREFERENCES_ADVANCED_APPLE_MODE_STATE];
	
	[self setHiddeHierarchyBox:(tState==NO)];
}

- (void) installationTypeChoiceDependencyEditDidRequest:(NSNotification *) inNotification
{
	// Hide the Action and Size columns
	
	[IBactionColumn_ retain];
	 
	[IBoutlineView_ removeTableColumn:IBactionColumn_];
	 
	[IBsizeColumn_ retain];
	 
	[IBoutlineView_ removeTableColumn:IBsizeColumn_];
	
	// Show the Index column
	
	[IBoutlineView_ addTableColumn:IBindentationColumn_];
	
	[IBindentationColumn_ release];
				
	[IBoutlineView_ moveColumn:[IBoutlineView_ numberOfColumns]-1 toColumn:0];
	
	[IBoutlineView_ sizeLastColumnToFit];
	
	// Disable selection & Contextual Menu
	
	choiceDependencyEditionOn_=YES;
}

- (void) installationTypeChoiceDependencyEndEditDidRequest:(NSNotification *) inNotification
{
	// Enable selection & Contextual Menu
	
	choiceDependencyEditionOn_=NO;
	
	// Hide the Index Column
	
	[IBindentationColumn_ retain];
	 
	[IBoutlineView_ removeTableColumn:IBindentationColumn_];
	
	// Show the Action and Size columns
	
	[IBoutlineView_ addTableColumn:IBactionColumn_];
	
	[IBactionColumn_ release];
				
	[IBoutlineView_ addTableColumn:IBsizeColumn_];
	
	[IBsizeColumn_ release];
	
	[IBoutlineView_ sizeToFit];
}

@end
