/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationViewInstallerPluginController.h"

#import "ICInstallerPluginLocalizer.h"

#import "ICIconsUtilities.h"

#import "ICDocument.h"

@implementation ICPresentationViewInstallerPluginController

+ (ICPresentationViewInstallerPluginController *) controller
{
	ICPresentationViewInstallerPluginController * nController;
    
    nController=[ICPresentationViewInstallerPluginController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICInstallerPluginView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICInstallerPluginView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
		else
		{
			[nController initLocalizationInformation];
		}
    }
    
    return [nController autorelease];
}

- (void) dealloc
{
	[plugInIcon_ release];
	
	[plugInNotFoundIcon_ release];
	
	[super dealloc];
}

- (void) awakeFromNib
{
	plugInIcon_=[[NSImage alloc] initWithContentsOfFile:@"/System/Library/CoreServices/CoreTypes.bundle/Contents/Resources/KEXT.icns"];
	
	if (plugInIcon_!=nil)
	{
		plugInNotFoundIcon_=[[NSImage alloc] initWithSize:NSMakeSize(128.0f,128.0f)/*[plugInIcon_ size]*/];
		
		if (plugInNotFoundIcon_!=nil)
		{
			NSImage * tImage;
			
			tImage=[ICIconsUtilities bigIconForType:NSFileTypeForHFSTypeCode(kQuestionMarkIcon)];
			
			if (tImage!=nil)
			{
				[plugInNotFoundIcon_ lockFocus];
			
				[plugInIcon_ compositeToPoint:NSZeroPoint operation:NSCompositeSourceOver fraction:1.0f];
			
				[tImage compositeToPoint:NSZeroPoint operation:NSCompositeSourceAtop fraction:1.0f];
				
				[plugInNotFoundIcon_ unlockFocus];
			}
			else
			{
				[plugInNotFoundIcon_ release];
				
				plugInNotFoundIcon_=nil;
			}
		}
	}
	
	
	
	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(viewFrameDidChange:) 
												 name:NSViewFrameDidChangeNotification 
											  object:IBview_];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument andLanguage:(NSString *) inLanguage
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument andLanguage:inLanguage];
	
	dictionary_=inDictionary;
	
	if (dictionary_!=nil)
	{
		NSWindow * tWindow;
		
		[dictionary_ retain];
		
		[self refreshIcon];
		
		// Register for Notifications
	
		tWindow=[[self document] windowForSheet];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidBecomeMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidBecomeMainNotification 
												   object:tWindow];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidResignMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidResignMainNotification 
												   object:tWindow];
	}
	else
	{
		// A COMPLETER
	}
}

#pragma mark -

- (void) refreshIcon
{
	if (dictionary_!=nil)
	{
		NSString * tPath;
		int tPathType;
		
		tPathType=[[dictionary_ objectForKey:ICDOCUMENT_PATH_TYPE] intValue];
			
		tPath=[dictionary_ objectForKey:ICDOCUMENT_PATH];
		
		if (tPath!=nil && [tPath length]>0)
		{
			NSFileManager * tFileManager;
			
			if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
			}
					
			tFileManager=[NSFileManager defaultManager];
								
			if (tFileManager!=nil)
			{
				BOOL isDirectory;
				
				if ([tFileManager fileExistsAtPath:tPath isDirectory:&isDirectory]==YES && isDirectory==YES)
				{
					NSBundle * tBundle;
			
					tBundle=[NSBundle bundleWithPath:tPath];
			
					if (tBundle!=nil)
					{
						if (plugInIcon_!=nil)
						{
							[IBiconView_ setImage:plugInIcon_];
						}
					}
					else
					{
						if (plugInNotFoundIcon_!=nil)
						{
							[IBiconView_ setImage:plugInNotFoundIcon_];
						}
					}
				}
				else
				{
					if (plugInNotFoundIcon_!=nil)
					{
						[IBiconView_ setImage:plugInNotFoundIcon_];
					}
				}
			}
		}
	}
}

#pragma mark -

- (NSString *) paneTitleForLanguage:(NSString *) inLanguage
{
	NSString * tLocalizedPaneTitle=nil;
	
	if (dictionary_!=nil)
	{
		NSString * tPath;
		int tPathType;
		
		tPathType=[[dictionary_ objectForKey:ICDOCUMENT_PATH_TYPE] intValue];
			
		tPath=[dictionary_ objectForKey:ICDOCUMENT_PATH];
		
		if (tPath!=nil && [tPath length]>0)
		{
			if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
			}
			else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
			{
				tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
			}
			
			if (tPath!=nil)
			{
				NSFileManager * tFileManager;
				
				tFileManager=[NSFileManager defaultManager];
								
				if (tFileManager!=nil)
				{
					BOOL isDirectory;
					
					if ([tFileManager fileExistsAtPath:tPath isDirectory:&isDirectory]==YES && isDirectory==YES)
					{
						NSBundle * tBundle;
					
						tBundle=[NSBundle bundleWithPath:tPath];
						
						if (tBundle!=nil)
						{
							return [ICInstallerPluginLocalizer localizedString:@"PaneTitle" forLanguage:inLanguage inExternalBundle:tBundle];
						}
						else
						{
							return [[ICInstallerPluginLocalizer defaultLocalizer] localizedString:@"Not Found" forLanguage:inLanguage inBundle:@"self"];
						}
					}
					else
					{
						return [[ICInstallerPluginLocalizer defaultLocalizer] localizedString:@"Not Found" forLanguage:inLanguage inBundle:@"self"];
					}
				}
			}
		}
	}
	
	return tLocalizedPaneTitle;
}

#pragma mark -

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	// We want to make sure the no display text label is centered
	
	NSRect tViewFrame;
	NSRect tIconFrame;
	
	// Center the Language Popup button
	
	tViewFrame=[IBview_ frame];
	
	tIconFrame=[IBiconView_ frame];
	
	tIconFrame.origin.x=_CGFloatRound(NSMidX(tViewFrame)-NSWidth(tIconFrame)*0.5f);
	
	tIconFrame.origin.y=_CGFloatRound(NSMidY(tViewFrame)-NSHeight(tIconFrame)*0.5f);
	
	[IBiconView_ setFrame:tIconFrame];
}

- (void) windowStateDidChange:(NSNotification *) inNotification
{
	[self refreshIcon];
}

@end
