/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationViewLicenseController.h"

#import "ICLanguageConverter+Native.h"

#import "ICDropDocumentView.h"

#import "ICTitleUtilities.h"

#import "ICLicenseUtilities.h"
#import "ICLicenseProvider.h"

#import "ICLocalizationUtilities.h"


#import "ICDocument.h"

@implementation ICPresentationViewLicenseController

+ (ICPresentationViewLicenseController *) controller
{
	ICPresentationViewLicenseController * nController;
    
    nController=[ICPresentationViewLicenseController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICLicenseView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICLicenseView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
		else
		{
			[nController initLocalizationInformation];
		}
    }
    
    return [nController autorelease];
}

- (void) awakeFromNib
{
	fileManager_=[NSFileManager defaultManager]; 
	
	[IBtextView_ setTextContainerInset:NSMakeSize(6.0f,6.0f)];
	
	[IBtextView_ setDrawsBackground:NO];
	
	[IBdropDocumentView_ setStepIsVisible:NO];
	
	// Register for Notifications
												   
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(viewFrameDidChange:) 
												 name:NSViewFrameDidChangeNotification 
											  object:IBview_];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument andLanguage:(NSString *) inLanguage
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument andLanguage:inLanguage];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE];
	
	if (dictionary_==nil)
	{
		dictionary_=[ICLicenseUtilities defaultLicenseDictionary];
		
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE];
		}
	}
	
	if (dictionary_!=nil)
	{
		NSWindow * tWindow;
		
		[dictionary_ retain];
		
		[self refreshViewForLanguage:inLanguage];
		
		// Register for Notifications
	
		tWindow=[[self document] windowForSheet];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidBecomeMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidBecomeMainNotification 
												   object:tWindow];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidResignMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidResignMainNotification 
												   object:tWindow];
												   
		[[NSNotificationCenter defaultCenter] removeObserver:self name:ICPRESENTATION_NOTIFICATION_LICENSE_SETTINGS_DID_CHANGE object:[self document]];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(licenseSettingsDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_LICENSE_SETTINGS_DID_CHANGE 
												   object:[self document]];
	}
	else
	{
		// A COMPLETER
	}
}

- (void) dealloc
{
	[cachedDocumentPath_ release];
	
	[cachedModificationDate_ release];
	
	[cachedButtons_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) prepareLicenseButtonsForLanguage:(NSString *) inLanguage
{
	if (inLanguage!=nil && cachedButtons_!=nil)
	{
		ICInstallerPluginLocalizer * tPluginLocalizer;
		NSButton * tButton;
		NSRect tFrame;
		CGFloat tMinX,tMaxX;
		
		tPluginLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
		
		// Print / Customize
		
		tButton=[cachedButtons_ objectAtIndex:0];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"Print..."
			  forLanguage:inLanguage
				 inBundle:@"self"]];
		
		tFrame=[tButton frame];
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		[tButton setFrame:tFrame];
		
		tMinX=NSMaxX(tFrame);
		
		// Save
		
		tButton=[cachedButtons_ objectAtIndex:1];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"Save..."
			  forLanguage:inLanguage
				 inBundle:@"self"]];
		
		tFrame=[tButton frame];
		
		[tButton sizeToFit];
		
		tFrame.origin.x=tMinX;
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		[tButton setFrame:tFrame];
		
		// Continue
		
		tButton=[cachedButtons_ objectAtIndex:3];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"CONTINUE"
			  forLanguage:inLanguage
				 inBundle:@"Main"]];
	
		tFrame=[tButton frame];
		
		tMaxX=NSMaxX(tFrame);
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		tFrame.origin.x=tMaxX-NSWidth(tFrame);
		
		[tButton setFrame:tFrame];
		
		tMaxX=NSMinX(tFrame);
		
		// Go Back
		
		tButton=[cachedButtons_ objectAtIndex:2];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"GOBACK"
			  forLanguage:inLanguage
				 inBundle:@"Main"]];
		
		tFrame=[tButton frame];
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		tFrame.origin.x=tMaxX-NSWidth(tFrame);
		
		[tButton setFrame:tFrame];
		
		[[tButton superview] setNeedsDisplay:YES];
	}
}

- (void) prepareButtons:(NSArray *) inButtonsArray forLanguage:(NSString *) inLanguage
{
	[cachedButtons_ release];
	
	cachedButtons_=[inButtonsArray retain];
	
	[self prepareLicenseButtonsForLanguage:inLanguage];
}

- (void) _refreshViewForLanguage:(NSString *) inLanguage
{
	[cachedModificationDate_ release];
	
	cachedModificationDate_=nil;
	
	[cachedDocumentPath_ release];
	
	cachedDocumentPath_=nil;
	
	if (cachedCustomLicenseDictionary_==nil || [cachedCustomLicenseDictionary_ count]==0)
	{
		[IBlicenseView_ setHidden:YES];

		[IBdropDocumentView_ setHidden:NO];
		
		cachedCustomLicenseDictionary_=nil;
		
		cachedKeywordsValueDictionary_=nil;
		
		[self prepareLicenseButtonsForLanguage:inLanguage];
	}
	else
	{
		NSMutableArray * tNativeLanguages;
		NSString * tRequestedNativeLanguage;
		NSDictionary * tDictionary;
		
		[cachedCustomLicenseDictionary_ retain];
		
		[cachedKeywordsValueDictionary_ retain];
		
		[IBlicenseView_ setHidden:NO];

		[IBdropDocumentView_ setHidden:YES];
		
		// Build Language PopupButton
		
		[IBlanguagePopupButton_ removeAllItems];
		
		tNativeLanguages=[[cachedCustomLicenseDictionary_ allKeys] mutableCopy];
		
		if (tNativeLanguages!=nil)
		{
			[tNativeLanguages sortUsingSelector:@selector(caseInsensitiveCompare:)];
			
			[IBlanguagePopupButton_ addItemsWithTitles:tNativeLanguages];
			
			[tNativeLanguages release];
		}
		
		// Select and Display the most appropriate Localized License Document
		
		tRequestedNativeLanguage=[[ICLanguageConverter defaultConverter] nativeForEnglish:inLanguage];
		
		tDictionary=[cachedCustomLicenseDictionary_ objectForKey:tRequestedNativeLanguage];
		
		if (tDictionary!=nil)
		{
			[self showLicenseForNativeLanguage:tRequestedNativeLanguage];
			
			[IBlanguagePopupButton_ selectItemWithTitle:tRequestedNativeLanguage];
		}
		else
		{
			NSEnumerator * tEnumerator;
			
			// Try to find the best approching language
			
			tEnumerator=[cachedCustomLicenseDictionary_ objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSDictionary * tLicenseDictionary;
				NSMutableArray * tEnglishLanguages;
			
				tEnglishLanguages=[NSMutableArray array];
				
				while (tLicenseDictionary=[tEnumerator nextObject])
				{
					NSString * tEnglishLanguage;
					
					tEnglishLanguage=[tLicenseDictionary objectForKey:ICDOCUMENT_LANGUAGE];
					
					if (tEnglishLanguage!=nil)
					{
						[tEnglishLanguages addObject:tEnglishLanguage];
					}
				}
				
				tEnglishLanguages=(NSMutableArray *) CFBundleCopyPreferredLocalizationsFromArray((CFArrayRef) tEnglishLanguages);
			
				if (tEnglishLanguages!=nil && [tEnglishLanguages count]>0)
				{
					NSString * tEnglishLanguage;
					
					tEnglishLanguage=[tEnglishLanguages objectAtIndex:0];
					
					[cachedLicenseLanguage_ release];
					
					cachedLicenseLanguage_=[[ICLanguageConverter defaultConverter] nativeForEnglish:tEnglishLanguage];
					
					[cachedLicenseLanguage_ retain];
					
					tDictionary=[cachedCustomLicenseDictionary_ objectForKey:cachedLicenseLanguage_];
		
					if (tDictionary!=nil)
					{
						[self showLicenseForNativeLanguage:cachedLicenseLanguage_];
						
						[IBlanguagePopupButton_ selectItemWithTitle:cachedLicenseLanguage_];
						
						[self prepareLicenseButtonsForLanguage:tEnglishLanguage];
					}
				}
				
				[tEnglishLanguages release];
			}
		}
	}
}

- (void) refreshViewForLanguage:(NSString *) inLanguage
{
	[cachedLicenseLanguage_ release];
	
	cachedLicenseLanguage_=nil;

	if (dictionary_!=nil && inLanguage!=nil)
	{
		NSNumber * tNumber;
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_MODE];
	
		if (tNumber!=nil)
		{
			NSArray * tLocalizations;
			NSString * tLicenseTemplateName;
			
			[cachedCustomLicenseDictionary_ release];
			
			cachedCustomLicenseDictionary_=nil;
			
			[cachedKeywordsValueDictionary_ release];
			
			cachedKeywordsValueDictionary_=nil;
			
			switch([tNumber intValue])
			{
				case ICLICENSE_MODE_CUSTOM:
				
					tLocalizations=[dictionary_ objectForKey:ICDOCUMENT_LOCALIZATIONS];
					
					if (tLocalizations!=nil)
					{
						cachedCustomLicenseDictionary_=[ICLocalizationUtilities validLocalizationsPathFromLocalizationsArray:tLocalizations projectPath:[document_ folder] referencePath:[document_ referenceFolderPath]];
						
						[self _refreshViewForLanguage:inLanguage];
						
						return;
					}
					
					break;
					
				case ICLICENSE_MODE_TEMPLATE:
				
					tLicenseTemplateName=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_NAME];
					
					if (tLicenseTemplateName!=nil)
					{
						tLocalizations=[[ICLicenseProvider defaultProvider] localizationsForLicenseWithName:tLicenseTemplateName];
					
						if (tLocalizations!=nil)
						{
							cachedCustomLicenseDictionary_=[ICLocalizationUtilities validLocalizationsPathFromLocalizationsArray:tLocalizations projectPath:[document_ folder] referencePath:[document_ referenceFolderPath]];
							
							cachedKeywordsValueDictionary_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_TEMPLATE_KEYWORDS];
							
							[self _refreshViewForLanguage:inLanguage];
							
							return;
						}
					}
					
					break;
			}
			
bail:

			[IBlicenseView_ setHidden:YES];
					
			[IBdropDocumentView_ setHidden:NO];
							
			cachedCustomLicenseDictionary_=nil;
			
			cachedKeywordsValueDictionary_=nil;
							
			[self prepareLicenseButtonsForLanguage:inLanguage];
		}
	}
}

- (BOOL) isLocalizedValueUsable:(id) inValue
{
	if (inValue!=nil)
	{
		if ([inValue isKindOfClass:[NSString class]]==YES)
		{
			// Title
			
			NSString * tString;
		
			tString=(NSString *) inValue;
			
			if ([tString length]>0)
			{
				return YES;
			}
		}
		else
		{
			// Path
			
			NSDictionary * tDictionary;
			NSString * tPath;
			
			tDictionary=(NSDictionary *) inValue;
			
			tPath=[tDictionary objectForKey:ICDOCUMENT_PATH];
			
			if (tPath!=nil && [tPath length]>0)
			{
				NSNumber * tNumber;
				
				tNumber=[tDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
				
				if (tNumber!=nil)
				{
					NSInteger tInteger;
					
					tInteger=[tNumber integerValue];
					
					if (tInteger==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						// Get the absolute path
						
						tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
					}
					else if (tInteger==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						// Get the absolute path
						
						tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
					}
				}
				
				if ([fileManager_ fileExistsAtPath:tPath]==YES)
				{
					return YES;
				}
			}
		}
	}
	
	return NO;
}

- (NSString *) paneTitleForLanguage:(NSString *) inLanguage
{
	NSString * tLocalizedPaneTitle;
	ICInstallerPluginLocalizer * tLocalizer;
	NSString * tLanguage;
	
	if (cachedLicenseLanguage_==nil)
	{
		tLanguage=inLanguage;
	}
	else
	{
		tLanguage=[[cachedCustomLicenseDictionary_ objectForKey:cachedLicenseLanguage_] objectForKey:ICDOCUMENT_LANGUAGE];
		
		if (tLanguage==nil)
		{
			tLanguage=inLanguage;
		}
	}
	
	tLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
	
	tLocalizedPaneTitle=[tLocalizer localizedString:paneTitleKey_ forLanguage:tLanguage inBundle:pluginName_];
                
	if (tLocalizedPaneTitle==nil)
	{
		tLanguage=[[ICLanguageConverter defaultConverter] ISOFromEnglish:tLanguage];
		
		tLocalizedPaneTitle=[tLocalizer localizedString:paneTitleKey_ forLanguage:tLanguage inBundle:pluginName_];
		
		if (tLocalizedPaneTitle==nil)
		{
			tLocalizedPaneTitle=[super paneTitleForLanguage:inLanguage];
		}
	}
	
	return tLocalizedPaneTitle;
}

#pragma mark -

- (IBAction) switchLicenseLanguage:(id) sender
{
	NSString * tSelectedLanguage;
	
	tSelectedLanguage=[sender titleOfSelectedItem];
	
	if (cachedLicenseLanguage_==nil || [cachedLicenseLanguage_ isEqualToString:tSelectedLanguage]==NO)
	{
		[cachedLicenseLanguage_ release];
		
		cachedLicenseLanguage_=[tSelectedLanguage retain];
		
		[self showLicenseForNativeLanguage:cachedLicenseLanguage_];
	}
}

- (void) showLicenseForNativeLanguage:(NSString *) inLanguage
{
	NSDictionary * tLicenseDictionary;
	
	tLicenseDictionary=[cachedCustomLicenseDictionary_ objectForKey:inLanguage];
	
	if (tLicenseDictionary!=nil)
	{
		NSString * tLanguage;
		NSString * tDocumentPath;
			
		tLanguage=[tLicenseDictionary objectForKey:ICDOCUMENT_LANGUAGE];
		
		tDocumentPath=[tLicenseDictionary objectForKey:ICDOCUMENT_PATH];
		
		// Update Text View
		
		if (tDocumentPath!=nil)
		{
			NSTextStorage * tTextStorage;
			BOOL success;
			NSNumber * tNumber;
			NSDictionary * tAttributesDictionary;
			
			cachedDocumentPath_=[tDocumentPath retain];
			
			[IBtextView_ setString:@""];
			
			tTextStorage=[IBtextView_ textStorage];
			
			[tTextStorage beginEditing];
			
			success = [tTextStorage readFromURL:[NSURL fileURLWithPath:cachedDocumentPath_]
										options:[ICPresentationViewController defaultStorageOptionsWithPath:cachedDocumentPath_]
							 documentAttributes:NULL];
			
			
			tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_MODE];
	
			if (tNumber!=nil)
			{
				if ([tNumber intValue]==ICLICENSE_MODE_TEMPLATE)
				{
					if (cachedKeywordsValueDictionary_!=nil && [cachedKeywordsValueDictionary_ count]>0)
					{
						[ICLicenseProvider replaceKeywords:cachedKeywordsValueDictionary_
										inAttributedString:tTextStorage];
					}
				}
			}
			
			[tTextStorage endEditing];
			
			[IBtextView_ scrollRangeToVisible:NSMakeRange(0,0)];
			
			tAttributesDictionary=[fileManager_ fileAttributesAtPath:cachedDocumentPath_ traverseLink:YES];
			
			if (tAttributesDictionary!=nil)
			{
				cachedModificationDate_=[[tAttributesDictionary objectForKey:NSFileModificationDate] retain];
			}
		}
		
		// Update Buttons
		
		if (tLanguage!=nil)
		{
			[self prepareLicenseButtonsForLanguage:tLanguage];
			
			[[NSNotificationCenter defaultCenter] postNotificationName:ICPRESENTATION_NOTIFICATION_LICENSE_DISPLAYEDLANGUAGE_DID_CHANGE
														object:[self document]
													  userInfo:nil];
		}
	}
}

#pragma mark -

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	// We want to make sure the no display text label is centered
	
	NSRect tViewFrame;
	NSRect tLabelFrame;
	
	// Center the Language Popup button
	
	tViewFrame=[IBlicenseView_ frame];
	
	tLabelFrame=[IBlanguagePopupButton_ frame];
	
	tLabelFrame.origin.x=_CGFloatRound(NSMidX(tViewFrame)-NSWidth(tLabelFrame)*0.5f);
	
	[IBlanguagePopupButton_ setFrame:tLabelFrame];
	
	// Center the no Display text label
	
	tViewFrame=[IBdropDocumentView_ frame];
	
	tLabelFrame=[IBnoDisplayText_ frame];
	
	tLabelFrame.origin.x=_CGFloatRound(NSMidX(tViewFrame)-NSWidth(tLabelFrame)*0.5f);
	
	tLabelFrame.origin.y=_CGFloatRound(NSMidY(tViewFrame)-NSHeight(tLabelFrame)*0.5f);
	
	[IBnoDisplayText_ setFrame:tLabelFrame];
}

- (void) documentTextDidChange:(NSString *) inPath
{
	if (inPath!=nil)
	{
		if (dictionary_!=nil)
		{		
			NSMutableArray * tLocalizations;
			NSNumber * tNumber;
			BOOL tDocumentDidReallyChanged=NO;
			
			tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_MODE];
			
			if (tNumber==nil || [tNumber intValue]!=ICLICENSE_MODE_CUSTOM)
			{
				// We were not using a License Document
				
				tDocumentDidReallyChanged=YES;
				
				[dictionary_ setObject:[NSNumber numberWithInt:ICLICENSE_MODE_CUSTOM] forKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE_MODE];
			}
			
			tLocalizations=[dictionary_ objectForKey:ICDOCUMENT_LOCALIZATIONS];
			
			if (tLocalizations==nil)
			{
				tLocalizations=[NSMutableArray array];
				
				if (tLocalizations!=nil)
				{
					[dictionary_ setObject:tLocalizations forKey:ICDOCUMENT_LOCALIZATIONS];
				}
			}
			
			if (tLocalizations!=nil)
			{
				NSString * tCurrentPreviewLanguage;
				
				// Get the current Preview language
				
				if (cachedLicenseLanguage_==nil)
				{
					tCurrentPreviewLanguage=[document_ documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
				}
				else
				{
					tCurrentPreviewLanguage=[[cachedCustomLicenseDictionary_ objectForKey:cachedLicenseLanguage_] objectForKey:ICDOCUMENT_LANGUAGE];
				}
				
				if (tCurrentPreviewLanguage!=nil)
				{
					NSMutableDictionary * tLocalizationDictionary=nil;
					NSMutableDictionary * tPathDictionary=nil;
					NSEnumerator * tEnumerator;
					
					// Look for the Language in the Localizations array
					
					tEnumerator=[tLocalizations objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						while (tLocalizationDictionary=[tEnumerator nextObject])
						{
							NSString * tLanguage;
							
							tLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
							
							if (tLanguage!=nil && [tLanguage isEqualToString:tCurrentPreviewLanguage]==YES)
							{
								tPathDictionary=[tLocalizationDictionary objectForKey:ICDOCUMENT_VALUE];
							}
						}
					}
					
					if (tPathDictionary!=nil)
					{
						NSString * tPath;
						int tPathType;
						
						tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] intValue];
							
						tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
						
						if (tPath!=nil && [tPath length]>0)
						{
							if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
							{
								tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
							}
							else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
							{
								tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
							}
							
							if ([tPath caseInsensitiveCompare:inPath]!=NSOrderedSame)
							{
								// This is not the same bakground picture
								
								if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
								{
									inPath=[inPath stringByRelativizingToPath:[document_ folder]];
								}
								else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
								{
									inPath=[inPath stringByRelativizingToPath:[document_ referenceFolderPath]];
								}
								
								[tPathDictionary setObject:inPath forKey:ICDOCUMENT_PATH];
								
								tDocumentDidReallyChanged=YES;
							}
						}
						else
						{
							// No ReadMe document had been set
							
							if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
							{
								inPath=[inPath stringByRelativizingToPath:[document_ folder]];
							}
							else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
							{
								inPath=[inPath stringByRelativizingToPath:[document_ referenceFolderPath]];
							}
							
							[tPathDictionary setObject:inPath forKey:ICDOCUMENT_PATH];
							
							[tPathDictionary setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
							
							tDocumentDidReallyChanged=YES;
						}
					}
					else
					{
						tLocalizationDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tCurrentPreviewLanguage,ICDOCUMENT_LANGUAGE,
																								  [NSMutableDictionary dictionaryWithObjectsAndKeys:inPath,ICDOCUMENT_PATH,
																																					[NSNumber numberWithInteger:ICDOCUMENT_PATH_TYPE_GLOBAL],ICDOCUMENT_PATH_TYPE,
																																					nil],ICDOCUMENT_VALUE,
																								  nil];
						
						if (tLocalizationDictionary!=nil)
						{
							[tLocalizations addObject:tLocalizationDictionary];
							
							[tLocalizations sortUsingSelector:@selector(compareLocalizationLanguage:)];
							
							tDocumentDidReallyChanged=YES;
						}
					}
				}
			}
			
			if (tDocumentDidReallyChanged==YES)
			{
				[ICLicenseUtilities notifyLicenseDidChangeWithSender:self];
				
				[self refreshViewForLanguage:[document_ documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE]];
				
				[self noteDocumentChange];
			}
		}
	}
}

- (void) licenseSettingsDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if ([tUserInfo objectForKey:ICPRESENTATION_NOTIFICATION_SENDER]!=self)
		{
			[self refreshViewForLanguage:[document_ documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE]];
		}
	}
}

- (void) windowStateDidChange:(NSNotification *) inNotification
{
	if (cachedDocumentPath_!=nil)
	{
		BOOL tShouldRefresh=NO;
	
		if ([fileManager_ fileExistsAtPath:cachedDocumentPath_]==YES)
		{
			if (cachedModificationDate_!=nil)
			{
				NSDictionary * tAttributesDictionary;
			
				tAttributesDictionary=[fileManager_ fileAttributesAtPath:cachedDocumentPath_ traverseLink:YES];
			
				if (tAttributesDictionary!=nil)
				{
					NSDate * tDate;
				
					tShouldRefresh=YES;
					
					tDate=[tAttributesDictionary objectForKey:NSFileModificationDate];
					
					if (tDate!=nil)
					{
						if ([tDate compare:cachedModificationDate_]!=NSOrderedDescending)
						{
							tShouldRefresh=NO;
						}
					}
				}
			}
		}
		else
		{
			tShouldRefresh=YES;
		}
	
		if (tShouldRefresh==YES)
		{
			[self refreshViewForLanguage:[document_ documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE]];
		}
	}
}

@end
