/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPresentationViewReadMeController.h"
#import "ICDropDocumentView.h"

#import "ICTitleUtilities.h"

#import "ICReadMeUtilities.h"

#import "ICLocalizationUtilities.h"

#import "ICDocument.h"

@implementation ICPresentationViewReadMeController

+ (ICPresentationViewReadMeController *) controller
{
	ICPresentationViewReadMeController * nController;
    
    nController=[ICPresentationViewReadMeController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICReadMeView" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICReadMeView"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
		else
		{
			[nController initLocalizationInformation];
		}
    }
    
    return [nController autorelease];
}

- (void) awakeFromNib
{
	fileManager_=[NSFileManager defaultManager]; 
	
	[IBtextView_ setTextContainerInset:NSMakeSize(6.0f,6.0f)];
	
	[IBtextView_ setDrawsBackground:NO];
	
	[IBdropDocumentView_ setStepIsVisible:NO];
	
	// Register for Notifications
											   
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(viewFrameDidChange:) 
												 name:NSViewFrameDidChangeNotification 
											  object:IBview_];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument andLanguage:(NSString *) inLanguage
{
	[presentationDictionary_ release];
	
	presentationDictionary_=[inDictionary retain];
	
	[super initializeWithDictionary:inDictionary forDocument:inDocument andLanguage:inLanguage];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_README];
	
	if (dictionary_==nil)
	{
		dictionary_=[ICReadMeUtilities defaultReadMeDictionary];
		
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PROJECT_PRESENTATION_README];
		}
	}
	
	if (dictionary_!=nil)
	{
		NSWindow * tWindow;
		
		[dictionary_ retain];
		
		[self refreshViewForLanguage:inLanguage];
		
		// Register for Notifications
	
		tWindow=[[self document] windowForSheet];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidBecomeMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidBecomeMainNotification 
												   object:tWindow];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidResignMainNotification object:tWindow];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidResignMainNotification 
												   object:tWindow];
												   
												   [[NSNotificationCenter defaultCenter] removeObserver:self name:ICPRESENTATION_NOTIFICATION_README_SETTINGS_DID_CHANGE object:[self document]];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(readMeSettingsDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_README_SETTINGS_DID_CHANGE 
												   object:[self document]];
	}
	else
	{
		// A COMPLETER
	}
}

- (void) dealloc
{
	[cachedDocumentPath_ release];
	
	[cachedModificationDate_ release];
	
	[presentationDictionary_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) prepareButtons:(NSArray *) inButtonsArray forLanguage:(NSString *) inLanguage
{
	if (inLanguage!=nil)
	{
		ICInstallerPluginLocalizer * tPluginLocalizer;
		NSButton * tButton;
		NSRect tFrame;
		CGFloat tMinX,tMaxX;
		
		tPluginLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
		
		// Print / Customize
		
		tButton=[inButtonsArray objectAtIndex:0];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"Print..."
			  forLanguage:inLanguage
				 inBundle:@"self"]];
		
		tFrame=[tButton frame];
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		[tButton setFrame:tFrame];
		
		tMinX=NSMaxX(tFrame);
		
		// Save
		
		tButton=[inButtonsArray objectAtIndex:1];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"Save..."
			  forLanguage:inLanguage
				 inBundle:@"self"]];
		
		tFrame=[tButton frame];
		
		[tButton sizeToFit];
		
		tFrame.origin.x=tMinX;
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		[tButton setFrame:tFrame];
		
		// Continue
		
		tButton=[inButtonsArray objectAtIndex:3];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"CONTINUE"
			  forLanguage:inLanguage
				 inBundle:@"Main"]];
	
		tFrame=[tButton frame];
		
		tMaxX=NSMaxX(tFrame);
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		tFrame.origin.x=tMaxX-NSWidth(tFrame);
		
		[tButton setFrame:tFrame];
		
		tMaxX=NSMinX(tFrame);
		
		// Go Back
		
		tButton=[inButtonsArray objectAtIndex:2];
		
		[tButton setHidden:NO];
		
		[tButton setTitle:[tPluginLocalizer localizedString:@"GOBACK"
			  forLanguage:inLanguage
				 inBundle:@"Main"]];
		
		tFrame=[tButton frame];
		
		[tButton sizeToFit];
		
		tFrame.size.width=NSWidth([tButton frame]);
		
		if (tFrame.size.width<=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH)
		{
			tFrame.size.width=ICPRESENTATIONVIEWCONTROLLER_MIN_BUTTON_WIDTH;
		}
		else
		{
			tFrame.size.width+=13.0f;
		}
		
		tFrame.origin.x=tMaxX-NSWidth(tFrame);
		
		[tButton setFrame:tFrame];
		
		[[tButton superview] setNeedsDisplay:YES];
	}
}

- (void) refreshViewForLanguage:(NSString *) inLanguage
{
	[cachedModificationDate_ release];
	
	cachedModificationDate_=nil;
	
	[cachedDocumentPath_ release];
	
	cachedDocumentPath_=nil;
	
	if (dictionary_!=nil && inLanguage!=nil)
	{
		NSString * tDocumentPath=nil;
		
		tDocumentPath=[ICReadMeUtilities readMeDocumentPathFromDictionary:dictionary_ forLanguage:inLanguage inDocument:document_];
		
		if (tDocumentPath==nil)
		{
			if ([IBscrollView_ isHidden]==NO)
			{
				[IBtextView_ setString:@""];
				
				[IBscrollView_ setHidden:YES];
			
				[IBdropDocumentView_ setHidden:NO];
			}
		}
		else
		{
			NSTextStorage * tTextStorage;
			BOOL success;
			NSDictionary * tAttributesDictionary;
			
			cachedDocumentPath_=[tDocumentPath retain];
						
			[IBtextView_ setString:@""];
			
			tTextStorage=[IBtextView_ textStorage];
			
			[tTextStorage beginEditing];
			
			success = [tTextStorage readFromURL:[NSURL fileURLWithPath:cachedDocumentPath_]
										options:[ICPresentationViewController defaultStorageOptionsWithPath:cachedDocumentPath_]
							 documentAttributes:NULL];
							 
			[tTextStorage endEditing];
			
			if ([IBscrollView_ isHidden]==YES)
			{
				[IBscrollView_ setHidden:NO];
			}

			[IBdropDocumentView_ setHidden:YES];
			
			[IBtextView_ scrollRangeToVisible:NSMakeRange(0,1)];
			
			tAttributesDictionary=[fileManager_ fileAttributesAtPath:cachedDocumentPath_ traverseLink:YES];
			
			if (tAttributesDictionary!=nil)
			{
				cachedModificationDate_=[[tAttributesDictionary objectForKey:NSFileModificationDate] retain];
			}
		}
	}
}

- (BOOL) isLocalizedValueUsable:(id) inValue
{
	if (inValue!=nil)
	{
		if ([inValue isKindOfClass:[NSString class]]==YES)
		{
			// Title
			
			NSString * tString;
		
			tString=(NSString *) inValue;
			
			if ([tString length]>0)
			{
				return YES;
			}
		}
		else
		{
			// Path
			
			NSDictionary * tDictionary;
			NSString * tPath;
			
			tDictionary=(NSDictionary *) inValue;
			
			tPath=[tDictionary objectForKey:ICDOCUMENT_PATH];
			
			if (tPath!=nil && [tPath length]>0)
			{
				NSNumber * tNumber;
				
				tNumber=[tDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
				
				if (tNumber!=nil)
				{
					NSInteger tInteger;
					
					tInteger=[tNumber integerValue];
					
					if (tInteger==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						// Get the absolute path
						
						tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
					}
					else if (tInteger==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						// Get the absolute path
						
						tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
					}
				}
				
				if ([fileManager_ fileExistsAtPath:tPath]==YES)
				{
					return YES;
				}
			}
		}
	}
	
	return NO;
}

#pragma mark -

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	// We want to make sure the no display text label is centered
	
	NSRect tViewFrame;
	NSRect tLabelFrame;
	
	tViewFrame=[IBdropDocumentView_ frame];
	
	tLabelFrame=[IBnoDisplayText_ frame];
	
	tLabelFrame.origin.x=_CGFloatRound(NSMidX(tViewFrame)-NSWidth(tLabelFrame)*0.5f);
	
	tLabelFrame.origin.y=_CGFloatRound(NSMidY(tViewFrame)-NSHeight(tLabelFrame)*0.5f);
	
	[IBnoDisplayText_ setFrame:tLabelFrame];
}

- (void) documentTextDidChange:(NSString *) inPath
{
	if (inPath!=nil)
	{
		if (dictionary_!=nil)
		{		
			NSMutableArray * tLocalizations;
			BOOL tDocumentDidReallyChanged=NO;
			
			tLocalizations=[dictionary_ objectForKey:ICDOCUMENT_LOCALIZATIONS];
			
			if (tLocalizations==nil)
			{
				tLocalizations=[NSMutableArray array];
				
				if (tLocalizations!=nil)
				{
					[dictionary_ setObject:tLocalizations forKey:ICDOCUMENT_LOCALIZATIONS];
				}
			}
			
			if (tLocalizations!=nil)
			{
				NSString * tCurrentPreviewLanguage;
				
				// Get the current Preview language
				
				tCurrentPreviewLanguage=[document_ documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
				
				if (tCurrentPreviewLanguage!=nil)
				{
					NSMutableDictionary * tLocalizationDictionary=nil;
					NSMutableDictionary * tPathDictionary=nil;
					NSEnumerator * tEnumerator;
					
					// Look for the Language in the Localizations array
					
					tEnumerator=[tLocalizations objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						while (tLocalizationDictionary=[tEnumerator nextObject])
						{
							NSString * tLanguage;
							
							tLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
							
							if (tLanguage!=nil && [tLanguage isEqualToString:tCurrentPreviewLanguage]==YES)
							{
								tPathDictionary=[tLocalizationDictionary objectForKey:ICDOCUMENT_VALUE];
							}
						}
					}
					
					if (tPathDictionary!=nil)
					{
						NSString * tPath;
						NSInteger tPathType;
						
						tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
							
						tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
						
						if (tPath!=nil && [tPath length]>0)
						{
							if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
							{
								tPath=[tPath stringByAbsolutingWithPath:[document_ folder]];
							}
							else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
							{
								tPath=[tPath stringByAbsolutingWithPath:[document_ referenceFolderPath]];
							}
								
							if ([tPath caseInsensitiveCompare:inPath]!=NSOrderedSame)
							{
								// This is not the same ReadMe document
								
								if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
								{
									inPath=[inPath stringByRelativizingToPath:[document_ folder]];
								}
								else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
								{
									inPath=[inPath stringByRelativizingToPath:[document_ referenceFolderPath]];
								}
								
								[tPathDictionary setObject:inPath forKey:ICDOCUMENT_PATH];
								
								tDocumentDidReallyChanged=YES;
							}
						}
						else
						{
							// No ReadMe document had been set
							
							if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
							{
								inPath=[inPath stringByRelativizingToPath:[document_ folder]];
							}
							else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
							{
								inPath=[inPath stringByRelativizingToPath:[document_ referenceFolderPath]];
							}
							
							[tPathDictionary setObject:inPath forKey:ICDOCUMENT_PATH];
							
							[tPathDictionary setObject:[NSNumber numberWithInt:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
							
							tDocumentDidReallyChanged=YES;
						}
					}
					else
					{
						tLocalizationDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tCurrentPreviewLanguage,ICDOCUMENT_LANGUAGE,
																								  [NSMutableDictionary dictionaryWithObjectsAndKeys:inPath,ICDOCUMENT_PATH,
																																					[NSNumber numberWithInteger:ICDOCUMENT_PATH_TYPE_GLOBAL],ICDOCUMENT_PATH_TYPE,
																																					nil],ICDOCUMENT_VALUE,
																								  nil];
						
						if (tLocalizationDictionary!=nil)
						{
							[tLocalizations addObject:tLocalizationDictionary];
							
							[tLocalizations sortUsingSelector:@selector(compareLocalizationLanguage:)];
							
							tDocumentDidReallyChanged=YES;
						}
					}
				}
			}
			
			if (tDocumentDidReallyChanged==YES)
			{
				[ICReadMeUtilities notifyReadMeDidChangeWithSender:self];
				
				[self refreshViewForLanguage:[document_ documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE]];
				
				[self noteDocumentChange];
			}
		}
	}
}

- (void) readMeSettingsDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if ([tUserInfo objectForKey:ICPRESENTATION_NOTIFICATION_SENDER]!=self)
		{
			[self refreshViewForLanguage:[document_ documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE]];
		}
	}
}

- (void) windowStateDidChange:(NSNotification *) inNotification
{
	if (cachedDocumentPath_!=nil)
	{
		BOOL tShouldRefresh=NO;
	
		if ([fileManager_ fileExistsAtPath:cachedDocumentPath_]==YES)
		{
			if (cachedModificationDate_!=nil)
			{
				NSDictionary * tAttributesDictionary;
			
				tAttributesDictionary=[fileManager_ fileAttributesAtPath:cachedDocumentPath_ traverseLink:YES];
			
				if (tAttributesDictionary!=nil)
				{
					NSDate * tDate;
				
					tShouldRefresh=YES;
					
					tDate=[tAttributesDictionary objectForKey:NSFileModificationDate];
					
					if (tDate!=nil)
					{
						if ([tDate compare:cachedModificationDate_]!=NSOrderedDescending)
						{
							tShouldRefresh=NO;
						}
					}
				}
			}
		}
		else
		{
			tShouldRefresh=YES;
		}
	
		if (tShouldRefresh==YES)
		{
			[self refreshViewForLanguage:[document_ documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE]];
		}
	}
}

@end
