/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectAssistantTemplateCell.h"

#import "NSBezierPath+RoundedRect.h"

#define ICONFRAME_WIDTH		88.0f
#define ICONFRAME_HEIGHT	88.0f

#define TEXTFRAME_RADIUS	8.0f

@implementation ICProjectAssistantTemplateCell

- (void) drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
	NSRect tFrame;
	NSImage * tImage;
	NSSize tSize;
	NSMutableAttributedString * tTitle=nil;
	
	tTitle=[[NSMutableAttributedString alloc] initWithAttributedString:[self attributedTitle]];
	
	if (_cFlags.highlighted || _cFlags.state)
	{
		NSBezierPath * tBezierPath;
		
		// Icon background
		
		tFrame=cellFrame;
		
		tFrame.origin.x=_CGFloatRound(NSMidX(cellFrame)-0.5f*ICONFRAME_WIDTH);
		
		tFrame.origin.y=0;
		
		tFrame.size.width=ICONFRAME_WIDTH;
		
		tFrame.size.height=ICONFRAME_HEIGHT;
		
		tBezierPath=[NSBezierPath bezierPathWithRoundedRect:tFrame radius:8.0f];
		
		if (tBezierPath!=nil)
		{
			[[NSColor colorWithDeviceWhite:0.8f alpha:1.0f] set];
			
			[tBezierPath fill];
		}
		
		// Text Background
	
		if (tTitle!=nil)
		{
			tSize=[tTitle size];
		
			tFrame.origin.x=_CGFloatRound(NSMidX(cellFrame)-0.5f*tSize.width)-TEXTFRAME_RADIUS-5.0f;
			
			tFrame.origin.y=NSMaxY(cellFrame)-2*TEXTFRAME_RADIUS;
			
			tFrame.size.width=tSize.width+2*TEXTFRAME_RADIUS+10.0f;
		
			tFrame.size.height=2*TEXTFRAME_RADIUS;
			
			tBezierPath=[NSBezierPath bezierPathWithRoundedRect:tFrame radius:TEXTFRAME_RADIUS];
		
			if (tBezierPath!=nil)
			{
				[[NSColor colorWithDeviceRed:0.2118f green:0.4627f blue:0.8510f alpha:1.0f] set];
				
				[tBezierPath fill];
			}
		}
	}
	
	// Draw Icon
	
	tImage=[[self image] copy];
	
	if (tImage!=nil)
	{
		[tImage setScalesWhenResized:YES];
		
		[tImage setSize:NSMakeSize(64.0f,64.0f)];
		
		tSize=[tImage size];
		
		tFrame.origin.x=_CGFloatRound(NSMidX(cellFrame)-0.5f*tSize.width);
		
		tFrame.origin.y=_CGFloatRound(0.5f*(ICONFRAME_WIDTH+tSize.height));;
		
		tFrame.size=tSize;
	
		[tImage compositeToPoint:tFrame.origin operation:NSCompositeSourceOver];
	
		[tImage release];
	}
	
	// Draw Text
	
	if (tTitle!=nil)
	{
		if (_cFlags.highlighted || _cFlags.state)
		{
			[tTitle addAttribute:NSForegroundColorAttributeName value:[NSColor whiteColor] range:NSMakeRange(0,[tTitle length])];
		}
		
		tSize=[tTitle size];
		
		[tTitle drawAtPoint:NSMakePoint(_CGFloatRound(NSMidX(cellFrame)-0.5f*tSize.width),NSMaxY(cellFrame)-2*TEXTFRAME_RADIUS-1.0f)];
	
		[tTitle release];
	}
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
	[self drawInteriorWithFrame:cellFrame inView:controlView];
}

@end
