/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectCommentsController.h"

@implementation ICProjectCommentsController

+ (ICProjectCommentsController *) projectCommentsController
{
	ICProjectCommentsController * nController=nil;
    
    nController=[ICProjectCommentsController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Project_Comments" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Project_Comments"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

#pragma mark -

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_COMMENTS];
	
	if (dictionary_!=nil)
	{
		NSData * tData;
		
		[dictionary_ retain];
		
		tData=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_COMMENTS_NOTES];
		
		//NSLog(@"%@",tData);
		
		if (tData==nil || [tData length]==0)
		{
			[IBtextView_ setString:@""];
		}
		else
		{
			NSTextStorage * tTextStorage;
			BOOL success;
			NSDictionary * tFontDictionary;
    
			tFontDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSFont systemFontOfSize:[NSFont systemFontSize]],NSFontAttributeName,nil];
																  
			tTextStorage=[IBtextView_ textStorage];
        
            [tTextStorage beginEditing];
			
            success = [tTextStorage readFromData:tData
                                        options:[NSDictionary dictionaryWithObjectsAndKeys:tFontDictionary,@"DefaultAttributes",nil]
                            documentAttributes:nil/*[NSDictionary dictionaryWithObjectsAndKeys:NSHTMLTextDocumentType,NSDocumentTypeDocumentAttribute,nil]*/];
			
			[tTextStorage endEditing];
		}
		
	}

	
}

- (void) viewWillBeRemoved:(id) sender
{
	NSData * tData;
	NSError * tError;
	
	tData=[[IBtextView_ textStorage] dataFromRange:NSMakeRange(0,[[IBtextView_ textStorage] length])
								documentAttributes:[NSDictionary dictionaryWithObjectsAndKeys:NSHTMLTextDocumentType,NSDocumentTypeDocumentAttribute,nil]
											 error:&tError];
	
	if (tData!=nil)
	{
		[dictionary_ setObject:tData forKey:ICDOCUMENT_PROJECT_COMMENTS_NOTES];
		
		modificationsHaveBeenReported_=YES;
	}
}

- (void) documentWillBeSaved:(NSNotification *) inNotification
{
	if (modificationsHaveBeenReported_==NO)
	{
		NSData * tData;
		NSError * tError;
	
		tData=[[IBtextView_ textStorage] dataFromRange:NSMakeRange(0,[[IBtextView_ textStorage] length])
									documentAttributes:[NSDictionary dictionaryWithObjectsAndKeys:NSHTMLTextDocumentType,NSDocumentTypeDocumentAttribute,nil]
												 error:&tError];
		
		if (tData!=nil)
		{
			[dictionary_ setObject:tData forKey:ICDOCUMENT_PROJECT_COMMENTS_NOTES];
		
			modificationsHaveBeenReported_=YES;
		}
	}
}

- (void)textDidChange:(NSNotification *) inNotification
{
	modificationsHaveBeenReported_=NO;
	
	// Notify Document Change
			
	[self noteDocumentChange];
}

@end
