/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectController.h"

#import "ICProjectSettingsController.h"
#import "ICProjectPresentationController.h"

#import "ICInstallationType+Constants.h"

#import "ICProjectRequirementsController.h"
#import "ICProjectCommentsController.h"

#import "ICPreferences+Constants.h"

#import "ICDocument.h"

@implementation ICProjectController

+ (ICProjectController *) projectController
{
	ICProjectController * nController=nil;
    
    nController=[ICProjectController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Project_Pane" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Project_Pane"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) awakeFromNib
{
	[super awakeFromNib];
}

#pragma mark -

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT];
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
		NSInteger tSelectedSegment=0;
		
		[dictionary_ retain];
		
		// Select the appropriate Segment
		
		tNumber=[document_ documentDefaultForKey:@"ui.project.selected.segment"];
		
		if (tNumber!=nil)
		{
			tSelectedSegment=[tNumber intValue];
		}
		else
		{
			tSelectedSegment=[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_PROJECT_PANE];
		}
		
		[IBsegmentedView_ setSelectedSegment:tSelectedSegment];
		
		[self showView:IBsegmentedView_];
		
		// Register for Notifications
	
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(installationTypeChoiceDependencyEditDidRequest:) 
													 name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_EDIT_DIDREQUEST
												   object:document_];
												   
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(installationTypeChoiceDependencyEndEditDidRequest:) 
													 name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_ENDEDIT_DIDREQUEST
												   object:document_];
	}
}
	

- (ICSegmentedController *) segmentedControllerWithTag:(NSInteger) inTag
{
	[document_ setDocumentDefault:[NSNumber numberWithInteger:inTag] forKey:@"ui.project.selected.segment"];
	
	switch(inTag)
	{
		case ICPROJECT_PANE_SETTINGS:
		
			return [ICProjectSettingsController projectSettingsController];
			
		case ICPROJECT_PANE_PRESENTATION:
		
			return [ICProjectPresentationController projectPresentationController];
			
		case ICPROJECT_PANE_REQUIREMENTS:
		
			return [ICProjectRequirementsController projectRequirementsController];
			
		case ICPROJECT_PANE_COMMENTS:
		
			return [ICProjectCommentsController projectCommentsController];
	}
	
	return nil;
}

#pragma mark -

- (void) installationTypeChoiceDependencyEditDidRequest:(NSNotification *) inNotification
{
	[IBsegmentedView_ setEnabled:NO];
}

- (void) installationTypeChoiceDependencyEndEditDidRequest:(NSNotification *) inNotification
{
	[IBsegmentedView_ setEnabled:YES];
}

@end
