/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICSegmentedController.h"

#import "ICPresentation+Constants.h"

#import "ICPresentationViewController.h"

@interface ICProjectPresentationController : ICSegmentedController
{
	IBOutlet id IBleftView_;
	
	IBOutlet id IBwindowView_;
	
	IBOutlet id IBproxyIconView_;
	
	IBOutlet id IBwindowTitleView_;
	
	IBOutlet id IBchapterTitleView_;
	
	IBOutlet id IBbackgroundView_;
	
	IBOutlet id IBinstallationStepsView_;
	
	IBOutlet id IBchapterContentView_;
	
	
	IBOutlet id IBprintButton_;
	
	IBOutlet id IBsaveButton_;
	
	IBOutlet id IBgoBackButton_;
	
	IBOutlet id IBcontinueButton_;
	
	
	IBOutlet id IBrightView_;
	
	IBOutlet id IBseparator_;
	
	IBOutlet id IBinspectorPopupButton_;
	
	IBOutlet id IBinspectorView_;
	
	
	IBOutlet id IBpluginsBox_;
	
	IBOutlet id IBpluginsAddButton_;
	
	IBOutlet id IBpluginsRemoveButton_;
	
	IBOutlet id IBaccessoryView_;
	
	IBOutlet id IBlanguagePreviewPopupButton_;
	
	id surgeryWindow_;
	
	// Data
	
	NSInteger cachedBuildFormat_;
	
	NSArray * navigationButtonsArray_;
	
	id currentInspectorController_;
	
	id currentViewController_;
	
	NSString * currentPreviewLanguage_;
	
	NSMutableArray * chaptersArray_;
	
	
	NSMutableArray * installationSteps_;
	
	NSMutableArray * visibleInstallationSteps_;

	// Drag and drop Data
	
	int internalDragStep_;
	
	CGFloat savedRightViewWidth_;
	
	CGFloat savedInspectorViewSpace_;
}

+ (ICProjectPresentationController *) projectPresentationController;

- (int) chapterTagForClassName:(NSString *) inName;

- (IBAction) addPlugIn:(id) sender;

- (void) addPluginDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo;

- (IBAction) removePlugIn:(id) sender;

- (void) removePlugInDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo;

- (IBAction) switchPreviewLanguage:(id) sender;

- (IBAction) switchInspectedView:(id) sender;

- (void) showViewWithTag:(NSInteger) inTag;

- (void) updateTitleViews;

+ (BOOL) shouldDrawGroxBoxForImage:(NSImage *) inImage andSettings:(NSDictionary *) inSettings;

- (void) updateBackgroundView;

// Proxy selectors

- (void) performIfPossibleSelector:(SEL) inSelector withObject:(id) inObject;

- (IBAction) group:(id) sender;

- (IBAction) ungroup:(id) sender;

- (IBAction) merge:(id) sender;

- (IBAction) separate:(id) sender;

// Notifications

- (void) windowViewDidResize:(NSNotification *) inNotification;

- (void) inspectorViewDidResize:(NSNotification *) inNotification;

- (void) titleSettingsDidChange:(NSNotification *) inNotification;

- (void) backgroundSettingsDidChange:(NSNotification *) inNotification;

- (void) readMeSettingsDidChange:(NSNotification *) inNotification;

- (void) licenseSettingsDidChange:(NSNotification *) inNotification;

- (void) licenseDisplayedLanguageDidChange:(NSNotification *) inNotification;

- (void) enlargeChoiceDependencyInspector:(NSNotification *) inNotification;

- (void) reduceChoiceDependencyInspector:(NSNotification *) inNotification;

- (void) leftViewDidResize:(NSNotification *) inNotification;

- (void) windowStateDidChange:(NSNotification *) inNotification;

@end
