/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectPresentationController.h"

#import "ICPreferences+Constants.h"

#import "ICPresentationPluginButton.h"

#import "ICLanguageConverter.h"
#import "ICInstallerLocalizer.h"

#import "ICBackgroundUtilities.h"

#import "ICPresentation+Notifications.h"

#import "ICLocalizationUtilities.h"

#import "ICInstallerPluginLocalizer.h"

#import "ICDocument.h"

#import "ICPresentationListView.h"

#import "ICPresentationImageView.h"

#import "ICInstallationSurgeryWindow.h"

//

#import "ICIconsUtilities.h"

#import "ICTitleUtilities.h"

#import "ICReadMeUtilities.h"

#import "ICLicenseUtilities.h"

#import "ICInstallationStepsUtilities.h"

#import "ICInstallationType+Constants.h"

#import "ICPresentationViewInstallationTypeController.h"

// Inspectors

#import "ICPresentationInspectorTitleController.h"

#import "ICPresentationInspectorBackgroundController.h"

#import "ICPresentationInspectorIntroductionController.h"

#import "ICPresentationInspectorReadMeController.h"

#import "ICPresentationInspectorSummaryController.h"

#import "ICPresentationInspectorInstallerPluginController.h"

#define ICPRESENTATIONINSPECTOR_ENLARGEMENT		170.0f

NSString * const ICPROJECTPRESENTATIONCONTROLLER_INTERNALSTEP_PBOARDTYPE=@"ICPROJECTPRESENTATIONCONTROLLER_INTERNALSTEP_PBOARDTYPE";

@implementation ICProjectPresentationController

+ (ICProjectPresentationController *) projectPresentationController
{
	ICProjectPresentationController * nController=nil;
    
    nController=[ICProjectPresentationController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Project_Presentation" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Project_Presentation"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	//NSLog(@"[ICProjectPresentationController dealloc]");
	
	[navigationButtonsArray_ release];
	
	[chaptersArray_ release];
	
	[currentPreviewLanguage_ release];
	
	[visibleInstallationSteps_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{
	NSMutableArray * tLanguageArray;
	NSUInteger i,tLanguageCount;
	NSArray * tItemsArray;
	NSImage * tPackageIcon;
	
	// Set the icon for the window
	
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
	
	tPackageIcon=[[NSImage alloc] initWithContentsOfFile:@"/System/Library/CoreServices/Installer.app/Contents/Resources/package.icns"];

#else
	
	tPackageIcon=[[NSImage alloc] initWithContentsOfFile:@"/Applications/Utilities/Installer.app/Contents/Resources/package.icns"];
	
#endif	

	if (tPackageIcon!=nil)
	{
		[IBproxyIconView_ setImage:tPackageIcon];
		
		[tPackageIcon release];
	}
	
	// Window Title
	
	[[IBwindowTitleView_ cell] setLineBreakMode:NSLineBreakByTruncatingTail];
	
	// Navigation Buttons
	
	
	navigationButtonsArray_=[[NSArray alloc] initWithObjects:IBprintButton_,
															 IBsaveButton_,
															 IBgoBackButton_,
															 IBcontinueButton_,
															 nil];
	
	
	[IBpluginsAddButton_ setPlus:YES];
	
	// Build the Preview In Menu
	
	[IBlanguagePreviewPopupButton_ removeAllItems];
    
    tLanguageArray=[[[ICInstallerLocalizer defaultLocalizer] localizations] mutableCopy];
    
    tLanguageCount=[tLanguageArray count];
    
    for(i=0;i<tLanguageCount;i++)
    {
        NSString * tEnglishName;
        
        tEnglishName=[[ICLanguageConverter defaultConverter] englishFromISO:[tLanguageArray objectAtIndex:i]];
		
        if (tEnglishName!=nil)
        {
            [tLanguageArray replaceObjectAtIndex:i withObject:tEnglishName];
        }
    }
    
    [tLanguageArray sortUsingSelector:@selector(caseInsensitiveCompare:)];
    
    [IBlanguagePreviewPopupButton_ addItemsWithTitles:tLanguageArray];
	
	[tLanguageArray release];
	
	tItemsArray=[IBlanguagePreviewPopupButton_ itemArray];
	
	if (tItemsArray!=nil)
	{
		NSUInteger tCount;
		
		tCount=[tItemsArray count];
		
		for(i=0;i<tCount;i++)
		{
			NSMenuItem * tMenuItem;
			NSString * tMenuTitle;
			
			tMenuItem=[tItemsArray objectAtIndex:i];
			
			tMenuTitle=[tMenuItem title];
			
			if (tMenuTitle!=nil)
			{
				NSString * tISOLanguageName;
				NSImage * tImage;
				
				tISOLanguageName=[[ICLanguageConverter defaultConverter] ISOFromEnglish:tMenuTitle];
				
				tImage=[NSImage imageNamed:[NSString stringWithFormat:@"flag_%@",tISOLanguageName]];
				
				if (tImage!=nil)
				{
					[tMenuItem setImage:tImage];
				}
			}
		}
	}
	
	// Enable Core Animation
	
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5

	//[IBbackgroundView_ setWantsLayer:YES];

#endif
	
	// Register for Drag and Drop
	
	[IBinstallationStepsView_ registerForDraggedTypes:[NSArray arrayWithObjects:ICPROJECTPRESENTATIONCONTROLLER_INTERNALSTEP_PBOARDTYPE,NSFilenamesPboardType,nil]];
	
	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(inspectorViewDidResize:) 
												 name:NSViewFrameDidChangeNotification 
											  object:IBinspectorView_];

	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(windowViewDidResize:) 
												 name:NSViewFrameDidChangeNotification 
											  object:IBwindowView_];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	NSString * tSavedPreviewLanguage;
	NSNumber * tNumber;
	
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	[visibleInstallationSteps_ release];
	
	visibleInstallationSteps_=nil;
	
	[currentPreviewLanguage_ release];
	
	currentPreviewLanguage_=nil;
	
	tSavedPreviewLanguage=[[self document] documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
	
	if (tSavedPreviewLanguage!=nil)
	{
		currentPreviewLanguage_=[tSavedPreviewLanguage retain];
	}
	else
	{
		// Set the current Preview language to the most prefered one
		
		NSMutableArray * tEnglishLanguageNames;
		
		tEnglishLanguageNames=[ICLocalizationUtilities englishLanguages];
		
		if (tEnglishLanguageNames!=nil)
		{
			NSArray * tPreferedLocalizations;
				
			tPreferedLocalizations=(NSArray *) CFBundleCopyPreferredLocalizationsFromArray((CFArrayRef) tEnglishLanguageNames);
			
			if (tPreferedLocalizations!=nil && [tPreferedLocalizations count]>0)
			{
				currentPreviewLanguage_=[[tPreferedLocalizations objectAtIndex:0] copy];
			}
			
			[tPreferedLocalizations release];
		}
		
		if (currentPreviewLanguage_==nil)
		{
			currentPreviewLanguage_=[[NSString alloc] initWithString:@"English"];
		}
		
		if (currentPreviewLanguage_!=nil)
		{
			[[self document] setDocumentDefault:currentPreviewLanguage_ forKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
		}
	}
	
	// Select the Preview language
	
	[IBlanguagePreviewPopupButton_ selectItemWithTitle:currentPreviewLanguage_];
	
	cachedBuildFormat_=ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_FLAT;
	
	tNumber=[[inDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS] objectForKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT];
	
	if (tNumber!=nil)
	{
		cachedBuildFormat_=[tNumber integerValue];
	}
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION];
	
	if (dictionary_!=nil)
	{
		NSInteger tSelectedStep=0;
		
		// Get the list of Installation Steps
		
		[installationSteps_ release];
		
		installationSteps_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_INSTALLATION_STEPS];
		
		// Check that we have all default Steps
		
		if (installationSteps_==nil)
		{
			// Create the steps
			
			installationSteps_=[ICInstallationStepsUtilities defaultInstallationSteps];
			
			if (installationSteps_!=nil)
			{
				[dictionary_ setObject:installationSteps_ forKey:ICDOCUMENT_PROJECT_INSTALLATION_STEPS];
			}
			else
			{
				// A COMPLETER
			}
		}
		else
		{
			// Check that the steps are correctly set
		
			// A COMPLETER
		}
		
		if (installationSteps_!=nil)
		{
			/*if (cachedBuildFormat_==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_FLAT)
			{
				visibleInstallationSteps_=[[NSMutableArray alloc] initWithCapacity:5];
				
				if (visibleInstallationSteps_!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[installationSteps_ objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSDictionary * tStepDictionary;
						
						while (tStepDictionary=[tEnumerator nextObject])
						{
							NSString * tControllerClassName;
							
							tControllerClassName=[tStepDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
							
							if ([tControllerClassName isEqualToString:@"ICPresentationViewInstallerPluginController"]==NO)
							{
								[visibleInstallationSteps_ addObject:tStepDictionary];
							}
						}
					}
				}
				else
				{
					// A COMPLETER
				}
			}
			else*/
			{
				visibleInstallationSteps_=[installationSteps_ retain];
			}
			
			/*[IBpluginsBox_ setHidden:(cachedBuildFormat_==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_FLAT)];*/
		}
		
		[chaptersArray_ release];
		
		// Initialize the Array of chapters and the Inspector Popup Button
		
		chaptersArray_=[[NSMutableArray  alloc] initWithCapacity:10];
		
		if (chaptersArray_!=nil)
		{
			NSMutableDictionary * tChapterDictionary;
			
			// Title
			
			tChapterDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Title",@""),ICPRESENTATION_CHAPTER_TITLE,
																				 [NSNumber numberWithBool:NO],ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED,
																				 @"ICPresentationInspectorTitleController",ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER_CLASS,
																				 @"ICPresentationViewIntroductionController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																				 [NSNumber numberWithInt:0],ICPRESENTATION_CHAPTER_INSPECTOR_TAG,
																				 nil];
			
			if (tChapterDictionary==nil)
			{
				// A COMPLETER
				
				return;
			}
			
			[chaptersArray_ addObject:tChapterDictionary];
			
			// Background
			
			tChapterDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Background",@""),ICPRESENTATION_CHAPTER_TITLE,
																				 [NSNumber numberWithBool:NO],ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED,
																				 @"ICPresentationInspectorBackgroundController",ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER_CLASS,
																				 @"ICPresentationViewIntroductionController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																				 [NSNumber numberWithInt:1],ICPRESENTATION_CHAPTER_INSPECTOR_TAG,
																				 nil];
			
			if (tChapterDictionary==nil)
			{
				// A COMPLETER
				
				return;
			}
			
			[chaptersArray_ addObject:tChapterDictionary];
			
			// A COMPLETER (prendre en compte l'ordre avec les plugins
			
			// Introduction
			
			tChapterDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Introduction",@""),ICPRESENTATION_CHAPTER_TITLE,
																				 [NSNumber numberWithBool:YES],ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED,
																				 @"ICPresentationInspectorIntroductionController",ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER_CLASS,
																				 @"ICPresentationViewIntroductionController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																				 [NSNumber numberWithInt:2],ICPRESENTATION_CHAPTER_INSPECTOR_TAG,
																				 nil];
			
			
			
			if (tChapterDictionary==nil)
			{
				// A COMPLETER
				
				return;
			}
			
			[chaptersArray_ addObject:tChapterDictionary];
			
			// Read Me
			
			tChapterDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Read Me",@""),ICPRESENTATION_CHAPTER_TITLE,
																				 [NSNumber numberWithBool:YES],ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED,
																				 @"ICPresentationInspectorReadMeController",ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER_CLASS,
																				 @"ICPresentationViewReadMeController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																				 [NSNumber numberWithInt:3],ICPRESENTATION_CHAPTER_INSPECTOR_TAG,
																				 nil];
			
			
			
			if (tChapterDictionary==nil)
			{
				// A COMPLETER
				
				return;
			}
			
			[chaptersArray_ addObject:tChapterDictionary];
			
			// License
			
			tChapterDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"License",@""),ICPRESENTATION_CHAPTER_TITLE,
																				 [NSNumber numberWithBool:YES],ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED,
																				 @"ICPresentationInspectorLicenseController",ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER_CLASS,
																				 @"ICPresentationViewLicenseController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																				 [NSNumber numberWithInt:4],ICPRESENTATION_CHAPTER_INSPECTOR_TAG,
																				 nil];
			
			
			
			if (tChapterDictionary==nil)
			{
				// A COMPLETER
				
				return;
			}
			
			[chaptersArray_ addObject:tChapterDictionary];
			
			// Installation Type
			
			tChapterDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Installation Type",@""),ICPRESENTATION_CHAPTER_TITLE,
																				 [NSNumber numberWithBool:YES],ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED,
																				 @"ICPresentationInspectorInstallationTypeController",ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER_CLASS,
																				 @"ICPresentationViewInstallationTypeController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																				 [NSNumber numberWithInt:5],ICPRESENTATION_CHAPTER_INSPECTOR_TAG,
																				 nil];
			
			
			
			if (tChapterDictionary==nil)
			{
				// A COMPLETER
				
				return;
			}
			
			[chaptersArray_ addObject:tChapterDictionary];
			
			// Summary
			
			tChapterDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Summary",@""),ICPRESENTATION_CHAPTER_TITLE,
																				 [NSNumber numberWithBool:YES],ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED,
																				 @"ICPresentationInspectorSummaryController",ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER_CLASS,
																				 @"ICPresentationViewSummaryController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																				 [NSNumber numberWithInt:6],ICPRESENTATION_CHAPTER_INSPECTOR_TAG,
																				 nil];
			
			
			
			if (tChapterDictionary==nil)
			{
				// A COMPLETER
				
				return;
			}
			
			[chaptersArray_ addObject:tChapterDictionary];
			
			// Installer Plugin
			
			tChapterDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Installer Plugin",@""),ICPRESENTATION_CHAPTER_TITLE,
																				 [NSNumber numberWithBool:YES],ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED,
																				 @"ICPresentationInspectorInstallerPluginController",ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER_CLASS,
																				 @"ICPresentationViewInstallerPluginController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																				 [NSNumber numberWithInt:-1],ICPRESENTATION_CHAPTER_INSPECTOR_TAG,
																				 nil];
			
			
			
			if (tChapterDictionary==nil)
			{
				// A COMPLETER
				
				return;
			}
			
			[chaptersArray_ addObject:tChapterDictionary];
		}
		
		// Find the selected step (if possible)
	
		tNumber=[[self document] documentDefaultForKey:@"ui.project.presentation.step.selected"];
	
		if (tNumber!=nil)
		{
			tSelectedStep=[tNumber integerValue];
			
			if (tSelectedStep>=[visibleInstallationSteps_ count])
			{
				tSelectedStep=0;
			}
		}
		
		[IBinstallationStepsView_ selectStepAtIndex:tSelectedStep];
		
		[self listViewSelectionDidChange:nil];
		
		// A COMPLETER
		
		[dictionary_ retain];
	
		// A COMPLETER
		
		// Build the Inspector Menu
	
		// A COMPLETER
		
		// Update Plugins UI depending on whether we can handle plugins or not
		
		// A COMPLETER (cachedBuildFormat_)
		
		// Initialize the Background View
		
		[self updateBackgroundView];
		
		// Register for Notifications
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(backgroundSettingsDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_BACKGROUND_SETTINGS_DID_CHANGE 
												   object:[self document]];


		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(titleSettingsDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_TITLE_SETTINGS_DID_CHANGE 
												   object:[self document]];

		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(readMeSettingsDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_README_SETTINGS_DID_CHANGE 
												   object:[self document]];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(licenseSettingsDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_LICENSE_SETTINGS_DID_CHANGE 
												   object:[self document]];
																																   
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(licenseDisplayedLanguageDidChange:) 
													 name:ICPRESENTATION_NOTIFICATION_LICENSE_DISPLAYEDLANGUAGE_DID_CHANGE 
												   object:[self document]];
												   
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(enlargeChoiceDependencyInspector:) 
													 name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_ENLARGE_INSPECTOR 
												   object:[self document]];
												   
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(reduceChoiceDependencyInspector:) 
													 name:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_CHOICE_REDUCE_INSPECTOR 
												   object:[self document]];
												   
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(leftViewDidResize:) 
													 name:NSViewFrameDidChangeNotification
												   object:IBleftView_];

		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidBecomeMainNotification 
												   object:[[self document] windowForSheet]];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(windowStateDidChange:) 
													 name:NSWindowDidResignMainNotification 
												   object:[[self document] windowForSheet]];
	}
}

#pragma mark -

- (void) viewWillBeRemoved:(id) sender
{
	if (currentViewController_!=nil)
	{
		[currentViewController_ viewWillBeRemoved:self];
	}
}

#pragma mark -

- (int) chapterTagForClassName:(NSString *) inName
{
	if (chaptersArray_!=nil && inName!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[chaptersArray_ objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tDictionary;
			
			while (tDictionary=[tEnumerator nextObject])
			{
				NSString * tClassNanme;
				
				tClassNanme=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
				
				if (tClassNanme!=nil && [tClassNanme isEqualToString:inName]==YES)
				{
					NSNumber * tNumber;
					
					tNumber=[tDictionary objectForKey: ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED];
					
					if (tNumber==nil || [tNumber boolValue]==YES)
					{
						return [[tDictionary objectForKey:ICPRESENTATION_CHAPTER_INSPECTOR_TAG] intValue];
					}
				}
			}
		}
	}

	return 0;
}

- (void) updateTitleViews
{
	NSString * tPaneTitle;
	NSString * tMostAppropriateLocalizedTitle=nil;
	NSDictionary * tTitleDictionary;
	NSMutableArray * tLocalizationsArray;
	NSArray * tSimplifiedLocalizationsArray;
	NSRect tWindowViewFrame;
	NSRect tProxyIconFrame;
	NSRect tWindowTitleFrame;
	CGFloat tAvailableWidth;
	CGFloat tMiddleXWindowView;
	
	// Refresh Chapter Title View
	
	tPaneTitle=[currentViewController_ paneTitleForLanguage:currentPreviewLanguage_];
	
	if (tPaneTitle!=nil)
	{
		[IBchapterTitleView_ setStringValue:tPaneTitle];
	}
	else
	{
		[IBchapterTitleView_ setStringValue:@""];
	}

	// Refresh Fake Window Title
	
	tTitleDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_TITLE];
		
	if (tTitleDictionary!=nil)
	{
		tLocalizationsArray=[tTitleDictionary objectForKey:ICDOCUMENT_LOCALIZATIONS];
		
		if (tLocalizationsArray!=nil)
		{
			tSimplifiedLocalizationsArray=[ICTitleUtilities clearedLocalizationsValue:tLocalizationsArray];
			
			tMostAppropriateLocalizedTitle=[ICLocalizationUtilities localizedValueForLanguage:currentPreviewLanguage_ inLocalizations:tSimplifiedLocalizationsArray lookForBestMatch:NO];
		}
	}
	
	if (tMostAppropriateLocalizedTitle==nil)
	{
		// Use Project document name
		
		tLocalizationsArray=[ICTitleUtilities defaultLocalizationsArrayForDocument:[self document]];
		
		if (tLocalizationsArray!=nil)
		{
			tSimplifiedLocalizationsArray=[ICTitleUtilities clearedLocalizationsValue:tLocalizationsArray];
			
			tMostAppropriateLocalizedTitle=[ICLocalizationUtilities localizedValueForLanguage:currentPreviewLanguage_ inLocalizations:tSimplifiedLocalizationsArray];
		}
	}
	
	if (tMostAppropriateLocalizedTitle!=nil)
	{
		ICInstallerPluginLocalizer * tPluginLocalizer;
		
		tPluginLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
		
		if (tPluginLocalizer!=nil)
		{
			NSString * tStringFormat;
			
			tStringFormat=[tPluginLocalizer localizedString:@"WindowTitle" forLanguage:currentPreviewLanguage_ inBundle:@"Main"];
			
			if (tStringFormat!=nil)
			{
				tMostAppropriateLocalizedTitle=[NSString stringWithFormat:tStringFormat,tMostAppropriateLocalizedTitle];
			}
		}
		
		[IBwindowTitleView_ setStringValue:tMostAppropriateLocalizedTitle];
	}
	else
	{
		[IBwindowTitleView_ setStringValue:@""];
	}
	
#define FAKEWINDOW_LEFTOFFSET	80.0f

#define FAKEWINDOW_RIGHTOFFSET	10.0f

#define FAKEWINDOW_ICON_INTERSPACE	2.0f

	tWindowViewFrame=[IBwindowView_ frame];
	
	tProxyIconFrame=[IBproxyIconView_ frame];
	
	tMiddleXWindowView=NSMidX(tWindowViewFrame);
	
	tAvailableWidth=NSWidth(tWindowViewFrame)-FAKEWINDOW_LEFTOFFSET-FAKEWINDOW_RIGHTOFFSET-NSWidth(tProxyIconFrame)-FAKEWINDOW_ICON_INTERSPACE;
	
	[IBwindowTitleView_ sizeToFit];
	
	tWindowTitleFrame=[IBwindowTitleView_ frame];
	
	if (NSWidth(tWindowTitleFrame) < (tAvailableWidth-1))
	{
		tWindowTitleFrame.size.width+=1.0f;
	}
	else
	{
		tWindowTitleFrame.size.width=tAvailableWidth;
	}
	
	tProxyIconFrame.origin.x=_CGFloatRound(tMiddleXWindowView-(NSWidth(tWindowTitleFrame)+NSWidth(tProxyIconFrame)+FAKEWINDOW_ICON_INTERSPACE)*0.5f);
	
	if (NSMinX(tProxyIconFrame)<=FAKEWINDOW_LEFTOFFSET)
	{
		tProxyIconFrame.origin.x=FAKEWINDOW_LEFTOFFSET;
	}
	
	tWindowTitleFrame.origin.x=NSMinX(tProxyIconFrame)+NSWidth(tProxyIconFrame)+FAKEWINDOW_ICON_INTERSPACE;
	
	[IBproxyIconView_ setFrame:tProxyIconFrame];
	
	[IBwindowTitleView_ setFrame:tWindowTitleFrame];
	
	[IBwindowView_ setNeedsDisplayInRect:NSMakeRect(FAKEWINDOW_LEFTOFFSET,NSHeight(tWindowViewFrame)-25.0f,NSWidth(tWindowViewFrame)-FAKEWINDOW_LEFTOFFSET-FAKEWINDOW_RIGHTOFFSET,25.0f)];
}

+ (BOOL) shouldDrawGroxBoxForImage:(NSImage *) inImage andSettings:(NSDictionary *) inSettings
{
	/*if (inImage!=nil && inSettings!=nil)
	{
		NSSize tSize;
		
		tSize=[inImage size];
		
		// A COMPLETER
	}*/

	return YES;
}

- (void) updateBackgroundView
{
	NSMutableDictionary * tBackgroundDictionary;
	
	tBackgroundDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND];
		
	if (tBackgroundDictionary==nil)
	{
		// Create default Background dictionary
		
		tBackgroundDictionary=[ICBackgroundUtilities defaultBackgroundDictionary];	
	}
	
	if (tBackgroundDictionary!=nil)
	{		
		NSNumber * tNumber;
		NSImage * tImage=nil;
		
		// Custom
		
		tNumber=[tBackgroundDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_CUSTOM];
		
		if (tNumber==nil || [tNumber boolValue]==NO)
		{
			// Load the default Background Picture

image_bail:
			
			tImage=[ICBackgroundUtilities defaultBackgroundPicture];
			
			if (tImage!=nil)
			{
				[IBbackgroundView_ setImage:tImage];
				
				[IBbackgroundView_ setImageAlignment:ICALIGNMENT_LEFT];
			
				[IBbackgroundView_ setImageScaling:ICSCALING_PROPORTIONALLY];
			}
			
			[IBbackgroundView_ setDrawGrowBox:YES];
		}
		else
		{
			// Load the custom Background Picture
			
			NSDictionary * tPathDictionary;
			
			tPathDictionary=[tBackgroundDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
			
			if (tPathDictionary!=nil)
			{
				NSString * tPath;
				
				tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
				
				if (tPath!=nil)
				{
					NSInteger tPathType;
				
					tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
					
					if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						tPath=[tPath stringByAbsolutingWithPath:[[self document] folder]];
					}
					else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						tPath=[tPath stringByAbsolutingWithPath:[[self document] referenceFolderPath]];
					}
					
					tImage=[[NSImage alloc] initWithContentsOfFile:tPath];
					
					if (tImage==nil)
					{
						// Display a question Mark
						
						tImage=[ICIconsUtilities bigIconForType:NSFileTypeForHFSTypeCode(kQuestionMarkIcon)];
						
						[IBbackgroundView_ setDrawGrowBox:NO];
					}
					else
					{
						// Find if the window will be resizable
						
						[IBbackgroundView_ setDrawGrowBox:[ICProjectPresentationController shouldDrawGroxBoxForImage:tImage andSettings:tBackgroundDictionary]];
						
						[tImage autorelease];
					}
				}
			}
			
			if (tImage==nil)
			{
				goto image_bail;
			}
			else
			{
				[IBbackgroundView_ setImage:tImage];
				
				tNumber=[tBackgroundDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_ALIGNMENT];
				
				if (tNumber==nil)
				{
					[IBbackgroundView_ setImageAlignment:ICALIGNMENT_LEFT];
				}
				else
				{
					[IBbackgroundView_ setImageAlignment:[tNumber intValue]];
				}
				
				tNumber=[tBackgroundDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_SCALING];
	
				if (tNumber==nil)
				{
					[IBbackgroundView_ setImageScaling:ICSCALING_TO_FIT];
				}
				else
				{
					[IBbackgroundView_ setImageScaling:[tNumber intValue]];
				}
			}
		}
	}
}

#pragma mark -

- (IBAction) addPlugIn:(id) sender
{
	NSOpenPanel * tOpenPanel;
    
	tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:YES];
	[tOpenPanel setAllowsMultipleSelection:YES];
	[tOpenPanel setDelegate:self];
	
    [tOpenPanel setPrompt:NSLocalizedString(@"Add",@"No comment")];
    
    [tOpenPanel beginSheetForDirectory:nil
                                  file:nil
                                 types:[NSArray arrayWithObject:@"bundle"]
                        modalForWindow:[[self document] windowForSheet]
                         modalDelegate:self
                        didEndSelector:@selector(addPluginDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (BOOL)panel:(id)sender shouldShowFilename:(NSString *) inPath
{
    if ([[inPath pathExtension] caseInsensitiveCompare:@"bundle"]==NSOrderedSame)
	{
		if ([ICInstallationStepsUtilities installationSteps:installationSteps_ ofDocument:[self document] containsInstallerPluginAtPath:inPath]==YES)
		{
			//return NO;	// A DECOMMENTER
		}
	}
	
	return YES;
}

- (void) addPluginDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        NSArray * tArray;
		
		
		tArray=[sheet filenames];
		
		if (tArray!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[tArray objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSString * tPath;
				NSInteger tInsertionIndex;
				NSBundle * tBundle;
				BOOL tModified=NO;
				NSInteger tPathType;
				
				tPathType=[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
				
				tInsertionIndex=[IBinstallationStepsView_ selectedStep]+1;
				
				while (tPath=[tEnumerator nextObject])
				{
					tBundle=[NSBundle bundleWithPath:tPath];
		
					// Create the dictionary
					
					if (tBundle!=nil)
					{
						NSMutableDictionary * tInstallerPluginDictionary;
						NSDictionary * tPathDictionary;
						
						if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
						{
							tPath=[tPath stringByRelativizingToPath:[[self document] folder]];
						}
						else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
						{
							tPath=[tPath stringByAbsolutingWithPath:[[self document] referenceFolderPath]];
						}
					
						tPathDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tPath,ICDOCUMENT_PATH,
																						  [NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																						  nil];
						
						tInstallerPluginDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:@"ICPresentationViewInstallerPluginController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																									 tPathDictionary,ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_INSTALLERPLUGIN_PATH,
																									 nil];
						
					
						if (tInstallerPluginDictionary!=nil)
						{
							[installationSteps_ insertObject:tInstallerPluginDictionary atIndex:tInsertionIndex];
							
							// Refresh the list view
							
							[IBinstallationStepsView_ setNeedsDisplay:YES];
							
							tInsertionIndex++;
							
							tModified=YES;
						}
					}
					else
					{
						// A COMPLETER
					}
				}
				
				if (tModified==YES)
				{
					[self noteDocumentChange];
				}
			}
		}
	}
}

- (IBAction) removePlugIn:(id) sender
{
	NSBeginAlertSheet(NSLocalizedString(@"Do you really want to remove this Installer plugin?",@"No comment"),
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [[self document] windowForSheet],
                      self,
                      @selector(removePlugInDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));

}

- (void) removePlugInDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSInteger tSelectedStep;
		
		tSelectedStep=[IBinstallationStepsView_ selectedStep];
		
		if (tSelectedStep>=0)
		{
			NSInteger tIndex;
			
			[installationSteps_ removeObjectAtIndex:tSelectedStep];
			
			// Find the first selectable Step
			
			tIndex=tSelectedStep;
			
			if (tIndex==0)
			{
				[IBinstallationStepsView_ selectStepAtIndex:0];
			}
			else
			{
				tIndex=tSelectedStep-1;
				
				for (;tIndex>=0;tIndex--)
				{
					if ([self listView:IBinstallationStepsView_ shouldSelectStep:tIndex]==YES)
					{
						[IBinstallationStepsView_ selectStepAtIndex:tIndex];
						
						break;
					}
				}
			}
			
			// Refresh the list view
			
			[self listViewSelectionDidChange:nil];
			
			[self noteDocumentChange];
		}
	}
}

- (IBAction) switchPreviewLanguage:(id) sender
{
	NSString * tSelectedPreviewLanguage;
	
	tSelectedPreviewLanguage=[sender titleOfSelectedItem];
	
	if ([tSelectedPreviewLanguage isEqualToString:currentPreviewLanguage_]==NO)
	{
		[currentPreviewLanguage_ release];
		
		currentPreviewLanguage_=[tSelectedPreviewLanguage copy];
		
		[[self document] setDocumentDefault:currentPreviewLanguage_ forKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
		
		if (currentViewController_!=nil)
		{
			// Refresh Window title and Pane title
		
			[self updateTitleViews];
			
			// Refresh Pane View
	
			[currentViewController_ refreshViewForLanguage:currentPreviewLanguage_];
		}
		
		// Refresh List View
		
		[IBinstallationStepsView_ setNeedsDisplay:YES];
	}
}

- (IBAction) switchInspectedView:(id) sender
{
	NSInteger tTag;
	NSDictionary * tChapterDictionary;
	
	// Check that the View is different
	
	tTag=[[sender selectedItem] tag];
	
	if (tTag>=0)
	{
		NSEnumerator * tEnumerator;
		
NS_DURING
		
		tEnumerator=[chaptersArray_ objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			while (tChapterDictionary=[tEnumerator nextObject])
			{
				if ([[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_INSPECTOR_TAG] intValue]==tTag)
				{
					break;
				}
			}
		}
		
NS_HANDLER

		tChapterDictionary=nil;

NS_ENDHANDLER

		if (tChapterDictionary!=nil)
		{
			id tController;
			
			tController=[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER];
			
			if (tController==nil || currentInspectorController_!=tController)
			{
				NSString * tViewControllerClassName;
				
				tViewControllerClassName=[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
				
				if (tViewControllerClassName!=nil)
				{
					// Select the View
					
					NSEnumerator * tStepEnumerator;
					
					tStepEnumerator=[visibleInstallationSteps_ objectEnumerator];
					
					if (tStepEnumerator!=nil)
					{
						NSDictionary * tDictionary;
						NSUInteger tIndex=0;
						
						while (tDictionary=[tStepEnumerator nextObject])
						{
							NSString * tClassName;
							
							tClassName=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
							
							if (tClassName!=nil && [tClassName isEqualToString:tViewControllerClassName]==YES)
							{
								break;
							}
							
							tIndex++;
						}
						
						if (tIndex<=[visibleInstallationSteps_ count])
						{
							[IBinstallationStepsView_ selectStepAtIndex:tIndex];
						}
					}
				}
				
				[self showViewWithTag:tTag];
				
				// Save the selected step
				
				[[self document] setDocumentDefault:[NSNumber numberWithInteger:[IBinstallationStepsView_ selectedStep]] forKey:@"ui.project.presentation.step.selected"];
				
				// A COMPLETER
		
				// Run the animation
		
				// A COMPLETER
			}
		}
	}
}

- (void) showViewWithTag:(NSInteger) inTag
{
	//if (currentInspectorController_!=nil)
	{
		NSView * tView;
		NSMutableDictionary * tChapterDictionary;
		NSEnumerator * tEnumerator;
		
		if (currentInspectorController_!=nil)
		{
			tView=[currentInspectorController_ view];
		
			[tView removeFromSuperview];
		}

NS_DURING
		
		tEnumerator=[chaptersArray_ objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			while (tChapterDictionary=[tEnumerator nextObject])
			{
				if ([[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_INSPECTOR_TAG] intValue]==inTag)
				{
					break;
				}
			}
		}
		
NS_HANDLER

		tChapterDictionary=nil;
		
		// A COMPLETER

NS_ENDHANDLER

		currentInspectorController_=[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER];
		
		if (currentInspectorController_==nil)
		{
			NSString * tClassName;
			
			tClassName=[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER_CLASS];
			
			if (tClassName!=nil)
			{
				Class tInspectorClass;
			
				tInspectorClass=NSClassFromString(tClassName);
				
				if (tInspectorClass!=nil)
				{
					currentInspectorController_=[tInspectorClass controller];
					
					if (currentInspectorController_!=nil)
					{
						[tChapterDictionary setObject:currentInspectorController_ forKey:ICPRESENTATION_CHAPTER_INSPECTOR_CONTROLLER];
					}
				}
				else
				{
					// A COMPLETER
				}
			}
			else
			{
				// A COMPLETER
			}
			
		}
		
		if (currentInspectorController_!=nil)
		{
			NSRect tFrame;
			NSUInteger tAutoresizemask;
			NSRect tBounds;
			NSNumber * tNumber;
			
			if (currentViewController_!=nil)
			{
				tView=[currentViewController_ view];
		
				[tView removeFromSuperview];
				
				tView=[currentViewController_ accessoryView];
		
				if (tView!=nil)
				{
					[tView removeFromSuperview];
				}
			}
			
			currentViewController_=[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER];
			
			if (currentViewController_==nil)
			{
				NSString * tClassName;
			
				tClassName=[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
				
				if (tClassName!=nil)
				{
					Class tViewClass;
				
					tViewClass=NSClassFromString(tClassName);
					
					if (tViewClass!=nil)
					{
						currentViewController_=[tViewClass controller];
						
						if (currentViewController_!=nil)
						{
							[tChapterDictionary setObject:currentViewController_ forKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER];
						}
					}
					else
					{
						// A COMPLETER
					}
				}
				else
				{
					// A COMPLETER
				}
			}
			
			// Set Inspector View
			
			tBounds=[IBinspectorView_ bounds];
			
			if ([currentInspectorController_ isMemberOfClass:[ICPresentationInspectorInstallerPluginController class]]==YES)
			{
				NSDictionary * tDictionary;
				NSMutableDictionary * tPathDictionary=nil;
				
NS_DURING

				tDictionary=[visibleInstallationSteps_ objectAtIndex:[IBinstallationStepsView_ selectedStep]];
	
NS_HANDLER

				tDictionary=nil;
	
NS_ENDHANDLER

				if (tDictionary!=nil)
				{
					tPathDictionary=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_INSTALLERPLUGIN_PATH];
				}							
				
				[currentInspectorController_ initializeWithDictionary:tPathDictionary forDocument:[self document]];
			}
			else
			{
				[currentInspectorController_ initializeWithDictionary:dictionary_ forDocument:[self document]];
			}
			
	
			tView=[currentInspectorController_ view];
			
			if (tView!=nil)
			{
				tAutoresizemask=[tView autoresizingMask];
			
				tFrame=[tView frame];
				
				if ((tAutoresizemask & NSViewWidthSizable)==NSViewWidthSizable)
				{
					tFrame.origin.x=0.0f;
					tFrame.size.width=NSWidth(tBounds);
				}
				else
				{
					// Center horizontally
					
					tFrame.origin.x=_CGFloatRound(NSMidX(tBounds)-NSWidth(tFrame)*0.5f);
				}
				
				if ((tAutoresizemask & NSViewHeightSizable)==NSViewHeightSizable)
				{
					tFrame.origin.y=0.0f;
					tFrame.size.height=NSHeight(tBounds);
				}
				else
				{
					// Center horizontally
					
					tFrame.origin.y=_CGFloatRound(NSMidY(tBounds)-NSHeight(tFrame)*0.5f);
				}
				
				[tView setFrame:tFrame];
				
				[IBinspectorView_ addSubview:tView];
			}
			
			// View controller
			
			if (currentViewController_!=nil)
			{
				NSString * tClassName;
				BOOL isInstallerPluginChapter=NO;
				id tAccessoryView;
				
				tClassName=[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
				
				if (tClassName!=nil)
				{
					if ([tClassName isEqualToString:@"ICPresentationViewInstallerPluginController"]==YES)
					{
						isInstallerPluginChapter=YES;
					}
				}
				
				// Set Navigation Buttons
				
				[currentViewController_ prepareButtons:navigationButtonsArray_ forLanguage:currentPreviewLanguage_];
				
				// Is Chapter enabled?
				
				tNumber=[tChapterDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_IS_ENABLED];
				
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					// View is disabled
				}
				else
				{
					// View is enabled
				}
				
				tBounds=[IBchapterContentView_ bounds];
				
				if (isInstallerPluginChapter==NO)
				{
					[currentViewController_ initializeWithDictionary:dictionary_ forDocument:[self document] andLanguage:currentPreviewLanguage_];
				}
				else
				{
					NSDictionary * tDictionary;
					NSMutableDictionary * tPathDictionary=nil;
					
NS_DURING

					tDictionary=[visibleInstallationSteps_ objectAtIndex:[IBinstallationStepsView_ selectedStep]];
					
					//NSLog(@"%@",tDictionary);
		
NS_HANDLER

					tDictionary=nil;
		
NS_ENDHANDLER
	
					if (tDictionary!=nil)
					{
						tPathDictionary=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_INSTALLERPLUGIN_PATH];
					}							
					
					[currentViewController_ initializeWithDictionary:tPathDictionary forDocument:[self document] andLanguage:currentPreviewLanguage_];
				}
				
				tView=[currentViewController_ view];
				
				// Set Pane title
				
				[self updateTitleViews];
				
				// Set the View
				
				[tView setFrame:tBounds];
				
				[IBchapterContentView_ addSubview:tView];
				
				tAccessoryView=[currentViewController_ accessoryView];
				
				if (tAccessoryView!=nil)
				{
					tBounds=[IBaccessoryView_ bounds];
					
					[tAccessoryView setFrame:tBounds];
				
					[IBaccessoryView_ addSubview:tAccessoryView];
				}
			}
		}

	}
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
	SEL tAction=[inMenuItem action];
	
	if (tAction==@selector(switchInspectedView:))
    {
		if ([inMenuItem tag]==-1)
		{
			return NO;
		}
    }
    
	if ([currentViewController_ respondsToSelector:tAction]==YES)
	{
		return [currentViewController_ validateMenuItem:inMenuItem];
	}
	
	if (tAction==@selector(group:) ||
		tAction==@selector(ungroup:) ||
		tAction==@selector(merge:) ||
		tAction==@selector(separate:))
	{
		return NO;
	}
	
    return YES;
}

- (void) performIfPossibleSelector:(SEL) inSelector withObject:(id) inObject
{
	if ([currentViewController_ respondsToSelector:inSelector]==YES)
	{
		[currentViewController_ performSelector:inSelector withObject:inObject];
	}
}

- (IBAction) group:(id) sender
{
	[self performIfPossibleSelector:@selector(group:) withObject:sender];
}

- (IBAction) ungroup:(id) sender
{
	[self performIfPossibleSelector:@selector(ungroup:) withObject:sender];
}

- (IBAction) merge:(id) sender
{
	[self performIfPossibleSelector:@selector(merge:) withObject:sender];
}

- (IBAction) separate:(id) sender
{
	[self performIfPossibleSelector:@selector(separate:) withObject:sender];
}

#pragma mark -

- (NSInteger) numberOfStepsInListView:(ICPresentationListView *) inListView
{
	if (visibleInstallationSteps_!=nil)
	{
		return [visibleInstallationSteps_ count];
	}
	
	return 0;
}

- (id) listView:(ICPresentationListView *) inListView objectForStep:(NSInteger) inStep
{
	if (visibleInstallationSteps_!=nil)
	{
		NSDictionary * tDictionary;
		
NS_DURING

		tDictionary=[visibleInstallationSteps_ objectAtIndex:inStep];
		
NS_HANDLER

		tDictionary=nil;
		
NS_ENDHANDLER
	
		if (tDictionary!=nil)
		{
			NSString * tClassName;
			
			tClassName=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
			
			if (tClassName!=nil)
			{
				if ([tClassName isEqualToString:@"ICPresentationViewInstallerPluginController"]==YES)
				{
					// This is an Installer Plugin
					
					NSDictionary * tPathDictionary;
					
					tPathDictionary=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_INSTALLERPLUGIN_PATH];
					
					if (tPathDictionary!=nil)
					{
						NSString * tPath;
						NSInteger tPathType;
						
						tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
							
						tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
						
						if (tPath!=nil && [tPath length]>0)
						{
							if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
							{
								tPath=[tPath stringByAbsolutingWithPath:[[self document] folder]];
							}
							else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
							{
								tPath=[tPath stringByAbsolutingWithPath:[[self document] referenceFolderPath]];
							}
					
							if (tPath!=nil)
							{
								NSFileManager * tFileManager;
								
								tFileManager=[NSFileManager defaultManager];
								
								if (tFileManager!=nil)
								{
									BOOL isDirectory;
									
									if ([tFileManager fileExistsAtPath:tPath isDirectory:&isDirectory]==YES && isDirectory==YES)
									{
										NSBundle * tBundle;
								
										tBundle=[NSBundle bundleWithPath:tPath];
								
										if (tBundle!=nil)
										{
											return [ICInstallerPluginLocalizer localizedString:@"InstallerSectionTitle" forLanguage:currentPreviewLanguage_ inExternalBundle:tBundle];
										}
										else
										{
											return [[ICInstallerPluginLocalizer defaultLocalizer] localizedString:@"Not Found" forLanguage:currentPreviewLanguage_ inBundle:@"self"];
										}
									}
									else
									{
										return [[ICInstallerPluginLocalizer defaultLocalizer] localizedString:@"Not Found" forLanguage:currentPreviewLanguage_ inBundle:@"self"];
									}
								}
							}
						}
					}
				}
				else
				{
					// Get the name from the View Controller list
					
					NSString * tLocalizedListTitle;
					ICInstallerPluginLocalizer * tLocalizer;
					NSString * tListTitleKey;
					NSString * tPluginBundleName;
					
					tLocalizer=[ICInstallerPluginLocalizer defaultLocalizer];
					
					tListTitleKey=[tDictionary objectForKey:ICPRESENTATIONVIEWCONTROLLER_LIST_TITLE_KEY];
					
					tPluginBundleName=[tDictionary objectForKey:ICPRESENTATIONVIEWCONTROLLER_PLUGIN_NAME];
					
					tLocalizedListTitle=[tLocalizer localizedString:tListTitleKey forLanguage:currentPreviewLanguage_ inBundle:tPluginBundleName];
								
					if (tLocalizedListTitle==nil)
					{
						NSString * tLanguage;
						
						tLocalizedListTitle=[tLocalizer localizedString:tListTitleKey forLanguage:currentPreviewLanguage_ inBundle:tPluginBundleName];
						
						if (tLocalizedListTitle==nil)
						{
							tLanguage=[NSString stringWithString:@"English"];
							
							tLocalizedListTitle=[tLocalizer localizedString:tListTitleKey forLanguage:tLanguage inBundle:tPluginBundleName];
						}
					}
					
					return tLocalizedListTitle;
				}
			}
			else
			{
				// A COMPLETER
			}
		}
	}
	
	return nil;
}

- (BOOL) listView:(ICPresentationListView *) inListView stepWillBeVisible:(NSInteger) inStep
{
	if (visibleInstallationSteps_!=nil)
	{
		NSDictionary * tDictionary;
		
NS_DURING

		tDictionary=[visibleInstallationSteps_ objectAtIndex:inStep];
		
NS_HANDLER

		tDictionary=nil;
		
NS_ENDHANDLER
	
		if (tDictionary!=nil)
		{
			NSString * tClassName;
			
			tClassName=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
			
			if (tClassName!=nil)
			{
				NSDictionary * tViewDictionary;
				
				if ([tClassName isEqualToString:@"ICPresentationViewReadMeController"]==YES)
				{
					if (dictionary_!=nil)
					{
						tViewDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_README];
					
						if(tViewDictionary!=nil)
						{
							if ([ICReadMeUtilities readMeDocumentPathFromDictionary:tViewDictionary forLanguage:currentPreviewLanguage_ inDocument:[self document]]!=nil)
							{
								return YES;
							}
						}
					}
				
					return NO;
				}
				else if ([tClassName isEqualToString:@"ICPresentationViewLicenseController"]==YES)
				{
					if (dictionary_!=nil)
					{
						tViewDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_LICENSE];
					
						if (tViewDictionary!=nil)
						{
							if ([ICLicenseUtilities licenseDocumentPathFromDictionary:tViewDictionary forLanguage:currentPreviewLanguage_ inDocument:[self document]]!=nil)
							{
								return YES;
							}
						}
					}
				
					return NO;
				}
				else if ([tClassName isEqualToString:@"ICPresentationViewInstallerPluginController"]==YES)
				{
					NSDictionary * tPathDictionary;
					
					tPathDictionary=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_INSTALLERPLUGIN_PATH];
					
					if (tPathDictionary!=nil)
					{
						NSString * tPath;
						NSInteger tPathType;
						
						tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
							
						tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
						
						if (tPath!=nil && [tPath length]>0)
						{
							if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
							{
								tPath=[tPath stringByAbsolutingWithPath:[[self document] folder]];
							}
							else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
							{
								tPath=[tPath stringByAbsolutingWithPath:[[self document] referenceFolderPath]];
							}
					
							if (tPath!=nil)
							{
								NSFileManager * tFileManager;
								
								tFileManager=[NSFileManager defaultManager];
								
								if (tFileManager!=nil)
								{
									BOOL isDirectory;
									
									if ([tFileManager fileExistsAtPath:tPath isDirectory:&isDirectory]==YES && isDirectory==YES)
									{
										NSBundle * tBundle;
								
										tBundle=[NSBundle bundleWithPath:tPath];
								
										if (tBundle!=nil)
										{
											return YES;
										}
										else
										{
											return NO;
										}
									}
									else
									{
										return NO;
									}
								}
							}
						}
					}
				}
			}
		}
	}

	return YES;
}

- (BOOL) listView:(ICPresentationListView *) inListView shouldSelectStep:(NSInteger) inStep
{
	if (visibleInstallationSteps_!=nil)
	{
		NSDictionary * tDictionary;
		
NS_DURING

		tDictionary=[visibleInstallationSteps_ objectAtIndex:inStep];
		
NS_HANDLER

		tDictionary=nil;
		
NS_ENDHANDLER
	
		if (tDictionary!=nil)
		{
			NSString * tClassName;
			
			tClassName=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
			
			if (tClassName!=nil)
			{
				if ([tClassName isEqualToString:@"ICPresentationViewDestinationSelectController"]==YES ||
					[tClassName isEqualToString:@"ICPresentationViewInstallationController"]==YES)
				{
					return NO;
				}
			}
		}
	}
	
	return YES;
}

#pragma mark -

- (BOOL) listView:(ICPresentationListView *) inListView writeStep:(NSInteger) inStep toPasteboard:(NSPasteboard*) inPasteboard
{
	if (inListView==IBinstallationStepsView_)
	{
		if (installationSteps_!=nil)
		{
			NSDictionary * tDictionary;
			
	NS_DURING

			tDictionary=[installationSteps_ objectAtIndex:inStep];
			
	NS_HANDLER

			tDictionary=nil;
			
	NS_ENDHANDLER
		
			if (tDictionary!=nil)
			{
				NSString * tClassName;
				
				tClassName=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
				
				if (tClassName!=nil)
				{
					if ([tClassName isEqualToString:@"ICPresentationViewInstallerPluginController"]==YES)
					{
						[inPasteboard declareTypes:[NSArray arrayWithObject:ICPROJECTPRESENTATIONCONTROLLER_INTERNALSTEP_PBOARDTYPE] owner:self];
		
						[inPasteboard setData:[NSData data] forType:ICPROJECTPRESENTATIONCONTROLLER_INTERNALSTEP_PBOARDTYPE]; 

						internalDragStep_=inStep;
						
						return YES;
					}
				}
			}
		}
	}
	
	return NO;
}

- (NSDragOperation) listView:(ICPresentationListView*) inListView validateDrop:(id <NSDraggingInfo>)info proposedStep:(NSInteger) inStep
{
	if (inListView==IBinstallationStepsView_ /*&& cachedBuildFormat_==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_BUNDLE*/)
	{
		if (inStep!=[visibleInstallationSteps_ count])
		{
		
			NSPasteboard * tPasteBoard;
			
			tPasteBoard=[info draggingPasteboard];
			
			if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:ICPROJECTPRESENTATIONCONTROLLER_INTERNALSTEP_PBOARDTYPE]]!=nil)
			{
				// We need to check it's an internal drag
				
				if ([info draggingSource]==IBinstallationStepsView_)
				{
					// Check that the step is acceptable
					
					if (internalDragStep_!=inStep && inStep!=(internalDragStep_+1))
					{
						return NSDragOperationMove;
					}
				}
			}
			else if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
			{
				// We need to check that the plugins are not already in the list
				
				NSArray * tArray;
				
				tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
							
				if (tArray!=nil)
				{
					NSEnumerator * tEnumerator;
				
					tEnumerator=[tArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSString * tFilePath;
						NSFileManager * tFileManager;
						
						tFileManager=[NSFileManager defaultManager];
						
						if (tFileManager!=nil)
						{
							while (tFilePath=[tEnumerator nextObject])
							{
								BOOL isDirectory;
						
								if ([tFileManager fileExistsAtPath:tFilePath isDirectory:&isDirectory]==NO || isDirectory==NO)
								{
									return NSDragOperationNone;
								}
								else
								{
									if ([[tFilePath pathExtension] caseInsensitiveCompare:@"bundle"]==NSOrderedSame)
									{
										if ([ICInstallationStepsUtilities installationSteps:installationSteps_ ofDocument:[self document] containsInstallerPluginAtPath:tFilePath]==YES)
										{
											return NSDragOperationNone;
										}
									}
								}
							}
							
							return NSDragOperationCopy;
						}
					}
				}
			}
		}
	}
	
	return NSDragOperationNone;
}

- (BOOL) listView:(ICPresentationListView*) inListView acceptDrop:(id <NSDraggingInfo>)info step:(NSInteger) inStep
{
	if (inListView==IBinstallationStepsView_ /*&& cachedBuildFormat_==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_BUNDLE*/)
	{
		NSPasteboard * tPasteBoard;
			
		tPasteBoard=[info draggingPasteboard];
		
		if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:ICPROJECTPRESENTATIONCONTROLLER_INTERNALSTEP_PBOARDTYPE]]!=nil)
		{
			// Switch Position of Installer Plugin
			
			NSDictionary * tDictionary;
		
NS_DURING

			tDictionary=[installationSteps_ objectAtIndex:internalDragStep_];
		
NS_HANDLER

			tDictionary=nil;
		
NS_ENDHANDLER
	
			if (tDictionary!=nil)
			{
				[tDictionary retain];
				
				[installationSteps_ removeObjectAtIndex:internalDragStep_];
				
				if (internalDragStep_<inStep)
				{
					inStep--;
				}
				
				[installationSteps_ insertObject:tDictionary atIndex:inStep];
				
				[tDictionary release];
				
				// Refresh the list view
				
				[IBinstallationStepsView_ selectStepAtIndex:inStep];
				
				[self listViewSelectionDidChange:nil];
			
				[self noteDocumentChange];
			}
			
			return YES;
		}
		else if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
		{
			// Add Installer Plugins
			
			NSArray * tArray;
		
			tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
		
			if (tArray!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tArray objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSString * tPath;
					int tInsertionIndex;
					NSBundle * tBundle;
					BOOL tModified=NO;
					NSInteger tPathType;
					
					tInsertionIndex=inStep;
					
					tPathType=[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
					
					while (tPath=[tEnumerator nextObject])
					{
						tBundle=[NSBundle bundleWithPath:tPath];
			
						// Create the dictionary
						
						if (tBundle!=nil)
						{
							NSMutableDictionary * tInstallerPluginDictionary;
							NSDictionary * tPathDictionary;
							
							if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
							{
								tPath=[tPath stringByRelativizingToPath:[[self document] folder]];
							}
							else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
							{
								tPath=[tPath stringByAbsolutingWithPath:[[self document] referenceFolderPath]];
							}
					
							tPathDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tPath,ICDOCUMENT_PATH,
																							  [NSNumber numberWithInteger:tPathType],ICDOCUMENT_PATH_TYPE,
																							  nil];
							
							tInstallerPluginDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:@"ICPresentationViewInstallerPluginController",ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS,
																										 tPathDictionary,ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_INSTALLERPLUGIN_PATH,
																										 nil];
							
						
							if (tInstallerPluginDictionary!=nil)
							{
								[installationSteps_ insertObject:tInstallerPluginDictionary atIndex:tInsertionIndex];
								
								// Refresh the list view
								
								[IBinstallationStepsView_ setNeedsDisplay:YES];
								
								tInsertionIndex++;
								
								tModified=YES;
							}
						}
						else
						{
							// A COMPLETER
						}
					}
					
					if (tModified==YES)
					{
						// Refresh the list view
						
						if ([tArray count]==1)
						{
							[IBinstallationStepsView_ selectStepAtIndex:inStep];
				
							[self listViewSelectionDidChange:nil];
						}
						else
						{
							[[IBinstallationStepsView_ window] invalidateCursorRectsForView:IBinstallationStepsView_];
							
							[IBinstallationStepsView_ setNeedsDisplay:YES];
						}
						
						[self noteDocumentChange];
						
						return YES;
					}
				}
			}
		}
	}
	
	return NO;
}


#pragma mark -


- (void) listViewSelectionDidChange:(NSNotification *)notification
{
	if (visibleInstallationSteps_!=nil)
	{
		NSDictionary * tDictionary;
		
NS_DURING

		tDictionary=[visibleInstallationSteps_ objectAtIndex:[IBinstallationStepsView_ selectedStep]];
		
NS_HANDLER

		tDictionary=nil;
		
NS_ENDHANDLER
	
		if (tDictionary!=nil)
		{
			NSString * tClassName;
			
			tClassName=[tDictionary objectForKey:ICPRESENTATION_CHAPTER_VIEW_CONTROLLER_CLASS];
			
			if (tClassName!=nil)
			{
				int tTag;
				
				if ([[currentViewController_ className] isEqualToString:tClassName]==NO || [tClassName isEqualToString:@"ICPresentationViewInstallerPluginController"]==YES)
				{
					tTag=[self chapterTagForClassName:tClassName];
					
					[IBinspectorPopupButton_ selectItemWithTag:tTag];
					
					[self showViewWithTag:tTag];
					
					// Save the selected step
				
					[[self document] setDocumentDefault:[NSNumber numberWithInteger:[IBinstallationStepsView_ selectedStep]] forKey:@"ui.project.presentation.step.selected"];
					
					[[IBinstallationStepsView_ window] invalidateCursorRectsForView:IBinstallationStepsView_];
					
					if ([tClassName isEqualToString:@"ICPresentationViewInstallerPluginController"]==YES)
					{
						[[IBinspectorPopupButton_ selectedItem] setEnabled:NO];
						
						[IBpluginsRemoveButton_ setEnabled:YES];
					}
					else
					{
						[IBpluginsRemoveButton_ setEnabled:NO];
					}
				}
			}
		}
	}
}

- (void) windowViewDidResize:(NSNotification *) inNotification
{
	[self updateTitleViews];
}

- (void) inspectorViewDidResize:(NSNotification *) inNotification
{
	NSView * tView;
	NSRect tFrame,tOldFrame;
	NSUInteger tAutoresizemask;
	NSRect tBounds;
	
	tBounds=[IBinspectorView_ bounds];
	
	tView=[currentInspectorController_ view];
	
	tAutoresizemask=[tView autoresizingMask];
	
	tFrame=tOldFrame=[tView frame];
	
	if ((tAutoresizemask & NSViewWidthSizable)==0)
	{
		// Center horizontally
		
		tFrame.origin.x=_CGFloatRound(NSMidX(tBounds)-NSWidth(tFrame)*0.5f);
	}
	
	if ((tAutoresizemask & NSViewHeightSizable)==0)
	{
		// Center horizontally
		
		tFrame.origin.y=_CGFloatRound(NSMidY(tBounds)-NSHeight(tFrame)*0.5f);
	}
	
	if (NSEqualRects(tFrame,tOldFrame)==NO)
	{
		[tView setFrame:tFrame];
	}
}

- (void) imageDidChange:(NSString *) inPath
{
	if (inPath!=nil)
	{
		NSMutableDictionary * tBackgroundDictionary;
	
		tBackgroundDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND];
		
		if (tBackgroundDictionary==nil)
		{
			// Create default Background dictionary
			
			tBackgroundDictionary=[ICBackgroundUtilities defaultBackgroundDictionary];
			
			if (tBackgroundDictionary!=nil)
			{
				[dictionary_ setObject:tBackgroundDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND];
			}
		}
	
		if (tBackgroundDictionary!=nil)
		{		
			NSMutableDictionary * tPathDictionary;
			NSNumber * tNumber;
			BOOL tImageDidReallyChanged=NO;
			
			tNumber=[tBackgroundDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_CUSTOM];
			
			if (tNumber==nil || [tNumber boolValue]==NO)
			{
				// We were not using a custom background picture
				
				tImageDidReallyChanged=YES;
				
				[tBackgroundDictionary setObject:[NSNumber numberWithBool:YES] forKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_CUSTOM];
			}
			
			tPathDictionary=[tBackgroundDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_BACKGROUND_PATH];
				
			if (tPathDictionary!=nil)
			{
				NSString * tPath;
				NSInteger tPathType;
				
				tPathType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
					
				tPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
				
				if (tPath!=nil && [tPath length]>0)
				{
					if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						tPath=[tPath stringByAbsolutingWithPath:[[self document] folder]];
					}
					else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						tPath=[tPath stringByAbsolutingWithPath:[[self document] referenceFolderPath]];
					}
					
					if ([tPath caseInsensitiveCompare:inPath]!=NSOrderedSame)
					{
						// This is not the same bakground picture
						
						if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
						{
							inPath=[inPath stringByRelativizingToPath:[[self document] folder]];
						}
						else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
						{
							inPath=[inPath stringByAbsolutingWithPath:[[self document] referenceFolderPath]];
						}
					
						[tPathDictionary setObject:inPath forKey:ICDOCUMENT_PATH];
						
						tImageDidReallyChanged=YES;
					}
				}
				else
				{
					// No custom background picture had been set
					
					if ([tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE]==nil)
					{
						tPathType=[[NSUserDefaults standardUserDefaults] integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
					}
					
					if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
					{
						inPath=[inPath stringByRelativizingToPath:[[self document] folder]];
					}
					else if (tPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
					{
						inPath=[inPath stringByAbsolutingWithPath:[[self document] referenceFolderPath]];
					}
						
					[tPathDictionary setObject:inPath forKey:ICDOCUMENT_PATH];
					
					[tPathDictionary setObject:[NSNumber numberWithInteger:tPathType] forKey:ICDOCUMENT_PATH_TYPE];
					
					tImageDidReallyChanged=YES;
				}
			}
			
			if (tImageDidReallyChanged==YES)
			{
				[IBbackgroundView_ setDrawGrowBox:[ICProjectPresentationController shouldDrawGroxBoxForImage:[IBbackgroundView_ image] andSettings:tBackgroundDictionary]];
				
				[ICBackgroundUtilities notifyBackgroundDidChangeWithSender:self];
				
				[self noteDocumentChange];
			}
		}
	}
}

- (void) titleSettingsDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if ([tUserInfo objectForKey:ICPRESENTATION_NOTIFICATION_SENDER]!=self)
		{
			[self updateTitleViews];
		}
	}
}

- (void) backgroundSettingsDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if ([tUserInfo objectForKey:ICPRESENTATION_NOTIFICATION_SENDER]!=self)
		{
			[self updateBackgroundView];
		}
	}
}

- (void) readMeSettingsDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		[IBinstallationStepsView_ setNeedsDisplay:YES];
	}
}

- (void) licenseSettingsDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		[IBinstallationStepsView_ setNeedsDisplay:YES];
	}
}

- (void) licenseDisplayedLanguageDidChange:(NSNotification *) inNotification
{
	NSString * tPaneTitle;
		
	// Refresh Chapter Title View
	
	tPaneTitle=[currentViewController_ paneTitleForLanguage:currentPreviewLanguage_];
	
	if (tPaneTitle!=nil)
	{
		[IBchapterTitleView_ setStringValue:tPaneTitle];
	}
}

- (void) enlargeChoiceDependencyInspector:(NSNotification *) inNotification
{
	//NSRect tViewBounds;
	NSRect tLeftViewFrame;
	NSRect tRightViewFrame;
	NSRect tFrame;
	
	//NSLog(@"%@",self);
	
	//tViewBounds=[IBview_ bounds];
	
	tLeftViewFrame=[IBleftView_ frame];
	
	tRightViewFrame=[IBrightView_ frame];
	
	savedRightViewWidth_=NSWidth(tRightViewFrame);
	
	// Hide the step selection widgets
	
	tFrame=[IBinspectorView_ frame];
	
	savedInspectorViewSpace_=NSHeight(tRightViewFrame)-NSHeight(tFrame);
	
	[IBinspectorPopupButton_ setHidden:YES];
	
	[IBseparator_ setHidden:YES];
	
	tFrame.size=tRightViewFrame.size;
	
	[IBinspectorView_ setFrame:tFrame];
	
	// Resize the Left and Right View
	
	tLeftViewFrame.size.width-=ICPRESENTATIONINSPECTOR_ENLARGEMENT;
	
	[IBleftView_ setFrame:tLeftViewFrame];
	
	tRightViewFrame.origin.x-=ICPRESENTATIONINSPECTOR_ENLARGEMENT;
	
	tRightViewFrame.size.width+=ICPRESENTATIONINSPECTOR_ENLARGEMENT;
	
	[IBrightView_ setFrame:tRightViewFrame];
	
	[IBview_ setNeedsDisplay:YES];
	
	// Show the Surgery field
	
	surgeryWindow_=[[ICInstallationSurgeryWindow alloc] initForView:IBleftView_];
	
	if (surgeryWindow_!=nil)
	{
		id tOutlineScrollView;
		
		tOutlineScrollView=[currentViewController_ outlineScrollView];
		
		tFrame=[tOutlineScrollView frame];
		
		tFrame=[IBleftView_ convertRect:tFrame fromView:[tOutlineScrollView superview]];
	
		[[IBleftView_ window] addChildWindow:surgeryWindow_ ordered:NSWindowAbove];
			
		[surgeryWindow_ setSurgeryViewFrame:tFrame];
		
		[surgeryWindow_ orderFront:self];
	}
}

- (void) reduceChoiceDependencyInspector:(NSNotification *) inNotification
{
	//NSRect tViewBounds;
	NSRect tLeftViewFrame;
	NSRect tRightViewFrame;
	NSRect tFrame;
	
	//tViewBounds=[IBview_ bounds];
	
	tLeftViewFrame=[IBleftView_ frame];
	
	tRightViewFrame=[IBrightView_ frame];
	
	// Remove Surgery field
	
	if (surgeryWindow_!=nil)
	{
		[[IBleftView_ window] removeChildWindow:surgeryWindow_];
		
		[surgeryWindow_ orderOut:self];
		
		[surgeryWindow_ release];
		
		surgeryWindow_=nil;
	}
	
	// Resize Left and Right View
	
	tLeftViewFrame.size.width+=ICPRESENTATIONINSPECTOR_ENLARGEMENT;
	
	[IBleftView_ setFrame:tLeftViewFrame];
	
	tRightViewFrame.origin.x+=ICPRESENTATIONINSPECTOR_ENLARGEMENT;
	
	tRightViewFrame.size.width-=ICPRESENTATIONINSPECTOR_ENLARGEMENT;
	
	[IBrightView_ setFrame:tRightViewFrame];
	
	// Show the step selection widgets
	
	[IBinspectorPopupButton_ setHidden:NO];
	
	[IBseparator_ setHidden:NO];
	
	tFrame=[IBinspectorView_ frame];
	
	tFrame.size.height=NSHeight(tRightViewFrame)-savedInspectorViewSpace_;
	
	[IBinspectorView_  setFrame:tFrame];
	
	[IBview_ setNeedsDisplay:YES];
}

- (void) leftViewDidResize:(NSNotification *) inNotification
{
	if (surgeryWindow_!=nil)
	{
		NSRect tFrame;
		NSRect tWindowFrame;
		
		tFrame=[IBleftView_ frame];
		
		tWindowFrame=[surgeryWindow_ frame];
		
		tWindowFrame.origin.y+=NSHeight(tWindowFrame)-NSHeight(tFrame);
		
		tWindowFrame.size=tFrame.size;
		
		[surgeryWindow_ setFrame:tWindowFrame display:YES];
	}
}

- (void) windowStateDidChange:(NSNotification *) inNotification
{
	if ([[currentViewController_ className] isEqualToString:@"ICPresentationViewInstallerPluginController"]==YES)
	{
		NSString * tPaneTitle;
	
		// Refresh Chapter Title View
		
		tPaneTitle=[currentViewController_ paneTitleForLanguage:currentPreviewLanguage_];
		
		if (tPaneTitle!=nil)
		{
			[IBchapterTitleView_ setStringValue:tPaneTitle];
		}
		else
		{
			[IBchapterTitleView_ setStringValue:@""];
		}
	}
	
	// Refresh Background
	
	[self updateBackgroundView];
}

@end
