/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICProjectRequirementMessageTableView.h"

#import "ICDocumentFormat+Constants.h"

@implementation ICProjectRequirementMessageTableView

- (NSArray *) _alternatingRowBackgroundColors
{
	NSArray * tColors;
	
	tColors=[super _alternatingRowBackgroundColors];
	
	if ([tColors count]==2)
	{
		return [NSArray arrayWithObjects:[tColors objectAtIndex:0],
										 [tColors objectAtIndex:0],
										 [tColors objectAtIndex:1],
										 [tColors objectAtIndex:1],
										 nil];
	}
	
	return tColors;
}

- (void) drawGridInClipRect:(NSRect)clipRect
{
	CGFloat rowHeight = [self rowHeight] + [self intercellSpacing].height;
    NSRect tBounds=[self bounds];
    NSRect highlightRect;
    CGFloat tColumWidth;
	NSBezierPath * tBezierPath;
	const CGFloat tPattern[2]={2.0f,5.0f};
	
	highlightRect.origin = NSMakePoint(NSMinX(tBounds), (int)(NSMinY(tBounds)/rowHeight)*rowHeight);
    highlightRect.size = NSMakeSize(NSWidth(tBounds), rowHeight + [self intercellSpacing].height);
    
	tColumWidth=[[self tableColumnWithIdentifier:ICDOCUMENT_LANGUAGE] width] + [self intercellSpacing].width;
	
	[[self gridColor] set];
	
	tBezierPath=[NSBezierPath bezierPath];
	
    while (NSMinY(highlightRect) < NSMaxY(tBounds))
    {
        NSRect clippedHighlightRect = NSIntersectionRect(highlightRect, tBounds);
        int tRowIndex;
		
		tRowIndex = (int)((NSMinY(highlightRect)+rowHeight*0.5f)/rowHeight);
			
		if ((tRowIndex%2)==0)
		{
			[tBezierPath setLineDash:tPattern count:0 phase:0];
			
			[tBezierPath moveToPoint:NSMakePoint(NSMinX(clippedHighlightRect),NSMinY(clippedHighlightRect)-0.5f)];
			
			[tBezierPath lineToPoint:NSMakePoint(NSMaxX(clippedHighlightRect),NSMinY(clippedHighlightRect)-0.5f)];
		}
		else
		{
			[tBezierPath setLineDash:tPattern count:2 phase:0];
			
			[tBezierPath moveToPoint:NSMakePoint(NSMinX(clippedHighlightRect)+tColumWidth,NSMinY(clippedHighlightRect)-0.5f)];
			
			[tBezierPath lineToPoint:NSMakePoint(NSMaxX(clippedHighlightRect),NSMinY(clippedHighlightRect)-0.5f)];
		}
		
		[tBezierPath stroke];
		
		[tBezierPath removeAllPoints];
		
        highlightRect.origin.y += rowHeight;
    }
	
	[tBezierPath setLineDash:tPattern count:0 phase:0];
			
	[tBezierPath moveToPoint:NSMakePoint(NSMinX(tBounds)+tColumWidth-0.5f,NSMinY(tBounds))];
	
	[tBezierPath lineToPoint:NSMakePoint(NSMinX(tBounds)+tColumWidth-0.5f,NSMaxY(tBounds))];
	
	[tBezierPath stroke];
}

@end
