/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectRequirementsController.h"
#import "ICProjectRequirementsUtility.h"

#import "ICProjectResourcesInspectorSelectionController.h"

#import "ICRequirementProjectEditorController.h"

#import "ICRequirementPluginsManager.h"
#import "ICRequirementPlugin_Converter.h"

NSString * const ICPROJECTREQUIREMENTS_HIERARCHY_DISCLOSED_STATE=@"ui.project.requirements.resources.disclosed";

NSString * const ICPROJECTREQUIREMENTS_SELECTION=@"ui.project.requirements.resources.selection";

NSString * const ICPROJECTREQUIREMENTS_IS_HEADER=@"ICPROJECTREQUIREMENTS_IS_HEADER";

@implementation ICProjectRequirementsController

+ (ICProjectRequirementsController *) projectRequirementsController
{
	ICProjectRequirementsController * nController=nil;
    
    nController=[ICProjectRequirementsController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Project_Requirements" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Project_Requirements"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) dealloc
{
	[cachedRequirements_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn = nil;
	
	[super awakeFromNib];
	
	[IBrequirementsArray_ setDoubleAction:@selector(editRequirement:)];
	
	[IBrequirementsArray_ setTarget:self];
	
	tTableColumn = [IBrequirementsArray_ tableColumnWithIdentifier:@"State"];
    
	if (tTableColumn!=nil)
	{
		NSButtonCell * tButttonCell;
		
		tButttonCell = [[NSButtonCell alloc] initTextCell: @""];
    
		if (tButttonCell!=nil)
		{
			[tButttonCell setControlSize:NSMiniControlSize];
			[tButttonCell setEditable: YES];
			[tButttonCell setButtonType: NSSwitchButton];
			[tButttonCell setImagePosition: NSImageOnly];
		
			[tTableColumn setDataCell:tButttonCell];
		
			[tButttonCell release];
		}
	}
	
	// Name
	
	tTableColumn = [IBrequirementsArray_ tableColumnWithIdentifier:@"Name"];
    
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
	
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
	}
	
	[IBoutlineView_ registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType,
                                                                      ICFileHierarchyPboardType,
																	  ICFileHierarchyExternalPboardType,
                                                                      nil]];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[cachedRequirements_ release];
	
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	cachedRessourcesSectionInitialHeight_=NSHeight([IBresourcesSection_ frame]);
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_REQUIREMENTS];
	
	if (dictionary_==nil)
	{
		// Build the default dictionary
		
		dictionary_=[ICProjectRequirementsUtility defaultProjectRequirementsDictionary];
		
		if (dictionary_!=nil)
		{
			[inDictionary setObject:dictionary_ forKey:ICDOCUMENT_PROJECT_REQUIREMENTS];
		}
	}
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
		
		[document_ updateFileBouncer];
		
		cachedFileBouncer_=[[document_ fileBouncer] retain];
		
		[dictionary_ retain];
		
		// Requirements
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_REQUIREMENTS_ROOT_VOLUME_ONLY];
		
		if (tNumber!=nil && [tNumber boolValue]==YES)
		{
			[IBoptionsRootVolumeOnly_ setState:NSOnState];
		}
		else
		{
			[IBoptionsRootVolumeOnly_ setState:NSOffState];
		}
		
		cachedRequirements_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_REQUIREMENTS_LIST];
		
		if (cachedRequirements_==nil)
		{
			cachedRequirements_=[NSMutableArray array];
			
			if (cachedRequirements_!=nil)
			{
				[dictionary_ setObject:cachedRequirements_ forKey:ICDOCUMENT_PROJECT_REQUIREMENTS_LIST];
			}
		}
		
		if (cachedRequirements_!=nil)
		{
			[cachedRequirements_ retain];
			
			[self updateCachedRequirements];
		}
		
		[self viewFrameDidChange:nil];
		
		// Resources
		
		[self performSelector:@selector(delayedHierarchyLoad:) withObject:nil afterDelay:0.1f];
		
		// Register for Notifications
		
		[[NSNotificationCenter defaultCenter] removeObserver:self name:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_FOLDERNAME_SHOULD_CHANGE object:document_];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(folderNameShouldChange:)
                                                 name:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_FOLDERNAME_SHOULD_CHANGE
                                               object:document_];
	
		[[NSNotificationCenter defaultCenter] removeObserver:self name:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_ATTRIBUTES_DID_CHANGE object:document_];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(fileAttributesDidChange:)
													 name:ICFILEHIERARCHYINSPECTOR_NOTIFICATION_ATTRIBUTES_DID_CHANGE
												   object:document_];
												   
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(viewFrameDidChange:) 
													 name:NSViewFrameDidChangeNotification 
												  object:IBview_];
		
	}

	
}

- (void) delayedHierarchyLoad:(id) sender
{
	// File Hierarchy
	
	fileTree_=[ICFileTree fileTreeWithArray:[dictionary_ objectForKey:ICDOCUMENT_PROJECT_REQUIREMENTS_FILE_HIERARCHY] projectPath:[document_ folder] referenceFolderPath:[document_ referenceFolderPath]];
	
	if (fileTree_!=nil)
	{
		[fileTree_ retain];
		
		[IBoutlineView_ reloadData];
		
		[self restoreDisclosedStates];
	}
	else
	{
	
		// A COMPLETER
	}
	
	// Restore Selection
	
	[self restoreSelection];
	
	[self outlineViewSelectionDidChange:nil];
}

- (IBAction) updateFiles:(id) sender
{
	// Save File Hierarchy if needed
	
	if (hierarchyChanged_==YES)
	{
		NSArray * tArray;
		
		tArray=[fileTree_ arrayWithProjectAtPath:[document_ folder] referenceFolderPath:[document_ referenceFolderPath]];
			
		if (tArray!=nil)
		{
			[dictionary_ setObject:tArray forKey:ICDOCUMENT_PROJECT_REQUIREMENTS_FILE_HIERARCHY];
		}
		
		hierarchyChanged_=NO;
	}
	
	// Notify Document Change
			
	[self noteDocumentChange];
}

- (NSString *) selectionKey
{
	return ICPROJECTREQUIREMENTS_SELECTION;
}

- (NSString *) disclosedStateKey
{
	return ICPROJECTREQUIREMENTS_HIERARCHY_DISCLOSED_STATE;
}

#pragma mark -

- (void) viewWillBeRemoved:(id) sender
{
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSViewFrameDidChangeNotification object:IBview_];
}

- (void) documentWillBeSaved:(NSNotification *) inNotification
{
}

#pragma mark -

- (IBAction) switchRootVolumeOnly:(id) sender
{
	if (dictionary_!=nil)
	{
		BOOL tValue;
	
		tValue=([sender state]==NSOnState);

		[dictionary_ setObject:[NSNumber numberWithBool:tValue] forKey:ICDOCUMENT_PROJECT_REQUIREMENTS_ROOT_VOLUME_ONLY];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

#pragma mark -

- (BOOL) tableView:(NSTableView *) inTableView isHeaderAtRow:(NSInteger) inRowIndex
{
	if (inTableView==IBrequirementsArray_)
	{
		if (inRowIndex>=0)
		{
			if (cachedFormattedRequirements_!=nil)
			{
				NSDictionary * tDictionary;
				NSNumber * tNumber;
				
				tDictionary=[cachedFormattedRequirements_ objectAtIndex:inRowIndex];
				
				tNumber=[tDictionary objectForKey:ICPROJECTREQUIREMENTS_IS_HEADER];
				
				return [tNumber boolValue];
			}
		}
	}
	
	return NO;
}

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (inTableView==IBrequirementsArray_)
	{
		if (cachedFormattedRequirements_!=nil)
		{
			return [cachedFormattedRequirements_ count];
		}
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (inTableView==IBrequirementsArray_)
	{
		if (cachedFormattedRequirements_!=nil)
		{
			NSDictionary * tDictionary;
			NSString * tColumnIdentifier;
			NSNumber * tNumber;
			
			tDictionary=[cachedFormattedRequirements_ objectAtIndex:inRowIndex];
			
			tNumber=[tDictionary objectForKey:ICPROJECTREQUIREMENTS_IS_HEADER];
			
			tColumnIdentifier=[inTableColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:@"State"])
			{
				return [tDictionary objectForKey:ICDOCUMENT_REQUIREMENT_STATE];
			}
			else if ([tColumnIdentifier isEqualToString:@"Name"])
			{
				if ([tNumber boolValue]==YES)
				{
					// Use Gray text
				
					NSAttributedString * tAttributedString;
				
					tAttributedString=[[NSAttributedString alloc] initWithString:[tDictionary objectForKey:ICDOCUMENT_REQUIREMENT_NAME]
																	  attributes:[NSDictionary dictionaryWithObjectsAndKeys:[NSColor whiteColor],NSForegroundColorAttributeName,
																															[NSFont boldSystemFontOfSize:[NSFont smallSystemFontSize]],NSFontAttributeName,
																															nil]];
						
				
					return [tAttributedString autorelease];
				}
				else
				{
					return [tDictionary objectForKey:ICDOCUMENT_REQUIREMENT_NAME];
				}
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	
	if (inTableView==IBrequirementsArray_)
	{
		if (cachedFormattedRequirements_!=nil)
		{
			NSMutableDictionary * tDictionary;
			
			tColumnIdentifier=[inTableColumn identifier];
		
			tDictionary=[cachedFormattedRequirements_ objectAtIndex:inRowIndex];
				
			if ([tColumnIdentifier isEqualToString:@"State"])
			{
				NSNumber * tNumber;
				
				tNumber=[tDictionary objectForKey:ICDOCUMENT_REQUIREMENT_STATE];
				
				if ([tNumber boolValue]!=[object boolValue])
				{
					[tDictionary setObject:object forKey:ICDOCUMENT_REQUIREMENT_STATE];
					
					[self noteDocumentChange];
				}
			}
			else if ([tColumnIdentifier isEqualToString:@"Name"])
			{
				NSString * tString;
							
				tString=[tDictionary objectForKey:ICDOCUMENT_REQUIREMENT_NAME];
							
				if ([tString isEqualToString:object]==NO)
				{
					NSEnumerator * tEnumerator;
			
					tEnumerator=[cachedRequirements_ objectEnumerator];
			
					if (tEnumerator!=nil)
					{
						NSDictionary * tOtherDictionary;
						
						while (tOtherDictionary=[tEnumerator nextObject])
						{
							if (tOtherDictionary!=tDictionary)
							{
								NSString * tOtherName;
							
								tOtherName=[tOtherDictionary objectForKey:ICDOCUMENT_REQUIREMENT_NAME];
							
								if (tOtherName!=nil)
								{
									if ([tOtherName isEqualToString:object]==YES)
									{
										NSBeep();
										
										[IBrequirementsArray_ editColumn:[IBrequirementsArray_ columnWithIdentifier:@"Name"] row:inRowIndex withEvent:nil select:YES];
										
										return;
									}
								}
							}
						}
						
						[tDictionary setObject:object forKey:ICDOCUMENT_REQUIREMENT_NAME];
					
						[self noteDocumentChange];
					}
				}
			}
		}
	}
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRow
{
	if (inTableView==IBrequirementsArray_)
	{
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		if ([tColumnIdentifier isEqualToString:@"State"]==YES)
		{
			NSDictionary * tDictionary;
			
			tDictionary=[cachedFormattedRequirements_ objectAtIndex:inRow];

			if ([[tDictionary objectForKey:ICPROJECTREQUIREMENTS_IS_HEADER] boolValue]==YES)
			{
				[(NSButtonCell *)cell setImagePosition: NSNoImage];
			}
			else
			{
				[(NSButtonCell *)cell setImagePosition: NSImageOnly];
			}
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView shouldEditTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRow
{
	if (inTableView==IBrequirementsArray_)
	{
		NSDictionary * tDictionary;
		
		tDictionary=[cachedFormattedRequirements_ objectAtIndex:inRow];

		if ([[tDictionary objectForKey:ICPROJECTREQUIREMENTS_IS_HEADER] boolValue]==YES)
		{
			return NO;
		}
	}
	
	return YES;
}

- (BOOL) tableView:(NSTableView *) inTableView shouldSelectRow:(NSInteger) inRow
{
	if (inTableView==IBrequirementsArray_)
	{
		NSDictionary * tDictionary;
		
		tDictionary=[cachedFormattedRequirements_ objectAtIndex:inRow];

		if ([[tDictionary objectForKey:ICPROJECTREQUIREMENTS_IS_HEADER] boolValue]==YES)
		{
			return NO;
		}
	}
	
	return YES;
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBrequirementsArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeRequirements:IBrequirementsRemoveButton_];
		}
		else
		{
			[self removeRequirementsDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBrequirementsArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBrequirementsRemoveButton_ isEnabled];
		}
	}
	
	return NO;
}

#pragma mark -

- (IBAction) addRequirement:(id) sender
{
	NSMutableDictionary * tMutableDictionary;
	
	// A COMPLETER	(identifier)
	
	tMutableDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],ICDOCUMENT_REQUIREMENT_STATE,
																		 @"fr.whitebox.Packages.requirement.cpu",ICDOCUMENT_REQUIREMENT_IDENTIFIER,
																		 [NSNumber numberWithInt:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR_FAIL],ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR,
																		 [NSMutableArray array],ICDOCUMENT_REQUIREMENT_MESSAGE,
																		 nil];

	if (tMutableDictionary!=nil)
	{
		if (requirementController_==nil)
		{
			requirementController_=[ICRequirementProjectEditorController newController];
		}
	
		if (requirementController_!=nil)
		{
			[requirementController_ showEditorForWindow:[document_ windowForSheet] withDictionary:tMutableDictionary delegate:self];
		}
	}
}

- (IBAction) editRequirement:(id) sender
{
	NSInteger tSelectedRow;
	
	tSelectedRow=[IBrequirementsArray_ selectedRow];
	
	if (tSelectedRow!=NSNotFound)
	{
		if (requirementController_==nil)
		{
			requirementController_=[ICRequirementProjectEditorController newController];
		}
	
		if (requirementController_!=nil)
		{
			NSMutableDictionary * tRequirementDictionary;
	
			tRequirementDictionary=[cachedFormattedRequirements_ objectAtIndex:tSelectedRow];
			
			editRequirementMode_=YES;
						
			[requirementController_ showEditorForWindow:[document_ windowForSheet] withDictionary:tRequirementDictionary delegate:self];
		}
	}
}

- (IBAction) removeRequirements:(id) sender
{
	NSString * tAlertTitle;
    
    if ([IBrequirementsArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this requirement?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these requirements?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeRequirementsDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeRequirementsDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBrequirementsArray_ selectedRowIndexes];
		
		[IBrequirementsArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			NSArray * tSelectedObjects;
			
			tSelectedObjects=[cachedFormattedRequirements_ objectsAtIndexes:tIndexSet];
			
			[cachedRequirements_ removeObjectsInArray:tSelectedObjects];
			
			[self updateCachedRequirements];
			
			[IBrequirementsRemoveButton_ setEnabled:NO];
			
			[self viewFrameDidChange:nil];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
	}
}

- (NSString *) uniqueRequirementNameWithBaseName:(NSString *) inBaseName
{
	NSString * tString;
    
	tString=nil;
	
	if (inBaseName!=nil)
	{
		NSUInteger _sIndex=0;
		NSUInteger i,tCount;
    
		tCount=[cachedRequirements_ count];
    
		do
		{
			NSDictionary * tRequirementDictionary;
			
			if (_sIndex>0)
			{
				tString=[[NSString alloc] initWithFormat:@"%@ %lu",inBaseName,_sIndex];
			}
			else
			{
				tString=[[NSString alloc] initWithString:inBaseName];
			}
			
			_sIndex++;
			
			for(i=0;i<tCount;i++)
			{
				tRequirementDictionary=[cachedRequirements_ objectAtIndex:i];
				
				if ([[tRequirementDictionary objectForKey:ICDOCUMENT_REQUIREMENT_NAME] caseInsensitiveCompare:tString]==NSOrderedSame)
				{
					break;
				}
			}
			
			if (i==tCount)
			{
				break;
			}
			
			[tString release];
		}
		while (_sIndex<65535);
    }
	
    return [tString autorelease];
}

- (void) requirementEditionDidEnd:(ICRequirementEditorController *) inEditor returnCode:(NSInteger) inReturnCode
{
	if (inReturnCode==NSOKButton)
	{
		NSMutableDictionary * tRequirementDictionary;
		
		tRequirementDictionary=[inEditor dictionary];
		
		if (tRequirementDictionary!=nil)
		{
			NSInteger tRowIndex;
			NSString * tSuggestedName;
			
			tSuggestedName=[tRequirementDictionary objectForKey:@"SuggestedName"];
			
			if (tSuggestedName!=nil)
			{
				NSString * tUniqueName;
				
				tUniqueName=[self uniqueRequirementNameWithBaseName:tSuggestedName];
				
				[tRequirementDictionary removeObjectForKey:@"SuggestedName"];
				
				[tRequirementDictionary setObject:tUniqueName forKey:ICDOCUMENT_REQUIREMENT_NAME];
			}
			
			if (editRequirementMode_==YES)
			{
				tRowIndex=[IBrequirementsArray_ selectedRow];
				
				if (tRowIndex!=NSNotFound)
				{
					id tObject;
					
					tObject=[cachedFormattedRequirements_ objectAtIndex:tRowIndex];
					
					tRowIndex=[cachedRequirements_ indexOfObject:tObject];
					
					[cachedRequirements_ replaceObjectAtIndex:tRowIndex withObject:tRequirementDictionary];
				
					[self updateCachedRequirements];
					
					tRowIndex=[cachedFormattedRequirements_ indexOfObject:tRequirementDictionary];
				
					[IBrequirementsArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				}
			}
			else
			{
				[cachedRequirements_ addObject:tRequirementDictionary];
				
				[self updateCachedRequirements];
				
				tRowIndex=[cachedFormattedRequirements_ indexOfObject:tRequirementDictionary];
				
				[IBrequirementsArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
				[IBrequirementsArray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
				
				[self viewFrameDidChange:nil];
			}
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
	}
	
	editRequirementMode_=NO;
}

- (void) updateCachedRequirements
{
	NSUInteger tRequirementsCount;
	NSMutableArray * tCachedTargetRequirements,* tCachedInstallationRequirements;
	
	[cachedFormattedRequirements_ release];
	
	cachedFormattedRequirements_=nil;
	
	tRequirementsCount=[cachedRequirements_ count];
	
	if (tRequirementsCount>0)
	{
		cachedFormattedRequirements_=[[NSMutableArray alloc] initWithCapacity:tRequirementsCount];
		
		tCachedTargetRequirements=[[NSMutableArray alloc] initWithCapacity:tRequirementsCount];
	
		tCachedInstallationRequirements=[[NSMutableArray alloc] initWithCapacity:tRequirementsCount];
	
		if (tCachedTargetRequirements!=nil && tCachedInstallationRequirements!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[cachedRequirements_ objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSDictionary * tRequirementDictionary;
				
				while (tRequirementDictionary=[tEnumerator nextObject])
				{
					NSString * tIdentifier;
										
					tIdentifier=[tRequirementDictionary objectForKey:ICDOCUMENT_REQUIREMENT_IDENTIFIER];
					
					if (tIdentifier!=nil)
					{
						id tRequirementConverter;
						
						// Find the appropriate Requirement Converter plugin
						
						tRequirementConverter=[[ICRequirementPluginsManager defaultManager] createPluginConverterControllerForIdentifier:tIdentifier];
						
						if (tRequirementConverter!=nil)
						{
							NSDictionary * tRequirementSettingsDictionary;
							
							tRequirementSettingsDictionary=[tRequirementDictionary objectForKey:ICDOCUMENT_REQUIREMENT_DICTIONARY];
							
							if (tRequirementSettingsDictionary!=nil)
							{
								int tRequirementType;
								NSNumber * tNumber;
								
								// Find if it's an installation or volumeCheck requirements
										
								tRequirementType=[tRequirementConverter requirementTypeWithParameters:tRequirementSettingsDictionary];
										
								if (tRequirementType==IC_REQUIREMENT_CHECK_TYPE_UNDEFINED)
								{
									tNumber=[tRequirementDictionary objectForKey:IC_REQUIREMENT_CHECK_TYPE];
									
									if (tNumber!=nil)
									{
										tRequirementType=[tNumber intValue];
									}
								}
								
								if (tRequirementType==IC_REQUIREMENT_CHECK_TYPE_INSTALLATION)
								{
									[tCachedInstallationRequirements addObject:tRequirementDictionary];
								}
								else if (tRequirementType==IC_REQUIREMENT_CHECK_TYPE_TARGET)
								{
									[tCachedTargetRequirements addObject:tRequirementDictionary];
								}
								else
								{
									// A COMPLETER
								}
							}
						}
					}
				}
				
				if ([tCachedInstallationRequirements count]>0)
				{
					[cachedFormattedRequirements_ addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Installation Requirements",@""),ICDOCUMENT_REQUIREMENT_NAME,
																									   [NSNumber numberWithBool:YES],ICPROJECTREQUIREMENTS_IS_HEADER,
																									   nil]];
																									   
					[cachedFormattedRequirements_ addObjectsFromArray:tCachedInstallationRequirements];
				}
				
				if ([tCachedTargetRequirements count]>0)
				{
					[cachedFormattedRequirements_ addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"Volume Requirements",@""),ICDOCUMENT_REQUIREMENT_NAME,
																									   [NSNumber numberWithBool:YES],ICPROJECTREQUIREMENTS_IS_HEADER,
																									   nil]];
																									   
					[cachedFormattedRequirements_ addObjectsFromArray:tCachedTargetRequirements];
				}
			}
		}
		else
		{
			// A COMPLETER
		
			NSLog(@"Low Memory");
		}
		
		[tCachedTargetRequirements release];
		
		[tCachedInstallationRequirements release];
	}
	
	[IBrequirementsArray_ reloadData];
}

/*#pragma mark -

- (IBAction) updatePreInstallPath:(id) sender
{
	NSMutableDictionary * tPathDictionary;
		
	tPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_REQUIREMENTS_PREINSTALL_PATH];
        
	if (tPathDictionary!=nil)
	{
		NSInteger tOldType,tNewType;
		NSString * tOldPath,* tNewPath;
		
		tOldPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
		
		tNewPath=[IBpreInstallPath_ stringValue];
		
		tOldType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
		
		tNewType=[IBpreInstallPath_ pathType];
		
		if ([tOldPath isEqualToString:tNewPath]==NO ||
			tOldType!=tNewType)
		{
			[tPathDictionary setObject:tNewPath forKey:ICDOCUMENT_PATH];
		
			[tPathDictionary setObject:[NSNumber numberWithInteger:tNewType] forKey:ICDOCUMENT_PATH_TYPE];
		
			[self noteDocumentChange];
		}
	}
}

- (IBAction) selectPreInstallPath:(id) sender
{
    NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:NO];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:NO];
    
    [tOpenPanel beginSheetForDirectory:[IBpreInstallPath_ absolutePath]
                                  file:nil
                                 types:nil
                        modalForWindow:[IBpreInstallPath_ window]
                         modalDelegate:self
                        didEndSelector:@selector(selectPreInstallPathDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) selectPreInstallPathDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        [IBpreInstallPath_ setAbsolutePath:[sheet filename]];
        
        [self updatePreInstallPath:IBpreInstallPath_];
    }
}

- (IBAction) revealPreInstallPathInFinder:(id) sender
{
    NSWorkspace * tWorkSpace;
    
    tWorkSpace=[NSWorkspace sharedWorkspace];
    
    [tWorkSpace selectFile:[IBpreInstallPath_ absolutePath] inFileViewerRootedAtPath:@""];
}

- (IBAction) updatePostInstallPath:(id) sender
{
	NSMutableDictionary * tPathDictionary;
		
	tPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_REQUIREMENTS_POSTINSTALL_PATH];
        
	if (tPathDictionary!=nil)
	{
		NSInteger tOldType,tNewType;
		NSString * tOldPath,* tNewPath;
		
		tOldPath=[tPathDictionary objectForKey:ICDOCUMENT_PATH];
		
		tNewPath=[IBpostInstallPath_ stringValue];
		
		tOldType=[[tPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
		
		tNewType=[IBpostInstallPath_ pathType];
		
		if ([tOldPath isEqualToString:tNewPath]==NO ||
			tOldType!=tNewType)
		{
			[tPathDictionary setObject:tNewPath forKey:ICDOCUMENT_PATH];
		
			[tPathDictionary setObject:[NSNumber numberWithInteger:tNewType] forKey:ICDOCUMENT_PATH_TYPE];
		
			[self noteDocumentChange];
		}
	}
}

- (IBAction) selectPostInstallPath:(id) sender
{
    NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:NO];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:NO];
    
    [tOpenPanel beginSheetForDirectory:[IBpostInstallPath_ absolutePath]
                                  file:nil
                                 types:nil
                        modalForWindow:[IBpostInstallPath_ window]
                         modalDelegate:self
                        didEndSelector:@selector(selectPostInstallPathDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) selectPostInstallPathDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        [IBpostInstallPath_ setAbsolutePath:[sheet filename]];
        
        [self updatePostInstallPath:IBpostInstallPath_];
    }
}

- (IBAction) revealPostInstallPathInFinder:(id) sender
{
    NSWorkspace * tWorkSpace;
    
    tWorkSpace=[NSWorkspace sharedWorkspace];
    
    [tWorkSpace selectFile:[IBpostInstallPath_ absolutePath] inFileViewerRootedAtPath:@""];
}

#pragma mark -

- (BOOL) textField:(ICFileTextField *) inTextField shouldAcceptFileAtPath:(NSString *) inPath
{
    if (inTextField==IBpreInstallPath_ ||
		inTextField==IBpostInstallPath_)
    {
        NSFileManager * tFileManager=[NSFileManager defaultManager];
        BOOL isDirectory;
    
        return ([tFileManager fileExistsAtPath:inPath isDirectory:&isDirectory]==YES && isDirectory==NO);
    }
    
    return NO;
}

- (BOOL) textField:(ICFileTextField *) inTextField didAcceptFileAtPath:(NSString *) inPath
{
    [(ICReferencedFileTextField *) inTextField setAbsolutePath:inPath];
    
    if (inTextField==IBpreInstallPath_)
    {
        [self updatePreInstallPath:IBpreInstallPath_];
    }
	else if (inTextField==IBpostInstallPath_)
    {
        [self updatePostInstallPath:IBpostInstallPath_];
    }
	
    
    return YES;
}*/

#pragma mark -

- (id) outlineView:(NSOutlineView *) inOutlineView objectValueForTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
    id objectValue = nil;
    ICFileNode * tFileNode;
    
    tFileNode=FILENODE_DATA(inItem);
    
    if([[inTableColumn identifier] isEqualToString: @"File"])
    {
        objectValue = [tFileNode fileName];
	}
    else
    if([[inTableColumn identifier] isEqualToString: @"Permissions"])
    {
        if ([tFileNode link]==YES)
        {
            objectValue= [FILENODE_DATA([inItem nodeParent]) privilegesStringRepresentationForLink];
        }
        else
        {
            objectValue= [tFileNode privilegesStringRepresentation];
        }
    }
    
    return objectValue;
}

- (void) outlineView:(NSOutlineView *) inOutlineView willDisplayCell:(NSCell *)cell forTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{    
    if ([[inTableColumn identifier] isEqualToString: @"File"])
    {
		ICFileNode * tFileNode;
        BOOL tFather=YES;
        NSImage * tIcon=nil;
		NSString * tFileName;
		int tFileType;
		NSColor * tColor;
		NSMutableDictionary * tMutableAttributesDictionary;
		
        tFileNode=FILENODE_DATA(inItem);
        
		tFileType=[tFileNode type];
		
		tFileName=[tFileNode fileName];
		
		tColor=[NSColor blackColor];
		
        switch(tFileType)
        {
           case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				
				if (tIcon==nil)
				{
					if (tFather==YES)
					{
						tIcon=genericFolderIcon_;
					}
					else
					{
						tIcon=genericDisabledFolderIcon_;
					}
                }
				
				if (tIcon!=nil)
				{
					[(ImageAndTextCell*)cell setImage:tIcon];
				}
				
				break;
            case ICDOCUMENT_FILE_REALFILE_NODE:
                
                [tFileNode fileExistenceOnDiskChanged];
                
                if ([tFileNode existsOnDisk]==NO)
                {
                    tColor=[NSColor redColor];
                }
                
                [(ImageAndTextCell*)cell setImage: [FILENODE_DATA(inItem) icon]];
				
                break;
        }
		
		tMutableAttributesDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tColor,NSForegroundColorAttributeName,
																					   [cell font],NSFontAttributeName,
																					   nil];
		
		if (cachedHighlightExcludedFiles_==YES && cachedFileBouncer_!=nil)
		{
			if ([inItem isItemExcludedByBouncer:cachedFileBouncer_]==YES)
			{
				[tMutableAttributesDictionary setObject:[NSNumber numberWithInt:NSUnderlineStyleSingle] forKey:NSStrikethroughStyleAttributeName];
			}
		}
		
		[fileNameFormatter_ setCachedAttributes:tMutableAttributesDictionary];
			
		[(ImageAndTextCell*)cell setStringValue:tFileName];
	}
}

#pragma mark -

- (BOOL) outlineView:(NSOutlineView *) inOutlineView writeItems:(NSArray*) inItems toPasteboard:(NSPasteboard*)pboard
{
    NSUInteger i,tCount;
    ICFileTree * tTree;
    NSMutableArray * tExternalDragArray;
	
    // We need to simplify the hierarchy and check no items is a base node
    
    tCount=[inItems count];
    
	[internalDragArray_ release];
	
    internalDragArray_=[[NSMutableArray alloc] initWithCapacity:tCount];
    
    tExternalDragArray=[NSMutableArray arrayWithCapacity:tCount];
    
    for(i=tCount;i>0;i--)
    {
        NSUInteger j;
		int tType;
		
		tTree=[inItems objectAtIndex:i-1];
        
		tType=[FILENODE_DATA(tTree) type];
		
        if (tType==ICDOCUMENT_FILE_BASE_NODE_INVISIBLE ||
			tType==ICDOCUMENT_FILE_BASE_NODE)
        {
            return NO;
        }
        
        for(j=i-1;j>0;j--)
        {
            if ([tTree isDescendantOfNode:[inItems objectAtIndex:j-1]]==YES)
            {
                break;
            }
        }
        
        if (j==0)
        {
            [internalDragArray_ insertObject:tTree atIndex:0];
            
            [tExternalDragArray insertObject:[tTree dictionaryWithProjectAtPath:nil referenceFolderPath:nil] atIndex:0];
        }
    }
    
    [pboard declareTypes:[NSArray arrayWithObjects: ICFileHierarchyPboardType,ICFileHierarchyExternalPboardType,nil] owner:self];
    
    [pboard setPropertyList:tExternalDragArray forType:ICFileHierarchyExternalPboardType];
	
    [pboard setPropertyList:[NSData data] forType:ICFileHierarchyPboardType]; 
    
    return YES;
}

- (NSDragOperation) outlineView:(NSOutlineView*) inOutlineView validateDrop:(id <NSDraggingInfo>)info proposedItem:(id) inItem proposedChildIndex:(NSInteger)childIndex
{
    NSPasteboard * tPasteBoard;
    NSArray * tArray;
    NSUInteger i,tCount;
    
    tPasteBoard=[info draggingPasteboard];
    
    switch(childIndex)
    {
        case NSOutlineViewDropOnItemIndex:
            break;
        default:
            
            if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject: NSFilenamesPboardType]]!=nil)
            {
                if ([fileTree_ numberOfChildren]==0)
				{
					[inOutlineView setDropItem:inItem dropChildIndex:-1];
				}
				else
				{
					NSString * tPath;
					ICFileTree * tChildTree;
					NSArray * tChildren;
					NSString * tName;
					NSUInteger k,tChildrenCount;
					BOOL find=NO;
					
					tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
							
					tCount=[tArray count];
							
					for(i=0;i<tCount;i++)
					{
						tPath=[tArray objectAtIndex:i];
								
						if ([inItem containsTreeWithName:[tPath lastPathComponent]]==YES)
						{
							return NSDragOperationNone;
						}
					}
					
					tPath=[tArray objectAtIndex:0];
					
					tName=[tPath lastPathComponent];
					
					tChildren=[inItem children];
					
					tChildrenCount=[tChildren count];
					
					for(k=0;k<tChildrenCount;k++)
					{
						tChildTree=[tChildren objectAtIndex:k];
						
						if ([tName compare:[FILENODE_DATA(tChildTree) fileName] options:NSCaseInsensitiveSearch]!=NSOrderedDescending)
						{
							[inOutlineView setDropItem:inItem dropChildIndex:k];
							find=YES;
							
							break;
						}
					}
					
					if (find==NO)
					{
						[inOutlineView setDropItem:inItem dropChildIndex:k];
					}
                }
				
                return NSDragOperationCopy;//[info draggingSourceOperationMask];
            }
            else
            {
                ICFileTree * tFileTree;
                NSString * tName=nil;
                
                if ([info draggingSource]==IBoutlineView_)
                {
					if ([inItem isDescendantOfNodeInArray:internalDragArray_]==YES)
                    {
                        return NSDragOperationNone;
                    }
                    
                    // We need to check the name and eventually switch the drop location
                    
                    tCount=[internalDragArray_ count];
                    
                    for(i=0;i<tCount;i++)
                    {
                        tFileTree=[internalDragArray_ objectAtIndex:i];
                        
                        tName=[FILENODE_DATA(tFileTree) fileName];
                        
                        if ([inItem containsTreeWithName:tName]==YES)
                        {
                            return NSDragOperationNone;
                        }
                    }
                    
                    tFileTree=[internalDragArray_ objectAtIndex:0];
                    
                    tName=[FILENODE_DATA(tFileTree) fileName];
                }
                else
                {
                    // External Drag
                    
                    NSArray * tExternalArray;
                    NSDictionary * tDictionary;
                    
                    tExternalArray=[[info draggingPasteboard] propertyListForType:ICFileHierarchyExternalPboardType];
                    
                    // We need to check the name and eventually switch the drop location
                    
                    tCount=[tExternalArray count];
                    
                    for(i=0;i<tCount;i++)
                    {
                        tDictionary=[tExternalArray objectAtIndex:i];
                        
                        if (tDictionary!=nil)
                        {
                            tName=[tDictionary objectForKey:ICDOCUMENT_PATH];
                        
                            if (tName==nil)
                            {
                                return NSDragOperationNone;
                            }
                            else
                            {
                                tName=[tName lastPathComponent];
                                
                                if ([inItem containsTreeWithName:tName]==YES)
                                {
                                    return NSDragOperationNone;
                                }
                            }
                        }
                    }
                    
                    tDictionary=[tExternalArray objectAtIndex:0];
                    
                    tName=[[tDictionary objectForKey:ICDOCUMENT_PATH] lastPathComponent];
                }
                    
                // Find the appropriate drop location based on the first name of the dropped objects
                
                {
                    ICFileTree * tChildTree;
                    NSArray * tChildren;
                    NSUInteger k,tChildrenCount;
                    BOOL find=NO;
                
                    tChildren=[inItem children];
                
                    tChildrenCount=[tChildren count];
                    
                    for(k=0;k<tChildrenCount;k++)
                    {
                        tChildTree=[tChildren objectAtIndex:k];
                        
                        if ([tName compare:[FILENODE_DATA(tChildTree) fileName] options:NSCaseInsensitiveSearch]!=NSOrderedDescending)
                        {
                            [inOutlineView setDropItem:inItem dropChildIndex:k];
							
                            find=NO;
                            
                            break;
                        }
                    }
                    
                    if (find==NO)
                    {
                        [inOutlineView setDropItem:inItem dropChildIndex:k];
                    }
                }
                
                return NSDragOperationGeneric;
            }
            break;
    }
    
    return NSDragOperationNone;
}

- (BOOL)outlineView:(NSOutlineView*) inOutlineView acceptDrop:(id <NSDraggingInfo>)info item:(id)targetItem childIndex:(NSInteger)childIndex
{
    ICFileTree * tFileTree;
    NSPasteboard * tPasteBoard;
    NSMutableArray * tNewSelectionArray;
    NSUInteger k,tNewCount;
    NSMutableIndexSet * tMutableIndexSet;
    tPasteBoard=[info draggingPasteboard];
    
	if (targetItem==nil)
	{
		targetItem=fileTree_;
	}
	
    tNewSelectionArray=[NSMutableArray array];
    
    if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject: NSFilenamesPboardType]]!=nil)
    {
        // Drag & Drop of a real item
        
        NSArray * tArray;
        BOOL tShowCustomizationDialog=YES;
		id tObject;
		
        tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
        
		tObject=[defaults_ objectForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
	
		if (tObject!=nil)
		{
			tShowCustomizationDialog=[defaults_ boolForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
		}
		
		if (tShowCustomizationDialog==YES)
		{
			[self beginFileSheetForFiles:tArray parents:[NSArray arrayWithObject:targetItem]];
		}
		else
		{
			[self _addFiles:tArray toParents:[NSArray arrayWithObject:targetItem] keepOwnerAndGroup:NO referenceStyle:[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE]];
		}
        
        return YES;
    }
    else
    {
        if ([info draggingSource]==IBoutlineView_)
        {
            NSMutableDictionary * tDisclosedDictionary;
			NSArray * tCoverageNodesArray;
			
			// Internal Drag, we need to move
            
			tDisclosedDictionary=[document_ documentDefaultForKey:[self disclosedStateKey]];
			
            // Find the coverage tree
			
			tCoverageNodesArray=[TreeNode minimumNodeCoverFromNodesInArray:internalDragArray_];
			
			if (tCoverageNodesArray!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tCoverageNodesArray objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSNumber * tNumber;
					
					while (tFileTree=[tEnumerator nextObject])
					{
						NSMutableArray * tChildFoldersArray;
		
						// Removed item and descendants from disclosed state dictionary (because the key will change)
						
						tChildFoldersArray=[tFileTree descendantFoldersIncludingSelf:YES];
						
						if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
						{
							NSUInteger i,tCount;
							
							tCount=[tChildFoldersArray count];
							
							for(i=tCount;i>0;i--)
							{
								ICFileTree * tGroupFileTree;
								NSString * tPath;
								
								tGroupFileTree=[tChildFoldersArray objectAtIndex:(i-1)];
								
								tPath=[tGroupFileTree filePath];
								
								if (tPath!=nil)
								{
									BOOL tRemoved=NO;
									
									tNumber=[tDisclosedDictionary objectForKey:tPath];
									
									if (tNumber==nil)
									{
										tRemoved=YES;
									}
									else
									{
										if ([tNumber boolValue]==NO)
										{
											tRemoved=YES;
										}
										
										[tDisclosedDictionary removeObjectForKey:tPath];
									}
									
									if (tRemoved==YES)
									{
										[tChildFoldersArray removeObject:tGroupFileTree];
									}
								}
							}
						}
						
						[tFileTree retain];
						
						[tFileTree removeFromParent];
						
						[targetItem insertSortedChild:tFileTree];
						
						[tFileTree release];
						
						[tNewSelectionArray addObject:tFileTree];
						
						// Put back the item and descendant folders disclosed state
			
						if (tChildFoldersArray!=nil && tDisclosedDictionary!=nil)
						{
							NSEnumerator * tChildEnumerator;
							
							tChildEnumerator=[tChildFoldersArray objectEnumerator];
							
							if (tChildEnumerator!=nil)
							{
								ICFileTree * tGroupFileTree;
								
								while (tGroupFileTree=[tChildEnumerator nextObject])
								{
									NSString * tPath;
									
									tPath=[tGroupFileTree filePath];
								
									if (tPath!=nil)
									{
										[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tPath];
									}
								}
							}
						}

					}
				}
			}
        }
        else
        {
            // External Drag, we copy
            
            NSArray * tExternalArray;
			NSUInteger i,tCount;
			NSNumber * tPathStyleNumber;
			
			tPathStyleNumber=[NSNumber numberWithLong:[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE]];
			
            tExternalArray=[tPasteBoard propertyListForType:ICFileHierarchyExternalPboardType];
            
            tCount=[tExternalArray count];
            
            for(i=0;i<tCount;i++)
            {
                NSMutableDictionary * tDictionary;
                
                tDictionary=[tExternalArray objectAtIndex:i];
                
				[tDictionary setObject:tPathStyleNumber forKey:ICDOCUMENT_PATH_TYPE];
				
                tFileTree=[ICFileTree fileNodeWithDictionary:tDictionary projectPath:[document_ folder] referenceFolderPath:[document_ referenceFolderPath]];
                
                [targetItem insertSortedChild:tFileTree];
                
                [tNewSelectionArray addObject:tFileTree];
            }
        }
    }
    
    [IBoutlineView_ deselectAll:nil];
    
    [IBoutlineView_ reloadData];
    
    tMutableIndexSet=[NSMutableIndexSet indexSet];
	
	if (tMutableIndexSet!=nil)
	{
		tNewCount=[tNewSelectionArray count];
		
		for(k=0;k<tNewCount;k++)
		{
			[tMutableIndexSet addIndex:[IBoutlineView_ rowForItem:[tNewSelectionArray objectAtIndex:k]]];
		}
		
		[IBoutlineView_ selectRowIndexes:tMutableIndexSet byExtendingSelection:NO];
	}
    
    hierarchyChanged_=YES;
	
	[self updateFiles:IBoutlineView_];
    
    return YES;
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
	SEL tAction;
	
	tAction=[inMenuItem action];
	
	if (tAction==@selector(newFolder:))
	{
		NSUInteger tSelectedCount;
		
		tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
		 
		if (tSelectedCount<=1)
		{
			return YES;
		}
		
		return NO;
	}
	
	return [super validateMenuItem:inMenuItem];
}

- (IBAction) newFolder:(id) sender
{
    NSInteger tRow;
    
    ICFileTree * tParentTree=nil;
    ICFileNode * tParentNode;
    ICFileTree * tNewFolderTree=nil;
    
    tRow=[IBoutlineView_ selectedRow];

    if (tRow==-1)
	{
		tParentTree=fileTree_;
	}
	else
	{
		id tItem;
		
		tItem=[IBoutlineView_ itemAtRow:tRow];
		
		switch([FILENODE_DATA(tItem) type])
		{
			case ICDOCUMENT_FILE_NEWFOLDER_NODE:
				tParentTree=(ICFileTree *) tItem;
				break;
			case ICDOCUMENT_FILE_REALFILE_NODE:
				tParentTree=(ICFileTree *) [tItem nodeParent];
				break;
		}
    }
	
    if (tParentTree!=nil)
    {
        if ([IBoutlineView_ isItemExpanded:tParentTree]==NO)
        {
            [IBoutlineView_ expandItem:tParentTree];
        }
    
        tParentNode=FILENODE_DATA(tParentTree);
            
        
        
        tNewFolderTree=[[ICFileTree alloc] initWithData:[ICFileNode folderFileNodeWithName:[ICFileTree uniqueNameWithParentFileTree:tParentTree]
                                                                                    icon:nil
                                                                                    user:[tParentNode uid]
                                                                                    group:[tParentNode gid]
                                                                                privileges:(0775 & ACCESSPERMS)]
                                                parent:nil
                                                children:[NSArray array]];
        
        if (tNewFolderTree!=nil)
        {
            [tParentTree insertSortedChild:tNewFolderTree];
            
            [tNewFolderTree release];
            
            hierarchyChanged_=YES;
            
            [IBoutlineView_ reloadData];
            
            tRow=[IBoutlineView_ rowForItem:tNewFolderTree];
            
            [IBoutlineView_ scrollRowToVisible:tRow];
            
            [IBoutlineView_ selectRow:tRow byExtendingSelection:NO];
            
            [IBoutlineView_ editColumn:[IBoutlineView_ columnWithIdentifier:@"File"] row:tRow withEvent:nil select:YES];
            
            [self updateFiles:IBoutlineView_];
        }
    }
}

- (IBAction) addFiles:(id)sender
{
    NSOpenPanel * tOpenPanel;
    NSInteger tSelectedRow;
    
	tSelectedRow=[IBoutlineView_ selectedRow];
    
    if (tSelectedRow==-1)
	{
		parentFileTree_=fileTree_;
	}
	else
	{
		parentFileTree_=[IBoutlineView_ itemAtRow:tSelectedRow];
	
		if ([FILENODE_DATA(parentFileTree_) type]==ICDOCUMENT_FILE_REALFILE_NODE)
		{
			parentFileTree_=(ICFileTree *) [parentFileTree_ nodeParent];
		}
    }
	
	[parentFileTree_ retain];
	
	 tOpenPanel=[NSOpenPanel openPanel];
	
	if ([[NSUserDefaults standardUserDefaults] boolForKey:ICPREFERENCES_FILES_SHOW_ALL_FILES]==YES)
	{
		[[tOpenPanel _navView] setShowsHiddenFiles:YES];	// @"AppleShowAllFiles" = YES
	}
	
    [tOpenPanel setResolvesAliases:YES];
    [tOpenPanel setCanChooseFiles:YES];
    [tOpenPanel setCanChooseDirectories:YES];
    [tOpenPanel setAllowsMultipleSelection:YES];
    [tOpenPanel setTreatsFilePackagesAsDirectories:YES];
    [tOpenPanel setDelegate:self];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Add...",@"No comment")];
    
    [tOpenPanel beginSheetForDirectory:nil
                                  file:nil
                                 types:nil
                        modalForWindow:[document_ windowForSheet]
                         modalDelegate:self
                        didEndSelector:@selector(addFilesPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) addFilesPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        NSArray * tFileNames;
		
		tFileNames=[sheet filenames];
		
		if (tFileNames!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[tFileNames objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSString * tPath;
				BOOL tShowCustomizationDialog=YES;
				id tObject;
				
				// A AMELIORER
				
				while (tPath=[tEnumerator nextObject])
				{
					if ([parentFileTree_ containsTreeWithName:[tPath lastPathComponent]]==YES)
					{
						NSBeep();
						
						return;
					}
				}
				
				tObject=[defaults_ objectForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
	
				if (tObject!=nil)
				{
					tShowCustomizationDialog=[defaults_ boolForKey: ICPREFERENCES_FILES_SHOW_CUSTOMIZATION_DIALOG];
				}
				
				if (tShowCustomizationDialog==YES)
				{
					NSDictionary * nContextualInformation;
					 
					 nContextualInformation=[[NSDictionary alloc] initWithObjectsAndKeys:tFileNames,IC_ADD_FILES,
																						 [NSArray arrayWithObject:parentFileTree_],IC_ADD_PARENTS,
																						 nil];
					 
					 
					
					[self performSelector:@selector(beginFileSheet:) withObject:nContextualInformation afterDelay:0.f];
				}
				else
				{
					[self _addFiles:tFileNames
						  toParents:[NSArray arrayWithObject:parentFileTree_] 
				  keepOwnerAndGroup:NO 
				     referenceStyle:[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE]];
				}
			}
		}
    }
	
	[parentFileTree_ release];
	
	parentFileTree_=nil;
}

- (void) beginFileSheetForFiles:(NSArray *) inFiles parents:(id) inParents
{
    NSInteger tDefaultReferenceStyle;
	NSDictionary * nContextualInformation;
	
    tDefaultReferenceStyle=[defaults_ integerForKey:ICPREFERENCES_GENERAL_DEFAULT_REFERENCE_STYLE];
    
    nContextualInformation=[[NSDictionary alloc] initWithObjectsAndKeys:inFiles,IC_ADD_FILES,
																		inParents,IC_ADD_PARENTS,
																		nil];
    
    // Prepare the sheet
    
    [IBreferenceStyle_ selectItemAtIndex:[IBreferenceStyle_ indexOfItemWithTag:tDefaultReferenceStyle]];
    
    [NSApp beginSheet:IBreferenceWindow_
       modalForWindow:[document_ windowForSheet]
        modalDelegate:self
       didEndSelector:@selector(fileSheetDidEnd:returnCode:contextInfo:)
          contextInfo:nContextualInformation];
}

- (void) fileSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
    NSDictionary * nContextualInformation;
    
    nContextualInformation=(NSDictionary *) contextInfo;
    
    if (returnCode==NSOKButton)
    {
		[self _addFiles:[nContextualInformation objectForKey:IC_ADD_FILES] 
			  toParents:[nContextualInformation objectForKey:IC_ADD_PARENTS] 
	  keepOwnerAndGroup:NO
	     referenceStyle:[[IBreferenceStyle_ selectedItem] tag]];
	}
    
    [nContextualInformation release];

    [sheet orderOut:self];
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBrequirementsArray_)
    {
		NSInteger tNumberOfSelectedRows;
		
		tNumberOfSelectedRows=[IBrequirementsArray_ numberOfSelectedRows];
		
		[IBrequirementsEditButton_ setEnabled:(tNumberOfSelectedRows==1)];
		
		[IBrequirementsRemoveButton_ setEnabled:(tNumberOfSelectedRows!=0)];
	}
	
}

- (void) outlineViewSelectionDidChange:(NSNotification *) inNotification
{
	NSInteger tSelectedCount;
	NSRect tBounds;
	NSView * tView;
	
	tSelectedCount=[IBoutlineView_ numberOfSelectedRows];
	
	if (tSelectedCount==0)
	{
		if (emptySelectionInspectorController_==nil)
		{
			emptySelectionInspectorController_=[ICFileHierarchyInspectorEmptySelectionController newController];
		}
		
		if (currentInspectorController_!=emptySelectionInspectorController_ || currentInspectorController_==nil)
		{
			tView=[currentInspectorController_ view];
		
			[tView removeFromSuperview];
		
			currentInspectorController_=emptySelectionInspectorController_;
			
			tView=[currentInspectorController_ view];
			
			tBounds=[IBinspectorView_ bounds];
			
			[tView setFrame:tBounds];
			
			[IBinspectorView_ addSubview:tView];
		}
		
		[IBremoveButton_ setEnabled:NO];
	}
	else
	{
		NSIndexSet * tIndexSet;
		
		[IBremoveButton_ setEnabled:YES];
		
		if (selectionInspectorController_==nil)
		{
			selectionInspectorController_=[ICProjectResourcesInspectorSelectionController newController];
		}
		
		if (currentInspectorController_!=selectionInspectorController_ || currentInspectorController_==nil)
		{
			tView=[currentInspectorController_ view];
		
			[tView removeFromSuperview];
		
			currentInspectorController_=selectionInspectorController_;
			
			tView=[currentInspectorController_ view];
			
			tBounds=[IBinspectorView_ bounds];
			
			[tView setFrame:tBounds];
			
			[IBinspectorView_ addSubview:tView];
		}
		
		tIndexSet=[IBoutlineView_ selectedRowIndexes];
		
		if (tIndexSet!=nil)
		{
			NSMutableArray * tSelectedItems;
			
			tSelectedItems=[NSMutableArray array];
			
			if (tSelectedItems!=nil)
			{
				NSUInteger tIndex;
				
				tIndex=[tIndexSet firstIndex];
				
				while (tIndex!=NSNotFound)
				{
					id tItem;
					
					tItem=[IBoutlineView_ itemAtRow:tIndex];
					
					if (tItem!=nil)
					{
						[tSelectedItems addObject:tItem];
						
						tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
					}
				}
				
				[selectionInspectorController_ initializeWithArray:tSelectedItems forDocument:document_];
			}
		}
	}
}

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	NSRect tBounds;
	NSUInteger tTotalNumber;
	CGFloat tAvailableHeight;
	CGFloat tRowHeight;
	CGFloat tMaximumRequirementsHeight;
	NSRect tResourcesFrame;
	NSRect tRequirementsFrame;
	NSSize tSize;
	
	tBounds=[IBview_ bounds];
	
	tResourcesFrame=[IBresourcesSection_ frame];
	
	tRequirementsFrame=[IBrequirementsSection_ frame];
	
	tTotalNumber=[cachedFormattedRequirements_ count];
	
	tAvailableHeight=NSHeight(tBounds)-cachedRessourcesSectionInitialHeight_;
	
	tRowHeight=[IBrequirementsArray_ rowHeight];
	
	if (tTotalNumber<3)
	{
		tTotalNumber=3;
	}
	
	tSize=[IBrequirementsArray_ intercellSpacing];
	
	tMaximumRequirementsHeight=NSHeight(tRequirementsFrame)-NSHeight([IBrequirementsScrollView_ frame])+tRowHeight*tTotalNumber+(tTotalNumber-1)*tSize.height+4;
	
	
	if (tMaximumRequirementsHeight<tAvailableHeight)
	{
		tRequirementsFrame.size.height=tMaximumRequirementsHeight;
		
		tResourcesFrame.size.height=NSHeight(tBounds)-tRequirementsFrame.size.height;
	}
	else
	{
		tRequirementsFrame.size.height=tAvailableHeight;
		
		tResourcesFrame.size.height=cachedRessourcesSectionInitialHeight_;
	}
	
	tResourcesFrame.origin.y=0.0f;
	
	tRequirementsFrame.origin.y=NSMaxY(tResourcesFrame);
	
	[IBresourcesSection_ setFrame:tResourcesFrame];
	
	[IBrequirementsSection_ setFrame:tRequirementsFrame];
	
	if (inNotification==nil)
	{
		[IBview_ setNeedsDisplay:YES];
	}
}

@end
