/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectRequirementsTableView.h"

#import "NSTableView+HeaderRow.h"

@implementation ICProjectRequirementsTableView

- (void) reloadData
{
	if (headersIndexSet_!=nil)
	{
		[headersIndexSet_ release];
		
		headersIndexSet_=nil;
	}

	[super reloadData];
}

- (void) highlightSelectionInClipRect:/**/(NSRect)clipRect
{
    CGFloat rowHeight = [self rowHeight] + [self intercellSpacing].height;
    NSRect tBounds=[self bounds];
    NSRect highlightRect;
    NSColor * tRowColor;
    static NSColor * sWhiteColor=nil;
	static NSColor * sGrayColor=nil;
	NSInteger tLastHeaderIndex=-2;
	
	
    if (sWhiteColor==nil)
    {
        sWhiteColor=[[NSColor whiteColor] retain];
    }
    
    highlightRect.origin = NSMakePoint(NSMinX(tBounds), (int)(NSMinY(tBounds)/rowHeight)*rowHeight);
    highlightRect.size = NSMakeSize(NSWidth(tBounds), rowHeight + [self intercellSpacing].height);
    
    if ([[[NSColor colorForControlTint:NSDefaultControlTint] colorNameComponent] isEqualToString:@"graphiteControlTintColor"]==NO)
	{
		tRowColor=[NSColor colorWithCalibratedRed:0.929f
											green:0.953f
											 blue:1.0f
											alpha:1.0f];
	}
	else
	{
		if (sGrayColor==nil)
		{
			sGrayColor=[[NSColor colorWithDeviceWhite:0.94f alpha:1.0f] retain];
		}
		
		tRowColor=sGrayColor;
	}
    
    while (NSMinY(highlightRect) < NSMaxY(tBounds))
    {
        NSRect clippedHighlightRect = NSIntersectionRect(highlightRect, tBounds);
        NSInteger tRowIndex;
		NSColor * tColor;
		
		tRowIndex = (int)((NSMinY(highlightRect)+rowHeight*0.5f)/rowHeight);
		
		if (tRowIndex<[self numberOfRows] &&
			[self delegate]!=nil &&
			[[self delegate] tableView:self isHeaderAtRow:tRowIndex]==YES)
		{
			if (headersIndexSet_==nil)
			{
				headersIndexSet_=[[NSMutableIndexSet alloc] initWithIndex:tRowIndex];
			}
			else
			{
				if ([headersIndexSet_ containsIndex:tRowIndex]==NO)
				{
					[headersIndexSet_ addIndex:tRowIndex];
				}
			}
		
			tColor=[NSColor colorWithDeviceRed:155/255.0f
										 green:178/255.0f
										  blue:232/255.0f
										 alpha:1.0f];
			
			tLastHeaderIndex=tRowIndex;
		}
		else
		{
			if (headersIndexSet_!=nil)
			{
				tLastHeaderIndex=[headersIndexSet_ indexLessThanIndex:tRowIndex];
			}
			
			tColor = (0 == ((tRowIndex-tLastHeaderIndex) & 0x1)) ? tRowColor : sWhiteColor;
		}
		
        [tColor set];
        
        NSRectFill(clippedHighlightRect);
		
        highlightRect.origin.y += rowHeight;
    }
    
    [super highlightSelectionInClipRect: clipRect];	// call superclass's behavior
}

@end
