/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectSettingsAdvancedOptionsListEditor.h"

@implementation ICProjectSettingsAdvancedOptionsListEditor

+ (id) showEditorForWindow:(NSWindow *) inDocumentWindow key:(NSString *) inKey value:(id) inValue description:(NSDictionary *) inDescription delegate:(id) inDelegate
{
	ICProjectSettingsAdvancedOptionsListEditor * nEditor=nil;
	
	if (inDocumentWindow!=nil && inKey!=nil && inDescription!=nil)
	{
		nEditor=[ICProjectSettingsAdvancedOptionsListEditor alloc];
		
		if (nEditor!=nil)
		{
			if ([NSBundle loadNibNamed:@"ICAdvancedOptionsListEditor" owner:nEditor]==NO)
			{
				NSBeep();
				
				NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICAdvancedOptionsListEditor"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
				
				[NSApp terminate:nil];
			}
			
			if ([nEditor initWithKey:inKey value:inValue description:inDescription delegate:inDelegate]!=nil)
			{
				[NSApp beginSheet:[nEditor window] modalForWindow:inDocumentWindow modalDelegate:nil didEndSelector:nil contextInfo:NULL];
			}
		
			
		}
		else
		{
			// A COMPLETER
		}
	}
	
	return nEditor;
}

- (id) initWithKey:(NSString *) inKey value:(id) inValue description:(NSDictionary *) inDescription delegate:(id) inDelegate
{
	self=[super initWithKey:inKey value:inValue description:inDescription delegate:inDelegate];

	if (self!=nil)
	{
		NSString * tEditorLabel;
		
		tEditorLabel=[inDescription objectForKey:@"LABEL"];
			
		if (tEditorLabel!=nil && [tEditorLabel length]>0)
		{
			[IBeditorLabel_ setStringValue:tEditorLabel];
		}
		else
		{
			[IBeditorLabel_ setStringValue:@"MISSING LABEL!!!!!!!!!"];
		}
		
		[IBeditorLabel_ sizeToFit];
		
		// Set the list of items
		
		items_=[[inDescription objectForKey:@"ITEMS"] retain];
		
		if (items_!=nil)
		{
			NSUInteger i,tCount;
			NSView * tContentView;
			NSRect tContentBounds;
			NSRect tFrame;
			NSRect tOldMatrixFrame;
			CGFloat tDeltaX=0.0f;
			CGFloat tDeltaY=0.0f;
			
			tCount=[items_ count];
			
			tOldMatrixFrame=[IBmatrix_ frame];
			
			[IBmatrix_ renewRows:tCount columns:1];
			
			for(i=0;i<tCount;i++)
			{
				NSButtonCell * tButtonCell;
			
				tButtonCell=(NSButtonCell *) [IBmatrix_ cellAtRow:i column:0];
				
				if (tButtonCell!=nil)
				{
					NSDictionary * tItemDictionary;
					NSString * tValue;
					NSString * tLabel;
					
					tItemDictionary=[items_ objectAtIndex:i];
					
					tLabel=[tItemDictionary objectForKey:@"LABEL"];
					
					if (tLabel==nil)
					{
						tLabel=@"MISSING LABEL!";
					}
					
					[tButtonCell setTitle:NSLocalizedStringFromTable(tLabel,@"Project",@"")];
					
					tValue=[tItemDictionary objectForKey:@"VALUE"];
					
					if (tValue!=nil)
					{
						if ([inValue containsObject:tValue]==YES)
						{
							[tButtonCell setState:NSOnState];
						}
						else
						{
							[tButtonCell setState:NSOffState];
						}
					}
					
				}
			}
			
			[IBmatrix_ sizeToFit];
			
			// Resize the window as required
			
			tContentView=[IBwindow_ contentView];
			
			tContentBounds=[tContentView bounds];
			
			tFrame=[IBeditorLabel_ frame];
			
			if ((NSMaxX(tFrame)+20.0f)>NSWidth(tContentBounds))
			{
				tDeltaX=NSMaxX(tFrame)+20.0f-tContentBounds.size.width;
			
				tContentBounds.size.width+=tDeltaX;
			}
			
			tFrame=[IBmatrix_ frame];
			
			if (NSMaxY(tFrame)!=NSMaxY(tOldMatrixFrame))
			{
				tDeltaY=NSMaxY(tFrame)-NSMaxY(tOldMatrixFrame);
			}
			
			if ((NSMaxX(tFrame)+20.0f)>NSWidth(tContentBounds))
			{
				tDeltaX=NSMaxX(tFrame)+20.0f-tContentBounds.size.width;
			}
			
			tFrame=[IBwindow_ frame];
			
			tFrame.size.width+=tDeltaX;
			
			tFrame.size.height+=tDeltaY;
			
			[IBwindow_ setFrame:tFrame display:NO animate:NO];
		}
	}

	return self;
}

- (void) dealloc
{
	[items_ release];
	
	[super dealloc];
}

#pragma mark -

- (id) value
{
	NSMutableArray * tList;
	
	tList=[NSMutableArray array];
	
	if (tList!=nil)
	{
		NSUInteger i,tCount;
	
		tCount=[items_ count];
				
		for(i=0;i<tCount;i++)
		{
			NSButtonCell * tButtonCell;
		
			tButtonCell=(NSButtonCell *) [IBmatrix_ cellAtRow:i column:0];
			
			if (tButtonCell!=nil)
			{
				NSDictionary * tItemDictionary;
				NSString * tValue;
				
				tItemDictionary=[items_ objectAtIndex:i];
				
				tValue=[tItemDictionary objectForKey:@"VALUE"];
				
				if (tValue!=nil)
				{
					if ([tButtonCell state]==NSOnState)
					{
						[tList addObject:tValue];
					}
				}
			}
		}
	}
	
	return tList;
}


@end
