/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectSettingsAdvancedOptionsOutlineView.h"
#import "ICProjectSettingsController.h"

@implementation NSObject (ICProjectSettingsAdvancedOptionsOutlineView)

- (BOOL) outlineView:(NSOutlineView *) inOutlineView doubleClickInTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{
	return NO;
}

@end

@implementation ICWhiteDiscloseButtonCell: NSButtonCell

- (id) init
{
	self=[super initTextCell:@""];
	
	if (self!=nil)
	{
		[self setBezelStyle:NSDisclosureBezelStyle];
		
		[self setButtonType:NSOnOffButton];
	}
	
	return self;
}

- (void) drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
	NSBezierPath * tBezierPath;
	
	tBezierPath=[NSBezierPath bezierPath];
	
	if (tBezierPath!=nil)
	{
		// Is the button pushed
		
		float tAngle;

#define MakeTrianglePoint(x,y) NSMakePoint(_CGFloatRound(NSMidX(cellFrame)+(x*cosf(tAngle)+y*sinf(tAngle)))-0.5f,_CGFloatRound(NSMidY(cellFrame)+(x*sinf(tAngle)-y*cosf(tAngle)))+0.5f)
		
		if (_cFlags.highlighted)
		{
			tAngle=2.4f;
			
			[[NSColor colorWithDeviceWhite:0.45f alpha:1.0f] set];

		}
		else
		{
			NSInteger tState;
		
			tState=[self state];
			
			if (tState==NSOnState)
			{
				tAngle=3.141592653f;
				
				[tBezierPath closePath];
			}
			else
			{
				tAngle=1.5707963265f;
			}
			
			[[NSColor whiteColor] set];
		}
		
		[tBezierPath moveToPoint:MakeTrianglePoint(-6.0f,-4.5f)];
				
		[tBezierPath lineToPoint:MakeTrianglePoint(-1.0f,4.0f)];
			
		[tBezierPath lineToPoint:MakeTrianglePoint(4.0f,-4.5f)];
			
		[tBezierPath closePath];
		
		[tBezierPath fill];
	}
	
	
}

@end


@implementation ICProjectSettingsAdvancedOptionsOutlineView

- (id)initWithCoder:(id) inCoder
{
	self=[super initWithCoder:inCoder];
	
	return self;
}

- (BOOL) acceptsFirstMouse:(NSEvent *)theEvent
{
    return acceptFirstClick_;
}

- (void) setAcceptFirstClick:(BOOL) aBool
{
    acceptFirstClick_=aBool;
}

#pragma mark -

#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5

- (NSRect) frameOfOutlineCellAtRow:(NSInteger) inRow
{
	NSRect tRect;
	
	tRect=[super frameOfOutlineCellAtRow:inRow];
	
	tRect.origin.x-=1.0f;
	
	return tRect;
}

#endif

- (void) reloadData
{
	if (headersIndexSet_!=nil)
	{
		[headersIndexSet_ release];
		
		headersIndexSet_=nil;
	}

	[super reloadData];
}

- (void) mouseDown:(NSEvent *) inEvent
{
	if ([inEvent clickCount]>1)
	{
		NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
		id tItem;
		NSTableColumn * tColumn;
		
		tItem=[self itemAtRow:[self rowAtPoint:tMouseLoc]];
		
		tColumn=[[self tableColumns] objectAtIndex:[self columnAtPoint:tMouseLoc]];
		
		if (tColumn!=nil && tItem!=nil)
		{
			if ([self delegate]!=nil && [[self delegate] outlineView:self doubleClickInTableColumn:tColumn item:tItem]==YES)
			{
				return;
			}
		}
	}
	
	[super mouseDown:inEvent];
}

- (void) highlightSelectionInClipRect:/**/(NSRect)clipRect
{
    CGFloat rowHeight = [self rowHeight] + [self intercellSpacing].height;
    NSRect visibleRect = [self visibleRect];
    NSRect highlightRect;
    NSColor * tRowColor;
    static NSColor * sWhiteColor=nil;
	static NSColor * sGrayColor=nil;
	NSInteger tLastHeaderIndex=-2;
	
	
    if (sWhiteColor==nil)
    {
        sWhiteColor=[[NSColor whiteColor] retain];
    }
    
    highlightRect.origin = NSMakePoint(NSMinX(visibleRect), (int)(NSMinY(clipRect)/rowHeight)*rowHeight);
    highlightRect.size = NSMakeSize(NSWidth(visibleRect), rowHeight + [self intercellSpacing].height);
    
    if ([[[NSColor colorForControlTint:NSDefaultControlTint] colorNameComponent] isEqualToString:@"graphiteControlTintColor"]==NO)
	{
		tRowColor=[NSColor colorWithCalibratedRed:0.929f
											green:0.953f
											blue:1.0f
											alpha:1.0f];
	}
	else
	{
		if (sGrayColor==nil)
		{
			sGrayColor=[[NSColor colorWithDeviceWhite:0.94f alpha:1.0f] retain];
		}
		
		tRowColor=sGrayColor;
	}
    
    while (NSMinY(highlightRect) < NSMaxY(clipRect))
    {
        NSRect clippedHighlightRect = NSIntersectionRect(highlightRect, clipRect);
        NSInteger tRowIndex;
		NSColor * tColor;
		BOOL tDrawSeparator=NO;
		
		tRowIndex = (int)((NSMinY(highlightRect)+rowHeight*0.5f)/rowHeight);
		
		if ([self delegate]!=nil && [[self delegate] isHeaderAtRow:tRowIndex]==YES)
		{
			if (headersIndexSet_==nil)
			{
				headersIndexSet_=[[NSMutableIndexSet alloc] initWithIndex:tRowIndex];
			}
			else
			{
				if ([headersIndexSet_ containsIndex:tRowIndex]==NO)
				{
					[headersIndexSet_ addIndex:tRowIndex];
				}
			}
		
			tColor=[NSColor colorWithDeviceRed:155/255.0f
										 green:178/255.0f
										  blue:232/255.0f
										 alpha:1.0f];
				
			if ([[self delegate] isHeaderAtRow:tRowIndex-1]==YES)
			{
				tDrawSeparator=YES;
			}
			
			tLastHeaderIndex=tRowIndex;
		}
		else
		{
			if (headersIndexSet_!=nil)
			{
				tLastHeaderIndex=[headersIndexSet_ indexLessThanIndex:tRowIndex];
			}
			
			tColor = (0 == ((tRowIndex-tLastHeaderIndex) & 0x1)) ? tRowColor : sWhiteColor;
		}
		
        [tColor set];
        
        NSRectFill(clippedHighlightRect);
        
		if (tDrawSeparator==YES)
		{
			[[NSColor whiteColor] set];
			
			[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(clippedHighlightRect),NSMinY(clippedHighlightRect)-0.5f)
									  toPoint:NSMakePoint(NSMaxX(clippedHighlightRect),NSMinY(clippedHighlightRect)-0.5f)];
		}
		
        highlightRect.origin.y += rowHeight;
    }
    
    [super highlightSelectionInClipRect: clipRect];	// call superclass's behavior
}

@end
