/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectSettingsAdvancedOptionsTree.h"

@implementation ICProjectSettingsAdvancedOptionsNodeData

+ (id) advancedOptionsNodeDataOfType:(int) inType withID:(NSString *) inID
{
	return [[[ICProjectSettingsAdvancedOptionsNodeData alloc] initWithType:inType andID:inID] autorelease];
}

+ (id) rootNodeData
{
	return [ICProjectSettingsAdvancedOptionsNodeData advancedOptionsNodeDataOfType:kAdvancedOptionsRootNode withID:nil];
}

- (id) initWithType:(int) inType andID:(NSString *) inID
{
	self=[super init];
	
	if (self!=nil)
	{
		type_=inType;
		
		id_=[inID retain];
	}
	
	return self;
}

- (void) dealloc
{
	[id_ release];

	[super dealloc];
}

#pragma mark -

- (int) type
{
	return type_;
}

- (NSString *) ID
{
	return [[id_ retain] autorelease];
}

@end


@implementation ICProjectSettingsAdvancedOptionsTreeNode

+ (id) advancedOptionsTreeNodeWithArray:(NSArray *) inArray
{
	ICProjectSettingsAdvancedOptionsTreeNode * tRootNode;
	
	tRootNode=[[ICProjectSettingsAdvancedOptionsTreeNode alloc] initWithData:[ICProjectSettingsAdvancedOptionsNodeData rootNodeData]
																	  parent:nil
																	children:[NSArray array]];
	
	if (tRootNode!=nil)
	{
		if (inArray!=nil)
		{
			NSEnumerator * tEnumerator;
		
			tEnumerator=[inArray objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSMutableDictionary * tItemDictionary;
				
				while (tItemDictionary=[tEnumerator nextObject])
				{
					ICProjectSettingsAdvancedOptionsTreeNode * tChildNode=nil;
					
					tChildNode=[ICProjectSettingsAdvancedOptionsTreeNode treeNodeWithDictionary:tItemDictionary];
					
					if (tChildNode!=nil)
					{
						[tRootNode insertChild:tChildNode
									   atIndex:[tRootNode numberOfChildren]];
					}
				}
			}
		}
	}
	
	return [tRootNode autorelease];
}

+ (id) treeNodeWithDictionary:(NSDictionary *) inDictionary
{
	ICProjectSettingsAdvancedOptionsTreeNode * tTreeNode=nil;
	
	if (inDictionary!=nil)
	{
		NSString * tID;
		
		tID=[inDictionary objectForKey:@"ID"];
		
		tTreeNode=[[ICProjectSettingsAdvancedOptionsTreeNode alloc] initWithData:[ICProjectSettingsAdvancedOptionsNodeData advancedOptionsNodeDataOfType:kAdvancedOptionsObject withID:tID]
																		  parent:nil
																		children:[NSArray array]];
		
		if (tTreeNode!=nil)
		{
			NSArray * tChildrenArray;
			
			tChildrenArray=[inDictionary objectForKey:@"CHILDREN"];
			
			if (tChildrenArray!=nil && [tChildrenArray count]>0)
			{
				NSEnumerator * tEnumerator;
					
				tEnumerator=[tChildrenArray objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSMutableDictionary * tItemDictionary;
			
					while (tItemDictionary=[tEnumerator nextObject])
					{
						ICProjectSettingsAdvancedOptionsTreeNode * tChildTreeNode=nil;
						
						tChildTreeNode=[ICProjectSettingsAdvancedOptionsTreeNode treeNodeWithDictionary:tItemDictionary];
						
						if (tChildTreeNode!=nil)
						{
							[tTreeNode insertChild:tChildTreeNode
										   atIndex:[tTreeNode numberOfChildren]];
						}
					}
				}
			}
		}
	}
	
	return [tTreeNode autorelease];
}

- (BOOL) isLeaf
{
	return ([self numberOfChildren]==0);
}

@end