/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectSettingsController.h"

#import "ICProjectSettingsAdvancedOptionsTextFieldCell.h"
#import "ICProjectSettingsAdvancedOptionsCheckBoxCell.h"

#import "ICProjectSettingsAdvancedOptionsListEditor.h"

#import "ICOffsetedPopupButtonCell.h"

#import "ICExclusionArraySeparatorCell.h"
#import "ICExclusionArrayPopupButtonCell.h"

#import "ICReferencedFileTextField.h"

#import "ICPreferences+Constants.h"

#import "ICInstallationStepsUtilities.h"

#import "ICIconsUtilities.h"

#import "ICCertificatesUtilities.h"

#import "ICDocument.h"

#import "ICProjectNameFormatter.h"

#import "ICPatternFormatter.h"

#import "TreeNode+Size.h"

#import <SecurityInterface/SFChooseIdentityPanel.h>
#import <SecurityInterface/SFCertificatePanel.h>
#include <Security/Security.h>

#import "ICCertificateWindow.h"

#import "ICCertificateStampView.h"

#import "ICCertificatePanel.h"

#define SAFENODE(n) 	((ICProjectSettingsAdvancedOptionsTreeNode*)((n!=nil)?(n):(tree_)))

NSString * const ICPROJECT_SETTINGS_ADVANCED_OPTIONS_DISCLOSED_STATE=@"ui.project.settings.options.advanced.disclosed";

@implementation ICProjectSettingsController

+ (ICProjectSettingsController *) projectSettingsController
{
	ICProjectSettingsController * nController=nil;
    
    nController=[ICProjectSettingsController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Project_Settings" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Project_Settings"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) dealloc
{
	[advancedOptionsDescription_ release];
	
	[tree_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{
	NSButtonCell * tPrototypeButtonCell = nil;
    NSTextFieldCell * tPrototypeTextCell=nil;
	ICProjectSettingsAdvancedOptionsTextFieldCell * tTextFieldCell;
	NSTableColumn * tTableColumn = nil;
	ICProjectNameFormatter * tFormatter;
	
	defaults_=[NSUserDefaults standardUserDefaults];
	
	// Build Name
	
	tFormatter=[ICProjectNameFormatter new];
	
	if (tFormatter!=nil)
	{
		[IBbuildName_ setFormatter:tFormatter];
		
		[tFormatter release];
	}
	
	// Excluded Files TableView
	
	tTableColumn = [IBexclusionArray_ tableColumnWithIdentifier: @"State"];
    
	tPrototypeButtonCell = [[NSButtonCell alloc] initTextCell: @""];
    
	if (tPrototypeButtonCell!=nil)
	{
		[tPrototypeButtonCell setControlSize:NSMiniControlSize];
		[tPrototypeButtonCell setEditable: YES];
		[tPrototypeButtonCell setButtonType: NSSwitchButton];
		[tPrototypeButtonCell setImagePosition: NSImageOnly];
    
		[tTableColumn setDataCell:tPrototypeButtonCell];
    
		[tPrototypeButtonCell release];
	}
	
	tTableColumn = [IBexclusionArray_ tableColumnWithIdentifier: @"Name"];
	
	tPrototypeTextCell=[tTableColumn dataCell];
	
	if (tPrototypeTextCell!=nil)
	{
		ICPatternFormatter * tFormatter;
		
		[tPrototypeTextCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
		
		tFormatter=[ICPatternFormatter new];
		
		if (tFormatter!=nil)
		{
			[tPrototypeTextCell setFormatter:tFormatter];
		
			[tFormatter release];
		}
	}
	
	tTableColumn = [IBexclusionArray_ tableColumnWithIdentifier: @"RegularExpression"];
    
	tPrototypeButtonCell = [[NSButtonCell alloc] initTextCell: @""];
    
	if (tPrototypeButtonCell!=nil)
	{
		[tPrototypeButtonCell setControlSize:NSMiniControlSize];
		[tPrototypeButtonCell setEditable: YES];
		[tPrototypeButtonCell setButtonType: NSSwitchButton];
		[tPrototypeButtonCell setImagePosition: NSImageOnly];
    
		[tTableColumn setDataCell:tPrototypeButtonCell];
    
		[tPrototypeButtonCell release];
	}
	
	// Advanced Options
	
	tTableColumn = [IBadvancedOptionsOutlineView_ tableColumnWithIdentifier: @"Key"];
    
	tTextFieldCell=[[ICProjectSettingsAdvancedOptionsTextFieldCell alloc] initTextCell:@""];
			
	if (tTextFieldCell!=nil)
	{
		[tTextFieldCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
		
		[tTextFieldCell setBordered:NO];
		
		[tTextFieldCell setEnabled:YES];
		
		[tTextFieldCell setEditable:YES];
		
		[tTextFieldCell setSelectable:YES];
		
		[tTextFieldCell setDrawsBackground:NO];
	
		[tTableColumn setDataCell:tTextFieldCell];
    
		[tTextFieldCell release];

	}
	
	/*tPrototypeTextCell=[tTableColumn dataCell];
	
	if (tPrototypeTextCell!=nil)
	{
		[tPrototypeTextCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	}*/
		
	// A COMPLETER
	
	advancedOptionsDescription_=[[NSDictionary alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"AdvancedOptions" ofType:@"plist"]];
	
	if (advancedOptionsDescription_!=nil)
	{
		cachedOptionsOrder_=[advancedOptionsDescription_ objectForKey:@"OPTIONS_ORDER"];
		
		cachedOptionsDescription_=[advancedOptionsDescription_ objectForKey:@"OPTIONS_DESCRIPTION"];
		
		// A COMPLETER
	}
	
	cachedBuildSectionInitialHeight_=NSHeight([IBbuildSection_ frame]);
	
	
	
	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(viewFrameDidChange:) 
														 name:NSViewFrameDidChangeNotification 
													  object:IBview_];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(advancedModeDidChange:) 
												 name:ICPREFERENCES_NOTIFICATION_ADVANCED_ADVANCED_MODE_DID_CHANGE
											   object:nil];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[tree_ release];
	
	tree_=nil;
	
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS];
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
		NSDictionary * tBuildPathDictionary;
		BOOL tState;
		NSString * tName;
		NSMenuItem * tMenuItem;
		
		[dictionary_ retain];
		
		// Name
		
		tName=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_NAME];
		
		if (tName!=nil)
		{
			[IBbuildName_ setStringValue:tName];
		}
		else
		{
			[IBbuildName_ setStringValue:@""];
		}
		
		// Build Path
        
		[IBbuildPath_ setDocument:inDocument];
		
        tBuildPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_PATH];
        
		if (tBuildPathDictionary!=nil)
		{
			NSString * tPath;
			NSInteger tType;
		
			tPath=[tBuildPathDictionary objectForKey:ICDOCUMENT_PATH];
			
			tNumber=[tBuildPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
        
			tType=[tNumber intValue];
				
			if (tType==0)
			{
				tType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			}

			[IBbuildPath_ _setPathType:tType];
			
			if (tPath!=nil)
			{
				[IBbuildPath_ setStringValue:tPath];
			}
			else
			{
				[IBbuildPath_ setStringValue:@""];
			}
		}
		else
		{
			[IBbuildPath_ _setPathType:ICDOCUMENT_PATH_TYPE_GLOBAL];
			
			[IBbuildPath_ setStringValue:@""];
		}
		
		// Reference Folder
		
		tMenuItem=[IBreferenceFolderPopupButton_ itemAtIndex:0];
		
		if (tMenuItem!=nil)
		{
			NSString * tReferenceFolderPath;
			NSImage * tImage=nil;
			
			tReferenceFolderPath=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH];
			
			if (tReferenceFolderPath==nil)
			{
				[tMenuItem setTitle:NSLocalizedStringFromTable(@"Project Folder",@"Project",@"")];
				
				[IBreferenceFolderPopupButton_ setToolTip:nil];
				
				tImage=[[NSWorkspace sharedWorkspace] iconForFile:[document_ folder]];
			}
			else
			{
				if ([tReferenceFolderPath isEqualToString:@"/"]==NO)
				{
					[tMenuItem setTitle:[tReferenceFolderPath lastPathComponent]];
				}
				else
				{
					[tMenuItem setTitle:@"/"];
				}
				
				tImage=[[NSWorkspace sharedWorkspace] iconForFile:tReferenceFolderPath];
				
				[IBreferenceFolderPopupButton_ setToolTip:tReferenceFolderPath];
			}
			
			if (tImage!=nil)
			{
				[tImage setScalesWhenResized:YES];
				
				[tImage setSize:NSMakeSize(16.0f,16.0)];
				
				[tMenuItem setImage:tImage];
			}
		}
		
		// Certificate
		
		cachedCertificateDictionary_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE];
		
		// Build Format
		
		cachedBuildFormat_=ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_FLAT;
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT];
		
		if (tNumber!=nil)
		{
			cachedBuildFormat_=[tNumber intValue];
		}
		
		[self setHiddenBuildFormatWarningLabel:(cachedBuildFormat_==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_BUNDLE)];
		
		[IBbuildFormatPopupButton_ selectItemWithTag:cachedBuildFormat_];
		
		
		[self performSelector:@selector(delayedShowHideCertificate:) withObject:nil afterDelay:0.1f];
		
		// Excluded Files
		
		cachedExcludedFilesArray_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES];
		
		[IBexclusionArray_ reloadData];
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PAYLOAD_ONLY];
		
		if (tNumber!=nil && [tNumber boolValue]==YES)
		{
			[IBexclusionOnlyPayloadCheckbox_ setState:NSOnState];
		}
		else
		{
			[IBexclusionOnlyPayloadCheckbox_ setState:NSOffState];
		}
		
		// Advanced Options
		
		tState=[defaults_ boolForKey:ICPREFERENCES_ADVANCED_ADVANCED_MODE_STATE];
		
		tree_=[ICProjectSettingsAdvancedOptionsTreeNode advancedOptionsTreeNodeWithArray:cachedOptionsOrder_];
		
		if (tree_!=nil)
		{
			[tree_ retain];
		}
		
		cachedAdvancedOptions_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_ADVANCED_OPTIONS];
		
		if (cachedAdvancedOptions_==nil)
		{
			cachedAdvancedOptions_=[NSMutableDictionary dictionary];
			
			if (cachedAdvancedOptions_!=nil)
			{
				[dictionary_ setObject:cachedAdvancedOptions_ forKey:ICDOCUMENT_PROJECT_SETTINGS_ADVANCED_OPTIONS];
			}
		}
		
		[self setHiddenAdvancedOptions:(tState==NO)];
	
		if (tState==YES)
		{
			[self viewFrameDidChange:nil];
		}
	}
}

#pragma mark -

- (void) viewWillBeRemoved:(id) sender
{
	[self showCertificateChildWindow:NO];
}

#pragma mark -

- (void) expandItem:(ICProjectSettingsAdvancedOptionsTreeNode *) inOptionsTree ifKeyOfDictionary:(NSMutableDictionary *) inDictionary
{
	if (inOptionsTree!=nil && [inDictionary count]>0)
	{
		if ([inOptionsTree isLeaf]==NO)
		{
			ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
			NSArray * tChildren;
			NSEnumerator * tEnumerator;
			NSNumber * tNumber;
			NSString * tKey;
			
			tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(inOptionsTree);
		
			tKey=[tNodeData ID];
			
			if (tKey!=nil)
			{
				[IBadvancedOptionsOutlineView_ expandItem:inOptionsTree];
				
				// Check children
				
				tChildren=[inOptionsTree children];
					
				tEnumerator=[tChildren objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					ICProjectSettingsAdvancedOptionsTreeNode * tOptionsTree;
					
					while (tOptionsTree=[tEnumerator nextObject])
					{
						[self expandItem:tOptionsTree ifKeyOfDictionary:inDictionary];
					}
				}
				
				tNumber=[inDictionary objectForKey:tKey];
				
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					[IBadvancedOptionsOutlineView_ collapseItem:inOptionsTree];
				}
			}
		}
	}
}

- (void) restoreAdvancedOptionsDisclosedStates
{
	NSDictionary * tDictionary;
	
	tDictionary=[document_ documentDefaultForKey:ICPROJECT_SETTINGS_ADVANCED_OPTIONS_DISCLOSED_STATE];
			
	if (tDictionary!=nil)
	{
		NSMutableDictionary * tDictionaryCopy;
		
		tDictionaryCopy=[tDictionary mutableCopy];
		
		if (tDictionaryCopy!=nil)
		{
			[self expandItem:((ICProjectSettingsAdvancedOptionsTreeNode *) [tree_ firstChild]) ifKeyOfDictionary:tDictionaryCopy];
		
			[tDictionaryCopy release];
		}
	}
	else
	{
		[self expandItemIfPossible:tree_];
	}
}

- (void) expandItemIfPossible:(ICProjectSettingsAdvancedOptionsTreeNode *) inItem
{
	if ([inItem isLeaf]==NO)
	{
		NSArray * tChildrenArray;
		
		[IBadvancedOptionsOutlineView_ expandItem:inItem];
		
		tChildrenArray=[inItem children];
		
		if (tChildrenArray!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[tChildrenArray objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				ICProjectSettingsAdvancedOptionsTreeNode * tChildNode;
				
				while (tChildNode=[tEnumerator nextObject])
				{
					[self expandItemIfPossible:tChildNode];
				}
			}
		}
	}
}

#pragma mark -

- (id) outlineView:(NSOutlineView *) inOutlineView child:(NSInteger) inIndex ofItem:(id) inItem
{
    if (inOutlineView==IBadvancedOptionsOutlineView_)
	{
		if (tree_!=nil)
		{
			return [SAFENODE(inItem) childAtIndex:inIndex];
		}
    }
	
    return nil;
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView isItemExpandable:(id) inItem
{
    if (inOutlineView==IBadvancedOptionsOutlineView_)
	{
		return ![inItem isLeaf];
	}
	
	return NO;
}

- (NSInteger) outlineView:(NSOutlineView *) inOutlineView numberOfChildrenOfItem:(id) inItem
{
    if (inOutlineView==IBadvancedOptionsOutlineView_)
	{
		if (tree_!=nil)
		{
			NSInteger tValue;
			
			tValue=[SAFENODE(inItem) numberOfChildren];
			
			return tValue;
		}
    }
	
    return 0;
}

- (id) outlineView:(NSOutlineView *) inOutlineView objectValueForTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
    if (inOutlineView==IBadvancedOptionsOutlineView_)
	{
		if (tree_!=nil && inItem!=nil && cachedOptionsOrder_!=nil && cachedOptionsDescription_!=nil && cachedAdvancedOptions_!=nil)
		{
			ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
			
			tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(inItem);
			
			if (tNodeData!=nil)
			{
				NSString * tColumnIdentifier;
				
				tColumnIdentifier=[inTableColumn identifier];
				
				if (tColumnIdentifier!=nil)
				{
					NSString * tKey;
					NSDictionary * tOptionDescription;
					
					tKey=[tNodeData ID];
					
					tOptionDescription=[cachedOptionsDescription_ objectForKey:tKey];
					
					if ([tColumnIdentifier isEqualToString: @"Key"])
					{
						return [tOptionDescription objectForKey:@"TITLE"];
					}
					else if ([tColumnIdentifier isEqualToString: @"Value"])
					{
						NSString * tType;
						
						tType=[tOptionDescription objectForKey:@"TYPE"];
						
						if ([tType isEqualToString:@"String"]==YES)
						{
							return [cachedAdvancedOptions_ objectForKey:tKey];
						}
						else if ([tType isEqualToString:@"List"]==YES)
						{
							NSArray * tArray;
							
							tArray=[cachedAdvancedOptions_ objectForKey:tKey];
							
							if (tArray!=nil)
							{
								if ([tArray count]>1)
								{
									return [tArray componentsJoinedByString:@" "];
								}
								else
								{
									return [tArray objectAtIndex:0];
								}
							}
						}
						else if ([tType isEqualToString:@"Boolean"]==YES)
						{
							return [cachedAdvancedOptions_ objectForKey:tKey];
						}
						else if ([tType isEqualToString:@"PopupButton"]==YES)
						{
							// A COMPLETER
						}
					}
				}
			}
		}
	}
	
	return nil;
}

- (void) outlineView:(NSOutlineView *) inOutlineView willDisplayCell:(NSCell *) inCell forTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{
	if (inOutlineView==IBadvancedOptionsOutlineView_)
	{
		if (tree_!=nil && inItem!=nil && cachedOptionsOrder_!=nil && cachedOptionsDescription_!=nil && cachedAdvancedOptions_!=nil)
		{
			ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
			
			tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(inItem);
			
			if (tNodeData!=nil)
			{
				NSString * tColumnIdentifier;
				
				tColumnIdentifier=[inTableColumn identifier];
				
				if (tColumnIdentifier!=nil)
				{
					NSString * tKey;
					NSDictionary * tOptionDescription;
					NSString * tType;
					
					tKey=[tNodeData ID];
					
					tOptionDescription=[cachedOptionsDescription_ objectForKey:tKey];
					
					tType=[tOptionDescription objectForKey:@"TYPE"];
					
					if ([tColumnIdentifier isEqualToString: @"Key"])
					{
						if ([tType isEqualToString:@"Header"]==YES)
						{
							[inCell setFont:[NSFont boldSystemFontOfSize:[NSFont smallSystemFontSize]]];
							
							[((NSTextFieldCell *) inCell) setTextColor:[NSColor whiteColor]];
												
							[inCell setAlignment:NSLeftTextAlignment];
						}
						else
						{
							[inCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
							
							[((NSTextFieldCell *) inCell) setTextColor:[NSColor blackColor]];
							
							[inCell setAlignment:NSRightTextAlignment];
						}
					}
					else if ([tColumnIdentifier isEqualToString: @"Value"])
					{
						if ([tType isEqualToString:@"PopupButton"]==YES)
						{
							[((NSPopUpButtonCell *) inCell) selectItemWithTitle:[cachedAdvancedOptions_ objectForKey:tKey]];
						}
					}
				}
			}
		}
	}
}

- (void) outlineView:(NSOutlineView *) inOutlineView setObjectValue:(id) inObject forTableColumn:(NSTableColumn *) inTableColumn byItem:(id) inItem
{
	if (inOutlineView==IBadvancedOptionsOutlineView_ && cachedOptionsOrder_!=nil && cachedOptionsDescription_ && cachedAdvancedOptions_!=nil)
	{
		if (tree_!=nil && inItem!=nil)
		{
			ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
			
			tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(inItem);
			
			if (tNodeData!=nil)
			{
				NSString * tColumnIdentifier;
				
				tColumnIdentifier=[inTableColumn identifier];
				
				if (tColumnIdentifier!=nil)
				{
					NSString * tKey;
					NSDictionary * tOptionDescription;
					
					tKey=[tNodeData ID];
					
					tOptionDescription=[cachedOptionsDescription_ objectForKey:tKey];
					
					if ([tColumnIdentifier isEqualToString: @"Value"])
					{
						NSString * tType;
						
						tType=[tOptionDescription objectForKey:@"TYPE"];
						
						if ([tType isEqualToString:@"String"]==YES)
						{
							[cachedAdvancedOptions_ setObject:inObject forKey:tKey];
							
							// Notify Document Change
					
							[self noteDocumentChange];
						}
						else if ([tType isEqualToString:@"List"]==YES)
						{
							NSArray * tArray;
							
							tArray=[inObject componentsSeparatedByString:@" "];
							
							if (tArray!=nil)
							{
								[cachedAdvancedOptions_ setObject:tArray forKey:tKey];
							}
							else
							{
								[cachedAdvancedOptions_ setObject:[NSArray array] forKey:tKey];
							}
							
							// Notify Document Change
					
							[self noteDocumentChange];
						}
						else if ([tType isEqualToString:@"Boolean"]==YES)
						{
							[cachedAdvancedOptions_ setObject:inObject forKey:tKey];
							
							// Notify Document Change
					
							[self noteDocumentChange];
						}
						else if ([tType isEqualToString:@"PopupButton"]==YES)
						{
							NSString * tTitle, * tOldTitle;
							
							tTitle=[[[inTableColumn dataCellForRow:[IBadvancedOptionsOutlineView_ rowForItem:inItem]] itemAtIndex:[inObject intValue]] title];
							
							tOldTitle=[cachedAdvancedOptions_ objectForKey:tKey];
							
							if ([tTitle isEqualToString:tOldTitle]==NO)
							{
								[cachedAdvancedOptions_ setObject:tTitle forKey:tKey];
								
								// Notify Document Change
					
								[self noteDocumentChange];
							}
						}
					}
				}
			}
		}
	}
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView shouldSelectItem:(id) inItem
{
	if (inOutlineView==IBadvancedOptionsOutlineView_)
	{
		if (tree_!=nil && inItem!=nil && cachedOptionsOrder_!=nil && cachedOptionsDescription_!=nil && cachedAdvancedOptions_!=nil)
		{
			ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
			
			tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(inItem);
			
			if (tNodeData!=nil)
			{
				NSString * tKey;
				NSDictionary * tOptionDescription;
				NSString * tType;
		
				tKey=[tNodeData ID];
		
				tOptionDescription=[cachedOptionsDescription_ objectForKey:tKey];
				
				tType=[tOptionDescription objectForKey:@"TYPE"];
				
				if ([tType isEqualToString:@"Header"]==YES)
				{
					return NO;
				}
			}
		}
	}
	
	return YES;
}

- (void) outlineView:(NSOutlineView *) inOutlineView willDisplayOutlineCell:(id) inCell forTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{
	if (inOutlineView==IBadvancedOptionsOutlineView_)
	{
		if (tree_!=nil && inItem!=nil && cachedOptionsOrder_!=nil && cachedOptionsDescription_!=nil && cachedAdvancedOptions_!=nil)
		{
			static NSImage * sCloseImage=nil;
			static NSImage * sDiscloseImage=nil;
			NSString * tPath;
			
			if (sCloseImage==nil)
			{
				tPath=[[NSBundle mainBundle] pathForResource:@"olv_close" ofType:@"tif"];
				
				if (tPath!=nil)
				{
					sCloseImage=[[NSImage alloc] initWithContentsOfFile:tPath];
				}
			}
			
			if (sDiscloseImage==nil)
			{
				tPath=[[NSBundle mainBundle] pathForResource:@"olv_disclose" ofType:@"tif"];
				
				if (tPath!=nil)
				{
					sDiscloseImage=[[NSImage alloc] initWithContentsOfFile:tPath];
				}
			}
			
			[inCell setImage:sDiscloseImage];
			
			[inCell setAlternateImage:sCloseImage];
		}
	}
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView shouldEditTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{
	if (inOutlineView==IBadvancedOptionsOutlineView_)
	{
		if (tree_!=nil && inItem!=nil && cachedOptionsOrder_!=nil && cachedOptionsDescription_!=nil && cachedAdvancedOptions_!=nil)
		{
			ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
			
			tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(inItem);
			
			if (tNodeData!=nil)
			{
				NSString * tKey;
				NSDictionary * tOptionDescription;
				NSString * tType;
		
				tKey=[tNodeData ID];
		
				tOptionDescription=[cachedOptionsDescription_ objectForKey:tKey];
				
				tType=[tOptionDescription objectForKey:@"TYPE"];
				
				if ([tType isEqualToString:@"Header"]==YES)
				{
					return NO;
				}
			}
		}
	}
	
	return YES;
}

- (CGFloat) outlineView:(NSOutlineView *) inOutlineView heightOfRowByItem:(id) inItem;
{
	if (inOutlineView==IBadvancedOptionsOutlineView_)
	{
		return 17.0f;
	}
	
	return 17.0f;
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView doubleClickInTableColumn:(NSTableColumn *) inTableColumn item:(id) inItem
{
	if (inOutlineView==IBadvancedOptionsOutlineView_ && cachedOptionsOrder_!=nil && cachedOptionsDescription_ && cachedAdvancedOptions_!=nil)
	{
		if (tree_!=nil && inItem!=nil)
		{
			ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
			
			tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(inItem);
			
			if (tNodeData!=nil)
			{
				NSString * tColumnIdentifier;
				
				tColumnIdentifier=[inTableColumn identifier];
				
				if (tColumnIdentifier!=nil)
				{
					NSString * tKey;
					NSDictionary * tOptionDescription;
					
					tKey=[tNodeData ID];
					
					tOptionDescription=[cachedOptionsDescription_ objectForKey:tKey];
					
					if ([tColumnIdentifier isEqualToString: @"Key"])
					{
						NSNumber * tNumber;
						
						tNumber=[tOptionDescription objectForKey:@"ADVANCED_EDITOR"];
						
						if (tNumber!=nil && [tNumber boolValue]==YES)
						{
							NSString * tType;
						
							tType=[tOptionDescription objectForKey:@"TYPE"];
						
							if ([tType isEqualToString:@"List"]==YES)
							{
								currentAdvancedOptionsEditor_=[ICProjectSettingsAdvancedOptionsListEditor showEditorForWindow:[[self view] window]
																														  key:tKey 
																														value:[cachedAdvancedOptions_ objectForKey:tKey]
																												  description:[tOptionDescription objectForKey:@"EDITOR"]
																													 delegate:self];
							}
							
							return YES;
						}
					}
				}
			}
		}
	}
	
	return NO;
}

- (void) editionDidEndWithResult:(int) inReturnCode andValue:(id) inValue forKey:(NSString *) inKey
{
	if (currentAdvancedOptionsEditor_!=nil)
	{
		if (inReturnCode==NSOKButton)
		{
			if (inValue!=nil)
			{
				[cachedAdvancedOptions_ setObject:inValue forKey:inKey];
			}
			else
			{
				[cachedAdvancedOptions_ removeObjectForKey:inKey];
			}
			
			[IBadvancedOptionsOutlineView_ reloadData];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
	
		[currentAdvancedOptionsEditor_ autorelease];
		
		currentAdvancedOptionsEditor_=nil;
	}
}

#pragma mark -


- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (dictionary_!=nil)
	{
		if (IBexclusionArray_==inTableView)
		{
			return [cachedExcludedFilesArray_ count];
		}
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{    
    if (dictionary_!=nil)
	{
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		if (IBexclusionArray_==inTableView && cachedExcludedFilesArray_!=nil)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSNumber * tNumber;
				
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
			tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
			
			if (tNumber==nil || [tNumber boolValue]==NO)
			{
				if ([tColumnIdentifier isEqualToString: @"State"])
				{
					return [tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_STATE];
				}
				else
				{
					NSArray * tPatternsArray;
					
					tPatternsArray=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY];
					
					if (tPatternsArray!=nil)
					{
						NSUInteger tCount;
						
						tCount=[tPatternsArray count];
						
						if ([tColumnIdentifier isEqualToString: @"RegularExpression"])
						{
							if (tCount==1)
							{
								return [[tPatternsArray objectAtIndex:0] objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_REGULAR_EXPRESSION];
							}
						}
						else if ([tColumnIdentifier isEqualToString: @"Name"])
						{
							NSString * tProxyName;
							
							tProxyName=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROXY_NAME];
							
							if (tProxyName!=nil || tCount==0)
							{
								return tProxyName;
							}
							else
							{
								NSString * tString;
								
								tString=[[tPatternsArray objectAtIndex:0] objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_STRING];
								
								return tString;
							}
						}
					}
				}
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (dictionary_!=nil)
	{
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		if (IBexclusionArray_==inTableView && cachedExcludedFilesArray_!=nil)
		{
			NSMutableDictionary * tExcludedFilesDictionary;
			NSNumber * tNumber;
				
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
			tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
			
			if (tNumber==nil || [tNumber boolValue]==NO)
			{
				if ([tColumnIdentifier isEqualToString: @"State"])
				{
					[tExcludedFilesDictionary setObject:object forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_STATE];
					
					// Notify Document Change
			
					[self noteDocumentChange];
				}
				else
				{
					tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
					
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						NSArray * tPatternsArray;
					
						tPatternsArray=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY];
						
						if (tPatternsArray!=nil)
						{
							NSUInteger tCount;
							NSMutableDictionary * tPatternDictionary;
							
							tCount=[tPatternsArray count];
							
							if (tCount==1)
							{
								tPatternDictionary=[tPatternsArray objectAtIndex:0];
								
								if ([tColumnIdentifier isEqualToString:@"RegularExpression"]==YES)
								{
									[tPatternDictionary setObject:object forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_REGULAR_EXPRESSION];
									
									// Notify Document Change
			
									[self noteDocumentChange];
								}
								else if ([tColumnIdentifier isEqualToString:@"Type"]==YES)
								{
									NSInteger tTag;
									
									tTag=[[[inTableColumn dataCellForRow:inRowIndex] itemAtIndex:[object intValue]] tag];
									
									tNumber=[tPatternDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE];
									
									if (tNumber==nil || tTag!=[tNumber intValue])
									{
										tNumber=[NSNumber numberWithInteger:tTag];
									
										[tPatternDictionary setObject:tNumber forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE];
									
										// Notify Document Change
			
										[self noteDocumentChange];
									}
								}
								else if ([tColumnIdentifier isEqualToString:@"Name"]==YES)
								{
									[tPatternDictionary setObject:object forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_STRING];
									
									// Notify Document Change
			
									[self noteDocumentChange];
								}
							}
						}
					}
				}
			}
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView shouldSelectRow:(NSInteger) inRowIndex
{
	if (inTableView==IBexclusionArray_)
	{
		if (inRowIndex>=0)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSNumber * tNumber;
			
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
			
			tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
		
			if (tNumber!=nil && [tNumber boolValue]==YES)
			{
				return NO;
			}
		}
	}
	
	return YES;
}

- (BOOL) isHeaderAtRow:(NSInteger) inRowIndex
{
	if (inRowIndex>=0)
	{
		ICProjectSettingsAdvancedOptionsTreeNode * tTreeNode;
		
		tTreeNode=(ICProjectSettingsAdvancedOptionsTreeNode *) [IBadvancedOptionsOutlineView_ itemAtRow:inRowIndex];
		
		if (tTreeNode!=nil)
		{
			ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
		
			tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(tTreeNode);
			
			if (tNodeData!=nil)
			{
				NSString * tKey;
				NSDictionary * tOptionDescription;
				NSString * tType;
			
				tKey=[tNodeData ID];
		
				tOptionDescription=[cachedOptionsDescription_ objectForKey:tKey];
				
				tType=[tOptionDescription objectForKey:@"TYPE"];
				
				if ([tType isEqualToString:@"Header"]==YES)
				{
					return YES;
				}
			}
		}
	}
	
	return NO;
}

- (CGFloat) tableView:(NSTableView *)inTableView heightOfRow:(NSInteger) inRowIndex
{
	if (inTableView==IBexclusionArray_)
	{
		NSDictionary * tExcludedFilesDictionary;
		NSNumber * tNumber;
				
		tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
			
		tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
			
		if (tNumber==nil || [tNumber boolValue]==NO)
		{
			return 15.0f;
		}
		else
		{
			return 8.0f;
		}
	}
	
	return 17.0f;
}
	


- (id) tableView:(NSTableView *) inTableView column:(NSTableColumn *) inColumn dataCellForRow:(int) inRowIndex
{
	if (inRowIndex>=0)
	{
		if (inTableView==IBexclusionArray_)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSString * tColumnIdentifier;
			NSNumber * tNumber;
			
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
			
			tColumnIdentifier=[inColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:@"Type"]==YES)
			{
				tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
						
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
						
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						ICOffsetedPopupButtonCell * tPopupButtonCell;
						
						tPopupButtonCell=[[ICExclusionArrayPopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
						
						if (tPopupButtonCell!=nil)
						{
							NSMenu * tMenu;
							
							[tPopupButtonCell setBordered:NO];
							
							[tPopupButtonCell setControlSize:NSSmallControlSize];
							
							[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
							
							tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
		
							if (tMenu!=nil)
							{
								id tMenuItem;
								static NSImage * tFolderImage=nil;
								static NSImage * tFileImage=nil;
								static NSImage * tFileOrFolderImage=nil;
								
								tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Files",@"Project",@"No comment")
																	 action:nil
															  keyEquivalent:@""];
							
								if (tMenuItem!=nil)
								{
									[tMenuItem setTag:ICFILE_TYPE_FILE];
								
									if (tFileImage==nil)
									{
										tFileImage=[[ICIconsUtilities smallIconForType:NSFileTypeForHFSTypeCode(kGenericDocumentIcon)] copy];
									}
									
									if (tFileImage!=nil)
									{
										[tMenuItem setImage:tFileImage];
									}
									
									[tMenu addItem:tMenuItem];
									
									[tMenuItem release];
								}
								
								tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Folders",@"Project",@"No comment")
																	 action:nil
															  keyEquivalent:@""];
							
								if (tMenuItem!=nil)
								{
									[tMenuItem setTag:ICFILE_TYPE_FOLDER];
								
									if (tFolderImage==nil)
									{
										tFolderImage=[[ICIconsUtilities smallIconForType:NSFileTypeForHFSTypeCode(kGenericFolderIcon)] copy];
									}
									
									if (tFolderImage!=nil)
									{
										[tMenuItem setImage:tFolderImage];
									}
									
									[tMenu addItem:tMenuItem];
									
									[tMenuItem release];
								}
								
								tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedStringFromTable(@"Files or Folders",@"Project",@"No comment")
																	 action:nil
															  keyEquivalent:@""];
							
								if (tMenuItem!=nil)
								{
									[tMenuItem setTag:ICFILE_TYPE_FILE_OR_FOLDER];
								
									if (tFileOrFolderImage==nil)
									{
										tFileOrFolderImage=[[NSImage alloc] initWithSize:NSMakeSize(16.0f,16.0f)];
										
										if (tFileOrFolderImage!=nil)
										{
											[tFileOrFolderImage lockFocus];
											
											if (tFileImage!=nil)
											{
												[tFileImage compositeToPoint:NSMakePoint(8.0f,0.0f) fromRect:NSMakeRect(8.0f,0.0f,8.0f,16.0f) operation:NSCompositeSourceOver];
											}
											
											if (tFolderImage!=nil)
											{
												[tFolderImage compositeToPoint:NSMakePoint(0.0f,0.0f) fromRect:NSMakeRect(0.0f,0.0f,8.0f,16.0f) operation:NSCompositeSourceOver];
											}
											
											[[NSColor blueColor] set];
											
											[NSBezierPath strokeLineFromPoint:NSMakePoint(7.5f,0.0f) toPoint:NSMakePoint(7.5f,16.0f)];
											
											[tFileOrFolderImage unlockFocus];
										
											[tFileOrFolderImage setScalesWhenResized:YES];
										}
									}
								
									if (tFileOrFolderImage!=nil)
									{
										[tMenuItem setImage:tFileOrFolderImage];
									}
									
									[tMenu addItem:tMenuItem];
									
									[tMenuItem release];
								}
								
								[tPopupButtonCell setMenu:tMenu];
								
								[tMenu release];
							}
							
							// A COMPLETER
							
							return [tPopupButtonCell autorelease];
						}
					}
				}
				else
				{
					NSCell * tCell;
					
					tCell=[[NSCell alloc] initTextCell:@""];
					
					if (tCell!=nil)
					{
						return [tCell autorelease];
					}
				}
			}
			else if ([tColumnIdentifier isEqualToString:@"Name"]==YES)
			{
				tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
						
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					NSTextFieldCell * tTextFieldCell;
					
					tTextFieldCell=[[ICProjectSettingsAdvancedOptionsTextFieldCell alloc] initTextCell:@""];
					
					if (tTextFieldCell!=nil)
					{
						ICPatternFormatter * tFormatter;

						[tTextFieldCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
						
						tFormatter=[ICPatternFormatter new];
						
						if (tFormatter!=nil)
						{
							[tTextFieldCell setFormatter:tFormatter];
						
							[tFormatter release];
						}
						
						[tTextFieldCell setBordered:NO];
						
						[tTextFieldCell setEnabled:YES];
						
						[tTextFieldCell setEditable:YES];
						
						[tTextFieldCell setSelectable:YES];
						
						[tTextFieldCell setDrawsBackground:NO];
					
						return [tTextFieldCell autorelease];
					}
				}
				else
				{
					ICExclusionArraySeparatorCell * tSeparatorCell;
					
					tSeparatorCell=[[ICExclusionArraySeparatorCell alloc] initTextCell:@""];
					
					if (tSeparatorCell!=nil)
					{
						return [tSeparatorCell autorelease];
					}
				}
			}
		}
		else if (inTableView==IBadvancedOptionsOutlineView_)
		{
			ICProjectSettingsAdvancedOptionsTreeNode * tTreeNode;
			ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
			
			tTreeNode=(ICProjectSettingsAdvancedOptionsTreeNode *) [IBadvancedOptionsOutlineView_ itemAtRow:inRowIndex];
			
			tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(tTreeNode);
				
			if (tNodeData!=nil)
			{
				NSString * tKey;
				NSDictionary * tOptionDescription;
				NSString * tType;
		
				tKey=[tNodeData ID];
			
				tOptionDescription=[cachedOptionsDescription_ objectForKey:tKey];
			
				tType=[tOptionDescription objectForKey:@"TYPE"];
				
				
				if ([tType isEqualToString:@"Header"]==YES)
				{
					NSTextFieldCell * tTextFieldCell;
					
					tTextFieldCell=[[NSTextFieldCell alloc] initTextCell:@""];
					
					if (tTextFieldCell!=nil)
					{
						[tTextFieldCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
						
						[tTextFieldCell setBordered:NO];
						
						[tTextFieldCell setEnabled:NO];
						
						[tTextFieldCell setEditable:NO];
						
						[tTextFieldCell setSelectable:NO];
					
						return [tTextFieldCell autorelease];
					}
				
				}
				else if ([tType isEqualToString:@"Boolean"]==YES)
				{
					ICProjectSettingsAdvancedOptionsCheckBoxCell * tButtonCell;
					
					tButtonCell=[[ICProjectSettingsAdvancedOptionsCheckBoxCell alloc] initTextCell:@""];
					
					if (tButtonCell!=nil)
					{
						[tButtonCell setControlSize:NSMiniControlSize];
						[tButtonCell setEditable: NO];
						[tButtonCell setButtonType: NSSwitchButton];
						[tButtonCell setImagePosition: NSImageOnly];
					
						return [tButtonCell autorelease];
					}
				}
				else if ([tType isEqualToString:@"String"]==YES ||
						 [tType isEqualToString:@"List"]==YES)
				{
					NSTextFieldCell * tTextFieldCell;
					
					tTextFieldCell=[[ICProjectSettingsAdvancedOptionsTextFieldCell alloc] initTextCell:@""];
					
					if (tTextFieldCell!=nil)
					{
						[tTextFieldCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
						
						[tTextFieldCell setBordered:NO];
						
						[tTextFieldCell setEnabled:YES];
						
						[tTextFieldCell setEditable:YES];
						
						[tTextFieldCell setSelectable:YES];
						
						[tTextFieldCell setDrawsBackground:NO];
					
						return [tTextFieldCell autorelease];
					}
				}
				else if ([tType isEqualToString:@"PopupButton"]==YES)
				{
					NSPopUpButtonCell * tPopupButtonCell;
					
					tPopupButtonCell=[[ICOffsetedPopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
					
					if (tPopupButtonCell!=nil)
					{
						NSArray * tItems;
						
						[tPopupButtonCell setBordered:NO];
						
						[tPopupButtonCell setControlSize:NSSmallControlSize];
						
						[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
						
						tItems=[tOptionDescription objectForKey:@"ITEMS"];
						
						if (tItems!=nil)
						{
							NSEnumerator * tEnumerator;
							
							tEnumerator=[tItems objectEnumerator];
							
							if (tEnumerator!=nil)
							{
								NSDictionary * tDictionary;
								
								while (tDictionary=[tEnumerator nextObject])
								{
									NSString * tTitle;
									
									tTitle=[tDictionary objectForKey:@"TITLE"];
									
									if (tTitle!=nil)
									{
										[tPopupButtonCell addItemWithTitle:tTitle];
									}
								}
							}
						}
						
						return [tPopupButtonCell autorelease];
					}
				}
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (dictionary_!=nil)
	{
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		if (IBexclusionArray_==inTableView && cachedExcludedFilesArray_!=nil)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSNumber * tNumber;
			
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
			tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
					
			if (tNumber!=nil && [tNumber boolValue]==YES)
			{
				if ([tColumnIdentifier isEqualToString: @"State"]==YES ||
					[tColumnIdentifier isEqualToString: @"RegularExpression"]==YES )
				{
					[(NSButtonCell *)cell setImagePosition: NSNoImage];
				}
			}
			else
			{
				if ([tColumnIdentifier isEqualToString: @"State"]==YES)
				{
					[(NSButtonCell *)cell setImagePosition: NSImageOnly];
				}
				else if ([tColumnIdentifier isEqualToString: @"Type"]==YES)
				{
					NSArray * tPatternsArray;
					
					tPatternsArray=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY];
					
					if (tPatternsArray!=nil)
					{
						NSUInteger tCount;
						
						tCount=[tPatternsArray count];
						
						if (tCount==1)
						{
							tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
						
							if (tNumber==nil || [tNumber boolValue]==NO)
							{
								[cell selectItemWithTag:[[[tPatternsArray objectAtIndex:0] objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE] intValue]];
							}
						}
					}
				}
				else if ([tColumnIdentifier isEqualToString: @"RegularExpression"]==YES)
				{
					NSArray * tPatternsArray;
					
					tPatternsArray=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY];
					
					if (tPatternsArray!=nil)
					{
						NSUInteger tCount;
						
						tCount=[tPatternsArray count];
						
						if (tCount==1)
						{
							tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
						
							if (tNumber!=nil && [tNumber boolValue]==YES)
							{
								[(NSButtonCell *)cell setImagePosition: NSNoImage];
							}
							else
							{
								[(NSButtonCell *)cell setImagePosition: NSImageOnly];
							}
						}
						else
						{
							[(NSButtonCell *)cell setImagePosition: NSNoImage];
						}
					}
				}
			}
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView shouldEditTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (dictionary_!=nil)
	{
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
	
		if (inTableView==IBexclusionArray_)
		{
			if ([tColumnIdentifier isEqualToString: @"Name"])
			{
				NSDictionary * tExcludedFilesDictionary;
				NSNumber * tNumber;
				
				tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
				tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
			
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
					
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						return YES;
					}
				}
			}
		}
	}
	
	return NO;
}

- (NSString *) tableView:(NSTableView *) inTableView toolTipForCell:(NSCell *)cell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex mouseLocation:(NSPoint)mouseLocation
{
	if (dictionary_!=nil)
	{
		if (inTableView==IBexclusionArray_)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSString * tString;
				
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
			tString=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROXY_TOOLTIP];
			
			if (tString!=nil)
			{
				return NSLocalizedStringFromTable(tString,@"ProjectSettings",@"");
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBexclusionArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeExcludedFile:IBexclusionRemoveButton_];
		}
		else
		{
			[self removeExcludedFilesDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBexclusionArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBexclusionRemoveButton_ isEnabled];
		}
	}
	
	return NO;
}

#pragma mark -

- (void) delayedShowHideCertificate:(id) inObject
{
	if (cachedBuildFormat_==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_FLAT && [cachedCertificateDictionary_ count]>0)
	{
		[self showCertificateChildWindow:YES];
	}
}

- (void) showCertificateChildWindow:(BOOL) inShow
{
	if (inShow==YES)
	{
		if (certificateWindow_==nil)
		{
			NSRect tCertificateFrame;
			NSRect tFrame;
			
			tCertificateFrame=[IBcertificateStamp_ frame];
		
			tFrame=[IBbuildSection_ bounds];
			
			tFrame=NSMakeRect(NSMaxX(tFrame)-NSWidth(tCertificateFrame)+30.0f,NSMaxY(tFrame)-NSHeight(tCertificateFrame)-5.0f,NSWidth(tCertificateFrame),NSHeight(tCertificateFrame));
			
			certificateWindow_=[[ICCertificateWindow alloc] initForView:IBbuildSection_ withFrame:tFrame addSubView:IBcertificateStamp_];
			
			if (certificateWindow_!=nil)
			{
				SecCertificateRef tCertificateRef;
		
				tCertificateRef=[ICCertificatesUtilities copyOfCertificateWithName:[cachedCertificateDictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE_NAME]];
				
				if (tCertificateRef!=NULL)
				{
					[IBcertificateStamp_ setIsMissing:NO];
					
					// Release Memory
				
					CFRelease(tCertificateRef);
				}
				else
				{
					[IBcertificateStamp_ setIsMissing:YES];
				}
				
				[[document_ windowForSheet] addChildWindow:certificateWindow_ ordered:NSWindowAbove];
					
				[certificateWindow_ orderFront:self];
			}
		}
	}
	else
	{
		if (certificateWindow_!=nil)
		{
			[[document_ windowForSheet] removeChildWindow:certificateWindow_];
		
			[certificateWindow_ orderOut:self];
			
			[certificateWindow_ release];
			
			certificateWindow_=nil;
		}
	}
}

- (IBAction) changeCertificate:(id) sender
{
	NSArray * tIdentitiesArray;
	
	tIdentitiesArray=[ICCertificatesUtilities availableIdentities];
	
	if (tIdentitiesArray==nil || [tIdentitiesArray count]==0)
	{
		NSBeginAlertSheet(NSLocalizedStringFromTable(@"No certificates that could be used to sign your distribution was found on this Mac.",@"Project",@"No comment"),
						  NSLocalizedString(@"OK",@"No comment"),
						  nil,
						  nil,
						  [document_ windowForSheet],
						  nil,
						  nil,
						  nil,
						  NULL,
						  NSLocalizedStringFromTable(@"Description Forthcoming",@"Project",@""));
	}
	else
	{
		SFChooseIdentityPanel * tChooseIdentityPanel;
	
		tChooseIdentityPanel=[[SFChooseIdentityPanel new] retain];		// A VERIFIER SUR 10.6
		
		[tChooseIdentityPanel setAlternateButtonTitle:NSLocalizedString(@"Cancel",@"")];
		
		[tChooseIdentityPanel beginSheetForWindow:[document_ windowForSheet]
									modalDelegate:self
								   didEndSelector:@selector(setIdentitySheetDidEnd:returnCode:contextInfo:)
									  contextInfo:tChooseIdentityPanel
									   identities:tIdentitiesArray
										  message:NSLocalizedStringFromTable(@"\nChoose a certificate to be used to sign the distribution.",@"Project",@"")];
	}
}

- (void) setIdentitySheetDidEnd:(NSWindow *) inSheet returnCode:(NSInteger) inReturnCode contextInfo:(void *) contextInfo
{
	SFChooseIdentityPanel * tChooseIdentityPanel;
		
	tChooseIdentityPanel=(SFChooseIdentityPanel *) contextInfo;
	
	if (tChooseIdentityPanel!=nil)
	{
		if (inReturnCode==NSOKButton)
		{
			SecIdentityRef tIdentityRef;
		
			tIdentityRef=[tChooseIdentityPanel identity];
		
			if (tIdentityRef!=NULL)
			{
				SecCertificateRef tCertificateRef;
				OSStatus tStatus;
				
				tStatus=SecIdentityCopyCertificate(tIdentityRef,&tCertificateRef);
				
				if (tStatus==noErr)
				{
					NSString * tCertificateIdentifier;
					NSString * tKeyChainPath;
					
					tKeyChainPath=[[NSString stringWithString:@"~/Library/Keychains/login.keychain"] stringByExpandingTildeInPath];

					SecCertificateInferLabel(tCertificateRef,(CFStringRef *) &tCertificateIdentifier);	// A VERIFIER SUR 10.7
					
					cachedCertificateDictionary_=[NSMutableDictionary dictionaryWithObjectsAndKeys:tCertificateIdentifier,ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE_NAME,
																								   tKeyChainPath,ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE_KEYCHAIN_PATH,
																								   nil];
					
					if (cachedCertificateDictionary_!=nil)
					{
						[dictionary_ setObject:cachedCertificateDictionary_ forKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE];
						
						[IBcertificateStamp_ setIsMissing:NO];
						
						[self showCertificateChildWindow:YES];
						
						// Notify Document Change
			
						[self noteDocumentChange];
					}
					else
					{
						// A COMPLETER
					}
					
					// Release Memory
					
					CFRelease(tCertificateRef);
				}
				else
				{
					// A COMPLETER
				}
			}
		}
		
		[tChooseIdentityPanel release];
	}
}

- (IBAction) viewCertificate:(id) sender
{
	SecCertificateRef tCertificateRef;
		
	tCertificateRef=[ICCertificatesUtilities copyOfCertificateWithName:[cachedCertificateDictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE_NAME]];
	
	if (tCertificateRef!=NULL)
	{
		ICCertificatePanel * tPanel;
		
		tPanel=[ICCertificatePanel certificatePanel];
		
		if (tPanel!=nil)
		{
			[tPanel beginSheetForCertificate:tCertificateRef
							  modalForWindow:[document_ windowForSheet] 
							   modalDelegate:self 
							  didEndSelector:@selector(certificateSheetDidEnd:returnCode:contextInfo:)
								 contextInfo:tCertificateRef];

		}
	}
}

- (void) certificateSheetDidEnd:(NSWindow *) inSheet returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
	SecCertificateRef tCertificateRef;
	
	tCertificateRef=(SecCertificateRef) contextInfo;
	
	// Release Memory
	
	CFRelease(tCertificateRef);
}

- (IBAction) removeCertificate:(id) sender
{
    NSBeginAlertSheet(NSLocalizedStringFromTable(@"Do you really want to remove this certificate?",@"Project",@"No comment"),
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeCertificateSheetDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeCertificateSheetDidEnd:(NSWindow *) sheet returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		[dictionary_ removeObjectForKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE];
		
		cachedCertificateDictionary_=nil;
		
		[self showCertificateChildWindow:NO];
		
		//[IBcertificateStamp_ setCertificate:NULL];	// A COMPLETER
		
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

#pragma mark -

- (IBAction) setName:(id) sender
{
	// A COMPLETER (Ne peut pas etre vide)
	
	[dictionary_ setObject:[IBbuildName_ stringValue] forKey:ICDOCUMENT_PROJECT_SETTINGS_NAME];
	
	// Notify Document Change
			
	[self noteDocumentChange];
}

- (IBAction) updateBuildPath:(id) sender
{
	NSMutableDictionary * tBuildPathDictionary;
		
	tBuildPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_PATH];
        
	if (tBuildPathDictionary!=nil)
	{
		NSInteger tOldType,tNewType;
		NSString * tOldPath,* tNewPath;
		
		tOldPath=[tBuildPathDictionary objectForKey:ICDOCUMENT_PATH];
		
		tNewPath=[IBbuildPath_ stringValue];
		
		tOldType=[[tBuildPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
		
		tNewType=[IBbuildPath_ pathType];
		
		if ([tOldPath isEqualToString:tNewPath]==NO ||
			tOldType!=tNewType)
		{
			[tBuildPathDictionary setObject:tNewPath forKey:ICDOCUMENT_PATH];
		
			[tBuildPathDictionary setObject:[NSNumber numberWithInteger:tNewType] forKey:ICDOCUMENT_PATH_TYPE];
		
			[self noteDocumentChange];
		}
	}
}

- (IBAction) selectBuildPath:(id) sender
{
    NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:NO];
    [tOpenPanel setCanChooseDirectories:YES];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:YES];
    
    [tOpenPanel beginSheetForDirectory:[IBbuildPath_ absolutePath]
                                  file:nil
                                 types:nil
                        modalForWindow:[document_ windowForSheet]
                         modalDelegate:self
                        didEndSelector:@selector(buildOpenPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) buildOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        [IBbuildPath_ setAbsolutePath:[sheet filename]];
        
        [self updateBuildPath:IBbuildPath_];
    }
}

- (IBAction) revealBuildPathInFinder:(id) sender
{
    NSWorkspace * tWorkSpace;
    
    tWorkSpace=[NSWorkspace sharedWorkspace];
    
    [tWorkSpace selectFile:[IBbuildPath_ absolutePath] inFileViewerRootedAtPath:@""];
}

#pragma mark -

- (IBAction) noAction:(id) sender
{
}

- (IBAction) setReferenceFolder:(id) sender
{
	 NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:NO];
    [tOpenPanel setCanChooseDirectories:YES];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:NO];
    
    [tOpenPanel beginSheetForDirectory:[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH]
                                  file:nil
                                 types:nil
                        modalForWindow:[document_ windowForSheet]
                         modalDelegate:self
                        didEndSelector:@selector(referenceFolderOpenPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) referenceFolderOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
		NSString * tFolderPath;
		
		tFolderPath=[sheet filename];
	
        if ([[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH] isEqualToString:tFolderPath]==NO)
		{
			NSMenuItem * tMenuItem;
			
			[dictionary_ setObject:tFolderPath forKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH];
		
			tMenuItem=[IBreferenceFolderPopupButton_ itemAtIndex:0];
			
			if (tMenuItem!=nil)
			{
				NSImage * tImage;
				
				[tMenuItem setTitle:[tFolderPath lastPathComponent]];
				
				tImage=[[NSWorkspace sharedWorkspace] iconForFile:tFolderPath];
				
				if (tImage!=nil)
				{
					[tImage setScalesWhenResized:YES];
					
					[tImage setSize:NSMakeSize(16.0f,16.0)];
					
					[tMenuItem setImage:tImage];
				}
			}
			
			[IBreferenceFolderPopupButton_ setToolTip:tFolderPath];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
    }
	
	[IBreferenceFolderPopupButton_ selectItemAtIndex:0];
}

- (IBAction) resetReferenceFolder:(id) sender
{
	NSMenuItem * tMenuItem;
	
	if ([dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH]!=nil)
	{
		[dictionary_ removeObjectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH];
		
		tMenuItem=[IBreferenceFolderPopupButton_ itemAtIndex:0];
				
		if (tMenuItem!=nil)
		{
			NSImage * tImage;
			
			[tMenuItem setTitle:NSLocalizedString(@"Project Folder",@"")];
			
			tImage=[[NSWorkspace sharedWorkspace] iconForFile:[document_ folder]];
			
			if (tImage!=nil)
			{
				[tImage setScalesWhenResized:YES];
				
				[tImage setSize:NSMakeSize(16.0f,16.0)];
				
				[tMenuItem setImage:tImage];
			}
		}
		
		[IBreferenceFolderPopupButton_ setToolTip:nil];
		
		// Notify Document Change
				
		[self noteDocumentChange];
	}
	
	[IBreferenceFolderPopupButton_ selectItemAtIndex:0];
}

#pragma mark -

- (IBAction) switchBuildFormat:(id) sender
{
	if (dictionary_!=nil)
	{
		cachedBuildFormat_=[[sender selectedItem] tag];

		if (cachedCertificateDictionary_!=nil && [cachedCertificateDictionary_ count]>0)
		{
			[self showCertificateChildWindow:(cachedBuildFormat_==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_FLAT)];
		}
		
		[self setHiddenBuildFormatWarningLabel:(cachedBuildFormat_==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_BUNDLE)];


		[dictionary_ setObject:[NSNumber numberWithInteger:cachedBuildFormat_] forKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

#pragma mark -

- (IBAction) addExcludedFile:(id) sender
{
	NSMutableDictionary * nPatternDictionary;
	
	nPatternDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:NO],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_REGULAR_EXPRESSION,
																		 @"",ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_STRING,
																		 [NSNumber numberWithInt:ICFILE_TYPE_FILE],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE,
																		 nil];
	
	if (nPatternDictionary!=nil)
	{
		NSMutableArray * nPatternsArray;
		
		nPatternsArray=[NSMutableArray arrayWithObject:nPatternDictionary];
		
		if (nPatternsArray!=nil)
		{
			NSMutableDictionary * nExcludedFilesDictionary;
	
			nExcludedFilesDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_STATE,
																						[NSNumber numberWithBool:NO],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED,
																						nPatternsArray,ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY,
																						nil];
																						
			if (nExcludedFilesDictionary!=nil)
			{
				NSUInteger tRowIndex;
				
				[IBexclusionArray_ deselectAll:nil];
			
				[cachedExcludedFilesArray_ addObject:nExcludedFilesDictionary];
				
				[IBexclusionArray_ reloadData];
				
				tRowIndex=[cachedExcludedFilesArray_ count]-1;
				
				[IBexclusionArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
				[IBexclusionArray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
				
				// Notify Document Change
			
				[self noteDocumentChange];
			}
		}
	}
}

- (IBAction) removeExcludedFile:(id) sender
{
	NSString * tAlertTitle;
    
    if ([IBexclusionArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedStringFromTable(@"Do you really want to remove this definition?",@"Project",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedStringFromTable(@"Do you really want to remove these definitions?",@"Project",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeExcludedFilesDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeExcludedFilesDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBexclusionArray_ selectedRowIndexes];
		
		[IBexclusionArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedExcludedFilesArray_ removeObjectsAtIndexes:tIndexSet];
			
			[IBexclusionArray_ reloadData];
			
			[IBexclusionRemoveButton_ setEnabled:NO];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
	}
}

- (IBAction) switchPayloadOnly:(id) sender
{
	BOOL tPayloadOnly;
	
	tPayloadOnly=([sender state]==NSOnState);
	
	[dictionary_ setObject:[NSNumber numberWithBool:tPayloadOnly] forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PAYLOAD_ONLY];
	
	// Notify Document Change
			
	[self noteDocumentChange];
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
    SEL tAction;
	
	tAction=[inMenuItem action];
	
	if (tAction==@selector(changeCertificate:))
    {
		if (cachedBuildFormat_==ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_FLAT)
		{
			if (cachedCertificateDictionary_==nil || [cachedCertificateDictionary_ count]==0)
			{
				[inMenuItem setTitle:NSLocalizedStringFromTable(@"Set Certificate...",@"Project",@"")];
			}
			else
			{
				[inMenuItem setTitle:NSLocalizedStringFromTable(@"Change Certificate...",@"Project",@"")];
			}
			
			return YES;
		}
		
		return NO;
	}
	else if (tAction==@selector(removeCertificate:))
    {
		if (cachedBuildFormat_!=ICDOCUMENT_PROJECT_SETTINGS_BUILD_FORMAT_FLAT || cachedCertificateDictionary_==nil || [cachedCertificateDictionary_ count]==0)
		{
			return NO;
		}
	}
	else if (tAction==@selector(revealBuildPathInFinder:))
    {
        NSString * tPath;
        NSFileManager * tFileManager;
        
        tPath=[IBbuildPath_ absolutePath];
        
        tFileManager=[NSFileManager defaultManager];
        
        return [tFileManager fileExistsAtPath:tPath];
    }
	else if (tAction==@selector(setReferenceFolder:))
	{
		if ([inMenuItem tag]==-1)
		{
			if (([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)==NSAlternateKeyMask)
			{
				[inMenuItem setTitle:NSLocalizedStringFromTable(@"Revert to Default",@"Project",@"")];
				
				[inMenuItem  setAction:@selector(resetReferenceFolder:)];
			}
		}
	}
	else if (tAction==@selector(resetReferenceFolder:))
	{
		if (([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)!=NSAlternateKeyMask)
		{
			[inMenuItem setTitle:NSLocalizedStringFromTable(@"Other...",@"Project",@"")];
			
			[inMenuItem  setAction:@selector(setReferenceFolder:)];
		}
	}

    return YES;
}

#pragma mark -

- (void) setHiddenBuildFormatWarningLabel:(BOOL) inHidden
{
	if (inHidden==NO)
	{
		NSDictionary * tProjectPresentationDictionary;
		
		tProjectPresentationDictionary=[[self document] projectPresentationDictionary];
		
		if (tProjectPresentationDictionary!=nil)
		{
			if ([ICInstallationStepsUtilities projectPresentationHasInstallerPlugins:tProjectPresentationDictionary]==YES)
			{
				[IBbuildFormatWarningLabel_ setHidden:NO];
				
				return;
			}
		}
	}
	
	[IBbuildFormatWarningLabel_ setHidden:YES];
}

- (void) setHiddenAdvancedOptions:(BOOL) inHidden
{
	id tView;
	
	tView=[self view];
	
	if (inHidden==YES && [IBadvancedOptionsSection_ isHidden]==NO)
	{
		// Hide Advanced Options
		
		NSRect tSectionFrame;
		NSRect tViewBounds;
		//float tOptionsSectionHeight;
		
		tViewBounds=[tView bounds];
		
		// Advanced Options Section
		
		[IBadvancedOptionsSection_ setHidden:YES];
		
		// Options Section
		
		/*tSectionFrame=[IBbuildSection_ frame];
		
		tSectionFrame.origin.y=0.0f;
		
		tOptionsSectionHeight=NSHeight(tSectionFrame);
		
		[IBbuildSection_ setFrame:tSectionFrame];
		
		[IBbuildSection_ setAutoresizingMask:NSViewWidthSizable|NSViewMaxYMargin];*/
		
		// Build Section
		
		tSectionFrame=[IBbuildSection_ frame];
		
		tSectionFrame.origin.y=0;
		
		tSectionFrame.size.height=NSHeight(tViewBounds)/*-tOptionsSectionHeight*/;
		
		[IBbuildSection_ setFrame:tSectionFrame];
		
		[IBexclusionArray_ setNextKeyView:IBbuildPath_];
		
		[IBbuildSection_ setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
		
		
		[tView setNeedsDisplay:YES];
	}
	else if (inHidden==NO)
	{
		if ([IBadvancedOptionsSection_ isHidden]==YES)
		{
			// Show Advanced Options
			
			[IBexclusionArray_ setNextKeyView:IBadvancedOptionsOutlineView_];
			
			[IBbuildSection_ setAutoresizingMask:NSViewWidthSizable|NSViewMinYMargin];
			
			
			[IBadvancedOptionsSection_ setHidden:NO];
			
			[IBadvancedOptionsOutlineView_ setNextKeyView:IBbuildPath_];
			
			[self viewFrameDidChange:nil];
			
			[tView setNeedsDisplay:YES];
		}
		
		[IBadvancedOptionsOutlineView_ reloadData];
		
		[self restoreAdvancedOptionsDisclosedStates];
	}
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    NSInteger tCount;
    
    if ([inNotification object]==IBexclusionArray_)
    {
		tCount=[IBexclusionArray_ numberOfSelectedRows];
		
		if (tCount==0)
		{
			[IBexclusionRemoveButton_ setEnabled:NO];
		}
		else
		{
			NSIndexSet * tIndexSet;
			
			tIndexSet=[IBexclusionArray_ selectedRowIndexes];
			
			if (tIndexSet!=nil)
			{
				NSUInteger tRowIndex;
				
				tRowIndex=[tIndexSet firstIndex];
				
				while (tRowIndex!=NSNotFound)
				{
					NSNumber * tNumber;
					NSDictionary * tExcludedFilesDictionary;
					
					tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:tRowIndex];
					
					tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
					
					if (tNumber!=nil && [tNumber boolValue]==YES)
					{
						[IBexclusionRemoveButton_ setEnabled:NO];
						
						return;
					}
					
					tRowIndex=[tIndexSet indexGreaterThanIndex:tRowIndex];
				}
				
				[IBexclusionRemoveButton_ setEnabled:YES];
			}
		}
	}
}

- (void) advancedModeDidChange:(NSNotification *) inNotification
{
	BOOL tState;
	
	tState=[defaults_ boolForKey:ICPREFERENCES_ADVANCED_ADVANCED_MODE_STATE];
	
	[self setHiddenAdvancedOptions:(tState==NO)];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSViewFrameDidChangeNotification object:IBview_];
					
	if (tState==YES)
	{
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(viewFrameDidChange:) 
													 name:NSViewFrameDidChangeNotification 
												  object:IBview_];
	}
}

- (void) outlineViewItemDidExpand:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			ICProjectSettingsAdvancedOptionsTreeNode * tOptionsNode;
			
			tOptionsNode=(ICProjectSettingsAdvancedOptionsTreeNode *) [tUserInfo objectForKey:@"NSObject"];
			
			if (tOptionsNode!=nil)
			{
				ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
				NSString * tKey;
				
				tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(tOptionsNode);
		
				tKey=[tNodeData ID];
					
				if (tKey!=nil)
				{
					NSMutableDictionary * tDisclosedDictionary;
			
					tDisclosedDictionary=[document_ documentDefaultForKey:ICPROJECT_SETTINGS_ADVANCED_OPTIONS_DISCLOSED_STATE];
					
					if (tDisclosedDictionary==nil)
					{
						tDisclosedDictionary=[NSMutableDictionary dictionary];
						
						if (tDisclosedDictionary!=nil)
						{
							[document_ setDocumentDefault:tDisclosedDictionary forKey:ICPROJECT_SETTINGS_ADVANCED_OPTIONS_DISCLOSED_STATE];
						}
						else
						{
							NSLog(@"[ICPackageFilesController outlineViewItemDidExpand:] Low memory");
						}
					}
					
					if (tDisclosedDictionary!=nil)
					{
						[tDisclosedDictionary setObject:[NSNumber numberWithBool:YES] forKey:tKey];
					}
				}
			}
		}
	}
}

- (void) outlineViewItemWillCollapse:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tUserInfo;
		
		tUserInfo=[inNotification userInfo];
		
		if (tUserInfo!=nil)
		{
			ICProjectSettingsAdvancedOptionsTreeNode * tOptionsNode;
			
			tOptionsNode=(ICProjectSettingsAdvancedOptionsTreeNode *) [tUserInfo objectForKey:@"NSObject"];
			
			if (tOptionsNode!=nil)
			{
				ICProjectSettingsAdvancedOptionsNodeData * tNodeData;
				NSString * tKey;
				
				tNodeData=(ICProjectSettingsAdvancedOptionsNodeData *) ADVANCED_OPTIONS_NODE_DATA(tOptionsNode);
		
				tKey=[tNodeData ID];
					
				if (tKey!=nil)
				{
					NSMutableDictionary * tDisclosedDictionary;
					
					tDisclosedDictionary=[document_ documentDefaultForKey:ICPROJECT_SETTINGS_ADVANCED_OPTIONS_DISCLOSED_STATE];
					
					if (tDisclosedDictionary!=nil)
					{
						// Check if the option key is down or not
						
						NSEvent * tCurrentEvent;
						
						tCurrentEvent=[NSApp currentEvent];
						
						if (tCurrentEvent==nil || (([tCurrentEvent modifierFlags] & NSAlternateKeyMask)==0))
						{
							if ([tKey isEqualToString:@"installer-script"]==NO)
							{
								// Check the parents state
							
								NSString * tParentKey;
								NSNumber * tNumber;
								
								tParentKey=[NSString stringWithString:tKey];
							
								do
								{
									tParentKey=[tParentKey stringByDeletingPathExtension];
									
									tNumber=[tDisclosedDictionary objectForKey:tParentKey];
									
									if (tNumber==nil || [tNumber boolValue]==NO)
									{
										// Parent is hidden
									
										return;
									}
									
									if ([tParentKey isEqualToString:@"installer-script"]==YES)
									{
										break;
									}
								}
								while (1);
							}
						}
						
						[tDisclosedDictionary removeObjectForKey:tKey];
					}
				}
			}
		}
	}
}

#pragma mark -

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	NSRect tBounds;
	
	tBounds=[IBview_ bounds];
	
	if ([defaults_ boolForKey:ICPREFERENCES_ADVANCED_ADVANCED_MODE_STATE]==YES)
	{
		NSUInteger tTotalNumber;
		CGFloat tAvailableHeight;
		CGFloat tRowHeight;
		CGFloat tMaximumAdvancedOptionsHeight;
		NSRect tAdvancedOptionsFrame;
		NSRect tBuildFrame;
	
		tAdvancedOptionsFrame=[IBadvancedOptionsSection_ frame];
		
		tBuildFrame=[IBbuildSection_ frame];
		
		tTotalNumber=([tree_ totalNumberOfNodes]-1);	// A CHANGER : DO NOT CHANGE USUALLY
		
		tAvailableHeight=NSHeight(tBounds)-cachedBuildSectionInitialHeight_;
		
		tRowHeight=[IBadvancedOptionsOutlineView_ rowHeight];
		
		if (tTotalNumber>0)
		{
			NSSize tSize;
		
			tSize=[IBadvancedOptionsOutlineView_ intercellSpacing];
		
			tMaximumAdvancedOptionsHeight=NSHeight(tAdvancedOptionsFrame)-NSHeight([IBadvancedOptionsScrollView_ frame])+tRowHeight*tTotalNumber+(tTotalNumber-1)*tSize.height+4;
		}
		else
		{
			tMaximumAdvancedOptionsHeight=NSHeight(tAdvancedOptionsFrame)-NSHeight([IBadvancedOptionsScrollView_ frame]);
		}
		
		tAdvancedOptionsFrame.origin.y=0.0f;
		
		if (tMaximumAdvancedOptionsHeight<tAvailableHeight)
		{
			tAdvancedOptionsFrame.size.height=tMaximumAdvancedOptionsHeight;
			
			[IBadvancedOptionsSection_ setFrame:tAdvancedOptionsFrame];
			
			
			tBuildFrame.size.height=NSHeight(tBounds)-tAdvancedOptionsFrame.size.height;
		}
		else
		{
			tAdvancedOptionsFrame.size.height=tAvailableHeight;
			
			[IBadvancedOptionsSection_ setFrame:tAdvancedOptionsFrame];
			
			tBuildFrame.size.height=cachedBuildSectionInitialHeight_;
		}
		
		tBuildFrame.origin.y=NSMaxY(tAdvancedOptionsFrame);
		
		[IBbuildSection_ setFrame:tBuildFrame];
	
		if (inNotification==nil)
		{
			[IBview_ setNeedsDisplay:YES];
		}
	}
	
	if (certificateWindow_!=nil)
	{
		NSRect tCertificateFrame;
		NSRect tFrame;
		NSRect tWindowFrame;
		
		tCertificateFrame=[IBcertificateStamp_ frame];
	
		tFrame=[IBbuildSection_ bounds];
		
		tFrame=NSMakeRect(NSMaxX(tFrame)-NSWidth(tCertificateFrame)+30.0f,NSMaxY(tFrame)-NSHeight(tCertificateFrame)-5.0f,NSWidth(tFrame),NSHeight(tFrame));
		
		
		tWindowFrame=[IBbuildSection_ convertRect:tFrame toView:nil];
		
		tWindowFrame.origin=[[IBbuildSection_ window] convertBaseToScreen:tWindowFrame.origin];
		
		[certificateWindow_ setFrameOrigin:tWindowFrame.origin];
	}
}

#pragma mark -

- (void) control:(NSControl *) inControl didFailToValidatePartialString:(NSString *)string errorDescription:(NSString *) inErrorDescription
{
	if ([inErrorDescription isEqualToString:@"Error"]==YES)
	{
		NSBeep();
	}
}

- (void) controlTextDidChange:(NSNotification *) inNotification
{
	NSString * tValue;
        
	tValue=[[[[inNotification userInfo] objectForKey:@"NSFieldEditor"] string] copy];
	
	if (tValue!=nil)
	{
		if ([inNotification object]==IBbuildName_)
		{
			[dictionary_ setObject:tValue forKey:ICDOCUMENT_PROJECT_SETTINGS_NAME];
		}
		
		[tValue release];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

#pragma mark -

- (BOOL) textField:(ICFileTextField *) inTextField shouldAcceptFileAtPath:(NSString *) inPath
{
    if (inTextField==IBbuildPath_)
    {
        NSFileManager * tFileManager=[NSFileManager defaultManager];
        BOOL isDirectory;
    
        return ([tFileManager fileExistsAtPath:inPath isDirectory:&isDirectory]==YES && isDirectory==YES);
    }
    
    return NO;
}

- (BOOL) textField:(ICFileTextField *) inTextField didAcceptFileAtPath:(NSString *) inPath
{
    
    
    [(ICReferencedFileTextField *) inTextField setAbsolutePath:inPath];
    
    if (inTextField==IBbuildPath_)
    {
        [self updateBuildPath:IBbuildPath_];
    }
    
    return YES;
}

@end
