/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectSettingsController_Package.h"

#import "ICOffsetedPopupButtonCell.h"

#import "ICExclusionArraySeparatorCell.h"
#import "ICExclusionArrayPopupButtonCell.h"

#import "ICReferencedFileTextField.h"

#import "ICPreferences+Constants.h"

#import "ICInstallationStepsUtilities.h"

#import "ICIconsUtilities.h"

#import "ICCertificatesUtilities.h"

#import "ICDocument.h"

#import "ICProjectSettingsAdvancedOptionsTextFieldCell.h"

#import "ICProjectNameFormatter.h"

#import "ICPatternFormatter.h"

#import <SecurityInterface/SFChooseIdentityPanel.h>
#import <SecurityInterface/SFCertificatePanel.h>
#include <Security/Security.h>

#import "ICCertificateWindow.h"

#import "ICCertificateStampView.h"

#import "ICCertificatePanel.h"

@implementation ICProjectSettingsController_Package

+ (ICProjectSettingsController_Package *) projectSettingsController
{
	ICProjectSettingsController_Package * nController=nil;
    
    nController=[ICProjectSettingsController_Package alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"Project_Settings_Package" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"Project_Settings_Package"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) dealloc
{
	
	[super dealloc];
}

#pragma mark -

- (void) awakeFromNib
{
	NSButtonCell * tPrototypeButtonCell = nil;
    NSTextFieldCell * tPrototypeTextCell=nil;
	NSTableColumn * tTableColumn = nil;
	ICProjectNameFormatter * tFormatter;
	
	defaults_=[NSUserDefaults standardUserDefaults];
	
	// Build Name
	
	tFormatter=[ICProjectNameFormatter new];
	
	if (tFormatter!=nil)
	{
		[IBbuildName_ setFormatter:tFormatter];
		
		[tFormatter release];
	}
	
	// Build Path
	
	// Excluded Files TableView
	
	tTableColumn = [IBexclusionArray_ tableColumnWithIdentifier: @"State"];
    
	tPrototypeButtonCell = [[NSButtonCell alloc] initTextCell: @""];
    
	if (tPrototypeButtonCell!=nil)
	{
		[tPrototypeButtonCell setControlSize:NSMiniControlSize];
		[tPrototypeButtonCell setEditable: YES];
		[tPrototypeButtonCell setButtonType: NSSwitchButton];
		[tPrototypeButtonCell setImagePosition: NSImageOnly];
    
		[tTableColumn setDataCell:tPrototypeButtonCell];
    
		[tPrototypeButtonCell release];
	}
	
	tTableColumn = [IBexclusionArray_ tableColumnWithIdentifier: @"Name"];
	
	tPrototypeTextCell=[tTableColumn dataCell];
	
	if (tPrototypeTextCell!=nil)
	{
		ICPatternFormatter * tFormatter;
		
		[tPrototypeTextCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
		
		tFormatter=[ICPatternFormatter new];
		
		if (tFormatter!=nil)
		{
			[tPrototypeTextCell setFormatter:tFormatter];
		
			[tFormatter release];
		}
	}
	
	tTableColumn = [IBexclusionArray_ tableColumnWithIdentifier: @"RegularExpression"];
    
	tPrototypeButtonCell = [[NSButtonCell alloc] initTextCell: @""];
    
	if (tPrototypeButtonCell!=nil)
	{
		[tPrototypeButtonCell setControlSize:NSMiniControlSize];
		[tPrototypeButtonCell setEditable: YES];
		[tPrototypeButtonCell setButtonType: NSSwitchButton];
		[tPrototypeButtonCell setImagePosition: NSImageOnly];
    
		[tTableColumn setDataCell:tPrototypeButtonCell];
    
		[tPrototypeButtonCell release];
	}
	
	cachedBuildSectionInitialHeight_=NSHeight([IBbuildSection_ frame]);
	
	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(viewFrameDidChange:) 
														 name:NSViewFrameDidChangeNotification 
													  object:IBview_];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS];
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
		NSDictionary * tBuildPathDictionary;
		NSString * tName;
		NSMenuItem * tMenuItem;
		
		[dictionary_ retain];
		
		// Name
		
		tName=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_NAME];
		
		if (tName!=nil)
		{
			[IBbuildName_ setStringValue:tName];
		}
		else
		{
			[IBbuildName_ setStringValue:@""];
		}
		
		// Build Path
        
		[IBbuildPath_ setDocument:inDocument];
		
        tBuildPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_PATH];
        
		if (tBuildPathDictionary!=nil)
		{
			NSString * tPath;
			NSInteger tType;
		
			tPath=[tBuildPathDictionary objectForKey:ICDOCUMENT_PATH];
			
			tNumber=[tBuildPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
        
			tType=[tNumber intValue];
				
			if (tType==0)
			{
				tType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			}

			[IBbuildPath_ _setPathType:tType];
			
			if (tPath!=nil)
			{
				[IBbuildPath_ setStringValue:tPath];
			}
			else
			{
				[IBbuildPath_ setStringValue:@""];
			}
		}
		else
		{
			[IBbuildPath_ _setPathType:ICDOCUMENT_PATH_TYPE_GLOBAL];
			
			[IBbuildPath_ setStringValue:@""];
		}
		
		// Reference Folder
		
		tMenuItem=[IBreferenceFolderPopupButton_ itemAtIndex:0];
		
		if (tMenuItem!=nil)
		{
			NSString * tReferenceFolderPath;
			NSImage * tImage=nil;
			
			tReferenceFolderPath=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH];
			
			if (tReferenceFolderPath==nil)
			{
				[tMenuItem setTitle:NSLocalizedString(@"Project Folder",@"")];
				
				[IBreferenceFolderPopupButton_ setToolTip:nil];
				
				tImage=[[NSWorkspace sharedWorkspace] iconForFile:[document_ folder]];
			}
			else
			{
				if ([tReferenceFolderPath isEqualToString:@"/"]==NO)
				{
					[tMenuItem setTitle:[tReferenceFolderPath lastPathComponent]];
				}
				else
				{
					[tMenuItem setTitle:@"/"];
				}
				
				tImage=[[NSWorkspace sharedWorkspace] iconForFile:tReferenceFolderPath];
				
				[IBreferenceFolderPopupButton_ setToolTip:tReferenceFolderPath];
			}
			
			if (tImage!=nil)
			{
				[tImage setScalesWhenResized:YES];
				
				[tImage setSize:NSMakeSize(16.0f,16.0)];
				
				[tMenuItem setImage:tImage];
			}
		}
		
		// Certificate
		
		cachedCertificateDictionary_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE];
		
		[self performSelector:@selector(delayedShowHideCertificate:) withObject:nil afterDelay:0.1f];
		
		// Excluded Files
		
		cachedExcludedFilesArray_=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES];
		
		[IBexclusionArray_ reloadData];
		
		tNumber=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PAYLOAD_ONLY];
		
		if (tNumber!=nil && [tNumber boolValue]==YES)
		{
			[IBexclusionOnlyPayloadCheckbox_ setState:NSOnState];
		}
		else
		{
			[IBexclusionOnlyPayloadCheckbox_ setState:NSOffState];
		}
	}
}

#pragma mark -

- (void) viewWillBeRemoved:(id) sender
{
	[self showCertificateChildWindow:NO];
}

#pragma mark -


- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (dictionary_!=nil)
	{
		if (IBexclusionArray_==inTableView)
		{
			return [cachedExcludedFilesArray_ count];
		}
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{    
    if (dictionary_!=nil)
	{
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		if (IBexclusionArray_==inTableView && cachedExcludedFilesArray_!=nil)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSNumber * tNumber;
				
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
			tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
			
			if (tNumber==nil || [tNumber boolValue]==NO)
			{
				if ([tColumnIdentifier isEqualToString: @"State"])
				{
					return [tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_STATE];
				}
				else
				{
					NSArray * tPatternsArray;
					
					tPatternsArray=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY];
					
					if (tPatternsArray!=nil)
					{
						NSUInteger tCount;
						
						tCount=[tPatternsArray count];
						
						if ([tColumnIdentifier isEqualToString: @"RegularExpression"])
						{
							if (tCount==1)
							{
								return [[tPatternsArray objectAtIndex:0] objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_REGULAR_EXPRESSION];
							}
						}
						else if ([tColumnIdentifier isEqualToString: @"Name"])
						{
							NSString * tProxyName;
							
							tProxyName=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROXY_NAME];
							
							if (tProxyName!=nil || tCount==0)
							{
								return tProxyName;
							}
							else
							{
								NSString * tString;
								
								tString=[[tPatternsArray objectAtIndex:0] objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_STRING];
								
								return tString;
							}
						}
					}
				}
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (dictionary_!=nil)
	{
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		if (IBexclusionArray_==inTableView && cachedExcludedFilesArray_!=nil)
		{
			NSMutableDictionary * tExcludedFilesDictionary;
			NSNumber * tNumber;
				
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
			tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
			
			if (tNumber==nil || [tNumber boolValue]==NO)
			{
				if ([tColumnIdentifier isEqualToString: @"State"])
				{
					[tExcludedFilesDictionary setObject:object forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_STATE];
					
					// Notify Document Change
			
					[self noteDocumentChange];
				}
				else
				{
					tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
					
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						NSArray * tPatternsArray;
					
						tPatternsArray=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY];
						
						if (tPatternsArray!=nil)
						{
							NSUInteger tCount;
							NSMutableDictionary * tPatternDictionary;
							
							tCount=[tPatternsArray count];
							
							if (tCount==1)
							{
								tPatternDictionary=[tPatternsArray objectAtIndex:0];
								
								if ([tColumnIdentifier isEqualToString:@"RegularExpression"]==YES)
								{
									[tPatternDictionary setObject:object forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_REGULAR_EXPRESSION];
									
									// Notify Document Change
			
									[self noteDocumentChange];
								}
								else if ([tColumnIdentifier isEqualToString:@"Type"]==YES)
								{
									NSInteger tTag;
									
									tTag=[[[inTableColumn dataCellForRow:inRowIndex] itemAtIndex:[object intValue]] tag];
									
									tNumber=[tPatternDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE];
									
									if (tNumber==nil || tTag!=[tNumber integerValue])
									{
										tNumber=[NSNumber numberWithInteger:tTag];
									
										[tPatternDictionary setObject:tNumber forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE];
									
										// Notify Document Change
			
										[self noteDocumentChange];
									}
								}
								else if ([tColumnIdentifier isEqualToString:@"Name"]==YES)
								{
									[tPatternDictionary setObject:object forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_STRING];
									
									// Notify Document Change
			
									[self noteDocumentChange];
								}
							}
						}
					}
				}
			}
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView shouldSelectRow:(NSInteger) inRowIndex
{
	if (inTableView==IBexclusionArray_)
	{
		if (inRowIndex>=0)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSNumber * tNumber;
			
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
			
			tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
		
			if (tNumber!=nil && [tNumber boolValue]==YES)
			{
				return NO;
			}
		}
	}
	
	return YES;
}

- (CGFloat) tableView:(NSTableView *)inTableView heightOfRow:(NSInteger) inRowIndex
{
	if (inTableView==IBexclusionArray_)
	{
		NSDictionary * tExcludedFilesDictionary;
		NSNumber * tNumber;
				
		tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
			
		tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
			
		if (tNumber==nil || [tNumber boolValue]==NO)
		{
			return 15.0f;
		}
		else
		{
			return 8.0f;
		}
	}
	
	return 17.0f;
}
	


- (id) tableView:(NSTableView *) inTableView column:(NSTableColumn *) inColumn dataCellForRow:(int) inRowIndex
{
	if (inRowIndex>=0)
	{
		if (inTableView==IBexclusionArray_)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSString * tColumnIdentifier;
			NSNumber * tNumber;
			
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
			
			tColumnIdentifier=[inColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:@"Type"]==YES)
			{
				tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
						
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
						
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						ICOffsetedPopupButtonCell * tPopupButtonCell;
						
						tPopupButtonCell=[[ICExclusionArrayPopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
						
						if (tPopupButtonCell!=nil)
						{
							NSMenu * tMenu;
							
							[tPopupButtonCell setBordered:NO];
							
							[tPopupButtonCell setControlSize:NSSmallControlSize];
							
							[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
							
							tMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
		
							if (tMenu!=nil)
							{
								id tMenuItem;
								static NSImage * tFolderImage=nil;
								static NSImage * tFileImage=nil;
								static NSImage * tFileOrFolderImage=nil;
								
								tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Files",@"No comment")
																	 action:nil
															  keyEquivalent:@""];
							
								if (tMenuItem!=nil)
								{
									[tMenuItem setTag:ICFILE_TYPE_FILE];
								
									if (tFileImage==nil)
									{
										tFileImage=[[ICIconsUtilities smallIconForType:NSFileTypeForHFSTypeCode(kGenericDocumentIcon)] copy];
									}
									
									if (tFileImage!=nil)
									{
										[tMenuItem setImage:tFileImage];
									}
									
									[tMenu addItem:tMenuItem];
									
									[tMenuItem release];
								}
								
								tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Folders",@"No comment")
																	 action:nil
															  keyEquivalent:@""];
							
								if (tMenuItem!=nil)
								{
									[tMenuItem setTag:ICFILE_TYPE_FOLDER];
								
									if (tFolderImage==nil)
									{
										tFolderImage=[[ICIconsUtilities smallIconForType:NSFileTypeForHFSTypeCode(kGenericFolderIcon)] copy];
									}
									
									if (tFolderImage!=nil)
									{
										[tMenuItem setImage:tFolderImage];
									}
									
									[tMenu addItem:tMenuItem];
									
									[tMenuItem release];
								}
								
								tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Files or Folders",@"No comment")
																	 action:nil
															  keyEquivalent:@""];
							
								if (tMenuItem!=nil)
								{
									[tMenuItem setTag:ICFILE_TYPE_FILE_OR_FOLDER];
								
									if (tFileOrFolderImage==nil)
									{
										tFileOrFolderImage=[[NSImage alloc] initWithSize:NSMakeSize(16.0f,16.0f)];
										
										if (tFileOrFolderImage!=nil)
										{
											[tFileOrFolderImage lockFocus];
											
											if (tFileImage!=nil)
											{
												[tFileImage compositeToPoint:NSMakePoint(8.0f,0.0f) fromRect:NSMakeRect(8.0f,0.0f,8.0f,16.0f) operation:NSCompositeSourceOver];
											}
											
											if (tFolderImage!=nil)
											{
												[tFolderImage compositeToPoint:NSMakePoint(0.0f,0.0f) fromRect:NSMakeRect(0.0f,0.0f,8.0f,16.0f) operation:NSCompositeSourceOver];
											}
											
											[[NSColor blueColor] set];
											
											[NSBezierPath strokeLineFromPoint:NSMakePoint(7.5f,0.0f) toPoint:NSMakePoint(7.5f,16.0f)];
											
											[tFileOrFolderImage unlockFocus];
										
											[tFileOrFolderImage setScalesWhenResized:YES];
										}
									}
								
									if (tFileOrFolderImage!=nil)
									{
										[tMenuItem setImage:tFileOrFolderImage];
									}
									
									[tMenu addItem:tMenuItem];
									
									[tMenuItem release];
								}
								
								[tPopupButtonCell setMenu:tMenu];
								
								[tMenu release];
							}
							
							// A COMPLETER
							
							return [tPopupButtonCell autorelease];
						}
					}
				}
				else
				{
					NSCell * tCell;
					
					tCell=[[NSCell alloc] initTextCell:@""];
					
					if (tCell!=nil)
					{
						return [tCell autorelease];
					}
				}
			}
			else if ([tColumnIdentifier isEqualToString:@"Name"]==YES)
			{
				tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
						
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					NSTextFieldCell * tTextFieldCell;
					
					tTextFieldCell=[[ICProjectSettingsAdvancedOptionsTextFieldCell alloc] initTextCell:@""];
					
					if (tTextFieldCell!=nil)
					{
						ICPatternFormatter * tFormatter;

						[tTextFieldCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
						
						tFormatter=[ICPatternFormatter new];
						
						if (tFormatter!=nil)
						{
							[tTextFieldCell setFormatter:tFormatter];
						
							[tFormatter release];
						}
						
						[tTextFieldCell setBordered:NO];
						
						[tTextFieldCell setEnabled:YES];
						
						[tTextFieldCell setEditable:YES];
						
						[tTextFieldCell setSelectable:YES];
						
						[tTextFieldCell setDrawsBackground:NO];
					
						return [tTextFieldCell autorelease];
					}
				}
				else
				{
					ICExclusionArraySeparatorCell * tSeparatorCell;
					
					tSeparatorCell=[[ICExclusionArraySeparatorCell alloc] initTextCell:@""];
					
					if (tSeparatorCell!=nil)
					{
						return [tSeparatorCell autorelease];
					}
				}
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (dictionary_!=nil)
	{
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		if (IBexclusionArray_==inTableView && cachedExcludedFilesArray_!=nil)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSNumber * tNumber;
			
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
			tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
					
			if (tNumber!=nil && [tNumber boolValue]==YES)
			{
				if ([tColumnIdentifier isEqualToString: @"State"]==YES ||
					[tColumnIdentifier isEqualToString: @"RegularExpression"]==YES )
				{
					[(NSButtonCell *)cell setImagePosition: NSNoImage];
				}
			}
			else
			{
				if ([tColumnIdentifier isEqualToString: @"State"]==YES)
				{
					[(NSButtonCell *)cell setImagePosition: NSImageOnly];
				}
				else if ([tColumnIdentifier isEqualToString: @"Type"]==YES)
				{
					NSArray * tPatternsArray;
					
					tPatternsArray=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY];
					
					if (tPatternsArray!=nil)
					{
						NSUInteger tCount;
						
						tCount=[tPatternsArray count];
						
						if (tCount==1)
						{
							tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
						
							if (tNumber==nil || [tNumber boolValue]==NO)
							{
								[cell selectItemWithTag:[[[tPatternsArray objectAtIndex:0] objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE] intValue]];
							}
						}
					}
				}
				else if ([tColumnIdentifier isEqualToString: @"RegularExpression"]==YES)
				{
					NSArray * tPatternsArray;
					
					tPatternsArray=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY];
					
					if (tPatternsArray!=nil)
					{
						NSUInteger tCount;
						
						tCount=[tPatternsArray count];
						
						if (tCount==1)
						{
							tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
						
							if (tNumber!=nil && [tNumber boolValue]==YES)
							{
								[(NSButtonCell *)cell setImagePosition: NSNoImage];
							}
							else
							{
								[(NSButtonCell *)cell setImagePosition: NSImageOnly];
							}
						}
						else
						{
							[(NSButtonCell *)cell setImagePosition: NSNoImage];
						}
					}
				}
			}
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView shouldEditTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (dictionary_!=nil)
	{
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
	
		if (inTableView==IBexclusionArray_)
		{
			if ([tColumnIdentifier isEqualToString: @"Name"])
			{
				NSDictionary * tExcludedFilesDictionary;
				NSNumber * tNumber;
				
				tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
				tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_SEPARATOR];
			
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
					
					if (tNumber==nil || [tNumber boolValue]==NO)
					{
						return YES;
					}
				}
			}
		}
	}
	
	return NO;
}

- (NSString *) tableView:(NSTableView *) inTableView toolTipForCell:(NSCell *)cell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex mouseLocation:(NSPoint)mouseLocation
{
	if (dictionary_!=nil)
	{
		if (inTableView==IBexclusionArray_)
		{
			NSDictionary * tExcludedFilesDictionary;
			NSString * tString;
				
			tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:inRowIndex];
				
			tString=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROXY_TOOLTIP];
			
			if (tString!=nil)
			{
				return NSLocalizedStringFromTable(tString,@"ProjectSettings",@"");
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBexclusionArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeExcludedFile:IBexclusionRemoveButton_];
		}
		else
		{
			[self removeExcludedFilesDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBexclusionArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBexclusionRemoveButton_ isEnabled];
		}
	}
	
	return NO;
}

#pragma mark -

- (void) delayedShowHideCertificate:(id) inObject
{
	if ([cachedCertificateDictionary_ count]>0)
	{
		[self showCertificateChildWindow:YES];
	}
}

- (void) showCertificateChildWindow:(BOOL) inShow
{
	if (inShow==YES)
	{
		if (certificateWindow_==nil)
		{
			NSRect tCertificateFrame;
			NSRect tFrame;
			
			tCertificateFrame=[IBcertificateStamp_ frame];
		
			tFrame=[IBbuildSection_ bounds];
			
			tFrame=NSMakeRect(NSMaxX(tFrame)-NSWidth(tCertificateFrame)+30.0f,NSMaxY(tFrame)-NSHeight(tCertificateFrame)-5.0f,NSWidth(tCertificateFrame),NSHeight(tCertificateFrame));
			
			certificateWindow_=[[ICCertificateWindow alloc] initForView:IBbuildSection_ withFrame:tFrame addSubView:IBcertificateStamp_];
			
			if (certificateWindow_!=nil)
			{
				SecCertificateRef tCertificateRef;
		
				tCertificateRef=[ICCertificatesUtilities copyOfCertificateWithName:[cachedCertificateDictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE_NAME]];
				
				if (tCertificateRef!=NULL)
				{
					[IBcertificateStamp_ setIsMissing:NO];
					
					// Release Memory
				
					CFRelease(tCertificateRef);
				}
				else
				{
					[IBcertificateStamp_ setIsMissing:YES];
				}
				
				[[document_ windowForSheet] addChildWindow:certificateWindow_ ordered:NSWindowAbove];
					
				[certificateWindow_ orderFront:self];
			}
		}
	}
	else
	{
		if (certificateWindow_!=nil)
		{
			[[document_ windowForSheet] removeChildWindow:certificateWindow_];
		
			[certificateWindow_ orderOut:self];
			
			[certificateWindow_ release];
			
			certificateWindow_=nil;
		}
	}
}

- (IBAction) changeCertificate:(id) sender
{
	NSArray * tIdentitiesArray;
	
	tIdentitiesArray=[ICCertificatesUtilities availableIdentities];
	
	if (tIdentitiesArray==nil || [tIdentitiesArray count]==0)
	{
		NSBeginAlertSheet(NSLocalizedStringFromTable(@"No certificates that could be used to sign your package was found on this Mac.",@"Project",@"No comment"),
						  NSLocalizedString(@"OK",@"No comment"),
						  nil,
						  nil,
						  [document_ windowForSheet],
						  nil,
						  nil,
						  nil,
						  NULL,
						  NSLocalizedStringFromTable(@"Description Forthcoming",@"Project",@""));
	}
	else
	{
		SFChooseIdentityPanel * tChooseIdentityPanel;
	
		tChooseIdentityPanel=[[SFChooseIdentityPanel new] retain];		// A VERIFIER SUR 10.6
		
		[tChooseIdentityPanel setAlternateButtonTitle:NSLocalizedString(@"Cancel",@"")];
		
		[tChooseIdentityPanel beginSheetForWindow:[document_ windowForSheet]
									modalDelegate:self
								   didEndSelector:@selector(setIdentitySheetDidEnd:returnCode:contextInfo:)
									  contextInfo:tChooseIdentityPanel
									   identities:tIdentitiesArray
										  message:NSLocalizedStringFromTable(@"\nChoose a certificate to be used to sign the package.",@"Project",@"")];
	}
}

- (void) setIdentitySheetDidEnd:(NSWindow *) inSheet returnCode:(NSInteger) inReturnCode contextInfo:(void *) contextInfo
{
	if (inReturnCode==NSOKButton)
	{
		SFChooseIdentityPanel * tChooseIdentityPanel;
		
		tChooseIdentityPanel=(SFChooseIdentityPanel *) contextInfo;
		
		if (tChooseIdentityPanel!=nil)
		{
			SecIdentityRef tIdentityRef;
		
			tIdentityRef=[tChooseIdentityPanel identity];
		
			if (tIdentityRef!=NULL)
			{
				SecCertificateRef tCertificateRef;
				OSStatus tStatus;
				
				tStatus=SecIdentityCopyCertificate(tIdentityRef,&tCertificateRef);
				
				if (tStatus==noErr)
				{
					NSString * tCertificateIdentifier;
					NSString * tKeyChainPath;
					
					tKeyChainPath=[[NSString stringWithString:@"~/Library/Keychains/login.keychain"] stringByExpandingTildeInPath];

					SecCertificateInferLabel(tCertificateRef,(CFStringRef *) &tCertificateIdentifier);	// A VERIFIER SUR 10.7
					
					cachedCertificateDictionary_=[NSMutableDictionary dictionaryWithObjectsAndKeys:tCertificateIdentifier,ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE_NAME,
																								   tKeyChainPath,ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE_KEYCHAIN_PATH,
																								   nil];
					
					if (cachedCertificateDictionary_!=nil)
					{
						[dictionary_ setObject:cachedCertificateDictionary_ forKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE];
					
						//[IBcertificateStamp_ setCertificate:tCertificateRef];	// A COMPLETER
						
						[self showCertificateChildWindow:YES];
						
						// Notify Document Change
			
						[self noteDocumentChange];
					}
					else
					{
						// A COMPLETER
					}
					
					// Release Memory
					
					CFRelease(tCertificateRef);
				}
				else
				{
					// A COMPLETER
				}
			}
			
			[tChooseIdentityPanel release];
		}
		
		// A COMPLETER
	}
}

- (IBAction) viewCertificate:(id) sender
{
	SecCertificateRef tCertificateRef;
		
	tCertificateRef=[ICCertificatesUtilities copyOfCertificateWithName:[cachedCertificateDictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE_NAME]];
	
	if (tCertificateRef!=NULL)
	{
		ICCertificatePanel * tPanel;
		
		tPanel=[ICCertificatePanel certificatePanel];
		
		if (tPanel!=nil)
		{
			[tPanel beginSheetForCertificate:tCertificateRef
							  modalForWindow:[document_ windowForSheet] 
							   modalDelegate:self 
							  didEndSelector:@selector(certificateSheetDidEnd:returnCode:contextInfo:)
								 contextInfo:tCertificateRef];

		}
	}
}

- (void) certificateSheetDidEnd:(NSWindow *) inSheet returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
	SecCertificateRef tCertificateRef;
	
	tCertificateRef=(SecCertificateRef) contextInfo;
	
	// Release Memory
	
	CFRelease(tCertificateRef);
}

- (IBAction) removeCertificate:(id) sender
{
    NSBeginAlertSheet(NSLocalizedStringFromTable(@"Do you really want to remove this certificate?",@"Project",@"No comment"),
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeCertificateSheetDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeCertificateSheetDidEnd:(NSWindow *) sheet returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		[dictionary_ removeObjectForKey:ICDOCUMENT_PROJECT_SETTINGS_CERTIFICATE];
		
		cachedCertificateDictionary_=nil;
		
		[self showCertificateChildWindow:NO];
		
		//[IBcertificateStamp_ setCertificate:NULL];	// A COMPLETER
		
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}


#pragma mark -

- (IBAction) setName:(id) sender
{
	NSString * tOldName;
	NSString * tNewName;
	
	
	tOldName=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_NAME];
	
	tNewName=[IBbuildName_ stringValue];
	
	if ([tNewName length]>0)
	{
		if ([tNewName compare:tOldName]!=NSOrderedSame)
		{
			[dictionary_ setObject:[IBbuildName_ stringValue] forKey:ICDOCUMENT_PROJECT_SETTINGS_NAME];
	
			// Notify Document Change
					
			[self noteDocumentChange];
		}
	}
	else
	{
		NSBeep();
		
		if (tOldName!=nil)
		{
			[IBbuildName_ setStringValue:tOldName];
		}
	}
}

- (IBAction) updateBuildPath:(id) sender
{
	NSMutableDictionary * tBuildPathDictionary;
		
	tBuildPathDictionary=[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_PATH];
        
	if (tBuildPathDictionary!=nil)
	{
		NSInteger tOldType,tNewType;
		NSString * tOldPath,* tNewPath;
		
		tOldPath=[tBuildPathDictionary objectForKey:ICDOCUMENT_PATH];
		
		tNewPath=[IBbuildPath_ stringValue];
		
		tOldType=[[tBuildPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
		
		tNewType=[IBbuildPath_ pathType];
		
		if ([tOldPath isEqualToString:tNewPath]==NO ||
			tOldType!=tNewType)
		{
			[tBuildPathDictionary setObject:tNewPath forKey:ICDOCUMENT_PATH];
		
			[tBuildPathDictionary setObject:[NSNumber numberWithInteger:tNewType] forKey:ICDOCUMENT_PATH_TYPE];
		
			[self noteDocumentChange];
		}
	}
}

- (IBAction) selectBuildPath:(id) sender
{
    NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:NO];
    [tOpenPanel setCanChooseDirectories:YES];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:YES];
    
    [tOpenPanel beginSheetForDirectory:[IBbuildPath_ absolutePath]
                                  file:nil
                                 types:nil
                        modalForWindow:[IBbuildPath_ window]
                         modalDelegate:self
                        didEndSelector:@selector(buildOpenPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) buildOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        [IBbuildPath_ setAbsolutePath:[sheet filename]];
        
        [self updateBuildPath:IBbuildPath_];
    }
}

- (IBAction) revealBuildPathInFinder:(id) sender
{
    NSWorkspace * tWorkSpace;
    
    tWorkSpace=[NSWorkspace sharedWorkspace];
    
    [tWorkSpace selectFile:[IBbuildPath_ absolutePath] inFileViewerRootedAtPath:@""];
}

#pragma mark -

- (IBAction) noAction:(id) sender
{
}

- (IBAction) setReferenceFolder:(id) sender
{
	 NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:NO];
    [tOpenPanel setCanChooseDirectories:YES];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:NO];
    
    [tOpenPanel beginSheetForDirectory:[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH]
                                  file:nil
                                 types:nil
                        modalForWindow:[document_ windowForSheet]
                         modalDelegate:self
                        didEndSelector:@selector(referenceFolderOpenPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) referenceFolderOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
		NSString * tFolderPath;
		
		tFolderPath=[sheet filename];
	
        if ([[dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH] isEqualToString:tFolderPath]==NO)
		{
			NSMenuItem * tMenuItem;
			
			[dictionary_ setObject:tFolderPath forKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH];
		
			tMenuItem=[IBreferenceFolderPopupButton_ itemAtIndex:0];
			
			if (tMenuItem!=nil)
			{
				NSImage * tImage;
				
				[tMenuItem setTitle:[tFolderPath lastPathComponent]];
				
				tImage=[[NSWorkspace sharedWorkspace] iconForFile:tFolderPath];
				
				if (tImage!=nil)
				{
					[tImage setScalesWhenResized:YES];
					
					[tImage setSize:NSMakeSize(16.0f,16.0)];
					
					[tMenuItem setImage:tImage];
				}
			}
			
			[IBreferenceFolderPopupButton_ setToolTip:tFolderPath];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
    }
	
	[IBreferenceFolderPopupButton_ selectItemAtIndex:0];
}

- (IBAction) resetReferenceFolder:(id) sender
{
	NSMenuItem * tMenuItem;
	
	if ([dictionary_ objectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH]!=nil)
	{
		[dictionary_ removeObjectForKey:ICDOCUMENT_PROJECT_SETTINGS_REFERENCE_FOLDER_PATH];
		
		tMenuItem=[IBreferenceFolderPopupButton_ itemAtIndex:0];
				
		if (tMenuItem!=nil)
		{
			NSImage * tImage;
			
			[tMenuItem setTitle:NSLocalizedString(@"Project Folder",@"")];
			
			tImage=[[NSWorkspace sharedWorkspace] iconForFile:[document_ folder]];
			
			if (tImage!=nil)
			{
				[tImage setScalesWhenResized:YES];
				
				[tImage setSize:NSMakeSize(16.0f,16.0)];
				
				[tMenuItem setImage:tImage];
			}
		}
		
		[IBreferenceFolderPopupButton_ setToolTip:nil];
		
		// Notify Document Change
				
		[self noteDocumentChange];
	}
	
	[IBreferenceFolderPopupButton_ selectItemAtIndex:0];
}

#pragma mark -

- (IBAction) addExcludedFile:(id) sender
{
	NSMutableDictionary * nPatternDictionary;
	
	nPatternDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:NO],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_REGULAR_EXPRESSION,
																		 @"",ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_STRING,
																		 [NSNumber numberWithInt:ICFILE_TYPE_FILE],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE,
																		 nil];
	
	if (nPatternDictionary!=nil)
	{
		NSMutableArray * nPatternsArray;
		
		nPatternsArray=[NSMutableArray arrayWithObject:nPatternDictionary];
		
		if (nPatternsArray!=nil)
		{
			NSMutableDictionary * nExcludedFilesDictionary;
	
			nExcludedFilesDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_STATE,
																						[NSNumber numberWithBool:NO],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED,
																						nPatternsArray,ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY,
																						nil];
																						
			if (nExcludedFilesDictionary!=nil)
			{
				NSUInteger tRowIndex;
				
				[IBexclusionArray_ deselectAll:nil];
			
				[cachedExcludedFilesArray_ addObject:nExcludedFilesDictionary];
				
				[IBexclusionArray_ reloadData];
				
				tRowIndex=[cachedExcludedFilesArray_ count]-1;
				
				[IBexclusionArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
				[IBexclusionArray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
				
				// Notify Document Change
			
				[self noteDocumentChange];
			}
		}
	}
}

- (IBAction) removeExcludedFile:(id) sender
{
	NSString * tAlertTitle;
    
    if ([IBexclusionArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this definition?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these definitions?",@"No comment");
    }
    
    NSBeginAlertSheet(tAlertTitle,
                      NSLocalizedString(@"Remove",@"No comment"),
                      NSLocalizedString(@"Cancel",@"No comment"),
                      nil,
                      [document_ windowForSheet],
                      self,
                      @selector(removeExcludedFilesDidEnd:returnCode:contextInfo:),
                      nil,
                      NULL,
                      NSLocalizedString(@"This cannot be undone.",@"No comment"));
}

- (void) removeExcludedFilesDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBexclusionArray_ selectedRowIndexes];
		
		[IBexclusionArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedExcludedFilesArray_ removeObjectsAtIndexes:tIndexSet];
			
			[IBexclusionArray_ reloadData];
			
			[IBexclusionRemoveButton_ setEnabled:NO];
			
			// Notify Document Change
			
			[self noteDocumentChange];
		}
	}
}

- (IBAction) switchPayloadOnly:(id) sender
{
	BOOL tPayloadOnly;
	
	tPayloadOnly=([sender state]==NSOnState);
	
	[dictionary_ setObject:[NSNumber numberWithBool:tPayloadOnly] forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PAYLOAD_ONLY];
	
	// Notify Document Change
			
	[self noteDocumentChange];
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
    SEL tAction;
	
	tAction=[inMenuItem action];
	
	if (tAction==@selector(changeCertificate:))
    {
		if (cachedCertificateDictionary_==nil || [cachedCertificateDictionary_ count]==0)
		{
			[inMenuItem setTitle:NSLocalizedStringFromTable(@"Set Certificate...",@"Project",@"")];
		}
		else
		{
			[inMenuItem setTitle:NSLocalizedStringFromTable(@"Change Certificate...",@"Project",@"")];
		}
		
		return YES;
	}
	else if (tAction==@selector(removeCertificate:))
    {
		if (cachedCertificateDictionary_==nil || [cachedCertificateDictionary_ count]==0)
		{
			return NO;
		}
	}
	if (tAction==@selector(revealBuildPathInFinder:))
    {
        NSString * tPath;
        NSFileManager * tFileManager;
        
        tPath=[IBbuildPath_ absolutePath];
        
        tFileManager=[NSFileManager defaultManager];
        
        return [tFileManager fileExistsAtPath:tPath];
    }
	else if (tAction==@selector(setReferenceFolder:))
	{
		if ([inMenuItem tag]==-1)
		{
			if (([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)==NSAlternateKeyMask)
			{
				[inMenuItem setTitle:NSLocalizedString(@"Revert to Default",@"")];
				
				[inMenuItem  setAction:@selector(resetReferenceFolder:)];
			}
		}
	}
	else if (tAction==@selector(resetReferenceFolder:))
	{
		if (([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)!=NSAlternateKeyMask)
		{
			[inMenuItem setTitle:NSLocalizedString(@"Other...",@"")];
			
			[inMenuItem  setAction:@selector(setReferenceFolder:)];
		}
	}

    return YES;
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    NSInteger tCount;
    
    if ([inNotification object]==IBexclusionArray_)
    {
		tCount=[IBexclusionArray_ numberOfSelectedRows];
		
		if (tCount==0)
		{
			[IBexclusionRemoveButton_ setEnabled:NO];
		}
		else
		{
			NSIndexSet * tIndexSet;
			
			tIndexSet=[IBexclusionArray_ selectedRowIndexes];
			
			if (tIndexSet!=nil)
			{
				NSUInteger tRowIndex;
				
				tRowIndex=[tIndexSet firstIndex];
				
				while (tRowIndex!=NSNotFound)
				{
					NSNumber * tNumber;
					NSDictionary * tExcludedFilesDictionary;
					
					tExcludedFilesDictionary=[cachedExcludedFilesArray_ objectAtIndex:tRowIndex];
					
					tNumber=[tExcludedFilesDictionary objectForKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_PROTECTED];
					
					if (tNumber!=nil && [tNumber boolValue]==YES)
					{
						[IBexclusionRemoveButton_ setEnabled:NO];
						
						return;
					}
					
					tRowIndex=[tIndexSet indexGreaterThanIndex:tRowIndex];
				}
				
				[IBexclusionRemoveButton_ setEnabled:YES];
			}
		}
	}
}

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	if (certificateWindow_!=nil)
	{
		NSRect tCertificateFrame;
		NSRect tFrame;
		NSRect tWindowFrame;
		
		tCertificateFrame=[IBcertificateStamp_ frame];
	
		tFrame=[IBbuildSection_ bounds];
		
		tFrame=NSMakeRect(NSMaxX(tFrame)-NSWidth(tCertificateFrame)+30.0f,NSMaxY(tFrame)-NSHeight(tCertificateFrame)-5.0f,NSWidth(tFrame),NSHeight(tFrame));
		
		
		tWindowFrame=[IBbuildSection_ convertRect:tFrame toView:nil];
		
		tWindowFrame.origin=[[IBbuildSection_ window] convertBaseToScreen:tWindowFrame.origin];
		
		[certificateWindow_ setFrameOrigin:tWindowFrame.origin];
	}
}

#pragma mark -

- (void) control:(NSControl *) inControl didFailToValidatePartialString:(NSString *)string errorDescription:(NSString *) inErrorDescription
{
	if ([inErrorDescription isEqualToString:@"Error"]==YES)
	{
		NSBeep();
	}
}

- (void) controlTextDidChange:(NSNotification *) inNotification
{
	NSString * tValue;
        
	tValue=[[[[inNotification userInfo] objectForKey:@"NSFieldEditor"] string] copy];
	
	if (tValue!=nil)
	{
		if ([inNotification object]==IBbuildName_)
		{
			[dictionary_ setObject:tValue forKey:ICDOCUMENT_PROJECT_SETTINGS_NAME];
		}
		
		[tValue release];
	
		// Notify Document Change
			
		[self noteDocumentChange];
	}
}

#pragma mark -

- (BOOL) textField:(ICFileTextField *) inTextField shouldAcceptFileAtPath:(NSString *) inPath
{
    if (inTextField==IBbuildPath_)
    {
        NSFileManager * tFileManager=[NSFileManager defaultManager];
        BOOL isDirectory;
    
        return ([tFileManager fileExistsAtPath:inPath isDirectory:&isDirectory]==YES && isDirectory==YES);
    }
    
    return NO;
}

- (BOOL) textField:(ICFileTextField *) inTextField didAcceptFileAtPath:(NSString *) inPath
{
    [(ICReferencedFileTextField *) inTextField setAbsolutePath:inPath];
    
    if (inTextField==IBbuildPath_)
    {
        [self updateBuildPath:IBbuildPath_];
    }
    
    return YES;
}

@end
