/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICQuickBuildDocument.h"

#include <sys/stat.h>
#include <unistd.h>

#import "ICUUIDFactory.h"

#import "ICApplicationController.h"

#import "ICFileTree.h"

#import "ICPackageFileUtility.h"

#import "ICSmartLocationDetector.h"

#import "ICDispatchBuildUtilities.h"

#import "ICProjectBuilder+Notifications.h"

#import "IBQuickBuildFeedbackController.h"

#import "ICQuickBuildWindow.h"

@implementation ICQuickBuildDocument

- (void) dealloc
{	
	[[NSDistributedNotificationCenter defaultCenter] removeObserver:self];
	
	[temporaryProjectPath_ release];
	
	[_UUID release];
	
	[super dealloc];
}

- (BOOL) readFromURL:(NSURL *) inURL ofType:(NSString *) inType error:(NSError **) outError
{
	NSString * tPath;
	
	tPath=[inURL path];
	
	[NSObject cancelPreviousPerformRequestsWithTarget:NSApp selector:@selector(terminate:) object:nil];
	
	if (tPath!=nil)
	{
		/* We only support bundles */
		
		NSBundle * tBundle;
		
		tBundle=[NSBundle bundleWithPath:tPath];
		
		if (tBundle!=nil)
		{
			NSArray * tPossibleLocations;
		
			tPossibleLocations=[ICSmartLocationDetector guessPotentialInstallationLocationsForFileAtPath:tPath];
		
			if (tPossibleLocations!=nil && [tPossibleLocations count]>1)
			{
				NSDictionary * tInfoDictionary;
				
				// Get the information from the bundle
			
				tInfoDictionary=[tBundle infoDictionary];
				
				if (tInfoDictionary!=nil)
				{
					NSMutableDictionary * tDocumentDictionary;
					
					tDocumentDictionary=[NSMutableDictionary dictionary];
					
					if (tDocumentDictionary!=nil)
					{
						NSMutableDictionary * tProjectDictionary;
						
						// Version
					
						[tDocumentDictionary setObject:[NSNumber numberWithInt:2] forKey:ICDOCUMENT_VERSION];
						
						// Type
						
						[tDocumentDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_TYPE_PACKAGE] forKey:ICDOCUMENT_TYPE];
						
						// PROJECT dictionary
						
						tProjectDictionary=[NSMutableDictionary dictionary];
						
						if (tProjectDictionary!=nil)
						{
							NSMutableDictionary * tProjectSettingsDictionary;
							
							[tDocumentDictionary setObject:tProjectDictionary forKey:ICDOCUMENT_PROJECT];
							
							// PROJECT SETTINGS		ICDOCUMENT_PROJECT_SETTINGS
							
							tProjectSettingsDictionary=[NSMutableDictionary dictionary];
							
							if (tProjectSettingsDictionary!=nil)
							{
								NSFileManager * tFileManager;
								NSMutableDictionary * tPackageSettingsDictionary;
								NSString * tString;
								NSString * tParentFolderPath;
								NSMutableArray * tMutableArray;
								NSArray * tArray;
								NSDictionary * tDictionary;
								
								[tProjectDictionary setObject:tProjectSettingsDictionary forKey:ICDOCUMENT_PROJECT_SETTINGS];
								
								// Name
								
								tString=[tInfoDictionary objectForKey:@"CFBundleName"];
								
								if (tString==nil)
								{
									tString=[tInfoDictionary objectForKey:@"CFBundleExecutable"];
									
									if (tString==nil)
									{
										tString=[[tPath lastPathComponent] stringByDeletingPathExtension];
									}
								}
								
								if (tString==nil)
								{
									NSLog(@"Low Memory");
										
									goto worseBail;
								}
								else
								{
									[tProjectSettingsDictionary setObject:tString forKey:ICDOCUMENT_PROJECT_SETTINGS_NAME];
								}
								
								_UUID=[[ICUUIDFactory string] copy];
				
								[[IBQuickBuildFeedbackController sharedController] addViewForUUID:_UUID fileName:[tString stringByAppendingPathExtension:@"pkg"]];
								
								// Build Path	ICDOCUMENT_PROJECT_SETTINGS_BUILD_PATH
								
								tFileManager=[NSFileManager defaultManager];
								
								tParentFolderPath=[tPath stringByDeletingLastPathComponent];
								
								if ([tFileManager isWritableFileAtPath:tParentFolderPath]==NO)
								{
									tParentFolderPath=NSHomeDirectory();
								}
								
								[tProjectSettingsDictionary setObject:[NSDictionary dictionaryWithObjectsAndKeys:tParentFolderPath,ICDOCUMENT_PATH,
																												 [NSNumber numberWithInt:ICDOCUMENT_PATH_TYPE_GLOBAL],ICDOCUMENT_PATH_TYPE,
																												 nil]
															   forKey:ICDOCUMENT_PROJECT_SETTINGS_BUILD_PATH];
								
								// Excluded files	ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES
								
								tMutableArray=[NSMutableArray array];
								
								if (tMutableArray==nil)
								{
									NSLog(@"Low Memory");
										
									goto lessBail;
								}
								
								[tProjectSettingsDictionary setObject:tMutableArray
															   forKey:ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES];
								
								
								tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:NO],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_REGULAR_EXPRESSION,
																					   @"",ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_STRING,
																					   [NSNumber numberWithInt:ICFILE_TYPE_FILE],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERN_ITEM_FILE_TYPE,
																					   nil];
								
								
								if (tDictionary==nil)
								{
									NSLog(@"Low Memory");
										
									goto lessBail;
								}
								
								tArray=[NSArray arrayWithObject:tDictionary];
								
								if (tArray==nil)
								{
									NSLog(@"Low Memory");
										
									goto lessBail;
								}
								
								tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tArray,ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_ARRAY,
																					   [NSNumber numberWithBool:YES],ICDOCUMENT_PROJECT_SETTINGS_EXCLUDED_FILES_PATTERNS_STATE,
																					   nil];
								
								if (tDictionary==nil)
								{
									NSLog(@"Low Memory");
										
									goto lessBail;
								}
								
								[tMutableArray addObject:tDictionary];
								
								// PACKAGE SETTINGS	ICDOCUMENT_PACKAGE_SETTINGS
							
								tPackageSettingsDictionary=[NSMutableDictionary dictionary];
							
								if (tPackageSettingsDictionary!=nil)
								{
									NSMutableDictionary * tPackageFilesDictionary;
									
									[tProjectDictionary setObject:tPackageSettingsDictionary forKey:ICDOCUMENT_PACKAGE_SETTINGS];
									
									// Identifier	ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER
									
									tString=[tBundle bundleIdentifier];
									
									if (tString!=nil)
									{
										NSMutableArray * tIdentifierComponents;
										
										tIdentifierComponents=[[tString componentsSeparatedByString:@"."] mutableCopy];
										
										if ([tIdentifierComponents count]>0)
										{
											[tIdentifierComponents insertObject:@"qb-pkg" atIndex:[tIdentifierComponents count]-1];
										}
										else
										{
											// A COMPLETER
										}
										
										tString=[tIdentifierComponents componentsJoinedByString:@"."];
										
										[tIdentifierComponents release];
									}
									else
									{
										NSLog(@"Missing bundle identifier");
										
										goto lessBail;
									}
									
									[tPackageSettingsDictionary setObject:tString forKey:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER];
									
									// Version	ICDOCUMENT_PACKAGE_SETTINGS_VERSION
									
									[tPackageSettingsDictionary setObject:@"1.0" forKey:ICDOCUMENT_PACKAGE_SETTINGS_VERSION];
									
									// Authentication	
									
									[tPackageSettingsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_AUTHENTICATION_ROOT] forKey:ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION];
									
									// Relocation	ICDOCUMENT_PACKAGE_SETTINGS_RELOCATABLE
									
									[tPackageSettingsDictionary setObject:[NSNumber numberWithBool:NO] forKey:ICDOCUMENT_PACKAGE_SETTINGS_RELOCATABLE];
									
									// Overwrite permissions	ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS
									
									[tPackageSettingsDictionary setObject:[NSNumber numberWithBool:NO] forKey:ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS];
									
									// Follow Symbolic Links	ICDOCUMENT_PACKAGE_SETTINGS_FOLLOW_SYMBOLIC_LINKS
									
									[tPackageSettingsDictionary setObject:[NSNumber numberWithBool:YES] forKey:ICDOCUMENT_PACKAGE_SETTINGS_FOLLOW_SYMBOLIC_LINKS];
									
									// Conclusion	ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION
									
									[tPackageSettingsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_NONE] forKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
									
									// PACKAGE FILES	ICDOCUMENT_PACKAGE_FILES
									
									tPackageFilesDictionary=[ICPackageFileUtility defaultPackageFileDictionary];
									
									if (tPackageFilesDictionary!=nil)
									{
										ICFileTree * tRootFileTree;
										
										[tProjectDictionary setObject:tPackageFilesDictionary forKey:ICDOCUMENT_PACKAGE_FILES];
										
										tRootFileTree=[ICFileTree fileTreeWithDictionary:[tPackageFilesDictionary objectForKey:ICDOCUMENT_PACKAGE_FILES_FILE_HIERARCHY] projectPath:@"/" referenceFolderPath:@"/"];
										
										if (tRootFileTree!=nil)
										{
											ICFileTree * tTree;
											ICFileTree * tFinalParentTree=nil;
											NSInteger i,tCount;
											
											tCount=[tPossibleLocations count];
											
											for(i=0;i<(tCount-1);i++)
											{
												NSString * tPossibleParentFolder;
												
												tPossibleParentFolder=[tPossibleLocations objectAtIndex:i];
												
												tTree=[((ICFileTree *) [tRootFileTree firstChild]) fileTreeAtPath:tPossibleParentFolder];
												
												if (tTree==nil)
												{
													if ([ICSmartLocationDetector canPathBeCreated:tPossibleParentFolder]==YES)
													{
														// Create Path
														
														tTree=[(ICFileTree *) [tRootFileTree firstChild] createMissingBranchesForPath:tPossibleParentFolder projectPath:@"/" referenceFolderPath:@"/"];
														
														if (tTree!=nil)
														{
															tFinalParentTree=tTree;
														
															break;
														}
													}
												}
												else
												{
													tFinalParentTree=tTree;
													
													break;
												}
											}
											
											if (tFinalParentTree!=nil)
											{
												id tFileNode;
												ICFileNode * tTargetNode=nil;
												struct stat tStat;
												
												tFileNode=[ICFileNode fileNodeWithType:ICDOCUMENT_FILE_REALFILE_NODE
																				  path:tPath
																			  pathType:ICDOCUMENT_PATH_TYPE_GLOBAL];
												
												// Owner and Group
												
						
												tTargetNode=FILENODE_DATA(tFinalParentTree);
												
												[tFileNode setUid:[tTargetNode uid]];
												[tFileNode setGid:[tTargetNode gid]];
												
												if (lstat([tPath fileSystemRepresentation], &tStat)==0)
												{
													ICFileTree * tFileTree;
													
													// Permissions
													
													[tFileNode setPrivileges: (tStat.st_mode & ALLPERMS)];
					
					
													tFileTree=[[ICFileTree alloc] initWithData:tFileNode
																						parent:tFinalParentTree
																					  children:nil];

													if (tFileTree!=nil)
													{
														ICFileTree * tHierarchyTree_;
														
														[tFinalParentTree insertSortedChild:tFileTree];
														
														[tFileTree release];
														
														tHierarchyTree_=(ICFileTree *) [tRootFileTree childAtIndex:0];
														
														if (tHierarchyTree_!=nil)
														{
															tDictionary=[tHierarchyTree_ dictionaryWithProjectAtPath:@"/" referenceFolderPath:@"/"];
															
															if (tDictionary!=nil)
															{
																char tTempDirectoryCPath[250]="/tmp/quickbuild.XXXXX";
																
																[tPackageFilesDictionary setObject:tDictionary forKey:ICDOCUMENT_PACKAGE_FILES_FILE_HIERARCHY];
																
																// Write to Disk to temp location
																
																if (mkdtemp(tTempDirectoryCPath)!=NULL)
																{
																	temporaryProjectPath_=[[NSString stringWithUTF8String:tTempDirectoryCPath] stringByAppendingPathComponent:@"temp_proj.pkgproj"];
																	
																	if ([tDocumentDictionary writeToFile:temporaryProjectPath_ atomically:YES]==YES)
																	{
																		[temporaryProjectPath_ retain];
																		
																		// Register for Builder notifications
																		
																		[[NSDistributedNotificationCenter defaultCenter] addObserver:self
																															selector:@selector(builderNotification:)
																																name:PACKAGES_BUILDER_NOTIFICATION
																															  object:nil
																															  suspensionBehavior:NSNotificationSuspensionBehaviorDeliverImmediately];
																		
																		
																		
																		// Build
																
																		[self performSelector:@selector(delayedBuild:) withObject:temporaryProjectPath_ afterDelay:1.0f];
																	}
																	else
																	{
																		// A COMPLETER
																	}
																	
																}
																else
																{
																	// A COMPLETER
																}
															}
														}
													}
													else
													{
														NSLog(@"Low Memory");
													}
												}
												else
												{
													// A COMPLETER
												}
											}
											else
											{
												// A COMPLETER
											}
										}
										else
										{
											NSLog(@"Low Memory");
										}
									
										//NSLog(@"%@",tDocumentDictionary);
									}
									else
									{
										// A COMPLETER
									}
								}
								else
								{
									NSLog(@"Low Memory");
								}
								
								goto lessBail;
							}
							else
							{
								NSLog(@"Low Memory");
							}
						}
						else
						{
							NSLog(@"Low Memory");
						}
					}
					else
					{
						NSLog(@"Low Memory");
					}
				}
				else
				{
					NSLog(@"Missing or corrupted Info.plist");
				}
			}
		}
	}
	
worseBail:

	NSBeep();

	if ([[NSApp delegate] launchedNormally]==NO)
	{
		if ([[[NSDocumentController sharedDocumentController] documents] count]==1)
		{
			[NSApp performSelector:@selector(terminate:) withObject:nil afterDelay:1.0f];
		}
		
	}
	
lessBail:

	// A COMPLETER

	return YES;
}

#pragma mark -

- (void) builderNotification:(NSNotification *)notification
{
    NSDictionary * tUserInfo;
    static int sProcessID=-1;
    
    if (sProcessID==-1)
    {
        sProcessID=[[NSProcessInfo processInfo] processIdentifier];
    }
    
    tUserInfo=[notification userInfo];
    
    if (tUserInfo!=nil)
    {
        NSNumber * tNumber;
        
        tNumber=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_PID];
        
        if (tNumber!=nil)
        {
            if ([tNumber intValue]==sProcessID)
            {
                NSString * tPath;
                
                tPath=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH];
                
                if (tPath!=nil)
                {
                    if ([tPath isEqualToString:temporaryProjectPath_]==YES)
                    {
                        int tEventInfoID;
						NSDictionary * tExtraData;
						//NSDictionary * tStackMetadata;
						
						tEventInfoID=[[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_ID] intValue];
						
						tExtraData=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA];
						
						//tStackMetadata=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_INFORMATION_STACK_METADATA];
						
						if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_DEBUG)
						{
							NSLog(@"-------------- Debug -----------------");
							
							NSLog(@"%@",tExtraData);
							
							NSLog(@"--------------------------------------");
						}
						else if (tEventInfoID==IC_BUILDER_NOTIFICATION_INFO_EVENT_STANDARD)
						{
							int tStep;
							int tState;
							
							tStep=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP] intValue];

							tState=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STATE] intValue];
							
							if (tState==IC_BUILDER_BUILD_STATE_END_FAILURE)
							{
								NSString * tSoundName;
		
								tSoundName=[[NSUserDefaults standardUserDefaults] stringForKey:ICPREFERENCES_BUILD_PLAY_ERRORS];
									
								if ([tSoundName length]>0)
								{
									[[NSSound soundNamed:tSoundName] play];
								}
								
								[[IBQuickBuildFeedbackController sharedController] setStatus:IC_QUICKBUILD_STATUS_FAILURE forUUID:_UUID];
								
								[[IBQuickBuildFeedbackController sharedController] performSelector:@selector(removeViewForUUID:) withObject:_UUID afterDelay:2.0f];
								
								if ([[NSApp delegate] launchedNormally]==NO)
								{
									if ([[[NSDocumentController sharedDocumentController] documents] count]==1)
									{
										[NSApp performSelector:@selector(terminate:) withObject:nil afterDelay:3.0f];
										
										return;
									}
									
								}
								
								[self close];
							}
							else
							{
								if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
								{
									if (tStep==IC_BUILDER_INFORMATION_STEP_PROJECT)
									{
										NSString * tSoundName;
										
										tSoundName=[[NSUserDefaults standardUserDefaults] stringForKey:ICPREFERENCES_BUILD_PLAY_SUCCESS];
										
										if ([tSoundName length]>0)
										{
											[[NSSound soundNamed:tSoundName] play];
										}
										
										[[IBQuickBuildFeedbackController sharedController] setStatus:IC_QUICKBUILD_STATUS_SUCCESS forUUID:_UUID];
										
										[[IBQuickBuildFeedbackController sharedController] performSelector:@selector(removeViewForUUID:) withObject:_UUID afterDelay:1.0f];
										
										if ([[NSApp delegate] launchedNormally]==NO)
										{
											if ([[[NSDocumentController sharedDocumentController] documents] count]==1)
											{
												[NSApp performSelector:@selector(terminate:) withObject:nil afterDelay:1.0f];
												
												return;
											}
										}
										
										[self close];
									}
								}
							}
						}
					}
				}
			}
		}
	}
}

- (void) delayedBuild:(NSString *) inPath
{
	int tResult;
	NSString * tSoundName;
	
	tResult=[ICDispatchBuildUtilities buildProjectAtPath:inPath withNotificationPath:inPath locationEnabled:YES];
																		
	switch (tResult)
	{
		case -1:
		
			tSoundName=[[NSUserDefaults standardUserDefaults] stringForKey:ICPREFERENCES_BUILD_PLAY_ERRORS];
				
			if ([tSoundName length]>0)
			{
				[[NSSound soundNamed:tSoundName] play];
			}
			
			NSRunAlertPanel(NSLocalizedStringFromTable(@"No signal from the packages_dispatcher process",@"Build",@"No comment"),
							NSLocalizedStringFromTable(@"The packages_dispatcher process is not responding. Packages can't build any project when this process is not running.",@"Build",@"No comment"),
							nil,
							nil,
							nil,
							nil);
			
			break;
			
		case NO:
		
			
		
			break;
	}
}

@end
