/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICReferencedPopupButton.h"
#import "ICDocumentFormat+Constants.h"

#import "ICReferencedUtilities.h"

#import "NSFileManager+fileTypes.h"

@implementation NSObject (ICReferencedPopupButton)

- (BOOL) acceptDrop
{
	return NO;
}

- (NSArray *) acceptedTypesForDrop
{
	return nil;
}

- (BOOL) acceptsDroppedFile:(NSString *) inPath
{
	NSArray * tArray;
	
	tArray=[self acceptedTypesForDrop];
					
	if (tArray!=nil && [[NSFileManager defaultManager] IC_fileAtPath:inPath matchesTypes:tArray]==YES)
	{
		return YES;
	}
	
	return NO;
}

- (void) referencedFileDidChange:(NSString *) inPath
{
}

@end

@implementation ICReferencedPopupButton

+ (Class) cellClass
{
    return [ICReferencedPopupButtonCell class];
}

- (void) dealloc
{
	[referencePopupButtonCell_ release];

	[delegate release];

	[super dealloc];
}

+ (void) setCellClass:(Class) inClass
{
}

#pragma mark -

- (void) setDelegate:(id) inDelegate
{
	if (delegate!=inDelegate)
	{
		[delegate release];
		
		delegate=[inDelegate retain];
		
		if (delegate!=nil && [delegate acceptDrop]==YES)
		{
			// Register for Drop
		
			[self unregisterDraggedTypes];
		
			[self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
		}
	}
}

- (void) setPathDoesNotExist:(BOOL) inBool
{
	[[self cell] setPathDoesNotExist:inBool];
	
	[super setNeedsDisplay:YES];
}

- (void) setEnabled:(BOOL) aBool
{
	[[self cell] setEnabled:aBool];

	[referencePopupButtonCell_ setEnabled:aBool];
}

- (NSInteger) pathType
{
    return [[self cell] pathType];
}

- (void) setPathType:(NSInteger) inPathType
{
	if (inPathType!=[self pathType])
	{
		[[self cell]  setPathType:inPathType];
	
		[referencePopupButtonCell_ selectItemWithTag:inPathType];
	
		[super setNeedsDisplay:YES];
	}
}

- (void) setFileNameWithPath:(NSString *) inFilePath
{
	NSMenuItem * tMenuItem;
	
	tMenuItem=[self itemAtIndex:0];
		
	if (tMenuItem!=nil)
	{
		if (inFilePath!=nil)
		{
			[tMenuItem setTitle:[inFilePath lastPathComponent]];
		}
		else
		{
			[tMenuItem setTitle:@"-"];
		}
	}
}

- (NSMenu *) referenceMenu
{
	return [[referenceMenu_ retain] autorelease];
}

- (void) setReferenceMenu:(NSMenu *) inMenu
{
	if (inMenu!=referenceMenu_)
	{
		[referenceMenu_ release];
		
		referenceMenu_=[inMenu copy];
	}
}

- (void) mouseDown:(NSEvent *) inEvent
{
	NSRect tBounds;
	NSPoint tMouseLoc=[self convertPoint:[inEvent locationInWindow] fromView:nil];
	
	tBounds=[self bounds];
	
	tBounds.size.width=ICREFERENCEDPOPUPBUTTONCELL_LEFT_OFFSET+4.0f;
	
	if (NSMouseInRect(tMouseLoc,tBounds,[self isFlipped])==YES)
	{
		tBounds.origin.x-=6.0f;
		
		[referencePopupButtonCell_ trackMouse:inEvent inRect:tBounds ofView:self untilMouseUp:YES];
	}
	else
	{
		[super mouseDown:inEvent];
	}
}

#pragma mark -

- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>)sender
{
    if (delegate!=nil)
	{
		NSPasteboard * tPasteBoard;
		NSDragOperation sourceDragMask;

		sourceDragMask = [sender draggingSourceOperationMask];
		
		tPasteBoard = [sender draggingPasteboard];

		if ( [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
		{
			if (sourceDragMask & NSDragOperationCopy)
			{
				NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
			
				if (tFiles!=nil && [tFiles count]==1)
				{
					NSString * tFilePath;
					
					tFilePath=[tFiles objectAtIndex:0];
					
					if ([delegate acceptsDroppedFile:tFilePath]==YES)
					{
						highlighted_=YES;
				
						[self setNeedsDisplay:YES];
				
						return NSDragOperationCopy;
					}
				}
			}
        }
    }
    
    return NSDragOperationNone;
}

- (BOOL) wantsPeriodicDraggingUpdates
{
	return NO;
}

- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
{
	return [self draggingEntered:sender];
}

- (BOOL) performDragOperation:(id <NSDraggingInfo>)sender
{
    if (delegate!=nil)
	{
		NSPasteboard * tPasteBoard;
		//NSDragOperation sourceDragMask;

		//sourceDragMask = [sender draggingSourceOperationMask];
		
		tPasteBoard = [sender draggingPasteboard];

		if ( [[tPasteBoard types] containsObject:NSFilenamesPboardType] )
		{
			NSArray * tFiles = [tPasteBoard propertyListForType:NSFilenamesPboardType];
			
			if (tFiles!=nil && [tFiles count]==1)
			{
				NSString * tFilePath;
				//NSArray * tFileTypes;
					
				tFilePath=[tFiles objectAtIndex:0];
					
				/*tFileTypes=[delegate acceptedTypesForDrop];
				
				if (tFileTypes!=nil && [[NSFileManager defaultManager] IC_fileAtPath:tFilePath matchesTypes:tFileTypes]==YES)
				{*/
					[delegate referencedFileDidChange:tFilePath];
					
					return YES;
				/*}*/
			}
		}
	}
	
    return NO;
}

- (BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	highlighted_=NO;
	
    [self setNeedsDisplay:YES];
    
    return YES;
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
    highlighted_=NO;
	
    [self setNeedsDisplay:YES];
}

#pragma mark -

- (IBAction) takeReferenceStyle:(id) sender
{
    if ([self pathType]!=[sender tag])
    {
    	//id tTarget;
		
		//tTarget=[[self itemAtIndex:2] target];
		
		[self setPathType:[sender tag]];
        
        /*if (tTarget!=nil && [tTarget respondsToSelector:@selector(referenceStyleDidChange:)]==YES)
        {
            [tTarget performSelector:@selector(referenceStyleDidChange:) withObject:self];
        }*/
		
		if (delegate!=nil && [delegate respondsToSelector:@selector(referenceStyleDidChange:)]==YES)
        {
            [delegate performSelector:@selector(referenceStyleDidChange:) withObject:self];
        }
    }
}

#pragma mark -

- (id)initWithCoder:(NSCoder *)aDecoder
{
    ICReferencedPopupButtonCell * tCell;
    self=[super initWithCoder:aDecoder];
    
    tCell=[[ICReferencedPopupButtonCell alloc] initTextCell:@""];
    
    if (tCell!=nil)
	{
		NSPopUpButtonCell * tOldCell;
		
		tOldCell=[self cell];
		
		[tCell setControlSize:[tOldCell controlSize]];
    
		[tCell setEnabled:YES];
		
		[tCell setPathType:ICDOCUMENT_PATH_TYPE_GLOBAL];
		
		[tCell setFont:[tOldCell font]];
		
		[tCell setMenu:[tOldCell menu]];
		
		[self setCell:tCell];
		
		[tCell release];
    }
	
	referencePopupButtonCell_=[[NSPopUpButtonCell alloc] initTextCell:@""];
    
    if (referencePopupButtonCell_!=nil)
	{
		[referencePopupButtonCell_ setControlSize:[tCell controlSize]];
		
		[referencePopupButtonCell_ setFont:[tCell font]];
		
		[referencePopupButtonCell_ setMenu:[ICReferencedUtilities referenceStyleMenuForTarget:self]];
		
		[referencePopupButtonCell_ selectItemWithTag:ICDOCUMENT_PATH_TYPE_GLOBAL];
    }
	
	if (delegate!=nil && [delegate acceptDrop]==YES)
	{
		// Register for Drop
	
		[self unregisterDraggedTypes];
	
		[self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
	}
	
    return self;
}

@end
