/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICReferencedPopupButtonCell.h"
#import "ICReferencedUtilities.h"

@implementation ICReferencedPopupButtonCell

- (void) setPathDoesNotExist:(BOOL) inBool
{
	pathDoesNotExist_=inBool;
}

- (NSInteger) pathType
{
    return pathType_;
}

- (void) setPathType:(NSInteger) inPathType
{
	pathType_=inPathType;
}

#pragma mark -

- (void) drawWithFrame:(NSRect) inFrame inView:(NSView *) inControlView
{
	NSMenuItem * tMenuItem;
	NSImage * tReferenceIcon;
	
    [self drawBorderAndBackgroundWithFrame:inFrame inView:inControlView];
	
	// Draw the Path Type icon
	
	tReferenceIcon=[ICReferencedUtilities imageForReferenceStyle:pathType_ controlSize:[self controlSize] popDown:YES];
	
	if (tReferenceIcon!=nil)
	{
		if ([self isEnabled]==NO)
		{
			[tReferenceIcon compositeToPoint:NSMakePoint(NSMinX(inFrame)+2.0f,NSMaxY(inFrame)-3.0f) operation:NSCompositeSourceOver fraction:0.5f];
		}
		else
		{
			[tReferenceIcon compositeToPoint:NSMakePoint(NSMinX(inFrame)+2.0f,NSMaxY(inFrame)-3.0f) operation:NSCompositeSourceOver];
		}
	}
	
	
	// Draw the menu item
	
	tMenuItem=[self selectedItem];
	
	if (tMenuItem!=nil)
	{
		NSImage * tIcon;
		NSRect tRect;
		float tOffset;
		NSDictionary * tAttributes;
		
		tIcon=[tMenuItem image];
		
		tRect=[self stateImageRectForBounds:inFrame];
		
		tRect.origin.x+=ICREFERENCEDPOPUPBUTTONCELL_LEFT_OFFSET;
		
		tOffset=ICREFERENCEDPOPUPBUTTONCELL_LEFT_OFFSET+2.0f;
		
		if (tIcon!=nil)
		{
			NSRect tOriginalRect;
			NSRect tDestinationRect;
			
			tOriginalRect.origin=NSZeroPoint;
			tOriginalRect.size=[tIcon size];
			
			tDestinationRect=NSMakeRect(NSMinX(inFrame)+tOffset,NSMinY(inFrame)+3.0f,15.0f,15.0f);
			
			if ([self isEnabled]==NO)
			{
				[tIcon drawInRect:tDestinationRect fromRect:tOriginalRect operation:NSCompositeSourceOver fraction:0.5f];
			}
			else
			{
				[tIcon drawInRect:tDestinationRect fromRect:tOriginalRect operation:NSCompositeSourceOver fraction:1.0f];
			}
			
			tOffset+=15.0f;
		}
		
		tRect=[self titleRectForBounds:inFrame];
		
		tRect.origin.x+=tOffset;
	
		tRect.size.width-=tOffset;
		
		tAttributes=[NSDictionary dictionaryWithObjectsAndKeys:[self font],NSFontAttributeName,
															   (pathDoesNotExist_==YES) ? [NSColor redColor] : nil,NSForegroundColorAttributeName,
															   nil];
		
		[[tMenuItem title] drawInRect:tRect withAttributes:tAttributes];
	}
}

- (BOOL) trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)flag;
{
	cellFrame.origin.x+=ICREFERENCEDPOPUPBUTTONCELL_LEFT_OFFSET;
	
	cellFrame.size.width-=ICREFERENCEDPOPUPBUTTONCELL_LEFT_OFFSET;

	return [super trackMouse:theEvent inRect:cellFrame ofView:controlView untilMouseUp:flag];
}

@end
