/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICReferencedUtilities.h"

#import "ICDocumentFormat+MinimumConstants.h"

@implementation ICReferencedUtilities

+ (NSImage *) imageForReferenceStyle:(NSInteger) inStyle controlSize:(NSControlSize) inControlSize popDown:(BOOL) inPopDown
{
    if (inPopDown==YES)
	{
		if (inControlSize==NSRegularControlSize)
		{
			static NSImage * sReferenceRegularPopDownImage=nil;
			static NSImage * sRelativeRegularPopDownImage=nil;
			static NSImage * sAbsoluteRegularPopDownImage=nil;
			
			switch(inStyle)
			{
				case ICDOCUMENT_PATH_TYPE_GLOBAL:
				
					if (sAbsoluteRegularPopDownImage==nil)
					{
						sAbsoluteRegularPopDownImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"AbsolutePopDown" ofType:@"tif"]];
					}
					
					return sAbsoluteRegularPopDownImage;
				
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
					
					if (sRelativeRegularPopDownImage==nil)
					{
						sRelativeRegularPopDownImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"RelativePopDown" ofType:@"tif"]];
					}
					
					return sRelativeRegularPopDownImage;
					
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
				
					if (sReferenceRegularPopDownImage==nil)
					{
						sReferenceRegularPopDownImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"ReferencePopDown" ofType:@"tif"]];
					}
					
					return sReferenceRegularPopDownImage;
			}
		}
		else
		{
			static NSImage * sReferenceSmallPopDownImage=nil;
			static NSImage * sRelativeSmallPopDownImage=nil;
			static NSImage * sAbsoluteSmallPopDownImage=nil;
			
			switch(inStyle)
			{
				case ICDOCUMENT_PATH_TYPE_GLOBAL:
				
					if (sAbsoluteSmallPopDownImage==nil)
					{
						sAbsoluteSmallPopDownImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"AbsoluteSmallPopDown" ofType:@"tif"]];
					}
					
					return sAbsoluteSmallPopDownImage;
				
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
					if (sRelativeSmallPopDownImage==nil)
					{
						sRelativeSmallPopDownImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"RelativeSmallPopDown" ofType:@"tif"]];
					}
					
					return sRelativeSmallPopDownImage;
					
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
				
					if (sReferenceSmallPopDownImage==nil)
					{
						sReferenceSmallPopDownImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"ReferenceSmallPopDown" ofType:@"tif"]];
					}
					
					return sReferenceSmallPopDownImage;
			}
		}
	}
	else
	{
		if (inControlSize==NSRegularControlSize)
		{
			static NSImage * sReferenceRegularImage=nil;
			static NSImage * sRelativeRegularImage=nil;
			static NSImage * sAbsoluteRegularImage=nil;
			static NSImage * sMixedImage=nil;
			
			switch(inStyle)
			{
				case ICDOCUMENT_PATH_TYPE_GLOBAL:
				
					if (sAbsoluteRegularImage==nil)
					{
						sAbsoluteRegularImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"AbsoluteRegularLeftEdge" ofType:@"tif"]];
					}
					
					return sAbsoluteRegularImage;
				
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
					
					if (sRelativeRegularImage==nil)
					{
						sRelativeRegularImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"RelativeRegularLeftEdge" ofType:@"tif"]];
					}
					
					return sRelativeRegularImage;
					
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
				
					if (sReferenceRegularImage==nil)
					{
						sReferenceRegularImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"ReferenceRegularLeftEdge" ofType:@"tif"]];
					}
					
					return sReferenceRegularImage;
				
				case ICDOCUMENT_PATH_TYPE_MIXED:
					
					if (sMixedImage==nil)
					{
						sMixedImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"MixedRegularLeftEdge" ofType:@"tif"]];
					}
					
					return sMixedImage;
			}
		}
		else
		{
			static NSImage * sReferenceSmallImage=nil;
			static NSImage * sRelativeSmallImage=nil;
			static NSImage * sAbsoluteSmallImage=nil;
			static NSImage * sMixedSmallImage=nil;
			
			switch(inStyle)
			{
				case ICDOCUMENT_PATH_TYPE_GLOBAL:
				
					if (sAbsoluteSmallImage==nil)
					{
						sAbsoluteSmallImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"AbsoluteSmallLeftEdge" ofType:@"tif"]];
					}
					
					return sAbsoluteSmallImage;
				
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
					
					if (sRelativeSmallImage==nil)
					{
						sRelativeSmallImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"RelativeSmallLeftEdge" ofType:@"tif"]];
					}
					
					return sRelativeSmallImage;
				
				case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
					
					if (sReferenceSmallImage==nil)
					{
						sReferenceSmallImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"ReferenceSmallLeftEdge" ofType:@"tif"]];
					}
					
					return sReferenceSmallImage;
							
				case ICDOCUMENT_PATH_TYPE_MIXED:
					
					if (sMixedSmallImage==nil)
					{
						sMixedSmallImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"MixedSmallLeftEdge" ofType:@"tif"]];
					}
					
					return sMixedSmallImage;
			}
		}
	}
	
    return nil;
}

+ (NSImage *) popupImageForReferenceStyle:(NSInteger) inStyle controlSize:(NSControlSize) inControlSize
{
    if (inControlSize==NSRegularControlSize)
	{
		static NSImage * sReferencePopupRegularImage=nil;
		static NSImage * sRelativePopupRegularImage=nil;
		static NSImage * sAbsolutePopupRegularImage=nil;
		
		switch(inStyle)
		{
			case ICDOCUMENT_PATH_TYPE_GLOBAL:
			
				if (sAbsolutePopupRegularImage==nil)
				{
					sAbsolutePopupRegularImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"PopAbsoluteRegular" ofType:@"tif"]];
				}
				
				return sAbsolutePopupRegularImage;
			
			case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
			
				if (sRelativePopupRegularImage==nil)
				{
					sRelativePopupRegularImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"PopRelativeRegular" ofType:@"tif"]];
				}
				
				return sRelativePopupRegularImage;
				
			case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
			
				if (sReferencePopupRegularImage==nil)
				{
					sReferencePopupRegularImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"PopReferenceRegular" ofType:@"tif"]];
				}
				
				return sReferencePopupRegularImage;
		}
	}
	else
	{
		static NSImage * sReferencePopupSmallImage=nil;
		static NSImage * sRelativePopupSmallImage=nil;
		static NSImage * sAbsolutePopupSmallImage=nil;
		
		switch(inStyle)
		{
			case ICDOCUMENT_PATH_TYPE_GLOBAL:
			
				if (sAbsolutePopupSmallImage==nil)
				{
					sAbsolutePopupSmallImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"PopAbsoluteSmall" ofType:@"tif"]];
				}
				
				return sAbsolutePopupSmallImage;
			
			case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT:
			
				if (sRelativePopupSmallImage==nil)
				{
					sRelativePopupSmallImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"PopRelativeSmall" ofType:@"tif"]];
				}
				
				return sRelativePopupSmallImage;
				
			case ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER:
			
				if (sReferencePopupSmallImage==nil)
				{
					sReferencePopupSmallImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"PopReferenceSmall" ofType:@"tif"]];
				}
				
				return sReferencePopupSmallImage;
		}
    }
	
    return nil;
}

+ (NSMenu *) referenceStyleMenuForTarget:(id) inTarget andAction:(SEL) inAction showReferenceFolder:(BOOL) inShowReferenceFolder
{
    NSMenu * nReferenceStyleMenu=nil;
    
    nReferenceStyleMenu=[[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@""];
    
    if (nReferenceStyleMenu!=nil)
    {
        id tMenuItem;
        NSImage * tImage;
        
        tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Relative to Reference Folder",@"No comment")
                                             action:inAction
                                      keyEquivalent:@""];
    
        [tMenuItem setTarget:inTarget];
        
        [tMenuItem setTag:ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER];
        
        tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Reference" ofType:@"tif"]];
        
        if (tImage!=nil)
        {
            [tMenuItem setImage:tImage];
        
            [tImage release];
        }
        
        [nReferenceStyleMenu addItem:tMenuItem];
        
        [tMenuItem release];
		
		
		tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Relative to Project",@"No comment")
                                             action:inAction
                                      keyEquivalent:@""];
    
        [tMenuItem setTarget:inTarget];
        
        [tMenuItem setTag:ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT];
        
        tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Relative" ofType:@"tif"]];
        
        if (tImage!=nil)
        {
            [tMenuItem setImage:tImage];
        
            [tImage release];
        }
        
        [nReferenceStyleMenu addItem:tMenuItem];
        
        [tMenuItem release];
        
		
		tMenuItem=[[NSMenuItem alloc] initWithTitle:NSLocalizedString(@"Absolute Path",@"No comment")
                                             action:inAction
                                      keyEquivalent:@""];
    
        [tMenuItem setTarget:inTarget];
        
        [tMenuItem setTag:ICDOCUMENT_PATH_TYPE_GLOBAL];
        
        tImage=[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Absolute" ofType:@"tif"]];
        
        if (tImage!=nil)
        {
            [tMenuItem setImage:tImage];
        
            [tImage release];
        }
        
        [nReferenceStyleMenu addItem:tMenuItem];
        
        [tMenuItem release];
    }
    
    return [nReferenceStyleMenu autorelease];
}

+ (NSMenu *) referenceStyleMenuForTarget:(id) inTarget andAction:(SEL) inAction
{
	return [ICReferencedUtilities referenceStyleMenuForTarget:inTarget andAction:inAction showReferenceFolder:NO];
}

+ (NSMenu *) referenceStyleMenuForTarget:(id) inTarget
{
    return [ICReferencedUtilities referenceStyleMenuForTarget:inTarget andAction:@selector(takeReferenceStyle:) showReferenceFolder:NO];
}

@end
