/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICRequirementChoiceEditorController.h"

#import "ICLocalizationUtilities.h"

#import "ICLanguagePopupButtonCell.h"

#import "ICDocument.h"

#import "ICDocumentSecondaryController.h"

@implementation ICRequirementChoiceEditorController

+ (ICRequirementChoiceEditorController *) newController
{
	ICRequirementChoiceEditorController * nController=nil;
    
    nController=[ICRequirementChoiceEditorController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICRequirementChoiceEditor" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICRequirementChoiceEditor"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return nController;
}

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn = nil;
	
	[super awakeFromNib];
	
	// Set up the messages array
	
	tTableColumn = [IBrequirementMessagesArray_ tableColumnWithIdentifier:ICDOCUMENT_LANGUAGE];
    
	if (tTableColumn!=nil)
	{
		ICLanguagePopupButtonCell * tPopupButtonCell = nil;
    
		tPopupButtonCell=[[ICLanguagePopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
		
		if (tPopupButtonCell!=nil)
		{
			[tPopupButtonCell setControlSize:NSSmallControlSize];
			
			[tPopupButtonCell setMenu:[ICLocalizationUtilities localizationMenu]];
			
			[tPopupButtonCell setBordered:NO];
			
			[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
			
			[tPopupButtonCell setAction:@selector(nothing:)];
			
			[tPopupButtonCell setTarget:self];
			
			[tTableColumn setDataCell:tPopupButtonCell];
			
			[tPopupButtonCell release];
		}
	}
	
	 tTableColumn = [IBrequirementMessagesArray_ tableColumnWithIdentifier:ICDOCUMENT_VALUE];
	
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
		
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
		
		[tTextFieldCell setWraps:YES];
	}
}

- (void) dealloc
{
	[cachedMessages_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) switchIdentifierView
{
	NSView * tPreviousKeyView;
	
	[super switchIdentifierView];
	
	[self updateMinMaxWindowSize];
	
	tPreviousKeyView=[currentUIController_ previousKeyView];
	
	if (tPreviousKeyView!=nil)
	{
		[IBrequirementMessagesArray_ setNextKeyView:tPreviousKeyView];
		
		[currentUIController_ setNextKeyView:IBrequirementMessagesArray_];
		
		[IBwindow_ makeFirstResponder:tPreviousKeyView];
	}
	else
	{
		[IBwindow_ makeFirstResponder:IBrequirementMessagesArray_];
	}
}

- (void) prepareUI
{
	NSButtonCell * tRadioButtonCell;
	NSNumber * tNumber;
	NSRect tFrame;
	
	[super prepareUI];
	
	// Requirement Behavior
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR];
	
	if (tNumber!=nil)
	{
		[IBrequirementBehaviorMatrix_ selectCellWithTag:[tNumber intValue]];
	}
	else
	{
		NSLog(@"[ICRequirementEditorController prepareUI]: Missing ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR value");
	}
	
	// Requirement Message
	
	cachedMessages_=[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_MESSAGE];
	
	if (cachedMessages_==nil)
	{
		cachedMessages_=[NSMutableArray array];
		
		[dictionary_ setObject:cachedMessages_ forKey:ICDOCUMENT_REQUIREMENT_MESSAGE];
	}
	
	if (cachedMessages_!=nil)
	{
		[cachedMessages_ retain];
	}
	
	[IBrequirementMessagesArray_ deselectAll:self];
	
	[IBrequirementMessagesRemoveButton_ setEnabled:NO];
	
	[IBrequirementMessagesArray_ reloadData];
	
	// Modify Radio Buttons if needed
	
	tFrame=[IBscrollView_ frame];
	
	if (visible_==YES)
	{
		tRadioButtonCell=[IBrequirementBehaviorMatrix_ cellWithTag:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR_CHOICE_UNSELECT_AND_HIDE];
		
		[tRadioButtonCell setTitle:NSLocalizedStringFromTable(@"Remove this choice",@"Requirements",@"")];
		
		
		
		[tRadioButtonCell setEnabled:YES];
		
		tRadioButtonCell=[IBrequirementBehaviorMatrix_ cellWithTag:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR_CHOICE_UNSELECT_AND_DISABLE];
		
		[tRadioButtonCell setTransparent:NO];
		
		if ([tRadioButtonCell isEnabled]==NO)
		{
			tFrame.size.height-=20.0f;
			
			[IBscrollView_ setFrame:tFrame];
		}
		
		[tRadioButtonCell setEnabled:YES];
		
		[tRadioButtonCell setTitle:NSLocalizedStringFromTable(@"Make this choice unselectable and show the following message in its tooltip:",@"Requirements",@"")];
	
		[IBrequirementMessagesAddButton_ setEnabled:![ICLocalizationUtilities allLanguagesUsedInLocalizations:cachedMessages_]];	// A COMPLETER (Depends if some languages are still available)
	}
	else
	{
		[dictionary_ setObject:[NSNumber numberWithInt:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR_CHOICE_UNSELECT_AND_DISABLE] forKey:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR];
		
		tRadioButtonCell=[IBrequirementBehaviorMatrix_ cellWithTag:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR_CHOICE_UNSELECT_AND_HIDE];
		
		
		
		[tRadioButtonCell setTransparent:NO];
		
		[tRadioButtonCell setTitle:NSLocalizedStringFromTable(@"Make this choice unselectable",@"Requirements",@"")];
		
		tRadioButtonCell=[IBrequirementBehaviorMatrix_ cellWithTag:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR_CHOICE_UNSELECT_AND_DISABLE];
		
		[tRadioButtonCell setTransparent:YES];
		
		if ([tRadioButtonCell isEnabled]==YES)
		{
			tFrame.size.height+=20.0f;
			
			[IBscrollView_ setFrame:tFrame];
		}
		
		[tRadioButtonCell setEnabled:NO];
	
		[IBrequirementBehaviorMatrix_ selectCellWithTag:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR_CHOICE_UNSELECT_AND_HIDE];
		
		[IBrequirementMessagesAddButton_ setEnabled:NO];
	}
}

- (void) showEditorForWindow:(NSWindow *) inWindow withDictionary:(NSMutableDictionary *) inDictionary visibility:(BOOL) inVisible delegate:(id) inDelegate
{
	[cachedMessages_ release];
	
	visible_=inVisible;
	
	[super showEditorForWindow:inWindow withDictionary:inDictionary delegate:inDelegate];
}

#pragma mark -

- (IBAction) switchBehavior:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedCell] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR];
	}
}

- (IBAction) nothing:(id) sender
{
}
#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
    SEL tAction=[inMenuItem action];
	
	if (tAction==@selector(nothing:))
	{
		NSInteger tSelectedCount;
		
		tSelectedCount=[IBrequirementMessagesArray_ numberOfSelectedRows];
		
		if (tSelectedCount!=1)
		{
			return NO;
		}
		else
		{
			NSString * tCurrentLanguageName;
			NSString * tTitle;
			
			tCurrentLanguageName=[[cachedMessages_ objectAtIndex:[IBrequirementMessagesArray_ selectedRow]] objectForKey:ICDOCUMENT_LANGUAGE];
			
			tTitle=[inMenuItem title];
			
			if ([tTitle isEqualToString:tCurrentLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[cachedMessages_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tTitle]==YES)
						{
							return NO;
						}
					}
				}
			}
		}
	}
    
    return YES;
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (cachedMessages_!=nil)
	{
		return [cachedMessages_ count];
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (cachedMessages_!=nil)
	{
		NSString * tColumnIdentifier;
		NSDictionary * tDictionary;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		tDictionary=[cachedMessages_ objectAtIndex:inRowIndex];
		
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
		{
			return [tDictionary objectForKey:ICDOCUMENT_VALUE];
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	
	if (cachedMessages_!=nil)
	{
		NSMutableDictionary * tDictionary;
		
		tColumnIdentifier=[inTableColumn identifier];
	
		tDictionary=[cachedMessages_ objectAtIndex:inRowIndex];
			
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			NSString * tLanguageName;
			
			// Check that the Language is different
			
			tLanguageName=[[inTableColumn dataCellForRow:inRowIndex] itemTitleAtIndex:[object intValue]];
			
			if ([[tDictionary objectForKey:ICDOCUMENT_LANGUAGE] isEqualToString:tLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				NSUInteger tSelectionIndex;
				
				tEnumerator=[cachedMessages_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tLanguageName]==YES)
						{
							NSBeep();
							
							return;
						}
					}
				}
				
				[tDictionary setObject:tLanguageName forKey:ICDOCUMENT_LANGUAGE];
				
				[inTableView deselectAll:self];
				
				[cachedMessages_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[inTableView reloadData];
				
				tSelectionIndex=[cachedMessages_ indexOfObject:tDictionary];
				
				if (tSelectionIndex!=NSNotFound)
				{
					[inTableView selectRowIndexes:[NSIndexSet indexSetWithIndex:tSelectionIndex] byExtendingSelection:NO];
				}
			}
		}
		else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
		{
			NSString * tTitleValue;
			
			tTitleValue=(NSString *) object;
			
			if ([[tDictionary objectForKey:ICDOCUMENT_VALUE] isEqualToString:tTitleValue]==NO)
			{
				[tDictionary setObject:tTitleValue forKey:ICDOCUMENT_VALUE];
			}
		}
	}
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{    
    if (cachedMessages_!=nil)
	{
		NSString * tColumnIdentifier;
	
		tColumnIdentifier=[inTableColumn identifier];
		
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			NSDictionary * tDictionary;
		
			tDictionary=[cachedMessages_ objectAtIndex:inRowIndex];
		
			[cell selectItemWithTitle:[tDictionary objectForKey:ICDOCUMENT_LANGUAGE]];
		}
	}
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBrequirementMessagesArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeRequirementMessage:IBrequirementMessagesRemoveButton_];
		}
		else
		{
			[self removeRequirementMessageDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBrequirementMessagesArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBrequirementMessagesRemoveButton_ isEnabled];
		}
	}
	
	return NO;
}

#pragma mark -

- (IBAction) addRequirementMessage:(id) sender
{
	if (cachedMessages_!=nil)
	{
		NSMutableDictionary * tNewLocalizedDescriptionDictionary;
		NSString * tLanguageName;
	
		tLanguageName=[[delegate document] documentDefaultForKey:ICDOCUMENT_PRESENTATION_PREVIEW_LANGUAGE];
							
		if ([ICLocalizationUtilities localizations:cachedMessages_ containsLanguage:tLanguageName]==YES)
		{
			tLanguageName=[ICLocalizationUtilities firstNewLanguageInLocalizations:cachedMessages_];
		}
		
		if (tLanguageName!=nil)
		{
			tNewLocalizedDescriptionDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tLanguageName,ICDOCUMENT_LANGUAGE,
																						   @"",ICDOCUMENT_VALUE,
																						   nil];
			
			if (tNewLocalizedDescriptionDictionary!=nil)
			{
				NSUInteger tRowIndex;
				
				[cachedMessages_ addObject:tNewLocalizedDescriptionDictionary];
				
				[cachedMessages_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[IBrequirementMessagesArray_ reloadData];
				
				tRowIndex=[cachedMessages_ indexOfObject:tNewLocalizedDescriptionDictionary];
				
				if (tRowIndex!=NSNotFound)
				{
					[IBrequirementMessagesArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
					[IBrequirementMessagesArray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
				}
			}
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (IBAction) removeRequirementMessage:(id) sender
{
	NSString * tAlertTitle;
    NSInteger tReturnCode;
	
    if ([IBrequirementMessagesArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this message?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these messages?",@"No comment");
    }
    
    tReturnCode=NSRunAlertPanel(tAlertTitle, NSLocalizedString(@"This cannot be undone.",@"No comment"), NSLocalizedString(@"Remove",@"No comment"), NSLocalizedString(@"Cancel",@"No comment"), nil);
	
	[self removeRequirementMessageDidEnd:nil returnCode:tReturnCode contextInfo:NULL];
}

- (void) removeRequirementMessageDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBrequirementMessagesArray_ selectedRowIndexes];
		
		[IBrequirementMessagesArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedMessages_ removeObjectsAtIndexes:tIndexSet];
			
			[IBrequirementMessagesArray_ reloadData];
			
			[IBrequirementMessagesRemoveButton_ setEnabled:NO];
		}
	}
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBrequirementMessagesArray_)
    {
		[IBrequirementMessagesAddButton_ setEnabled:![ICLocalizationUtilities allLanguagesUsedInLocalizations:cachedMessages_]];
		
		[IBrequirementMessagesRemoveButton_ setEnabled:([IBrequirementMessagesArray_ numberOfSelectedRows]!=0)];
	}
}

@end
