/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementEditorController.h"

#import "NSDictionary+MutableDeepCopy.h"

#import "ICRequirementPluginsManager.h"

#import "ICPluginsManager+AppKit.h"

#import "ICDocumentWindow.h"

@implementation ICRequirementEditorController

- (void) dealloc
{
	[dictionary_ release];
	
	[cachedValues_ release];
	
	[super dealloc];
}

- (void) awakeFromNib
{
	NSArray * tTitles;
	
	defaultContentWidth_=NSWidth([[IBwindow_ contentView] frame]);
	
	// Requirement Popup
	
	[IBrequirementTypePopupButton_ removeAllItems];
		
	tTitles=[[ICRequirementPluginsManager defaultManager] allPluginsNameSorted];
	
	if (tTitles!=nil)
	{
		[IBrequirementTypePopupButton_ addItemsWithTitles:tTitles];
	}
}

#pragma mark -

- (id) delegate
{
	return delegate;
}

- (NSMutableDictionary *) dictionary
{
	return [[dictionary_ retain] autorelease];
}

#pragma mark -

- (void) updateMinMaxWindowSize
{
	NSSize tSize;
		
	tSize=[[IBwindow_ contentView] frame].size;
		
	if ([currentUIController_ windowCanBeResized]==YES)
	{
		NSRect tContentFrame;
		NSRect tRequirementFrame;
		
		tContentFrame=[[IBwindow_ contentView] frame];
		
		tRequirementFrame=[IBrequirementView_ frame];
		
		tContentFrame.size.height=NSHeight(tContentFrame)-NSHeight(tRequirementFrame)+[currentUIController_ minHeight];
		
		[IBwindow_ setContentMinSize:NSMakeSize(defaultContentWidth_, NSHeight(tContentFrame))];
		
		
		
		[IBwindow_ setContentMaxSize:NSMakeSize(2000.0f,2000.0f)];
	
		//NSLog(@"Min: %@ Max: %@",NSStringFromSize(NSMakeSize(defaultContentWidth_, NSHeight(tContentFrame))),NSStringFromSize(NSMakeSize(2000.0f,2000.0f)) );
	}
	else
	{
		tSize.width=defaultContentWidth_;
		
		[IBwindow_ setContentMinSize:tSize];
		
		[IBwindow_ setContentMaxSize:tSize];
		
		//NSLog(@"Min: %@ Max: %@",NSStringFromSize(tSize),NSStringFromSize(tSize) );
	}
}

- (void) switchIdentifierView
{
	NSString * tRequirementIdentifier;
	
	if (currentUIController_!=nil)
	{
		[currentUIController_ viewWillBeRemoved:self];
		
		[currentUIController_ release];
		
		currentUIController_=nil;
	}
	
	if (currentView_!=nil)
	{
		[currentView_ removeFromSuperview];
	
		currentView_=nil;
	}

	tRequirementIdentifier=[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_IDENTIFIER];
	
	if (tRequirementIdentifier!=nil)
	{
		[IBrequirementTypeIcon_ setImage:[[ICRequirementPluginsManager defaultManager] iconForIdentifier:tRequirementIdentifier]];
		
		currentUIController_=[[ICRequirementPluginsManager defaultManager] createPluginUIControllerForIdentifier:tRequirementIdentifier];
		
		if (currentUIController_!=nil)
		{
			currentView_=[currentUIController_ view];
			
			if (currentView_!=nil)
			{
				NSRect tBounds;
				NSRect tCurrentViewBounds;
				NSRect tOldWindowFrame;
				NSRect tNewWindowFrame;
				NSRect tComputeRect;
				
				NSMutableDictionary * tValuesDictionary;
				
				tBounds=[IBrequirementView_ bounds];
				
				tCurrentViewBounds=[currentView_ bounds];
				
				if ([currentUIController_ windowCanBeResized]==YES)
				{
					NSString * tSizeString;
					NSString * tKey;
					
					tKey=[NSString stringWithFormat:@"%@.size",[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_IDENTIFIER]];
					
					tSizeString=[[NSUserDefaults standardUserDefaults] objectForKey:tKey];
					
					if (tSizeString!=nil)
					{
						tCurrentViewBounds=NSRectFromString(tSizeString);
					}
					
					[IBwindow_ setShowsResizeIndicator:YES];
				}
				else
				{
					[IBwindow_ setShowsResizeIndicator:NO];
				}
				
				[IBwindow_ setMinSize:NSMakeSize(50.0f,50.0f)];
				
				
				tOldWindowFrame=[IBwindow_ frame];
				
				tComputeRect=NSMakeRect(0,0,NSWidth(tCurrentViewBounds)-NSWidth(tBounds),NSHeight(tCurrentViewBounds)-NSHeight(tBounds));
				
				tComputeRect=[NSWindow frameRectForContentRect:tComputeRect styleMask:NSBorderlessWindowMask];
				
				tNewWindowFrame.size=NSMakeSize(NSWidth(tOldWindowFrame)+NSWidth(tComputeRect),NSHeight(tOldWindowFrame)+NSHeight(tComputeRect));
				
				tNewWindowFrame.origin.x=_CGFloatFloor(NSMidX(tOldWindowFrame)-NSWidth(tNewWindowFrame)*0.5f);
				tNewWindowFrame.origin.y=NSMaxY(tOldWindowFrame)-NSHeight(tNewWindowFrame);
				
				
				
				tValuesDictionary=[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_DICTIONARY];
				
				if (tValuesDictionary==nil)
				{
					tValuesDictionary=[currentUIController_ defaultDictionary];
					
					if (tValuesDictionary!=nil)
					{
						[dictionary_ setObject:tValuesDictionary forKey:ICDOCUMENT_REQUIREMENT_DICTIONARY];
					}
					else
					{
						// A COMPLETER
					}
				}
				
				[currentUIController_ initializeWithDictionary:tValuesDictionary forDocument:[delegate document]];
				
				[IBwindow_ setFrame:tNewWindowFrame display:YES animate:NO];
				
				tBounds=[IBrequirementView_ bounds];
				
				if ([currentUIController_ windowCanBeResized]==YES)
				{
					[currentView_ setAutoresizingMask:NSViewWidthSizable+NSViewHeightSizable];
				}
				
				[currentView_ setFrame:tBounds];
				
				[IBrequirementView_ addSubview:currentView_];
			}
		}
	}
}

- (void) prepareUI
{
	NSString * tRequirementIdentifier;
	
	// Set the Requirement
	
	tRequirementIdentifier=[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_IDENTIFIER];
	
	if (tRequirementIdentifier!=nil)
	{
		NSString * tLocalizedName;
	
		tLocalizedName=[[ICRequirementPluginsManager defaultManager] localizedPluginNameForIdentifier:tRequirementIdentifier];
		
		[IBrequirementTypePopupButton_ selectItemWithTitle:tLocalizedName];
		
		[self switchIdentifierView];
	}
	else
	{
		NSLog(@"[ICRequirementEditorController prepareUI]: Missing ICDOCUMENT_REQUIREMENT_IDENTIFIER value");
	}
	
	// Convert to Script button
	
	/*if ([tRequirementIdentifier isEqualToString:@""]==YES)	// A COMPLETER
	{
		[IBrequirementConvertToScriptButton_ setEnabled:NO];
	}
	else
	{
		[IBrequirementConvertToScriptButton_ setEnabled:YES];
	}*/
}

- (void) showEditorForWindow:(NSWindow *) inWindow withDictionary:(NSMutableDictionary *) inDictionary delegate:(id) inDelegate
{
	[dictionary_ release];
	
	dictionary_=nil;
	
	if (inWindow!=nil && inDictionary!=nil && inDelegate!=nil)
	{
		if (cachedValues_==nil)
		{
			cachedValues_=[[NSMutableDictionary alloc] initWithCapacity:[[[ICRequirementPluginsManager defaultManager] allPluginsIdentifier] count]];
		}
		
		dictionary_=[inDictionary mutableDeepCopy];
		
		delegate=inDelegate;
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(optionKeyDidChange:)
													 name:ICWINDOW_NOTIFICATION_OPTIONKEY_DID_CHANGE
												   object:IBwindow_];
		
		[self prepareUI];
		
		
		
		[NSApp beginSheet:IBwindow_
		   modalForWindow:inWindow
		    modalDelegate:self
		   didEndSelector:nil
		      contextInfo:NULL];
	}
}

- (IBAction) endDialog:(id) sender
{
	NSMutableDictionary * tRequirementDictionary=nil;
	
	[IBwindow_ makeFirstResponder:nil];

	if ([currentUIController_ windowCanBeResized]==YES)
	{
		if (currentView_!=nil)
		{
			NSRect tBounds;
			NSString * tKey;
			
			tBounds=[currentView_ bounds];
			
			tKey=[NSString stringWithFormat:@"%@.size",[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_IDENTIFIER]];
			
			[[NSUserDefaults standardUserDefaults] setObject:NSStringFromRect(tBounds) forKey:tKey];
			
			[currentView_ setAutoresizingMask:0];
		}
	}
	
	if ([sender tag]==NSOKButton)
	{
		tRequirementDictionary=[currentUIController_ dictionary];
	
		if (tRequirementDictionary!=nil)
		{
			[dictionary_ setObject:tRequirementDictionary forKey:ICDOCUMENT_REQUIREMENT_DICTIONARY];
		
			if (delegate!=nil)
			{
				NSString * tRequirementName;
				
				tRequirementName=[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_NAME];
				
				if (tRequirementName==nil)
				{
					[dictionary_ setObject:[IBrequirementTypePopupButton_ titleOfSelectedItem] forKey:@"SuggestedName"];
				}
				
				[delegate requirementEditionDidEnd:self returnCode:NSOKButton];
			}
		}
		else
		{
			NSLog(@"[ICRequirementEditorController endDialog:] A COMPLETER");
		}
	}
	else
	{
		[delegate requirementEditionDidEnd:self returnCode:[sender tag]];
	}
	
	[cachedValues_ release];
	
	cachedValues_=nil;
	
	[NSApp endSheet:IBwindow_];
	
	[IBwindow_ orderOut:self];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self name:ICWINDOW_NOTIFICATION_OPTIONKEY_DID_CHANGE object:IBwindow_];
}

#pragma mark - 

- (IBAction) switchRequirementType:(id) sender
{
	NSString * tIdentifier;
	
	tIdentifier=[[ICRequirementPluginsManager defaultManager] identifierForLocalizedPluginName:[sender titleOfSelectedItem]];
	
	if ([tIdentifier isEqualToString:[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_IDENTIFIER]]==NO)
	{
		NSMutableDictionary * tMutableDictionary;
		
		[IBwindow_ makeFirstResponder:nil];
		
		if (currentUIController_!=nil)
		{
			tMutableDictionary=[currentUIController_ dictionary];
		
			if (tMutableDictionary!=nil)
			{
				[cachedValues_ setObject:tMutableDictionary forKey:[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_IDENTIFIER]];
			}
			
			if ([currentUIController_ windowCanBeResized]==YES)
			{
				if (currentView_!=nil)
				{
					NSRect tBounds;
					NSString * tKey;
					
					tBounds=[currentView_ bounds];
					
					tKey=[NSString stringWithFormat:@"%@.size",[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_IDENTIFIER]];
					
					[[NSUserDefaults standardUserDefaults] setObject:NSStringFromRect(tBounds) forKey:tKey];
					
					[currentView_ setAutoresizingMask:0];
				}
			}
		}
		
		[dictionary_ setObject:tIdentifier forKey:ICDOCUMENT_REQUIREMENT_IDENTIFIER];
		
		tMutableDictionary=[cachedValues_ objectForKey:tIdentifier];
		
		if (tMutableDictionary!=nil)
		{
			[dictionary_ setObject:tMutableDictionary forKey:ICDOCUMENT_REQUIREMENT_DICTIONARY];
		}
		else
		{
			[dictionary_ removeObjectForKey:ICDOCUMENT_REQUIREMENT_DICTIONARY];
		}
		
		[self switchIdentifierView];
	}
}

- (IBAction) convertToScript:(id) sender
{
}

#pragma mark -

- (void) optionKeyDidChange:(NSNotification *) inNotification
{
	if (inNotification!=nil)
	{
		NSDictionary * tDictionary;
		
		tDictionary=[inNotification userInfo];
		
		if (tDictionary!=nil)
		{
			NSNumber * tNumber;
			
			tNumber=[tDictionary objectForKey:ICWINDOW_OPTIONKEY_STATE];
			
			if (tNumber!=nil)
			{
				[currentUIController_ optionKeyStateDidChange:[tNumber boolValue]];
			}
		}
	}
}

@end

@implementation NSObject (RequirementEdition)

- (void) requirementEditionDidEnd:(ICRequirementEditorController *) inEditor returnCode:(NSInteger) inReturnCode
{
}

@end
