/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementProjectEditorBehaviorController.h"

@implementation ICRequirementProjectEditorBehaviorController

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary parentController:(id) inParentController
{
	NSNumber * tNumber;
	
	[dictionary_ release];
	
	dictionary_=[inDictionary retain];
	
	// Requirement Behavior
	
	tNumber=[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR];
	
	if (tNumber!=nil)
	{
		[IBrequirementBehaviorMatrix_ selectCellWithTag:[tNumber intValue]];
	}
	else
	{
		NSLog(@"[ICRequirementPorjectEditorBehaviorController initializeWithDictionary:]: Missing ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR value");
	}
	
	// Requirement Message
	
	cachedMessages_=[dictionary_ objectForKey:ICDOCUMENT_REQUIREMENT_MESSAGE];
	
	if (cachedMessages_==nil)
	{
		cachedMessages_=[NSMutableArray array];
		
		[dictionary_ setObject:cachedMessages_ forKey:ICDOCUMENT_REQUIREMENT_MESSAGE];
	}
	
	if (cachedMessages_!=nil)
	{
		[cachedMessages_ retain];
	}
	
	[IBrequirementMessagesArray_ deselectAll:self];
	
	[IBrequirementMessagesRemoveButton_ setEnabled:NO];
	
	[IBrequirementMessagesArray_ reloadData];
}

- (void) dealloc
{
	[cachedMessages_ release];
	
	[dictionary_ release];

	[super dealloc];
}

#pragma mark -

- (id) view
{
	return IBview_;
}

- (id) arrayOutlet
{
	return IBrequirementMessagesArray_;
}

#pragma mark -

- (IBAction) switchBehavior:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedCell] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:ICDOCUMENT_REQUIREMENT_FAILURE_BEHAVIOR];
	}
}

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (cachedMessages_!=nil)
	{
		return [cachedMessages_ count];
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	// To implement in the subclass
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBrequirementMessagesArray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeRequirementMessage:IBrequirementMessagesRemoveButton_];
		}
		else
		{
			[self removeRequirementMessageDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBrequirementMessagesArray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBrequirementMessagesRemoveButton_ isEnabled];
		}
	}
	
	return NO;
}

#pragma mark -

- (IBAction) nothing:(id) sender
{
}

#pragma mark -

- (IBAction) addRequirementMessage:(id) sender
{
	// To implement in the subclass
}

- (IBAction) removeRequirementMessage:(id) sender
{
	NSString * tAlertTitle;
    NSInteger tReturnCode;
	
    if ([IBrequirementMessagesArray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove this message?",@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedString(@"Do you really want to remove these messages?",@"No comment");
    }
    
    tReturnCode=NSRunAlertPanel(tAlertTitle, NSLocalizedString(@"This cannot be undone.",@"No comment"), NSLocalizedString(@"Remove",@"No comment"), NSLocalizedString(@"Cancel",@"No comment"), nil);
	
	[self removeRequirementMessageDidEnd:nil returnCode:tReturnCode contextInfo:NULL];
}

- (void) removeRequirementMessageDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBrequirementMessagesArray_ selectedRowIndexes];
		
		[IBrequirementMessagesArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedMessages_ removeObjectsAtIndexes:tIndexSet];
			
			[IBrequirementMessagesArray_ reloadData];
			
			[IBrequirementMessagesRemoveButton_ setEnabled:NO];
		}
	}
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBrequirementMessagesArray_)
    {
		[IBrequirementMessagesAddButton_ setEnabled:![ICLocalizationUtilities allLanguagesUsedInLocalizations:cachedMessages_]];
		
		[IBrequirementMessagesRemoveButton_ setEnabled:([IBrequirementMessagesArray_ numberOfSelectedRows]!=0)];
	}
}

@end
