/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementProjectEditorBehaviorController_Installation.h"

#import "ICLanguagePopupButtonCell.h"

#import "ICRequirementFormatters.h"

@implementation ICRequirementProjectEditorBehaviorController_Installation

+ (ICRequirementProjectEditorBehaviorController_Installation *) controller
{
	ICRequirementProjectEditorBehaviorController_Installation * nController;
    
    nController=[ICRequirementProjectEditorBehaviorController_Installation alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICRequirementProjectEditor_Installation" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICRequirementProjectEditor_Installation"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn = nil;
	
	// Set up the messages array
	
	 tTableColumn = [IBrequirementMessagesArray_ tableColumnWithIdentifier:ICDOCUMENT_VALUE];
	
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
		
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
		
		[tTextFieldCell setWraps:YES];
	}
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
    SEL tAction=[inMenuItem action];
	
	if (tAction==@selector(nothing:))
	{
		NSInteger tSelectedCount;
		
		tSelectedCount=[IBrequirementMessagesArray_ numberOfSelectedRows];
		
		if (tSelectedCount!=1)
		{
			return NO;
		}
		else
		{
			NSString * tCurrentLanguageName;
			NSString * tTitle;
			
			tCurrentLanguageName=[[cachedMessages_ objectAtIndex:[IBrequirementMessagesArray_ selectedRow]/2] objectForKey:ICDOCUMENT_LANGUAGE];
			
			tTitle=[inMenuItem title];
			
			if ([tTitle isEqualToString:tCurrentLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[cachedMessages_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tTitle]==YES)
						{
							return NO;
						}
					}
				}
			}
		}
	}
    
    return YES;
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (cachedMessages_!=nil)
	{
		return 2*[cachedMessages_ count];
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (cachedMessages_!=nil)
	{
		NSString * tColumnIdentifier;
		NSDictionary * tDictionary;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		tDictionary=[cachedMessages_ objectAtIndex:(inRowIndex/2)];
		
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
		{
			NSString * tValue=nil;
		
			if ((inRowIndex%2)==0)
			{
				tValue=[tDictionary objectForKey:ICDOCUMENT_VALUE];
			}
			else
			{
				tValue=[tDictionary objectForKey:ICDOCUMENT_SECONDARY_VALUE];
			}
			
			return tValue;
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	
	if (cachedMessages_!=nil)
	{
		NSMutableDictionary * tDictionary;
		
		tColumnIdentifier=[inTableColumn identifier];
	
		tDictionary=[cachedMessages_ objectAtIndex:(inRowIndex/2)];
			
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			NSString * tLanguageName;
			
			// Check that the Language is different
			
			tLanguageName=[[inTableColumn dataCellForRow:(inRowIndex%2)] itemTitleAtIndex:[object intValue]];
			
			if ([[tDictionary objectForKey:ICDOCUMENT_LANGUAGE] isEqualToString:tLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				NSUInteger tSelectionIndex;
				
				tEnumerator=[cachedMessages_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tLanguageName]==YES)
						{
							NSBeep();
							
							return;
						}
					}
				}
				
				[tDictionary setObject:tLanguageName forKey:ICDOCUMENT_LANGUAGE];
				
				[inTableView deselectAll:self];
				
				[cachedMessages_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[inTableView reloadData];
				
				tSelectionIndex=[cachedMessages_ indexOfObject:tDictionary];
				
				if (tSelectionIndex!=NSNotFound)
				{
					[inTableView selectRowIndexes:[NSIndexSet indexSetWithIndex:tSelectionIndex*2] byExtendingSelection:NO];
				}
			}
		}
		else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
		{
			NSString * tValue;
			
			tValue=(NSString *) object;
			
			if ((inRowIndex%2)==0)
			{
				if ([[tDictionary objectForKey:ICDOCUMENT_VALUE] isEqualToString:tValue]==NO)
				{
					[tDictionary setObject:tValue forKey:ICDOCUMENT_VALUE];
				}
			}
			else
			{
				if ([[tDictionary objectForKey:ICDOCUMENT_SECONDARY_VALUE] isEqualToString:tValue]==NO)
				{
					[tDictionary setObject:tValue forKey:ICDOCUMENT_SECONDARY_VALUE];
				}
			}
		}
	}
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{    
    if (cachedMessages_!=nil)
	{
		NSString * tColumnIdentifier;
	
		tColumnIdentifier=[inTableColumn identifier];
		
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			if ((inRowIndex%2)==0)
			{
				NSDictionary * tDictionary;
		
				tDictionary=[cachedMessages_ objectAtIndex:(inRowIndex/2)];
		
				[cell selectItemWithTitle:[tDictionary objectForKey:ICDOCUMENT_LANGUAGE]];
			}
		}
		else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
		{
			id tFormatter=nil;
			
			if ((inRowIndex%2)==0)
			{
				tFormatter=[ICTitleFormatter new];
			}
			else
			{
				tFormatter=[ICMessageFormatter new];
			}
			
			if (tFormatter!=nil)
			{
				[cell setFormatter:tFormatter];
			
				[tFormatter release];
			}
		}
	}
}

- (id) tableView:(NSTableView *) inTableView column:(NSTableColumn *) inColumn dataCellForRow:(NSInteger) inRowIndex
{
	if (inRowIndex>=0)
	{
		if (inTableView==IBrequirementMessagesArray_)
		{
			NSString * tColumnIdentifier;
			
			tColumnIdentifier=[inColumn identifier];
			
			if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE]==YES)
			{
				if ((inRowIndex%2)==0)
				{
					ICLanguagePopupButtonCell * tPopupButtonCell = nil;
    
					tPopupButtonCell=[[ICLanguagePopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
					
					if (tPopupButtonCell!=nil)
					{
						[tPopupButtonCell setControlSize:NSSmallControlSize];
						
						[tPopupButtonCell setMenu:[ICLocalizationUtilities localizationMenu]];
						
						[tPopupButtonCell setBordered:NO];
						
						[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
						
						[tPopupButtonCell setAction:@selector(nothing:)];
						
						[tPopupButtonCell setTarget:self];
						
						return [tPopupButtonCell autorelease];
					}
				}
				else
				{
					NSCell * tCell;
					
					tCell=[[NSCell alloc] initTextCell:@""];
					
					if (tCell!=nil)
					{
						return [tCell autorelease];
					}
				}
			}
		}
	}
	
	return nil;
}

#pragma mark -

- (IBAction) addRequirementMessage:(id) sender
{
	if (cachedMessages_!=nil)
	{
		NSMutableDictionary * tNewLocalizedDescriptionDictionary;
		NSString * tLanguageName;
	
		tLanguageName=[ICLocalizationUtilities firstNewLanguageInLocalizations:cachedMessages_];
		
		if (tLanguageName!=nil)
		{
			tNewLocalizedDescriptionDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tLanguageName,ICDOCUMENT_LANGUAGE,
																						   @"",ICDOCUMENT_VALUE,
																						   @"",ICDOCUMENT_SECONDARY_VALUE,
																						   nil];
			
			if (tNewLocalizedDescriptionDictionary!=nil)
			{
				NSUInteger tRowIndex;
				
				[cachedMessages_ addObject:tNewLocalizedDescriptionDictionary];
				
				[cachedMessages_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[IBrequirementMessagesArray_ reloadData];
				
				tRowIndex=[cachedMessages_ indexOfObject:tNewLocalizedDescriptionDictionary];
				
				if (tRowIndex!=NSNotFound)
				{
					[IBrequirementMessagesArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex*2] byExtendingSelection:NO];
				
					[IBrequirementMessagesArray_ editColumn:1 row:tRowIndex*2 withEvent:nil select:YES];
				}
			}
		}
		else
		{
			// A COMPLETER
		}
	}
}

- (void) removeRequirementMessageDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
    if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBrequirementMessagesArray_ selectedRowIndexes];
		
		[IBrequirementMessagesArray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			// A COMPLETER
		
			[cachedMessages_ removeObjectsAtIndexes:tIndexSet];
			
			[IBrequirementMessagesArray_ reloadData];
			
			[IBrequirementMessagesRemoveButton_ setEnabled:NO];
		}
	}
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBrequirementMessagesArray_)
    {
		NSMutableIndexSet * tIndexSet;
		NSUInteger tIndex;
		
		tIndexSet=[[IBrequirementMessagesArray_ selectedRowIndexes] mutableCopy];
		
		tIndex=[tIndexSet firstIndex];
		
		while (tIndex!=NSNotFound)
		{
			tIndex=[tIndexSet indexGreaterThanIndex:tIndex];
		}
		
		
		[IBrequirementMessagesAddButton_ setEnabled:![ICLocalizationUtilities allLanguagesUsedInLocalizations:cachedMessages_]];
		
		[IBrequirementMessagesRemoveButton_ setEnabled:([IBrequirementMessagesArray_ numberOfSelectedRows]!=0)];
	}
}

@end
