/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementProjectEditorBehaviorController_Volume.h"

#import "ICLanguagePopupButtonCell.h"

@implementation ICRequirementProjectEditorBehaviorController_Volume

+ (ICRequirementProjectEditorBehaviorController_Volume *) controller
{
	ICRequirementProjectEditorBehaviorController_Volume * nController;
    
    nController=[ICRequirementProjectEditorBehaviorController_Volume alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICRequirementProjectEditor_Volume" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICRequirementProjectEditor_Volume"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return [nController autorelease];
}

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn;
	
	//[super awakeFromNib];
	
	// Set up the messages array
	
	tTableColumn = [IBrequirementMessagesArray_ tableColumnWithIdentifier:ICDOCUMENT_LANGUAGE];
    
	if (tTableColumn!=nil)
	{
		ICLanguagePopupButtonCell * tPopupButtonCell = nil;
    
		tPopupButtonCell=[[ICLanguagePopupButtonCell alloc] initTextCell:@"" pullsDown:NO];
		
		if (tPopupButtonCell!=nil)
		{
			[tPopupButtonCell setControlSize:NSSmallControlSize];
			
			[tPopupButtonCell setMenu:[ICLocalizationUtilities localizationMenu]];
			
			[tPopupButtonCell setBordered:NO];
			
			[tPopupButtonCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
			
			[tPopupButtonCell setAction:@selector(nothing:)];
			
			[tPopupButtonCell setTarget:self];
			
			[tTableColumn setDataCell:tPopupButtonCell];
			
			[tPopupButtonCell release];
		}
	}
	
	 tTableColumn = [IBrequirementMessagesArray_ tableColumnWithIdentifier:ICDOCUMENT_VALUE];
	
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
		
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
		
		[tTextFieldCell setWraps:YES];
	}
}

#pragma mark -

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (cachedMessages_!=nil)
	{
		NSString * tColumnIdentifier;
		NSDictionary * tDictionary;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		tDictionary=[cachedMessages_ objectAtIndex:inRowIndex];
		
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
		{
			return [tDictionary objectForKey:ICDOCUMENT_VALUE];
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	
	if (cachedMessages_!=nil)
	{
		NSMutableDictionary * tDictionary;
		
		tColumnIdentifier=[inTableColumn identifier];
	
		tDictionary=[cachedMessages_ objectAtIndex:inRowIndex];
			
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			NSString * tLanguageName;
			
			// Check that the Language is different
			
			tLanguageName=[[inTableColumn dataCellForRow:inRowIndex] itemTitleAtIndex:[object intValue]];
			
			if ([[tDictionary objectForKey:ICDOCUMENT_LANGUAGE] isEqualToString:tLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				NSUInteger tSelectionIndex;
				
				tEnumerator=[cachedMessages_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tLanguageName]==YES)
						{
							NSBeep();
							
							return;
						}
					}
				}
				
				[tDictionary setObject:tLanguageName forKey:ICDOCUMENT_LANGUAGE];
				
				[inTableView deselectAll:self];
				
				[cachedMessages_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[inTableView reloadData];
				
				tSelectionIndex=[cachedMessages_ indexOfObject:tDictionary];
				
				if (tSelectionIndex!=NSNotFound)
				{
					[inTableView selectRowIndexes:[NSIndexSet indexSetWithIndex:tSelectionIndex] byExtendingSelection:NO];
				}
			}
		}
		else if ([tColumnIdentifier isEqualToString:ICDOCUMENT_VALUE])
		{
			NSString * tTitleValue;
			
			tTitleValue=(NSString *) object;
			
			if ([[tDictionary objectForKey:ICDOCUMENT_VALUE] isEqualToString:tTitleValue]==NO)
			{
				[tDictionary setObject:tTitleValue forKey:ICDOCUMENT_VALUE];
			}
		}
	}
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{    
    if (cachedMessages_!=nil)
	{
		NSString * tColumnIdentifier;
	
		tColumnIdentifier=[inTableColumn identifier];
		
		if ([tColumnIdentifier isEqualToString:ICDOCUMENT_LANGUAGE])
		{
			NSDictionary * tDictionary;
		
			tDictionary=[cachedMessages_ objectAtIndex:inRowIndex];
		
			[cell selectItemWithTitle:[tDictionary objectForKey:ICDOCUMENT_LANGUAGE]];
		}
	}
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
    SEL tAction=[inMenuItem action];
	
	if (tAction==@selector(nothing:))
	{
		NSInteger tSelectedCount;
		
		tSelectedCount=[IBrequirementMessagesArray_ numberOfSelectedRows];
		
		if (tSelectedCount!=1)
		{
			return NO;
		}
		else
		{
			NSString * tCurrentLanguageName;
			NSString * tTitle;
			
			tCurrentLanguageName=[[cachedMessages_ objectAtIndex:[IBrequirementMessagesArray_ selectedRow]] objectForKey:ICDOCUMENT_LANGUAGE];
			
			tTitle=[inMenuItem title];
			
			if ([tTitle isEqualToString:tCurrentLanguageName]==NO)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[cachedMessages_ objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tLocalizationDictionary;
					
					while (tLocalizationDictionary=[tEnumerator nextObject])
					{
						NSString * tExistingLanguage;
						
						tExistingLanguage=[tLocalizationDictionary objectForKey:ICDOCUMENT_LANGUAGE];
						
						if ([tExistingLanguage isEqualToString:tTitle]==YES)
						{
							return NO;
						}
					}
				}
			}
		}
	}
    
    return YES;
}

#pragma mark -

- (IBAction) addRequirementMessage:(id) sender
{
	if (cachedMessages_!=nil)
	{
		NSMutableDictionary * tNewLocalizedDescriptionDictionary;
		NSString * tLanguageName;
	
		tLanguageName=[ICLocalizationUtilities firstNewLanguageInLocalizations:cachedMessages_];
		
		if (tLanguageName!=nil)
		{
			tNewLocalizedDescriptionDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tLanguageName,ICDOCUMENT_LANGUAGE,
																						   @"",ICDOCUMENT_VALUE,
																						   nil];
			
			if (tNewLocalizedDescriptionDictionary!=nil)
			{
				NSUInteger tRowIndex;
				
				[cachedMessages_ addObject:tNewLocalizedDescriptionDictionary];
				
				[cachedMessages_ sortUsingSelector:@selector(compareLocalizationLanguage:)];
				
				[IBrequirementMessagesArray_ reloadData];
				
				tRowIndex=[cachedMessages_ indexOfObject:tNewLocalizedDescriptionDictionary];
				
				if (tRowIndex!=NSNotFound)
				{
					[IBrequirementMessagesArray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
					[IBrequirementMessagesArray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
				}
			}
		}
		else
		{
			// A COMPLETER
		}
	}
}

@end
