/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementProjectEditorController.h"

#import "ICLocalizationUtilities.h"

#import "ICLanguagePopupButtonCell.h"

#import "ICDocument.h"

#import "ICDocumentSecondaryController.h"

#import "ICRequirementProjectEditorBehaviorController_Volume.h"

#import "ICRequirementProjectEditorBehaviorController_Installation.h"

@implementation ICRequirementProjectEditorController

+ (ICRequirementProjectEditorController *) newController
{
	ICRequirementProjectEditorController * nController;
    
    nController=[ICRequirementProjectEditorController alloc];
    
    if (nController!=nil)
    {
        if ([NSBundle loadNibNamed:@"ICRequirementProjectEditor" owner:nController]==NO)
        {
            NSBeep();
            
            NSRunCriticalAlertPanel([NSString stringWithFormat:NSLocalizedString(@"Packages can't find the \"%@\".nib resource file.",@"No comment"),@"ICRequirementProjectEditor"],NSLocalizedString(@"Packages will stop running to avoid improper behavior.\nYou should check that the application has not been corrupted.",@"No comment"), nil,nil,nil);
            
            [NSApp terminate:nil];
        }
    }
    
    return nController;
}

- (void) showEditorForWindow:(NSWindow *) inWindow withDictionary:(NSMutableDictionary *) inDictionary delegate:(id) inDelegate
{
	cachedRequirementCheckType_=-100;
	
	[super showEditorForWindow:inWindow withDictionary:inDictionary delegate:inDelegate];
}

- (void) switchIdentifierView
{
	NSView * tPreviousKeyView;
	
	if (currentUIController_!=nil)
	{
		[[NSNotificationCenter defaultCenter] removeObserver:self
														name:ICREQUIREMENTPLUGIN_NOTIFICATION_TYPE_DIDCHANGE 
													  object:currentUIController_];
	}
	
	[super switchIdentifierView];
	
	if (currentUIController_!=nil)
	{
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(requirementCheckTypeDidChange:)
													 name:ICREQUIREMENTPLUGIN_NOTIFICATION_TYPE_DIDCHANGE 
												   object:currentUIController_];
	
		[self requirementCheckTypeDidChange:nil];
	}
	
	tPreviousKeyView=[currentUIController_ previousKeyView];
	
	if (tPreviousKeyView!=nil)
	{
		[[currentBehaviorUIController_ arrayOutlet] setNextKeyView:tPreviousKeyView];
		
		[currentUIController_ setNextKeyView:[currentBehaviorUIController_ arrayOutlet]];
		
		[IBwindow_ makeFirstResponder:tPreviousKeyView];
	}
	else
	{
		[IBwindow_ makeFirstResponder:[currentBehaviorUIController_ arrayOutlet]];
	}
}

- (void) showBehaviorViewForCheckType:(int) inRequirementCheckType
{
	[IBwindow_ setContentMinSize:NSMakeSize(defaultContentWidth_,50.0f)];
	
	if (currentBehaviorUIController_!=nil)
	{
		if ([currentBehaviorUIController_ viewCanBeRemoved:self]==NO)
		{
			return;
		}
		
		[currentBehaviorUIController_ viewWillBeRemoved:self];
		
		if (currentBehaviorView_!=nil)
		{
			[currentBehaviorView_ removeFromSuperview];
			
			currentBehaviorView_=nil;
		}
		
		[currentBehaviorUIController_ release];
		
		currentBehaviorUIController_=nil;
	}
	
	if (inRequirementCheckType==IC_REQUIREMENT_CHECK_TYPE_TARGET)
	{
		currentBehaviorUIController_=[ICRequirementProjectEditorBehaviorController_Volume controller];
	}
	else if (inRequirementCheckType==IC_REQUIREMENT_CHECK_TYPE_INSTALLATION)
	{
		currentBehaviorUIController_=[ICRequirementProjectEditorBehaviorController_Installation controller];
	}
	else
	{
		// A COMPLETER
	}
	
	if (currentBehaviorUIController_!=nil)
	{
		NSRect tBounds;
		NSRect tCurrentViewBounds;
		NSRect tOldWindowFrame;
		NSRect tNewWindowFrame;
		NSRect tComputeRect;
		
		[currentBehaviorUIController_ retain];
		
		currentBehaviorView_=[currentBehaviorUIController_ view];
		
		tBounds=[IBbehaviorView_ bounds];
		
		tCurrentViewBounds=[currentBehaviorView_ bounds];
		
		tCurrentViewBounds.size.width=NSWidth(tBounds);
		
		[currentBehaviorView_ setFrame:tCurrentViewBounds];
		
		tOldWindowFrame=[IBwindow_ frame];
		
		tComputeRect=NSMakeRect(0,0,0,NSHeight(tCurrentViewBounds)-NSHeight(tBounds));
		
		tComputeRect=[NSWindow frameRectForContentRect:tComputeRect styleMask:NSBorderlessWindowMask];
		
		tNewWindowFrame.size=NSMakeSize(tOldWindowFrame.size.width,NSHeight(tOldWindowFrame)+NSHeight(tComputeRect));
		
		tNewWindowFrame.origin.x=NSMinX(tOldWindowFrame);
		tNewWindowFrame.origin.y=NSMaxY(tOldWindowFrame)-NSHeight(tNewWindowFrame);
		
		// Initialize View widgets
		
		[currentBehaviorUIController_ initializeWithDictionary:dictionary_ parentController:self];
		
		[IBbehaviorView_ addSubview:currentBehaviorView_];
		
		[IBbehaviorView_ setAutoresizingMask:NSViewWidthSizable+NSViewHeightSizable];
		
		[IBrequirementView_ setAutoresizingMask:NSViewMinYMargin+NSViewWidthSizable];
		
		[IBcheckTypeView_ setAutoresizingMask:NSViewMinYMargin+NSViewWidthSizable];
		
		[IBwindow_ setFrame:tNewWindowFrame display:YES animate:NO];
	
		[IBrequirementView_ setAutoresizingMask:NSViewWidthSizable+NSViewHeightSizable];
		
		[IBcheckTypeView_ setAutoresizingMask:NSViewMaxYMargin+NSViewWidthSizable];
		
		[IBbehaviorView_ setAutoresizingMask:NSViewMaxYMargin+NSViewWidthSizable];
	}
	else
	{
		NSLog(@"[ICRequirementProjectEditorController showBehaviorViewForCheckType:] Missing controller for Check Type: %d",inRequirementCheckType);
	}
	
	[self updateMinMaxWindowSize];
}

- (IBAction) switchRequirementCheckType:(id) sender
{
	if (dictionary_!=nil)
	{
		int tRequirementCheckType;
		
		tRequirementCheckType=([sender state]==NSOnState) ? IC_REQUIREMENT_CHECK_TYPE_TARGET : IC_REQUIREMENT_CHECK_TYPE_INSTALLATION;
		
		if (cachedRequirementCheckType_!=tRequirementCheckType)
		{
			[dictionary_ setObject:[NSNumber numberWithInt:tRequirementCheckType] forKey:IC_REQUIREMENT_CHECK_TYPE];
	
			cachedRequirementCheckType_=tRequirementCheckType;
			
			// Update Behavior View
		
			[self showBehaviorViewForCheckType:cachedRequirementCheckType_];
		}
	}
}

#pragma mark -

- (void) requirementCheckTypeDidChange:(NSNotification *) inNotification
{
	int tRequirementCheckType;
	
	tRequirementCheckType=[currentUIController_ requirementType];
	
	if (tRequirementCheckType==IC_REQUIREMENT_CHECK_TYPE_UNDEFINED)
	{
		NSNumber * tNumber;
		
		[IBrequirementTypeCheckBox_ setEnabled:YES];
		
		tRequirementCheckType=IC_REQUIREMENT_CHECK_TYPE_INSTALLATION;
			
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_CHECK_TYPE];
		
		if (tNumber!=nil)
		{
			tRequirementCheckType=[tNumber intValue];
		}
		else
		{
			[dictionary_ setObject:[NSNumber numberWithInt:tRequirementCheckType] forKey:IC_REQUIREMENT_CHECK_TYPE];
		}
		
		if (tRequirementCheckType==IC_REQUIREMENT_CHECK_TYPE_TARGET)
		{
			[IBrequirementTypeCheckBox_ setState:NSOnState];
		}
		else if (tRequirementCheckType==IC_REQUIREMENT_CHECK_TYPE_INSTALLATION)
		{
			[IBrequirementTypeCheckBox_ setState:NSOffState];
		}
		else
		{
			NSLog(@"[ICRequirementProjectEditorController requirementCheckTypeDidChange:] Unsupported Requirement Check Type");
		}
	}
	else
	{
		[IBrequirementTypeCheckBox_ setEnabled:NO];
		
		[dictionary_ setObject:[NSNumber numberWithInt:tRequirementCheckType] forKey:IC_REQUIREMENT_CHECK_TYPE];
			
		if (tRequirementCheckType==IC_REQUIREMENT_CHECK_TYPE_TARGET)
		{
			[IBrequirementTypeCheckBox_ setState:NSOnState];
		}
		else if (tRequirementCheckType==IC_REQUIREMENT_CHECK_TYPE_INSTALLATION)
		{
			[IBrequirementTypeCheckBox_ setState:NSOffState];
		}
		else
		{
			NSLog(@"[ICRequirementProjectEditorController requirementCheckTypeDidChange:] Unsupported Requirement Check Type");
		}
	}
	
	
	if (cachedRequirementCheckType_!=tRequirementCheckType)
	{
		cachedRequirementCheckType_=tRequirementCheckType;
		
		// Update Behavior View
				
		[self showBehaviorViewForCheckType:cachedRequirementCheckType_];
	}
	else
	{
		[self updateMinMaxWindowSize];
	}
}

@end
