/*
 Copyright (c) 2008-2010, Stephane Sudre
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 - Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ICSmartLocationDetector.h"

#include <sys/stat.h>

@implementation ICSmartLocationDetector

+ (NSArray *) guessPotentialInstallationLocationsForFileAtPath:(NSString *) inPath
{
	NSMutableArray * tPotentialLocations=nil;

	if (inPath!=nil)
	{
		struct stat tStat;
		
		if (lstat([inPath fileSystemRepresentation], &tStat)==0)
		{
			NSBundle * tBundle;
			mode_t tMode;
			NSString * tParentFolder;
			NSString * tExtension;
			
			tMode=(tStat.st_mode & S_IFMT);
			
			switch (tMode)
			{
				case S_IFDIR:
				
					tBundle=[NSBundle bundleWithPath:inPath];
					
					if (tBundle!=nil)
					{
						// A Bundle
						
						tExtension=[inPath pathExtension];
						
						tParentFolder=[inPath stringByDeletingLastPathComponent];
							
						tPotentialLocations=[NSMutableArray arrayWithObject:tParentFolder];
						
						if (tExtension!=nil && [tExtension length]>0)
						{
							// With an extension
							
							if ([tExtension caseInsensitiveCompare:@"app"]==NSOrderedSame)
							{
								// An application
								
								if ([tParentFolder hasPrefix:@"/Applications/Utilities"]==YES)
								{
									[tPotentialLocations addObject:@"/Applications/Utilities/"];
								}
								else
								{
									[tPotentialLocations addObject:@"/Applications/"];
								}
							}
							else if ([tExtension caseInsensitiveCompare:@"prefPane"]==NSOrderedSame)
							{
								// A Preference Pane
								
								[tPotentialLocations addObject:@"/Library/PreferencePanes/"];
							}
							else if ([tExtension caseInsensitiveCompare:@"framework"]==NSOrderedSame)
							{
								// A Framework
								
								[tPotentialLocations addObject:@"/Library/Frameworks/"];
							}
							else if ([tExtension caseInsensitiveCompare:@"saver"]==NSOrderedSame)
							{
								// A Screen Saver
								
								[tPotentialLocations addObject:@"/Library/Screen Savers/"];
							}
							else if ([tExtension caseInsensitiveCompare:@"wdgt"]==NSOrderedSame)
							{
								// A Dashboard Widget
								
								[tPotentialLocations addObject:@"/Library/Widgets/"];
							}
							else if ([tExtension caseInsensitiveCompare:@"webplugin"]==NSOrderedSame)
							{
								// A WebKit plugin
								
								[tPotentialLocations addObject:@"/Library/Internet Plug-Ins/"];
							}
							else if ([tExtension caseInsensitiveCompare:@"service"]==NSOrderedSame)
							{
								// An Internet Plug-in
								
								[tPotentialLocations addObject:@"/Library/Services/"];
							}
							else if ([tExtension caseInsensitiveCompare:@"service"]==NSOrderedSame)
							{
								// A Service
								
								[tPotentialLocations addObject:@"/Library/Services/"];
							}
							else if ([tExtension caseInsensitiveCompare:@"component"]==NSOrderedSame)
							{
								// A QuickTime plugin?
								
								[tPotentialLocations addObject:@"/Library/QuickTime/"];
								
								[tPotentialLocations addObject:@"/Library/Components/"];
							}
							else if ([tExtension caseInsensitiveCompare:@"dict"]==NSOrderedSame)
							{
								// A Dictionary base
								
								[tPotentialLocations addObject:@"/Library/Dictionaries/"];
							}
							else if ([tExtension caseInsensitiveCompare:@"mdimporter"]==NSOrderedSame)
							{
								// A Spotlight importer
								
								[tPotentialLocations addObject:@"/Library/Spotlight"];
							}
							else if ([tExtension caseInsensitiveCompare:@"colorPicker"]==NSOrderedSame)
							{
								// A Color Picker
								
								[tPotentialLocations addObject:@"/Library/ColorPickers"];
							}
							else if ([tExtension caseInsensitiveCompare:@"ApertureEdit"]==NSOrderedSame ||
									 [tExtension caseInsensitiveCompare:@"ApertureExport"]==NSOrderedSame)
							{
								// An Aperture Plug-in
								
								[tPotentialLocations addObject:@"/Library/Application Support/Aperture/Plug-Ins"];
							}
						}
					}
					else
					{
						// Just a folder
					}
					
					break;
			
				case S_IFREG:
					
					tExtension=[inPath pathExtension];
					
					tParentFolder=[inPath stringByDeletingLastPathComponent];
					
					tPotentialLocations=[NSMutableArray arrayWithObject:tParentFolder];
					
					if (tExtension!=nil && [tExtension length]>0)
					{
						// With an extension
						
						if ([tExtension caseInsensitiveCompare:@"ttf"]==NSOrderedSame ||
							[tExtension caseInsensitiveCompare:@"dfont"]==NSOrderedSame)
						{
							// A Font
							
							[tPotentialLocations addObject:@"/Library/Fonts/"];
						}
						else if ([tExtension caseInsensitiveCompare:@"keychain"]==NSOrderedSame)
						{
							// A Keychain file
							
							[tPotentialLocations addObject:@"/Library/Keychains/"];
						}
					}
					
					break;
					
				case S_IFLNK:
					
					tParentFolder=[inPath stringByDeletingLastPathComponent];
						
					tPotentialLocations=[NSMutableArray arrayWithObject:tParentFolder];
					
					break;
			}
			
			if (tPotentialLocations!=nil)
			{
				[tPotentialLocations addObject:@"/"];
			}
		}
	}
	
	return tPotentialLocations;
}

+ (BOOL) canPathBeCreated:(NSString *) inPath
{
	if ([inPath hasPrefix:@"/Applications"]==YES ||
		[inPath hasPrefix:@"/Library"]==YES ||
		[inPath hasPrefix:@"/usr/local"]==YES)
	{
		return YES;
	}

	return NO;
}

@end
