/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICSquareReferencedPopupButton.h"

#import "ICReferencedUtilities.h"

#import "ICDocumentFormat+MinimumConstants.h"

@implementation ICSquareReferencedPopupButtonCell

+ (NSImage *) imageForPathType:(NSInteger) inPathType;
{
	NSImage * tImage=nil;

	if (inPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
	{
		static NSImage * tRelativePathImage=nil;
		NSString * tPath;
		
		if (tRelativePathImage==nil)
		{
			tPath=[[NSBundle mainBundle] pathForResource:@"RelativeSquareSmall" ofType:@"tif"];
			
			if (tPath!=nil)
			{
				tRelativePathImage=[[NSImage alloc] initWithContentsOfFile:tPath];
			}
		}
		
		tImage=tRelativePathImage;
	}
	else if (inPathType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_REFERENCE_FOLDER)
	{
		static NSImage * tReferencePathImage=nil;
		NSString * tPath;
		
		if (tReferencePathImage==nil)
		{
			tPath=[[NSBundle mainBundle] pathForResource:@"ReferenceSquareSmall" ofType:@"tif"];
			
			if (tPath!=nil)
			{
				tReferencePathImage=[[NSImage alloc] initWithContentsOfFile:tPath];
			}
		}
		
		tImage=tReferencePathImage;
	}
	else if (inPathType==ICDOCUMENT_PATH_TYPE_GLOBAL)
	{
		static NSImage * tAbsolutePathImage=nil;
		NSString * tPath;
		
		if (tAbsolutePathImage==nil)
		{
			tPath=[[NSBundle mainBundle] pathForResource:@"AbsoluteSquareSmall" ofType:@"tif"];
			
			if (tPath!=nil)
			{
				tAbsolutePathImage=[[NSImage alloc] initWithContentsOfFile:tPath];
			}
		}
		
		tImage=tAbsolutePathImage;
	}

	return tImage;
}


- (void) drawWithFrame:(NSRect) inFrame inView:(NSView *) inControlView
{
	NSMenuItem * tMenuItem;
	
	tMenuItem=[self selectedItem];
	
	if (tMenuItem!=nil)
	{
		NSInteger tTag;
		NSImage * tImage;
		
		tTag=[tMenuItem tag];
		
		tImage=[ICSquareReferencedPopupButtonCell imageForPathType:tTag];
		
		if (tImage!=nil)
		{
			[tImage compositeToPoint:NSMakePoint(0,NSMaxY(inFrame)) operation:NSCompositeSourceOver];
		}
	}
}

@end


@implementation ICSquareReferencedPopupButton

+ (Class) cellClass
{
    return [ICSquareReferencedPopupButtonCell class];
}

+ (void) setCellClass:(Class) inClass
{
}

#pragma mark -

- (id)initWithCoder:(NSCoder *)aDecoder
{
    ICSquareReferencedPopupButtonCell * tCell;
	NSRect tFrame;
	
    self=[super initWithCoder:aDecoder];
    
	tFrame=[self frame];
	
	tFrame.size.height=20.0f;
	
	[self setFrame:tFrame];
	
    tCell=[[ICSquareReferencedPopupButtonCell alloc] initTextCell:@""];
    
    if (tCell!=nil)
	{
		NSPopUpButtonCell * tOldCell;
		
		tOldCell=[self cell];
		
		[tCell setControlSize:[tOldCell controlSize]];
    
		[tCell setEnabled:YES];
		
		[tCell setFont:[tOldCell font]];
		
		[tCell setMenu:[ICReferencedUtilities referenceStyleMenuForTarget:[self target] andAction:[self action]]];
		
		[self setCell:tCell];
		
		[tCell release];
    }
	
    return self;
}

@end
