/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICTemplateUtilities.h"

#import "NSString+Karelia.h"

@implementation ICTemplateUtilities

+ (id) resolveHierarchy:(id) inHierarchy withKeywordDictionary:(NSDictionary *) inKWDictionary
{
    if ([inHierarchy isKindOfClass:[NSArray class]]==YES)
    {
        return [ICTemplateUtilities resolveArray:inHierarchy withKeywordDictionary:inKWDictionary];
    }
    else
    if ([inHierarchy isKindOfClass:[NSDictionary class]]==YES)
    {
        return [ICTemplateUtilities resolveDictionary:inHierarchy withKeywordDictionary:inKWDictionary];
    }
    
    return nil;
}

+ (NSMutableArray *) resolveArray:(NSArray *) inArray withKeywordDictionary:(NSDictionary *) inKWDictionary
{
    NSMutableArray * tMutableArray;
    NSUInteger i,tCount;
    
    tMutableArray=[[inArray mutableCopy] autorelease];
    
    tCount=[tMutableArray count];
    
    for(i=0;i<tCount;i++)
    {
        id tObject;
        
        tObject=[tMutableArray objectAtIndex:i];
        
        if ([tObject isKindOfClass:[NSArray class]]==YES)
        {
            [tMutableArray replaceObjectAtIndex:i
                                     withObject:[ICTemplateUtilities resolveArray:tObject withKeywordDictionary:inKWDictionary]];
        }
        else
        if ([tObject isKindOfClass:[NSDictionary class]]==YES)
        {
            [tMutableArray replaceObjectAtIndex:i
                                     withObject:[ICTemplateUtilities resolveDictionary:tObject withKeywordDictionary:inKWDictionary]];
        }
        else
        if ([tObject isKindOfClass:[NSString class]]==YES)
        {
            [tMutableArray replaceObjectAtIndex:i
                                     withObject:[tObject replaceAllTextBetweenString:@"%%"
                                                                           andString:@"%%"
                                                                      fromDictionary:inKWDictionary]];
        }
    }
    
    return tMutableArray;
}

+ (NSMutableDictionary *) resolveDictionary:(NSDictionary *) inDictionary withKeywordDictionary:(NSDictionary *) inKWDictionary
{
    NSMutableDictionary * tMutableDictionary;
    NSArray * tKeysArray;
    NSUInteger i,tCount;
    
    tMutableDictionary=[inDictionary mutableCopy];
    
    tKeysArray=[tMutableDictionary allKeys];
    
    tCount=[tKeysArray count];
    
    for(i=0;i<tCount;i++)
    {
        id tObject;
        id tKey;
        
        tKey=[tKeysArray objectAtIndex:i];
        
        tObject=[tMutableDictionary objectForKey:tKey];
        
        if ([tObject isKindOfClass:[NSArray class]]==YES)
        {
            [tMutableDictionary setObject:[ICTemplateUtilities resolveArray:tObject withKeywordDictionary:inKWDictionary]
                                   forKey:tKey];
        }
        else
        if ([tObject isKindOfClass:[NSDictionary class]]==YES)
        {
            [tMutableDictionary setObject:[ICTemplateUtilities resolveDictionary:tObject withKeywordDictionary:inKWDictionary]
                                   forKey:tKey];
        }
        else
        if ([tObject isKindOfClass:[NSString class]]==YES)
        {
            [tMutableDictionary setObject:[tObject replaceAllTextBetweenString:@"%%"
                                                                     andString:@"%%"
                                                                fromDictionary:inKWDictionary]
                                   forKey:tKey];
        }
    }
    
    return [tMutableDictionary autorelease];
}

@end
