/*
Copyright (c) 2007-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICTitleUtilities.h"

@implementation ICTitleUtilities

+ (NSMutableArray *) defaultLocalizationsArrayForDocument:(NSDocument *) inDocument
{
	NSMutableArray * tDefaultLocalizationsArray=nil;
	
	if (inDocument!=nil)
	{
		tDefaultLocalizationsArray=[NSMutableArray array];
			
		if (tDefaultLocalizationsArray!=nil)
		{
			NSMutableDictionary * tDefaultTitleDictionary;
			NSString * tFileName;
			
			tFileName=[[[inDocument fileName] lastPathComponent] stringByDeletingPathExtension];
			
			tDefaultTitleDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:@"English",ICDOCUMENT_LANGUAGE,
																					  tFileName,ICDOCUMENT_VALUE,
																					  nil];
		
			if (tDefaultTitleDictionary!=nil)
			{
				[tDefaultLocalizationsArray addObject:tDefaultTitleDictionary];
			}
			else
			{
				// A COMPLETER
			
				tDefaultLocalizationsArray=nil;
			}
		}
	}
	
	return tDefaultLocalizationsArray;
}

+ (NSArray *) clearedLocalizationsValue:(NSMutableArray *) inArray
{
	NSMutableArray * tClearedArray=nil;
	
	if (inArray!=nil && [inArray count]>0)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tDictionary;
			
			tClearedArray=[NSMutableArray array];
			
			while (tDictionary=[tEnumerator nextObject])
			{
				NSString * tString;
				
				tString=[tDictionary objectForKey:ICDOCUMENT_VALUE];
				
				if (tString!=nil)
				{
					if ([tString length]>0)
					{
						[tClearedArray addObject:tDictionary];
					}
				}
			}
		}
	}
	
	return tClearedArray;
}

+ (NSMutableDictionary *) defaultTitleDictionaryForDocument:(NSDocument *) inDocument
{
	NSMutableDictionary * tDefaultDictionary=nil;
	
	if (inDocument!=nil)
	{
		tDefaultDictionary=[NSMutableDictionary dictionary];
		
		if (tDefaultDictionary!=nil)
		{
			NSMutableArray * tDefaultLocalizations;
			
			tDefaultLocalizations=[ICTitleUtilities defaultLocalizationsArrayForDocument:inDocument];
			
			if (tDefaultLocalizations!=nil)
			{
				[tDefaultDictionary setObject:tDefaultLocalizations forKey:ICDOCUMENT_LOCALIZATIONS];
			}
			else
			{
				tDefaultDictionary=nil;
				
				// A COMPLETER
			}
		}
	}
	
	return tDefaultDictionary;
}

+ (void) notifyTitleDidChangeWithSender:(ICDocumentSecondaryController *) sender
{
	[[NSNotificationCenter defaultCenter] postNotificationName:ICPRESENTATION_NOTIFICATION_TITLE_SETTINGS_DID_CHANGE
														object:[sender document]
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:sender,ICPRESENTATION_NOTIFICATION_SENDER,nil]];
}

@end
