/*
Copyright (c) 2008, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectAssistantController.h"

#import "ICProjectAssistantTypeChoiceController.h"s

#import "ICPreferences+Constants.h"

#import "ICPreferencesUtilities.h"

#import "ICTemplateUtilities.h"

#import "ICUUIDFactory.h"

@implementation ICProjectAssistantController

+ (id) sharedProjectAssistantController
{
    static ICProjectAssistantController * sProjectAssistantController=nil;
    
    if (sProjectAssistantController==nil)
    {
        sProjectAssistantController=[ICProjectAssistantController alloc];
		
		if (sProjectAssistantController!=nil)
		{
			if ([NSBundle loadNibNamed:@"ProjectCreationAssistant" owner:sProjectAssistantController]==NO)
			{
				NSLog(@"A problem occured while loading the nib file \"ProjectCreationAssistant\".");
				
				NSBeep();
				
				return nil;
			}
		}
		else
		{
			// A COMPLETER
		}
    }
    
    return sProjectAssistantController;
}

- (void) awakeFromNib
{
	BOOL tShowOnLaunch;
	
	defaults_=[NSUserDefaults standardUserDefaults];

	fileManager_=[NSFileManager defaultManager];
	
	// Initialize the Start and Stop controller
    
    [typeChoiceController_ setMainController:self];
    
    [stopPaneController_ setMainController:self];
	
	// Show On Launch
	
	tShowOnLaunch=[defaults_ boolForKey:ICPREFERENCES_DONT_SHOW_ON_LAUNCH];
	
	[IBshowOnLaunchCheckBox_ setState:(tShowOnLaunch ? NSOffState : NSOnState)];
}

- (void) dealloc
{
    [projectAssistantInfoArray_ release];
    
    [projectAssistantControllerArray_ release];

    [super dealloc];
}

#pragma mark -

- (void) createNewProject
{
    // Create the new Process Engine
    
    [assistantEngine_ release];
    
    assistantEngine_=[ICProjectAssistantEngine new];
    
    // Clean the UI
    
    if (currentPaneView_!=nil)
    {
        [currentPaneView_ removeFromSuperview];
    }
    
    [IBnextButton_ setTitle:NSLocalizedString(@"Next",@"No comment")];
    
    [IBpreviousButton_ setEnabled:NO];
    [IBnextButton_ setEnabled:NO];
    
    currentPaneIndex_=0;
    
    currentPaneController_=typeChoiceController_;
    
    currentPaneView_=[currentPaneController_ view];
	
	[currentPaneView_ setFrame:[IBtopView_ bounds]];
	
	[IBtopView_ addSubview:currentPaneView_];
	
	[typeChoiceController_ initPaneWithEngine:assistantEngine_];
    
    [NSApp runModalForWindow:IBwindow_];
}

#pragma mark -

- (ICProjectAssistantPaneController *) paneControllerAtIndex:(NSUInteger) inIndex
{
    NSUInteger tArrayCount=[projectAssistantControllerArray_ count];
    
    if (inIndex!=NSNotFound && inIndex<tArrayCount)
    {
        return [projectAssistantControllerArray_ objectAtIndex:inIndex];
    }
    
    return nil;
}

- (NSUInteger) indexOfPaneControllerWithName:(NSString *) inName
{
    NSUInteger tArrayCount=[projectAssistantControllerArray_ count];
    NSUInteger i;
    
    for(i=0;i<tArrayCount;i++)
    {
        if ([[[projectAssistantInfoArray_ objectAtIndex:i] objectForKey:@"Name"] isEqualToString:inName]==YES)
        {
            return i;
        }
    }
    
    return NSNotFound;
}

#pragma mark -

- (IBAction) cancel:(id) sender
{
    [NSApp stopModal];
    
    [IBwindow_ orderOut:self];
}

- (IBAction) next:(id) sender
{
    if ([currentPaneController_ checkPaneValuesWithEngine:assistantEngine_]==YES)
    {
        if (currentPaneIndex_==([projectAssistantControllerArray_ count]-1))
        {
            // This is the last pane
                
            [self finishSetUp:nil];
        }
        else
        {
            ICProjectAssistantPaneController * tPaneController;
            NSUInteger tIndex;
            NSString * tNextPaneName;
            
            if (currentPaneIndex_==0)
            {
                // Look for the Plugin.bundle bundle
            
                NSDictionary * tTemplateDictionary;
                NSString * tPlugInPath;
                NSDictionary * tDictionary;
                
                // Release memory
                
                [projectAssistantInfoArray_ release];
                [projectAssistantControllerArray_ release];
                
                // Initialize projectAssistantInfoArray_
                
                projectAssistantInfoArray_=[[NSMutableArray alloc] initWithCapacity:2];
                
                tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:@"",@"Class Name",
                                                                    @"",@"Nib Name",
                                                                    @"Start",@"Name",
                                                                    nil];
                
                [projectAssistantInfoArray_ addObject:tDictionary];
                
                tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:@"",@"Class Name",
                                                                    @"",@"Nib Name",
                                                                    @"Stop",@"Name",
                                                                    nil];
                
                [projectAssistantInfoArray_ addObject:tDictionary];
                
                // Initialize projectAssistantControllerArray_
                
                projectAssistantControllerArray_=[[NSMutableArray alloc] initWithCapacity:2];
                
                [projectAssistantControllerArray_ addObject:typeChoiceController_];
                [projectAssistantControllerArray_ addObject:stopPaneController_];
                
                tTemplateDictionary=[assistantEngine_ templateDictionary];
                
                if (tTemplateDictionary!=nil)
				{
					NSBundle * tTemplateBundle;
					
					tTemplateBundle=[tTemplateDictionary objectForKey:ICPROJECT_TEMPLATE_BUNDLE];
					
					if (tTemplateBundle!=nil)
					{
						tPlugInPath=[tTemplateBundle pathForResource:@"Plugin" ofType:@"bundle"];
					
						if (tPlugInPath!=nil)
						{
							pluginBundle_=[NSBundle bundleWithPath:tPlugInPath];
                    
							if (pluginBundle_!=nil)
							{
								NSString * tPaneListPath;
								
								if ([pluginBundle_ load]==YES)
								{
									tPaneListPath=[pluginBundle_ pathForResource:@"PaneList" ofType:@"plist"];
								
									if (tPaneListPath!=nil)
									{
										NSArray * tArray;
										
										tArray=[NSArray arrayWithContentsOfFile:tPaneListPath];
										
										if (tArray!=nil)
										{
											NSUInteger i,tCount;
											NSUInteger tLast;
											
											tCount=[tArray count];
											
											tLast=[projectAssistantInfoArray_ count]-1;
											
											// Complete projectAssistantInfoArray_ and projectAssistantControllerArray_
											
											for(i=0;i<tCount;i++)
											{
												tDictionary=[tArray objectAtIndex:i];
												
												if (tDictionary!=nil)
												{
													id tAssistantPaneController;
													
													[projectAssistantInfoArray_ insertObject:tDictionary atIndex:tLast];
													
													tAssistantPaneController=[NSClassFromString([tDictionary objectForKey:@"Class Name"]) alloc];
													
													[tAssistantPaneController loadPaneNib:[tDictionary objectForKey:@"Nib Name"] withMainController:self];
													
													[projectAssistantControllerArray_ insertObject:tAssistantPaneController atIndex:tLast];
													
													[tAssistantPaneController release];
													
													tLast++;
												}
											}
										}
										else
										{
											NSLog(@"PaneList can't be read");
											
											// A COMPLETER
										}
									}
									else
									{
										NSLog(@"PaneList not found");
										
										// A COMPLETER
									}
								}
								else
								{
									NSLog(@"Bundle can't be loaded");
										
									// A COMPLETER
								}
							}
							else
							{
								NSLog(@"bundle can't be initialized");
								
								// A COMPLETER
							}
						}
						else
						{
							// A COMPLETER
						}
					}
				}
			}
            
            // Set next Pane for the first pane
            
            [typeChoiceController_ setNextPaneName:[[projectAssistantInfoArray_ objectAtIndex:1] objectForKey:@"Name"]];
            
            tNextPaneName=[currentPaneController_ nextPaneName];
            
            if (tNextPaneName!=nil)
            {
                tIndex=[self indexOfPaneControllerWithName:tNextPaneName];
            }
            else
            {
                tIndex=currentPaneIndex_+1;
            }
            
			[currentPaneController_ viewWillBeRemoved:self];
			
            tPaneController = [self paneControllerAtIndex:tIndex];
            
            if (tPaneController!=nil)
            {
                // Set up the new content of the box
                
				currentPaneController_=tPaneController;
				
                [currentPaneView_ removeFromSuperview];
                
				currentPaneView_=[currentPaneController_ view];
				
                [currentPaneView_ setFrame:[IBtopView_ bounds]];
	
				[IBtopView_ addSubview:currentPaneView_];
            }
            else
            {
                NSLog(@"The next pane was not found");
                return;
            }
            
            if (currentPaneIndex_==0)
            {
                [IBpreviousButton_ setEnabled:YES];
            }
            
            [currentPaneController_ setPreviousPaneIndex:currentPaneIndex_];
            
            currentPaneIndex_=tIndex;
            currentPaneController_=tPaneController;
            
            if (currentPaneIndex_==([projectAssistantInfoArray_ count] -1))
            {
                [IBnextButton_ setEnabled:NO];
                
                [IBnextButton_ setTitle:NSLocalizedString(@"Finish",@"No comment")];
            }
            else
            {
                [IBnextButton_ setEnabled:YES];
            }
            
            [currentPaneController_ initPaneWithEngine:assistantEngine_];
        }
    }
}

- (IBAction) previous:(id) sender
{
    ICProjectAssistantPaneController * tPaneController;
    NSUInteger tIndex;
    
    tIndex=[currentPaneController_ previousPaneIndex];
    
    if (tIndex==NSNotFound)
    {
        NSLog(@"No previous pane is defined for this pane");
        return;
    }
    
	[currentPaneController_ viewWillBeRemoved:self];
	
    tPaneController = [self paneControllerAtIndex:tIndex];
    
    if (tPaneController!=nil)
    {
        // Set up the new content of the box
        
        [currentPaneView_ removeFromSuperview];
        
		currentPaneController_=tPaneController;
		
        currentPaneView_=[currentPaneController_ view];
	
		[currentPaneView_ setFrame:[IBtopView_ bounds]];
	
		[IBtopView_ addSubview:currentPaneView_];
    }
    else
    {
        NSLog(@"The previous pane was not found");
		
        return;
    }
    
    currentPaneIndex_=tIndex;
    
    if (currentPaneIndex_!=([projectAssistantControllerArray_ count] -1))
    {
        [IBnextButton_ setEnabled:YES];
    
        [IBnextButton_ setTitle:NSLocalizedString(@"Next",@"No comment")];
    }
    
    if (currentPaneIndex_==0)
    {
        [IBpreviousButton_ setEnabled:NO];
    }
}

- (void) processPaneController:(ICProjectAssistantPaneController *) inPaneController withEngine:(id) inEngine
{
    ICProjectAssistantPaneController * tPaneController;
    NSUInteger tIndex;
    
    tIndex=[inPaneController previousPaneIndex];
    
    if (tIndex>0)
    {
        tPaneController = [self paneControllerAtIndex:tIndex];
        
        if (tPaneController!=nil)
        {
            [self processPaneController:tPaneController withEngine:inEngine];
        }
    }

    [inPaneController processWithEngine:inEngine];
}

-(void) finishSetUp:(id) sender
{
    NSString * tProjectDirectory;
    NSString * tProjectName;
    NSString * tNewPath;
    NSDictionary * tTemplateDictionary;
	
    tProjectName=[assistantEngine_ projectName];
    
    tProjectDirectory=[assistantEngine_ projectDirectory];
    
    tNewPath=[tProjectDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.pkgproj",tProjectName]];
    
    // Create the Project Folder
    
	tTemplateDictionary=[assistantEngine_ templateDictionary];
	
	if (tTemplateDictionary!=nil)
	{
		NSBundle * tTemplateBundle;
					
		tTemplateBundle=[tTemplateDictionary objectForKey:ICPROJECT_TEMPLATE_BUNDLE];
		
		if (tTemplateBundle!=nil)
		{
			NSString * tSourcePath;
		
			tSourcePath=[tTemplateBundle pathForResource:@"Template" ofType:@"pkgproj"];
	
			if ([fileManager_ fileExistsAtPath:tProjectDirectory]==NO)
			{
				if ([fileManager_ createDirectoryAtPath:tProjectDirectory attributes:nil]==NO)
				{
					NSBeep();
				
					NSBeginAlertSheet([NSString stringWithFormat:NSLocalizedString(@"Packages is not able to create the project directory at \"%@\".",@"No comment"),tProjectDirectory],
									  nil,
									  nil,
									  nil,
									  IBwindow_,
									  nil,
									  nil,
									  nil,
									  NULL,
									  NSLocalizedString(@"Check that you have the required write privileges",@"No comment"));
					
					return;
				}
			}
			
			if (NO==[fileManager_ copyPath:tSourcePath
									toPath:tNewPath
								   handler:nil])
			{
				NSBeep();
				
				NSBeginAlertSheet(NSLocalizedString(@"Packages is not able to duplicate the template.",@"No comment"),
								nil,
								nil,
								nil,
								IBwindow_,
								nil,
								nil,
								nil,
								NULL,
								[NSString stringWithFormat:NSLocalizedString(@"Check that you have write privileges for \"%@\".",@"No comment"),[tProjectDirectory stringByDeletingLastPathComponent]]);
			}
			else
			{
				NSDictionary * tProjectDictionary;
				NSDictionary * tKeywordsDictionary;
				NSMutableDictionary * tFileAttributesDictionary;
				NSString * tCompanyName=nil;
				NSString * tCompanyPackageIdentifier=nil;
				NSError * tError;
				NSDictionary * tTemplatesKeywords;
				
				[assistantEngine_ setFinalProjectPath:tNewPath];
				
				tTemplatesKeywords=[defaults_ dictionaryForKey:ICPREFERENCES_TEMPLATES_KEYWORDS];
				
				if (tTemplatesKeywords!=nil)
				{
					tCompanyPackageIdentifier=[tTemplatesKeywords objectForKey:ICPREFERENCES_TEMPLATES_COMPANY_IDENTIFIER_PREFIX];
					
					tCompanyName=[tTemplatesKeywords objectForKey:ICPREFERENCES_TEMPLATES_KEY_COMPANY_NAME];
				}
				
				if (tCompanyPackageIdentifier==nil)
				{
					tCompanyPackageIdentifier=[ICPreferencesUtilities defaultPackageIdentifierPrefix];
				}
				
				if (tCompanyName==nil)
				{
					tCompanyName=[ICPreferencesUtilities defaultCompanyName];
				}
				
				// Replace the %%MACRO%%
				
				
				tKeywordsDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tProjectName,@"PROJECT_NAME",
																			   tCompanyName,@"COMPANY_NAME",
																			   tCompanyPackageIdentifier,@"COMPANY_PACKAGE_IDENTIFIER",
																			   [NSString stringWithFormat:@"%d",[[NSCalendarDate calendarDate] yearOfCommonEra]],@"YEAR",
																			   tProjectDirectory,@"PROJECT_PATH",
																			   [ICUUIDFactory string],@"UUID",
																			   nil];
					
				tProjectDictionary=[NSDictionary dictionaryWithContentsOfFile:tNewPath];
				
				tProjectDictionary=[ICTemplateUtilities resolveDictionary:tProjectDictionary
													withKeywordDictionary:tKeywordsDictionary];
					
					
				[assistantEngine_ startProcessWithProjectDictionary:(NSMutableDictionary *) tProjectDictionary];
				
				[self processPaneController:currentPaneController_ withEngine:assistantEngine_];
				
				[assistantEngine_ endProcess];
				
				[tProjectDictionary writeToFile:tNewPath atomically:YES];
				
				tFileAttributesDictionary=[[fileManager_ fileAttributesAtPath:tNewPath traverseLink:NO] mutableCopy];
				
				[tFileAttributesDictionary setObject:[NSNumber numberWithBool:YES] forKey:NSFileExtensionHidden];

				[fileManager_ changeFileAttributes:tFileAttributesDictionary
											atPath:tNewPath];
				
				[tFileAttributesDictionary release];
																						
				[NSApp stopModal];

				[IBwindow_ orderOut:self];
				
				// Open the new project
				
				if ([[NSDocumentController sharedDocumentController] openDocumentWithContentsOfURL:[NSURL fileURLWithPath:tNewPath] display:YES error:&tError]==nil)
				{
					// A COMPLETER error on opening
				}
			}
		}
	}
}

- (void) setEnableNextButton:(BOOL) aBool
{
    if ([IBnextButton_ isEnabled]!=aBool)
    {
        [IBnextButton_ setEnabled:aBool];
    }
}

#pragma mark -

- (IBAction) switchShowOnLaunch:(id) sender
{
	[defaults_ setBool:([sender state]==NSOffState) forKey:ICPREFERENCES_DONT_SHOW_ON_LAUNCH];
}

#pragma mark -

- (void) upgradedDocument:(NSDocument *) inDocument didSave:(BOOL) inDidSave contextInfo:(void *) contextInfo
{
	if (inDidSave==YES)
	{
		NSURL * tURL;
		NSError * tError;
		
		tURL=[inDocument fileURL];
		
		// Open the modified file
		
		[inDocument close];
		
		// Open the new project
			
		if ([[NSDocumentController sharedDocumentController] openDocumentWithContentsOfURL:tURL display:YES error:&tError]==nil)
		{
			// A COMPLETER error on opening
		}
	}
	else
	{
		[inDocument close];
	
		// Show Alert
		
		// A COMPLETER
	}
}

@end
