/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectAssistantStopPaneController.h"
#import "ICFileNameFormatter.h"
#import "ICProjectAssistantEngine.h"
#import "ICProjectAssistantController.h"

#import "ICFilePathTextField.h"

@implementation ICProjectAssistantStopPaneController

- (void) dealloc
{
	[oldProjectName_ release];

	[super dealloc];
}

- (void) awakeFromNib
{
    ICFileNameFormatter * tFormatter;
    
    tFormatter=[ICFileNameFormatter new];
    
	if (tFormatter!=nil)
	{
		[tFormatter setCantStartWithADot:NO];
    
		[IBprojectName_ setFormatter:tFormatter];
    
		[tFormatter release];
	}
	
	oldProjectName_=[NSMutableString new];
}

- (void) initPaneWithEngine:(id) inEngine
{
    NSString * tString;
    
    tString=[inEngine projectName];
    
    if (tString==nil)
    {
        tString=@"";
		
		projectDirectoryWasChangedByUser_=NO;
		
		[mainController_ setEnableNextButton:NO];
		
		[IBtipLabel_ setStringValue:@""];
    }
    else
    {
        [mainController_ setEnableNextButton:([tString length]>0)];
    }
	
	[oldProjectName_ setString:tString];
    
    [IBprojectName_ setStringValue:tString];
    
    tString=[inEngine projectDirectory];
    
    if (tString==nil || [tString length]==0)
    {
        tString=@"~/";
		
		[mainController_ setEnableNextButton:NO];
		
		[IBtipLabel_ setStringValue:@""];
    }
    
    [IBprojectDirectory_ setStringValue:tString];
    
    [[IBprojectName_ window] makeFirstResponder:IBprojectName_];
}

#pragma mark -

- (void) controlTextDidEndEditing:(NSNotification *) inNotification
{
	if ([inNotification object]==IBprojectDirectory_)
	{
		NSString * tProjectDirectory;
		
		tProjectDirectory=[IBprojectDirectory_ stringValue];
		
		if ([tProjectDirectory hasSuffix:@"/"]==NO)
		{
			tProjectDirectory=[tProjectDirectory stringByAppendingString:@"/"];
			
			[IBprojectDirectory_ setStringValue:tProjectDirectory];
		}
	}
}

- (void) controlTextDidChange:(NSNotification *) inNotification
{
    NSString * tProjectName=nil;
	NSString * tProjectDirectory;
	
	tProjectName=[IBprojectName_ stringValue];
	
	tProjectDirectory=[IBprojectDirectory_ stringValue];
	
	// Update the tip label
	
	if ([inNotification object]==IBprojectName_ ||
        [inNotification object]==IBprojectDirectory_)
    {
        if ([[[[inNotification userInfo] objectForKey:@"NSFieldEditor"] string] length]==0)
        {
            [mainController_ setEnableNextButton:NO];
        }
        else
        {
            [mainController_ setEnableNextButton:YES];
        }
        
        // A COMPLETER (meilleure gestion de la chaine vide dans l'un des champs)
    }
	
	if ([inNotification object]==IBprojectName_)
	{
		if (projectDirectoryWasChangedByUser_==NO)
		{
			if ([oldProjectName_ length]>0)
			{
				NSString * tLastPathComponent;
				
				tLastPathComponent=[tProjectDirectory lastPathComponent];
				
				if ([tLastPathComponent isEqualToString:oldProjectName_])
				{
					tProjectDirectory=[[tProjectDirectory stringByDeletingLastPathComponent] stringByAppendingPathComponent:tProjectName];
				}
				else
				{
					tProjectDirectory=[tProjectDirectory stringByAppendingPathComponent:tProjectName];
				}
			}
			else
			{
				tProjectDirectory=[tProjectDirectory stringByAppendingPathComponent:tProjectName];
			}
			
			if ([tProjectDirectory hasSuffix:@"/"]==NO)
			{
				tProjectDirectory=[tProjectDirectory stringByAppendingString:@"/"];
			}
			
			[IBprojectDirectory_ setStringValue:tProjectDirectory];
			
			[oldProjectName_ setString:tProjectName];
		}
	}
	else if ([inNotification object]==IBprojectDirectory_)
	{
		projectDirectoryWasChangedByUser_=YES;
	}
	
	// Update the tip label
	
	if ([tProjectName length]==0 || [tProjectDirectory length]==0)
	{
		[IBtipLabel_ setStringValue:@""];
	}
	else
	{
		NSString * tTipMessage;
			
		tTipMessage=[NSString stringWithFormat:NSLocalizedString(@"The project directory %@ will be created if necessary, and the project file %@.packproj will be created therein.",@""),tProjectDirectory,tProjectName];
		
		[IBtipLabel_ setStringValue:tTipMessage];
	}
}

- (void)control:(NSControl *)control didFailToValidatePartialString:(NSString *)string errorDescription:(NSString *)error
{
    if (control==IBprojectName_)
    {
        NSBeep();
    }
}

#pragma mark -

- (IBAction) selectDirectory:(id) sender
{
    NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setCanChooseFiles:NO];
    [tOpenPanel setCanChooseDirectories:YES];
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    
    [tOpenPanel setCanCreateDirectories:YES];
    
    [tOpenPanel beginSheetForDirectory:[[IBprojectDirectory_ stringValue] stringByExpandingTildeInPath]
                                  file:nil
                                 types:nil
                        modalForWindow:[IBprojectName_ window]
                         modalDelegate:self
                        didEndSelector:@selector(assistantOpenPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) assistantOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(NSInteger)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        BOOL tOldValue;
		NSString * tDirectoryPath;
		
		tDirectoryPath=[[sheet directory] stringByAbbreviatingWithTildeInPath];
		
		if (projectDirectoryWasChangedByUser_==NO)
		{
			tDirectoryPath=[tDirectoryPath stringByAppendingPathComponent:[IBprojectName_ stringValue]];
		}
		
		if ([tDirectoryPath hasSuffix:@"/"]==NO)
		{
			tDirectoryPath=[tDirectoryPath stringByAppendingString:@"/"];
		}
		
		tOldValue=projectDirectoryWasChangedByUser_;
		
		[IBprojectDirectory_ setStringValue:tDirectoryPath];
		
		projectDirectoryWasChangedByUser_=tOldValue;
    }
}

#pragma mark -

- (BOOL) checkPaneValuesWithEngine:(id) inEngine
{
    NSString * tProjectDirectory;
    BOOL isDirectory;
    NSFileManager * tFileManager;
    NSString * tProjectName;
	NSString * tNewPath;
		
    tFileManager=[NSFileManager defaultManager];
    
    tProjectDirectory=[[IBprojectDirectory_ stringValue] stringByExpandingTildeInPath];
    
    if ([tFileManager fileExistsAtPath:tProjectDirectory isDirectory:&isDirectory]==NO)
    {
        NSString * tParentProjectDirectory;
		
		tParentProjectDirectory=[tProjectDirectory stringByDeletingLastPathComponent];
		
		if ([tFileManager fileExistsAtPath:tParentProjectDirectory isDirectory:&isDirectory]==NO)
		{
			NSBeep();
			
			NSBeginAlertSheet(NSLocalizedString(@"The parent directory of the project directory you specified does not exist.",@"No comment"),
								nil,
								nil,
								nil,
								[IBprojectDirectory_ window],
								nil,
								nil,
								nil,
								NULL,
								NSLocalizedString(@"Please select another location for the project.",@"No comment"));
		
			return NO;
		}
		else
		{
			if (isDirectory==NO)
			{
				NSBeep();
			
				NSBeginAlertSheet(NSLocalizedString(@"The parent directory of the project directory you specified is not a directory.",@"No comment"),
									nil,
									nil,
									nil,
									[IBprojectDirectory_ window],
									nil,
									nil,
									nil,
									NULL,
									NSLocalizedString(@"Please select another location for the project.",@"No comment"));
			
				return NO;
			}
		}
	}
    else
    {
        if (isDirectory==NO)
        {
            NSBeep();
        
            NSBeginAlertSheet(NSLocalizedString(@"A file with the same name as the project directory you specified already exists.",@"No comment"),
                                nil,
                                nil,
                                nil,
                                [IBprojectDirectory_ window],
                                nil,
                                nil,
                                nil,
                                NULL,
                                NSLocalizedString(@"Please select another location for the project.",@"No comment"));
        
			return NO;
		}
	}
   
	// Check the name of the file
		
	tProjectName=[IBprojectName_ stringValue];
		
	// Check that there's not already a file with that name
	
	tNewPath=[tProjectDirectory stringByAppendingPathComponent:tProjectName];
	
	if ([tFileManager fileExistsAtPath:tNewPath]==YES)
	{
		NSBeep();

		NSBeginAlertSheet(NSLocalizedString(@"A file with this name already exists at this location.",@"No comment"),
							nil,
							nil,
							nil,
							[IBprojectDirectory_ window],
							nil,
							nil,
							nil,
							NULL,
							NSLocalizedString(@"Please select another location or name for the project.",@"No comment"));
	}
	else
	{
		[inEngine setProjectName:tProjectName];

		[inEngine setProjectDirectory:tProjectDirectory];

		return YES;
	}
        
    return NO;
}

#pragma mark -

- (NSDragOperation) textField:(ICFilePathTextField *) inTextField validateDrop:(id <NSDraggingInfo>)info;
{
	/*NSPasteboard * tPasteBoard;
			
	tPasteBoard=[info draggingPasteboard];
	
	if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
	{
		NSArray * tArray;
			
		tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
					
		if (tArray!=nil)
		{
			if ([tArray count]==1)
			{
				NSString * tFilePath;
				BOOL isDirectory;
				
				tFilePath=[tArray objectAtIndex:0];
				
				if ([[NSFileManager defaultManager] fileExistsAtPath:tFilePath isDirectory:&isDirectory]==YES && isDirectory==YES)
				{
					return NSDragOperationCopy;
				}
			}
		}
	}*/
	
	return NSDragOperationNone;
}

- (BOOL) textField:(ICFilePathTextField *) inTextField acceptDrop:(id <NSDraggingInfo>)info;
{
    /*NSPasteboard * tPasteBoard;
			
	tPasteBoard=[info draggingPasteboard];
	
	if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
	{
		NSArray * tArray;
			
		tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
					
		if (tArray!=nil)
		{
			if ([tArray count]==1)
			{
				NSString * tFilePath;
				BOOL isDirectory;
				
				tFilePath=[tArray objectAtIndex:0];
				
				if ([[NSFileManager defaultManager] fileExistsAtPath:tFilePath isDirectory:&isDirectory]==YES && isDirectory==YES)
				{
					[IBprojectDirectory_ setStringValue:[tFilePath stringByAbbreviatingWithTildeInPath]];

					return YES;
				}
			}
		}
	}*/
		
	return NO;
}

@end
