/*
Copyright (c) 2008, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICProjectAssistantTypeChoiceController.h"
#import "ICProjectAssistantController.h"
#import "ICProjectAssistantEngine.h"

#import "ICProjectAssistantTemplateCell.h" 

#import "NSImage+GrayScale.h"

@implementation ICProjectAssistantTypeChoiceController

- (void) awakeFromNib
{
	templates_=[self newTemplatesArray];
	
	if (templates_!=nil)
	{
		// Populate the Matrix
	
		ICProjectAssistantTemplateCell * tPrototypeCell;
	
		tPrototypeCell=[[ICProjectAssistantTemplateCell alloc] initTextCell:@""];
		
		if (tPrototypeCell!=nil)
		{
			NSUInteger tCount,i;
		
			[tPrototypeCell setButtonType:NSOnOffButton];
			
			[IBmatrix_ setPrototype:tPrototypeCell];
			
			[tPrototypeCell release];
			
			tCount=[templates_ count];
			
			[IBmatrix_ removeRow:0];
			
			[IBmatrix_ renewRows:1 columns:tCount];
			
			[IBmatrix_ sizeToCells];
			
			for(i=0;i<tCount;i++)
			{
				NSDictionary * tTemplateDictionary;
				ICProjectAssistantTemplateCell * tTemplateCell;
				
				tTemplateDictionary=[templates_ objectAtIndex:i];
				
				tTemplateCell=(ICProjectAssistantTemplateCell *) [IBmatrix_ cellAtRow:0 column:i];
				
				if (tTemplateCell!=nil)
				{
					NSNumber * tNumber;
					
					// Tag
					
					[tTemplateCell setTag:i];
					
					// Title
					
					[tTemplateCell setTitle:[tTemplateDictionary objectForKey:ICPROJECT_TEMPLATE_NAME]];
					
					// Image
					
					[tTemplateCell setImage:[tTemplateDictionary objectForKey:ICPROJECT_TEMPLATE_ICON]];
					
					// Supported
					
					tNumber=[tTemplateDictionary objectForKey:ICPROJECT_TEMPLATE_SUPPORTED];
					
					if ([tNumber boolValue]==NO)
					{
						[tTemplateCell setEnabled:NO];
					}
				}
			}
		}
	}
}

- (void) initPaneWithEngine:(id) inEngine
{
    NSDictionary * tTemplateDictionary;
	NSInteger tTag=0;
	
	tTemplateDictionary=[inEngine templateDictionary];
	
	if (tTemplateDictionary!=nil)
	{
		tTag=[templates_ indexOfObject:tTemplateDictionary];
		
		if (tTag==NSNotFound)
		{
			tTag=0;
		}
	}
	
	[IBmatrix_ selectCellWithTag:tTag];
	
	[self selectTemplate:IBmatrix_];
	
	[mainController_ setEnableNextButton:YES];
}

- (void) dealloc
{
    [templates_ release];
	
	[nextPaneName_ release];
    
    [super dealloc];
}

#pragma mark -

- (NSArray *) newTemplatesArray
{
	NSMutableArray * tMutableArray=nil;
	NSFileManager * tFileManager;
	NSString * tFolderPath;
	
	tFileManager=[NSFileManager defaultManager];
	
	tFolderPath=[NSString stringWithString:@"/Library/Application Support/fr.whitebox.packages/Projects Templates"];
	
	if (tFolderPath!=nil)
	{
		NSArray * tTemplatesNames;
		
		tTemplatesNames=[tFileManager directoryContentsAtPath:tFolderPath];
		
		if (tTemplatesNames!=nil)
		{
			tMutableArray=[[NSMutableArray alloc] initWithCapacity:10];
			
			if (tMutableArray!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tTemplatesNames objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSString * tName;
					
					while (tName=[tEnumerator nextObject])
					{
						NSString * tPath;
						
						tPath=[tFolderPath stringByAppendingPathComponent:tName];
						
						if (tPath!=nil)
						{
							BOOL isDirectory;
							
							if ([tFileManager fileExistsAtPath:tPath isDirectory:&isDirectory]==YES && isDirectory==YES)
							{
								NSBundle * tBundle;
								
								tBundle=[NSBundle bundleWithPath:tPath];
								
								if (tBundle!=nil)
								{
									NSMutableDictionary * tTemplateDictionary;
									
									tTemplateDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tBundle,ICPROJECT_TEMPLATE_BUNDLE,nil];
									
									if (tTemplateDictionary!=nil)
									{
										NSString * tString;
										NSString * tIconPath;
										NSNumber * tNumber;
										BOOL tSupported=YES;
										
										// Name
										
										tString=[tBundle objectForInfoDictionaryKey:ICPROJECT_TEMPLATE_NAME];
										
										if (tString==nil)
										{
											tString=[tPath stringByDeletingPathExtension];
										}
										
										if (tString!=nil)
										{
											[tTemplateDictionary setObject:tString forKey:ICPROJECT_TEMPLATE_NAME];
										}
										
										// Description
										
										tString=[tBundle objectForInfoDictionaryKey:ICPROJECT_TEMPLATE_DESCRIPTION];
										
										if (tString==nil)
										{
											tString=@"";
										}
										
										if (tString!=nil)
										{
											[tTemplateDictionary setObject:tString forKey:ICPROJECT_TEMPLATE_DESCRIPTION];
										}
										
										// Supported
										
										tNumber=[tBundle objectForInfoDictionaryKey:ICPROJECT_TEMPLATE_SUPPORTED];
										
										if (tNumber==nil)
										{
											tNumber=[NSNumber numberWithBool:YES];
										}
										
										if (tNumber!=nil)
										{
											[tTemplateDictionary setObject:tNumber forKey:ICPROJECT_TEMPLATE_SUPPORTED];
										}
										
										tSupported=[tNumber boolValue];
										
										// Icon
										
										tIconPath=[tBundle pathForResource:@"Icon" ofType:@"icns"];
										
										if (tIconPath!=nil)
										{
											NSImage * tImage;
											
											tImage=[[NSImage alloc] initWithContentsOfFile:tIconPath];
											
											if (tImage!=nil)
											{
												if (tSupported==NO)
												{
													[tTemplateDictionary setObject:[tImage grayscale] forKey:ICPROJECT_TEMPLATE_ICON];
												}
												else
												{
													[tTemplateDictionary setObject:tImage forKey:ICPROJECT_TEMPLATE_ICON];
												}
												
												[tImage release];
											}
										}
										
										// Template File
										
										tString=[tBundle pathForResource:@"Template" ofType:@"icns"];
										
										if (tString!=nil)
										{
											[tTemplateDictionary setObject:tString forKey:ICPROJECT_TEMPLATE_PATH];
										}
										
										// A COMPLETER

										[tMutableArray addObject:tTemplateDictionary];
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	return tMutableArray;
}

- (NSString *) nextPaneName
{
    return [[nextPaneName_ retain] autorelease];
}

- (void) setNextPaneName:(NSString *) inNextPaneName
{
    if (nextPaneName_!=inNextPaneName)
    {
        [nextPaneName_ release];
        
        nextPaneName_=[inNextPaneName copy];
    }
}

#pragma mark -

- (IBAction) selectTemplate:(id) sender
{
	NSInteger tTag;
	NSDictionary * tDictionary;
	
	tTag=[[sender selectedCell] tag];
	
	tDictionary=[templates_ objectAtIndex:tTag];

	if (tDictionary!=currentTemplateDictionary_)
	{
		currentTemplateDictionary_=tDictionary;
		
		// Update the Description
		
		[IBdescriptionIcon_ setImage:[currentTemplateDictionary_ objectForKey:ICPROJECT_TEMPLATE_ICON]];
		
		[IBdescriptionTitle_ setStringValue:[currentTemplateDictionary_ objectForKey:ICPROJECT_TEMPLATE_NAME]];
		
		[IBdescriptionDescription_ setStringValue:[currentTemplateDictionary_ objectForKey:ICPROJECT_TEMPLATE_DESCRIPTION]];
	}
}

#pragma mark -

- (BOOL) checkPaneValuesWithEngine:(id) inEngine
{
    [inEngine setTemplateDictionary:currentTemplateDictionary_];
    
    return YES;
}

- (void) processWithEngine:(id) inEngine
{
    /*
      Overload this method to process the data provided by the user in this pane
      Don't forget to add the AKProcessEngine subclass header in the code
    */
}

@end
