/*
Copyright (c) 2009, Todd Ditchendorf
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the Todd Ditchendorf nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*

Moving Todd source code back to Obj-C 1.0

Copyright (c) 2009, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICGutterView.h"

@implementation ICGutterView

- (void) awakeFromNib
{
	attributes_=[[NSDictionary alloc] initWithObjectsAndKeys:[NSFont systemFontOfSize:9.0f], NSFontAttributeName,
															 [NSColor darkGrayColor], NSForegroundColorAttributeName,
															 nil];
}

#pragma mark -

- (void) setLineNumberRects:(NSArray *) inArray
{
	if (inArray!=lineNumberRects)
	{
		[lineNumberRects release];
		
		lineNumberRects=[inArray copy];
	}
}

- (void) setStartLineNumber:(unsigned long) inStartLineNumber
{
	startLineNumber=inStartLineNumber;
}

#pragma mark -

- (BOOL) isOpaque
{
	return NO;
}

- (BOOL) isFlipped
{
	return YES;
}

- (unsigned long) autoresizingMask
{
	return NSViewHeightSizable;
}

- (void) drawRect:(NSRect) inRect
{
	NSPoint p1,p2;
	CGFloat tWidth;
	NSRect tBounds;
	
	tBounds=[self bounds];
	
	// Draw Background
	
	[[NSColor colorWithCalibratedWhite:0.933f alpha:1.0f] set];
	
	//inRect.size.height-=1.0f;
	
	NSRectFill(inRect);
	
	//inRect.size.height+=1.0f;
    
	if (NSMaxX(inRect)==NSMaxX(tBounds))
	{
		[[NSColor colorWithCalibratedWhite:0.5765f alpha:1.0f] set];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(tBounds)-0.5f,NSMinY(inRect)) toPoint:NSMakePoint(NSMaxX(tBounds)-0.5f,NSMaxY(inRect))];
	}
	
	/*if (NSMaxY(inRect)==NSMaxY(tBounds))
	{
		[[NSColor colorWithCalibratedWhite:0.7451f alpha:1.0f] set];
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(tBounds),NSMaxY(inRect)-0.5f) toPoint:NSMakePoint(NSMaxX(tBounds),NSMaxY(inRect)-0.5f)];
	}*/
	
	
	
    tWidth = NSWidth(inRect);
    
	p1 = NSMakePoint(tWidth + 1., 0.);
    
	p2 = NSMakePoint(tWidth + 1., NSHeight(inRect));
    
	[NSBezierPath strokeLineFromPoint:p1 toPoint:p2];
    
    if ([lineNumberRects count]>0)
	{
		NSUInteger i,tCount;
		NSSize tMaxSize;
		
		tMaxSize=NSMakeSize(800,400);
		
		tCount = startLineNumber + [lineNumberRects count];
		
		for (i=startLineNumber ; i < tCount; i++)
		{
			NSRect tRect;
			//float x = 0.;
			NSString * tString;
			NSAttributedString * tAttributedString;
			
			tRect = [[lineNumberRects objectAtIndex:i - startLineNumber] rectValue];

			// set the x origin of the number according to the number of digits it contains
			
			// center the number vertically for tall lines
			
			if (tRect.origin.y)
			{
				tRect.origin.y += NSHeight(tRect)*0.5 - 6.;
			}
			
			tString = [[NSNumber numberWithUnsignedLong:i + 1] stringValue];
			
			tRect.origin.x=tWidth-NSWidth([tString boundingRectWithSize:tMaxSize options:0 attributes:attributes_])-2.0f;
			
			tAttributedString = [[NSAttributedString alloc] initWithString:tString attributes:attributes_];
			
			[tAttributedString drawAtPoint:tRect.origin];
			
			[tAttributedString release];
		}
	}
}

@end
