/*
Copyright (c) 2009-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "NSRangeUtilities.h"

@implementation NSValue (SortRange)

- (NSComparisonResult) compareRangeLocation:(NSValue *) inValue
{
	if (inValue!=nil)
	{
		NSRange tRange,tOtherRange;
		
		tRange=[self rangeValue];
		
		tOtherRange=[inValue rangeValue];
		
		if (tRange.location<tOtherRange.location)
		{
			return NSOrderedAscending;
		}
		else if (tRange.location>tOtherRange.location)
		{
			return NSOrderedDescending;
		}
	}
	
	return NSOrderedSame;
}

@end

@implementation NSRangeUtilities

+ (NSUInteger) indexOfRangeIncludingLocation:(NSUInteger) inLocation withinRanges:(NSArray *) inArray
{
	if (inArray!=nil)
	{
		NSUInteger tIndex,tCount;
		
		tCount=[inArray count];
		
		for(tIndex=0;tIndex<tCount;tIndex++)
		{
			NSValue * tValue;
			NSRange tRange;
			
			tValue=[inArray objectAtIndex:tIndex];
			
			tRange=[tValue rangeValue];
				
			if (NSLocationInRange(inLocation,tRange)==YES)
			{
				return tIndex;
			}
		}
	}
	
	
	return NSNotFound;
}

+ (BOOL) location:(NSUInteger) inLocation isInsideRanges:(NSArray *) inArray
{
	if (inArray!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSValue * tValue;
			
			while (tValue=[tEnumerator nextObject])
			{
				NSRange tOtherRange;
				
				tOtherRange=[tValue rangeValue];
				
				if (NSLocationInRange(inLocation,tOtherRange)==YES)
				{
					return YES;
				}
			}
		}
	}
	
	return NO;
}

+ (BOOL) range:(NSRange) inRange intersectsRanges:(NSArray *) inArray
{
	if (inArray!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSValue * tValue;
			
			while (tValue=[tEnumerator nextObject])
			{
				NSRange tOtherRange;
				
				tOtherRange=[tValue rangeValue];
				
				if (NSIntersectionRange(inRange,tOtherRange).length!=0)
				{
					return YES;
				}
			}
		}
	}
	
	return NO;
}

+ (BOOL) range:(NSRange) inRange intersectsSortedRanges:(NSArray *) inArray
{
	if (inArray!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSValue * tValue;
			
			while (tValue=[tEnumerator nextObject])
			{
				NSRange tOtherRange;
				
				tOtherRange=[tValue rangeValue];
				
				if (tOtherRange.location>=NSMaxRange(inRange))
				{
					return NO;
				}
				
				if (NSIntersectionRange(inRange,tOtherRange).length!=0)
				{
					return YES;
				}
			}
		}
	}
	
	return NO;
}

+ (NSArray *) rangesFromRange:(NSRange) inRange excludingRanges:(NSArray *) inRanges
{
	NSMutableArray * tAvailableRanges=nil;
	
	if (inRanges==nil)
	{
		return [NSArray arrayWithObject:[NSValue valueWithRange:inRange]];
	}
	else
	{
		NSMutableArray * tSortedRangesArray;
		
		tSortedRangesArray=[inRanges mutableCopy];
		
		if (tSortedRangesArray!=nil)
		{
			[tSortedRangesArray sortUsingSelector:@selector(compareRangeLocation:)];
			
			tAvailableRanges=[NSMutableArray array];
			
			if (tAvailableRanges!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tSortedRangesArray objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSValue * tValue;
					NSRange tNextAvailableRange;
					NSValue * tAvailableValue;
					
					tNextAvailableRange=inRange;
					
					while (tValue=[tEnumerator nextObject])
					{
						NSRange tExcludedRange;
						NSUInteger tMaxExcludedRange;
						NSUInteger tMaxAvailableRange;
										
						tExcludedRange=[tValue rangeValue];
						
						tMaxExcludedRange=NSMaxRange(tExcludedRange);
						
						tMaxAvailableRange=NSMaxRange(tNextAvailableRange);
						
						if (tExcludedRange.location<tMaxAvailableRange)
						{
							if (tExcludedRange.location>tNextAvailableRange.location)
							{
								tAvailableValue=[NSValue valueWithRange:NSMakeRange(tNextAvailableRange.location,tExcludedRange.location-tNextAvailableRange.location)];
								
								[tAvailableRanges addObject:tAvailableValue];
								
								if (tMaxExcludedRange<tMaxAvailableRange)
								{
									tNextAvailableRange.location=tMaxExcludedRange;
									
									tNextAvailableRange.length=tMaxAvailableRange-tMaxExcludedRange;
								}
								else
								{
									tNextAvailableRange.length=0;
									
									break;
								}
							}
							else if (tMaxExcludedRange>tNextAvailableRange.location)
							{
								NSRange tReallyExcludedRange;
								
								tReallyExcludedRange.location=tNextAvailableRange.location;
								
								tReallyExcludedRange.length=tMaxExcludedRange-tNextAvailableRange.location;
								
								if (NSMaxRange(tReallyExcludedRange)>=tMaxAvailableRange)
								{
									tNextAvailableRange.length=0;
									
									break;
								}
								
								tNextAvailableRange.location=NSMaxRange(tReallyExcludedRange);
									
								tNextAvailableRange.length=tMaxAvailableRange-NSMaxRange(tReallyExcludedRange);
							}
						}
					}
					
					if (tNextAvailableRange.length>0)
					{
						tAvailableValue=[NSValue valueWithRange:tNextAvailableRange];
								
						[tAvailableRanges addObject:tAvailableValue];
					}
				}
			}
			
			[tSortedRangesArray release];
		}
	}
	
	return tAvailableRanges;
}

+ (NSArray *) sortedRanges:(NSArray *) inArray intersectingRange:(NSRange) inRange
{
	NSMutableArray * tMutableArray=nil;
	
	if (inArray!=nil && (inRange.length>0))
	{
		NSUInteger tCount,tIndex;
		
		tMutableArray=[NSMutableArray array];
		
		tCount=[inArray count];
		
		for(tIndex=0;tIndex<tCount;tIndex++)
		{
			NSValue * tValue;
			NSRange tRange;
			
			tValue=[inArray objectAtIndex:tIndex];
			
			tRange=[tValue rangeValue];
			
			if (tRange.location>=NSMaxRange(inRange))
			{
				break;
			}
			
			if (NSIntersectionRange(tRange,inRange).length>0)
			{
				[tMutableArray addObject:tValue];
			}
		}
	}
	
	return tMutableArray;
}

@end
