#import "ICLocatorConverter_JavaScript_Controller.h"

#import "ICLocator_JavaScript+Constants.h"

@implementation ICLocatorConverter_JavaScript_Controller

- (NSArray *) elementsWithSettings:(NSDictionary *) inSettings withUniqueSearchID:(NSString *) inUniqueUUID error:(NSDictionary **) outErrorDictionary
{
	NSXMLElement * tSearchElement;
	
	if (outErrorDictionary!=nil)
	{
		*outErrorDictionary=nil;
	}
	
	tSearchElement=(NSXMLElement *) [NSXMLNode elementWithName:@"search"];
			
	if (tSearchElement!=nil)
	{
		id tAttribute;
		NSString * tFunctionName;
		NSMutableArray * tMutableArray;
		NSString * tStringParameters=@"";
		NSUInteger tIndex,tCount;
		BOOL nonEmptyParameterFound=NO;
		NSXMLElement * tScriptElement;
			
		// type
		
		tAttribute=[NSXMLNode attributeWithName:@"type" stringValue:@"script"];
		
		if (tAttribute!=nil)
		{
			[tSearchElement addAttribute:tAttribute];
		}
		else
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																			nil];
			
			goto bail;
		}
		
		// id
		
		tAttribute=[NSXMLNode attributeWithName:@"id" stringValue:inUniqueUUID];
		
		if (tAttribute!=nil)
		{
			[tSearchElement addAttribute:tAttribute];
		}
		else
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																			nil];
			
			goto bail;
		}
		
		// script
		
		tFunctionName=[inSettings objectForKey:IC_LOCATOR_JAVASCRIPT_FUNCTION];
	
		if (tFunctionName==nil)
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorMissingParameter],ICLocatorConverterErrorCode,
																		   IC_LOCATOR_JAVASCRIPT_FUNCTION,ICLocatorConverterErrorParameter,
																		   nil];
		
			goto bail;
		}
		else if ([tFunctionName length]==0)
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorInvalidParameter],ICLocatorConverterErrorCode,
																		   IC_LOCATOR_JAVASCRIPT_FUNCTION,ICLocatorConverterErrorParameter,
																		   nil];
		
			goto bail;
		}
		
		tMutableArray=[[[inSettings objectForKey:IC_LOCATOR_JAVASCRIPT_PARAMETERS] mutableCopy] autorelease];
	
		tCount=[tMutableArray count];
			
		for(tIndex=tCount;tIndex>0;tIndex--)
		{
			NSMutableString * tMutableString;
			
			tMutableString=[[[tMutableArray objectAtIndex:(tIndex-1)] mutableCopy] autorelease];
			
			CFStringTrimWhitespace((CFMutableStringRef) tMutableString);
			
			if ([tMutableString length]==0)
			{
				if (nonEmptyParameterFound==YES)
				{
					*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorInvalidParameter],ICLocatorConverterErrorCode,
																				   IC_LOCATOR_JAVASCRIPT_PARAMETERS,ICLocatorConverterErrorParameter,
																				   nil];
					return nil;
				}
			}
			else
			{
				nonEmptyParameterFound=YES;
			}
			
			if ([tMutableString length]==0)
			{
				[tMutableArray removeObjectAtIndex:(tIndex-1)];
			}
			else
			{
				[tMutableArray replaceObjectAtIndex:(tIndex-1) withObject:tMutableString];
			}
		}
		
		if ([tMutableArray count]>0)
		{
			tStringParameters=[tMutableArray componentsJoinedByString:@","];
		}
		
		
		tAttribute=[NSXMLNode attributeWithName:@"script" stringValue:[NSString stringWithFormat:@"%@(%@)",tFunctionName,tStringParameters]];
		
		if (tAttribute!=nil)
		{
			[tSearchElement addAttribute:tAttribute];
		}
		else
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																			nil];
			
			goto bail;
		}
		
		tScriptElement=(NSXMLElement *) [NSXMLNode elementWithName:@"script"];
		
		if (tScriptElement!=nil)
		{
			// Add JavaScript code
			
			NSMutableString * tMutableSourceCode;
			NSUInteger tLength;
				
			tMutableSourceCode=[[[inSettings objectForKey:IC_LOCATOR_JAVASCRIPT_SOURCE_CODE] mutableCopy] autorelease];
			
			tLength=[tMutableSourceCode length];
			
			// Add a tab at the beginning of each line
				
			if (tLength>0)
			{
				NSXMLNode * tNode;
				
				[tMutableSourceCode replaceOccurrencesOfString:@"\n" withString:@"\n\t\t" options:0 range:NSMakeRange(0,tLength)];
				
				[tMutableSourceCode insertString:@"\n\n\t\t" atIndex:0];
			
				[tMutableSourceCode appendString:@"\n\n        "];
			
				tNode=[NSXMLNode textWithStringValue:tMutableSourceCode];

				if (tNode!=nil)
				{
					[tScriptElement addChild:tNode];
				
					[tSearchElement addChild:tScriptElement];
			
					return [NSArray arrayWithObject:tSearchElement];
				}
				else
				{
					*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																					nil];
				}
			}
			else
			{
				*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorMissingParameter],ICLocatorConverterErrorCode,
																			   IC_LOCATOR_JAVASCRIPT_SOURCE_CODE,ICLocatorConverterErrorInvalidParameter,
																			   nil];
			}
		}
		else
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																			nil];
		}
	}
	else
	{
		*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																		nil];
	}

bail:
	
	return nil;
}

@end
