#import "ICLocatorConverter_Standard_Controller.h"

#import "ICLocator_Standard+Constants.h"

@implementation ICLocatorConverter_Standard_Controller

- (NSArray *) elementsWithSettings:(NSDictionary *) inSettings withUniqueSearchID:(NSString *) inUniqueUUID error:(NSDictionary **) outErrorDictionary
{
	NSXMLElement * tSearchElement;
	
	if (outErrorDictionary!=nil)
	{
		*outErrorDictionary=nil;
	}
	
	tSearchElement=(NSXMLElement *) [NSXMLNode elementWithName:@"search"];
			
	if (tSearchElement!=nil)
	{
		id tAttribute;
		NSString * tBundleIdentifier;
		NSString * tDefaultPath;
		NSXMLElement * tBundleElement;
		BOOL tPreferDefaultPath=NO;
		NSNumber * tNumber;
		NSString * tSubID=nil;
		
		tNumber=[inSettings objectForKey:IC_LOCATOR_STANDARD_PREFER_DEFAULT_PATH];
		
		if (tNumber!=nil)
		{
			tPreferDefaultPath=[tNumber boolValue];
		}
		
		// type
		
		tAttribute=[NSXMLNode attributeWithName:@"type" stringValue:@"component"];
		
		if (tAttribute!=nil)
		{
			[tSearchElement addAttribute:tAttribute];
		}
		else
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																			nil];
			
			goto bail;
		}
		
		// id
		
		if (tPreferDefaultPath==NO)
		{
			tAttribute=[NSXMLNode attributeWithName:@"id" stringValue:inUniqueUUID];
		}
		else
		{
			tSubID=[inUniqueUUID stringByAppendingString:@".1"];
			
			tAttribute=[NSXMLNode attributeWithName:@"id" stringValue:tSubID];
		}
		
		if (tAttribute!=nil)
		{
			[tSearchElement addAttribute:tAttribute];
		}
		else
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																			nil];
			
			goto bail;
		}
		
		
		tBundleIdentifier=[inSettings objectForKey:IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER];
	
		if (tBundleIdentifier==nil)
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorMissingParameter],ICLocatorConverterErrorCode,
																		   IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER,ICLocatorConverterErrorParameter,
																		   nil];
		
			goto bail;
		}
		else if ([tBundleIdentifier length]==0)
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorMissingParameter],ICLocatorConverterErrorCode,
																		   IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER,ICLocatorConverterErrorInvalidParameter,
																		   nil];
		
			goto bail;
		}
		
		tDefaultPath=[inSettings objectForKey:IC_LOCATOR_STANDARD_DEFAULT_PATH];
	
		if (tDefaultPath==nil)
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorMissingParameter],ICLocatorConverterErrorCode,
																		   IC_LOCATOR_STANDARD_DEFAULT_PATH,ICLocatorConverterErrorParameter,
																		   nil];
		
			goto bail;
		}
			
		tBundleElement=(NSXMLElement *) [NSXMLNode elementWithName:@"bundle"];
		
		if (tBundleElement!=nil)
		{
			// CFBundleIdentifier
			
			tAttribute=[NSXMLNode attributeWithName:@"CFBundleIdentifier" stringValue:tBundleIdentifier];
	
			if (tAttribute!=nil)
			{
				[tBundleElement addAttribute:tAttribute];
			}
			else
			{
				*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																				nil];
		
				goto bail;
			}
			
			tDefaultPath=[tDefaultPath stringByStandardizingPath];
			
			if ([tDefaultPath length]>0)
			{
				// path
			
				tAttribute=[NSXMLNode attributeWithName:@"path" stringValue:tDefaultPath];
		
				if (tAttribute!=nil)
				{
					[tBundleElement addAttribute:tAttribute];
				}
				else
				{
					*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																					nil];
			
					goto bail;
				}
			}
			else
			{
				tPreferDefaultPath=NO;
			}
			
			[tSearchElement addChild:tBundleElement];
			
			
			if (tPreferDefaultPath==YES)
			{
				NSXMLElement * tSearchSortElement;
	
				tSearchSortElement=(NSXMLElement *) [NSXMLNode elementWithName:@"search"];
						
				if (tSearchSortElement!=nil)
				{
					NSXMLElement * tScriptElement;
					
					// type
					
					tAttribute=[NSXMLNode attributeWithName:@"type" stringValue:@"script"];
					
					if (tAttribute!=nil)
					{
						[tSearchSortElement addAttribute:tAttribute];
					}
					else
					{
						*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																						nil];
						
						goto bail;
					}
					
					// id
					
					tAttribute=[NSXMLNode attributeWithName:@"id" stringValue:inUniqueUUID];
					
					if (tAttribute!=nil)
					{
						[tSearchSortElement addAttribute:tAttribute];
					}
					else
					{
						*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																						nil];
						
						goto bail;
					}
					
					// script
					
					tAttribute=[NSXMLNode attributeWithName:@"script" stringValue:@"searchDefaultPath()"];
					
					if (tAttribute!=nil)
					{
						[tSearchSortElement addAttribute:tAttribute];
					}
					else
					{
						*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																						nil];
						
						goto bail;
					}
					
					tScriptElement=(NSXMLElement *) [NSXMLNode elementWithName:@"script"];
		
					if (tScriptElement!=nil)
					{
						// Add JavaScript code
						
						NSMutableString * tMutableSourceCode;
						NSUInteger tLength;
						NSXMLNode * tNode;
						
						tMutableSourceCode=[NSMutableString stringWithFormat:@"function searchDefaultPath()\n\
{\n\
\tvar tResults;\n\
\n\
\ttResults=my.search.results['%@'];\n\
\n\
\tif (typeof(tResults) == 'object')\n\
\t{\n\
\t\tvar tCount;\n\
\n\
\t\ttCount=tResults.length;\n\
\n\
\t\tif (tCount>0)\n\
\t\t{\n\
\t\t\tvar i;\n\
\n\
\t\t\tfor(i=0;i<tCount;i++)\n\
\t\t\t{\n\
\t\t\t\tif (tResults[i] == '%@')\n\
\t\t\t\t{\n\
\t\t\t\t\treturn tResults[i];\n\
\t\t\t\t}\n\
\t\t\t}\n\
\t\t}\n\
\t}\n\
\n\
\treturn tResults;\n\
}",tSubID,tDefaultPath];
	
	
						tLength=[tMutableSourceCode length];
						
						// Add a tab at the beginning of each line
							
						[tMutableSourceCode replaceOccurrencesOfString:@"\n" withString:@"\n\t\t" options:0 range:NSMakeRange(0,tLength)];
							
						[tMutableSourceCode insertString:@"\n\n\t\t" atIndex:0];
					
						[tMutableSourceCode appendString:@"\n\n        "];
					
						tNode=[NSXMLNode textWithStringValue:tMutableSourceCode];

						if (tNode!=nil)
						{
							[tScriptElement addChild:tNode];
						
							[tSearchSortElement addChild:tScriptElement];
					
							return [NSArray arrayWithObjects:tSearchElement,tSearchSortElement,nil];
						}
						else
						{
							*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																							nil];
						}
					}
					else
					{
						*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																						nil];
					}
				}
				else
				{
					*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																					nil];
				}
			}
			else
			{
				return [NSArray arrayWithObject:tSearchElement];
			}
		}
		else
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																			nil];
		}
	}
	else
	{
		*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICLocatorConverterErrorLowMemory],ICLocatorConverterErrorCode,
																		nil];
	}

bail:
	
	return nil;
}

@end
