#import "ICLocatorUI_Standard_Controller.h"

#import "NSDictionary+MutableDeepCopy.h"

#import "ICLocator_Standard+Constants.h"

#import "ICDocumentFormat+Constants.h"

@implementation ICLocatorUI_Standard_Controller

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn = nil;
	
	// Array
	
	tTableColumn = [IBarray_ tableColumnWithIdentifier:@"BundleIdentifier"];
    
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
	
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
	}
	
	// Name
	
	tTableColumn = [IBarray_ tableColumnWithIdentifier:@"Path"];
    
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
	
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
	}
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary mutableDeepCopy];
	
	if (dictionary_!=nil)
	{
		// Arguments
		
		cachedList_=[[dictionary_ objectForKey:IC_LOCATOR_STANDARD_LIST] retain];
		
		[IBaddButton_ setEnabled:YES];
		
		[IBremoveButton_ setEnabled:NO];
		
		[IBarray_ reloadData];
		
		[IBarray_ deselectAll:self];
	}
}

- (void) dealloc
{
	[cachedList_ release];

	[super dealloc];
}

#pragma mark -

- (NSMutableDictionary *) defaultDictionaryWithCommonValues:(NSDictionary *) inDictionary
{
	return [NSMutableDictionary dictionaryWithObjectsAndKeys:[NSMutableArray arrayWithObjects:[NSMutableDictionary dictionaryWithObjectsAndKeys:[inDictionary objectForKey:IC_LOCATOR_BUNDLE_IDENTIFIER],IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER,
																																				[inDictionary objectForKey:IC_LOCATOR_PATH],IC_LOCATOR_STANDARD_DEFAULT_PATH,
																																				nil],nil],IC_LOCATOR_STANDARD_LIST,
															 nil];
}

- (NSView *) previousKeyView
{
	return IBarray_;
}

- (void) setNextKeyView:(NSView *) inView
{
	[IBarray_ setNextKeyView:inView];
}

#pragma mark -

- (int) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (cachedList_!=nil)
	{
		return [cachedList_ count];
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(int) inRowIndex
{
	if (cachedList_!=nil)
	{
		NSMutableDictionary * tDictionary;
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];

		tDictionary=[cachedList_ objectAtIndex:inRowIndex];
		
		if ([tColumnIdentifier isEqualToString: @"BundleIdentifier"])
		{
			return [tDictionary objectForKey:IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER];
		}
		else if ([tColumnIdentifier isEqualToString: @"Path"])
		{
			return [tDictionary objectForKey:IC_LOCATOR_STANDARD_DEFAULT_PATH];
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(int) inRowIndex
{
	if (inTableView==IBarray_)
	{
		if (cachedList_!=nil)
		{
			NSMutableDictionary * tDictionary;
			NSString * tColumnIdentifier;
			
			tColumnIdentifier=[inTableColumn identifier];
		
			tDictionary=[cachedList_ objectAtIndex:inRowIndex];
			
			if ([tColumnIdentifier isEqualToString: @"BundleIdentifier"])
			{
				[tDictionary setObject:object forKey:IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER];
			}
			else if ([tColumnIdentifier isEqualToString: @"Path"])
			{
				[tDictionary setObject:object forKey:IC_LOCATOR_STANDARD_DEFAULT_PATH];
			}
		}
	}
}

- (void) control:(NSControl *) inControl didFailToValidatePartialString:(NSString *) inPartialString errorDescription:(NSString *) inError
{
	if (inError!=nil && [inError isEqualToString:@"NSBeep"]==YES)
	{
		NSBeep();
	}
}

#pragma mark -

- (IBAction) addBundleKey:(id) sender
{
	unsigned int tRowIndex;
	
	tRowIndex=[cachedList_ count];

	[cachedList_ addObject:[NSMutableDictionary dictionaryWithObjectsAndKeys:@"",IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER,
																			 @"/",IC_LOCATOR_STANDARD_DEFAULT_PATH,
																			 nil]];
	
	[IBarray_ deselectAll:self];
			
	[IBarray_ reloadData];
	
	[IBarray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
	[IBarray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
}

- (IBAction) removeBundleKey:(id) sender
{
	NSIndexSet * tIndexSet;
			
	tIndexSet=[IBarray_ selectedRowIndexes];
	
	[IBarray_ deselectAll:self];
	
	if (tIndexSet!=nil)
	{
		[cachedList_ removeObjectsAtIndexes:tIndexSet];
		
		[IBarray_ reloadData];
		
		[IBremoveButton_ setEnabled:NO];
	}
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBarray_)
    {
		[IBremoveButton_ setEnabled:([IBarray_ numberOfSelectedRows]!=0)];
	}
}

@end
