#import "ICLocatorUI_Standard_Controller.h"

#import "NSDictionary+MutableDeepCopy.h"

#import "ICLocator_Standard+Constants.h"

#import "ICBundleIdentifierFormatter.h"

@implementation ICLocatorUI_Standard_Controller

- (void) awakeFromNib
{
	ICBundleIdentifierFormatter * tFormatter;
	
	tFormatter=[ICBundleIdentifierFormatter new];
	
	if (tFormatter!=nil)
	{
		[IBbundleIdentifierTextField_ setFormatter:tFormatter];
		
		[tFormatter release];
	}
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary mutableDeepCopy];
	
	if (dictionary_!=nil)
	{
		NSString * tString;
		
		// Arguments
		
		tString=[dictionary_ objectForKey:IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER];
		
		if (tString!=nil)
		{
			[IBbundleIdentifierTextField_ setStringValue:tString];
		}
		else
		{
			[IBbundleIdentifierTextField_ setStringValue:@""];
		}
		
		
		[IBdefaultPathCheckBox_ setEnabled:NO];
		
		[IBdefaultPathCheckBox_ setState:NSOffState];
		
		tString=[dictionary_ objectForKey:IC_LOCATOR_STANDARD_DEFAULT_PATH];
		
		if (tString!=nil)
		{
			[IBdefaultPathTextField_ setStringValue:tString];
			
			if ([tString length]>0)
			{
				NSNumber * tNumber;
				
				tNumber=[dictionary_ objectForKey:IC_LOCATOR_STANDARD_PREFER_DEFAULT_PATH];
				
				[IBdefaultPathCheckBox_ setEnabled:YES];
				
				if (tNumber==nil || [tNumber boolValue]==NO)
				{
					[IBdefaultPathCheckBox_ setState:NSOffState];
				}
				else
				{
					[IBdefaultPathCheckBox_ setState:NSOnState];
				}
			}
		}
		else
		{
			[IBdefaultPathTextField_ setStringValue:@""];
		}
	}
}

- (void) dealloc
{
	[super dealloc];
}

#pragma mark -

- (NSMutableDictionary *) defaultDictionaryWithCommonValues:(NSDictionary *) inDictionary
{
	return [NSMutableDictionary dictionaryWithObjectsAndKeys:[inDictionary objectForKey:IC_LOCATOR_BUNDLE_IDENTIFIER],IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER,
															 [inDictionary objectForKey:IC_LOCATOR_PATH],IC_LOCATOR_STANDARD_DEFAULT_PATH,
															 nil];
}

- (NSView *) previousKeyView
{
	return IBbundleIdentifierTextField_;
}

- (void) setNextKeyView:(NSView *) inView
{
	[IBdefaultPathTextField_ setNextKeyView:inView];
}

#pragma mark -

- (IBAction) setBundleIdentifier:(id) sender
{
	NSString * tStringValue;
	
	tStringValue=[IBbundleIdentifierTextField_ stringValue];
	
	if (tStringValue!=nil)
	{
		[dictionary_ setObject:tStringValue forKey:IC_LOCATOR_STANDARD_BUNDLE_IDENTIFIER];
	}
}

- (IBAction) setDefaultPath:(id) sender
{
	NSString * tStringValue;
	
	tStringValue=[IBdefaultPathTextField_ stringValue];
	
	if (tStringValue!=nil)
	{
		[dictionary_ setObject:tStringValue forKey:IC_LOCATOR_STANDARD_DEFAULT_PATH];
	}
}

- (IBAction) switchUseDefaultPath:(id) sender
{
	NSNumber * tNumber;
	
	tNumber=[NSNumber numberWithBool:([IBdefaultPathCheckBox_ state]==NSOnState)];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_LOCATOR_STANDARD_PREFER_DEFAULT_PATH];
	}
}

#pragma mark -

- (void) control:(NSControl *) inControl didFailToValidatePartialString:(NSString *) inPartialString errorDescription:(NSString *) inError
{
	if (inError!=nil && [inError isEqualToString:@"NSBeep"]==YES)
	{
		NSBeep();
	}
}

- (void) controlTextDidChange:(NSNotification *) inNotification
{
	if ([inNotification object]==IBdefaultPathTextField_)
	{
		BOOL tValue;
		
		tValue=([[IBdefaultPathTextField_ stringValue] length]>0);
		
		[IBdefaultPathCheckBox_ setEnabled:tValue];
		
		if (tValue==NO)
		{
			NSNumber * tNumber;
			
			tNumber=[NSNumber numberWithBool:tValue];
			
			[IBdefaultPathCheckBox_ setState:NSOffState];
			
			if (tNumber!=nil)
			{
				[dictionary_ setObject:tNumber forKey:IC_LOCATOR_STANDARD_PREFER_DEFAULT_PATH];
			}
		}
	}
}

@end
