#import "ICRequirementUI_CPU_Controller.h"

#import "ICRequirement_CPU+Constants.h"

@implementation ICRequirementUI_CPU_Controller

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary mutableDeepCopy];
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
		NSInteger tTag;
		NSInteger tSubTag;
		
		// Minimum Number of CPU Cores
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_CPU_MINIMUM_CPU_CORES_COUNT];
		
		if (tNumber==nil)
		{
			tTag=1;
		}
		else
		{
			tTag=[tNumber integerValue];
		}
		
		[IBminimumCPUCoresCount_ selectSegmentWithTag:tTag];
		
		// CPU Family
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_CPU_ARCHITECTURE_FAMILY];
		
		if (tNumber==nil)
		{
			tTag=1;
		}
		else
		{
			tTag=[tNumber integerValue];
		}
		
		[IBCPUArchitecture_ selectSegmentWithTag:tTag];
		
		// PowerPC
		
		if (tTag==CPU_ARCHITECTURE_INTEL)
		{
			[IBPowerPCArchitecture_ setEnabled:NO];
			
			[IBPowerPCArchitecture_ selectSegmentWithTag:CPU_ARCHITECTURE_TYPE_ANY];
		}
		else
		{
			tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_CPU_POWERPC_ARCHITECTURE_TYPE];
		
			if (tNumber==nil)
			{
				tSubTag=CPU_ARCHITECTURE_TYPE_ANY;
			}
			else
			{
				tSubTag=[tNumber integerValue];
			}
		
			[IBPowerPCArchitecture_ selectSegmentWithTag:tSubTag];
		}
		
		// Intel
		
		if (tTag==CPU_ARCHITECTURE_POWERPC)
		{
			[IBIntelArchitecture_ setEnabled:NO];
			
			[IBIntelArchitecture_ selectSegmentWithTag:CPU_ARCHITECTURE_TYPE_ANY];
		}
		else
		{
			tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_CPU_INTEL_ARCHITECTURE_TYPE];
		
			if (tNumber==nil)
			{
				tSubTag=CPU_ARCHITECTURE_TYPE_ANY;
			}
			else
			{
				tSubTag=[tNumber integerValue];
			}
		
			[IBIntelArchitecture_ selectSegmentWithTag:tSubTag];
		}
		
		// Minimum CPU Frequency
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_CPU_MINIMUM_FREQUENCY];
		
		if (tNumber==nil)
		{
			tTag=CPU_MINIMUM_FREQUENCY;
		}
		else
		{
			tTag=[tNumber integerValue];
		}
		
		[IBminimumCPUFrequency_ selectItemWithTag:tTag];
	}
}

#pragma mark -

- (NSMutableDictionary *) defaultDictionary
{
	return [NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInteger:1],IC_REQUIREMENT_CPU_MINIMUM_CPU_CORES_COUNT,
															 [NSNumber numberWithInteger:CPU_ARCHITECTURE_ANY],IC_REQUIREMENT_CPU_ARCHITECTURE_FAMILY,
															 [NSNumber numberWithInteger:CPU_ARCHITECTURE_TYPE_ANY],IC_REQUIREMENT_CPU_POWERPC_ARCHITECTURE_TYPE,
															 [NSNumber numberWithInteger:CPU_ARCHITECTURE_TYPE_ANY],IC_REQUIREMENT_CPU_INTEL_ARCHITECTURE_TYPE,
															 [NSNumber numberWithInteger:CPU_MINIMUM_FREQUENCY],IC_REQUIREMENT_CPU_MINIMUM_FREQUENCY,
															 nil];
}

- (int) requirementType
{
	return IC_REQUIREMENT_CHECK_TYPE_INSTALLATION;
}

#pragma mark -

- (IBAction) switchMinimumCPUCoresCount:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender cell] tagForSegment:[sender selectedSegment]];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_CPU_MINIMUM_CPU_CORES_COUNT];
	}
}

- (IBAction) switchCPUArchitecture:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender cell] tagForSegment:[sender selectedSegment]];
	
	tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_CPU_ARCHITECTURE_FAMILY];
	
	if (tNumber!=nil && [tNumber integerValue]!=tTag)
	{
		tNumber=[NSNumber numberWithInteger:tTag];
		
		if (tNumber!=nil)
		{
			[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_CPU_ARCHITECTURE_FAMILY];
			
			// PowerPC Type
			
			if (tTag==CPU_ARCHITECTURE_INTEL)
			{
				[IBPowerPCArchitecture_ setEnabled:NO];
				
				[dictionary_ setObject:[NSNumber numberWithInteger:CPU_ARCHITECTURE_TYPE_ANY] forKey:IC_REQUIREMENT_CPU_POWERPC_ARCHITECTURE_TYPE];
				
				[IBPowerPCArchitecture_ selectSegmentWithTag:CPU_ARCHITECTURE_TYPE_ANY];
			}
			else
			{
				[IBPowerPCArchitecture_ setEnabled:YES];
				
				[IBPowerPCArchitecture_ selectSegmentWithTag:[[dictionary_ objectForKey:IC_REQUIREMENT_CPU_POWERPC_ARCHITECTURE_TYPE] intValue]];
			}
			
			// Intel
			
			if (tTag==CPU_ARCHITECTURE_POWERPC)
			{
				[IBIntelArchitecture_ setEnabled:NO];
				
				[dictionary_ setObject:[NSNumber numberWithInteger:CPU_ARCHITECTURE_TYPE_ANY] forKey:IC_REQUIREMENT_CPU_INTEL_ARCHITECTURE_TYPE];
				
				[IBIntelArchitecture_ selectSegmentWithTag:CPU_ARCHITECTURE_TYPE_ANY];
			}
			else
			{
				[IBIntelArchitecture_ setEnabled:YES];
				
				[IBIntelArchitecture_ selectSegmentWithTag:[[dictionary_ objectForKey:IC_REQUIREMENT_CPU_INTEL_ARCHITECTURE_TYPE] intValue]];
			}
		}
	}
}

- (IBAction) switchPowerPCArchitecture:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender cell] tagForSegment:[sender selectedSegment]];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_CPU_POWERPC_ARCHITECTURE_TYPE];
	}
}

- (IBAction) switchIntelArchitecture:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender cell] tagForSegment:[sender selectedSegment]];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_CPU_INTEL_ARCHITECTURE_TYPE];
	}
}

- (IBAction) switchMinimumCPUFrequency:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_CPU_MINIMUM_FREQUENCY];
	}
}

@end
