#import "ICRequirementUI_Files_Controller.h"

#import "ICRequirement_Files+Constants.h"

#import "ICAbsolutePathFormatter.h"

@implementation ICRequirementUI_Files_Controller

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn = nil;
	
	// Path Names
    
    tTableColumn = [IBarray_ tableColumnWithIdentifier:@"Path"];
	
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
		
		if (tTextFieldCell!=nil)
		{
			ICAbsolutePathFormatter * tFormatter;
			
			tFormatter=[ICAbsolutePathFormatter new];
			
			if (tFormatter!=nil)
			{
				[tTextFieldCell setFormatter:tFormatter];
				
				[tFormatter release];
			}
			
			[tTextFieldCell setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
		}
	}
	
	[IBarray_ registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary mutableDeepCopy];
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
		NSInteger tTag;
		
		// Selector
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_FILES_SELECTOR];
		
		if (tNumber==nil)
		{
			tTag=IC_REQUIREMENT_FILES_SELECTOR_ANY;
		}
		else
		{
			tTag=[tNumber integerValue];
		}
		
		[IBselector_ selectItemWithTag:tTag];
		
		// Condition
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_FILES_CONDITION];
		
		if (tNumber==nil)
		{
			tTag=IC_REQUIREMENT_FILES_CONDITION_EXIST;
		}
		else
		{
			tTag=[tNumber integerValue];
		}
		
		[IBcondition_ selectItemWithTag:tTag];
		
		// Disk Type
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_FILES_DISK_TYPE];
		
		if (tNumber==nil)
		{
			tTag=IC_FILES_DISK_TYPE_DESTINATION;
		}
		else
		{
			tTag=[tNumber integerValue];
		}
		
		[IBdiskType_ selectItemWithTag:tTag];
		
		// Files
		
		cachedFiles_=[[dictionary_ objectForKey:IC_REQUIREMENT_FILES_LIST] retain];
		
		if (cachedFiles_!=nil)
		{
			[cachedFiles_ sortUsingSelector:@selector(caseInsensitiveCompare:)];
		}
		
		[IBaddButton_ setEnabled:YES];
		
		[IBremoveButton_ setEnabled:NO];
		
		[IBarray_ reloadData];
		
		[IBarray_ deselectAll:self];
	}
}

- (void) dealloc
{
	[cachedFiles_ release];

	[super dealloc];
}

#pragma mark -

- (NSMutableDictionary *) defaultDictionary
{
	return [NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:IC_REQUIREMENT_FILES_SELECTOR_ANY],IC_REQUIREMENT_FILES_SELECTOR,
															 [NSNumber numberWithInt:IC_REQUIREMENT_FILES_CONDITION_EXIST],IC_REQUIREMENT_FILES_CONDITION,
															 [NSNumber numberWithInt:IC_FILES_DISK_TYPE_DESTINATION],IC_REQUIREMENT_FILES_DISK_TYPE,
															 [NSMutableArray array],IC_REQUIREMENT_FILES_LIST,
															 nil];
}

- (int) requirementType
{
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
	
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_FILES_DISK_TYPE];
		
		if (tNumber!=nil)
		{
			NSInteger tDiskType;
			
			tDiskType=[tNumber integerValue];
			
			if (tDiskType==IC_FILES_DISK_TYPE_STARTUP_DISK)
			{
				return IC_REQUIREMENT_CHECK_TYPE_INSTALLATION;
			}
			else if (tDiskType==IC_FILES_DISK_TYPE_DESTINATION)
			{
				return IC_REQUIREMENT_CHECK_TYPE_TARGET;
			}
		}
	}
	
	return IC_REQUIREMENT_CHECK_TYPE_UNDEFINED;
}

#pragma mark -

- (NSView *) previousKeyView
{
	return IBarray_;
}

- (void) setNextKeyView:(NSView *) inView
{
	[IBarray_ setNextKeyView:inView];
}

- (IBAction) switchSelector:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_FILES_SELECTOR];
	}
}

- (IBAction) switchCondition:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_FILES_CONDITION];
	}
}

- (IBAction) switchDiskType:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_FILES_DISK_TYPE];
	
		[self noteCheckTypeChange];
	}
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (cachedFiles_!=nil)
	{
		return [cachedFiles_ count];
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (cachedFiles_!=nil)
	{
		return [cachedFiles_ objectAtIndex:inRowIndex];
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (inTableView==IBarray_)
	{
		if (cachedFiles_!=nil)
		{
			NSUInteger tIndex;
			
			tIndex=[cachedFiles_ indexOfObject:object];
			
			if (tIndex!=inRowIndex)
			{
				if (tIndex!=NSNotFound)
				{
					NSBeep();
					
					return;
				}
				else
				{
					[cachedFiles_ replaceObjectAtIndex:inRowIndex withObject:object];
					
					[cachedFiles_ sortUsingSelector:@selector(caseInsensitiveCompare:)];
			
					[IBarray_ deselectAll:self];
			
					[IBarray_ reloadData];
			
					[IBarray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:[cachedFiles_ indexOfObject:object]] byExtendingSelection:NO];
				}
			}
		}
	}
}

- (void) control:(NSControl *) inControl didFailToValidatePartialString:(NSString *) inPartialString errorDescription:(NSString *) inError
{
	if (inError!=nil && [inError isEqualToString:@"NSBeep"]==YES)
	{
		NSBeep();
	}
}

#pragma mark -

- (BOOL) tableView:(NSTableView *) inTableView validateAction:(SEL) inSelector
{
	if (inTableView==IBarray_)
	{
		if (inSelector==@selector(delete:))
		{
			return [IBremoveButton_ isEnabled];
		}
	}
	
	return NO;
}

- (void) tableView:(NSTableView *) inTableView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
	if (inTableView==IBarray_)
	{
		if (inConfirmationRequired==YES)
		{
			[self removeFile:IBremoveButton_];
		}
		else
		{
			[self removeFileDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:NULL];
		}
	}
}

#pragma mark -

- (NSDragOperation) tableView:(NSTableView*) inTableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(NSInteger) inRow proposedDropOperation:(NSTableViewDropOperation) inOperation
{
    if (cachedFiles_!=nil)
	{
		NSPasteboard * tPasteBoard;
			
		tPasteBoard=[info draggingPasteboard];
		
		if (inTableView==IBarray_)
		{
			if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
			{
				NSArray * tArray;
				
				tArray=(NSArray *) [tPasteBoard propertyListForType:NSFilenamesPboardType];
							
				if (tArray!=nil)
				{
					NSEnumerator * tEnumerator;
					
					tEnumerator=[tArray objectEnumerator];
					
					if (tEnumerator!=nil)
					{
						NSString * tPath;
						
						while (tPath=[tEnumerator nextObject])
						{
							if ([cachedFiles_ containsObject:tPath]==NO)
							{
								[IBarray_ setDropRow:-1 dropOperation:NSTableViewDropOn];
				
								return NSDragOperationCopy;
							}
						}
					}
				}
			}
		}
	}
	
	return NSDragOperationNone;
}

- (BOOL) tableView:(NSTableView*) inTableView acceptDrop:(id <NSDraggingInfo>)info row:(NSInteger) inRow dropOperation:(NSTableViewDropOperation) inOperation
{
    if (cachedFiles_!=nil)
	{
		NSPasteboard * tPasteBoard;
			
		tPasteBoard=[info draggingPasteboard];
		
		if (inTableView==IBarray_)
		{
			if ([tPasteBoard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]]!=nil)
			{
				NSArray * tArray;
					
				tArray=[tPasteBoard propertyListForType:NSFilenamesPboardType];
				
				if (tArray!=nil)
				{
					NSMutableIndexSet * tIndexSet;
					
					tIndexSet=[NSMutableIndexSet indexSet];
					
					if (tIndexSet!=nil)
					{
						NSUInteger i,tCount;
						
						tCount=[tArray count];
						
						for(i=0;i<tCount;i++)
						{
							if ([cachedFiles_ containsObject:[tArray objectAtIndex:i]]==NO)
							{
								[tIndexSet addIndex:i];
							}
						}
						
						[cachedFiles_ addObjectsFromArray:[tArray objectsAtIndexes:tIndexSet]];
						
						[cachedFiles_ sortUsingSelector:@selector(caseInsensitiveCompare:)];
			
						[IBarray_ deselectAll:self];
			
						[IBarray_ reloadData];
			
						[IBremoveButton_ setEnabled:NO];
			
						return YES;
					}
				}
			}
		}
	}
	
	return NO;
}


#pragma mark -

- (void) optionKeyStateDidChange:(BOOL) isOptionKeyPressed
{
	if (isOptionKeyPressed==YES)
	{
		[IBaddButton_ setAction:@selector(addExistingFile:)];
	}
	else
	{
		[IBaddButton_ setAction:@selector(addFile:)];
	}
}

- (IBAction) addFile:(id) sender
{
	NSUInteger tRowIndex;
	
	tRowIndex=[cachedFiles_ count];

	[cachedFiles_ addObject:[NSString stringWithString:@"/"]];
	
	[IBarray_ deselectAll:self];
			
	[IBarray_ reloadData];
	
	[IBarray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
	[IBarray_ editColumn:0 row:tRowIndex withEvent:nil select:YES];
}

- (IBAction) addExistingFile:(id) sender
{
	NSOpenPanel * tOpenPanel;
	
	tOpenPanel=[NSOpenPanel openPanel];
	
	if (tOpenPanel!=nil)
	{
		NSString * tLabel;
		
		tLabel=NSLocalizedStringFromTableInBundle(@"Add",@"Localizable",[NSBundle bundleForClass:[self class]],@"No comment");
		
		[tOpenPanel setCanChooseDirectories:YES];
		
		[tOpenPanel setTitle:tLabel];
		
		[tOpenPanel setPrompt:tLabel];
		
		[[tOpenPanel _navView] setShowsHiddenFiles:YES];
		
		[tOpenPanel setAllowsMultipleSelection:YES];
	
		if ([tOpenPanel runModalForDirectory:nil file:nil types:nil]==NSOKButton)
		{
			NSMutableArray * tPaths;
			
			tPaths=[[tOpenPanel filenames] mutableCopy];
			
			if (tPaths!=nil)
			{
				NSUInteger tCount,tIndex;
				
				tCount=[tPaths count];
				
				for(tIndex=tCount;tIndex>0;tIndex--)
				{
					NSString * tPath;
					
					tPath=[tPaths objectAtIndex:tIndex-1];
					
					if ([cachedFiles_ containsObject:tPath]==YES)
					{
						[tPaths removeObjectAtIndex:tIndex-1];
					}
				}
				
				if ([tPaths count]>0)
				{
					[cachedFiles_ addObjectsFromArray:tPaths];
			
					[cachedFiles_ sortUsingSelector:@selector(caseInsensitiveCompare:)];
					
					[IBarray_ deselectAll:self];
					
					[IBarray_ reloadData];
					
					[IBremoveButton_ setEnabled:NO];
				}
				else
				{
					NSBeep();
				}
				
				[tPaths release];
			}
		}
	}
}

- (IBAction) removeFile:(id) sender
{
	NSString * tAlertTitle;
    NSInteger tReturnCode;
	
    if ([IBarray_ numberOfSelectedRows]==1)
    {
        tAlertTitle=NSLocalizedStringFromTableInBundle(@"Do you really want to remove this item?",@"Localizable",[NSBundle bundleForClass:[self class]],@"No comment");
    }
    else
    {
        tAlertTitle=NSLocalizedStringFromTableInBundle(@"Do you really want to remove these items?",@"Localizable",[NSBundle bundleForClass:[self class]],@"No comment");
    }
    
    tReturnCode=NSRunAlertPanel(tAlertTitle, NSLocalizedString(@"This cannot be undone.",@"No comment"), NSLocalizedString(@"Remove",@"No comment"), NSLocalizedString(@"Cancel",@"No comment"), nil);
	
	[self removeFileDidEnd:nil returnCode:tReturnCode contextInfo:NULL];
}

- (void) removeFileDidEnd:(NSWindow *) inWindow returnCode:(NSInteger) returnCode contextInfo:(void *) contextInfo
{
	if (returnCode==NSAlertDefaultReturn)
    {
		NSIndexSet * tIndexSet;
			
		tIndexSet=[IBarray_ selectedRowIndexes];
		
		[IBarray_ deselectAll:self];
		
		if (tIndexSet!=nil)
		{
			[cachedFiles_ removeObjectsAtIndexes:tIndexSet];
			
			[IBarray_ reloadData];
			
			[IBremoveButton_ setEnabled:NO];
		}
	}
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBarray_)
    {
		[IBremoveButton_ setEnabled:([IBarray_ numberOfSelectedRows]!=0)];
	}
}

@end
