#import "ICRequirementConverter_JavaScript_Controller.h"

#import "ICRequirement_JavaScript+Constants.h"
											
@implementation ICRequirementConverter_JavaScript_Controller

- (int) requirementTypeWithParameters:(NSDictionary *) inParameters
{
	return IC_REQUIREMENT_CHECK_TYPE_INSTALLATION; 
}

- (NSDictionary *) sharedFunctionsImplementation
{
	NSMutableString * tSharedSourceCode;
	
	tSharedSourceCode=[self rootObjectForKey:IC_REQUIREMENT_JAVASCRIPT_SHARED_SOURCE_CODE];
	
	if (tSharedSourceCode!=nil)
	{
		static NSString * sJavaScript_UUID=nil;
		
		if (sJavaScript_UUID==nil)
		{
			CFUUIDRef tUUIDRef;
			id tObject;
			
			tObject=tSharedSourceCode;
			
			tUUIDRef=CFUUIDCreate(kCFAllocatorDefault);
	
			if (tUUIDRef!=NULL)
			{
				NSMutableString * tMutableSourceCode;
				NSUInteger tLength;
				
				tLength=[tSharedSourceCode length];
				
				tMutableSourceCode=[[tSharedSourceCode mutableCopy] autorelease];
				
				// Add a tab at the beginning of each line
				
				if (tLength>0)
				{
					[tMutableSourceCode insertString:@"\t" atIndex:0];
				
					[tMutableSourceCode replaceOccurrencesOfString:@"\n" withString:@"\n\t" options:0 range:NSMakeRange(0,tLength)];
				}
				
				sJavaScript_UUID=(NSString *) CFUUIDCreateString(kCFAllocatorDefault,tUUIDRef);
			
				tObject=tMutableSourceCode;
			
				// Release Memory
		
				CFRelease(tUUIDRef);
			}
			
			if (sJavaScript_UUID!=nil && tObject!=nil)
			{
				return [NSDictionary dictionaryWithObjectsAndKeys:tObject,sJavaScript_UUID,nil];
			}
		}
	}
	
	return nil;
}

- (NSString *) invocationWithFormat:(NSString *) inFormat parameters:(NSDictionary *) inParameters index:(int) inIndex error:(NSDictionary **) outErrorDictionary
{
	NSString * tInvocation=nil;
	NSNumber * tNumber;
	NSInteger tReturnValue=IC_REQUIREMENT_JAVASCRIPT_RETURN_VALUE_TRUE;
	NSString * tFunctionName;
	
	tNumber=[inParameters objectForKey:IC_REQUIREMENT_JAVASCRIPT_RETURN_VALUE];
	
	if (tNumber!=nil)
	{
		tReturnValue=[tNumber integerValue];
	}
	
	tFunctionName=[inParameters objectForKey:IC_REQUIREMENT_JAVASCRIPT_FUNCTION];
		
	if (tFunctionName!=nil)
	{
		if ([tFunctionName length]>0)
		{
			NSMutableArray * tMutableArray;
			NSString * tStringParameters=@"";
			NSUInteger tIndex,tCount;
			BOOL nonEmptyParameterFound=NO;
			
			tMutableArray=[[[inParameters objectForKey:IC_REQUIREMENT_JAVASCRIPT_PARAMETERS] mutableCopy] autorelease];
			
			tCount=[tMutableArray count];
			
			for(tIndex=tCount;tIndex>0;tIndex--)
			{
				NSMutableString * tMutableString;
				
				tMutableString=[[[tMutableArray objectAtIndex:(tIndex-1)] mutableCopy] autorelease];
				
				CFStringTrimWhitespace((CFMutableStringRef) tMutableString);
				
				if ([tMutableString length]==0)
				{
					if (nonEmptyParameterFound==YES)
					{
						*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorInvalidParameter],ICRequirementConverterErrorCode,
																					   IC_REQUIREMENT_JAVASCRIPT_PARAMETERS,ICRequirementConverterErrorParameter,
																					   nil];
						return nil;
					}
				}
				else
				{
					nonEmptyParameterFound=YES;
				}
				
				if ([tMutableString length]==0)
				{
					[tMutableArray removeObjectAtIndex:(tIndex-1)];
				}
				else
				{
					[tMutableArray replaceObjectAtIndex:(tIndex-1) withObject:tMutableString];
				}
			}
			
			if ([tMutableArray count]>0)
			{
				tStringParameters=[tMutableArray componentsJoinedByString:@","];
			}
			
			tInvocation=[NSString stringWithFormat:@"(%@(%@)==%@)",tFunctionName,tStringParameters,(tReturnValue==IC_REQUIREMENT_JAVASCRIPT_RETURN_VALUE_TRUE) ? @"true" : @"false"];
		}
		else
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorInvalidParameter],ICRequirementConverterErrorCode,
																		   IC_REQUIREMENT_JAVASCRIPT_FUNCTION,ICRequirementConverterErrorParameter,
																		   nil];
		}
	}
	else
	{
		*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorMissingParameter],ICRequirementConverterErrorCode,
																	   IC_REQUIREMENT_JAVASCRIPT_FUNCTION,ICRequirementConverterErrorParameter,
																	   nil];
	}
	
	return tInvocation;
}

@end
