#import "ICRequirementUI_JavaScript_Controller.h"

#import "ICRequirement_JavaScript+Constants.h"

#import "ICSourceTextViewDelegate.h"

#import "ICSourceTextView+Constants.h"

@implementation ICRequirementUI_JavaScript_Controller

- (void) awakeFromNib
{
	/*NSTableColumn * tTableColumn = nil;
	
	// Path Names
    
    tTableColumn = [IBarray_ tableColumnWithIdentifier:@"Value"];
	
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
		
		if (tTextFieldCell!=nil)
		{
			[tTextFieldCell setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
		}
	}*/
	
	// Register for Notifications
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(functionsListDidChange:)
											     name:IC_SOURCE_TEXTVIEW_FUNCTIONS_LIST_DID_CHANGE
											   object:IBtextViewDelegate_];
											   
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(showDocumentationForKeyword:)
											     name:ICSOURCETEXTVIEW_NOTIFICATION_SHOULD_SHOW_DOCUMENTATION_FOR_KEYWORD
											   object:IBtextView_];
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary mutableDeepCopy];
	
	if (dictionary_!=nil)
	{
		NSString * tString;
		NSNumber * tNumber;
		NSInteger tTag;
		
		// Shared Source Code
		
		tString=[[inDocument rootObjectForKey:IC_REQUIREMENT_JAVASCRIPT_SHARED_SOURCE_CODE] copy];
		
		if (tString!=nil)
		{
			[IBtextView_ setString:tString];
			
			[IBtextViewDelegate_ textDidChange:nil];
			
			[tString release];
		}
		
		// Function
		
		tString=[dictionary_ objectForKey:IC_REQUIREMENT_JAVASCRIPT_FUNCTION];
		
		if (tString!=nil)
		{
			[IBfunctionsComboBox_ setStringValue:tString];
		}
		else
		{
			[IBfunctionsComboBox_ setStringValue:@""];
		}
		
		// Parameters
		
		cachedParameters_=[dictionary_ objectForKey:IC_REQUIREMENT_JAVASCRIPT_PARAMETERS];
		
		if (cachedParameters_==nil)
		{
			cachedParameters_=[NSMutableArray array];
			
			[dictionary_ setObject:cachedParameters_ forKey:IC_REQUIREMENT_JAVASCRIPT_PARAMETERS];
		}
		
		[cachedParameters_ retain];
		
		[IBaddButton_ setEnabled:YES];
		
		[IBremoveButton_ setEnabled:NO];
		
		[IBarray_ reloadData];
		
		[IBarray_ deselectAll:self];
		
		// Return Value
		
		tNumber=[inDictionary objectForKey:IC_REQUIREMENT_JAVASCRIPT_RETURN_VALUE];
		
		if (tNumber==nil)
		{
			tTag=IC_REQUIREMENT_JAVASCRIPT_RETURN_VALUE_TRUE;
		}
		else
		{
			tTag=[tNumber integerValue];
		}
		
		[IBreturnValuePopUpButton_ selectItemWithTag:tTag];
	}
}

- (NSMutableDictionary *) dictionary
{
	NSString * tString;
	
	tString=[[IBtextView_ string] copy];
	
	[document_ setRootObject:tString forKey:IC_REQUIREMENT_JAVASCRIPT_SHARED_SOURCE_CODE];
	
	[tString release];
	
	return [super dictionary];
}

#pragma mark -

- (BOOL) windowCanBeResized
{
	return YES;
}

- (NSMutableDictionary *) defaultDictionary
{
	return [NSMutableDictionary dictionary];
}

- (int) requirementType
{
	return IC_REQUIREMENT_CHECK_TYPE_UNDEFINED;
}

- (NSView *) previousKeyView
{
	return IBtextView_;
}

- (void) setNextKeyView:(NSView *) inView
{
	[IBarray_ setNextKeyView:inView];
}

#pragma mark -

- (CGFloat) minHeight
{
	return 310.0f;
}

#pragma mark -

- (NSInteger) numberOfItemsInComboBox:(NSComboBox *) inComboBox
{
	if (cachedFunctionPrototypes_!=nil)
	{
		return [cachedFunctionPrototypes_ count];
	}
	
	return 0;
}

- (id) comboBox:(NSComboBox *) inComboBox objectValueForItemAtIndex:(NSInteger) inIndex
{
	if (cachedFunctionPrototypes_!=nil && inIndex<[cachedFunctionPrototypes_ count])
	{
		return [cachedFunctionPrototypes_ objectAtIndex:inIndex];
	}
	
	return nil;
}

- (NSUInteger) comboBox:(NSComboBox *) inComboBox indexOfItemWithStringValue:(NSString *) inString
{
	if (cachedFunctionPrototypes_!=nil)
	{
		return [cachedFunctionPrototypes_ indexOfObject:inString];
	}
	
	return NSNotFound;
}

/*- (NSString *)comboBox:(NSComboBox *) inComboBox completedString:(NSString *) inString
{
}*/

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    NSUInteger tParametersCount=0;
	NSUInteger tPrototypeParametersCount=0;
	
	if (cachedParameters_!=nil)
	{
		tParametersCount=[cachedParameters_ count];
	}
	
	if (cachedFunctionPrototypeParameters_!=nil)
	{
		tPrototypeParametersCount=[cachedFunctionPrototypeParameters_ count];
	}
	
	if (tPrototypeParametersCount>tParametersCount)
	{
		return tPrototypeParametersCount;
	}
	
	return tParametersCount;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (cachedParameters_!=nil)
	{
		NSString * tColumnIdentifier;
	
		tColumnIdentifier=[inTableColumn identifier];
		
		if ([tColumnIdentifier isEqualToString:@"Value"]==YES)
		{
			if (inRowIndex<[cachedParameters_ count])
			{
				NSMutableString * tMutableString;
				
				tMutableString=[[cachedParameters_ objectAtIndex:inRowIndex] mutableCopy];
				
				if (tMutableString!=nil)
				{
					[tMutableString autorelease];
					
					CFStringTrimWhitespace((CFMutableStringRef) tMutableString);
				
					if ([tMutableString length]>0)
					{
						return tMutableString;
					}
				}
			}
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	
	tColumnIdentifier=[inTableColumn identifier];
	
	if ([tColumnIdentifier isEqualToString:@"Value"]==YES)
	{
		if (cachedParameters_!=nil)
		{
			NSUInteger tCount;
			
			tCount=[cachedParameters_ count];
			
			if (inRowIndex>=tCount)
			{
				NSUInteger tIndex;
				
				for(tIndex=tCount;tIndex<inRowIndex;tIndex++)
				{
					[cachedParameters_ addObject:@""];
				}
				
				[cachedParameters_ addObject:object];
			}
			else
			{
				[cachedParameters_ replaceObjectAtIndex:inRowIndex withObject:object];
			}
		}
	}
}

- (void) tableView:(NSTableView *) inTableView willDisplayCell:(id) cell forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	NSString * tColumnIdentifier;
	
	tColumnIdentifier=[inTableColumn identifier];
	
	if ([tColumnIdentifier isEqualToString:@"Value"]==YES)
	{
		if (cachedFunctionPrototypeParameters_!=nil)
		{
			if (inRowIndex<[cachedFunctionPrototypeParameters_ count])
			{
				NSTextFieldCell * tTextFieldCell;
				
				tTextFieldCell = (NSTextFieldCell *) cell;
			
				[tTextFieldCell setPlaceholderString:[cachedFunctionPrototypeParameters_ objectAtIndex:inRowIndex]];
			}
		}
	}
}

#pragma mark -

- (IBAction) setFunctionName:(id) sender
{
	NSString * tString;
	
	[self controlTextDidChange:[NSNotification notificationWithName:NSTextDidChangeNotification object:IBfunctionsComboBox_]];
	
	tString=[IBfunctionsComboBox_ stringValue];
	
	if (tString!=nil)
	{
		[dictionary_ setObject:tString forKey:IC_REQUIREMENT_JAVASCRIPT_FUNCTION];
	}
}

- (IBAction) addParameter:(id) sender
{
	NSUInteger tRowIndex;
	
	tRowIndex=[cachedParameters_ count];

	[cachedParameters_ addObject:[NSString stringWithString:@""]];
	
	[IBarray_ deselectAll:self];
			
	[IBarray_ reloadData];
	
	[IBarray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
				
	[IBarray_ editColumn:0 row:tRowIndex withEvent:nil select:YES];
}

- (IBAction) removeParameters:(id) sender
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBarray_ selectedRowIndexes];
	
	if (tIndexSet!=nil)
	{
		[cachedParameters_ removeObjectsAtIndexes:tIndexSet];
	
		[IBarray_ deselectAll:self];
		
		[IBarray_ reloadData];
		
		//[IBremoveButton_ setEnabled:NO];
	}
}

- (IBAction) switchReturnValue:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_JAVASCRIPT_RETURN_VALUE];
	}
}

#pragma mark -

- (void) controlTextDidChange:(NSNotification *) inNotification
{
	if ([inNotification object]==IBfunctionsComboBox_)
	{
		NSString * tFunctionName;
		
		tFunctionName=[IBfunctionsComboBox_ stringValue];
		
		[cachedFunctionPrototypeParameters_ release];
		
		cachedFunctionPrototypeParameters_=nil;
		
		if (tFunctionName!=nil)
		{
			cachedFunctionPrototypeParameters_=[[IBtextViewDelegate_ parametersForFunctionNamed:tFunctionName] retain];
		}
		
		[IBarray_ reloadData];
	}
}

- (void) functionsListDidChange:(NSNotification *) inNotification
{
	NSString * tFunctionName;
	
	[cachedFunctionPrototypes_ release];
	
	cachedFunctionPrototypes_=nil;
	
	// Refresh Combox Box Menu

	cachedFunctionPrototypes_=[[IBtextViewDelegate_ sortedFunctionsList] retain];
	
	tFunctionName=[IBfunctionsComboBox_ stringValue];
	
	[IBfunctionsComboBox_ reloadData];
	
	[cachedFunctionPrototypeParameters_ release];
	
	cachedFunctionPrototypeParameters_=nil;
	
	if (tFunctionName!=nil)
	{
		cachedFunctionPrototypeParameters_=[[IBtextViewDelegate_ parametersForFunctionNamed:tFunctionName] retain];
	}
	
	[IBarray_ reloadData];
}

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBarray_)
    {
		[IBremoveButton_ setEnabled:([IBarray_ numberOfSelectedRows]!=0)];
	}
}

- (void) showDocumentationForKeyword:(NSNotification *) inNotification
{
	if ([inNotification object]==IBtextView_)
    {
		NSString * tKeyword;
		
		tKeyword=[[inNotification userInfo] objectForKey:ICSOURCETEXTVIEW_NOTIFICATION_KEYWORD];
		
		if (tKeyword!=nil && [tKeyword length]>0)
		{
			static NSDictionary * sJavaScriptDocumentationReference=nil;
			
			if (sJavaScriptDocumentationReference==nil)
			{
				NSString * tPath;
				
				tPath=[[NSBundle bundleWithIdentifier:@"fr.whitebox.Packages.requirement.javascript.ui"] pathForResource:@"" ofType:@"plist"];
				
				if (tPath!=nil)
				{
					sJavaScriptDocumentationReference=[[NSDictionary alloc] initWithContentsOfFile:tPath];
				}
			}
			
			if (sJavaScriptDocumentationReference!=nil)
			{
				NSArray * tOccurrences;
				
				tOccurrences=[sJavaScriptDocumentationReference objectForKey:tKeyword];
				
				if (tOccurrences!=nil)
				{
					NSUInteger tCount;
					
					tCount=[tOccurrences count];
					
					if (tCount==1)
					{
						NSDictionary * tDictionary;
						NSString * tURLString;
						
						tDictionary=[tOccurrences objectAtIndex:0];
						
						tURLString=[tDictionary objectForKey:@"URL"];
						
						if (tURLString!=nil)
						{
							NSURL * tURL=nil;
    
							tURL=[NSURL URLWithString:tURLString];
							
							if (tURL!=nil)
							{
								[[NSWorkspace sharedWorkspace] openURL:tURL];
							}
						}
					}
					else if (tCount>1)
					{
						// A COMPLETER
					}
				}
			}
			else
			{
				NSBeep();
			}
		}
	}
}

@end
