/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementConverter_OS_Controller.h"

#import "ICRequirement_OS+Constants.h"


@implementation ICRequirementConverter_OS_Controller

- (int) requirementTypeWithParameters:(NSDictionary *) inParameters
{
	if (inParameters!=nil)
	{
		NSNumber * tNumber;
	
		tNumber=[inParameters objectForKey:IC_REQUIREMENT_OS_DISK_TYPE];
		
		if (tNumber!=nil)
		{
			NSInteger tDiskType;
			
			tDiskType=[tNumber integerValue];
			
			if (tDiskType==IC_OS_DISK_TYPE_STARTUP_DISK)
			{
				return IC_REQUIREMENT_CHECK_TYPE_INSTALLATION;
			}
			else if (tDiskType==IC_OS_DISK_TYPE_DESTINATION)
			{
				return IC_REQUIREMENT_CHECK_TYPE_TARGET;
			}
		}
	}
	
	return IC_REQUIREMENT_CHECK_TYPE_UNDEFINED;
}

- (NSString *) invocationWithFormat:(NSString *) inFormat parameters:(NSDictionary *) inParameters index:(int) inIndex error:(NSDictionary **) outErrorDictionary
{
	NSString * tInvocation=nil;
	NSNumber * tNumber;
	
	tNumber=[inParameters objectForKey:IC_REQUIREMENT_OS_DISK_TYPE];
	
	if (tNumber!=nil)
	{
		NSInteger tDiskType;
		
		tDiskType=[tNumber integerValue];
		
		if (tDiskType>=IC_OS_DISK_TYPE_DESTINATION && tDiskType<=IC_OS_DISK_TYPE_STARTUP_DISK)
		{
			NSString * tDiskTypeString=@"IC_DISK_TYPE_DESTINATION";
			
			tNumber=[inParameters objectForKey:IC_REQUIREMENT_OS_MINIMUM_VERSION];
			
			if (tNumber!=nil)
			{
				NSInteger tMinimumVersion;
				
				tMinimumVersion=[tNumber integerValue];
				
				if (tMinimumVersion>=0)
				{
					NSString * tMinimumVersionString;
					
					if (tMinimumVersion==IC_OS_MINIMUM_VERSION_NOT_INSTALLED)
					{
						tInvocation=[NSString stringWithFormat:inFormat,tDiskTypeString,@"false",@"''",@"IC_OS_DISTRIBUTION_TYPE_ANY"];
					}
					else
					{
						if (tDiskType==IC_OS_DISK_TYPE_STARTUP_DISK)
						{
							tDiskTypeString=@"IC_DISK_TYPE_STARTUP_DISK";
						}
						
						/* Create Minimum Version String */
						
						if ((tMinimumVersion%100)==0)
						{
							tMinimumVersionString=[NSString stringWithFormat:@"'%d.%d'",tMinimumVersion/10000,((tMinimumVersion/100)%100)];
						}
						else
						{
							tMinimumVersionString=[NSString stringWithFormat:@"'%d.%d.%d'",tMinimumVersion/10000,((tMinimumVersion/100)%100),(tMinimumVersion%100)];
						}
						
						tNumber=[inParameters objectForKey:IC_REQUIREMENT_OS_DISTRIBUTION_TYPE];
			
						if (tNumber!=nil)
						{
							NSInteger tDistributionType;
							
							tDistributionType=[tNumber integerValue];
							
							if (tDistributionType>=IC_OS_DISTRIBUTION_TYPE_ANY && tDistributionType<=IC_OS_DISTRIBUTION_TYPE_SERVER)
							{
								NSString * tDistributionTypeString=@"IC_OS_DISTRIBUTION_TYPE_ANY";
								
								if (tDistributionType==IC_OS_DISTRIBUTION_TYPE_CLIENT)
								{
									tDistributionTypeString=@"IC_OS_DISTRIBUTION_TYPE_CLIENT";
								}
								else if (tDistributionType==IC_OS_DISTRIBUTION_TYPE_CLIENT)
								{
									tDistributionTypeString=@"IC_OS_DISTRIBUTION_TYPE_SERVER";
								}
								
								
								tInvocation=[NSString stringWithFormat:inFormat,tDiskTypeString,@"true",tMinimumVersionString,tDistributionTypeString];
							}
							else
							{
								*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorInvalidParameter],ICRequirementConverterErrorCode,
																							   IC_REQUIREMENT_OS_DISTRIBUTION_TYPE,ICRequirementConverterErrorParameter,
																							   nil];
							}
						}
						else
						{
							*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorMissingParameter],ICRequirementConverterErrorCode,
																						   IC_REQUIREMENT_OS_DISTRIBUTION_TYPE,ICRequirementConverterErrorParameter,
																						   nil];
						}
						
					}
				}
				else
				{
					*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorInvalidParameter],ICRequirementConverterErrorCode,
																				   IC_REQUIREMENT_OS_MINIMUM_VERSION,ICRequirementConverterErrorParameter,
																				   nil];
				}
				
			}
			else
			{
				*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorMissingParameter],ICRequirementConverterErrorCode,
																			   IC_REQUIREMENT_OS_MINIMUM_VERSION,ICRequirementConverterErrorParameter,
																			   nil];
			}
				
			
			
		}
		else
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorInvalidParameter],ICRequirementConverterErrorCode,
																		   IC_REQUIREMENT_OS_DISK_TYPE,ICRequirementConverterErrorParameter,
																		   nil];
		}
	}
	else
	{
		*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorMissingParameter],ICRequirementConverterErrorCode,
																	   IC_REQUIREMENT_OS_DISK_TYPE,ICRequirementConverterErrorParameter,
																	   nil];
	}
	
	return tInvocation;
}

@end
