/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementUI_OS_Controller.h"

#import "ICRequirement_OS+Constants.h"

@implementation ICRequirementUI_OS_Controller

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary mutableDeepCopy];
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
		NSInteger tTag;
		
		// Disk Type
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_OS_DISK_TYPE];
		
		if (tNumber==nil)
		{
			tTag=IC_OS_DISK_TYPE_DESTINATION;
		}
		else
		{
			tTag=[tNumber integerValue];
		}
		
		[IBdiskType_ selectItemWithTag:tTag];
		
		// Minimum Version
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_OS_MINIMUM_VERSION];
		
		if (tNumber==nil)
		{
			tTag=IC_OS_MINIMUM_VERSION_1050;
		}
		else
		{
			tTag=[tNumber integerValue];
		}
		
		[IBminimumVersion_ selectItemWithTag:tTag];
		
		// Distribution Type
		
		if (tTag==IC_OS_MINIMUM_VERSION_NOT_INSTALLED)
		{
			[IBdistribution_ setEnabled:NO];
			
			[IBdistribution_ selectSegmentWithTag:IC_OS_DISTRIBUTION_TYPE_ANY];
		}
		else
		{
			tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_OS_DISTRIBUTION_TYPE];
		
			if (tNumber==nil)
			{
				tTag=IC_OS_DISTRIBUTION_TYPE_ANY;
			}
			else
			{
				tTag=[tNumber intValue];
			}
			
			[IBdistribution_ selectSegmentWithTag:tTag];
		}
	}
}

#pragma mark -

- (NSMutableDictionary *) defaultDictionary
{
	return [NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInteger:IC_OS_DISK_TYPE_DESTINATION],IC_REQUIREMENT_OS_DISK_TYPE,
															 [NSNumber numberWithInteger:IC_OS_MINIMUM_VERSION_1050],IC_REQUIREMENT_OS_MINIMUM_VERSION,
															 [NSNumber numberWithInteger:IC_OS_DISTRIBUTION_TYPE_ANY],IC_REQUIREMENT_OS_DISTRIBUTION_TYPE,
															 nil];
}

- (int) requirementType
{
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
	
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_OS_DISK_TYPE];
		
		if (tNumber!=nil)
		{
			int tDiskType;
			
			tDiskType=[tNumber integerValue];
			
			if (tDiskType==IC_OS_DISK_TYPE_STARTUP_DISK)
			{
				return IC_REQUIREMENT_CHECK_TYPE_INSTALLATION;
			}
			else if (tDiskType==IC_OS_DISK_TYPE_DESTINATION)
			{
				return IC_REQUIREMENT_CHECK_TYPE_TARGET;
			}
		}
	}
	
	return IC_REQUIREMENT_CHECK_TYPE_UNDEFINED;
}

#pragma mark -

- (BOOL) validateMenuItem:(NSMenuItem *) inMenuItem
{    
    if ([inMenuItem action]==@selector(switchMinimumVersion:))
    {
		if ([inMenuItem tag]==IC_OS_MINIMUM_VERSION_NOT_INSTALLED)
		{
			NSNumber * tNumber;
			NSInteger tTag;
			
			tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_OS_DISK_TYPE];
			
			if (tNumber==nil)
			{
				tTag=IC_OS_DISK_TYPE_DESTINATION;
			}
			else
			{
				tTag=[tNumber integerValue];
			}
		
			return (tTag!=IC_OS_DISK_TYPE_STARTUP_DISK);
		}
	}
	
	return YES;
}

#pragma mark -

- (IBAction) switchDiskType:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_OS_DISK_TYPE];
		
		if (tTag==IC_OS_DISK_TYPE_STARTUP_DISK)
		{
			tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_OS_MINIMUM_VERSION];
			
			tTag=[tNumber intValue];
			
			if (tTag==IC_OS_MINIMUM_VERSION_NOT_INSTALLED)
			{
				tTag=IC_OS_MINIMUM_VERSION_1050;
				
				tNumber=[NSNumber numberWithInteger:tTag];
				
				if (tNumber!=nil)
				{
					[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_OS_MINIMUM_VERSION];
				}
				
				[IBminimumVersion_ selectItemWithTag:tTag];
			}
			
			[IBdistribution_ setEnabled:YES];
		}
		
		[self noteCheckTypeChange];
	}
}

- (IBAction) switchMinimumVersion:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_OS_MINIMUM_VERSION];
		
		if (tTag==IC_OS_MINIMUM_VERSION_NOT_INSTALLED)
		{
			[dictionary_ setObject:[NSNumber numberWithInteger:IC_OS_DISTRIBUTION_TYPE_ANY] forKey:IC_REQUIREMENT_OS_DISTRIBUTION_TYPE];
			
			[IBdistribution_ setEnabled:NO];
			
			[IBdistribution_ selectSegmentWithTag:IC_OS_DISTRIBUTION_TYPE_ANY];
		}
		else
		{
			[IBdistribution_ setEnabled:YES];
		}		
	}
}

- (IBAction) switchDistribution:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender cell] tagForSegment:[sender selectedSegment]];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_OS_DISTRIBUTION_TYPE];
	}
}

@end
