/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementConverter_Script_Controller.h"

#import "ICRequirement_Script+Constants.h"

#import "ICDocumentFormat+MinimumConstants.h"

@implementation ICRequirementConverter_Script_Controller

- (NSDictionary *) requiredOptionsValuesWithParameters:(NSDictionary *) inParameters
{
	return [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],@"installer-script.options:allow-external-scripts",nil];
}

- (NSArray *) requiredAdditionalResourcesWithParameters:(NSDictionary *) inParameters
{
	NSNumber * tEmbedScript;
		
	tEmbedScript=[inParameters objectForKey:IC_REQUIREMENT_SCRIPT_EMBED];
		
	if (tEmbedScript!=nil && [tEmbedScript boolValue]==YES)
	{
		NSDictionary * tDictionary;
		
		tDictionary=[inParameters objectForKey:IC_REQUIREMENT_SCRIPT_PATH];
		
		if (tDictionary!=nil)
		{
			return [NSArray arrayWithObject:[NSDictionary dictionaryWithObjectsAndKeys:tDictionary,@"PATH",[NSNumber numberWithInt:0775],@"PERMISSIONS",nil]];
		}
	}
	
	return nil;
}

- (int) requirementTypeWithParameters:(NSDictionary *) inParameters
{
	return IC_REQUIREMENT_CHECK_TYPE_UNDEFINED;
}

- (NSString *) variablesWithIndex:(int) inIndex tabulationDepth:(NSString *) inTabulationDepth parameters:(NSDictionary *) inParameters error:(NSDictionary **) outErrorDictionary
{
	NSArray * tArray;
	
	tArray=[inParameters objectForKey:IC_REQUIREMENT_SCRIPT_ARGUMENTS];
	
	if (tArray!=nil)
	{
		NSUInteger tCount;
		NSMutableString * tMutableString;
		
		tCount=[tArray count];
		
		tMutableString=[NSMutableString stringWithFormat:@"%@var tScriptArguments%d=new Array(",inTabulationDepth,inIndex];
		
		if (tMutableString!=nil)
		{
			NSUInteger i;
			int tIndex=0;
			
			for(i=0;i<tCount;i++)
			{
				NSDictionary * tArgumentDictionary;
				NSNumber * tNumber;
				
				tArgumentDictionary=[tArray objectAtIndex:i];
				
				tNumber=[tArgumentDictionary objectForKey:IC_REQUIREMENT_SCRIPT_ARGUMENT_STATE];
				
				if (tNumber!=nil && [tNumber boolValue]==YES)
				{
					NSString * tArgumentValue;
					
					tArgumentValue=[tArgumentDictionary objectForKey:IC_REQUIREMENT_SCRIPT_ARGUMENT_VALUE];
					
					if ([tArgumentValue length]>0)
					{
						/* Escape the string (" > < &") */
					
						tArgumentValue=(NSString *) CFXMLCreateStringByEscapingEntities(kCFAllocatorDefault,(CFStringRef) tArgumentValue,NULL);
						
						if (tArgumentValue!=nil)
						{
							if (tIndex!=0)
							{
								[tMutableString appendFormat:@",\n%@                             ",inTabulationDepth];
							}
							
							[tMutableString appendFormat:@"\'%@\'",tArgumentValue];
							
							tIndex++;
						
							[tArgumentValue release];
						}
						else
						{
							/* A COMPLETER */
						}
						
						
					}
				}
			}
			
			[tMutableString appendString:@");\n\n"];
			
			return tMutableString;
		}
	}
	else
	{
		*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorMissingParameter],ICRequirementConverterErrorCode,
																	   IC_REQUIREMENT_SCRIPT_ARGUMENTS,ICRequirementConverterErrorParameter,
																	   nil];
	}
	
	return nil;
}

- (NSString *) invocationWithFormat:(NSString *) inFormat parameters:(NSDictionary *) inParameters index:(int) inIndex error:(NSDictionary **) outErrorDictionary
{
	NSString * tInvocation=nil;
	NSString * tPath;
	
	
	tPath=[[inParameters objectForKey:IC_REQUIREMENT_SCRIPT_PATH] objectForKey:ICDOCUMENT_PATH];
	
	if (tPath==nil)
	{
		*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorMissingParameter],ICRequirementConverterErrorCode,
																	   IC_REQUIREMENT_SCRIPT_PATH,ICRequirementConverterErrorParameter,
																	   nil];
	}
	else
	{
		NSNumber * tEmbedScript;
		
		tEmbedScript=[inParameters objectForKey:IC_REQUIREMENT_SCRIPT_EMBED];
		
		if (tEmbedScript==nil)
		{
			*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorMissingParameter],ICRequirementConverterErrorCode,
																		   IC_REQUIREMENT_SCRIPT_EMBED,ICRequirementConverterErrorParameter,
																		   nil];
		}
		else
		{
			NSNumber * tComparatorNumber;
			
			if ([tEmbedScript boolValue]==YES)
			{
				tPath=[tPath lastPathComponent];
			}
			
			tComparatorNumber=[inParameters objectForKey:IC_REQUIREMENT_SCRIPT_COMPARATOR];
			
			if (tComparatorNumber==nil)
			{
				*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorMissingParameter],ICRequirementConverterErrorCode,
																			   IC_REQUIREMENT_SCRIPT_COMPARATOR,ICRequirementConverterErrorParameter,
																			   nil];
			}
			else
			{
				NSString * tComparatorString=nil;
				int tComparator;
				
				tComparator=[tComparatorNumber intValue];
				
				if (tComparator==IC_REQUIREMENT_COMPARATOR_IS_LESS)
				{
					tComparatorString=@"IC_COMPARATOR_IS_LESS";
				}
				else if (tComparator==IC_REQUIREMENT_COMPARATOR_IS_EQUAL)
				{
					tComparatorString=@"IC_COMPARATOR_IS_EQUAL";
				}
				else if (tComparator==IC_REQUIREMENT_COMPARATOR_IS_GREATER)
				{
					tComparatorString=@"IC_COMPARATOR_IS_GREATER";
				}
				else if (tComparator==IC_REQUIREMENT_COMPARATOR_IS_NOT_EQUAL)
				{
					tComparatorString=@"IC_COMPARATOR_IS_NOT_EQUAL";
				}
				else
				{
					*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorInvalidParameter],ICRequirementConverterErrorCode,
																				   IC_REQUIREMENT_SCRIPT_COMPARATOR,ICRequirementConverterErrorParameter,
																				   nil];
				}
				
				if (tComparatorString!=nil)
				{
					NSNumber * tReturnValueNumber;
					
					tReturnValueNumber=[inParameters objectForKey:IC_REQUIREMENT_SCRIPT_VALUE];
					
					if (tReturnValueNumber==nil)
					{
						*outErrorDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICRequirementConverterErrorMissingParameter],ICRequirementConverterErrorCode,
																					   IC_REQUIREMENT_SCRIPT_VALUE,ICRequirementConverterErrorParameter,
																					   nil];
					}
					else
					{
						tInvocation=[NSString stringWithFormat:inFormat,tPath,[NSString stringWithFormat:@"tScriptArguments%d",inIndex],tComparatorString,[tReturnValueNumber intValue]];
					}
				}
			}
		}
	}
	
	return tInvocation;
}

@end
