/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementUI_Script_Controller.h"

#import "ICIntegerFormatter.h"

#import "ICRequirement_Script+Constants.h"

#import "ICReferencedFileTextField.h"

#import "ICDocumentFormat+Constants.h"

@implementation ICRequirementUI_Script_Controller

- (void) awakeFromNib
{
	NSTableColumn * tTableColumn = nil;
	ICIntegerFormatter * tFormatter;
	
	// Return Value
	
	tFormatter=[ICIntegerFormatter new];
	
	if (tFormatter!=nil)
	{
		[IBreturnValue_ setFormatter:tFormatter];
		
		[tFormatter release];
	}
	
	// Array
	
	tTableColumn = [IBarray_ tableColumnWithIdentifier:@"State"];
    
	if (tTableColumn!=nil)
	{
		NSButtonCell * tButttonCell;
		
		tButttonCell = [[NSButtonCell alloc] initTextCell: @""];
		
		if (tButttonCell!=nil)
		{
			[tButttonCell setControlSize:NSMiniControlSize];
			[tButttonCell setEditable: YES];
			[tButttonCell setButtonType: NSSwitchButton];
			[tButttonCell setImagePosition: NSImageOnly];
			
			[tTableColumn setDataCell:tButttonCell];
			
			[tButttonCell release];
		}
	}
	
	// Name
	
	tTableColumn = [IBarray_ tableColumnWithIdentifier:@"Value"];
    
	if (tTableColumn!=nil)
	{
		NSCell * tTextFieldCell;
		
		tTextFieldCell = [tTableColumn dataCell];
		
		[tTextFieldCell setFont:[NSFont systemFontOfSize:11.0f]];
	}
}

- (void) initializeWithDictionary:(NSMutableDictionary *) inDictionary forDocument:(id) inDocument
{
	[super initializeWithDictionary:inDictionary forDocument:inDocument];
	
	dictionary_=[inDictionary mutableDeepCopy];
	
	if (dictionary_!=nil)
	{
		NSNumber * tNumber;
		NSInteger tState=NSOnState;
		NSInteger tTag=IC_REQUIREMENT_COMPARATOR_IS_EQUAL;
		NSMutableDictionary * tScriptPathDictionary;
		
		[IBembedCheckBox_ setEnabled:YES];
		
		// Embed
		
		[IBembeddedWarning_ setHidden:YES];
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_SCRIPT_EMBED];
		
		if (tNumber!=nil)
		{
			tState=([tNumber boolValue]==YES) ? NSOnState : NSOffState;
		}
		
		[IBembedCheckBox_ setState:tState];
		
		if (tState==NSOnState)
		{
			if ([document_ isFlatDistribution]==YES)
			{
				[IBembeddedWarning_ setHidden:NO];
			}
		}
		
		[IBembedCheckBox_ setEnabled:YES];
		
		// Script Path
		
		[IBscriptPath_ setDocument:inDocument];
		
		tScriptPathDictionary=[dictionary_ objectForKey:IC_REQUIREMENT_SCRIPT_PATH];
        
		if (tScriptPathDictionary!=nil)
		{
			NSString * tPath;
			NSInteger tType;
			
			tPath=[tScriptPathDictionary objectForKey:ICDOCUMENT_PATH];
			
			tNumber=[tScriptPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE];
			
			tType=[tNumber integerValue];
			
			if (tType==0)
			{
				tType=ICDOCUMENT_PATH_TYPE_GLOBAL;
			}
			
			[IBscriptPath_ _setPathType:tType];
			
			if (tPath!=nil)
			{
				[IBscriptPath_ setStringValue:tPath];
			}
			else
			{
				[IBscriptPath_ setStringValue:@""];
			}
			
			if (tType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
			{
				[IBembedCheckBox_ setEnabled:NO];
				
				[IBembedCheckBox_ setState:NSOffState];
			}
			else
			{
				[IBembedCheckBox_ setEnabled:YES];
			}
		}
		else
		{
			[dictionary_ setObject:[NSMutableDictionary dictionary] forKey:IC_REQUIREMENT_SCRIPT_PATH];
			
			[IBscriptPath_ _setPathType:ICDOCUMENT_PATH_TYPE_GLOBAL];
			
			[IBscriptPath_ setStringValue:@""];
		}
		
		
		
		// Arguments
		
		cachedArguments_=[[dictionary_ objectForKey:IC_REQUIREMENT_SCRIPT_ARGUMENTS] retain];
		
		[IBaddButton_ setEnabled:YES];
		
		[IBremoveButton_ setEnabled:NO];
		
		[IBarray_ reloadData];
		
		[IBarray_ deselectAll:self];
		
		// Comparator
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_SCRIPT_COMPARATOR];
		
		if (tNumber!=nil)
		{
			tTag=[tNumber intValue];
		}
		
		if ([IBcomparator_ selectItemWithTag:tTag]==NO)
		{
			// A COMPLETER
		}
		
		// Return Value
		
		tNumber=[dictionary_ objectForKey:IC_REQUIREMENT_SCRIPT_VALUE];
		
		if (tNumber==nil)
		{
			[IBreturnValue_ setStringValue:@"0"];
		}
		else
		{
			[IBreturnValue_ setStringValue:[tNumber stringValue]];
		}
	}
}

- (void) dealloc
{
	[cachedArguments_ release];
	
	[super dealloc];
}

#pragma mark -

- (NSMutableDictionary *) defaultDictionary
{
	return [NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],IC_REQUIREMENT_SCRIPT_EMBED,
			[NSMutableArray array],IC_REQUIREMENT_SCRIPT_ARGUMENTS,
			[NSNumber numberWithInt:IC_REQUIREMENT_COMPARATOR_IS_EQUAL],IC_REQUIREMENT_SCRIPT_COMPARATOR,
			[NSNumber numberWithInt:0],IC_REQUIREMENT_SCRIPT_VALUE,
			nil];
}

- (int) requirementType
{
	return IC_REQUIREMENT_CHECK_TYPE_UNDEFINED;
}

- (NSView *) previousKeyView
{
	return IBscriptPath_;
}

- (void) setNextKeyView:(NSView *) inView
{
	[IBreturnValue_ setNextKeyView:inView];
}

#pragma mark -

- (NSInteger) numberOfRowsInTableView:(NSTableView *) inTableView
{
    if (cachedArguments_!=nil)
	{
		return [cachedArguments_ count];
	}
	
	return 0;
}

- (id) tableView:(NSTableView *) inTableView objectValueForTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (cachedArguments_!=nil)
	{
		NSMutableDictionary * tDictionary;
		NSString * tColumnIdentifier;
		
		tColumnIdentifier=[inTableColumn identifier];
		
		tDictionary=[cachedArguments_ objectAtIndex:inRowIndex];
		
		if ([tColumnIdentifier isEqualToString: @"State"])
		{
			return [tDictionary objectForKey:IC_REQUIREMENT_SCRIPT_ARGUMENT_STATE];
		}
		else if ([tColumnIdentifier isEqualToString: @"Value"])
		{
			return [tDictionary objectForKey:IC_REQUIREMENT_SCRIPT_ARGUMENT_VALUE];
		}
	}
	
	return nil;
}

- (void) tableView:(NSTableView *) inTableView setObjectValue:(id) object forTableColumn:(NSTableColumn *) inTableColumn row:(NSInteger) inRowIndex
{
	if (inTableView==IBarray_)
	{
		if (cachedArguments_!=nil)
		{
			NSMutableDictionary * tDictionary;
			NSString * tColumnIdentifier;
			
			tColumnIdentifier=[inTableColumn identifier];
			
			tDictionary=[cachedArguments_ objectAtIndex:inRowIndex];
			
			if ([tColumnIdentifier isEqualToString: @"State"])
			{
				NSNumber * tNumber;
				
				tNumber=[tDictionary objectForKey:IC_REQUIREMENT_SCRIPT_ARGUMENT_STATE];
				
				if ([tNumber boolValue]!=[object boolValue])
				{
					[tDictionary setObject:object forKey:IC_REQUIREMENT_SCRIPT_ARGUMENT_STATE];
					
				}
			}
			else if ([tColumnIdentifier isEqualToString: @"Value"])
			{
				[tDictionary setObject:object forKey:IC_REQUIREMENT_SCRIPT_ARGUMENT_VALUE];
			}
		}
	}
}

- (void) control:(NSControl *) inControl didFailToValidatePartialString:(NSString *) inPartialString errorDescription:(NSString *) inError
{
	if (inError!=nil && [inError isEqualToString:@"NSBeep"]==YES)
	{
		NSBeep();
	}
}

#pragma mark -

- (IBAction) updateScriptPath:(id) sender
{
	NSMutableDictionary * tScriptPathDictionary;
	
	tScriptPathDictionary=[dictionary_ objectForKey:IC_REQUIREMENT_SCRIPT_PATH];
	
	if (tScriptPathDictionary!=nil)
	{
		NSInteger tOldType,tNewType;
		NSString * tOldPath,* tNewPath;
		
		tOldPath=[tScriptPathDictionary objectForKey:ICDOCUMENT_PATH];
		
		tNewPath=[IBscriptPath_ stringValue];
		
		tOldType=[[tScriptPathDictionary objectForKey:ICDOCUMENT_PATH_TYPE] integerValue];
		
		tNewType=[IBscriptPath_ pathType];
		
		if ([tOldPath isEqualToString:tNewPath]==NO ||
			tOldType!=tNewType)
		{
			[tScriptPathDictionary setObject:tNewPath forKey:ICDOCUMENT_PATH];
			
			[tScriptPathDictionary setObject:[NSNumber numberWithInteger:tNewType] forKey:ICDOCUMENT_PATH_TYPE];
		}
	}
}

- (IBAction) selectScriptPath:(id) sender
{
    NSOpenPanel * tOpenPanel;
    
    tOpenPanel=[NSOpenPanel openPanel];
    
    [tOpenPanel setPrompt:NSLocalizedString(@"Choose",@"No comment")];
    
    [tOpenPanel beginSheetForDirectory:[IBscriptPath_ absolutePath]
                                  file:nil
                                 types:nil
                        modalForWindow:[IBscriptPath_ window]
                         modalDelegate:self
                        didEndSelector:@selector(scriptPathOpenPanelDidEnd:returnCode:contextInfo:)
                           contextInfo:NULL];
}

- (void) scriptPathOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode==NSOKButton)
    {
        [IBscriptPath_ setAbsolutePath:[sheet filename]];
        
        [self updateScriptPath:IBscriptPath_];
    }
}

- (IBAction) revealScriptPathInFinder:(id) sender
{
    NSWorkspace * tWorkSpace;
    
    tWorkSpace=[NSWorkspace sharedWorkspace];
    
    [tWorkSpace selectFile:[IBscriptPath_ absolutePath] inFileViewerRootedAtPath:@""];
}

- (IBAction) switchEmbed:(id) sender
{
	NSNumber * tNumber;
	NSInteger tState;
	
	tState=[sender state];
	
	tNumber=[NSNumber numberWithBool:(tState==NSOnState)];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_SCRIPT_EMBED];
	}
	
	[IBembeddedWarning_ setHidden:YES];
	
	if (tState==NSOnState)
	{
		if ([document_ isFlatDistribution]==YES)
		{
			[IBembeddedWarning_ setHidden:NO];
		}
	}
}

- (IBAction) addArgument:(id) sender
{
	NSUInteger tRowIndex;
	
	tRowIndex=[cachedArguments_ count];
	
	[cachedArguments_ addObject:[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],IC_REQUIREMENT_SCRIPT_ARGUMENT_STATE,
								 @"",IC_REQUIREMENT_SCRIPT_ARGUMENT_VALUE,
								 nil]];
	
	[IBarray_ deselectAll:self];
	
	[IBarray_ reloadData];
	
	[IBarray_ selectRowIndexes:[NSIndexSet indexSetWithIndex:tRowIndex] byExtendingSelection:NO];
	
	[IBarray_ editColumn:1 row:tRowIndex withEvent:nil select:YES];
}

- (IBAction) removeArgument:(id) sender
{
	NSIndexSet * tIndexSet;
	
	tIndexSet=[IBarray_ selectedRowIndexes];
	
	[IBarray_ deselectAll:self];
	
	if (tIndexSet!=nil)
	{
		[cachedArguments_ removeObjectsAtIndexes:tIndexSet];
		
		[IBarray_ reloadData];
		
		[IBremoveButton_ setEnabled:NO];
	}
}

- (IBAction) switchComparator:(id) sender
{
	NSInteger tTag;
	NSNumber * tNumber;
	
	tTag=[[sender selectedItem] tag];
	
	tNumber=[NSNumber numberWithInteger:tTag];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_SCRIPT_COMPARATOR];
	}
}

- (IBAction) setReturnValue:(id) sender
{
	NSString * tStringValue;
	NSNumber * tNumber;
	
	tStringValue=[IBreturnValue_ stringValue];
	
	tNumber=[NSNumber numberWithInt:[tStringValue intValue]];
	
	if (tNumber!=nil)
	{
		[dictionary_ setObject:tNumber forKey:IC_REQUIREMENT_SCRIPT_VALUE];
	}
}

#pragma mark -

- (void) tableViewSelectionDidChange:(NSNotification *) inNotification
{
    if ([inNotification object]==IBarray_)
    {
		[IBremoveButton_ setEnabled:([IBarray_ numberOfSelectedRows]!=0)];
	}
}

- (void) controlTextDidChange:(NSNotification *) inNotification
{
	if ([inNotification object]==IBscriptPath_)
	{
		NSInteger tType;
		
		tType=[IBscriptPath_ pathType];
		
		if (tType==ICDOCUMENT_PATH_TYPE_RELATIVE_TO_PROJECT)
		{
			[IBembedCheckBox_ setState:NSOnState];
			
			[IBembedCheckBox_ setEnabled:NO];
		}
		else
		{
			[IBembedCheckBox_ setEnabled:YES];
		}
	}
}

- (BOOL) textField:(ICFileTextField *) inTextField shouldAcceptFileAtPath:(NSString *) inPath
{
    if (inTextField==IBscriptPath_)
    {
        NSFileManager * tFileManager=[NSFileManager defaultManager];
        BOOL isDirectory;
		
        return ([tFileManager fileExistsAtPath:inPath isDirectory:&isDirectory]==NO && isDirectory==NO);
    }
    
    return NO;
}

- (BOOL) textField:(ICFileTextField *) inTextField didAcceptFileAtPath:(NSString *) inPath
{
    
    
    [(ICReferencedFileTextField *) inTextField setAbsolutePath:inPath];
    
    if (inTextField==IBscriptPath_)
    {
        [self updateScriptPath:IBscriptPath_];
    }
    
    return YES;
}

@end
