/*
Copyright (c) 2004-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICCertificatesUtilities.h"

@implementation ICCertificatesUtilities

+ (NSArray *) availableIdentities
{
	return [ICCertificatesUtilities availableIdentitiesForKeyChainAtPath:nil];
}

+ (NSArray *) availableIdentitiesForKeyChainAtPath:(NSString *) inPath
{
	NSMutableArray * tMutableArray;
	
	tMutableArray=[NSMutableArray array];
	
	if (tMutableArray!=nil)
	{
		SecIdentitySearchRef tSecuritySearchRef;
		CFTypeRef tTypeRef=NULL;
		OSStatus tStatus;
		
		if (inPath!=nil)
		{
			SecKeychainRef tKeyChainRef;
			
			tStatus=SecKeychainOpen([inPath fileSystemRepresentation],&tKeyChainRef);
			
			if (tStatus!=noErr)
			{
				NSLog(@"SecKeychainOpen error:%%d",tStatus);
				
				switch(tStatus)
				{
					case errSecNoSuchKeychain:
					
						break;
				}
			}
			else
			{
				tTypeRef=tKeyChainRef;
			}
		}
		
		tStatus=SecIdentitySearchCreate(tTypeRef,CSSM_KEYUSE_SIGN,&tSecuritySearchRef);
		
		if (tStatus==noErr)
		{
			SecIdentityRef tIdentityRef;
			
			while (SecIdentitySearchCopyNext(tSecuritySearchRef,&tIdentityRef)!=errSecItemNotFound)
			{
				[tMutableArray addObject:(id) tIdentityRef];
				
				// Release Memory
				
				CFRelease(tIdentityRef);
			}
			
			// Release
			
			CFRelease(tSecuritySearchRef);
		}
		else
		{
			NSLog(@"SecIdentitySearchCreate error:%%d",tStatus);
		}
		
		if (tTypeRef!=NULL)
		{
			// Release
			
			CFRelease(tTypeRef);
		}
	}
	else
	{
		// A COMPLETER
	}
	
	return tMutableArray;
}

+ (SecCertificateRef) copyOfCertificateWithName:(NSString *) inName
{
	return [ICCertificatesUtilities copyOfCertificateWithName:inName atPath:nil];
}

+ (SecCertificateRef) copyOfCertificateWithName:(NSString *) inName atPath:(NSString *) inPath
{
	NSArray * tArray;
	SecCertificateRef tCertificateRef=nil;
	
	tArray=[ICCertificatesUtilities availableIdentitiesForKeyChainAtPath:inPath];
	
	if (tArray!=nil && inName!=nil)
	{
		SecIdentityRef tIdentityRef;
		NSUInteger i,tCount;
		
		tCount=[tArray count];
		
		for(i=0;i<tCount;i++)
		{
			NSString * tCertificateIdentifier=nil;
			
			tIdentityRef=(SecIdentityRef) [tArray objectAtIndex:i];
			
			SecIdentityCopyCertificate(tIdentityRef,&tCertificateRef);
			
			SecCertificateInferLabel(tCertificateRef,(CFStringRef *) &tCertificateIdentifier);	// A VERIFIER SUR 10.7

			if ([tCertificateIdentifier isEqualToString:inName]==YES)
			{
				break;
			}
			
			// Release Memory
			
			CFRelease(tCertificateRef);
			
			tCertificateRef=NULL;
		}
	}
	
	return tCertificateRef;
}

+ (SecIdentityRef) identityWithName:(NSString *) inName atPath:(NSString *) inPath
{
	NSArray * tArray;
	
	tArray=[ICCertificatesUtilities availableIdentitiesForKeyChainAtPath:inPath];
	
	if (tArray!=nil && inName!=nil)
	{
		SecIdentityRef tIdentityRef;
		NSUInteger i,tCount;
		
		tCount=[tArray count];
		
		for(i=0;i<tCount;i++)
		{
			NSString * tCertificateIdentifier=nil;
			SecCertificateRef tCertificateRef;
			
			tIdentityRef=(SecIdentityRef) [tArray objectAtIndex:i];
			
			SecIdentityCopyCertificate(tIdentityRef,&tCertificateRef);
			
			SecCertificateInferLabel(tCertificateRef,(CFStringRef *) &tCertificateIdentifier);	// A VERIFIER SUR 10.7
			
			if ([tCertificateIdentifier isEqualToString:inName]==YES)
			{
				CFRelease(tCertificateRef);
				
				return tIdentityRef;
			}
			
			// Release Memory
			
			CFRelease(tCertificateRef);
		}
	}
	
	return NULL;
}

@end
