/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICInstallationTypeUtilities.h"

#import "ICUUIDFactory.h"

#import "ICDocumentFormat+Constants.h"

@implementation ICInstallationTypeUtilities

+ (NSMutableDictionary *) defaultItemDictionaryWithPackageDictionary:(NSDictionary *) inPackageDictionary
{
	NSMutableDictionary * tItemDictionary;
	
	tItemDictionary=[NSMutableDictionary dictionary];
	
	if (tItemDictionary!=nil)
	{
		NSString * tUUID;
		NSMutableArray * tLocalizationsArray;
		
		// Type
		
		[tItemDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
		
		// Item UUID
		
		[tItemDictionary setObject:[ICUUIDFactory string] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
		
		// Package UUID
		
		tUUID=[inPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID];
		
		if (tUUID!=nil)
		{
			[tItemDictionary setObject:tUUID forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_PACKAGE_UUID];
		}
		else
		{
			NSLog(@"[ICInstallationTypeUtilities defaultItemDictionaryWithPackageDictionary:] Missing Information : ICDOCUMENT_PACKAGE_UUID");
		}
		
		// TITLE
			
		tLocalizationsArray=[NSMutableArray array];

		if (tLocalizationsArray!=nil)
		{
			[tItemDictionary setObject:tLocalizationsArray forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TITLE];
		}
		else
		{
			NSLog(@"[ICInstallationTypeUtilities defaultItemDictionaryWithPackageDictionary:] Not enough memory");
		}
		
		// DESCRIPTION
		
		tLocalizationsArray=[NSMutableArray array];

		if (tLocalizationsArray!=nil)
		{
			[tItemDictionary setObject:tLocalizationsArray forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_DESCRIPTION];
		
		}
		else
		{
			NSLog(@"[ICInstallationTypeUtilities defaultItemDictionaryWithPackageDictionary:] Not enough memory");
		}
		
		// TOOLTIP
		
		[tItemDictionary setObject:[NSMutableArray array] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TOOLTIP];
		
		// OPTIONS
		
		[tItemDictionary setObject:[NSMutableDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE_SELECTED],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_STATE,
																					 [NSNumber numberWithBool:NO],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS_HIDDEN,
																					 nil]
		
							forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_OPTIONS];
	}
	
	return tItemDictionary;
}

+ (NSMutableDictionary *) defaultHierarchyDictionaryWithPackages:(NSArray *) inPackagesArray
{
	NSMutableDictionary * tHierarchyDictionary=nil;
	NSMutableArray * tItemsList;
		
	tItemsList=[NSMutableArray array];
	
	if (tItemsList!=nil)
	{
		if (inPackagesArray!=nil)
		{
			NSEnumerator * tEnumerator;
			
			tEnumerator=[inPackagesArray objectEnumerator];
			
			if (tEnumerator!=nil)
			{
				NSDictionary * tPackageDictionary;
				
				while (tPackageDictionary=[tEnumerator nextObject])
				{
					NSMutableDictionary * tItemDictionary;
					
					tItemDictionary=[ICInstallationTypeUtilities defaultItemDictionaryWithPackageDictionary:tPackageDictionary];
					
					if (tItemDictionary!=nil)
					{
						[tItemsList addObject:tItemDictionary];
					}
				}
			}
		}
			
		tHierarchyDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:tItemsList,ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST,
																			   [NSMutableDictionary dictionary],ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_REMOVED,
																			   nil];
	}
	else
	{
		NSLog(@"[ICInstallationTypeUtilities defaultHierarchyDictionaryWithPackages:] Not enough memory");
	}
	
	return tHierarchyDictionary;
}

+ (NSMutableDictionary *) defaultInstallationTypeDictionaryWithPackages:(NSArray *) inPackagesArray
{
	NSMutableDictionary * tMutableDictionary=nil;
	
	tMutableDictionary=[NSMutableDictionary dictionary];
	
	if (tMutableDictionary!=nil)
	{
		NSMutableDictionary * tHierarchyDictionary;
		
		tHierarchyDictionary=[ICInstallationTypeUtilities defaultHierarchyDictionaryWithPackages:inPackagesArray];
		
		if (tHierarchyDictionary!=nil)
		{
			NSMutableDictionary * tHierarchiesDictionary;
			
			tHierarchiesDictionary=[NSMutableDictionary dictionary];
			
			if (tHierarchiesDictionary!=nil)
			{
				[tHierarchiesDictionary setObject:tHierarchyDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHY_INSTALLER];
				
				[tMutableDictionary setObject:tHierarchiesDictionary forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
					
				[tMutableDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_MODE_ALLOW] forKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE];
			}
			else
			{
				NSLog(@"[ICInstallationTypeUtilities defaultInstallationTypeDictionaryWithPackages:] Not enough memory");
		
				tMutableDictionary=nil;
			}
		}
	}
	else
	{
		NSLog(@"[ICInstallationTypeUtilities defaultInstallationTypeDictionaryWithPackages:] Not enough memory");
	
		tMutableDictionary=nil;
	}
	
	return tMutableDictionary;
}

+ (NSMutableArray *) allUUIDSForPackages:(NSArray *) inPackagesArray
{
	NSMutableArray * tMutableArray=nil;
	
	if (inPackagesArray!=nil)
	{
		NSEnumerator * tEnumerator;
	
		tMutableArray=[NSMutableArray array];
	
		tEnumerator=[inPackagesArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tPackageDictionary;
					
			while (tPackageDictionary=[tEnumerator nextObject])
			{
				NSMutableDictionary * tItemDictionary;
				
				tItemDictionary=[NSMutableDictionary dictionary];
				
				if (tItemDictionary!=nil)
				{
					NSString * tUUID;
					
					// Package UUID
					
					tUUID=[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID];
					
					if (tUUID!=nil)
					{
						[tMutableArray addObject:tUUID];
					}
				}
			}
		}
	}
	
	return tMutableArray;
}

+ (NSMutableArray *) allChoicesUUIDForChoices:(NSArray *) inChoicesArray
{
	NSMutableArray * tMutableArray=nil;
	
	if (inChoicesArray!=nil)
	{
		NSEnumerator * tEnumerator;
	
		tMutableArray=[NSMutableArray array];
	
		tEnumerator=[inChoicesArray objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tChoiceDictionary;
					
			while (tChoiceDictionary=[tEnumerator nextObject])
			{
				NSString * tChoiceUUID;
				
				tChoiceUUID=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_UUID];
				
				if (tChoiceUUID!=nil)
				{
					[tMutableArray addObject:tChoiceUUID];
				}
			}
		}
	}
	
	return tMutableArray;
}

@end
