/*
Copyright (c) 2004-2009, Stphane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICLicenseProvider.h"
#import "NSString+Karelia.h"

#import "ICDocumentFormat+Constants.h"

@implementation ICLicenseProvider

+ (ICLicenseProvider *) defaultProvider
{
    static ICLicenseProvider * sLicenseProvider=nil;
    
    if (sLicenseProvider==nil)
    {
        sLicenseProvider=[[ICLicenseProvider alloc] init];
    }
    
    return sLicenseProvider;
}

- (id) init
{
    self=[super init];
    
    if (self!=nil)
    {
        NSFileManager * tFileManager;
        NSString * tFolderPath;
        NSArray * tTemplateTypes;
        NSString * tSubPath=nil;
        NSUInteger i,tTypeCount;
        NSArray * tLibraryArray;
        NSEnumerator * tEnumerator;
        NSString * tLibraryPath;
        
        // Find the Template licenses folder
        
		tFileManager=[NSFileManager defaultManager];
		
        tLibraryArray=NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,NSLocalDomainMask,NO);
        
        tEnumerator=[tLibraryArray objectEnumerator];
        
        while (tLibraryPath=[tEnumerator nextObject])
        {
            tFolderPath=[tLibraryPath stringByAppendingPathComponent:@"Application Support/fr.whitebox.packages/Licenses Templates"];
            
            tTemplateTypes=[tFileManager directoryContentsAtPath:tFolderPath];
            
            tTypeCount=[tTemplateTypes count];
            
            licensesDictionary_=[[NSMutableDictionary alloc] initWithCapacity:tTypeCount];
            
            for(i=0;i<tTypeCount;i++)
            {
                NSString * tTypePath;
                
                BOOL isDirectory;
                
                tSubPath=[tTemplateTypes objectAtIndex:i];
                
                tTypePath=[tFolderPath stringByAppendingPathComponent:tSubPath];
                
                if ([tFileManager fileExistsAtPath:tTypePath isDirectory:&isDirectory]==YES && isDirectory==YES)
                {
                    NSArray * tLanguages;
                    NSUInteger j,tLanguagesCount;
                    NSString * tLanguageName;
                    NSMutableArray * tLicenseAvailableLanguages;
                    NSMutableDictionary * tLicenseObject;
                    NSArray * tKeywordsArray;
					NSDictionary * tSLADictionary;
					NSString * tSLAString=nil;
					
                    tLanguages=[tFileManager directoryContentsAtPath:tTypePath];
                    
                    tLanguagesCount=[tLanguages count];
                    
                    tLicenseAvailableLanguages=[NSMutableArray arrayWithCapacity:tLanguagesCount];
                    
					tSLADictionary=[NSDictionary dictionaryWithContentsOfFile:[tTypePath stringByAppendingPathComponent:@"sla.plist"]];
					
					if (tSLADictionary!=nil)
					{
						tSLAString=[tSLADictionary objectForKey:@"sla"];
					}
					
					tKeywordsArray=[NSArray arrayWithContentsOfFile:[tTypePath stringByAppendingPathComponent:@"Keywords.plist"]];
					
                    for(j=0;j<tLanguagesCount;j++)
                    {
                        NSString * tLanguagePath;
                        
                        tLanguageName=[tLanguages objectAtIndex:j];
                        
                        tLanguagePath=[tTypePath stringByAppendingPathComponent:tLanguageName];
                        
                        if ([tFileManager fileExistsAtPath:tLanguagePath isDirectory:&isDirectory]==YES && isDirectory==YES)
                        {
                            NSString * tLicensePath;
                            
                            // Check the coherence of the Folder before adding the file
                            
							tLicensePath=[tLanguagePath stringByAppendingPathComponent:@"License.rtf"];
							
                            if ([tFileManager fileExistsAtPath:tLicensePath isDirectory:&isDirectory]==NO || isDirectory==YES)
                            {
                                continue;
                            }
                            
							[tLicenseAvailableLanguages addObject:[NSDictionary dictionaryWithObjectsAndKeys:[NSDictionary dictionaryWithObjectsAndKeys:tLicensePath,ICDOCUMENT_PATH,
																																						[NSNumber numberWithInteger:ICDOCUMENT_PATH_TYPE_GLOBAL],ICDOCUMENT_PATH_TYPE,
																																						nil],ICDOCUMENT_VALUE,
																											 [tLanguageName stringByDeletingPathExtension],ICDOCUMENT_LANGUAGE,
																											 nil]];
						}
                    }
                    
                    tLicenseObject=[NSMutableDictionary dictionaryWithObjectsAndKeys:tLicenseAvailableLanguages,ICDOCUMENT_LOCALIZATIONS,
                                                                              tKeywordsArray,@"Keywords",
                                                                              nil];
                                                                            
                    
					if (tSLAString!=nil)
					{
						[tLicenseObject setObject:tSLAString forKey:@"sla"];
					}
					
                    [licensesDictionary_ setObject:tLicenseObject forKey:tSubPath];
                }
            }
        }
    }
    
    return self;
}

- (void) dealloc
{
    [licensesDictionary_ release];
    
    [super dealloc];
}

- (NSArray *) allLicenses
{
    return [licensesDictionary_ allKeys];
}

- (NSString *) slaForLicenseWithName:(NSString *) inName
{
    if (licensesDictionary_!=nil && inName!=nil)
	{
		NSDictionary * tLicenseDictionary;
		
		tLicenseDictionary=[licensesDictionary_ objectForKey:inName];
		
		if (tLicenseDictionary!=nil)
		{
			return [tLicenseDictionary objectForKey:@"sla"];
		}
	}
    
    return nil;
}

- (NSArray *) licenseKeywordsForLicenseWithName:(NSString *) inName
{
    if (licensesDictionary_!=nil && inName!=nil)
	{
		NSDictionary * tLicenseDictionary;
		
		tLicenseDictionary=[licensesDictionary_ objectForKey:inName];
		
		if (tLicenseDictionary!=nil)
		{
			return [tLicenseDictionary objectForKey:@"Keywords"];
		}
	}
    
    return nil;
}

- (NSArray *) localizationsForLicenseWithName:(NSString *) inName
{
	if (licensesDictionary_!=nil && inName!=nil)
	{
		NSDictionary * tLicenseDictionary;
		
		tLicenseDictionary=[licensesDictionary_ objectForKey:inName];
		
		if (tLicenseDictionary!=nil)
		{
			return [tLicenseDictionary objectForKey:ICDOCUMENT_LOCALIZATIONS];
		}
	}
    
    return nil;
}

+ (void) replaceKeywords:(NSDictionary *) inDictionary inAttributedString:(NSMutableAttributedString *) inMutableAttributedString
{
    NSString * tString;
    BOOL keepOn=YES;
    NSRange tFoundRange={0,0};
	
    tString=[inMutableAttributedString string];
    
    do
    {
        tFoundRange = [tString rangeFromString:@"%%" toString:@"%%" options:0 range:NSMakeRange(NSMaxRange(tFoundRange),[tString length]-NSMaxRange(tFoundRange))];
        
        if (tFoundRange.location == NSNotFound)
        {
            keepOn=NO;
        }
        else
        {
            NSString * tKey;
            NSString * tValue;
            
            tKey=[tString substringWithRange:NSMakeRange(tFoundRange.location+2,tFoundRange.length-4)];
            
            tValue=[inDictionary objectForKey:tKey];
            
            if (tValue!=nil)
            {
                [inMutableAttributedString replaceCharactersInRange:tFoundRange
                                                         withString:tValue];
            
                tString=[inMutableAttributedString string];
				
				tFoundRange.length=[tValue length];
            }
        }
        
        
    }
    while (YES==keepOn);
}

+ (void) replaceKeywords:(NSDictionary *) inDictionary inString:(NSMutableString *) inMutableString
{
    BOOL keepOn=YES;
    
    do
    {
        NSRange foundRange = [inMutableString rangeFromString:@"%%" toString:@"%%" options:0 range:NSMakeRange(0,[inMutableString length])];
        
        if (foundRange.location == NSNotFound)
        {
            keepOn=NO;
        }
        else
        {
            NSString * tKey;
            NSString * tValue;
            
            tKey=[inMutableString substringWithRange:NSMakeRange(foundRange.location+2,foundRange.length-4)];
            
            tValue=[inDictionary objectForKey:tKey];
            
            if (tValue!=nil)
            {
                [inMutableString replaceCharactersInRange:foundRange
                                                         withString:tValue];
            }
            else
            {
                break;
            }
        }
        
        
    }
    while (YES==keepOn);
}

@end
