/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPackageUtilities.h"

#import "ICDocumentFormat+Constants.h"

#import "ICUUIDFactory.h"

#include <xar/xar.h>

int32_t ixar_extract_tobuffersz(xar_t x, xar_file_t f, char **buffer, size_t *size)
{
	const char *sizestring = NULL;
	
	if(0 != xar_prop_get(f,"data/size",&sizestring))
	{
		return -1;
	}

	*size = strtoull(sizestring, (char **)NULL, 10);
	
	*buffer = malloc(*size);
	
	if(!(*buffer))
	{
		return -1;
	}
	
	return xar_arcmod_extract(x,f,NULL,*buffer,*size);
}

@implementation ICPackageUtilities

+ (BOOL) isUniquePackageName:(NSString *) inName forPackage:(NSDictionary *) inDictionary withExistingPackages:(NSArray *) inPackages
{
	if (inName!=nil && inPackages!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inPackages objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tDictionary;
			
			while (tDictionary=[tEnumerator nextObject])
			{
				if (tDictionary!=inDictionary)
				{
					if ([[[tDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME] caseInsensitiveCompare:inName]==NSOrderedSame)
					{
						return NO;
					}
				}
			}
		}
	}
	
	return YES;
}

+ (BOOL) isUniquePackageUUID:(NSString *) inUUID forPackage:(NSDictionary *) inDictionary withExistingPackages:(NSArray *) inPackages
{
	if (inUUID!=nil && inPackages!=nil)
	{
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inPackages objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tDictionary;
			
			while (tDictionary=[tEnumerator nextObject])
			{
				if (tDictionary!=inDictionary)
				{
					if ([[tDictionary objectForKey:ICDOCUMENT_PACKAGE_UUID] caseInsensitiveCompare:inUUID]==NSOrderedSame)
					{
						return NO;
					}
				}
			}
		}
	}
	
	return YES;
}

+ (NSString *) uniquePackageIdentifierWithExistingPackages:(NSArray *) inPackages originalIdentifier:(NSString *) inIdentifier
{
	NSString * tString=nil;
	
	if (inIdentifier!=nil)
	{
		NSUInteger _sIndex=0;
		NSUInteger i,tCount;
    
		tCount=[inPackages count];
    
		do
		{
			NSDictionary * tPackageDictionary;
			
			[tString release];
			
			if (_sIndex>0)
			{
				tString=[[NSString alloc] initWithFormat:@"%@_%lu",inIdentifier,_sIndex];
			}
			else
			{
				tString=[[NSString alloc] initWithString:inIdentifier];
			}
			
			_sIndex++;
			
			for(i=0;i<tCount;i++)
			{
				tPackageDictionary=[inPackages objectAtIndex:i];
				
				if ([[[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER] caseInsensitiveCompare:tString]==NSOrderedSame)
				{
					break;
				}
			}
			
			if (i==tCount)
			{
				break;
			}
		}
		while (_sIndex<65535);
    }
	
    return [tString autorelease];
}

+ (NSString *) uniquePackageNameWithExistingPackages:(NSArray *) inPackages baseName:(NSString *) inBaseName
{
	NSString * tString;
    
	tString=nil;
	
	if (inBaseName!=nil)
	{
		NSUInteger _sIndex=0;
		NSUInteger i,tCount;
    
		tCount=[inPackages count];
    
		do
		{
			NSDictionary * tPackageDictionary;
			
			[tString release];
			
			if (_sIndex>0)
			{
				tString=[[NSString alloc] initWithFormat:@"%@ %lu",inBaseName,_sIndex];
			}
			else
			{
				tString=[[NSString alloc] initWithString:inBaseName];
			}
			
			_sIndex++;
			
			for(i=0;i<tCount;i++)
			{
				tPackageDictionary=[inPackages objectAtIndex:i];
				
				if ([[[tPackageDictionary objectForKey:ICDOCUMENT_PACKAGE_SETTINGS] objectForKey:ICDOCUMENT_PACKAGE_SETTINGS_NAME] caseInsensitiveCompare:tString]==NSOrderedSame)
				{
					break;
				}
			}
			
			if (i==tCount)
			{
				break;
			}
		}
		while (_sIndex<65535);
    }
	
    return [tString autorelease];
}

+ (NSString *) uniquePackageNameWithExistingPackages:(NSArray *) inPackages
{
	static NSString * sLocalizedBaseName=nil;
    
	if (sLocalizedBaseName==nil)
    {
        sLocalizedBaseName=[[NSString alloc] initWithString:NSLocalizedString(@"untitled package",@"No comment")];
    }
    
    return [ICPackageUtilities uniquePackageNameWithExistingPackages:inPackages baseName:sLocalizedBaseName];
}



+ (NSMutableDictionary *) importedPackageDictionaryWithContentsOfFile:(NSString *) inPath
{
	NSMutableDictionary * tMutablePackageDictionary=nil;
	
	if (inPath!=nil)
	{
		tMutablePackageDictionary=[NSMutableDictionary dictionary];
	
		if (tMutablePackageDictionary!=nil)
		{
			NSMutableDictionary * tMutableSettingsDictionary;
			
			// PACKAGE TYPE
			
			[tMutablePackageDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGE_TYPE_IMPORTED] forKey:ICDOCUMENT_PACKAGE_TYPE];
			
			// UUID
			
			[tMutablePackageDictionary setObject:[ICUUIDFactory string] forKey:ICDOCUMENT_PACKAGE_UUID];
			
			// SETTINGS
			
			tMutableSettingsDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[[inPath lastPathComponent] stringByDeletingPathExtension],ICDOCUMENT_PACKAGE_SETTINGS_NAME,
																						 [NSNumber numberWithInt:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED],ICDOCUMENT_PACKAGE_SETTINGS_LOCATION,
																						 nil];
																						 
			if (tMutableSettingsDictionary!=nil)
			{
				// A COMPLETER
				
				[tMutablePackageDictionary setObject:tMutableSettingsDictionary forKey:ICDOCUMENT_PACKAGE_SETTINGS];
			}
			else
			{
				NSLog(@"[ICPackageUtilities importedPackageDictionaryWithContentsOfFile:existingPackages:] Unable to create new package Settings dictionary");
				
				tMutablePackageDictionary=nil;
			}
			
			// Default Scripts
			
			// A COMPLETER
		}
		else
		{
			NSLog(@"[ICPackageUtilities importedPackageDictionaryWithContentsOfFile:existingPackages:] Unable to create new package dictionary");
		}
	}
	
	return tMutablePackageDictionary;
}

+ (BOOL) isFlatPackageAtPath:(NSString *) inPath
{
	if (inPath!=nil)
	{
		const char * tCFilePath;
		
		tCFilePath=[[NSFileManager defaultManager] fileSystemRepresentationWithPath:inPath];
		
		if (tCFilePath!=nil)
		{
			// Check the Magic Cookie
		
			int fd;
		
			fd=open(tCFilePath,O_RDONLY);
			
			if (fd!=0)
			{
				size_t tReadSize;
				char tMagicCookie[4];
				
				tReadSize=read(fd,tMagicCookie,4*sizeof(char));
				
				close(fd);
				
				if (tReadSize==4)
				{
					if (tMagicCookie[0]=='x' &&
						tMagicCookie[1]=='a' &&
						tMagicCookie[2]=='r' &&
						tMagicCookie[3]=='!')
					{
						// Check that it's a pure package xar archive
						
						xar_t tArchive;

						BOOL isFlatPackage=NO;
						
						tArchive=xar_open(tCFilePath,READ);
					
						if (tArchive!=NULL)
						{
							xar_iter_t tIterator;
							
							tIterator=xar_iter_new();
							
							if (tIterator!=NULL)
							{
								xar_file_t tFile;
								
								for(tFile=xar_file_first(tArchive,tIterator);tFile!=NULL;tFile=xar_file_next(tIterator))
								{
									const char * tValue=NULL;
										
									xar_prop_get(tFile,"name",&tValue);
										
									if (!strcmp(tValue,"Distribution"))
									{
										// It's a Distribution Script, not a FLAT package
										
										isFlatPackage=NO;
										
										break;
									}
									else if (!strcmp(tValue,"PackageInfo"))
									{
										/*char * tBuffer;
										size_t tSize;*/
										
										// Check it's a file
										
										xar_prop_get(tFile,"type",&tValue);
										
										if (!strcmp(tValue,"file"))
										{
											isFlatPackage=YES;
										}
									}
								}
								
								xar_iter_free(tIterator);
							}
			
							xar_close(tArchive);
						}
						
						return isFlatPackage;
					}
				}
			}
		}
	}

	return NO;
}

+ (NSMutableDictionary *) packageSettingsDictionaryWithContentsOfFile:(NSString *) inPath
{
	NSMutableDictionary * tPackageSettingsDictionary=nil;
	
	if (inPath!=nil)
	{
		xar_t tArchive;

		tArchive=xar_open([[NSFileManager defaultManager] fileSystemRepresentationWithPath:inPath],READ);
	
		if (tArchive!=NULL)
		{
			xar_iter_t tIterator;
			
			tIterator=xar_iter_new();
			
			if (tIterator!=NULL)
			{
				xar_file_t tFile;
				
				for(tFile=xar_file_first(tArchive,tIterator);tFile!=NULL;tFile=xar_file_next(tIterator))
				{
					const char * tValue=NULL;
						
					xar_prop_get(tFile,"name",&tValue);
						
					if (!strcmp(tValue,"Distribution"))
					{
						// It's a Distribution Script, not a FLAT package
						
						tPackageSettingsDictionary=nil;
						
						break;
					}
					else if (!strcmp(tValue,"PackageInfo"))
					{
						char * tBuffer;
						size_t tSize;
						
						// Check it's a file
						
						xar_prop_get(tFile,"type",&tValue);
						
						if (!strcmp(tValue,"file"))
						{
							if (ixar_extract_tobuffersz(tArchive,tFile,&tBuffer,&tSize)==0)
							{
								NSData * tData;
								
								tData=[NSData dataWithBytes:tBuffer length:tSize];
								
								if (tData!=nil)
								{
									NSXMLDocument * tXMLDocument;
									NSError * tError;
									
									tXMLDocument=[[NSXMLDocument alloc] initWithData:tData options:0 error:&tError];
									
									if (tXMLDocument!=nil)
									{
										NSArray * tNodes;
										
										tNodes=[tXMLDocument nodesForXPath:@"pkg-info" error:&tError];
										
										if ([tNodes count]>0)
										{
											NSXMLElement * tElement;
											
											tElement=(NSXMLElement *) [tNodes objectAtIndex:0];
										
											if (tElement!=nil)
											{
												NSArray * tAttributes;
												
												tAttributes=[tElement attributes];
												
												if (tAttributes!=nil)
												{
													NSEnumerator * tEnumerator;
													
													tEnumerator=[tAttributes objectEnumerator];
													
													tPackageSettingsDictionary=[NSMutableDictionary dictionaryWithObjectsAndKeys:[[inPath lastPathComponent] stringByDeletingPathExtension],ICDOCUMENT_PACKAGE_SETTINGS_NAME,
																																 @"",ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER,
																																 @"",ICDOCUMENT_PACKAGE_SETTINGS_VERSION,
																																 [NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_AUTHENTICATION_NONE],ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION,
																																 [NSNumber numberWithBool:NO],ICDOCUMENT_PACKAGE_SETTINGS_RELOCATABLE,
																																 [NSNumber numberWithBool:NO],ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS,
																																 [NSNumber numberWithBool:NO],ICDOCUMENT_PACKAGE_SETTINGS_FOLLOW_SYMBOLIC_LINKS,
																																 [NSNumber numberWithBool:NO],ICDOCUMENT_PACKAGE_SETTINGS_USE_HFSPLUS_COMPRESSION,
																																 [NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_NONE],ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION,
																																 [NSNumber numberWithInt:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_EMBEDDED],ICDOCUMENT_PACKAGE_SETTINGS_LOCATION,
																																 [NSNumber numberWithInt:0],ICDOCUMENT_PACKAGE_SETTINGS_PAYLOAD_SIZE,
																																 nil];
													
													if (tEnumerator!=nil)
													{
														NSXMLNode * tAttributeNode;
														
														while (tAttributeNode=[tEnumerator nextObject])
														{
															NSString * tAttributeName;
															NSString * tStringValue;
															
															tAttributeName=[tAttributeNode name];
															
															tStringValue=[tAttributeNode stringValue];
															
															if ([tAttributeName isEqualToString:@"identifier"]==YES)
															{
																[tPackageSettingsDictionary setObject:tStringValue forKey:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER];
															}
															else if ([tAttributeName isEqualToString:@"relocatable"]==YES)
															{
																tStringValue=[tStringValue lowercaseString];
																
																[tPackageSettingsDictionary setObject:[NSNumber numberWithBool:[tStringValue isEqualToString:@"yes"] || [tStringValue isEqualToString:@"true"]] forKey:ICDOCUMENT_PACKAGE_SETTINGS_RELOCATABLE];
															}
															else if ([tAttributeName isEqualToString:@"overwrite-permissions"]==YES)
															{
																tStringValue=[tStringValue lowercaseString];
																
																[tPackageSettingsDictionary setObject:[NSNumber numberWithBool:[tStringValue isEqualToString:@"yes"] || [tStringValue isEqualToString:@"true"]] forKey:ICDOCUMENT_PACKAGE_SETTINGS_OVERWRITE_PERMISSIONS];
															}
															else if ([tAttributeName isEqualToString:@"followSymLinks"]==YES)
															{
																tStringValue=[tStringValue lowercaseString];
																
																[tPackageSettingsDictionary setObject:[NSNumber numberWithBool:[tStringValue isEqualToString:@"yes"] || [tStringValue isEqualToString:@"true"]] forKey:ICDOCUMENT_PACKAGE_SETTINGS_FOLLOW_SYMBOLIC_LINKS];
															}
															else if ([tAttributeName isEqualToString:@"useHFSPlusCompression"]==YES)
															{
																tStringValue=[tStringValue lowercaseString];
																
																[tPackageSettingsDictionary setObject:[NSNumber numberWithBool:[tStringValue isEqualToString:@"yes"] || [tStringValue isEqualToString:@"true"]] forKey:ICDOCUMENT_PACKAGE_SETTINGS_USE_HFSPLUS_COMPRESSION];
															}
															else if ([tAttributeName isEqualToString:@"auth"]==YES)
															{
																tStringValue=[tStringValue lowercaseString];
																
																if ([tStringValue isEqualToString:@"root"]==YES)
																{
																	[tPackageSettingsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_AUTHENTICATION_ROOT] forKey:ICDOCUMENT_PACKAGE_SETTINGS_AUTHENTICATION];
																}
															}
															else if ([tAttributeName isEqualToString:@"postinstall-action"]==YES)
															{
																tStringValue=[tStringValue lowercaseString];
																
																if ([tStringValue isEqualToString:@"restart"]==YES)
																{
																	[tPackageSettingsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_REQUIRE_RESTART] forKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
																}
																else if ([tStringValue isEqualToString:@"shutdown"]==YES)
																{
																	[tPackageSettingsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_REQUIRE_SHUTDOWN] forKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
																}
																else if ([tStringValue isEqualToString:@"logout"]==YES)
																{
																	[tPackageSettingsDictionary setObject:[NSNumber numberWithInt:ICDOCUMENT_PACKAGES_SETTINGS_CONCLUSION_ACTION_REQUIRE_LOGOUT] forKey:ICDOCUMENT_PACKAGE_SETTINGS_CONCLUSION_ACTION];
																}
															}
															else if ([tAttributeName isEqualToString:@"version"]==YES)
															{
																[tPackageSettingsDictionary setObject:tStringValue forKey:ICDOCUMENT_PACKAGE_SETTINGS_VERSION];
															}
														}
													}
													
													tNodes=[tXMLDocument nodesForXPath:@"pkg-info/payload" error:&tError];
										
													if ([tNodes count]>0)
													{
														tElement=(NSXMLElement *) [tNodes objectAtIndex:0];
													
														if (tElement!=nil)
														{
															tAttributes=[tElement attributes];
															
															if (tAttributes!=nil)
															{
																tEnumerator=[tAttributes objectEnumerator];
																
																if (tEnumerator!=nil)
																{
																	NSXMLNode * tAttributeNode;
																	
																	while (tAttributeNode=[tEnumerator nextObject])
																	{
																		NSString * tAttributeName;
																		NSString * tStringValue;
																		
																		tAttributeName=[tAttributeNode name];
																		
																		tStringValue=[tAttributeNode stringValue];
																		
																		if ([tAttributeName isEqualToString:@"installKBytes"]==YES)
																		{
#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5
																			
																			[tPackageSettingsDictionary setObject:[NSNumber numberWithInt:[tStringValue intValue]] forKey:ICDOCUMENT_PACKAGE_SETTINGS_PAYLOAD_SIZE];
																			
#else
																			
																			[tPackageSettingsDictionary setObject:[NSNumber numberWithInteger:[tStringValue integerValue]] forKey:ICDOCUMENT_PACKAGE_SETTINGS_PAYLOAD_SIZE];
																			
#endif
																			
																			
																			
																			break;
																		}
																	}
																}
															}
														}
													}
												}
											}
										}
										else
										{
											// A COMPLETER
										}
										
										[tXMLDocument release];
									}
									else
									{
										// A COMPLETER
									}
								}
								
								free(tBuffer);
							}
							/*else
							{
								// Error during extraction
								
								// A COMPLETER
							}*/
						}
						else
						{
							// Broken format
							
							// A COMPLETER
							
							tPackageSettingsDictionary=nil;
						
							break;
						}
					}
				}
			
				xar_iter_free(tIterator);
			}
			
			xar_close(tArchive);
		}
		else
		{
			// Not a xar archive probably
			
			// A COMPLETER
		}
	}
	
	return tPackageSettingsDictionary;
}

@end
