/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICRequirementPlugin_Converter.h"

#import "ICDocumentFormat+SharedGlobalData.h"

@implementation ICRequirementPlugin_Converter

- (id) init
{
	self=[super init];
	
	if (self!=nil)
	{
		bundle_=[NSBundle bundleForClass:[self class]];
		
		if (bundle_!=nil)
		{
			[bundle_ retain];
		}
		else
		{
			NSLog(@"[ICRequirementPlugin_Converter init] Unable to locate the bundle");
		}
	}
	
	return self;
}

- (void) dealloc
{
	[bundle_ release];
	
	[super dealloc];
}

#pragma mark -

- (void) setDocumentDictionary:(NSDictionary *) inDictionary
{
	documentDictionary_=inDictionary;
}

- (id) rootObjectForKey:(NSString *) inKey
{
	if (documentDictionary_!=nil && inKey!=nil)
	{
		NSMutableDictionary * tGlobalDataDictionary;
		
		tGlobalDataDictionary=[documentDictionary_ objectForKey:ICDOCUMENT_SHARED_GLOBAL_DATA_DEFINE];
		
		if (tGlobalDataDictionary!=nil)
		{
			return [tGlobalDataDictionary objectForKey:inKey];
		}
	}
	
	return nil;
}

- (NSDictionary *) requiredOptionsValuesWithParameters:(NSDictionary *) inParameters
{
	return nil;
}

- (NSArray *) requiredAdditionalResourcesWithParameters:(NSDictionary *) inParameters
{
	return nil;
}

- (int) requirementTypeWithParameters:(NSDictionary *) inParameters
{
	return IC_REQUIREMENT_CHECK_TYPE_UNDEFINED;
}

- (NSArray *) sharedConstantsNames
{
	NSArray * tArray=nil;
	
	if (bundle_!=nil)
	{
		NSString * tPath;
		
		tPath=[bundle_ pathForResource:@"Constants" ofType:@"plist"];
		
		if (tPath!=nil)
		{
			NSDictionary * tDictionary;
			
			tDictionary=[NSDictionary dictionaryWithContentsOfFile:tPath];
			
			if (tDictionary!=nil)
			{
				tArray=[tDictionary objectForKey:@"Names"];
			}
		}
	}
	
	return tArray;
}

- (NSString *) constantsForNames:(NSArray *) inNames
{
	NSMutableString * tString=nil;
	
	if (bundle_!=nil)
	{
		NSString * tPath;
		
		tPath=[bundle_ pathForResource:@"Constants" ofType:@"plist"];
		
		if (tPath!=nil)
		{
			NSDictionary * tDictionary;
			
			tDictionary=[NSDictionary dictionaryWithContentsOfFile:tPath];
			
			if (tDictionary!=nil)
			{
				NSDictionary * tDefinitionsDictionary;
				
				tDefinitionsDictionary=[tDictionary objectForKey:@"Definitions"];
				
				if (tDefinitionsDictionary!=nil)
				{
					tString=[NSMutableString string];
					
					if (tString!=nil || inNames==nil || [inNames count]==0)
					{
						NSEnumerator * tEnumerator;
						
						tEnumerator=[inNames objectEnumerator];
						
						if (tEnumerator!=nil)
						{
							NSString * tConstantName;
							
							while (tConstantName=[tEnumerator nextObject])
							{
								NSString * tValue;
								
								tValue=[tDefinitionsDictionary objectForKey:tConstantName];
								
								if (tValue!=nil)
								{
									NSString * tDefinitionString;
									
									tDefinitionString=[NSString stringWithFormat:@"\tconst %@=%@;\n",tConstantName,tValue];
									
									if (tDefinitionString!=nil)
									{
										[tString appendString:tDefinitionString];
									}
								}
							}
						}
						else
						{
							tString=nil;
						}
					}
				}
			}
		}
	}
	
	return tString;
}

- (NSString *) variablesWithIndex:(int) inIndex tabulationDepth:(NSString *) inTabulationDepth parameters:(NSDictionary *) inParameters error:(NSDictionary **) outErrorDictionary
{
	return @"";
}

- (NSDictionary *) sharedConstants
{
	NSDictionary * tDictionary=nil;
	
	if (bundle_!=nil)
	{
		NSString * tPath;
		
		tPath=[bundle_ pathForResource:@"Constants" ofType:@"plist"];
		
		if (tPath!=nil)
		{
			tDictionary=[NSDictionary dictionaryWithContentsOfFile:tPath];
		}
	}
	
	return tDictionary;
}

- (NSDictionary *) sharedFunctionsImplementation
{
	NSDictionary * tDictionary=nil;
	
	if (bundle_!=nil)
	{
		NSString * tPath;
		
		tPath=[bundle_ pathForResource:@"Functions" ofType:@"plist"];
		
		if (tPath!=nil)
		{
			tDictionary=[NSDictionary dictionaryWithContentsOfFile:tPath];
		}
	}
	
	return tDictionary;
}

- (NSString *) invocationWithFormat:(NSString *) inFormat parameters:(NSDictionary *) inParameters index:(int) inIndex error:(NSDictionary **) outErrorDictionary
{
	// To implement in subclass
	
	return nil;
}

- (NSString *) invocationWithParameters:(NSDictionary *) inParameters index:(int) inIndex error:(NSDictionary **) outErrorDictionary
{
	NSString * tString=nil;
	
	if (bundle_!=nil)
	{
		tString=NSLocalizedStringFromTableInBundle(@"InvocationFormat",@"Code",bundle_,@"");
		
		if (tString!=nil && inParameters!=nil)
		{
			return [self invocationWithFormat:tString parameters:inParameters index:inIndex error:outErrorDictionary];
		}
	}
	
	return tString;
}

@end
