/*
Copyright (c) 2007-2009, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "WBOutlineView.h"

@implementation NSObject (WBOutlineViewDelegate)

- (void) outlineView:(NSOutlineView *) inOutlineView deleteSelectedRowsWithConfirmationRequired:(BOOL) inConfirmationRequired
{
}

- (BOOL) outlineView:(NSOutlineView *) inOutlineView validateAction:(SEL) inSelector
{
	return NO;
}

@end

@implementation WBOutlineView

- (BOOL)validateMenuItem:(NSMenuItem *)aMenuItem
{
    if ([aMenuItem action]==@selector(delete:))
    {
        BOOL tShouldDelete=NO;
		
		if ([self delegate]!=nil)
		{
			tShouldDelete=[[self delegate] outlineView:self validateAction:@selector(delete:)];
		}
        
        return tShouldDelete;
    }
    
    return YES;
}

- (void) keyDown:(NSEvent *) theEvent
{
    NSString * tString;
    NSUInteger stringLength;
    NSUInteger i;
    unichar tChar;
    
    tString= [theEvent characters];
    
	stringLength=[tString length];
    
    for(i=0;i<stringLength;i++)
    {
        tChar=[tString characterAtIndex:i];
        
        if (tChar==0x7F)
        {
            BOOL tShouldDelete=YES;
			
			if ([self delegate]!=nil)
			{
				tShouldDelete=[[self delegate] outlineView:self validateAction:@selector(delete:)];
			}
			
			if (tShouldDelete==YES)
            {
				[[self delegate] outlineView:self deleteSelectedRowsWithConfirmationRequired:(([theEvent modifierFlags]&NSAlternateKeyMask)!=NSAlternateKeyMask)];
            }
            else
            {
                NSBeep();
            }
            
            return;
        }
    }
    
    [super keyDown:theEvent];
}

- (IBAction) delete:(id) sender
{
    if ([self delegate]!=nil)
	{
		[[self delegate] outlineView:self deleteSelectedRowsWithConfirmationRequired:YES];
	}
}

#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5

- (void) textDidEndEditing:(NSNotification *)notification;
{
    if ([[[notification userInfo] valueForKey:@"NSTextMovement"] intValue] == NSReturnTextMovement)
    {
		NSMutableDictionary *newUserInfo;

		newUserInfo = [[NSMutableDictionary alloc] initWithDictionary:[notification userInfo]];

		[newUserInfo setObject:[NSNumber numberWithInt:NSIllegalTextMovement] forKey:@"NSTextMovement"];

		notification = [NSNotification notificationWithName:[notification name]
													 object:[notification object]
												   userInfo:newUserInfo];

		[super textDidEndEditing:notification];

		[newUserInfo release];

		[[self window] makeFirstResponder:self];
    }
    else
    {
		[super textDidEndEditing:notification];
    }
}

#endif

@end
