/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICBuildDistributedNotificationCenter.h"

#import "ICBuildStack.h"

@implementation ICBuildDistributedNotificationCenter

+ (ICBuildDistributedNotificationCenter *) defaultCenter
{
	static ICBuildDistributedNotificationCenter * sDefaultCenter=nil;
	
	if (sDefaultCenter==nil)
	{
		sDefaultCenter=[ICBuildDistributedNotificationCenter new];
	}
	
	return sDefaultCenter;
}

- (id) init
{
	self=[super init];
	
	if (self!=nil)
	{
		distributedNotificationCenter_=[NSDistributedNotificationCenter defaultCenter];
	}
	
	return self;
}

- (void) dealloc
{
	[distributedNotificationCenter_ release];
	
	[attachedProcessID_ release];

	[attachedProjectPath_ release];

	[super dealloc];
}



#pragma mark -

- (void) attachToProcess:(pid_t) inPID andProject:(NSString *) inPath
{
	[attachedProcessID_ release];
	
	attachedProcessID_=[[NSNumber alloc] initWithUnsignedInt:inPID];
	
	[attachedProjectPath_ release];
	
	attachedProjectPath_=[inPath copy];
}


#pragma mark -

- (void) postNotificationRequest:(NSUInteger) inRequestID metadata:(NSDictionary *) inDictionary
{
	if (attachedProcessID_!=nil && attachedProjectPath_!=nil && distributedNotificationCenter_!=nil)
	{
		NSDictionary * tDictionary;
        NSNumber * tStepNumber;
		
        tStepNumber=[NSNumber numberWithUnsignedLong:inRequestID];
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:attachedProcessID_,IC_BUILDER_NOTIFICATION_INFO_PID,
															   attachedProjectPath_,IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
															   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_REQUEST],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
                                                               tStepNumber,IC_BUILDER_INFORMATION_STEP,
															   inDictionary,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
															   nil];
                                                            
        [distributedNotificationCenter_ postNotificationName:PACKAGES_BUILDER_NOTIFICATION object:nil userInfo:tDictionary options:NSNotificationPostToAllSessions];
	}
}

- (void) postNotificationWithState:(NSUInteger) inState forStep:(NSUInteger) inStep metadata:(NSDictionary *) inDictionary
{
	if (attachedProcessID_!=nil && attachedProjectPath_!=nil && distributedNotificationCenter_!=nil)
	{
		NSDictionary * tDictionary;
        NSNumber * tStepNumber;
		
        if (IC_BUILDER_INFORMATION_STEP_CURRENT==inStep)
		{
			tStepNumber=[NSNumber numberWithUnsignedLong:[[[[ICBuildStack sharedBuildStack] stack] lastObject] intValue]];
		}
		else
		{
			tStepNumber=[NSNumber numberWithUnsignedLong:inStep];
		}
		
		if (inState==IC_BUILDER_BUILD_STATE_BEGIN)
		{
			[[ICBuildStack sharedBuildStack] addStep:inStep withMetadata:inDictionary];
		}
		
		tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:attachedProcessID_,IC_BUILDER_NOTIFICATION_INFO_PID,
															   attachedProjectPath_,IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
															   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_STANDARD],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
                                                               tStepNumber,IC_BUILDER_INFORMATION_STEP,
															   [NSNumber numberWithUnsignedLong:inState],IC_BUILDER_INFORMATION_STATE,
															   [[ICBuildStack sharedBuildStack] stack],IC_BUILDER_NOTIFICATION_INFO_EVENT_INFORMATION_STACK,
															   [[ICBuildStack sharedBuildStack] metadata],IC_BUILDER_NOTIFICATION_INFO_EVENT_INFORMATION_STACK_METADATA,
															   inDictionary,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
															   nil];
                                                            
        //NSLog(@"%@",tDictionary);
		
		[distributedNotificationCenter_ postNotificationName:PACKAGES_BUILDER_NOTIFICATION object:nil userInfo:tDictionary options:NSNotificationPostToAllSessions];
	
		if (inState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
		{
			[[ICBuildStack sharedBuildStack] removeLastStep];
		}
	}
}

- (void) postNotificationDebugInformation:(NSDictionary *) inDictionary
{
	if (attachedProcessID_!=nil && attachedProjectPath_!=nil && distributedNotificationCenter_!=nil)
	{
		NSDictionary * tDictionary;
        
        tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:attachedProcessID_,IC_BUILDER_NOTIFICATION_INFO_PID,
															   attachedProjectPath_,IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
															   [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_DEBUG],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
                                                               [[ICBuildStack sharedBuildStack] stack],IC_BUILDER_NOTIFICATION_INFO_EVENT_INFORMATION_STACK,
															   [[ICBuildStack sharedBuildStack] metadata],IC_BUILDER_NOTIFICATION_INFO_EVENT_INFORMATION_STACK_METADATA,
															   inDictionary,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
															   nil];
                                                            
        [distributedNotificationCenter_ postNotificationName:PACKAGES_BUILDER_NOTIFICATION object:nil userInfo:tDictionary options:NSNotificationPostToAllSessions];
	}
}

- (void) postNotificationBuildProgressionInformation:(NSDictionary *) inDictionary
{
	if (attachedProcessID_!=nil && attachedProjectPath_!=nil && distributedNotificationCenter_!=nil)
	{
		NSDictionary * tDictionary;
        
        tDictionary=[NSDictionary dictionaryWithObjectsAndKeys:attachedProcessID_,IC_BUILDER_NOTIFICATION_INFO_PID,
					 attachedProjectPath_,IC_BUILDER_NOTIFICATION_INFO_PROJECT_PATH,
					 [NSNumber numberWithInt:IC_BUILDER_NOTIFICATION_INFO_EVENT_BUILD_PROGRESSION],IC_BUILDER_NOTIFICATION_INFO_EVENT_ID,
					 inDictionary,IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA,
					 nil];
		
        [distributedNotificationCenter_ postNotificationName:PACKAGES_BUILDER_NOTIFICATION object:nil userInfo:tDictionary options:NSNotificationPostToAllSessions];
	}
}

@end
