/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICInstallationHierarchyUtilities.h"

#import "ICDocumentFormat+Constants.h"

#import "ICInstallationTypeUtilities.h"

@implementation ICInstallationHierarchyUtilities

+ (void) updateListOfPackages:(NSMutableArray *) inMutableArray withChoices:(NSArray *) inChoices
{
	if (inMutableArray!=nil && inChoices!=nil)
	{
		NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
		NSEnumerator * tEnumerator;
		
		tEnumerator=[inChoices objectEnumerator];
		
		if (tEnumerator!=nil)
		{
			NSDictionary * tChoiceDictionary;
			
			while (tChoiceDictionary=[tEnumerator nextObject])
			{
				NSNumber * tNumber;
				
				tNumber=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE];
				
				if ([tNumber intValue]==ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_TYPE_PACKAGE)
				{
					NSString * tPackageUUID;
					
					tPackageUUID=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_PACKAGE_UUID];
					
					if ([inMutableArray containsObject:tPackageUUID]==NO)
					{
						[inMutableArray addObject:tPackageUUID];
					}
				}
				else
				{
					// It's a group
					
					NSMutableArray * tChildrenArray;
					
					tChildrenArray=[tChoiceDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_LIST_ITEM_GROUP_CHILDREN];
					
					if (tChoiceDictionary!=nil)
					{
						[ICInstallationHierarchyUtilities updateListOfPackages:inMutableArray withChoices:tChildrenArray];
					}
				}
			}
		}
		
		[pool release];
	}
}

+ (NSArray *) packagesUUIDEnabledInPresentationDictionary:(NSDictionary *) inPresentationDictionary
{
	NSMutableArray * tMutableArray=nil;
	
	if (inPresentationDictionary!=nil)
	{
		NSDictionary * tInstallationTypeDictionary;
		
		tInstallationTypeDictionary=[inPresentationDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE];
		
		if (tInstallationTypeDictionary!=nil)
		{
			NSDictionary * tHierarchiesDictionary;
		
			tHierarchiesDictionary=[tInstallationTypeDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES];
			
			if (tHierarchiesDictionary!=nil)
			{
				NSEnumerator * tEnumerator;
				
				tEnumerator=[tHierarchiesDictionary objectEnumerator];
				
				if (tEnumerator!=nil)
				{
					NSDictionary * tDictionary;
					
					tMutableArray=[NSMutableArray array];
					
					if (tMutableArray!=nil)
					{
						while (tDictionary=[tEnumerator nextObject])
						{
							NSArray * tArray;
						
							tArray=[tDictionary objectForKey:ICDOCUMENT_PROJECT_PRESENTATION_INSTALLATION_TYPE_HIERARCHIES_LIST];
						
							if (tArray!=nil)
							{
								[ICInstallationHierarchyUtilities updateListOfPackages:tMutableArray withChoices:tArray];
							}
						}
					}
				}
			}
		}
	}
	
	return tMutableArray;
}

@end
