/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICPayloadUtilities.h"

#import "ICBuildDistributedNotificationCenter.h"

#import "ICDocumentFormat+Constants.h"

@implementation ICPayloadUtilities

+ (NSMutableDictionary *) firstItemForHierarchy:(NSMutableDictionary *) inDictionary andDefaultLocation:(NSString *) inPath
{
    NSMutableDictionary * tDictionary=inDictionary;
    
    if ([inPath isEqualToString:@"/"]==NO)
    {
        NSArray * tPathComponents;
        NSUInteger i,tCount;
        NSString * tComponent;
        
        tPathComponents=[inPath componentsSeparatedByString:@"/"];
        
        tCount=[tPathComponents count];
        
        for(i=0;i<tCount;i++)
        {
            NSArray * tChildArray;
            NSUInteger j,tChildrenCount;
            NSMutableDictionary * tChildDictionary;
            NSString * tPathName;
            
            tComponent=[tPathComponents objectAtIndex:i];
            
            tChildArray=[tDictionary objectForKey:ICDOCUMENT_PACKAGE_FILES_FILE_CHILDREN];
            
            tChildrenCount=[tChildArray count];
            
            for(j=0;j<tChildrenCount;j++)
            {
                tChildDictionary=[tChildArray objectAtIndex:j];
                
                tPathName=[tChildDictionary objectForKey:ICDOCUMENT_PATH];
                
                if (tPathName==nil)
                {
                    // A COMPLETER
                }
                
                if ([tPathName isEqualToString:tComponent]==YES)
                {
                    tDictionary=tChildDictionary;
                    
                    break;
                }
            }
        }
    }
    
    return tDictionary;
}

+ (int) optimizeHierarchy:(NSMutableDictionary *) inDictionary
{
    NSNumber * tNumber;
    int tType;
    NSMutableArray * tChildren;
    NSUInteger i,tCount;
    BOOL tFound=NO;
    NSMutableDictionary * tChildDictionary;
    
    tNumber=[inDictionary objectForKey:ICDOCUMENT_PACKAGE_FILES_FILE_TYPE];
        
    if (tNumber==nil)
    {
        // Missing Information
		
		[[ICBuildDistributedNotificationCenter defaultCenter] postNotificationWithState:IC_BUILDER_BUILD_STATE_END_FAILURE
																				forStep:IC_BUILDER_INFORMATION_STEP_CURRENT
																			   metadata:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithUnsignedLong:IC_BUILDER_FAILURE_REASON_MISSING_INFORMATION],IC_BUILDER_FAILURE_REASON,
																																   ICDOCUMENT_PACKAGE_FILES_FILE_TYPE,IC_BUILDER_FAILURE_TAG_NAME,
																																   nil]]; 
        return -1;
    }
    
    tType=[tNumber intValue];
    
    switch(tType)
    {
        case ICDOCUMENT_FILE_BASE_NODE_INVISIBLE:
		case ICDOCUMENT_FILE_ROOT_NODE:
		case ICDOCUMENT_FILE_BASE_NODE:
            break;
        case ICDOCUMENT_FILE_NEWFOLDER_NODE:
            return 0;
        case ICDOCUMENT_FILE_REALFILE_NODE:
            return 0;
        default:
            
			// Incorrect Value
			
			[[ICBuildDistributedNotificationCenter defaultCenter] postNotificationWithState:IC_BUILDER_BUILD_STATE_END_FAILURE
																				forStep:IC_BUILDER_INFORMATION_STEP_CURRENT
																			   metadata:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithUnsignedLong:IC_BUILDER_FAILURE_REASON_INCORRECT_VALUE],IC_BUILDER_FAILURE_REASON,
																																   ICDOCUMENT_PACKAGE_FILES_FILE_TYPE,IC_BUILDER_FAILURE_TAG_NAME,
																																   nil]]; 
			
			return -1;
    }
    
    tChildren=[inDictionary objectForKey:ICDOCUMENT_PACKAGE_FILES_FILE_CHILDREN];
    
    if (tChildren==nil)
    {
       // Missing Information
	   
	   [[ICBuildDistributedNotificationCenter defaultCenter] postNotificationWithState:IC_BUILDER_BUILD_STATE_END_FAILURE
																				forStep:IC_BUILDER_INFORMATION_STEP_CURRENT
																			   metadata:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithUnsignedLong:IC_BUILDER_FAILURE_REASON_MISSING_INFORMATION],IC_BUILDER_FAILURE_REASON,
																																   ICDOCUMENT_PACKAGE_FILES_FILE_CHILDREN,IC_BUILDER_FAILURE_TAG_NAME,
																																   nil]];   
        return -1;
    }
    
    tCount=[tChildren count];
    
    for(i=0;i<tCount;i++)
    {
        int tReturnCode;
		
		tChildDictionary=[tChildren objectAtIndex:i];
        
        tReturnCode=[self optimizeHierarchy:tChildDictionary];
		
		if (tReturnCode==-1)
		{
			return -1;
		}
		
		if (tReturnCode==1)
        {
            [tChildren removeObjectAtIndex:i];
            
            tCount--;
            i--;
        }
        else
        {
            tFound=YES;
        }
    }
    
    if (tFound==NO && (tType==ICDOCUMENT_FILE_BASE_NODE_INVISIBLE || tType==ICDOCUMENT_FILE_BASE_NODE))
    {
        return 1;
    }
    
    return 0;
}

@end
