/*
Copyright (c) 2008-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import <Foundation/Foundation.h>

#import "ICProjectBuilder.h"

#import "ICBuilderLogManager.h"

#include <unistd.h>
#include <sys/types.h>

int main (int argc, const char * argv[])
{
    static char optstring[]="F:f:u:g:p:n:t:s:d";
	int optch;
	
	if (argc>=5)
    {
		NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
		NSString * tProjectPath=nil;
		NSString * tReferenceFolderPath=nil;
		int tUserID=-1;
        int tGroupID=-1;
		int tProcessID=0;
		NSString * tScratchPath=nil;
        NSString * tSplitForksToolName=nil;
		NSString * tNotificationPath=nil;
		BOOL tDebug=NO;
		
		while ((optch = getopt(argc,(char **)argv,optstring))!=-1)
		{
			switch(optch)
			{
				case 'd':
					
					tDebug=YES;
					
					break;
				
				case 'F':
				
					if (*optarg!=0)
					{
						tReferenceFolderPath=[NSString stringWithUTF8String:optarg];
					}
					else
					{
						[[ICBuilderLogManager sharedLogManager] logMessageWithLevel:ASL_LEVEL_CRIT format:@"Missing reference folder path (-F)"];
						
						return 1;
					}
					
					break;
				
				case 'f':
				
					if (*optarg!=0)
					{
						tProjectPath=[NSString stringWithUTF8String:optarg];
					}
					else
					{
						[[ICBuilderLogManager sharedLogManager] logMessageWithLevel:ASL_LEVEL_CRIT format:@"Missing file path (-f)"];
						
						return 1;
					}
					
					break;
					
				case 'u':
				
					if (*optarg!=0)
					{
						tUserID=atoi(optarg);
					}
					else
					{
						[[ICBuilderLogManager sharedLogManager] logMessageWithLevel:ASL_LEVEL_CRIT format:@"Missing User ID (-u)"];
						
						return 1;
					}
					
					break;
					
				case 'g':
				
					if (*optarg!=0)
					{
						tGroupID=atoi(optarg);
					}
					else
					{
						[[ICBuilderLogManager sharedLogManager] logMessageWithLevel:ASL_LEVEL_CRIT format:@"Missing Group ID (-g)"];
						
						return 1;
					}
					
					break;
					
				case 'p':
				
					if (*optarg!=0)
					{
						tProcessID=atoi(optarg);
					}
					else
					{
						[[ICBuilderLogManager sharedLogManager] logMessageWithLevel:ASL_LEVEL_CRIT format:@"Missing Process ID (-p)"];
						
						return 1;
					}
					
					break;
					
				case 'n':
				
					if (*optarg!=0)
					{
						tNotificationPath=[NSString stringWithUTF8String:optarg];
					}
					else
					{
						[[ICBuilderLogManager sharedLogManager] logMessageWithLevel:ASL_LEVEL_CRIT format:@"Missing Notification Path (-n)"];
						
						return 1;
					}
					
					break;
					
				case 't':
				
					if (*optarg!=0)
					{
						tSplitForksToolName=[NSString stringWithUTF8String:optarg];
					}
					else
					{
						[[ICBuilderLogManager sharedLogManager] logMessageWithLevel:ASL_LEVEL_CRIT format:@"Missing SplitForks tool name (-t)"];
						
						return 1;
					}
					
					break;
					
				case 's':
					
					if (*optarg!=0)
					{
						tScratchPath=[NSString stringWithUTF8String:optarg];
					}
					else
					{
						[[ICBuilderLogManager sharedLogManager] logMessageWithLevel:ASL_LEVEL_CRIT format:@"Missing Scracth path (-s)"];
						
						return 1;
					}
					
					break;
				
				default:
				
					break;
			}
		}
		
		if (tProjectPath!=nil &&
			tUserID!=-1 &&
			tGroupID!=-1 &&
			tProcessID!=0 &&
			tNotificationPath!=nil)
		{
			ICProjectBuilder * tProjectBuilder;
			
			tProjectBuilder=[ICProjectBuilder sharedProjectBuilder];
			
			if (tProjectBuilder!=nil)
			{
				[tProjectBuilder buildProjectAtPath:tProjectPath
									   forProcessID:tProcessID
										 withUserID:tUserID
											groupID:tGroupID
								   notificationPath:tNotificationPath
								 splitForksToolName:tSplitForksToolName
										scratchPath:tScratchPath
									referenceFolder:tReferenceFolderPath
											  debug:tDebug];
        
				[tProjectBuilder release];
			}
		}
		
		[pool release];
    }
    else
	{
		return 1;
	}
    
    return 0;
}
