/*
Copyright (c) 2009-2010, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* Thanks to Thomas for the references on mach port messaging with launchd */

#import "ICDispatcher.h"
#include <launch.h>

#define AUTOQUIT_DELAY	300.0f	// 5 Minutes

CFDataRef MessagePortCallBack(CFMessagePortRef local, SInt32 msgid, CFDataRef data, void *info);

CFDataRef MessagePortCallBack(CFMessagePortRef local, SInt32 msgid, CFDataRef data, void *info)
{
	ICDispatcher * tDispatcher;
	
	tDispatcher=(ICDispatcher *) info;
	
	return [tDispatcher processMessagePortMessage:msgid withData:data];
}

@implementation ICDispatcher

- (id) init
{
	self=[super init];
	
	if (self!=nil)
	{
		NSProcessInfo * tProcessInfo;
		
		messagePortContext_.version=0;
		messagePortContext_.info=(void *) self;
		messagePortContext_.retain=NULL;
		messagePortContext_.release=NULL;
		messagePortContext_.copyDescription=NULL;
		
		// Get the path
		
		processPath_=nil;
		
		tProcessInfo=[NSProcessInfo processInfo];
		
		if (tProcessInfo!=nil)
		{
			NSArray * tArguments;
			
			tArguments=[tProcessInfo arguments];
			
			if (tArguments!=nil)
			{
				processPath_=[[[tArguments objectAtIndex:0] stringByDeletingLastPathComponent] copy];
			}
		}
	}
	
	return self;
}

- (void) dealloc
{
	[processPath_ release];
	
	[super dealloc];
}

#pragma mark -

- (BOOL) run
{
#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5

	[self handlePortMessage:nil];
	
	return YES;

#else

	launch_data_t tCheckinRequest;
	BOOL tShouldRun=NO;
	
	tCheckinRequest=launch_data_new_string(LAUNCH_KEY_CHECKIN);
	
	if (tCheckinRequest!=NULL)
	{
		launch_data_t tCheckinResponse;
		
		tCheckinResponse=launch_msg(tCheckinRequest);
		
		if (tCheckinResponse!=NULL)
		{
			switch(launch_data_get_type(tCheckinResponse))
			{
				case LAUNCH_DATA_ERRNO:
					{
						int tError;
						
						tError=launch_data_get_errno(tCheckinResponse);
					
						NSLog(@"Error: %d",tError);
					}
					
					break;
					
				case LAUNCH_DATA_DICTIONARY:
					
					{
						launch_data_t tMachServicesDictionary;
					
						tMachServicesDictionary = launch_data_dict_lookup(tCheckinResponse, LAUNCH_JOBKEY_MACHSERVICES);
						
						if (tMachServicesDictionary!=NULL)
						{
							launch_data_t tMachServicePortName;
							
							tMachServicePortName=launch_data_dict_lookup(tMachServicesDictionary, ICDISPATCH_MACHPORT_NAME );
							
							if (tMachServicePortName!=NULL)
							{
								mach_port_t tMachPort;
								NSPort * tPort;
								
								tMachPort=launch_data_get_machport(tMachServicePortName);
								
								tPort = [[NSMachPort alloc] initWithMachPort:tMachPort];
								
								if (tPort!=nil)
								{
									[tPort setDelegate:self];
								
									[[NSRunLoop currentRunLoop] addPort:tPort forMode:NSDefaultRunLoopMode];
								
									tShouldRun=YES;
								}
							}
							else
							{
								NSLog(@"MachPort Name not found");
							}
						}
						else
						{
							NSLog(@"Mach Service Dictionary not found");
						}
					}
					
					break;
					
				default:
				
					NSLog(@"Unknwon error");
				
					break;
			}
			
			launch_data_free(tCheckinResponse);
		}
		else
		{
			// A COMPLETER
		}
		
		launch_data_free(tCheckinRequest);
	}
	else
	{
		NSLog(@"Memory Low");
	}
	
	if (tShouldRun==YES)
	{
		[[NSRunLoop currentRunLoop] run];
		
		return YES;
	}
	
	return NO;
	
#endif
}

- (void) terminate:(id) sender
{
	exit(0);
}

- (CFDataRef) processMessagePortMessage:(SInt32) inMessageID withData:(CFDataRef) inData
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    
    if (inData)
    {
        NSTask * tBuildTask;
        NSMutableArray *args = [NSMutableArray array];
        NSDictionary * tDictionary;
        CFStringRef tErrorString;
        
        tDictionary=(NSDictionary *) CFPropertyListCreateFromXMLData(kCFAllocatorDefault,inData,kCFPropertyListImmutable,&tErrorString);
        
        if (tDictionary!=nil)
        {
            NSString * tString;
			NSNumber * tNumber;
			
            tBuildTask=[NSTask new];
            
			// Project Path
					
			[args addObject:@"-f"];
			
			[args addObject:[tDictionary objectForKey:ICDISPATCH_PROJECT_PATH]];
            
			// User ID
			
			[args addObject:@"-u"];
			
            [args addObject:[[tDictionary objectForKey:ICDISPATCH_USER_ID] stringValue]];
            
            // Group ID
			
			[args addObject:@"-g"];
			
			[args addObject:[[tDictionary objectForKey:ICDISPATCH_GROUP_ID] stringValue]];
            
			// Process ID
			
			[args addObject:@"-p"];
			
            [args addObject:[[tDictionary objectForKey:ICDISPATCH_PROCESS_ID] stringValue]];	
            
            // Notification Path
			
			[args addObject:@"-n"];
			
			[args addObject:[tDictionary objectForKey:ICDISPATCH_NOTIFICATION_PATH]];		
			
			// Tool to use to split forks
			
			tString=[tDictionary objectForKey:ICDISPATCH_SPLITFORKS_TOOL];
			
			if (tString!=nil)
			{
				[args addObject:@"-t"];
				
				[args addObject:tString];	
            }
			
            // Scratch Path
			
			tString=[tDictionary objectForKey:ICDISPATCH_SCRATCH_PATH];
            
            if (tString!=nil)
            {
                [args addObject:@"-s"];
				
				[args addObject:tString];						
            }
			
			// Debug Mode
			
			tNumber=[tDictionary objectForKey:ICDISPATCH_DEBUG_MODE];
            
            if (tNumber!=nil && [tNumber boolValue]==YES)
            {
                [args addObject:@"-d"];
            }
			
			// Reference Folder
			
			tString=[tDictionary objectForKey:ICDISPATCH_REFERENCE_FOLDER];
            
            if (tString!=nil)
            {
                [args addObject:@"-F"];
				
				[args addObject:tString];						
            }
            
			[tBuildTask setLaunchPath:[processPath_ stringByAppendingPathComponent:@"packages_builder"]];
			
            [tBuildTask setArguments:args];
            
            [tBuildTask launch];
            
            [tDictionary release];
			
			// Delay auto-quit
			
			[NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(terminate:) object:nil]; 
			
			[self performSelector:@selector(terminate:) withObject:nil afterDelay:AUTOQUIT_DELAY];
        }
        else
        {
            // Release Memory
            
            CFRelease(tErrorString);
        }
    }
    
    [pool release];
    
    return NULL;
}

- (void) handlePortMessage:(NSPortMessage *) inMessage
{
	if (messagePortInitialized_==NO)
	{
		CFMessagePortRef tLocalMessagePort;
		Boolean shouldFreeInfo=FALSE;
		
		tLocalMessagePort =CFMessagePortCreateLocal(kCFAllocatorDefault, (CFStringRef) ICDISPATCH_MESSAGEPORT_NAME,MessagePortCallBack, &messagePortContext_, &shouldFreeInfo);
    
		if (tLocalMessagePort!=NULL)
		{
			CFRunLoopSourceRef tRunLoopSource;
			
			tRunLoopSource =CFMessagePortCreateRunLoopSource(kCFAllocatorDefault, tLocalMessagePort,0);
			
			if (tRunLoopSource!=NULL)
			{
				CFRunLoopAddSource(CFRunLoopGetCurrent(), tRunLoopSource,kCFRunLoopDefaultMode);
				
				messagePortInitialized_=YES;
			}
		}
#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5

		[[NSRunLoop currentRunLoop] run];

#else		
		[self performSelector:@selector(terminate:) withObject:nil afterDelay:AUTOQUIT_DELAY];
#endif

	}
}

@end
