/*
Copyright (c) 2004-2010, Stéphane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "ICBuildObserver.h"

#import "ICProjectBuilder+Notifications.h"

#import "ICProjectBuilder+SignatureRequest.h"

#import "ICSignatureFactory.h"

#import "ICDocumentFormat+Constants.h"

@implementation ICBuildObserver

- (void) setVerbose:(BOOL) inVerbose
{
    verbose_=inVerbose;
}

- (void) processBuildNotification:(NSNotification *) notification
{
    NSDictionary * tUserInfo;
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    static int sProcessID=-1;
    
    if (sProcessID==-1)
    {
        sProcessID=[[NSProcessInfo processInfo] processIdentifier];
    }
    
    tUserInfo=[notification userInfo];
    
    if (tUserInfo!=nil)
    {
        NSNumber * tNumber;
        
        tNumber=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_PID];
        
        if (tNumber!=nil)
        {
            /* Check the notification is for us */
            
            if ([tNumber intValue]==sProcessID)
            {
                // Event ID
				
				tNumber=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_ID];
				
				if (tNumber!=nil)
				{
					int tEventType;
					
					tEventType=[tNumber intValue];
					
					if (tEventType==IC_BUILDER_NOTIFICATION_INFO_EVENT_DEBUG)
					{
						//NSLog(@"DEBUG_DATA: %@",[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA]);
					}
					else if (tEventType==IC_BUILDER_NOTIFICATION_INFO_EVENT_STANDARD)
					{
						NSUInteger tStep=0;
						NSUInteger tState=0;
						NSDictionary * tStackMetadata;
						NSString * tStepString=@"";
						
						tStackMetadata=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_INFORMATION_STACK_METADATA];
						
						// Step
						
						tNumber=[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP];
						
						if (tNumber!=nil)
						{
							tStep=[tNumber unsignedLongValue];
							
							tStepString=[tNumber stringValue];
						}
						else
						{
							// A COMPLETER
						}
						
						
						
						// State
						
						tNumber=[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STATE];
						
						if (tNumber!=nil)
						{
							tState=[tNumber unsignedLongValue];
						}
						else
						{
							// A COMPLETER
						}
						
						if (tState==IC_BUILDER_BUILD_STATE_END_FAILURE)
						{
							static BOOL sFirstFailureReport=YES;
							static BOOL sKnownErrorFound=NO;
							NSDictionary * tMetadata;
							
							if (sFirstFailureReport==YES)
							{
								if (verbose_==YES)
								{
									(void)fprintf(stdout, "\n\n");
								}
								
								(void)fprintf(stdout, "==============================================================================\n");
							
								(void)fprintf(stdout, "ERROR:\n");
								
								sFirstFailureReport=NO;
							}
							
							tMetadata=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA];
							
							if (tMetadata==nil)
							{
								if (tStep==IC_BUILDER_INFORMATION_STEP_PROJECT)
								{
									if (sKnownErrorFound==NO)
									{
										(void)fprintf(stdout, "\nDescription:\n\nUnknow Error\n");
									}
								}
							}
							else
							{
								tNumber=[tMetadata objectForKey:IC_BUILDER_FAILURE_REASON];
								
								if (tNumber!=nil)
								{
									NSUInteger tFailureReason;
									NSArray * tStepsArray;
									
									tFailureReason=[tNumber unsignedLongValue];
									
									if (tFailureReason==IC_BUILDER_FAILURE_REASON_UNKNOWN)
									{
										if (tStep==IC_BUILDER_INFORMATION_STEP_PROJECT)
										{
											if (sKnownErrorFound==NO)
											{
												(void)fprintf(stdout, "Description:\n\nUnknow Error\n");
												
												sKnownErrorFound=YES;
											}
										}
									}
									else
									{
										sKnownErrorFound=YES;
										
										(void)fprintf(stdout, "\nDescription:\n\n");
										
										if (tFailureReason==IC_BUILDER_FAILURE_REASON_OUT_OF_MEMORY)
										{
											(void)fprintf(stdout, "Not enough memory to perform operation.");
										}
										else
										{
											NSString * tTag;
											NSString * tFilePath;
											BOOL isFolder;
											BOOL isTool;
											NSString * tTitle=nil;
											
											tTag=[tMetadata objectForKey:IC_BUILDER_FAILURE_TAG_NAME];
											
											tFilePath=[tMetadata objectForKey:IC_BUILDER_INFORMATION_FILE_PATH];
											
											isFolder=[[tMetadata objectForKey:IC_BUILDER_FAILURE_FILE_IS_FOLDER] boolValue];
											
											isTool=[[tMetadata objectForKey:IC_BUILDER_FAILURE_FILE_IS_TOOL] boolValue];
											
											switch(tFailureReason)
											{
												case IC_BUILDER_FAILURE_REASON_MISSING_INFORMATION:
													
													tTitle=nil;
														
													if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH]==YES)
													{
														tTitle=[NSString stringWithFormat:@"The location of the referenced package has not been fully defined.",tTag];
													}
													
													if (tTitle==nil)
													{
														tTitle=[NSString stringWithFormat:@"Missing information for tag '%@'",tTag];
													}
													
													(void)fprintf(stdout, [tTitle UTF8String]);
													
													break;
													
												case IC_BUILDER_FAILURE_REASON_MISSING_BUILD_DATA:
												
													(void)fprintf(stdout, "Missing build data for tag '%s'\n",[tTag UTF8String]);
												
													break;
													
												case IC_BUILDER_FAILURE_REASON_INCORRECT_VALUE:
												
													(void)fprintf(stdout, "Incorrect value for object with tag '%s'\n",[tTag UTF8String]);
													
													break;
												
												case IC_BUILDER_FAILURE_REASON_FILE_INCORRECT_TYPE:
													
													(void)fprintf(stdout, "Incorrect type for file at path \"%s\"",[[tFilePath lastPathComponent] UTF8String]);
													
													break;
												
												case IC_BUILDER_FAILURE_REASON_FILE_INSUFFICIENT_PERMISSIONS_WRITE:
												
													(void)fprintf(stdout, "Insufficient privileges to write at path \"%s\"",[[tFilePath lastPathComponent] UTF8String]);
												
													break;
												
												case IC_BUILDER_FAILURE_REASON_FILE_INSUFFICIENT_PERMISSIONS_SET:
												
													(void)fprintf(stdout, "Insufficient privileges to set permissions for path \"%s\"",[[tFilePath lastPathComponent] UTF8String]);
												
													break;
												
												case IC_BUILDER_FAILURE_REASON_OPERATION_FAILED:
												
													(void)fprintf(stdout, "%s returned error code '%d'",[[tFilePath lastPathComponent] UTF8String],[[tMetadata objectForKey:IC_BUILDER_FAILURE_TOOL_RETURN_CODE] intValue]);
													
													break;
												
												case IC_BUILDER_FAILURE_REASON_SPLIT_NON_HFS_VOLUME:
												
													(void)fprintf(stdout, "The Temporary Build Path is on a non HFS volume");
												
													break;
												
												case IC_BUILDER_FAILURE_REASON_LICENSE_MISSING_TEMPLATE:
												
													(void)fprintf(stdout, "License template for \"%s\" can not be found",[tTag UTF8String]);
													
													break;
												
												case IC_BUILDER_FAILURE_REASON_PACKAGE_SAME_NAME:
													
													// A COMPLETER
													
													break;
														
												case IC_BUILDER_FAILURE_REASON_STRING_EMPTY:
												
													tTitle=nil;
														
													if (tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE_INFO)
													{
														if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_IDENTIFIER]==YES)
														{
															tTitle=@"The identifier of the package can not be empty.";
														}
														else if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_VERSION]==YES)
														{
															tTitle=@"The version of the package can not be empty.";
														}
													}
													else if (tStep==IC_BUILDER_INFORMATION_STEP_DISTRIBUTION)
													{
														if ([tTag isEqualToString:ICDOCUMENT_PROJECT_SETTINGS_NAME]==YES)
														{
															tTitle=@"The name of the project can not be empty.";
														}
														
													}
													else if (tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE_CREATE)
													{
														if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_NAME]==YES)
														{
															tTitle=@"The name of the package can not be empty.";
														}
													}
													else
													{
														if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_SETTINGS_LOCATION_PATH]==YES)
														{
															tTitle=@"The location of the referenced package has not been fully defined.";
														}
													}
													
													if (tTitle==nil)
													{
														tTitle=[NSString stringWithFormat:@"String can not be empty for tag '%@'",tTag];
													}
													
													(void)fprintf(stdout, [tTitle UTF8String]);
													
													break;
												
												case IC_BUILDER_FAILURE_REASON_REQUIREMENT_MISSING_CONVERTER:
													
													(void)fprintf(stdout, "Converter not found for requirement of type '%s'",[tTag UTF8String]);
													
													break;
													
												case IC_BUILDER_FAILURE_REASON_REQUIREMENT_MISSING_CODE:
													
													(void)fprintf(stdout, "No code generated for requirement of type '%s'",[tTag UTF8String]);
													
													break;
												
												case IC_BUILDER_FAILURE_REASON_LOCATOR_MISSING_CONVERTER:
												
													(void)fprintf(stdout, "Converter not found for locator of type '%s'",[tTag UTF8String]);
													
													break;
													
												case IC_BUILDER_FAILURE_REASON_LOCATOR_CONVERSION_ERROR:
												
													(void)fprintf(stdout, "No code generated for locator of type '%s'",[tTag UTF8String]);
													
													break;
												
												case IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_FOUND:
												
													tTitle=nil;
														
													if (tStep==IC_BUILDER_INFORMATION_STEP_PACKAGE_IMPORT)
													{
														if (tTag!=nil)
														{
															if ([tTag isEqualToString:ICDOCUMENT_PACKAGE_REFERENCE_PATH]==YES)
															{
																tTitle=[NSString stringWithFormat:@"Unable to find package at path '%@'",tFilePath];
															}
														}
													}
													
													if (tTitle==nil)
													{
														if (isTool==YES)
														{
															tTitle=[NSString stringWithFormat:@"Unable to find tool '%@'",[tFilePath lastPathComponent]];
														}
														else if (isFolder==YES)
														{
															tTitle=[NSString stringWithFormat:@"Unable to find folder at path '%@'",tFilePath];
														}
														else
														{
															tTitle=[NSString stringWithFormat:@"Unable to find file at path '%@'",tFilePath];
														}
													}
													
													(void)fprintf(stdout, [tTitle UTF8String]);
													
													break;
												
												case IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_CREATED:
												
													if (isFolder==YES)
													{
														(void)fprintf(stdout, "Unable to create folder at path '%s'\n",[tFilePath UTF8String]);
													}
													else
													{
														(void)fprintf(stdout, "Unable to create file at path '%s'\n",[tFilePath UTF8String]);
													}
													
													break;
													
												case IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_COPIED:
												
													(void)fprintf(stdout, "Unable to copy item at path '%s'",[tFilePath UTF8String]);
													
													tNumber=[tMetadata objectForKey:IC_BUILDER_FAILURE_SECONDARY_REASON];
													
													if (tNumber!=nil)
													{
														if ([tNumber unsignedLongValue]==IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_FOUND)
														{
															(void)fprintf(stdout, " because the item could not be found");
														}
													}
													
													(void)fprintf(stdout, "\n");
													
													break;
													
												case IC_BUILDER_FAILURE_REASON_FILE_CANNOT_BE_DELETED:
												
													if (isFolder==YES)
													{
														(void)fprintf(stdout, "Unable to delete folder at path '%s'\n",[tFilePath UTF8String]);
													}
													else
													{
														(void)fprintf(stdout, "Unable to delete file at path '%s'\n",[tFilePath UTF8String]);
													}
													
													break;
													
												case IC_BUILDER_FAILURE_REASON_SIGNATURE_GENERIC_ERROR:
														
														(void)fprintf(stdout, "Unable to sign the data");
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_SIGNATURE_TIME_OUT:
														
														(void)fprintf(stdout, "Signing operation timed out");
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_SIGNATURE_AUTH_DENIED:
													
														(void)fprintf(stdout, "Signing operation denied");
														
														break;
														
													case IC_BUILDER_FAILURE_REASON_SIGNATURE_CERTIFICATE_NOT_FOUND:
													
														(void)fprintf(stdout, "Unable to find signing certificate");
														
														break;
											}
										}
									}
									
									if (verbose_==NO)
									{
										tStepsArray=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_INFORMATION_STACK];
									
										if (tStepsArray!=nil)
										{
											NSEnumerator * tEnumerator;
											
											tEnumerator=[tStepsArray objectEnumerator];
											
											if (tEnumerator!=nil)
											{
												NSString * tStepString;
												BOOL tFirstStep=YES;
												
												(void)fprintf(stdout, "\nStep:\n\n");
												
												while (tStepString=[tEnumerator nextObject])
												{
													int tStepValue;
													NSDictionary * tStepMetadata;
													
													if (tFirstStep==NO)
													{
														(void)fprintf(stdout, " > ");
													}
													
													tStepValue=[tStepString intValue];
													
													switch(tStepValue)
													{
														case IC_BUILDER_INFORMATION_STEP_PROJECT:
															
															(void)fprintf(stdout, "Project");
															
															break;
															
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION:
															
															(void)fprintf(stdout, "Distribution");
															
															break;
														
														case IC_BUILDER_INFORMATION_STEP_PACKAGE:
														
															(void)fprintf(stdout, "Package");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_PROJECT_BUILD_FOLDER:
														
															(void)fprintf(stdout, "Build Folder");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_PROJECT_CLEAN:
														
															(void)fprintf(stdout, "Clean");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_BACKGROUND_IMAGE:
														
															(void)fprintf(stdout, "Background");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_WELCOME_MESSAGE:
														
															(void)fprintf(stdout, "Introduction");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_README_MESSAGE:
														
															(void)fprintf(stdout, "ReadMe");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_LICENSE_DOCUMENT:
														
															(void)fprintf(stdout, "License");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_CONCLUSION_MESSAGE:
														
															(void)fprintf(stdout, "Conclusion");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_SCRIPT:
														
															(void)fprintf(stdout, "Definition");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_CHOICES_HIERARCHIES:
														
															(void)fprintf(stdout, "Choices Hierarchies");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_INSTALLATION_VOLUME_REQUIREMENTS:
														
															(void)fprintf(stdout, "Installation|Volume Requirements");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_JAVASCRIPT:
														
															(void)fprintf(stdout, "JavaScript Scripts");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_RESOURCES:
														
															(void)fprintf(stdout, "Resources");
														
															break;
															
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_SCRIPTS:
														
															(void)fprintf(stdout, "Scripts");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_INSTALLER_PLUGINS:
														
															(void)fprintf(stdout, "Plugins");
														
															break;
														
														case IC_BUILDER_INFORMATION_STEP_PACKAGE_CREATE:
														case IC_BUILDER_INFORMATION_STEP_PACKAGE_REFERENCE:
														case IC_BUILDER_INFORMATION_STEP_PACKAGE_IMPORT:
														
															(void)fprintf(stdout, "Package");
															
															tStepMetadata=[tStackMetadata objectForKey:tStepString];
															
															if (tStepMetadata!=nil)
															{
																NSString * tPackageName;
																
																tPackageName=[tStepMetadata objectForKey:IC_BUILDER_INFORMATION_PACKAGE_NAME];
																
																if (tPackageName!=nil)
																{
																	(void)fprintf(stdout, " '%s'",[tPackageName UTF8String]);
																}
															}
															
															break;
															
														case IC_BUILDER_INFORMATION_STEP_PACKAGE_INFO:
														
															(void)fprintf(stdout, "PackageInfo");
														
															break;
															
														case IC_BUILDER_INFORMATION_STEP_PACKAGE_PAYLOAD:
														
															(void)fprintf(stdout, "Payload");
														
															break;
															
														case IC_BUILDER_INFORMATION_STEP_SCRIPTS_PAYLOAD:
														
															(void)fprintf(stdout, "Scripts");
														
															break;
															
														case IC_BUILDER_INFORMATION_STEP_PAYLOAD_ASSEMBLE:
															
															(void)fprintf(stdout, "Assemble");
															
															break;
															
														case IC_BUILDER_INFORMATION_STEP_PAYLOAD_SPLIT:
														
															(void)fprintf(stdout, "Split Forks");
														
															break;
															
														case IC_BUILDER_INFORMATION_STEP_PAYLOAD_BOM:
															
															(void)fprintf(stdout, "Bom");
															
															break;
															
														case IC_BUILDER_INFORMATION_STEP_PAYLOAD_PAX:
														
															(void)fprintf(stdout, "Pax");
														
															break;
														
													}
													
													tFirstStep=NO;
												}
												
												(void)fprintf(stdout, "\n");
											}
										}
									}
								}
							}
							
							if (tStep==IC_BUILDER_INFORMATION_STEP_PROJECT)
							{
								(void)fprintf(stdout, "\n==============================================================================");
								
								(void)fprintf(stdout, "\nBuild Failed\n");
								
								exit(1);
							}
						}
						else
						{
							char * tStateString=NULL;
							char * tOffsetString="";
							
							if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
							{
								tStateString="";
							}
							else if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
							{
								tStateString=" (Completed)";
							}
							else
							{
								// Incorrect Value
								
								// A COMPLETER
							}
							
							if (buildDistribution_==YES)
							{
								tOffsetString="\t";
							}
							else if (tState==IC_BUILDER_BUILD_STATE_END_SUCCESS)
							{
								tOffsetString="";
							}
							
							if (verbose_==YES)
							{
								switch(tStep)
								{
									case IC_BUILDER_INFORMATION_STEP_PROJECT:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											NSString * tProjectPath;
											struct tm *tTimeStructure;
											
											startTime_=time(NULL);
											
											tTimeStructure=localtime(&startTime_);
											
											tProjectPath=[[tStackMetadata objectForKey:tStepString] objectForKey:IC_BUILDER_INFORMATION_FILE_PATH];
											
											(void)fprintf(stdout, "Building Project (%02d:%02d:%02d) at path: %s \n",tTimeStructure->tm_hour,tTimeStructure->tm_min,tTimeStructure->tm_sec,[tProjectPath fileSystemRepresentation]);
											(void)fprintf(stdout, "------------------------------------------------------------------------------\n\n");
										}
										else
										{
											double tDiffTime;
											
											tDiffTime=difftime(time(NULL),startTime_);
											
											(void)fprintf(stdout, "\n------------------------------------------------------------------------------\n");
											
											if (tDiffTime>=2.0)
											{
												(void)fprintf(stdout, "Build Successful (%g seconds)\n",tDiffTime);
											}
											else if (tDiffTime<1.0)
											{
												(void)fprintf(stdout, "Build Successful (less than a second)\n");
											}
											else
											{
												(void)fprintf(stdout, "Build Successful (1 second)\n");
											}
											
											exit(0);
										}
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											buildDistribution_=YES;
											
											(void)fprintf(stdout, "Distribution\n\n");
										}
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PACKAGE:
									
										break;
										
									
									case IC_BUILDER_INFORMATION_STEP_PROJECT_BUILD_FOLDER:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "Build Folder");
										}
										else
										{
											(void)fprintf(stdout, " (Done)\n\n");
										}
										
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PROJECT_CLEAN:
										
										(void)fprintf(stdout, "\tClean%s\n",tStateString);
										
										break;
										
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_BACKGROUND_IMAGE:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\t\tBackground picture");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_WELCOME_MESSAGE:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\t\tIntroduction");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_README_MESSAGE:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\t\tRead Me");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
									
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_LICENSE_DOCUMENT:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\t\tLicense");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
									
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_CONCLUSION_MESSAGE:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\t\tConclusion");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
									
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_SCRIPT:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\tDefinition\n\n");
										}
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_CHOICES_HIERARCHIES:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\n\t\tChoices hierarchies");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
									
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_JAVASCRIPT:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\n\t\tJavaScript scripts");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
									
										break;
										
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_INSTALLATION_VOLUME_REQUIREMENTS:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\t\tInstallation|Volume requirements");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n\n");
										}
										
										break;
									
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_RESOURCES:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\n\tResources");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_SCRIPTS:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "%s\tScripts\n\n",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, "%s\t\n",tOffsetString);
										}
										break;
									
									case IC_BUILDER_INFORMATION_STEP_DISTRIBUTION_INSTALLER_PLUGINS:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "\n\tPlugins");
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_INFO:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "%s\tPackageInfo",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_PAYLOAD:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "%s\tPayload\n\n",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, "%s\t\n",tOffsetString);
										}
										break;
										
									
									case IC_BUILDER_INFORMATION_STEP_SCRIPTS_PAYLOAD:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "%s\tScripts\n\n",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, "%s\t\n",tOffsetString);
										}
										break;
									
									
									case IC_BUILDER_INFORMATION_STEP_PAYLOAD_ASSEMBLE:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "%s\t\tAssemble file hierarchy",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}

										break;
									
									case IC_BUILDER_INFORMATION_STEP_PAYLOAD_SPLIT:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "%s\t\tSplit forks",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
									
										break;
									
									case IC_BUILDER_INFORMATION_STEP_PAYLOAD_BOM:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "%s\t\tCreate bill of material",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}
										
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PAYLOAD_PAX:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											(void)fprintf(stdout, "%s\t\tCreate pax archive",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
										}

										
										break;
									
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_CREATE:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											NSString * tPackageName;
											
											tPackageName=[[tStackMetadata objectForKey:tStepString] objectForKey:IC_BUILDER_INFORMATION_PACKAGE_NAME];
											
											(void)fprintf(stdout, "%sPackage \"%s\"\n\n",tOffsetString,[tPackageName UTF8String]);
										}
										else
										{
											(void)fprintf(stdout, "\n%s----------------------------------------------\n\n",tOffsetString);
										}
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_REFERENCE:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											NSString * tPackageName;
											
											tPackageName=[[tStackMetadata objectForKey:tStepString] objectForKey:IC_BUILDER_INFORMATION_PACKAGE_NAME];
											
											(void)fprintf(stdout, "%sPackage \"%s\"\n\n",tOffsetString,[tPackageName UTF8String]);
											(void)fprintf(stdout, "%s\tReference package",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
											
											(void)fprintf(stdout, "\n%s----------------------------------------------\n\n",tOffsetString);
										}
										
										break;
									
										break;
										
									case IC_BUILDER_INFORMATION_STEP_PACKAGE_IMPORT:
									
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											NSString * tPackageName;
											
											tPackageName=[[tStackMetadata objectForKey:tStepString] objectForKey:IC_BUILDER_INFORMATION_PACKAGE_NAME];
											
											(void)fprintf(stdout, "%sPackage \"%s\"\n\n",tOffsetString,[tPackageName UTF8String]);
											(void)fprintf(stdout, "%s\tCopy imported package",tOffsetString);
										}
										else
										{
											(void)fprintf(stdout, " (done)\n");
											
											(void)fprintf(stdout, "\n%s----------------------------------------------\n\n",tOffsetString);
										}
										
										break;
								}
							}
							else
							{
								switch(tStep)
								{
									case IC_BUILDER_INFORMATION_STEP_PROJECT:
										
										if (tState==IC_BUILDER_BUILD_STATE_BEGIN)
										{
											startTime_=time(NULL);
										}
										else
										{
											double tDiffTime;
											
											tDiffTime=difftime(time(NULL),startTime_);
											
											if (tDiffTime>=2.0)
											{
												(void)fprintf(stdout, "Build Successful (%g seconds)\n",tDiffTime);
											}
											else if (tDiffTime<1.0)
											{
												(void)fprintf(stdout, "Build Successful (less than a second)\n");
											}
											else
											{
												(void)fprintf(stdout, "Build Successful (1 second)\n");
											}
											
											exit(0);
										}
										
										break;
								}
							}
							
						}
					}
					else if (tEventType==IC_BUILDER_NOTIFICATION_REQUEST)
					{
						int tRequestID;
							
						tRequestID=[[tUserInfo objectForKey:IC_BUILDER_INFORMATION_STEP] intValue];
						
						if (tRequestID==IC_BUILDER_NOTIFICATION_REQUEST_SIGN_DATA)
						{
							NSDictionary * tMetadata;
							
							tMetadata=[tUserInfo objectForKey:IC_BUILDER_NOTIFICATION_INFO_EVENT_EXTRA_DATA];
							
							if ([ICSignatureFactory performSignatureRequestWithDictionary:tMetadata]==NO)
							{
								// A COMPLETER
							}
						}
					}
					else
					{
						// Incorrect Value
						
						// A COMPLETER
					}
				}
            }
        }
    }
    
    [pool release];
}

@end
